/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             
 * Project: CORVAL2
 *
 * Contact:
 * Omar Halabi
 * e-mail: oha@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

#ifndef ADAPTOR_H
#define ADAPTOR_H

#include <stdio.h>

extern FILE * logStream;
//ProfileBody * sBody;
//IOR * sIOR = 0;




/*-------------------------------------------------------------------------
 * These are the functions of the GCI operational interface. The adaptation
 * process is the process of implementing these functions to allow the
 * generated code to communicate with the system to be tested. Here is a 
 * short description of what the functions must do:
 *
 * GciSend          - Send something to the IUT through a PCO
 * GciImplicitSend  - Send something to the IUT, not through a PCO (but for
 *                    example to the test operator) to affect the current
 *                    status of the IUT
 * GciStartTimer    - Start a given timer
 * GciCancelTimer   - Cancel a given timer
 * GciReadTimer     - Read the value of a given timer
 * GciSnapshot      - Check the status of the receive channels, timers and
 *                    possibly test components (concurrent TTCN)
 * GciLog           - Function used for logging
 * GciConfig        - Create configuration (concurrent TTCN)
 * GciCreate        - Create test component (concurrent TTCN)
 */

extern "C"
{
GciStatus GciSend( int pco, GciValue* object );
GciStatus GciImplicitSend( GciValue* object );
GciStatus GciStartTimer( int timer, long value, int unit );
GciStatus GciCancelTimer( int timer );
long GciReadTimer( int timer );
GciStatus GciSnapshot( );
GciStatus GciConfig( GciConf conf );
GciStatus GciCreate( int comp, char* tree, GciValue* args );
GciStatus GciLog( int logId, char* logString );
void GciLogPrint( char* string, char* logString, int isRow );



/*-------------------------------------------------------------------------
 * These are the functions of the GCI behaviour interface. These three
 * functions are used to inform the TTCN runtime behaviour about:
 *
 * GciReceive       - Something has been received on the given PCO
 * GciTimeout       - The given timer has timed out
 * GciDone          - The given test component has terminated (concurrent TTCN)
 */

  GciStatus GciReceive( int pco, GciValue* object ); 
  GciStatus GciTimeout( int timer );
  GciStatus GciDone( int ptc );
  extern FILE* logStream;
 
/*------------------------------------------------------------------------
 * Global definition of the Log Stream used by the GciLog function, might
 * be used by other user written code in order to have a conformant logging.
 */


};

/*---------------------------------------------------------------------------
 *
 * Name: AdTimeStruct
 *
 * Description:
 *
 *  The AdTimeStruct structure is used as the basic notion of timers. The
 *  structure has a couple of attribute values, namely
 *    - An identifier, the descriptor used by GCI
 *     
 */

typedef struct AdTimeStruct
  {
    int Id;                     /* GCI timer id */
    int hd;                     /* rpoll timer id, AD_TIMER_STOPPED */
    long Duration;              /* in ms */
    long StartTime;             /* when started */
    int Unit;
    struct AdTimeStruct *Next;  /* to next struct */
  }
AdTimeStruct;

//extern FILE * logStream;
#endif /* ADAPTOR_H */
