//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/iiop/src/cdrCoder.cc
//
// Modifications:
// $Log: cdrCoder.cpp,v $
// Revision 1.4  2001/01/31 19:41:09  mli
// Update.
//
// Revision 1.1.1.1  2000/02/09 17:07:11  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.12  1999/07/05 15:11:27  stephen
// Overruled 105 mod and fixed memory problem.
//
// Revision 1.11  1999/04/30 12:33:10  ian
// SR105 Changed memory deallocation
//
// Revision 1.10  1999/04/26 14:57:50  james
// Changed CORBA::Char* to char*
//
// Revision 1.9  1997/12/04 14:10:48  stephen
// IOR shift operator added.
//
// Revision 1.8  1997/11/21 13:01:23  stephen
// Support for marshalling into a vsOrbString class.
//
// Revision 1.7  1997/10/22 13:47:42  stephen
// UMR fixed by call to memset.
//
// Revision 1.6  1997/09/19 08:45:51  stephen
// Post merge edits.
//
// Revision 1.5  1997/09/18 14:38:36  damien
// Win32 changes
//
// Revision 1.4  1997/09/12 07:51:00  stephen
// Finalised byte swapping in operators.
//
// Revision 1.3  1997/09/11 14:44:07  stephen
// Support added for either endianness within coder.
//
// Revision 1.2  1997/09/04 13:48:06  stephen
// Added operator<< for ProfileBody class.
//
// Revision 1.1  1997/09/02 14:38:56  stephen
// Renamed cdrCoder to have cpp extension.
//
// Revision 1.1  1997/08/29 12:05:17  stephen
// Initial IIOP sources
//


#include <iiop.hh>
#include <string.h>
#ifdef _WIN32
#include <assert.h>
#endif

cdrCoder::cdrCoder() :
    m_Stream(0),
    m_BufIndx(0),
    m_Flipped(0),
    m_AllocLength(0),
    m_StreamEndian(0),
    m_StreamLength(0)
{
    setEndian();
    
}

cdrCoder::cdrCoder(CORBA::ULong initSize) :
    m_Stream(new unsigned char[initSize]),
    m_AllocLength(initSize),
    m_Flipped(0)
{
    memset(m_Stream, 0, initSize);
    
    m_BufIndx = m_Stream;
    setEndian();
}

cdrCoder::cdrCoder( unsigned char * buffer, CORBA::ULong bufLen) 
{
    m_AllocLength = bufLen;
    m_StreamLength = bufLen;
    m_Stream = 0;
    m_Stream = new unsigned char [m_AllocLength];
    ::memcpy(m_Stream, buffer, m_AllocLength);
    m_BufIndx = m_Stream;
    m_Flipped = 0;
    setEndian();
}


void
cdrCoder::initialise( unsigned char * buffer, CORBA::ULong bufLen) 
{
    m_AllocLength = bufLen;
    m_StreamLength = bufLen;

    m_Stream = new unsigned char [m_AllocLength];
    ::memcpy((char *)m_Stream, (const char *)buffer, m_AllocLength);
    m_BufIndx = m_Stream;
    m_Flipped = 0;
    setEndian();
}


cdrCoder::~cdrCoder()
{
	if (m_Stream)
		delete m_Stream;

    m_Stream = 0;
    m_BufIndx = 0;
    m_AllocLength = 0;
    m_StreamLength = 0;
}

void
cdrCoder::alignAndGrow(CORBA::ULong sizeOfType)
{
    // align buffer on type boundry
    m_BufIndx = ( unsigned char *)
	(((CORBA::ULong)(m_BufIndx + sizeOfType - 1)) &~(sizeOfType - 1));

    // how much remaining space is there
    // in the current stream
    unsigned char * endOfStream = m_Stream + m_AllocLength;

    CORBA::ULong outOfSpace = endOfStream - m_BufIndx >= sizeOfType;

    // oops get more space
    if (outOfSpace)
	;
}


cdrCoder &
cdrCoder::operator<<(const CORBA::Char &c)
{
    alignAndGrow(CDR_CHAR_SZ);
    
    *m_BufIndx = c;
    m_BufIndx++;
    
    return *this;    
}

cdrCoder &
cdrCoder::marshallOctet(const CORBA::Octet &o)
{
    alignAndGrow(CDR_OCTET_SZ);

    *m_BufIndx = o;
    m_BufIndx++;
    
    return *this;    
}


cdrCoder &
cdrCoder::operator<<(vsOrbSeqOctet &s)
{
    CORBA::ULong i = 0;
    CORBA::ULong seqLength = s.length();

    *this << seqLength;
    
    for (i = 0; i < seqLength ; i++)
	marshallOctet(s[i]);

    return *this;    
}

cdrCoder &
cdrCoder::operator<<(const char *s)
{
    int i = 0;
    
    CORBA::ULong strLen = ::strlen(s);

    *this << strLen + 1;

    for (i; i < strLen; i++)
	marshallOctet(s[i]);

    *this << '\0';
    
    return *this;    
}

cdrCoder &
cdrCoder::operator<<(const CORBA::Boolean &b)
{
    alignAndGrow(CDR_BOOL_SZ);

    *m_BufIndx = b;
    m_BufIndx++;
    return *this;    
}

cdrCoder &
cdrCoder::operator<<(const CORBA::Short &s)
{
    alignAndGrow(CDR_SHORT_SZ);

    if (m_Flipped)
    {
	*(unsigned  char*)m_BufIndx++ = ((unsigned char*)&s)[1];
	*(unsigned char*)m_BufIndx++ = ((unsigned char*)&s)[0];
    }
    else
    {
	*(CORBA::Short *)m_BufIndx = s;
	m_BufIndx += CDR_SHORT_SZ;
    }
    
    return *this;    
}

cdrCoder &
cdrCoder::operator<<(const CORBA::UShort &us)
{
    alignAndGrow(CDR_USHORT_SZ);

    if (m_Flipped)
    {
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&us)[1];
	*( unsigned char*)m_BufIndx++ = (( unsigned char*)&us)[0];
    }
    else
    {
	*(CORBA::UShort *)m_BufIndx = us;
	m_BufIndx += CDR_USHORT_SZ;
    }
    
    return *this;
}

cdrCoder &
cdrCoder::operator<<(const CORBA::Long &l)
{
    alignAndGrow(CDR_LONG_SZ);

    if (m_Flipped)
    {
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&l)[3];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&l)[2];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&l)[1];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&l)[0];
    }
    else
    {
	*(CORBA::Long *)m_BufIndx = l;
	m_BufIndx += CDR_LONG_SZ;
    }
    
    return *this;
}

cdrCoder &
cdrCoder::operator<<(const CORBA::ULong &ul)
{
    alignAndGrow(CDR_ULONG_SZ);

    if (m_Flipped)
    {
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ul)[3];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ul)[2];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ul)[1];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ul)[0];
    }
    else
    {
	*(CORBA::ULong *)m_BufIndx = ul;
	m_BufIndx += CDR_ULONG_SZ;
    }
    
    return *this;
}

cdrCoder &
cdrCoder::operator<<(const CORBA::Float &f)
{
    alignAndGrow(CDR_FLOAT_SZ);

    if (m_Flipped)
    {
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&f)[3];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&f)[2];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&f)[1];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&f)[0];
    }
    else
    {
	*(CORBA::Float *)m_BufIndx = f;
	m_BufIndx += CDR_FLOAT_SZ;
    }
    
    return *this;
}

cdrCoder &
cdrCoder::operator<<(const CORBA::LongLong &ll)
{
    alignAndGrow(CDR_LONGLONG_SZ);

    if (m_Flipped)
    {
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ll)[7];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ll)[6];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ll)[5];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ll)[4];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ll)[3];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ll)[2];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ll)[1];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ll)[0];
    }
    else
    {
	*(CORBA::LongLong *)m_BufIndx = ll;
	m_BufIndx += CDR_LONGLONG_SZ;
    }
    
    return *this;
}

cdrCoder &
cdrCoder::operator<<(const CORBA::ULongLong &ull)
{
    alignAndGrow(CDR_ULONGLONG_SZ);

    if (m_Flipped)
    {
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ull)[7];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ull)[6];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ull)[5];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ull)[4];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ull)[3];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ull)[2];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ull)[1];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&ull)[0];
    }
    else
    {
	*(CORBA::ULongLong *)m_BufIndx = ull;
	m_BufIndx += CDR_ULONGLONG_SZ;
    }
    
    return *this;
}



cdrCoder &
cdrCoder::operator<<(const CORBA::Double &d)
{
    alignAndGrow(CDR_DOUBLE_SZ);

    if (m_Flipped)
    {
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&d)[7];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&d)[6];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&d)[5];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&d)[4];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&d)[3];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&d)[2];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&d)[1];
	*(unsigned  char*)m_BufIndx++ = ((unsigned  char*)&d)[0];
    }
    else
    {
	*(CORBA::Double *)m_BufIndx = d;
	m_BufIndx += CDR_DOUBLE_SZ;
    }
    
    return *this;
}

cdrCoder&
cdrCoder::operator<<(vsOrbSeqServiceContext &seq)
{
    
    CORBA::ULong i = 0;
    CORBA::ULong seqLength = seq.length();

    // The first part of a request is the service context data
    *this << seqLength;

    if (seqLength)
    {
	for (i; i < seqLength ; i++)
	{
	    // Each element is an identifier, data length
	    // and length number of elements
	    
	    *this << seq[i].contextId();
	    *this << seq[i].contextData();
	}
    }

    return *this;
}

cdrCoder&
cdrCoder::operator<<(ProfileBody &p)
{
    *this << m_OurEndian;
    
    //    *this << p.ver().major() << p.ver().minor();
    this->marshallOctet(p.ver().major());
    this->marshallOctet(p.ver().minor());
    *this << p.host();
    *this << p.port();
    *this << p.objectKey();

    return *this;
}

cdrCoder&
cdrCoder::operator<<(IOR &ior)
{
    *this << ior.typeId();

    CORBA::ULong i = 0;
    CORBA::ULong seqLength = ior.sequence().length();

    // The first part of a request is the service context data
    *this << seqLength;

    if (seqLength)
    {
	for (i; i < seqLength ; i++)
	{
	    // Each element is an identifier, data length
	    // and length number of elements
	    *this << ior.sequence()[i].id();
	    *this << ior.sequence()[i].data();
	}
    }
    return *this;
}

cdrCoder &
cdrCoder::operator>>(CORBA::Char &s)
{
    alignAndGrow(CDR_CHAR_SZ);

    s = *m_BufIndx;
    m_BufIndx++;

    return *this;
}

cdrCoder &
cdrCoder::unmarshallOctet(CORBA::Octet &o)
{
    alignAndGrow(CDR_OCTET_SZ);

    o = *m_BufIndx;
    m_BufIndx++;
    
    return *this;
}

cdrCoder &
cdrCoder::operator>>(vsOrbSeqOctet &s)
{
    int i = 0;
    
    for (i = 0; i < s.length() ; i++)
	unmarshallOctet(s[i]);

    return *this;    
}

cdrCoder &
cdrCoder::operator>>(vsOrbString &s)
{
    CORBA::ULong i = 0;
    
    CORBA::ULong strLen;
    
    *this >> strLen;

    s.reinit(strLen);
    
    for (i; i < strLen; i++)
	unmarshallOctet(s[i]);
    
    return *this;    
}

cdrCoder &
cdrCoder::operator>>(char *&s)
{
    CORBA::ULong i = 0;
    
    CORBA::ULong strLen;
    
    *this >> strLen;

    s = new char[strLen];
    
    for (i; i < strLen; i++)
    {
	// Prevent warning about temporaries being created
      unsigned char c;
      unmarshallOctet(c);
	s[i] = c;
    }
    
    return *this;    
}

cdrCoder &
cdrCoder::operator>>(CORBA::Boolean &b)
{
    alignAndGrow(CDR_BOOL_SZ);
    
    b = *m_BufIndx;
    m_BufIndx++;

    return *this;
}

cdrCoder &
cdrCoder::operator>>(CORBA::Short &s)
{
    alignAndGrow(CDR_SHORT_SZ);
    
    if (SameEndian())
    {
      s = *(CORBA::Short*)m_BufIndx;
      m_BufIndx += CDR_SHORT_SZ;
    }
    else
    {
     unsigned  char *target = (unsigned  char *)&s;
      target[1] = *m_BufIndx++;
      target[0] = *m_BufIndx++;
    }

    return *this;
}

cdrCoder &
cdrCoder::operator>>(CORBA::UShort &us)
{
    alignAndGrow(CDR_USHORT_SZ);

    if (SameEndian())
    {
      us = *(CORBA::UShort*)m_BufIndx;
      m_BufIndx += CDR_USHORT_SZ;
    }
    else
    {
      unsigned char *target = (unsigned char *)&us;
      target[1] = *m_BufIndx++;
      target[0] = *m_BufIndx++;
    }
    return *this;
}

cdrCoder &
cdrCoder::operator>>(CORBA::Long &l)
{
    alignAndGrow(CDR_LONG_SZ);

    if (SameEndian())
    {
      l = *(CORBA::Long*)m_BufIndx;
      m_BufIndx += CDR_LONG_SZ;
    }
    else
    {
      unsigned char *target = (unsigned  char *)&l;
      target[3] = *m_BufIndx++;
      target[2] = *m_BufIndx++;
      target[1] = *m_BufIndx++;
      target[0] = *m_BufIndx++;
    }
    return *this;
}

cdrCoder &
cdrCoder::operator>>(CORBA::ULong &ul)
{
    alignAndGrow(CDR_ULONG_SZ);

    if (SameEndian())
    {
      ul = *(CORBA::ULong*)m_BufIndx;
      m_BufIndx += CDR_ULONG_SZ;
    }
    else
    {
      unsigned  char *target = (unsigned  char *)&ul;
      target[3] = *m_BufIndx++;
      target[2] = *m_BufIndx++;
      target[1] = *m_BufIndx++;
      target[0] = *m_BufIndx++;
    }
    return *this;
}

cdrCoder &
cdrCoder::operator>>(CORBA::Float &f)
{
    alignAndGrow(CDR_FLOAT_SZ);

    if (SameEndian())
    {
      f = *(CORBA::Float*)m_BufIndx;
      m_BufIndx += CDR_FLOAT_SZ;
    }
    else
    {
      unsigned char *target = (unsigned  char *)&f;
      target[3] = *m_BufIndx++;
      target[2] = *m_BufIndx++;
      target[1] = *m_BufIndx++;
      target[0] = *m_BufIndx++;
    }
    return *this;
}

cdrCoder &
cdrCoder::operator>>(CORBA::LongLong &ll)
{
    alignAndGrow(CDR_LONGLONG_SZ);

    if (SameEndian())
    {
      ll = *(CORBA::LongLong*)m_BufIndx;
      m_BufIndx += CDR_LONGLONG_SZ;
    }
    else
    {
      unsigned char *target = (unsigned char *)&ll;
      target[7] = *m_BufIndx++;
      target[6] = *m_BufIndx++;
      target[5] = *m_BufIndx++;
      target[4] = *m_BufIndx++;
      target[3] = *m_BufIndx++;
      target[2] = *m_BufIndx++;
      target[1] = *m_BufIndx++;
      target[0] = *m_BufIndx++;
    }
    return *this;
}

cdrCoder &
cdrCoder::operator>>(CORBA::ULongLong &ull)
{
    alignAndGrow(CDR_ULONGLONG_SZ);

    if (SameEndian())
    {
      ull = *(CORBA::ULongLong*)m_BufIndx;
      m_BufIndx += CDR_ULONGLONG_SZ;
    }
    else
    {
      unsigned char *target = (unsigned  char *)&ull;
      target[7] = *m_BufIndx++;
      target[6] = *m_BufIndx++;
      target[5] = *m_BufIndx++;
      target[4] = *m_BufIndx++;
      target[3] = *m_BufIndx++;
      target[2] = *m_BufIndx++;
      target[1] = *m_BufIndx++;
      target[0] = *m_BufIndx++;
    }
    return *this;
}

cdrCoder &
cdrCoder::operator>>(CORBA::Double &d)
{
    alignAndGrow(CDR_DOUBLE_SZ);

    if (SameEndian())
    {
      d = *(CORBA::Double*)m_BufIndx;
      m_BufIndx += CDR_DOUBLE_SZ;
    }
    else
    {
      unsigned char *target = (unsigned  char *)&d;
      target[7] = *m_BufIndx++;
      target[6] = *m_BufIndx++;
      target[5] = *m_BufIndx++;
      target[4] = *m_BufIndx++;
      target[3] = *m_BufIndx++;
      target[2] = *m_BufIndx++;
      target[1] = *m_BufIndx++;
      target[0] = *m_BufIndx++;
    }
    return *this;
}

cdrCoder &
cdrCoder::operator>>(vsOrbSeqServiceContext &seq)
{
    // The first part of a request is the service context data
    CORBA::ULong i = 0;
    CORBA::ULong seqLength;

    *this >> seqLength;

    if (seqLength)
    {
	seq.grow(seqLength);
	seq.length(seqLength);
	
	for (i; i < seqLength ; i++)
	{
	    // Each element is an identifier, data length
	    // and length number of elements
	    CORBA::ULong localId;
	    CORBA::ULong octLength;

	    *this >> localId;
	    seq[i].contextId(localId);

	    *this >> octLength;
	    
	    seq[i].contextData().grow(octLength);
	    seq[i].contextData().length(octLength);
	    *this >> seq[i].contextData();
	}
    }
    return *this;
}

cdrCoder&
cdrCoder::operator>>(IOR *&ior)
{
    char * typeInfo = 0;

    *this >> typeInfo;

    ior->typeId(typeInfo);

		if (typeInfo)
			delete typeInfo;

    CORBA::ULong i = 0;
    CORBA::ULong seqLength;

    *this >> seqLength;

    ior->sequence().grow(seqLength);
    ior->sequence().length(seqLength);
    
    if (seqLength)
    {
	for (i; i < seqLength ; i++)
	{
	    // Each element is an identifier, data length
	    // and length number of elements
	    CORBA::ULong localId;
	    CORBA::ULong octLength;

	    *this >> localId;

	    ior->sequence()[i].id(localId);
	    
	    *this >> octLength;
	    
	    ior->sequence()[i].data().grow(octLength);
	    ior->sequence()[i].data().length(octLength);

	    *this >> ior->sequence()[i].data();
	}
    }
    return *this;
}


