/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             
 * Project: CORVAL2
 *
 * Contact:
 * Omar Halabi
 * e-mail: oha@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/
#include <stdio.h>
#include <iiop.hh>
#ifdef _WIN32
#include <assert.h>
#endif

// Hex array used to hexify() IORs
char * m_hexChars = "0123456789abcdef";

vsOrbSeqOctet::vsOrbSeqOctet()
{
    m_Maximum = 0;
    m_Length = 0;
    m_Data = 0;
    m_ReleaseFlag = 1;
}

vsOrbSeqOctet::vsOrbSeqOctet(const vsOrbSeqOctet& seqRef)
{
    m_Length = seqRef.m_Length;
    m_ReleaseFlag = 1;

    CORBA::ULong max = m_Maximum = seqRef.m_Maximum;
    if (max) {
        m_Data = allocbuf (max);
        for (CORBA::ULong index = 0; index < m_Length; index++) {
            m_Data[index] = seqRef.m_Data[index];
        }
    }
    else
        m_Data = NULL;
}

vsOrbSeqOctet::vsOrbSeqOctet(CORBA::ULong max)
{
    if (max) {
        m_Data = allocbuf (max);
    }
    else
        m_Data = 0;

    m_Length = 0;
    m_Maximum = max;
    m_ReleaseFlag = 1;
}


void
vsOrbSeqOctet::grow(CORBA::ULong max)
{
    if (max) {
        m_Data = allocbuf (max);
    }
    else
        m_Data = 0;

    m_Length = 0;
    m_Maximum = max;
    m_ReleaseFlag = 1;
}

vsOrbSeqOctet::vsOrbSeqOctet(CORBA::ULong max,
			     CORBA::ULong length,
			     CORBA::Octet* data,
			     CORBA::Boolean release)
{
    m_Maximum = max;
    m_Length = length;
    m_Data = data;
    m_ReleaseFlag = release;
}






vsOrbSeqOctet&
vsOrbSeqOctet::operator=(const vsOrbSeqOctet &seqRef)
{
    if (this == &seqRef) return *this;
    if (m_Data && m_ReleaseFlag) {
    }
    if (!m_ReleaseFlag || (seqRef.m_Maximum > m_Maximum)) {
        if (m_Data && m_ReleaseFlag) {
            freebuf (m_Data);
        }
        m_Data = allocbuf (seqRef.m_Maximum);
        m_Maximum = seqRef.m_Maximum;
        m_ReleaseFlag = 1;
    }
    m_Length = seqRef.m_Length;
    for (CORBA::ULong index = 0; index < m_Length; index++) {
      m_Data[index] = seqRef.m_Data[index];
      //  m_Data[index+1] = '\0'; 
    }
    
    return *this;
}

vsOrbSeqOctet::~vsOrbSeqOctet()
{
    if (m_ReleaseFlag && m_Data)
	delete [] m_Data;
}

CORBA::Octet*
vsOrbSeqOctet::allocbuf(CORBA::ULong numElems)
{
    return (new unsigned char[numElems]);
}

void
vsOrbSeqOctet::freebuf(CORBA::Octet* data)
{
    if (data)
	delete [] data;
}

CORBA::ULong
vsOrbSeqOctet::maximum() const
{
    return m_Maximum;
}

CORBA::ULong
vsOrbSeqOctet::length() const
{
    return m_Length ? m_Length : 0;
}

void
vsOrbSeqOctet::length (CORBA::ULong len)
{
    m_Length = len;
}

CORBA::Octet&
vsOrbSeqOctet::operator [] (CORBA::ULong index)
{
    if (index >= m_Length)
    {
	cout <<"Overflow of vsOrbSeqOctet from operator[]" << endl;
	assert(0);
    }
    
    return m_Data[index];
}

const CORBA::Octet&
vsOrbSeqOctet::operator [] (CORBA::ULong index) const
{
    if (index >= m_Length)
    {
	cout <<"Overflow of vsOrbSeqOctet from operator[] const" << endl;
	assert(0);
    }
    
    return m_Data[index];
}

vsOrbSeqServiceContext::~vsOrbSeqServiceContext()
{
    if (m_ReleaseFlag && m_Data)
	delete [] m_Data;
}



vsOrbSeqServiceContext::vsOrbSeqServiceContext()
{
    m_ReleaseFlag = 1;
    m_Length = 0;
    m_Maximum = 0;
    m_Data = 0;
}


vsOrbSeqServiceContext::vsOrbSeqServiceContext(
    CORBA::ULong max,
    CORBA::ULong length,
    ServiceContext* data,
    CORBA::Boolean release)
{
    m_ReleaseFlag = release;
    m_Length = length;
    m_Maximum = max;
    m_Data = data;
}

vsOrbSeqServiceContext::vsOrbSeqServiceContext(
    const vsOrbSeqServiceContext &sevSeq)
{
    m_Length = sevSeq.m_Length;
    m_ReleaseFlag = 1;
    CORBA::ULong max = m_Maximum = sevSeq.m_Maximum;
    if (max) {
        m_Data = allocbuf (max);
        for (CORBA::ULong index = 0; index < m_Length; index++) {
            m_Data[index] = sevSeq.m_Data[index];
        }
    }
    else
        m_Data = NULL;
}

vsOrbSeqServiceContext::vsOrbSeqServiceContext(CORBA::ULong max) {
    if (max) {
        m_Data = allocbuf (max);
    }
    else
        m_Data = NULL;

    m_Length = 0;
    m_ReleaseFlag = 1;
    m_Maximum = max;
}

void
vsOrbSeqServiceContext::grow(CORBA::ULong max) {
    if (max) {
        m_Data = allocbuf (max);
    }
    else
        m_Data = NULL;

    m_Length = 0;
    m_ReleaseFlag = 1;
    m_Maximum = max;
}

vsOrbSeqServiceContext&
vsOrbSeqServiceContext::operator=(const vsOrbSeqServiceContext &srvCntxtSeq)
{
    if (this == &srvCntxtSeq) return *this;
    if (m_Data && m_ReleaseFlag) {
    }
    if (!m_ReleaseFlag || (srvCntxtSeq.m_Maximum > m_Maximum)) {
        if (m_Data && m_ReleaseFlag) {
            freebuf (m_Data);
        }
        m_Data = allocbuf (srvCntxtSeq.m_Maximum);
        m_Maximum = srvCntxtSeq.m_Maximum;
        m_ReleaseFlag = 1;
    }
    m_Length = srvCntxtSeq.m_Length;
    for (CORBA::ULong index = 0; index < m_Length; index++) {
        m_Data[index] = srvCntxtSeq.m_Data[index];
    }
    return *this;
}

CORBA::ULong
vsOrbSeqServiceContext::maximum() const
{
    return m_Maximum;
}

CORBA::ULong
vsOrbSeqServiceContext::length() const
{
    return m_Length;
}

void
vsOrbSeqServiceContext::length(CORBA::ULong len) {
    if (len > m_Maximum) {
        vsOrbSeqServiceContext tempSeq (*this);
        if ( m_ReleaseFlag && m_Data) {
            freebuf (m_Data);
        }
        m_ReleaseFlag = 1;
        m_Length = len;
        m_Maximum = len;
        m_Data = allocbuf(len);
        for (CORBA::ULong index = 0; index < tempSeq.m_Length; index++) {
            m_Data[index] = tempSeq.m_Data[index];
        }
    }
    else if (len < m_Length) {
        if (m_ReleaseFlag) {
        }
        m_Length = len;
    } else {
        m_Length = len;
    }
}

ServiceContext&
vsOrbSeqServiceContext::operator [] (CORBA::ULong index)  {

    if (index >= m_Length)
    {
	cout <<"Overflow of vsOrbSeqServiceContext from operator[]" << endl;
	assert(0);
    }
    
    return m_Data[index];
}

const ServiceContext&
vsOrbSeqServiceContext::operator [] (CORBA::ULong index) const {

    if (index >= m_Length)
    {
	cout <<"Overflow of vsOrbSeqServiceContext from operator[] const" << endl;
	assert(0);
    }
    
    return m_Data[index];
}

ServiceContext*
vsOrbSeqServiceContext::allocbuf(CORBA::ULong nelems)
{
    return (new ServiceContext[nelems]);
}

void
vsOrbSeqServiceContext::freebuf(ServiceContext* data)
{
    if (data)
	delete [] data;
}


ServiceContext::~ServiceContext() {
}

ServiceContext::ServiceContext(const ServiceContext &srvCntxt) 
	 : m_ContextData(srvCntxt.m_ContextData)
{
    m_ContextId = srvCntxt.m_ContextId;
}

ServiceContext::ServiceContext() {
    m_ContextId = 0;
    m_ContextData = 0;
}

ServiceContext &
ServiceContext::operator=(const ServiceContext& srvCntxt)
{
    if (this == &srvCntxt)
	return *this;

    m_ContextId = srvCntxt.m_ContextId;
    m_ContextData = srvCntxt.m_ContextData;

    return *this;
}

vsOrbSeqTaggedProfile::~vsOrbSeqTaggedProfile()
{
    if (m_Data && m_ReleaseFlag)
        freebuf (m_Data);
}

vsOrbSeqTaggedProfile::vsOrbSeqTaggedProfile () {
    m_ReleaseFlag = 1;
    m_Length = 0;
    m_Maximum = 0;
    m_Data = 0;
}


vsOrbSeqTaggedProfile::vsOrbSeqTaggedProfile (
    CORBA::ULong max,
    CORBA::ULong length,
    TaggedProfile* data,
    CORBA::Boolean release)
{
    m_ReleaseFlag = release;
    m_Length = length;
    m_Maximum = max;
    m_Data = data;
}

vsOrbSeqTaggedProfile::vsOrbSeqTaggedProfile (
    const vsOrbSeqTaggedProfile &taggedProfileSeq)
{
    m_Length = taggedProfileSeq.m_Length;
    m_ReleaseFlag = 1;
    CORBA::ULong max = m_Maximum = taggedProfileSeq.m_Maximum;
    if (max) {
        m_Data = allocbuf (max);
        for (CORBA::ULong index = 0; index < m_Length; index++) {
            m_Data[index] = taggedProfileSeq.m_Data[index];
        }
    } else
        m_Data = NULL;
}

vsOrbSeqTaggedProfile::vsOrbSeqTaggedProfile (CORBA::ULong max) {
    if (max) {
        m_Data = allocbuf (max);
    } else
        m_Data = NULL;
    m_Length = 0;
    m_ReleaseFlag = 1;
    m_Maximum = max;
}

void
vsOrbSeqTaggedProfile::grow (CORBA::ULong max) {
    if (max) {
        m_Data = allocbuf (max);
    } else
        m_Data = NULL;
    m_Length = 0;
    m_ReleaseFlag = 1;
    m_Maximum = max;
}

vsOrbSeqTaggedProfile& vsOrbSeqTaggedProfile::operator= (
    const vsOrbSeqTaggedProfile &taggedProfileSeq)
{
    if (this == &taggedProfileSeq) return *this;
    if (m_Data && m_ReleaseFlag) {
    }
    if (!m_ReleaseFlag || (taggedProfileSeq.m_Maximum > m_Maximum)) {
        if (m_Data && m_ReleaseFlag) {
            freebuf (m_Data);
        }
        m_Data = allocbuf (taggedProfileSeq.m_Maximum);
        m_Maximum = taggedProfileSeq.m_Maximum;
        m_ReleaseFlag = 1;
    }
    m_Length = taggedProfileSeq.m_Length;
    for (CORBA::ULong index = 0; index < m_Length; index++) {
        m_Data[index] = taggedProfileSeq.m_Data[index];
    }
    return *this;
}

CORBA::ULong vsOrbSeqTaggedProfile::maximum () const
{
    return m_Maximum;
}

CORBA::ULong vsOrbSeqTaggedProfile::length () const
{
    return m_Length;
}

void vsOrbSeqTaggedProfile::length (CORBA::ULong len) {
    if (len > m_Maximum) {
        vsOrbSeqTaggedProfile tempSeq (*this);
        if (m_Data && m_ReleaseFlag) {
            {
            }
            freebuf (m_Data);
        }
        m_ReleaseFlag = 1;
        m_Length = len;
        m_Maximum = len;
        m_Data = allocbuf(len);
        for (CORBA::ULong index = 0; index < tempSeq.m_Length; index++) {
            m_Data[index] = tempSeq.m_Data[index];
        }
    } else if (len < m_Length) {
        if (m_ReleaseFlag) {
        }
        m_Length = len;
    } else {
        m_Length = len;
    }
}

TaggedProfile& vsOrbSeqTaggedProfile::operator [] (CORBA::ULong index)  {

    if (index >= m_Length)
    {
	cout <<"Overflow of vsOrbSeqTaggedProfile from operator[]" << endl;
	assert(0);
    }
    
    return m_Data[index];
}

const TaggedProfile& vsOrbSeqTaggedProfile::operator []
(CORBA::ULong index) const {

    if (index >= m_Length)
    {
	cout <<"Overflow of vsOrbSeqTaggedProfile from operator[] const" << endl;
	assert(0);
    }
    
    return m_Data[index];
}

TaggedProfile* vsOrbSeqTaggedProfile::allocbuf (CORBA::ULong nelems)
{
    return (new TaggedProfile[nelems]);
}

void vsOrbSeqTaggedProfile::freebuf (TaggedProfile* data)
{
    if (data) delete [] data;
}

TaggedProfile ::~TaggedProfile () {
}

TaggedProfile::TaggedProfile (const TaggedProfile &taggedProfile) 
	 : profile_data(taggedProfile.profile_data) {
    profile_id = taggedProfile.profile_id;
}

TaggedProfile::TaggedProfile () {
}

TaggedProfile &TaggedProfile::operator= (
    const TaggedProfile& taggedProfile)
{
    if (this == &taggedProfile) return *this;
    profile_id = taggedProfile.profile_id;
    profile_data = taggedProfile.profile_data;
    return *this;
}

vsOrbSeqTaggedComponent::~vsOrbSeqTaggedComponent()
{
    if (m_Data && m_ReleaseFlag)
        freebuf (m_Data);
}

vsOrbSeqTaggedComponent::vsOrbSeqTaggedComponent () {
    m_ReleaseFlag = 1;
    m_Length = 0;
    m_Maximum = 0;
    m_Data = 0;
}


vsOrbSeqTaggedComponent::vsOrbSeqTaggedComponent (
    CORBA::ULong max,
    CORBA::ULong length,
    TaggedComponent* data,
    CORBA::Boolean release)
{
    m_ReleaseFlag = release;
    m_Length = length;
    m_Maximum = max;
    m_Data = data;
}

vsOrbSeqTaggedComponent::vsOrbSeqTaggedComponent (
    const vsOrbSeqTaggedComponent &taggedComponentSeq)
{
    m_Length = taggedComponentSeq.m_Length;
    m_ReleaseFlag = 1;
    CORBA::ULong max = m_Maximum = taggedComponentSeq.m_Maximum;
    if (max) {
        m_Data = allocbuf (max);
        for (CORBA::ULong index = 0; index < m_Length; index++) {
            m_Data[index] = taggedComponentSeq.m_Data[index];
        }
    } else
        m_Data = NULL;
}

vsOrbSeqTaggedComponent::vsOrbSeqTaggedComponent (CORBA::ULong max) {
    if (max) {
        m_Data = allocbuf (max);
    } else
        m_Data = NULL;
    m_Length = 0;
    m_ReleaseFlag = 1;
    m_Maximum = max;
}

vsOrbSeqTaggedComponent& vsOrbSeqTaggedComponent::operator= (
    const vsOrbSeqTaggedComponent &taggedComponentSeq)
{
    if (this == &taggedComponentSeq) return *this;
    if (m_Data && m_ReleaseFlag) {
    }
    if (!m_ReleaseFlag || (taggedComponentSeq.m_Maximum > m_Maximum)) {
        if (m_Data && m_ReleaseFlag) {
            freebuf (m_Data);
        }
        m_Data = allocbuf (taggedComponentSeq.m_Maximum);
        m_Maximum = taggedComponentSeq.m_Maximum;
        m_ReleaseFlag = 1;
    }
    m_Length = taggedComponentSeq.m_Length;
    for (CORBA::ULong index = 0; index < m_Length; index++) {
        m_Data[index] = taggedComponentSeq.m_Data[index];
    }
    return *this;
}

CORBA::ULong vsOrbSeqTaggedComponent::maximum () const
{
    return m_Maximum;
}

CORBA::ULong vsOrbSeqTaggedComponent::length () const
{
    return m_Length;
}

void vsOrbSeqTaggedComponent::length (CORBA::ULong len) {
    if (len > m_Maximum) {
        vsOrbSeqTaggedComponent tempSeq (*this);
        if (m_Data && m_ReleaseFlag) {
            {
            }
            freebuf (m_Data);
        }
        m_ReleaseFlag = 1;
        m_Length = len;
        m_Maximum = len;
        m_Data = allocbuf(len);
        for (CORBA::ULong index = 0; index < tempSeq.m_Length; index++) {
            m_Data[index] = tempSeq.m_Data[index];
        }
    } else if (len < m_Length) {
        if (m_ReleaseFlag) {
        }
        m_Length = len;
    } else {
        m_Length = len;
    }
}

TaggedComponent& vsOrbSeqTaggedComponent::operator [] (CORBA::ULong index)  {

    if (index >= m_Length)
    {
	cout <<"Overflow of vsOrbSeqTaggedComponent from operator[]" << endl;
	assert(0);
    }
    
    return m_Data[index];
}

const TaggedComponent& vsOrbSeqTaggedComponent::operator [] (CORBA::ULong index) const {

    if (index >= m_Length)
    {
	cout <<"Overflow of vsOrbSeqTaggedComponent from operator[] const" << endl;
	assert(0);
    }
    
    return m_Data[index];
}

TaggedComponent* vsOrbSeqTaggedComponent::allocbuf (CORBA::ULong nelems)
{
    return (new TaggedComponent[nelems]);
}

void vsOrbSeqTaggedComponent::freebuf (TaggedComponent* data)
{
    if (data) delete [] data;
}

TaggedComponent ::~TaggedComponent () {
}

TaggedComponent::TaggedComponent (const TaggedComponent &taggedComponent) 
	 : component_data(taggedComponent.component_data) {
    component_id = taggedComponent.component_id;
}

TaggedComponent::TaggedComponent () {
}

TaggedComponent &TaggedComponent::operator= (
    const TaggedComponent& taggedComponent)
{
    if (this == &taggedComponent) return *this;
    component_id = taggedComponent.component_id;
    component_data = taggedComponent.component_data;
    return *this;
}



IIOPMessageHeader::IIOPMessageHeader( unsigned char * buffer,
				     CORBA::ULong length) :
    m_cdrCoder(buffer, length)
   
{
    m_cdrCoder.setEndian();
}


IIOPMessageHeader::IIOPMessageHeader(GciValue * Obj) : 
      m_cdrCoder(4096),
      m_Size(0)
{
  
  char  * car;
  
  GciValue * giopGciValue;
  GciValue * verGciValue;
  GciValue * aGciValue;
 

if(Obj != NULL)
 {
   if (GciGetType(Obj))
	  {
	 
    aGciValue = GciGetField( Obj, 1);
    giopGciValue = GciGetField( aGciValue, 1);
    verGciValue = GciGetField( aGciValue, 2);
   
    car = GciGetVisibleString(GciGetField(aGciValue, 1));
    m_Cookie[0] = car[0];
    m_Cookie[1] = car[1]; 
    m_Cookie[2] = car[2];
    m_Cookie[3] = car[3];

    m_Ver.m_Major = atol( GciGetOCTET_STRING(GciGetField(verGciValue, 1)));
    m_Ver.m_Minor = atol(GciGetOCTET_STRING(GciGetField(verGciValue, 2)));
    m_ByteSex = m_cdrCoder.OurEndian(); // GciGetBOOLEAN(GciGetField(aGciValue, 3))
    m_Flags = CORBA::Octet( m_cdrCoder.OurEndian());
    m_MsgType = atol(GciGetOCTET_STRING(GciGetField(aGciValue, 4)));
    m_Size    = GciGetINTEGER(GciGetField(aGciValue, 5));
  

	  }
 }
 
}

IIOPMessageHeader::~IIOPMessageHeader()
{
}

CORBA::ULong
IIOPMessageHeader::marshall()
{
  m_cdrCoder << m_Cookie[0] << m_Cookie[1] << m_Cookie[2] << m_Cookie[3];
  
  m_cdrCoder.marshallOctet(m_Ver.m_Major);
  m_cdrCoder.marshallOctet(m_Ver.m_Minor);
 
  switch(m_Ver.m_Minor)
    {
    case 0:
      m_cdrCoder << m_ByteSex;
      break;
    case 1:
      m_cdrCoder.marshallOctet(m_Flags);
      
      break;
    case 2:
      m_cdrCoder.marshallOctet(m_Flags);
      
      break;
    default:
      fprintf( stderr, "Error: UNKNOWN MINOR\n");
    }   
  
  
  m_cdrCoder.marshallOctet(m_MsgType);
  m_cdrCoder << m_Size;
  
  return 0;
}

int
IIOPMessageHeader::unmarshall()
{
  m_cdrCoder >> m_Cookie[0] >> m_Cookie[1] >> m_Cookie[2] >> m_Cookie[3];
  if ( ! (m_Cookie[0] == 'G' && m_Cookie[1] == 'I'
	  && m_Cookie[2] == 'O' && m_Cookie[3] == 'P') )
    return -1;
  //m_cdrCoder >> m_Ver.m_Major >> m_Ver.m_Minor;
  m_cdrCoder.unmarshallOctet(m_Ver.m_Major);
  m_cdrCoder.unmarshallOctet(m_Ver.m_Minor); 
  
  
  switch(m_Ver.m_Minor)
    {
    case 0:
      m_cdrCoder >> m_ByteSex;
      m_cdrCoder.StreamEndian(m_ByteSex);
      break;
      
    case 1:
      m_cdrCoder.unmarshallOctet(m_Flags);
      m_cdrCoder.StreamEndian(m_Flags);
      break;
      
    case 2:
      m_cdrCoder.unmarshallOctet(m_Flags);
      m_cdrCoder.StreamEndian(m_Flags);
      break;
      
    default:
      printf(" UNKNOWN MINOR IN THE UNMARSHALL\n");
    }  
   
  
    m_cdrCoder.unmarshallOctet(m_MsgType);
    

    if (m_MsgType > Fragment)
	return -1;
    
    m_cdrCoder >> m_Size;
    return 0;
}




IIOPRequestMessage::IIOPRequestMessage(GciValue * Obj) :
    IIOPMessageHeader(Obj)
  
{
  GciValue * reqHdrGciValue;
  reqHdrGciValue = GciGetField(Obj, 2);
   
  m_ServiceContext = 0;
  m_RequestId = GciGetINTEGER(GciGetField(reqHdrGciValue, 2));
  m_ResponseExpected = GciGetBOOLEAN(GciGetField(reqHdrGciValue, 3)); 
  m_ObjectKey = getObjectKey();
  m_Operation = GciGetGraphicString(GciGetField(reqHdrGciValue, 5));
  m_Principal = "oha";
  
}


IIOPRequestMessage::~IIOPRequestMessage()
{
  
}


CORBA::ULong
IIOPRequestMessage::marshall()
{

  
  // Build the basic header structure
  IIOPMessageHeader::marshall();
    
  // Marshall additional request information
  m_cdrCoder << m_ServiceContext;
 
  
  m_cdrCoder << m_RequestId;
  m_cdrCoder << m_ResponseExpected;
  
  // Marshall the objectKey 
  m_cdrCoder << m_ObjectKey;
  // Finally marshall the operation and user name 

  m_cdrCoder << m_Operation;
  m_cdrCoder << m_Principal ;
 

 return 0;
}


int
IIOPRequestMessage::unmarshall()
{
    // Build the basic header structure
    if (IIOPMessageHeader::unmarshall() == -1)
	return -1;

    
    // Unmarshall additional request information
    m_cdrCoder >> m_ServiceContext;
    m_cdrCoder >> m_RequestId;
    m_cdrCoder >> m_ResponseExpected;
   
    // Unmarshall the objectKey 
    CORBA::ULong seqLength = 0;
    
    m_cdrCoder >> seqLength;
    
    m_ObjectKey.grow(seqLength);
    m_ObjectKey.length(seqLength);
    
    m_cdrCoder >> m_ObjectKey;

    // Unmarshall operation and principal
    m_cdrCoder >> m_Operation;
    //    m_cdrCoder >> m_Principal;
    
    return 0;
}

IIOPRequestMessage_1_1::IIOPRequestMessage_1_1(GciValue * Obj) :
    IIOPMessageHeader(Obj)
  
{
  GciValue * reqHdrGciValue;
  reqHdrGciValue = GciGetField(Obj, 2);
   
  m_ServiceContext = 0;
  m_RequestId = GciGetINTEGER(GciGetField(reqHdrGciValue, 2));
  m_ResponseExpected = GciGetBOOLEAN(GciGetField(reqHdrGciValue, 3)); 
  m_Reserved1= atol(GciGetOCTET_STRING(GciGetField(reqHdrGciValue, 4)));
  m_Reserved2 = atol(GciGetOCTET_STRING(GciGetField(reqHdrGciValue, 5)));
  m_Reserved3 = atol(GciGetOCTET_STRING(GciGetField(reqHdrGciValue, 6)));
  m_ObjectKey = getObjectKey();
  m_Operation = GciGetGraphicString(GciGetField(reqHdrGciValue, 8));
  m_Principal = "oha";
  
}


IIOPRequestMessage_1_1::~IIOPRequestMessage_1_1()
{
  
}


CORBA::ULong
IIOPRequestMessage_1_1::marshall()
{


  IIOPMessageHeader::marshall();
 
  m_cdrCoder << m_ServiceContext;
  m_cdrCoder << m_RequestId;
  m_cdrCoder << m_ResponseExpected;
  m_cdrCoder.marshallOctet(m_Reserved1);
  m_cdrCoder.marshallOctet(m_Reserved2);
  m_cdrCoder.marshallOctet(m_Reserved3);
  m_cdrCoder << m_ObjectKey;
  m_cdrCoder << m_Operation;
  m_cdrCoder << m_Principal ;
  return 0;
}


int
IIOPRequestMessage_1_1::unmarshall()
{
    
    if (IIOPMessageHeader::unmarshall() == -1)
	return -1;

    m_cdrCoder >> m_ServiceContext;
    m_cdrCoder >> m_RequestId;
    m_cdrCoder >> m_ResponseExpected;
    m_cdrCoder.unmarshallOctet(m_Reserved1);
    m_cdrCoder.unmarshallOctet(m_Reserved2);
    m_cdrCoder.unmarshallOctet(m_Reserved3); 
    CORBA::ULong seqLength = 0;
    m_cdrCoder >> seqLength;
    m_ObjectKey.grow(seqLength);
    m_ObjectKey.length(seqLength);
    m_cdrCoder >> m_ObjectKey;
    m_cdrCoder >> m_Operation;
    return 0;
}


IIOPRequestMessage_1_2::IIOPRequestMessage_1_2(GciValue * Obj) :
    IIOPMessageHeader(Obj)
  
{
  GciValue * reqHdrGciValue;
  reqHdrGciValue = GciGetField(Obj, 2);
   
  m_RequestId = GciGetINTEGER(GciGetField(reqHdrGciValue, 1));
  m_ResponseFlags =atol(GciGetOCTET_STRING(GciGetField(reqHdrGciValue, 2))); 
  m_Reserved1=  atol(GciGetOCTET_STRING(GciGetField(reqHdrGciValue, 3)));
  m_Reserved2 = atol(GciGetOCTET_STRING(GciGetField(reqHdrGciValue, 4)));
  m_Reserved3 = atol(GciGetOCTET_STRING(GciGetField(reqHdrGciValue, 5)));
  m_AddressingDis = 0;
  m_ObjectKey = getObjectKey();
  m_Operation = GciGetGraphicString(GciGetField(reqHdrGciValue, 7));
  m_ServiceContext = 0;
  
}


IIOPRequestMessage_1_2::~IIOPRequestMessage_1_2()
{
 
}


CORBA::ULong
IIOPRequestMessage_1_2::marshall()
{ 
  
  IIOPMessageHeader::marshall();

   m_cdrCoder << m_RequestId;
  m_cdrCoder.marshallOctet(m_ResponseFlags);
  m_cdrCoder.marshallOctet(m_Reserved1);
  m_cdrCoder.marshallOctet(m_Reserved2);
  m_cdrCoder.marshallOctet(m_Reserved3);
  m_cdrCoder << m_AddressingDis;
  m_cdrCoder << m_ObjectKey;
  m_cdrCoder << m_Operation;
  m_cdrCoder << m_ServiceContext;
  
  return 0;
}


int
IIOPRequestMessage_1_2::unmarshall()
{
    
    if (IIOPMessageHeader::unmarshall() == -1)
	return -1;

    m_cdrCoder >> m_RequestId;
    m_cdrCoder.unmarshallOctet(m_ResponseFlags);
    m_cdrCoder.unmarshallOctet(m_Reserved1);
    m_cdrCoder.unmarshallOctet(m_Reserved2);
    m_cdrCoder.unmarshallOctet(m_Reserved3); 
    m_cdrCoder >> m_AddressingDis;
    CORBA::ULong seqLength = 0;
    m_cdrCoder >> seqLength;
    m_ObjectKey.grow(seqLength);
    m_ObjectKey.length(seqLength);
    m_cdrCoder >> m_ObjectKey;
    m_cdrCoder >> m_Operation;
    m_cdrCoder >> m_ServiceContext;
    return 0;
}



					      		   
IIOPReplyMessage::IIOPReplyMessage( unsigned char * buffer,
				   CORBA::ULong length) :
    IIOPMessageHeader(buffer, length),
    m_ServiceContext(0),
    m_RequestId(0),
    m_ReplyStatus((ReplyStatusType)0)
    
{
}

IIOPReplyMessage::~IIOPReplyMessage()
{
}


CORBA::ULong
IIOPReplyMessage::marshall()
{
    // Build the basic header structure
    IIOPMessageHeader::marshall();
    // Marshall service context data
    m_cdrCoder << m_ServiceContext;
    // Marshall the request id and reply type
    m_cdrCoder << m_RequestId;    
    m_cdrCoder << (CORBA::ULong)m_ReplyStatus;
    return 0;
}

int
IIOPReplyMessage::unmarshall()
{
  // Build the basic header structure
   if (IIOPMessageHeader::unmarshall() == -1)
  return -1;
   // Unmarshall service context data
    m_cdrCoder >> m_ServiceContext;
    // Unmarshall the request id and reply type
    CORBA::ULong tmpStatus = 0;
    m_cdrCoder >> m_RequestId;
    m_cdrCoder >> tmpStatus;
    m_ReplyStatus = (ReplyStatusType)tmpStatus;
    return 0;
}


IIOPReplyMessage_1_1::IIOPReplyMessage_1_1( unsigned char * buffer,
				   CORBA::ULong length) :
    IIOPMessageHeader(buffer, length),
    m_ServiceContext(0),
    m_RequestId(0),
    m_ReplyStatus((ReplyStatusType)0)
{
}

IIOPReplyMessage_1_1::~IIOPReplyMessage_1_1()
{
}


CORBA::ULong
IIOPReplyMessage_1_1::marshall()
{
   
    IIOPMessageHeader::marshall();
    m_cdrCoder << m_ServiceContext;
    m_cdrCoder << m_RequestId;    
    m_cdrCoder << (CORBA::ULong)m_ReplyStatus;
    return 0;
}

int
IIOPReplyMessage_1_1::unmarshall()
{
  
   if (IIOPMessageHeader::unmarshall() == -1)
  return -1;
    
 
    m_cdrCoder >> m_ServiceContext;
    CORBA::ULong tmpStatus = 0;
    m_cdrCoder >> m_RequestId;
    m_cdrCoder >> tmpStatus;
    m_ReplyStatus = (ReplyStatusType)tmpStatus;
    return 0;
}


IIOPReplyMessage_1_2::IIOPReplyMessage_1_2( unsigned char * buffer,
				   CORBA::ULong length) :
    IIOPMessageHeader(buffer, length),
   
    m_RequestId(0),
    m_ReplyStatus((ReplyStatusType)0),
    m_ServiceContext(0)
{
}

IIOPReplyMessage_1_2::~IIOPReplyMessage_1_2()
{
}


CORBA::ULong
IIOPReplyMessage_1_2::marshall()
{
    IIOPMessageHeader::marshall();
    m_cdrCoder << m_RequestId;    
    m_cdrCoder << (CORBA::ULong)m_ReplyStatus;
    m_cdrCoder << m_ServiceContext;
    return 0;
}

int
IIOPReplyMessage_1_2::unmarshall()
{
  
   if (IIOPMessageHeader::unmarshall() == -1)
  return -1;
    
 
  CORBA::ULong tmpStatus = 0;
  m_cdrCoder >> m_RequestId;
  m_cdrCoder >> tmpStatus;
  m_ReplyStatus = (ReplyStatusType)tmpStatus;
  m_cdrCoder >> m_ServiceContext;
  return 0;
}

		  
IIOPLocateRequestMessage::IIOPLocateRequestMessage(GciValue * Obj) :
  IIOPMessageHeader(Obj)
  
    
{
  
  GciValue * msgHdrGciValue;
  msgHdrGciValue = GciGetField(Obj, 2); 

  m_RequestId = GciGetINTEGER(GciGetField(msgHdrGciValue, 1));

  m_ObjectKey = getObjectKey();
}

IIOPLocateRequestMessage::~IIOPLocateRequestMessage()
{

 
}

CORBA::ULong
IIOPLocateRequestMessage::marshall()
{
  
  
    // Build the basic header structure
    IIOPMessageHeader::marshall();

    // Marshall request id
    m_cdrCoder << m_RequestId;
     
    // Marshall the target objectKey 
    m_cdrCoder << m_ObjectKey;
   
    return 0;
}

int
IIOPLocateRequestMessage::unmarshall()
{
    // Build the basic header structure
    if (IIOPMessageHeader::unmarshall() == -1)
	return -1;
    
    // Unmarshall request id
    m_cdrCoder >> m_RequestId;

    // Unmarshall the objectKey 
    CORBA::ULong seqLength = 0;
    
    m_cdrCoder >> seqLength;
    m_ObjectKey.grow(seqLength);
    m_ObjectKey.length(seqLength);
    
    m_cdrCoder >> m_ObjectKey;
    return 0;
}



IIOPLocateRequestMessage_1_2::IIOPLocateRequestMessage_1_2(GciValue * Obj) :
  IIOPMessageHeader(Obj)
  
    
{
  
  GciValue * msgHdrGciValue;
  msgHdrGciValue = GciGetField(Obj, 2); 

  m_RequestId = GciGetINTEGER(GciGetField(msgHdrGciValue, 1));
  m_AddressingDis = 0;
  m_ObjectKey = getObjectKey();
}

IIOPLocateRequestMessage_1_2::~IIOPLocateRequestMessage_1_2()
{

 
}

CORBA::ULong
IIOPLocateRequestMessage_1_2::marshall()
{
  
  
    // Build the basic header structure
    IIOPMessageHeader::marshall();

    // Marshall request id
    m_cdrCoder << m_RequestId;
     m_cdrCoder <<  m_AddressingDis ;
  
    // Marshall the target objectKey 
    m_cdrCoder << m_ObjectKey;
  
    return 0;
}

int
IIOPLocateRequestMessage_1_2::unmarshall()
{
    // Build the basic header structure
    if (IIOPMessageHeader::unmarshall() == -1)
	return -1;
    
    // Unmarshall request id
    m_cdrCoder >> m_RequestId;

    // Unmarshall the objectKey 
    CORBA::ULong seqLength = 0;
    
    m_cdrCoder >> seqLength;
    m_ObjectKey.grow(seqLength);
    m_ObjectKey.length(seqLength);
    
    m_cdrCoder >> m_ObjectKey;
    return 0;
}



IIOPLocateReplyMessage::IIOPLocateReplyMessage(unsigned char * buffer,
				       CORBA::ULong length) :
    IIOPMessageHeader( buffer, length),
    m_RequestId(0),
    m_LocateStatus((LocateStatusType)0)
{
}


IIOPLocateReplyMessage::~IIOPLocateReplyMessage()
{
}

CORBA::ULong
IIOPLocateReplyMessage::marshall()
{
    // Build the basic header structure
    IIOPMessageHeader::marshall();

    // Marshall the request id and reply type
    m_cdrCoder << m_RequestId;
    m_cdrCoder << (CORBA::ULong)m_LocateStatus;
   
   
    return 0;
}

int
IIOPLocateReplyMessage::unmarshall()
{
  //  Build the basic header structure
   if (IIOPMessageHeader::unmarshall() == -1)
  return -1;
    
    // Unmarshall the request id and reply type
    CORBA::ULong tmpStatus = 0;

    m_cdrCoder >> m_RequestId;
    m_cdrCoder >> tmpStatus;
    m_LocateStatus = (LocateStatusType)tmpStatus;
    
    return 0;
}





IIOPCancelRequestMessage::IIOPCancelRequestMessage(GciValue * Obj) :
  IIOPMessageHeader(Obj)
 
{
  GciValue * msgHdrGciValue;
  msgHdrGciValue = GciGetField(Obj, 2); 

  m_RequestId = GciGetINTEGER(GciGetField(msgHdrGciValue, 1));

}

IIOPCancelRequestMessage::~IIOPCancelRequestMessage()
{
 
}

CORBA::ULong
IIOPCancelRequestMessage::marshall()
{
    // Build the basic header structure
    IIOPMessageHeader::marshall();

    // Marshall request id
    m_cdrCoder << m_RequestId;
   
    return 0;
}

int
IIOPCancelRequestMessage::unmarshall()
{
  return 0;
}


IIOPFragmentMessage::IIOPFragmentMessage(GciValue * Obj) :
  IIOPMessageHeader(Obj)
   
{

  GciValue * msgHdrGciValue;
  msgHdrGciValue = GciGetField(Obj, 2); 

  m_RequestId = GciGetINTEGER(GciGetField(msgHdrGciValue, 1));

}

IIOPFragmentMessage::~IIOPFragmentMessage()
{
}

CORBA::ULong
IIOPFragmentMessage::marshall()
{
    // Build the basic header structure
    IIOPMessageHeader::marshall();

    // Marshall request id
    m_cdrCoder << m_RequestId;
    return 0;
}

int
IIOPFragmentMessage::unmarshall()
{
  if (IIOPMessageHeader::unmarshall() == -1)
    return -1;
  m_cdrCoder  >> m_RequestId;
  return 0;
}

	   
IOR::IOR () :
    m_TypeId(0),
    m_Coder(2048)
{
    m_InternetProfile = new ProfileBody;
}

IOR::IOR (CORBA::Char *stringForm)
{
    m_TypeId = 0;
    m_SeqProfiles = 0;
    m_InternetProfile = new ProfileBody;

    fromString(stringForm);
}

IOR::~IOR() 
{
    if (m_InternetProfile)
	delete m_InternetProfile;

    m_InternetProfile = 0;

    if (m_TypeId)
	delete m_TypeId;

    m_TypeId = 0;
}




IOR *
IOR::fromString(CORBA::Char * stringForm)
{
    
    if (stringForm != 0 &&
	stringForm[0] == 'I' &&
	stringForm[1] == 'O' &&
	stringForm[2] == 'R' &&
	stringForm[3] == ':')
    {
	// stringified IORs are hexified
	// We need to unhex it before we can unmarshall

	Encapsulation * encap = 0;

	encap = unHexify(&stringForm[4]);

	// Initialise the coder with the unhex'd stream
	m_Coder.initialise(encap->buffer(), encap->length());

	CORBA::Boolean gender;

	// read and set the endian flag
	m_Coder >> gender;
	m_Coder.StreamEndian(gender);

	// initialise the typeId
	m_Coder >> m_TypeId;

	CORBA::ULong seqLength = 0;

	// How many sequence elements?
	m_Coder >> seqLength;

	//cout << "IOR contains " << seqLength << " outer profiles" << endl;
	
	// Alloc and initialise the sequence
	m_SeqProfiles.grow(seqLength);
	m_SeqProfiles.length(seqLength);

	// For each element :
	//
	// read the profileId
	// read its data portion
	for (CORBA::ULong i = 0; i < seqLength; i++)
	{
	    CORBA::ULong tmp = 0;
	    
	    m_Coder >> tmp;

	    //cout << "ProfileId: " << tmp << endl;
				
	    m_SeqProfiles[i].profile_id = tmp;

	    CORBA::ULong tmpOctLength = 0;
	    
	    m_Coder >> tmpOctLength;
	    vsOrbSeqOctet tmpSeqOctet(tmpOctLength);
	    tmpSeqOctet.length(tmpOctLength);

	    m_Coder >> tmpSeqOctet;
	    
	    m_SeqProfiles[i].profile_data = tmpSeqOctet;
	}

	if (encap)
	    delete encap;
    }
    return 0;
}
			   

CORBA::Char*
IOR::toString() {

    m_Coder.setEndian();

    m_Coder << m_Coder.OurEndian();
    
    m_Coder << m_TypeId;

    // Assume only a TAG_INTERNET_IOP profile for the moment
    CORBA::ULong seqLen = 1;
    m_Coder << seqLen;

    m_Coder << (CORBA::ULong )TAG_INTERNET_IOP;

    cdrCoder tmpCoder(512);

    if (m_Coder.m_Flipped)
	tmpCoder.flipEnds();
    
    tmpCoder.setEndian();
    
    tmpCoder << *m_InternetProfile;

    vsOrbSeqOctet seqOct(tmpCoder.bufLen(),
			 tmpCoder.bufLen(),
			 tmpCoder.stream());
    
    
    m_Coder << seqOct;

    Encapsulation * encap = new Encapsulation(m_Coder.stream(),
					      m_Coder.bufLen());
	
    unsigned char *buf = hexify(encap);
    
    CORBA::Char * stringForm = new char [ m_Coder.bufLen() * 2 + 5];
    strcpy (stringForm, "IOR:");
    strcat (stringForm, (char *)buf);
    delete buf;
    
    delete encap;

    return stringForm;
};



ProfileBody*
IOR::extractInternetTag()
{
    // Build up the actual profile body structures
    
    for (CORBA::ULong i = 0; i < m_SeqProfiles.length(); i++)
    {
	if (m_SeqProfiles[i].profile_id == TAG_INTERNET_IOP)
	{
	    // get a new octet seq
	    vsOrbSeqOctet tmpSeq(m_SeqProfiles[i].data());

	    cdrCoder tmpCoder;

	    // set that up
	    tmpCoder.initialise(tmpSeq.data(), tmpSeq.length());

	    CORBA::Octet major;
	    CORBA::Octet minor;
	    CORBA::Boolean actualEnd;

	    // stream off the actual endian flag for the encapsulation
	    // and posibbly flip the coders setting 
	    tmpCoder >> actualEnd;
	    tmpCoder.StreamEndian(actualEnd);

	    // stream off the version block
	    //tmpCoder >> major;
	    //tmpCoder >> minor;
            tmpCoder.unmarshallOctet(major);
            tmpCoder.unmarshallOctet(minor);
            
	    m_InternetProfile->ver().major(major);
	    m_InternetProfile->ver().minor(minor);

	    CORBA::Char * tmpHost = 0;
	    CORBA::UShort tmpPort = 0;

	    tmpCoder >> tmpHost;

	    m_InternetProfile->host(tmpHost);
	    delete tmpHost;

	    tmpCoder >> tmpPort;
	    m_InternetProfile->port(tmpPort);

	    CORBA::ULong tmpOctLength = 0;
	    
	    tmpCoder >> tmpOctLength;

	    vsOrbSeqOctet tmpSeqOctet(tmpOctLength);
	    tmpSeqOctet.length(tmpOctLength);

	    tmpCoder >> tmpSeqOctet;

	    m_InternetProfile->objectKey(tmpSeqOctet);
	}
    }
    return m_InternetProfile;
}

Encapsulation *
IOR::unHexify(char * hexForm)
{
    Encapsulation *e = new Encapsulation(::strlen(hexForm) / 2);
    
    CORBA::ULong i;
    for (i=0; i < e->length() && *hexForm != '\0'; i++)
    {
	unsigned char * ref = e->buffer();
	
	hexToByte(hexForm, ref[i]);
	hexForm +=2;
    }

    if ( i != e->length())
	return 0;

    return e;
}

void
IOR::hexToNibble(char in, unsigned char &out)
{
   out = 0;
   switch (in)
   {
       case '1': case '2': case '3': case '4': case '5':
       case '6': case '7': case '8': case '9': case '0':
            out = in - '0';
            break;
       case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
            out = in - 'A' + 10;
            break;
       case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
            out = in - 'a' + 10;
            break;
       default:
            ;
   }
}

void
IOR::hexToByte(char *s, unsigned char &c)
{
    c = 0;

    if (s == 0)
       return;

    unsigned char tmp;
    hexToNibble(s[0], tmp);
    c |= tmp << 4;
    hexToNibble(s[1], tmp);
    c |= tmp;
}

void
IOR::byteToHex(unsigned char c, char *s)
{
    s[0] = m_hexChars[(c >> 4) & 0x0f];
    s[1] = m_hexChars[c & 0x0f];
}

unsigned char *
IOR::hexify(Encapsulation *e)
{
    if (e == 0)
	return 0;

    CORBA::ULong i;
    CORBA::ULong eLen = e->length();
    
    unsigned char *ptr, *ret = 0;

    ptr = new unsigned char [ eLen * 2 + 1];

    ret = ptr;

    for (i = 0; i < eLen; i++)
    {
	byteToHex( *(e->buffer() + i), (char *)ptr);
	ptr += 2;
    }

    *ptr = '\0';
    return ret;
}


CORBA::Boolean
ProfileBody::operator==(ProfileBody& rhs)
{
    CORBA::Boolean equal = 1;
    

    
    if ( (this->ver().major() != rhs.ver().major())
	|| (this->ver().minor() != rhs.ver().minor()) )
    {
	
		   
	
	equal = 0;
    }

    if (this->port() != rhs.port())
    {
	equal = 0;
    }
    
    return equal;
}

			  
	      
