/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             
 * Project: CORVAL2
 *
 * Contact:
 * Omar Halabi
 * e-mail: oha@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

#ifndef VSORB_IIOP_HH
#define VSORB_IIOP_HH

#include <gci.h>
#include "orb_gw.h"
#ifdef _WIN32
#include <iostream.h>
#include <winsock.h>
#else
#include <stream.h>
#include <string.h>
#include <assert.h>
#include <sys/types.h>
#include <netinet/in.h>
#endif

#define CDR_CHAR_SZ    1
#define CDR_BOOL_SZ    1
#define CDR_OCTET_SZ   1

#define CDR_SHORT_SZ   2
#define CDR_USHORT_SZ  2

#define CDR_LONG_SZ    4
#define CDR_ULONG_SZ   4

#define CDR_LONGLONG_SZ    8
#define CDR_ULONGLONG_SZ   8

#define CDR_FLOAT_SZ   4

#define CDR_DOUBLE_SZ  8

#define TAG_INTERNET_IOP 0

#define TRUE 1
#define FALSE 0

class vsOrbSeqOctet;
class vsOrbString;
class vsOrbSeqServiceContext;
class ProfileBody;
class IOR;

class cdrCoder
{

	friend class IOR;
    
    protected:

    // Big or little endian
    CORBA::Boolean   m_OurEndian;
    CORBA::Boolean   m_StreamEndian;

    // Are we pretending to be the opposite architecture
    CORBA::Boolean   m_Flipped;

    // Marshalled stream
    unsigned char *  m_Stream;

    // Current index into stream
    unsigned char *  m_BufIndx;

    // Allocated and used Stream length
    CORBA::ULong     m_AllocLength;
    CORBA::ULong     m_StreamLength;

    // Align buffer and grow if necessary
    void alignAndGrow(CORBA::ULong);

    
public:

    // usual constructors
    cdrCoder();
    cdrCoder(CORBA::ULong);
    cdrCoder( unsigned char * buffer, CORBA::ULong bufLen);
    ~cdrCoder();


    void initialise(unsigned char * buffer, CORBA::ULong bufLen);

    // Insertion operations for basic types
    cdrCoder &operator<<(const CORBA::Char &c);
    cdrCoder &operator<<(const CORBA::Boolean &b);  
    cdrCoder &operator<<(const CORBA::Short &s);
    cdrCoder &operator<<(const CORBA::UShort &us);
    cdrCoder &operator<<(const CORBA::Long &l);
    cdrCoder &operator<<(const CORBA::ULong &ul);
    cdrCoder &operator<<(const CORBA::Float &f);
    cdrCoder &operator<<(const CORBA::LongLong &ll);
    cdrCoder &operator<<(const CORBA::ULongLong &ull);
    cdrCoder &operator<<(const CORBA::Double &d);
    cdrCoder &operator<<(const char *s);
    cdrCoder &operator<<(vsOrbSeqOctet &s);
    cdrCoder &operator<<(vsOrbSeqServiceContext &seq);
    cdrCoder &operator<<(ProfileBody &p);
    cdrCoder &operator<<(IOR &ior);

    cdrCoder &marshallOctet(const CORBA::Octet &o);
    
    // Extraction operators for basic types
    cdrCoder &operator>>(CORBA::Char &c);
    cdrCoder &operator>>(CORBA::Boolean &b);  
    cdrCoder &operator>>(CORBA::Short &s);
    cdrCoder &operator>>(CORBA::UShort &us);
    cdrCoder &operator>>(CORBA::Long &l);
    cdrCoder &operator>>(CORBA::ULong &ul);
    cdrCoder &operator>>(CORBA::Float &f);
    cdrCoder &operator>>(CORBA::LongLong &ll);
    cdrCoder &operator>>(CORBA::ULongLong &ull);
    cdrCoder &operator>>(CORBA::Double &d);
    cdrCoder &operator>>(char *&s);
    cdrCoder &operator>>(vsOrbString &s);
    cdrCoder &operator>>(vsOrbSeqOctet &s);
    cdrCoder &operator>>(vsOrbSeqServiceContext &seq);
    cdrCoder &operator>>(IOR *&ior);

    // Auxilliary routines
    cdrCoder &unmarshallOctet(CORBA::Octet&o);
    
    // Initialise our endian flag
    //
    // TCP/IP byte ordering is big endian. By checking the
    // result of the htonl routine we can infer whether the
    // runtime architecture is the same as the networks.
    //
    // Intels are little endian and SPARCs etc are big.
    //
    // IIOP specifies a value of false for big endian machines
    // and true for little endian machines. So if (x == y) we are
    // a big endian architecture so the boolean flag in the coder
    // is set to be false.
    void setEndian() {

	CORBA::ULong x = 1234;
	CORBA::ULong y = htonl(x);

	if (x == y)
	    m_OurEndian = 0; 
	else
	    m_OurEndian = 1;
    }
    
    void setEndian(CORBA::Boolean whichEnd) {
	m_OurEndian = whichEnd;
    }
    
	CORBA::Boolean OurEndian() {
	return m_Flipped ? !m_OurEndian : m_OurEndian;
    }

    CORBA::Boolean StreamEndian() {
	return m_StreamEndian;
    }

    void StreamEndian (CORBA::Boolean whichEnd) {
	m_StreamEndian = whichEnd;
    }

    CORBA::Boolean SameEndian() {
	if (m_OurEndian == m_StreamEndian)
	    return 1;
	else
	    return 0;
    }

    unsigned char * stream() {
	return m_Stream;
    }

    CORBA::ULong curPos() {
	return m_BufIndx - m_Stream;
    }

    unsigned char * seek(CORBA::ULong numBytes) {
	return m_Stream+=numBytes/4;
    }

    void rewind(CORBA::ULong numBytes) {
	    m_BufIndx-=numBytes;
    }

    CORBA::ULong bufLen() {
	return m_BufIndx - m_Stream;
    }

    CORBA::ULong bufLeft() {
	return m_StreamLength-(m_BufIndx - m_Stream);
    }

    CORBA::ULong StreamLength() {
	return m_StreamLength;
    }

    void StreamLength(CORBA::ULong length) {
	m_BufIndx = m_Stream;
	m_BufIndx = m_BufIndx + 8;

	CORBA::ULong actualLen = length - 12;
	
	*this << actualLen;
    }

    void flipEnds() {
	m_Flipped = TRUE;
    }
};

class vsOrbSeqOctet
{
    CORBA::ULong m_Maximum;
    CORBA::ULong m_Length;
    CORBA::Octet* m_Data;
    unsigned char m_ReleaseFlag;

    public:
    vsOrbSeqOctet& operator= (const vsOrbSeqOctet&);
    vsOrbSeqOctet (const vsOrbSeqOctet&);

    vsOrbSeqOctet (CORBA::ULong max);
    vsOrbSeqOctet (CORBA::ULong max,
		   CORBA::ULong length,
		   CORBA::Octet* data,
		   CORBA::Boolean release = 0);
    vsOrbSeqOctet ();

    ~vsOrbSeqOctet ();

    static CORBA::Octet* allocbuf(CORBA::ULong numElems);
    static void freebuf(CORBA::Octet* data);

    CORBA::ULong maximum() const;

    CORBA::ULong length() const;

    void grow (CORBA::ULong max);
    void length (CORBA::ULong len);

    CORBA::Octet* data() {
	return m_Data;
    }
    
    CORBA::Octet& operator [] (CORBA::ULong index);
    const CORBA::Octet& operator [] (CORBA::ULong index) const;
};

typedef CORBA::ULong iiop_ServiceId;
typedef CORBA::ULong ComponentId;


class ServiceContext {
    
    iiop_ServiceId m_ContextId;
    vsOrbSeqOctet m_ContextData;

    public:
    ServiceContext(const ServiceContext &);
    ServiceContext();
    
    ~ServiceContext();
    ServiceContext& operator= (const ServiceContext&);

    iiop_ServiceId contextId() const {
	return m_ContextId;
    }

    void contextId(iiop_ServiceId id) {
	m_ContextId = id;
    }
    
    vsOrbSeqOctet& contextData() {
	return m_ContextData;
    }

    void contextData(vsOrbSeqOctet *seqOct) {
	m_ContextData = *seqOct;
    }

  

    
};

typedef ServiceContext* ServiceContext_ptr;

class vsOrbSeqServiceContext
{
public:
    CORBA::ULong m_Maximum;
    CORBA::ULong m_Length;
    ServiceContext* m_Data;
    unsigned char m_ReleaseFlag;

    public:
    vsOrbSeqServiceContext& operator= (const vsOrbSeqServiceContext&);
    vsOrbSeqServiceContext (const vsOrbSeqServiceContext&);

    vsOrbSeqServiceContext (CORBA::ULong max);
    vsOrbSeqServiceContext (CORBA::ULong max,
			    CORBA::ULong length,
			    ServiceContext* data,
			    CORBA::Boolean release = 0);
    vsOrbSeqServiceContext ();

    ~vsOrbSeqServiceContext ();

    static ServiceContext* allocbuf(CORBA::ULong numElems);
    static void freebuf(ServiceContext* data);
    
    CORBA::ULong maximum() const;
    CORBA::ULong length() const;
    void grow (CORBA::ULong len);
    void length (CORBA::ULong len);
    
    ServiceContext& operator [] (CORBA::ULong index);
    const ServiceContext& operator [] (CORBA::ULong index) const; 
};

typedef vsOrbSeqServiceContext ServiceContextList;

class TaggedProfile
{

	friend class IOR;
    
    CORBA::ULong profile_id;
    vsOrbSeqOctet profile_data;

    public:
    TaggedProfile(const TaggedProfile &);
    TaggedProfile();
    ~TaggedProfile();
    TaggedProfile& operator= (const TaggedProfile&);

    CORBA::ULong id() {
	return profile_id;
    }

    void id(CORBA::ULong id) {
	profile_id = id;
    }

    vsOrbSeqOctet& data() {
	return profile_data;
    }
    
    void data(vsOrbSeqOctet *seqOct) {
	profile_data = *seqOct;
    }
};

typedef TaggedProfile* TaggedProfile_ptr;

class vsOrbSeqTaggedProfile
{

	friend class IOR;

    CORBA::ULong m_Maximum;
    CORBA::ULong m_Length;
    TaggedProfile* m_Data;
    unsigned char m_ReleaseFlag;
    
    public:
    vsOrbSeqTaggedProfile& operator= (const vsOrbSeqTaggedProfile&);
    vsOrbSeqTaggedProfile (const vsOrbSeqTaggedProfile&);

    vsOrbSeqTaggedProfile (CORBA::ULong max);
    vsOrbSeqTaggedProfile (CORBA::ULong max,
			   CORBA::ULong length,
			   TaggedProfile* data,
			   CORBA::Boolean release = 0);
    vsOrbSeqTaggedProfile ();
    
    ~vsOrbSeqTaggedProfile ();
    
    void grow (CORBA::ULong max);
    static TaggedProfile* allocbuf(CORBA::ULong numElems);
    static void freebuf(TaggedProfile* data);
    
    CORBA::ULong maximum() const;
    CORBA::ULong length() const;
    void length (CORBA::ULong len);

    TaggedProfile& operator [] (CORBA::ULong index);
    const TaggedProfile& operator [] (CORBA::ULong index) const;
};


typedef vsOrbSeqTaggedProfile* vsOrbSeqTaggedProfile_ptr;


class TaggedComponent
{
    ComponentId component_id;
    vsOrbSeqOctet component_data;

    public:
    TaggedComponent(const TaggedComponent &);
    TaggedComponent();
    ~TaggedComponent();
    TaggedComponent& operator= (const TaggedComponent&);
};

typedef TaggedComponent* TaggedComponent_ptr;

class vsOrbSeqTaggedComponent
{
    CORBA::ULong m_Maximum;
    CORBA::ULong m_Length;
    TaggedComponent* m_Data;
    unsigned char m_ReleaseFlag;
    
    public:
    vsOrbSeqTaggedComponent& operator= (const vsOrbSeqTaggedComponent&);
    vsOrbSeqTaggedComponent (const vsOrbSeqTaggedComponent&);

    vsOrbSeqTaggedComponent (CORBA::ULong max);
    vsOrbSeqTaggedComponent (CORBA::ULong max,
			     CORBA::ULong length,
			     TaggedComponent* data,
			     CORBA::Boolean release = 0);
    vsOrbSeqTaggedComponent ();
    
    ~vsOrbSeqTaggedComponent ();
    
    static TaggedComponent* allocbuf(CORBA::ULong numElems);
    static void freebuf(TaggedComponent* data);
    
    CORBA::ULong maximum() const;
    CORBA::ULong length() const;
    void length (CORBA::ULong len);

    TaggedComponent& operator [] (CORBA::ULong index);
    const TaggedComponent& operator [] (CORBA::ULong index) const;
};


typedef vsOrbSeqTaggedComponent* vsOrbSeqTaggedComponent_ptr;

typedef CORBA::Char  MagicCookie[4];

class Version {

public:
    
    CORBA::Octet  m_Major;
    CORBA::Octet  m_Minor;

    Version() : m_Major(1), m_Minor(0) {
    }
    
    Version(const Version &v) {
	m_Major = v.m_Major;
	m_Minor = v.m_Minor;
    }
    
    ~Version() {}

    CORBA::Octet major() {
	return m_Major;
    }

    void major(CORBA::Octet o) {
	m_Major = o;
    }
    
    CORBA::Octet minor() {
	return m_Minor;
    }

    void minor(CORBA::Octet o) {
	m_Minor = o;
    }
};


vsOrbSeqOctet& getObjectKey();

enum IIOPMsgType {
    Request,
    Reply,
    CancelRequest,
    LocateRequest,
    LocateReply,
    CloseConnection,
    MessageError,
    Fragment,
    REQUEST
};

enum ReplyStatusType {
    NO_EXCEPTION,
    USER_EXCEPTION,
    SYSTEM_EXCEPTION,
    
  // Added in GIOP1.2
    LOCATION_FORWARD,
    LOCATION_FORWARD_PERM,
    NEEDS_ADDRESSING_MODE
};


enum LocateStatusType {
    UNKNOWN_OBJECT,
    OBJECT_HERE,
    OBJECT_FORWARD,
    
 // Added in GIOP1.2
    OBJECT_FORWARD_PERM,
    LOC_SYSTEM_EXCEPTION,
    LOC_NEEDS_ADDRESSING_MODE


};


class IIOPMessageHeader
{
    
public:
  MagicCookie      m_Cookie;
  Version          m_Ver;
  CORBA::Boolean     m_ByteSex;
  CORBA::Octet      m_Flags;
  CORBA::Octet     m_MsgType;
  CORBA::ULong     m_Size;
  
  cdrCoder         m_cdrCoder;
    
public:
  IIOPMessageHeader ( unsigned char * buffer, CORBA::ULong length);
  IIOPMessageHeader (GciValue * Obj);
  ~IIOPMessageHeader ();

    void ver (const Version& ver)
    {
	m_Ver.m_Major = ver.m_Major;
	m_Ver.m_Minor = ver.m_Minor;
    }
    
    const Version& ver ()
    {
	return m_Ver;
    }
  
    void byte_order (CORBA::Boolean byte_order)
    {
	m_ByteSex = byte_order;
	m_cdrCoder.StreamEndian(byte_order);
    }
    
    CORBA::Boolean byte_order ()
    {
	return m_ByteSex;
    }
  void flags_type (CORBA::Octet flags_type)
    {
	m_Flags = flags_type;
    }
    
    CORBA::Octet flags_type ()
    {
	return m_Flags;
    }  
  
   void message_type (CORBA::Octet message_type)
    {
	m_MsgType = message_type;
    }
    
    CORBA::Octet message_type ()
    {
	return m_MsgType;
    }
    
    void message_size (CORBA::ULong message_size)
    {
	m_Size = message_size;
    }
    
    CORBA::ULong message_size ()
    {
	return m_Size;
    }
  
    virtual CORBA::ULong marshall();

   virtual int unmarshall();

    IIOPMessageHeader &operator<<(const CORBA::Char &c) {
	m_cdrCoder << c;
	return *this;
    }
    
    IIOPMessageHeader &operator<<(const CORBA::Boolean &b) {  
	m_cdrCoder << b;
	return *this;
    }
    
    IIOPMessageHeader &operator<<(const CORBA::Short &s) {
	m_cdrCoder << s;
	return *this;
    }
    
    IIOPMessageHeader &operator<<(const CORBA::UShort &us) {
	m_cdrCoder << us;
	return *this;
    }

    IIOPMessageHeader &operator<<(const CORBA::Long &l) {
	m_cdrCoder << l;
	return *this;
    }

    IIOPMessageHeader &operator<<(const CORBA::ULong &ul) {
	m_cdrCoder << ul;
	return *this;
    }
    
    IIOPMessageHeader &operator<<(const CORBA::Float &f) {
	m_cdrCoder << f;
	return *this;
    }
  
  IIOPMessageHeader &operator<<(const CORBA::LongLong &ll) {
	m_cdrCoder << ll;
	return *this;
    }
    
IIOPMessageHeader &operator<<(const CORBA::ULongLong &ull) {
	m_cdrCoder << ull;
	return *this;
    }


 IIOPMessageHeader &operator<<(const CORBA::Double &d) {
	m_cdrCoder << d;
	return *this;
    }

    
    
    
    IIOPMessageHeader &operator<<(const char *s) {
	m_cdrCoder << s;
	return *this;
    }

  IIOPMessageHeader &operator<<(vsOrbSeqOctet &s) {
      m_cdrCoder << s;
	return *this;
    }

  
  

    IIOPMessageHeader &operator<<(IOR &ior) {
	m_cdrCoder << ior;
	return *this;
    }

    IIOPMessageHeader &operator>>(CORBA::Char &c) {
	m_cdrCoder >> c;
	return *this;
    }
    
    IIOPMessageHeader &operator>>(CORBA::Boolean &b) {  
	m_cdrCoder >> b;
	return *this;
    }
    
    IIOPMessageHeader &operator>>(CORBA::Short &s) {
	m_cdrCoder >> s;
	return *this;
    }
    
    IIOPMessageHeader &operator>>(CORBA::UShort &us) {
	m_cdrCoder >> us;
	return *this;
    }

    IIOPMessageHeader &operator>>(CORBA::Long &l) {
	m_cdrCoder >> l;
	return *this;
    }

    IIOPMessageHeader &operator>>(CORBA::ULong &ul) {
	m_cdrCoder >> ul;
	return *this;
    }
    
    IIOPMessageHeader &operator>>(CORBA::Float &f) {
	m_cdrCoder >> f;
	return *this;
    }

  IIOPMessageHeader &operator>>(CORBA::LongLong &ll) {
	m_cdrCoder >> ll;
	return *this;
    }
  IIOPMessageHeader &operator>>(CORBA::ULongLong &ull) {
	m_cdrCoder >> ull;
	return *this;
    }
    

    IIOPMessageHeader &operator>>(CORBA::Double &d) {
	m_cdrCoder >> d;
	return *this;
    }
    
    IIOPMessageHeader &operator>>(char *&s) {
	m_cdrCoder >> s;
	return *this;
    }

    IIOPMessageHeader &operator>>(vsOrbString &s) {
	m_cdrCoder >> s;
	return *this;
    }

    IIOPMessageHeader &operator>>(vsOrbSeqOctet &s) {
	m_cdrCoder >> s;
	return *this;
    }

    IIOPMessageHeader &operator>>(IOR *&ior) {
	m_cdrCoder >> ior;
	return *this;
    }

   unsigned char * getMarshalledBuffer() {
	return m_cdrCoder.stream();
    }

    CORBA::ULong getMarshalledBufferLen() {
	return m_cdrCoder.bufLen();
    }

    void updateStreamLength(CORBA::ULong length) {
	m_cdrCoder.StreamLength(length);
    }
    
    void flipEnds()
    {
	m_ByteSex = m_ByteSex ? FALSE : TRUE;
	m_cdrCoder.flipEnds();
    }
    
    CORBA::ULong curPos() {
	return m_cdrCoder.curPos();
    }

    unsigned char * seek(CORBA::ULong numBytes) {
	return m_cdrCoder.seek(numBytes);
    }

    void rewind(CORBA::ULong numBytes) {
	    m_cdrCoder.rewind(numBytes);
    }

  unsigned  char * head() {
	return m_cdrCoder.stream();
    }
    
    CORBA::Boolean StreamEndian() {
	return m_cdrCoder.StreamEndian();
    }

    void StreamEndian (CORBA::Boolean whichEnd) {
	m_cdrCoder.StreamEndian(whichEnd);
    }

};



    





class IIOPRequestMessage: public virtual IIOPMessageHeader
{

  public:
  ServiceContextList   m_ServiceContext; 
  CORBA::ULong         m_RequestId; 
  CORBA::Boolean       m_ResponseExpected; 
  vsOrbSeqOctet        m_ObjectKey;
  char              *  m_Operation; 
  char              *  m_Principal; 

    public:
    IIOPRequestMessage(GciValue * Obj);
  
    ~IIOPRequestMessage();
    

    CORBA::ULong marshall();

   int unmarshall();
 
};

class IIOPRequestMessage_1_1 : public virtual IIOPMessageHeader
{

 public:
  ServiceContextList   m_ServiceContext; 
  CORBA::ULong         m_RequestId; 
  CORBA::Boolean       m_ResponseExpected; 
  CORBA::Octet         m_Reserved1;
  CORBA::Octet         m_Reserved2;
  CORBA::Octet         m_Reserved3;
  vsOrbSeqOctet        m_ObjectKey;
  char              *  m_Operation; 
  char              *  m_Principal; 

public:
  IIOPRequestMessage_1_1(GciValue * Obj);
  
  ~IIOPRequestMessage_1_1();
  CORBA::ULong marshall();
  int unmarshall();
 
};

class IIOPRequestMessage_1_2 : public virtual IIOPMessageHeader
{

 public:
  
  CORBA::ULong         m_RequestId; 
  CORBA::Octet         m_ResponseFlags; 
  CORBA::Octet         m_Reserved1;
  CORBA::Octet         m_Reserved2;
  CORBA::Octet         m_Reserved3;
  CORBA::Short         m_AddressingDis;
  vsOrbSeqOctet        m_ObjectKey;
  char              *  m_Operation; 
   ServiceContextList   m_ServiceContext; 
  
public:
  IIOPRequestMessage_1_2(GciValue * Obj);
  
  ~IIOPRequestMessage_1_2();
  CORBA::ULong marshall();
  int unmarshall();
 
};





class IIOPReplyMessage: public virtual IIOPMessageHeader
{

public:
  ServiceContextList   m_ServiceContext; 
  CORBA::ULong         m_RequestId; 
  ReplyStatusType      m_ReplyStatus;

  
 public:
  IIOPReplyMessage (unsigned char * buffer, CORBA::ULong length);
  IIOPReplyMessage ();
  ~IIOPReplyMessage ();

 
    CORBA::ULong marshall();

  int unmarshall();
  

};

class IIOPReplyMessage_1_1: public virtual IIOPMessageHeader
{

public:
  ServiceContextList   m_ServiceContext; 
  CORBA::ULong         m_RequestId; 
  ReplyStatusType      m_ReplyStatus;

  

 public:
  IIOPReplyMessage_1_1 (unsigned char * buffer, CORBA::ULong length);
  IIOPReplyMessage_1_1 ();
  ~IIOPReplyMessage_1_1 ();

  
    CORBA::ULong marshall();

  int unmarshall();
  

};

class IIOPReplyMessage_1_2: public virtual IIOPMessageHeader
{

public:
  
  CORBA::ULong         m_RequestId; 
  ReplyStatusType      m_ReplyStatus;
  ServiceContextList   m_ServiceContext; 
  

 public:
  IIOPReplyMessage_1_2 (unsigned char * buffer, CORBA::ULong length);
  IIOPReplyMessage_1_2 ();
  ~IIOPReplyMessage_1_2 ();

 
    CORBA::ULong marshall();

  int unmarshall();
  

};




class IIOPCancelRequestMessage: public virtual IIOPMessageHeader
{
    
    public:
    CORBA::ULong  m_RequestId;
    
    public:
    IIOPCancelRequestMessage (GciValue * Obj);
    ~IIOPCancelRequestMessage ();

    void request_id (CORBA::ULong request_id)
    {
	m_RequestId = request_id;
    }
    
    CORBA::ULong request_id ()
    {
	return m_RequestId;
    }
    
  CORBA::ULong marshall();

  int unmarshall();
 

};




class IIOPLocateRequestMessage: public virtual IIOPMessageHeader
{
    
public:
  CORBA::ULong    m_RequestId;
  vsOrbSeqOctet   m_ObjectKey; 
  
  
public:
  IIOPLocateRequestMessage (GciValue * Obj);
  ~IIOPLocateRequestMessage ();

  void request_id (CORBA::ULong request_id)
    {
      m_RequestId = request_id;
    }
    
  CORBA::ULong request_id ()
    {
      return m_RequestId;
    }
    
    
  void object_key (const vsOrbSeqOctet& object_key)
    {
      m_ObjectKey = object_key;
    }
  
  vsOrbSeqOctet& object_key ()
    {
      return m_ObjectKey;
    }
   
    
  CORBA::ULong marshall();

  int unmarshall();

 
};


class IIOPLocateRequestMessage_1_2: public virtual IIOPMessageHeader
{
    
public:
  CORBA::ULong    m_RequestId;
  CORBA::Short    m_AddressingDis; 
  vsOrbSeqOctet   m_ObjectKey; 

public:
  IIOPLocateRequestMessage_1_2 (GciValue * Obj);
  ~IIOPLocateRequestMessage_1_2 ();

  void request_id (CORBA::ULong request_id)
    {
      m_RequestId = request_id;
    }
    
  CORBA::ULong request_id ()
    {
      return m_RequestId;
    }
    
    
  void object_key (const vsOrbSeqOctet& object_key)
    {
      m_ObjectKey = object_key;
    }
  
  vsOrbSeqOctet& object_key ()
    {
      return m_ObjectKey;
    }
   
    
  CORBA::ULong marshall();

  int unmarshall();

 
};







class IIOPLocateReplyMessage: public virtual IIOPMessageHeader
{
    
    public:
    CORBA::ULong      m_RequestId;
    LocateStatusType  m_LocateStatus;
    
    public:
    IIOPLocateReplyMessage ( unsigned char * buffer, CORBA::ULong length);
    IIOPLocateReplyMessage ();
    ~IIOPLocateReplyMessage ();

    void request_id (CORBA::ULong request_id)
    {
	m_RequestId = request_id;
    }
    
    CORBA::ULong request_id ()
    {
	return m_RequestId;
    }
    
    void locate_status (LocateStatusType locate_status)
    {
	m_LocateStatus = locate_status;
    }
    
    LocateStatusType locate_status ()
    {
	return m_LocateStatus;
    }
    
    CORBA::ULong marshall();

  int unmarshall();

  
};


class IIOPFragmentMessage: public virtual IIOPMessageHeader
{
    
    public:
    CORBA::ULong      m_RequestId;
    
    public:
    IIOPFragmentMessage (GciValue * Obj);
    ~IIOPFragmentMessage ();

    void request_id (CORBA::ULong request_id)
    {
	m_RequestId = request_id;
    }
    
    CORBA::ULong request_id ()
    {
	return m_RequestId;
    }
   
    
  CORBA::ULong marshall();

  int unmarshall();
};



class Encapsulation
{

    protected:
    
    CORBA::ULong    m_Length;
     unsigned char * m_Buffer;
    
    public:
    
    Encapsulation (CORBA::ULong l) {
	m_Length = l;
	if (l == 0)
	    m_Buffer = 0;
	else {
	    m_Buffer = new unsigned char [l];
	    memset(m_Buffer, 111, l);
	}
    }
    
    Encapsulation (unsigned char * buf, CORBA::ULong l) {
	m_Length = l;
	if (l == 0)
	    m_Buffer = 0;
	else {
	    m_Buffer = new unsigned char [l];
	    memcpy(m_Buffer, buf, l);
	}
    }

    ~Encapsulation() {
	if (m_Buffer)
	    delete m_Buffer;
	
	m_Buffer = 0;
	m_Length = 0;
    }

  unsigned  char * buffer() {
	return m_Buffer;
    }

    CORBA::ULong length() {
	return m_Length;
    }
};


class ProfileBody
{

friend class IOR;
    
protected:
    
    Version         m_Ver;
    char          * m_Host;
    CORBA::UShort   m_Port;
    vsOrbSeqOctet   m_ObjectKey;

public:

    ProfileBody() : m_Host(0), m_Port(0) {
    }

    ~ProfileBody() {
	if (m_Host)
	    delete m_Host;

	m_Host = 0;

	m_ObjectKey = 0;
    }

    CORBA::Boolean operator== (ProfileBody& rhs);
	
    
    Version& ver() {
	return m_Ver;
    }

    void ver(Version ver) {
	m_Ver.m_Major = ver.m_Major;
	m_Ver.m_Minor = ver.m_Minor;
    }
    
    char * host() {
	return m_Host;
    }

    void host(char * host) {
	if (m_Host)
	    delete m_Host;

	CORBA::ULong strLen = ::strlen(host) +1;
	
	m_Host = new char[strLen];
	::memcpy((char *)m_Host, (const char *)host, strLen);
    }

    CORBA::UShort port() {
	return m_Port;
    }
    
    void port(CORBA::UShort port) {
	m_Port = port;
    }

    vsOrbSeqOctet& objectKey() {
	return m_ObjectKey;
    }

    void objectKey(vsOrbSeqOctet& key) {
	m_ObjectKey = key;
    }   

};

class IOR
{

    protected:

    char          * m_TypeId;
    vsOrbSeqTaggedProfile  m_SeqProfiles;
    ProfileBody      * m_InternetProfile;
    cdrCoder               m_Coder;

    public:

    IOR();
    IOR (char *);
    ~IOR();

    IOR *  fromString(char * stringForm);
    char * toString();

    Encapsulation * unHexify(char * hexForm);
    unsigned char * hexify(Encapsulation * e);

    void hexToNibble(char, unsigned char&);

    void hexToByte(char *, unsigned char&);
    void byteToHex(unsigned char, char*);

    char * typeId() {
	return m_TypeId;
    }

    void typeId(char * newType) {
	if (m_TypeId)
	    delete m_TypeId;

	CORBA::ULong strLen = ::strlen(newType) +1;
	
	m_TypeId = new char[strLen];
	::memcpy((char *)m_TypeId, (const char *)newType, strLen);
    }

    vsOrbSeqTaggedProfile& sequence() {
	return m_SeqProfiles;
    }

    ProfileBody* profileBody() {
	return m_InternetProfile;
    }
    
    ProfileBody* extractInternetTag();

    void insertInternetTag(ProfileBody* profile) {
	if (m_InternetProfile)
	    delete m_InternetProfile;
	
	m_InternetProfile = profile;
    }

    cdrCoder& coder() {
	return m_Coder;
    }
    

    void flipEnds() {
	m_Coder.flipEnds();
    }
};

class vsOrbString
{
    friend class cdrCoder;

private:
    unsigned char * m_string;
    
    vsOrbString(const vsOrbString &);

    vsOrbString& operator= (const vsOrbString&);

public:
    
    vsOrbString() {
	m_string = 0;
    }
    
    vsOrbString(CORBA::Long len) {
	m_string = new unsigned char [len];
    }
    
    ~vsOrbString() {
	if (m_string)
	    delete [] m_string;

	m_string = 0;
    }

    operator char * () {
	return (char *)m_string;
    }
    
    operator unsigned char * () {
	return m_string;
    }
    
    unsigned char& operator [] (CORBA::ULong index) {
	    return m_string[index];
    }

    const unsigned char& operator [] (CORBA::ULong index) const {
	    return m_string[index];
    }
    
    CORBA::Boolean isValid() const {
	return (0 != m_string);
    }
    
    unsigned char * reinit(CORBA::Long len) {
	if (m_string)
	    delete [] m_string;
	
	return m_string = new unsigned char [len];
    }
};

class cacheMsg
{
    
private:

    CORBA::Boolean m_sex;
    IIOPMessageHeader &m_msg;

public:
    cacheMsg(IIOPMessageHeader &msg) : m_msg(msg){
	m_sex = msg.byte_order();
    }

    ~cacheMsg() {
	m_msg.byte_order(m_sex);
    }
};
				
#endif

