/*---------------------------------------------------------------------------
* File:   static.c
* Author: ITEX C-Code Generator
*
* Copyright 1991-1999 Telelogic AB, All rights reserved.
* 
* This Program is owned by Telelogic and is protected by national
* copyright laws and international copyright treaties. Telelogic
* grants you the right to use this Program on one computer or in one
* local computer network at any one time.  Under this License you may
* only modify the source code for the purpose of adapting it to your
* environment. You must reproduce and include any copyright and
* trademark notices on all copies of the source code.  You may not
* use, copy, merge, modify or transfer the Program except as provided
* in this License.  Telelogic does not warrant that the Program will
* meet your requirements or that the operation of the Program will be
* uninterrupted and error free. You are solely responsible that the
* selection of the Program and the modification of the source code
* will achieve your intended results and that the results are
* actually obtained.
*/


#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "valrep.h"
#include "static.h"
#include "gci.h"

#ifdef GENMSC
#include "mscgen.h"
#endif

#ifdef DEBUG
#define IcEnter(function) fprintf(stderr, "Enter: %s\n", (function))
#else
#define IcEnter(function) 0
#endif

#ifdef DEBUG
#define IcLeave(function) fprintf(stderr, "Leave: %s\n", (function))
#else
#define IcLeave(function) 0
#endif

extern int IcNoOfPCOs;
extern int IcNoOfCPs;
extern int IcNoOfPTCs;

extern int GcFirstPTCD;

extern VcPCOQueue IcPCOList[];

VcValue *IcObject = NULL;

static VcTimerType *VcTimers = NULL;

int cleanupFlag = 0;

static GciConf IcCurrentConfig = NULL;

/*--------------------------------------------------------------------
*
*/

typedef struct IcComponentStatus
{
  int *pco_list;
  int pco_count;
  enum
  {
    Ready,
      Waiting,
      None
  } status;
  int descriptor;
} IcStatus; 

static IcStatus *component_status = NULL;
static int       component_count  = 0;

/*--------------------------------------------------------------------
*
*/

void IcStatusCreate(GciConf config)
{
  IcEnter("IcStatusCreate");
  
  if (component_status != NULL)
    free((void *) component_status); /* Free pcolist as well */
  
                                     /* Create default context, i.e. a context containing the variables
                                     * default values:
  */
  VcCreateDefaultContext();
  
  if (config == NULL) {
    int i = 0;
    
    component_count  = 1;
    component_status = (IcStatus *) malloc(sizeof(IcStatus));
    
    component_status[0].status     = Ready;
    component_status[0].descriptor = 0;
    component_status[0].pco_count = GciGetNoOfPCOs();
    component_status[0].pco_list = (int *) 
      malloc(sizeof(int) * component_status[0].pco_count);
    
    for (i = 0; i < component_status[0].pco_count; ++i)
      component_status[0].pco_list[i] = GciGetPCO(i);
    
    VcContextCreateMTC(0, "MTC");
  }
  else {
    int i = 0;
    IcSymTabEntry *entry = NULL;
    
    component_count  = GciGetNoOfComponents(config);
    component_status = (IcStatus *) 
      malloc(sizeof(IcStatus) * component_count);
    
    for (i = 0; i < component_count; ++i) {
      GciComponent *c = NULL;
      char **list = NULL;
      int d = 0, j = 0, pco_count = 0;
      
      c = GciGetComponent(config, i);
      d = GciGetComponentDescriptor(c);
      
      list = GciGetComponentPCOs(c);
      while (*list != NULL) {
        ++pco_count, ++list;
      }
      if (GciGetComponentType(c) == GciMTC) {
        component_status[i].status = Ready;
        VcContextCreateMTC(d, GciGetComponentName(c));
      }
      else {
        component_status[i].status = None;
      }
      component_status[i].descriptor = d;
      component_status[i].pco_count = pco_count;
      component_status[i].pco_list = (int *) 
        malloc(sizeof(int) * component_status[i].pco_count);
      
      list = GciGetComponentPCOs(c);
      for (j = 0; j < component_status[i].pco_count; ++j, ++list) {
        for (entry = IcSymTab; entry->Name != NULL; ++entry) {
          if (strcmp(entry->Name, *list) == 0) {
            component_status[i].pco_list[j] = entry->Type;
            break;
          }
        }
      }
    }
  }
  IcLeave("IcStatusCreate");
}

/*--------------------------------------------------------------------
*
*/

static
int IcStatusIndex(void)
{
  int i = 0, descriptor = -1, result = -1;
  
  IcEnter("IcStatusIndex");
  
  descriptor = VcContextGetComponent();
  for (i = 0; i < component_count; ++i)
    if (component_status[i].descriptor == descriptor) {
      result = i;
      break;
    }
    if (result < 0)
      fprintf(stderr, "Error: failed to find VcContext in IcStatus\n");
    
    IcLeave("IcStatusIndex");
    
    return result;
}

/*--------------------------------------------------------------------
*
*/

static
int IcStatusGetPCOCount(void)
{
  int index = -1, result = 0;
  
  IcEnter("IcStatusGetPCOCount");
  if ((index = IcStatusIndex()) >= 0)
    result = component_status[index].pco_count;
  IcLeave("IcStatusGetPCOCount");
  
  return result;
}

static
int IcStatusGetPCO(int pco)
{
  int index = -1, result = 0;
  
  IcEnter("IcStatusGetPCO");
  if ((index = IcStatusIndex()) >= 0)
    result = component_status[index].pco_list[pco];
  IcLeave("IcStatusGetPCO");
  
  return result;
}

/*--------------------------------------------------------------------
*
*/

Bool IcIsReady(void)
{
  Bool result = FALSE;
  int index = -1;
  
  IcEnter("IcIsReady");
  if ((index = IcStatusIndex()) >= 0)
    result = (component_status[index].status == Ready);
  IcLeave("IcIsReady");
  
  return result;
}

/*--------------------------------------------------------------------
*
*/

void IcSetAllReady(void)
{
  int i = 0;
  
  IcEnter("IcSetAllReady");
  for (i = 0; i < component_count; ++i)
    if (component_status[i].status != None)
      component_status[i].status = Ready;
    IcLeave("IcSetAllReady");
}

/*--------------------------------------------------------------------
*
*/

static
int IcAllWaiting(void)
{
  int i = 0, result = TRUE;
  
  IcEnter("IcAllWaiting");
  for (i = 0; i < component_count; ++i)
    if (component_status[i].status == Ready) {
      result = FALSE;
      break;
    }
    IcLeave("IcAllWaiting");
    
    return result;
}

/*--------------------------------------------------------------------
*
*/

static
void IcSetWaiting(void)
{
  int index = -1;
  
  IcEnter("IcSetWaiting");
  if ((index = IcStatusIndex()) >= 0)
    component_status[index].status = Waiting;
  IcLeave("IcSetWaiting");
}

/*--------------------------------------------------------------------
*
*/

static
void IcSetNone(void)
{
  int index = -1;
  
  IcEnter("IcSetNone");
  if ((index = IcStatusIndex()) >= 0)
    component_status[index].status = None;
  IcLeave("IcSetNone");
}

/*--------------------------------------------------------------------
*
*/

static
void IcSetReady(void)
{
  int index = -1;
  
  IcEnter("IcSetReady");
  if ((index = IcStatusIndex()) >= 0)
    component_status[index].status = Ready;
  IcLeave("IcSetReady");
}

/*--------------------------------------------------------------------
* Concurrency part.
*/

void (*TaskInit)(void) = NULL;
void (*TaskCreate)(void *(*)(void *),int,char *,GciValue *,char *) = NULL;
void (*TaskDelete)(void) = NULL;
void (*TaskReschedule)(void) = NULL;
void (*TaskCleanup)(void) = NULL;

int IcInternalConcurrency(void)
{
  return TaskInit != NULL;
}

static
int IcIsCP(int pcod)
{
  return 
    GcFirstPCOD + IcNoOfPCOs <= pcod && 
    pcod < GcFirstPCOD + IcNoOfPCOs + IcNoOfCPs;
}

static 
int IcIsLegalPCO(int pcod)
{
  int pco_count = IcStatusGetPCOCount(), i = 0, result = 0;
  
  for (i = 0; i < pco_count; ++i)
    if ((result = IcStatusGetPCO(i)) == pcod)
      break;
    return result;
}

static 
int IcIsLegalCP(int pcod)
{
  return IcIsCP(pcod);
}

/*--------------------------------------------------------------------
* Name:   TaskInit
* Author: ITEX C Code Generator
*
*/

void IcTaskInit(void) 
{
  IcEnter("IcTaskInit");
  if (TaskInit != NULL)
    TaskInit();
  IcLeave("IcTaskInit");
}

/*--------------------------------------------------------------------
* Name:   TaskCreate
* Author: ITEX C Code Generator
*
*/

void IcTaskCreate(void *(*function)(void *), 
                  int component, 
                  char *tree, 
                  GciValue *args, 
                  char *name)
{
  IcEnter("IcTaskCreate");
  if (TaskCreate != NULL)
    TaskCreate(function, component, tree, args, name);
  IcLeave("IcTaskCreate");
}

/*--------------------------------------------------------------------
* Name:   TaskDelete
* Author: ITEX C Code Generator
*
*/

void IcTaskDelete(void)
{
  IcEnter("IcTaskDelete");
  if (TaskDelete != NULL)
    TaskDelete();
  IcLeave("IcTaskDelete");
}

/*--------------------------------------------------------------------
* Name:   TaskReschedule
* Author: ITEX C Code Generator
*
*/

void IcTaskReschedule(void)
{
  IcEnter("IcTaskReschedule");
  if (TaskReschedule != NULL)
    TaskReschedule();
  IcLeave("IcTaskReschedule");
}

/*--------------------------------------------------------------------
* Name:   TaskCleanup
* Author: ITEX C Code Generator
*
*/

void IcTaskCleanup(void)
{
  IcEnter("IcTaskCleanup");
  if (TaskCleanup != NULL) {
    cleanupFlag = 1;
    TaskCleanup();
    cleanupFlag = 0;
  }
  IcLeave("IcTaskCleanup");
}

/*--------------------------------------------------------------------
*
*/

/* Global vars */

int IcGlobalVerdict = IcNone;

#define STRSIZE 255

/* These are functions that the user must write to make it
* work. Please refer to the INTOOL/GCI document for more details. */

extern int GciConfig(GciConf);
extern int GciSnapshot(void);
/* extern int GciLog(int, char *); */
extern int GciCreate(int, char *, VcValue *);
extern int GciSend(int, VcValue *);
extern int GciCancelTimer(int);
extern long GciReadTimer(int);
extern int GciImplicitSend(VcValue *);
extern int GciStartTimer(int, long, int);
extern VcValue *GciReadTSPar(char *, char *);
extern int GcFirstTimerD;

/* These three functions are generated */

extern VcPCOQueue *IcGetPCO(int);
extern VcTimerType *IcGetTimer(int);
extern VcPTCType *IcGetPTC(int);

/*-------------------------------------------------------------------------
* Section: Misc Stuff
* 
* Description:
*  This section contains misc functions.
*-----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
* Name:    IcConcurrentInit
* Creator: ITEX C-Code Generator
*
*/

void IcConcurrentInit(void)
{
  IcEnter("IcConcurrentInit");
  IcTaskInit();
  IcLeave("IcConcurrentInit");
}

/*-------------------------------------------------------------------------
* Name:    IcConcurrentCleanup
* Creator: ITEX C-Code Generator
*
*/

void IcConcurrentCleanup(void)
{
  IcEnter("IcConcurrentCleanup");
  IcTaskCleanup();      /* Await for all PTCs to terminate */
  IcLeave("IcConcurrentCleanup");
}

/*-------------------------------------------------------------------------
* Name:    IcConfig
* Creator: ITEX C-Code Generator
*
*/

int IcConfig(GciConf config)
{
  int result = 0;
  
  IcEnter("IcConfig");
  
  /* Save the current configuration which may be used to figure out
  test component names and CPs etc. */
  
  IcCurrentConfig = config;
  
  /* Create and setup a component status structure that we can keep
  * during an execution of a test case. */
  
  if (IcInternalConcurrency()) 
    IcStatusCreate(config);
  else
    IcStatusCreate(NULL);
  
    /* If we have an actual configuration we have a concurrent testcase
    * and we let the user gain control in GciConfig.  Otherwise we are
  * done and can return GciOk. */
  
  result = (config != NULL) ? GciConfig(config) : GciOk;
  
  IcLeave("IcConfig");
  
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcSnapshot
* Creator: ITEX C-Code Generator
*
* Description:
*  The snapshot function.
*
* Return Value:
*  Status result from GciSnapshot.
*
*/

int IcSnapshot(void)
{
  VcContext *save_context = NULL;
  static int result = GciOk;
  
  IcEnter("IcSnapshot");
  
  /* Save this threads context into a local variable.  This will store
  * the context pointer until we return from the reschedule function. */
  
  save_context = context;
  
  /* Wait for all threads to get here */
  
  IcSetWaiting();
  while (!IcAllWaiting()) {
    if (IcIsReady())
      break;
    IcTaskReschedule();
    VcContextSwitch(save_context);
    if (cleanupFlag)
      return GciNotOk;
  }
  
  /* All but the MTC wait here. The MTC handles the snapshot. */
  
#ifndef ANY_THREAD_DOES_SNAPSHOT
  while (IcAllWaiting()) {
    if (VcContextGetComponent() == MTCD) {
#endif
      VcMallocType strategy = VcGetAlloc();
      VcSetPermAlloc(); 
      result = GciSnapshot();
      IcSetAllReady();
      VcSetMalloc(strategy);
#ifndef ANY_THREAD_DOES_SNAPSHOT
    }    
    IcTaskReschedule();
    VcContextSwitch(save_context);
  }
#endif
  
  /* Return the status of the operation.  In the case of something
  * went wrong and the execution we convert the return value to '0'
  * to indicate that something went wrong and we should terminate
  * execution of this test. */
  
  if ( result == IcEXECUTIONSTOPPED )
    result = 0;
  
  IcLeave("IcSnapshot");
  
  return result;
}

/*---------------------------------------------------------------------------
* Name:   IcConcurrencyCreate
*
* Internal code. Not to be altered at adaptation time. 
*  
* This code is the main routine of parallel test components.  First
* it unmangles the data given as a void pointer, then it calls the
* generated behaviour tree, and finally it exits with the return code
* from that behaviour.  
*/

static
void *IcConcurrencyCreate(void *args)
{
  IcArguments *arguments = NULL;
  VcValue *permanent = NULL;
  VcContext *save_context = NULL;
  VcMallocType strategy;
  
  IcEnter("IcConcurrencyCreate");
  
  arguments = (IcArguments *) args;
  
#ifdef GENMSC
  MscCreateComponent( IcGetComponentName(), arguments->name ); /* ??? */
#endif
  
  VcContextCreatePTC(arguments->component, arguments->name);
  save_context = context;
  
  strategy = VcGetAlloc();
  VcSetPermAlloc();
  if (arguments->arguments == NULL)
    permanent = VcMkSEQUENCE(0);
  else
    permanent = VcCopy(arguments->arguments);
  VcSetMalloc(strategy);
  
  IcSetReady();
  GciStartTestComponent(arguments->tree, permanent);
  VcContextSwitch(save_context);
  IcSetNone();
  
  VcRmValue(permanent);
  
  GciDone(VcContextGetComponent());
  VcContextDelete();
  IcTaskDelete();
  
  IcLeave("IcConcurrencyCreate");
  
  return NULL;
}

/*-------------------------------------------------------------------------
* Name:    IcGetComponentName
* Creator: ITEX C-Code Generator
*
*/

const char *IcGetComponentName(void)
{
  const char *result = NULL;
  IcEnter("IcGetComponentName");
  result = VcContextGetComponentName();
  IcLeave("IcGetComponentName");
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcGetOtherCPEndComponentName
* Description:
*
*  Given a CP and a current configuration, find the name of the component
*  that owns the other end.
*
*  If there is no such component or if it is not a CP, just return the
*  name of the CP/PCO that was provided as parameter...
*/

static const char *
IcGetOtherCPEndComponentName( int cp )
{
  const char * cpname = GciGetPCOName( cp );
  if ( IcInternalConcurrency() && IcCurrentConfig ) {
    int ix;
    for ( ix = 0; ix < GciGetNoOfComponents( IcCurrentConfig ); ++ix ) {
      GciComponent * comp = GciGetComponent( IcCurrentConfig, ix );
      char ** tmp_cp = GciGetComponentCPs( comp );
      
      if ( GciGetComponentDescriptor(comp) == VcContextGetComponent() )
        continue;       /* CP is not connected to self */
      
      while ( *tmp_cp )
        if (!strcmp( cpname, *tmp_cp ) )
          return GciGetComponentName( comp );
        else
          ++tmp_cp;
    }
    GciLog(IcLogMessage, "CP not part of current configuration");
    return "Unknown_PTC"; /* Dummy return to get rid of warning! */
  }
  else
    return cpname;
}

const char *
IcGetNameOfComponentOwningPCO( int PCOD )
{
  const char * compname = "MTC";
  const char * pconame = GciGetPCOName( PCOD );
  
  if ( IcCurrentConfig ) {
    int ix;
    for ( ix = 0; ix < GciGetNoOfComponents( IcCurrentConfig ); ++ix ) {
      GciComponent * comp = GciGetComponent( IcCurrentConfig, ix );
      char ** tmp_pco = GciGetComponentPCOs( comp );
      while ( *tmp_pco )
        if (!strcmp( pconame, *tmp_pco ) )
          return GciGetComponentName( comp );
        else
          ++tmp_pco;
    }
  }
  return compname;
}

/*-------------------------------------------------------------------------
* Name:    IcCreate
* Creator: ITEX C-Code Generator
*
* Description:
*  The function for CREATE.
*
* Return Value:
*/

int IcCreate(int c, char *tree, VcValue *args, char *name)
{
  VcContext *save_context = NULL;
  int result = GciNotOk;
  
  IcEnter("IcCreate");
  
  result = GciCreate(c, tree, args);
  save_context = context;
  
  IcGetPTC(c)->Status = VcSTATUSCREATED;
  IcTaskCreate(IcConcurrencyCreate, c, tree, args, name);
  VcContextSwitch(save_context);
  
  IcLeave("IcCreate");
  
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcDone
* Creator: ITEX C-Code Generator
*
* Description:
*  The function for DONE. Not implemented!
*
* Return Value:
*  
*
*/

Bool IcDone(int ptc)
{
  Bool result       = FALSE;
  int  component    = VcContextGetComponent();
  int  n;
  
  IcEnter("IcDone");
  
#ifdef DEBUG
  fprintf( stdout, "Number of available PTCs are %d, starting from %d.\n", 
    IcNoOfPTCs, GcFirstPTCD );
#endif
  
    /* The argument IcALL_PTCs denotes the use of a DONE statement with an empty
    argument list, meaning the status of all PTCs. According to 15.9.10.2 of
    the ISO 9646-3 standard, the MTC is the only component that can use a 
  DONE statement with an empty list. */
  
  if ( ptc == IcALL_PTCs ) {
    result = TRUE;
    
    /* Loop over all PTCs and exit if we find one that is running. */
    for ( n=0; result && n<IcNoOfPTCs; ++n ) {
      
    /* As we must be in the MTC to be here, we can compare with the current
    component ID to skip checking the active component for status.
      (It is never VcSTATUSDONE at this moment, of course) */
      
      if ( component != GcFirstPTCD+n ) {
        result = ( IcGetPTC(GcFirstPTCD+n)->Status == VcSTATUSDONE );
#ifdef DEBUG
        fprintf( stdout, "Checking PTC %d, status is %d\n", GcFirstPTCD+n, 
          IcGetPTC(GcFirstPTCD+n)->Status );
#endif
        
      }
#ifdef DEBUG
      else {
        fprintf( stdout, "Skipping this component at id %d\n", n+GcFirstPTCD );
      }
#endif
    }
  }
  else {
#ifdef DEBUG
    fprintf( stdout, "Checking status of PTC %d!\n", ptc );
#endif
    result = IcGetPTC(ptc)->Status == VcSTATUSDONE;
  }
  
  IcLeave("IcDone");
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcReadTSPar
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called for every test suite parameter that has to
*  be read.
*
* Return Value:
*  Status result from GciReadTSPar.
*
*/

VcValue *IcReadTSPar(char *x, char *y)
{
  VcValue *result = NULL;
  IcEnter("IcReadTSPar");
  result = GciReadTSPar(x, y);
  IcLeave("IcReadTSPar");
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcFileLog
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called for logging.
*
* Return Value:
*  Status result from GciLog.
*
*/

Bool IcFileLog(int id, char *string)
{
  Bool result = FALSE;
  
  IcEnter("IcFileLog");
  
  if (id == GciLogVerdict || id == GciLogPVerdict) {
    const char *verdict = NULL;
    char buffer[STRSIZE];
    
    verdict = IcVerdict2Str(IcGlobalVerdict);
    sprintf(buffer, "%s %s", verdict, string);
    
    result = GciLog(id, buffer);
  }
  else
    result = GciLog(id, string);
  
  IcLeave("IcFileLog");
  
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcStartTimer
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to start a timer.
*
* Return Value:
*  Status result from GciStartTimer.
*
*/

int IcStartTimer(int TimerD, VcValue *value, int nr, char *id, char *t)
{
  int result = GciNotOk;
  
  IcEnter("IcStartTimer");
  
  if (VcGetBaseType(VcGetType(value)) == VcINTEGERD) {
    int          timer_id = ConvertTimerIdtoContextId(TimerD);
    VcTimerType *timer    = IcGetTimer(timer_id);
    VcBaseVal    duration = VcGetINTEGER(value);
    
    if (timer) {
      if (duration < 0 && timer->Duration != NULL)
        duration = VcGetINTEGER(timer->Duration);
      
      if (duration >= 0) {
        char msg[STRSIZE];
        
        timer->Status = VcSTATUSRUNNING;
        result = GciStartTimer(timer_id, duration, timer->Unit);
        
        sprintf(msg, "[%d] %s (%s) %s", nr, id, VcContextGetComponentName(), t);
        IcLog(GciLogStartT, msg);
#ifdef GENMSC
        MscStartTimer( IcGetComponentName(), timer->Name, duration );
#endif
      }
    }
    else
      fprintf(stderr, "Tried to start timer without default duration!\n");
  }
  else
    fprintf(stderr, "Value for timer must be an INTEGER!\n");
  
  IcLeave("IcStartTimer");
  
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcCancelTimer
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to cancel a timer.
*
* Return Value:
*  Status result from GciCancelTimer.
*
*/

Bool IcCancelTimer(int TimerD, int nr, char *name)
{
  char msg[STRSIZE];
  Bool result = FALSE;
  int  timer_id;
  VcTimerType *timer = NULL;
  
  IcEnter("IcCancelTimer");
  
  if (TimerD == GcANY_TIMERD) {
    int i, timers = GciGetNoOfTimers();
    for (i = 0; i < timers; ++i) {
      timer_id = ConvertTimerIdtoContextId(GcFirstTimerD + i);
      timer    = IcGetTimer(timer_id);
      if (timer) {
        if (timer->Status == VcSTATUSRUNNING) {
          result = GciCancelTimer(timer_id);
#ifdef GENMSC
          if (timer->Status == VcSTATUSRUNNING)
            MscCancelTimer(IcGetComponentName(), timer->Name);
#endif
          timer->Status = VcSTATUSSTOPPED;
          
          sprintf(msg, "[%s] %s (%s)", "implicit", GciGetTimerName(GcFirstTimerD + i), VcContextGetComponentName());
          IcLog(GciLogCancelT, msg);
        }
      }
    }
  }
  else {
    timer_id = ConvertTimerIdtoContextId(TimerD);
    timer    = IcGetTimer(timer_id);
    if (timer) {
      if (timer->Status == VcSTATUSRUNNING) {
#ifdef GENMSC
        if (timer->Status == VcSTATUSRUNNING)
          MscCancelTimer(IcGetComponentName(), timer->Name);
#endif
        result = GciCancelTimer(timer_id);
        timer->Status = VcSTATUSSTOPPED;
        sprintf(msg, "[%d] %s (%s)", nr, name, VcContextGetComponentName());
        IcLog(GciLogCancelT, msg);
      }
    }
  }
  
  IcLeave("IcCancelTimer");
  
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcReadTimer
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to read a timer
*
* Return Value:
*  Status result from GciReadTimer.
*
*/

VcValue *IcReadTimer(int TimerD)
{
  VcValue *result = NULL;
  int      timer_id;
  
  IcEnter("IcReadTimer");
  
  timer_id = ConvertTimerIdtoContextId(TimerD);
  result = VcMkINTEGER((VcBaseVal) GciReadTimer(timer_id));
  
  IcLeave("IcReadTimer");
  
  return result;
}



/*-------------------------------------------------------------------------
* Name:    IcSend
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to send away an object.
*
* Return Value:
*  Status result from GciSend.
*
*/

Bool IcSend(int PCOD, VcValue *value, int nr, char *pco, char *type)
{
  char msg[STRSIZE];
  int result = 0;
  
  IcEnter("IcSend");
  
  if (IcIsCP(PCOD) && IcInternalConcurrency()) {
    VcMallocType strategy;
    
    IcSetAllReady();
    
    strategy = VcGetAlloc();
    VcSetPermAlloc();
    result = GciReceive(PCOD, VcCopy(value));
    VcSetMalloc(strategy);
    
#ifdef GENMSC
    {
      /* Internal CP message */
      char * comment = (char*) malloc( strlen( GciGetPCOName( PCOD ) ) + 10 );
      strcpy( comment, "via " );
      strcat( comment, GciGetPCOName( PCOD ) );
      MscOut( MscInternalMessage, IcGetComponentName(), 
        IcGetOtherCPEndComponentName( PCOD ), value, comment );
      free( (void*) comment );
    }
#endif
  }
  else {
#ifdef GENMSC
    if (!IcIsCP( PCOD ) ) {
      /* This is for sending a message to IUT, no delay in channel */
      MscOut( MscSendMessage, 
        IcGetComponentName(), GciGetPCOName(PCOD), value, NULL ); 
      MscIn( MscSendMessage, 
        IcGetComponentName(), GciGetPCOName(PCOD), NULL );
    }
#endif
    result = GciSend(PCOD, value);
  }
  
  sprintf(msg, "[%d] %s ! %s", nr, pco, type);
  GciLog(GciLogSendE, msg);
  
  IcLeave("IcSend");
  
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcImplicitSend
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to perform an implicit send.
*
* Return Value:
*  Status result from GciImplicitSend.
*
*/

Bool IcImplicitSend(VcValue *value)
{
  Bool result = FALSE;
  IcEnter("IcImplicitSend");
#ifdef GENMSC
  MscImplicitSend( IcGetComponentName(), value );
#endif
  result = GciImplicitSend(value);
  IcLeave("IcImplicitSend");
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcReceive
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to see if we have received something.
*
* Return Value:
*  
*
*/

Bool IcReceive(int PCOD, VcValue *value)
{
  VcPCOQueueObject *queue = NULL;
  Bool result = FALSE;
  
  IcEnter("IcReceive");
  
  queue = IcGetPCO(PCOD)->Queue;
  if (IcIsCP(PCOD) && IcInternalConcurrency()) {
    
  /* Find the first relevant message.  This has to be done in order
    * to be able to handle internal two way communication (CM/CP). */
    
    while (queue != NULL && queue->sender == context)
      queue = queue->Next;
  }
  
  /* See if the received message matched the receive constraint. */
  
  if (queue != NULL)
    result = VcMatch(value, queue->First);
  
  IcLeave("IcReceive");
  
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcSetVerdict
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to set a verdict.
*
* Return Value:
*  
* */

Bool IcSetVerdict(int verdict)
{
  IcEnter("IcSetVerdict");
  
  if (verdict != IcR) {
    if (verdict < VcContextGetVerdict())
      VcContextSetVerdict(verdict);
    if (verdict < IcGlobalVerdict)
      IcGlobalVerdict = verdict;
  }
  
#ifdef GENMSC
  MscPreliminaryVerdict( IcGetComponentName(), IcVerdict2Str( verdict ) );
#endif
  
  IcLeave("IcSetVerdict");
  
  return TRUE;
}

/*-------------------------------------------------------------------------
* Name:    IcGetVerdict
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to get a verdict.
*
* Return Value:
*  
*
*/

int IcGetVerdict(void)
{
  int result = 0;
  IcEnter("IcGetVerdict");
  result = IcGlobalVerdict;
  IcLeave("IcGetVerdict");
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcGetReceiveObject
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to get the received object.
*
* Return Value:
*  The received object.
*
*/

VcValue *IcGetReceiveObject(int PCOD)
{
  VcValue *result = NULL;
  VcPCOQueueObject *queue = NULL;
  
  IcEnter("IcGetReceiveObject");
  
  queue = IcGetPCO(PCOD)->Queue;
  if (IcIsCP(PCOD) && IcInternalConcurrency())
    while (queue != NULL && queue->sender == context)
      queue = queue->Next;
    result = (queue != NULL) ? queue->First : NULL;
    
    IcLeave("IcGetReceiveObject");
    
    return result;
}

int IcClearPCOs(void)
{
  int i = 0;
  
  for (i = 0; i < IcNoOfPCOs; ++i)
    if (IcPCOList[i].Type != NULL) {
      VcPCOQueueObject *queue = NULL;
      
      queue = IcPCOList[i].Queue;
      while (queue != NULL) {
        VcPCOQueueObject *hold = NULL;
        
        hold = queue;
        queue = queue->Next;
        
        VcRmValue(hold->First);
        hold->First = NULL;
        hold->Next  = NULL;
        VCFREE((char *) hold, sizeof(*hold), "");
      }
      IcPCOList[i].Queue = NULL;
    }
    
    return TRUE;
}

/*-------------------------------------------------------------------------
* Name:    IcRemoveObject
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to remove an object from a PCO queue.
*
* Return Value:
*  
*
*/

int IcRemoveObject(int PCOD, int nr, char *pco, char *type)
{
  VcPCOQueueObject *queue = NULL;
  int result = TRUE;
  
  IcEnter("IcRemoveObject");
  
  queue = IcGetPCO(PCOD)->Queue;
  if (queue != NULL) {
    char msg[STRSIZE];
    
    if (IcIsCP(PCOD) && IcInternalConcurrency()) {
      VcPCOQueueObject *prev = NULL;
      
      /* Find the first relevant message.  This has to be done in
      * order to be able to handle internal two way communication
      * (CM/CP).  The first relevant message is the one we have
      * matched and is thus the one we want to remove now.*/
      
      prev = queue;
      while (queue != NULL && queue->sender == context)
        prev = queue, queue = queue->Next;
      
        /* Now we do the actual remove.  queue should point at the
        * relevant object and prev on the object before it.  If prev
        * and queue are pointing at the same object we want to remove
      * the first object in the queue. */
      
      if (queue != NULL) {
        if (queue == prev)
          IcGetPCO(PCOD)->Queue = queue->Next;
        else
          prev->Next = queue->Next;
      }
    }
    else
      IcGetPCO(PCOD)->Queue = queue->Next;
    
#ifdef GENMSC
    if ( IcInternalConcurrency() && IcIsCP( PCOD ) )
      MscIn( MscInternalMessage, IcGetOtherCPEndComponentName(PCOD), 
      IcGetComponentName(), NULL );
    else
      MscIn( MscReceiveMessage, GciGetPCOName(PCOD), 
      IcGetComponentName(), NULL );
#endif
    
    VcRmValue(queue->First);
    queue->First = NULL;
    queue->Next  = NULL;
    VCFREE((char *) queue, sizeof(*queue), "IcRemoveObject");
    
    sprintf(msg, "[%d] %s ? %s", nr, pco, type);
    GciLog((strcmp("OTHERWISE", type) == 0) ? GciLogOtherE : GciLogRecE, msg);
  }
  else
    result = FALSE;
  
  IcLeave("IcRemoveObject");
  
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcOtherwise
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to see if we got an OTHERWISE.
*
* Return Value:
*  
*
*/

int IcOtherwise(int PCOD)
{
  int result = 0;
  
  IcEnter("IcOtherwise");
  
  if (IcIsCP(PCOD) && IcInternalConcurrency()) {
    VcPCOQueueObject *queue = IcGetPCO(PCOD)->Queue;
    
    /* Find the first relevant message.  This has to be done in order
    * to be able to handle internal two way communication (CM/CP). */
    
    while (queue != NULL && queue->sender == context)
      queue = queue->Next;
    
    result = (queue != NULL);
  }    
  else if (IcIsLegalPCO(PCOD)) {
    result = (IcGetPCO(PCOD)->Queue != NULL);
  }
  
  IcLeave("IcOtherwise");
  
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcGetEcbStatus
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to get the status of the execution block.
*
* Return Value:
*  
* */

int IcGetEcbStatus(IcECB *ecb)
{
  int result = 0;
  IcEnter("IcGetEcbStatus");
  result = ecb->status;
  IcLeave("IcGetEcbStatus");
  return result;
}

/*-------------------------------------------------------------------------
* Name:    IcSetEcbStatus
* Creator: ITEX C-Code Generator
*
* Description:
*  This function is called to set the execution block status.
*
* Return Value:
*  
*
*/

int IcSetEcbStatus(IcECB * ecb, int status, int lineNr)
{
  char line[10];
  
  IcEnter("IcSetEcbStatus");
  
  ecb->status = status;
  if (lineNr == IcNOMATCH) {
    sprintf(line, "%d", lineNr);
    GciLog(GciLogNOMatch, line);
  }
  
  IcLeave("IcSetEcbStatus");
  
  return TRUE;
}

VcType *IcSymTabStr2Type(const char *name)
{
  int i = 0;
  VcType *result = NULL;
  
  IcEnter("IcSymTabStr2Type");
  
  for (i = 0; IcSymTab[i].Name != NULL; ++i)
    if (strcmp(IcSymTab[i].Name, name) == 0) {
      result = *IcSymTab[i].TypeRef;
      break;
    }
    
    IcLeave("IcSymTabStr2Type");
    
    return result;
}

#define IcSymTabOffset 100

const char *IcSymTabTag2Str(int tag)
{
  const char *name = NULL, *result = NULL;
  
  IcEnter("IcSymTabTag2Str");
  
  if ((name = VcTag2Str(tag)) != NULL)
    result = name;
  else
    result = IcSymTab[tag - IcSymTabOffset].Name;
  
  IcLeave("IcSymTabTag2Str");
  
  return result;
}

/*-------------------------------------------------------------------------
* Section: Timer Stuff
* 
* Description:
*  This section contains functions that deal with timer related actions.
*-----------------------------------------------------------------------*/


VcTimerType *IcContextGetTimer(int timer)
{
  int components = (IcNoOfPTCs > 0 ? IcNoOfPTCs : 1);
  
  if (VcTimers == NULL)
    IcInitTimers();
  
  if (timer < 0 || timer >= IcNoOfTimers * components) {
    fprintf(stderr, "IcContextGetTimer got an illegal timer ID %d (%d)\n", timer, IcNoOfTimers);
    return NULL;
  }
  
  return &VcTimers[timer];
}

int IcInitTimers()
{
  int i = 0, amount = 0;
  
  IcEnter("IcInitTimers");
  
  if (VcTimers == NULL) {
    int i;
    int j;
    int timers = GciGetNoOfTimers();
    int components = (IcNoOfPTCs > 0 ? IcNoOfPTCs : 1);
    
    VcTimers = (VcTimerType *) malloc(sizeof(VcTimerType) * timers * components);
    for (i = 0; i < components; ++i)
      for (j = 0; j < timers; ++j) {
        VcTimerType *defaultTimer = IcGetDefaultTimer(GcFirstTimerD + j);
        if (defaultTimer != NULL) {
          int index = i * timers + j;
          VcTimers[index].Name      = defaultTimer->Name;
          VcTimers[index].Duration  = defaultTimer->Duration;
          VcTimers[index].Unit      = defaultTimer->Unit;
          VcTimers[index].StartTime = 0;
          VcTimers[index].Status    = VcSTATUSSTOPPED;
        }
      }
  }
  
  IcLeave("IcInitTimers");
  
  return TRUE;
}

int IcTimeout(int TimerD)
{
  int result = FALSE;
  
  IcEnter("IcTimeout");
  
  if (TimerD == GcANY_TIMERD) {
    int i;
    int components = (IcNoOfPTCs > 0 ? IcNoOfPTCs : 1);
    int amount = GciGetNoOfTimers() * components;
    
    for (i = 0; i < amount; ++i) {
      VcTimerType *timer = IcContextGetTimer(i);
      if (timer) {
        if (timer->Status == VcSTATUSTO) {
          timer->Status = VcSTATUSSTOPPED;
#ifdef GENMSC
          MscTimeoutTimer(IcGetComponentName(), timer->Name);
#endif
          result = TRUE;
        }
      }
    }
  }
  else {
    VcTimerType *timer = IcGetTimer(ConvertTimerIdtoContextId(TimerD));
    if (timer) {
      if (timer->Status == VcSTATUSTO) {
        timer->Status = VcSTATUSSTOPPED;
#ifdef GENMSC
        MscTimeoutTimer( IcGetComponentName(), timer->Name );
#endif
        result = TRUE;
      }
    }
  }
  
  IcLeave("IcTimeout");
  
  return result;
}

int IcSetTimerName(int TimerD, char *Name)
{
  VcTimerType *Timer = NULL;
  
  IcEnter("IcSetTimerName");
  
  Timer = IcGetDefaultTimer(TimerD);
  Timer->Name = (char *) malloc(sizeof(char) * (strlen(Name) + 1));
  strcpy(Timer->Name, Name);
  
  IcLeave("IcSetTimerName");
  
  return TRUE;
}

int IcSetTimerDuration(int TimerD, VcValue * duration)
{
  IcEnter("IcSetTimerDuration");
  IcGetDefaultTimer(TimerD)->Duration = duration;
  IcLeave("IcSetTimerDuration");
  return TRUE;
}

VcValue *IcGetTimerDuration(int TimerD)
{
  VcValue *result = NULL;
  IcEnter("IcGetTimerDuration");
  result = IcGetTimer(TimerD)->Duration;
  IcLeave("IcGetTimerDuration");
  return result;
}

int IcSetTimerUnit(int TimerD, VcUnit unit)
{
  IcEnter("IcSetTimerUnit");
  IcGetDefaultTimer(TimerD)->Unit = unit;
  IcLeave("IcSetTimerUnit");
  return TRUE;
}

VcUnit IcGetTimerUnit(int TimerD)
{
  VcUnit result = 0;
  IcEnter("IcGetTimerUnit");
  result = IcGetTimer(TimerD)->Unit;
  IcLeave("IcGetTimerUnit");
  return result;
}

/*-------------------------------------------------------------------------
* Section: PCO Stuff
* 
* Description:
*  This section contains functions that deal with pco related actions.
*-----------------------------------------------------------------------*/

int 
IcSetPCOName(int PCOD, char *Name)
{
  VcPCOQueue *Pco = NULL;
  
  IcEnter("IcSetPCOName");
  
  Pco = IcGetPCO(PCOD);
  Pco->Name = (char *) malloc(sizeof(char) * (strlen(Name) + 1));
  strcpy(Pco->Name, Name);
  
  IcLeave("IcSetPCOName");
  
  return TRUE;
}

int IcSetPCOType(int PCOD, char *Type)
{
  VcPCOQueue *Pco = NULL;
  
  IcEnter("IcSetPCOType");
  
  Pco = IcGetPCO(PCOD);
  Pco->Type = (char *) malloc(sizeof(char) * (strlen(Type) + 1));
  strcpy(Pco->Type, Type);
  
  IcLeave("IcSetPCOType");
  
  return TRUE;
}

int IcSetPCORole(int PCOD, int Role)
{
  IcEnter("IcSetPCORole");
  IcGetPCO(PCOD)->Role = Role;
  IcLeave("IcSetPCORole");
  return TRUE;
}

/*-------------------------------------------------------------------------
* Section: Utility stuff
* 
* Description:
*  This section contains functions that are nice to have
*-----------------------------------------------------------------------*/

const char *IcVerdict2Str(int v)
{
  const char *result = NULL;
  
  IcEnter("IcVerdict2Str");
  
  switch (v) {
  case IcFAIL:
    result = "FAIL";
    break;
  case IcINCONC:
    result = "INCONC";
    break;
  case IcPASS:
    result = "PASS";
    break;
  case IcNone:
    result = "None";
    break;
  case IcR:
    result = IcGlobalVerdict != IcR ? IcVerdict2Str(IcGlobalVerdict) : "R";
    break;
  default:
    result = "";
    break;
  }
  
  IcLeave("IcVerdict2Str");
  
  return result;
}

int IcStr2Verdict(const char *str)
{
  int result = -1;
  
  IcEnter("IcStr2Verdict");
  
  if (strcmp(str, "FAIL") == 0)
    result = IcFAIL;
  else if (strcmp(str, "INCONC") == 0)
    result = IcINCONC;
  else if (strcmp(str, "PASS") == 0)
    result = IcPASS;
  else if (strcmp(str, "None") == 0)
    result = IcNone;
  else if (strcmp(str, "R") == 0)
    result = IcGlobalVerdict;     
  else
    result = -1;
  
  IcLeave("IcStr2Verdict");
  
  return result;
}

/***************************
* Converts a timer identifier to the PTC/MTC specific identifier.
* Does this by adding a multiple of the total number of timers
* to the timer identifier
***************************/

int ConvertTimerIdtoContextId(int id)
{
  int component = VcContextGetComponent();
  if (component == MTCD)
    return id;
  else
    return id + (component - GcFirstPTCD) * IcNoOfTimers;
}

/***************************
* Converts a contect specific timer id to the "base" timer id.
***************************/

int ConvertContextIdToTimerId(int id)
{
  int component = VcContextGetComponent();
  if (component == MTCD)
    return id;
  else
    return id - (component - GcFirstPTCD) * IcNoOfTimers;
}

/* End of File */

