/*---------------------------------------------------------------------------
 * File:   static.h
 * Author: ITEX C-Code Generator
 *
 * Copyright 1991-1999 Telelogic AB, All rights reserved.
 * 
 * This Program is owned by Telelogic and is protected by national
 * copyright laws and international copyright treaties. Telelogic
 * grants you the right to use this Program on one computer or in one
 * local computer network at any one time.  Under this License you may
 * only modify the source code for the purpose of adapting it to your
 * environment. You must reproduce and include any copyright and
 * trademark notices on all copies of the source code.  You may not
 * use, copy, merge, modify or transfer the Program except as provided
 * in this License.  Telelogic does not warrant that the Program will
 * meet your requirements or that the operation of the Program will be
 * uninterrupted and error free. You are solely responsible that the
 * selection of the Program and the modification of the source code
 * will achieve your intended results and that the results are
 * actually obtained.
 */


#ifndef STATIC_H
#define STATIC_H

#include "valrep.h"

/*----------------------------------------------------------------------
 * Section: Defines
 */

#define IcNOMATCH           -1
#define IcNOMATCH1          -2
#define IcTESTCASEERROR     -3
#define IcEXECUTIONERROR    -4
#define IcEXECUTIONSTOPPED  -5

#define IcMATCH              1
#define IcVERDICT            2

#define IcFAIL               0
#define IcINCONC             1
#define IcPASS               2
#define IcNone               3
#define IcR                  4

#define IcALL_PTCs           -1

/*----------------------------------------------------------------------
 * Section: Enums
 */

typedef enum
  {
    IcLogMatchValue = 0,
    IcLogNoMatchValue = 1,
    IcLogMessage = 2
  }
IcLogValues;

/*----------------------------------------------------------------------
 * Section: Structures
 */

typedef struct IcECB {int status;} IcECB;

struct IcComponent
{
  char *Name;
  int Id;
  char **UsedPCOs;
  char **UsedCPs;
};
typedef struct IcComponent IcComponent;

struct IcSymTabEntry
{
  char *Name;
  int Type;
  VcType **TypeRef;
  VcValue **Val;
  int (*Behav)();
  char **tgList;
  
  IcComponent **config;
};
typedef struct IcSymTabEntry IcSymTabEntry;


typedef struct IcArguments
{
  int component;
  char * tree;
  VcValue *arguments;
  char *name;
} IcArguments;

/*----------------------------------------------------------------------
 * Section: Globals
 */

extern IcSymTabEntry IcSymTab[];
extern int IcGlobalVerdict;

/*----------------------------------------------------------------------
 * Section: Functions
 */

extern void IcSetAllReady(void);
extern int IcSnapshot(void);
extern int IcSend(int, VcValue *, int, char *, char *);
extern int IcImplicitSend(VcValue *);
extern int IcReceive(int, VcValue *);
extern int IcSetVerdict(int);
extern int IcGetVerdict(void);
extern int IcOtherwise(int);
extern int IcGetEcbStatus(IcECB *);
extern int IcSetEcbStatus(IcECB *, int, int);
extern int IcAssign(VcValue *, VcValue *);
extern VcValue *IcGetReceiveObject(int);
extern int IcRemoveObject(int, int, char *, char *);
extern int IcFileLog(int, char *);
extern int IcStartTimer(int, VcValue *, int, char *, char *);
extern int IcCancelTimer(int, int, char *);
extern VcValue *IcReadTimer(int);
extern VcValue *IcReadTSPar(char *, char *);
extern int IcCreate(int, char *, VcValue *, char *);
extern const char *IcGetComponentName(void);
extern int IcDone(int);
extern VcType *IcSymTabStr2Type(const char *);
extern const char *IcSymTabTag2Str(int);

/*-------------------------------------------------------------------------
 * Section: Timer Stuff
 * 
 * Description:
 *  This section contains functions that deal with timer related actions.
 *-----------------------------------------------------------------------*/

#define IcTIMERDEFVALUE VcMkINTEGER(-1)
#define IcCANCELALLTIMERS -1
#define GcANY_TIMERD -1

extern int IcTimeout(int TimerD);
extern VcValue *IcGetTimerDuration(int TimerD);
extern int IcSetTimerName(int TimerD, char *Name);
extern int IcSetTimerDuration(int TimerD, VcValue * duration);
extern VcUnit IcGetTimerUnit(int TimerD);
extern int IcSetTimerUnit(int TimerD, VcUnit unit);
extern int IcInitTimers();
extern VcTimerType* IcContextGetTimer( int );
extern VcTimerType* IcGetDefaultTimer( int );
extern int IcNoOfTimers;
extern VcTimerType* VcContextGetTimers(void);
extern void VcContextSetTimers( int, VcTimerType*);
extern const char * IcGetNameOfComponentOwningPCO( int PCOD );

/*-------------------------------------------------------------------------
 * Section: PCO Stuff
 * 
 * Description:
 *  This section contains functions that deal with pco related actions.
 *-----------------------------------------------------------------------*/

extern int IcClearPCOs(void);
extern int IcSetPCOType(int PCOD, char *Type);
extern int IcSetPCORole(int PCOD, int Role);
extern int IcSetPCOName(int PCOD, char *Name);

/*-------------------------------------------------------------------------
 * Section: Utility stuff
 * 
 * Description:
 *  This section contains functions that are nice to have
 *-----------------------------------------------------------------------*/

extern VcValue *IcObject;
extern const char *IcVerdict2Str(int v);
extern int IcStr2Verdict(const char *str);
extern int IcInternalConcurrency(void);

/*--------------------------------------------------------------------
 * Section: Concurrency
 *
 *------------------------------------------------------------------*/

extern void (*TaskInit)(void);
extern void (*TaskCreate)(void *(*)(void *),int,char *,VcValue *,char *);
extern void (*TaskDelete)(void);
extern void (*TaskReschedule)(void);
extern void (*TaskCleanup)(void);

#endif
