// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Hello.h>

#include <stdlib.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    //
    // Get "hello" object
    //
    CORBA::Object_var obj = orb -> string_to_object("relfile:/Hello.ref");
    if(CORBA::is_nil(obj))
    {
	cerr << argv[0] << ": cannot read IOR from Hello.ref" << endl;
	return EXIT_FAILURE;
    }
    
    Hello_var hello = Hello::_narrow(obj);
    assert(!CORBA::is_nil(hello));
    
    //
    // Main loop
    //
     cout << "Enter 'h' for hello or 'x' for exit:\n";
    char c;
    do
    {
    cout << "> ";
    cin >> c;
    if(c == 'h')
     hello -> say_hello();
	    }
     while(cin.good() && c != 'x');

    hello -> lookForValue();
    hello -> returnDouble();
    hello -> returnLong();
    hello -> returnUlong();
    hello -> returnFloat();
    hello -> returnLonglong();
    hello -> returnUlonglong();
    hello -> returnChar();
    hello -> returnBoolean();
    hello -> returnShort();
    hello -> returnUshort();
    hello -> returnOctet();
    hello -> returnStruct();
    hello -> returnEnum();
    hello -> returnString();
    hello -> returnWstring();
    hello -> returnSequence();
    hello -> returnUnion();
    hello -> returnArray();
    
    //Pseudo-Object Types
    
    hello -> returnTKWstring();
    hello -> returnTKStruct();
    hello -> returnTKUnion();
    hello -> returnTKEnum();
    hello -> returnTKSequence();
    hello -> returnTKArray();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
