/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/

 /*********************************************************************
 *                                                                    *
 *    Name:   adPCO.cpp                                               *
 *                                                                    *
 *    Author: Mang Li                                                 *
 *            GMD Fokus, TIP                                          *
 *                                                                    *
 *    Description: This file contains the implementation of           *
 *        the class AdPCOList to support the local management of      *
 *        PCOs and CPs.                                               *
 *                                                                    *
 *********************************************************************/


#include <iostream.h>
#include <string.h>

// GCI header files
#include "gci.h"
#include "s_aux.h"

#include "adPCO.h"

extern int VERBOSITY;
extern void verbose_print(char* fmt);
#define VERBOSE(F)	verbose_print F

//---------------
// nameToId      |
//---------------

int AdPCOList::nameToId(const char* pcoName)
{
  int i;
  for(i = GcFirstPCOD; i < GcFirstPCOD + IcNoOfPCOs + IcNoOfCPs; i++) 
    if(!strcmp(GciGetPCOName(i), pcoName))
      return i;
  return -1;
}


//---------------
// Constructor   |
//---------------

AdPCOList::AdPCOList()
{
  theList_ = NULL;
}


//---------------
// Destructor    |
//---------------

AdPCOList::~AdPCOList()
{
  if (theList_ != NULL)
    {
      AdPCOStruct* aPCO = theList_;
      AdPCOStruct* tmp;
      
      for(; aPCO != NULL; aPCO = tmp)
        {
          tmp = aPCO->next;
          if (aPCO->pcoName != NULL)
            delete[] aPCO->pcoName;
          aPCO->next = NULL;
          delete aPCO;
        }
      theList_ = NULL;
    }
}


//---------------
// resetPCOs     |
//---------------

void AdPCOList::resetPCOs()
{
  if (theList_ != NULL)
    {
      AdPCOStruct* aPCO = theList_;
      AdPCOStruct* tmp;
      
      for(; aPCO != NULL; aPCO = tmp)
        {
          tmp = aPCO->next;
          if (aPCO->pcoName != NULL)
            delete[] aPCO->pcoName;
          aPCO->next = NULL;
          delete aPCO;
        }
      theList_ = NULL;
    }
  return;
}


//---------------
// addPCObyID    |
//---------------

void AdPCOList::addPCObyID(int pcod)
{
  AdPCOStruct* aPCO = theList_;
  for(; aPCO != NULL; aPCO = aPCO->next)
    {
      if (aPCO->pcoID == pcod)
        {
          if (VERBOSITY > 0)
            cout<<"AdPCOList::addPCObyID: The PCO exits already, "
                <<"it remains unchanged."
                << endl;
          return;
        }
    }
        
  aPCO = new AdPCOStruct;
  aPCO->pcoID = pcod;

  char* pcon = GciGetPCOName(pcod);
  if (pcon != NULL)
    {
      aPCO->pcoName = new char[strlen(pcon)+1];
      strcpy(aPCO->pcoName, pcon);
    }
  
  aPCO->next = theList_;
  theList_ = aPCO;
  if (VERBOSITY > 0)
    cout<<"AdPCOList::addPCObyID: A PCO named '" << pcon 
        <<"' is added to the PCO list." << endl;
}


//---------------
// addPCObyName  |
//---------------

void AdPCOList::addPCObyName(const char* pcoName)
{
  AdPCOStruct* aPCO = theList_;
  for(; aPCO != NULL; aPCO = aPCO->next)
    {
      if (strcmp(aPCO->pcoName, pcoName) == 0)
        {
          if (VERBOSITY > 0)
            cout<<"AdPCOList::addPCObyName: The PCO exits already, "
                <<"it remains unchanged."
                << endl;
          return;
        }
    }
        
  aPCO = new AdPCOStruct;
  aPCO->pcoID = -1; // Marks for "currently unknown"

  aPCO->pcoName = new char[strlen(pcoName)+1];
  strcpy(aPCO->pcoName, pcoName);
  
  aPCO->next = theList_;
  theList_ = aPCO;
  if (VERBOSITY > 0)
    cout<<"AdPCOList::aPCObyName: A PCO named '" << pcoName 
        <<"' is added to the PCO list." << endl;  
}


//---------------
// getPCOId      |
//---------------

int AdPCOList::getPCOId(const char* pcoName)
{
  // Get PCO descriptor of pcoID

  AdPCOStruct* aPCO = theList_;
  
  for (; aPCO != NULL; aPCO = aPCO->next)
    {
      if (strcmp(aPCO->pcoName, pcoName) == 0)
        {
          if (aPCO->pcoID != -1)
            return(aPCO->pcoID);
          else
            return(nameToId(aPCO->pcoName));
        }
    }
  return -1;
              
}


//---------------
// pcoIdReady    |
//---------------

int AdPCOList::pcoIsReady(const char* pcoName, int pcod)
{  
  AdPCOStruct* aPCO = theList_;
  for(; aPCO != NULL; aPCO = aPCO->next)
    {
      if ((aPCO->pcoID == pcod) || (strcmp(aPCO->pcoName,pcoName) == 0))
        {
          // Add the PCO ID if absent
          if (aPCO->pcoID == -1)
            aPCO->pcoID = pcod;
          return ISTRUE;
        }
    }
  return ISFALSE;
}
