/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/


/*********************************************************************
 *                                                                    *
 *    Name:   gwCoDec.h                                               *
 *                                                                    *
 *    Author: Mang Li                                                 *
 *            GMD Fokus, TIP                                          *
 *                                                                    *
 *    Description: This header file contains the declaration of       *
 *        the class GWCoDec to support encoding and decoding.         *
 *                                                                    *
 *                                                                    *
 *********************************************************************/

extern int VERBOSITY;
extern void verbose_print(char* fmt);
#define VERBOSE(F)	verbose_print F


#define IDLTYPE_PREFIX "IDL:"
#define IDLTYPE_SUFIX ":1.0"

// GCI SEQUENCE index begins with 1 instead of 0
#define GCI_SEQ_OFFSET 1

// Prefix for IDL types
#define BASIC_TYPE_PRE "CORBA__"
#define UNION_TYPE_PRE "UNION__"
#define VALUEBOX_TYPE_PRE "VALUEBOX__"
#define VALUE_TYPE_PRE "VALUETYPE__"
#define INTF_TYPE_PRE "INTF__"
#define ABSTRACT_INTF_TYPE_PRE "ABSTR_INTF__"
#define EXCEPTION_TYPE_PRE "EXCEPT__"
#define STRUCT_TYPE_PRE "STRUCT__"
#define ARRAY_TYPE_PRE "ARRAY__"
#define SEQ_TYPE_PRE "SEQ__"
#define ENUM_TYPE_PRE "ENUM__"

// Prefix for identification of ASP types
#define REQUEST_ASP "pCALL__"
#define REPLY_ASP "pREPLY__"
#define RAISE_ASP "pRAISE__"

#define MAX_NUM 5


class GWCoDec
{
private:
  CORBA::ORB_var orb_;
  CORBA::Repository_var repository_;
  DynamicAny::DynAnyFactory_var dynAnyFactory_;

public:
  GWCoDec(CORBA::ORB_ptr);
  ~GWCoDec();

private:

  // Construct the IDL Type - replace '__' by '/', add prefix 'IDL:' 
  // and sufix ':1.0'.
  char* getIDLType(const char*, char*);
  
  // Get the GCI type descriptor from the RepositoryId (TypeCode id)
  // e.g. 'IDL:TestInterface/BadInput:1.0'
  // and 'IDL:omg.org/CORBA/COMM_FAILURE:1.0'
  int typeDescFromTCid(CORBA::Any*);


  // Resolve the appropriate type descriptor from the type or tag
  // of the GciValue. decrs is the number to decrease from the
  // top level. typeName passes the name of the type in correspondence
  // with the returned descriptor. It can be different to the type
  // of value, in particular if decrs is not 0.
  char* typeDescFromGCIValue(int decrs, char*& typeName, GciValue* value);

  // Encode an ASP field to a CORBA::Any.
  // decrs is the number to decrease from the top level, when
  // determining the type definition
  CORBA::Any* encodeField(int decrs, GciValue* value);

  // Decode an ASP field from a CORBA::Any
  GciValue* decodeField(CORBA::Any*);

public:
  
  // Convert a GciValue to a GWMessage 
  GciStatus makeGWMessage(GciValue*, TOR::GWMessage*&);

  // Convert a GWMessage to a GciValue
  GciStatus makeGciObj(const TOR::GWMessage*, GciValue*&);

  // Register value factory
  void register_this_factory(const char* id);

  // Unregister value factory
  void unregister_this_factory(const char* id);
};

//find out which system exception was raised
void whichSysExcept(CORBA::SystemException* sx);

