/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/


/*********************************************************************
 *                                                                    *
 *    Name:   adTimer.cpp                                             *
 *                                                                    *
 *    Author: Mang Li                                                 *
 *            GMD Fokus, TIP                                          *
 *                                                                    *
 *    Description: This file contains the implementation of           *
 *        the class AdTimerList to support the local management of    *
 *        timers.                                                     *
 *                                                                    *
 *********************************************************************/


#include <iostream.h>
#include <string.h>
#include <sys/time.h>

// GCI header files
#include "gci.h"
#include "s_aux.h"

#include "adTimer.h"

// Indication of an error
#define ERROR_IND -1

extern int VERBOSITY;
extern void verbose_print(char* fmt);
#define VERBOSE(F)	verbose_print F


//---------------
// Constructor   |
//---------------

AdTimerList::AdTimerList()
{
  theList_ = NULL;
}


//---------------
// Destructor    |
//---------------

AdTimerList::~AdTimerList()
{
  if (theList_ != NULL)
    {
      AdTimerStruct* aTimer = theList_;
      AdTimerStruct* tmp;
      
      for(; aTimer != NULL; aTimer = tmp)
        {
          tmp = aTimer->next;
          aTimer->next = NULL;
          delete aTimer;
        }
      theList_ = NULL;
    }
}


//---------------
// resetTimers   |
//---------------

void AdTimerList::resetTimers()
{
  if (theList_ != NULL)
    {
      AdTimerStruct* aTimer = theList_;
      AdTimerStruct* tmp;
      
      for(; aTimer != NULL; aTimer = tmp)
        {
          tmp = aTimer->next;
          aTimer->next = NULL;
          delete aTimer;
        }
 
      theList_ = NULL;
    }
  return;
  
}



//---------------
// addTimerbyID  |
//---------------

GciStatus AdTimerList::addTimerbyID(int tid, long timerval, int unit)
{
  AdTimerStruct* aTimer = theList_;
  for(; aTimer != NULL; aTimer = aTimer->next)
    {
      if (aTimer->timerID == tid)
        {
          aTimer->duration = timerval;
          aTimer->unit = unit; 
          if(gettimeofday (&aTimer->startTime, NULL) == 0)
            {
              aTimer->status = RUNNING;
              if (VERBOSITY > 0)
                cout<<"AdTimerList::addTimerbyID: The timer exits already, "
                    <<"startTime updated."
                    << endl;
              return  GciOk;
            }
          else
            {
              if (VERBOSITY > 0)
                cout<<"adTimer:addTimerbyID: gettimeofday() failed." << endl;
              return GciNotOk;
            }
        }
    }
        
  aTimer = new AdTimerStruct;
  aTimer->timerID = tid;
  aTimer->unit = unit;
  aTimer->duration = timerval;
  if( gettimeofday (&aTimer->startTime, NULL) == 0)
    {
      aTimer->status = RUNNING;
      aTimer->next = theList_;
      theList_ = aTimer;
      if (VERBOSITY > 0)
        cout<<"AdTimerList::addTimerbyID: A Timer named '" 
            << GciGetTimerName(tid) 
            <<"' is added to the Timer list." << endl;
      return GciOk;
    }
  else
    {
      if (VERBOSITY > 0)
        cout<<"adTimer:addTimerbyID: gettimeofday() failed." << endl;
      return GciNotOk;
    }
  
}


//-------------
// stopTimer   |
//-------------

GciStatus AdTimerList::stopTimer(int tid)
{
  AdTimerStruct* aTimer = theList_;
  for(; aTimer != NULL; aTimer = aTimer->next)
    {
      if (aTimer->timerID == tid)
        {
          aTimer->status = STOPPED;
          if (VERBOSITY > 0)
            cout<<"adTimer: stopTimer: the timer '"
                << GciGetTimerName(tid) <<"' is stopped." << endl;
          return GciOk;
        }
    }
  if (VERBOSITY > 0)
    cout<<"adTimer: stopTimer: the timer '"
        << GciGetTimerName(tid) <<"' is not in the timer list." << endl;
  return GciNotOk;
}
      

//-------------
// readTimer   |
//-------------

long AdTimerList::readTimer(int tid)
{
  AdTimerStruct* aTimer = theList_;
  timeval tv;
  long s, us;
  long value;
  
  if(gettimeofday (&tv, NULL) == 0)
    {
      s = tv.tv_sec;
      us = tv.tv_usec;
      if (us < 0)
        {
          us += 1000000;
          s -= 1;
        }
      
      for(; aTimer != NULL; aTimer = aTimer->next)
        {
          if (aTimer->timerID == tid)
            {
              if (aTimer->status == RUNNING)
                {
                  switch(aTimer->unit)
                    {
                    case Vcs:
                      {
                        value = s - aTimer->startTime.tv_sec
                          + (us - aTimer->startTime.tv_usec) /1000000;
                        break;
                      }
                    case Vcms:
                      {
                        value = (s - aTimer->startTime.tv_sec)*1000
                          + (us - aTimer->startTime.tv_usec) /1000;
                        break;
                      }
                    case Vcus:
                      {
                        value = (s - aTimer->startTime.tv_sec)*1000000
                          + (us - aTimer->startTime.tv_usec);
                        break;
                      }
                    default:
                      {
                        if (VERBOSITY > 0)
                          cout<<"adTimer: readTimer: "
                              <<"the unit is not s, us!" << endl;
                        value = ERROR_IND;
                      }
                    }
                  return value;
                }
              else
                {
                  if (VERBOSITY > 0)
                    cout<<"adTimer: readTimer: "
                        <<"the timer '" << GciGetTimerName(tid)
                        <<"' is not running." << endl;
                  return ERROR_IND;
                }
            }
        }
      if (VERBOSITY > 0)
        cout<<"adTimer: readTimer: the timer '"
            << GciGetTimerName(tid) <<"' is not in the timer list." << endl;
      return ERROR_IND;
    }
  else
    {
      if (VERBOSITY > 0)
        cout<< "adTimer: readTimer: gettimeofday failed."<< endl;
      return ERROR_IND;
    }
}


//-------------
// timeout     |
//-------------


int AdTimerList::timeout()
{  
  AdTimerStruct* aTimer = theList_;

  timeval tv;
  long s, us;
  int tid;
    
  if(gettimeofday (&tv, NULL) == 0)
    {
      s = tv.tv_sec;
      us = tv.tv_usec;
      if (us < 0)
        {
          us += 1000000;
          s -= 1;
        }
      
      for(; aTimer != NULL; aTimer = aTimer->next)
        {
          if (aTimer->status == RUNNING)
            {
              long tmp;
              
              switch(aTimer->unit)
                {
                case Vcs:
                  {
                    tmp = s - aTimer->startTime.tv_sec
                      + (us - aTimer->startTime.tv_usec) /1000000;
                    break;
                  }
                case Vcms:
                  {
                    tmp = (s - aTimer->startTime.tv_sec)*1000
                      + (us - aTimer->startTime.tv_usec) /1000;
                    break;
                  }
                case Vcus:
                  {
                    tmp = (s - aTimer->startTime.tv_sec)*1000000
                      + (us - aTimer->startTime.tv_usec);
                    break;
                  }
                default:
                  {
                    if (VERBOSITY > 0)
                      cout<<"adTimer: timeout: "
                          <<"the unit is not s, us!" << endl;
                    return ERROR_IND;
                  }
                }
              
              if(tmp >= aTimer->duration)
                {
                  tid = aTimer->timerID;
                  aTimer->status = STOPPED;
                  
                  if (VERBOSITY > 0)
                    cout<<"adTimer: timeout for timer '" 
                        <<GciGetTimerName(tid) <<"'" << endl;
                  return tid;
                }
            }
        }
      tid = 0;
      return tid;
    }
  else
    {
      if (VERBOSITY > 0)
        cout<< "adTimer: timeout: gettimeofday failed."<< endl;
      return ERROR_IND;
    }
  
}

