/*-------------------------------------------------------------------------
 * Name:   gci.h
 * Author: ITEX C-Code Generator
 *
 * Copyright 1991-1999 Telelogic AB, All rights reserved.
 *
 * This Program is owned by Telelogic and is protected by national
 * copyright laws and international copyright treaties. Telelogic
 * grants you the right to use this Program on one computer or in one
 * local computer network at any one time.  Under this License you may
 * only modify the source code for the purpose of adapting it to your
 * environment. You must reproduce and include any copyright and
 * trademark notices on all copies of the source code.  You may not
 * use, copy, merge, modify or transfer the Program except as provided
 * in this License.  Telelogic does not warrant that the Program will
 * meet your requirements or that the operation of the Program will be
 * uninterrupted and error free. You are solely responsible that the
 * selection of the Program and the modification of the source code
 * will achieve your intended results and that the results are
 * actually obtained.
 */

#ifndef GCISOFT_H
#define GCISOFT_H

#define CCG_VERSION 116

#include "static.h"
#include "valrep.h"

/*-------------------------------------------------------------------------
 *	The Timer Units.
 */
typedef enum {
  Gcimin = 60,	/* Minutes */
  Gcis = 0,	/* seconds */
  Gcims = -3,	/* milli seconds */
  Gcius = -6,	/* micro seconds */
  Gcins = -9,	/* nano seconds */
  Gcips = -12	/* pico seconds */
} GciTimeUnit;

typedef int GciPCOID;

typedef struct GciAddress {
  int     type;        /*  Address type 0 - 100 is reserved by Telelogic */
  char*   buffer;      /* The address stored in a buffer */
} GciAddress;

typedef int GciTimerID;

/***********************************************************************
  Structure for our timer representation. This should probably not 
  be located here
  */
typedef enum{ 
    GCIRUNNING, 
    GCISTOPPED, 
    GCIEXPIRED 
} GciTimerStatus;

typedef struct GciTime {
  unsigned long        time_val;
  GciTimeUnit unit;
} GciTime;

/*-------------------------------------------------------------------------
  Position pointer used by iterators
  */
typedef void* GciPosition;

/*-------------------------------------------------------------------------
 *	Concurrent Test Component Types.
 */

#define GciMTC                    0	/* Master Test Component */
#define GciPTC                    1	/* Paralell Test Component */


/*-------------------------------------------------------------------------
 *	The Status Type.
 */

typedef enum
  {
    GciNotOk = 0,
    GciOk = 1
  }
GciStatus;


/*-------------------------------------------------------------------------
 *	The Verdict Type returned by GciStart*.
 */

typedef enum
  {
    GciFail = 0,
    GciInConc = 1,
    GciPass = 2,
    GciNone = 3
  }
GciVerdict;


/*-------------------------------------------------------------------------
 *	The log event types for GciLog.
 */

typedef enum
  {
    GciLogStartTC = 100,
    GciLogStopTC = 101,
    GciLogStartDEF = 102,
    GciLogStopDEF = 103,
    GciLogVerdict = 104,
    GciLogPVerdict = 105,
    GciLogMatch = 106,
    GciLogNOMatch = 107,
    GciLogSendE = 108,
    GciLogRecE = 109,
    GciLogOtherE = 110,
    GciLogTimeoutE = 111,
    GciLogAssign = 112,
    GciLogStartT = 113,
    GciLogStopT = 114,
    GciLogCancelT = 115,
    GciLogReadT = 116,
    GciLogAttach = 117,
    GciLogImplSend = 118,
    GciLogGoto = 119,
    GciLogRec = 120,
    GciLogTimeout = 121
  }
GciLogValues;


/*-------------------------------------------------------------------------
 *	The GciValue is implemented by an internal type. Only refer to
 *	GciValue.
 */

typedef VcValue GciValue; /* Regard this as 'void', never 'VcValue'!! */


/*-------------------------------------------------------------------------
 *	Related type:
 */

typedef VcReal	GciReal;


/*-------------------------------------------------------------------------
 *	Used by GciGetTestCaseList and GciGetTestCaseGroupList.
 */

typedef char *	GciTCList;


/*-------------------------------------------------------------------------
 *	Constructors and access methods for (atomic) GciValues.
 */
#ifdef __cplusplus
extern "C" {
#endif

GciValue *	GciMkINTEGER(long);
long		GciGetINTEGER(GciValue *);
GciValue *	GciMkBOOLEAN(int);
Bool		GciGetBOOLEAN(GciValue *);
GciValue *	GciMkREAL(int, int, int);
GciReal *	GciGetREAL(GciValue *);
GciValue *	GciMkBIT_STRING(const char *);
char *		GciGetBIT_STRING(GciValue *);
GciValue *	GciMkHEXSTRING(const char *);
char *		GciGetHEXSTRING(GciValue *);
GciValue *	GciMkOCTET_STRING(const char *);
char *		GciGetOCTET_STRING(GciValue *);
GciValue *	GciMkNumericString(const char *);
char *		GciGetNumericString(GciValue *);
GciValue *	GciMkPrintableString(const char *);
char *		GciGetPrintableString(GciValue *);
GciValue *	GciMkTeletexString(const char *);
char *		GciGetTeletexString(GciValue *);
GciValue *	GciMkVideotexString(const char *);
char *		GciGetVideotexString(GciValue *);
GciValue *	GciMkVisibleString(const char *);
char *		GciGetVisibleString(GciValue *);
GciValue *	GciMkIA5String(const char *);
char *		GciGetIA5String(GciValue *);
GciValue *	GciMkT61String(const char *);
char *		GciGetT61String(GciValue *);
GciValue *	GciMkISO646String(const char *);
char *		GciGetISO646String(GciValue * value);
GciValue *	GciMkGraphicString(const char *);
char *		GciGetGraphicString(GciValue *);
GciValue *	GciMkGeneralString(const char *);
char *		GciGetGeneralString(GciValue *);
GciValue *	GciMkGeneralizedTime(char *);
char *		GciGetGeneralizedTime(GciValue *);
GciValue *	GciMkUTCTime(char *);
char *		GciGetUTCTime(GciValue *);


/*-------------------------------------------------------------------------
 *	Constructors and access methods for (special) GciValues.
 */

GciValue *	GciMkENUMERATED(int);
int		GciGetENUMERATED(GciValue *);
GciValue *	GciMkCHOICE(char *, GciValue *);
GciValue *	GciGetCHOICE(GciValue *);
char *		GciGetCHOICEName(GciValue * value);

GciValue *	GciMkObjectDescriptor(const char *);
char *		GciGetObjectDescriptor(GciValue *);
GciValue *	GciMkNULL(void);
int		GciGetNULL(GciValue *);
GciValue *	GciMkANY(GciValue *);
GciValue *	GciGetANY(GciValue *);
GciValue *	GciMkR_TYPE(int);
int		GciGetR_TYPE(GciValue *);
GciValue *	GciMkPDU(GciValue *);
GciValue *	GciGetPDU(GciValue *);

GciValue *  GciMkOBJECT_IDENTIFIER();
GciValue *  GciAddOBJECT_IDENTIFIERComponent(GciValue*, int comp); 
int         GciOBJECT_IDENTIFIERSize(GciValue*);
int         GciGetOBJECT_IDENTIFIERComponent(GciValue*, int index);

/*-------------------------------------------------------------------------
 *	Constructors and access methods for (composite) GciValues.
 */

GciValue *	GciMkSEQUENCE(int);
GciValue *	GciSetField(GciValue *, int, GciValue *);
GciValue *	GciGetField(GciValue *, int);
int		GciSeqSize(GciValue *);
GciValue *	GciMkSEQUENCEOF(void);
GciValue *	GciAddElem(GciValue *, GciValue *);
GciValue *	GciGetElem(GciValue *, int);
int		GciSeqOfSize(GciValue *);
GciValue *      GciMkSET(int);
GciValue *      GciMkSETOF(void);


/*-------------------------------------------------------------------------
 *	Declarations of functions provided by ITEX/CCG.
 */

GciStatus	GciInit(void);
GciStatus	GciExit(void);
GciStatus	GciReceive(int, GciValue *);
int       	GciIsSelected(const char *);
GciVerdict	GciStartTestCase(const char *);
GciTCList *	GciGetTestCaseList(void);
GciTCList *	GciGetTestCaseGroupList(void);
GciValue *	GciGetValue(const char *);
GciStatus	GciTimeout(int TimerD);
GciStatus	GciDone(int PTCD);


/*-------------------------------------------------------------------------
 *	Type, Tag and Class retrieval and manipulation functions.
 */

int		GciGetType(GciValue *);
int		GciGetBaseType(GciValue * );
GciValue *	GciSetType(int, GciValue *);
GciValue *	GciTagType(char, int, Bool, GciValue *);
int		GciGetTag(GciValue *, int);
char		GciGetClass(GciValue *, int);
Bool		GciGetImplicit(GciValue *, int);
const char *	GciGetTypeNameString(int);
int		GciGetTypeDescriptor(const char *);

int             GciGetOMIT(GciValue *value);

/*-------------------------------------------------------------------------
 *	Retrieval of some static attributes from the TTCN suite, and also
 *	index/descriptor translation.
 */

char *		GciGetTimerName(int TimerD);
char *		GciGetPCOName(int PCOD);

int		GciGetNoOfTimers();
int		GciGetNoOfPCOs();
int		GciGetPCO( int index );
int		GciGetPCOIndex( int desc );
int		GciGetTimer( int index );
int		GciGetTimerIndex( int desc );


/*-------------------------------------------------------------------------
 *	Functions and types used for implementation of concurrent TTCN,
 *	These are not needed for non-concurrent TTCN.
 */

typedef IcComponent **GciConf;
typedef IcComponent GciComponent;

GciVerdict	GciStartTestComponent(VcString TreeName, GciValue * args);

int		GciGetNoOfComponents(GciConf conf);
GciComponent *	GciGetComponent(GciConf conf, int index);
char *		GciGetComponentName(GciComponent * comp);
int		GciGetComponentDescriptor(GciComponent * comp);
int		GciGetComponentType(GciComponent * comp);
char **		GciGetComponentCPs(GciComponent * comp);
char **		GciGetComponentPCOs(GciComponent * comp);

/* Should be in static.h */
extern void IcStatusCreate(GciConf config);

 extern GciStatus GciLog( int logId, char* logString );

#ifdef __cplusplus
}
#endif

#endif /* GCISOFT_H */
