/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/


/*-----------------------------------------------------------------
 * File Name: gwAdaptor.c
 * Author: ITEX C Code Generator 
 *         + Mang Li (GMD FOKUS, TIP)
 * 
 * Description:
 *  If this file has been generated, the user didn't have an adaptor file
 *  before. The file is generated for the user as a template to allow the
 *  compilation to go through, but also to show the user the functions that
 *  need to be implemented. Observe that the user only needs to implement
 *  the functions he/she knows he/she will use. If for example a test suite
 *  doesn't use timers, there is no need to implement the timer functions.
 *  If a function is called which the user has not implemented a message
 *  will be printed to inform the user about this.
 * 
 *----------------------------------------------------------------*/

#ifdef USE_TTMAN
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <string.h>

#include "if_ttman.h"

char tm_usgtxt[] =
"GCI-ETS (GW) together with TSP1-FE\n"
"Usage: ttcnsrv [-h] [-v...] tc-name conf-name"
"\n"
"	-h		print this info\n"
"	-v		verbose, more v's give more verbosity\n"
"	tc-name		name of the test component \n"
"	conf-name	name of the test configuration\n";

static FIFO_t fifo;
static int 	TestCaseRun=FALSE;
static int	parametersReadIn=FALSE;
static int      cancel_exec_flag=FALSE; 
static int	error_code;
static char*    RunningTestCaseName;
static AsnList	myParam;
char	*conf_name;
#endif

#include <iostream.h>
#include <fstream.h>
#include <math.h>

#ifdef LINUX
#include <getopt.h>
#endif

#ifdef UNIX
#include <stdarg.h>
#endif

#ifdef WIN32
#include <sys/timeb.h>
#endif
 
#include <orb_gw.h>
#include <Gateway.h>
#include <obv_test_impl.h>

// Adapted header files
#include "gci.h"
#include "valrep.h"

#include "s_type.h"
#include "s_decl.h"

#include "s_constr.h"
#include "s_dyn.h"
#include "s_aux.h"

#include "adPCO.h"
#include "adTimer.h"
#include "gwCoDec.h"
#include "gwAdaptor.h"
  
int VERBOSITY=0;
#define DO_VERBOSE
#ifdef DO_VERBOSE

void verbose_print(int level, char *fmt,...)
 {
	va_list         ap;
	if (VERBOSITY >= level) {
		va_start(ap, fmt);
		//vinform(fmt, ap);
		cout << fmt << endl;
		va_end(ap);
	}
};
#define VERBOSE(F)	verbose_print F

#define CHECK(C,F)	if(!(C)) verbose_print F
#define VERBOSE_PRINT(L,N,F)
#define BUG(F)		bug F
#define WARN(F) 	warn F
#define PERROR(F) 	error F
#define MKSTR(S)	#S

#endif

// Indication of an error
#define WRONG_OBJ_REF "-1"
#define ERROR_IND -1

#define PAR_FILE_NAME "TSPar"
#define REFFILE_SUFFIX ".ref"
#define LOG_FILE_NAME "ETS.log"

extern void whichSysExcept(CORBA::SystemException* sx);


//---------------
// Variables     |
//---------------

// CORBA ORB
CORBA::ORB_var orb_;

// GatewayMain
TOR::GatewayMain_var gwm_;

// PCO list
AdPCOList* pcoList_;

// Timer list
AdTimerList* timerList_;

// CoDec
GWCoDec* gwCoDec_;

// For using MSC editor to print logs
//int useMSC = FALSE;

//---------------
// Help function |
//---------------

// Get object reference from file
CORBA::Object_ptr AdGetRefFromFile(const char* objName)
{
  CORBA::String_var refFile = CORBA::string_dup(objName);
  refFile += REFFILE_SUFFIX;
  if (VERBOSITY > 0)
    cout << "AdGetRefFromFile: Get object reference from file '"
         << refFile << "'" << endl;
  
  ifstream in;
  in.open(refFile);
  assert(!in.fail());
      
  char implStr[1000];
  in >> implStr;
  in.close();
  CORBA::Object_var obj = orb_->string_to_object(implStr);
  
  if (!CORBA::is_nil(obj))
    return CORBA::Object::_duplicate(obj);
  else
    return CORBA::Object::_nil();
  
}

void AdCleanUp()
{
  timerList_->resetTimers();
  pcoList_->resetPCOs();
  gwm_->GWDestroy();
}

//-----------------
// TTman adaptation|
//-----------------

#ifdef USE_TTMAN

/*
 * remove  all its nodes, and the the dataElemnts. 
 * This is taken from the snacc distribution and modified
 */
void SetOctet(AsnOcts * F, char *s)
{
  if (ASNOCTS_PRESENT(F)) {
    FreeAsnOcts(F);
  }
  F->octs = (char *) Asn1Alloc(strlen(s) + 1);
  strcpy(F->octs, s);
  F->octetLen = strlen(F->octs);
};

void AsnListClear(AsnList * L)
{
  AsnListFirst(L);
  while (L->count > 0) {
    Asn1Free(CURR_LIST_ELMT(L));
    AsnListRemove(L);
  }
};

void CopyParam(Param * FROM, Param * TO)
{
  Param          *tmpParam;
  tmpParam =  (Param *) Asn1Alloc(sizeof(Param));
  SetOctet(&(TO)->param_id, FROM->param_id.octs);
  SetOctet(&(TO)->param_value, FROM->param_value.octs);
};

void CopyParamList(AsnList * FROM, AsnList * TO)
{
  Param          *tmpParam;
  Param         **H;

  for (FROM->curr = FROM->first; 
       (FROM->curr &&  (tmpParam = (Param *) FROM->curr->data)); 
       FROM->curr = FROM->curr->next)
    {    
      H = (Param **) AsnListAppend(TO);
      *H = (Param *) Asn1Alloc(sizeof(Param));
      CopyParam(tmpParam, *H);
    }
};

#define SEND_TPI(PDU)				\
   tpi_outfunc((PDU));				\
  if((PDU))  FreeTPI_PDUs((PDU));

# define SEND_TPI_FINAL_VERDICT(VERDICT, ERROR)					\
{										\
	TPI_PDUs *ppdu;								\
	ppdu= (TPI_PDUs *) Asn1Alloc(sizeof(TPI_PDUs));					\
	ppdu->choiceId = TPI_PDUs::TPI_PDUS_TPI_FINAL_VERDICT;				\
	ppdu->a.tpi_final_verdict = (TPI_FINAL_VERDICT_Body*) Asn1Alloc(sizeof(TPI_FINAL_VERDICT_Body));	\
	ppdu->a.tpi_final_verdict->verdict_value = (VerdictValue) (VERDICT);	\
	ppdu->a.tpi_final_verdict->error_code = (FeErrorCode) (ERROR);		\
	SEND_TPI(ppdu);								\
};

# define SEND_TPI_CREATED(ERROR)					\
{									\
	TPI_PDUs *ppdu;							\
	ppdu= (TPI_PDUs *) Asn1Alloc(sizeof(TPI_PDUs));				\
	ppdu->choiceId = TPI_PDUs::TPI_PDUS_TPI_CREATED;				\
	ppdu->a.tpi_created = (TPI_CREATED_Body*) Asn1Alloc(sizeof(TPI_CREATED_Body));	\
	ppdu->a.tpi_created->error_code = (FeErrorCode) (ERROR);	\
	SEND_TPI(ppdu);							\
};

# define SEND_TPI_CANCELED(ERROR)					\
{									\
	TPI_PDUs *ppdu;							\
	ppdu= (TPI_PDUs *) Asn1Alloc(sizeof(TPI_PDUs));				\
	ppdu->choiceId = TPI_PDUs::TPI_PDUS_TPI_CANCELED;				\
	ppdu->a.tpi_canceled = (TPI_CANCELED_Body*) Asn1Alloc(sizeof(TPI_CANCELED_Body));	\
	ppdu->a.tpi_canceled->error_code = (FeErrorCode) (ERROR);	\
	SEND_TPI(ppdu);							\
};

# define SEND_TPI_CREATE(TCOID, TEST_CASE_ID, TREE_ID, PARAM)		\
{									\
	TPI_PDUs *ppdu;							\
	Param         **H;						\
									\
	ppdu= (TPI_PDUs *) Asn1Alloc(sizeof(TPI_PDUs));				\
	ppdu->choiceId = TPI_PDUs::TPI_PDUS_TPI_CREATE;				\
	ppdu->a.tpi_create = Asn1Alloc(sizeof(TPI_CREATE_Body));	\
	SetOctet(&ppdu->a.tpi_create->tco_id, (TCOID));			\
	SetOctet(&ppdu->a.tpi_create->test_case_id, (TEST_CASE_ID));	\
	SetOctet(&ppdu->a.tpi_create->tree, (TREE_ID));		\
        ppdu->a.tpi_create->param_list = AsnListNew(sizeof(void *));	\
/* Right now, all parameters are stored as list in the first parameter */  \
	H = (Param **) AsnListAppend(ppdu->a.tpi_create->param_list);		\
	*H = (Param *) Asn1Alloc(sizeof(Param));					\
	SetOctet(&(*H)->param_id,"");					\
	SetOctet(&(*H)->param_value, (PARAM));				\
	SEND_TPI(ppdu);							\
};

# define SEND_TPI_INFO(TCOID, CP_ID, COORD_MSG)				\
{									\
	TPI_PDUs *ppdu;							\
									\
	ppdu= (TPI_PDUs *) Asn1Alloc(sizeof(TPI_PDUs));				\
	ppdu->choiceId = TPI_PDUs::TPI_PDUS_TPI_INFO_REQ;				\
	ppdu->a.tpi_info_req = Asn1Alloc(sizeof(TPI_INFO_REQ_Body));	\
	SetOctet(&ppdu->a.tpi_info_req->tco_id, (TCOID));		\
	SetOctet(&ppdu->a.tpi_info_req->cp_id, (CP_ID));		\
	SetOctet(&ppdu->a.tpi_info_req->msg, (COORD_MSG));		\
	SEND_TPI(ppdu);							\
};

/*--------------------------------------------------------------------- 
 *
 * Name: openFIFO
 *
 * Description:
 *  Open the named pipes (aka FIFO) created by the parent
 *  e.g. FIFO_NAME = /tmp/tsp1MTC.3456_out
 *  Note: in and out refer to the parent, hence reading from "*_out" and
 *  writing to "*_in"
 * 
 * Return Value: int
 *	 0 	if ok
 *	-1	else
 */
int openFIFO()
{
  int length;
  char str_pid[20];
  sprintf(str_pid,".%d",(int) getpid());
  
  length = strlen(LOCATION)+strlen(fifo.myTcoId)+strlen("_out")+strlen(str_pid);
  
  if(length > MAX_STR_LEN) {
    BUG(("The FIFO-name is too long. total_length %d vs. MAX_STR_LEN %d. \n"
	 "Aborting...", length, MAX_STR_LEN));
  }
  
  strcpy(fifo.fifo_out,"");
  strcat(fifo.fifo_out, LOCATION);
  strcat(fifo.fifo_out, fifo.myTcoId);
  strcat(fifo.fifo_out, str_pid);
  strcpy(fifo.fifo_in, fifo.fifo_out);
  strcat(fifo.fifo_in, "_out");
  strcat(fifo.fifo_out, "_in");
  
  
  if((fifo.readfd = open(fifo.fifo_in,0)) < 0 ) {
    PERROR(("can't open FIFO for reading"));
    perror("eTC terminating");
    exit(-1);
  }
  if((fifo.writefd = open(fifo.fifo_out,1)) < 0) {
    PERROR(("can't open FIFO for writing"));
    perror("eTC terminating");
    exit(-1);
  }
  
  return(0);
};

/*--------------------------------------------------------------------- 
 *
 * Name: fifo_outfunc
 *
 * Description:
 *	Function is called either from the tpi_outfunc or the gci_outfunc.
 *	It builds the message, tags the type and send it across the 
 * 	Fifo
 *
 * Return Value:  = 0 - on success
 *	         != 0 - else
 */
int fifo_outfunc(int type, char *buf, int len)
{
  Mesg 	*mesg;
  /* Build a message suitable for transmission across the FIFO*/
  mesg = (Mesg *) malloc(sizeof(Mesg));
  mesg->mesg_len = len;
  mesg->mesg_type = type;
  if( len > MAXMESGDATA ) {
    PERROR(("Data to transmit over FIFO too long. MAXMESGDATA = %d",MAXMESGDATA));
    return(-1);
  }
  memcpy(mesg->mesg_data, buf, mesg->mesg_len);
  /* Write the PDU on the line */
  if (mesg_send(fifo.writefd, mesg) < 0) {
    perror("write");
    return(-1);
  }
  free(mesg);
  return(0);
  
}

/*--------------------------------------------------------------------- 
 *
 * Name: fifo_infunc
 *
 * Description:
 *	Function is called via poll_dispatch. Some fifo_mesg are treated immediately
 *	in this function other signals, like coordination messages are queued up 
 *
 * Return Value: void
 */

void fifo_infunc(int fd, int mask, void *vtpi_infunc)
{
  Mesg 		*rcvBuf;
  int		size;
  
  /* no input paramters are expected, therefore ignoring varg */
  
  rcvBuf = (Mesg *) malloc(MAXMESGDATA+MESGHDRSIZE);
  if ((size =   mesg_recv(fd, rcvBuf, DATAGRAM_SERVICE)) < 0) {	
    PERROR (("eTC: fifo_infunc - read error %s",fifo.myTcoId));
    return;
  }
  if (!size) {
    /* The FIFO was closed at the other end.
       It looks like the FrontEnd does not exist anymore 
       Therefore .. */     
    poll_unregister(fifo.readfd);
    /* Close the fd's */
    close(fifo.readfd);
    close(fifo.writefd);
    /* Terminate */
    exit(0);
  }
  VERBOSE((3,"eTC %s: Received something on the FIFO", fifo.myTcoId));
  /* 1st check whether GciPDU or TpiPdu */
  if(rcvBuf->mesg_type == TPI_PDU) {
    TPI_PDUs 	decoded_pdu;
    jmp_buf 	env;
    int 		decode_err;
    AsnLen 		decoded_len;
    SBuf 		buf;
    int 		val;
    
    SBufInstallData (&buf, (char *) rcvBuf->mesg_data, rcvBuf->mesg_len);
    decoded_len = 0;
    decode_err = FALSE;
    if ((val = setjmp (env)) == 0) {
      BDecTPI_PDUs (&buf, &decoded_pdu, &decoded_len, env);
    } else {
      decode_err = TRUE;
      VERBOSE((1,"eTC %s: ERROR - TPI-Decode routines returned %d\n", fifo.myTcoId, val));
    }
    /* processing of tpi_pdu is done here */
    tpi_infunc(&decoded_pdu); /* Calling the appropriate tpi_infunc */
  } else if (rcvBuf->mesg_type == GCI_PDU) {
    /* The only GCI-PDU defined for now is the GCI-SET-PARAMETER Pdu */
    GCI_PDUs 	decoded_pdu;
    jmp_buf 	env;
    int 		decode_err;
    AsnLen 		decoded_len;
    SBuf 		buf;
    int 		val;
    
    SBufInstallData (&buf, (char *) rcvBuf->mesg_data, rcvBuf->mesg_len);
    decoded_len = 0;
    decode_err = FALSE;
    if ((val = setjmp (env)) == 0) {
      BDecGCI_PDUs (&buf, &decoded_pdu, &decoded_len, env);
    } else {
      decode_err = TRUE;
      VERBOSE((1,"eTC %s: ERROR - Decode routines returned %d\n", fifo.myTcoId, val));
    }
    /* processing of tpi_pdu is done here, because only one PDU is defined */
    if (decoded_pdu.choiceId != GCI_PDUs::GCI_PDUS_GCI_SET_PARAMETER) {
      WARN(("eTC %s: GCI-PDU is not GCI-SET-PARAMETER and therefore not supported",fifo.myTcoId));	
    }
    AsnListClear(&myParam);
    CopyParamList(decoded_pdu.a.gci_set_parameter->param_list, &myParam);
    FreeGCI_PDUs(&decoded_pdu);
    parametersReadIn = TRUE;
  } else {
    VERBOSE((1,"eTC %s: Received something on FIFO %s which is neither TPI-PDU nor GCI-PDU",fifo.myTcoId, fifo.fifo_in));
  }
  return;
}
/*
 * tpi_out -
 * 
 * This function is called whenever a tpi_pdu 
 * should be transmitted.
 */

void tpi_outfunc(TPI_PDUs *pdu)
{
  char            data[MAXMESGDATA];
  SBuf            outputBuf;
  unsigned long int encodedLen;
  
  /* Convert the TPI_PDU to a buffer. */
  SBufInit(&outputBuf, data, sizeof(data));
  SBufResetInWriteRvsMode(&outputBuf);
  /* encode the PDU. The result is in outputBuf */
  encodedLen = BEncTPI_PDUs(&outputBuf, pdu);
  /* after encoding a value ALWAYS check for write error in the buffer. */
  if ((encodedLen <= 0) || (SBufWriteError(&outputBuf))) {
    panic(("failed to encode TPI_PDU"));
  }
  
  fifo_outfunc(TPI_PDU, SBufDataPtr(&outputBuf), SBufDataLen(&outputBuf));
  return;
};

/*--------------------------------------------------------------------- 
 *
 * Name: tpi_infunc
 *
 * Description:
 * 	Processing of an incoming TPI-PDU
 * 
 * Return Value: void */

void tpi_infunc(TPI_PDUs *m)
{
  int ver = 3; /* The verdict */
    
  switch(m->choiceId) 
    {
    case TPI_PDUs::TPI_PDUS_TPI_CREATE:	
      VERBOSE((3,"eTC %s: received TPI_CREATE", fifo.myTcoId));
      error_code = ERROK;
      cancel_exec_flag = FALSE;
      GciInit(); 	/* Hopefully the parameters are allready there */

      SEND_TPI_CREATED(error_code);
      if(error_code != ERROK) 
	{
	  VERBOSE((1,"eTC %s: Error %d occured in GciInit(). Aborting TC Creation. Waiting for new creation", fifo.myTcoId, error_code)); 
	  return;
	}
      if(RunningTestCaseName) 
	free(RunningTestCaseName);
      RunningTestCaseName = strdup(m->a.tpi_create->test_case_id.octs);
      if (strcmp(m->a.tpi_create->tree.octs, "")==0) 
	{
	  /* If the treeId is emtpy, then I've to be the MTC */
	  VERBOSE((3,"eTC %s: Tree of TPI_CREATE is empty. I've to be the MTC", fifo.myTcoId));
	  ver = GciStartTestCase(RunningTestCaseName);
	  SEND_TPI_FINAL_VERDICT(ver, ERROK);
      
	  /* Clean-up */
	  if(RunningTestCaseName) 
	    { 
	      free(RunningTestCaseName);
	      RunningTestCaseName = (char *) NULL;
	    }
	  TestCaseRun = TRUE;

	} 
      else 
	{ 
	  /* I'm a PTC */
	  /* C-TTCN unused */
	}
      AdCleanUp();
      break;
    
    case TPI_PDUs::TPI_PDUS_TPI_INFO_IND:
      /* C-TTCN unused 
      GciReceive(getPCOId(m->a.tpi_info_ind->cp_id.octs),GeneralStringDecode(m->a.tpi_info_ind->msg.octs));
      */
      break;
    case TPI_PDUs::TPI_PDUS_TPI_LAST_VERDICT:
      /* C-TTCN unused */
      break;
    case TPI_PDUs::TPI_PDUS_TPI_UPDATE_VERDICT:
      /* Not implemented in GCI */
      WARN(("Update Verdicts are not implemented in GCI"));
    break;
    
    case TPI_PDUs::TPI_PDUS_TPI_CANCEL:
      cancel_exec_flag = TRUE;
      SEND_TPI_CANCELED(ERROK);
      break;
    case TPI_PDUs::TPI_PDUS_TPI_UPDATED_VERDICT:
    case TPI_PDUs::TPI_PDUS_TPI_FINAL_VERDICT:
    case TPI_PDUs::TPI_PDUS_TPI_CANCELED:
    case TPI_PDUs::TPI_PDUS_TPI_INFO_REQ:
    case TPI_PDUs::TPI_PDUS_TPI_CREATED:
    default:	
      WARN(("eTC %s: This TPI-Message should not be received",fifo.myTcoId));
    }
};

#endif

//-----------------------
// Test suite operations |
//-----------------------

// getObjRef

GciValue* getObjRef(GciValue* objName)
{
  GciValue* theValue;
  GciValue* buf;
  CORBA::Object_var obj;
  const char* name = GciGetGraphicString(objName);

  obj = AdGetRefFromFile(name);
  if (!CORBA::is_nil(obj))
    {
      CORBA::String_var aStr = CORBA::string_dup(orb_->object_to_string(obj));
      buf = GciMkIA5String(aStr);
      theValue = GciSetType(GcCORBA__ObjectD, buf);
      if (VERBOSITY > 0)
        cout << "getObjRef: The reference of the object '" 
             << name << "' is ready." << endl;
    }
  else
    {
      buf = GciMkIA5String(WRONG_OBJ_REF); //Marks for wrong reference
      theValue = GciSetType(GcCORBA__ObjectD, buf);
      if (VERBOSITY > 0)
        cout << "getObjRef: Getting reference of the object '" 
             << name << "' is failed." << endl;
    }
  return(theValue);
}

// activatePCOtoSUTServer

GciValue* activatePCOtoSUTServer(GciValue* pcoName, 
                                 GciValue * objectName, GciValue* refStr)
{
  GciValue * theValue;
  CORBA::String_var aStr = CORBA::string_dup(GciGetIA5String(refStr));
  CORBA::String_var bStr = CORBA::string_dup(GciGetGraphicString(pcoName));
  
  if (strcmp(aStr, WRONG_OBJ_REF) != 0)
    {
      CORBA::Object_var obj = orb_->string_to_object(aStr);
      if(!CORBA::is_nil(obj))
        {
          TOR::GWConnInfo_var aConn = new TOR::GWConnInfo;
          aConn->connType = TOR::PCOtoSUTServer;
          aConn->pName = CORBA::string_dup(bStr);
          aConn->objInfo.objName = 
            CORBA::string_dup(GciGetGraphicString(objectName));
          aConn->objInfo.objRef = CORBA::Object::_duplicate(obj);
          
          if (gwm_->GWAddConnection(aConn) == TOR::GWOK)
            {
              pcoList_->addPCObyName(bStr);
              theValue = GciMkBOOLEAN(true);
              if (VERBOSITY > 0)
                cout <<"activatePCOtoSUTServer: A GatewayClient is connected"
                     <<" with the PCO '" << bStr <<"'" << endl;
            }
          else
            {
              theValue = GciMkBOOLEAN(false);
              if (VERBOSITY > 0)
                cout <<"activatePCOtoSUTServer: "
                     <<"Failed due to GWAddConnection()." << endl;
            }
        }
      else
        {
          theValue = GciMkBOOLEAN(false);
          if (VERBOSITY > 0)
            cout <<"activatePCOtoSUTServer: Failed due to null reference." 
                 << endl;
        }
    }
  else 
    {
      theValue = GciMkBOOLEAN(false);
      if (VERBOSITY > 0)
        cout <<"activatePCOtoSUTServer: Failed due to wrong reference string." 
             << endl;
    }
  return(theValue);
}


// activatePCOtoSUTClient
 
GciValue* activatePCOtoSUTClient(GciValue * pcoName, 
                                 GciValue * objectName, GciValue * ref)
{
  GciValue* theValue;
  GciValue* buf;
  CORBA::String_var bStr = CORBA::string_dup(GciGetGraphicString(pcoName));
  CORBA::String_var cStr = CORBA::string_dup(GciGetGraphicString(objectName));

  TOR::GWConnInfo_var aConn = new TOR::GWConnInfo;
  aConn->connType = TOR::PCOtoSUTClient;
  aConn->pName = CORBA::string_dup(bStr);
  aConn->objInfo.objName = CORBA::string_dup(cStr);
  aConn->objInfo.objRef = CORBA::Object::_nil();
          
  if (gwm_->GWAddConnection(aConn) == TOR::GWOK)
    {
      CORBA::Object_var obj = AdGetRefFromFile(cStr);

      if (!CORBA::is_nil(obj))
        {
          CORBA::String_var aStr = 
            CORBA::string_dup(orb_->object_to_string(obj));
          buf = GciMkIA5String(aStr);
          ref = GciSetType(GcCORBA__ObjectD, buf);
          theValue = GciMkBOOLEAN(true);
          pcoList_->addPCObyName(bStr);
          if (VERBOSITY > 0)
            cout <<"activatePCOtoSUTClient: A GatewayServer is connected"
                 <<" with the PCO '" << bStr <<"'" << endl;
        }
      else
        {
          buf = GciMkIA5String(WRONG_OBJ_REF); //Marks for wrong reference
          ref = GciSetType(GcCORBA__ObjectD, buf);
          theValue = GciMkBOOLEAN(false);
          if (VERBOSITY > 0)
            cout << "activatePCOtoSUTClient: Can not get the reference " 
                 <<" of the emulated server object." << endl;
        }
    }
  else
    {
      buf = GciMkIA5String(WRONG_OBJ_REF); //Marks for wrong reference
      ref = GciSetType(GcCORBA__ObjectD, buf);
      theValue = GciMkBOOLEAN(false);
      if (VERBOSITY > 0)
        cout <<"activatePCOtoSUTClient: Failed due to GWAddConnection()." 
             << endl;
    }

  return(theValue);
}

GciValue* mkIntBase2( GciValue * exp )
{
  /* Make integer with base 2 and exponent exp. */
  //is it correct to use long for all the cases?

  long y = GciGetINTEGER(exp);
  double res;
  res= pow ((double) 2, (double) y);
  
  return (GciMkINTEGER((long) res));  
}


GciValue* getIFD(  )
{

  /* Get TestInterfaceD reference. */
  CORBA::Object_var obj = AdGetRefFromFile("OBVTest::TestInterfaceD");
  GciValue* buf;
  GciValue* theValue;

  if (!CORBA::is_nil(obj))
    {
      CORBA::String_var aStr = CORBA::string_dup(orb_->object_to_string(obj));
      buf = GciMkIA5String(aStr);
      theValue = GciSetType(GcINTF__OBVTest__TestInterfaceDD, buf);
    }
  else
    {
      buf = GciMkIA5String(WRONG_OBJ_REF); //Marks for wrong reference
      theValue = GciSetType(GcINTF__OBVTest__TestInterfaceDD, buf);
    }
  return(theValue);

}


GciValue* getIFBfromVLC(  )
{

  /* Get TestInterfaceB reference from a TestValueC instance. */
  CORBA::Object_var obj = AdGetRefFromFile("OBVTest::TestInterfaceB");
  GciValue* buf;
  GciValue* theValue;

  if (!CORBA::is_nil(obj))
    {
      CORBA::String_var aStr = CORBA::string_dup(orb_->object_to_string(obj));
      buf = GciMkIA5String(aStr);
      theValue = GciSetType(GcINTF__OBVTest__TestInterfaceBD, buf);
    }
  else
    {
      buf = GciMkIA5String(WRONG_OBJ_REF); //Marks for wrong reference
      theValue = GciSetType(GcINTF__OBVTest__TestInterfaceBD, buf);
    }
  return(theValue);
}


GciValue* getIFBdummy(  )
{

  /* Get a TestInterfaceB dummy reference. */

  CORBA::Object_var obj = AdGetRefFromFile("TestInterfaceB_dummy");
  GciValue* buf;
  GciValue* theValue;

  if (!CORBA::is_nil(obj))
    {
      CORBA::String_var aStr = CORBA::string_dup(orb_->object_to_string(obj));
      buf = GciMkIA5String(aStr);
      theValue = GciSetType(GcINTF__OBVTest__TestInterfaceBD, buf);
    }
  else
    {
      buf = GciMkIA5String(WRONG_OBJ_REF); //Marks for wrong reference
      theValue = GciSetType(GcINTF__OBVTest__TestInterfaceBD, buf);
    }
  return(theValue);

}


GciValue* registerValueFactory( GciValue* rcv, GciValue * rid )
{
  /* Create a factory for the value with rid and register it with the ORB. 
rcv determines whether the server or the client is the receiver of the value,
for which a value factory should be registered. */

  bool ircv = GciGetBOOLEAN(rcv);
  const char* vln = GciGetGraphicString(rid);
  CORBA::Object_var obj;
  OBVTest::TestValueSaver_var saver;

  if(ircv == 1)
    {
      // Server is the receiver of the value
      gwCoDec_ -> register_this_factory(vln);
      return(GciMkBOOLEAN(true));
    }
  else
    {
      // Client is the receiver of the value
      
      obj = AdGetRefFromFile("OBVTest::TestHelper");
      if(CORBA::is_nil(obj))
	{
	  if(VERBOSITY > 0)
	    cout <<"registerValueFactory failed because "
		 <<"TestHelper is not available."<<endl;
	  return(GciMkBOOLEAN(false));
	}
      OBVTest::TestHelper_var ifh =
	OBVTest::TestHelper::_narrow(obj);
      assert(!CORBA::is_nil(ifh));
      saver = ifh -> get_value_saver();
      saver -> register_this_factory(vln);
      
      return(GciMkBOOLEAN(true));
    }
}


GciValue* unregisterValueFactory( GciValue * rcv, GciValue * rid )
{

  /* Unregister a factory with the ORB. rcv determines whether at the server or at the client side the value factory should be unregistered. */

  bool ircv = GciGetBOOLEAN(rcv);
  const char* vln = GciGetGraphicString(rid);
  CORBA::Object_var obj;
  OBVTest::TestValueSaver_var saver;

  if(ircv == 1)
    {
      // Server is the receiver of the value
      gwCoDec_ -> unregister_this_factory(vln);
      return(GciMkBOOLEAN(true));
    }
  else
    {
      // Client is the receiver of the value
      obj = AdGetRefFromFile("OBVTest::TestHelper");
      if(CORBA::is_nil(obj))
	{
	  if(VERBOSITY > 0)
	    cout <<"unregisterValueFactory failed because "
		 <<"TestHelper is not available."<<endl;
	  return(GciMkBOOLEAN(false));
	}
      OBVTest::TestHelper_var ifh =
	OBVTest::TestHelper::_narrow(obj);
      assert(!CORBA::is_nil(ifh));
      saver = ifh -> get_value_saver();
      saver -> unregister_this_factory(vln);
      return(GciMkBOOLEAN(true));
    }

}


GciValue* is_valid_ref( GciValue * rf )
{
  /*Check if the reference is a valid reference of TestInterfaceB.*/

  CORBA::String_var ref = CORBA::string_dup(GciGetGraphicString(rf));
  CORBA::Object_var obj = orb_->string_to_object(ref);
  if(!CORBA::is_nil(obj))
  {
    OBVTest::TestInterfaceB_var ifb = OBVTest::TestInterfaceB::_narrow(obj);
    if(!CORBA::is_nil(ifb))
      return(GciMkBOOLEAN(true));
  }
     return(GciMkBOOLEAN(false));
}

GciValue* is_valid_vld( GciValue * vld )
{
  /*Check if the value instance of type TestValueD is valid.*/
  const char* s = GciGetGraphicString(GciGetField(vld, 1));
  CORBA::Long l = GciGetINTEGER(GciGetField(vld, 2));
  CORBA::Long id = GciGetINTEGER(GciGetField(vld, 3));

  if(VERBOSITY > 0)
    cout<<"vld_trunc_vla: name "<<s<<", age "<<l
	<<", id "<<id<<"."<<endl;
  if(strcmp(s,"john")==0 && l==30)
    return(GciMkBOOLEAN(true));
  return(GciMkBOOLEAN(false));
}

/*------------------------------------------------------------------------- 
 * Name: logStream
 * Author: ITEX C Code Generator
 *
 * Description: 
 *  Used for logging by GciLog and GciLogPrint. Assigned a value in main().
 *  This declaration is not part of the Gci Interface. 
 */

FILE * logStream;

extern "C"
{
  GciValue* GciReadTSPar(char* par, char* pRef);
  
};


/*-------------------------------------------------------------------------
 * Name: GciReadTSPar
 * Author: ITEX C Code Generator 
 *         + Mang Li (GMD Fokus, TIP)
 *
 * Description:
*  If the user is using test suite parameters, this function will be
 *  called once for each of those before running. The first argument to
 *  the function is the name of the test suite parameter and the second
 *  is a reference to where the value can be found. Mostly the second
 *  parameter is used to define a given file or other media from where
 *  the value of the parameter should be fetched.
 *
 * Return Value: GciValue
 *  Return a GciValue with the parameter that was read.
 */

#ifdef USE_TTMAN
GciValue* GciReadTSPar( char *par, char *pRef )
{
  GciValue *v;
  Param	 *tmpParam;

  /* Where the parameters already read in ?*/

  VERBOSE((0,"eTC %s: GciReadTSPar entered"));
  if(!parametersReadIn) 
    {
      VERBOSE((0,"eTC %s: We didn't receive parameters yet"));
      error_code = ERRBADSE;
      return((GciValue *) NULL);
    }

  cout<<"GciReadTSPar."<<endl;
 
  for (myParam.curr = myParam.first; 
       (myParam.curr &&  (tmpParam = (Param *) myParam.curr->data)); 
       myParam.curr = myParam.curr->next)
    {
      if(strcmp(tmpParam->param_id.octs, par) == 0) 
	{
	  if((v=GeneralStringDecode(tmpParam->param_value.octs)) == NULL) 
	    {
	      VERBOSE((0,"eTC %s: GciReadTSPar: Fatal: Failed parsing test suite parameter."));
	    }
	  return(v);
	}
    }
  return((GciValue *) NULL);
}

#else
GciValue* GciReadTSPar( char* par, char* pRef )
{
  GciValue* value = NULL;
  const char* refFile = PAR_FILE_NAME;
  if (VERBOSITY > 0)
    cout << "GciReadTSPar: Get TS parameters from file '"
         << refFile << "'" << endl;

  // The parameter file is organized as such:
  // one line for each parameter with the following elements and order:
  // 'ParameterID TypeofParameter ValueofParameter'
  // e.g. 'GcPropertyName_usr1 GraphicString password'.

  // Currently, the follwing types are impelmented:
  // GraphicString, BOOLEAN, INTEGER, OCTET_STRING, 

  ifstream in;
  in.open(refFile);
  assert(!in.fail());

  char parName[1000];
  char parType[100];
  
  while(!in.eof())
    {
      in >> parName;
      if(strcmp(parName, par) == 0)
        {
          in >> parType;
          if(strcmp(parType, "GraphicString") == 0)
            {
              char tmp[1000];
              in >> tmp;
              //Empty string is encoded as "-" in TSPar
              if (strcmp(tmp, "-") == 0)
                value = GciMkGraphicString("");
              else
                value = GciMkGraphicString(tmp);
              if (VERBOSITY > 0)
                cout<<"GciReadTSPar: The GraphicString '" 
                    << tmp << "' is encoded." << endl;

              break;
            }
          else if (strcmp(parType, "BOOLEAN") == 0)
            {
              int tmp;
              in >> tmp;
              value = GciMkBOOLEAN(tmp);
              if (VERBOSITY > 0)
                cout<<"GciReadTSPar: The BOOLEAN '" 
                    << tmp << "' is encoded." << endl;
              break;
            }
          else if (strcmp(parType, "INTEGER") == 0)
            {
              long tmp;
              in >> tmp;
              value = GciMkINTEGER(tmp);
              if (VERBOSITY > 0)
                cout<<"GciReadTSPar: The INTEGER '" 
                    << tmp << "' is encoded." << endl;
              break;
            }
          else if(strcmp(parType, "OCTET_STRING") == 0)
            {
              char tmp[1000];
              in >> tmp;
              value = GciMkOCTET_STRING(tmp);
              if (VERBOSITY > 0)
                cout<<"GciReadTSPar: The OCTET_STRING '" 
                    << tmp << "' is encoded." << endl;
              break;
            }
        }
      else
        {
          in >> parType;
          in >> parType;
        }
    }
  in.close();
  return value;
  
}
#endif


/*-------------------------------------------------------------------------
 * Name: GciSend
 * Author: ITEX C Code Generator
 *         + Mang Li (GMD Fokus, TIP)
 *
 * Description:
 *  This function is called from the TTCN runtime behaviour when something
 *  needs to be sent to the implementation under test. The first argument
 *  is a unique descriptor for the PCO that will be used and the second
 *  argument is the object that needs to be sent. Observe that some kind of
 *  encoding will probably be needed (implemented by the user) before the
 *  object/message is sent away.
 * 
 * Return Value: GciStatus 
 *  GciOk if the sending worked as it should.
 *  GciNotOk if the sending didn't work at all.
 */

GciStatus GciSend( int pcod, GciValue* object )
{
  CORBA::String_var pcoName = CORBA::string_dup(GciGetPCOName(pcod));
  assert(pcoName != NULL);
  
  if (pcoList_->pcoIsReady(pcoName, pcod) == ISTRUE)
    {
      TOR::GWMessage* amsg;
      if(gwCoDec_->makeGWMessage(object, amsg) == GciOk)
        {
          try
            {
              if (gwm_->GWSend(pcoName, *amsg) == TOR::GWOK)
                {
		  if(VERBOSITY > 0)
		    cout<<"GciSend: A message is dispatched via GatewayMain."
			<<endl;
                  return GciOk;
                }
              else
                {
                  if(VERBOSITY > 0)
                    cout <<"GciSend: Message dispatching "
                         <<"via GatewayMain returns not OK." << endl;
                  return GciNotOk;
                }
            }
          catch(CORBA::SystemException& ex)
            {
              if(VERBOSITY > 0)
                {
                  cout <<"GciSend: Message dispatching "
                       <<"via GatewayMain failed "
                       <<"due to CORBA system exception." << endl;
                  whichSysExcept(&ex);
                }
              return GciNotOk;
            }
          catch(...)
            {
              if(VERBOSITY > 0)
                cout <<"GciSend: Message dispatching "
                     <<"to GatewayMain failed due to unknown failure." 
                     << endl;
              return GciNotOk;
            }
        }
      else
        {
          if(VERBOSITY > 0)
            cout <<"GciSend: makeGWMessage() failed." << endl;
          return GciNotOk;
        }
    }
  if(VERBOSITY > 0)
    cout << "GciSend: The PCO '" << pcoName 
         <<"' is not ready. It shall be "
         <<"activated first." << endl;
  return GciNotOk;
}



/*-------------------------------------------------------------------------
 * Name: GciImplicitSend
 * Author: ITEX C Code Generator
 *
 * Description:
 *  This function does a simular task as GciSend, but the sending is not
 *  done through a PCO. An implicit send is typically used for example to
 *  send some message to the test operator to have him/her perform some 
 *  action on the implementation under test that can not be controlled
 *  through a PCO.
 *
 * Return Value: GciStatus
 *  GciOk if the sending worked as it should.
 *  GciNotOk if the sending didn't work at all.
 */

GciStatus GciImplicitSend( GciValue* msg )
{
  fprintf( stderr, "GciImplicitSend in adaptor.c not implemented!!\n" );

  return GciOk;  
}



/*-------------------------------------------------------------------------
 * Name: GciStartTimer
 * Author: ITEX C Code Generator
 *         + Mang Li (GMD Fokus, TIP)
 *
 * Description:
 *  This function is called when a timer needs to be started. The first
 *  argument is a unique descriptor for the timer, the second argument is
 *  the duration value and the last argument is the unit.
 *
 * Return Value: GciStatus
 *  GciOk if the timer was successfully started.
 *  GciNotOk if the timer couldn't be started at all.
 */

GciStatus GciStartTimer( int timerd, long timerVal, int unit )
{
  if (timerList_->addTimerbyID(timerd, timerVal, unit) == GciOk)
    return GciOk;
  else
    return GciNotOk;
}



/*-------------------------------------------------------------------------
 * Name: GciCancelTimer
 * Author: ITEX C Code Generator
 *         + Mang Li (GMD Fokus, TIP)
 *
 * Description:
 *  This function is called when a timer needs to be stopped. The only
 *  argument the function needs is the unique descriptor for the timer.
 *
 * Return Value: GciStatus
 *  GciOk if the timer could be stopped.
 *  GciNotOk if the timer couldn't be stopped at all.
 */

GciStatus GciCancelTimer( int timerd )
{
  if (timerList_->stopTimer(timerd) == GciOk)
    return GciOk;
  else
    return GciNotOk;
}



/*-------------------------------------------------------------------------
 * Name: GciReadTimer
 * Author: ITEX C Code Generator
 *         + Mang Li (GMD Fokus, TIP)
 *
 * Description:
 *  This function is called when a timer needs to be read. The function
 *  needs only one argument which is the unique descriptor for the timer
 *  to be read and returns the timer value.
 *
 * Return Value: long
 *  A long representing the value of the timer.
 */

long GciReadTimer( int timerd )
{
  long value = timerList_->readTimer(timerd);
  if(value != ERROR_IND)
    return value;
  else
    return 0;
}



/*-------------------------------------------------------------------------
 * Name: GciSnapshot
 * Author: ITEX C Code Generator
 *         + Mang Li (GMD Fokus, TIP)
 *
 * Description:
 *  This is a very important function for the execution process. In this
 *  function the user must check all PCOs and timers in the system. If the
 *  user is running concurrent TTCN the termination of different test
 *  components must be detected and handled here too. The functions used
 *  to report nack to the TTCN runtime behaviour are:
 * 
 *  GciReceive     - Report that something has been received on a PCO
 *  GciTimeout     - Report that a timer has timed out
 *  GciDone        - Report that a test component has terminated its execution
 *
 * Return Value: GciStatus
 *  GciOk if everything in snapshot went fine.
 *  GciNotOk if something catastrofic occured while doing the snapshot.
 */

GciStatus GciSnapshot( )
{
  TOR::GWMessage* rcvmsg;
  char* pcoName;
        
  for( ; ; )
    {
      try
        {
          if(gwm_->GWReceive(pcoName, rcvmsg) == TOR::MsgReady)
            {
              // Get PCO descriptor of pcoID
              const int pcod = pcoList_->getPCOId(pcoName);
              if (pcod != -1)
                {
                  GciValue* gciObj;
                  if (gwCoDec_->makeGciObj(rcvmsg, gciObj) == GciOk) 
                    {
                      if (GciReceive(pcod, gciObj) == GciOk)
                        {
			  if(VERBOSITY > 0)
			    cout<<"GciSnapshot: GciReceive() successful."
				<<endl;
                          return GciOk;
                        }
                      else
                        {
                          cout<<"GciSnapshot: GciReceive() failed." << endl;
                          return GciNotOk;
                        }
                    }
                  else
                    {
                      VERBOSE((0,"GciSnapshot: makeGciObj() failed."));
                      return GciNotOk;
                    }
                }
              else
                {
                  if(VERBOSITY > 0)
                    cout << "GciSnapshot: The PCO '" << pcoName 
                         <<"' is not ready. It shall be "
                         <<"activated first." << endl;
                  return GciNotOk;
                }
            }
          else
            {
              // Check timers
              int tid = timerList_->timeout();
              if(tid != ERROR_IND)
                {
                  if (tid > 0)
                    {
                      if(GciTimeout(tid) == GciOk)
                        {
                          VERBOSE((0,"GciSnapshot: GciTimeout() successful."));
                          return GciOk;
                        }
                      else
                        {
                          if(VERBOSITY > 0)
                            cout<<"GciSnapshot: GciTimeout() failed." << endl;
                          return GciNotOk;
                        }
                    }
                  else
                    return GciOk;
                }
              else
                {
                  if(VERBOSITY > 0)
                    cout<<"GciSnapshot: timerList_->timeout failed." << endl;
                  return GciNotOk;
                }
            }
        }
      catch(CORBA::SystemException &ex)
        {
          if(VERBOSITY > 0)
            cout <<"GciSnapshot: Message receiving "
                 <<"via GatewayMain failed "
                 <<"due to CORBA exception." << ex << endl;
          return GciNotOk;
        }
      catch(...)
        {
          if(VERBOSITY > 0)
            cout <<"GciSnapshot: Message receiving "
                 <<"from GatewayMain failed due to unknown failure." 
                 << endl;
          return GciNotOk;
        }
    }
    
}



/*-------------------------------------------------------------------------
 * Name: GciConfig
 * Author: ITEX C Code Generator
 *
 * Description:
 *  This function is only called when concurrent TTCN is used. Some TTCN
 *  tables with dynamic behaviour allow the user to specify a configuration
 *  to use for a given test. The information in the configuration can be
 *  traversed using the following managment functions:
 *  
 *  GciGetNoOfComponents         - Number of components in the configuration
 *  GciGetComponent              - Get a given component in the configuration
 *  GciGetComponentName          - Get the name of a component
 *  GciGetComponentDescriptor    - Get the descriptor of a component
 *  GciGetComponentType          - Get the type of a given component
 *  GciGetComponentCPs           - Get the CPs the component uses
 *  GciGetComponentPCOs          - Get the PCOs the component uses
 *
 * Return Value: GciStatus
 *  GciOk if the configuration could be created.
 *  GciNotOk if the configuration could not be successfully created.
 */

GciStatus GciConfig(GciConf config)
{
  fprintf( stderr, "GciConfig not implemented!!\n" );
  return GciOk;
}



/*-------------------------------------------------------------------------
 * Name: GciCreate
 * Author: ITEX C Code Generator
 *
 * Description:
 *  This function is called when running concurrent TTCN when a test
 *  component needs to be created. The first argument is a unique descriptor
 *  for the component, the second is the behaviour tree to execute and
 *  the last argument is a list of arguments for the test component.
 *
 * Return Value: GciStatus
 *  GciOk if the component could be created
 *  GciNotOk if the component could not be created at all.
 */

GciStatus GciCreate( int comp, char* tree, GciValue* args )
{
  fprintf( stderr, "GciCreate not implemented!!\n" );  

  return GciOk;
}



/*-------------------------------------------------------------------------
 * Name: GciLogPrint
 * Author: ITEX C Code Generator
 *
 * Description:
 *  This is simply a help function for GciLog.
 *
 * Return Value: void
 *  No return value.
 *
 */

void GciLogPrint( char* string, char* str, int isRow )
{
  int maxWithCol1 = 30;
  int maxWithCol2 = 5;
  
  if( isRow )
    fprintf( logStream, "%-*sRow: %s\n", maxWithCol1, string, str );
  else
    fprintf( logStream, "%-*s%s\n", maxWithCol1 + maxWithCol2, string, str );
}



/*-------------------------------------------------------------------------
 * Name: GciLog
 * Author: ITEX C Code Generator
 *
 * Description:
 *  This function is called every time something needs to be logged. Normally
 *  the user would have to write this function too, but we provide here an
 *  implementation to give some extra help on the way. The user is of course
 *  free to totally rewrite this function.
 *
 * Return Value: GciStatus
 *  GciOk if the logging went fine.
 *  GciNotOk if the logging couldn't be performed.
 */

GciStatus GciLog( int logId, char* logString )
{
  switch( logId )
    {
    case IcLogMatchValue:
      fprintf( logStream, "%s\n", logString );
      break;
    case IcLogNoMatchValue:
      fprintf( logStream, "Failed %s\n", logString );
      break;
    case IcLogMessage:
      fprintf( logStream, "%s\n", logString );
      break;
    case GciLogStartTC:
      GciLogPrint( "Start Test Case:", logString, 0 );
      break;
    case GciLogStopTC:
      GciLogPrint( "Test Case ended:", logString, 0 );
      break;
    case GciLogStartDEF:
      GciLogPrint( "Start Default:", logString, 0 );
      break;
    case GciLogStopDEF:
      GciLogPrint( "Default ended:", logString, 0 );
      break;
    case GciLogVerdict:
      GciLogPrint( "Final verdict:", logString, 0 );
      break;
    case GciLogPVerdict:
      GciLogPrint( "Preliminary verdict:", logString, 0 );
      break;
    case GciLogMatch:
      GciLogPrint( "Line matched:", logString, 0 );
      break;
    case GciLogNOMatch:
      GciLogPrint( "Line not matched:", logString, 0 );
      break;
    case GciLogSendE:
      GciLogPrint( "Send event:", logString, 1 );
      break;
    case GciLogRecE:
      GciLogPrint( "Receive event:", logString, 1 );
      break;
    case GciLogOtherE:
      GciLogPrint( "Otherwise event:", logString, 1 );
      break;
    case GciLogTimeoutE:
      GciLogPrint( "TimeOut:", logString, 1 );
      break;
    case GciLogAssign:
      GciLogPrint( "Assignment:", logString, 1 );
      break;
    case GciLogStartT:
      GciLogPrint( "StartTimer:", logString, 1 );
      break;
    case GciLogStopT:
      GciLogPrint( "StopTimer:", logString, 0 );
      break;
    case GciLogCancelT:
      GciLogPrint( "CancelTimer:", logString, 1 );
      break;
    case GciLogReadT:
      GciLogPrint( "ReadTimer:", logString, 1 );
      break;
    case GciLogAttach:
      GciLogPrint( "Attach:", logString, 1 );
      break;
    case GciLogImplSend:
      GciLogPrint( "ImplicitSend:", logString, 1 );
      break;
    case GciLogGoto:
      GciLogPrint( "Goto:", logString, 1 );
      break;
    case GciLogRec:
      GciLogPrint( "Received on PCO:", logString, 0 );
      break;
    case GciLogTimeout:
      GciLogPrint( "Timed out Timer:", logString, 0 );
      break;
    default:
      fprintf( logStream, "Log descriptor %d string %s not implemented!\n"
	      , logId, logString );
      fflush( logStream );
      return GciNotOk;
      break;
    }

  fflush( logStream );  
  return GciOk;
}



/*-------------------------------------------------------------------------
 * Name:   main
 * Author: ITEX C Code Generator
 *         + Mang Li (GMD Fokus, TIP)
 *
 * Description:
 *  This is a simple main function that shows how some managment functions
 *  can be used to run tests from a given test suite. First a list of test
 *  cases is fetched and these names are presented to the user. After that
 *  the test case the user wants to run is executed and the verdict is then
 *  presented back.
 *
 * Return Value: integer
 *  A value reporting the success or failure of the test run.
 */

#ifdef USE_TTMAN
int main(int argc, char** argv)
{
  int orb_agc = 3;
  char *orb_arg[3];
  extern char *optarg;	/* For getopt */
  extern int 	optind;		/* For getopt */
  int		opt;
      
  fprintf(stderr,"\n");
  while((opt = getopt(argc, argv, "hv:")) != EOF)
    switch(opt) 
      {
      case 'h':
	fprintf(stderr, tm_usgtxt);
	exit(0);
      case 'v':
	VERBOSITY=atoi(optarg);;
	break;
      case '?':
	fprintf(stderr, tm_usgtxt);
	exit(-1);
      }
  
  if((argc-optind < 2)) 
    {
      fprintf(stderr, tm_usgtxt);
      exit(-1);
    };
  //VERBOSE((0,"eTC: Verbosity level"));
      
  // Log stream
  if((logStream=fopen(LOG_FILE_NAME, "w")) == NULL)
    {
      fprintf (stderr, "main: could not open log file.");
      logStream = stdout;
    }
    
  /* first parameter = TestComponentId*/
  if(strlen(argv[optind]) > MAX_STR_LEN) 
    {
      printf("Passed parameter, i.e. TestComponent too long. MAX_STR_LEN = %d", MAX_STR_LEN);
      return(-1);
    } 
  else 
    {
      /* initializing fifo */
      //bzero(&fifo,sizeof(fifo));
      //fifo = (FIFO_t *) malloc(sizeof(FIFO_t *));
      strcpy(fifo.myTcoId, argv[optind]);
      cout<<"main: my TCO_ID is: "<<fifo.myTcoId<<endl;
    }
    
  /* second parameter = configuration_name */
  optind++;
  if(!argv[optind]) 
    {
      printf("No session_id, i.e. configuration given. Terminating.");
      return(-1);
    } 
  else 
    {
      conf_name = (char *) malloc(strlen(argv[optind])+1);
      strcpy(conf_name, argv[optind]);
      cout<<"TestSuite will run with configuration:" <<conf_name<<endl;
    }
  
#define ORBSWITCH "-ORBservice"
#define REPSWITCH "InterfaceRepository"
  
  orb_arg[0] = (char *) malloc(strlen(ORBSWITCH)+1);
  strcpy(orb_arg[0],ORBSWITCH);
  orb_arg[1] = (char *) malloc(strlen(REPSWITCH)+1);
  strcpy(orb_arg[1],REPSWITCH);
  
  ifstream in;
  in.open("Repository.ref");
  assert(!in.fail());
  
  char* implStr = new char[1000];
  in >> implStr;
  in.close();
  orb_arg[2] = implStr;
  
  // Init ORB and get the reference of the GatewayMain
  
  orb_ = CORBA::ORB_init(orb_agc, orb_arg);
  assert(!CORBA::is_nil(orb_));
  
  // Get the appropriate GatewayMain reference            
  CORBA::Object_ptr anObj = AdGetRefFromFile("GatewayMain");
  assert (!CORBA::is_nil(anObj));
  
  gwm_ = TOR::GatewayMain::_narrow(anObj);
  assert (!CORBA::is_nil(gwm_));
  VERBOSE((0,"Main: Get reference of GatewayMain finished."));
  gwm_ -> GWInit();
    
  // Init PCO/CP list
  pcoList_ = new AdPCOList;
  
  // Init Timer list
  timerList_ = new AdTimerList;
  
  // Init CoDec
  gwCoDec_ = new GWCoDec(orb_);
  
  if(openFIFO(fifo)) {
    printf("eTC %s: couldn't open FIFO's. Terminating",
	    fifo.myTcoId);
    return(-1);
  }

  fifo.read_hd = poll_register (fifo.readfd, fifo_infunc, 
				NULL, POLL_IN);
  InitNibbleMem(512, 512);
  RunningTestCaseName = (char *) NULL;
  error_code = ERROK;
  while(1)
    poll_dispatch(1);
  
  return 0; 
}

#else
int main(int argc, char** argv)
{
  try
    {
      int i, ver;
      char** list;
      CORBA::String_var name;
      char* verdicts[] = { "FAIL", "INCONCLUSIVE", "PASS", "NONE" };

      // Set verbosity
      int ac = 0;
      while (ac < argc)
        {
          if (strcmp (argv[ac], "-v") == 0)
            {
              VERBOSITY = 1;
              break;
            }
          ac++;
        }      

      GciInit();

      // Init ORB and get the reference of the GatewayMain
      orb_ = CORBA::ORB_init(argc, argv);
      assert(!CORBA::is_nil(orb_));
      
      const char* gwmFile = "GatewayMain";
      CORBA::Object_var anObj = AdGetRefFromFile(gwmFile);
      assert (!CORBA::is_nil(anObj));
      
      gwm_ = TOR::GatewayMain::_narrow(anObj);
      assert (!CORBA::is_nil(gwm_));
      VERBOSE((0,"Main: Get reference of GatewayMain finished."));
      gwm_ -> GWInit();
      
      // Init PCO list
      pcoList_ = new AdPCOList;

      // Init Timer list
      timerList_ = new AdTimerList;
      
      // Init CoDec
      gwCoDec_ = new GWCoDec(orb_);

      // Log stream
      if((logStream=fopen(LOG_FILE_NAME, "w")) == NULL)
        {
          fprintf (stderr, "main: could not open log file.");
          logStream = stdout;
        }

      list = GciGetTestCaseList();  
      printf("Available Test Cases :\n\n");
      for (i = 0; list[i] != NULL; ++i)
        printf("%s\n", list[i]);
      printf("\n Type 'all' to have all test cases to be executed,");
      printf("\n or enter the name of the test to be selected, ");
      printf("\n or 'q' to exit: ");
      cin >> name;

      if(strcmp(name, "all") == 0)
        {
          for (i = 0; list[i] != NULL; ++i)
            {
              GciLogPrint( "==========>>>", "", 0 );    
              ver = GciStartTestCase(list[i]);
              fprintf(stderr, "\nFINAL VERDICT = %s\n", verdicts[ver]);
              GciLogPrint( "<<<==========", "", 0 );
              GciLogPrint( "", "", 0 );
              GciLogPrint( "", "", 0 );
              GciLogPrint( "", "", 0 );

              timerList_->resetTimers();
              pcoList_->resetPCOs();
              gwm_->GWDestroy();
            }
        }
      else{
        while(strcmp(name, "q") != 0)
          {
            GciLogPrint( "==========>>>", "", 0 );
            ver = GciStartTestCase(name);
            fprintf(stderr, "\nFINAL VERDICT = %s\n", verdicts[ver]);
            GciLogPrint( "<<<==========", "", 0 );
            GciLogPrint( "", "", 0 );
            GciLogPrint( "", "", 0 );
            GciLogPrint( "", "", 0 );
            
            timerList_->resetTimers();
            pcoList_->resetPCOs();
            gwm_->GWDestroy();
            
            printf("Available Test Cases :\n\n");
            for (i = 0; list[i] != NULL; ++i)
              printf("%s\n", list[i]);
            printf("\n Select the next test case or 'q' to exit: ");
            cin >> name;
          }
      }
      
    }
   catch(CORBA::SystemException& ex)
    {
      if(VERBOSITY > 0)
        {
          cout << "Main: CORBA system exception." << endl;
          whichSysExcept(&ex);
        }
      return 1;
    }
  catch(...)
    {
      cout << "Main: Unknown failure." << endl;
      return 1;
    }
  return 0; 
}

#endif

/* End of File */
