/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/


/*------------------------------------------------------------------
 * Name: gwAdaptor.h
 * Author: ITEX C Code Generator 
 *        + Mang Li (GMD Fokus, TIP)
 *
 * Description:
 *  This header file is a template file generated by the ITEX C Code
 *  Generator to provide the user with a quick way to get started with
 *  the adaptation process.
 * 
 * 
 * Revision: $Id: gwAdaptor.h,v 1.1 2001/01/31 17:20:53 mli Exp $ 
 *-----------------------------------------------------------------*/

#ifndef GWADAPTOR_H
#define GWADAPTOR_H

#include <stdio.h>
/*-------------------------------------------------------------------------
 * These are the functions of the GCI operational interface. The adaptation
 * process is the process of implementing these functions to allow the
 * generated code to communicate with the system to be tested. Here is a 
 * short description of what the functions must do:
 *
 * GciSend          - Send something to the IUT through a PCO
 * GciImplicitSend  - Send something to the IUT, not through a PCO (but for
 *                    example to the test operator) to affect the current
 *                    status of the IUT
 * GciStartTimer    - Start a given timer
 * GciCancelTimer   - Cancel a given timer
 * GciReadTimer     - Read the value of a given timer
 * GciSnapshot      - Check the status of the receive channels, timers and
 *                    possibly test components (concurrent TTCN)
 * GciLog           - Function used for logging
 * GciConfig        - Create configuration (concurrent TTCN)
 * GciCreate        - Create test component (concurrent TTCN)
 */
extern "C" {
GciStatus GciSend( int pco, GciValue* object );
GciStatus GciImplicitSend( GciValue* object );
GciStatus GciStartTimer( int timer, long value, int unit );
GciStatus GciCancelTimer( int timer );
long GciReadTimer( int timer );
GciStatus GciSnapshot( );
GciStatus GciConfig( GciConf conf );
GciStatus GciCreate( int comp, char* tree, GciValue* args );
}

/*Declared in gci.h
GciStatus GciLog( int logId, char* logString );*/

void GciLogPrint( char* string, char* logString, int isRow );



/*-------------------------------------------------------------------------
 * These are the functions of the GCI behaviour interface. These three
 * functions are used to inform the TTCN runtime behaviour about:
 *
 * GciReceive       - Something has been received on the given PCO
 * GciTimeout       - The given timer has timed out
 * GciDone          - The given test component has terminated (concurrent TTCN)
 */
/*Declared in gci.h
GciStatus GciReceive( int pco, GciValue* object ); 
GciStatus GciTimeout( int timer );
GciStatus GciDone( int ptc );
*/


/*------------------------------------------------------------------------
 * Global definition of the Log Stream used by the GciLog function, might
 * be used by other user written code in order to have a conformant logging.
 */

extern FILE* logStream;



// /***************
// *              *
// * Proto types  *
// *              *
// ***************/

// Test suite operations

extern "C" {
  GciValue* getObjRef(GciValue* objName);
  GciValue* activatePCOtoSUTClient(GciValue * pcoName, GciValue * objectName, GciValue * ref);
  GciValue* activatePCOtoSUTServer(GciValue* pcoName, GciValue * objectName, GciValue* refStr);
  GciValue* mkIntBase2( GciValue * exp );
  GciValue* getIFD(  );
  GciValue* getIFBfromVLC(  );
  GciValue* getIFBdummy(  );
  GciValue* registerValueFactory( GciValue * rcv, GciValue * rid );
  GciValue* unregisterValueFactory( GciValue * rcv, GciValue * rid );
  GciValue* is_valid_ref( GciValue * rf );
  GciValue* is_valid_vld( GciValue * vld );
  
}
// Help function to obtain object references

CORBA::Object_ptr AdGetRefFromFile(const char* objName);
void AdCleanUp();

#endif /* GWADAPTOR_H */
