/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/


/**********************************************************************
 *                                                                    *
 *    Name:   gwCoDec.cpp                                             *
 *                                                                    *
 *    Author: Mang Li                                                 *
 *            GMD FOKUS, TIP                                          *
 *                                                                    *
 *    Description: This file contains the implementation of           *
 *        the class GWCoDec to support encoding and decoding.         *
 *                                                                    *
 *                                                                    *
 *********************************************************************/

#include <stdlib.h>
#include <iostream.h>
#include <math.h>

// CORBA and Gateway includes
#include <orb_gw.h>
#include <Gateway.h>
#include "obv_test_impl.h"

// GCI header files
#include "gci.h"
#include "s_aux.h"

#include "gwCoDec.h"




//--------------
// Constructor  |
//--------------

GWCoDec::GWCoDec(CORBA::ORB_ptr theORB)
  :orb_(CORBA::ORB::_duplicate(theORB))
{
  // Find the Interface Repository
  CORBA::Object_var obj = 
    orb_->resolve_initial_references("InterfaceRepository");
  assert(!CORBA::is_nil(obj));
  repository_ = CORBA::Repository::_narrow(obj);

  CORBA::Object_var obj2 = orb_->resolve_initial_references("DynAnyFactory");
  assert(!CORBA::is_nil(obj2));
  dynAnyFactory_ = DynamicAny::DynAnyFactory::_narrow(obj2);
}

//--------------
// Destructor   |
//--------------

GWCoDec::~GWCoDec()
{
  if(!CORBA::is_nil(orb_))
    orb_->destroy();
  
}

//--------------
// getIDLType   |
//--------------

char* GWCoDec::getIDLType(const char* theType, char* prefix)
{
  char* tmp;
  CORBA::String_var idlType;
    
  //get rid of the prefix, e.g. ENUM__!!!!
  if(strcmp(prefix,UNION_TYPE_PRE)==0 || 
     strcmp(prefix,VALUEBOX_TYPE_PRE)==0 || 
     strcmp(prefix,VALUE_TYPE_PRE)==0 || 
     strcmp(prefix,INTF_TYPE_PRE)==0 || 
     strcmp(prefix,ABSTRACT_INTF_TYPE_PRE)==0 || 
     strcmp(prefix,STRUCT_TYPE_PRE)==0 || 
     strcmp(prefix,ARRAY_TYPE_PRE)==0 || 
     strcmp(prefix,SEQ_TYPE_PRE)==0 || 
     strcmp(prefix,ENUM_TYPE_PRE)==0 ||
     strcmp(prefix,EXCEPTION_TYPE_PRE)==0)

    for (int k=0; theType[k]!= '\0'; k++)
      {
        if((theType[k]=='_') && (theType[k+1]=='_'))
          {  
            tmp = new char[strlen(theType)-k-1];
            strcpy(tmp, &theType[k+2]);
            break;
          }
      }
  else
    {
      tmp = new char[strlen(theType)+1];
      strcpy(tmp, (char*) theType);
    }

  // Replace '__' by '/'
  char* buf = new char[strlen(tmp)+1];
  CORBA::ULong i;
  CORBA::ULong j = 0;
  for (i=0; tmp[i] != '\0'; i++)
    {
      if (tmp[i] == '_')
        {
          if (tmp[i+1] == '_')
            {
              i++;
              buf[j] = '/';
              j++;
            }
          else
            {
              buf[j] = tmp[i];
              j++;
            }
        }
      else
        {
          buf[j] = tmp[i];
          j++;
        }
    }
  char* body = new char[j+1];
  strncpy(body, buf, j);
  body[j] = '\0';
  delete[] buf;
  delete[] tmp;
  
  // Add prefix and suffix
  idlType += CORBA::string_dup(IDLTYPE_PREFIX); 
  idlType += CORBA::string_dup(body);
  idlType += CORBA::string_dup(IDLTYPE_SUFIX);
  delete[] body;
  
  char* result = new char[strlen(idlType)+1];
  strcpy(result,idlType);

  if (VERBOSITY > 0)
    cout<<"getIDLType: '" << result <<"'." << endl;
    
  return result;
    
}


//--------------------
// typeDescFromTCid   |
//--------------------

// Get the GCI type descriptor from the RepositoryId(TypeCode id)
int GWCoDec::typeDescFromTCid(CORBA::Any* any)
{
  char* tmp;
  char* pref; 

  long tk = any->type()->kind();
  if(tk == CORBA::tk_objref ||
     tk == CORBA::tk_struct ||
     tk == CORBA::tk_union ||
     tk == CORBA::tk_enum ||
     tk == CORBA::tk_value ||
     tk == CORBA::tk_value_box ||
     tk == CORBA::tk_abstract_interface ||
     tk == CORBA::tk_except)
    {
      const char* s = any->type()->id();
      tmp = new char[strlen(s)+1];
      strcpy(tmp, s);

      //remember the prefix for struct, sequence, value, etc.
      if(tk == CORBA::tk_objref)
        {
          pref = new char[strlen(INTF_TYPE_PRE)+1];
          strcpy(pref, INTF_TYPE_PRE);
        }
      else if(tk == CORBA::tk_struct)
        {
          pref = new char[strlen(STRUCT_TYPE_PRE)+1];
          strcpy(pref, STRUCT_TYPE_PRE);
        }
      else if(tk == CORBA::tk_union)
        {
          pref = new char[strlen(UNION_TYPE_PRE)+1];
          strcpy(pref, UNION_TYPE_PRE);
        }
      else if(tk == CORBA::tk_enum)
        {
          pref = new char[strlen(ENUM_TYPE_PRE)+1];
          strcpy(pref, ENUM_TYPE_PRE);
        }
      else if(tk == CORBA::tk_value)
        {
          pref = new char[strlen(VALUE_TYPE_PRE)+1];
          strcpy(pref, VALUE_TYPE_PRE);
        }
      else if(tk == CORBA::tk_value_box)
        {
          pref = new char[strlen(VALUEBOX_TYPE_PRE)+1];
          strcpy(pref, VALUEBOX_TYPE_PRE);
        }
      else if(tk == CORBA::tk_abstract_interface)
        {
          pref = new char[strlen(ABSTRACT_INTF_TYPE_PRE)+1];
          strcpy(pref, ABSTRACT_INTF_TYPE_PRE);
        }
      else if(tk == CORBA::tk_except)
        {
          pref = new char[strlen(EXCEPTION_TYPE_PRE)+1];
          strcpy(pref, EXCEPTION_TYPE_PRE);
        }
              
    }
  // id() does not apply to sequence and array
  // how to get there own id, not only the content type id?
  else if(tk == CORBA::tk_sequence)
    {
      pref = new char[strlen(SEQ_TYPE_PRE)+1];
      strcpy(pref, SEQ_TYPE_PRE);
      const char* s = any->type()->content_type()->id();
      tmp = new char[strlen(s)+1];
      strcpy(tmp, s);
    }
  else if(tk == CORBA::tk_array)
    {
      pref = new char[strlen(ARRAY_TYPE_PRE)+1];
      strcpy(pref, ARRAY_TYPE_PRE);
      const char* s = any->type()->content_type()->id();
      tmp = new char[strlen(s)+1];
      strcpy(tmp, s);
    }
  else
    {
      if (VERBOSITY > 0)
        cout<<"typeDescFromTCid: kind not found on the list." << endl;
      return 0;
    }
  if (VERBOSITY > 0)
    cout<<"typeDescFromTCid: RepositoryId of the type is '" 
        << tmp <<"'" << endl;

  // The RepositoryId e.g. 'IDL:TestInterface/BadInput:1.0'
  // and 'IDL:omg.org/CORBA/COMM_FAILURE:1.0'
  
  unsigned long ol = strlen(tmp);
  unsigned long pl = strlen("IDL:");
  unsigned long tl = strlen(":1.0");
    
  if(strncmp(tmp, "IDL:", pl) == 0)
    if(strncmp(tmp, "IDL:omg.org/", strlen("IDL:omg.org/")) == 0)
      pl = strlen("IDL:omg.org/");
  
  ol = ol - pl;
  char* s1 = new char[ol+1];
  strcpy(s1, &tmp[pl]);
  
  ol = ol - tl;
  char* s2 = new char[ol+1];
  strncpy(s2, s1, ol);
  s2[ol] = '\0';
  delete[] s1;

  //Number of slashes which mark the scopes.
  unsigned sl = 0;
  unsigned long i;
  unsigned long j = 0;
  for (i=0; i<strlen(s2); i++)
    {
      if (s2[i] == '/')
        sl++;
    }
  
  char* s3 = new char[ol+1+sl];
  for (i=0; i<ol; i++)
    {
      if (s2[i] == '/')
        {
          s3[j] = '_';
          s3[j+1] = '_';
          j = j+2;
        }
      else
        {
          s3[j] = s2[i];
          j++;
        }
    }
  s3[ol+sl] ='\0'; 
  delete[] s2;
  
  CORBA::String_var typen = CORBA::string_dup(pref);
  typen += CORBA::string_dup(s3);
  delete[] s3;
  
  int typedes = GciGetTypeDescriptor(typen);

  if (VERBOSITY > 0)
    cout<<"typeDescFromTCid: The descriptor of the type '"
        << GciGetTypeNameString(typedes) << "' is resolved." <<endl;
  return typedes;
}



//------------------------
// typeDescFromGCIValue   |
//------------------------

char* GWCoDec::typeDescFromGCIValue(int decrs, char*& typeName, GciValue* value)
{
  int tpn;
  char* td;
  unsigned n = 0;

  VcType* tmp = VcGetType(value);
  while((tmp != NULL) && (decrs > 0))
    {
      tmp = VcGetParent(tmp);
      decrs--;
    }
  tpn = VcGetName(tmp);
  td = (char*) GciGetTypeNameString(tpn);
  typeName = td;
  
  while ((n < MAX_NUM) && (td != NULL))
    {
      if (VERBOSITY > 0)
        cout<<"typeDescFromGCIValue: looking for "<< td 
            <<" the type description/abbr."<< endl;
  
      if (strncmp(td, BASIC_TYPE_PRE, strlen(BASIC_TYPE_PRE)) ==0)
        return (CORBA::string_dup(td));
      
      else if (strncmp(td, UNION_TYPE_PRE, strlen(UNION_TYPE_PRE)) ==0)
        return (CORBA::string_dup(UNION_TYPE_PRE));
      
      else if (strncmp(td, VALUEBOX_TYPE_PRE, strlen(VALUEBOX_TYPE_PRE)) ==0)
        return (CORBA::string_dup(VALUEBOX_TYPE_PRE));

      else if (strncmp(td, VALUE_TYPE_PRE, strlen(VALUE_TYPE_PRE)) ==0)
        return (CORBA::string_dup(VALUE_TYPE_PRE));

      else if (strncmp(td, INTF_TYPE_PRE, strlen(INTF_TYPE_PRE)) ==0)
        return (CORBA::string_dup(INTF_TYPE_PRE));

      else if (strncmp(td, STRUCT_TYPE_PRE, strlen(STRUCT_TYPE_PRE)) ==0)
        return (CORBA::string_dup(STRUCT_TYPE_PRE));

      else if (strncmp(td, SEQ_TYPE_PRE, strlen(SEQ_TYPE_PRE)) ==0)
        return (CORBA::string_dup(SEQ_TYPE_PRE));

      else if (strncmp(td, ARRAY_TYPE_PRE, strlen(ARRAY_TYPE_PRE)) ==0)
        return (CORBA::string_dup(ARRAY_TYPE_PRE));

      else if (strncmp(td, ENUM_TYPE_PRE, strlen(ENUM_TYPE_PRE)) ==0)
        return (CORBA::string_dup(ENUM_TYPE_PRE));
      
      //allows muli-demensional arrays
      else if (strcmp(td, "SEQUENCE_OF") ==0)
        return (CORBA::string_dup("ARRAY_SEQOF"));
      else
        {
          tmp = VcGetParent(tmp);
          tpn = VcGetName(tmp);
          td = (char*) GciGetTypeNameString(tpn);
        }
      n++;
      
    }
 if (VERBOSITY > 0)
   cout<<"typeDescFromGCIValue: not found on the list, exceed level: "
       << n << endl;  
}


//-----------------
// encodeField     |
//-----------------

// Encode an ASP field to a CORBA::Any
CORBA::Any* GWCoDec::encodeField(int decrs, GciValue* aField)
{
  CORBA::Any* any = new CORBA::Any;
  char* typeName;
  char* theType = typeDescFromGCIValue(decrs, typeName, aField);
  if (VERBOSITY > 0)  
    cout<<"encodeField: The type descriptor of the type: " <<
      typeName<<" is: "<< theType <<" ." <<endl;
  
  if(strcmp(theType, "CORBA__Long") == 0)
    {
      CORBA::Long tmp;
      tmp = GciGetINTEGER(aField);
      if (VERBOSITY > 0)
	cout<<"encodeField: a long '"<< tmp <<"'."<< endl;
      *any <<= tmp;
    }
  else if(strcmp (theType, "CORBA__ULong") == 0)
    {
      CORBA::ULong tmp;
      tmp = GciGetINTEGER(aField);
      if (VERBOSITY > 0)
	cout<<"encode: an unsigned long '"<< tmp <<"'."<< endl;
      *any <<= tmp;
    }
  else if(strcmp (theType, "CORBA__LongLong") == 0)
     {
       CORBA::LongLong tmp;
       tmp = GciGetINTEGER(aField);
       if (VERBOSITY > 0)
         cout<<"encodeField: a long long."<< tmp << endl;
       *any <<= tmp;
     }
  else if(strcmp (theType, "CORBA__ULongLong") == 0)
    {
      CORBA::ULongLong tmp;
      tmp = GciGetINTEGER(aField);
      if (VERBOSITY > 0)
        cout<<"encodeField: an unsigned long long."<< tmp << endl;
      *any <<= tmp;
    }
  else if(strcmp (theType, "CORBA__Short") == 0)
      {
        CORBA::Short tmp;
        tmp = GciGetINTEGER(aField);
        if (VERBOSITY > 0)
          cout<<"encodeField: a short '"<< tmp <<"'."<< endl;
        *any <<= tmp;
      }
   else if(strcmp (theType, "CORBA__UShort") == 0)
      {
        CORBA::UShort tmp;
        tmp = GciGetINTEGER(aField);
        if (VERBOSITY > 0)
          cout<<"encodeField: an unsigned short '"<< tmp <<"'."<< endl;
        *any <<= tmp;
      }
  else if(strcmp (theType, "CORBA__Boolean") == 0)
    {
      CORBA::Boolean tmp;
      tmp = GciGetBOOLEAN(aField);
      if (VERBOSITY > 0)
        cout<<"encodeField: a boolean '"<< tmp <<"'."<< endl;
      *any <<= CORBA::Any::from_boolean(tmp);
    }
  else if(strcmp (theType, "CORBA__Octet") == 0)
    {
      char* s1 = GciGetOCTET_STRING(aField);
      assert(strlen(s1) == 8);
      CORBA::Octet oc = strtol(s1,NULL,2);
      *any <<= CORBA::Any::from_octet(oc);
      if (VERBOSITY > 0)
	cout<<"encodeField: Octet '"<< oc <<"' is encoded." << endl;
    }
  else if(strcmp (theType, "CORBA__Object") == 0)
    {
      if (VERBOSITY > 0)
	cout<<"encodeField: an object reference." << endl;
      char* s = GciGetIA5String(aField);
      CORBA::Object_var obj = orb_->string_to_object(s);
      assert(!CORBA::is_nil(obj));
      *any <<= obj;
    }
  else if(strcmp (theType, "CORBA__Float") == 0)
    {
      /*
      VcReal* areal = VcGetREAL (aField);
      unsigned base = VcGetBase(areal);
      int ep = VcGetExponent(areal);*/

      assert(GciSeqSize(aField) == 3);
      long mt = GciGetINTEGER(GciGetField(aField, 0+GCI_SEQ_OFFSET));
      unsigned base = GciGetINTEGER(GciGetField(aField, 1+GCI_SEQ_OFFSET));
      int ep = GciGetINTEGER(GciGetField(aField, 2+GCI_SEQ_OFFSET));
      assert(base == 10);
      assert(ep == -6);
      
      //CORBA::Float fl = ldexp(mt, ep);
      CORBA::Float fl = mt * pow (base, ep);
      
      if (VERBOSITY > 0)
	cout<<"encodeField: a float: "<<fl<<endl;
      *any <<= fl;
    }
  else if(strcmp (theType, "CORBA__Double") == 0)
    {
      /*
      VcReal* areal = VcGetREAL (aField);
      long mt = VcGetMantissa(areal);
      unsigned base = VcGetBase(areal);
      int ep = VcGetExponent(areal);
      */
      assert(GciSeqSize(aField) == 3);
      long mt = GciGetINTEGER(GciGetField(aField, 0+GCI_SEQ_OFFSET));
      unsigned base = GciGetINTEGER(GciGetField(aField, 1+GCI_SEQ_OFFSET));
      int ep = GciGetINTEGER(GciGetField(aField, 2+GCI_SEQ_OFFSET));
      assert(base == 10);
      assert(ep == -15);

      //CORBA::Double fl = ldexp(mt, ep);
      // Range -15 seems to be too large. Therefore use -10.
      // See also decode of double.
      ep = -10;
      CORBA::Double fl = mt * pow (base, ep);
      if (VERBOSITY > 0)
	cout<<"encodeField: a double: "<<fl<<endl;
      *any <<= fl;
    }
  else if(strcmp (theType, "CORBA__Char") == 0)
    {
      char* tmp = GciGetGraphicString(aField);
      char ach = tmp[0];
      if (VERBOSITY > 0)
	cout<<"encodeField: a char '"<< tmp << "'."<< endl;
      *any <<= CORBA::Any::from_char(ach);
    }
  else if(strcmp(theType,  "CORBA__String") == 0)
    {
      const char* tmp = GciGetGraphicString(aField);
      assert(tmp != NULL);
      if (VERBOSITY > 0)
	cout<<"encodeField: a string '"<< tmp << "'."<< endl;
      *any <<= CORBA::Any::from_string(tmp,0);
    }
  else if(strcmp(theType,  "CORBA__WChar") == 0)
    {
      //What needs a wchar more than a char?
      char* tmp = GciGetGraphicString(aField);
      char ach = tmp[0];
      if (VERBOSITY > 0)
        cout<<"encodeField: a wchar '"<< ach << "'" << endl;
      *any <<= CORBA::Any::from_wchar(ach);
    }
  else if(strcmp(theType,  "CORBA__WString") == 0)
    {
      //What needs a wstring more than a string?
      char* tmp = GciGetGraphicString(aField);
      assert(tmp != NULL);
      if (VERBOSITY > 0)
        cout<<"encodeField: a string '"<< tmp << "'."<< endl;
      *any <<= CORBA::Any::from_string(tmp,0);
    }
  //struct  
  else if(strcmp(theType,  STRUCT_TYPE_PRE) == 0)
    {
      if (VERBOSITY > 0)
        cout<<"encodeField: a struct."<< endl;
      unsigned long k;
      GciValue* theField;
      CORBA::ULong len = GciSeqSize(aField);
      
      CORBA::String_var idltype = 
        CORBA::string_dup(getIDLType(typeName, theType));
      CORBA::Contained_var contained =
        repository_ -> lookup_id(idltype);
      assert(!CORBA::is_nil(contained));
      CORBA::StructDef_var structdef =  CORBA::StructDef::_narrow(contained);
      assert(!CORBA::is_nil(structdef));
      CORBA::StructMemberSeq_var memseq = structdef -> members();
      
      CORBA::TypeCode_var newtc = structdef -> type();      
      DynamicAny::DynAny_var da = 
        dynAnyFactory_->create_dyn_any_from_type_code(newtc);
      DynamicAny::DynStruct_var dyn_struct = 
        DynamicAny::DynStruct::_narrow(da);
      
      DynamicAny::NameValuePairSeq cnt;
      cnt.length(len);
      assert(cnt.length() == dyn_struct->type()->member_count());
      CORBA::Any_var inany;

      for (k = 0; k < len; k++)
        {
          theField = GciGetField(aField, k+GCI_SEQ_OFFSET);
          assert(theField != NULL);
          cnt[k].id = CORBA::string_dup(memseq[k].name);
          inany = encodeField(0, theField);
          assert(inany != NULL);
          cnt[k].value = inany;
        }
      try
	{
	  dyn_struct->set_members(cnt);
	  any = dyn_struct->to_any();
	}
      catch(DynamicAny::DynAny::InvalidValue& ex)
	{
	  if (VERBOSITY > 0)
	    cout << "encodeField: struct: invalid value." << endl;
	}
    }

  //sequence
  else if(strcmp(theType,  SEQ_TYPE_PRE) == 0)
    {
      if (VERBOSITY > 0)
        cout<<"encodeField: a sequence."<< endl;
      unsigned long k;
      CORBA::ULong len = GciSeqOfSize(aField);

      CORBA::String_var idltype = 
        CORBA::string_dup(getIDLType(typeName, theType));

      CORBA::Contained_var contained =
        repository_ -> lookup_id(idltype);
      assert(!CORBA::is_nil(contained));
      CORBA::AliasDef_var alias =  CORBA::AliasDef::_narrow(contained);
      assert(!CORBA::is_nil(alias));
      CORBA::TypeCode_var newtc = alias -> original_type_def() -> type();     

      DynamicAny::DynAny_var da = 
        dynAnyFactory_->create_dyn_any_from_type_code(newtc);
      DynamicAny::DynSequence_var dyn_seq  = 
        DynamicAny::DynSequence::_narrow(da);
      
      dyn_seq->set_length(len);
      
      DynamicAny::AnySeq seq;
      seq.length(len);
      CORBA::Any_var inany;
      GciValue* theField;      

      for (k = 0; k < len; k++)
        {
          theField = GciGetElem(aField, k);
          assert(theField != NULL);
          inany = encodeField(0, theField);
          seq[k] = inany;
        }
      try
        {
          dyn_seq->set_elements(seq);
          any = dyn_seq->to_any();
        }
      catch(DynamicAny::DynAny::InvalidValue& ex)
        {
          if (VERBOSITY > 0)
            cout << "encodeField: sequence: invalid value." << endl;
        }
    }

  //enum
  else if(strcmp(theType,  ENUM_TYPE_PRE) == 0)
    {
      if (VERBOSITY > 0)
        cout<<"encodeField: an enum."<< endl;
      CORBA::String_var idltype = 
        CORBA::string_dup(getIDLType(typeName, theType));

      CORBA::Contained_var contained =
	repository_ -> lookup_id(idltype);
      assert(!CORBA::is_nil(contained));
      CORBA::EnumDef_var enumdef =  CORBA::EnumDef::_narrow(contained);
      assert(!CORBA::is_nil(enumdef));
      
      CORBA::EnumMemberSeq_var mems = enumdef->members();
      CORBA::TypeCode_var newtc = enumdef -> type();
      
      DynamicAny::DynAny_var da = 
        dynAnyFactory_->create_dyn_any_from_type_code(newtc);
      DynamicAny::DynEnum_var dyn_enum = 
        DynamicAny::DynEnum::_narrow(da);
      
      // Add value
      // The number returned by GciGetENUMERATED is identical to the
      // one specified in the ATS (in parenthesis). While the ordering
      // of the elments in the ATS is not significant.
      // However, since the number involved is used to find the
      // proper string, the numbering in the ATS shall obey the ordering
      // in the IDL spec. More over, the numbering in the ATS 
      // shall begins with 0 and be counted continuously.
      CORBA::ULong ul = GciGetENUMERATED(aField);
      CORBA::String_var vl = CORBA::string_dup(mems[ul]);
      try
	{
	  dyn_enum->set_as_string(vl);
	  if (VERBOSITY > 0)
	    cout<<"encodeField: enum: value in string'" << vl
		<< "' is encoded." << endl;
	  any = dyn_enum->to_any();
	}
      catch(DynamicAny::DynAny::InvalidValue& ex)
	{
	  if (VERBOSITY > 0)
	    cout << "encodeField: enum: invalid value." << endl;
	}
    }

  //union
  else if(strcmp(theType,  UNION_TYPE_PRE) == 0)
    {
      if (VERBOSITY > 0)
	cout<<"encodeField: a unoin."<<endl;

      CORBA::String_var idltype = 
        CORBA::string_dup(getIDLType(typeName, theType));

      CORBA::Contained_var contained =
	repository_ -> lookup_id(idltype);
      assert(!CORBA::is_nil(contained));
      CORBA::UnionDef_var uniondef =  CORBA::UnionDef::_narrow(contained);
      assert(!CORBA::is_nil(uniondef));
      
      CORBA::TypeCode_var newtc = uniondef -> type();
      DynamicAny::DynAny_var da = 
        dynAnyFactory_->create_dyn_any_from_type_code(newtc);
      DynamicAny::DynUnion_var dyn_union = DynamicAny::DynUnion::_narrow(da);
      
      CORBA::TypeCode_var disType = uniondef->discriminator_type();
      
      // Discriminator
      GciValue* discvalue = GciGetField(aField, GCI_SEQ_OFFSET);
      assert(discvalue != NULL);
      CORBA::Any_var d = encodeField(0, discvalue);
      assert(disType->equivalent(d->type()));
      DynamicAny::DynAny_var dis = dynAnyFactory_->create_dyn_any(*d);
      assert(!CORBA::is_nil(dis));
      
      try
        {
          dyn_union -> set_discriminator(dis);
        }
      catch(DynamicAny::DynAny::InvalidValue& ex)
	{
	  if (VERBOSITY > 0)
	    cout << "encodeField: union discriminator: invalid value." << endl;
	} 
	      
      // Body
      GciValue* bodyvalue = GciGetField(aField, GCI_SEQ_OFFSET+1);
      CORBA::Any_var b = encodeField(0, GciGetCHOICE(bodyvalue));
      assert(b->type()->kind() == dyn_union->member_kind());
      DynamicAny::DynAny_var body = dyn_union -> member();
      
      try
        {
          body -> from_any(b.in());
          any = dyn_union->to_any();
        }
      catch(DynamicAny::DynAny::InvalidValue& ex)
	{
	  if (VERBOSITY > 0)
	    cout << "encodeField: union body: invalid value." << endl;
	}    
      catch(DynamicAny::DynAny::TypeMismatch& ex)
	{
	  if (VERBOSITY > 0)
	    cout << "encodeField: union body: type mismatch." << endl;
	}    
    }

  //array
  else if(strcmp(theType,  ARRAY_TYPE_PRE) == 0)
    {
      if (VERBOSITY > 0)
	cout<<"encodeField: array."<< endl;

      CORBA::ULong len = GciSeqOfSize(aField);

      CORBA::String_var idltype = 
        CORBA::string_dup(getIDLType(typeName, theType));

      CORBA::Contained_var contained =
        repository_ -> lookup_id(idltype);
      assert(!CORBA::is_nil(contained));
      CORBA::AliasDef_var alias =  CORBA::AliasDef::_narrow(contained);
      assert(!CORBA::is_nil(alias));
      CORBA::TypeCode_var newtc = alias->original_type_def()->type();     

      DynamicAny::DynAny_var da = 
        dynAnyFactory_->create_dyn_any_from_type_code(newtc);
      DynamicAny::DynArray_var dyn_array  = 
        DynamicAny::DynArray::_narrow(da);
      
      DynamicAny::AnySeq ary;
      ary.length(len);
      CORBA::Any_var inany;
      GciValue* theField;      
      
      for (CORBA::ULong k = 0; k < len; k++)
        {
          theField = GciGetElem(aField, k);
          assert(theField != NULL);
          inany = encodeField(0, theField);
          ary[k] = inany;
        }
      try
        {
          dyn_array->set_elements(ary);
          any = dyn_array->to_any();
        }
      catch(DynamicAny::DynAny::InvalidValue& ex)
        {
          if (VERBOSITY > 0)
            cout << "encodeField: array: invalid value." << endl;
        }
    }
  else if(strcmp(theType,  "ARRAY_SEQOF") == 0)
    {
      if (VERBOSITY > 0)
	cout<<"encodeField: sequence_of (slice) "
            <<"for multi-demensional arrays."<< endl;

      CORBA::ULong len = GciSeqOfSize(aField);
      assert (len > 0);

      CORBA::Any_var inany;
      GciValue* theField = GciGetElem(aField, 0);
      inany = encodeField(0, theField);

      CORBA::TypeCode_var elm_tc = inany -> type();
      CORBA::TypeCode_var tc_sl = orb_ -> create_array_tc(len, elm_tc);
      DynamicAny::DynAny_var da 
        = dynAnyFactory_->create_dyn_any_from_type_code(tc_sl);
      DynamicAny::DynArray_var dyn_array_sl  = 
        DynamicAny::DynArray::_narrow(da);

      DynamicAny::AnySeq ary;
      ary.length(len);
      ary[0] = inany;
      for (CORBA::ULong k = 1; k < len; k++)
        {
          theField = GciGetElem(aField, k);
          assert(theField != NULL);
          inany = encodeField(0, theField);
          ary[k] = inany;
        }
      try
        {
          dyn_array_sl->set_elements(ary);
          any = dyn_array_sl->to_any();
        }
      catch(DynamicAny::DynAny::InvalidValue& ex)
        {
          if (VERBOSITY > 0)
            cout << "encodeField: array slice: invalid value." << endl;
        }
    }
  //exception  
  else if(strcmp(theType,  EXCEPTION_TYPE_PRE) == 0)
    {
      // exception is handled the same as struct
      // but, does TypeCode set correctly, i.e. tk_except?
      if (VERBOSITY > 0)
        cout<<"encodeField: an exception."<< endl;
      unsigned long k;
      GciValue* theField;
      CORBA::ULong len = GciSeqSize(aField);

      CORBA::String_var idltype = 
        CORBA::string_dup(getIDLType(typeName, theType));

      CORBA::Contained_var contained =
        repository_ -> lookup_id(idltype);
      assert(!CORBA::is_nil(contained));
      CORBA::StructDef_var structdef =  CORBA::StructDef::_narrow(contained);
      assert(!CORBA::is_nil(structdef));
      CORBA::StructMemberSeq_var memseq = structdef -> members();
      
      CORBA::TypeCode_var newtc = structdef -> type();      
      DynamicAny::DynAny_var da = 
        dynAnyFactory_->create_dyn_any_from_type_code(newtc);
      DynamicAny::DynStruct_var dyn_struct = 
        DynamicAny::DynStruct::_narrow(da);
      
      DynamicAny::NameValuePairSeq cnt;
      cnt.length(len);
      assert(cnt.length() == dyn_struct->type()->member_count());
      CORBA::Any_var inany;

      for (k = 0; k < len; k++)
        {
          theField = GciGetField(aField, k+GCI_SEQ_OFFSET);
          assert(theField != NULL);
          cnt[k].id = CORBA::string_dup(memseq[k].name);
          inany = encodeField(0, theField);
          assert(inany != NULL);
          cnt[k].value = inany;
        }
      try
	{
	  dyn_struct->set_members(cnt);
	  any = dyn_struct->to_any();
	}
      catch(DynamicAny::DynAny::InvalidValue& ex)
	{
	  if (VERBOSITY > 0)
	    cout << "encodeField: struct: invalid value." << endl;
	}
    }

   //valuebox
  else if(strcmp(theType,  VALUEBOX_TYPE_PRE) == 0)
    {
      if (VERBOSITY > 0)
        cout <<"encodeField: valuebox."<<endl;

      CORBA::String_var id = 
        CORBA::string_dup(getIDLType(typeName, theType));

      CORBA::Contained_var contd = repository_ -> lookup_id(id);
      assert(!CORBA::is_nil(contd));
      
      CORBA::ValueBoxDef_var vbdef = CORBA::ValueBoxDef::_narrow(contd);
      assert(!CORBA::is_nil(vbdef));
      
      CORBA::TypeCode_var vtc = vbdef -> type();
      
      DynamicAny::DynAny_var da = 
        dynAnyFactory_->create_dyn_any_from_type_code(vtc);
      
      DynamicAny::DynValue_var dyn_vl = 
        DynamicAny::DynValue::_narrow(da);
      
      DynamicAny::NameValuePairSeq cnt;
      cnt.length(1);
            
      CORBA::Any_var inany;

      cnt[0].id = CORBA::string_dup("");
      inany = encodeField(1, aField);
      cnt[0].value = inany;
	
      try{
        dyn_vl -> set_members(cnt);
        any = dyn_vl -> to_any();    
      }
      catch(DynamicAny::DynAny::InvalidValue& ex)
        {
          if (VERBOSITY > 0)
            cout << "encodeField: valuebox: invalid value." << endl;
        }

    }
  else if(strcmp(theType, VALUE_TYPE_PRE) ==0)
    {
      if (VERBOSITY > 0)
        cout <<"encodeField: value type."<<endl;

      CORBA::ULong len = GciSeqSize(aField);

      CORBA::String_var id = 
        CORBA::string_dup(getIDLType(typeName, theType));      
      CORBA::Contained_var contd = repository_ -> lookup_id(id);
      assert(!CORBA::is_nil(contd));
      CORBA::ValueDef_var vdef = CORBA::ValueDef::_narrow(contd);
      assert(!CORBA::is_nil(vdef));

      CORBA::ValueDef::FullValueDescription_var vdes =
        vdef -> describe_value();
      CORBA::TypeCode_var vtc = vdef -> type();

      DynamicAny::DynAny_var da = 
        dynAnyFactory_->create_dyn_any_from_type_code(vtc);
      DynamicAny::DynValue_var dyn_vl = 
        DynamicAny::DynValue::_narrow(da);

      DynamicAny::NameValuePairSeq cnt;
      cnt.length(len);
      // why not in case of truncatable?
      // assert(cnt.length() == dyn_vl ->type()->member_count());
      assert(cnt.length() == vdes->members.length());
      CORBA::Any_var inany;
      GciValue* theField;

      for (unsigned long k = 0; k < len; k++)
        {
          theField = GciGetField(aField, k+GCI_SEQ_OFFSET);
          assert(theField != NULL);
          cnt[k].id = CORBA::string_dup(vdes->members[k].name);
          inany = encodeField(0, theField);
          cnt[k].value = inany;
        }
      try
	{	
          dyn_vl -> set_members(cnt);
	  any = dyn_vl -> to_any();    
        }
      catch(DynamicAny::DynAny::InvalidValue& ex)
        {
          if (VERBOSITY > 0)
            cout << "encodeField: value: invalid value." << endl;
        }
    }
  else if(strcmp(theType,  INTF_TYPE_PRE) == 0)
    {
      if (VERBOSITY > 0)
	cout<<"encodeField: an interface." << endl;
      char* s = GciGetIA5String(aField);
      CORBA::Object_var obj = orb_->string_to_object(s);
      assert(!CORBA::is_nil(obj));
      *any <<= obj;
    }
  else
    {
      if (VERBOSITY > 0)
        cout<<"encodeField: nothing matched." <<endl;
    }
  return any;
}


//---------------
// decodeField   |
//---------------

// Decode an ASP field from a CORBA::Any
GciValue* GWCoDec::decodeField(CORBA::Any* any)
{
  GciValue* aField;
  GciValue* tmp;
  GciValue* value;
  int typeDesc = 0;
  int is_alias = FALSE;
  
  int tk = any->type()->kind();

  CORBA::TypeCode_ptr tc;
  if (tk == CORBA::tk_alias)
    {
      tc = any->type()->content_type();
      // GCI seems to be not interested in alias types
      // (typedef). Needs check for other cases
      //      is_alias = TRUE;
      while (tc->kind() == CORBA::tk_alias)
        tc = tc->content_type();
      tk = tc->kind();
    }
          
  switch(tk)
    {
    case CORBA::tk_null:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A null type." << endl;
        tmp = GciMkNULL();
        value = GciSetType(GcCORBA__VoidD, tmp);
        break;
      }
    case CORBA::tk_void:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A void type." << endl;
        tmp = GciMkNULL();
        value = GciSetType(GcCORBA__VoidD, tmp);
        break;
      }
    case CORBA::tk_long:
      {
        CORBA::Long l;
        *any >>= l;
        if (VERBOSITY > 0)
          cout<<"decodeField: A long type:" << l <<endl;
        tmp = GciMkINTEGER(l);
        value = GciSetType(GcCORBA__LongD, tmp);
        break;
      }
    case CORBA::tk_short:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A short type." << endl;
        CORBA::Short s;
        *any >>= s;
        tmp = GciMkINTEGER(s);
        value = GciSetType(GcCORBA__ShortD, tmp);
        break;
      }
    case CORBA::tk_ulong:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: An unsigned long type." << endl;
        CORBA::ULong ul;
        *any >>= ul;
        tmp = GciMkINTEGER(ul);
        value = GciSetType(GcCORBA__ULongD, tmp);
        break;
      }
    case CORBA::tk_longlong:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A long long type." << endl;
        CORBA::LongLong ul;
        *any >>= ul;
        tmp = GciMkINTEGER(ul);
        value = GciSetType(GcCORBA__LongLongD, tmp);
        break;
      }
    case CORBA::tk_ulonglong:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: An unsigned long long type." << endl;
        CORBA::ULongLong ul;
        *any >>= ul;
        tmp = GciMkINTEGER(ul);
        value = GciSetType(GcCORBA__ULongLongD, tmp);
        break;
      }
    case CORBA::tk_ushort:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: An unsigned short type." << endl;
        CORBA::UShort us;
        *any >>= us;
        tmp = GciMkINTEGER(us);
        value = GciSetType(GcCORBA__UShortD, tmp);
        break;
      }
    case CORBA::tk_float:
      {
        CORBA::Float fl;
        *any >>= fl;
        long mt = (long) (fl * pow (10, 6));
        //        tmp = GciMkREAL(mt, 10, -6);
        tmp = GciMkSEQUENCE(3);
        GciSetField(tmp, 0+GCI_SEQ_OFFSET, GciMkINTEGER(mt));
        GciSetField(tmp, 1+GCI_SEQ_OFFSET, GciMkINTEGER(10));
        GciSetField(tmp, 2+GCI_SEQ_OFFSET, GciMkINTEGER(-6));
        
        if (VERBOSITY > 0)
          cout<<"decodeField: float mt: "<<mt 
              << "."<<endl;
        value = GciSetType(GcCORBA__FloatD, tmp);
        break;
      }
    case CORBA::tk_double:
      {
        CORBA::Double fl;
        *any >>= fl;
	// Range 15 seems to be too large. Therefore use 10.
	// See also encode of double.
	//long mt = (long) (fl * pow (10, 15));
        long mt = (long) (fl * pow (10, 10));
        //tmp = GciMkREAL(mt, 10, -15);
        //long mt = (long) frexp(fl, &ep);
        
        tmp = GciMkSEQUENCE(3);
        GciSetField(tmp, 0+GCI_SEQ_OFFSET, GciMkINTEGER(mt));
        GciSetField(tmp, 1+GCI_SEQ_OFFSET, GciMkINTEGER(10));
        GciSetField(tmp, 2+GCI_SEQ_OFFSET, GciMkINTEGER(-15));
        
        if (VERBOSITY > 0)
          cout<<"decodeField: double mt: "<<mt 
              << "."<<endl;
        value = GciSetType(GcCORBA__DoubleD, tmp);
        break;
      }
    case CORBA::tk_char:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A char type." << endl;
        CORBA::Char ch;
        char* aStr = new char[2];
        *any >>= CORBA::Any::to_char(ch);
        aStr[0] = ch;
        aStr[1] = '\0';
        tmp = GciMkGraphicString(CORBA::string_dup(aStr));
        value = GciSetType(GcCORBA__CharD, tmp);
        delete[] aStr;
        break;
      }
    case CORBA::tk_wchar:
      {
        //What needs a wchar more than a char?
        if (VERBOSITY > 0)
          cout<<"decodeField: A schar type." << endl;
        CORBA::WChar ch;
        char* aStr = new char[2];
        *any >>= CORBA::Any::to_wchar(ch);
        aStr[0] = ch;
        aStr[1] = '\0';
        tmp = GciMkGraphicString(CORBA::string_dup(aStr));
        value = GciSetType(GcCORBA__WCharD, tmp);
        delete[] aStr;
        break;
      }
    case CORBA::tk_boolean:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A boolean type." << endl;
        CORBA::Boolean b;
        *any >>= CORBA::Any::to_boolean(b);
        tmp = GciMkBOOLEAN(b);
        value = GciSetType(GcCORBA__BooleanD, tmp);
        break;
      }
    case CORBA::tk_octet:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: An octet type." << endl;
        CORBA::Octet oc;
        *any >>= CORBA::Any::to_octet(oc);
        unsigned long lg = oc;
        
        char* s = new char [9];
        unsigned long r;
        int i, j;
        for(i=7, j=0; i>-1; i--, j++)
          {
            r = (unsigned long) pow ((double) 2, (double) i);
            if(lg >= r)
                {
                  s[j] = '1';
                  lg = lg - r;
                }
            else
                s[j] = '0';
          }
        s[8] = '\0';

        tmp = GciMkOCTET_STRING(s);
        value = GciSetType(GcCORBA__OctetD, tmp);
        break;
      }
    case CORBA::tk_string:
      {
        const char* s;
        *any >>= CORBA::Any::to_string(s,0);
        if (VERBOSITY > 0)
          cout<<"decodeField: A string type." << s <<endl;
        tmp = GciMkGraphicString(s);
        value = GciSetType(GcCORBA__StringD, tmp);
        break;
      }
    case CORBA::tk_wstring:
      {
        //What needs a wstring more than a string?
        if (VERBOSITY > 0)
          cout<<"decodeField: A wstring type." << endl;
        CORBA::WString_var ws;
        *any >>= CORBA::Any::to_wstring(ws,0);
        char* s;
        s[0]= ws[0];
        tmp = GciMkGraphicString(s);
        value = GciSetType(GcCORBA__WStringD, tmp);
        break;
      }
    case CORBA::tk_objref:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: An object reference type."<<endl;
        CORBA::Object_var anObj;
        *any >>= CORBA::Any::to_object(anObj);
        CORBA::String_var s = orb_->object_to_string(anObj);
        tmp = GciMkIA5String(s);
        typeDesc = typeDescFromTCid(any);
        if(typeDesc != 0)
          value = GciSetType(typeDesc, tmp);
        else
          value = NULL;
        break;
      }
    case CORBA::tk_any:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: An any type - ???."
              <<"Should not come here." << endl;
        value = NULL;
        break;
      }
    case CORBA::tk_TypeCode:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A TypeCode type." 
              <<" Not implemented." << endl;
        value = NULL;
        break;
      }
    case CORBA::tk_sequence:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A sequence type." << endl;

        DynamicAny::DynAny_var dyn_any = 
          dynAnyFactory_->create_dyn_any(*any);
        DynamicAny::DynSequence_var dyn_seq = 
          DynamicAny::DynSequence::_narrow(dyn_any);
        assert (!CORBA::is_nil(dyn_seq));

        DynamicAny::AnySeq_var members = dyn_seq->get_elements();
        unsigned long len = members->length();
        value = GciMkSEQUENCEOF();
        
        GciValue* theField;
        CORBA::Any_var anymem = new CORBA::Any;
        for (unsigned long i=0; i<len; i++)
          {
            *anymem = members[i];
            theField = decodeField(anymem);
            assert(theField != NULL);
            GciAddElem(value, theField);
          }
        // need to set type descriptor?
        // if yes, then is_alias = TRUE; 
        break;
      }
    case CORBA::tk_struct:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A struct type." << endl;

        DynamicAny::DynAny_var dyn_any = 
          dynAnyFactory_->create_dyn_any(*any);
        DynamicAny::DynStruct_var dyn_struct = 
          DynamicAny::DynStruct::_narrow(dyn_any);
        assert (!CORBA::is_nil(dyn_struct));
        
        DynamicAny::NameValuePairSeq_var members = dyn_struct->get_members();
        unsigned long len = members->length();
        tmp = GciMkSEQUENCE(len);
        if (len > 0)
          {
            GciValue* theField;
            CORBA::Any_var anymem = new CORBA::Any;
            for (unsigned long i=0; i<len; i++)
              {
                *anymem = members[i].value;
                theField = decodeField(anymem);
                assert(theField != NULL);
                GciSetField(tmp, i+GCI_SEQ_OFFSET, theField);
              }
          }
        typeDesc = typeDescFromTCid(any);
        if(typeDesc != 0)
            value = GciSetType(typeDesc, tmp);
        else
          value = NULL;
        break;
      }
    case CORBA::tk_Principal:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A Principal type." 
              <<"Not implemented." << endl;
        value = NULL;
        break;
      }
    case CORBA::tk_union:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A union type." << endl;

        DynamicAny::DynAny_var dyn_any = 
          dynAnyFactory_->create_dyn_any(*any);
        DynamicAny::DynUnion_var dyn_union = 
          DynamicAny::DynUnion::_narrow(dyn_any);
        assert (!CORBA::is_nil(dyn_union));

        tmp = GciMkSEQUENCE(2);

        // Disciminator
        DynamicAny::DynAny_var dis = dyn_union -> get_discriminator();
        CORBA::Any_var discany = dis -> to_any();
        GciValue* discvalue = decodeField(discany);
        assert(discvalue != NULL);
        GciSetField(tmp, GCI_SEQ_OFFSET, discvalue);

        //body
        DynamicAny::DynAny_var ubody = dyn_union -> member();
        CORBA::Any_var bodyany = ubody -> to_any();
        GciValue* bodyvalue = decodeField(bodyany);
        assert(bodyvalue != NULL);

        CORBA::String_var mn = dyn_union->member_name();
        if (VERBOSITY > 0)
          cout<<"decodeField: union: member name is: "
            <<mn << endl;
        GciValue* body = GciMkCHOICE(CORBA::string_dup(mn), bodyvalue);
        GciSetField(tmp, GCI_SEQ_OFFSET+1, body);

        typeDesc = typeDescFromTCid(any);
        if (typeDesc != 0)
          value = GciSetType(typeDesc, tmp);
        else
          value = NULL;
        break;
      }
    case CORBA::tk_enum:
      { 
        if (VERBOSITY > 0)
          cout<<"decodeField: An enum type." << endl;

        DynamicAny::DynAny_var dyn_any = 
          dynAnyFactory_->create_dyn_any(*any);
        DynamicAny::DynEnum_var dyn_enum = 
          DynamicAny::DynEnum::_narrow(dyn_any);
        assert (!CORBA::is_nil(dyn_enum));

        // value as string
        // Not used by GCI/ITEX3.2! 
        const char* s = dyn_enum->get_as_string();
        if (VERBOSITY > 0)
          cout<<"decodeField: enum : value in string '" << s
              <<"'." << endl;
        CORBA::ULong num = dyn_enum->get_as_ulong();
        if (VERBOSITY > 0)
          cout<<"decodeField: enum : value in ulong '" << num
              <<"'." << endl;
        tmp = GciMkENUMERATED(num);
        typeDesc = typeDescFromTCid(any);
        if(typeDesc != 0)
            value = GciSetType(typeDesc, tmp);
        else
          value = NULL;
        break;
      }
    
    case CORBA::tk_array:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: An array type." << endl;

        DynamicAny::DynAny_var dyn_any = 
          dynAnyFactory_->create_dyn_any(*any);
        DynamicAny::DynArray_var dyn_array = 
          DynamicAny::DynArray::_narrow(dyn_any);
        assert (!CORBA::is_nil(dyn_array));
        
        DynamicAny::AnySeq_var members = dyn_array->get_elements();
        unsigned long len = dyn_array->component_count();
        value = GciMkSEQUENCEOF();
        
        GciValue* theField;
        CORBA::Any_var anymem = new CORBA::Any;
        for (unsigned long i=0; i<len; i++)
          {
            *anymem = members[i];
            theField = decodeField(anymem);
            assert(theField != NULL);
            GciAddElem(value, theField);
          }
        /*
        typeDesc = typeDescFromTCid(any);
        if(typeDesc != 0)
            value = GciSetType(typeDesc, tmp);
        else
        value = NULL;
        */
        break;
      }
    case CORBA::tk_alias:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: An alias type - "
              <<"shall not come here, because this type is already "
              <<"considered at the beginning of the procedure." 
              << endl;
        value = NULL;
        break;
      }
    case CORBA::tk_except:
      {
        //DynStruct can be used to handle exception values 9.2.6/CORBA2.3)
        if (VERBOSITY > 0)
          cout<<"decodeField: An except type." << endl;
        
        DynamicAny::DynAny_var dyn_any = 
          dynAnyFactory_->create_dyn_any(*any);
        DynamicAny::DynStruct_var dyn_struct = 
          DynamicAny::DynStruct::_narrow(dyn_any);
        assert (!CORBA::is_nil(dyn_struct));
        
        DynamicAny::NameValuePairSeq_var members = dyn_struct->get_members();
        unsigned long len = members->length();
        tmp = GciMkSEQUENCE(len);
        if (len > 0)
          {
            GciValue* theField;
            CORBA::Any_var anymem = new CORBA::Any;
            for (unsigned long i=0; i<len; i++)
              {
                *anymem = members[i].value;
                theField = decodeField(anymem);
                assert(theField != NULL);
                GciSetField(tmp, i+GCI_SEQ_OFFSET, theField);
              }
          }
        typeDesc = typeDescFromTCid(any);
        if(typeDesc != 0)
            value = GciSetType(typeDesc, tmp);
        else
          value = NULL;
        break;

      }
    case CORBA::tk_value:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A value type." << endl;

        DynamicAny::DynAny_var dyn_any = 
          dynAnyFactory_->create_dyn_any(*any);
        DynamicAny::DynValue_var dyn_value = 
          DynamicAny::DynValue::_narrow(dyn_any);
        assert (!CORBA::is_nil(dyn_value));
        
        DynamicAny::NameValuePairSeq_var members = dyn_value->get_members();
        unsigned long len = members->length();
        tmp = GciMkSEQUENCE(len);
        if (len > 0)
          {
            GciValue* theField;
            CORBA::Any_var anymem = new CORBA::Any;
            for (unsigned long i=0; i<len; i++)
              {
                *anymem = members[i].value;
                theField = decodeField(anymem);
                assert(theField != NULL);
                GciSetField(tmp, i+GCI_SEQ_OFFSET, theField);
              }
          }
        typeDesc = typeDescFromTCid(any);
        if(typeDesc != 0)
            value = GciSetType(typeDesc, tmp);
        else
          value = NULL;
        break;
      }
    case CORBA::tk_value_box:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: A valuebox type." << endl;

        DynamicAny::DynAny_var dyn_any = 
          dynAnyFactory_->create_dyn_any(*any);
        DynamicAny::DynValue_var dyn_value = 
          DynamicAny::DynValue::_narrow(dyn_any);
        assert (!CORBA::is_nil(dyn_value));
        
        DynamicAny::NameValuePairSeq_var members = dyn_value->get_members();
        assert(members->length() == 1);
        
        CORBA::Any_var anymem = new CORBA::Any;
        *anymem = members[0].value;
        tmp = decodeField(anymem);
        assert(value != NULL);

        typeDesc = typeDescFromTCid(any);
        if(typeDesc != 0)
            value = GciSetType(typeDesc, tmp);
        else
          value = NULL;
        break;
      }
    default:
      {
        if (VERBOSITY > 0)
          cout<<"decodeField: nothing matched." << endl;
        value = NULL;
        break;
      }
    }

  if(is_alias == TRUE)
    {
      typeDesc = typeDescFromTCid(any);
      aField = GciSetType(typeDesc, value);
    }
  else
    aField = value;
  
  return aField;
}


//-----------------
// makeGWMessage   |
//-----------------

// Convert a ASP message to a GWMessage 
GciStatus GWCoDec::makeGWMessage(GciValue* asp, TOR::GWMessage*& imsg)
{
  TOR::GWMessage_var msg = new TOR::GWMessage;
    
  CORBA::String_var aspID = 
    CORBA::string_dup(GciGetTypeNameString(GciGetType(asp)));
  if (VERBOSITY > 0)
    cout <<"makeGWMessage: The type name of the ASP is '" 
         << aspID <<"'" << endl;
  
  TOR::GWMsgContentType cntType;
  if(strncmp(aspID, REQUEST_ASP, strlen(REQUEST_ASP))==0)
    cntType = TOR::Parameters;
  else if (strncmp(aspID, REPLY_ASP, strlen(REPLY_ASP))==0)
    cntType = TOR::ReturnAndParameters;
  else if (strncmp(aspID, REQUEST_ASP, strlen(RAISE_ASP))==0)
    cntType = TOR::IsException;
  else
    {
      // Wrong prefix
      if (VERBOSITY > 0)
        cout<<"makeGWMessage: Wrong prefix - encoding is not executed." 
            << endl;
      return GciNotOk;
    }  
  
  // The object name
  msg->objectName = 
    CORBA::string_dup(GciGetGraphicString
                      (GciGetField(asp, 0+GCI_SEQ_OFFSET)));
  if (VERBOSITY > 0)
    cout <<"makeGWMessage: The object name is '" 
         << msg->objectName <<"'"<<endl;

  // The operation name
  msg->operationName = 
    CORBA::string_dup(GciGetGraphicString
                      (GciGetField(asp, 1+GCI_SEQ_OFFSET)));
  if (VERBOSITY > 0)
    cout <<"makeGWMessage: The operation name is '" 
         << msg->operationName <<"'"<<endl;


  // Complete the msg content
  switch(cntType)
    {
    case TOR::Parameters:
      {
        // Probably a request message
        CORBA::ULong len = GciSeqSize(asp) -2;
                        
        if(len == 0)
          msg->cntType = TOR::IsVoid; // No parameter
        else
          {
            msg->cntType = TOR::Parameters;
            TOR::GWParamList_var pList = new TOR::GWParamList;
            pList->length(len);
            TOR::GWElem_var elm;
            GciValue* theField;
            for (unsigned long k = 0; k < len; k++)
              {
                theField = GciGetField(asp, k+2+GCI_SEQ_OFFSET);
                assert(theField != NULL);
                elm = encodeField(0, theField);
                pList[k] = elm;
              }
            msg->content.paramList = pList;
          }
        imsg = new TOR::GWMessage(msg);
        return GciOk;
      }
    case TOR::ReturnAndParameters:
      {
        // A reply message
        unsigned long len =  GciSeqSize(asp)-2;
        
        // The return value field is used even
        // for void type.
        assert (len >= 1);
        
	msg->cntType = TOR::ReturnValue;
	TOR::GWElem_var rv;
	GciValue* theField = GciGetField(asp, 2+GCI_SEQ_OFFSET);
	if (GciGetType(theField) != GcCORBA__VoidD)
	  {
	    rv = encodeField(0, theField);
	    msg->content.returnValue = rv;
	  }
	          
        if (len > 1)
          {
            // Parameters
            msg->cntType = TOR::ReturnAndParameters;
            TOR::GWParamList_var pList = new TOR::GWParamList;
            pList->length(len-1);
            TOR::GWElem_var elm;
            GciValue* theField;
            for (unsigned long k = 0; k < len-1; k++)
              {
                theField = GciGetField(asp, k+3+GCI_SEQ_OFFSET);
                elm = encodeField(0, theField);
                pList[k] = elm;
              }
            msg->content.paramList = pList;
          }
        imsg = new TOR::GWMessage(msg);
        return GciOk;
      }
    case TOR::IsException:
      {
        //todo***: check mapping!
        // An exception message (only user-defined exceptions!)
        // IDL exception types are mapped to SEQUENCE
        // An operation raise is mapped to CHOICE
        msg->cntType = TOR::IsException;
        GciValue* sub = GciGetCHOICE(asp);
        TOR::GWElem_var ex= encodeField(0, sub);
        msg->content.except = ex;
        imsg = new TOR::GWMessage(msg);
        return GciOk;
      }
    default:
      {
        return GciNotOk;
      }
    }
}

//--------------
// makeGciObj   |
//--------------
     
// Convert a GWMessage to an ASP message
GciStatus GWCoDec::makeGciObj(const TOR::GWMessage* msg, GciValue*& asp)
{
  CORBA::String_var opn = msg->operationName;
  CORBA::String_var tmpName = msg->objectName;
  
  // For scope name, replace '::' by '__'
  for (unsigned long i=0; i<strlen(tmpName); i++)
    {
      if (tmpName[i] == ':')
        tmpName[i] = '_';
    }
    
  // append operation name

  tmpName += "__";
  tmpName += opn;
  if (VERBOSITY > 0)
    cout<<"makeGciObj: The second part of the ASP type name is '" 
        << tmpName <<"'" <<endl;
  
  CORBA::String_var typeName;
  GciValue* value;

  switch (msg->cntType)
    {
    case TOR::Parameters:
      {
        // A request msg
        typeName = CORBA::string_dup(REQUEST_ASP);
        if (VERBOSITY > 0)
          cout<<"makeGciObj: The first part of the ASP type name is '" 
              << typeName <<"'" <<endl;
                       
        // Construct the fields of the ASP
        CORBA::ULong len =  msg->content.paramList.length();
        value = GciMkSEQUENCE(len+2);
        
        // Object name and operation name
        GciSetField(value, 0+GCI_SEQ_OFFSET, 
                    GciMkGraphicString(msg->objectName));
        GciSetField(value, 1+GCI_SEQ_OFFSET, 
                    GciMkGraphicString(msg->operationName));
        
        // Other fields
        for(CORBA::ULong i=0; i<len; i++)
          {
             GciValue* theField;
             CORBA::Any_var any = new CORBA::Any;
             *any = msg->content.paramList[i];
             theField = decodeField(any);
             assert(theField != NULL);
             GciSetField(value, i+2+GCI_SEQ_OFFSET, theField);
          }
        break;
      }
    case TOR::ReturnValue:
      {
        // A reply msg
        typeName = CORBA::string_dup(REPLY_ASP);
        if (VERBOSITY > 0)
          cout<<"makeGciObj: The first part of the ASP type name is '" 
              << typeName <<"'" <<endl;

        // Construct the field of the ASP
        value = GciMkSEQUENCE(3);
        
        // Object name and operation name
        GciSetField(value, 0+GCI_SEQ_OFFSET, 
                    GciMkGraphicString(msg->objectName));
        GciSetField(value, 1+GCI_SEQ_OFFSET, 
                    GciMkGraphicString(msg->operationName));
        
        CORBA::Any_var any = new CORBA::Any;
        *any = msg->content.returnValue;
        GciValue* theField;
        theField = decodeField(any);
        assert(theField != NULL);
        GciSetField(value, 2+GCI_SEQ_OFFSET, theField);
        break;
      }
    case TOR::ReturnAndParameters:
      {
        // A reply msg
        if (VERBOSITY > 0)
          cout<<"makeGciObj: A reply msg with return value and parameter(s)." 
              << endl;
        typeName = CORBA::string_dup(REPLY_ASP);
        if (VERBOSITY > 0)
          cout<<"makeGciObj: The first part of the ASP type name is '" 
              << typeName <<"'" <<endl;
        
        // Construct the fields of the ASP
        CORBA::ULong len = msg->content.paramList.length();
        value = GciMkSEQUENCE(len+3);

        // Object name and operation name
        GciSetField(value, 0+GCI_SEQ_OFFSET, 
                    GciMkGraphicString(msg->objectName));
        GciSetField(value, 1+GCI_SEQ_OFFSET, 
                    GciMkGraphicString(msg->operationName));
                        
        // First field for return value
        CORBA::Any_var rv = new CORBA::Any;
        *rv = msg->content.returnValue;
        GciValue* rField;
        rField = decodeField(rv);
        assert(rField != NULL);
        GciSetField(value, 2+GCI_SEQ_OFFSET, rField);
                
        // Other fields
        for(CORBA::ULong i=0; i<len; i++)
          {
             GciValue* theField;
             CORBA::Any_var any = new CORBA::Any;
             *any = msg->content.paramList[i];
             theField = decodeField(any);
             assert(theField != NULL);
             GciSetField(value, i+3+GCI_SEQ_OFFSET, theField);
          }
        break;
      }
    case TOR::IsVoid:
      {
        // A request msg
        typeName = CORBA::string_dup(REQUEST_ASP);
        if (VERBOSITY > 0)
          cout<<"makeGciObj: The first part of the ASP type name is '" 
              << typeName <<"'" <<endl;
        value = GciMkSEQUENCE(2);
        
        // Object name and operation name
        GciSetField(value, 0+GCI_SEQ_OFFSET, 
                    GciMkGraphicString(msg->objectName));
        GciSetField(value, 1+GCI_SEQ_OFFSET, 
                    GciMkGraphicString(msg->operationName));
        
        break;
      }
    case TOR::IsException:
      {
        // An exception
        CORBA::Any_var eany = new CORBA::Any;
        *eany = msg->content.except;
        
        CORBA::String_var eid = CORBA::string_dup(eany->type()->name());
        cout <<"makeGciObj: to decode exception: "
             <<"name is: "<< eid <<endl;

        int typed = typeDescFromTCid(eany);
        // To be generalized for all CORBA exceptions and possible
        // Gateway exceptions.

        if(typed == GcEXCEPT__CORBA__COMM_FAILURED || 
           typed == GcEXCEPT__CORBA__UNKNOWND ||
           typed == GcEXCEPT__CORBA__TRANSIENTD)
       
          {
            // A CORBA system exception
            // To do - An ASP may be defined for system exceptions,
            // since a system exception (in general)
            // shall not cause verdict FAIL,
            // but INCONCLUSIVE. Currently, system exeptions are
            // caught usually by OTHERWISE.

            if (VERBOSITY > 0)
              cout<<"makeGciObj: a system exception."<< endl;
            
            char* s1 = (char*) GciGetTypeNameString(typed);
            char* s2 = new char[strlen(s1)+1-strlen("EXCEPT__")];
            strcpy(s2, &s1[8]);
            if (VERBOSITY > 0)
              cout<<"makeGciObj: a system exception:"
                  << s2 << endl;

            value = GciMkSEQUENCE(0);
            typeName = CORBA::string_dup(RAISE_ASP);
            tmpName = CORBA::string_dup(s2);
            delete[] s2;
            
          }
        else
          {
            // A user-defined exception - assumption!
            if (VERBOSITY > 0)
              cout<<"makeGciObj: a user exception."<< endl;

            value = GciMkSEQUENCE(3);
        
            // Object name and operation name
            GciSetField(value, 0+GCI_SEQ_OFFSET, 
                        GciMkGraphicString(msg->objectName));
            GciSetField(value, 1+GCI_SEQ_OFFSET, 
                        GciMkGraphicString(msg->operationName));
        
            GciValue* theField = decodeField(eany);
            assert(theField != NULL);
            GciValue* eField = GciMkCHOICE(CORBA::string_dup(eid), theField);
            GciSetField(value, 2+GCI_SEQ_OFFSET, eField);
          }
        break;
      }
    }
  
  // Construct the ASP type name
  typeName += tmpName;
  if (VERBOSITY > 0)
    cout<<"makeGciObj: The constructed ASP type name is '" << typeName
        << "'" << endl;
  
  // Set type name
  GciSetType(GciGetTypeDescriptor(typeName), value);
  asp = value;
  return GciOk;
                         
}

void GWCoDec::register_this_factory(const char* id)
{
  if((orb_ -> lookup_value_factory(id))!= NULL)
    {
      if(VERBOSITY > 0)
	cout <<"gwCoDec: register_this_factory: "<<
	  id << " already registered."<< endl;
      return;
    }

  CORBA::Contained_var contained = repository_ -> lookup_id(id);
  assert(!CORBA::is_nil(contained));

  CORBA::ValueFactoryBase_var vbf, oldf;

  if(strcmp(id, "IDL:OBVTest/TestValueA:1.0") == 0)
    vbf = new TestValueAFactory_impl();

  else if(strcmp(id, "IDL:OBVTest/TestValueB:1.0") == 0)
    vbf = new TestValueBFactory_impl();

  else if(strcmp(id, "IDL:OBVTest/TestValueC:1.0") == 0) 
    vbf = new TestValueCFactory_impl();

  else if(strcmp(id, "IDL:OBVTest/TestValueD:1.0") == 0) 
    vbf = new TestValueDFactory_impl();

  else if(strcmp(id, "IDL:OBVTest/TestShortBox:1.0") == 0)
    vbf = new OBVTest::TestShortBox_init;

  else if(strcmp(id, "IDL:OBVTest/TestUShortBox:1.0") == 0)
    vbf = new OBVTest::TestUShortBox_init;

  else if(strcmp(id, "IDL:OBVTest/TestLongBox:1.0") == 0)
    vbf = new OBVTest::TestLongBox_init;

  else if(strcmp(id, "IDL:OBVTest/TestULongBox:1.0") == 0)
    vbf = new OBVTest::TestULongBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestLLongBox:1.0") == 0)
    vbf = new OBVTest::TestLLongBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestULLongBox:1.0") == 0)
    vbf = new OBVTest::TestULLongBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestBooleanBox:1.0") == 0)
    vbf = new OBVTest::TestBooleanBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestOctetBox:1.0") == 0)
    vbf = new OBVTest::TestOctetBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestCharBox:1.0") == 0)
    vbf = new OBVTest::TestCharBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestWCharBox:1.0") == 0)
    vbf = new OBVTest::TestWCharBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestFloatBox:1.0") == 0)
    vbf = new OBVTest::TestFloatBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestDoubleBox:1.0") == 0)
    vbf = new OBVTest::TestDoubleBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestStringBox:1.0") == 0)
    vbf = new OBVTest::TestStringBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestEnumBox:1.0") == 0)
    vbf = new OBVTest::TestEnumBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestArrayBox:1.0") == 0)
    vbf = new OBVTest::TestArrayBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestFixStructBox:1.0") == 0)
    vbf = new OBVTest::TestFixStructBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestVarStructBox:1.0") == 0)
    vbf = new OBVTest::TestVarStructBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestFixUnionBox:1.0") == 0)
    vbf = new OBVTest::TestFixUnionBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestVarUnionBox:1.0") == 0)
    vbf = new OBVTest::TestVarUnionBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestShSeqBox:1.0") == 0)
    vbf = new OBVTest::TestShSeqBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestStrSeqBox:1.0") == 0)
    vbf = new OBVTest::TestStrSeqBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestAnyBox:1.0") == 0)
    vbf = new OBVTest::TestAnyBox_init;
  else
    cout <<"gwCoDec: register_this_factory: should not come here."
         <<"strcmp failed."<<endl;

  oldf = orb_ -> register_value_factory(id, vbf);
  if(VERBOSITY > 0)
    cout <<"gwCoDec: register_this_factory: "<<
         id <<endl;
  return;
}

void GWCoDec::unregister_this_factory(const char* id)
{
  if((orb_ -> lookup_value_factory(id))!= NULL)
    {
      orb_ -> unregister_value_factory(id);
      if(VERBOSITY > 0)
        cout <<"gwCoDec: unregister_this_factory: "<<
          id <<endl;
    }
  return;
}


void whichSysExcept(CORBA::SystemException* sx)
{
  if(CORBA::BAD_PARAM::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: bad parm."<<endl;

  else if(CORBA::INV_OBJREF::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: invalid objref."<<endl;

  else if(CORBA::MARSHAL::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: marshal."<<endl;

  else if(CORBA::BAD_TYPECODE::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: bad typecode."<<endl;
  
  else if(CORBA::BAD_OPERATION::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: bad operation."<<endl;
  
  else if(CORBA::NO_RESPONSE::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: no response."<<endl;
  
  else if(CORBA::NO_IMPLEMENT::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: no implement."<<endl;
  
  else if(CORBA::COMM_FAILURE::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: comm failure."<<endl;

  else if(CORBA::TRANSIENT::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: transient."<<endl;
  
  else if(CORBA::INV_IDENT::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: inv ident."<<endl;
  
  else if(CORBA::INV_FLAG::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: inv flag."<<endl;
  
  else if(CORBA::INTF_REPOS::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: intf repository."<<endl;
  
  else if(CORBA::BAD_CONTEXT::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: bad context."<<endl;
  
  else if(CORBA::OBJ_ADAPTER::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: obj adapter."<<endl;
  
  else if(CORBA::DATA_CONVERSION::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: data conversion."<<endl;
  
  else if(CORBA::OBJECT_NOT_EXIST::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: object not exist."<<endl;
  
  else if(CORBA::INVALID_TRANSACTION::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: invalid transaction."<<endl;
  
  else if (CORBA::UNKNOWN::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: unknown."<<endl;

  else
    cout<<"whichSysExcept: not found on the list."<<endl;
  
  return;
  
 }
