/*---------------------------------------------------------------------------
 * 
 *    File Name: coding.c
 *    Author:    Stephan Pietsch
 *    Revision:  $Id: string_coding.c,v 1.1 2001/01/31 17:20:54 mli Exp $
 *
 *    These are the functions for encoding and decoding of 
 *    PDUs for transmission
 *
 *
 *  Encoding and Decoding
 *
 *	Transfer syntax:
 *
 *	Message		::= Value ';'
 *	Value		::= Choice | Sequence | SequenceOf | Base
 *      Choice          ::=   TYPEID '<' ValueList '>'
 *	SequenceOf	::=   TYPEID '{' ValueList '}' 
 *			    | TYPEID '[' ValueList ']' 
 *			    |  '[' ValueList ']'
 *	Sequence	::=   TYPEID '{' ValueList '}' 
 *                          | TYPEID '[' ValueList ']' 
 *			    | '{' ValueList '}'
 *	ValueList	::= <empty> | Value { ,Value }
 *	Base		::=   Boolean | Number | BString | HString 
 *                          | OString | Enumerated | CString 
 *	Boolean		::= TRUE | FALSE
 *	Number		::= [- ][0-9][0-9]*
 *	Bstring		::= `'`[01]*`'`'B'
 *	Hstring		::= `'`[0-9A-F]*`'`'H'
 *	Ostring		::= `'`([0-9A-F][0-9A-F])*`'`'O'
 *      Enumerated      ::= TYPEID  
 *	Cstring		::= TYPEID TextString
 *      TextString	::= `"` [^"]* `"`
 *	TYPEID		::= [a-zA-Z][_0-9a-zA-Z]* -- same name as in ATS
 *	
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include "string_coding.h"

#define TRUE 1
#define FALSE 0
#define MSG_BUF_SIZE             1024 

static char *rcsid="$Id: string_coding.c,v 1.1 2001/01/31 17:20:54 mli Exp $";

char *GeneralStringEncode_Value ( GciValue *object, char *buf)
{
  int i;
  char tmp[MSG_BUF_SIZE];
  
  switch( GciGetType( object))
    {
    case VcBOOLEAND:
      sprintf( tmp, "%s ", GciGetBOOLEAN( object) ? "TRUE" : "FALSE");
      strcat( buf, tmp);
      break;
    case VcINTEGERD:
      sprintf( tmp, "%ld ", GciGetINTEGER( object));
      strcat( buf, tmp);
      break;
    case VcBIT_STRINGD:
      if( GciGetBIT_STRING( object) != NULL) {
	sprintf( tmp, "'%s'B ", GciGetBIT_STRING( object));
	strcat( buf, tmp);
      }
      break;
    case VcOCTET_STRINGD:
      if( GciGetOCTET_STRING( object) != NULL) {
	sprintf( tmp, "'%s'O ", GciGetOCTET_STRING( object));
	strcat( buf, tmp);
      }
      break;
    case VcIA5StringD:
      if( GciGetIA5String( object) != NULL) {
	sprintf( tmp, "\"%s\" ", GciGetIA5String( object));
	strcat( buf, tmp);
      }
      break;
    case VcPrintableStringD:
      if( GciGetPrintableString( object) != NULL) {
	sprintf( tmp, "\"%s\" ", GciGetPrintableString( object));
	strcat( buf, tmp);
      }
      break;
    case VcENUMERATEDD:
      if( GciGetENUMERATED( object) != 0L) {
	sprintf( tmp, "%d ", GciGetENUMERATED( object));
	strcat( buf, tmp);
      }
      break;
    case VcSEQUENCED:
    case VcSETD:
      if( ( GciGetTypeNameString( GciGetType( object)) != NULL) && ( GciSeqSize( object) > 0)) {
	strcat( buf, GciGetTypeNameString( GciGetType( object)));
	strcat( buf, " { ");
	for (i = 0; i < GciSeqSize( object);) {
	  i++;
	  buf = GeneralStringEncode_Value( GciGetField( object, i), buf);
	  if( i < GciSeqOfSize( object))
	    strcat( buf, ",  ");
	}
	strcat( buf, " } ");
      }
      break;
    case VcSET_OFD:
    case VcSEQUENCE_OFD:
      if( ( GciGetTypeNameString( GciGetType( object)) != NULL) && ( GciSeqOfSize( object) > 0)) {
	strcat( buf, GciGetTypeNameString( GciGetType( object)));
	strcat( buf, " [ ");
	for (i = 0; i < GciSeqOfSize( object);) {
	  buf = GeneralStringEncode_Value( GciGetElem( object, i), buf);
	  i++;
	  if( i < GciSeqOfSize( object))
	    strcat( buf, ",  ");
	}
	strcat( buf, " ] ");
      }
      break;
    default:
      if(GciGetTypeNameString( GciGetType( object)) != NULL) {
	strcat( buf, GciGetTypeNameString( GciGetType( object)));
	strcat( buf, " { ");
	if(GciSeqSize( object) > 0) {
	  for( i = 0; i < GciSeqSize( object);) {
	    i++;
	    buf = GeneralStringEncode_Value( GciGetField( object, i), buf);
	    if( i < GciSeqOfSize( object))
	      strcat( buf, ", "); 
	  }
	}
	strcat( buf, " } ");
      }
    }
  /*  printf( "current encode buffer: %s\n", buf);*/
  return buf;
}

char *GeneralStringEncode( GciValue *object)
{
  char *buf;

  if ( object != NULL) {
    buf = (char *)malloc( MSG_BUF_SIZE);

    GeneralStringEncode_Value( object, buf);
    /*    strcat( buf, ";\0"); */
    return buf;
  }
  else {
    fprintf( stderr, "GeneralStringEncode: NULL object to encode !\n");
    return "";
  }
}

/*---------------------------------------------------------------------------
 *
 *    Decoder functions
 */

long generaldecode_integer;
char generaldecode_string[2048];

/* tokens */

#define GENERALDECODE__ERROR	       -1
#define GENERALDECODE__OTHER		0
#define GENERALDECODE__SEQUENCE         1
#define GENERALDECODE__CHOICE           2
#define GENERALDECODE__BOOLEAN		3
#define GENERALDECODE__INTEGER		4
#define GENERALDECODE__BSTRING		5
#define GENERALDECODE__HSTRING		6
#define GENERALDECODE__OSTRING		7
#define GENERALDECODE__CSTRING		8
#define GENERALDECODE__PSTRING		9
#define GENERALDECODE__ENUMERATED      10
#define GENERALDECODE__GSTRING	       11

int GeneralStringDecode__LookAhead( char **msg)
{
  int  c, i;
  char *pos = generaldecode_string;

  while(( c = ( *((*msg)++))) == ',' || isspace( c)) /* do nothing */ ;
  switch( c) 
       {
	 case '}':
	 case ']':
	 case '>':
	       return GENERALDECODE__OTHER;
	 case ';':
	       *(*msg--);
	       return GENERALDECODE__OTHER;
	 case '\'':
	       *(pos++) = c = *((*msg)++);
	       while( c != '\'')
		    *(pos++) = c = *((*msg)++);
	       *(--pos) = '\0';
	       switch( *((*msg)++))
		    {
		      case 'B':
			    return GENERALDECODE__BSTRING;
		      case 'H':
			    return GENERALDECODE__HSTRING;
		      case 'O':
			    return GENERALDECODE__OSTRING;
		      case 'G':
			    return GENERALDECODE__GSTRING;
		      default:
			    return GENERALDECODE__ERROR;
		    }
	 case '"':
	       sscanf( *msg, "%[_0-9a-zA-Z]", generaldecode_string);
	       for( i = -1; i != strlen( generaldecode_string); i++) 
		    *((*msg)++);
	       return GENERALDECODE__PSTRING;
	 case '{':
	 case '[':
	       sprintf (generaldecode_string, "SEQUENCE%s", c == '{' ? "" : "_OF");
	       return GENERALDECODE__SEQUENCE;
	 case '-':
	       sscanf( *msg, "%d", &generaldecode_integer);
	       generaldecode_integer = -generaldecode_integer;
	       return GENERALDECODE__INTEGER;
	 default: 
	       *((*msg)--);
	       if( isdigit( c)) {
		    sscanf( *msg, "%d", &generaldecode_integer);
		    while( !isspace( *((*msg)++)));
		    return GENERALDECODE__INTEGER;
	       }
	       sscanf( *msg, "%[_0-9a-zA-Z]", generaldecode_string);
	       for( i = 0; i < strlen( generaldecode_string); i++)
		    *((*msg)++);
	       if( !strcmp( "TRUE", generaldecode_string)) {
		    generaldecode_integer = 1;
		    return GENERALDECODE__BOOLEAN;
	       }
	       if( !strcmp( "FALSE", generaldecode_string)) {
		    generaldecode_integer = 0;
		    return GENERALDECODE__BOOLEAN;
	       }
	       while( isspace( c = *((*msg)++)));/* skip blanks */
	       if( c == '"') {			/* character string */
		    sscanf( *msg, "%[^\"]", generaldecode_string);
		    *((*msg)++);		/* remove " */
		    return GENERALDECODE__CSTRING;
	       }
	       if( c == '<') {
		    return GENERALDECODE__CHOICE;
	       }
	       if( c == '{' || c == '[') {
		    return GENERALDECODE__SEQUENCE;
	       }
	       return GENERALDECODE__ERROR;
       }
}


GciValue *GeneralStringDecode__Value( char **msg)
{
  GciValue *objt;
  int       type_d;
  
  switch( GeneralStringDecode__LookAhead( msg))
    {
    case GENERALDECODE__SEQUENCE: 
      {
	GciValue *item;

	objt = GciMkSEQUENCEOF();
	GciSetType( GciGetTypeDescriptor( generaldecode_string), objt);
	for ( item = GeneralStringDecode__Value( msg); GciGetType( item) !=  VcNULLD; item = GeneralStringDecode__Value( msg)) {
	  GciAddElem( objt, item);
	}
	break; 
      }
    case GENERALDECODE__CHOICE: 
      {
	type_d = GciGetTypeDescriptor( generaldecode_string);
	objt = GciMkCHOICE( "GcVERSIONS", GeneralStringDecode__Value( msg));
	GciSetType( type_d, objt);
	break; 
      }
    case GENERALDECODE__BOOLEAN:
      objt = GciMkBOOLEAN (generaldecode_integer);
      break;
    case GENERALDECODE__INTEGER:
      objt = GciMkINTEGER (generaldecode_integer);
      break;
    case GENERALDECODE__BSTRING:
      objt = GciMkBIT_STRING (generaldecode_string);
      break;
    case GENERALDECODE__HSTRING:
      objt = GciMkHEXSTRING (generaldecode_string);
      break;
    case GENERALDECODE__OSTRING:
      objt = GciMkOCTET_STRING (generaldecode_string);
      break;
    case GENERALDECODE__CSTRING:
      objt = GciMkIA5String (generaldecode_string);
      break;
    case GENERALDECODE__PSTRING:
      objt = GciMkPrintableString( generaldecode_string);
      break;
    case GENERALDECODE__GSTRING:
      objt = GciMkPrintableString( generaldecode_string);
      break;
    case GENERALDECODE__ENUMERATED:
      objt = GciMkENUMERATED( generaldecode_integer);
      break;
    case GENERALDECODE__ERROR:
    default:
      objt = GciMkNULL();/*( GciValue *) NULL;*/
    }
  return objt;
}

GciValue *GeneralStringDecode( char *msg)
{
  GciValue *val;
printf("GeneralStringDecode: %s\n", msg);
  val = GeneralStringDecode__Value( &msg);
  if( val) {
       printf("return\n");
       return val;
  }
  else {
    fprintf( stderr, "can't decode received message!\n");
    return NULL;
  }
}

