/*--------------------------------------------------------------------
 * Name:   valrep.h
 * Author: ITEX C-Code Generator
 *
 * Copyright 1991-2000 Telelogic AB, All rights reserved.
 * 
 * This Program is owned by Telelogic and is protected by national
 * copyright laws and international copyright treaties. Telelogic
 * grants you the right to use this Program on one computer or in one
 * local computer network at any one time.  Under this License you may
 * only modify the source code for the purpose of adapting it to your
 * environment. You must reproduce and include any copyright and
 * trademark notices on all copies of the source code.  You may not
 * use, copy, merge, modify or transfer the Program except as provided
 * in this License.  Telelogic does not warrant that the Program will
 * meet your requirements or that the operation of the Program will be
 * uninterrupted and error free. You are solely responsible that the
 * selection of the Program and the modification of the source code
 * will achieve your intended results and that the results are
 * actually obtained.
 *
 * This is the interface to the value representation in the ccg.  The
 * functions corresponding to this specifications can be found in the
 * file valrep.c
 * 
 * Additional documentation on the use of the functions, the data
 * representation model and the mapping from the TTCN/ASN.1 can be
 * found in the document ITEX CCG.
 */

#ifndef VALREP_H
#define VALREP_H

#ifdef _MSC_VER
#pragma warning(disable : 4100) /* unreferenced formal parameter                */
#pragma warning(disable : 4127) /* conditional expression is constant           */
#pragma warning(disable : 4505) /* unreferenced local function has been removed */
#pragma warning(disable : 4706) /* assignment within conditional expression     */
#pragma warning(disable : 4711) /* function selected for inline expansion       */
#endif

#include <stdlib.h>
#include <limits.h>

/*--------------------------------------------------------------------
 * Forward declarations for structs used below...
 */
#ifdef __cplusplus
extern "C" {
#endif

typedef struct VcTypeList VcTypeList;
typedef struct VcValue VcValue;
typedef struct VcType VcType;
typedef struct VcRestriction VcRestriction;
typedef struct VcList VcList;
typedef struct VcReal VcReal;
typedef struct VcChoice VcChoice;
typedef struct VcPCOQueue VcPCOQueue;
typedef struct VcPCOQueueObject VcPCOQueueObject;
typedef struct VcTimerType VcTimerType;
typedef struct VcPTCType VcPTCType;

#ifdef __cplusplus
}
#endif

/* Borland C doesn't like empty parameter lists.  Make sure there
 * isn't any empty parameter lists. */

/* Do NOT remove this define or the logging will crash!! */

#ifndef LOCAL
#define CCGLOG			/* The Beta always has logging! */
#ifdef CCGLOG
#define IcLog(id, str) IcFileLog(id, str)
#else
#define IcLog(id, str)
#endif
#endif

/*--------------------------------------------------------------------
 * The following defines are for the different ASN.1 types...
 */

#define VcUndefinedD            -1
#define VcBOOLEAND               1
#define VcINTEGERD               2
#define VcBIT_STRINGD            3
#define VcOCTET_STRINGD          4
#define VcNULLD                  5
#define VcOBJECT_IDENTIFIERD     6
#define VcObjectDescriptorD      7
#define VcEXTERNALD              8
#define VcREALD                  9
#define VcENUMERATEDD           10
#define VcHEXSTRINGD            11
#define VcSEQUENCED             16
#define VcSEQUENCE_OFD          28
#define VcSETD                  17
#define VcSET_OFD               29
#define VcNumericStringD        18
#define VcPrintableStringD      19
#define VcTeletexStringD        20
#define VcVideotexStringD       21
#define VcIA5StringD            22
#define VcUTCTimeD              23
#define VcGeneralizedTimeD      24
#define VcGraphicStringD        25
#define VcVisibleStringD        26
#define VcGeneralStringD        27
#define VcR_TYPED               30
#define VcCHOICED               31
#define VcPDUD                  32
#define VcTAGGEDD               33
#define VcANYD                  34
#define VcISO646StringD         35
#define VcT61StringD            36
#define VcLocalTypeD            99	/* Any kind of local type. */
#define VcSTRINGD               98	/* Any kind of string. */
#define VcSTRUCTD               97	/* Any kind of struct. */

#define VcLT                      1
#define VcUT                      2

/*--------------------------------------------------------------------
 * Special values
 */

#ifndef TRUE
#define TRUE                     1
#endif

#ifndef FALSE
#define FALSE                    0
#endif

#ifndef NULL
#define NULL                     0
#endif

#ifndef VcINFINITY
#define VcINFINITY               INT_MAX
#endif
#ifndef VcMINUSINFINITY
#define VcMINUSINFINITY          INT_MIN
#endif

#ifndef VcMAX
#define VcMAX                    VcMkINTEGER(INT_MAX)
#endif
#ifndef VcMIN
#define VcMIN                    VcMkINTEGER(INT_MIN)
#endif

/*--------------------------------------------------------------------
 * The classes for the tags. And the implicit flag.
 */

#define VcNONE                  0
#define VcUNIVERSAL             1
#define VcAPPLICATION           2
#define VcPRIVATE               3
#define VcCONTEXT               4

#define VcIMPLICIT              1
#define VcEXPLICIT              0

/*--------------------------------------------------------------------
 * The attributes for the types.
 */

#define VcOPTIONALATTR             1

/*--------------------------------------------------------------------
 * The attributes for the values.
 */

#define VcEMPTYATTR                 0
#define VcIF_PRESENTATTR            1
#define VcOMITATTR                  2
#define VcANYONEATTR                4
#define VcANYORNONEATTR             8
#define VcVALUELISTATTR            16
#define VcCOMPLEMENTATTR           32
#define VcVALUERANGEATTR           64
#define VcSUBSETATTR              128
#define VcSUPERSETATTR            256
#define VcPERMUTATIONATTR         512

/*--------------------------------------------------------------------
 * The different restrictions...
 */

#define VcSIZERANGE             1
#define VcSIZE                  2
#define VcVALUERANGE            3
#define VcSINGLEVALUE           4
#define VcPERMITTEDALPHABET     5
#define VcCONTAINED             6

/*--------------------------------------------------------------------
 * The timer durations
 */

#define Vcmin                   60	/* Minutes       */
#define Vcs                      0	/* seconds       */
#define Vcms                    -3	/* mili seconds  */
#define Vcus                    -6	/* mVcro seconds */
#define Vcns                    -9	/* nano seconds  */
#define Vcps                    -12	/* pico seconds  */

/*--------------------------------------------------------------------
 * The different operands on the types.
 */

typedef char VcOperator;

/* Operators */

VcValue *VcNegate(VcValue *);
VcValue *VcAdd(VcValue *, VcValue *);
VcValue *VcSubtract(VcValue *, VcValue *);
VcValue *VcMultiply(VcValue *, VcValue *);
VcValue *VcDivide(VcValue *, VcValue *);
VcValue *VcModulo(VcValue *, VcValue *);
VcValue *VcEQ(VcValue *, VcValue *);
VcValue *VcNEQ(VcValue *, VcValue *);
VcValue *VcGT(VcValue *, VcValue *);
VcValue *VcLTN(VcValue *, VcValue *);
VcValue *VcGEQ(VcValue *, VcValue *);
VcValue *VcLEQ(VcValue *, VcValue *);
VcValue *VcNOT(VcValue *);
VcValue *VcOR(VcValue *, VcValue *);
VcValue *VcAND(VcValue *, VcValue *);

/* For backward compatibility. */
typedef struct VcRestriction VcResList;

/*--------------------------------------------------------------------
 * Some typedefs to make this implementation flexible in case we would
 * like to change the implementation of a given type...
 */

typedef int Bool;

typedef int VcTypeName;
typedef char VcTypeClass;
typedef int VcAttribute;	/* More than 8 attributes -> need int. */
typedef int VcTypeTag;
typedef long VcBaseVal;
typedef char *VcString;

typedef char *VcEnumerated;

typedef int VcPCO;		/* PCO descriptor */
typedef int VcTimer;		/* Timer descriptor. */
typedef int VcUnit;		/* How long is one (s, ms, us, ps, ns) */
typedef int VcTime;		/* An instance in time   */
typedef int VcStatus;		/* running man */
typedef int VcTimeDelta;	/* From here to eternity */

/*---------------------------------------------------- 
 * TTCN operations me must be able to perform on the values.
 * 
 */

/* Conversion operations */

VcValue *VcHEX_TO_INT(VcValue * HexValue);
VcValue *VcBIT_TO_INT(VcValue * BitValue);
VcValue *VcINT_TO_HEX(VcValue * IntValue, VcValue * Length);
VcValue *VcINT_TO_BIT(VcValue * IntValue, VcValue * Length);

/* Local function used by INT_TO_BIT */

void Vcrev(char *Wrong, char *Right);

/* Other predefined operations */

VcValue *VcIS_PRESENT(VcValue * Val);
VcValue *VcNUMBER_OF_ELEMENTS(VcValue * DataObjectReference);
VcValue *VcIS_CHOSEN(VcValue * DataObjectReference);
VcValue *VcLENGTH_OF(VcValue * DataObjectReference);

/*--------------------------------------------------------------------
 * Real Arithmetic 
 */

/* NOT IMPLEMENTED! */

/*--------------------------------------------------------------------
 * Functions to create a VcValue out of a C-type representation.
 */
#ifdef __cplusplus
extern "C" {
#endif

VcValue *VcMkLIST(void);
VcValue *VcMkUndefined(void);
VcValue *VcMkBOOLEAN(VcBaseVal Val);
VcValue *VcMkINTEGER(VcBaseVal Val);
VcValue *VcMkENUMERATED(VcBaseVal Val);
VcValue *VcMkBIT_STRING(VcString Val);
VcValue *VcMkHEXSTRING(VcString Val);
VcValue *VcMkOCTET_STRING(VcString Val);
VcValue *VcMkREAL(VcBaseVal mantissa, VcBaseVal base, VcBaseVal exponent);
VcValue *VcMkOBJECT_IDENTIFIER(VcValue* Val);
VcValue *VcMkObjectDescriptor(VcString Val);
VcValue *VcMkUTCTime(VcString Val);
VcValue *VcMkGeneralizedTime(VcString Val);
VcValue *VcMkCSTRING(VcString Val);
VcValue *VcMkR_TYPE(VcBaseVal Val);
VcValue *VcMkNULL(void);
VcValue *VcMkPDU(VcValue * Val);
VcValue *VcMkANY(VcValue * Val);
VcValue *VcMkSEQUENCE(int fields);
VcValue *VcMkCHOICE(VcValue * Val, VcString name);
VcValue *VcMkSEQUENCE_OF(VcType * Type);
VcValue *VcMkLIST(void);
VcValue *VcMkSET(int fields);
VcValue *VcMkSET_OF(VcType * Type);

VcValue *VcSetListType(VcType *, VcValue *);
VcType *VcGetListType(VcValue *);

/*--------------------------------------------------------------------
 * Functions to retrieve a C-Value from the internal representation.
 */

VcBaseVal VcGetBOOLEAN(VcValue * Val);
VcBaseVal VcGetR_TYPE(VcValue * Val);
VcBaseVal VcGetINTEGER(VcValue * Val);
VcString VcGetBIT_STRING(VcValue * Val);
VcString VcGetHEXSTRING(VcValue * Val);
VcString VcGetOCTET_STRING(VcValue * Val);
VcReal *VcGetREAL(VcValue * Val);
VcValue *VcGetOBJECT_IDENTIFIER(VcValue * Val);
VcString VcGetObjectDescriptor(VcValue * Val);
VcString VcGetUTCTime(VcValue * Val);
VcString VcGetGeneralizedTime(VcValue * Val);
VcString VcGetCSTRING(VcValue * Val);
VcValue *VcGetCHOICE(VcString name, VcType* type, VcValue * Val);
VcValue *VcGetANY(VcValue * Val);
VcValue *VcGetPDU(VcValue * Val);

VcBaseVal VcGetMantissa(VcReal *);
VcBaseVal VcGetBase(VcReal *);
VcBaseVal VcGetExponent(VcReal *);

void VcSetMantissa(VcReal *, VcBaseVal);
void VcSetBase(VcReal *, VcBaseVal);
void VcSetExponent(VcReal *, VcBaseVal);

/* OBJECT IDENTIFIER */
VcValue* VcAddOBJID_Component(VcValue* oid, int comp);
int VcGetOBJID_Size(VcValue* value);
int VcGetOBJID_Component(VcValue* oid, unsigned int index);

/* Getting the parts */

VcString VcGetCHOICEName(VcValue * Val);

/*--------------------------------------------------------------------
 * Functions to manipulate type and attributes in the internal
 * representation.
 */

VcValue *VcSetType(VcType *, VcValue *);
VcType *VcGetType(VcValue *);
VcValue *VcSetValueAttribute(VcAttribute, VcValue *);
Bool VcGetValueAttribute(VcAttribute, VcValue *);
VcType *VcStr2Type(const char *);
VcType *VcTag2Type(int);
const char *VcTag2Str(int);

/*--------------------------------------------------------------------
 * Compound functions to save code.
 */

VcValue *VcMkOmit(VcType * type);
VcValue *VcMkAnyOne(VcType * type);
VcValue *VcMkAnyOrNone(VcType * type);

/*--------------------------------------------------------------------
 * Functions to manipulate complex Values in the internal
 * representation.
 */

VcValue *VcAddElem(VcValue * Item, VcValue * List);	/* Add elem first */
VcValue *VcAddElemZ(VcValue * Item, VcValue * List);	/* Add elem last */
VcValue *VcGetElem(int Index, VcValue * List);
VcValue *VcSetField(VcValue * Val, int FieldNo, VcValue * Seq);
VcValue *VcGetField(int Index, VcValue * Seq);
int VcSeqSize(VcValue * Seq);
int VcSeqOfSize(VcValue * SeqOf);

/*--------------------------------------------------------------------
 * Type
 */

VcType *VcMkType(VcTypeName Name);
VcType *VcReType(VcType * Type, VcTypeName Name);

VcType *VcTagType(VcTypeClass Class, VcTypeTag Tag, Bool Implicit,
		  VcType * Type);
VcTypeName VcGetName(VcType * Type);
VcTypeTag VcGetTag(VcType * Type);
VcTypeClass VcGetClass(VcType * Type);
Bool VcGetImplicit(VcType * Type);

VcType *VcSetDefaultValue(VcValue** Val, VcType * Type);

VcType *VcSetTypeAttribute(VcAttribute, VcType * Type);
Bool VcGetTypeAttribute(VcAttribute, VcType * Type);

VcType *VcSetParent(VcType *, VcType *);
VcType *VcGetParent(VcType *);
int VcGetLength(VcType * Type);
VcType *VcAddLocal(char *Name, VcType * LocalType, VcType * Parent);
VcType *VcRegisterLocal(VcType * HolderType, VcType * Parent);
VcType *VcGetLocal(char *Name, VcType * Parent);
VcType *VcAddRes(VcRestriction *, VcType *);

void VcEnumMap(char *data, VcType * Type);
char *VcEnumGet(char *name, VcType * Type);

/*--------------------------------------------------------------------
 * Restrictions
 */

typedef unsigned int VcRestrictionType;

/*--------------------------------------------------------------------
 * The functions for adding restrictions to a type.
 */

VcRestriction *VcMkRes(VcRestrictionType);
VcRestriction *VcResAddElem(VcValue **, VcRestriction *);
VcRestriction *VcAddContainedType(VcType *, VcRestriction *);

/* Value dependency backpatching */

Bool VcValueDepForBackPatch(VcValue** value_location, VcValue** resval);
Bool VcBackPatchValueDep();
#ifdef __cplusplus
}
#endif
/*--------------------------------------------------------------------
 * Function for creating all the base types in the type pool.
 */

void VcBaseTypes(void);

extern VcType *VcUndefined;
extern VcType *VcINTEGER;
extern VcType *VcBOOLEAN;
extern VcType *VcBIT_STRING;
extern VcType *VcSEQUENCE;
extern VcType *VcREAL;
extern VcType *VcCHOICE;
extern VcType *VcR_TYPE;
extern VcType *VcSEQUENCE_OF;
extern VcType *VcOCTET_STRING;
extern VcType *VcPDU;
extern VcType *VcHEXSTRING;
extern VcType *VcSET;
extern VcType *VcSET_OF;
extern VcType *VcNULL;
extern VcType *VcIA5String;
extern VcType *VcENUMERATED;
extern VcType *VcTAGGED;
extern VcType *VcOBJECT_IDENTIFIER;
extern VcType *VcANY;
extern VcType *VcObjectDescriptor;
extern VcType *VcGeneralizedTime;
extern VcType *VcUTCTime;
extern VcType *VcNumericString;
extern VcType *VcPrintableString;
extern VcType *VcTeletexString;
extern VcType *VcVideotexString;
extern VcType *VcT61String;
extern VcType *VcISO646String;
extern VcType *VcGraphicString;
extern VcType *VcVisibleString;
extern VcType *VcGeneralString;

/*--------------------------------------------------------------------
 * Memory handling.
 */

void *VcMalloc(size_t, char *);
void VcFree(void *, size_t, char *);

#ifdef DEBUG_MEMORY
#define VCMALLOC(x, y) VcMalloc((x), (y))
#define VCFREE(x, y, z) VcFree((x), (y), (z))
#else
#define VCMALLOC(x, y) VcMalloc((x), "")
#define VCFREE(x, y, z) VcFree((x), (y), "")
#endif

/* Backward compatibility. */
#define MALLOC VCMALLOC
#define FREE   VCFREE

extern void VcMkFrame(void);
extern void VcFreeFrame(void);

extern void VcSetPermAlloc(void);
extern void VcSetTempAlloc(void);

typedef int VcMallocType;

extern int VcGetAlloc(void);
extern void VcSetMalloc(VcMallocType);
extern void VcInitMem(void);
extern void VcMemPrint(void);

/* Internal concurrent TTCN - memory context creation/switching */

typedef struct VcContext VcContext;

extern void VcCreateDefaultContext(void);
extern void VcResetTCVars(void);
extern void VcContextCreateMTC(int, const char *);
extern void VcContextCreatePTC(int, const char *);
extern void VcContextDelete(void);

extern void VcContextSwitch(VcContext *);

extern int VcContextGetComponent(void);
extern void VcContextSetID(int);
extern const char *VcContextGetComponentName(void);
extern VcTimerType* VcContextGetTimers(void);
extern void VcContextSetTimers(int, VcTimerType*);

extern int VcContextGetVerdict(void);
extern void VcContextSetVerdict(int);

extern void VcMemoryReport(const char *, void *);

extern VcContext *context;


#define VcSTATIC 1
#define VcTMP    2

/*--------------------------------------------------------------------
 *  Restrictions for the base types.
 */

Bool VcIsHex(VcString String);
Bool VcIsOct(VcString String);
Bool VcIsBit(VcString String);
Bool VcIsNum(VcString String);
Bool VcIsNumStr(VcString String);
Bool VcIsTime(VcString String);

/*--------------------------------------------------------------------
 * The matching function.
 * The reference object is the only one that may contain wildcards.
 */

Bool VcMatch(VcValue * Ref, VcValue * Val);	/* Match two values */

extern Bool vcRelaxedTagMatching; /* Defaults to FALSE, i.e. strict matching */

/*--------------------------------------------------------------------
 *  Handling Values
 */

void VcAssign(VcValue * Obj1, VcValue * Obj2, int Mem, char *name);
void VcRmValue(VcValue * Val);
VcValue *VcCopy(VcValue * Val);

/*--------------------------------------------------------------------
 * Restrictions for the basic complex types.
 */

VcValue *VcSUBSET(VcValue * DataObject1, VcValue * DataObject2);
VcValue *VcSUPERSET(VcValue * DataObject1, VcValue * DataObject2);
VcValue *VcPERMUTATION(VcValue * DataObject1, VcValue * DataObject2);

/*--------------------------------------------------------------------
 * The PCO queue.
 */

extern int GcFirstPCOD;
extern int GcFirstTimerD;

struct VcPCOQueueObject 
{
  void *sender;
  VcValue *First;
  struct VcPCOQueueObject *Next;
};

struct VcPCOQueue
{
  char *Name;
  char *Type;
  int Role;
  struct VcPCOQueueObject *Queue; /* In queue */
  VcValue *Out;			/* Out queue */
};

#define VcDEFAULT_PCO 0

void VcInitPCO(VcPCO pco, char *name, char *role);	/* Set up one PCO */

/*--------------------------------------------------------------------
 * The timer structure.
 */

struct VcTimerType
  {
    char *Name;			/* The name of the timer */
    VcValue *Duration;
    VcUnit Unit;		/* The unit of the duration */
    VcTime StartTime;
    VcStatus Status;
  };

struct VcPTCType
  {
    VcStatus Status;
  };

#define VcDEFAULTTIMER 0

#define VcSTATUSSTOPPED 0
#define VcSTATUSRUNNING 1
#define VcSTATUSTO      2
#define VcSTATUSCREATED 3
#define VcSTATUSDONE    4

Bool VcBefore(VcTime Start, VcValue * Duration, VcUnit Unit, VcTime End);
void VcSetTime(VcTime Time);
VcTime Now(void);

VcTypeName VcGetBaseType(VcType * type);
VcTypeClass VcGetBaseClass(VcType * type);
VcTypeTag VcGetBaseTag(VcType * type);

/*--------------------------------------------------------------------
 * 
 */

extern void VcPush(void);
extern void VcAddMember(VcType *type, const char *name);
extern void VcRegisterMembers(VcType *);
extern void VcSequenceOfType(VcType *sequence, VcType *type);
extern VcType* VcGetSequenceOfType(VcType *type);
/*--------------------------------------------------------------------
 * 
 */

unsigned VcGetMemberCount(VcTypeName type);
VcTypeName VcGetMemberType(VcTypeName type, unsigned index);
VcType* VcGetMemberVcType(VcType* vc_type, unsigned index);
const char *VcGetMemberName(VcTypeName type, unsigned index);

extern int MTCD;

#endif
