/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/


/*********************************************************************
 *                                                                    *
 *    Name:  GatewayClient_impl.cpp                                   *
 *                                                                    *
 *    Author: Mang Li                                                 *
 *            GMD Fokus, TIP                                          *
 *                                                                    *
 *    Description: This file contains the implementation of the       *
 *        class TOR_GatewayClient_impl.                               *
 *                                                                    *
 *                                                                    *
 *********************************************************************/

#include "orb_gw.h"

#include "GatewayClient_impl.h"

extern int VERBOSITY;
extern void verbose_print(char* fmt);
#define VERBOSE(F)	verbose_print F

//-------------
// Constructor |
//-------------

TOR_GatewayClient_impl::TOR_GatewayClient_impl(const char* objn, 
                                               const CORBA::ORB_ptr orb, 
                                               const CORBA::Object_ptr object)
  : orb_(CORBA::ORB::_duplicate(orb)),
    obj_(CORBA::Object::_duplicate(object)),
    objName_(CORBA::string_dup(objn)),
    interfDef_(CORBA::InterfaceDef::_nil()),
    inMsgQueue_(new TOR::GWMsgQueue),
    requestList_(new RequestList)
{
  VERBOSE(("GatewayClient_impl: in Constructor"));
  VERBOSE(("GatewayClient_impl: The object name is >>"));
  VERBOSE((objName_));

  CORBA::Object_var obj = orb_ -> 
    resolve_initial_references("InterfaceRepository");
  assert(!CORBA::is_nil(obj));
  								       
  CORBA::Repository_var repository = CORBA::Repository::_narrow(obj);
  assert(!CORBA::is_nil(repository));
  
  CORBA::String_var id = CORBA::string_dup(getIDLType(objn));
  CORBA::Contained_var contained = repository -> lookup_id(id);
  assert(!CORBA::is_nil(contained));
  interfDef_ = CORBA::InterfaceDef::_narrow(contained);
  assert(!CORBA::is_nil(interfDef_));
}


//-------------
// Destructor  |
//-------------

TOR_GatewayClient_impl::~TOR_GatewayClient_impl ()
{
  VERBOSE(("GatewayClient_impl: in Destructor."));
    
  // Remove all elments in the incoming message queue
  inMsgQueue_->length(0);

  // Remove all elements in the request queue
  requestList_->length(0);    

  if(!CORBA::is_nil(orb_))
    orb_->destroy();
}


//--------------
// getIDLType   |
//--------------

char* TOR_GatewayClient_impl::getIDLType(const char* objName)
{
  CORBA::String_var tmp = CORBA::string_dup(objName);
  char* buf = new char[strlen(tmp)+1];
  CORBA::String_var idlType;
  
  // Replace '::' by '/'
  CORBA::ULong i = 0;
  CORBA::ULong j = 0;
  for (i = 0; tmp[i] != '\0'; i++)
    {
      if ((tmp[i] == ':') && tmp[i+1] == ':')
        {
          buf[j] = '/';
          i++;
          j++;
        }
      else
        {
          buf[j] = tmp[i];
          j++;
        }
    }
  char* body = new char[j+1];
  strncpy(body, buf, j);
  body[j] = '\0';
  delete[] buf;
  
  // Add prefix and suffix
  idlType += CORBA::string_dup(IDLTYPE_PREFIX); 
  idlType += CORBA::string_dup(body);
  idlType += CORBA::string_dup(IDLTYPE_SUFIX);
  delete[] body;
  
  char* result = new char[strlen(idlType)+1];
  strcpy(result,idlType);
  return result;
}


//-------------------
// findOperationDef  |
//-------------------

CORBA::OperationDef_ptr TOR_GatewayClient_impl::findOperationDef(const char* opname)
{
  // First, search in the current interface
  CORBA::InterfaceDef::FullInterfaceDescription_var current =
    interfDef_ -> describe_interface();
  for(CORBA::ULong i = 0 ; i < current->operations.length() ; i++)
    {
      if (strcmp(current->operations[i].name, opname) == 0)
        {
          CORBA::Contained_var contained = interfDef_->lookup(opname);
          assert(!CORBA::is_nil(contained));
          CORBA::OperationDef_var opDef = 
            CORBA::OperationDef::_narrow(contained);
          assert(!CORBA::is_nil(opDef));
          return  (CORBA::OperationDef::_duplicate(opDef));
        }
    }
    
  // Search in the base interfaces
  CORBA::InterfaceDefSeq_var seq = interfDef_->base_interfaces();
  for(CORBA::ULong m = 0; m < seq->length(); m++)
    {
      CORBA::InterfaceDef::FullInterfaceDescription_var base =
        seq[m] -> describe_interface();

      for(CORBA::ULong n = 0 ; n < base->operations.length() ; n++)
        {
          if (strcmp(base->operations[n].name, opname) == 0)
            {
              CORBA::Contained_var contained = seq[m]->lookup(opname);
              assert(!CORBA::is_nil(contained));
              CORBA::OperationDef_var opDef = CORBA::OperationDef::_narrow(contained);
              assert(!CORBA::is_nil(opDef));
              
              if(VERBOSITY > 1)
                {
                  CORBA::Contained::Description_var description = 
                    opDef -> describe();
                  const CORBA::OperationDescription* odes;
                  description -> value >>= odes;
                }
              
              return  (CORBA::OperationDef::_duplicate(opDef));
            }
        }
    }
  return  (CORBA::OperationDef::_nil());
}


//---------------
// sendOutMsg    |
//---------------

TOR::GWStatus TOR_GatewayClient_impl::sendOutMsg(const TOR::GWMessage& imsg)
  throw(CORBA::SystemException)
{
  TOR::GWMessage* msg = new TOR::GWMessage(imsg);
  
  try
    {
      assert (strcmp (objName_, msg->objectName) == 0);
      VERBOSE(("GatewayClient_impl: In the method sendOutMsg->) >>"));
      VERBOSE(("for the object >>"));
      VERBOSE((objName_ ));
      
      // Create Request using Interface Repository
      // cout << "GatewayClient_impl: Create operation list using Interface "
      //   << "Repository ." << endl;
      CORBA::OperationDef_var opDef = findOperationDef(msg->operationName);
      assert(!CORBA::is_nil(opDef));

      // List of arguments
      CORBA::NVList_ptr argList;  
      orb_ -> create_operation_list(opDef, argList);
      assert(!CORBA::is_nil(argList));
  
      // Type code of return value
      CORBA::TypeCode_ptr rvType = opDef -> result();
      assert(!CORBA::is_nil(rvType));
  
      TOR::GWQElem* inQElem = new TOR::GWQElem;
      TOR::GWMessage* inMsg = new TOR::GWMessage;
  
      inQElem->flag = TOR::MsgNotReady;
      
      inMsg->objectName = msg->objectName;
      inMsg->operationName = msg->operationName;
      inMsg->cntType = TOR::IsVoid;

      if (CORBA::is_nil(argList))  
        {
          // No arguments
          if (rvType->kind() != CORBA::tk_null && rvType->kind() != CORBA::tk_void)
            {
              inMsg->cntType = TOR::ReturnValue;
              VERBOSE(("GatewayClient_impl: The operation >>"));
              VERBOSE((inMsg->operationName));
              VERBOSE(("expects no parameters but return value."));
            }
          else
            {
              inMsg->cntType = TOR::IsVoid;
              VERBOSE(("GatewayClient_impl: The operation >>'"));
              VERBOSE((inMsg->operationName));
              VERBOSE(("expects no parameters and no return value."));
            }
        }
      else
        {
          // With arguments
          if (rvType->kind() != CORBA::tk_null && rvType->kind() != CORBA::tk_void)
            {
              inMsg->cntType = TOR::ReturnAndParameters;
              VERBOSE(("GatewayClient_impl: The operation >>'"));
              VERBOSE((inMsg->operationName));
              VERBOSE(("expects parameters and return value."));
            }
          else
            {
              inMsg->cntType  = TOR::Parameters;
              VERBOSE(("GatewayClient_impl: The operation >>'"));
              VERBOSE((inMsg->operationName));
              VERBOSE(("expects parameters but no return value."));
            }
          
          CORBA::ULong m = 0;
          // Set values for in/inout parameters
          for (CORBA::ULong i = 0; i < argList -> count(); i++)
            {
              if(argList->item(i)->flags() == (CORBA::ARG_IN || CORBA::ARG_INOUT))
                {
                  // Check type code
                  // This may be replace by an exception
                  // Because the Gateway shall only pass the messages, 
                  // the correctness of the message content 
                  // shall be controlled by
                  // the TTCN run-time, the ORB and the SUT server
                  // The same for message.cntType checking!!!

                  // The type code checking is turned off
                  // CORBA::TypeCode_ptr tc1 = argList->item(i)->value()->type();
                  // CORBA::TypeCode_var tc2 = msg.content.paramList[m].type();
                  // assert (tc1->kind()  == tc2->kind());
                  
                  // Update the value
                  *(argList->item(i)->value()) = msg->content.paramList[m];
                  m++;
                }
            }
        }
      
            
      // Create an exception list
      CORBA::ExceptionList_ptr exList;
      orb_->create_exception_list(exList);
      
      CORBA::ContextList_ptr cxtList;
      orb_->create_context_list(cxtList);
      
      CORBA::ExceptionDefSeq_var excSeq = opDef -> exceptions();
      for(CORBA::ULong e=0; e < excSeq->length(); e++)
        {
          CORBA::TypeCode_var tc = 
            CORBA::TypeCode::_duplicate(excSeq[e]->type());
          exList->add(tc);
        }
  
  
      // Get a pointer to the Request object      
      CORBA::NamedValue_ptr result;
      orb_->create_named_value(result);
      
      CORBA::Context_ptr ctx;
      orb_ -> get_default_context(ctx);
      
      CORBA::Request_ptr request;
       
      //obj_->_create_request (ctx, msg.operationName, argList, result, exList, cxtList, request, 0);

      obj_->_create_request (ctx, msg->operationName, argList, result, request, 0);

      request->set_return_type(rvType);
      request->send_deferred();

      // Save the pointer to the Request object
      requestList_->length(requestList_->length()+1);
      requestList_[requestList_->length()-1] = CORBA::Request::_duplicate(request);
      
      // Append the element to the message queue
      inQElem->message = *inMsg;

      inMsgQueue_->length(inMsgQueue_->length()+1);
      inMsgQueue_[inMsgQueue_->length()-1] = *inQElem;      
      
      return TOR::GWOK;
    }
   catch(CORBA::SystemException& ex)
    {
      //cerr << &ex << endl;
      cout << "GatewayClient_impl: sendOutMsg() failed "
           <<"due to CORBA system exception." << endl;
      return TOR::GWNotOK;
    }
  catch(...)
    {
      cout << "GatewayClient_impl: sendOutMsg() failed "
           <<"due to unknown failure." << endl;
      return TOR::GWNotOK;
    }
}

 
//------------
// getInMsg   |
//------------

TOR::GWMsgFlag TOR_GatewayClient_impl::getInMsg(TOR::GWMessage_out msg)
  throw(CORBA::SystemException)
{
  //  cout << "GatewayClient_impl: In the method getInMsg()." << endl;
  
  try
    {
      for (CORBA::ULong i=0; i< inMsgQueue_->length(); i++)
        {
          if(inMsgQueue_[i].flag == TOR::MsgNotReady)
            {
              if(requestList_[i]->poll_response())
                {
                  if(requestList_[i]->env()->exception() == NULL)
                    {
                      VERBOSE(("GatewayClient_impl: No exceptions are returned by the operation >>"));
                      VERBOSE((inMsgQueue_[i].message.operationName));
                      
                      switch (inMsgQueue_[i].message.cntType)
                        {
                        case (TOR::Parameters):
                          {
                            // Fill the place of return value with NULL,
                            // in order to mark the message as a reply message.
                            inMsgQueue_[i].message.cntType = TOR::ReturnAndParameters;
                            CORBA::Any* rv = new CORBA::Any;
                            rv->replace(CORBA::_tc_void, 0);
                            
                            // Save all inout/out parameters
                            CORBA::NVList_ptr list = requestList_[i]->arguments();
                            assert(!CORBA::is_nil(list));
                                                        
                            TOR::GWParamList_var pl = new TOR::GWParamList;
                            for (CORBA::ULong j=0; j < list->count(); j++)
                              {
                                if(list->item(j)->flags() != CORBA::ARG_IN)
                                  {
                                    CORBA::Any* any = new CORBA::Any;
                                    *any = *(list->item(j)->value());
                                    pl->length(pl->length()+1);
                                    pl[pl->length()-1] = *any;
                                  }
                              }
                            
                            TOR::GWMsgContent* ct = new TOR::GWMsgContent;
                            ct->paramList = *pl;
                            ct->returnValue = *rv;
                            inMsgQueue_[i].message.content = *ct;
                            
                            VERBOSE(("GatewayClient_impl: Values of the parameters for the operation >>"));
                            VERBOSE((inMsgQueue_[i].message.operationName));
                            VERBOSE(("are updated"));
                            
                            break;
                          }
                        case (TOR::ReturnValue):
                          {
                            CORBA::NamedValue_ptr rv = requestList_[i]->result();
                            assert(!CORBA::is_nil(rv));
                            
                            // Save return value
                            CORBA::Any* any = new CORBA::Any;
                            *any = *(rv->value());
                            
                            TOR::GWMsgContent* ct = new TOR::GWMsgContent;
                            ct->returnValue = *any;
                            inMsgQueue_[i].message.content = *ct;
                            VERBOSE(("GatewayClient_impl: Values of the return value for the operation >>"));
                            VERBOSE((inMsgQueue_[i].message.operationName));
                            VERBOSE(("is updated"));

                            break;
                          }
                        case (TOR::ReturnAndParameters):
                          {
                            CORBA::NVList_ptr list = requestList_[i]->arguments();
                            assert(!CORBA::is_nil(list));
                            
                            // Save all inout/out parameters
                            TOR::GWParamList_var pl = new TOR::GWParamList;
                            for (CORBA::ULong j=0; j < list->count(); j++)
                              {
                                if(list->item(j)->flags() != CORBA::ARG_IN)
                                  {
                                    CORBA::Any* any = new CORBA::Any;
                                    *any = *(list->item(j)->value());
                                    pl->length(pl->length()+1);
                                    pl[pl->length()-1] = *any;
                                  }
                              }
                        
                            TOR::GWMsgContent* ct = new TOR::GWMsgContent;
                            ct->paramList = *pl;
                            
                            CORBA::NamedValue_ptr rv = requestList_[i]->result();
                            assert(!CORBA::is_nil(rv));
                            
                            // Save return value
                            CORBA::Any* any = new CORBA::Any;
                            *any = *(rv->value());
                            ct->returnValue = *any;
                            
                            inMsgQueue_[i].message.content = *ct;

                            VERBOSE(("GatewayClient_impl: Values of the parameters and the return value for the operation >>"));
                            VERBOSE((inMsgQueue_[i].message.operationName));
                            VERBOSE(("are updated"));

                            break;
                          }
                        case (TOR::IsVoid):
                          {
                            // Fill the place of return value with NULL,
                            // in order to mark the message as a reply message.
                            inMsgQueue_[i].message.cntType = TOR::ReturnValue;
                            CORBA::Any* rv = new CORBA::Any;
                            rv->replace(CORBA::_tc_void, 0);

                            TOR::GWMsgContent* ct = new TOR::GWMsgContent;
                            ct->returnValue = *rv;
                            
                            inMsgQueue_[i].message.content = *ct;
                            break;
                          }
                        default:
                          {
                            // Shall not come to here.
                            cout<<"GatewayClient_impl: Error message content "
                                <<"type." << endl;
                            break;
                          }
                        } // End of switch
                    }
                  else
                    {
                      // Exceptions
                      inMsgQueue_[i].message.cntType = TOR::IsException;
                      CORBA::Exception* ex = requestList_[i]->env()->exception();
                      CORBA::UnknownUserException* uuex = CORBA::UnknownUserException::_downcast(ex);
                      if (uuex != NULL)
                        {
                          cout << "GatewayClient_impl: "
                               <<"Unknown user exception." << endl;
                          CORBA::Any* any = new CORBA::Any;
                          *any = uuex->exception();
                          TOR::GWMsgContent* ct = new TOR::GWMsgContent;
                          ct->except = *any;
                          inMsgQueue_[i].message.content = *ct;
                        }
                      else 
                        {
                          CORBA::SystemException* sx = CORBA::SystemException::_downcast(ex);
                          if (sx != NULL)
                            {
                              //CORBA system exception
                              cout << "GatewayClient_impl: "
                                   <<"CORBA system exception." << endl;
                              whichSysExcept(sx);
                              
                              CORBA::Any* any = new CORBA::Any;
                              *any <<= new CORBA::COMM_FAILURE(0, CORBA::COMPLETED_MAYBE);
                              TOR::GWMsgContent* ct = new TOR::GWMsgContent;
                              ct->except = *any;
                              inMsgQueue_[i].message.content = *ct;
                            }
                          else
                            {
                              //Unknown exception
                              cout << "GatewayClient_impl: "
                                   <<"Unknown exception." << endl;
                              CORBA::Any* any = new CORBA::Any;
                              *any <<= new CORBA::UNKNOWN(0, CORBA::COMPLETED_MAYBE);
                              TOR::GWMsgContent* ct = new TOR::GWMsgContent;
                              ct->except = *any;
                              inMsgQueue_[i].message.content = *ct;
                            }
                        }
                    }
                  inMsgQueue_[i].flag = TOR::MsgReady;
                }
              else
                {
                  // VERBOSE(("GatewayClient_impl: Response not ready."));
                }
            }
        }
      
      // Pass the first completed message in the incoming message queue 
      // as argument back
      
      for (CORBA::ULong k=0; k< inMsgQueue_->length(); k++)
        {
          if (inMsgQueue_[k].flag == TOR::MsgReady)
            {
              VERBOSE(("GatewayClient_impl: Return the first element in the incoming message queue."));
              
              msg = new TOR::GWMessage(inMsgQueue_[k].message);
              
	      //***todo, first try
              // Omnibroker extension
              // inMsgQueue_->remove(k);
              // requestList_->remove(k);
              TOR::GWMsgQueue::freebuf(&inMsgQueue_[k]);
	      CORBA::release(requestList_[k]); 

              return TOR::MsgReady;
            }
        }
      // VERBOSE(("GatewayClient_impl: No message is ready for return."));

      // A dump message
      TOR::GWMessage_var dump = new TOR::GWMessage;
      dump->objectName = CORBA::string_dup("");
      dump->operationName = CORBA::string_dup("");
      dump->cntType = TOR::IsVoid;
      
      msg = new TOR::GWMessage(dump);
      return TOR::MsgNotReady;
    }
  catch(CORBA::SystemException& ex)
    {
      cerr << &ex << endl;
      cout << "GatewayClient_impl: getInMsg() failed due to CORBA "
           << "system exception." << endl;

      // A dump message
      TOR::GWMessage_var dump = new TOR::GWMessage;
      dump->objectName = CORBA::string_dup("");
      dump->operationName = CORBA::string_dup("");
      dump->cntType = TOR::IsVoid;

      msg = new TOR::GWMessage(dump);
      return TOR::MsgNotReady;
    }
  catch(...)
    {
      cout << "GatewayClient_impl: getInMsg() failed due to unknown failure." 
           << endl;

      // A dump message
      TOR::GWMessage_var dump = new TOR::GWMessage;
      dump->objectName = CORBA::string_dup("");
      dump->operationName = CORBA::string_dup("");
      dump->cntType = TOR::IsVoid;

      msg = new TOR::GWMessage(dump);
      return TOR::MsgNotReady;
    }
}


void whichSysExcept(CORBA::SystemException* sx)
{
  if(CORBA::BAD_PARAM::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: bad parm."<<endl;

  else if(CORBA::INV_OBJREF::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: invalid objref."<<endl;

  else if(CORBA::MARSHAL::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: marshal."<<endl;

  else if(CORBA::BAD_TYPECODE::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: bad typecode."<<endl;
  
  else if(CORBA::BAD_OPERATION::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: bad operation."<<endl;
  
  else if(CORBA::NO_RESPONSE::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: no response."<<endl;
  
  else if(CORBA::NO_IMPLEMENT::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: no implement."<<endl;
  
  else if(CORBA::COMM_FAILURE::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: comm failure."<<endl;

  else if(CORBA::TRANSIENT::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: transient."<<endl;
  
  else if(CORBA::INV_IDENT::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: inv ident."<<endl;
  
  else if(CORBA::INV_FLAG::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: inv flag."<<endl;
  
  else if(CORBA::INTF_REPOS::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: intf repository."<<endl;
  
  else if(CORBA::BAD_CONTEXT::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: bad context."<<endl;
  
  else if(CORBA::OBJ_ADAPTER::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: obj adapter."<<endl;
  
  else if(CORBA::DATA_CONVERSION::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: data conversion."<<endl;
  
  else if(CORBA::OBJECT_NOT_EXIST::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: object not exist."<<endl;
  
  else if(CORBA::INVALID_TRANSACTION::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: invalid transaction."<<endl;
  
  else if (CORBA::UNKNOWN::_downcast(sx) !=NULL)
    cout<<"whichSysExcept: unknown."<<endl;

  else
    cout<<"whichSysExcept: not found on the list."<<endl;
  
  return;
  
 }


  

