/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/

/*********************************************************************
 *                                                                    *
 *    Name:  GatewayClient_impl.h                                     *
 *                                                                    *
 *    Author: Mang Li                                                 *
 *            GMD Fokus, TIP                                          *
 *                                                                    *
 *    Description: This header file contains the declaration of the   *
 *        class TOR_GatewayClient_impl.                               *
 *                                                                    *
 *********************************************************************/


#include <Gateway_skel.h>

#define IDLTYPE_PREFIX "IDL:"
#define IDLTYPE_SUFIX ":1.0"

typedef CORBA::ORB::RequestSeq RequestList;
typedef CORBA::ORB::RequestSeq_var RequestList_var;

class TOR_GatewayClient_impl : public POA_TOR::GatewayClient
{
public:

  TOR_GatewayClient_impl(const char*, const CORBA::ORB_ptr, 
                         const CORBA::Object_ptr);
  ~TOR_GatewayClient_impl();

protected:
  friend class TOR_GatewayMain_impl;
  
private:

  // The ORB
  CORBA::ORB_var orb_;

  // The requested object
  CORBA::Object_var obj_;
  CORBA::String_var objName_;
  
  // Interface repository
  CORBA::InterfaceDef_var interfDef_;

  // Message queue for incoming replies or exceptions from ORB
  TOR::GWMsgQueue_var inMsgQueue_;

  // List to store references of CORBA Request objects
  RequestList_var requestList_;

private:

  // Help function to construct scoped names for the use of
  // interface repositories.
  char* getIDLType(const char*);

  // Help function to find the OperationDef for an operation.
  // Since inherited operations can not always be found in the
  // current InterfaceDef, base interfaces must be searched.
  CORBA::OperationDef_ptr findOperationDef(const char* opname);
    
public:

  // Process an outgoing request
  virtual TOR::GWStatus sendOutMsg(const TOR::GWMessage&)
    throw(CORBA::SystemException); 

  // Get an incoming reply or exception
  virtual TOR::GWMsgFlag getInMsg(TOR::GWMessage_out msg)
    throw(CORBA::SystemException); 
}; 


void whichSysExcept(CORBA::SystemException* sx);
  
  
  
  
