/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/


/*********************************************************************
 *                                                                    *
 *    Name:  GatewayClient_impl.cpp                                   *
 *                                                                    *
 *    Author: Mang Li                                                 *
 *            GMD Fokus, TIP                                          *
 *                                                                    *
 *    Description: This file contains the implementation of the       *
 *        class TOR_GatewayClient_impl.                               *
 *                                                                    *
 *********************************************************************/



#include <orb_gw.h>

#include "GatewayMain_impl.h"
#include "GatewayClient_impl.h"
#include "GatewayServer_impl.h"

#include <fstream.h>

extern int VERBOSITY;
extern void verbose_print(char* fmt);
#define VERBOSE(F)	verbose_print F

extern void whichSysExcept(CORBA::SystemException* sx);


//-------------
// Constructor |
//-------------

TOR_GatewayMain_impl::TOR_GatewayMain_impl(const CORBA::ORB_ptr orb, PortableServer::POA_ptr poa, const PortableServer::POAManager_ptr manager, OBVTest::TestValueSaver_ptr saver)
  : orb_(CORBA::ORB::_duplicate(orb)),
    poa_(PortableServer::POA::_duplicate(poa)),
    poamgr_(PortableServer::POAManager::_duplicate(manager)),
    PCOList_(new TOR::GWConnections),
    cmQueue_(new TOR::GWMsgQueue),
    saver_(OBVTest::TestValueSaver::_duplicate(saver))
        
{
  VERBOSE(("GatewayMain_impl: in the constructor."));

}


//------------
// Destructor |
//------------

TOR_GatewayMain_impl::~TOR_GatewayMain_impl()
{
  VERBOSE(("GatewayMain_impl: in the destructor."));
  for (CORBA::ULong i = 0; i <  PCOList_->length(); i++)
    CORBA::release(PCOList_[i].objInfo.objRef);

  PCOList_->length(0);
  cmQueue_->length(0);   
  CORBA::release(orb_);
}


//----------
// GWInit   |
//----------

TOR::GWStatus TOR_GatewayMain_impl::GWInit()
  throw(CORBA::SystemException)
{
  for (CORBA::ULong i = 0; i <  PCOList_->length(); i++)
    CORBA::release(PCOList_[i].objInfo.objRef);

  PCOList_->length(0);
  cmQueue_->length(0);
  saver_ -> register_all_factories();
  return TOR::GWOK;
}


//-------------
// GWDestroy   |
//-------------

void TOR_GatewayMain_impl::GWDestroy()
  throw(CORBA::SystemException)
{
  for (CORBA::ULong i = 0; i <  PCOList_->length(); i++)
    CORBA::release(PCOList_[i].objInfo.objRef);

  PCOList_->length(0);
  cmQueue_->length(0);

}


//----------------
// here_is_a_CM   |
//----------------

void TOR_GatewayMain_impl::here_is_a_CM(const TOR::GWMessage& msg)
  throw(CORBA::SystemException)
{
  TOR::GWQElem* inQElem = new TOR::GWQElem;
  inQElem->flag = TOR::MsgReady;
  TOR::GWMessage* inMsg = new TOR::GWMessage(msg);
  inQElem->message = *inMsg;
}


//---------------
// GWConfigure   |
//---------------

TOR::GWStatus TOR_GatewayMain_impl::GWConfigure(CORBA::Boolean concurrent,
                                     const TOR::GWComponentConfig& config)
  throw(CORBA::SystemException)
{
  try
    {
      componentID_ = config.componentID;
      
      // Create PCO list
      // cout << "GatewayMain_impl: Creating PCO list..." << endl;
      TOR::GWConnInfo* aPCO;
      if (!concurrent)
        {
          // Plain TTCN
          for (CORBA::ULong i = 0; i <  config.connections.length(); i++)
            { 
              aPCO = new TOR::GWConnInfo(config.connections[i]);   

              switch (aPCO->connType)
                {
                case(TOR::PCOtoSUTServer):
                  {
                    // Connection to an SUT server object
                    TOR_GatewayClient_impl* clt = new TOR_GatewayClient_impl(aPCO->objInfo.objName, orb_, aPCO->objInfo.objRef);

		    TOR::GatewayClient_var cltvar = clt->_this();
                    assert (!CORBA::is_nil(cltvar));
                    aPCO->objInfo.objRef = CORBA::Object::_duplicate(cltvar);
                    if(VERBOSITY > 0)
                      cout<< "GatewayMain_impl: a GatewayClient is created for"
                          <<" the SUT object '" << aPCO->objInfo.objName
                          << "' which is reachable via the PCO '" 
                          << aPCO->pName << "'" << endl;

                    break;
                  }
                case(TOR::PCOtoSUTClient):
                  {
                    // Connected to an SUT client object. The server is locally
                    // emulated by a GatewayServer
                                                 
                    CORBA::String_var srvn =  aPCO->objInfo.objName;
                    TOR_GatewayServer_impl* srv = 
		      new TOR_GatewayServer_impl(srvn, orb_, poa_);
                    TOR::GatewayServer_var srvvar = srv -> _this();
                    assert (!CORBA::is_nil(srvvar));
		    aPCO->objInfo.objRef = CORBA::Object::_duplicate(srvvar);

                    if(VERBOSITY > 0)
                      cout<< "GatewayMain_impl: a GatewayServer is created for"
                          <<" the emulated object '" << aPCO->objInfo.objName
                          << "' which is reachable via the PCO '" 
                          << aPCO->pName << "'" << endl;
                    break;
                  }
                
                case(TOR::CPtoTestComponent):
                  {
                    // Shall not come to here.
                    return TOR::GWNotOK;
                  }
                default:
                  {
                    // Shall not come to here.
                    return TOR::GWNotOK;
                  }
                }
              PCOList_->length(PCOList_->length()+1);
              PCOList_[PCOList_->length()-1] = *aPCO;
              
            }
        }
      else
        {
          // Concurrent TTCN
          for (CORBA::ULong i = 0; i <  config.connections.length(); i++)
            { 
              aPCO = new TOR::GWConnInfo(config.connections[i]);                    
              switch (aPCO->connType)
                {
                case(TOR::PCOtoSUTServer):
                  {
                    // Connection to an SUT server object
                    TOR_GatewayClient_impl* clt = new TOR_GatewayClient_impl(aPCO->objInfo.objName, orb_, aPCO->objInfo.objRef);
                    TOR::GatewayClient_var cltvar = clt->_this();
                    assert (!CORBA::is_nil(cltvar));
                    aPCO->objInfo.objRef = CORBA::Object::_duplicate(cltvar);

                    if(VERBOSITY > 0)
                      cout<< "GatewayMain_impl: a GatewayClient is created for"
                          <<" the SUT object '" << aPCO->objInfo.objName
                          << "' which is reachable via the PCO '" 
                          << aPCO->pName << "'" << endl;

                    break;
                  }
                case(TOR::PCOtoSUTClient):
                  {
                    // Connected to an SUT client object. The server is locally
                    // emulated by a GatewayServer
                                        
                    CORBA::String_var srvn =  aPCO->objInfo.objName;
                    TOR_GatewayServer_impl* srv = 
		      new TOR_GatewayServer_impl(srvn, orb_, poa_);
                    TOR::GatewayServer_var srvvar = srv -> _this();
                    assert (!CORBA::is_nil(srvvar));
		    aPCO->objInfo.objRef = CORBA::Object::_duplicate(srvvar);
                    if(VERBOSITY > 0)
                      cout<< "GatewayMain_impl: a GatewayServer is created for"
                          <<" the emulated object '" << aPCO->objInfo.objName
                          << "' which is reachable via the PCO '" 
                          << aPCO->pName << "'" << endl;

                    break;
                  }
                case(TOR::CPtoTestComponent):
                  {
                    // Build a CP to another test component
                    // No need to create a new instance, since the partner test
                    // component is represented by a GatewayMain, 
                    // whose reference is included in GWConnInfo.
                    return TOR::GWOK;
                  }
                default:
                  {
                    // Shall not come to here.
                    return TOR::GWNotOK;
                  }
                }
              PCOList_->length(PCOList_->length()+1);
              PCOList_[PCOList_->length()-1] = *aPCO;
            }
        }
      VERBOSE(("GatewayMain_impl: The PCO/CP list is ready."));
      return TOR::GWOK;
    }
  catch(CORBA::SystemException& ex)
    {
      cout << "GatewayMain_impl: Configuration failed "
           <<"due to CORBA system exception." << endl;
      whichSysExcept(&ex);
      return TOR::GWNotOK;
    }
  catch(...)
    {
      cout << "GatewayMain_impl: Configuration failed "
           <<"due to unknown failure." << endl;
      return TOR::GWNotOK;
    }
}


//------------------
// GWAddConnection  |
//------------------

TOR::GWStatus TOR_GatewayMain_impl::GWAddConnection
(const TOR::GWConnInfo& connInfo)
  throw(CORBA::SystemException)
{
  try
    {
      // Check the PCO list whether a connection via the PCO
      // with pcoID already exits
      for (CORBA::ULong i = 0; i <  PCOList_->length(); i++)
        {
          if (strcmp(connInfo.pName, PCOList_[i].pName) == 0)
            {
              cout <<"GatewayMain_impl: A connection via the PCO '"
                   << connInfo.pName <<"' already exits." << endl;
              if (PCOList_[i].connType == connInfo.connType)
                {
                  if(VERBOSITY > 0)
                    cout <<"GatewayMain_impl: ConnType is correct," 
                         <<" the PCO list remains unchanged."<< endl;
                  return TOR::GWOK;
                }
              else
                {
                  if(VERBOSITY > 0)
                    cout <<"GatewayMain_impl: ConnType is not correct,"
                         <<" the PCO list remains unchanged." << endl;
                  return TOR::GWNotOK;
                }
            }
        }
      
      TOR::GWConnInfo* aPCO = new TOR::GWConnInfo(connInfo);
      
      switch (aPCO->connType)
        {
        case(TOR::PCOtoSUTServer):
          {
            // Connection to an SUT server object
            TOR_GatewayClient_impl* clt = new TOR_GatewayClient_impl(aPCO->objInfo.objName, orb_, aPCO->objInfo.objRef);
            TOR::GatewayClient_var cltvar = clt->_this();
            assert (!CORBA::is_nil(cltvar));
	    aPCO->objInfo.objRef = CORBA::Object::_duplicate(cltvar);

            if(VERBOSITY > 0)
              cout<< "GatewayMain_impl: a GatewayClient is created for"
                  <<" the SUT object '" << aPCO->objInfo.objName
                  << "' which is reachable via the PCO '" 
                  << aPCO->pName << "'" << endl;
            
            break;
          }
        case(TOR::PCOtoSUTClient):
          {
            // Connected to an SUT client object. The server is locally
            // emulated by a GatewayServer
                                        
            CORBA::String_var srvn =  aPCO->objInfo.objName;
            TOR_GatewayServer_impl* srv = 
	      new TOR_GatewayServer_impl(srvn, orb_, poa_);
            TOR::GatewayServer_var srvvar = srv -> _this();
            assert (!CORBA::is_nil(srvvar));
	    aPCO->objInfo.objRef = CORBA::Object::_duplicate(srvvar);
                      
            if(VERBOSITY > 0)          
              cout<< "GatewayMain_impl: a GatewayServer is created for"
                  <<" the emulated object '" << aPCO->objInfo.objName
                  << "' which is reachable via the PCO '" 
                  << aPCO->pName << "'" << endl;

            break;
          }
        case(TOR::CPtoTestComponent):
          {
            // Build a CP to another test component
            // No need to create a new instance, since the partner test
            // component is represented by a GatewayMain, whose reference
            // is included in GWConnInfo.
            return TOR::GWOK;
          }
        default:
          {
            // Shall not come to here.
            return TOR::GWNotOK;
          }
        }
      PCOList_->length(PCOList_->length()+1);
      PCOList_[PCOList_->length()-1] = *aPCO;
      return TOR::GWOK;
    }
  catch(CORBA::SystemException& ex)
    {
      cout << "GatewayMain_impl: system exception raised: "
	   << ex << endl; 
      cout <<"Failed adding new connection." << endl;
      whichSysExcept(&ex);
      return TOR::GWNotOK;
    }
  catch(CORBA::Exception &ex)
    {
      cout << "GatewayMain_impl: exception raised :"<< ex << endl;
      cout <<"Failed adding new connection." << endl;
    }
  catch(...)
    {
      cout << "GatewayMain_impl: Failed adding new connection." << endl;
      return TOR::GWNotOK;
    }
}


//------------
// GWSend     |
//------------

TOR::GWStatus TOR_GatewayMain_impl::GWSend
(const char* pcoID, const TOR::GWMessage& msg)
  throw(CORBA::SystemException)
{
  try{
    // Send TOR::GWMsg
    for(CORBA::ULong i=0; i < PCOList_->length(); i++)
      {           
	if (strcmp(pcoID, PCOList_[i].pName) == 0)
	  {
	    switch (PCOList_[i].connType)
	      {
	      case(TOR::PCOtoSUTServer):
		{
		  TOR::GatewayClient_var clt = 
		    TOR::GatewayClient::_narrow(PCOList_[i].objInfo.objRef);
		  if(!CORBA::is_nil(clt))
		    {
		      if(clt->sendOutMsg(msg) == TOR::GWOK)
			{
			  if(VERBOSITY > 0)
			    cout << "GatewayMain_impl: Send message "
				 << "via GatewayClient '"
				 << PCOList_[i].pName <<"' completed. " << endl;
			  return TOR::GWOK;
			}
		      else
			{
			  cout << "GatewayMain_impl: Failed sending message "
			       <<"via the GatewayClient '"
			       << PCOList_[i].pName << "'." << endl;
			  return TOR::GWNotOK;
			}
		    }
		  else
		    {
		      cout << "GatewayMain_impl: Failed sending message "
			   <<"via the GatewayClient '"
			   << PCOList_[i].pName << "'." << endl;
		      return TOR::GWNotOK;
		    }
		}
	      case(TOR::PCOtoSUTClient):
		{
		  TOR::GatewayServer_var srv = TOR::GatewayServer::_narrow(PCOList_[i].objInfo.objRef);
		  if(!CORBA::is_nil(srv))
		    {
		      if(srv->sendOutMsg(msg) == TOR::GWOK)
			{
			  if(VERBOSITY > 0)
			    cout << "GatewayMain_impl: Send message "
				 << "via GatewayServer '"
				 << PCOList_[i].pName <<"' completed. " << endl;
			  return TOR::GWOK;
			}
		      else
			{
			  cout << "GatewayMain_impl: Failed sending message "
			       <<"via the GatewayServer '"
			       << PCOList_[i].pName << "'." << endl;
			  return TOR::GWNotOK;
			}
		    }
		  else
		    {
		      cout << "GatewayMain_impl: Failed sending message "
			   <<"via the GatewayServer '"
			   << PCOList_[i].pName << "'." << endl;
		      return TOR::GWNotOK;
		    }
		}
	      case(TOR::CPtoTestComponent):
		{
		  TOR::GatewayMain_var tc = TOR::GatewayMain::_narrow(PCOList_[i].objInfo.objRef);
		  // Include CP id in the objectName field
		  TOR::GWMessage_var cm = new TOR::GWMessage(msg);
		  cm->objectName = PCOList_[i].pName;
		  
		  if(!CORBA::is_nil(tc))
		    {
		      try
			{
			  tc->here_is_a_CM(cm);
			  if(VERBOSITY > 0)
			    cout << "GatewayMain_impl: Send message "
				 << "via CP '"
				 << PCOList_[i].pName <<"' completed. " << endl;
			  return TOR::GWOK;
			}
		      catch(...)
			{
			  cout << "GatewayMain_impl: Failed sending message "
			       <<"via CP '"
			       << PCOList_[i].pName << "'." << endl;
			  return TOR::GWNotOK;
			}
		    }
		  else
		    {
		      cout << "GatewayMain_impl: Failed sending message "
			   <<"via CP '"
			   << PCOList_[i].pName << "'." << endl;
		      return TOR::GWNotOK;
		    }
		}
	      default:
		{
		  break;
		}
	      }
	  }
      }    
  }
  catch (CORBA::Exception &ex)
    {
      cout << "GatewayMain_impl: Failed sending message "
	   <<"due to exception " 
	   << ex << endl;
      return TOR::GWNotOK;
    }
  return TOR::GWNotOK;
}


//--------------
// GWReceive    |
//--------------

TOR::GWMsgFlag TOR_GatewayMain_impl::GWReceive
(CORBA::String_out pcoID,TOR::GWMessage_out msg)
  throw(CORBA::SystemException)
{
  try
    {
      TOR::GWMessage_var amsg;
      for(CORBA::ULong i=0; i < PCOList_->length(); i++)
	{
	  // The searching order is significant! 
	  // Should all objects(message queues) be treated equally?
	  
	  switch (PCOList_[i].connType)
	    {
	    case(TOR::PCOtoSUTServer):
	      {
		TOR::GatewayClient_var clt = TOR::GatewayClient::_narrow(PCOList_[i].objInfo.objRef);
		if(!CORBA::is_nil(clt))
		  {
		    if(clt->getInMsg(amsg) == TOR::MsgReady)
		      {
			if(VERBOSITY > 0)
			  cout <<"GatewayMain_impl: Message is prepared by the "
			       <<"GatewayClient '" 
			       << PCOList_[i].pName <<"'" << endl;
			msg = new TOR::GWMessage(amsg);
			pcoID =  CORBA::string_dup(PCOList_[i].pName);
			return TOR::MsgReady;
		      }
		    else
		      break;
		  }
		else
		  break;
	      }
	    case(TOR::PCOtoSUTClient):
	      {
		TOR::GatewayServer_var srv = TOR::GatewayServer::_narrow(PCOList_[i].objInfo.objRef);
		if(!CORBA::is_nil(srv))
		  {
		    if(srv->getInMsg(amsg) == TOR::MsgReady)
		      {
			if(VERBOSITY > 0)
			  cout <<"GatewayMain_impl: Message is prepared by the "
			       <<"GatewayServer '" 
			       << PCOList_[i].pName <<"'" << endl;
			msg = new TOR::GWMessage(amsg);
			pcoID =  CORBA::string_dup(PCOList_[i].pName);
			return TOR::MsgReady;
		      }
		    else
		      break;
		  }
		else
		  break;
	      }
	    case(TOR::CPtoTestComponent):
	      {
		break;
	      }
	    }
	}
    }
  catch (CORBA::Exception &ex)
    {
      cout << "GatewayMain_impl: Failed receiving message "
	   <<"due to exception " 
	   << ex << endl;
      return TOR::MsgNotReady;
    }

  if (cmQueue_->length() > 1)
    {
      msg = new TOR::GWMessage(cmQueue_[0].message);
      pcoID = CORBA::string_dup(msg->objectName);
      cmQueue_->length(0);      
      return TOR::MsgReady;
    }

  // A dump message
  TOR::GWMessage_var dump = new TOR::GWMessage;
  dump->objectName = CORBA::string_dup("");
  dump->operationName = CORBA::string_dup("");
  dump->cntType = TOR::IsVoid;

  msg = new TOR::GWMessage(dump);
  pcoID = CORBA::string_dup("");
  return TOR::MsgNotReady;
}


