/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/

/*********************************************************************
 *                                                                    *
 *    Name:  GatewayServer_impl.cpp                                   *
 *                                                                    *
 *    Author: Mang Li                                                 * 
 *            GMD Fokus, TIP                                          *
 *                                                                    *
 *    Description: This file contains the implementation of the       *
 *        classes TOR_DIR and TOR_GatewayServer_impl.                 *
 *                                                                    *
 *                                                                    *
 *********************************************************************/

#include <orb_gw.h>

#include "GatewayServer_impl.h"
#include <fstream.h>

extern int VERBOSITY;
extern void verbose_print(char* fmt);
#define VERBOSE(F)	verbose_print F

using namespace CORBA;

#include "obv_test.h"
using namespace OBVTest;


// /****************
// *               *
// *   TOR_DIR     *
// *               *                          
// ****************/


//-------------
// Constructor |
//-------------

TOR_DIR::TOR_DIR(const char* name, const CORBA::ORB_ptr orb, TOR_GatewayServer_impl* srv)
  : orb_(CORBA::ORB::_duplicate(orb)),
    idlType_(CORBA::string_dup("")),
    objName_(CORBA::string_dup(name))
{
  VERBOSE(("TOR_DIR: in constructor."));
  //  cout << "TOR_DIR: objec name: "<<objName_<<endl;
  theServer_ = srv;
  assert(theServer_ != NULL);  
  getIDLType();   
}


//------------
// Destructor |
//------------

TOR_DIR::~TOR_DIR()
{
  VERBOSE(("TOR_DIR: in destructor."));
  //    CORBA::release(theServer_);
}


//-----------
// destroy   |
//-----------

void TOR_DIR::destroy()
{
  // ***todo
  //CORBA::release(theServer_);
  //CORBA::release(this);
  
}


//--------------
// getIDLType   |
//--------------

void TOR_DIR::getIDLType()
{
  CORBA::String_var tmp = CORBA::string_dup(objName_);
  char* buf = new char[strlen(tmp)+1];
  
  // Replace '::' by '/'
  CORBA::ULong i = 0;
  CORBA::ULong j = 0;
  for (i = 0; tmp[i] != '\0'; i++)
    {
      if (tmp[i] == ':')
        {
          if (tmp[i+1] == ':')
            {
              i++;
              buf[j] = '/';
              j++;
            }
          else
            {
              buf[j] = tmp[i];
              j++;
            }
        }
      else
        {
          buf[j] = tmp[i];
          j++;
        }
    }
  char* body = new char[j+1];
  strncpy(body, buf, j);
  body[j] = '\0';
  delete[] buf;

  idlType_ += CORBA::string_dup(IDLTYPE_PREFIX); 
  idlType_ += CORBA::string_dup(body);
  idlType_ += CORBA::string_dup(IDLTYPE_SUFIX);
  delete[] body;
}


//----------
// _is_a    |
//----------

CORBA::Boolean TOR_DIR::_is_a(const char* name)
  throw(CORBA::SystemException)
{
  if(strcmp(name, idlType_) == 0)
    {
      return true;
    }
  else
    {
      //***todo
      //return CORBA::Object::_is_a(name);
      return PortableServer::ServantBase::_is_a(name);
    }
  return false;
}


//-----------------
// _get_interface  |
//-----------------

CORBA::InterfaceDef_ptr TOR_DIR::_get_interface()
  throw(CORBA::SystemException)
{
  CORBA::Object_var obj;
  
  try
    {
      obj = orb_ -> resolve_initial_references("InterfaceRepository");
      assert(!CORBA::is_nil(obj));
      
    }
  catch(const CORBA::ORB::InvalidName&)
    {
      throw CORBA::INTF_REPOS(22, CORBA::COMPLETED_NO);
    }

  if(CORBA::is_nil(obj))
    {
      throw CORBA::INTF_REPOS(22, CORBA::COMPLETED_NO);
    }
								       
  CORBA::Repository_var repository = CORBA::Repository::_narrow(obj);

  if(CORBA::is_nil(repository))
    {
      throw CORBA::INTF_REPOS(22, CORBA::COMPLETED_NO);
    }

  CORBA::Contained_var contained =
    repository -> lookup_id(idlType_);

  if(CORBA::is_nil(contained))
    return CORBA::InterfaceDef::_nil();

  return CORBA::InterfaceDef::_narrow(contained);
}

CORBA::RepositoryId 
TOR_DIR::_primary_interface (const PortableServer::ObjectId&,
			     PortableServer::POA_ptr)
{
  return string_dup(idlType_);
}

//-------------------
// findOperationDef  |
//-------------------

CORBA::OperationDef_ptr TOR_DIR::findOperationDef(const char* opname)
{
  // First, search in the current interface
  CORBA::InterfaceDef_var interfDef = this->_get_interface();
  CORBA::InterfaceDef::FullInterfaceDescription_var current =
    interfDef-> describe_interface();
  for(CORBA::ULong i = 0 ; i < current->operations.length() ; i++)
    {
      if (strcmp(current->operations[i].name, opname) == 0)
        {
          CORBA::Contained_var contained = interfDef->lookup(opname);
          assert(!CORBA::is_nil(contained));
          CORBA::OperationDef_var opDef = 
            CORBA::OperationDef::_narrow(contained);
          assert(!CORBA::is_nil(opDef));
          return  (CORBA::OperationDef::_duplicate(opDef));
        }
    }

  // Search in the base interfaces
  CORBA::InterfaceDefSeq_var seq = interfDef->base_interfaces();
  for(CORBA::ULong m=0; m<seq->length(); m++)
    {
      CORBA::InterfaceDef::FullInterfaceDescription_var base =
        seq[m] -> describe_interface();

      for(CORBA::ULong n = 0 ; n < base->operations.length() ; n++)
        {
          if (strcmp(base->operations[n].name, opname) == 0)
            {
              CORBA::Contained_var contained = seq[m]->lookup(opname);
              assert(!CORBA::is_nil(contained));
              CORBA::OperationDef_var opDef = CORBA::OperationDef::_narrow(contained);
              assert(!CORBA::is_nil(opDef));
              return  (CORBA::OperationDef::_duplicate(opDef));
            }
        }
    }
  return  (CORBA::OperationDef::_nil());  
}


//------------
// invoke     |
//------------

void TOR_DIR::invoke(CORBA::ServerRequest_ptr request) throw()
{
  /*
  NVList_ptr list;
  orb_ -> create_list(0, list);
  list -> add(ARG_IN) -> value() -> replace(_tc_TestShortBox, 0);
  list -> add(ARG_OUT) -> value() -> replace(_tc_short, 0);
  request -> arguments(list);

  //const Any &a = *(list-> item(0) -> value());
  TestShortBox* a1;
  *(list-> item(0) -> value()) >>= a1;
  cout <<"hhh a1:"<< a1->_value()<<endl;

  Any *b = list-> item(1) -> value();
  Short b1 = 543;
  *b <<= b1;
  return;
  */

  // JTC enables multi-threading
  JTCSynchronized synchronized(*this);
  
  TOR::GWMessage_var inMsg = new TOR::GWMessage;
  
  inMsg->objectName = CORBA::string_dup(objName_);
  inMsg->operationName = CORBA::string_dup(request->operation());
  inMsg->cntType = TOR::IsVoid;

  if (VERBOSITY > 0)
    cout << "TOR_DIR: In the method invoke() for the object '"
         << inMsg->objectName << "' and the operation '"
         << inMsg->operationName << "'" << endl;
  
  // Get the NVList
  CORBA::OperationDef_var opDef = findOperationDef(request->operation());
  assert(!CORBA::is_nil(opDef));

  // Type code of return value
  CORBA::TypeCode_var rvType = opDef -> result();

  // List of arguments
  CORBA::NVList_ptr pList;
  orb_ -> create_operation_list (opDef, pList);  
  request -> arguments(pList);

  if (CORBA::is_nil(pList))  // No arguments
    {
      if (VERBOSITY > 0)
	{
	  if (rvType->kind() != CORBA::tk_null && 
	      rvType->kind() != CORBA::tk_void)
            cout << "TOR_DIR:The operation '" 
                 << inMsg->operationName << "'"
                 << "expects no parameters but return value." << endl;
	  else
	    cout << "TOR_DIR:The operation'" 
                 << inMsg->operationName << "'"
                 << "expects no parameters and no return value." 
                 << endl;
        }
    }
  else // Arguments
    {
      inMsg->cntType  = TOR::Parameters;
      
      if (VERBOSITY > 0)
	{
	  if (rvType->kind() != CORBA::tk_null && 
	      rvType->kind() != CORBA::tk_void)
	    cout << "TOR_DIR:The operation'" 
                 << inMsg->operationName << "'"
                 << "expects parameters and return value." << endl;
	  else
            cout << "TOR_DIR:The operation'" 
                 << inMsg->operationName << "'"
                 << "expects parameters but no return value." << endl;
        }
            
      // Save all in/inout parameters in the inMsg
      TOR::GWParamList_var pl = new TOR::GWParamList;
      for (CORBA::ULong i = 0; i < pList -> count(); i++)
        {
          if(pList->item(i)->flags() != CORBA::ARG_OUT)
            {
              if (VERBOSITY > 0)
                cout << "TOR_DIR:Save the '" << i 
                     << "th' parameter of mode '" << pList->item(i)->flags() 
                     << "' in the incoming message" <<endl;
	      pl->length(pl->length()+1);
	      pl[pl->length()-1] =*(pList->item(i)->value());
            }
        }

      TOR::GWMsgContent* ct = new TOR::GWMsgContent;
      ct->paramList = *pl;
      inMsg->content = *ct;
    }

  // Add the msg in the server's queue
  theServer_->addTo(*inMsg);
  

  // 
  //  Waiting for the reply ...
  //

  VERBOSE(("TOR_DIR: Waiting for the reply ..."));
  TOR::GWMessage* amsg;
  while (theServer_->removeFrom(request->operation(), amsg) == TOR::MsgNotReady)
    {
      // JTC function
      try
        {
	  wait();
        }
      catch (const JTCInterruptedException&)
        {
          CORBA::Any* any = new CORBA::Any;
          *any <<= new CORBA::NO_RESPONSE(1, CORBA::COMPLETED_MAYBE);
          request->set_exception(*any);
	  return;
        }

    }
  
  switch(amsg->cntType)
    {
    case (TOR::Parameters):
      {
        // Shall not come to here
        VERBOSE(("TOR_DIR: Failure - the reply message shall contain at least the return value field, which is NULL if the type of the return result is void."));
        
        CORBA::Any* any = new CORBA::Any;
        *any <<= new CORBA::NO_RESPONSE(1, CORBA::COMPLETED_MAYBE);
	request->set_exception(*any);
        break;
      }
    case (TOR::ReturnValue):
      {
        // Set return value
        if (rvType->kind() != CORBA::tk_null 
	    && rvType->kind() != CORBA::tk_void)
          {
	    request -> set_result(amsg->content.returnValue);
            if (VERBOSITY > 0)
              cout << "TOR_DIR: Return value "
                   << "of the operation '" << amsg->operationName 
                   <<"' updated." << endl;
          }
        else
          if (VERBOSITY > 0)
            cout << "TOR_DIR: The operation '" 
                 << amsg->operationName 
                 <<"' does not expect a return result." << endl;
        break;
      }
    case (TOR::ReturnAndParameters):
      {
        // Set inout/out parameter values
        if(amsg->content.paramList.length() > 0)
          {
            CORBA::ULong m = 0;
            for (CORBA::ULong k=0; k < pList->count(); k++)
              {
                if((pList->item(k)->flags() == CORBA::ARG_INOUT) || 
		   (pList->item(k)->flags() == CORBA::ARG_OUT))
                  {
		    // Check type code
                    // To be considered if the type checking shall be
                    // done by the Gateway.
		    // For types that use alias this checking will fail,
		    // but the operation will be successful. Therefore
		    // type code checking is no more used.

                    //CORBA::TypeCode_var tc1 = pList->item(k)->value()->type();
                    //CORBA::TypeCode_var tc2 = amsg->content.paramList[m].type();
                    //assert (tc1->kind()  == tc2->kind());
		    *(pList->item(k)->value()) = amsg->content.paramList[m];
                    m++;
                  }
              }
          }
                
        // Set return value
        //if(amsg->content.returnValue.value()!=NULL)
        if (rvType->kind() != CORBA::tk_null && rvType->kind() != CORBA::tk_void) 
	  {
	    request -> set_result(amsg->content.returnValue);
          }
        if (VERBOSITY > 0)
          cout << "TOR_DIR: Return value (or void) and values for " 
               <<"inout/out parameters of the operation '" 
               << amsg->operationName <<"' updated." << endl;
        break;
      }
    case (TOR::IsException):
      {
        assert(amsg->content.except.value()!=NULL);
	request -> set_exception(amsg->content.except);
        if (VERBOSITY > 0)
          cout << "TOR_DIR: Exception raised for the operation '" 
               << amsg->operationName <<"' updated." << endl;
        break;
      }
    case (TOR::IsVoid):
      {
        // Shall not come to here
        VERBOSE(("TOR_DIR: Failure - the reply message shall contain at least the return value field, which is NULL if the type of the return result is void."));
        CORBA::Any* any = new CORBA::Any;
        *any <<= new CORBA::NO_RESPONSE(1, CORBA::COMPLETED_MAYBE);
        request->set_exception(*any);
        break;
      }
    default:
      {
        CORBA::Any* any = new CORBA::Any;
        *any <<= new CORBA::NO_RESPONSE(1, CORBA::COMPLETED_MAYBE);
        request->set_exception(*any);
        break;
      }
    }
  return;
  
}




// /*************************************
//  *                                   *
//  *        TOR_GatewayServer_impl     *
//  *                                   *
//  ************************************/


//-------------
// Constructor |
//-------------

TOR_GatewayServer_impl::TOR_GatewayServer_impl(const char* name, const CORBA::ORB_ptr orb, PortableServer::POA_ptr poa)
  : orb_(CORBA::ORB::_duplicate(orb)),
    poa_(PortableServer::POA::_duplicate(poa)),
    inMsgQueue_(new TOR::GWMsgQueue),
    outMsgQueue_(new TOR::GWMsgQueue)
{
  VERBOSE(("GatewayServer_impl: in constructor."));
  try
    {
      theDIR_ = new TOR_DIR (name, orb_, this);
      PortableServer::ObjectId_var id = poa_ -> activate_object(theDIR_);
      CORBA::Object_var pDSI =
	poa_ -> create_reference_with_id(id, theDIR_->idlType_);
      CORBA::String_var ostr = orb -> object_to_string(pDSI);

      CORBA::String_var refF = name;
      refF += ".ref";
      ofstream outF(refF);
      assert(!outF.fail());
      outF << ostr << endl;
      outF.close();
      if (VERBOSITY > 0)
	cout << "GatewayServer_impl: Reference of the DIR is saved in file '"
	     << refF <<"'" << endl;
    }
  catch (CORBA::Exception &ex)
    {
      cout <<"TOR_GatewayServer_impl: exception raised: "<< ex<<endl;
    }
}


//------------
// Destructor |
//------------

TOR_GatewayServer_impl::~TOR_GatewayServer_impl()
{
  VERBOSE(("GatewayServer_impl: in destructor."));
  inMsgQueue_->length(0);
  outMsgQueue_->length(0);
}


//----------
// addTo    |
//----------

TOR::GWStatus TOR_GatewayServer_impl::addTo(const TOR::GWMessage& msg)
{
  TOR::GWQElem* inQElem = new TOR::GWQElem;
  inQElem->message = msg;
  inQElem->flag = TOR::MsgReady;

  inMsgQueue_->length(inMsgQueue_->length()+1);
  inMsgQueue_[inMsgQueue_->length()-1] = *inQElem;
  
  return TOR::GWOK;
}


//--------------
// removeFrom   |
//--------------

TOR::GWMsgFlag TOR_GatewayServer_impl::removeFrom(const char* opName, TOR::GWMessage_out msg)
{
  // Other method to identify certain client in the case
  // that multiple clients invoke the same operation???
  for (CORBA::ULong i=0; i < outMsgQueue_->length(); i++)
    {
      if (strcmp(outMsgQueue_[i].message.operationName, opName) == 0 
	  && outMsgQueue_[i].flag == TOR::MsgReady)
        {
          msg = new TOR::GWMessage(outMsgQueue_[i].message);
	  for (CORBA::ULong j=i; j < outMsgQueue_->length()-1; j++)
	    outMsgQueue_[j] = outMsgQueue_[j+1];
	  outMsgQueue_->length(outMsgQueue_->length()-1);
          VERBOSE(("GatewayServer_impl:Message ready for DIR."));
          return TOR::MsgReady;
        }
    }
  
  // No message is ready for DIR.
  return TOR::MsgNotReady;
}


//-------------
// getInMsg    |
//-------------

TOR::GWMsgFlag TOR_GatewayServer_impl::getInMsg(TOR::GWMessage_out msg)
  throw(CORBA::SystemException)
{
  // Pass the first completed message in the incoming message queue 
  // as argument back

  for (CORBA::ULong i=0; i < inMsgQueue_->length(); i++)
    {
      if (inMsgQueue_[i].flag == TOR::MsgReady)
        {
          VERBOSE(("GatewayServer_impl: Return an incoming message."));
          msg = new TOR::GWMessage(inMsgQueue_[i].message);
	  for (CORBA::ULong j=i; j < inMsgQueue_->length()-1; j++)
	    inMsgQueue_[j] = inMsgQueue_[j+1];
	  inMsgQueue_->length(inMsgQueue_->length()-1);
          return TOR::MsgReady;
        }
    }
  
   // No message is ready for return.
  return TOR::MsgNotReady;
}


//--------------
// sendOutMsg   |
//--------------

TOR::GWStatus TOR_GatewayServer_impl::sendOutMsg(const TOR::GWMessage& msg)
  throw(CORBA::SystemException)
{
  // JTC enables multi-threading
  JTCSynchronized synchronized(*theDIR_);
  try
    {
      // New element in the outgoing message queue
      TOR::GWQElem* outQElem = new TOR::GWQElem;
      TOR::GWMessage* amsg = new TOR::GWMessage(msg);
  
      outQElem->message = *amsg;
      outQElem->flag = TOR::MsgReady;

      outMsgQueue_->length(outMsgQueue_->length()+1);
      outMsgQueue_[outMsgQueue_->length()-1] = *outQElem;
      theDIR_->notify();
      return TOR::GWOK;
    }
  catch(CORBA::SystemException& ex)
    {
      cerr << &ex << endl;
      cout << "GatewayServer_impl: sendOutMsg() failed "
           <<"due to CORBA system exception." << endl;
      return TOR::GWNotOK;
    }
  catch(...)
    {
      cout << "GatewayServer_impl: sendOutMsg() failed "
           <<"due to unknown failure." << endl;
      return TOR::GWNotOK;
    }
}


