/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/


/*********************************************************************
 *                                                                    *
 *    Name:  GatewayServer_impl.h                                     *
 *                                                                    *
 *    Author: Mang Li                                                 *
 *            GMD Fokus, TIP                                          *
 *                                                                    *
 *    Description: This header file contains the declaration of the   *
 *        classes TOR_DIR and TOR_GatewayServer_impl.                 *
 *                                                                    *
 *********************************************************************/


#include <Gateway_skel.h>

// Multi-Threading support - not supported by CYGNUS
#include <JTC/JTC.h>

#define IDLTYPE_PREFIX "IDL:"
#define IDLTYPE_SUFIX ":1.0"


class TOR_GatewayServer_impl;

// The Dynamic Implementation Routine
class TOR_DIR : virtual public PortableServer::DynamicImplementation,
		JTCMonitor 
{
  friend class TOR_GatewayServer_impl;
  
public:

  TOR_DIR(const char*, const CORBA::ORB_ptr, TOR_GatewayServer_impl*);
  ~TOR_DIR();

protected:
  void destroy();

private:
  
  CORBA::ORB_var orb_;
  
  // The scoped name of the represented object.
  CORBA::String_var idlType_;

  // Name of the represented object.
  const CORBA::String_var objName_; 

  // The owner GatwayServer
  TOR_GatewayServer_impl* theServer_;

private:

  // Help function to construct scoped names for the use of
  // interface repositories.
  void getIDLType();

  // Find the OperationDef for an operation
  // Since inherited operations can not always be found in the
  // current InterfaceDef, base interfaces must be searched.
  CORBA::OperationDef_ptr findOperationDef(const char* opname);

public:

  // Overloading suggested by ORBacus vendor, in order
  // to support the use of interface repositories
  virtual CORBA::Boolean _is_a(const char*)
    throw(CORBA::SystemException);
  virtual CORBA::InterfaceDef_ptr _get_interface()
    throw(CORBA::SystemException);

  virtual CORBA::RepositoryId
    _primary_interface(const PortableServer::ObjectId&,
		       PortableServer::POA_ptr);
public:
  
  // The mandatory operation of DIR
  virtual void invoke(CORBA::ServerRequest_ptr) throw ();
};


class TOR_GatewayServer_impl : public POA_TOR::GatewayServer
{
  friend class TOR_DIR;
public:

  TOR_GatewayServer_impl(const char*, const CORBA::ORB_ptr, PortableServer::POA_ptr);
  ~TOR_GatewayServer_impl();

private:
  
  CORBA::ORB_var orb_; // ORB
  PortableServer::POA_var poa_;
  // Message queue for incoming requests from ORB
  TOR::GWMsgQueue_var inMsgQueue_;

  // Message queue for outgoing replies or exceptions to ORB
  TOR::GWMsgQueue_var outMsgQueue_;

private:
  
  // Help methods called by TOR_DIR to access messages in the
  // in and out msg queue.
  TOR::GWStatus addTo(const TOR::GWMessage&);
  TOR::GWMsgFlag removeFrom(const char*, TOR::GWMessage_out);

public:
  
  // The DIR
  TOR_DIR* theDIR_;
  
  // Get an incoming request from ORB
  virtual TOR::GWMsgFlag getInMsg(TOR::GWMessage_out msg)
    throw(CORBA::SystemException);

  // Process an outgoing reply or exception to ORB
  virtual TOR::GWStatus sendOutMsg(const TOR::GWMessage&)
    throw(CORBA::SystemException); 

}; 
