/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/

/*********************************************************************
 *                                                                    *
 *    Name:  Key.cpp                                                  *
 *                                                                    *
 *    Author: Mang Li                                                 * 
 *            GMD Fokus, TIP                                          *
 *                                                                    *
 *    Description: This file contains the main function of            *
 *           the gateway.                                             *
 *                                                                    *
 *                                                                    *
 *********************************************************************/


#include <orb_gw.h>
#include "GatewayMain_impl.h"
#include "obv_test_impl.h"

#include <fstream.h>

int VERBOSITY = 0;
#define DO_VERBOSE
#ifdef DO_VERBOSE
  
void verbose_print(char* fmt)
{
  if (VERBOSITY > 0)
    cout << fmt << endl;
  
};

#define VERBOSE(F)	verbose_print F
#endif


int main(int argc, char* argv[], char*[])
{
  // Set verbosity
  int ac = 0;
  while (ac < argc)
    {
      if (strcmp (argv[ac], "-v") == 0)
        {
          VERBOSITY = 1;
          break;
        }
      ac++;
    }


  //------------------
  // Init ORB and BOA |
  //------------------
  try
    {
      VERBOSE(("Key: Init ORB and BOA."));
            
      CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
      assert(!CORBA::is_nil(orb));
      
      CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
      PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poaObj);
    
      PortableServer::POAManager_var manager = rootPOA -> the_POAManager();

      TestValueSaver_impl* impl = new TestValueSaver_impl(orb);
      OBVTest::TestValueSaver_var saver = impl -> _this();

      //------------------
      // Init GatewayMain |
      //------------------

      VERBOSE(("Key: Init GatewayMain."));
      TOR_GatewayMain_impl* gwm = 
	new TOR_GatewayMain_impl(orb, rootPOA, manager, saver);
      
      PortableServer::ServantBase_var servant = gwm;
      TOR::GatewayMain_var gwmObj = gwm -> _this();
            
      // Save reference for GatewayMain in file
      CORBA::String_var ostr = orb->object_to_string(gwmObj);
      
      const char* refF = "GatewayMain.ref";
      ofstream outF(refF);
      assert(!outF.fail());
      outF << ostr << endl;
      outF.close();
      cout <<"Key: GatewayMain is ready." << endl;

      manager -> activate();
      orb -> run();

      if(!CORBA::is_nil(orb))
        orb -> destroy();
        
      return 0;
    }
  catch(CORBA::SystemException& ex)
    {
      cout << "Key: Failed creating GatewayMain" << endl;
      return 1;
    }
  catch(...)
    {
      cout << "Key: Failed creating GatewayMain" << endl;
      return 1;
    }

 
}



