/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/


#include <orb_gw.h>

#include <obv_test_impl.h>

#include <stdlib.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int VERBOSITY = 0;
#define DO_VERBOSE
#ifdef DO_VERBOSE
void verbose_print(char* fmt)
{
  if (VERBOSITY > 0)
    cout << fmt << endl;
  
};
#define VERBOSE(F)	verbose_print F
#endif


int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{

    // Resolve RootPOA
    //
    CORBA::Object_var obj;
    CORBA::String_var ref;
        
    obj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var root = PortableServer::POA::_narrow(obj);
    PortableServer::POAManager_var manager = root -> the_POAManager();

    TestValueSaver_impl* impl = new TestValueSaver_impl(orb);
    OBVTest::TestValueSaver_var saver = impl -> _this();

    //
    // Instantiate and activate the TestInterfaceA servant
    //
    TestInterfaceA_impl* aimpl = new TestInterfaceA_impl(saver);
    obj = aimpl -> _this();

    const char* aFile = "OBVTest::TestInterfaceA.ref";
    ofstream outa(aFile);
    if(outa.fail())
    {
        cerr << argv[0] << ": can't open `" << aFile << "'" << endl;
        return EXIT_FAILURE;
    }

    ref = orb -> object_to_string(obj);
    outa << ref << endl;
    outa.close();

    //
    // Instantiate and activate the TestInterfaceC servant
    //
    TestInterfaceC_impl* cimpl = new TestInterfaceC_impl(orb, saver);
    obj = cimpl -> _this();

    const char* cFile = "OBVTest::TestInterfaceC.ref";
    ofstream outc(cFile);
    if(outc.fail())
    {
        cerr << argv[0] << ": can't open `" << cFile << "'" << endl;
        return EXIT_FAILURE;
    }

    ref = orb -> object_to_string(obj);
    outc << ref << endl;
    outc.close();

    //
    // Instantiate and activate the TestInterfaceB servant
    // from a TestValueC instance
    //

    TestValueCFactory_impl* cf = new TestValueCFactory_impl();
    OBVTest::TestValueC_var vlc = cf -> create("A TestValueC instance.");
    // Conformant usage ?
    TestValueC_impl* imp_vlc =
      dynamic_cast<TestValueC_impl*>(vlc.in());
    OBVTest::TestInterfaceB_var ifb = imp_vlc -> _this();
    obj = CORBA::Object::_duplicate(ifb);
    
    const char* bFile = "OBVTest::TestInterfaceB.ref";
    ofstream outb(bFile);
    if(outb.fail())
    {
        cerr << argv[0] << ": can't open `" << bFile << "'" << endl;
        return EXIT_FAILURE;
    }

    ref = orb -> object_to_string(obj);
    outb << ref << endl;
    outb.close();
    cout <<"Server is ready." << endl;
    //
    // Run implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
