/******************************************************************** 
 * Copyright (c) 2000                                               *
 * GMD FOKUS                                                        *
 * Kaiserin-Augusta-Allee 31                                        *
 * 10589 Berlin                                                     *
 * Germany                                                          *
 * Tel: +49-30-3463-7000                                            *
 * Fax: +49-30-3463-8000                                            *
 *                                                                  *        
 * All rights reserved. No part of this source code may be          *
 * reproduced, stored in a retrieval system, or transmitted,        *
 * in any form or by any means, electronic, mechanical,             *
 * photocopying, recording or otherwise,                            *
 * except as stated in the end-user licence agreement,              * 
 * without the prior permission of the copyright owners.            *
 *                                                                  *
 * Project: CORVAL2                                                 *
 *                                                                  *
 * Contact:                                                         *
 * Mang Li                                                          *
 * e-mail: m.li@fokus.gmd.de                                        *
 *                                                                  *
 * Date: Oct. 2000                                                  *
 ********************************************************************/


/* obv_test_impl.cpp */

#include "orb_gw.h"
#include "obv_test_impl.h"
#include <math.h>

extern int VERBOSITY;

using namespace CORBA;
using namespace OBVTest;

//
//TestInterfaceA_impl
//

TestInterfaceA_impl::TestInterfaceA_impl(TestValueSaver_ptr saver) 
  :saver_(TestValueSaver::_duplicate(saver))
{

}

TestInterfaceA_impl::~TestInterfaceA_impl()
{
}


Res TestInterfaceA_impl::val_sub_intf(TestInterfaceB_ptr ifb)
  throw(SystemException)
{
  try
    {
      ifb -> pong();
    }
  catch (...)
    {
      return IS_NOK;
    }
  
  return IS_OK;
}

Res TestInterfaceA_impl::val_sub_val(TestValueD* vld)
  throw(SystemException)
{
  try
    {
      TestValueA* vla = TestValueA::_downcast(vld);
      if (vla != NULL)
        {
          vla->name();
          vla->age();
        }
    
      //more detail later to address differences between
      //base and derived types.
    }
  catch (...)
    {
      return IS_NOK;
    }
  
  return IS_OK;
}


Res TestInterfaceA_impl::sub_abstr_intf(TestAbstrInterfaceA_ptr abif) 
  throw(SystemException)
{
  try
    {
      String_var nm;
      abif -> describe_it(nm);
    }
  catch(SystemException& ex)
    {
      cout <<"server: sub_abstr_intf: sytem exception: "
           << ex.reason()<<endl;
      
      return IS_NOK;
    }
  catch (...)
    {
      return IS_NOK;
    }
  
  return IS_OK;
}

TestValueSaver_ptr TestInterfaceA_impl::get_value_saver()
  throw(SystemException)
{
  return TestValueSaver::_duplicate(saver_);  
}

//
//TestInterfaceB_impl
//
void TestInterfaceB_impl::pong()
{
  if(VERBOSITY > 0)
    cout << "pong of TestInterfaceB called."<< endl;
  return;  
}



//
//TestInterfaceC_impl
//

TestInterfaceC_impl::TestInterfaceC_impl(ORB_ptr orb, TestValueSaver_ptr saver)
  :orb_(ORB::_duplicate(orb)),
   saver_(TestValueSaver::_duplicate(saver))
{

}

TestInterfaceC_impl::~TestInterfaceC_impl()
{
}

Res TestInterfaceC_impl::hello(String_out s) throw(SystemException)
{
  if(VERBOSITY>0)
    cout <<s<<endl;
  return IS_OK;
}

Res TestInterfaceC_impl::in_shortbox(TestShortBox* b,
                                     Short_out value)
  throw(SystemException)
{
  Short inb;
  inb = b->_boxed_in();
  TestShortBox_var outb = new TestShortBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
}

Res TestInterfaceC_impl::out_shortbox(Short value,
				      TestShortBox_out b)
  throw(SystemException)
{
  b = new TestShortBox(value);
  return IS_OK;
}


Res TestInterfaceC_impl::in_ushortbox(TestUShortBox* b,
				      UShort_out value)
  throw(SystemException)
{
  UShort inb;
  inb = b->_boxed_in();
  TestUShortBox_var outb = new TestUShortBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
}

Res TestInterfaceC_impl::out_ushortbox(UShort value,
				       TestUShortBox_out b)
  throw(SystemException)
{
  b = new TestUShortBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_longbox(TestLongBox* b,
				    Long_out value)
  throw(SystemException)
{
  Long inb;
  inb = b->_boxed_in();
  TestLongBox_var outb = new TestLongBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
}

Res TestInterfaceC_impl::out_longbox(Long value,
				     TestLongBox_out b)
  throw(SystemException)
{
  b = new TestLongBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_ulongbox(TestULongBox* b,
				     ULong_out value)
  throw(SystemException)
{
  ULong inb;
  inb = b->_boxed_in();
  TestULongBox_var outb = new TestULongBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
}

Res TestInterfaceC_impl::out_ulongbox(ULong value,
				      TestULongBox_out b)
  throw(SystemException)
{
  b = new TestULongBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_llongbox(TestLLongBox* b,
				     LongLong_out value)
  throw(SystemException)
{
  LongLong inb;
  inb = b->_boxed_in();
  TestLLongBox_var outb = new TestLLongBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
}


Res TestInterfaceC_impl::out_llongbox(LongLong value,
				      TestLLongBox_out b)
  throw(SystemException)
{
  b = new TestLLongBox(value);
  return IS_OK;
}


Res TestInterfaceC_impl::in_ullongbox(TestULLongBox* b,
				      ULongLong_out value)
  throw(SystemException)
{
  ULongLong inb;
  inb = b->_boxed_in();
  TestULLongBox_var outb = new TestULLongBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
}


Res TestInterfaceC_impl::out_ullongbox(ULongLong value,
				       TestULLongBox_out b)
  throw(SystemException)
{
  b = new TestULLongBox(value);
  return IS_OK;
}


Res TestInterfaceC_impl::in_booleanbox(TestBooleanBox* b,
				       Boolean_out value)
  throw(SystemException)
{
  Boolean inb;
  inb = b->_boxed_in();
  TestBooleanBox_var outb = new TestBooleanBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
}


Res TestInterfaceC_impl::out_booleanbox(Boolean value,
					TestBooleanBox_out b)
  throw(SystemException)
{
  b = new TestBooleanBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_octetbox(TestOctetBox* b,
				     Octet_out value)
  throw(SystemException)
{
  Octet inb;
  inb = b->_boxed_in();
  TestOctetBox_var outb = new TestOctetBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
}

Res TestInterfaceC_impl::out_octetbox(Octet value,
                                      TestOctetBox_out b)
  throw(SystemException)
{
  b = new TestOctetBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_charbox(TestCharBox* b,
                                    Char_out value)
  throw(SystemException)
{ 
  Char inb;
  inb = b->_boxed_in();
  TestCharBox_var outb = new TestCharBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
}

Res TestInterfaceC_impl::out_charbox(Char value,
                                     TestCharBox_out b)
  throw(SystemException)
{
  b = new TestCharBox(value);
  return IS_OK;
 }

Res TestInterfaceC_impl::in_wcharbox(TestWCharBox* b,
                                     WChar_out value)
  throw(SystemException)
{
  WChar inb;
  inb = b->_boxed_in();
  TestWCharBox_var outb = new TestWCharBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
}

Res TestInterfaceC_impl::out_wcharbox(WChar value,
                                      TestWCharBox_out b)
  throw(SystemException)
{
  b = new TestWCharBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_floatbox(TestFloatBox* b,
                                     Float_out value)
  throw(SystemException)
{
  Float inb;
  inb = b->_boxed_in();
  TestFloatBox_var outb = new TestFloatBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
 }


Res TestInterfaceC_impl::out_floatbox(Float value,
                                      TestFloatBox_out b)
  throw(SystemException)
{
  b = new TestFloatBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_doublebox(TestDoubleBox* b,
                                      Double_out value)
  throw(SystemException)
{
  Double inb;
  inb = b->_boxed_in();
  TestDoubleBox_var outb = new TestDoubleBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
 }

Res TestInterfaceC_impl::out_doublebox(Double value,
                                       TestDoubleBox_out b)
  throw(SystemException)
{
  b = new TestDoubleBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_stringbox(TestStringBox* b,
                                      String_out value)
  throw(SystemException)
{
  String_var inb;
  inb = b->_boxed_in();
  value = string_dup(inb);
  return IS_OK;
 }

Res TestInterfaceC_impl::out_stringbox(const char* value,
                                       TestStringBox_out b)
  throw(SystemException)
{
  b = new TestStringBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_enumbox(TestEnumBox* b,
                                    TestEnum_out value)
  throw(SystemException)
{
  TestEnum inb;
  inb = b->_boxed_in();
  TestEnumBox_var outb = new TestEnumBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
 }

Res TestInterfaceC_impl::out_enumbox(TestEnum value,
                                     TestEnumBox_out b)
  throw(SystemException)
{
  b = new TestEnumBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_arraybox(TestArrayBox* b,
                                     TestArray_out value)
  throw(SystemException)
{
  const TestArray_slice* inb = b->_boxed_in();
  value[0] = * TestArray_dup(inb);
  return IS_OK;
 }

Res TestInterfaceC_impl::out_arraybox(const TestArray value,
                                      TestArrayBox_out b)
  throw(SystemException)
{
  b = new TestArrayBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_fstructbox(TestFixStructBox* b,
                                       TestFixStruct_out value)
  throw(SystemException)
{
  TestFixStruct inb;
  inb = b->_boxed_in();
  TestFixStructBox_var outb = new TestFixStructBox(inb);
  value = outb->_boxed_out();
  return IS_OK;
}

Res TestInterfaceC_impl::out_fstructbox(const TestFixStruct& value, 
					TestFixStructBox_out b) 
  throw(SystemException)
{
  b = new TestFixStructBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_vstructbox(TestVarStructBox* b,
                                       TestVarStruct_out value)
  throw(SystemException)
{
  TestVarStruct inb;
  inb = b->_boxed_in();
  value = new TestVarStruct(inb);
  return IS_OK;
}

Res TestInterfaceC_impl::out_vstructbox(const TestVarStruct& value,
                                        TestVarStructBox_out b)
  throw(SystemException)
{
  b = new TestVarStructBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_funionbox(TestFixUnionBox* b,
                                      TestFixUnion_out value)
  throw(SystemException)
{
  TestFixUnion_var inb;
  inb = b->_boxed_in();
  value = inb.out();
  return IS_OK;
}

Res TestInterfaceC_impl::out_funionbox(const TestFixUnion& value, 
				       TestFixUnionBox_out b)
  throw(SystemException)
{
  b = new TestFixUnionBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_vunionbox(TestVarUnionBox* b,
                                      TestVarUnion_out value)
  throw(SystemException)
{
  TestVarUnion inb;
  inb = b->_boxed_in();
  value = new TestVarUnion(inb);
  return IS_OK;
}

Res TestInterfaceC_impl::out_vunionbox(const TestVarUnion& value, 
				       TestVarUnionBox_out b)
  throw(SystemException)
{
  b = new TestVarUnionBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_shseqbox(TestShSeqBox* b,
                                     TestShSeq_out value)
  throw(SystemException)
{
  TestShSeq inb;
  inb = b->_boxed_in();
  value = new TestShSeq(inb);
  return IS_OK;
}

Res TestInterfaceC_impl::out_shseqbox(const TestShSeq& value,
                                      TestShSeqBox_out b)
  throw(SystemException)
{
  b = new TestShSeqBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_strseqbox(TestStrSeqBox* b,
                                      TestStrSeq_out value)
  throw(SystemException)
{
  TestStrSeq inb;
  inb = b->_boxed_in();
  value = new TestStrSeq(inb);
  return IS_OK;
}

Res TestInterfaceC_impl::out_strseqbox(const TestStrSeq& value, 
				       TestStrSeqBox_out b)
  throw(SystemException)
{
  b = new TestStrSeqBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::in_anybox(TestAnyBox* b,
                                   Any_out value)
  throw(SystemException)
{
  Any inb;
  inb = b->_boxed_in();
  value = new Any(inb);
  return IS_OK;
}

Res TestInterfaceC_impl::out_anybox(const Any& value,
                                    TestAnyBox_out b)
  throw(SystemException)
{
  b = new TestAnyBox(value);
  return IS_OK;
}

Res TestInterfaceC_impl::val_in_struct(TestValueA* v,
				       Long n,
                                       TestValMbStruct_out s)
  throw(SystemException)
{
  s = new TestValMbStruct();
  TestValueA_init* af = new TestValueAFactory_impl();
  TestValueA* vla = af -> create("A TestValueA instance.");

  vla ->name(string_dup(v->name()));
  vla ->age(v->age());
  
  s->nr = n;  
  s->valmb = vla;
  return IS_OK;
}

Res TestInterfaceC_impl::val_out_struct(const TestValMbStruct& s, 
					TestValueA_out v)
  throw(SystemException)
{
  try
    {
      s.valmb->name();
      s.valmb->age();
      
      TestValueA_init* af = new TestValueAFactory_impl();  
      v = af -> create("A TestValueA instance.");
      
      v->name(s.valmb->name());
      v->age(s.valmb->age());
  
      return IS_OK;
    }
  catch(CORBA::Exception &ex)
    {
      cout<<"val_out_struct: exception."<<ex<<endl;
      return IS_NOK;
    }
}

TestValueSaver_ptr TestInterfaceC_impl::get_value_saver()
  throw(SystemException)
{
  return TestValueSaver::_duplicate(saver_);
  
}

//
//TestInterfaceD_impl
//

TestInterfaceD_impl::TestInterfaceD_impl(const char* name)
  :myname_(name)
{ 
}


void TestInterfaceD_impl::describe_it(String_out id)
{
  id = string_dup(myname_);
  
}

TestInterfaceD_impl::~TestInterfaceD_impl()
{ 
}

//
// TestValueA_impl
//

TestValueA_impl::TestValueA_impl()
  :myname_("")
{

}

TestValueA_impl::TestValueA_impl(const char* name)
  :myname_(name)
{ 
  
}

TestValueA_impl::~TestValueA_impl()
{
}

void TestValueA_impl::print()
{
}

OBVTest::TestValueA* TestValueA_impl::change(OBVTest::TestValueA* inval, 
					     OBVTest::TestValueA*& ioval, 
					     OBVTest::TestValueA_out outval)
{
  cout<<"not implemented."<<endl;
}


//
//  TestValueAFactory_impl
//

ValueBase* TestValueAFactory_impl::create_for_unmarshal()
{
  return new TestValueA_impl();
}


TestValueA* TestValueAFactory_impl::create(const char* name)
{ 
  return new TestValueA_impl(name);
}


//
// TestValueB_impl
//

TestValueB_impl::TestValueB_impl()
  :myname_("")
{
  
}

TestValueB_impl::TestValueB_impl(const char* name)
  :myname_(name)
{ 
  
}

TestValueB_impl::~TestValueB_impl()
{
}

void TestValueB_impl::describe_it(String_out id)
{ 
  id = string_dup(myname_);
  
}


//
//  TestValueBFactory_impl
//

ValueBase* TestValueBFactory_impl::create_for_unmarshal()
{
  return new TestValueB_impl();
}


TestValueB* TestValueBFactory_impl::create(const char* name)
{ 
  return new TestValueB_impl(name);
}


//
// TestValueC_impl
//

TestValueC_impl::TestValueC_impl()
  :myname_("")
{

}

TestValueC_impl::TestValueC_impl(const char* name)
  :myname_(name)
{ 
  
}

TestValueC_impl::~TestValueC_impl()
{
}

void TestValueC_impl::ping()
{
  if(VERBOSITY > 0)
    cout <<"ping of TestValueC called."<<endl;
  return;
}

void TestValueC_impl::pong()
{ 
  if(VERBOSITY > 0)
    cout << "pong of TestValueC called."<< endl;
  return;
}

//
//  TestValueCFactory_impl
//

ValueBase* TestValueCFactory_impl::create_for_unmarshal()
{
  return new TestValueC_impl();
}


TestValueC* TestValueCFactory_impl::create(const char* name)
{ 
  return new TestValueC_impl(name);
}

//
// TestValueD_impl
//

TestValueD_impl::TestValueD_impl()
  :myid_(0)
{

}

TestValueD_impl::TestValueD_impl(Long id)
  :myid_(id)
{ 
  
}

TestValueD_impl::~TestValueD_impl()
{
}

void TestValueD_impl::id(CORBA::Long d)
{
  myid_ = d;
  return;
}

CORBA::Long TestValueD_impl::id()
{
  return myid_;
}


void TestValueD_impl::print()
{
  cout<<"print: not implemented."<<endl;
  return;
}

OBVTest::TestValueA* TestValueD_impl::change(OBVTest::TestValueA* inval, 
                                             OBVTest::TestValueA*& ioval, 
                                             OBVTest::TestValueA_out outval)
{
  cout<<"not implemented."<<endl;
  
}


//
//  TestValueDFactory_impl
//

ValueBase* TestValueDFactory_impl::create_for_unmarshal()
{
  return new TestValueD_impl();
}


TestValueD* TestValueDFactory_impl::create(Long id)
{ 
  return new TestValueD_impl(id);
}


//
// TestValueSaver_impl
//

TestValueSaver_impl::TestValueSaver_impl (ORB_ptr orb)
  :orb_(ORB::_duplicate(orb))
  
{
  Object_var obj = orb_ -> 
    resolve_initial_references("InterfaceRepository");
  assert(!is_nil(obj));
  								       
  repository_ = Repository::_narrow(obj);
  assert(!is_nil(repository_));
}

TestValueSaver_impl::~TestValueSaver_impl ()
{
  if(!is_nil(orb_)){
    orb_->destroy();
    release(orb_);
  }
  
}

void TestValueSaver_impl::register_all_factories()
  throw(SystemException)
{
  ValueFactoryBase_var vbf, oldf;
 
  // Register factory of TestValueA
  vbf  = new TestValueAFactory_impl();
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestValueA:1.0", vbf);
  
  // Register factory of TestValueB
  vbf = new TestValueBFactory_impl();
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestValueB:1.0", vbf);
  
  // Register factory of TestValueC
  vbf = new TestValueCFactory_impl();
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestValueC:1.0", vbf);
  
  // Register factory of TestValueD
  vbf = new TestValueDFactory_impl();
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestValueD:1.0", vbf);
  
  
  // Register valuebox factories
  
  vbf = new TestShortBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestShortBox:1.0", vbf);
  
  vbf = new TestUShortBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestUShortBox:1.0", vbf);
  
  vbf = new TestLongBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestLongBox:1.0", vbf);
  
  vbf = new TestULongBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestULongBox:1.0", vbf);
  
  vbf = new TestLLongBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestLLongBox:1.0", vbf);
  
  vbf = new TestULLongBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestULLongBox:1.0", vbf);
  
  vbf = new TestBooleanBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestBooleanBox:1.0", vbf);
  
  vbf = new TestOctetBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestOctetBox:1.0", vbf);
  
  vbf = new TestCharBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestCharBox:1.0", vbf);
  
  vbf = new TestWCharBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestWCharBox:1.0", vbf);
  
  vbf = new TestFloatBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestFloatBox:1.0", vbf);
  
  vbf = new TestDoubleBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestDoubleBox:1.0", vbf);
  
  vbf = new TestStringBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestStringBox:1.0", vbf);
  
  vbf = new TestEnumBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestEnumBox:1.0", vbf);
  
  vbf = new TestArrayBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestArrayBox:1.0", vbf);
  
  vbf = new TestFixStructBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestFixStructBox:1.0", vbf);
  
  vbf = new TestVarStructBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestVarStructBox:1.0", vbf);
  
  vbf = new TestFixUnionBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestFixUnionBox:1.0", vbf);
  
  vbf = new TestVarUnionBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestVarUnionBox:1.0", vbf);
  
  vbf = new TestShSeqBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestShSeqBox:1.0", vbf);
  
  vbf = new TestStrSeqBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestStrSeqBox:1.0", vbf);
  
  vbf = new TestAnyBox_init;
  oldf = orb_ -> register_value_factory("IDL:OBVTest/TestAnyBox:1.0", vbf);

  if(VERBOSITY > 0)
    cout <<"register_all_factory."<<endl;  
  return;
}

void TestValueSaver_impl::register_this_factory(const char* id)
  throw(SystemException)
{
  if((orb_ -> lookup_value_factory(id))!= NULL)
    {
      if(VERBOSITY > 0)
	cout <<"register_this_factory: "<<
	  id << " already registered."<< endl;
      return;
    }

  Contained_var contained = repository_ -> lookup_id(id);
  assert(!is_nil(contained));

  ValueFactoryBase_var vbf, oldf;

  if(strcmp(id, "IDL:OBVTest/TestValueA:1.0") == 0)
    vbf = new TestValueAFactory_impl();
  
  else if(strcmp(id, "IDL:OBVTest/TestValueB:1.0") == 0)
    vbf = new TestValueBFactory_impl();
  
  else if(strcmp(id, "IDL:OBVTest/TestValueC:1.0") == 0) 
    vbf = new TestValueCFactory_impl();
  
  else if(strcmp(id, "IDL:OBVTest/TestValueD:1.0") == 0) 
    vbf = new TestValueDFactory_impl();
  
  else if(strcmp(id, "IDL:OBVTest/TestShortBox:1.0") == 0)
    vbf = new TestShortBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestUShortBox:1.0") == 0)
    vbf = new TestUShortBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestLongBox:1.0") == 0)
    vbf = new TestLongBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestULongBox:1.0") == 0)
    vbf = new TestULongBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestLLongBox:1.0") == 0)
    vbf = new TestLLongBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestULLongBox:1.0") == 0)
    vbf = new TestULLongBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestBooleanBox:1.0") == 0)
    vbf = new TestBooleanBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestOctetBox:1.0") == 0)
    vbf = new TestOctetBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestCharBox:1.0") == 0)
    vbf = new TestCharBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestWCharBox:1.0") == 0)
    vbf = new TestWCharBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestFloatBox:1.0") == 0)
    vbf = new TestFloatBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestDoubleBox:1.0") == 0)
    vbf = new TestDoubleBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestStringBox:1.0") == 0)
    vbf = new TestStringBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestEnumBox:1.0") == 0)
    vbf = new TestEnumBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestArrayBox:1.0") == 0)
    vbf = new TestArrayBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestFixStructBox:1.0") == 0)
    vbf = new TestFixStructBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestVarStructBox:1.0") == 0)
    vbf = new TestVarStructBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestFixUnionBox:1.0") == 0)
    vbf = new TestFixUnionBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestVarUnionBox:1.0") == 0)
    vbf = new TestVarUnionBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestShSeqBox:1.0") == 0)
    vbf = new TestShSeqBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestStrSeqBox:1.0") == 0)
    vbf = new TestStrSeqBox_init;
  
  else if(strcmp(id, "IDL:OBVTest/TestAnyBox:1.0") == 0)
    vbf = new TestAnyBox_init;
  else
    cout <<"register_this_factory: should not come here."
         <<"strcmp failed."<<endl;
  
  oldf = orb_ -> register_value_factory(id, vbf);
  if(VERBOSITY > 0)
    cout <<"register_this_factory: "<<
      id <<endl;
  return;
}

void TestValueSaver_impl::unregister_this_factory(const char* id)
  throw(SystemException)
{
  if((orb_ -> lookup_value_factory(id))!= NULL)
    {
      orb_ -> unregister_value_factory(id);
      if(VERBOSITY > 0)
        cout <<"unregister_this_factory: " <<
          id <<endl;
    }
  return;
}

//
// TestHelper_impl
//

TestHelper_impl::TestHelper_impl(ORB_ptr orb, 
				 TestValueSaver_ptr saver)
  :orb_(ORB::_duplicate(orb)),
   saver_(TestValueSaver::_duplicate(saver))
{
}

void TestHelper_impl::make_call(const char* ifid, const char* opid, 
				Res_out rst)
  throw(SystemException)
{
  Object_var obj;
  rst = IS_NOK;
  
  if(strcmp(ifid, "OBVTest::TestInterfaceC") == 0)
    {
      // Obtain TestInterfaceC
      obj = orb_ -> string_to_object("relfile:/OBVTest::TestInterfaceC.ref");
      assert(!is_nil(obj));
      TestInterfaceC_var ifc = TestInterfaceC::_narrow(obj);
      assert(!is_nil(ifc)); 

      if(strcmp(opid, "in_shortbox") == 0)
	{
	  Short osh = 0;
	  TestShortBox_var svb = new TestShortBox(777);
	  try
	    {
	      ifc->in_shortbox(svb, osh);
	      if(VERBOSITY>0)
		cout<<"in_shortbox successful :"<< osh <<endl;
	      if (osh == 777)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_shortbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_shortbox") == 0)
	{
	  Short ish = 32767;
	  TestShortBox_var svb;
	  try
	    {
	      ifc->out_shortbox(ish, svb);
	      if(VERBOSITY>0)
		cout<<"out_shortbox successful :"<< svb->_value() <<endl;
	      if (svb->_value() == 32767)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_shortbox failed due to exception: "
		  << ex << endl;
	    }
	  
	}
      else if(strcmp(opid, "in_ushortbox") == 0)
	{
	  UShort o = 0;
	  TestUShortBox_var b = new TestUShortBox(65535);
	  try
	    {
	      ifc->in_ushortbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_ushortbox successful." << endl;
	      if (o == 65535)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_ushortbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_ushortbox") == 0)
	{
	  UShort i = 65535;
	  TestUShortBox_var b;
	  try
	    {
	      ifc->out_ushortbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_ushortbox successful" << endl;
	      if (b->_value() == 65535)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_ushortbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_longbox") == 0)
	{
	  Long o = 0;
	  TestLongBox_var b = new TestLongBox(-21474836);
	  try
	    {
	      ifc->in_longbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_longbox successful." << endl;
	      if (o == -21474836)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_longbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_longbox") == 0)
	{
	  Long i = -21474836;
	  TestLongBox_var b;
	  try
	    {
	      ifc->out_longbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_longbox successful" << endl;
	      if (b->_value() == -21474836)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_longbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_ulongbox") == 0)
	{
	  ULong o = 0;
	  TestULongBox_var b = new TestULongBox(2147483647);
	  try
	    {
	      ifc->in_ulongbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_ulongbox successful." << endl;
	      if (o == 2147483647)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_ulongbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_ulongbox") == 0)
	{
	  ULong i = 2147483647;
	  TestULongBox_var b;
	  try
	    {
	      ifc->out_ulongbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_ulongbox successful" << endl;
	      if (b->_value() == 2147483647)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_ulongbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_llongbox") == 0)
	{
	  LongLong o = 0;
	  TestLLongBox_var b = new TestLLongBox(32767);
	  try
	    {
	      ifc->in_llongbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_llongbox successful." << endl;
	      if (o == 32767)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_llongbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_llongbox") == 0)
	{
	  LongLong i = 32767;
	  TestLLongBox_var b;
	  try
	    {
	      ifc->out_llongbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_llongbox successful" << endl;
	      if (b->_value() == 32767)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_llongbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_ullongbox") == 0)
	{
	  ULongLong o = 0;
	  TestULLongBox_var b = new TestULLongBox(32767);
	  try
	    {
	      ifc->in_ullongbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_ullongbox successful." << endl;
	      if (o == 32767)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_ullongbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_ullongbox") == 0)
	{
	  ULongLong i = 32767;
	  TestULLongBox_var b;
	  try
	    {
	      ifc->out_ullongbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_ullongbox successful" << endl;
	      if (b->_value() == 32767)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_ullongbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_booleanbox") == 0)
	{
	  Boolean o = 0;
	  TestBooleanBox_var b = new TestBooleanBox(1);
	  try
	    {
	      ifc->in_booleanbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_booleanbox successful." << endl;
	      if (o == 1)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_booleanbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_booleanbox") == 0)
	{
	  Boolean i = 0;
	  TestBooleanBox_var b;
	  try
	    {
	      ifc->out_booleanbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_booleanbox successful" << endl;
	      if (b->_value() == 0)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_booleanbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_octetbox") == 0)
	{
	  Octet o;
	  TestOctetBox_var b = new TestOctetBox('G');
	  try
	    {
	      ifc->in_octetbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_octetbox successful." << endl;
	      if (o == 'G')
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_octetbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_octetbox") == 0)
	{
	  Octet i = 'G';
	  TestOctetBox_var b;
	  try
	    {
	      ifc->out_octetbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_octetbox successful" << endl;
	      if (b->_value() == 'G')
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_octetbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_charbox") == 0)
	{
	  Char o;
	  TestCharBox_var b = new TestCharBox('M');
	  try
	    {
	      ifc->in_charbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_charbox successful." << endl;
	      if (o == 'M')
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_charbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_charbox") == 0)
	{
	  Char i = 'D';
	  TestCharBox_var b;
	  try
	    {
	      ifc->out_charbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_charbox successful" << endl;
	      if (b->_value() == 'D')
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_charbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_wcharbox") == 0)
	{
	  WChar o;
	  TestWCharBox_var b = new TestWCharBox('$');
	  try
	    {
	      ifc->in_wcharbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_wcharbox successful." << endl;
	      if (o == '$')
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_wcharbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_wcharbox") == 0)
	{
	  WChar i = '}';
	  TestWCharBox_var b;
	  try
	    {
	      ifc->out_wcharbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_wcharbox successful" << endl;
	      if (b->_value() == '}')
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_wcharbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_floatbox") == 0)
	{
	  Float o;
	  TestFloatBox_var b = new TestFloatBox(0.789403);
	  try
	    {
	      ifc->in_floatbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_floatbox successful." << endl;
	      if ((short)fabs(o*10) == 7)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_floatbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_floatbox") == 0)
	{
	  Float i = 0.789403;
	  TestFloatBox_var b;
	  try
	    {
	      ifc->out_floatbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_floatbox successful" << endl;
	      if ((short)fabs(b->_value()*10) == 7)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_floatbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_doublebox") == 0)
	{
	  Double o;
	  TestDoubleBox_var b = new TestDoubleBox(0.0034567893);
	  try
	    {
	      ifc->in_doublebox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_doublebox successful." << endl;
	      if ((short)fabs(o*1000) == 3)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_doublebox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_doublebox") == 0)
	{
	  Double i = 0.0034567893;
	  TestDoubleBox_var b;
	  try
	    {
	      ifc->out_doublebox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_doublebox successful" << endl;
	      Double a = b->_value();
	      if ((short)fabs(a*1000) == 3)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_doublebox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_stringbox") == 0)
	{
	  String_var o;
	  TestStringBox_var b = new TestStringBox("GMD FOKUS");
	  try
	    {
	      ifc->in_stringbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_stringbox successful." << endl;
	      if (strcmp(o, "GMD FOKUS") == 0)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_stringbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_stringbox") == 0)
	{
	  String_var i = "GMD FOKUS";
	  TestStringBox_var b;
	  try
	    {
	      ifc->out_stringbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_stringbox successful" << endl;
	      if (strcmp(b->_value(), "GMD FOKUS") == 0)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_stringbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_enumbox") == 0)
	{
	  TestEnum o;
	  TestEnumBox_var b = new TestEnumBox(blue);
	  try
	    {
	      ifc->in_enumbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_enumbox successful." << endl;
	      if (o == blue)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_enumbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_enumbox") == 0)
	{
	  TestEnum i = red;
	  TestEnumBox_var b;
	  try
	    {
	      ifc->out_enumbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_enumbox successful" << endl;
	      if (b->_value() == red)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_enumbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_arraybox") == 0)
	{
	  TestArray o, a;
	  a [0][1] = 687;
	  a [0][2] = 435;
	  TestArrayBox_var b = new TestArrayBox(a);
	  try
	    {
	      ifc->in_arraybox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_arraybox successful." << endl;
	      if (o[0][1] == 687 && o[0][2] == 435)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_arraybox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_arraybox") == 0)
	{
	  TestArray i;
	  i [0][1] = 111;
	  i [0][2] = 222;
	  TestArrayBox_var b;
	  try
	    {
	      ifc->out_arraybox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_arraybox successful" << endl;
	      TestArray_slice a;
	      a = *(b->_value());
	      if (a[1] == 111 && a[2] == 222)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_arraybox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_fstructbox") == 0)
	{
	  TestFixStruct o, a;
	  a.x = 357;
	  a.y = 950;
	  a.radius = 0.05;
	  TestFixStructBox_var b = new TestFixStructBox(a);
	  try
	    {
	      ifc->in_fstructbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_fstructbox successful." << endl;
	      if (o.x==357 && o.y==950 && o.radius==0.05)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_fstructbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_fstructbox") == 0)
	{
	  TestFixStruct i, a;
	  i.x = 357;
	  i.y = 950;
	  i.radius = 0.05;
	  TestFixStructBox_var b;
	  try
	    {
	      ifc->out_fstructbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_fstructbox successful" << endl;
	      a = b->_value();
	      if (a.x==357 && a.y==950 && a.radius==0.05)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_fstructbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_vstructbox") == 0)
	{
	  TestVarStruct* o;
	  TestVarStructBox_var b = new TestVarStructBox();
	  b->name("joe");
	  b->email("joe@yahoo.com");
	  try
	    {
	      ifc->in_vstructbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_vstructbox successful." << endl;
	      if (strcmp(o->name,"joe")==0 &&
		  strcmp(o->email,"joe@yahoo.com")==0)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_vstructbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_vstructbox") == 0)
	{
	  TestVarStruct i, a;
	  i.name = "joe";
	  i.email = "joe@yahoo.com";
	  TestVarStructBox_var b;
	  try
	    {
	      ifc->out_vstructbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_vstructbox successful" << endl;
	      a = b->_value();
	      if (strcmp(a.name,"joe")==0 &&
		  strcmp(a.email,"joe@yahoo.com")==0)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_vstructbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_funionbox") == 0)
	{
	  TestFixUnion o, a;
	  a.o('o');
	  a._d(true);
	  TestFixUnionBox_var b = new TestFixUnionBox(a);
	  try
	    {
	      ifc->in_funionbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_funionbox successful." << endl;
	      if (o._d()==true && o.o()=='o')
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_funionbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_funionbox") == 0)
	{
	  TestFixUnion i, a;
	  i.d(0.0309);
	  i._d(false);
	  TestFixUnionBox_var b;
	  try
	    {
	      ifc->out_funionbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_funionbox successful" << endl;
	      a = b->_value();
	      if (a._d()==false && (short)fabs(a.d()*100)==3)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_funionbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_vunionbox") == 0)
	{
	  TestVarUnion_var o, a;
	  a = new TestVarUnion();
	  a->s("tip");
	  a->_d(0);
	  TestVarUnionBox_var b = new TestVarUnionBox(*a);
	  try
	    {
	      ifc->in_vunionbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_vunionbox successful." << endl;
	      if (o->_d()==0 && strcmp(o->s(),"tip")==0)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_vunionbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_vunionbox") == 0)
	{
	  TestVarUnion_var i;
	  TestFixStruct a;
	  a.x = 357;
	  a.y = 950;
	  i = new TestVarUnion();
	  i->fs(a);
	  i->_d(9);
	  TestVarUnionBox_var b;
	  try
	    {
	      ifc->out_vunionbox(*i, b);
	      if(VERBOSITY>0)
		cout<<"out_vunionbox successful" << endl;
	      TestVarUnion i1 = b->_value();
	      if (i1._d()==9 && i1.fs().x==357 && i1.fs().y==950)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_vunionbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_shseqbox") == 0)
	{
	  TestShSeq_var o;
	  TestShSeq a;
	  a.length(2);
	  a[0] = 88;
	  a[1] = 99;
	  TestShSeqBox_var b = new TestShSeqBox(a);
	  try
	    {
	      ifc->in_shseqbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_shseqbox successful." << endl;
	      if (o->length() == 2)
		if (o[0]==88 && o[1]==99)
		  rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_shseqbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_shseqbox") == 0)
	{
	  TestShSeq i, a;
	  i.length(2);
	  i[0] = 88;
	  i[1] = 99;
	  TestShSeqBox_var b;
	  try
	    {
	      ifc->out_shseqbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_shseqbox successful" << endl;
	      a = b->_value();
	      if (a[0]==88 && a[1]==99)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_shseqbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_strseqbox") == 0)
	{
	  TestStrSeq_var o;
	  TestStrSeq a;
	  a.length(3);
	  a[0] = "Berlin";
	  a[1] = "10589";
	  a[2] = "34637000";
	  TestStrSeqBox_var b = new TestStrSeqBox(a);
	  try
	    {
	      ifc->in_strseqbox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_strseqbox successful." << endl;
	      if (o->length() == 3)
		if (strcmp(o[0],"Berlin") == 0)
		  rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_strseqbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_strseqbox") == 0)
	{
	  TestStrSeq i, a;
	  TestStrSeqBox_var b;
	  i.length(0);

	  try
	    {
	      ifc->out_strseqbox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_strseqbox successful" << endl;
	      a = b->_value();
	      if (a.length() == 0)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_strseqbox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "in_anybox") == 0)
	{
	  Any_var o, a;
	  *a <<= (Short) 911;
	  TestAnyBox_var b = new TestAnyBox(a);
	  try
	    {
	      ifc->in_anybox(b, o);
	      if(VERBOSITY>0)
		cout<<"in_anybox successful." << endl;
	      Short a1;
	      *o >>= a1;
	      if (a1 == 911)
		rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"in_anybox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "out_anybox") == 0)
	{
	  Any_var i;
	  TestAnyBox_var b;
	  *i <<= "";
	  try
	    {
	      ifc->out_anybox(i, b);
	      if(VERBOSITY>0)
		cout<<"out_anybox successful" << endl;
	      const char* a;
	      (b->_value()) >>= a;
	      if (strcmp(a, "")== 0)
		rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_anybox failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "val_in_struct") == 0)
	{
	  TestValueA_init* af = new TestValueAFactory_impl();
	  TestValueA_var v = af -> create("A TestValueA instance.");
	  v->name("john");
	  v->age(30);
	  Long n = 105;
	  TestValMbStruct_var s;
	  try
	    {
	      ifc->val_in_struct(v, n, s);
	      if(VERBOSITY>0)
		cout<<"val_in_struct successful." << endl;
	      if(s->nr == 105)
		if(strcmp(s->valmb->name(), "john")==0 &&
		   s->valmb->age()==30)
		   rst = IS_OK;
	    }   
	  catch (Exception &ex)
	    {
	      cout<<"val_in_struct failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "val_out_struct") == 0)
	{
	  TestValueA_init* af = new TestValueAFactory_impl();
	  TestValueA_var v = af -> create("A TestValueA instance.");
	  v->name("john");
	  v->age(30);
	  Long n = 105;
	  TestValMbStruct_var s = new TestValMbStruct();
	  s->nr = n;
	  s->valmb = v;
	  TestValueA_var a;
	  try
	    {
	      ifc->val_out_struct(s, a);
	      if(VERBOSITY>0)
		cout<<"val_out_struct successful" << endl;
	      if(strcmp(a->name(), "john")==0 &&
		 a->age()==30)
	      rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"val_out_struct failed due to exception: "
		  << ex << endl;
	    }
	}
      else if(strcmp(opid, "hello") == 0)
	{
	  try
	    {
	      String_var s;
	      ifc -> hello(s);
	      if(VERBOSITY>0)
		cout<<"hello successful: "<<s <<endl;
	      rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"out_shortbox failed due to exception: "
		  << ex <<endl;
	    }
	  catch (...)
	    {
	      cout<<"out_shortbox failed."<<endl;
	    }
	}
    }
  if(strcmp(ifid, "OBVTest::TestInterfaceA") == 0)
    {
      // Obtain TestInterfaceA
      obj = orb_ -> string_to_object("relfile:/OBVTest::TestInterfaceA.ref");
      assert(!is_nil(obj));
      TestInterfaceA_var ifa = TestInterfaceA::_narrow(obj);
      assert(!is_nil(ifa));

      if(strcmp(opid, "val_sub_intf") == 0)
	{
	  obj = orb_ -> string_to_object("relfile:/OBVTest::TestInterfaceB.ref");
	  assert(!is_nil(obj));
	  TestInterfaceB_var ifb = TestInterfaceB::_narrow(obj);
	  assert(!is_nil(ifb));
	  try
	    {
	      ifa->val_sub_intf(ifb);
	      if(VERBOSITY>0)
		cout<<"val_sub_intf executed." << endl;
	      rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"val_sub_intf failed due to exception: "
		  << ex << endl;
	    }

	}
      else if(strcmp(opid, "val_sub_val") == 0)
	{
	  TestValueD_init* df = new TestValueDFactory_impl();
	  TestValueD_var vld = df -> create(105);
	  vld->name("john");
	  vld->age(30);
	  try
	    {
	      ifa->val_sub_val(vld);
	      if(VERBOSITY>0)
		cout<<"val_sub_val executed." << endl;
	      rst = IS_OK;
	    }
	  catch (Exception &ex)
	    {
	      cout<<"val_sub_val failed due to exception: "
		  << ex << endl;
	    }
	}
    }
  return;
}

TestValueSaver_ptr TestHelper_impl::get_value_saver()
  throw(SystemException)
{
  return TestValueSaver::_duplicate(saver_);  
}
