//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/c++/inc/Any.hh
//
// Modifications:
// $Log: Any.hh,v $
// Revision 1.1.1.1  2000/02/09 17:07:08  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.1.1.1  1997/07/21 12:26:50  stephen
// Initial vsOrb code base
//
//

class Any {

public:

    Any(VSORB_ENV2);
    Any( const Any& VSORB_ENV);
    Any( TypeCode_ptr tc, void *value, Boolean release = FALSE VSORB_ENV);
    ~Any(VSORB_ENV2);
    
    Any &operator=( const Any&);
    void operator<<=( Short);
    void operator<<=( UShort);
    void operator<<=( Long);
    void operator<<=( ULong);
    void operator<<=( Float);
    void operator<<=( Double);
    void operator<<=( const Any&);
    void operator<<=( const char*);
    Boolean operator>>=( Short&) const;
    Boolean operator>>=( UShort&) const;
    Boolean operator>>=( Long&) const;
    Boolean operator>>=( ULong&) const;
    Boolean operator>>=( Float&) const;
    Boolean operator>>=( Double&) const;
    Boolean operator>>=( Any&) const;
    Boolean operator>>=( char*&) const;
    
    // special types needed for boolean, octet, char,
    // and bounded string insertion

    struct from_boolean {
	from_boolean(Boolean b);
	Boolean val;
    };

    struct from_octet {
	from_octet(Octet o);
	Octet val;
    };

    struct from_char {
	from_char(Char c);
	Char val;
    };

    struct from_string { 
	from_string(char* s, ULong b);
	char *val;
	ULong bound;
    };

    void operator<<=( from_boolean);
    void operator<<=( from_char);
    void operator<<=( from_octet);
    void operator<<=( from_string);

    // special types needed for boolean, octet, char extraction
    // these are suggested implementations only
    struct to_boolean {
	to_boolean(Boolean &b);
	Boolean &ref;
    };

    struct to_char {
	to_char(Char &c);
	Char &ref;
    };

    struct to_octet {
	to_octet(Octet &o);
	Octet &ref;
    };

    struct to_object {
	to_object(Object_ptr &obj);
	Object_ptr &ref;
    };

    struct to_string {
	to_string(char *& s, ULong b);
	char *& val;
	ULong bound;
    };

    Boolean operator>>=( to_boolean) const;
    Boolean operator>>=( to_char) const;
    Boolean operator>>=( to_octet) const;
    Boolean operator>>=( to_object) const;
    Boolean operator>>=( to_string) const;
    
    void replace( TypeCode_ptr, void *value,
		 Boolean release = FALSE VSORB_ENV);

    TypeCode_ptr type(VSORB_ENV2) const;
    const void *value(VSORB_ENV2) const;

    private:
    // these are hidden and should not be implemented
    // so as to catch erroneous attempts to insert or extract
    // multiple IDL types mapped to unsigned char
    void operator<<=( unsigned char);
    Boolean operator>>=( unsigned char&) const;
};

//
// Any_var Class
//

class Any_var {

public:

    Any_var(VSORB_ENV2);
    Any_var( Any *a);
    Any_var( const Any_var &a);
    ~Any_var(VSORB_ENV2);
    Any_var &operator=( Any *a);
    Any_var &operator=( const Any_var &a);
    Any *operator->();

    // other conversion operators for parameter passing
    void operator<<=( Short);
    void operator<<=( UShort);
    void operator<<=( Long);
    void operator<<=( ULong);
    void operator<<=( Float);
    void operator<<=( Double);
    void operator<<=( const Any&);
    void operator<<=( const char*);

    void operator<<=( Any::from_boolean);
    void operator<<=( Any::from_char);
    void operator<<=( Any::from_octet);
    void operator<<=( Any::from_string);

    Boolean operator>>=( Short&) const;
    Boolean operator>>=( UShort&) const;
    Boolean operator>>=( Long&) const;
    Boolean operator>>=( ULong&) const;
    Boolean operator>>=( Float&) const;
    Boolean operator>>=( Double&) const;
    Boolean operator>>=( Any&) const;
    Boolean operator>>=( char*&) const;

    Boolean operator>>=( Any::to_boolean) const;
    Boolean operator>>=( Any::to_char) const;
    Boolean operator>>=( Any::to_octet) const;
    Boolean operator>>=( Any::to_object) const;
    Boolean operator>>=( Any::to_string) const;
};
