//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/c++/inc/Exceptions.hh
//
// Modifications:
// $Log: Exceptions.hh,v $
// Revision 1.2  2000/05/24 13:41:33  corval2
// #undef minor before using it since some operating systems have this defined
// as a macro
//
// Revision 1.1.1.1  2000/02/09 17:07:08  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.2  1997/08/20 12:37:13  james
// _narrow method added to exceptions
//
//

// This avoids name clashes with a minor macro already defined under Linux
#undef minor

//
// Base class for exceptions
//

class Exception
{

public:

    Exception (const Exception&);
    ~Exception ();
    Exception& operator= (const Exception&);
    
protected:

    Exception ();
};

//
// Status and Type information
//

enum CompletionStatus { COMPLETED_YES, COMPLETED_NO, COMPLETED_MAYBE };
enum exception_type {NO_EXCEPTION, USER_EXCEPTION, SYSTEM_EXCEPTION};


//
// User exceptions
//

class UserException : public Exception
{ 

public:

    UserException();
    UserException( const UserException &);
    ~UserException();
    UserException &operator=( const UserException &);

		static UserException *_narrow(Exception *);
};


//
// Unknown User exceptions
//

class UnknownUserException : public UserException
{

public:

    Any &exception();

		static UnknownUserException *_narrow(Exception *);
};


//
// System exception class
//

class SystemException : public Exception
{

public:
    SystemException();
    SystemException(const SystemException &);
    SystemException(ULong minor, CompletionStatus status);
    ~SystemException();

    SystemException &operator=(const SystemException &);

    ULong minor() const;
    void minor(ULong);
    
    CompletionStatus completed() const;
    void completed(CompletionStatus);

		static SystemException *_narrow(Exception *);
};

class UNKNOWN : public virtual SystemException 
{ 
public:
    UNKNOWN();
    UNKNOWN(const SystemException &);
    UNKNOWN(ULong minor, CompletionStatus status);
		static UNKNOWN *_narrow(Exception *);
};

class BAD_PARAM : public virtual SystemException 
{ 
public:
    BAD_PARAM();
    BAD_PARAM(const SystemException &);
    BAD_PARAM(ULong minor, CompletionStatus status);
		static BAD_PARAM *_narrow(Exception *);
};

class NO_MEMORY : public virtual SystemException 
{ 
public:
    NO_MEMORY();
    NO_MEMORY(const SystemException &);
    NO_MEMORY(ULong minor, CompletionStatus status);
		static NO_MEMORY *_narrow(Exception *);
};

class IMP_LIMIT : public virtual SystemException 
{ 
public:
    IMP_LIMIT();
    IMP_LIMIT(const SystemException &);
    IMP_LIMIT(ULong minor, CompletionStatus status);
		static IMP_LIMIT *_narrow(Exception *);
};

class COMM_FAILURE : public virtual SystemException 
{ 
public:
   COMM_FAILURE();
    COMM_FAILURE(const SystemException &);
    COMM_FAILURE(ULong minor, CompletionStatus status);
		static COMM_FAILURE *_narrow(Exception *);
};

class INV_OBJREF : public virtual SystemException 
{ 
public:
    INV_OBJREF();
    INV_OBJREF(const SystemException &);
    INV_OBJREF(ULong minor, CompletionStatus status);
		static INV_OBJREF *_narrow(Exception *);
};

class NO_PERMISSION : public virtual SystemException 
{ 
public:
    NO_PERMISSION();
    NO_PERMISSION(const SystemException &);
    NO_PERMISSION(ULong minor, CompletionStatus status);
		static NO_PERMISSION *_narrow(Exception *);
};

class INTERNAL : public virtual SystemException 
{ 
public:
    INTERNAL();
    INTERNAL(const SystemException &);
    INTERNAL(ULong minor, CompletionStatus status);
		static INTERNAL *_narrow(Exception *);
};

class MARSHAL : public virtual SystemException 
{ 
public:
    MARSHAL();
    MARSHAL(const SystemException &);
    MARSHAL(ULong minor, CompletionStatus status);
		static MARSHAL *_narrow(Exception *);
};

class INITIALIZE : public virtual SystemException 
{ 
public:
    INITIALIZE();
    INITIALIZE(const SystemException &);
    INITIALIZE(ULong minor, CompletionStatus status);
		static INITIALIZE *_narrow(Exception *);
};

class NO_IMPLEMENT : public virtual SystemException 
{ 
public:
    NO_IMPLEMENT();
    NO_IMPLEMENT(const SystemException &);
    NO_IMPLEMENT(ULong minor, CompletionStatus status);
		static NO_IMPLEMENT *_narrow(Exception *);
};

class BAD_TYPECODE : public virtual SystemException 
{ 
public:
    BAD_TYPECODE();
    BAD_TYPECODE(const SystemException &);
    BAD_TYPECODE(ULong minor, CompletionStatus status);
		static BAD_TYPECODE *_narrow(Exception *);
};

class BAD_OPERATION : public virtual SystemException 
{ 
public:
    BAD_OPERATION();
    BAD_OPERATION(const SystemException &);
    BAD_OPERATION(ULong minor, CompletionStatus status);
		static BAD_OPERATION *_narrow(Exception *);
};

class NO_RESOURCES : public virtual SystemException 
{ 
public:
    NO_RESOURCES();
    NO_RESOURCES(const SystemException &);
    NO_RESOURCES(ULong minor, CompletionStatus status);
		static NO_RESOURCES *_narrow(Exception *);
};

class NO_RESPONSE : public virtual SystemException 
{ 
public:
    NO_RESPONSE();
    NO_RESPONSE(const SystemException &);
    NO_RESPONSE(ULong minor, CompletionStatus status);
		static NO_RESPONSE *_narrow(Exception *);
};

class PERSIST_STORE : public virtual SystemException 
{ 
public:
    PERSIST_STORE();
    PERSIST_STORE(const SystemException &);
    PERSIST_STORE(ULong minor, CompletionStatus status);
		static PERSIST_STORE *_narrow(Exception *);
};

class BAD_INV_ORDER : public virtual SystemException 
{ 
public:
    BAD_INV_ORDER();
    BAD_INV_ORDER(const SystemException &);
    BAD_INV_ORDER(ULong minor, CompletionStatus status);
		static BAD_INV_ORDER *_narrow(Exception *);
};

class TRANSIENT : public virtual SystemException 
{ 
public:
    TRANSIENT();
    TRANSIENT(const SystemException &);
    TRANSIENT(ULong minor, CompletionStatus status);
		static TRANSIENT *_narrow(Exception *);
};

class FREE_MEM : public virtual SystemException 
{ 
public:
    FREE_MEM();
    FREE_MEM(const SystemException &);
    FREE_MEM(ULong minor, CompletionStatus status);
		static FREE_MEM *_narrow(Exception *);
};

class INV_IDENT : public virtual SystemException 
{ 
public:
    INV_IDENT();
    INV_IDENT(const SystemException &);
    INV_IDENT(ULong minor, CompletionStatus status);
		static INV_IDENT *_narrow(Exception *);
};

class INV_FLAG : public virtual SystemException 
{ 
public:
    INV_FLAG();
    INV_FLAG(const SystemException &);
    INV_FLAG(ULong minor, CompletionStatus status);
		static INV_FLAG *_narrow(Exception *);
};

class INTF_REPOS : public virtual SystemException 
{ 
public:
    INTF_REPOS();
    INTF_REPOS(const SystemException &);
    INTF_REPOS(ULong minor, CompletionStatus status);
		static INTF_REPOS *_narrow(Exception *);
};

class BAD_CONTEXT : public virtual SystemException 
{ 
public:
    BAD_CONTEXT();
    BAD_CONTEXT(const SystemException &);
    BAD_CONTEXT(ULong minor, CompletionStatus status);
		static BAD_CONTEXT *_narrow(Exception *);
};

class OBJ_ADAPTER : public virtual SystemException 
{ 
public:
    OBJ_ADAPTER();
    OBJ_ADAPTER(const SystemException &);
    OBJ_ADAPTER(ULong minor, CompletionStatus status);
		static OBJ_ADAPTER *_narrow(Exception *);
};

class DATA_CONVERSION : public virtual SystemException 
{ 
public:
    DATA_CONVERSION();
    DATA_CONVERSION(const SystemException &);
    DATA_CONVERSION(ULong minor, CompletionStatus status);
		static DATA_CONVERSION *_narrow(Exception *);
};

class OBJECT_NOT_EXIST : public virtual SystemException 
{ 
public:
    OBJECT_NOT_EXIST();
    OBJECT_NOT_EXIST(const SystemException &);
    OBJECT_NOT_EXIST(ULong minor, CompletionStatus status);
		static OBJECT_NOT_EXIST *_narrow(Exception *);
};

