//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/c++/inc/IR.hh
//
// Modifications:
// $Log: IR.hh,v $
// Revision 1.3  2000/10/19 18:41:04  uwes
// Fixed error reported on HP_UX
//
// Revision 1.2  2000/02/11 15:57:17  mli
// T_sequence -> CORBA::T_sequence
// See the change in vsorbref.hh
//
// Revision 1.1.1.1  1997/07/21 12:26:51  stephen
// Initial vsOrb code base
//
//


typedef String_var Identifier_var;		// see Issues.
typedef String_var ScopedName_var;		// see Issues.
typedef String_var RepositoryId_var;		// see Issues.
typedef String_var VersionSpec_var;		// see Issues.


typedef char *ScopedName;
typedef char *VersionSpec;

enum DefinitionKind {
    dk_none,
    dk_all,
    dk_Attribute,
    dk_Constant,
    dk_Exception,
    dk_Interface,
    dk_Module,
    dk_Operation,
    dk_Typedef,
    dk_Alias,
    dk_Struct,
    dk_Union,
    dk_Enum,
    dk_Primitive,
    dk_String,
    dk_Sequence,
    dk_Array,
    dk_Repository
    };

class IRObject;

typedef IRObject * IRObject_ptr;

class IRObject :public virtual Object{
    public:
    static IRObject_ptr _duplicate(IRObject_ptr obj);
    static IRObject_ptr _narrow(Object_ptr obj);
    static IRObject_ptr _nil();

    DefinitionKind def_kind(VSORB_ENV2);
    void destroy(VSORB_ENV2);
};

typedef T_var<IRObject> IRObject_var;

class Contained;
class Repository;
class Container;

typedef Contained * Contained_ptr;
typedef T_var<Contained> Contained_var;
typedef Repository * Repository_ptr;
typedef T_var<Repository> Repository_var;
typedef Container * Container_ptr;
typedef T_var<Container> Container_var;

class Contained : public virtual IRObject {
    public:
    static Contained_ptr _duplicate(Contained_ptr obj);
    static Contained_ptr _narrow(Object_ptr obj);
    static Contained_ptr _nil();

    RepositoryId id(VSORB_ENV2);
    void id(const char * VSORB_ENV);
    Identifier name(VSORB_ENV2);
    void name(const char * VSORB_ENV);
    VersionSpec version(VSORB_ENV2);
    void version(const char * VSORB_ENV);

    Container_ptr defined_in(VSORB_ENV2);
    ScopedName absolute_name(VSORB_ENV2);
    Repository_ptr containing_repository(VSORB_ENV2);

    struct Description
    {
	DefinitionKind kind;
	Any value;
    };
    typedef T_var<Description> Description_var;

    Description* describe(VSORB_ENV2);

    void move(
	Container_ptr new_container,
	const char *new_name,
	const char *new_version VSORB_ENV);
};

class ModuleDef;
class ConstantDef;
class IDLType;
class StructDef;
class UnionDef;
class EnumDef;
class AliasDef;
class ExceptionDef;
//class OperationDef;
//class InterfaceDef;

typedef ModuleDef * ModuleDef_ptr;
typedef ConstantDef * ConstantDef_ptr;
typedef IDLType * IDLType_ptr;
typedef StructDef * StructDef_ptr;
typedef UnionDef * UnionDef_ptr;
typedef EnumDef * EnumDef_ptr;
typedef AliasDef * AliasDef_ptr;
typedef ExceptionDef * ExceptionDef_ptr;

typedef T_var<ModuleDef> ModuleDef_var;
typedef T_var<ConstantDef> ConstantDef_var;
typedef T_var<IDLType> IDLType_var;
typedef T_var<StructDef> StructDef_var;
typedef T_var<UnionDef> UnionDef_var;
typedef T_var<EnumDef> EnumDef_var;
typedef T_var<AliasDef> AliasDef_var;
typedef T_var<ExceptionDef> ExceptionDef_var;
typedef T_var<OperationDef> OperationDef_var;
typedef T_var<InterfaceDef> InterfaceDef_var;

typedef T_var<TypeCode> TypeCode_var;

typedef T_sequence<InterfaceDef> InterfaceDefSeq;
typedef T_sequence<Contained> ContainedSeq;

typedef ContainedSeq * ContainedSeq_ptr;

struct StructMember {
    Identifier_var name;
    TypeCode_var type;
    IDLType_var type_def;
};

typedef T_var<StructMember> StructMember_var;

typedef T_sequence<StructMember> StructMemberSeq;

struct UnionMember {
    Identifier_var name;
    Any label;
    TypeCode_var type;
    IDLType_var type_def;
};
typedef T_var<UnionMember> UnionMember_var;

typedef T_sequence<UnionMember> UnionMemberSeq;
typedef T_sequence<Identifier> EnumMemberSeq;


class Container : public virtual IRObject {
    public:
    static Container_ptr _duplicate(Container_ptr obj);
    static Container_ptr _narrow(Object_ptr obj);
    static Container_ptr _nil();

    Contained_ptr lookup(const char * search_name VSORB_ENV);

    ContainedSeq_ptr contents(DefinitionKind limit_type,
			      Boolean exclude_inherited VSORB_ENV);

    ContainedSeq_ptr lookup_name(const char * search_name,
				 Long levels_to_search,
				 DefinitionKind limit_type,
				 Boolean exclude_inherited VSORB_ENV);

    struct Description {
	Contained_var contained_object;
	DefinitionKind kind;
	Any value;
    };

    typedef CORBA::T_sequence<Description> DescriptionSeq;

    typedef T_var<Description> Description_var;

    typedef DescriptionSeq * DescriptionSeq_ptr;

    DescriptionSeq_ptr describe_contents(DefinitionKind limit_type,
					 Boolean exclude_inherited,
					 Long max_returned_objs VSORB_ENV);

    ModuleDef_ptr create_module(const char * id,
				const char * name,
				const char * version VSORB_ENV);

    ConstantDef_ptr create_constant(const char * id,
				    const char * name,
				    const char * version,
				    const IDLType_ptr type,
				    const Any& value VSORB_ENV);

    StructDef_ptr create_struct(const char * id,
				const char * name,
				const char * version,
				const StructMemberSeq& members VSORB_ENV);

    UnionDef_ptr create_union(const char * id,
			      const char * name,
			      const char * version,
			      const IDLType_ptr discriminator_type,
			      const UnionMemberSeq& members VSORB_ENV);

    EnumDef_ptr create_enum(const char * id,
			    const char * name,
			    const char * version,
			    const EnumMemberSeq& members VSORB_ENV);

    AliasDef_ptr create_alias(const char * id,
			      const char * name,
			      const char * version,
			      const IDLType_ptr original_type VSORB_ENV);

    InterfaceDef_ptr create_interface(const char * id,
				      const char * name,
				      const char * version,
				      const InterfaceDefSeq& base_interfaces VSORB_ENV);

    ExceptionDef_ptr create_exception(const char * id,
				      const char * name,
				      const char * version,
				      const StructMemberSeq& members VSORB_ENV);
};

class IDLType : public virtual IRObject {
    public:
    static IDLType_ptr _duplicate(IDLType_ptr obj);
    static IDLType_ptr _narrow(Object_ptr obj);
    static IDLType_ptr _nil();

    TypeCode_ptr type(VSORB_ENV2);
};

class PrimitiveDef;
class StringDef;
class SequenceDef;
class ArrayDef;

typedef PrimitiveDef * PrimitiveDef_ptr;
typedef T_var<PrimitiveDef> PrimitiveDef_var;
typedef StringDef * StringDef_ptr;
typedef T_var<StringDef> StringDef_var;
typedef SequenceDef * SequenceDef_ptr;
typedef T_var<SequenceDef> SequenceDef_var;
typedef ArrayDef * ArrayDef_ptr;
typedef T_var<ArrayDef> ArrayDef_var;

enum PrimitiveKind {
    pk_null,
    pk_void,
    pk_short,
    pk_long,
    pk_ushort,
    pk_ulong,
    pk_float,
    pk_double,
    pk_boolean,
    pk_char,
    pk_octet,
    pk_any,
    pk_TypeCode,
    pk_Principal,
    pk_string,
    pk_objref
    };

class Repository : public virtual Container {
    public:
    static Repository_ptr _duplicate(Repository_ptr obj);
    static Repository_ptr _narrow(Object_ptr obj);
    static Repository_ptr _nil();

    Contained_ptr lookup_id(const char * search_id VSORB_ENV);

    PrimitiveDef_ptr get_primitive(PrimitiveKind kind VSORB_ENV);

    StringDef_ptr create_string(ULong bound VSORB_ENV);

    SequenceDef_ptr create_sequence(ULong bound,
				    const IDLType_ptr element_type VSORB_ENV);

    ArrayDef_ptr create_array(ULong length,
			      IDLType_ptr element_type VSORB_ENV);
};

class ModuleDef : public virtual Container, public virtual Contained {
    public:
    static ModuleDef_ptr _duplicate(ModuleDef_ptr obj);
    static ModuleDef_ptr _narrow(Object_ptr obj);
    static ModuleDef_ptr _nil();
};

struct ModuleDescription {
    Identifier_var name;
    RepositoryId_var id;
    RepositoryId_var defined_in;
    VersionSpec_var version;
};

typedef T_var<ModuleDescription> ModuleDescription_var;

class ConstantDef : public virtual Contained {
    public:
    static ConstantDef_ptr _duplicate(ConstantDef_ptr obj);
    static ConstantDef_ptr _narrow(Object_ptr obj);
    static ConstantDef_ptr _nil();

    TypeCode_ptr type(VSORB_ENV2);
    IDLType_ptr type_def(VSORB_ENV2);
    void type_def(IDLType_ptr VSORB_ENV);
    Any* value(VSORB_ENV2);
    void value(const Any& VSORB_ENV);
};

struct ConstantDescription {
    Identifier_var name;
    RepositoryId_var id;
    RepositoryId_var defined_in;
    VersionSpec_var version;
    TypeCode_var type;
    Any value;
};
typedef T_var<ConstantDescription> ConstantDescription_var;

class TypedefDef;
typedef TypedefDef * TypedefDef_ptr;
class TypedefDef : public virtual Contained, public virtual IDLType {
    public:
    static TypedefDef_ptr _duplicate(TypedefDef_ptr obj);
    static TypedefDef_ptr _narrow(Object_ptr obj);
    static TypedefDef_ptr _nil();
};

typedef T_var<TypedefDef> TypedefDef_var;

struct TypeDescription {
    Identifier_var name;
    RepositoryId_var id;
    RepositoryId_var defined_in;
    VersionSpec_var version;
    TypeCode_var type;
};
typedef T_var<TypeDescription> TypeDescription_var;

class StructDef : public virtual TypedefDef {
    public:
    static StructDef_ptr _duplicate(StructDef_ptr obj);
    static StructDef_ptr _narrow(Object_ptr obj);
    static StructDef_ptr _nil();

    StructMemberSeq* members(VSORB_ENV2);
    void members(const StructMemberSeq& VSORB_ENV);
};

class UnionDef : public virtual TypedefDef {
    public:
    static UnionDef_ptr _duplicate(UnionDef_ptr obj);
    static UnionDef_ptr _narrow(Object_ptr obj);
    static UnionDef_ptr _nil();

    TypeCode_ptr discriminator_type(VSORB_ENV2);
    IDLType_ptr discriminator_type_def(VSORB_ENV2);
    void discriminator_type_def(IDLType_ptr VSORB_ENV);
    UnionMemberSeq* members(VSORB_ENV2);
    void members(const UnionMemberSeq& VSORB_ENV);
};

class EnumDef : public virtual TypedefDef {
    public:
    static EnumDef_ptr _duplicate(EnumDef_ptr obj);
    static EnumDef_ptr _narrow(Object_ptr obj);
    static EnumDef_ptr _nil();

    EnumMemberSeq * members(VSORB_ENV2);
    void members(const EnumMemberSeq & VSORB_ENV);

};

class AliasDef : public virtual TypedefDef {
    public:
    static AliasDef_ptr _duplicate(AliasDef_ptr obj);
    static AliasDef_ptr _narrow(Object_ptr obj);
    static AliasDef_ptr _nil();

    IDLType_ptr original_type_def(VSORB_ENV2);
    void original_type_def(IDLType_ptr VSORB_ENV);
};

class PrimitiveDef : public virtual IDLType {
    public:
    static PrimitiveDef_ptr _duplicate(PrimitiveDef_ptr obj);
    static PrimitiveDef_ptr _narrow(Object_ptr obj);
    static PrimitiveDef_ptr _nil();

    PrimitiveKind kind(VSORB_ENV2);
};

class StringDef : public virtual IDLType {
    public:
    static StringDef_ptr _duplicate(StringDef_ptr obj);
    static StringDef_ptr _narrow(Object_ptr obj);
    static StringDef_ptr _nil();

    ULong bound(VSORB_ENV2);
    void bound(ULong VSORB_ENV);
};

class SequenceDef : public virtual IDLType {
    public:
    static SequenceDef_ptr _duplicate(SequenceDef_ptr obj);
    static SequenceDef_ptr _narrow(Object_ptr obj);
    static SequenceDef_ptr _nil();

    ULong bound(VSORB_ENV2);
    void bound(ULong VSORB_ENV);
    TypeCode_ptr element_type(VSORB_ENV2);
    IDLType_ptr element_type_def(VSORB_ENV2);
    void element_type_def(IDLType_ptr VSORB_ENV);
};

class ArrayDef : public virtual IDLType {
    public:
    static ArrayDef_ptr _duplicate(ArrayDef_ptr obj);
    static ArrayDef_ptr _narrow(Object_ptr obj);
    static ArrayDef_ptr _nil();

    ULong length(VSORB_ENV2);
    void length(ULong VSORB_ENV);
    TypeCode_ptr element_type(VSORB_ENV2);
    IDLType_ptr element_type_def(VSORB_ENV2);
    void element_type_def(IDLType_ptr VSORB_ENV);
};

class ExceptionDef : public virtual Contained {
    public:
    static ExceptionDef_ptr _duplicate(ExceptionDef_ptr obj);
    static ExceptionDef_ptr _narrow(Object_ptr obj);
    static ExceptionDef_ptr _nil();

    TypeCode_ptr type(VSORB_ENV2);
    StructMemberSeq * members(VSORB_ENV2);
    void members(const StructMemberSeq& VSORB_ENV);
};

struct ExceptionDescription {
    Identifier_var name;
    RepositoryId_var id;
    RepositoryId_var defined_in;
    VersionSpec_var version;
    TypeCode_var type;
};
typedef T_var<ExceptionDescription> ExceptionDescription_var;

enum AttributeMode {
    ATTR_NORMAL,
    ATTR_READONLY
    };


class AttributeDef;

typedef AttributeDef * AttributeDef_ptr;

class AttributeDef : public virtual Contained {
    public:
    static AttributeDef_ptr _duplicate(AttributeDef_ptr obj);
    static AttributeDef_ptr _narrow(Object_ptr obj);
    static AttributeDef_ptr _nil();

    TypeCode_ptr type(VSORB_ENV2);
    IDLType_ptr type_def(VSORB_ENV2);
    void type_def(IDLType_ptr VSORB_ENV);
    AttributeMode mode(VSORB_ENV2);
    void mode(AttributeMode VSORB_ENV);
};

typedef T_var<AttributeDef> AttributeDef_var;

struct AttributeDescription {
    Identifier_var name;
    RepositoryId_var id;
    RepositoryId_var defined_in;
    VersionSpec_var version;
    TypeCode_var type;
    AttributeMode mode;
};
typedef T_var<AttributeDescription> AttributeDescription_var;

enum OperationMode {
    OP_NORMAL,
    OP_ONEWAY
    };

enum ParameterMode {
    PARAM_IN,
    PARAM_OUT,
    PARAM_INOUT
    };

struct ParameterDescription {
    Identifier_var name;
    TypeCode_var type;
    IDLType_var type_def;
    ParameterMode mode;
};
typedef T_var<ParameterDescription> ParameterDescription_var;

typedef T_sequence<ParameterDescription> ParDescriptionSeq;

typedef Identifier ContextIdentifier;
typedef Identifier_var ContextIdentifier_var;
typedef T_sequence<ContextIdentifier> ContextIdSeq;

typedef T_sequence<ExceptionDef> ExceptionDefSeq;
typedef T_sequence<ExceptionDescription> ExcDescriptionSeq;

class OperationDef : public virtual Contained {
    public:
    static OperationDef_ptr _duplicate(OperationDef_ptr obj);
    static OperationDef_ptr _narrow(Object_ptr obj);
    static OperationDef_ptr _nil();

    TypeCode_ptr result(VSORB_ENV2);
    IDLType_ptr result_def(VSORB_ENV2);
    void result_def(IDLType_ptr VSORB_ENV);
    ParDescriptionSeq* params(VSORB_ENV2);
    void params(const ParDescriptionSeq& VSORB_ENV);
    OperationMode mode(VSORB_ENV2);
    void mode(OperationMode VSORB_ENV);
    ContextIdSeq* contexts(VSORB_ENV2);
    void contexts(const ContextIdSeq& VSORB_ENV);
    ExceptionDefSeq* exceptions(VSORB_ENV2);
    void exceptions(const ExceptionDefSeq& VSORB_ENV);
};

struct OperationDescription {
    Identifier_var name;
    RepositoryId_var id;
    RepositoryId_var defined_in;
    VersionSpec_var version;
    TypeCode_var result;
    OperationMode mode;
    ContextIdSeq contexts;
    ParDescriptionSeq parameters;
    ExcDescriptionSeq exceptions;
};
typedef T_var<OperationDescription> OperationDescription_var;

typedef T_sequence<RepositoryId> RepositoryIdSeq;
typedef T_sequence<OperationDescription> OpDescriptionSeq;
typedef T_sequence<AttributeDescription> AttrDescriptionSeq;

class InterfaceDef : public virtual Container, public virtual Contained, public
virtual IDLType {
    public:
    static InterfaceDef_ptr _duplicate(InterfaceDef_ptr obj);
    static InterfaceDef_ptr _narrow(Object_ptr obj);
    static InterfaceDef_ptr _nil();

    InterfaceDefSeq* base_interfaces(VSORB_ENV2);
    void base_interfaces(const InterfaceDefSeq & VSORB_ENV);

    Boolean is_a(const char * interface_id VSORB_ENV);

    struct FullInterfaceDescription {
	Identifier_var name;
	RepositoryId_var id;
	RepositoryId_var defined_in;
	VersionSpec_var version;
	OpDescriptionSeq operations;
	AttrDescriptionSeq attributes;
	RepositoryIdSeq base_interfaces;
	TypeCode_var type;
    };
    typedef T_var<FullInterfaceDescription> FullInterfaceDescription_var;

    FullInterfaceDescription* describe_interface(VSORB_ENV2);

    AttributeDef_ptr create_attribute(const char * id,
				      const char * name,
				      const char * version,
				      const IDLType_ptr type,
				      AttributeMode mode VSORB_ENV);

    OperationDef_ptr create_operation(const char * id,
				      const char * name,
				      const char * version,
				      const IDLType_ptr result,
				      OperationMode mode,
				      const ParDescriptionSeq& params,
				      const ExceptionDefSeq& exceptions,
				      const ContextIdSeq& contexts VSORB_ENV);
};

struct InterfaceDescription {
    Identifier_var name;
    RepositoryId_var id;
    RepositoryId_var defined_in;
    VersionSpec_var version;
    RepositoryIdSeq base_interfaces;
};
typedef T_var<InterfaceDescription> InterfaceDescription_var;


