//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/iiop/inc/transport.hh
//
// Modifications:
// $Log: transport.hh,v $
// Revision 1.4  2000/11/13 14:51:21  uwes
// unistd.h already includes sys/select.h
//
// Revision 1.3  2000/09/05 14:46:07  corval2
// Fixed includes for CC 5.1
//
// Revision 1.2  2000/05/24 09:44:49  mli
// remove 'extern "C" int gethostname'
//
// Revision 1.1.1.1  2000/02/09 17:07:11  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.14  1999/07/27 11:12:21  cwatson
// Typedef provided for socket type.
//
// Revision 1.13  1999/04/26 14:57:49  james
// Changed CORBA::Char* to char*
//
// Revision 1.12  1998/06/04 16:11:05  stephen
// Mismatched bracket incorrectly checked-in.
//
// Revision 1.11  1998/06/04 13:52:11  stephen
// Added cleanUp function to save FD resources.
//
// Revision 1.10  1997/12/12 15:09:10  stephen
// Signed/Unsigned mismatch in prototype of transmit/receive.
//
// Revision 1.9  1997/11/01 15:30:56  ian
// Signed/Unsigned mismatch in func return.
//
// Revision 1.8  1997/10/30 10:49:12  ian
// Removed unneeded macro.
//
// Revision 1.7  1997/10/28 15:25:29  stephen
// Added timeout to processEvent.
//
// Revision 1.6  1997/10/22 13:49:11  stephen
// *** empty log message ***
//
// Revision 1.5  1997/10/02 18:53:20  stephen
// Synchronisation point.
//
// Revision 1.4  1997/09/11 14:51:06  stephen
// Event loop modelled correctly.
//
// Revision 1.3  1997/09/08 13:02:00  damien
// win32 changes
//
// Revision 1.1  1997/09/03 09:36:40  stephen
// Intial TCP/IP support class.
//
//

#ifndef VSORB_TRANSPORT_HH
#define VSORB_TRANSPORT_HH

#include <iostream.h>

#ifndef WIN32

#include <stdio.h>

#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>
#include <string.h>
#include <sys/socket.h>
#include <errno.h>
#else
#include <winsock.h>
#endif

#include <vsorb.hh>

#ifndef WIN32
typedef CORBA::ULong vsSocket;
#else
typedef SOCKET vsSocket;
#endif

class Transport
{

protected:

    // All active file descriptors
    fd_set m_FdSet;
    CORBA::ULong m_fdCount;
    
    // Our host and port information
    CORBA::UShort m_Port;
    char* m_IPAddress;

    CORBA::Boolean m_Connected;

    // Cache our listening FD
    vsSocket m_ListenFd;

    // Connected client and server
    vsSocket m_ClientFd;
    vsSocket m_ServerFd;
    
    Transport();
    
private:

    static Transport* m_Instance;
    
public:

    ~Transport();
    
    static Transport* Instance();
    CORBA::Boolean initialise();

    // Opens new connection
    CORBA::Long connectTo(char *host, CORBA::UShort port);

    // send data on socket
    int transmit(CORBA::ULong fd,
			  unsigned char* data,
			  CORBA::ULong length);

    // reads data from socket
    int receive(CORBA::ULong fd,
		unsigned char*  data,
		CORBA::ULong length,
		CORBA::ULong flags = 0);

    // any activity on connected sockets?
    CORBA::Boolean waitOnClient();
    CORBA::Boolean isEventPending();
    CORBA::Long processEvent(CORBA::ULong timeout);
    
    // Utility routines
    char * IPAddress() {
	return m_IPAddress;
    };

    CORBA::UShort listeningPort() {
	return m_Port;
    };
    
    CORBA::Boolean AlreadyConnected() {
	return m_Connected;
    };

    void AlreadyConnected(CORBA::Boolean b) {
	m_Connected = b;
    };

    CORBA::ULong serverFd() {
	return m_ServerFd;
    };
    
    CORBA::ULong clientFd() {
	return m_ClientFd;
    };

    void cleanUp() {
#ifdef WIN32
        closesocket(m_ServerFd);
#else
        close(m_ServerFd);
#endif
        FD_CLR(m_ServerFd, &m_FdSet);
    };
    
};
#endif
