//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/iiop/src/decode.cpp
//
// Modifications:
// $Log: decode.cpp,v $
// Revision 1.2  2000/02/11 16:04:02  mli
// Return type of some decode* methods from
// 'unsigned long ' changed to 'long'.
//
// Revision 1.1.1.1  2000/02/09 17:07:11  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.21  1999/07/05 15:16:18  stephen
// Changed SR105 fix for memory problems.
//
// Revision 1.20  1999/04/30 12:33:11  ian
// SR105 Changed memory deallocation
//
// Revision 1.19  1998/09/04 15:40:04  james
// mode and getSequenceLength ernie functions called
//
// Revision 1.18  1998/08/28 14:51:53  james
// IsEquivalent enhanced
//
// Revision 1.17  1998/06/16 09:24:27  stephen
// Indirected aliases support added.
//
// Revision 1.16  1998/06/04 14:04:40  stephen
// Typecode debugging information improved.
//
// Revision 1.15  1998/05/11 10:16:10  stephen
// Improved diagnostic information in error cases.
//
// Revision 1.14  1997/12/11 15:12:11  james
// Fixed Typecode indirection calculations
//
// Revision 1.13  1997/12/11 11:08:02  stephen
// Mismatch in declaration for tcIndirect buffer.
//
// Revision 1.12  1997/12/09 12:09:30  cwatson
// Changed %f to %g in FloatType::decodeIIOP()
//
// Revision 1.11  1997/12/05 16:29:05  stephen
// Sequence now bound at a max of 3.
//
// Revision 1.10  1997/12/04 14:26:11  stephen
// TypeCode indirection support as well as type equivialence checks.
//
// Revision 1.8  1997/11/21 13:19:12  cwatson
// Some extra casts needed on NT
//
// Revision 1.7  1997/11/21 13:01:47  stephen
// decodeTypeCode functions to marshall Any's.
//
// Revision 1.6  1997/11/04 17:04:05  james
// Updated Union Handling
//
// Revision 1.5  1997/10/30 16:05:06  james
// Union handling modified
//
// Revision 1.4  1997/10/30 12:04:36  james
// Bad tet_printf fixed
//
// Revision 1.3  1997/10/30 10:50:38  ian
// Extra printf removed.
//
// Revision 1.2  1997/10/23 14:50:24  stephen
// Added extra debug infor under error conditions.
//
// Revision 1.1  1997/10/22 13:46:15  stephen
// IIOP decoding functions used by parse tree objects.
//


#include <iiop.hh>
#include <grammar.hh>
#include <utilities.hh>
#include <find.hh>
#include <ernie.hh>
#include <vsorb.hh>
#include <stdio.h>

char  name[1024]={'\0'};

void AddName(char *p);
void RemoveName();
void AddIndex(int i);
void RemoveIndex();
CORBA::Long processTK(CORBA::ULong, CORBA::ULong,CORBA::ULong);    
int IsEquivalent(Node *p1, Node *p2);
long processIndirection(Node * currentNode,
				CORBA::ULong curPos,
				CORBA::Long indirection);


Ernie ernie;
extern Node** tcIndirect;
extern ProfileBody *pBody;
extern ProfileBody *sBody;

long
Node::decodeIIOP(IIOPMessageHeader &msg)
{
    // Do nothing for the base class
    return 0;
}

long
Node::decodeTypeCode(IIOPMessageHeader &msg)
{
    // Do nothing for the base class
    return 0;
}

long
ChoiceNode::decodeIIOP(IIOPMessageHeader &msg)
{
    // When there's a choice let the child decide
    if (child != 0)
	return child->decodeIIOP(msg);
    else
	return 0;
}

long
ChoiceNode::decodeTypeCode(IIOPMessageHeader &msg)
{
    // When there's a choice let the child decide
    if (child != 0)
	return child->decodeTypeCode(msg);
    else
	return 0;
}

long
SequenceNode::decodeIIOP(IIOPMessageHeader &msg)
{
    // For each Id in the stored array
    // check its associated child is valid
    // and let the child handle the decoding
    for(int i = 0; childIds[i] != 0; ++i)
	if (childNodes[i] != 0)
	    if (childNodes[i]->decodeIIOP(msg) != 0)
		return -1;
    
    return 0;
}

long
SequenceNode::decodeTypeCode(IIOPMessageHeader &msg)
{
    // For each Id in the stored array
    // check its associated child is valid
    // and let the child handle the decoding
    for(int i = 0; childIds[i] != 0; ++i)
	if (childNodes[i] != 0)
	    if (childNodes[i]->decodeTypeCode(msg) != 0)
		return -1;
    return 0;
}

long
ListNode::decodeIIOP(IIOPMessageHeader &msg)
{
    // First nil child is the end of the list
    // Again have the child do the actual
    // typed decoding
    for(int i = 0; childNodes[i] != 0; ++i)
	if (childNodes[i]->decodeIIOP(msg) != 0)
	    return -1;
    return 0;
}

long
ListNode::decodeTypeCode(IIOPMessageHeader &msg)
{
    // First nil child is the end of the list
    // Again have the child do the actual
    // typed decoding
    for(int i = 0; childNodes[i] != 0; ++i)
	if (childNodes[i]->decodeTypeCode(msg) != 0)
	    return -1;
    return 0;
}

long
CharType::decodeIIOP(IIOPMessageHeader &msg)
{
    int pos = msg.curPos();
    msg >> value;

    if (ernie.checkChar(value))
	return 0;
    else
    {
	tet_printf("proxy: decoding error in Char %s at location %d",name,pos);
	tet_printf("proxy: expected 0x%x, got 0x%x",
		   (int )ernie.getCharAgain(), (int )value);
	return -1;
    }
}

long
CharType::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;
    
    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_char 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_char,msg.curPos());

    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;
	
	return processIndirection(this, msg.curPos(), indirection);
    }

    
    return 0;
}

long
StringType::decodeIIOP(IIOPMessageHeader &msg)
{
    int pos = msg.curPos();
    msg >> value;

    if (childNodes[0])
    {
	unsigned long strBound = childNodes[0]->exprValue();
	if (ernie.checkString(value, strBound))
	    return 0;
	else
	{
	    tet_printf("proxy: decoding error in bounded String %s at location %d",name,pos);
	    tet_printf("proxy: expected %s, got %s",
		       ernie.getStringAgain(), value);
	    return -1;
	}
    }
    else
    {
	if (ernie.checkString(value))
	    return 0;
	else
	{
	    tet_printf("proxy: decoding error in String %s at location %d",name,pos);
	    tet_printf("proxy: expected %s, got %s",
		       ernie.getStringAgain(), value);
	    return -1;
	}
    }
}

long
StringType::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;
    
    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_string 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_string,msg.curPos());

    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }
    else
    {
	// should check bound is correct wrt IDL defn.
	CORBA::ULong b;
	msg >> b;

	if (childNodes[0] == 0 && b!= 0)
	{
	    tet_printf("proxy: decoding error in length of unbounded string %s",name);
	    tet_printf("proxy: expected 0, got %d", b);
	    //return -1;
	}
    }   
    return 0;
}

long
BooleanType::decodeIIOP(IIOPMessageHeader &msg)
{
    int pos = msg.curPos();
    msg >> value;

    if (ernie.checkBoolean(value))
    	return 0;
    else
    {
	tet_printf("proxy: decoding error in Boolean %s at location %d",name,pos);
	tet_printf("proxy: expected %d, got %d",
		   (unsigned long)ernie.getBooleanAgain(),
		   (unsigned long)value);
	return -1;
    }
}

long
BooleanType::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;

    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_boolean 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_boolean,msg.curPos());

    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    return 0;
}

long
OctetType::decodeIIOP(IIOPMessageHeader &msg)
{
    int pos = msg.curPos();
    msg >> value;

    if (ernie.checkOctet(value))
    	return 0;
    else
    {
	tet_printf("proxy: decoding error in Octet %s at location %d",name,pos);
	tet_printf("proxy: expected 0x%x, got 0x%x",
		   (int )ernie.getOctetAgain(), (int )value);
	return -1;
    }
    
}

long
OctetType::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;

    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_octet 0x%x at %d", this, msg.curPos()/4);
#endif
    

    ret = processTK(tk, CORBA::tk_octet,msg.curPos());

    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    return 0;
}


long
FloatType::decodeIIOP(IIOPMessageHeader &msg)
{
    int pos = msg.curPos();
    msg >> value;

    if (ernie.checkFloat(value))
    	return 0;
    else
    {
	tet_printf("proxy: decoding error in Float %s at location %d",name,pos);
	tet_printf("proxy: expected %g, got %g",
		   ernie.getFloatAgain(), value);
	return -1;
    }
}

long
FloatType::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;

    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_float 0x%x at %d", this, msg.curPos()/4);
#endif
    

    ret = processTK(tk, CORBA::tk_float,msg.curPos());

    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    return 0;
}

long
DoubleType::decodeIIOP(IIOPMessageHeader &msg)
{
    int pos = msg.curPos();
    msg >> value;

    if (ernie.checkDouble(value))
    	return 0;
    else
    {
	tet_printf("proxy: decoding error in Double %s at location %d",name,pos);
	tet_printf("proxy: expected %e, got %e",
		   ernie.getDoubleAgain(), value);
	return -1;
    }
}

long
DoubleType::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;
    
    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_double 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_double,msg.curPos());
    
    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    return 0;
}

long
SignedLongInt::decodeIIOP(IIOPMessageHeader &msg)
{
    int pos = msg.curPos();
    msg >> value;

    if (ernie.checkLong(value))
    	return 0;
    else
    {
	tet_printf("proxy: decoding error in Long %s at location %d",name,pos);
	tet_printf("proxy: expected %lx, got %lx",
		   ernie.getLongAgain(), value);
	return -1;
    }
}

long
SignedLongInt::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;

    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_long 0x%x at %d", this, msg.curPos()/4);
#endif
    

    ret = processTK(tk, CORBA::tk_long,msg.curPos());
    
    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    return 0;
}

long
SignedShortInt::decodeIIOP(IIOPMessageHeader &msg)
{
    int pos = msg.curPos();
    msg >> value;

    if (ernie.checkShort(value))
    	return 0;
    else
    {
	tet_printf("proxy: decoding error in Short %s at location %d",name,pos);
	tet_printf("proxy: expected 0x%x, got 0x%x",
		   (int)ernie.getShortAgain(), (int)value);
	return -1;
    }
}

long
SignedShortInt::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;

    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_short 0x%x at %d", this, msg.curPos()/4);
#endif
    

    ret = processTK(tk, CORBA::tk_short,msg.curPos());
    
    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    return 0;
}


long
UnsignedLongInt::decodeIIOP(IIOPMessageHeader &msg)
{
    int pos = msg.curPos();
    msg >> value;

    if (ernie.checkULong(value))
    	return 0;
    else
    {
	tet_printf("proxy: decoding error in ULong %s at location %d",name,pos);
	tet_printf("proxy: expected 0x%x, got 0x%x",
		   ernie.getULongAgain(), value);
	return -1;
    }
}

long
UnsignedLongInt::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;

    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_ulong 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_ulong,msg.curPos());
    
    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    return 0;
}

long
UnsignedShortInt::decodeIIOP(IIOPMessageHeader &msg)
{
    int pos = msg.curPos();
    msg >> value;

    if (ernie.checkUShort(value))
    	return 0;
    else
    {
	tet_printf("proxy: decoding error in UShort %s at location %d",name,pos);
	tet_printf("proxy: expected 0x%x, got 0x%x",
		   (unsigned int)ernie.getUShortAgain(),
		   (unsigned int)value);
	return -1;
    }
}

long
UnsignedShortInt::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;
    
    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_ushort 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_ushort,msg.curPos());
    
    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    return 0;
}

long
OpDcl::decodeIIOP(IIOPMessageHeader &msg)
{
  // an operation must have some return specifier
    // and at least an empty parameter list
    if ( (childNodes[0] == 0) || (childNodes[2] == 0) )
    {
	tet_infoline("proxy: Parse error. OpDcl::decodeIIOP()");
	return -1;
    }

    // Requests and Replies differ wrt the marshalling 
    // of in's, inout's, out's and possible return types 
    switch (msg.message_type())
    {
	case Request:
	{
	    // process any parameters
	    if (childNodes[2]->decodeIIOP(msg) != 0)
		return -1;
	    
	    if (childNodes[4] != 0)
		return childNodes[4]->decodeIIOP(msg);
	}
	
	break;
	case Reply:
	{
	    // process return type first
#ifdef DEBUG
	tet_printf("proxy: Processing Return Value starting at location %d", msg.curPos());
#endif
	    if (childNodes[0]->decodeIIOP(msg) != 0)
		return -1;

	    // then any parameters
	    if (childNodes[2]->decodeIIOP(msg) != 0)
		return -1;
	}
	break;
	default:
        {
	    tet_infoline("proxy: Shouldn't call decodeIIOP for this type");
	    return -1;
	}
    }
    return 0;
}

long
ParamDcl::decodeIIOP(IIOPMessageHeader &msg)
{
    CORBA::ULong ret = 0;
    
    if ( (childNodes[0] == 0) || (childNodes[1] == 0) )
    {
	tet_infoline("proxy: Parse error. ParamDcl::decodeIIOP()");
	return -1;
    }
    
    switch (msg.message_type())
    {
	case Request:
	{
	    // as long as its not an out parameter
	    // we should decode if for requests
	    if (childNodes[0]->Choice() != 1)
	    {
		AddName((_Identifier &)(*this)[2][0]);
                ernie.mode((childNodes[0]->Choice() == 2)?1:0);
#ifdef DEBUG
	tet_printf("proxy: Processing Parameter %s starting at location %d", name, msg.curPos());
#endif
		ret = childNodes[1]->decodeIIOP(msg);
		RemoveName();
		
		return ret;
	    }
	}
	
	break;
	case Reply:
	{
	    // when its a reply ignore all in parameters
	    if (childNodes[0]->Choice() != 0)
	    {
		AddName((_Identifier &)(*this)[2][0]);
                ernie.mode((childNodes[0]->Choice() == 2)?2:0);
#ifdef DEBUG
	tet_printf("proxy: Processing Parameter %s starting at location %d", name, msg.curPos());
#endif
		ret = childNodes[1]->decodeIIOP(msg);
		RemoveName();
		
		return ret;
	    }
	}
	break;
	default:
        {
	    tet_infoline("proxy: Shouldn't call decodeIIOP for this type");
	    return -1;
	}
    }
    return 0;
}

long
_ScopedName::decodeIIOP(IIOPMessageHeader &msg)
{
    // Get our own definition
    Node *ptr;
    ptr = FindDefinition(*this, *this,2);

    if (!ptr)
    {
	tet_infoline("proxy: Coundn't find a type definition");
	tet_infoline("proxy: Inconsistent IDL in use");
	return -1;
    }

    if (IsATypeCode(ptr))
    {
#ifdef DEBUG
    tet_printf("proxy: Processing TypeCode %s starting at location %d", name, msg.curPos());
#endif
	if (value == 0)
	    value = findTypeCodeDef();

	return value->decodeTypeCode(msg);
    }
    else
    {
#ifdef DEBUG
        char buf[1024];
        Node &node = *this;

        if (node[IdScopedNameA] != IdNotInitialised)
          strcpy(buf,"::");
        else
          buf[0] = '\0';

        for (int i = 0; i < node[IdScopedNameB].Length(); ++i)
        {
          if (i != 0)
            strcat(buf,"::");
          strcat(buf,(_Identifier&)node[IdScopedNameB][i]);
        }
         
	tet_printf("proxy: looking up ScopedName %s",buf);
#endif
	// Take a deep copy of the tree branch
	if (value == 0)
	{
	    value = NewNode(ptr->Parent(), ptr->Id());
	    value->operator=(*ptr);
	}
	
	// Traverse this part of the tree
	return value->decodeIIOP(msg);
    }
}

long
_ScopedName::decodeTypeCode(IIOPMessageHeader &msg)
{
    // Get our own definition
    Node *ptr;
    ptr = FindDefinition(*this, *this,2);

    if (!ptr)
    {
	tet_infoline("proxy: Coundn't find ourselves");
	return -1;
    }

    if (IsATypeCode(ptr))
    {
	CORBA::ULong tk;
	CORBA::Long ret;

	msg >> tk;
	tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
	tet_printf("proxy: storing tk_TypeCode 0x%x at %d", this, msg.curPos()/4);
#endif
	

	ret = processTK(tk, CORBA::tk_TypeCode,msg.curPos());

	if (ret == -1)
	    return ret;
	else if (ret == -2)
	{
	    CORBA::Long indirection;
	    msg >> indirection;

	    return processIndirection(this, msg.curPos(), indirection);
	}

	return 0;
    }
    else
    {
	// Take a deep copy of the tree branch
	if (value == 0)
	{
	    value = NewNode(ptr->Parent(), ptr->Id());
	    value->operator=(*ptr);
	}
    
	// Traverse this part of the tree
	return value->decodeTypeCode(msg);
    }
}

long
EnumType::decodeIIOP(IIOPMessageHeader &msg)
{
    msg >> value;

    if (value == ernie.getULong() % childNodes[1]->Length())
	return 0;
    else
    {
	tet_printf("proxy: decoding error in Enumerated Type %s",name);
	tet_printf("proxy: expected %d, got %d",
		   ernie.getULong()  % childNodes[1]->Length(), value);
	return -1;
    }
}

long
EnumType::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;
    
    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_enum 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_enum,msg.curPos());

    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }
    
    // Enums are a complex type when used
    // in an Any. Hence they marshall as
    // CDR encapsulations. 
    CORBA::Boolean encapSex = 0;
    CORBA::ULong encapLen = 0;

    // cache the message byte order
    cacheMsg cache(msg);

    // get the encapsulation length
    msg >> encapLen;

    // get the encapsulations byte order
    msg >> encapSex;

    // temporarily set the coder to the endianness
    // that occurs in the encapsulation
    msg.byte_order(encapSex);

    vsOrbString repId;
    vsOrbString ename;

    unsigned long count = 0;

    msg >> repId;

    // repId can't be null
    if (!repId.isValid())
	return -1;

#ifdef DEBUG
    tet_printf("proxy: enum repId is %s", (char *)repId);
#endif

    msg >> ename;
    // name can't be null
    if (!ename.isValid())
	return -1;

#ifdef DEBUG
    tet_printf("proxy: enum name is %s", (char *)ename);
#endif

    msg >> count;

    if (count != childNodes[1]->Length())
    {
	tet_printf("proxy: incorrect enumerator count in TypeCode %s",name);
	tet_printf("proxy: expected %d, got %d at location %d", childNodes[1]->Length(), count, msg.curPos()-4);
	return -1;
    }

    return childNodes[1]->decodeTypeCode(msg);
}

long
Enumerator::decodeTypeCode(IIOPMessageHeader &msg)
{
    vsOrbString memberName;
    
    msg >> memberName;
    
    if (!strcmp((char *)*(_Identifier*)childNodes[0], memberName))
	return 0;
    else
	return -1;
}

long
SequenceType::decodeIIOP(IIOPMessageHeader &msg)
{
    // Get the sequence length
    msg >> seqLen;

    CORBA::ULong v;

    if (childNodes[1] != 0)
      v = ernie.getSequenceLength(childNodes[1]->exprValue());
    else
      v = ernie.getSequenceLength(0);

    if (seqLen != v)
    {
      tet_printf("proxy: %s", name);
      tet_printf("proxy: decoding error in length of sequence %s at location %d",name,msg.curPos()-4);
      tet_printf("proxy: expected 0x%x, got 0x%x",v , seqLen);
      return -1;
    }
    
    
    // Allocate that many elements
    value = (seqLen == 0) ? 0 : new SimpleTypeSpec[seqLen];

    int i = 0;

    // Iterate over decoding the appropriate type
    for (i; i < seqLen; i++)
    {
	AddIndex(i);
	
	(ChoiceNode&)(value[i]) = (*childNodes[0]);
	value[i].Parent(this);
	if (value[i].decodeIIOP(msg) != 0)
	    return -1;
	RemoveIndex();
    }
    return 0;
}

long
SequenceType::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;

    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_sequence 0x%x at %d", this, msg.curPos()/4);
#endif
    

    ret = processTK(tk, CORBA::tk_sequence,msg.curPos());

    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    unsigned long count = 0;

    // Sequences are a complex type when used
    // in an Any. Hence they marshall as
    // CDR encapsulations. 
    CORBA::Boolean encapSex = 0;
    CORBA::ULong encapLen = 0;

    // Cache the message byte order
    cacheMsg cache(msg);

    // get the encapsulations length
    msg >> encapLen;

    // get the encapsulations byte order
    msg >> encapSex;

    // temporarily set the coder to the endianness
    // that occurs in the encapsulation
    msg.byte_order(encapSex);

    if (childNodes[0]->decodeTypeCode(msg) != 0)
	return -1;
    
    msg >> count;

    // unbounded sequence
    if (childNodes[1] == 0)
    {
	if (count != 0)
	{
	    tet_infoline("proxy: unbounded seq must have zero length in TypeCode");
	    tet_printf("proxy: expected 0, got %d at location %d", count, msg.curPos()-4);
	    return -1;
	}
    }
    else
	if (count != (unsigned long)childNodes[1]->exprValue())
	{
	    tet_printf("proxy: incorrect bounded seq length in TypeCode %s",name);
	    tet_printf("proxy: expected %d, got %d at location %d", (unsigned long)childNodes[1]->exprValue(), count, msg.curPos()-4);
	    return -1;
	}
     return 0;
}

long
ArrayDeclarator::decodeIIOP(IIOPMessageHeader &msg)
{
    // Find associated TypeSpec
    Node * ptr;
    for (ptr = parent;
	 ptr != 0 && ptr->Id() != IdTypeDeclarator &&
	 ptr->Id() != IdElementSpec &&
	 ptr->Id() != IdMember;
	 ptr=ptr->Parent())
	;

    if (ptr != 0)
	ptr = &(*ptr)[0];

    // Check TypeSpec is non nil
    if (ptr == 0)
    {
	tet_infoline("proxy: error ArrayDeclarator::decodeIIOP()");
	tet_infoline("proxy: Unable to find TypSpec");
	return -1;
    }

    int i = 0;

    // Call exprValue to calculate array length
    if (childNodes[1] != 0)
    {
	// Allocate sufficient storage for array
	seqLen = childNodes[1]->exprValue();
	value = new TypeSpec[seqLen];

	// Decode each array element
	for (i; i < seqLen; i++)
	{
	    AddIndex(i);
	    (ChoiceNode&)(value[i]) = *ptr;
	    value[i].Parent(ptr->Parent());
	    if (value[i].decodeIIOP(msg) != 0)
	    {
		RemoveIndex();
		return -1;
	    }
	    RemoveIndex();
	}
    }
    else
	tet_infoline("proxy: error ArrayDeclarator::decodeIIOP()");
    
    return 0;
}

long
FixedArraySizeList::decodeTypeCode(IIOPMessageHeader &msg)
{
  return childNodes[0]->decodeTypeCode(msg);
}

long
FixedArraySize::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;
    
    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_array 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_array,msg.curPos());

    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    unsigned long count = 0;

    // Arrays are a complex type when used
    // in an Any. Hence they marshall as
    // CDR encapsulations. 
    CORBA::Boolean encapSex = 0;
    CORBA::ULong encapLen = 0;

    // cache the message byte order
    cacheMsg cache(msg);

    // get the encapsulation length
    msg >> encapLen;

    // get the encapsulations byte order
    msg >> encapSex;

    // temporarily set the coder to the endianness
    // that occurs in the encapsulation
    msg.byte_order(encapSex);

    Node * ptr;
    int index = 0;
    
    // init thing
    index = findNextFAS(this, parent);

    if (index == 0)
    {
	// Find associated TypeSpec
	for (ptr = parent;
	     ptr != 0 && ptr->Id() != IdTypeDeclarator &&
	     ptr->Id() != IdElementSpec &&
	     ptr->Id() != IdMember;
	     ptr=ptr->Parent())
	    ;
	
	if (ptr != 0)
	    ptr = &(*ptr)[0];
	
	// Check TypeSpec is non nil
	if (ptr == 0)
	{
	    tet_infoline("proxy: error FixedArraySize::decodeTypeCode()");
	    tet_infoline("proxy: Unable to find TypSpec");
	    return -1;
	}
    }
    else
	ptr = &(*parent)[index];
    
    
    if (ptr->decodeTypeCode(msg) != 0)
	return -1;
    
    msg >> count;

    if (count != (unsigned long)child->exprValue())
    {
	tet_printf("proxy: incorrect array bound length in TypeCode %s",name);
	tet_printf("proxy: expected %d, got %d at location %d", (unsigned long)child->exprValue(), count, msg.curPos()-4);
	return -1;
    }

    return 0;
}

long
AttrDcl::decodeIIOP(IIOPMessageHeader &msg)
{
#ifdef DEBUG
	tet_printf("proxy: Processing Attribute starting at location %d", msg.curPos());
#endif

    // Prevent Declarator List from processing message but
    // have the ParamTypeSpec do the work instead
    if (childNodes[1] != 0)
	return childNodes[1]->decodeIIOP(msg);
    else
    {
	tet_infoline("proxy: AttrDcl::decodeIIOP() element must have a ParamTypeSpec");
	return 0;
    }
}

long
ElementSpec::decodeIIOP(IIOPMessageHeader &msg)
{
    // Prevent TypSpec from processing message but
    // have the Declarator do the work instead
    if (childNodes[1] != 0)
	return childNodes[1]->decodeIIOP(msg);
    else
    {
	tet_infoline("proxy: ElementSpec::decodeIIOP() element must have a declarator");
    
	
	return 0;
    }
}

long
ElementSpec::decodeTypeCode(IIOPMessageHeader &msg)
{
    // Prevent TypSpec from processing message but
    // have the Declarator do the work instead
    if (childNodes[1] != 0)
	return childNodes[1]->decodeTypeCode(msg);
    else
    {
	tet_infoline("proxy: ElementSpec::decodeIIOP() element must have a declarator");
	return 0;
    }
}

long
TypeDeclarator::decodeIIOP(IIOPMessageHeader &msg)
{
    // Prevent TypSpec from processing message but
    // have the Declarators do the work instead
    if (childNodes[1] != 0)
	return childNodes[1]->decodeIIOP(msg);
    else
    {
	tet_infoline("proxy: TypeDeclarator::decodeIIOP() element must have a declarators");
	
	return 0;
    }
}

long
TypeDeclarator::decodeTypeCode(IIOPMessageHeader &msg)
{
    // Prevent TypSpec from processing message but
    // have the Declarators do the work instead
    if (childNodes[1] != 0)
	return childNodes[1]->decodeTypeCode(msg);
    else
    {
	tet_infoline("proxy: TypeDeclarator::decodeTypeCode() element must have a declarators");
	
	return 0;
    }
}

long
Member::decodeIIOP(IIOPMessageHeader &msg)
{
    // Prevent TypSpec from processing message but
    // have the Declarator do the work instead
    if (childNodes[1] != 0)
	return childNodes[1]->decodeIIOP(msg);
    else
    {
	tet_infoline("proxy: Member::decodeIIOP() element must have a declarators");
	
	return 0;
    }
}

long
Member::decodeTypeCode(IIOPMessageHeader &msg)
{
    // Prevent TypSpec from processing message but
    // have the Declarator do the work instead
    if (childNodes[1] != 0)
	return childNodes[1]->decodeTypeCode(msg);
    else
    {
	tet_infoline("proxy: Member::decodeTypeCode() element must have a declarators");
	
	return 0;
    }
}

long
StructType::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;
    
    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_struct 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_struct,msg.curPos());
    
    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    // Structs are a complex type when used
    // in an Any. Hence they marshall as
    // CDR encapsulations. 
    CORBA::Boolean encapSex = 0;
    CORBA::ULong encapLen = 0;

    // cache the message byte order
    cacheMsg cache(msg);

    // get the encapsulation length
    msg >> encapLen;

    // get the encapsulations byte order
    msg >> encapSex;

    // temporarily set the coder to the endianness
    // that occurs in the encapsulation
    msg.byte_order(encapSex);
    
    vsOrbString repId;
    vsOrbString TypeName;
    unsigned long count = 0;

    msg >> repId;

    // repId can't be null
    if (!repId.isValid())
	return -1;

#ifdef DEBUG
    tet_printf("proxy: struct repId is %s", (char *)repId);
#endif
    
    msg >> TypeName;
    // TypeName can't be null
    if (!TypeName.isValid())
	return -1;

#ifdef DEBUG
    tet_printf("proxy: struct name is %s", (char *)TypeName);
#endif
    
    msg >> count;

    if (count != CountMembers((*this)[0]))
    {
	tet_printf("proxy: incorrect struct member count in TypeCode %s",name);
	tet_printf("proxy: expected %d, got %d at location %d",
		   CountMembers((*this)[0]), count, msg.curPos()-4);
	return -1;
    }
    
    return childNodes[0]->decodeTypeCode(msg);
}


long
ExceptDcl::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;
    
    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_except 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_except,msg.curPos());

    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    // Exceptions are a complex type when used
    // in an Any. Hence they marshall as
    // CDR encapsulations. 
    CORBA::Boolean encapSex = 0;
    CORBA::ULong encapLen = 0;

    // cache the message byte order
    cacheMsg cache(msg);

    // get the encapsulation length
    msg >> encapLen;

    // get the encapsulations byte order
    msg >> encapSex;

    // temporarily set the coder to the endianness
    // that occurs in the encapsulation
    msg.byte_order(encapSex);

    vsOrbString repId;
    vsOrbString TypeName;
    unsigned long count = 0;

    msg >> repId;
    // repId can't be null
    if (!repId.isValid())
	return -1;

#ifdef DEBUG
    tet_printf("proxy: except repId is %s", (char *)repId);
#endif
    
    msg >> TypeName;
    // TypeName can't be null
    if (!TypeName.isValid())
	return -1;

#ifdef DEBUG
    tet_printf("proxy: except name is %s", (char *)TypeName);
#endif
    
    msg >> count;

    if (count != childNodes[1]->Length())
    {
	tet_printf("proxy: incorrect except member count in TypeCode %s",name);
	tet_printf("proxy: expected %d, got %d at location %d", childNodes[1]->Length(), count, msg.curPos()-4);
	return -1;
    }
    
    return childNodes[1]->decodeTypeCode(msg);
}


long
SimpleDeclarator::decodeIIOP(IIOPMessageHeader &msg)
{
    // Find associated TypeSpec
    Node * ptr;
    for (ptr = parent;
	 ptr != 0 && ptr->Id() != IdTypeDeclarator &&
	 ptr->Id() != IdElementSpec &&
	 ptr->Id() != IdMember;
	 ptr=ptr->Parent())
	;

    if (ptr != 0)
	ptr = &(*ptr)[0];

    // Check TypeSpec is non nil
    if (ptr == 0)
    {
	tet_infoline("proxy: error SimpleDeclarator::decodeIIOP()");
	tet_infoline("proxy: Unable to find TypSpec");
	return -1;
    }

    if (value == 0)
    {
	value = NewNode(ptr->Parent(), ptr->Id());
	value->operator=(*ptr);
    }
    
    // Traverse this part of the tree
    AddName((_Identifier &)*childNodes[0]);

    CORBA::ULong ret = value->decodeIIOP(msg);

    RemoveName();
    return ret;
    
}

long
SimpleDeclarator::decodeTypeCode(IIOPMessageHeader &msg)
{
    // Find associated TypeSpec
    Node * ptr;
    for (ptr = parent;
	 ptr != 0 && ptr->Id() != IdTypeDeclarator &&
	 ptr->Id() != IdElementSpec &&
	 ptr->Id() != IdMember;
	 ptr=ptr->Parent())
	;

    if (ptr != 0)
	ptr = &(*ptr)[0];

    // Check TypeSpec is non nil
    if (ptr == 0)
    {
	tet_infoline("proxy: error SimpleDeclarator::decodeTypeCode()");
	tet_infoline("proxy: Unable to find TypeSpec");
	return -1;
    }

    if (value == 0)
    {
		value = NewNode(ptr->Parent(), ptr->Id());
		value->operator=(*ptr);
    }
    
    // Traverse this part of the tree
    return value->decodeTypeCode(msg);
}

long
Declarator::decodeTypeCode(IIOPMessageHeader &msg)
{
	Node * ptr;
  for (ptr = parent;
      ptr != 0 && ptr->Id() != IdTypeDeclarator &&
      ptr->Id() != IdElementSpec &&
      ptr->Id() != IdMember;
      ptr=ptr->Parent())
    ;

  if (ptr == 0)
  {
    tet_infoline("proxy: error Declarator::decodeTypeCode()");
    tet_infoline("proxy: Null pointer found when searching for TypeDeclarator");
    return -1;
  }

  switch (ptr->Id()) {
  case IdMember:
    {
      vsOrbString name;
      msg >> name;

#ifdef DEBUG
      tet_printf("proxy: processing struct member %s", (char *)name);
#endif
    }
    // No Break
  default:
    return child->decodeTypeCode(msg);
    break;
  case IdTypeDeclarator:
		// Typedefs translate to aliases which are complex types
		CORBA::ULong tk;
		CORBA::Long ret;

		msg >> tk;

    // Aliases are optional within a TypeCode, 
    // so if the next TCKind is not tk_alias, 
    // assume that it is inplace.
    if (tk != CORBA::tk_alias
     && tk != 0xFFFFFFFF)
    {
      // rewind the coder 
      msg.rewind(4);
      return child->decodeTypeCode(msg);
    }
    else
    {
  		tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    	tet_printf("proxy: storing tk_alias 0x%x at %d", this, msg.curPos()/4);
#endif

	    ret = processTK(tk, CORBA::tk_alias,msg.curPos());
    
	    if (ret == -1)
	      return ret;
      else if (ret == -2)
      {
        CORBA::Long indirection;
        msg >> indirection;

        return processIndirection(this, msg.curPos(), indirection);
      }

      // Aliases are a complex type when used
      // in an Any. Hence they marshall as
      // CDR encapsulations. 
      CORBA::Boolean encapSex = 0;
      CORBA::ULong encapLen = 0;

      // cache the message byte order
      cacheMsg cache(msg);

      // get the encapsulation length
      msg >> encapLen;

      // get the encapsulations byte order
      msg >> encapSex;

      // temporarily set the coder to the endianness
      // that occurs in the encapsulation
      msg.byte_order(encapSex);

      vsOrbString repId;
      vsOrbString name;

      msg >> repId;

      // repId can't be null
      if (!repId.isValid())
        return -1;

#ifdef DEBUG
      tet_printf("proxy: alias repId is %s", (char *)repId);
#endif
    
      msg >> name;
      // name can't be null
      if (!name.isValid())
        return -1;

#ifdef DEBUG
      tet_printf("proxy: alias name is %s", (char *)name);
#endif
    
      return child->decodeTypeCode(msg);

    }
  }
}

long
UnionSwitch::decodeIIOP(IIOPMessageHeader &msg)
{
    if (childNodes[0] == 0 || childNodes[1] == 0)
    {
	tet_infoline("proxy: error UnionSwitch::decodeIIOP badly formed UnionSwitch");
	return -1;
    }

    // Get the descriminator
    switch (FindTypeSpecifier((*this)[0]))
    {
	case tsShort :
        {
	    short s;
	    msg >> s;
	    d = (long)s;
			ernie.getULong();
	    break;
	}
    
	case tsUShort :
        {
	    unsigned short us;
	    msg >> us;
	    d = (unsigned long)us;
			ernie.getULong();
	    break;
	}
	case tsLong :
        {
	    long l;
	    msg >> l;
	    d = l;
			ernie.getULong();
	    break;
	}
	case tsEnum :
	case tsULong :
        {
	    unsigned long ul;
	    msg >> ul;
	    d = ul;
			ernie.getULong();
	    break;
	}
	case tsBoolean :
        {
	    unsigned char b;
	    msg >> b;
	    d = (VSORB_Boolean)b;
			ernie.getULong();
	    break;
	}
	case tsChar :
        {
	    char c;
	    msg >> c;
	    d = (unsigned char)c;
			ernie.getULong();
	    break;
	}
	default :
        {
	    tet_infoline("proxy: Illegal union descriminator type selected");
	    return -1;
	}
    }


    
    // Now we have the descriminator
    Node *ptr;
    Node *defaultNode;

    // Select the correct ElementSpec using the discriminator
    // If an exact match can not be found, use the defaultNode
    if ((ptr = findElementSpec((*this)[1],d,defaultNode)) == 0)
	ptr = defaultNode;

    if (ptr == 0)
    {
	tet_infoline("proxy: error UnionSwitch::decodeIIOP()");
	return -1;
    }
    
    // Make a local copy of the selected ElementSpec
    if (value == 0)
    {
	value = NewNode(ptr->Parent(), ptr->Id());
	value->operator=(*ptr);
    }
    
    // Traverse this part of the tree
    return value->decodeIIOP(msg);
}

long
UnionSwitch::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;
    
    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_union 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_union,msg.curPos());

    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    // Unions are a complex type when used
    // in an Any. Hence they marshall as
    // CDR encapsulations. 
    CORBA::Boolean encapSex = 0;
    CORBA::ULong encapLen = 0;

    // cache the message byte order
    cacheMsg cache(msg);

    // get the encapsulation length
    msg >> encapLen;

    // get the encapsulations byte order
    msg >> encapSex;

    // temporarily set the coder to the endianness
    // that occurs in the encapsulation
    msg.byte_order(encapSex);

    vsOrbString repId;
    vsOrbString TypeName;

    msg >> repId;
    // repId can't be null
    if (!repId.isValid())
	return -1;

#ifdef DEBUG
    tet_printf("proxy: union repId is %s", (char *)repId);
#endif
    
    msg >> TypeName;
    // TypeName can't be null
    if (!TypeName.isValid())
	return -1;

#ifdef DEBUG
    tet_printf("proxy: union name is %s", (char *)TypeName);
#endif

    if (childNodes[0]->decodeTypeCode(msg) != 0)
	return -1;

    long defaultLabel = 0;
    unsigned long count = 0;

    msg >> defaultLabel;
    msg >> count;

    if (CountCaseLabels(*this, 1) == CountCaseLabels(*this, 0))
    {
	// therefore there is no default label
	if (defaultLabel >= 0)
	{
	    tet_printf("proxy: error expected negative defaultLabel indicator in %s",name);
	    tet_printf("proxy: expected < 0, got %d", defaultLabel);
	    return -1;
	}
    }
    else
    {
	// check implied default is valid 
    }    

    if (count != CountCaseLabels(*this, 1))
    {
	tet_printf("proxy: error in case label count for union %s",name);
	tet_printf("proxy: expected %d, got %d",
		   CountCaseLabels(*this, 1), count);
	return -1;
    }
    
    return childNodes[1]->decodeTypeCode(msg);
}

long
AnyType::decodeIIOP(IIOPMessageHeader &msg)
{

  value = findTypeCodeDef((++ernie.anyCount < 10)?0:11);
    
    if (value->decodeTypeCode(msg) == 0)
	return value->decodeIIOP(msg);

    return -1;
}

long
CaseLabel::decodeTypeCode(IIOPMessageHeader &msg)
{
    // Get the descriminator
    switch (FindTypeSpecifier(*this))
    {
	case tsShort :
        {
	    short s;
	    msg >> s;
	    if (choice == 0)
	    {
		if (child->exprValue() != (long)s)
		{
		    tet_printf("proxy: error in case label value %s",name);
		    tet_printf("proxy: expected %d, got %d",
			       (CORBA::Long)child->exprValue(), s);
		    return -1;
		}
	    }
	    
	    break;
	}
    
	case tsUShort :
        {
	    unsigned short us;
	    msg >> us;
	    if (choice == 0)
	    {
			unsigned long value = child->exprValue();
		if (value != us)
		{
		    tet_printf("proxy: error in case label value %s",name);
		    tet_printf("proxy: expected %u, got %u",
			       (CORBA::ULong)child->exprValue(), us);
		    return -1;
		}
	    }
	    break;
	}
	case tsLong :
        {
	    long l;
	    msg >> l;
	    if (choice == 0)
	    {
		if (child->exprValue() != l)
		{
		    tet_printf("proxy: error in case label value %s",name);
		    tet_printf("proxy: expected %d, got %d",
			       (CORBA::Long)child->exprValue(), l);
		    return -1;
		}
	    }
	    break;
	}
	case tsEnum :
	case tsULong :
        {
	    unsigned long ul;
	    msg >> ul;
	    if (choice == 0)
	    {
				unsigned long value;
				value = child->exprValue();
		if (value != ul)
		{
		    tet_printf("proxy: error in case label value %s",name);
		    tet_printf("proxy: expected %u, got %u",
			       (CORBA::ULong)child->exprValue(), ul);
		    return -1;
		}
	    }
	    break;
	}
	case tsBoolean :
        {
	    CORBA::Boolean b;
	    msg >> b;
	    if (choice == 0)
	    {
		
		VSORB_Boolean value;
		
		value = (VSORB_Boolean)child->exprValue();
		
		if (value != b)
		{
		    tet_printf("proxy: error in case label value %s",name);
		    tet_printf("proxy: expected %d, got %d",
			       (CORBA::ULong)value,
			       (CORBA::ULong)b);
		    return -1;
		}
	    }
	    break;
	}
	case tsChar :
        {
	    unsigned char c;
	    msg >> c;
	    if (choice == 0)
	    {
		unsigned char value = (unsigned char)child->exprValue();
		
		if (value != c)
		{
		    tet_printf("proxy: error in case label value %s",name);
		    tet_printf("proxy: expected %u, got %u",
			       (CORBA::ULong)value,
			       (CORBA::ULong)c);
		    return -1;
		}
	    }
	    break;
	}
	default :
        {
	    tet_infoline("proxy: Illegal union descriminator type selected");
	    return -1;
	}
    }

    vsOrbString name;

    msg >> name;
    // name can't be null
    if (!name.isValid())
	return -1;

#ifdef DEBUG
    tet_printf("proxy: label name is %s", (char *)name);
#endif

    Node * ptr;
    for (ptr = parent;
	 ptr != 0 && ptr->Id() != IdCase;
	 ptr=ptr->Parent())
	;

    if (ptr != 0)
	ptr = &(*ptr)[1];

    // Check TypeSpec is non nil
    if (ptr == 0)
    {
	tet_infoline("proxy: error CaseLabel::decodeTypeCode()");
	tet_infoline("proxy: Unable to find ElementSpec");
	return -1;
    }

    return ptr->decodeTypeCode(msg);
}

long
Case::decodeTypeCode(IIOPMessageHeader &msg)
{
    // Prevent ElementSpec from processing message but
    // have the Declarator do the work instead
    if (childNodes[0] != 0)
	return childNodes[0]->decodeTypeCode(msg);
    else
    {
	tet_infoline("proxy: Case::decodeTypeCode() element must have an ElementSpec");
	return 0;
    }
}


long
AnyType::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;
    
    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_any 0x%x at %d", this, msg.curPos()/4);
#endif

    
    ret = processTK(tk, CORBA::tk_any,msg.curPos());
    
    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }
    
    return 0;
}

extern IOR * pIOR;

long
ObjectType::decodeIIOP(IIOPMessageHeader &msg)
{
    IOR * ior = new IOR;

    CORBA::Boolean mend;
    long ret = -1;
    
    mend = msg.StreamEndian();

    ior->coder().StreamEndian(msg.StreamEndian());

    msg >> ior;

    ProfileBody * msgBody = ior->extractInternetTag();

#ifdef DEBUG
    tet_printf("proxy: IOR has typeId [%s]", ior->typeId());

    
    tet_printf("proxy:   Version:       %d.%d\n",
	       (CORBA::ULong)msgBody->ver().major(),
	       (CORBA::ULong)msgBody->ver().minor());
    
    tet_printf("proxy:   Host:          %s\n",msgBody->host());
    tet_printf("proxy:   Port:          %d\n",msgBody->port());
#endif

    if (msg.message_type() == 0)
    {
	if (*pBody == *msgBody)
	    ret = 0;
    }	
    else
	if (*sBody == *msgBody)
	    ret = 0;


    if (ret == -1)
	tet_infoline("proxy: incorrect TAG_INTERNET_PROFILE");

    delete ior;
    
    return ret;
}

long
ObjectType::decodeTypeCode(IIOPMessageHeader &msg)
{
    CORBA::ULong tk;
    CORBA::Long ret;
    
    msg >> tk;
    tcIndirect[msg.curPos()/4] = this;
#ifdef DEBUG
    tet_printf("proxy: storing tk_objref 0x%x at %d", this, msg.curPos()/4);
#endif


    ret = processTK(tk, CORBA::tk_objref,msg.curPos());
    
    if (ret == -1)
	return ret;
    else if (ret == -2)
    {
	CORBA::Long indirection;
	msg >> indirection;

	return processIndirection(this, msg.curPos(), indirection);
    }

    // ObjectRefs are a complex type when used
    // in an Any. Hence they marshall as
    // CDR encapsulations. 
    CORBA::Boolean encapSex = 0;
    CORBA::ULong encapLen = 0;

    // cache the message byte order
    cacheMsg cache(msg);

    // get the encapsulation length
    msg >> encapLen;

    // get the encapsulations byte order
    msg >> encapSex;

    // temporarily set the coder to the endianness
    // that occurs in the encapsulation
    msg.byte_order(encapSex);
    
    vsOrbString repId;
    vsOrbString name;
    unsigned long count = 0;

    msg >> repId;

    // repId can't be null
    if (!repId.isValid())
	return -1;

#ifdef DEBUG
    tet_printf("proxy: objref repId is %s", (char *)repId);
#endif
    
    msg >> name;
    // name can't be null
    if (!name.isValid())
	return -1;

#ifdef DEBUG
    tet_printf("proxy: objref name is %s", (char *)name);
#endif
    
    return 0;
}

long
ContextExpr::decodeIIOP(IIOPMessageHeader &msg)
{
    int index = 0;
    CORBA::ULong seqLen = 0;
    
    msg >> seqLen;

    if (seqLen != Length()*2)
    {
	tet_printf("proxy: decoding error in length of context %s",name);
	tet_printf("proxy: expected %d, got %d",
		   Length()*2, seqLen);
	return -1;
    }
    
    vsOrbString *cntxtPairs = new vsOrbString[seqLen];
    
    for (index; index < seqLen; index++)
    {
	msg >> cntxtPairs[index];
#ifdef DEBUG
	tet_printf("proxy: cntxt name [%d] - %s", index, (char *)cntxtPairs[index]);
#endif	
	index++;
	msg >> cntxtPairs[index];
#ifdef DEBUG
	tet_printf("proxy: cntxt value [%d] - %s", index, (char *)cntxtPairs[index]);
#endif
    }

    for (index = 0; index < Length(); index++)
    {
	char * cname = (char *)(StringLiteral&)(*childNodes[index]);
	for (int i = 0; i < seqLen; i+=2)
	{
	    if (!strcmp(cntxtPairs[i], cname))
	    {
		if (!ernie.checkString(cntxtPairs[i+1]))
		{
		    tet_printf("proxy: decoding error in Context string %s", cname);
		    tet_printf("proxy: expected %s, got %s",
			       ernie.getStringAgain(), (char *)cntxtPairs[i+1]);
		    if (cntxtPairs)
			delete [] cntxtPairs;
		    
		    return -1;
		}
		break;
	    }
	}
    }

    if (cntxtPairs)
	delete [] cntxtPairs;
    
    return 0;
}

void
AddName(char *p)
{
    if (strlen(name))
	strcat(name,".");
    strcat(name,p);
}

void
RemoveName()
{
    int i;
    
    for (i = strlen(name);
	 i > 0
	 && name[i] != '.';
	 --i)
	;
    
    name[i] = '\0';
}

void
AddIndex(int i)
{
    sprintf(name+strlen(name),"[%d]",i);
}

void
RemoveIndex()
{
    int i;
    
    for (i = strlen(name); i > 0 && name[i] != '['; --i)
	 ;
	 
    name[i] = '\0';
}


CORBA::Long
processTK(CORBA::ULong got, CORBA::ULong want, CORBA::ULong pos)
{

#ifdef DEBUG
    tet_printf("proxy: expected 0x%x, got 0x%x", want, got);
#endif

    if (got == 0xffffffff)
	return -2;
    
    if (got != want)
    {
	tet_printf("proxy: incorrect tk_kind value in TypeCode %s",name);
	tet_printf("proxy: expected 0x%x, got 0x%x at location %d", want, got, pos-4);
	return -1;
    }

    return 0;
}

// returns 1 if they are the same
int IsEquivalent(Node *p1, Node *p2)
{

  if (p1 == 0 && p2 == 0)
    return 1;

  if (p1 == 0 || p2 == 0)
    return 0;

  if (p1->Id() != p2->Id())
    return 0;

  if (p1->Id() == IdConstExp)
  {
    // equivalent if the expressions evaluated to the same value
    ExprValue e1;
    ExprValue e2;

    e1 = p1->exprValue();
    e2 = p2->exprValue();
    // Be carefult not to compare two expression of a different type
    if (e1.Type() != e2.Type())
      return 0;

    if (e1 == e2)
      return 1;
    else
      return 0;
  }

  if (p1->Length() != p2->Length())
    return 0;

	for (int i = 0; i < p1->Length(); ++i)
	{
		if (!IsEquivalent(&(*p1)[i],&(*p2)[i]))
  		return 0;
  }
  return 1;
}

long
processIndirection(Node * currentNode,
		   CORBA::ULong curPos,
		   CORBA::Long indirection)
{
    CORBA::Long align = indirection%4;
    if (align < 0)
      align = -align;

    CORBA::ULong index = curPos + indirection + align;
    index = index/4;

#ifdef DEBUG
    tet_printf("proxy: typecode indirection of %d bytes at location %d", indirection,curPos-4);
    tet_printf("proxy: searching tclist at %d", index);
#endif
    
    Node * recurse = tcIndirect[index];

#ifdef DEBUG
    tet_printf("proxy: found pointer 0x%x at tclist[%d] ", recurse,index);
#endif
    
    if (recurse == 0)
    {
	tet_printf("proxy: Typecode Indirection (%d) at location %d does not point to the start of a typecode definition",indirection,curPos-4);
	return -1;
    }
    else
    {
      Node *type=0;

      if (currentNode->Id() == IdDeclarator)
      {
        _ScopedName sn;

        if ((*currentNode)[IdSimpleDeclarator])
          BuildFullyScopedName((*currentNode)[IdSimpleDeclarator],sn);
        else
          BuildFullyScopedName((*currentNode)[IdComplexDeclarator][IdArrayDeclarator],sn);

        type = FindDefinition(specification,sn);

        while (type != 0
          && *type != IdSequenceType
          && *type != IdStringType
          && *type != IdStructType
          && *type != IdUnionType
          && *type != IdEnumType
          && *type != IdScopedName
          && (*type)[0] != IdNotInitialised)
          type = &(*type)[0];
      }

      if (IsEquivalent(currentNode, recurse)
        || (type != 0 && IsEquivalent(type, recurse)))
      {
#ifdef DEBUG
	tet_infoline("proxy: found a match");
#endif
	return 0;
      }
      else
      {
	tet_printf("proxy: Typecode Indirection (%d) at location %d does not point to the a typecode definition of the expected type",indirection,curPos-4);
	return -1;
      }
    }
}
