//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/iiop/src/find.cpp
//
// Modifications:
// $Log: find.cpp,v $
// Revision 1.2  2000/02/11 16:04:30  mli
// Decl. of index p extracted out of the 'for' loop (2x).
//
// Revision 1.11  1998/09/04 15:41:03  james
// max parameter added to findTypeCodeDef
//
// Revision 1.10  1998/08/28 14:51:27  james
// BuildFullyScopedName added
//
// Revision 1.9  1998/05/11 10:26:36  stephen
// Node processing simplified.
//
// Revision 1.8  1997/12/12 15:11:05  stephen
// Extra debug info removed.
//
// Revision 1.7  1997/12/05 16:31:41  stephen
// Bug in count func.
//
// Revision 1.6  1997/12/04 14:11:42  stephen
// Additional typecodes added to findTCDef().
//
// Revision 1.5  1997/11/25 15:32:52  stephen
// TypeCode support added.
//
// Revision 1.4  1997/11/21 13:02:06  stephen
// New helper functions.
//
// Revision 1.3  1997/10/22 13:48:21  stephen
// Common code moved to parser directory.
//
// Revision 1.2  1997/10/02 18:53:25  stephen
// Synchronisation point.
//
// Revision 1.1  1997/09/19 08:59:48  stephen
// Auxiliary routines to assit tree navigation.
//

#include <stdlib.h>
#include <iostream.h>

#include "find.hh"

Node *
findAttrDef(const _Identifier &name, Node &ptr)
{
    switch(ptr.Id())
    {
	case IdNotInitialised :
	    return 0;
	
	// Check attribute declaration
	// Search for simple declarator and then
	// rewind and check were in an attribute declarator
	case IdSimpleDeclarator :
	if (ptr[IdIdentifier] != IdNotInitialised
	    && ((_Identifier&)(ptr[IdIdentifier])) == name)
	    {
		Node * p;
		
		// Find our grandparent
		p = ptr.Parent();
		if (p->Parent() != 0)
		    p = p->Parent();

		// Check gramps is an attribute declaration
		if (p != 0 && p->Id() == IdAttrDcl)
		    return p;
	    }
	
	break;
	// Continue the search into the child nodes
	default:
	Node *ret;
	for (int i = 0; i < ptr.Length(); ++i)
	{
	    if ((ret = findAttrDef(name,ptr[i])) != 0)
		return ret;
	}
    }
    return 0;
}

Node *
findOpDef(const _Identifier &name, Node &ptr)
{
    switch(ptr.Id())
    {
	case IdNotInitialised:
	    return 0;
	
	// Check operation declaration matches name
	case IdOpDcl :
	if (ptr[IdIdentifier] != IdNotInitialised 
	    && ((_Identifier&)(ptr[IdIdentifier])) == name)
	    return &ptr;
	break;
	// Continue the search into the child nodes
	default:
	Node *ret;
	for (int i = 0; i < ptr.Length(); ++i)
	{
	    if ((ret = findOpDef(name,ptr[i])) != 0)
		return ret;
	}
    }
    return 0;
}


Node *
findTypeCodeDef(unsigned long max)
{
    int numTypes = 0;
    Node * n = 0;

    if (max == 0)
    {
      numTypes = CountTypes(specification);
      numTypes = ernie.getULong()%(13+numTypes);
    }
    else
      numTypes = ernie.getULong()%max;


    switch(numTypes)
    {
          case   0: {
	      n = NewNode(0,IdUnsignedLongInt);
#ifdef DEBUG
	      ::tet_infoline("proxy: findTCDef creating new node of type: ULong\n");
#endif
	      break;
	  }
		    
	  case   1: {
	      n = NewNode(0,IdSignedLongInt);
#ifdef DEBUG
	      ::tet_infoline("proxy: findTCDef creating new node of type: Long\n");
#endif
	      break;
	  }
		    
	  case   2: {
	      n = NewNode(0,IdUnsignedShortInt);
#ifdef DEBUG
	      ::tet_infoline("proxy: findTCDef creating new node of type: UShort\n");
#endif
	      break;
	  }
		    
	  case   3: {
	      n = NewNode(0,IdSignedShortInt);
#ifdef DEBUG
	      ::tet_infoline("proxy: findTCDef creating new node of type: Short\n");
#endif
	      break;
	  }
		    
	  case   4: {
	      n = NewNode(0,IdBooleanType);
#ifdef DEBUG
	      ::tet_infoline("proxy: findTCDef creating new node of type: Boolean\n");
#endif
	      break;
	  }
		    
	  case   5: {
	      n = NewNode(0,IdOctetType);
#ifdef DEBUG
	      ::tet_infoline("proxy: findTCDef creating new node of type: Octet\n");
#endif
	      break;
	  }
		    
	  case   6: {
	      n = NewNode(0,IdCharType);
#ifdef DEBUG
	      ::tet_infoline("proxy: findTCDef creating new node of type: Char\n");
#endif
	      break;
	  }
		    
	  case   7: {
	      n = NewNode(0,IdFloatType);
#ifdef DEBUG
	      ::tet_infoline("proxy: findTCDef creating new node of type: Float\n");
#endif
	      break;
	  }
		    
	  case   8: {
	      n = NewNode(0,IdDoubleType);
#ifdef DEBUG
	      ::tet_infoline("proxy: findTCDef creating new node of type: Double\n");
#endif
	      break;
	  }
		    
	  case   9: {
	      n = NewNode(0,IdStringType);
#ifdef DEBUG
	      ::tet_infoline("proxy: findTCDef creating new node of type: String\n");
#endif
	      break;
	  }
		    
	  case  10: {
	      n = NewNode(0,IdStringType);
	      CORBA::ULong  b;
	      b = ernie.getULong()%30;
	      b++;

#ifdef DEBUG
    ::tet_printf("proxy: string bound of %d bytes\n", b);
#endif
	      PositiveIntConst *positiveIntConst = new PositiveIntConst;
	      ConstExp		*constExp 	 = new ConstExp;
	      OrExpr		*orExpr 	 = new OrExpr;
	      XorExpr		*xorExpr	 = new XorExpr;
	      AndExpr		*andExpr	 = new AndExpr;
	      ShiftExpr		*shiftExpr	 = new ShiftExpr;
	      AddExpr		*addExpr	 = new AddExpr;
	      MultExpr		*multExpr	 = new MultExpr;
	      UnaryExpr		*unaryExpr	 = new UnaryExpr;
	      PrimaryExpr       *primaryExpr     = new PrimaryExpr;
	      Literal		*literal         = new Literal;

	      ExprValue expr;

	      expr = b;
	      
	      (*literal) = expr;
	      (*primaryExpr) << literal;
	      (*unaryExpr) << primaryExpr;
	      (*multExpr) << unaryExpr;
	      (*addExpr) << multExpr;
	      (*shiftExpr) << addExpr;
	      (*andExpr) << shiftExpr;
	      (*xorExpr) << andExpr;
	      (*orExpr)	<< xorExpr;
	      (*constExp)  << orExpr;
	      (*positiveIntConst) << constExp;

	      (*(StringType*)(n)) << positiveIntConst;

#ifdef DEBUG
	      ::tet_infoline("proxy: findTCDef creating new node of type: BoundedString\n");
#endif
	      break;
	  }

	case   11: {
	    ScopedNameB scopedNameB;
	    _Identifier identifier;

	    identifier = "CORBA";
	    scopedNameB << identifier;
	    identifier = "TypeCode";
	    scopedNameB << identifier;	    	
	    
	    n = NewNode(0,IdScopedName);
	    (_ScopedName &)(*n) << scopedNameB;
	    
#ifdef DEBUG
	    ::tet_infoline("proxy: findTCDef creating new node of type: TypeCode\n");
#endif
	    break;
	}
		    
	case   12: {
	    n = NewNode(0,IdAnyType);
#ifdef DEBUG
	    ::tet_infoline("proxy: findTCDef creating new node of type: Any\n");
#endif
	    break;
	}
		    
	default: {
	    Node * p = 0;
	    p = getTypeByNumber(specification, numTypes-13);
#ifdef DEBUG
    ::tet_printf("proxy: found type at %x\n", p);
#endif

	    n = NewNode(p->Parent(), p->Id());
	    n->operator=(*p);
#ifdef DEBUG
	    tet_printf("proxy: looking for %d", numTypes-13);
#endif
	    break;
	      
	}
    }

    return n;
}

Node *
findElementSpec(Node &ptr, ExprValue exp, Node *& defaultNode)
{
    Node * ret;

	switch(ptr.Id())
	{
		case IdNotInitialised :
	    return 0;
		case IdCaseLabel :
		{
	    if (ptr.Choice() == 0)
	    {
		if (ptr[IdConstExp] == 0)
		{
		    tet_infoline("proxy: internal error in findElementSpec");
		    return 0;
		}
		
		if (exp == ptr[IdConstExp].exprValue())
		{
                  Node *p = ptr.Parent();
                  for (p;
			 p != 0 && p->Id() != IdCase;
			 p = p->Parent())
			;
		    
		    return &(*p)[IdElementSpec];
		}
	    }
	    else
	    {
		return findElementSpec(ptr[IdDefaultLabel],exp,defaultNode);
	    }
	}
	break;

  case IdCase :
		return findElementSpec(ptr[IdCaseLabelList],exp,defaultNode);
		break;
	
	case IdDefaultLabel :
	{
          Node *p = ptr.Parent();
          for (p;
		 p != 0 && p->Id() != IdCase;
		 p = p->Parent())
		;
	    defaultNode = &(*p)[IdElementSpec];
	}
	
	break;

	default:
	for (int i = 0; i < ptr.Length(); ++i)
	{
	    if ((ret = findElementSpec(ptr[i], exp, defaultNode)) != 0)
		return ret;
	}
    }
    return 0;
}


Node *
findCaseLabel(Node &ptr,int number)
{
  Node *p;
    
  if (ptr == IdNotInitialised)
      return 0;
  
  if (ptr == IdCaseLabel
      && number == CountPreceedingCaseLabels(ptr))
      return &ptr;
  
  for (int i = 0; i < ptr.Length(); ++i)
  {
      if ((p= findCaseLabel(ptr[i],number)) != 0)
	  return p;
  }
  
  return 0;
}


Node *
getTypeByNumberInternal(Node &ptr,int &n)
{
  Node *tmp;

 
  switch (ptr.Id())
  {
      case IdNotInitialised:
        return 0;
      case IdTypeDeclarator :
	if (n < ptr[IdDeclarators].Length())
	    return &ptr[IdDeclarators][n];
	else
	    n-=ptr[IdDeclarators].Length();
	break;
      case IdStructType :
      case IdUnionType :
      case IdEnumType :
	if (n == 0)
	    return &ptr;
	else
	    --n;
	break;
      case IdStringType:
	if (ptr.Parent() != 0 && ptr.Parent()->Id() ==
	    IdParamTypeSpec && ptr[0] != IdNotInitialised)
	{
	    if (n == 0)
		return &ptr;
	    else
		--n;
	}
	break;
  }
  
  for (int i = 0; i < ptr.Length(); ++i)
  {
      if ((tmp=getTypeByNumberInternal(ptr[i],n)) != 0)
	  return tmp;
  }
  return 0;
}

Node *
getTypeByNumber(Node &ptr,int n)
{
  return getTypeByNumberInternal(ptr,n);
}


int
findNextFAS(Node * child, Node * parent)
{
    int i = 0;

    for (i; i < parent->Length() - 1; i++)
    {
	if (child == &(*parent)[i])
	    return i+1;
    }
    return 0;
}



void BuildFullyScopedName_Internal(Node &ptr,ScopedNameB &sn)
{
	switch (ptr.Id())
	{
    case IdNotInitialised :
          return;
		case IdModule :
		case IdInterfaceDcl :
		case IdStructType :
		case IdUnionType :
		case IdExceptDcl :
		case IdOpDcl :
		case IdSimpleDeclarator :
          if (ptr.Parent() != 0)
				    BuildFullyScopedName_Internal(*ptr.Parent(),sn);
					sn << ptr[IdIdentifier];
					break;
		default:
          if (ptr.Parent() != 0)
  				  BuildFullyScopedName_Internal(*ptr.Parent(),sn);
	}
}

void BuildFullyScopedName(Node &ptr,_ScopedName &sn)
{
	ScopedNameA	scopedNameA;
	ScopedNameB	scopedNameB;

	if (ptr.Id() == IdNotInitialised)
		return;

  if (ptr.Parent() != 0)
    BuildFullyScopedName_Internal(*ptr.Parent(),scopedNameB);
	scopedNameB << ptr[IdIdentifier];
	sn << scopedNameA;
	sn << scopedNameB;
}
