//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/iiop/src/transport.cpp
//
// Modifications:
// $Log: transport.cpp,v $
// Revision 1.2  2000/05/24 14:38:07  corval2
// Fixed types and added include for <sys/time.h<, required by Linux
//
// Revision 1.1.1.1  2000/02/09 17:07:11  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.19  1999/04/29 11:21:57  cwatson
// Revision comments changed.
//
// Revision 1.18  1999/04/28 11:55:42  cwatson
// SR100 Moved the listen() call.
//
// Revision 1.17  1999/04/26 14:57:53  james
// Changed CORBA::Char* to char*
//
// Revision 1.16  1998/08/28 14:49:40  james
// new recv loop for when MSG_PEEK flag is set
//
// Revision 1.15  1998/06/04 13:54:34  stephen
// desctructor cleanup improved.
//
// Revision 1.14  1997/12/12 15:11:26  stephen
// Signed/unsigned mismatch in transmit/receive methods.
//
// Revision 1.13  1997/11/25 15:34:12  stephen
// Minor updates.
//
// Revision 1.12  1997/11/20 14:44:32  cwatson
// Make sure the socket number assigned is the higher
// of the server and client socket numbers.
//
// Revision 1.11  1997/11/01 15:26:55  ian
// Listen fd not initialised.
//
// Revision 1.10  1997/10/30 15:14:48  ian
// Incorect initialisation of temp var.
//
// Revision 1.9  1997/10/30 10:49:45  ian
// Macro replaced with an assignment.
//
// Revision 1.8  1997/10/28 15:25:33  stephen
// Added timeout to processEvent.
//
// Revision 1.7  1997/10/22 13:46:40  stephen
// Minor updates.
//
// Revision 1.6  1997/09/19 08:44:35  stephen
// Post merge edits.
//
// Revision 1.5  1997/09/18 14:40:06  damien
// Win32 changes
//
// Revision 1.4  1997/09/11 14:49:39  stephen
// Have processEvent() loop working now.
//
// Revision 1.3  1997/09/08 12:59:58  damien
// win32 changes
//
// Revision 1.2  1997/09/04 13:55:27  stephen
// Added initialise() to setup listening port and IP address.
//
// Revision 1.1  1997/09/03 09:35:20  stephen
// Initial TCP/IP support class.
//
//

#ifndef WIN32
#include <unistd.h>
#include <sys/time.h>
#endif
#include <transport.hh>
#include <tet_api.h>

Transport* Transport::m_Instance;

Transport* Transport::Instance() {
    if (m_Instance == 0)
	m_Instance = new Transport;

    return m_Instance;
};

Transport::Transport() :
    m_Connected(0),
    m_IPAddress(0),
    m_ClientFd(0),
    m_ServerFd(0),
    m_ListenFd(0),
    m_fdCount(0),
    m_Port(0)
{
};

CORBA::Boolean
Transport::initialise() {
#ifdef WIN32

    WORD wVersionRequested;
    WSADATA wsaData;
    int err;
 
    wVersionRequested = MAKEWORD( 2, 0 );
 
    err = WSAStartup( wVersionRequested, &wsaData );
    if ( err != 0 )
        return 0;


#endif

	FD_ZERO(&m_FdSet);

    struct hostent *he;
    struct sockaddr_in socketInfo;
    socklen_t len = sizeof(sockaddr_in);

    const int maxHost = 257;
    char hostName[maxHost];

    // Get out hostname in order to lookup our hostent struct
    if( gethostname(hostName, maxHost) )
	return 0;

    // Use the hostName to get IP address info
    if( (he = gethostbyname(hostName)) == 0 ) 
	return 0;
    
    memcpy(&socketInfo.sin_addr, he->h_addr, he->h_length);

    // Convert to a string
    if( (m_IPAddress = inet_ntoa(socketInfo.sin_addr)) == 0)
	return 0;

        // Now create a socket first
    if ((m_ListenFd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	return 0;

    socketInfo.sin_family = AF_INET;
    socketInfo.sin_addr.s_addr = htonl(INADDR_ANY);
    socketInfo.sin_port = htons(m_Port);

    // Bind to port number 0 to get a random port from the OS
    if ((bind(m_ListenFd, (struct sockaddr *)&socketInfo,
	      sizeof(sockaddr))) != 0)
	return 0;
    
#if 1
    listen(m_ListenFd, 10);
#endif

    // Now find out our allocated port number
    if (getsockname(m_ListenFd, (struct sockaddr *)&socketInfo, &len) < 0)
	return 0;
    
    m_Port = ntohs(socketInfo.sin_port);

    return 1;
};

Transport::~Transport() {

#ifdef WIN32
    closesocket(m_ClientFd);
    closesocket(m_ServerFd);
    closesocket(m_ListenFd);
    
    WSACleanup();
#else
    close(m_ClientFd);
    close(m_ServerFd);
    close(m_ListenFd);
#endif

    FD_CLR(m_ClientFd, &m_FdSet);
    FD_CLR(m_ServerFd, &m_FdSet);
};

CORBA::Long
Transport::connectTo(char *host, CORBA::UShort port)
{
    struct sockaddr_in socketInfo;
    struct hostent *hp;
    
    if (( m_ServerFd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	return -1;
    
    socketInfo.sin_family = AF_INET;
    socketInfo.sin_addr.s_addr = htonl(INADDR_ANY);
    socketInfo.sin_port = htons(port);
    
    if (!(hp = gethostbyname(host))) 
	return -1;
    
    memcpy(&socketInfo.sin_addr, hp->h_addr, hp->h_length);
    
    if ((connect(m_ServerFd, (struct sockaddr *) &socketInfo,
		 sizeof(sockaddr)))==-1)
	return -1;

    FD_SET(m_ServerFd, &m_FdSet);
    if (m_ServerFd > m_fdCount)
	m_fdCount = m_ServerFd;
    
    return m_ServerFd;
};

int
Transport::transmit(CORBA::ULong fd,
		    unsigned char* data,
		    CORBA::ULong length)
{
    int numBytes = 0;
#ifndef WIN32
    return numBytes = write(fd, (void *)data, length);
#else
    return numBytes = send(fd, (char *)data, length, 0);
#endif
};

int
Transport::receive(CORBA::ULong fd,
		   unsigned char*  data,
		   CORBA::ULong length,
		   CORBA::ULong flags)
{
    int totalSoFar = 0;
    int numBytes = 0;
    unsigned char * copy = data;

    if (flags != MSG_PEEK)
    {
      do 
      {
	numBytes = recv(fd, (char *)copy, length, flags);

	if (numBytes == -1)
	    return numBytes;
	
	copy += numBytes;
	totalSoFar = totalSoFar + numBytes;
	length = length - numBytes;
      } while( length != 0 );
      return totalSoFar;
    }
    else
    {
      int numTries = 0;

      numBytes = recv(fd, (char *)data, length, flags);
      while (numBytes != length && numTries++ < 10)
      {
#ifdef WIN32
        Sleep(1000);
#else
        sleep(1);
#endif
        numBytes = recv(fd, (char *)data, length, flags);
      }
      return numBytes;
    }
};


CORBA::Boolean
Transport::waitOnClient()
{
    struct sockaddr addr;
    socklen_t addrlen = sizeof(addr);

#if 0
    if (listen(m_ListenFd, 10) != 0)
	return FALSE;
    else
#endif
    {
	m_ClientFd = accept(m_ListenFd, &addr, &addrlen);
	
	if (m_ClientFd < 0)
	    return FALSE;
	
	FD_SET(m_ClientFd, &m_FdSet);
	if (m_ClientFd > m_fdCount)
	    m_fdCount = m_ClientFd;
    }
    AlreadyConnected(TRUE);
    return TRUE;
};

CORBA::Boolean
Transport::isEventPending ()
{
    fd_set tmpFdSet;
    
    struct timeval timeout;
    timeout.tv_usec = 1;
    timeout.tv_sec = 1;
    CORBA::ULong result = 0;

    if (!AlreadyConnected()) 
	waitOnClient();

memcpy(&tmpFdSet, &m_FdSet, sizeof(fd_set));

    result = select(m_fdCount+1, &tmpFdSet, NULL, NULL, &timeout);

    return (result != 0);
}

CORBA::Long
Transport::processEvent(CORBA::ULong timeout)
{
    fd_set tmpFdSet;
    
    struct timeval t;
    t.tv_usec = 0;
    t.tv_sec = timeout;
    int result = 0;

    if (!AlreadyConnected()) 
	waitOnClient();

    memcpy(&tmpFdSet, &m_FdSet, sizeof(fd_set));
    
    result = select(m_fdCount+1, &tmpFdSet, NULL, NULL, &t);

#ifdef WIN32
    if (result == SOCKET_ERROR)
    {
	tet_printf("proxy: transport error: %d", WSAGetLastError()); 
	return -1;
    }
#else
    if (result == -1)
    {	
	tet_printf("proxy: transport error: %d", errno); 
	return -1;
    }
#endif
    
    if (result > 0)
    {
    	for (CORBA::Long j = 0; j < m_fdCount+1; j++) {
	    if (FD_ISSET(j,&tmpFdSet))		
		return j;
	}
    }
    return 0;
}
