//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <iostream.h>
#include <fstream.h>
#include <string.h>

#ifdef IIOP_FUNCTIONS
#include "vsorb.hh"
#else
#include "corba_supp.hh"
#endif

#include "ernie.hh"

static const char *Printables = " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";

Ernie::Ernie(void):m_zranx(21288629),m_zrany(362436069),m_zranz(1613801),
m_zranc(1),m_zrann(1131199209)
{
    lastString = 0;
}

Ernie::~Ernie(void)
{
    if (lastString != 0)
	delete lastString;
}

void 
Ernie::seed(CORBA::ULong newVal)
{
    m_zrann=newVal;

    m_zranx=21288629;
    m_zrany=362436069;
    m_zranz=1613801;
    m_zranc=1;

    anyCount=0;
    sequenceCount=0;
    inoutMode = 0;
}

void 
Ernie::seed(char *newVal)
{
    CORBA::ULong	x=1;
    char	*ptr;

    for (ptr = newVal; *ptr != '\0'; ++ptr)
	x*=(CORBA::ULong)*ptr;
    
    m_zrann=x;

    m_zranx=21288629;
    m_zrany=362436069;
    m_zranz=1613801;
    m_zranc=1;

    anyCount=0;
    sequenceCount=0;
    inoutMode = 0;
}

void 
Ernie::seed(const char *newVal)
{
    CORBA::ULong	x=1;
    const char	*ptr;

    for (ptr = newVal; *ptr != '\0'; ++ptr)
	x*=(CORBA::ULong)*ptr;
    
    m_zrann=x;

    m_zranx=21288629;
    m_zrany=362436069;
    m_zranz=1613801;
    m_zranc=1;

    anyCount=0;
    sequenceCount=0;
    inoutMode = 0;
}

void 
Ernie::mode(CORBA::ULong m)
{
  // 0 = NOT an inout parameter (Normal behaviour)
  // 1 = INOUT parameter and handling request
  // 2 = INOUT parameter and handling reply
  inoutMode = m;
}

CORBA::ULong 
Ernie::seed(void)
{
    return m_zrann;
}

CORBA::ULong
Ernie::randULong(void)
{
    CORBA::Long s;
    if(m_zrany>m_zranx+m_zranc)
    {
	s=m_zrany-(m_zranx+m_zranc);
	m_zranc=0;
    }
    else
    {
	s=(m_zranx+m_zranc)-m_zrany-18;
	m_zranc=1;
    }
    m_zranx=m_zrany;
    m_zrany=m_zranz; 
    return ((m_zranz=s) + (m_zrann=69069*m_zrann+1013904243));
}

CORBA::ULong
Ernie::getSequenceLength(CORBA::ULong bound)
{
  lastULong = 0;

  // INOUT sequences are always 0 in replies
  if (inoutMode != 2)
  {
    if (++sequenceCount < 100)
      lastULong = randULong() % ((bound!=0)?bound:3);
  }
  
  return lastULong;
}

CORBA::ULong
Ernie::getULong(void)
{
    lastULong = randULong();
    return lastULong;
}

CORBA::Long
Ernie::getLong(void)
{
    CORBA::ULong p = randULong();
    CORBA::Long q = p>>1;
    lastLong = (p>2147483647)?q:-q;
    return lastLong;
}

CORBA::UShort
Ernie::getUShort(void)
{
    CORBA::ULong p = randULong();
    lastUShort = (CORBA::UShort)(p%65536);		
    return lastUShort;
}

CORBA::Short
Ernie::randShort(void)
{
    CORBA::ULong p = randULong();
    return (CORBA::Short)(p%65536);	
}

CORBA::Short
Ernie::getShort(void)
{
    lastShort = randShort();
    return lastShort;
}

CORBA::Boolean
Ernie::getBoolean(void)
{
    CORBA::ULong p = randULong();
    lastBoolean = (CORBA::Boolean)(p%2);
    return lastBoolean;
}

CORBA::Octet
Ernie::getOctet(void)
{
    CORBA::ULong p = randULong();
    lastOctet = (CORBA::Octet)(p%256);
		return lastOctet;
}

CORBA::Char
Ernie::randChar(void)
{
    CORBA::ULong p = randULong();
    return (CORBA::Char)(p%256);	
}

CORBA::Char
Ernie::getChar(void)
{
    lastChar = randChar();
    return lastChar;
}

CORBA::Float
Ernie::getFloat(void)
{
    char tmp[64];
    char *stop;
    float s;
    CORBA::Short p = randShort()%10;
    CORBA::ULong q = randULong();
    CORBA::Short r = randShort()%38;

    sprintf(tmp,"%d.%ue%d",p,q,r);    
    lastFloat = (CORBA::Float)strtod(tmp,&stop);	
    return lastFloat;
}

CORBA::Double
Ernie::getDouble(void)
{
    char tmp[128];
    char *stop;
    CORBA::Short p = randShort()%10;
    CORBA::ULong q = randULong();
    CORBA::Short r = randShort()%308;

    sprintf(tmp,"%d.%ue%d",p,q,r);
    lastDouble = (CORBA::Double)strtod(tmp,&stop); 
    return lastDouble;
}



char*
Ernie::getString(int min, int max)
{
    CORBA::ULong	l;
    int	i;

  	l = min + getULong()%(max-min);

    if (lastString)
    {
	delete lastString;
	lastString = 0;
    }
    lastString = new char[l+1];

    for (i = 0; i < l; ++i)
    {
	lastString[i] = Printables[randULong() % strlen(Printables)];
    }
    lastString[i] = '\0';

    return(lastString);
}

char*
Ernie::getString(void)
{
  switch (inoutMode) {
  default: // NOT an inout parameter (Normal behaviour)
    return getString(0,100);
  case 1: // INOUT parameter and handling request
    return getString(20,100);
  case 2: // INOUT parameter and handling reply
    return getString(0,20);
  }
}
  

char*
Ernie::getString(int max)
{
  int min = 20;

  if (max > 100)
    max = 100;

  if (max <= 20)
    min = max/2;

  switch (inoutMode) {
  default: // NOT an inout parameter (Normal behaviour)
    return getString(0,max);
  case 1: // INOUT parameter and building request
    return getString(min,max);
  case 2: // INOUT parameter and building reply
    return getString(0,min);
  }
}


CORBA::ULong 
Ernie::getULongAgain(void)
{
    return lastULong;
}

CORBA::Long 
Ernie::getLongAgain(void)
{
    return lastLong;
}

CORBA::Short 
Ernie::getShortAgain(void)
{
    return lastShort;
}

CORBA::UShort 
Ernie::getUShortAgain(void)
{
    return lastUShort;
}

CORBA::Boolean 
Ernie::getBooleanAgain(void)
{
    return lastBoolean;
}

CORBA::Octet 
Ernie::getOctetAgain(void)
{
    return lastOctet;
}

CORBA::Char 
Ernie::getCharAgain(void)
{
    return lastChar;
}

CORBA::Float 
Ernie::getFloatAgain(void)
{
    return lastFloat;
}

CORBA::Double 
Ernie::getDoubleAgain(void)
{
    return lastDouble;
}

char* 
Ernie::getStringAgain(void)
{
    return lastString;
}


int Ernie::checkULong(const CORBA::ULong v)
{
    return (v == getULong());
}

int Ernie::checkLong(const CORBA::Long v)
{
    return (v == getLong());
}

int Ernie::checkUShort(const CORBA::UShort v)
{
    return (v == getUShort());
}

int Ernie::checkShort(const CORBA::Short v)
{
    return (v == getShort());
}

int Ernie::checkBoolean(const CORBA::Boolean v)
{
    return (v == getBoolean());
}

int Ernie::checkOctet(const CORBA::Octet v)
{
    return (v == getOctet());
}

int Ernie::checkChar(const CORBA::Char v)
{
    return (v == getChar());
}

int Ernie::checkFloat(const CORBA::Float v)
{
    CORBA::Float delta;
    CORBA::Float min;
    CORBA::Float max;
    
    delta = getFloat()/1000;
    if (delta < 0.0)
	delta = -delta;

    max = getFloatAgain() + delta;
    min = getFloatAgain() - delta;

    return ((v >= min) && (v <= max));
}

int Ernie::checkDouble(const CORBA::Double v)
{
    CORBA::Double  delta;
    
    delta = getDouble()/100000;
    if (delta < 0.0)
	delta = -delta;
    
    return ((v >= (getDoubleAgain()-delta)) && (v <= (getDoubleAgain()+delta)));
}

int Ernie::checkString(const char* v)
{
    return (!strcmp(v,getString()));
}

int Ernie::checkString(const char* v,int b)
{
    return (!strcmp(v,getString(b)));
}



char *ULongToString(CORBA::ULong p)
{
    static char	buf[100];

    sprintf(buf,"%u",(unsigned long)p);
    return buf;
}

char *LongToString(CORBA::Long p)
{
    static char	buf[100];

    sprintf(buf,"%d", (long)p);
    return buf;
}

char *ShortToString(CORBA::Short p)
{
    static char	buf[100];

    sprintf(buf,"%d", (short)p);
    return buf;
}

char *UShortToString(CORBA::UShort p)
{
    static char	buf[100];

    sprintf(buf,"%u", (unsigned short)p);
    return buf;
}

const char *BooleanToString(CORBA::Boolean p)
{
    static char	buf[100];

    if (p == (CORBA::Boolean)1)
	return "TRUE";
    if (p == (CORBA::Boolean)0)
	return "FALSE";

    sprintf(buf,"%d",  (unsigned long)p);
    return buf;
}

char *OctetToString(CORBA::Octet p)
{
    static char	buf[100];

    sprintf(buf,"'\\x%x'", (unsigned long)p);
    return buf;
}

char *CharToString(CORBA::Char p)
{
  static char	buf[100];

  if (p != 0 && strchr("\\\?\'\"",(int)p) != 0)
    sprintf(buf,"'\\%c'",p);
  else if (isprint(p))
	  sprintf(buf,"'%c'", (char)p);
  else
	  sprintf(buf,"'\\x%x'", (unsigned long)p);

    return buf;
}

char *FloatToString(CORBA::Float p)
{
    static char	buf[100];

    sprintf(buf,"%g", (float)p);
    return buf;
}

char *DoubleToString(CORBA::Double p)
{
    static char	buf[100];

    sprintf(buf,"%g", (double)p);
    return buf;
}

char *StringToString(char *p)
{
  static char	buf[1024];
	char	buf1[10];
  unsigned char *ptr;
  buf[0] = '\0';

  for (ptr = (unsigned char *)p; *ptr != 0; ++ptr)
  {
    if (strchr("\\\?\'\"",(int)*ptr) != 0)
      sprintf(buf1,"\\%c",*ptr);
    else if (isprint(*ptr))
		  sprintf(buf1,"%c", *ptr);
	  else
		  sprintf(buf1,"\\x%2.2x", (unsigned long)*ptr);

    strcat(buf,buf1);
    ++p;
  }
  
  return buf;
}

