//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2000 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#ifndef _EXPRVALUE_HH
#define _EXPRVALUE_HH

#include "types.hh"

typedef enum {VSORB_False, VSORB_True} VSORB_Boolean;


class ExprValue
{
  EvalType  type;

  union {
    long                 Long;
    unsigned long       ULong;
    double               Double;
    VSORB_Boolean        Boolean;
    unsigned char       Char;
    char                 *String;
  }value;

  // Some useful functions
  void CorrectIntegerType();
  int  IsInteger() const;

public:
  ExprValue();
  ExprValue(ExprValue &);
  ExprValue(const ExprValue &);
  ~ExprValue();


  // Assignment Operators
  ExprValue &operator=(const long);
  ExprValue &operator=(const unsigned long);
  ExprValue &operator=(const double);
  ExprValue &operator=(const VSORB_Boolean);
  ExprValue &operator=(const unsigned char);
  ExprValue &operator=(const char *);
  ExprValue &operator=(const ExprValue &);

  // Cast Operators
  operator long() const;
  operator unsigned long() const;
  operator unsigned char() const;
  operator VSORB_Boolean() const;

  // Unary Operators
  ExprValue operator-();
  ExprValue operator+();
  ExprValue operator~();
  ExprValue &operator--();
  ExprValue &operator++();

  // Infix Operators
  ExprValue operator*(const ExprValue &);
  ExprValue operator/(const ExprValue &);
  ExprValue operator+(const ExprValue &);
  ExprValue operator-(const ExprValue &);
  ExprValue operator%(const ExprValue &);
  ExprValue operator<<(const ExprValue &);
  ExprValue operator>>(const ExprValue &);
  ExprValue operator&(const ExprValue &);
  ExprValue operator|(const ExprValue &);
  ExprValue operator^(const ExprValue &);

  int operator>(const long) const;
  int operator<(const long) const;
  int operator>=(const long) const;
  int operator<=(const long) const;
  int operator==(const long) const;
  int operator==(const ExprValue &) const;
  int operator!=(const long) const;

  long PreserveTypeAssign(long v);
  EvalType  Type() const;
  char *string() const;
};

fstream &operator<<(fstream &output, const ExprValue v);
ostream &operator<<(ostream &output, const ExprValue v);

#endif

