//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/nodes/include/grammar.hh
//
// Modifications:
// $Log: grammar.hh,v $
// Revision 1.2  2000/02/11 16:03:15  mli
// Return type of some decode* methods from
// 'unsigned long ' changed to 'long'.
//
// Revision 1.1.1.1  2000/02/09 17:07:13  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.6  1998/05/11 10:29:37  stephen
// exprValue() and decodeTypeCode() added to some classes.
//
// Revision 1.5  1997/12/04 14:14:12  stephen
// IIOP functions added to ObjectType and ContextExpr.
//
// Revision 1.4  1997/11/21 13:02:25  stephen
// Extra decodeTypeCode functions.
//
// Revision 1.3  1997/10/20 17:29:36  stephen
// Added case label counting functions.
//
// Revision 1.2  1997/10/14 10:23:56  james
// Extended types removed
//
// Revision 1.1  1997/10/06 11:35:47  james
// Initial Sources


#ifndef _GRAMMAR_HH
#define _GRAMMAR_HH

#include "node.hh"
class IOR;


// Not part of the grammar, but used to store list of previously defined types
class ScopedNameBList : public ListNode
{
public:
  ScopedNameBList();
  ScopedNameBList(Node *p, RecursionCount r);
  ScopedNameBList(const ScopedNameBList &);
};



class Specification : public ListNode
{
public:
  Specification();
  Specification(Node *p, RecursionCount r);
  Specification(const Specification &);
};

class Definition : public ChoiceNode
{
public:
  Definition();
  Definition(Node *p, RecursionCount r);
  Definition(const Definition &);
};


class Module : public SequenceNode
{
public:
  Module();
  Module(Node *p, RecursionCount r);
  Module(const Module &);
};


class Definitions : public ListNode
{
public:
  Definitions();
  Definitions(Node *p, RecursionCount r);
  Definitions(const Definitions &);
};



class Interface : public ChoiceNode
{
public:
  Interface();
  Interface(Node *p, RecursionCount r);
  Interface(const Interface &);
};


class InterfaceDcl : public SequenceNode
{
public:
  InterfaceDcl();
  InterfaceDcl(Node *p, RecursionCount r);
  InterfaceDcl(const InterfaceDcl &);
};


class ForwardDcl : public SequenceNode
{
public:
  ForwardDcl();
  ForwardDcl(Node *p, RecursionCount r);
  ForwardDcl(const ForwardDcl &);
};


class InterfaceBody : public ListNode
{
public:
  InterfaceBody();
  InterfaceBody(Node *p, RecursionCount r);
  InterfaceBody(const InterfaceBody &);
};


class Export : public ChoiceNode
{
public:
  Export();
  Export(Node *p, RecursionCount r);
  Export(const Export &);
};


class InheritanceSpec : public ListNode
{
  ScopedNameBList  exceptions;
  int  index;

public:
  InheritanceSpec();
  InheritanceSpec(Node *p, RecursionCount r);
  InheritanceSpec(const InheritanceSpec &);

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();
#endif
};




class _ScopedName : public SequenceNode
{
  ScopedNameBList  possibleValues;
  int              index;

  Node *value;
public:
  _ScopedName();
  _ScopedName(Node *p, RecursionCount r);
  _ScopedName(const _ScopedName &);
  ~_ScopedName();

  ExprValue exprValue();

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();
  int assignValue(ExprValue, ExprValue);
#endif
};


class ScopedNameA : public Node
{
public:
  ScopedNameA();
  ScopedNameA(Node *p, RecursionCount r);
  ScopedNameA(const ScopedNameA &);
};

class ScopedNameB : public ListNode
{
public:
  ScopedNameB();
  ScopedNameB(Node *p, RecursionCount r);
  ScopedNameB(const ScopedNameB &);
};


class ConstDcl : public SequenceNode
{
  int SetRange(ExprValue &GreaterThan, ExprValue &LessThan);

public:
  ConstDcl();
  ConstDcl(Node *p, RecursionCount r);
  ConstDcl(const ConstDcl &);

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();
#endif
};




class ConstType : public ChoiceNode
{
public:
  ConstType();
  ConstType(Node *p, RecursionCount r);
  ConstType(const ConstType &);
};


class PositiveIntConst : public ChoiceNode
{
public:
  PositiveIntConst();
  PositiveIntConst(Node *p, RecursionCount r);
  PositiveIntConst(const PositiveIntConst &);

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();
#endif
};


class TypeDcl : public ChoiceNode
{
public:
  TypeDcl();
  TypeDcl(Node *p, RecursionCount r);
  TypeDcl(const TypeDcl &);
};


class TypeDeclarator : public SequenceNode
{
public:
  TypeDeclarator();
  TypeDeclarator(Node *p, RecursionCount r);
  TypeDeclarator(const TypeDeclarator &);


#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  

};



class TypeSpec : public ChoiceNode
{
public:
  TypeSpec();
  TypeSpec(Node *p, RecursionCount r);
  TypeSpec(const TypeSpec &);
};




class SimpleTypeSpec : public ChoiceNode
{
public:
  SimpleTypeSpec();
  SimpleTypeSpec(Node *p, RecursionCount r);
  SimpleTypeSpec(const SimpleTypeSpec &);
};









class BaseTypeSpec : public ChoiceNode
{
public:
  BaseTypeSpec();
  BaseTypeSpec(Node *p, RecursionCount r);
  BaseTypeSpec(const BaseTypeSpec &);
};


class ObjectType : public Node
{
    IOR * value;
    
public:
  ObjectType();
  ObjectType(Node *p, RecursionCount r);
  ObjectType(const ObjectType &);
#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};





class TemplateTypeSpec : public ChoiceNode
{
public:
  TemplateTypeSpec();
  TemplateTypeSpec(Node *p, RecursionCount r);
  TemplateTypeSpec(const TemplateTypeSpec &);
};




class ConstrTypeSpec : public ChoiceNode
{
public:
  ConstrTypeSpec();
  ConstrTypeSpec(Node *p, RecursionCount r);
  ConstrTypeSpec(const ConstrTypeSpec &);
};


class Declarators : public ListNode
{
public:
  Declarators();
  Declarators(Node *p, RecursionCount r);
  Declarators(const Declarators &);
};



class Declarator : public ChoiceNode
{
public:
  Declarator();
  Declarator(Node *p, RecursionCount r);
  Declarator(const Declarator &);

#ifdef IIOP_FUNCTIONS
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class SimpleDeclarator : public SequenceNode
{
  Node *value;
public:
  SimpleDeclarator();
  SimpleDeclarator(Node *p, RecursionCount r);
  SimpleDeclarator(const SimpleDeclarator &);
  ~SimpleDeclarator();

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class ComplexDeclarator : public SequenceNode
{
public:
  ComplexDeclarator();
  ComplexDeclarator(Node *p, RecursionCount r);
  ComplexDeclarator(const ComplexDeclarator &);
};




class FloatingPtType : public ChoiceNode
{
public:
  FloatingPtType();
  FloatingPtType(Node *p, RecursionCount r);
  FloatingPtType(const FloatingPtType &);
};

class FloatType : public Node
{
  float value;
public:
  FloatType();
  FloatType(Node *p, RecursionCount r);
  FloatType(const FloatType &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};

class DoubleType : public Node
{
  double value;
public:
  DoubleType();
  DoubleType(Node *p, RecursionCount r);
  DoubleType(const DoubleType &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class IntegerType : public ChoiceNode
{
public:
  IntegerType();
  IntegerType(Node *p, RecursionCount r);
  IntegerType(const IntegerType &);
};




class SignedInt : public ChoiceNode
{
public:
  SignedInt();
  SignedInt(Node *p, RecursionCount r);
  SignedInt(const SignedInt &);
};


class SignedLongInt : public Node
{
  long value;
public:
  SignedLongInt();
  SignedLongInt(Node *p, RecursionCount r);
  SignedLongInt(const SignedLongInt &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class SignedShortInt : public Node
{
  short value;
public:
  SignedShortInt();
  SignedShortInt(Node *p, RecursionCount r);
  SignedShortInt(const SignedShortInt &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};




class UnsignedInt : public ChoiceNode
{
public:
  UnsignedInt();
  UnsignedInt(Node *p, RecursionCount r);
  UnsignedInt(const UnsignedInt &);
};



class UnsignedLongInt : public Node
{
  unsigned long value;
public:
  UnsignedLongInt();
  UnsignedLongInt(Node *p, RecursionCount r);
  UnsignedLongInt(const UnsignedLongInt &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class UnsignedShortInt : public Node
{
  unsigned short value;
public:
  UnsignedShortInt();
  UnsignedShortInt(Node *p, RecursionCount r);
  UnsignedShortInt(const UnsignedShortInt &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class CharType : public Node
{
  char value;
public:
  CharType();
  CharType(Node *p, RecursionCount r);
  CharType(const CharType &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class BooleanType : public Node
{
  unsigned char value;
public:
  BooleanType();
  BooleanType(Node *p, RecursionCount r);
  BooleanType(const BooleanType &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class OctetType : public Node
{
  unsigned char value;
public:
  OctetType();
  OctetType(Node *p, RecursionCount r);
  OctetType(const OctetType &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class AnyType : public Node
{
  Node * value;  
public:
  AnyType();
  ~AnyType();
  AnyType(Node *p, RecursionCount r);
  AnyType(const AnyType &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class StructType : public SequenceNode
{
public:
  StructType();
  StructType(Node *p, RecursionCount r);
  StructType(const StructType &);
#ifdef IIOP_FUNCTIONS
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class MemberList : public ListNode
{
public:
  MemberList();
  MemberList(Node *p, RecursionCount r);
  MemberList(const MemberList &);
};


class Member : public SequenceNode
{
public:
  Member();
  Member(Node *p, RecursionCount r);
  Member(const Member &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};



class UnionType : public SequenceNode
{
public:
  UnionType();
  UnionType(Node *p, RecursionCount r);
  UnionType(const UnionType &);
};

class EnumType : public SequenceNode
{
  unsigned long value;
public:
  EnumType();
  EnumType(Node *p, RecursionCount r);
  EnumType(const EnumType &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class UnionSwitch : public SequenceNode
{
  ExprValue d;
  Node *value;

#ifdef GENERATOR_FUNCTIONS
  ExprValue values[MAX_LABELS];
  EnumType  enumType;
  int       enumIndex;
#endif

public:
  UnionSwitch();
  UnionSwitch(Node *p, RecursionCount r);
  UnionSwitch(const UnionSwitch &);
  ~UnionSwitch();

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();

  void CreateValues();
  Node *CreateLabel();
#endif

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};



class SwitchTypeSpec : public ChoiceNode
{
public:
  SwitchTypeSpec();
  SwitchTypeSpec(Node *p, RecursionCount r);
  SwitchTypeSpec(const SwitchTypeSpec &);
};


class SwitchBody : public ListNode
{
public:
  SwitchBody();
  SwitchBody(Node *p, RecursionCount r);
  SwitchBody(const SwitchBody &);

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();
#endif
};


class Case : public SequenceNode
{
public:
  Case();
  Case(Node *p, RecursionCount r);
  Case(const Case &);

#ifdef IIOP_FUNCTIONS
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class CaseLabelList : public ListNode
{
public:
  CaseLabelList();
  CaseLabelList(Node *p, RecursionCount r);
  CaseLabelList(const CaseLabelList &);
};



class CaseLabel : public ChoiceNode
{
public:
  CaseLabel();
  CaseLabel(Node *p, RecursionCount r);
  CaseLabel(const CaseLabel &);

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();
#endif

#ifdef IIOP_FUNCTIONS
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};

class DefaultLabel : public Node
{
public:
  DefaultLabel();
  DefaultLabel(Node *p, RecursionCount r);
  DefaultLabel(const DefaultLabel &);
};


class ElementSpec : public SequenceNode
{
public:
  ElementSpec();
  ElementSpec(Node *p, RecursionCount r);
  ElementSpec(const ElementSpec &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class EnumeratorList : public ListNode
{
public:
  EnumeratorList();
  EnumeratorList(Node *p, RecursionCount r);
  EnumeratorList(const EnumeratorList &);
};


class Enumerator : public SequenceNode
{
public:
  Enumerator();
  Enumerator(Node *p, RecursionCount r);
  Enumerator(const Enumerator &);

  ExprValue exprValue();

#ifdef IIOP_FUNCTIONS
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};



class SequenceType : public SequenceNode
{
  unsigned long seqLen;
  SimpleTypeSpec *value;
public:
  SequenceType();
  SequenceType(Node *p, RecursionCount r);
  SequenceType(const SequenceType &);
  ~SequenceType();

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};



class StringType : public SequenceNode
{
  char *value;
public:
  StringType();
  StringType(Node *p, RecursionCount r);
  StringType(const StringType &);
  ~StringType();

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class ArrayDeclarator : public SequenceNode
{
  unsigned long seqLen;
  TypeSpec *value;
public:
  ArrayDeclarator();
  ArrayDeclarator(Node *p, RecursionCount r);
  ArrayDeclarator(const ArrayDeclarator &);
  ~ArrayDeclarator();

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
#endif  
};


class FixedArraySizeList : public ListNode
{
public:
  FixedArraySizeList();
  FixedArraySizeList(Node *p, RecursionCount r);
  FixedArraySizeList(const FixedArraySizeList &);

  ExprValue exprValue();

#ifdef IIOP_FUNCTIONS
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};


class FixedArraySize : public ChoiceNode
{
public:
  FixedArraySize();
  FixedArraySize(Node *p, RecursionCount r);
  FixedArraySize(const FixedArraySize &);

#ifdef IIOP_FUNCTIONS
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif  
};



class AttrDcl : public SequenceNode
{
public:
  AttrDcl();
  AttrDcl(Node *p, RecursionCount r);
  AttrDcl(const AttrDcl &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
//  long decodeTypeCode(IIOPMessageHeader &msg);
#endif
};


class Readonly : public Node
{
public:
  Readonly();
  Readonly(Node *p, RecursionCount r);
  Readonly(const Readonly &);
};


class SimpleDeclaratorList : public ListNode
{
public:
  SimpleDeclaratorList();
  SimpleDeclaratorList(Node *p, RecursionCount r);
  SimpleDeclaratorList(const SimpleDeclaratorList &);
};


class ExceptDcl : public SequenceNode
{
public:
  ExceptDcl();
  ExceptDcl(Node *p, RecursionCount r);
  ExceptDcl(const ExceptDcl &);
#ifdef IIOP_FUNCTIONS
  long decodeTypeCode(IIOPMessageHeader &msg);
#endif
};



class OpDcl : public SequenceNode
{
public:
  OpDcl();
  OpDcl(Node *p, RecursionCount r);
  OpDcl(const OpDcl &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
#endif  
};



class OpTypeSpec : public ChoiceNode
{
public:
  OpTypeSpec();
  OpTypeSpec(Node *p, RecursionCount r);
  OpTypeSpec(const OpTypeSpec &);
};


class OpTypeSpecA : public SequenceNode
{
public:
  OpTypeSpecA();
  OpTypeSpecA(Node *p, RecursionCount r);
  OpTypeSpecA(const OpTypeSpecA &);
};

class OpTypeSpecB : public Node
{
public:
  OpTypeSpecB();
  OpTypeSpecB(Node *p, RecursionCount r);
  OpTypeSpecB(const OpTypeSpecB &);
};

class OpAttribute : public Node
{
public:
  OpAttribute();
  OpAttribute(Node *p, RecursionCount r);
  OpAttribute(const OpAttribute &);
};

class ParameterDcls : public SequenceNode
{
public:
  ParameterDcls();
  ParameterDcls(Node *p, RecursionCount r);
  ParameterDcls(const ParameterDcls &);
};


class ParameterDclsA : public ListNode
{
public:
  ParameterDclsA();
  ParameterDclsA(Node *p, RecursionCount r);
  ParameterDclsA(const ParameterDclsA &);
};



class ParamDcl : public SequenceNode
{
public:
  ParamDcl();
  ParamDcl(Node *p, RecursionCount r);
  ParamDcl(const ParamDcl &);

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
#endif  
};




class ParamAttribute : public SimpleChoiceNode
{
public:
  ParamAttribute();
  ParamAttribute(Node *p, RecursionCount r);
  ParamAttribute(const ParamAttribute &);
};



class RaisesExpr : public ListNode
{
  ScopedNameBList  exceptions;
  int  index;

public:
  RaisesExpr();
  RaisesExpr(Node *p, RecursionCount r);
  RaisesExpr(const RaisesExpr &);

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();
#endif
};



class ContextExpr : public ListNode
{
public:
  ContextExpr();
  ContextExpr(Node *p, RecursionCount r);
  ContextExpr(const ContextExpr &);
#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader &msg);
#endif  
};






class ParamTypeSpec : public ChoiceNode
{
public:
  ParamTypeSpec();
  ParamTypeSpec(Node *p, RecursionCount r);
  ParamTypeSpec(const ParamTypeSpec &);
};



#endif

