//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/utils/nodes/include/literals.hh
//
// Modifications:
// $Log: literals.hh,v $
// Revision 1.1.1.1  2000/02/09 17:07:13  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1999/07/05 15:03:46  cwatson
// Added dtor to StringLiteral class
//
// Revision 1.3  1997/12/04 14:14:42  stephen
// Cast operator added for StringLIteral type.
//
// Revision 1.2  1997/10/14 10:23:57  james
// Extended types removed
//
// Revision 1.1  1997/10/06 11:35:48  james
// Initial Sources
//
//

#ifndef _LITERALS_HH
#define _LITERALS_HH

#include "node.hh"

#define MAX_LEN_IDENTIFIER 100

class _Identifier : public Node
{
  char  *value;

public:
  _Identifier();
  _Identifier(Node *p, RecursionCount r);
  _Identifier(const _Identifier &r);
  ~_Identifier();

  Node &operator=(const Node &);
  Node &operator=(const char *);
  int operator==(const _Identifier &) const;

  operator char *();

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
#endif
};

class IntegerLiteral : public Node
{
  ExprValue value;

public:
  IntegerLiteral();
  IntegerLiteral(Node *p, RecursionCount r);
  IntegerLiteral(const IntegerLiteral &r);

  Node &operator=(const char *);
  Node &operator=(const Node &);

  ExprValue exprValue();
#ifdef GENERATOR_FUNCTIONS
  int assignValue(ExprValue, ExprValue);
#endif
};


class CharacterLiteral : public Node
{
  char  *value;

public:
  CharacterLiteral();
  CharacterLiteral(Node *p, RecursionCount r);
  CharacterLiteral(const CharacterLiteral &r);
  ~CharacterLiteral();

  Node &operator=(const char *);
  Node &operator=(const Node &);

#ifdef GENERATOR_FUNCTIONS
  int  Initialise();
#endif

  ExprValue exprValue();
};


class StringLiteral : public Node
{
  char  *value;

public:
  StringLiteral();
  StringLiteral(Node *p, RecursionCount r);
  StringLiteral(const StringLiteral &r);
  ~StringLiteral();

  Node &operator=(const char *);
  Node &operator=(const Node &);

  ExprValue exprValue();

  operator char * ();
};


class FloatingPtLiteral : public Node
{
  ExprValue value;

public:
  FloatingPtLiteral();
  FloatingPtLiteral(Node *p, RecursionCount r);
  FloatingPtLiteral(const FloatingPtLiteral &r);

  Node &operator=(const char *);
  Node &operator=(const Node &);

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
#endif

  ExprValue exprValue();
};


class BooleanLiteral : public SimpleChoiceNode
{
public:
  BooleanLiteral();
  BooleanLiteral(Node *p, RecursionCount r);
  BooleanLiteral(const BooleanLiteral &r);
 
  Node &operator=(const char *);
 
 
  ExprValue exprValue();
};



class Literal : public ChoiceNode
{
public:
  Literal();
  Literal(Node *p, RecursionCount r);

  Literal &operator=(ExprValue &);

};

 
#endif

