//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/utils/nodes/include/node.hh
//
// Modifications:
// $Log: node.hh,v $
// Revision 1.2  2000/02/11 16:03:00  mli
// Return type of some decode* methods from
// 'unsigned long ' changed to 'long'.
//
// Revision 1.1.1.1  2000/02/09 17:07:13  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.3  1998/05/11 10:31:02  stephen
// Index operator now returns a Node&.
//
// Revision 1.2  1997/10/16 14:23:38  james
// Reduced memory usage
//
// Revision 1.1  1997/10/06 11:35:49  james
// Initial Sources
//

#ifndef _NODE_HH
#define _NODE_HH

#include "exprvalue.hh"

#ifdef IIOP_FUNCTIONS
class IIOPMessageHeader;
#endif

typedef unsigned char RecursionCount[MAX_NODE_ID];
#define MAX_CHILD_IDS 10

class Node
{
protected:
  Node            *parent;
  NodeId          nodeId;

#ifdef GENERATOR_FUNCTIONS
  RecursionCount  recursionCount;
#endif

public:
  Node();
  Node(NodeId n);
  Node(Node *, NodeId n, RecursionCount r);
  Node(const Node &);
  virtual ~Node();

  NodeId  Id() const;
  operator NodeId() const;
  Node    *Parent() const;
  void    Parent(Node *);
  virtual int Choice() const;

#ifdef GENERATOR_FUNCTIONS
  virtual int Initialise();
  virtual int Next();
  virtual int assignValue(ExprValue GreaterThan, ExprValue LessThan);
#endif

  virtual Node &operator=(const Node &);
  virtual Node &operator[](int);
  virtual Node &operator[](NodeId);
  virtual Node &operator[](NodeId) const;
  virtual int  IsEmpty();
  virtual int  Length();

  virtual ExprValue exprValue();

#ifdef IIOP_FUNCTIONS
  virtual long decodeIIOP(IIOPMessageHeader&);
  virtual long decodeTypeCode(IIOPMessageHeader&);
#endif
};

// Node factory function, implemented in factory.cpp
Node *NewNode(Node *, NodeId id, RecursionCount r = 0);

class SimpleChoiceNode : public Node
{
protected:
  int              choice;
  int              maxChoice;

public:
  SimpleChoiceNode();
  SimpleChoiceNode(NodeId n);
  SimpleChoiceNode(Node *, NodeId n, RecursionCount r);
  SimpleChoiceNode(const SimpleChoiceNode &);
  ~SimpleChoiceNode();

  int Choice() const;

  Node &operator=(const Node &);
  Node &operator=(const int);

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();
#endif
};

class ChoiceNode : public Node
{

protected:
  int             choice;
  NodeId          childIds[MAX_CHILD_IDS];
  Node            *child;

public:
  ChoiceNode();
  ChoiceNode(NodeId n);
  ChoiceNode(Node *, NodeId n, RecursionCount r);
  ChoiceNode(const ChoiceNode &);
  ~ChoiceNode();

  int Choice() const;

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();
  int assignValue(ExprValue GreaterThan, ExprValue LessThan);
#endif

  Node &operator=(const Node &);
  ChoiceNode &operator<<(Node *);
  ChoiceNode &operator<<(Node &);

  Node &operator[](int);
  Node &operator[](NodeId);
  Node &operator[](NodeId) const;
  int  IsEmpty();
  int  Length();

  ExprValue exprValue();

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader&);
  long decodeTypeCode(IIOPMessageHeader&);
#endif
};

class SequenceNode : public Node
{

protected:
  NodeId          childIds[MAX_CHILD_IDS];
  short           childIsOptional[MAX_CHILD_IDS];
  Node            *childNodes[MAX_CHILD_IDS];

public:
  SequenceNode();
  SequenceNode(NodeId n);
  SequenceNode(Node *, NodeId n, RecursionCount r);
  SequenceNode(const SequenceNode &);
  ~SequenceNode();

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();
#endif

  Node &operator=(const Node &);
  SequenceNode &operator<<(Node *);
  SequenceNode &operator<<(Node &);

  Node &operator[](int);
  Node &operator[](NodeId);
  Node &operator[](NodeId) const;
  int  IsEmpty();
  int  Length();

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader&);
  long decodeTypeCode(IIOPMessageHeader&);
#endif
};

class ListNode : public Node
{

protected:
  NodeId          childId;
  char            *separator;
  Node            **childNodes;
	int							maxChildNodes;

public:
  ListNode();
  ListNode(NodeId n);
  ListNode(Node *, NodeId n, RecursionCount r);
  ListNode(const ListNode &);
  ~ListNode();

  Node &operator=(const Node &);
  ListNode &operator<<(Node *);
  ListNode &operator<<(Node &);

  Node &operator[](int);
  Node &operator[](NodeId);
  Node &operator[](NodeId) const;
  int  IsEmpty();
  int  Length();
  char *Separator();

#ifdef GENERATOR_FUNCTIONS
  int Initialise();
  int Next();
#endif

#ifdef IIOP_FUNCTIONS
  long decodeIIOP(IIOPMessageHeader&);
  long decodeTypeCode(IIOPMessageHeader&);
#endif
};

#endif
