//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/utils/nodes/src/constexpr.cpp
//
// Modifications:
// $Log: constexpr.cpp,v $
// Revision 1.1.1.1  2000/02/09 17:07:13  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.3  1998/05/11 10:42:37  stephen
// Reference semantics changed for Node param.
//
// Revision 1.2  1997/10/14 10:24:06  james
// Extended types removed
//
// Revision 1.1  1997/10/06 11:35:56  james
// Initial Sources
//

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <ctype.h>
 
#include "constexpr.hh"
#include "utilities.hh"

#include "grammar.hh"
#include "literals.hh"
#include "outputidl.hh"



ConstExp::ConstExp() : ChoiceNode(IdConstExp)
{
  childIds[0] = IdOrExpr;
}

ConstExp::ConstExp(Node *p, RecursionCount r) : ChoiceNode(p,IdConstExp,r)
{
  childIds[0] = IdOrExpr;
}



OrExpr::OrExpr() : ChoiceNode(IdOrExpr)
{
  childIds[0] = IdXorExpr;
  childIds[1] = IdOrOper;
}

OrExpr::OrExpr(Node *p, RecursionCount r) : ChoiceNode(p,IdOrExpr,r)
{
  childIds[0] = IdXorExpr;
  switch(GetEvalType(*this))
  {
    case etLong :
    case etULong :
            childIds[1] = IdOrOper;
            break;
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
    case etEnum :
            break;
    case etUndefined :
    default :
            cerr << "Error: " << nodeId << " evaluation type undefined" << endl;
  }
}



OrOper::OrOper() : SequenceNode(IdOrOper)
{
  childIds[0] = IdOrExpr;
  childIds[1] = IdXorExpr;
}

OrOper::OrOper(Node *p, RecursionCount r) : SequenceNode(p,IdOrOper,r)
{
  childIds[0] = IdOrExpr;
  childIds[1] = IdXorExpr;
}


ExprValue OrOper::exprValue()
{
  ExprValue  v;

  if (childNodes[0] != 0 && childNodes[1] != 0)
    v = childNodes[0]->exprValue() | childNodes[1]->exprValue();

  return v;
}


XorExpr::XorExpr() : ChoiceNode(IdXorExpr)
{
  childIds[0] = IdAndExpr;
  childIds[1] = IdXorOper;
}

XorExpr::XorExpr(Node *p, RecursionCount r) : ChoiceNode(p,IdXorExpr,r)
{
  childIds[0] = IdAndExpr;
  switch(GetEvalType(*this))
  {
    case etLong :
    case etULong :
            childIds[1] = IdXorOper;
            break;
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
    case etEnum :
            break;
    case etUndefined :
    default :
            cerr << "Error: " << nodeId << " evaluation type undefined" << endl;
  }
}



XorOper::XorOper() : SequenceNode(IdXorOper)
{
  childIds[0] = IdXorExpr;
  childIds[1] = IdAndExpr;
}

XorOper::XorOper(Node *p, RecursionCount r) : SequenceNode(p,IdXorOper,r)
{
  childIds[0] = IdXorExpr;
  childIds[1] = IdAndExpr;
}


ExprValue XorOper::exprValue()
{
  ExprValue  v;

  if (childNodes[0] != 0 && childNodes[1] != 0)
    v = childNodes[0]->exprValue() ^ childNodes[1]->exprValue();

  return v;
}




AndExpr::AndExpr() : ChoiceNode(IdAndExpr)
{
  childIds[0] = IdShiftExpr;
  childIds[1] = IdAndOper;
}

AndExpr::AndExpr(Node *p, RecursionCount r) : ChoiceNode(p,IdAndExpr,r)
{
  childIds[0] = IdShiftExpr;
  switch(GetEvalType(*this))
  {
    case etLong :
    case etULong :
            childIds[1] = IdAndOper;
            break;
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
    case etEnum :
            break;
    case etUndefined :
    default :
            cerr << "Error: " << nodeId << " evaluation type undefined" << endl;
  }
}



AndOper::AndOper() : SequenceNode(IdAndOper)
{
  childIds[0] = IdAndExpr;
  childIds[1] = IdShiftExpr;
}

AndOper::AndOper(Node *p, RecursionCount r) : SequenceNode(p,IdAndOper,r)
{
  childIds[0] = IdAndExpr;
  childIds[1] = IdShiftExpr;
}


ExprValue AndOper::exprValue()
{
  ExprValue  v;

  if (childNodes[0] != 0 && childNodes[1] != 0)
    v = childNodes[0]->exprValue() & childNodes[1]->exprValue();

  return v;
}



ShiftExpr::ShiftExpr() : ChoiceNode(IdShiftExpr)
{
  childIds[0] = IdAddExpr;
  childIds[1] = IdShiftRightOper;
  childIds[2] = IdShiftLeftOper;
}

ShiftExpr::ShiftExpr(Node *p, RecursionCount r) : ChoiceNode(p,IdShiftExpr,r)
{
  childIds[0] = IdAddExpr;
  switch(GetEvalType(*this))
  {
    case etLong :
    case etULong :
            childIds[1] = IdShiftRightOper;
            childIds[2] = IdShiftLeftOper;
            break;
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
    case etEnum :
            break;
    case etUndefined :
    default :
            cerr << "Error: " << nodeId << " evaluation type undefined" << endl;
  }
}



ShiftRightOper::ShiftRightOper() : SequenceNode(IdShiftRightOper)
{
  childIds[0] = IdShiftExpr;
  childIds[1] = IdAddExpr;
}

ShiftRightOper::ShiftRightOper(Node *p, RecursionCount r) : SequenceNode(p,IdShiftRightOper,r)
{
  childIds[0] = IdShiftExpr;
  childIds[1] = IdAddExpr;
}


ExprValue ShiftRightOper::exprValue()
{
  ExprValue  v;

  if (childNodes[0] != 0 && childNodes[1] != 0)
    v = childNodes[0]->exprValue() >> childNodes[1]->exprValue();

  return v;
}


ShiftLeftOper::ShiftLeftOper() : SequenceNode(IdShiftLeftOper)
{
  childIds[0] = IdShiftExpr;
  childIds[1] = IdAddExpr;
}

ShiftLeftOper::ShiftLeftOper(Node *p, RecursionCount r) : SequenceNode(p,IdShiftLeftOper,r)
{
  childIds[0] = IdShiftExpr;
  childIds[1] = IdAddExpr;
}


ExprValue ShiftLeftOper::exprValue()
{
  ExprValue  v;

  if (childNodes[0] != 0 && childNodes[1] != 0)
    v = childNodes[0]->exprValue() << childNodes[1]->exprValue();

  return v;
}


AddExpr::AddExpr() : ChoiceNode(IdAddExpr)
{
  childIds[0] = IdMultExpr;
  childIds[1] = IdAddOper;
  childIds[2] = IdSubtractOper;
}

AddExpr::AddExpr(Node *p, RecursionCount r) : ChoiceNode(p,IdAddExpr,r)
{
  childIds[0] = IdMultExpr;
  switch(GetEvalType(*this))
  {
    case etLong :
    case etULong :
    case etDouble :
            childIds[1] = IdAddOper;
            childIds[2] = IdSubtractOper;
            break;
    case etBoolean :
    case etChar :
    case etString :
    case etEnum :
            break;
    case etUndefined :
    default :
            cerr << "Error: " << nodeId << " evaluation type undefined" << endl;
  }
}


AddOper::AddOper() : SequenceNode(IdAddOper)
{
  childIds[0] = IdAddExpr;
  childIds[1] = IdMultExpr;
}

AddOper::AddOper(Node *p, RecursionCount r) : SequenceNode(p,IdAddOper,r)
{
  childIds[0] = IdAddExpr;
  childIds[1] = IdMultExpr;
}

ExprValue AddOper::exprValue()
{
  ExprValue  v;

  if (childNodes[0] != 0 && childNodes[1] != 0)
    v = childNodes[0]->exprValue() + childNodes[1]->exprValue();

  return v;
}


SubtractOper::SubtractOper() : SequenceNode(IdSubtractOper)
{
  childIds[0] = IdAddExpr;
  childIds[1] = IdMultExpr;
}

SubtractOper::SubtractOper(Node *p, RecursionCount r) : SequenceNode(p,IdSubtractOper,r)
{
  childIds[0] = IdAddExpr;
  childIds[1] = IdMultExpr;
}


ExprValue SubtractOper::exprValue()
{
  ExprValue  v;

  if (childNodes[0] != 0 && childNodes[1] != 0)
    v = childNodes[0]->exprValue() - childNodes[1]->exprValue();

  return v;
}



MultExpr::MultExpr() : ChoiceNode(IdMultExpr)
{
  childIds[0] = IdUnaryExpr;
  childIds[1] = IdMultOper;
  childIds[2] = IdDivOper;
  childIds[3] = IdRemainderOper;
}

MultExpr::MultExpr(Node *p, RecursionCount r) : ChoiceNode(p,IdMultExpr,r)
{
  childIds[0] = IdUnaryExpr;
  switch(GetEvalType(*this))
  {
    case etLong :
    case etULong :
            childIds[1] = IdMultOper;
            childIds[2] = IdDivOper;
            childIds[3] = IdRemainderOper;
            break;
    case etDouble :
            childIds[1] = IdMultOper;
            childIds[2] = IdDivOper;
            break;
    case etBoolean :
    case etChar :
    case etString :
    case etEnum :
            break;
    case etUndefined :
    default :
            cerr << "Error: " << nodeId << " evaluation type undefined" << endl;
  }
}




MultOper::MultOper() : SequenceNode(IdMultOper)
{
  childIds[0] = IdMultExpr;
  childIds[1] = IdUnaryExpr;
}

MultOper::MultOper(Node *p, RecursionCount r) : SequenceNode(p,IdMultOper,r)
{
  childIds[0] = IdMultExpr;
  childIds[1] = IdUnaryExpr;
}


ExprValue MultOper::exprValue()
{
  ExprValue  v;

  if (childNodes[0] != 0 && childNodes[1] != 0)
    v = childNodes[0]->exprValue() * childNodes[1]->exprValue();

  return v;
}



DivOper::DivOper() : SequenceNode(IdDivOper)
{
  childIds[0] = IdMultExpr;
  childIds[1] = IdUnaryExpr;
}

DivOper::DivOper(Node *p, RecursionCount r) : SequenceNode(p,IdDivOper,r)
{
  childIds[0] = IdMultExpr;
  childIds[1] = IdUnaryExpr;
}


ExprValue DivOper::exprValue()
{
  ExprValue  v;

  if (childNodes[0] != 0 && childNodes[1] != 0)
    v = childNodes[0]->exprValue() / childNodes[1]->exprValue();

  return v;
}


RemainderOper::RemainderOper() : SequenceNode(IdRemainderOper)
{
  childIds[0] = IdMultExpr;
  childIds[1] = IdUnaryExpr;
}

RemainderOper::RemainderOper(Node *p, RecursionCount r) : SequenceNode(p,IdRemainderOper,r)
{
  childIds[0] = IdMultExpr;
  childIds[1] = IdUnaryExpr;
}


ExprValue RemainderOper::exprValue()
{
  ExprValue  v;

  if (childNodes[0] != 0 && childNodes[1] != 0)
    v = childNodes[0]->exprValue() % childNodes[1]->exprValue();

  return v;
}


UnaryExpr::UnaryExpr() : ChoiceNode(IdUnaryExpr)
{
  childIds[0] = IdPrimaryExpr;
  childIds[1] = IdUnaryMinusOper;  // "-"
  childIds[2] = IdUnaryPlusOper;  // "+"
  childIds[3] = IdUnaryNotOper;    // "~"
}

UnaryExpr::UnaryExpr(Node *p, RecursionCount r) : ChoiceNode(p,IdUnaryExpr,r)
{
  switch(GetEvalType(*this))
  {
    case etLong :
    case etULong :
            childIds[0] = IdPrimaryExpr;
            childIds[1] = IdUnaryMinusOper;  // "-"
            childIds[2] = IdUnaryPlusOper;  // "+"
            //childIds[3] = IdUnaryNotOper;  // "~"
            break;
    case etDouble :
            childIds[0] = IdPrimaryExpr;
            childIds[1] = IdUnaryMinusOper;  // "-"
            childIds[2] = IdUnaryPlusOper;  // "+"
            break;
    case etBoolean :
    case etChar :
    case etString :
    case etEnum :
            childIds[0] = IdPrimaryExpr;
            break;
    case etUndefined :
    default :
            cerr << "Error: " << nodeId << " evaluation type undefined" << endl;
  }
}


UnaryMinusOper::UnaryMinusOper() : ChoiceNode(IdUnaryMinusOper)
{
  childIds[0] = IdPrimaryExpr;
}

UnaryMinusOper::UnaryMinusOper(Node *p, RecursionCount r) : ChoiceNode(p,IdUnaryMinusOper,r)
{
  childIds[0] = IdPrimaryExpr;
}


ExprValue UnaryMinusOper::exprValue()
{
  ExprValue r;

  if (child != 0)
    r = -(child->exprValue());

  return r;
}


UnaryPlusOper::UnaryPlusOper() : ChoiceNode(IdUnaryPlusOper)
{
  childIds[0] = IdPrimaryExpr;
}

UnaryPlusOper::UnaryPlusOper(Node *p, RecursionCount r) : ChoiceNode(p,IdUnaryPlusOper,r)
{
  childIds[0] = IdPrimaryExpr;
}


UnaryNotOper::UnaryNotOper() : ChoiceNode(IdUnaryNotOper)
{
  childIds[0] = IdPrimaryExpr;
}

UnaryNotOper::UnaryNotOper(Node *p, RecursionCount r) : ChoiceNode(p,IdUnaryNotOper,r)
{
  childIds[0] = IdPrimaryExpr;
}


ExprValue UnaryNotOper::exprValue()
{
  ExprValue r;

  if (child != 0)
    r = ~(child->exprValue());

  return r;
}


PrimaryExpr::PrimaryExpr() : ChoiceNode(IdPrimaryExpr)
{
  childIds[0] = IdLiteral;
  childIds[1] = IdConstExp;
  childIds[2] = IdScopedName;
}

PrimaryExpr::PrimaryExpr(Node *p, RecursionCount r) : ChoiceNode(p,IdPrimaryExpr,r)
{
  switch(GetEvalType(*this))
  {
    case etLong :
    case etULong :
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
            childIds[0] = IdLiteral;
            childIds[1] = IdConstExp;
            childIds[2] = IdScopedName;
            break;
    case etEnum :
            childIds[0] = IdConstExp;
            childIds[1] = IdScopedName;
            break;
    case etUndefined :
    default :
            cerr << "Error: " << nodeId << " evaluation type undefined" << endl;
  }
}


