//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2000 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "exprvalue.hh"

ExprValue::ExprValue()
{
  type = etUndefined;
}

ExprValue::ExprValue(ExprValue &v)
{
  type = v.type;
  switch(type)
  {
    case etUndefined : 
                        break;
    case etLong :         value.Long = v.value.Long;
                        break;
    case etULong :      value.ULong = v.value.ULong;
                        break;
    case etDouble :      value.Double = v.value.Double;
                        break;
    case etBoolean :    value.Boolean = v.value.Boolean;
                        break;
    case etChar :         value.Char = v.value.Char;
                        break;
    case etString :      value.String = new char[strlen(v.value.String)+1];
                        strcpy(value.String,v.value.String);
                        break;
  }
}

ExprValue::ExprValue(const ExprValue &v)
{
  type = v.type;
  switch(type)
  {
    case etUndefined : 
                        break;
    case etLong :         value.Long = v.value.Long;
                        break;
    case etULong :      value.ULong = v.value.ULong;
                        break;
    case etDouble :      value.Double = v.value.Double;
                        break;
    case etBoolean :    value.Boolean = v.value.Boolean;
                        break;
    case etChar :         value.Char = v.value.Char;
                        break;
    case etString :      value.String = new char[strlen(v.value.String)+1];
                        strcpy(value.String,v.value.String);
                        break;
  }
}


ExprValue::~ExprValue()
{
  switch(type)
  {
    case etString: 
            delete value.String;
            break;
  }
}

int ExprValue::IsInteger() const
{
  return(type == etLong 
      || type == etULong);
}

void ExprValue::CorrectIntegerType()
{
  if (type == etLong)
  {
    unsigned long tmp;

    if (value.Long > 0)
    {
      tmp = (unsigned long)value.Long;
      type = etULong;
      value.ULong = tmp;
    }
  }
}

// Assignment Operators

ExprValue &ExprValue::operator=(const long v)
{
  type = etLong;
  value.Long = v;
  CorrectIntegerType();
  return *this;
}

ExprValue &ExprValue::operator=(const unsigned long v)
{
  type = etULong;
  value.ULong = v;
  CorrectIntegerType();
  return *this;
}

ExprValue &ExprValue::operator=(const double v)
{
  type = etDouble;
  value.Double = v;
  return *this;
}

ExprValue &ExprValue::operator=(const VSORB_Boolean v)
{
  type = etBoolean;
  value.Boolean = v;
  return *this;
}

ExprValue &ExprValue::operator=(const unsigned char v)
{
  type = etChar;
  value.Char = v;
  return *this;
}


ExprValue &ExprValue::operator=(const char * v)
{
  type = etString;
  value.String = new char[strlen(v)+1];
  strcpy(value.String,v);
  return *this;
}

ExprValue &ExprValue::operator=(const ExprValue & v)
{
  if (&v != this)
  {
    type = v.type;
    switch(type)
    {
      case etUndefined :   
                          break;
      case etLong :         value.Long = v.value.Long;
                          break;
      case etULong :      value.ULong = v.value.ULong;
                          break;
      case etDouble :      value.Double = v.value.Double;
                          break;
      case etBoolean :    value.Boolean = v.value.Boolean;
                          break;
      case etChar :         value.Char = v.value.Char;
                          break;
      case etString :      value.String = new char[strlen(v.value.String)+1];
                          strcpy(value.String,v.value.String);
                          break;
    }
    CorrectIntegerType();
  }

  return *this;
}

ExprValue::operator long() const
{
  switch(type)
  {
    case etLong :         return (long) value.Long;
    case etULong :      return (long) value.ULong;
    case etBoolean :    return (long) value.Boolean;
    case etUndefined :   
    case etDouble :
    case etChar :
    case etString :
      cerr << "operator (long) can not be applied to a sub-expression of type "
           << type;
      exit(EXIT_FAILURE);
  }
  return 0;
}

ExprValue::operator unsigned long() const
{
  switch(type)
  {
    case etLong :         return (unsigned long) value.Long;
    case etULong :      return (unsigned long) value.ULong;
    case etBoolean :    return (unsigned long) value.Boolean;
    case etChar :       return (unsigned long) value.Char;
    case etUndefined :   
    case etDouble :
    case etString :
      cerr << "operator (unsigned long) can not be applied to a sub-expression of type "
           << type;
      exit(EXIT_FAILURE);
  }
  return 0;
}

ExprValue::operator unsigned char() const
{
  switch(type)
  {
    case etChar :        return value.Char;
    default :
      cerr << "operator (unsigned char) can not be applied to a sub-expression of type "
           << type;
      exit(EXIT_FAILURE);
  }
  return 0;
}

ExprValue::operator VSORB_Boolean() const
{
  switch(type)
  {
    case etBoolean :    return value.Boolean;
    default :
      cerr << "operator (Boolean) can not be applied to a sub-expression of type "
           << type;
      exit(EXIT_FAILURE);
  }
  return VSORB_False;
}

ExprValue ExprValue::operator-()
{
  ExprValue v;

  v.type = type;

  switch(type)
  {
    case etLong :       v.value.Long = -value.Long;
                        break;
    case etDouble :      v.value.Double = -value.Double;
                        break;
    case etULong :      v.type = etLong;
                        v.value.Long = -(long)value.ULong;
                        break;
    case etUndefined :
    case etBoolean :
    case etChar :
    case etString :
      cerr << "Unary operator - can not be applied to a sub-expression of type "
           << type;
      exit(EXIT_FAILURE);
  }
  CorrectIntegerType();
  return v;
}


ExprValue ExprValue::operator+()
{
  ExprValue v;

  v.type = type;

  switch(type)
  {
    case etLong :       v.value.Long = +value.Long;
                        break;
    case etULong :      v.value.ULong = +value.ULong;
                        break;
    case etDouble :      v.value.Double = +value.Double;
                        break;
    case etUndefined :
    case etBoolean :
    case etChar :
    case etString :
      cerr << "Unary operator + can not be applied to a sub-expression of type "
           << type;
      exit(EXIT_FAILURE);
  }
  CorrectIntegerType();
  return v;
}


ExprValue ExprValue::operator~()
{
  ExprValue v;

  v.type = type;

  switch(type)
  {
    case etLong :       v.value.Long = ~value.Long;
                        break;
    case etULong :      v.value.ULong = ~value.ULong;
                        break;
    case etUndefined :
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
      cerr << "Unary operator ~ can not be applied to a sub-expression of type "
           << type << endl;
      exit(EXIT_FAILURE);
  }
  CorrectIntegerType();
  return v;
}

ExprValue &ExprValue::operator--()
{
  switch(type)
  {
    case etLong :       --value.Long;
                        break;
    case etULong :      --value.ULong;
                        break;
    case etUndefined :
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
      cerr << "Unary operator -- can not be applied to a sub-expression of type "
           << type << endl;
      exit(EXIT_FAILURE);
  }

  return *this;
}

ExprValue &ExprValue::operator++()
{
  switch(type)
  {
    case etLong :       ++value.Long;
                        break;
    case etULong :      ++value.ULong;
                        break;
    case etUndefined :
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
      cerr << "Unary operator ++ can not be applied to a sub-expression of type "
           << type << endl;
      exit(EXIT_FAILURE);
  }

  return *this;
}

ExprValue ExprValue::operator*(const ExprValue &r)
{
  ExprValue v;

  v.type = type;

  if (type == r.type)
  {
    switch(type)
    {
      case etLong :       v.value.Long = value.Long *  r.value.Long;
                          break;
      case etULong :      v.value.ULong = value.ULong * r.value.ULong;
                          break;
      case etDouble :      v.value.Double = value.Double * r.value.Double;
                          break;
      case etUndefined :
      case etBoolean :
      case etChar :
      case etString :
        cerr << "operator * can not be applied to a sub-expressions of type "
              << type << endl;
        exit(EXIT_FAILURE);
    }
  }
  else
  {
    if (IsInteger() && r.IsInteger())
    {
    	v = (long)*this * (long)r;
    }
    else
    {
        cerr << "operator * can not combine sub-expressions of different types";
        cerr << "(" << type << "," << r.type << ")";
        exit(EXIT_FAILURE);
    }
  }

  v.CorrectIntegerType();
  return v;
}

ExprValue ExprValue::operator/(const ExprValue &r)
{
  ExprValue v;

  if (r == 0L)
  {
    v= (unsigned long)0;
    return v;
  }

  v.type = type;

  if (type == r.type)
  {
    switch(type)
    {
      case etLong :       v.value.Long = value.Long / r.value.Long;
                          break;
      case etULong :      v.value.ULong = value.ULong / r.value.ULong;
                          break;
      case etDouble :      v.value.Double = value.Double / r.value.Double;
                          break;
      case etUndefined :
      case etBoolean :
      case etChar :
      case etString :
        cerr << "operator / can not be applied to a sub-expressions of type "
              << type << endl;
        exit(EXIT_FAILURE);
    }
  }
  else
  {
    if (IsInteger() && r.IsInteger())
    {
       v = (long)*this / (long)r;
    }
    else
    {
        cerr << "operator / can not combine sub-expressions of different types";
        cerr << "(" << type << "," << r.type << ")";
        exit(EXIT_FAILURE);
    }
  }
  v.CorrectIntegerType();
  return v;
}

ExprValue ExprValue::operator+(const ExprValue &r)
{
  ExprValue v;

  v.type = type;

  if (type == r.type)
  {
    switch(type)
    {
      case etLong :       v.value.Long = value.Long +  r.value.Long;
                          break;
      case etULong :      v.value.ULong = value.ULong + r.value.ULong;
                          break;
      case etDouble :      v.value.Double = value.Double + r.value.Double;
                          break;
      case etUndefined :
      case etBoolean :
      case etChar :
      case etString :
        cerr << "operator + can not be applied to a sub-expressions of type "
              << type << endl;
        exit(EXIT_FAILURE);
    }
  }
  else
  {
    if (IsInteger() && r.IsInteger())
    {
    	v = (long)*this + (long)r;
    }
    else
    {
        cerr << "operator + can not combine sub-expressions of different types";
        cerr << "(" << type << "," << r.type << ")";
        exit(EXIT_FAILURE);
    }
  }

  v.CorrectIntegerType();
  return v;
}


ExprValue ExprValue::operator-(const ExprValue &r)
{
  ExprValue v;

  v.type = type;

  if (type == r.type)
  {
    switch(type)
    {
      case etLong :       v.value.Long = value.Long - r.value.Long;
                          break;
      case etULong :      if (value.ULong >= r.value.ULong)
                            v.value.ULong = value.ULong - r.value.ULong;
                          else
                          {
                            v.type = etLong;
                            v.value.Long = (long)value.ULong - (long)r.value.ULong;
                          }
                          break;
      case etDouble :      v.value.Double = value.Double - r.value.Double;
                          break;
      case etUndefined :
      case etBoolean :
      case etChar :
      case etString :
        cerr << "operator - can not be applied to a sub-expressions of type "
              << type << endl;
        exit(EXIT_FAILURE);
    }
  }
  else
  {
    if (IsInteger() && r.IsInteger())
    {
    	v = (long)*this - (long)r;
    }
    else
    {
      cerr << "operator - can not combine sub-expressions of different types";
      cerr << "(" << type << "," << r.type << ")";
      exit(EXIT_FAILURE);
    }
  }

  v.CorrectIntegerType();
  return v;
}


ExprValue ExprValue::operator%(const ExprValue &r)
{
  ExprValue v;

  if (r == 0L)
  {
    v= (unsigned long)0;
    return v;
  }
  v.type = type;

  if (type == r.type)
  {
    switch(type)
    {
      case etLong :       v.value.Long = value.Long % r.value.Long;
                          break;
      case etULong :
                          if (value.ULong >= r.value.ULong)
                            v.value.ULong = value.ULong % r.value.ULong;
                          else
                          {
                            v.type = etLong;
                            v.value.Long = (long)value.ULong % (long)r.value.ULong;
                          }
                          break;
      case etUndefined :
      case etDouble :
      case etBoolean :
      case etChar :
      case etString :
        cerr << "operator % can not be applied to a sub-expressions of type "
              << type << endl;
        exit(EXIT_FAILURE);
    }
  }
  else
  {
    if (IsInteger() && r.IsInteger())
    {
      v = (long)*this % (long)r;
    }
    else
    {
      cerr << "operator % can not combine sub-expressions of different types";
      cerr << "(" << type << "," << r.type << ")";
      exit(EXIT_FAILURE);
    }
  }

  v.CorrectIntegerType();
  return v;
}



ExprValue ExprValue::operator<<(const ExprValue &r)
{
  ExprValue v;

  if (type != r.type)
  {
      cerr << "operator << can not combine sub-expressions of different types";
        cerr << "(" << type << "," << r.type << ")";
      exit(EXIT_FAILURE);
  }

  switch(type)
  {
    case etLong :       v.value.Long = value.Long << r.value.Long;
                        break;
    case etULong :      v.value.ULong = value.ULong << r.value.ULong;
                        break;
    case etUndefined :
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
      cerr << "operator << can not be applied to a sub-expressions of type "
           << type << endl;
      exit(EXIT_FAILURE);
  }
  v.CorrectIntegerType();
  return v;
}

ExprValue ExprValue::operator>>(const ExprValue &r)
{
  ExprValue v;

  if (type != r.type)
  {
      cerr << "operator >> can not combine sub-expressions of different types";
        cerr << "(" << type << "," << r.type << ")";
      exit(EXIT_FAILURE);
  }

  switch(type)
  {
    case etLong :       v.value.Long = value.Long >> r.value.Long;
                        break;
    case etULong :      v.value.ULong = value.ULong >> r.value.ULong;
                        break;
    case etUndefined :
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
      cerr << "operator >> can not be applied to a sub-expressions of type "
           << type << endl;
      exit(EXIT_FAILURE);
  }
  v.CorrectIntegerType();
  return v;
}


ExprValue ExprValue::operator&(const ExprValue &r)
{
  ExprValue v;

  if (type != r.type)
  {
      cerr << "operator & can not combine sub-expressions of different types";
        cerr << "(" << type << "," << r.type << ")";
      exit(EXIT_FAILURE);
  }

  switch(type)
  {
    case etLong :       v.value.Long = value.Long & r.value.Long;
                        break;
    case etULong :      v.value.ULong = value.ULong & r.value.ULong;
                        break;
    case etUndefined :
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
      cerr << "operator & can not be applied to a sub-expressions of type "
           << type << endl;
      exit(EXIT_FAILURE);
  }
  v.CorrectIntegerType();
  return v;
}


ExprValue ExprValue::operator|(const ExprValue &r)
{
  ExprValue v;

  if (type != r.type)
  {
      cerr << "operator | can not combine sub-expressions of different types";
        cerr << "(" << type << "," << r.type << ")";
      exit(EXIT_FAILURE);
  }

  switch(type)
  {
    case etLong :       v.value.Long = value.Long | r.value.Long;
                        break;
    case etULong :      v.value.ULong = value.ULong | r.value.ULong;
                        break;
    case etUndefined :
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
      cerr << "operator | can not be applied to a sub-expressions of type "
           << type << endl;
      exit(EXIT_FAILURE);
  }
  v.CorrectIntegerType();
  return v;
}


ExprValue ExprValue::operator^(const ExprValue &r)
{
  ExprValue v;

  if (type != r.type)
  {
      cerr << "operator ^ can not combine sub-expressions of different types";
        cerr << "(" << type << "," << r.type << ")";
      exit(EXIT_FAILURE);
  }

  switch(type)
  {
    case etLong :       v.value.Long = value.Long ^ r.value.Long;
                        break;
    case etULong :      v.value.ULong = value.ULong ^ r.value.ULong;
                        break;
    case etUndefined :
    case etDouble :
    case etBoolean :
    case etChar :
    case etString :
      cerr << "operator ^ can not be applied to a sub-expressions of type "
           << type << endl;
      exit(EXIT_FAILURE);
  }
  v.CorrectIntegerType();
  return v;
}


long ExprValue::PreserveTypeAssign(long v)
{
  switch(type)
  {
    case etLong :       return value.Long = v;
                        break;
    case etULong :      return value.ULong = (unsigned long)v;
                        break;
    case etDouble :      return (long)(value.Double = (double)v);
                        break;
    case etBoolean :
    case etChar :
    case etString :
    case etUndefined :
      cerr << "Error: PreserveTypeAssign() called on ExprValue of type" << type << endl;
      exit(EXIT_FAILURE);
  }
  return v;
}


int ExprValue::operator>(const long v) const
{
  switch(type)
  {
    case etLong :       return value.Long > v;
    case etULong :      return value.ULong > (unsigned long)v;
    case etDouble :      return value.Double > v;
    case etBoolean :
    case etChar :
    case etString :
    case etUndefined :
      cerr << "Error: operator > called on ExprValue of type" << type << endl;
      exit(EXIT_FAILURE);
  }
  return 0;
}

int ExprValue::operator<(const long v) const
{
  switch(type)
  {
    case etLong :       return value.Long < v;
    case etULong :      return value.ULong < (unsigned long)v;
    case etDouble :      return value.Double < v;
    case etBoolean :
    case etChar :
    case etString :
    case etUndefined :
      cerr << "Error: operator < called on ExprValue of type" << type << endl;
      exit(EXIT_FAILURE);
  }
  return 0;
}

int ExprValue::operator>=(const long v) const
{
  switch(type)
  {
    case etLong :       return value.Long >= v;
    case etULong :      return value.ULong >= (unsigned long)v;
    case etDouble :      return value.Double >= v;
    case etBoolean :
    case etChar :
    case etString :
    case etUndefined :
      cerr << "Error: operator >= called on ExprValue of type" << type << endl;
      exit(EXIT_FAILURE);
  }
  return 0;
}

int ExprValue::operator<=(const long v) const
{
  switch(type)
  {
    case etLong :       return value.Long <= v;
    case etULong :      return value.ULong <= (unsigned long)v;
    case etDouble :      return value.Double <= v;
    case etBoolean :
    case etChar :
    case etString :
    case etUndefined :
      cerr << "Error: operator <= called on ExprValue of type" << type << endl;
      exit(EXIT_FAILURE);
  }
  return 0;
}

int ExprValue::operator==(const long v) const
{
  switch(type)
  {
    case etLong :       return value.Long == v;
    case etULong :      return value.ULong == (unsigned long)v;
    case etDouble :      return value.Double == v;
    case etBoolean :
    case etChar :
    case etString :
    case etUndefined :
      cerr << "Error: operator == called on ExprValue of type" << type << endl;
      exit(EXIT_FAILURE);
  }
  return 0;
}

int ExprValue::operator!=(const long v) const
{
  switch(type)
  {
    case etLong :       return value.Long != v;
    case etULong :      return value.ULong != (unsigned long)v;
    case etDouble :      return value.Double != v;
    case etBoolean :
    case etChar :
    case etString :
    case etUndefined :
      cerr << "Error: operator != called on ExprValue of type" << type << endl;
      exit(EXIT_FAILURE);
  }
  return 0;
}

EvalType ExprValue::Type() const
{
  return type;
}

char * ExprValue::string() const
{
  char *ptr = 0;

  switch(type)
  {
    case etLong :       
          ptr = new char[40];
          sprintf(ptr,"%d",value.Long);
          break;      
    case etULong :      
          ptr = new char[100];
          sprintf(ptr,"%u",value.ULong);
          break;      
    case etDouble :      
          ptr = new char[100];
          sprintf(ptr,"%g",value.Double);
          break;      
    case etBoolean :
          ptr = new char[6];
          strcpy(ptr,(value.Boolean == VSORB_True)?"TRUE":"FALSE");
          break;      
    case etChar :
          ptr = new char[10];
          if (value.Char != 0 && strchr("\\\?\'\"",(int)value.Char) != 0)
          {
            sprintf(ptr,"'\\%c'",value.Char);
          }
          else if (isprint(value.Char) && value.Char != '\0')
          {
            sprintf(ptr,"'%c'",value.Char);
          }
          else
          {
            sprintf(ptr,"'\\%3.3o'",value.Char);
          }

          break;      
    case etString :
          ptr = new char[strlen(value.String)+1];
          strcpy(ptr,value.String);
          break;      
    case etUndefined :
          ptr = new char[30];
          strcpy(ptr,"ExprValue::Undefined");
          break;      
  }

  return ptr;
}

int ExprValue::operator==(const ExprValue &v) const
{
  switch(type)
  {
    case etLong :       return value.Long == (long)v;
    case etEnum :        
    case etULong :      return value.ULong == (unsigned long)v;
    case etBoolean :    return value.Boolean == (VSORB_Boolean)v;
    case etChar :        return value.Char == (unsigned char)v;
    case etDouble :  
    case etString :
    case etUndefined :
    default:
      cerr << "Error: operator == called on ExprValue of type" << type << endl;
      exit(EXIT_FAILURE);
  }
  return 0;
}

 
 
fstream &operator<<(fstream &output, const ExprValue v)
{
  char *ptr;
 
  ptr = v.string();
  output << ptr;
  delete ptr;
  return output;
}
 
ostream &operator<<(ostream &output, const ExprValue v)
{
  char *ptr;
 
  ptr = v.string();
  output << ptr;
  delete ptr;
  return output;
}


