//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/utils/nodes/src/factory.cpp
//
// Modifications:
// $Log: factory.cpp,v $
// Revision 1.1.1.1  2000/02/09 17:07:14  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1998/05/11 10:47:42  stephen
// Bogus statement removed.
//
// Revision 1.3  1997/12/12 15:12:31  stephen
// Defensive coding around default NewNode code.
//
// Revision 1.2  1997/10/14 10:24:09  james
// Extended types removed
//
// Revision 1.1  1997/10/06 11:35:58  james
// Initial Sources
//

#include <stdlib.h>
 
#include "grammar.hh"
#include "literals.hh"
#include "constexpr.hh"

#include <stdio.h>
Node *NewNode(Node *p,NodeId id, RecursionCount r)
{
  Node *       node;

  switch (id)
  {
    case IdSpecification :        node =  (Node *)new Specification(p,r);
                                  break;
    case IdDefinition :           node =  (Node *)new Definition(p,r);
                                  break;
    case IdModule :               node =  (Node *)new Module(p,r);
                                  break;
    case IdDefinitions :           node =  (Node *)new Definitions(p,r);
                                  break;
    case IdInterface :             node =  (Node *)new Interface(p,r);
                                  break;
    case IdInterfaceDcl :         node =  (Node *)new InterfaceDcl(p,r);
                                  break;
    case IdForwardDcl :           node =  (Node *)new ForwardDcl(p,r);
                                  break;
    case IdInterfaceBody :         node =  (Node *)new InterfaceBody(p,r);
                                  break;
    case IdExport :               node =  (Node *)new Export(p,r);
                                  break;
    case IdInheritanceSpec :       node =  (Node *)new InheritanceSpec(p,r);
                                  break;
    case IdScopedName :           node =  (Node *)new _ScopedName(p,r);
                                  break;
    case IdScopedNameA :           node =  (Node *)new ScopedNameA(p,r);
                                  break;
    case IdScopedNameB :           node =  (Node *)new ScopedNameB(p,r);
                                  break;
    case IdConstDcl :             node =  (Node *)new ConstDcl(p,r);
                                  break;
    case IdConstType :             node =  (Node *)new ConstType(p,r);
                                  break;
    case IdConstExp :             node =  (Node *)new ConstExp(p,r);
                                  break;
    case IdOrExpr :               node =  (Node *)new OrExpr(p,r);
                                  break;
    case IdOrOper :               node =  (Node *)new OrOper(p,r);
                                  break;
    case IdXorExpr :               node =  (Node *)new XorExpr(p,r);
                                  break;
    case IdXorOper :               node =  (Node *)new XorOper(p,r);
                                  break;
    case IdAndExpr :               node =  (Node *)new AndExpr(p,r);
                                  break;
    case IdAndOper :               node =  (Node *)new AndOper(p,r);
                                  break;
    case IdShiftExpr :             node =  (Node *)new ShiftExpr(p,r);
                                  break;
    case IdShiftRightOper :       node =  (Node *)new ShiftRightOper(p,r);
                                  break;
    case IdShiftLeftOper :         node =  (Node *)new ShiftLeftOper(p,r);
                                  break;
    case IdAddExpr :               node =  (Node *)new AddExpr(p,r);
                                  break;
    case IdAddOper :               node =  (Node *)new AddOper(p,r);
                                  break;
    case IdSubtractOper :         node =  (Node *)new SubtractOper(p,r);
                                  break;
    case IdMultExpr :             node =  (Node *)new MultExpr(p,r);
                                  break;
    case IdMultOper :             node =  (Node *)new MultOper(p,r);
                                  break;
    case IdDivOper :               node =  (Node *)new DivOper(p,r);
                                  break;
    case IdRemainderOper :         node =  (Node *)new RemainderOper(p,r);
                                  break;
    case IdUnaryExpr :             node =  (Node *)new UnaryExpr(p,r);
                                  break;
    case IdUnaryMinusOper :       node =  (Node *)new UnaryMinusOper(p,r);
                                  break;
    case IdUnaryPlusOper :         node =  (Node *)new UnaryPlusOper(p,r);
                                  break;
    case IdUnaryNotOper :         node =  (Node *)new UnaryNotOper(p,r);
                                  break;
    case IdPrimaryExpr :           node =  (Node *)new PrimaryExpr(p,r);
                                  break;
    case IdLiteral :               node =  (Node *)new Literal(p,r);
                                  break;
    case IdBooleanLiteral :       node =  (Node *)new BooleanLiteral(p,r);
                                  break;
    case IdPositiveIntConst :      node =  (Node *)new PositiveIntConst(p,r);
                                  break;
    case IdTypeDcl :               node =  (Node *)new TypeDcl(p,r);
                                  break;
    case IdTypeDeclarator :       node =  (Node *)new TypeDeclarator(p,r);
                                  break;
    case IdTypeSpec :             node =  (Node *)new TypeSpec(p,r);
                                  break;
    case IdSimpleTypeSpec :       node =  (Node *)new SimpleTypeSpec(p,r);
                                  break;
    case IdBaseTypeSpec :         node =  (Node *)new BaseTypeSpec(p,r);
                                  break;
    case IdObjectType :           node =  (Node *)new ObjectType(p,r);
                                  break;
    case IdTemplateTypeSpec :      node =  (Node *)new TemplateTypeSpec(p,r);
                                  break;
    case IdConstrTypeSpec :       node =  (Node *)new ConstrTypeSpec(p,r);
                                  break;
    case IdDeclarators :           node =  (Node *)new Declarators(p,r);
                                  break;
    case IdDeclarator :           node =  (Node *)new Declarator(p,r);
                                  break;
    case IdSimpleDeclarator :      node =  (Node *)new SimpleDeclarator(p,r);
                                  break;
    case IdComplexDeclarator :    node =  (Node *)new ComplexDeclarator(p,r);
                                  break;
    case IdFloatingPtType :       node =  (Node *)new FloatingPtType(p,r);
                                  break;
    case IdFloatType :             node =  (Node *)new FloatType(p,r);
                                  break;
    case IdDoubleType :           node =  (Node *)new DoubleType(p,r);
                                  break;
    case IdIntegerType :           node =  (Node *)new IntegerType(p,r);
                                  break;
    case IdSignedInt :             node =  (Node *)new SignedInt(p,r);
                                  break;
    case IdSignedLongInt :         node =  (Node *)new SignedLongInt(p,r);
                                  break;
    case IdSignedShortInt :       node =  (Node *)new SignedShortInt(p,r);
                                  break;
    case IdUnsignedInt :           node =  (Node *)new UnsignedInt(p,r);
                                  break;
    case IdUnsignedLongInt :       node =  (Node *)new UnsignedLongInt(p,r);
                                  break;
    case IdUnsignedShortInt :     node =  (Node *)new UnsignedShortInt(p,r);
                                  break;
    case IdCharType :             node =  (Node *)new CharType(p,r);
                                  break;
    case IdBooleanType :           node =  (Node *)new BooleanType(p,r);
                                  break;
    case IdOctetType :             node =  (Node *)new OctetType(p,r);
                                  break;
    case IdAnyType :               node =  (Node *)new AnyType(p,r);
                                  break;
    case IdStructType :           node =  (Node *)new StructType(p,r);
                                  break;
    case IdMemberList :           node =  (Node *)new MemberList(p,r);
                                  break;
    case IdMember :               node =  (Node *)new Member(p,r);
                                  break;
    case IdUnionType :             node =  (Node *)new UnionType(p,r);
                                  break;
    case IdUnionSwitch :          node =  (Node *)new UnionSwitch(p,r);
                                  break;
    case IdSwitchTypeSpec :       node =  (Node *)new SwitchTypeSpec (p,r);
                                  break;
    case IdSwitchBody :           node =  (Node *)new SwitchBody(p,r);
                                  break;
    case IdCase :                 node =  (Node *)new Case(p,r);
                                  break;
    case IdCaseLabel :             node =  (Node *)new CaseLabel(p,r);
                                  break;
    case IdDefaultLabel :         node =  (Node *)new DefaultLabel(p,r);
                                  break;
    case IdCaseLabelList :         node =  (Node *)new CaseLabelList(p,r);
                                  break;
    case IdElementSpec :           node =  (Node *)new ElementSpec(p,r);
                                  break;
    case IdEnumType :             node =  (Node *)new EnumType(p,r);
                                  break;
    case IdEnumerator :           node =  (Node *)new Enumerator(p,r);
                                  break;
    case IdEnumeratorList :       node =  (Node *)new EnumeratorList(p,r);
                                  break;
    case IdSequenceType :         node =  (Node *)new SequenceType(p,r);
                                  break;
    case IdStringType :           node =  (Node *)new StringType(p,r);
                                  break;
    case IdArrayDeclarator :      node =  (Node *)new ArrayDeclarator(p,r);
                                  break;
    case IdFixedArraySizeList :   node =  (Node *)new FixedArraySizeList(p,r);
                                  break;
    case IdFixedArraySize :       node =  (Node *)new FixedArraySize(p,r);
                                  break;
    case IdAttrDcl :              node =  (Node *)new AttrDcl(p,r);
                                  break;
    case IdReadonly :             node =  (Node *)new Readonly(p,r);
                                  break;
    case IdSimpleDeclaratorList : node =  (Node *)new SimpleDeclaratorList(p,r);
                                  break;
    case IdExceptDcl :            node =  (Node *)new ExceptDcl(p,r);
                                  break;
    case IdOpDcl :                node =  (Node *)new OpDcl(p,r);
                                  break;
    case IdOpTypeSpec :           node =  (Node *)new OpTypeSpec(p,r);
                                  break;
    case IdOpTypeSpecA :          node =  (Node *)new OpTypeSpecA(p,r);
                                  break;
    case IdOpTypeSpecB :          node =  (Node *)new OpTypeSpecB(p,r);
                                  break;
    case IdOpAttribute :          node =  (Node *)new OpAttribute(p,r);
                                  break;
    case IdParameterDcls :        node =  (Node *)new ParameterDcls(p,r);
                                  break;
    case IdParameterDclsA :       node =  (Node *)new ParameterDclsA(p,r);
                                  break;
    case IdParamDcl :             node =  (Node *)new ParamDcl(p,r);
                                  break;
    case IdParamAttribute :       node =  (Node *)new ParamAttribute(p,r);
                                  break;
    case IdRaisesExpr :           node =  (Node *)new RaisesExpr(p,r);
                                  break;
    case IdContextExpr :          node =  (Node *)new ContextExpr(p,r);
                                  break;
    case IdParamTypeSpec :        node =  (Node *)new ParamTypeSpec(p,r);
                                  break;
    case IdIdentifier :           node =  (Node *)new _Identifier(p,r);
                                  break;
    case IdIntegerLiteral :       node =  (Node *)new IntegerLiteral(p,r);
                                  break;
    case IdCharacterLiteral :     node =  (Node *)new CharacterLiteral(p,r);
                                  break;
    case IdStringLiteral :        node =  (Node *)new StringLiteral(p,r);
                                  break;
    case IdFloatingPtLiteral :    node =  (Node *)new FloatingPtLiteral(p,r);
                                  break;
    default :                     node =  0;
  }
  if (node == 0)
  {
    if (p != 0)
      printf("Error: Node %d is unable to create node %d\n\n", p->Id(), id);
    else
      printf("Error: Factory is unable to create node %d\n\n", id);

    fflush(stdout);

    exit(EXIT_FAILURE);
  }
  return node;
}
