//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/utils/nodes/src/grammar.cpp
//
// Modifications:
// $Log: grammar.cpp,v $
// Revision 1.2  2000/02/11 16:05:32  mli
// Decl of a ExprValue extracted.
//
// Revision 1.4  1998/05/11 10:49:04  stephen
// Enums evaluate as unsigned longs.
//
// Revision 1.3  1997/12/04 14:15:09  stephen
// AnyType destructor added.
//
// Revision 1.2  1997/10/14 10:24:11  james
// Extended types removed
//
// Revision 1.1  1997/10/06 11:36:00  james
// Initial Sources
//
//

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>

#include "grammar.hh"
#include "utilities.hh"
   
Specification::Specification() : ListNode(IdSpecification)
{
  childId = IdDefinition;
}

Specification::Specification(Node *p, RecursionCount r) : ListNode(p,IdSpecification,r)
{
  childId = IdDefinition;
}

Specification::Specification(const Specification &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}



Definition::Definition() : ChoiceNode(IdDefinition)
{
  childIds[0] = IdTypeDcl;
  childIds[1] = IdConstDcl;
  childIds[2] = IdExceptDcl;
  childIds[3] = IdInterface;
  childIds[4] = IdModule;
}

Definition::Definition(Node *p, RecursionCount r) : ChoiceNode(p,IdDefinition,r)
{
  childIds[0] = IdTypeDcl;
  childIds[1] = IdConstDcl;
  childIds[2] = IdExceptDcl;
  childIds[3] = IdInterface;
  childIds[4] = IdModule;
}

Definition::Definition(const Definition &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}


Module::Module() : SequenceNode(IdModule)
{
  childIds[0] = IdIdentifier;
  childIds[1] = IdDefinitions;
}

Module::Module(Node *p, RecursionCount r) : SequenceNode(p,IdModule,r)
{
  childIds[0] = IdIdentifier;
  childIds[1] = IdDefinitions;
}

Module::Module(const Module &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}

Definitions::Definitions() : ListNode(IdDefinitions)
{
  childId = IdDefinition;
}

Definitions::Definitions(Node *p, RecursionCount r) : ListNode(p,IdDefinitions,r)
{
  childId = IdDefinition;
}

Definitions::Definitions(const Definitions &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}



Interface::Interface() : ChoiceNode(IdInterface)
{
  childIds[0] = IdInterfaceDcl;
  childIds[1] = IdForwardDcl;
}

Interface::Interface(Node *p, RecursionCount r) : ChoiceNode(p,IdInterface,r)
{
  childIds[0] = IdInterfaceDcl;
  childIds[1] = IdForwardDcl;
}

Interface::Interface(const Interface &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}


InterfaceDcl::InterfaceDcl() : SequenceNode(IdInterfaceDcl)
{
  // Note that the order of inheritancespec and identifier are reversed to 
  // prevent interface from inheriting from itself
  childIds[0] = IdInheritanceSpec;
  childIds[1] = IdIdentifier;
  childIds[2] = IdInterfaceBody;

  childIsOptional[0] = 1;
  childIsOptional[2] = 1;
}

InterfaceDcl::InterfaceDcl(Node *p, RecursionCount r) : SequenceNode(p,IdInterfaceDcl,r)
{
  // Note that the order of inheritancespec and identifier are reversed to 
  // prevent interface from inheriting from itself
  childIds[1] = IdIdentifier;
  childIds[0] = IdInheritanceSpec;
  childIds[2] = IdInterfaceBody;

  childIsOptional[0] = 1;
  childIsOptional[2] = 1;
}

InterfaceDcl::InterfaceDcl(const InterfaceDcl &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}


ForwardDcl::ForwardDcl() : SequenceNode(IdForwardDcl)
{
  childIds[0] = IdIdentifier;
}

ForwardDcl::ForwardDcl(Node *p, RecursionCount r) : SequenceNode(p,IdForwardDcl,r)
{
  childIds[0] = IdIdentifier;
}

ForwardDcl::ForwardDcl(const ForwardDcl &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}


InterfaceBody::InterfaceBody() : ListNode(IdInterfaceBody)
{
  childId = IdExport;
}

InterfaceBody::InterfaceBody(Node *p, RecursionCount r) : ListNode(p,IdInterfaceBody,r)
{
  childId = IdExport;
}

InterfaceBody::InterfaceBody(const InterfaceBody &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}



Export::Export() : ChoiceNode(IdExport)
{
  childIds[0] = IdTypeDcl;
  childIds[1] = IdConstDcl;
  childIds[2] = IdExceptDcl;
  childIds[3] = IdAttrDcl;
  childIds[4] = IdOpDcl;
}

Export::Export(Node *p, RecursionCount r) : ChoiceNode(p,IdExport,r)
{
  childIds[0] = IdTypeDcl;
  childIds[1] = IdConstDcl;
  childIds[2] = IdExceptDcl;
  childIds[3] = IdAttrDcl;
  childIds[4] = IdOpDcl;
}

Export::Export(const Export &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



InheritanceSpec::InheritanceSpec() : ListNode(IdInheritanceSpec)
{
  childId = IdScopedName;
  separator = ", ";
}

InheritanceSpec::InheritanceSpec(Node *p, RecursionCount r) : ListNode(p,IdInheritanceSpec,r)
{
  childId = IdScopedName;
  separator = ", ";
}

InheritanceSpec::InheritanceSpec(const InheritanceSpec &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}

_ScopedName::_ScopedName() : SequenceNode(IdScopedName)
{
  childIds[0] = IdScopedNameA;
  childIds[1] = IdScopedNameB;


  childIsOptional[0] = 1;
  value = 0;
}

_ScopedName::_ScopedName(Node *p, RecursionCount r) : SequenceNode(p,IdScopedName,r)
{
  childIds[0] = IdScopedNameA;
  childIds[1] = IdScopedNameB;

  childIsOptional[0] = 1;

  value = 0;
}

_ScopedName::_ScopedName(const _ScopedName &r)
{
  RecursionCount rc;

  if (&r != this)
  {
    *((SequenceNode*)(this)) = *((Node*)(&r));
       
     memset(rc, 0, sizeof(RecursionCount));
     value = NewNode(r.Parent(), r.Id(), rc);
   }
}

_ScopedName::~_ScopedName()
{
  if (value != 0)
    delete value;
  value = 0;
}

ExprValue
_ScopedName::exprValue()
{
  Node *p;
  
  if ((p = FindDefinition(*this,(_ScopedName&)*this)) == 0)
  {
    cerr << "Error: Unable to find definition of " << this << " in call to exprValue" << endl;
    exit(EXIT_FAILURE);
  }

  ExprValue ret;
  ret = p->exprValue();
  return ret;
}
 


ScopedNameA::ScopedNameA() : Node(IdScopedNameA)
{
}

ScopedNameA::ScopedNameA(Node *p, RecursionCount r) : Node(p,IdScopedNameA,r)
{
}

ScopedNameA::ScopedNameA(const ScopedNameA &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}



ScopedNameB::ScopedNameB() : ListNode(IdScopedNameB)
{
  childId = IdIdentifier;
  separator = "::";
}

ScopedNameB::ScopedNameB(Node *p, RecursionCount r) : ListNode(p,IdScopedNameB,r)
{
  childId = IdIdentifier;
  separator = "::";
}

ScopedNameB::ScopedNameB(const ScopedNameB &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}



ConstDcl::ConstDcl() : SequenceNode(IdConstDcl)
{
  // Note that the order of ConstExp and Identifier are reversed to prevent circular definitions
  childIds[0] = IdConstType;
  childIds[1] = IdConstExp;
  childIds[2] = IdIdentifier;
}

ConstDcl::ConstDcl(Node *p, RecursionCount r) : SequenceNode(p,IdConstDcl,r)
{
  // Note that the order of ConstExp and Identifier are reversed to prevent circular definitions
  childIds[0] = IdConstType;
  childIds[1] = IdConstExp;
  childIds[2] = IdIdentifier;
}

ConstDcl::ConstDcl(const ConstDcl &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}




ConstType::ConstType() : ChoiceNode(IdConstType)
{
  childIds[0] = IdIntegerType;
  childIds[1] = IdCharType;
  childIds[2] = IdBooleanType;
  childIds[3] = IdFloatingPtType;
  childIds[4] = IdStringType;
  childIds[5] = IdScopedName;
}

ConstType::ConstType(Node *p, RecursionCount r) : ChoiceNode(p,IdConstType,r)
{
  childIds[0] = IdIntegerType;
  childIds[1] = IdCharType;
  childIds[2] = IdBooleanType;
  childIds[3] = IdFloatingPtType;
  childIds[4] = IdStringType;
  childIds[5] = IdScopedName;
}

ConstType::ConstType(const ConstType &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}


PositiveIntConst::PositiveIntConst() : ChoiceNode(IdPositiveIntConst)
{
  childIds[0] = IdConstExp;
}

PositiveIntConst::PositiveIntConst(Node *p, RecursionCount r) : ChoiceNode(p,IdPositiveIntConst,r)
{
  childIds[0] = IdConstExp;
}

PositiveIntConst::PositiveIntConst(const PositiveIntConst &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}




TypeDcl::TypeDcl() : ChoiceNode(IdTypeDcl)
{
  childIds[0] = IdTypeDeclarator;
  childIds[1] = IdConstrTypeSpec;
}

TypeDcl::TypeDcl(Node *p, RecursionCount r) : ChoiceNode(p,IdTypeDcl,r)
{
  childIds[0] = IdTypeDeclarator;
  childIds[1] = IdConstrTypeSpec;
}

TypeDcl::TypeDcl(const TypeDcl &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}


TypeDeclarator::TypeDeclarator() : SequenceNode(IdTypeDeclarator)
{
  childIds[0] = IdTypeSpec;
  childIds[1] = IdDeclarators;
}

TypeDeclarator::TypeDeclarator(Node *p, RecursionCount r) : SequenceNode(p,IdTypeDeclarator,r)
{
  childIds[0] = IdTypeSpec;
  childIds[1] = IdDeclarators;
}

TypeDeclarator::TypeDeclarator(const TypeDeclarator &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}



TypeSpec::TypeSpec() : ChoiceNode(IdTypeSpec)
{
  childIds[0] = IdSimpleTypeSpec;
  childIds[1] = IdConstrTypeSpec;
}

TypeSpec::TypeSpec(Node *p, RecursionCount r) : ChoiceNode(p,IdTypeSpec,r)
{
  childIds[0] = IdSimpleTypeSpec;
  childIds[1] = IdConstrTypeSpec;
}

TypeSpec::TypeSpec(const TypeSpec &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



SimpleTypeSpec::SimpleTypeSpec() : ChoiceNode(IdSimpleTypeSpec)
{
  childIds[0] = IdBaseTypeSpec;
  childIds[1] = IdTemplateTypeSpec;
  childIds[2] = IdScopedName;
}

SimpleTypeSpec::SimpleTypeSpec(Node *p, RecursionCount r) : ChoiceNode(p,IdSimpleTypeSpec,r)
{
  childIds[0] = IdBaseTypeSpec;
  childIds[1] = IdTemplateTypeSpec;
  childIds[2] = IdScopedName;
}

SimpleTypeSpec::SimpleTypeSpec(const SimpleTypeSpec &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}


BaseTypeSpec::BaseTypeSpec() : ChoiceNode(IdBaseTypeSpec)
{
  childIds[0] = IdFloatingPtType;
  childIds[1] = IdIntegerType;
  childIds[2] = IdCharType;
  childIds[3] = IdBooleanType;
  childIds[4] = IdOctetType;
  childIds[5] = IdAnyType;
  childIds[6] = IdObjectType;
}

BaseTypeSpec::BaseTypeSpec(Node *p, RecursionCount r) : ChoiceNode(p,IdBaseTypeSpec,r)
{
  childIds[0] = IdFloatingPtType;
  childIds[1] = IdIntegerType;
  childIds[2] = IdCharType;
  childIds[3] = IdBooleanType;
  childIds[4] = IdOctetType;
  childIds[5] = IdAnyType;
  childIds[6] = IdObjectType;
}

BaseTypeSpec::BaseTypeSpec(const BaseTypeSpec &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



ObjectType::ObjectType() : Node(IdObjectType)
{
}

ObjectType::ObjectType(Node *p, RecursionCount r) : Node(p,IdObjectType,r)
{
}

ObjectType::ObjectType(const ObjectType &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}



TemplateTypeSpec::TemplateTypeSpec() : ChoiceNode(IdTemplateTypeSpec)
{
  childIds[0] = IdSequenceType;
  childIds[1] = IdStringType;
}

TemplateTypeSpec::TemplateTypeSpec(Node *p, RecursionCount r) : ChoiceNode(p,IdTemplateTypeSpec,r)
{
  childIds[0] = IdSequenceType;
  childIds[1] = IdStringType;
}

TemplateTypeSpec::TemplateTypeSpec(const TemplateTypeSpec &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



ConstrTypeSpec::ConstrTypeSpec() : ChoiceNode(IdConstrTypeSpec)
{
  childIds[0] = IdStructType;
  childIds[1] = IdUnionType;
  childIds[2] = IdEnumType;
}

ConstrTypeSpec::ConstrTypeSpec(Node *p, RecursionCount r) : ChoiceNode(p,IdConstrTypeSpec,r)
{
  childIds[0] = IdStructType;
  childIds[1] = IdUnionType;
  childIds[2] = IdEnumType;
}

ConstrTypeSpec::ConstrTypeSpec(const ConstrTypeSpec &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



Declarators::Declarators() : ListNode(IdDeclarators)
{
  childId = IdDeclarator;
  separator = ", ";
}

Declarators::Declarators(Node *p, RecursionCount r) : ListNode(p,IdDeclarators,r)
{
  childId = IdDeclarator;
  separator = ", ";
}

Declarators::Declarators(const Declarators &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}



Declarator::Declarator() : ChoiceNode(IdDeclarator)
{
  childIds[0] = IdSimpleDeclarator;
  childIds[1] = IdComplexDeclarator;
}

Declarator::Declarator(Node *p, RecursionCount r) : ChoiceNode(p,IdDeclarator,r)
{
  childIds[0] = IdSimpleDeclarator;
  childIds[1] = IdComplexDeclarator;
}

Declarator::Declarator(const Declarator &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}


SimpleDeclarator::SimpleDeclarator() : SequenceNode(IdSimpleDeclarator)
{
  childIds[0] = IdIdentifier;

  value = 0;
}

SimpleDeclarator::SimpleDeclarator(Node *p, RecursionCount r) : SequenceNode(p,IdSimpleDeclarator,r)
{
  childIds[0] = IdIdentifier;

  value = 0;
}

SimpleDeclarator::SimpleDeclarator(const SimpleDeclarator &r)
{
  RecursionCount rc;

  if (&r != this)
  {
    *((SequenceNode*)(this)) = *((Node*)(&r));

    memset(rc, 0, sizeof(RecursionCount));
    value = NewNode(r.Parent(), r.Id(), rc);
  }
}

SimpleDeclarator::~SimpleDeclarator()
{
  if (value != 0)
    delete value;
  value = 0;
}



ComplexDeclarator::ComplexDeclarator() : SequenceNode(IdComplexDeclarator)
{
  childIds[0] = IdArrayDeclarator;
}

ComplexDeclarator::ComplexDeclarator(Node *p, RecursionCount r) : SequenceNode(p,IdComplexDeclarator,r)
{
  childIds[0] = IdArrayDeclarator;
}

ComplexDeclarator::ComplexDeclarator(const ComplexDeclarator &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}



FloatingPtType::FloatingPtType() : ChoiceNode(IdFloatingPtType)
{
  childIds[0] = IdFloatType;
  childIds[1] = IdDoubleType;
}

FloatingPtType::FloatingPtType(Node *p, RecursionCount r) : ChoiceNode(p,IdFloatingPtType,r)
{
  childIds[0] = IdFloatType;
  childIds[1] = IdDoubleType;
}

FloatingPtType::FloatingPtType(const FloatingPtType &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



FloatType::FloatType() : Node(IdFloatType)
{
}

FloatType::FloatType(Node *p, RecursionCount r) : Node(p,IdFloatType,r)
{
}

FloatType::FloatType(const FloatType &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}


DoubleType::DoubleType() : Node(IdDoubleType)
{
}

DoubleType::DoubleType(Node *p, RecursionCount r) : Node(p,IdDoubleType,r)
{
}

DoubleType::DoubleType(const DoubleType &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}


IntegerType::IntegerType() : ChoiceNode(IdIntegerType)
{
  childIds[0] = IdSignedInt;
  childIds[1] = IdUnsignedInt;
}

IntegerType::IntegerType(Node *p, RecursionCount r) : ChoiceNode(p,IdIntegerType,r)
{
  childIds[0] = IdSignedInt;
  childIds[1] = IdUnsignedInt;
}

IntegerType::IntegerType(const IntegerType &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



SignedInt::SignedInt() : ChoiceNode(IdSignedInt)
{
  childIds[0] = IdSignedLongInt;
  childIds[1] = IdSignedShortInt;
}

SignedInt::SignedInt(Node *p, RecursionCount r) : ChoiceNode(p,IdSignedInt,r)
{
  childIds[0] = IdSignedLongInt;
  childIds[1] = IdSignedShortInt;
}

SignedInt::SignedInt(const SignedInt &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



SignedLongInt::SignedLongInt() : Node(IdSignedLongInt)
{
}

SignedLongInt::SignedLongInt(Node *p, RecursionCount r) : Node(p,IdSignedLongInt,r)
{
}

SignedLongInt::SignedLongInt(const SignedLongInt &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}



SignedShortInt::SignedShortInt() : Node(IdSignedShortInt)
{
}

SignedShortInt::SignedShortInt(Node *p, RecursionCount r) : Node(p,IdSignedShortInt,r)
{
}

SignedShortInt::SignedShortInt(const SignedShortInt &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}



UnsignedInt::UnsignedInt() : ChoiceNode(IdUnsignedInt)
{
  childIds[0] = IdUnsignedLongInt;
  childIds[1] = IdUnsignedShortInt;
}

UnsignedInt::UnsignedInt(Node *p, RecursionCount r) : ChoiceNode(p,IdUnsignedInt,r)
{
  childIds[0] = IdUnsignedLongInt;
  childIds[1] = IdUnsignedShortInt;
}

UnsignedInt::UnsignedInt(const UnsignedInt &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}




UnsignedLongInt::UnsignedLongInt() : Node(IdUnsignedLongInt)
{
}

UnsignedLongInt::UnsignedLongInt(Node *p, RecursionCount r) : Node(p,IdUnsignedLongInt,r)
{
}

UnsignedLongInt::UnsignedLongInt(const UnsignedLongInt &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}



UnsignedShortInt::UnsignedShortInt() : Node(IdUnsignedShortInt)
{
}

UnsignedShortInt::UnsignedShortInt(Node *p, RecursionCount r) : Node(p,IdUnsignedShortInt,r)
{
}

UnsignedShortInt::UnsignedShortInt(const UnsignedShortInt &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}



CharType::CharType() : Node(IdCharType)
{
}

CharType::CharType(Node *p, RecursionCount r) : Node(p,IdCharType,r)
{
}

CharType::CharType(const CharType &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}




BooleanType::BooleanType() : Node(IdBooleanType)
{
}

BooleanType::BooleanType(Node *p, RecursionCount r) : Node(p,IdBooleanType,r)
{
}

BooleanType::BooleanType(const BooleanType &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}



OctetType::OctetType() : Node(IdOctetType)
{
}

OctetType::OctetType(Node *p, RecursionCount r) : Node(p,IdOctetType,r)
{
}

OctetType::OctetType(const OctetType &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}



AnyType::AnyType() : Node(IdAnyType)
{
    value = 0;
}

AnyType::~AnyType()
{
    if (value)
	delete value;

    value = 0;
}

AnyType::AnyType(Node *p, RecursionCount r) : Node(p,IdAnyType,r)
{
    value = 0;
}

AnyType::AnyType(const AnyType &r)
{
    value = 0;
  *((Node*)(this)) = *((Node*)(&r));
}



StructType::StructType() : SequenceNode(IdStructType)
{
  // Note that the order of MemberList and Identifier are reversed to prevent circular definitions
  childIds[0] = IdMemberList;
  childIds[1] = IdIdentifier;
}

StructType::StructType(Node *p, RecursionCount r) : SequenceNode(p,IdStructType,r)
{
  // Note that the order of MemberList and Identifier are reversed to prevent circular definitions
  childIds[0] = IdMemberList;
  childIds[1] = IdIdentifier;
}

StructType::StructType(const StructType &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}


MemberList::MemberList() : ListNode(IdMemberList)
{
  childId = IdMember;
}

MemberList::MemberList(Node *p, RecursionCount r) : ListNode(p,IdMemberList,r)
{
  childId = IdMember;
}

MemberList::MemberList(const MemberList &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}



Member::Member() : SequenceNode(IdMember)
{
  childIds[0] = IdTypeSpec;
  childIds[1] = IdDeclarators;
}

Member::Member(Node *p, RecursionCount r) : SequenceNode(p,IdMember,r)
{
  childIds[0] = IdTypeSpec;
  childIds[1] = IdDeclarators;
}

Member::Member(const Member &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}



UnionType::UnionType() : SequenceNode(IdUnionType)
{
  // Note that the order of UnionSwitch and Identifier are reversed to prevent circular definitions
  childIds[0] = IdUnionSwitch;
  childIds[1] = IdIdentifier;
}

UnionType::UnionType(Node *p, RecursionCount r) : SequenceNode(p,IdUnionType,r)
{
  // Note that the order of UnionSwitch and Identifier are reversed to prevent circular definitions
  childIds[0] = IdUnionSwitch;
  childIds[1] = IdIdentifier;
}

UnionType::UnionType(const UnionType &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}


UnionSwitch::UnionSwitch() : SequenceNode(IdUnionSwitch)
{
  childIds[0] = IdSwitchTypeSpec;
  childIds[1] = IdSwitchBody;

  value = 0;
}

UnionSwitch::UnionSwitch(Node *p, RecursionCount r) : SequenceNode(p,IdUnionSwitch,r)
{
  childIds[0] = IdSwitchTypeSpec;
  childIds[1] = IdSwitchBody;

  value = 0;
}

UnionSwitch::UnionSwitch(const UnionSwitch &r)
{
  RecursionCount rc;

  if (&r != this)
  {
    *((SequenceNode*)(this)) = *((Node*)(&r));

    memset(rc, 0, sizeof(RecursionCount));
    value = NewNode(r.Parent(), r.Id(), rc);
  }
}

UnionSwitch::~UnionSwitch()
{
  if (value)
    delete value;
  value = 0;
}


SwitchTypeSpec::SwitchTypeSpec() : ChoiceNode(IdSwitchTypeSpec)
{
  childIds[0] = IdIntegerType;
  childIds[1] = IdCharType;
  childIds[2] = IdBooleanType;
  childIds[3] = IdEnumType;
  childIds[4] = IdScopedName;
}

SwitchTypeSpec::SwitchTypeSpec(Node *p, RecursionCount r) : ChoiceNode(p,IdSwitchTypeSpec,r)
{
  childIds[0] = IdIntegerType;
  childIds[1] = IdCharType;
  childIds[2] = IdBooleanType;
  childIds[3] = IdEnumType;
  childIds[4] = IdScopedName;
}

SwitchTypeSpec::SwitchTypeSpec(const SwitchTypeSpec &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



SwitchBody::SwitchBody() : ListNode(IdSwitchBody)
{
  childId = IdCase;
}

SwitchBody::SwitchBody(Node *p, RecursionCount r) : ListNode(p,IdSwitchBody,r)
{
  childId = IdCase;
}

SwitchBody::SwitchBody(const SwitchBody &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}



Case::Case() : SequenceNode(IdCase)
{
  childIds[0] = IdCaseLabelList;
  childIds[1] = IdElementSpec;
}

Case::Case(Node *p, RecursionCount r) : SequenceNode(p,IdCase,r)
{
  childIds[0] = IdCaseLabelList;
  childIds[1] = IdElementSpec;
}

Case::Case(const Case &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}


CaseLabelList::CaseLabelList() : ListNode(IdCaseLabelList)
{
  childId = IdCaseLabel;
}

CaseLabelList::CaseLabelList(Node *p, RecursionCount r) : ListNode(p,IdCaseLabelList,r)
{
  childId = IdCaseLabel;
}

CaseLabelList::CaseLabelList(const CaseLabelList &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}


CaseLabel::CaseLabel() : ChoiceNode(IdCaseLabel)
{
  childIds[0] = IdConstExp;
  childIds[1] = IdDefaultLabel;
}

CaseLabel::CaseLabel(Node *p, RecursionCount r) : ChoiceNode(p,IdCaseLabel,r)
{
  childIds[0] = IdConstExp;
  childIds[1] = IdDefaultLabel;
}

CaseLabel::CaseLabel(const CaseLabel &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



DefaultLabel::DefaultLabel() : Node(IdDefaultLabel)
{
}

DefaultLabel::DefaultLabel(Node *p, RecursionCount r) : Node(p,IdDefaultLabel,r)
{
}

DefaultLabel::DefaultLabel(const DefaultLabel &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}


ElementSpec::ElementSpec() : SequenceNode(IdElementSpec)
{
  childIds[0] = IdTypeSpec;
  childIds[1] = IdDeclarator;
}

ElementSpec::ElementSpec(Node *p, RecursionCount r) : SequenceNode(p,IdElementSpec,r)
{
  childIds[0] = IdTypeSpec;
  childIds[1] = IdDeclarator;
}

ElementSpec::ElementSpec(const ElementSpec &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}



EnumType::EnumType() : SequenceNode(IdEnumType)
{
  childIds[0] = IdIdentifier;
  childIds[1] = IdEnumeratorList;

  value = 0;
}

EnumType::EnumType(Node *p, RecursionCount r) : SequenceNode(p,IdEnumType,r)
{
  childIds[0] = IdIdentifier;
  childIds[1] = IdEnumeratorList;

  value = 0;
}

EnumType::EnumType(const EnumType &r)
{
  if (&r != this)
  {
    *((SequenceNode*)(this)) = *((Node*)(&r));
    value = r.value;
  }
}



EnumeratorList::EnumeratorList() : ListNode(IdEnumeratorList)
{
  childId = IdEnumerator;
  separator = ",";
}

EnumeratorList::EnumeratorList(Node *p, RecursionCount r) : ListNode(p,IdEnumeratorList,r)
{
  childId = IdEnumerator;
  separator = ",";
}

EnumeratorList::EnumeratorList(const EnumeratorList &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}



Enumerator::Enumerator() : SequenceNode(IdEnumerator)
{
  childIds[0] = IdIdentifier;
}

Enumerator::Enumerator(Node *p, RecursionCount r) : SequenceNode(p,IdEnumerator,r)
{
  childIds[0] = IdIdentifier;
}

Enumerator::Enumerator(const Enumerator &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}

ExprValue
Enumerator::exprValue()
{
  ExprValue ret;
  Node &p = *parent;


  for (int i = 0; i < p.Length(); ++i)
  {
    if (&p[i] == this)
      ret = (unsigned long)i;
  }

  return ret;
}


SequenceType::SequenceType() : SequenceNode(IdSequenceType)
{
  childIds[0] = IdSimpleTypeSpec;
  childIds[1] = IdPositiveIntConst;
  childIsOptional[1] = 1;

  value = 0;
}

SequenceType::SequenceType(Node *p, RecursionCount r) : SequenceNode(p,IdSequenceType,r)
{
  childIds[0] = IdSimpleTypeSpec;
  childIds[1] = IdPositiveIntConst;
  childIsOptional[1] = 1;

  value = 0;
}

SequenceType::SequenceType(const SequenceType &r)
{
  if (&r != this)
  {
    *((SequenceNode*)(this)) = *((Node*)(&r));
    if (r.value != 0)
    {
      int i = 0;
      seqLen = r.seqLen;
      value = new SimpleTypeSpec[seqLen];
      for (i = 0; i < seqLen ; i++)
        (ChoiceNode&)(value[i]) = (const Node&)r.value[i];
    }
    else
      value = 0;
  }
}

SequenceType::~SequenceType()
{
  if (value != 0)
    delete [] value;
 
  value = 0;
  seqLen = 0;
}




StringType::StringType() : SequenceNode(IdStringType)
{
  childIds[0] = IdPositiveIntConst;
  childIsOptional[0] = 1;

  value = 0;
}

StringType::StringType(Node *p, RecursionCount r) : SequenceNode(p,IdStringType,r)
{
  childIds[0] = IdPositiveIntConst;
  childIsOptional[0] = 1;

  value = 0;
}

StringType::StringType(const StringType &r)
{
  if (&r != this)
  {
    *((SequenceNode*)(this)) = *((Node*)(&r));
    if (r.value)
    {
      value = new char[strlen(r.value)+1];
      strcpy(value, r.value);
    }
    else
      value = 0;
  }
}

StringType::~StringType()
{
  if (value != 0)
    delete value;
  value = 0;
}




ArrayDeclarator::ArrayDeclarator() : SequenceNode(IdArrayDeclarator)
{
  childIds[0] = IdIdentifier;
  childIds[1] = IdFixedArraySizeList;

  value = 0;
}

ArrayDeclarator::ArrayDeclarator(Node *p, RecursionCount r) : SequenceNode(p,IdArrayDeclarator,r)
{
  childIds[0] = IdIdentifier;
  childIds[1] = IdFixedArraySizeList;

  value = 0;
}

ArrayDeclarator::ArrayDeclarator(const ArrayDeclarator &r)
{
  if (&r != this)
  {
    *((SequenceNode*)(this)) = *((Node*)(&r));
    if (r.value != 0)
    {
      int i = 0;
      seqLen = r.seqLen;
      value = new TypeSpec[seqLen];
      for (i = 0; i < seqLen ; i++)
        (ChoiceNode&)(value[i]) = (const Node&)r.value[i];
    }
    else
      value = 0;
  }
}

ArrayDeclarator::~ArrayDeclarator()
{
  if (value)
    delete [] value;

  value = 0;
  seqLen = 0;
}



FixedArraySizeList::FixedArraySizeList() : ListNode(IdFixedArraySizeList)
{
  childId = IdFixedArraySize;
}

FixedArraySizeList::FixedArraySizeList(Node *p, RecursionCount r) : ListNode(p,IdFixedArraySizeList,r)
{
  childId = IdFixedArraySize;
}

FixedArraySizeList::FixedArraySizeList(const FixedArraySizeList &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}


ExprValue
FixedArraySizeList::exprValue()
{
  int i = 1;
  ExprValue ret;

  if (childNodes[0] == 0)
    ret = (unsigned long)0;
  else
  {
    ret = childNodes[0]->exprValue();
    for (i; i < Length(); i++)
      ret = ret * childNodes[i]->exprValue();
  }
  return ret;
}
 

FixedArraySize::FixedArraySize() : ChoiceNode(IdFixedArraySize)
{
  childIds[0] = IdPositiveIntConst;
}

FixedArraySize::FixedArraySize(Node *p, RecursionCount r) : ChoiceNode(p,IdFixedArraySize,r)
{
  childIds[0] = IdPositiveIntConst;
}

FixedArraySize::FixedArraySize(const FixedArraySize &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



AttrDcl::AttrDcl() : SequenceNode(IdAttrDcl)
{
  childIds[0] = IdReadonly;
  childIds[1] = IdParamTypeSpec;
  childIds[2] = IdSimpleDeclaratorList;

  childIsOptional[0] = 1;
}

AttrDcl::AttrDcl(Node *p, RecursionCount r) : SequenceNode(p,IdAttrDcl,r)
{
  childIds[0] = IdReadonly;
  childIds[1] = IdParamTypeSpec;
  childIds[2] = IdSimpleDeclaratorList;

  childIsOptional[0] = 1;
}

AttrDcl::AttrDcl(const AttrDcl &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}



Readonly::Readonly() : Node(IdReadonly)
{
}

Readonly::Readonly(Node *p, RecursionCount r) : Node(p,IdReadonly,r)
{
}

Readonly::Readonly(const Readonly &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}



SimpleDeclaratorList::SimpleDeclaratorList() : ListNode(IdSimpleDeclaratorList)
{
  childId = IdSimpleDeclarator;
  separator = ", ";
}

SimpleDeclaratorList::SimpleDeclaratorList(Node *p, RecursionCount r) : ListNode(p,IdSimpleDeclaratorList,r)
{
  childId = IdSimpleDeclarator;
  separator = ", ";
}

SimpleDeclaratorList::SimpleDeclaratorList(const SimpleDeclaratorList &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}



ExceptDcl::ExceptDcl() : SequenceNode(IdExceptDcl)
{
  childIds[0] = IdIdentifier;
  childIds[1] = IdMemberList;

  childIsOptional[1] = 1;
}

ExceptDcl::ExceptDcl(Node *p, RecursionCount r) : SequenceNode(p,IdExceptDcl,r)
{
  childIds[0] = IdIdentifier;
  childIds[1] = IdMemberList;

  childIsOptional[1] = 1;
}

ExceptDcl::ExceptDcl(const ExceptDcl &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}



OpDcl::OpDcl() : SequenceNode(IdOpDcl)
{
  childIds[0] = IdOpTypeSpec;
  childIds[1] = IdIdentifier;
  childIds[2] = IdParameterDcls;
  childIds[3] = IdRaisesExpr;
  childIds[4] = IdContextExpr;

  childIsOptional[3] = 1;
  childIsOptional[4] = 1;
}

OpDcl::OpDcl(Node *p, RecursionCount r) : SequenceNode(p,IdOpDcl,r)
{
  childIds[0] = IdOpTypeSpec;
  childIds[1] = IdIdentifier;
  childIds[2] = IdParameterDcls;
  childIds[3] = IdRaisesExpr;
  childIds[4] = IdContextExpr;

  childIsOptional[3] = 1;
  childIsOptional[4] = 1;
}

OpDcl::OpDcl(const OpDcl &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}




OpTypeSpec::OpTypeSpec() : ChoiceNode(IdOpTypeSpec)
{
  childIds[0] = IdParamTypeSpec;
  childIds[1] = IdOpTypeSpecA;
}

OpTypeSpec::OpTypeSpec(Node *p, RecursionCount r) : ChoiceNode(p,IdOpTypeSpec,r)
{
  childIds[0] = IdParamTypeSpec;
  childIds[1] = IdOpTypeSpecA;
}

OpTypeSpec::OpTypeSpec(const OpTypeSpec &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}



OpTypeSpecA::OpTypeSpecA() : SequenceNode(IdOpTypeSpecA)
{
  childIds[0] = IdOpAttribute;
  childIds[1] = IdOpTypeSpecB;

  childIsOptional[0] = 1;
}

OpTypeSpecA::OpTypeSpecA(Node *p, RecursionCount r) : SequenceNode(p,IdOpTypeSpecA,r)
{
  childIds[0] = IdOpAttribute;
  childIds[1] = IdOpTypeSpecB;

  childIsOptional[0] = 1;
}

OpTypeSpecA::OpTypeSpecA(const OpTypeSpecA &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}



OpTypeSpecB::OpTypeSpecB() : Node(IdOpTypeSpecB)
{
}

OpTypeSpecB::OpTypeSpecB(Node *p, RecursionCount r) : Node(p,IdOpTypeSpecB,r)
{
}

OpTypeSpecB::OpTypeSpecB(const OpTypeSpecB &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}



OpAttribute::OpAttribute() : Node(IdOpAttribute)
{
}

OpAttribute::OpAttribute(Node *p, RecursionCount r) : Node(p,IdOpAttribute,r)
{
}

OpAttribute::OpAttribute(const OpAttribute &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}



ParameterDcls::ParameterDcls() : SequenceNode(IdParameterDcls)
{
  childIds[0] = IdParameterDclsA;

  childIsOptional[0] = 1;
}

ParameterDcls::ParameterDcls(Node *p, RecursionCount r) : SequenceNode(p,IdParameterDcls,r)
{
  childIds[0] = IdParameterDclsA;

  childIsOptional[0] = 1;
}

ParameterDcls::ParameterDcls(const ParameterDcls &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}



ParameterDclsA::ParameterDclsA() : ListNode(IdParameterDclsA)
{
  childId = IdParamDcl;
  separator = ",";
}

ParameterDclsA::ParameterDclsA(Node *p, RecursionCount r) : ListNode(p,IdParameterDclsA,r)
{
  childId = IdParamDcl;
  separator = ",";
}

ParameterDclsA::ParameterDclsA(const ParameterDclsA &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}



ParamDcl::ParamDcl() : SequenceNode(IdParamDcl)
{
  childIds[0] = IdParamAttribute;
  childIds[1] = IdParamTypeSpec;
  childIds[2] = IdSimpleDeclarator;
}

ParamDcl::ParamDcl(Node *p, RecursionCount r) : SequenceNode(p,IdParamDcl,r)
{
  childIds[0] = IdParamAttribute;
  childIds[1] = IdParamTypeSpec;
  childIds[2] = IdSimpleDeclarator;
}

ParamDcl::ParamDcl(const ParamDcl &r)
{
  *((SequenceNode*)(this)) = *((Node*)(&r));
}



ParamAttribute::ParamAttribute() : SimpleChoiceNode(IdParamAttribute)
{
  maxChoice = 2;
}

ParamAttribute::ParamAttribute(Node *p, RecursionCount r) : SimpleChoiceNode(p,IdParamAttribute,r)
{
  maxChoice = 2;
}

ParamAttribute::ParamAttribute(const ParamAttribute &r)
{
  *((SimpleChoiceNode*)(this)) = *((Node*)(&r));
}



RaisesExpr::RaisesExpr() : ListNode(IdRaisesExpr)
{
  childId = IdScopedName;
  separator = ", ";
  index = 0;
}

RaisesExpr::RaisesExpr(Node *p, RecursionCount r) : ListNode(p,IdRaisesExpr,r)
{
  childId = IdScopedName;
  separator = ", ";
  index = 0;
}

RaisesExpr::RaisesExpr(const RaisesExpr &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}





ContextExpr::ContextExpr() : ListNode(IdContextExpr)
{
  childId = IdStringLiteral;
  separator = ", ";
}

ContextExpr::ContextExpr(Node *p, RecursionCount r) : ListNode(p,IdContextExpr,r)
{
  childId = IdStringLiteral;
  separator = ", ";
}

ContextExpr::ContextExpr(const ContextExpr &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}



ParamTypeSpec::ParamTypeSpec() : ChoiceNode(IdParamTypeSpec)
{
  childIds[0] = IdBaseTypeSpec;
  childIds[1] = IdStringType;
  childIds[2] = IdScopedName;
}

ParamTypeSpec::ParamTypeSpec(Node *p, RecursionCount r) : ChoiceNode(p,IdParamTypeSpec,r)
{
  childIds[0] = IdBaseTypeSpec;
  childIds[1] = IdStringType;
  childIds[2] = IdScopedName;
}


ParamTypeSpec::ParamTypeSpec(const ParamTypeSpec &r)
{
  *((ChoiceNode*)(this)) = *((Node*)(&r));
}





ScopedNameBList::ScopedNameBList() : ListNode(IdScopedNameBList)
{
  childId = IdScopedNameB;
  separator = "\n";
}

ScopedNameBList::ScopedNameBList(Node *p, RecursionCount r) : ListNode(p,IdScopedNameBList,r)
{
  childId = IdScopedNameB;
}

ScopedNameBList::ScopedNameBList(const ScopedNameBList &r)
{
  *((ListNode*)(this)) = *((Node*)(&r));
}


