//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/utils/nodes/src/literals.cpp
//
// Modifications:
// $Log: literals.cpp,v $
// Revision 1.1.1.1  2000/02/09 17:07:14  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.9  1999/07/05 15:04:01  cwatson
// Added dtor to StringLiteral class.
//
// Revision 1.8  1999/07/05 09:57:46  cwatson
// Weird overwrite of value in StringLiteral class
//
// Revision 1.7  1998/05/11 10:51:48  stephen
// IntegerLiterals initialised to 0.
//
// Revision 1.6  1997/12/09 11:03:09  james
// long double fix
//
// Revision 1.5  1997/12/08 14:25:35  james
// Fixed cast operator
//
// Revision 1.4  1997/12/04 14:15:29  stephen
// Cast operator added.
//
// Revision 1.3  1997/11/26 20:30:14  james
// Fixed char literal problem
//
// Revision 1.2  1997/10/14 10:24:12  james
// Extended types removed
//
// Revision 1.1  1997/10/06 11:36:01  james
// Initial Sources
//



#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <ctype.h>
  
#include "literals.hh"
#include "utilities.hh"

#include "grammar.hh"
#include "constexpr.hh"
#include "outputidl.hh"



_Identifier::_Identifier() : Node(IdIdentifier)
{
  value = 0;
}

_Identifier::_Identifier(Node *p, RecursionCount r) : Node(p,IdIdentifier,r)
{
  value = 0;
}

_Identifier::_Identifier(const _Identifier &r)
{
  *((Node*)(this)) = *((Node*)(&r));

  if (r.value)
  {
    value = new char[strlen(r.value)+1];
    strcpy(value, r.value);
  }
  else
    value = 0;
}

_Identifier::~_Identifier()
{
  if (value != 0)
    delete value;
  value = 0;
}

Node &_Identifier::operator=(const Node &r)
{
  if (&r != this)
  {
    if (value != 0)
      delete value;

    Node::operator=(r);
    if (((_Identifier&)r).value)
    {
      value = new char[strlen(((_Identifier&)r).value)+1];
      strcpy(value, ((_Identifier&)r).value);
    }
    else
      value = 0;
  }

  return *this;
}

Node &_Identifier::operator=(const char *r)
{
  if (value != 0)
    delete value;

   if (r)
   {
     value = new char[strlen(r)+1];
     strcpy(value, r);
   }
   else
    value = 0;

  return *this;
}

int _Identifier::operator==(const _Identifier &r) const
{
  if (r.Id() == IdIdentifier
   && value != 0
   && r.value != 0
   && !strcmp(value, r.value))
    return 1;
  else
    return 0;
}


_Identifier::operator char *() 
{
  return value;
}


IntegerLiteral::IntegerLiteral() : Node(IdIntegerLiteral)
{
  value = (unsigned long)3;
}

IntegerLiteral::IntegerLiteral(Node *p, RecursionCount r) : Node(p,IdIntegerLiteral,r)
{
  value = (unsigned long)3;
}

IntegerLiteral::IntegerLiteral(const IntegerLiteral &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}

Node &IntegerLiteral::operator=(const char *r)
{
  unsigned long  l=0;
  int            base;

  if (r != 0)
  {
    if (r[0] == '0')
    {
      if (r[1] == 'x' || r[1] == 'X')
        base = 16;
      else
        base = 8;
    }
    else
      base = 10;

  	switch (base)
  	{
    	case 8:
          	sscanf(r+1,"%o",&l);
          	break;
    	case 10:
          	sscanf(r,"%d",&l);
          	break;
    	case 16:
          	sscanf(r+2,"%x",&l);
          	break;
  	}

  	value = (unsigned long) l;
	}
  return *this;
}

Node &IntegerLiteral::operator=(const Node &r)
{
  if (&r != this)
  {
    Node::operator=(r);
  
    value = ((IntegerLiteral*)&r)->value;
  }

  return *this;
}


ExprValue IntegerLiteral::exprValue()
{
  return value;
}



CharacterLiteral::CharacterLiteral() : Node(IdCharacterLiteral)
{
  value = 0;
}

CharacterLiteral::CharacterLiteral(Node *p, RecursionCount r) : Node(p,IdCharacterLiteral,r)
{
  value = 0;
}

CharacterLiteral::CharacterLiteral(const CharacterLiteral &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}

CharacterLiteral::~CharacterLiteral()
{
  if (value != 0)
    delete value;
}

Node &CharacterLiteral::operator=(const char *r)
{
  if (value != 0)
    delete value;

  value = new char[strlen(r)+1];
  strcpy(value,r);

  return *this;
}

Node &CharacterLiteral::operator=(const Node &r)
{
  if (&r != this)
  {
    Node::operator=(r);
  
    value = new char[strlen(((CharacterLiteral*)&r)->value)+1];
    strcpy(value,((CharacterLiteral*)&r)->value);
  }

  return *this;
}



#define isoctal(x) ((x) >= '0' && (x) <= '8')

ExprValue CharacterLiteral::exprValue()
{
  ExprValue  v;

  if (value != 0)
  {
    if (value[0] == '\'' 
     && value[1] != '\\'
     && value[2] == '\'')
    {
      v = (unsigned char)value[1];
    }
    else if (value[0] == '\'' 
     && value[1] == '\\' 
     && isoctal(value[2])
     && ((value[3] == '\'') 
        || (isoctal(value[3]) && value[4] == '\'')
        || (isoctal(value[3]) && isoctal(value[4]) && value[5] == '\'')))
    {
      int  i;
      sscanf(value+2,"%o",&i);
      v = (unsigned char) i;

    }
    else if (value[0] == '\'' 
     && value[1] == '\\' 
     && value[2] == 'x' 
     && isxdigit(value[3])
     && ((value[4] == '\'') || (isxdigit(value[4]) && value[5] == '\'')))
    {
      int  i;
      sscanf(value+3,"%x",&i);
      v = (unsigned char) i;
    }
    else if (value[0] == '\'' 
     && value[1] == '\\'
     && value[3] == '\'')
    {
      switch(value[2])
      {
        case 'n' :  v = (unsigned char)'\n';
                    break;
        case 't' :  v = (unsigned char)'\t';
                    break;
        case 'v' :  v = (unsigned char)'\v';
                    break;
        case 'b' :  v = (unsigned char)'\b';
                    break;
        case 'r' :  v = (unsigned char)'\r';
                    break;
        case 'f' :  v = (unsigned char)'\f';
                    break;
        case 'a' :  v = (unsigned char)'\a';
                    break;
        case '\\' :  v = (unsigned char)'\\';
                    break;
        case '?' :  v = (unsigned char)'\?';
                    break;
        case '\'' :  v = (unsigned char)'\'';
                    break;
        case '\"' :  v = (unsigned char)'\"';
                    break;
        default :
            cerr << "Error: badly formed character litersl " << value << endl;
            exit(EXIT_FAILURE);
      }
    }
    else
    {
      cerr << "Error: badly formed Character Literal" << value << endl;
      exit(EXIT_FAILURE);
    }
  }
  return v;
}


StringLiteral::StringLiteral() : Node(IdStringLiteral)
{
  value = 0;
}

StringLiteral::StringLiteral(Node *p, RecursionCount r) : Node(p,IdStringLiteral,r)
{
  value = 0;
}

StringLiteral::StringLiteral(const StringLiteral &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}

StringLiteral::~StringLiteral()
{
	if (value != 0)
		delete value;

  value = 0;
}

Node &StringLiteral::operator=(const char *r)
{
  if (value != 0)
    delete value;

  value = new char[strlen(r)+1];
  strcpy(value,r);

  return *this;
}

Node &StringLiteral::operator=(const Node &r)
{
  if (&r != this)
  {
    Node::operator=(r);
  
    value = new char[strlen(((StringLiteral*)&r)->value)+1];
    strcpy(value,((StringLiteral*)&r)->value);
  }

  return *this;
}


ExprValue StringLiteral::exprValue()
{
  ExprValue  v;

  v = value;
  return v;
}

StringLiteral::operator char * ()
{
    return value;
}


FloatingPtLiteral::FloatingPtLiteral() : Node(IdFloatingPtLiteral)
{
  value = 123.2;
}
 
FloatingPtLiteral::FloatingPtLiteral(Node *p, RecursionCount r) : Node(p,IdFloatingPtLiteral,r)
{
  value = 123.2;
}
 
FloatingPtLiteral::FloatingPtLiteral(const FloatingPtLiteral &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}
 

Node &FloatingPtLiteral::operator=(const char *r)
{
  double d;

  sscanf(r,"%lg",&d);

  value = d;
  
  return *this;
}

Node &FloatingPtLiteral::operator=(const Node &r)
{
  if (&r != this)
  {
    Node::operator=(r);
  
    value = ((FloatingPtLiteral*)&r)->value;
  }

  return *this;
}



ExprValue FloatingPtLiteral::exprValue()
{
  return value;
}


BooleanLiteral::BooleanLiteral() : SimpleChoiceNode(IdBooleanLiteral)
{
  maxChoice = 1;
}
 
BooleanLiteral::BooleanLiteral(Node *p, RecursionCount r) : SimpleChoiceNode(p,IdBooleanLiteral,r)
{
  maxChoice = 1;
}
 
BooleanLiteral::BooleanLiteral(const BooleanLiteral &r)
{
  *((Node*)(this)) = *((Node*)(&r));
}

Node &BooleanLiteral::operator=(const char *r)
{
  if (!strcmp(r,"FALSE"))
    choice = 0;
  else if (!strcmp(r,"TRUE"))
    choice = 1;

  return *this;
}


 
ExprValue BooleanLiteral::exprValue()
{
  ExprValue v;
 
  switch(choice)
  {
    case 0 :  v = VSORB_False;
            break;
    case 1 :  v = VSORB_True;
            break;
  }
 
  return v;
}
 


Literal::Literal() : ChoiceNode(IdLiteral)
{
  childIds[0] = IdIntegerLiteral;
  childIds[1] = IdFloatingPtLiteral;
  childIds[2] = IdBooleanLiteral;
  childIds[3] = IdCharacterLiteral;
  childIds[4] = IdStringLiteral;
}

Literal::Literal(Node *p, RecursionCount r) : ChoiceNode(p,IdLiteral,r)
{
  switch(GetEvalType(*this))
  {
    case etLong :
    case etULong :
            childIds[0] = IdIntegerLiteral;
            break;
    case etDouble :
            childIds[0] = IdFloatingPtLiteral;
            break;
    case etBoolean :
            childIds[0] = IdBooleanLiteral;
            break;
    case etChar :
            childIds[0] = IdCharacterLiteral;
            break;
    case etString :
            childIds[0] = IdStringLiteral;
            break;
    case etEnum :
            cerr << "Literal can not satisify requirement for enumerator" << endl;
            break;
    case etUndefined :
    default :
            cerr << "Error: " << nodeId << " evaluation type undefined" << endl;
  }
}


Literal & Literal::operator=(ExprValue &v)
{
  char *ptr = 0;
  IntegerLiteral  integerLiteral;
  FloatingPtLiteral  floatingPtLiteral;
  BooleanLiteral  booleanLiteral;
  CharacterLiteral  characterLiteral;
  StringLiteral  stringLiteral;


  ptr = v.string();

  switch(v.Type())
  {
    case etLong :
    case etULong :
            integerLiteral = ptr;
            *this << integerLiteral;
            break;
    case etDouble :
            floatingPtLiteral = ptr;
            *this << floatingPtLiteral;
            break;
    case etBoolean :
            booleanLiteral = ptr;
            *this << booleanLiteral;
            break;
    case etChar :
            characterLiteral = ptr;
            *this << characterLiteral;
            break;
    case etString :
            stringLiteral = ptr;
            *this << stringLiteral;
            break;
    case etEnum :
            cerr << "Literal can not satisify requirement for enumerator" << endl;
            break;
    case etUndefined :
    default :
            cerr << "Error: " << nodeId << " evaluation type undefined" << endl;
  }

  if (ptr != 0)
    delete ptr;
  return *this;
}


