//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/utils/nodes/src/outputidl.cpp
//
// Modifications:
// $Log: outputidl.cpp,v $
// Revision 1.1.1.1  2000/02/09 17:07:14  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.6  1998/05/11 10:53:48  stephen
// Change to node reference semantics propagated.
//
// Revision 1.5  1997/11/17 11:10:15  james
// Enhanced formating
//
// Revision 1.4  1997/10/14 10:24:13  james
// Extended types removed
//
// Revision 1.3  1997/10/08 09:55:41  james
// tidy up
//
// Revision 1.2  1997/10/07 10:21:49  james
// FindDefinition function modified
//
// Revision 1.1  1997/10/06 11:36:02  james
// Initial Sources
//

#include <stdlib.h>
 
#include "grammar.hh"
#include "literals.hh"
#include "constexpr.hh"
#include "outputidl.hh"

static int NumTabs = 0;

ostream &idltabs(ostream &output)
{
  output << endl;

  for (int i = NumTabs; i >0; --i)
    output << "  ";

  return output;
}

ostream &operator<<(ostream &output, Node &node)
{
  switch (node.Id())
  {
    case IdNotInitialised :
                                  break;
    case IdSpecification :        output <<  (Specification&)node;
                                  break;
    case IdDefinition :           output <<  (Definition&)node;
                                  break;
    case IdModule :               output <<  (Module&)node;
                                  break;
    case IdDefinitions :          output <<  (Definitions&)node;
                                  break;
    case IdInterface :            output <<  (Interface&)node;
                                  break;
    case IdInterfaceDcl :         output <<  (InterfaceDcl&)node;
                                  break;
    case IdForwardDcl :           output <<  (ForwardDcl&)node;
                                  break;
    case IdInterfaceBody :        output <<  (InterfaceBody&)node;
                                  break;
    case IdExport :               output <<  (Export&)node;
                                  break;
    case IdInheritanceSpec :      output <<  (InheritanceSpec&)node;
                                  break;
    case IdScopedName :           output <<  (_ScopedName&)node;
                                  break;
    case IdScopedNameA :          output <<  (ScopedNameA&)node;
                                  break;
    case IdScopedNameB :          output <<  (ScopedNameB&)node;
                                  break;
    case IdConstDcl :             output <<  (ConstDcl&)node;
                                  break;
    case IdConstType :            output <<  (ConstType&)node;
                                  break;
    case IdConstExp :             output <<  (ConstExp&)node;
                                  break;
    case IdOrExpr :               output <<  (OrExpr&)node;
                                  break;
    case IdOrOper :               output <<  (OrOper&)node;
                                  break;
    case IdXorExpr :              output <<  (XorExpr&)node;
                                  break;
    case IdXorOper :              output <<  (XorOper&)node;
                                  break;
    case IdAndExpr :              output <<  (AndExpr&)node;
                                  break;
    case IdAndOper :              output <<  (AndOper&)node;
                                  break;
    case IdShiftExpr :            output <<  (ShiftExpr&)node;
                                  break;
    case IdShiftRightOper :       output <<  (ShiftRightOper&)node;
                                  break;
    case IdShiftLeftOper :        output <<  (ShiftLeftOper&)node;
                                  break;
    case IdAddExpr :              output <<  (AddExpr&)node;
                                  break;
    case IdAddOper :              output <<  (AddOper&)node;
                                  break;
    case IdSubtractOper :         output <<  (SubtractOper&)node;
                                  break;
    case IdMultExpr :             output <<  (MultExpr&)node;
                                  break;
    case IdMultOper :             output <<  (MultOper&)node;
                                  break;
    case IdDivOper :              output <<  (DivOper&)node;
                                  break;
    case IdRemainderOper :        output <<  (RemainderOper&)node;
                                  break;
    case IdUnaryExpr :            output <<  (UnaryExpr&)node;
                                  break;
    case IdUnaryMinusOper :       output <<  (UnaryMinusOper&)node;
                                  break;
    case IdUnaryPlusOper :        output <<  (UnaryPlusOper&)node;
                                  break;
    case IdUnaryNotOper :         output <<  (UnaryNotOper&)node;
                                  break;
    case IdPrimaryExpr :          output <<  (PrimaryExpr&)node;
                                  break;
    case IdLiteral :              output <<  (Literal&)node;
                                  break;
    case IdBooleanLiteral :       output <<  (BooleanLiteral&)node;
                                  break;
    case IdPositiveIntConst :     output <<  (PositiveIntConst&)node;
                                  break;
    case IdTypeDcl :              output <<  (TypeDcl&)node;
                                  break;
    case IdTypeDeclarator :       output <<  (TypeDeclarator&)node;
                                  break;
    case IdTypeSpec :             output <<  (TypeSpec&)node;
                                  break;
    case IdSimpleTypeSpec :       output <<  (SimpleTypeSpec&)node;
                                  break;
    case IdBaseTypeSpec :         output <<  (BaseTypeSpec&)node;
                                  break;
    case IdObjectType :           output <<  (ObjectType&)node;
                                  break;
    case IdTemplateTypeSpec :     output <<  (TemplateTypeSpec&)node;
                                  break;
    case IdConstrTypeSpec :       output <<  (ConstrTypeSpec&)node;
                                  break;
    case IdDeclarators :          output <<  (Declarators&)node;
                                  break;
    case IdDeclarator :           output <<  (Declarator&)node;
                                  break;
    case IdSimpleDeclarator :     output <<  (SimpleDeclarator&)node;
                                  break;
    case IdComplexDeclarator :    output <<  (ComplexDeclarator&)node;
                                  break;
    case IdFloatingPtType :       output <<  (FloatingPtType&)node;
                                  break;
    case IdFloatType :            output <<  (FloatType&)node;
                                  break;
    case IdDoubleType :           output <<  (DoubleType&)node;
                                  break;
    case IdIntegerType :          output <<  (IntegerType&)node;
                                  break;
    case IdSignedInt :            output <<  (SignedInt&)node;
                                  break;
    case IdSignedLongInt :        output <<  (SignedLongInt&)node;
                                  break;
    case IdSignedShortInt :       output <<  (SignedShortInt&)node;
                                  break;
    case IdUnsignedInt :          output <<  (UnsignedInt&)node;
                                  break;
    case IdUnsignedLongInt :      output <<  (UnsignedLongInt&)node;
                                  break;
    case IdUnsignedShortInt :     output <<  (UnsignedShortInt&)node;
                                  break;
    case IdCharType :             output <<  (CharType&)node;
                                  break;
    case IdBooleanType :          output <<  (BooleanType&)node;
                                  break;
    case IdOctetType :            output <<  (OctetType&)node;
                                  break;
    case IdAnyType :              output <<  (AnyType&)node;
                                  break;
    case IdStructType :           output <<  (StructType&)node;
                                  break;
    case IdMemberList :           output <<  (MemberList&)node;
                                  break;
    case IdMember :               output <<  (Member&)node;
                                  break;
    case IdUnionType :            output <<  (UnionType&)node;
                                  break;
    case IdUnionSwitch :          output <<  (UnionSwitch&)node;
                                  break;
    case IdSwitchTypeSpec :       output <<  (SwitchTypeSpec &)node;
                                  break;
    case IdSwitchBody :           output <<  (SwitchBody&)node;
                                  break;
    case IdCase :                 output <<  (Case&)node;
                                  break;
    case IdCaseLabel :            output <<  (CaseLabel&)node;
                                  break;
    case IdDefaultLabel :         output <<  (DefaultLabel&)node;
                                  break;
    case IdCaseLabelList :        output <<  (CaseLabelList&)node;
                                  break;
    case IdElementSpec :          output <<  (ElementSpec&)node;
                                  break;
    case IdEnumType :             output <<  (EnumType&)node;
                                  break;
    case IdEnumerator :           output <<  (Enumerator&)node;
                                  break;
    case IdEnumeratorList :       output <<  (EnumeratorList&)node;
                                  break;
    case IdSequenceType :         output <<  (SequenceType&)node;
                                  break;
    case IdStringType :           output <<  (StringType&)node;
                                  break;
    case IdArrayDeclarator :      output <<  (ArrayDeclarator&)node;
                                  break;
    case IdFixedArraySizeList :   output <<  (FixedArraySizeList&)node;
                                  break;
    case IdFixedArraySize :       output <<  (FixedArraySize&)node;
                                  break;
    case IdAttrDcl :              output <<  (AttrDcl&)node;
                                  break;
    case IdReadonly :             output <<  (Readonly&)node;
                                  break;
    case IdSimpleDeclaratorList : output <<  (SimpleDeclaratorList&)node;
                                  break;
    case IdExceptDcl :            output <<  (ExceptDcl&)node;
                                  break;
    case IdOpDcl :                output <<  (OpDcl&)node;
                                  break;
    case IdOpTypeSpec :           output <<  (OpTypeSpec&)node;
                                  break;
    case IdOpTypeSpecA :          output <<  (OpTypeSpecA&)node;
                                  break;
    case IdOpTypeSpecB :          output <<  (OpTypeSpecB&)node;
                                  break;
    case IdOpAttribute :          output <<  (OpAttribute&)node;
                                  break;
    case IdParameterDcls :        output <<  (ParameterDcls&)node;
                                  break;
    case IdParameterDclsA :       output <<  (ParameterDclsA&)node;
                                  break;
    case IdParamDcl :             output <<  (ParamDcl&)node;
                                  break;
    case IdParamAttribute :       output <<  (ParamAttribute&)node;
                                  break;
    case IdRaisesExpr :           output <<  (RaisesExpr&)node;
                                  break;
    case IdContextExpr :          output <<  (ContextExpr&)node;
                                  break;
    case IdParamTypeSpec :        output <<  (ParamTypeSpec&)node;
                                  break;
    case IdIdentifier :           output <<  (_Identifier&)node;
                                  break;
    case IdIntegerLiteral :       output <<  (IntegerLiteral&)node;
                                  break;
    case IdCharacterLiteral :     output <<  (CharacterLiteral&)node;
                                  break;
    case IdStringLiteral :        output <<  (StringLiteral&)node;
                                  break;
    case IdFloatingPtLiteral :    output <<  (FloatingPtLiteral&)node;
                                  break;
    default :                     
        cerr  << "Error: << operator failed to recognise " << node.Id() << endl;
        exit(EXIT_FAILURE);
  }
  return output;
}

ostream &operator<<(ostream &output, Node *node)
{
  if (node != 0)
    output << *node;
  return output;
}


ostream &operator<<(ostream &output, ChoiceNode &node)
{
  output << node[0];
  return output;
}

ostream &operator<<(ostream &output, SequenceNode &node)
{
  for(int i = 0; i < node.Length(); ++i)
    output << node[i];
  return output;
}

ostream &operator<<(ostream &output, ListNode &node)
{
  for(int i = 0; i < node.Length(); ++i)
  {
    output << node[i];
    if (node.Separator() != 0 && node[i+1] != IdNotInitialised)
      output << node.Separator();
  }
  return output;
}

ostream &operator<<(ostream &output,  Specification &node)
{
  output << (ListNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  Definition &node)
{
  output << idltabs;
  output << (ChoiceNode &)node;
  output << ";\n";
  return output;
}

ostream &operator<<(ostream &output,  Module &node)
{
  output << idltabs;
  output << "module ";
  output << node[IdIdentifier];
  output << "{" ;
  output << node[IdDefinitions];
  output << idltabs;
  output << "}";
  return output;
}

ostream &operator<<(ostream &output,  Definitions &node)
{
  ++NumTabs;
  output << (ListNode&)node;
  --NumTabs;
  return output;
}

ostream &operator<<(ostream &output,  Interface &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  InterfaceDcl &node)
{
  output << "interface ";
  output << node[IdIdentifier];
  output << node[IdInheritanceSpec];
  output << "{";
  output << node[IdInterfaceBody];
  output << idltabs;
  output << "}";

  return output;
}

ostream &operator<<(ostream &output,  ForwardDcl &node)
{
  output << "interface ";
  output << node[IdIdentifier];
  return output;
}

ostream &operator<<(ostream &output,  InterfaceBody &node)
{
  ++NumTabs;
  output << (ListNode&)node;
  --NumTabs;
  return output;
}

ostream &operator<<(ostream &output,  Export &node)
{
  output << "\n";
  output << idltabs;
  output << (ChoiceNode&)node;
  output << ";";
  return output;
}

ostream &operator<<(ostream &output,  InheritanceSpec &node)
{
  if (node.Length() != 0)
  {
    output << " : ";
    output << (ListNode&)node;
  }
  return output;
}

ostream &operator<<(ostream &output,  _ScopedName &node)
{
  output << (SequenceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  ScopedNameA &node)
{
  output << "::";
  return output;
}

ostream &operator<<(ostream &output,  ScopedNameB &node)
{
  output << (ListNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  ConstDcl &node)
{
  output << idltabs;
  output << "const ";
  output << node[IdConstType];
  output << " ";
  output << node[IdIdentifier];
  output << "=";
  output << node[IdConstExp];

  return output;
}

ostream &operator<<(ostream &output,  ConstType &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  ConstExp &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  OrExpr &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  OrOper &node)
{
  output << node[IdOrExpr];
  output << "|";
  output << node[IdXorExpr];
  return output;
}

ostream &operator<<(ostream &output,  XorExpr &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  XorOper &node)
{
  output << node[IdXorExpr];
  output << "^";
  output << node[IdAndExpr];
  return output;
}

ostream &operator<<(ostream &output,  AndExpr &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  AndOper &node)
{
  output << node[IdAndExpr];
  output << "&";
  output << node[IdShiftExpr];
  return output;
}

ostream &operator<<(ostream &output,  ShiftExpr &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  ShiftRightOper &node)
{
  output << node[IdShiftExpr];
  output << ">>";
  output << node[IdAddExpr];
  return output;
}

ostream &operator<<(ostream &output,  ShiftLeftOper &node)
{
  output << node[IdShiftExpr];
  output << "<<";
  output << node[IdAddExpr];
  return output;
}

ostream &operator<<(ostream &output,  AddExpr &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  AddOper &node)
{
  output << node[IdAddExpr];
  output << "+";
  output << node[IdMultExpr];
  return output;
}

ostream &operator<<(ostream &output,  SubtractOper &node)
{
  output << node[IdAddExpr];
  output << "-";
  output << node[IdMultExpr];
  return output;
}

ostream &operator<<(ostream &output,  MultExpr &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  MultOper &node)
{
  output << node[IdMultExpr];
  output << "*";
  output << node[IdUnaryExpr];
  return output;
}

ostream &operator<<(ostream &output,  DivOper &node)
{
  output << node[IdMultExpr];
  output << "/";
  output << node[IdUnaryExpr];
  return output;
}

ostream &operator<<(ostream &output,  RemainderOper &node)
{
  output << node[IdMultExpr];
  output << "%";
  output << node[IdUnaryExpr];
  return output;
}

ostream &operator<<(ostream &output,  UnaryExpr &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  UnaryMinusOper &node)
{
  output << "-";
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  UnaryPlusOper &node)
{
  output << "+";
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  UnaryNotOper &node)
{
  output << "~";
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  PrimaryExpr &node)
{
  if (node.Choice() == 1)
    output << "(";
  output << (ChoiceNode&)node;
  if (node.Choice() == 1)
    output << ")";
  return output;
}

ostream &operator<<(ostream &output,  Literal &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  BooleanLiteral &node)
{
  if (node.Choice() == 0)
    output << "FALSE";
  else
    output << "TRUE";
  return output;
}

ostream &operator<<(ostream &output,  PositiveIntConst &node)
{
  output << (SequenceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  TypeDcl &node)
{
  if (node.Choice() == 0)
    output << "typedef ";
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  TypeDeclarator &node)
{
  output << node[IdTypeSpec];
  output << " ";
  output << node[IdDeclarators];
  return output;
}

ostream &operator<<(ostream &output,  TypeSpec &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  SimpleTypeSpec &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  BaseTypeSpec &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  ObjectType &node)
{
  output << "Object ";
  return output;
}

ostream &operator<<(ostream &output,  TemplateTypeSpec &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  ConstrTypeSpec &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  Declarators &node)
{
  output << (ListNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  Declarator &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  SimpleDeclarator &node)
{
  output << (SequenceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  ComplexDeclarator &node)
{
  output << (SequenceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  FloatingPtType &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  FloatType &node)
{
  output << "float";
  return output;
}

ostream &operator<<(ostream &output,  DoubleType &node)
{
  output << "double";
  return output;
}

ostream &operator<<(ostream &output,  IntegerType &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  SignedInt &node)
{  
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  SignedLongInt &node)
{
  output << "long";
  return output;
}

ostream &operator<<(ostream &output,  SignedShortInt &node)
{
  output << "short";
  return output;
}

ostream &operator<<(ostream &output,  UnsignedInt &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  UnsignedLongInt &node)
{
  output << "unsigned long";
  return output;
}

ostream &operator<<(ostream &output,  UnsignedShortInt &node)
{
  output << "unsigned short";
  return output;
}

ostream &operator<<(ostream &output,  CharType &node)
{
  output << "char";
  return output;
}

ostream &operator<<(ostream &output,  BooleanType &node)
{
  output << "boolean";
  return output;
}

ostream &operator<<(ostream &output,  OctetType &node)
{
  output << "octet";
  return output;
}

ostream &operator<<(ostream &output,  AnyType &node)
{
  output << "any";
  return output;
}

ostream &operator<<(ostream &output,  StructType &node)
{
  output << "struct ";
  output << node[IdIdentifier];
  output << "{";
  output << node[IdMemberList];
  output << idltabs;
  output << "}";
  return output;
}

ostream &operator<<(ostream &output,  MemberList &node)
{
  ++NumTabs;
  output << (ListNode&)node;
  --NumTabs;
  return output;
}

ostream &operator<<(ostream &output,  Member &node)
{
  output << idltabs;
  output << node[IdTypeSpec];
  output << " ";
  output << node[IdDeclarators];
  output << ";";
  return output;
}

ostream &operator<<(ostream &output,  UnionType &node)
{
  output << "union ";
  output << node[IdIdentifier];
  output << node[IdUnionSwitch];
  return output;
}

ostream &operator<<(ostream &output,  UnionSwitch &node)
{
  output << " switch (";
  output << node[IdSwitchTypeSpec];
  if (node[IdSwitchTypeSpec].Choice() == 3)
    output << idltabs;
  output << ") ";
  output << "{";
  output << node[IdSwitchBody];
  output << idltabs;
  output << "}";
  return output;
}

ostream &operator<<(ostream &output,  SwitchTypeSpec &node)
{
  ++NumTabs;
  if (node.Choice() == 3)
    output << idltabs;
  output << (ChoiceNode&)node;
  --NumTabs;
  return output;
}

ostream &operator<<(ostream &output,  SwitchBody &node)
{
  ++NumTabs;
  output << (ListNode&)node;
  --NumTabs;
  return output;
}

ostream &operator<<(ostream &output,  Case &node)
{
  output << (SequenceNode&)node;
  output << ";";
  return output;
}

ostream &operator<<(ostream &output,  CaseLabel &node)
{
  output << idltabs;
  if (node.Choice() == 0)
    output << "case ";
  output << (ChoiceNode&)node;
  output << ":";
  
  return output;
}

ostream &operator<<(ostream &output,  DefaultLabel &node)
{
  output << "default ";
  return output;
}

ostream &operator<<(ostream &output,  CaseLabelList &node)
{
  output << (ListNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  ElementSpec &node)
{
  ++NumTabs;
  output << idltabs;
  output << node[IdTypeSpec];
  output << " ";
  output << node[IdDeclarator];
  --NumTabs;
  return output;
}

ostream &operator<<(ostream &output,  EnumType &node)
{
  output << "enum ";
  output << node[IdIdentifier];
  output << "{";
  output << node[IdEnumeratorList];
  output << idltabs;
  output << "}";
  return output;
}

ostream &operator<<(ostream &output,  Enumerator &node)
{
  output << idltabs;
  output << node[IdIdentifier];
  return output;
}

ostream &operator<<(ostream &output,  EnumeratorList &node)
{
  ++NumTabs;
  output << (ListNode&)node;
  --NumTabs;
  return output;
}

ostream &operator<<(ostream &output,  SequenceType &node)
{
  output << "sequence<";
  output << node[IdSimpleTypeSpec];
  if (node[IdPositiveIntConst].Id() != IdNotInitialised)
  {
    output << ",";
    output << node[IdPositiveIntConst];
  }
  output << "> ";
  return output;
}

ostream &operator<<(ostream &output,  StringType &node)
{
  output << "string";
  if (node[IdPositiveIntConst].Id() != IdNotInitialised)
  {
    output << "<";
    output << node[IdPositiveIntConst];
    output << "> ";
  }
  return output;
}

ostream &operator<<(ostream &output,  ArrayDeclarator &node)
{
  output << (SequenceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  FixedArraySizeList &node)
{
  output << (ListNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  FixedArraySize &node)
{
  output <<  "[";
  output << node[IdPositiveIntConst];
  output << "]";
  return output;
}

ostream &operator<<(ostream &output,  AttrDcl &node)
{
  output << node[IdReadonly];
  output << "attribute ";
  output << node[IdParamTypeSpec];
  output << " ";
  output << node[IdSimpleDeclaratorList];
  return output;
}

ostream &operator<<(ostream &output,  Readonly &node)
{
  output << "readonly ";
  return output;
}

ostream &operator<<(ostream &output,  SimpleDeclaratorList &node)
{
  output << (ListNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  ExceptDcl &node)
{
  output << "exception ";
  output << node[IdIdentifier];
  output << "{";
  output << node[IdMemberList];
  output << idltabs;
  output << "}";
  return output;
}

ostream &operator<<(ostream &output,  OpDcl &node)
{
  output << node[IdOpTypeSpec];
  output << " ";
  output << node[IdIdentifier];
  output << node[IdParameterDcls];
  output << node[IdRaisesExpr];
  output << node[IdContextExpr];
  return output;
}

ostream &operator<<(ostream &output,  OpTypeSpec &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  OpTypeSpecA &node)
{
  output << (SequenceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  OpTypeSpecB &node)
{
  output << "void ";
  return output;
}

ostream &operator<<(ostream &output,  OpAttribute &node)
{
  output << "oneway ";
  return output;
}

ostream &operator<<(ostream &output,  ParameterDcls &node)
{
  output << "(";
  if (node[IdParameterDclsA].Id() != IdNotInitialised)
  {
    output << node[IdParameterDclsA];
    output << idltabs;
  }
  output << ")";
  return output;
}

ostream &operator<<(ostream &output,  ParameterDclsA &node)
{
  output << (ListNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  ParamDcl &node)
{
  ++NumTabs;
  output << idltabs;
  output << node[IdParamAttribute];
  output << node[IdParamTypeSpec];
  output << " ";
  output << node[IdSimpleDeclarator];
  --NumTabs;
  return output;
}

ostream &operator<<(ostream &output,  ParamAttribute &node)
{
  switch(node.Choice())
  {
    case 0 :  output << "in    ";
              break;
    case 1 :  output << "out   ";
              break;
    case 2 :  output << "inout ";
              break;
  }
  return output;
}

ostream &operator<<(ostream &output,  RaisesExpr &node)
{
  if (node.Length() != 0)
  {
    output << " raises (";
    output << (ListNode&)node;
    output << ")";
  }
  return output;
}

ostream &operator<<(ostream &output,  ContextExpr &node)
{
  output << " context (";
  output << (ListNode&)node;
  output << ")";
  return output;
}

ostream &operator<<(ostream &output,  ParamTypeSpec &node)
{
  output << (ChoiceNode&)node;
  return output;
}

ostream &operator<<(ostream &output,  _Identifier &node)
{
  output << (char *)node;
  return output;
}

ostream &operator<<(ostream &output,  IntegerLiteral &node)
{
  output << node.exprValue();
  return output;
}

ostream &operator<<(ostream &output,  CharacterLiteral &node)
{
  output << node.exprValue();
  return output;
}

ostream &operator<<(ostream &output,  StringLiteral &node)
{
  output << node.exprValue();
  return output;
}

ostream &operator<<(ostream &output,  FloatingPtLiteral &node)
{
  output << node.exprValue();
  return output;
}


