//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/utils/nodes/src/utilities.cpp
//
// Modifications:
// $Log: utilities.cpp,v $
// Revision 1.2  2000/02/11 16:06:24  mli
// Decl. of index p extracted out of the 'for' loop.
//
// Revision 1.11  1998/05/11 10:55:01  stephen
// Change to node reference semantics propagated.
//
// Revision 1.10  1997/12/05 12:31:02  james
// Anon bounded string count as types
//
// Revision 1.9  1997/12/01 10:55:09  james
// CountMembers added
//
// Revision 1.8  1997/11/24 12:18:32  james
// TypeCode support added
//
// Revision 1.7  1997/11/17 11:08:52  james
// CountTypes function added
//
// Revision 1.6  1997/10/20 17:29:41  stephen
// Added case label counting functions.
//
// Revision 1.5  1997/10/16 14:23:44  james
// Reduced memory usage
//
// Revision 1.4  1997/10/14 10:24:19  james
// Extended types removed
//
// Revision 1.3  1997/10/08 09:55:42  james
// tidy up
//
// Revision 1.2  1997/10/07 10:21:50  james
// FindDefinition function modified
//
// Revision 1.1  1997/10/06 11:36:07  james
// Initial Sources

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
 
#include "grammar.hh"
#include "literals.hh"
#include "utilities.hh"

#include "constexpr.hh"
#include "outputidl.hh"

static ObjectType  VSORB_TypeCode;

EvalType GetEvalType(Node &node)
{
  EvalType evalType;

  switch(FindTypeSpecifier(node))
  {
     case tsShort:
     case tsLong:
          evalType = etLong;
          break;
     case tsUShort:
     case tsULong:
          evalType = etULong;
          break;
     case tsFloat:
     case tsDouble:
          evalType = etDouble;
          break;
     case tsBoolean:
          evalType = etBoolean;
          break;
     case tsChar:
          evalType = etChar;
          break;
     case tsString:
          evalType = etString;
          break;
     case tsEnum:
          evalType = etEnum;
          break;
     case tsUndefined: 
    default:
          evalType = etUndefined;
          break;
  }
  //cout << node->Id() << " type is " << evalType << endl;
  return evalType;
}


TypeSpecifier FindTypeSpecifier(Node &ptr)
{
  TypeSpecifier ret;

  switch(ptr.Id())
  {
    case IdNotInitialised:
          return tsUndefined;
    case IdConstDcl :
          return FindTypeSpecifier(ptr[0]);
    case IdPositiveIntConst :
          return tsULong;
    case IdUnionType :
          return FindTypeSpecifier(ptr[IdUnionSwitch]);
    case IdUnionSwitch :
          return FindTypeSpecifier(ptr[IdSwitchTypeSpec]);
    case IdCaseLabel :
          {
            Node *p;
            
            for (p = ptr.Parent();
                  p != 0 && p->Id() != IdUnionSwitch;
                  p = p->Parent())
              ;
            if (p)
              return FindTypeSpecifier(*p);
          }
    case IdScopedName :
      {
        Node *p;
        if ((p = FindDefinition(ptr,(_ScopedName&)ptr)) != 0)
          return FindTypeSpecifier(*p);
      }

    case IdConstExp :
            if (ptr.Parent())
              return FindTypeSpecifier(*ptr.Parent());
        
    case IdOrExpr :
    case IdOrOper :
    case IdXorExpr :
    case IdXorOper :
    case IdAndExpr :
    case IdAndOper :
    case IdShiftExpr :
    case IdShiftRightOper :
    case IdShiftLeftOper :
    case IdAddExpr :
    case IdAddOper :
    case IdSubtractOper :
    case IdMultExpr :
    case IdMultOper :
    case IdDivOper :
    case IdRemainderOper :
    case IdUnaryExpr :
    case IdUnaryMinusOper :
    case IdUnaryPlusOper :
    case IdUnaryNotOper :
    case IdPrimaryExpr :
    case IdLiteral :
    case IdBooleanLiteral :
    case IdIntegerLiteral :
    case IdCharacterLiteral :
    case IdStringLiteral :
    case IdFloatingPtLiteral :
          {
            Node *p;
            

            for (p = ptr.Parent();
                  p != 0 && p->Id() != IdConstExp;
                  p = p->Parent())
              ;
            if (p)
              return FindTypeSpecifier(*p);
          }
          break;

    case IdFloatType :
          return tsFloat;
    case IdDoubleType :
          return tsDouble;
    case IdSignedLongInt :
          return tsLong;
    case IdSignedShortInt :
          return tsShort;
    case IdUnsignedLongInt :
          return tsULong;
    case IdUnsignedShortInt :
          return tsUShort;
    case IdCharType :
          return tsChar;
    case IdBooleanType :
          return tsBoolean;
    case IdOctetType :
          return tsOctet;
    case IdAnyType :
          return tsAny;
    case IdEnumType :
        return tsEnum;
    case IdStringType :
        return tsString;

    default:
      for (int i = 0; i < ptr.Length(); ++i)
      {
        if ((ret = FindTypeSpecifier(ptr[i])) != tsUndefined)
          return ret;
      }
  }
  return tsUndefined;
}
 

Node *FindNode(const _Identifier &name, Node &ptr)
{
  switch(ptr.Id())
  {
    case IdNotInitialised:
      return 0;
      // Check if these nodes are the required type, but
      // do not continue the search into these nodes.
    case IdModule :
    case IdInterfaceDcl :
    case IdStructType :
    case IdUnionType :
    case IdExceptDcl :
    case IdOpDcl :

    case IdConstDcl :
    case IdSimpleDeclarator :
    case IdArrayDeclarator :
    case IdEnumerator :
      if (ptr[IdIdentifier] != IdNotInitialised
       && ((_Identifier&)ptr[IdIdentifier]) == name)
        return &ptr;
      break;

      // Check if EnumType is the required type, but do not break, as the
      // enumerators must be searched also
    case IdEnumType :
      if (ptr[IdIdentifier] != IdNotInitialised
       && ((_Identifier&)ptr[IdIdentifier]) == name)
        return &ptr;

      // Continue the search into the child nodes
    default:
      Node *ret;
      for (int i = 0; i < ptr.Length(); ++i)
      {
        if ((ret = FindNode(name,ptr[i])) != 0)
          return ret;
      }
  }
  return 0;
}


// Nodes that introduce a Scoped Level : 
//     module, interface, structure, union, operation, exception

// This function will return a pointer to a Node that will be one of the 
// following types:
//    Module, InterfaceDcl, ConstDcl, Simpledeclarator, StructType, UnionType, 
//    EnumType, Enumerator, ArrayDeclarator, ExceptDcl, OpDcl
// ForwardDcls will be ignored for the purposes of this function

Node *FindDefinition(
    Node &start,
    const _ScopedName  name,
    int flag)
{
  Node *ptr;
  int  nameIndex=0;

  if (start == IdNotInitialised)
  {
    cerr << "Error: FindDefintion was passed a uninitialised start value" << endl;
    exit(EXIT_FAILURE);
  }

  if (name[IdScopedNameB] == IdNotInitialised)
  {
    cerr << "Error: FindDefinition was passed a ScopedName with no Identifiers" << endl;
    cerr << "       Start node is " << start.Id();

    Node *p;
    for (p = &start; p != 0 && p->Parent() != 0; p = p->Parent());
    cerr << p;
    exit(EXIT_FAILURE);
  }

  // Check for CORBA::TypeCode
  if (((SequenceNode&)name)[1].Length() == 2)
  {
    if (!strcmp("CORBA",(_Identifier&)((SequenceNode&)name)[IdScopedNameB][0])
    && !strcmp("TypeCode",(_Identifier&)((SequenceNode&)name)[IdScopedNameB][1]))
      return &VSORB_TypeCode;
  }

  // If scopedname containes a leading scoped delimiter
  // then start search from the file scope
  if (name[IdScopedNameA] != IdNotInitialised)
  {
    ptr = &start;
    while (ptr->Parent() != 0)
      ptr = ptr->Parent();
  }
  else
  {
    // Otherwise, climb the tree until the scope contining the first
    // identifier listed in the scoped name is found

    ptr = &start;

    while (ptr != 0)
    {
      while (ptr != 0
          && ptr->Id() != IdSpecification
          && ptr->Id() != IdDefinitions
          && ptr->Id() != IdMemberList
          && ptr->Id() != IdUnionSwitch
          && ptr->Id() != IdInterfaceBody)
        ptr = ptr->Parent();

      if (ptr == 0)
        break;

      if (FindNode(
          (_Identifier&)name[IdScopedNameB][0],
          *ptr) != 0)
        break;

      ptr = ptr->Parent();
    }
  }

  
  // ptr now points to the node where the search should begin

  for (nameIndex = 0; ptr != 0 && nameIndex < name[IdScopedNameB].Length(); ++nameIndex)
  {
    ptr = FindNode((_Identifier&)name[IdScopedNameB][nameIndex],*ptr);

    if (name[IdScopedNameB][nameIndex+1] != IdNotInitialised)
    {
      switch(ptr->Id())
      {
        case IdModule :
          ptr = &(*ptr)[IdDefinitions];
          break;
        case IdInterfaceDcl :
          ptr = &(*ptr)[IdInterfaceBody];
          break;
        case IdStructType :
          ptr = &(*ptr)[IdMemberList];
          break;
        case IdUnionType :
          ptr = &(*ptr)[IdUnionSwitch];
          break;
        case IdExceptDcl :
          ptr = &(*ptr)[IdMemberList];
          break;
        case IdOpDcl :
          ptr = &(*ptr)[IdParameterDcls];
          break;
        default: 
          break;
      }
    }
  }


  switch (flag) {
  case 0:
    break;
  case 1:
    // If a SimpleDeclarator was found,
    // then return a pointer to the associated TypeSpec
    if (ptr != 0 && ptr->Id() == IdSimpleDeclarator)
    {
      for (ptr=ptr->Parent();
        ptr != 0 && ptr->Id() != IdTypeDeclarator;
        ptr=ptr->Parent())
        ;
       ptr = &(*ptr)[0];
    }
    break;
  case 2:
    // return a pointer to a node of type Declarator
    // Used by the IIOP proxy to ensure typedefs are handled properly
    if (ptr != 0)
    {
      if (ptr->Id() == IdSimpleDeclarator)
        ptr = ptr->Parent();
      else if (ptr->Id() == IdArrayDeclarator)
        ptr = ptr->Parent()->Parent();
    }
  }

  return ptr;
}

int IsATypeCode(
    Node *node)
{
  return (node == &VSORB_TypeCode);
}

int CountCaseLabels(Node &ptr,int countDefault)
{
  int count = 0;

  if (ptr == IdNotInitialised)
    return 0;

  if (ptr == IdElementSpec) // Don't process ElementSpec for fear of counting the labels on nested unions
    return 0;

  if (ptr == IdCaseLabel
   && (ptr.Choice() == 0 || countDefault != 0))
    return 1;

  for (int i = 0; i < ptr.Length(); ++i)
    count += CountCaseLabels(ptr[i],countDefault);

  return count;
}

int CountPreceedingCaseLabels(Node &start)
{
  int count = 0;

  // Find the SwitchBody node
  Node *switchBody = &start;
  for (;
       switchBody != 0 && switchBody->Id() != IdSwitchBody;
       switchBody = switchBody->Parent())
    ;

  if (switchBody == 0)
    return 0;

  // for each Case in SwitchBody
  for (int a = 0; a < switchBody->Length(); ++a)
  {
    Node &caseLabelList = (*switchBody)[a][IdCaseLabelList];

    // for each CaseLabel in the CaseLabelList
    for (int b = 0; b < caseLabelList.Length(); ++b)
    {
      if (&caseLabelList[b] == &start)
        return count;

      if (caseLabelList[b].Choice() == 0)
        ++count;
    }
  }

  return count;
}

int CountTypes(Node &ptr)
{
  int  count = 0;
 
  switch (ptr.Id())
  {
    case IdNotInitialised:
      return count;
    case IdTypeDeclarator :
                  count += ptr[IdDeclarators].Length();
                  break;
    case IdStructType :
    case IdUnionType :
    case IdEnumType :
                  ++count;
                  break;
    case IdStringType :
      if (ptr.Parent() != 0 && ptr.Parent()->Id() == IdParamTypeSpec && ptr[0] != IdNotInitialised)
	  ++count;
      break;
  }
  for (int i = 0; i < ptr.Length(); ++i)
    count+=CountTypes(ptr[i]);

  return count;
}

int CountMembers(Node &ptr)
{
  int count = 0;
 
  switch (ptr.Id()) {
  case IdNotInitialised:
    return 0;
  case IdDeclarator:
    return 1;
  case IdMember:
    return CountMembers(ptr[IdDeclarators]);
  }

  for (int i = 0; i < ptr.Length(); ++i)
    count += CountMembers(ptr[i]);
  return count;
}


