/*
   Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
  
   All rights reserved. No part of this source code may be reproduced,
   stored in a retrieval system, or transmitted, in any form or by any
   means, electronic, mechanical, photocopying, recording or otherwise,
   except as stated in the end-user licence agreement, without the prior
   permission of the copyright owners.
  
   Developed for X/Open by ApTest Ireland Limited.
  
   Project: VSORB
  
   File: src/utils/reports/vsorbpr.c
  
   Purpose: Report print formatter used by vsvsorbpr
  
   Modifications:
   $Log: vsorbpr.c,v $
   Revision 1.1.1.1  2000/02/09 17:07:15  mli
   Imported TOG's VSOrb 1.1.1

   Revision 1.2  1997/12/01 10:38:51  ian
   Fixed fprintfs.

   Revision 1.1  1997/10/31 12:39:14  ian
   First version of report generators.


*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>

#define OPTIONS ":s:o:l:"

char 	*linestore[66*2000];

int  VSORBgetopt (int argc, char * const argv [], char *defined_options_p);
extern int VSORBoptind, VSORBoptopt;
extern char *VSORBoptarg;

int main(int, char *[]);

int main(int argc, char *argv[])
{
	int     optlet, lines=54, nchar, mchar;
	char	*theorg="", *thesystem="";
	char	thedate[128];
	time_t	thetime;
	char	foot1[80];
	char	foot2[80];
	int	linecount=0;
	int	i, j, k;
	char	linebuf[1024];
	int	storecnt = 0;
	char	*lptr;
	int	pagecount=0, pagetotal=0;

	VSORBoptind= 0;

	while ((optlet = VSORBgetopt(argc, argv, OPTIONS))!= -1) {
		switch (optlet) {
		case 'o':
			theorg = VSORBoptarg;
			break;
		case 's':
			thesystem=VSORBoptarg;
			break;
		case 'l':
			lines=atoi(VSORBoptarg);
			break;
		case ':':
			fprintf(stderr, "vsorbpr: option -%c requires an operand\n", VSORBoptopt);
			exit(1);
		}
	}

#ifdef CDEBUG
printf("org = %s\n", theorg);
printf("system = %s\n", thesystem);
printf("lines = %d\n", lines);
#endif
	if (lines < 40) {
		fprintf(stderr, "vsorbpr: lines per page must be at least 40, is %d\n", lines);
		exit(1);
	}
	time(&thetime);
	strftime(thedate, sizeof(thedate), "%b %d, %Y", localtime(&thetime));
#ifdef CDEBUG
printf("date = %s\n", thedate);
#endif

	memset(foot1, ' ', sizeof(foot1));
	memset(foot2, ' ', sizeof(foot1));

	i = strlen(thedate)+1;
	j = strlen(thesystem)+strlen("System: ");
	if (j > (80 - i)) {
		fprintf(stderr, "vsorbpr: too many characters in system name\n");
		exit(1);
	}
	sprintf(foot1, "System: %s", thesystem);
	foot1[j] = ' ';
	sprintf(&foot1[80-i], "%s", thedate);

	k = strlen(theorg) + strlen("Organization: ");
	j = strlen("Page  of  ")+3+3+1;
	if (j > (80 - k)) {
		fprintf(stderr, "vsorbpr: too many characters in organization name\n");
		exit(1);
	}
	sprintf(foot2, "Organization: %s", theorg);
	foot2[k] = ' ';

	/*first count the number of pages while storing and breaking up lines*/
	while (fgets(linebuf, sizeof(linebuf), stdin) != 0) {
		nchar = strlen(linebuf);
		lptr = linebuf;
		while (nchar > 0) {
			mchar = (nchar < 80) ? nchar : 80;
			linestore[storecnt] = (char *)malloc(mchar +2);
			if (linestore[storecnt] == NULL) {
				fprintf(stderr, "vsorbpr: malloc failed");
				exit(1);
			}
			strncpy(linestore[storecnt], lptr, mchar);
			if (linestore[storecnt][mchar-1] != '\n') {
				linestore[storecnt][mchar] = '\n';
				linestore[storecnt][mchar+1] = 0;
			}
			else
				linestore[storecnt][mchar] = 0;
			nchar -= 80;
			lptr += 80;
			if (linebuf[0] == 0xc) {
				pagetotal++;
				linecount = 0;
			}
			else {
				linecount++;
				if (linecount == (lines -5)) {
					pagetotal++;
					linecount = 0;
				}
			}
			storecnt++;
			if (storecnt == (sizeof(linestore)/sizeof(char *))) {
				fprintf(stderr, "vsorbpr: sorry, cannot handle a report this large");
				exit(1);
			}
		}
	}
	pagetotal++;

	/*now spew em out*/
	linecount = 0;
	pagecount = 1;
	for (i = 0; i < storecnt; i++) {
		if (*linestore[i] == 0xc) {
			for (j = 0; j < (lines-linecount-5); j++)
				printf("\n");
			printf("\n\n%s\n", foot1);
			sprintf(linebuf, "Page %d of %d", pagecount, pagetotal);
			strcpy(&foot2[80-strlen(linebuf)-1], linebuf);
			printf("%s\n\f", foot2);
			pagecount++;
			linecount = 0;
			continue;
		}
		printf("%s", linestore[i]);
		linecount++;
		if (linecount == lines-5) {
			printf("\n\n%s\n", foot1);
			sprintf(linebuf, "Page %d of %d", pagecount, pagetotal);
			strcpy(&foot2[80-strlen(linebuf)-1], linebuf);
			printf("%s\n\f", foot2);
			pagecount++;
			linecount = 0;
		}
	}
	for (j = 0; j < (lines-linecount-5); j++)
		printf("\n");
	printf("\n\n%s\n", foot1);
	sprintf(linebuf, "Page %d of %d", pagecount, pagetotal);
	strcpy(&foot2[80-strlen(linebuf)-1], linebuf);
	printf("%s\n", foot2);
	pagecount++;
	linecount = 0;
	return 0;
}
