/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* interface DynAny                                */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

    DynamicAny::DynAny_ptr GDAny;    
    void get_dynany(CORBA::Any p_a)
    {   
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb(); 
	CORBA::Object_ptr obj = orb->resolve_initial_references("DynAnyFactory");
	DynamicAny::DynAnyFactory_ptr daf = DynamicAny::DynAnyFactory::_narrow(obj);
	GDAny = daf->create_dyn_any(p_a);
    }

    void provide_struct_dany(DynamicAny::DynAny_ptr da)
    {
	CORBA::Short val_short = 1;
	CORBA::Long val_long = 2;

	DynamicAny::DynStruct_ptr dyn_struct = DynamicAny::DynStruct::_narrow(da);
	dyn_struct->insert_short(val_short);
	dyn_struct->next();
	dyn_struct->insert_long(val_long);
    }

/*------------------------------------------------------*/
/* insert_octet */

VSBoolean VSinsert_octet(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }
   
    CORBA::Any A;
    CORBA::Octet O;

    try {	
	O = 1;
	(*da)->insert_octet(O);   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_octet */

VSBoolean VSget_octet(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_any:
	  {
	      CORBA::Any A;
	      A <<= (CORBA::Short) 0;
	      (*da)->insert_any(A);
	  }
	  break;
	  case CORBA::tk_octet:
	  {
	      CORBA::Octet O = 1;
	      (*da)->insert_octet(O);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::Octet rO;
    
    try {
	rO = (*da)->get_octet();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rO == 1)
	return TRUE;
    else
	return FALSE;
} 

/*------------------------------------------------------*/
/* insert_typecode */

VSBoolean VSinsert_typecode(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }   
    
    CORBA::TypeCode_var TC;
    try {	
	TC = CORBA::_tc_short;
	(*da)->insert_typecode(TC);   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_typecode */

VSBoolean VSget_typecode(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_TypeCode:
	  {
	      CORBA::TypeCode_ptr TC = CORBA::_tc_short;
	      (*da)->insert_typecode(TC);	     
	  }
	  break;	  
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;
	  default: 
	  {
	      CORBA::Any tmp;
	      tmp <<= (CORBA::Short) 0;

	      get_dynany(tmp);	       
	      (*da)->insert_dyn_any(GDAny);
	  }
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::TypeCode_var rTC;
    
    try {
	rTC = (*da)->get_typecode();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rTC->kind() == CORBA::tk_short)
	return TRUE;
    else
	return FALSE;
}

/*------------------------------------------------------*/
/* insert_dyn_any */

VSBoolean VSinsert_dyn_any(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }   
    
    DynamicAny::DynAny_var DA;
    try {
	CORBA::Any tmp;
	tmp <<= (CORBA::Short) 0;

	get_dynany(tmp);	       
	(*da)->insert_dyn_any(GDAny);
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_dyn_any */

VSBoolean VSget_dyn_any(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_TypeCode:
	  {
	      CORBA::TypeCode_ptr TC = CORBA::_tc_short;
	      (*da)->insert_typecode(TC);	     
	  }
	  break;	  
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;
	  default: 
	  {
	      CORBA::Any tmp;
	      tmp <<= (CORBA::Short) 0;

	      get_dynany(tmp);	       
	      (*da)->insert_dyn_any(GDAny);
	  }
	}
    }	
    catch (...) {
	return FALSE;
    }    

    DynamicAny::DynAny_var rDA;
    
    try {
	rDA = (*da)->get_dyn_any();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    return (rDA->equal(GDAny));
}
	

/*------------------------------------------------------*/
/* insert_boolean */

VSBoolean VSinsert_boolean(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::Boolean boo;
    CORBA::Long L;

    try {
	if (iv == 1) {
	    boo = TRUE;
	    (*da)->insert_boolean(boo);
	}
	else {
	    L = 0;
	    (*da)->insert_boolean(L);
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_boolean */

VSBoolean VSget_boolean(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_boolean:
	  {
	      CORBA::Boolean boo = TRUE;
	      (*da)->insert_boolean(boo);
	  }
	  break;
	  case CORBA::tk_long:
	  {
	      CORBA::Long L = 2;
	      (*da)->insert_long(L);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::Boolean rboo;
    
    try {
	rboo = (*da)->get_boolean();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    if (rboo == TRUE)    
	return (VSBoolean) TRUE;
    else
	return FALSE;
}

/*------------------------------------------------------*/
/* insert_long */

VSBoolean VSinsert_long(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::Boolean boo;
    CORBA::Long L;

    try {
	if (iv == 1) {
	    L = 0;
	    (*da)->insert_long(L);
	}
	else {
	    boo = TRUE;
	    (*da)->insert_long(boo);
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_long */

VSBoolean VSget_long(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_boolean:
	  {
	      CORBA::Boolean boo = TRUE;
	      (*da)->insert_boolean(boo);
	  }
	  break;
	  case CORBA::tk_long:
	  {
	      CORBA::Long L = 0;
	      (*da)->insert_long(L);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::Long rL;
    
    try {
	rL = (*da)->get_long();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rL == 0)    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}


/*------------------------------------------------------*/
/* insert_short */

VSBoolean VSinsert_short(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::Short S;
    CORBA::UShort US;

    try {
	if (iv == 1) {
	    S = 0;
	    (*da)->insert_short(S);
	}
	else {
	    US = 1;
	    (*da)->insert_short(US);
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
} 


/*------------------------------------------------------*/
/* get_short */

VSBoolean VSget_short(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_short:
	  {
	      CORBA::Short S = 0;
	      (*da)->insert_short(S);
	  }
	  break;
	  case CORBA::tk_ushort:
	  {
	      CORBA::UShort US = 1;
	      (*da)->insert_ushort(US);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::Short rS;
    
    try {
	rS = (*da)->get_short();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rS == 0)    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}

/*------------------------------------------------------*/
/* insert_ushort */

VSBoolean VSinsert_ushort(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::Short S;
    CORBA::UShort US;

    try {
	if (iv == 1) {
	    US = 1;
	    (*da)->insert_ushort(US);
	}
	else {
	    S = 0;
	    (*da)->insert_ushort(S);
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_ushort */

VSBoolean VSget_ushort(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_short:
	  {
	      CORBA::Short S = 0;
	      (*da)->insert_short(S);
	  }
	  break;
	  case CORBA::tk_ushort:
	  {
	      CORBA::UShort US = 1;
	      (*da)->insert_ushort(US);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::UShort rUS;
    
    try {
	rUS = (*da)->get_ushort();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rUS == 1)    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}

/*------------------------------------------------------*/
/* insert_longlong */

VSBoolean VSinsert_longlong(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::LongLong LL;
    CORBA::ULongLong ULL;

    try {
	if (iv == 1) {
	    LL = 0;
	    (*da)->insert_longlong(LL);
	}
	else {
	    ULL = 1;
	    (*da)->insert_longlong(ULL);
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_longlong */

VSBoolean VSget_longlong(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_longlong:
	  {
	      CORBA::LongLong LL = 0;
	      (*da)->insert_longlong(LL);
	  }
	  break;
	  case CORBA::tk_ulonglong:
	  {
	      CORBA::ULongLong ULL = 1;
	      (*da)->insert_ulonglong(ULL);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::LongLong rLL;
    
    try {
	rLL = (*da)->get_longlong();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rLL == 0)    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}

/*------------------------------------------------------*/
/* insert_ulonglong */

VSBoolean VSinsert_ulonglong(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::LongLong LL;
    CORBA::ULongLong ULL;

    try {
	if (iv == 1) {
	    ULL = 1;
	    (*da)->insert_ulonglong(ULL);
	}
	else {
	    LL = 0;
	    (*da)->insert_ulonglong(LL);
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_ulonglong */

VSBoolean VSget_ulonglong(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_longlong:
	  {
	      CORBA::LongLong LL = 0;
	      (*da)->insert_longlong(LL);
	  }
	  break;
	  case CORBA::tk_ulonglong:
	  {
	      CORBA::ULongLong ULL = 1;
	      (*da)->insert_ulonglong(ULL);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::ULongLong rULL;
    
    try {
	rULL = (*da)->get_ulonglong();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rULL == 1)    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}

/*------------------------------------------------------*/
/* insert_ulong */

VSBoolean VSinsert_ulong(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::Double D;
    CORBA::ULong UL;

    try {
	if (iv == 1) {
	    UL = 1;
	    (*da)->insert_ulong(UL);
	}
	else {
	    D = 1.2;
	    (*da)->insert_ulong(D);
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_ulong */

VSBoolean VSget_ulong(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_ulong:
	  {
	      CORBA::ULong UL = 1;
	      (*da)->insert_ulong(UL);
	  }
	  break;
	  case CORBA::tk_double:
	  {
	      CORBA::Double D = 1.2;
	      (*da)->insert_double(D);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::ULong rUL;
    
    try {
	rUL = (*da)->get_ulong();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rUL == 1)    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}


/*------------------------------------------------------*/
/* insert_double */

VSBoolean VSinsert_double(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::Double D;
    CORBA::ULong UL;

    try {
	if (iv == 1) {
	    D = 1.2;
	    (*da)->insert_double(D);
	}
	else {
	    UL = 1;
	    (*da)->insert_double(UL);
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
} 

/*------------------------------------------------------*/
/* get_double */

VSBoolean VSget_double(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_ulong:
	  {
	      CORBA::ULong UL = 1;
	      (*da)->insert_ulong(UL);
	  }
	  break;
	  case CORBA::tk_double:
	  {
	      CORBA::Double D = 1.2;
	      (*da)->insert_double(D);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::Double rD;
    
    try {
	rD = (*da)->get_double();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rD == 1.2)    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}

/*------------------------------------------------------*/
/* insert_longdouble */

VSBoolean VSinsert_longdouble(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::LongDouble LD;
    CORBA::Float F;

    try {
	if (iv == 1) {
	    LD = 1.2;
	    (*da)->insert_longdouble(LD);
	}
	else {
	    F = 2.2;
	    (*da)->insert_longdouble(F);
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_longdouble */

VSBoolean VSget_longdouble(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_float:
	  {
	      CORBA::Float F = 2.2;
	      (*da)->insert_float(F);
	  }
	  break;
	  case CORBA::tk_longdouble:
	  {
	      CORBA::LongDouble LD = 1.2;
	      (*da)->insert_longdouble(LD);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::Double rLD;
    
    try {
	rLD = (*da)->get_longdouble();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rLD == 1.2)    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}

/*------------------------------------------------------*/
/* insert_float */

VSBoolean VSinsert_float(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::LongDouble LD;
    CORBA::Float F;

    try {
	if (iv == 1) {
	     F = 2.2;
	    (*da)->insert_float(F);
	}
	else {
	    LD = 1.2;
	    (*da)->insert_float(LD);	   
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_float */

VSBoolean VSget_float(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_float:
	  {
	      CORBA::Float F = 2.2;
	      (*da)->insert_float(F);
	  }
	  break;
	  case CORBA::tk_longdouble:
	  {
	      CORBA::LongDouble LD = 1.2;
	      (*da)->insert_longdouble(LD);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::Double rF;
    
    try {
	rF = (*da)->get_float();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rF == 2.2)    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}

/*------------------------------------------------------*/
/* insert_char */

VSBoolean VSinsert_char(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::Char C;
    CORBA::WChar WC;

    try {
	if (iv == 1) {
	     C = 'a';
	    (*da)->insert_char(C);
	}
	else {
	    WC = L'b';
	    (*da)->insert_char(WC);	   
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_char */

VSBoolean VSget_char(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_char:
	  {
	      CORBA::Char C = 'a';
	      (*da)->insert_char(C);
	  }
	  break;
	  case CORBA::tk_wchar:
	  {
	      CORBA::WChar WC = L'b';
	      (*da)->insert_wchar(WC);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::Char rC;
    
    try {
	rC = (*da)->get_char();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rC == 'a')    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}

/*------------------------------------------------------*/
/* insert_wchar */

VSBoolean VSinsert_wchar(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::Char C;
    CORBA::WChar WC;

    try {
	if (iv == 1) {
	      WC = L'b';
	    (*da)->insert_wchar(WC);
	}
	else {
	    C = 'a';
	    (*da)->insert_wchar(C);	   
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_wchar */

VSBoolean VSget_wchar(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_char:
	  {
	      CORBA::Char C = 'a';
	      (*da)->insert_char(C);
	  }
	  break;
	  case CORBA::tk_wchar:
	  {
	      CORBA::WChar WC = L'b';
	      (*da)->insert_wchar(WC);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::WChar rWC;
    
    try {
	rWC = (*da)->get_wchar();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (rWC == 'b')    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}

/*------------------------------------------------------*/
/* insert_string */

VSBoolean VSinsert_string(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    try {
	if (iv == 1) {
	    CORBA::Char* S = "a";
	    (*da)->insert_string(S);	     
	}
	else {
	    // Compile error
	    CORBA::Char* WS = "b";
	    (*da)->insert_string(WS);	   
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_string */

VSBoolean VSget_string(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_string:
	  {
	      CORBA::Char* S = "a";
	      (*da)->insert_string(S);
	  }
	  break;
	  case CORBA::tk_wstring:
	  {
	      CORBA::WChar* WS = L"b";
	      (*da)->insert_wstring(WS);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::Char* rS;
    
    try {
	rS= (*da)->get_string();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    if (strcmp(rS, "a") == 0)    
	return (VSBoolean) TRUE;
    else 
	return (VSBoolean) FALSE;
}

/*------------------------------------------------------*/
/* insert_wstring */

VSBoolean VSinsert_wstring(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }

   
    CORBA::WChar* S;
    CORBA::WChar* WS;

    try {
	if (iv == 1) {
	      WS = L"b";
	    (*da)->insert_wstring(WS);
	}
	else {
	    S = L"a";
	    (*da)->insert_wstring(S);	   
	}   
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_wstring */

VSBoolean VSget_wstring(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_char:
	  {
	      CORBA::Char* S = "a";
	      (*da)->insert_string(S);
	  }
	  break;
	  case CORBA::tk_wstring:
	  {
	      CORBA::WChar* WS = L"b";
	      (*da)->insert_wstring(WS);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::WChar* rWS;
    
    try {
	rWS= (*da)->get_wstring();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    // Compare impossible
    return TRUE;  
}

/*------------------------------------------------------*/
/* insert_reference */

VSBoolean VSinsert_reference(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{	
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }
       
    CORBA::ValueBase* val;
	
    CORBA::Object_ptr obj;
	
    try {
	TestInterface_impl *I = new TestInterface_impl();
	obj = I ->_this();
	
	(*da)->insert_reference(obj);	   
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;    
}

/*------------------------------------------------------*/
/* get_reference */

VSBoolean VSget_reference(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    
    CORBA::Object_ptr obj;
    CORBA::ValueBase* val;
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	TestInterface_impl *I = new TestInterface_impl();
	obj = I ->_this();


	TestValue_impl *V = new TestValue_impl();
	val = V->_copy_value();

	switch (tc->kind()) {
	  case CORBA::tk_objref:
	  {	       
	      (*da)->insert_reference(obj);
	  }
	  break;
	  case CORBA::tk_value:
	  {	      
	      (*da)->insert_val(val);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::Object_ptr robj;
    
    try {
	robj= (*da)->get_reference();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    return (obj->_is_equivalent(robj));  
}

/*------------------------------------------------------*/
/* insert_val */

VSBoolean VSinsert_val(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }
       
    CORBA::ValueBase* val;
	
    try {
	TestValue_impl *V = new TestValue_impl();
	val = V->_copy_value();

	(*da)->insert_val(val);	   
    }   
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_val */

VSBoolean VSget_val(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    
    CORBA::Object_ptr obj;
    CORBA::ValueBase* val;
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	TestInterface_impl *I = new TestInterface_impl();
	obj = I ->_this();

	TestValue_impl *V = new TestValue_impl();
	val = V->_copy_value();

	switch (tc->kind()) {
	  case CORBA::tk_objref:
	  {	       
	      (*da)->insert_reference(obj);
	  }
	  break;
	  case CORBA::tk_value:
	  {	      
	      (*da)->insert_val(val);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::ValueBase* rval;
    
    try {
	rval= (*da)->get_val();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    return TRUE;  
}

/*------------------------------------------------------*/
/* insert_any */

VSBoolean VSinsert_any(VSDynAny_ptr *p_da, VSBoolean p_iv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Boolean iv = (CORBA::Boolean) p_iv;

    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	if (tc->kind() == CORBA::tk_struct)
	{
	    provide_struct_dany(*da);
	    (*da)->seek(-1);
	}
    }
    catch (...) {
	return FALSE;
    }
   
    CORBA::Any A;
    CORBA::Octet O;

    try {
	A <<= (CORBA::Short) 0;
	(*da)->insert_any(A);	   
    }   
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_any */

VSBoolean VSget_any(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
        
    try {    
	CORBA::TypeCode_ptr tc = (*da)->type();

	switch (tc->kind()) {
	  case CORBA::tk_any:
	  {
	      CORBA::Any A;
	      A <<= (CORBA::Short) 0;
	      (*da)->insert_any(A);
	  }
	  break;
	  case CORBA::tk_octet:
	  {
	      CORBA::Octet O = 1;
	      (*da)->insert_octet(O);
	  }
	  break;
	  case CORBA::tk_struct:
	  {
	      provide_struct_dany(*da);
	      (*da)->seek(-1);
	  }
	  break;		
	}
    }	
    catch (...) {
	return FALSE;
    }    

    CORBA::Any_var rA;
    
    try {
	rA = (*da)->get_any();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    CORBA::TypeCode_var a_tc;
    try {
	a_tc = rA->type();	
    }
    catch (...) {
	return FALSE; 
    }

    if (a_tc->kind() == CORBA::tk_short)
    {
	CORBA::Short a_short;
	rA >>= a_short;
	return (a_short == 0);
    }
    else
	return FALSE;
}
       


} // end 	
