/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* interface DynAnyFactory                         */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/
/* component_count */

VSULong VScomponent_count(VSDynAny_ptr *p_da, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;

    CORBA::ULong count;

    try {
	count = (*da)->component_count();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSULong) -1;
    }
    catch (...) {
	return (VSULong) -1;
    }
    
    return (VSULong) count;
}

/*------------------------------------------------------*/
/* seek */

VSBoolean VSseek(VSDynAny_ptr *p_da, VSLong p_index, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Long index = (CORBA::Long) p_index;
    
    CORBA::Boolean boo;
    
    try {
	boo = (*da)->seek(index);
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) 0;
    }
    catch (...) {
	return (VSBoolean) 0;
    }
    
    return (VSBoolean) boo;
}

/*------------------------------------------------------*/
/* rewind */

VSBoolean VSrewind(VSDynAny_ptr *p_da, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
 
    try {
	(*da)->rewind();
    }
     catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* next */
VSBoolean VSnext(VSDynAny_ptr *p_da, VSULong current_positon, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Long position = (CORBA::Long) current_positon;

    CORBA::TypeCode_var da_tc;
     try {
	da_tc = (*da)->type();	
    }
    catch (...) {
	return FALSE; 
    }
     
     CORBA::Boolean boo;
     try {
	 switch (da_tc->kind()) {
	   case CORBA::tk_short:      
		 boo = (*da)->next();
		 break;	     
	   case CORBA::tk_struct:
	   {
	       if (position == 0)	     
		   boo = (*da)->next();
	       if (position == 1)
	       {
		   CORBA::ULong count = (*da)->component_count();
		   if((*da)->seek(count-1))
		       boo = (*da)->next();
	       }	
	   }
	   break;
	 }
     }	
     catch(CORBA::Exception &ex) {
	 CORBA::Exception *t = exception_convert(&ex);
	 if(env)
	     env->exception(t);
	 return (VSBoolean) -1;
     }
     catch (...) {
	 return (VSBoolean) -1;
     }
     
     return (VSBoolean) boo;
}

/*------------------------------------------------------*/
/* current_component */

VSDynAny_ptr *VScurrent_component(VSDynAny_ptr *p_da, VSULong p_position, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Long position = (CORBA::Long) p_position;

    CORBA::TypeCode_var da_tc;
    try {
	da_tc = (*da)->type();	
    }
    catch (...) {
	return FALSE; 
    }
     
    DynamicAny::DynAny_ptr rda;
    try {
	switch (da_tc->kind()) {
	  case CORBA::tk_short:      
		rda = (*da)->current_component();
		break;	     
	  case CORBA::tk_struct:
	  {
	      if (position == 1)	     
		  rda = (*da)->current_component();
	      if (position == 2)
	      {
		  CORBA::ULong count = (*da)->component_count();
		  if((*da)->seek(count))
		      rda = (*da)->current_component();
	      }	
	  }
	  break;
	}
    }	
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSDynAny_ptr *) 0;
    }
    catch (...) {
	return (VSDynAny_ptr *) 0;
    }

    DynamicAny::DynAny_ptr *tmp = new DynamicAny::DynAny_ptr(rda);
    return (VSDynAny_ptr *) tmp;
}

/*------------------------------------------------------*/
/* *VStype */

VSTypeCode_ptr *VStype(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    
    CORBA::TypeCode_ptr tc;
    
    try {
	tc = (*da)->type();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSTypeCode_ptr *) 0;
    }
    catch (...) {
	return (VSTypeCode_ptr *) 0;
    }
    
    CORBA::TypeCode_ptr *rtc = new CORBA::TypeCode_ptr(tc);
    return (VSTypeCode_ptr *)  rtc;
}

/*------------------------------------------------------------*/
/* Initialize a DynAny object from another DynAny object */

VSBoolean VSassign(VSDynAny_ptr *p_da1, VSDynAny_ptr *p_da2, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da1 = (DynamicAny::DynAny_ptr *) p_da1;
    DynamicAny::DynAny_ptr *da2 = (DynamicAny::DynAny_ptr *) p_da2;

    try {
	(*da1)->assign(*da2);
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    return(VSBoolean) TRUE;
}    

/*--------------------------------------------------------*/
/* Compare DynAny values */
	
VSBoolean VSequal(VSDynAny_ptr *p_da1, VSDynAny_ptr *p_da2, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da1 = (DynamicAny::DynAny_ptr *) p_da1;
    DynamicAny::DynAny_ptr *da2 = (DynamicAny::DynAny_ptr *) p_da2;

    CORBA::Boolean boo;
    try {
	boo = (*da1)->equal(*da2);
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    return(VSBoolean) boo; 
}    

/*--------------------------------------------------------*/
/* Initialize a DynAny object from an any value */

VSBoolean VSfrom_any(VSDynAny_ptr *p_dastr, VSAny_ptr p_astr, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::Any *astr = (CORBA::Any *) p_astr;
    DynamicAny::DynAny_ptr *dastr = (DynamicAny::DynAny_ptr *) p_dastr;

    CORBA::Boolean boo;
    try {
	(*dastr)->from_any(*astr);
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    return(VSBoolean) boo;
}

/*--------------------------------------------------------*/
/* Generate an any value from a DynAny object */
VSAny_ptr *VSto_any(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;

    CORBA::Any_var a;
    try {
	a = (*da)->to_any();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSAny_ptr *) 0;
    }
    catch (...) {
	return (VSAny_ptr *) 0;
    }

    CORBA::Any_var *ra = new CORBA::Any_var(a);
    return (VSAny_ptr *) ra;
}

/*--------------------------------------------------------*/
/* Destroy a DynAny object */

VSBoolean VSdestroy(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;

    try {
	(*da)->destroy();
    }  
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }

    return(VSBoolean) TRUE;
}

/*--------------------------------------------------------*/
/* Create a copy of a DynAny object */
  
VSDynAny_ptr *VScopy(VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;

    DynamicAny::DynAny_ptr tmp;
    try {
	tmp = (*da)->copy();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSDynAny_ptr *) 0;
    }
    catch (...) {
	return (VSDynAny_ptr *) 0;
    }

    DynamicAny::DynAny_ptr *rda = new DynamicAny::DynAny_ptr(tmp);
    return (VSDynAny_ptr *) rda;
}

}	
