/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* Interface DynAny                                */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include <t1_aux.h>

//-----------------------------------------------------------

int return_correct_count(VSDynAny_ptr *p_da, VSULong p_count)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::ULong count = (CORBA::ULong) p_count;

    CORBA::TypeCode_var da_tc;
     try {
	da_tc = (*da)->type();	
    }
    catch (...) {
	return FALSE; 
    }

     switch (da_tc->kind()) {
       case CORBA::tk_short:      
	     return (count == 0);       	
	     break;	     
       case CORBA::tk_struct:
	     return (count == 2);
	     break;
	     /* case CORBA::tk_array:
	     return (count == 3);
	     break; */
       default:
	     return FALSE;
	     break;
     }
}

int count_greater_index(VSDynAny_ptr *p_da, VSLong p_index)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Long index = (CORBA::Long) p_index;
    
    CORBA::ULong count = (*da)->component_count();
   
    if (count >= index)
	return TRUE;
    else
	return FALSE;
}

int equal_seek0(VSDynAny_ptr *p_da)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    
    DynamicAny::DynAny_ptr seekComp, rewindComp;
    rewindComp = (*da)->current_component();
    
    if ((*da)->seek(0))
	seekComp= (*da)->current_component();
    else
	return FALSE;

    return (rewindComp->equal(seekComp));
}


int comp_count(VSDynAny_ptr *p_da)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    
    CORBA::ULong count = (*da)->component_count();

    return count;
}

int valid_DynAny(VSDynAny_ptr *p_da)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;

    return (da ? TRUE : FALSE);
}

int is_nil_dany(VSDynAny_ptr *p_da)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
   
    return (da ? TRUE : FALSE);
}




int return_correct_TypeCode(VSTypeCode_ptr *p_tc)
{
    CORBA::TypeCode_ptr *tc = (CORBA::TypeCode_ptr *) p_tc;
    
    if ((*tc)->kind() == CORBA::tk_short)
	return TRUE;
    else
	return FALSE;
}

int same_TypeCode(VSDynAny_ptr *p_da, VSDynAny_ptr *p_dap)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    DynamicAny::DynAny_ptr *dap = (DynamicAny::DynAny_ptr *) p_dap;

    CORBA::TypeCode_var da_tc, dap_tc;
    

    try {
	da_tc = (*da)->type();
	dap_tc = (*dap)->type();	
    }
    catch (...) {
	return FALSE; 
    }

    if (da_tc->kind() == dap_tc->kind())
	return TRUE;
    else
	return FALSE;
}

int same_dany(VSDynAny_ptr *p_da, VSDynAny_ptr *p_dap)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    DynamicAny::DynAny_ptr *dap = (DynamicAny::DynAny_ptr *) p_dap;

    return ((*da)->equal(*dap));
}

int equal_DynAnyValue(VSDynAny_ptr *p_da, VSDynAny_ptr *p_dap)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    DynamicAny::DynAny_ptr *dap = (DynamicAny::DynAny_ptr *) p_dap;

    CORBA::TypeCode_var da_tc, dap_tc;
    try {
	da_tc = (*da)->type();
	dap_tc = (*dap)->type();	
    }
    catch (...) {
	return FALSE; 
    }

    if (da_tc->kind() == dap_tc->kind())
    {
	switch (da_tc->kind()) {
	  case CORBA::tk_short:		
	  {
	      CORBA::Short da_short, dap_short;
	      
	      da_short = (*da)->get_short();
	      dap_short = (*dap)->get_short();
	      return (da_short == dap_short);
	  }
	  break;
	  default:
		return FALSE;
		break;
	}
    }
    else
	return FALSE; 
}

int equal_AnyTypeCode(VSDynAny_ptr *p_da, VSAny_ptr p_a)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Any *a = (CORBA::Any *) p_a;

    CORBA::TypeCode_var da_tc, a_tc;
    

    try {
	da_tc = (*da)->type();
	a_tc = a->type();	
    }
    catch (...) {
	return FALSE; 
    }

    if (da_tc->kind() == a_tc->kind())
	return TRUE;
    else
	return FALSE;
}

int equal_AnyValue(VSDynAny_ptr *p_da, VSAny_ptr p_a)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Any *a = (CORBA::Any *) p_a;

    CORBA::TypeCode_var da_tc, a_tc;
    

    try {
	da_tc = (*da)->type();
	a_tc = a->type();	
    }
    catch (...) {
	return FALSE; 
    }

    if (da_tc->kind() == a_tc->kind())
    {
	switch (da_tc->kind()) {
	  case CORBA::tk_string:
	  {
	      const char *a_str, *da_str;
	      *a >>= a_str;
	      da_str = (*da)->get_string();
	      
	      return (a_str == da_str);
	  }
	  break;
	  case CORBA::tk_short:
	  {
	      CORBA::Short a_short, da_short;
	      *a >>= a_short;
	      da_short = (*da)->get_short();
	      
	      return (a_short == da_short);
	  }
	  break;
	  default:
		return FALSE;
		break;
	}
    }
    else
	return FALSE; 
}


int equal_Any(VSDynAny_ptr *p_da, VSAny_ptr *p_a)
{
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;
    CORBA::Any_var *a = (CORBA::Any_var *) p_a;

    CORBA::TypeCode_var da_tc, a_tc;
    

    try {
	da_tc = (*da)->type();
	a_tc = (*a)->type();	
    }
    catch (...) {
	return FALSE; 
    }

    if (da_tc->kind() == a_tc->kind())
    {
	switch (da_tc->kind()) {
	  case CORBA::tk_short:
	  {
	      CORBA::Short a_short, da_short;
	      *a >>= a_short;
	      da_short = (*da)->get_short();
	      
	      return (a_short == da_short);
	  }
	  break;
	  default:
		return FALSE;
		break;
	}
    }
    else
	return FALSE; 
}

int valid_AnyValue(VSAny_ptr p_a)
{
    CORBA::Any *a = (CORBA::Any *) p_a;

     const char *a_str;

    *a >>= a_str;
    if (a_str == NULL)
	return FALSE;
    else
	return TRUE;
}

}

