/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* Interface DynArray                             */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include <t1_aux.h>

//-----------------------------------------------------------

int valid_type_any_seq(VSDynArray_ptr *p_dArray, VSAnySeq_ptr p_aSeq)
{
    DynamicAny::DynArray_ptr *dArray = (DynamicAny::DynArray_ptr *) p_dArray;
    DynamicAny::AnySeq *aSeq = (DynamicAny::AnySeq *) p_aSeq;
    
    if(!dArray || !aSeq) 
	return FALSE;    

    CORBA::ULong count = (*dArray)->component_count();
    CORBA::ULong len = aSeq->length();
    
    if (count != len)
	return FALSE;

    CORBA::ULong i = 0;
    while (i < count)
    {
	CORBA::Any a;
	CORBA::TypeCode_var dArray_tc, array_tc, any_tc;

	a = (*aSeq)[i];
	CORBA::Any *any_array = new CORBA::Any(a);
	any_tc = any_array->type();
	dArray_tc = (*dArray)->type();
	array_tc = dArray_tc->content_type();  
	(*dArray)->seek(i);

	 if (array_tc->kind() != any_tc->kind())
	    return FALSE;
	i++;
    }
    
    return TRUE;  
}

int valid_value_any_seq(VSDynArray_ptr *p_dArray, VSAnySeq_ptr p_aSeq)
{
    DynamicAny::DynArray_ptr *dArray = (DynamicAny::DynArray_ptr *) p_dArray;
    DynamicAny::AnySeq *aSeq = (DynamicAny::AnySeq *) p_aSeq;
    
    if(!dArray || !aSeq) 
	return FALSE;    

    CORBA::ULong count = (*dArray)->component_count();
    CORBA::ULong len = aSeq->length();

    if (count == len)
	return TRUE;
    else
	return FALSE;
}

int is_correct_anyseq(VSDynArray_ptr *p_dArray, VSAnySeq_ptr p_aSeq)
{
    DynamicAny::DynArray_ptr *dArray = (DynamicAny::DynArray_ptr *) p_dArray;
    DynamicAny::AnySeq *aSeq = (DynamicAny::AnySeq *) p_aSeq;
    
    if(!dArray || !aSeq) 
	return FALSE;    

    CORBA::ULong count = (*dArray)->component_count();
    CORBA::ULong len = aSeq->length();
    
    if (count != len)
	return FALSE;

    CORBA::ULong i = 0;
    while (i < count)
    {
	CORBA::Any a;
	CORBA::TypeCode_var dArray_tc, array_tc, any_tc;

	a = (*aSeq)[i];
	CORBA::Any *any_array = new CORBA::Any(a);
	any_tc = any_array->type();
	dArray_tc = (*dArray)->type();
	array_tc = dArray_tc->content_type();  
	(*dArray)->seek(i);

	 if (array_tc->kind() != any_tc->kind())
	    return FALSE;
	i++;
    }
    
    return TRUE; 
}

int valid_value_dynany_seq(VSDynArray_ptr *p_dArray, VSDynAnySeq_ptr p_dSeq)
{
    DynamicAny::DynArray_ptr *dArray = (DynamicAny::DynArray_ptr *) p_dArray;
    DynamicAny::DynAnySeq *dSeq = (DynamicAny::DynAnySeq *) p_dSeq;
    
    if(!dArray || !dSeq) 
	return FALSE;    

    CORBA::ULong count = (*dArray)->component_count();
    CORBA::ULong len = dSeq->length();
    
    if (count != len)
	return FALSE;
    else
       return TRUE; 
}

int valid_type_dynany_seq(VSDynArray_ptr *p_dArray, VSDynAnySeq_ptr p_dSeq)
{
    DynamicAny::DynArray_ptr *dArray = (DynamicAny::DynArray_ptr *) p_dArray;
    DynamicAny::DynAnySeq *dSeq = (DynamicAny::DynAnySeq *) p_dSeq;
    
    if(!dArray || !dSeq) 
	return FALSE;    

    CORBA::ULong count = (*dArray)->component_count();
    CORBA::ULong len = dSeq->length();
    
    if (count != len)
	return FALSE;

    CORBA::ULong i = 0;
    while (i < count)
    {
	CORBA::Any a;
	CORBA::TypeCode_var dArray_tc, array_tc, dynany_tc;

	
	dynany_tc = (*dSeq)[i]->type();
	dArray_tc = (*dArray)->type();
	array_tc = dArray_tc->content_type();   
	(*dArray)->seek(i);

	 if (array_tc->kind() != dynany_tc->kind())
	    return FALSE;
	i++;
    }
    
    return TRUE; 
}

int is_correct_DynAnySeq(VSDynArray_ptr *p_dArray, VSDynAnySeq_ptr p_dSeq)
{
    DynamicAny::DynArray_ptr *dArray = (DynamicAny::DynArray_ptr *) p_dArray;
    DynamicAny::DynAnySeq *dSeq = (DynamicAny::DynAnySeq *) p_dSeq;
    
    if(!dArray || !dSeq) 
	return FALSE;    

    CORBA::ULong count = (*dArray)->component_count();
    CORBA::ULong len = dSeq->length();
    
    if (count != len)
	return FALSE;

    CORBA::ULong i = 0;
    while (i < count)
    {
	CORBA::Any a;
	CORBA::TypeCode_var dArray_tc, array_tc, dynany_tc;
	
	dynany_tc = (*dSeq)[i]->type();
	dArray_tc = (*dArray)->type();
	array_tc = dArray_tc->content_type();   
	(*dArray)->seek(i);

	 if (array_tc->kind() != dynany_tc->kind())
	    return FALSE;
	i++;
    }
    
    return TRUE;
}

} // end
