/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* Interface DynArray                              */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include <t1i_tdi.h>
#include <corba_defs.h>

    DynamicAny::DynAny_ptr get_DAny(CORBA::TypeCode_ptr p_tc)
    {   
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb(); 
	CORBA::Object_ptr obj = orb->resolve_initial_references("DynAnyFactory");
	DynamicAny::DynAnyFactory_ptr daf = DynamicAny::DynAnyFactory::_narrow(obj);
	DynamicAny::DynAny_ptr dany = daf ->create_dyn_any_from_type_code(p_tc);
	return dany;
    }

    CORBA::TypeCode_ptr provide_array_tc()
    {
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();

	CORBA::TypeCode_ptr array_tc = orb->create_array_tc(2, CORBA::_tc_long);
	return array_tc;
    } 

/*------------------------------------------------------------------*/
/* test variable VSDynArray_ptr * dArray(anonymous prop) */
int provide_dArray (VSDynArray_ptr **testvar, enum prop_dArray_prop_1 prop_1)
{
    DynamicAny::DynArray_ptr *tvar; 
    switch(prop_1) {
      case PdArray_prop_1_valid:
      {
	  try { 
	      CORBA::TypeCode_ptr array_tc = provide_array_tc();	      
	      DynamicAny::DynAny_ptr da = get_DAny(array_tc);
	      
	      DynamicAny::DynArray_ptr darray = DynamicAny::DynArray::_narrow(da);
	      tvar = new DynamicAny::DynArray_ptr(darray);
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
    }
    
    *testvar = (VSDynArray_ptr *) tvar;
    return TRUE;
}

int relinquish_dArray (VSDynArray_ptr **testvar, enum prop_dArray_prop_1 prop_1)
{
    DynamicAny::DynArray_ptr *tvar = (DynamicAny::DynArray_ptr *) *testvar;
    
    switch(prop_1) {
      case PdArray_prop_1_valid:
	    (*tvar)->destroy();
	    break;
    }
    
    return TRUE;
}


/*------------------------------------------------------------------*/
/* test variable VSAnySeq_ptr aSeq(anonymous prop) */
int provide_aSeq (VSAnySeq_ptr *testvar, enum prop_aSeq_prop_1 prop_1)
{
    DynamicAny::AnySeq *tvar;
    CORBA::Any x;
    
    
    switch(prop_1) {
      case PaSeq_prop_1_valid:
      {
	  try {
	      DynamicAny::AnySeq aSeq;
	      aSeq.length(2);	     
	      
	      x <<= (CORBA::Long) 0;
	      aSeq[0] = x;

	      x <<= (CORBA::Long ) 1;
	      aSeq[1] = x;
 	  
	      tvar = new DynamicAny::AnySeq(aSeq);
	  }
	  catch(...) {
	      return FALSE;
	  }
      }
      break;
      case  PaSeq_prop_1_typeinvalid:
      {
	  try {
	      DynamicAny::AnySeq aSeq;
	      aSeq.length(2);
	  
	      x <<= (CORBA::Short) 0;
	      aSeq[0] = x;

	      x <<= (CORBA::Short) 1;
	      aSeq[1] = x;
	  
	      tvar = new  DynamicAny::AnySeq(aSeq);
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
      case PaSeq_prop_1_valinvalid:
      {
	  try {
	      DynamicAny::AnySeq aSeq;
	      aSeq.length(1);
	  
	      x <<= (CORBA::Long) 0;
	      aSeq[0] = x;
	  
	      tvar = new  DynamicAny::AnySeq(aSeq);
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;    
    }
    
    *testvar = (VSAnySeq_ptr) tvar;
    return TRUE ;
}

int relinquish_aSeq (VSAnySeq_ptr *testvar, enum prop_aSeq_prop_1 prop_1)
{
    DynamicAny::AnySeq *tvar = (DynamicAny::AnySeq *) *testvar;

    switch(prop_1) {
      case PaSeq_prop_1_valid:
      case  PaSeq_prop_1_typeinvalid:
      case PaSeq_prop_1_valinvalid:
	    delete tvar;
	    break;
    }

    return TRUE;
}

/*------------------------------------------------------------------*/
/* test variable VSDynAnySeq_ptr dSeq(anonymous prop) */
int provide_dSeq (VSDynAnySeq_ptr *testvar, enum prop_dSeq_prop_1 prop_1)
{
    DynamicAny::DynAnySeq *tvar;
    DynamicAny::DynAny_var da;

    switch(prop_1) {
      case PdSeq_prop_1_valid:
      {
	  try {
	      DynamicAny::DynAnySeq dSeq;
	      dSeq.length(2);	  
	      da = get_DAny(CORBA::_tc_long);

	      da->insert_long(0);
	      dSeq[0] = da;

	      da->insert_long(1);
	      dSeq[1] = da;
	  
	      tvar = new DynamicAny::DynAnySeq(dSeq);
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
      case  PdSeq_prop_1_typeinvalid:
      {
	  try {
	      DynamicAny::DynAnySeq dSeq;
	      dSeq.length(1);	  
	      da = get_DAny(CORBA::_tc_long);

	      da->insert_long(0);
	      dSeq[0] = da;	 
	  
	      tvar = new DynamicAny::DynAnySeq(dSeq);
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
      case PdSeq_prop_1_valinvalid:
      {
	  try {
	      DynamicAny::DynAnySeq dSeq;
	      dSeq.length(2);	  
	      da = get_DAny(CORBA::_tc_short);

	      da->insert_short(0);
	      dSeq[0] = da;

	      da->insert_short(1);
	      dSeq[1] = da;
	  
	      tvar = new DynamicAny::DynAnySeq(dSeq);
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
    }
    
    *testvar = (VSDynAnySeq_ptr) tvar;
    return TRUE;
}

int relinquish_dSeq (VSDynAnySeq_ptr *testvar, enum prop_dSeq_prop_1 prop_1)
{   
    DynamicAny::DynAnySeq *tvar = (DynamicAny::DynAnySeq *) *testvar;

    switch(prop_1) {
      case PdSeq_prop_1_valid:
      case  PdSeq_prop_1_typeinvalid:
      case PdSeq_prop_1_valinvalid:
	    delete tvar;
	    break;
    } 

    return TRUE;
}

} // end
