/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* interface DynArray                             */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/
/* set_length */

VSBoolean VSset_length(VSDynSequence_ptr *p_dSeq, VSULong p_len, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    CORBA::ULong len = (CORBA::ULong) p_len;

    try {
	(*dSeq)->set_length(len);
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSBoolean) FALSE;
    }
    catch (...) {
	(VSBoolean) FALSE;
    }

    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_length */

VSULong VSget_length(VSDynSequence_ptr *p_dSeq, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev; 
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
   
    try {
	(*dSeq)->set_length(1);
    }
    catch (...) {
	return (VSULong) 0;
    }

    CORBA::ULong glen;
    try {
	glen = (*dSeq)->get_length();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSULong) 0;
    }
    catch (...) {
	(VSULong) 0;
    }

    return (VSULong) glen;
}


/*------------------------------------------------------*/
/* set_elements */

VSBoolean VSset_elements(VSDynSequence_ptr *p_dSeq, VSAnySeq_ptr p_aSeq, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    DynamicAny::AnySeq *aSeq = (DynamicAny::AnySeq *) p_aSeq;
    
    try {
	(*dSeq)->set_elements(*aSeq);
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSBoolean) FALSE;
    }
    catch (...) {
	(VSBoolean) FALSE;
    }

    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_elements */

VSAnySeq_ptr VSget_elements(VSDynSequence_ptr *p_dSeq, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    
    DynamicAny::AnySeq *aSeq; 
    try {
	aSeq = (*dSeq)->get_elements();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSAnySeq_ptr) 0;
    }
    catch (...) {
	(VSAnySeq_ptr) 0;
    }    
    
    return (VSAnySeq_ptr) aSeq;
} 

/*------------------------------------------------------*/
/* set_elements_as_dyn_any */

VSBoolean VSset_elements_as_dyn_any(VSDynSequence_ptr *p_dSeq, VSDynAnySeq_ptr p_seq, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    DynamicAny::DynAnySeq *seq = (DynamicAny::DynAnySeq *) p_seq;

    CORBA::Boolean boo;
    try {
	(*dSeq)->set_elements_as_dyn_any(*seq);
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSBoolean) FALSE;
    }
    catch (...) {
	(VSBoolean) FALSE;
    }

    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_elements_as_dyn_any */

VSDynAnySeq_ptr VSget_elements_as_dyn_any(VSDynSequence_ptr *p_dSeq, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    
    DynamicAny::DynAnySeq *seq; 
    try {
	seq = (*dSeq)->get_elements_as_dyn_any();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSDynAnySeq_ptr) 0;
    }
    catch (...) {
	(VSDynAnySeq_ptr) 0;
    }    
    
    return (VSDynAnySeq_ptr) seq;
}

} // end
