/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* Interface DynArray                             */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include <t1_aux.h>

//-----------------------------------------------------------

int len_exceed_boundSeq(VSDynSequence_ptr *p_dSeq, VSULong p_len)
{
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    CORBA::ULong len = (CORBA::ULong) p_len;

    CORBA::ULong glen;

    glen = (*dSeq)->get_length();

    return (glen < len);
}


int return_correct_len(VSULong p_len)
{
    CORBA::ULong len = (CORBA::ULong) p_len;
    return (len == 1);
}

int valid_type_any_seq(VSDynSequence_ptr *p_dSeq, VSAnySeq_ptr p_aSeq)
{
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    DynamicAny::AnySeq *aSeq = (DynamicAny::AnySeq *) p_aSeq;
    
    if(!dSeq || !aSeq) 
	return FALSE;    

    CORBA::ULong count = (*dSeq)->component_count();
    CORBA::ULong len = aSeq->length();
    
    if (count < len)
	return FALSE;

    CORBA::ULong i = 0;
    while (i < count)
    {
	CORBA::Any a;
	CORBA::TypeCode_var dSeq_tc, seq_tc, any_tc;

	a = (*aSeq)[i];
	CORBA::Any *any_seq = new CORBA::Any(a);
	any_tc = any_seq->type();
	dSeq_tc = (*dSeq)->type();
	seq_tc = dSeq_tc->content_type(); 
 
	(*dSeq)->seek(i);

	if (seq_tc->kind() != any_tc->kind())
	    return FALSE;
	i++;
    }
    
    return TRUE;  
}

int valid_value_any_seq(VSDynSequence_ptr *p_dSeq, VSAnySeq_ptr p_aSeq)
{
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    DynamicAny::AnySeq *aSeq = (DynamicAny::AnySeq *) p_aSeq;
    
    if(!dSeq || !aSeq) 
	return FALSE;    

    CORBA::ULong count = (*dSeq)->component_count();
    CORBA::ULong len = aSeq->length();

    if (count >= len)
	return TRUE;
    else
	return FALSE;
}

int is_correct_anyseq(VSDynSequence_ptr *p_dSeq, VSAnySeq_ptr p_aSeq)
{
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    DynamicAny::AnySeq *aSeq = (DynamicAny::AnySeq *) p_aSeq;
    
    if(!dSeq || !aSeq) 
	return FALSE;    

    CORBA::ULong count = (*dSeq)->component_count();
    CORBA::ULong len = aSeq->length();
    
    if (count < len)
	return FALSE;

    CORBA::ULong i = 0;
    while (i < count)
    {
	CORBA::Any a;
	CORBA::TypeCode_var dSeq_tc, seq_tc, any_tc;

	a = (*aSeq)[i];
	CORBA::Any *any_seq = new CORBA::Any(a);
	any_tc = any_seq->type();
	dSeq_tc = (*dSeq)->type();
	seq_tc = dSeq_tc->content_type();  
	(*dSeq)->seek(i);

	 if (seq_tc->kind() != any_tc->kind())
	    return FALSE;
	i++;
    }
    
    return TRUE; 
}

int valid_value_dynany_seq(VSDynSequence_ptr *p_dSeq, VSDynAnySeq_ptr p_seq)
{
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    DynamicAny::DynAnySeq *seq = (DynamicAny::DynAnySeq *) p_seq;
    
    if(!dSeq || !seq) 
	return FALSE;    

    CORBA::ULong count = (*dSeq)->component_count();
    CORBA::ULong len = seq->length();
    
    if (count >= len)
	return TRUE;
    else
       return FALSE; 
}

int valid_type_dynany_seq(VSDynSequence_ptr *p_dSeq, VSDynAnySeq_ptr p_seq)
{
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    DynamicAny::DynAnySeq *seq = (DynamicAny::DynAnySeq *) p_seq;
    
    if(!dSeq || !seq) 
	return FALSE;    

    CORBA::ULong count = (*dSeq)->component_count();
    CORBA::ULong len = seq->length();
    
    if (count < len)
	return FALSE;

    CORBA::ULong i = 0;
    while (i < count)
    {
	CORBA::Any a;
	CORBA::TypeCode_var dSeq_tc, seq_tc, dynany_tc;

	
	dynany_tc = (*seq)[i]->type();
	dSeq_tc = (*dSeq)->type();
	seq_tc = dSeq_tc->content_type();   
	(*dSeq)->seek(i);

	 if (seq_tc->kind() != dynany_tc->kind())
	    return FALSE;
	i++;
    }
    
    return TRUE; 
}

int is_correct_DynAnySeq(VSDynSequence_ptr *p_dSeq, VSDynAnySeq_ptr p_seq)
{
    DynamicAny::DynSequence_ptr *dSeq = (DynamicAny::DynSequence_ptr *) p_dSeq;
    DynamicAny::DynAnySeq *seq = (DynamicAny::DynAnySeq *) p_seq;
    
    if(!dSeq || !seq) 
	return FALSE;    

    CORBA::ULong count = (*dSeq)->component_count();
    CORBA::ULong len = seq->length();
    
    if (count < len)
	return FALSE;

    CORBA::ULong i = 0;
    while (i < count)
    {
	CORBA::Any a;
	CORBA::TypeCode_var dSeq_tc, seq_tc, dynany_tc;
	
	dynany_tc = (*seq)[i]->type();
	dSeq_tc = (*dSeq)->type();
	seq_tc = dSeq_tc->content_type();   
	(*dSeq)->seek(i);

	 if (seq_tc->kind() != dynany_tc->kind())
	    return FALSE;
	i++;
    }
    
    return TRUE;
}

} // end

