/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* interface DynStruct                             */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/
/* current_member_name */

VSString VScurrent_member_name(VSDynStruct_ptr *p_ds, VSLong position, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynStruct_ptr *ds = (DynamicAny::DynStruct_ptr *) p_ds;
    CORBA::Long pos = (CORBA::Long) position;

    try {
	(*ds)->seek(pos);
    }
    catch (...) {
	return (VSString) NULL;
    }

    char* name;
    try {
	name = (*ds)->current_member_name();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSString) NULL;
    }
    catch (...) {
	(VSString) NULL;
    }

    return (VSString) name;
}

/*------------------------------------------------------*/
/* current_member_kind */

VSTCKind VScurrent_member_kind(VSDynStruct_ptr *p_ds, VSLong position, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynStruct_ptr *ds = (DynamicAny::DynStruct_ptr *) p_ds;
    CORBA::Long pos = (CORBA::Long) position;

    try {
	(*ds)->seek(pos);
    }
    catch (...) {
	return (VSTCKind) NULL;
    }

    CORBA::TCKind tcKind;
    try {
	tcKind = (*ds)->current_member_kind();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSTCKind) 0;
    }
    catch (...) {
	(VSTCKind) 0;
    }

    return (VSTCKind) tcKind;
}

/*------------------------------------------------------*/
/* set_members */

VSBoolean VSset_members(VSDynStruct_ptr *p_ds, VSNameValuePairSeq_ptr p_nv, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynStruct_ptr *ds = (DynamicAny::DynStruct_ptr *) p_ds;
    DynamicAny::NameValuePairSeq *nv = (DynamicAny::NameValuePairSeq *) p_nv;

    CORBA::Boolean boo;
    try {
	(*ds)->set_members(*nv);
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSBoolean) FALSE;
    }
    catch (...) {
	(VSBoolean) FALSE;
    }

    return (VSBoolean) TRUE;
}

/*------------------------------------------------*/
/* get_members */

VSNameValuePairSeq_ptr VSget_members(VSDynStruct_ptr *p_ds, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynStruct_ptr *ds = (DynamicAny::DynStruct_ptr *) p_ds;

    DynamicAny::NameValuePairSeq *nv; 
    try {
	nv = (*ds)->get_members();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSNameValuePairSeq_ptr) 0;
    }
    catch (...) {
	(VSNameValuePairSeq_ptr) 0;
    }    
    
    return (VSNameValuePairSeq_ptr) nv;
}


/*------------------------------------------------*/
/* set_members_as_dyn_any */

VSBoolean VSset_members_as_dyn_any(VSDynStruct_ptr *p_ds, VSNameDynAnyPairSeq_ptr p_nda, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynStruct_ptr *ds = (DynamicAny::DynStruct_ptr *) p_ds;
    DynamicAny::NameDynAnyPairSeq *nda = (DynamicAny::NameDynAnyPairSeq *) p_nda;
        
    try {
	(*ds)->set_members_as_dyn_any(*nda);
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSBoolean) FALSE;
    }
    catch (...) {
	(VSBoolean) FALSE;
    }

    return (VSBoolean) TRUE;
}

/*------------------------------------------------*/
/* get_members_as_dyn_any*/

VSNameDynAnyPairSeq_ptr VSget_members_as_dyn_any(VSDynStruct_ptr *p_ds, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynStruct_ptr *ds = (DynamicAny::DynStruct_ptr *) p_ds;

    DynamicAny::NameDynAnyPairSeq *nda; 
    try {
	nda = (*ds)->get_members_as_dyn_any();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSNameDynAnyPairSeq_ptr) 0;
    }
    catch (...) {
	(VSNameDynAnyPairSeq_ptr) 0;
    }    
    
    return (VSNameDynAnyPairSeq_ptr) nda;
}

}

