/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* interface DynStruct                             */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>


/*------------------------------------------------------*/
/* set_discriminator */

VSBoolean VSset_discriminator(VSDynUnion_ptr *p_du, VSDynAny_ptr *p_da, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynUnion_ptr *du = (DynamicAny::DynUnion_ptr *) p_du;
    DynamicAny::DynAny_ptr *da = (DynamicAny::DynAny_ptr *) p_da;

    try {
	(*du)->set_discriminator(*da);
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSBoolean) FALSE;
    }
    catch (...) {
	(VSBoolean) FALSE;
    }

    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* get_discriminator */

VSDynAny_ptr *VSget_discriminator(VSDynUnion_ptr *p_du, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynUnion_ptr *du = (DynamicAny::DynUnion_ptr *) p_du;
    
    DynamicAny::DynAny_ptr da;
    try {
	da = (*du)->get_discriminator();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSDynAny_ptr *) 0;
    }
    catch (...) {
	(VSDynAny_ptr *) 0;
    }
    
    DynamicAny::DynAny_ptr *rda = new DynamicAny::DynAny_ptr(da);
    return (VSDynAny_ptr *) rda;
 }

/*------------------------------------------------------*/    
/* set_to_default_member */

VSBoolean VSset_to_default_member(VSDynUnion_ptr *p_du, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynUnion_ptr *du = (DynamicAny::DynUnion_ptr *) p_du;
    
    try {
	(*du)->set_to_default_member();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSBoolean) FALSE;
    }
    catch (...) {
	(VSBoolean) FALSE;
    }

    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/    
/* set_to_no_active_member */

VSBoolean VSset_to_no_active_member(VSDynUnion_ptr *p_du, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynUnion_ptr *du = (DynamicAny::DynUnion_ptr *) p_du;

     try {
	(*du)->set_to_no_active_member();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSBoolean) FALSE;
    }
    catch (...) {
	(VSBoolean) FALSE;
    }

    return (VSBoolean) TRUE;
}

/*------------------------------------------------*/
/* has_no_active_member */

VSBoolean VShas_no_active_member(VSDynUnion_ptr *p_du, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynUnion_ptr *du = (DynamicAny::DynUnion_ptr *) p_du;  
    
     
    CORBA::Boolean boo;
    try {
	boo = (*du)->has_no_active_member();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSBoolean) FALSE;
    }
    catch (...) {
	(VSBoolean) FALSE;
    }

    return (VSBoolean) boo;
}

/*------------------------------------------------*/
/* discriminator_kind*/

VSTCKind VSdiscriminator_kind (VSDynUnion_ptr *p_du, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynUnion_ptr *du = (DynamicAny::DynUnion_ptr *) p_du;
    
    CORBA::TCKind tk;
    try {
	tk = (*du)->discriminator_kind();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSTCKind) 0;
    }
    catch (...) {
	(VSTCKind) 0;
    }

    return (VSTCKind) tk;
}

/*------------------------------------------------*/
/* member_kind*/

VSTCKind VSmember_kind (VSDynUnion_ptr *p_du, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynUnion_ptr *du = (DynamicAny::DynUnion_ptr *) p_du;  

    CORBA::TCKind tk;
    try {
	tk = (*du)->member_kind();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSTCKind) 0;
    }
    catch (...) {
	(VSTCKind) 0;
    }

    return (VSTCKind) tk;
}

/*------------------------------------------------*/
/* member*/

VSDynAny_ptr *VSmember(VSDynUnion_ptr *p_du, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynUnion_ptr *du = (DynamicAny::DynUnion_ptr *) p_du;   

    DynamicAny::DynAny_ptr memda;
    try {
	memda = (*du)->member();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSDynAny_ptr) 0;
    }
    catch (...) {
	(VSDynAny_ptr) 0;
    }

    DynamicAny::DynAny_ptr *rany = new DynamicAny::DynAny_ptr(memda);
    return (VSDynAny_ptr *) rany;
}

/*------------------------------------------------*/
/* member_name */

VSString VSmember_name(VSDynUnion_ptr *p_du, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynUnion_ptr *du = (DynamicAny::DynUnion_ptr *) p_du;
   
    char *name;
    try {
	name = (*du)->member_name();
    }
    catch(CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	(VSString) 0;
    }
    catch (...) {
	(VSString) 0;
    }
    
    return (VSString) name;
}

} // end
