/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/


/***************************************************/
/* Interface DynUnion                              */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include <t1i_tdi.h>
#include <corba_defs.h>

        
    DynamicAny::DynAny_ptr get_DAny(CORBA::TypeCode_ptr p_tc)
    {   
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb(); 
	CORBA::Object_ptr obj = orb->resolve_initial_references("DynAnyFactory");
	DynamicAny::DynAnyFactory_ptr daf = DynamicAny::DynAnyFactory::_narrow(obj);
	DynamicAny::DynAny_ptr dany = daf ->create_dyn_any_from_type_code(p_tc);
	return dany;
    }

    CORBA::TypeCode_ptr provide_enum_tc()
    {
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();

	CORBA::EnumMemberSeq mems(3);
	mems.length(3);

	mems[0] = CORBA::string_dup("red");
	mems[1] = CORBA::string_dup("green");
	mems[2] = CORBA::string_dup("blue");

	
	CORBA::TypeCode_ptr enum_tc = orb->create_enum_tc(
	    "IDL:DAnyTest/TestEnum:1.0",
	    "TestEnum",
	    mems );
	return enum_tc;
    }
    
    CORBA::TypeCode_ptr provide_union_tc(CORBA::TypeCode_ptr tc_enum)
    {
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();

	CORBA::UnionMemberSeq mems(3);
	mems.length(3);
	CORBA::Any a;

	a <<= DAnyTest::red;
	mems[0].name = CORBA::string_dup("s");
	mems[0].label = a;
	mems[0].type = CORBA::_tc_short;


	a <<=  DAnyTest::green;
	mems[1].name = CORBA::string_dup("l");
	mems[1].label = a;
	mems[1].type = CORBA::_tc_long;
	


	a <<=  DAnyTest::blue;
	mems[2].name = CORBA::string_dup("f");
	mems[2].label = a;
	mems[2].type = CORBA::_tc_float;

	CORBA::TypeCode_ptr union_tc;
	
	union_tc = orb->create_union_tc(
	    "IDL:DAnyTest/TestUnion:1.0",
	    "TestUnion",
	    tc_enum, // or use DAnyTest::_tc_TestEnum
	    mems);
	
	return union_tc;
    }     


/*------------------------------------------------------------------*/
/* test variable VSDynUnion_ptr * DUnion(anonymous prop) */  

int provide_DUnion (VSDynUnion_ptr **testvar, enum prop_DUnion_prop_1 prop_1)
{
    DynamicAny::DynUnion_ptr *tvar;
   
    switch(prop_1) {
      case PDUnion_prop_1_ndf_mb:
      {
	  try {		     
	      DynamicAny::DynAny_ptr da = get_DAny(DAnyTest::_tc_TestUnion);
	      
	      DynamicAny::DynUnion_ptr du = DynamicAny::DynUnion::_narrow(da);
	      tvar = new DynamicAny::DynUnion_ptr(du);	     
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
      case PDUnion_prop_1_df_mb:
      {
	  try {	      
	      DynamicAny::DynAny_ptr da = get_DAny(DAnyTest::_tc_TUnionD);
	      
	      DynamicAny::DynUnion_ptr du = DynamicAny::DynUnion::_narrow(da);
	      tvar = new DynamicAny::DynUnion_ptr(du);	     
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
    }

    *testvar = (VSDynUnion_ptr *) tvar;
    return TRUE;
}

int relinquish_DUnion (VSDynUnion_ptr **testvar, enum prop_DUnion_prop_1 prop_1)
{
    DynamicAny::DynUnion_ptr *tvar = (DynamicAny::DynUnion_ptr *) *testvar;
    switch(prop_1) {
      case PDUnion_prop_1_ndf_mb:
      case PDUnion_prop_1_df_mb:
	     (*tvar)->destroy();
	     break;
     }
    return TRUE;
}


/*------------------------------------------------------------------*/
/* test variable VSDynAny_ptr * discriminator(anonymous prop) */

int provide_discriminator(VSDynAny_ptr **testvar, enum prop_discriminator_prop_1 prop_1)
{
    DynamicAny::DynAny_ptr *tvar;
    CORBA::TypeCode_ptr a;

    switch(prop_1) {
      case Pdiscriminator_prop_1_dEnum:
      {
	  try {
	      CORBA::TypeCode_ptr enum_tc = provide_enum_tc();
	      DynamicAny::DynAny_ptr da = get_DAny(enum_tc);
	      DynamicAny::DynEnum_var du = DynamicAny::DynEnum::_narrow(da);
	      du->set_as_string("red"); 
	      tvar = new DynamicAny::DynAny_ptr(da);
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;	   
      case Pdiscriminator_prop_1_dLong:
      {
	  try {
	      CORBA::TypeCode_ptr tc = CORBA::_tc_long;
	      DynamicAny::DynAny_ptr da = get_DAny(tc); 
	      da->insert_long(1);
	      tvar = new DynamicAny::DynAny_ptr(da);
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
      case Pdiscriminator_prop_1_invalid:
      {
	  try {
	      CORBA::TypeCode_ptr tc = CORBA::_tc_short;
	      DynamicAny::DynAny_ptr da = get_DAny(tc);
	      da->insert_short(1);
	      tvar = new DynamicAny::DynAny_ptr(da);
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break; 
    }

    *testvar = (VSDynAny_ptr *) tvar;
    return TRUE;
}

int relinquish_discriminator(VSDynAny_ptr **testvar, enum prop_discriminator_prop_1 prop_1)
{
    DynamicAny::DynAny_ptr *tvar = (DynamicAny::DynAny_ptr *) *testvar;
    switch(prop_1) {
      case Pdiscriminator_prop_1_dEnum:
      case Pdiscriminator_prop_1_dLong:
      case Pdiscriminator_prop_1_invalid:
	    (*tvar)->destroy();
	    break;
    }
    return TRUE;
}


/*------------------------------------------------------------------*/
/* test variable VSDynUnion_ptr * du(anonymous prop) */  

int provide_du (VSDynUnion_ptr **testvar, enum prop_du_prop_1 prop_1)
{
    DynamicAny::DynUnion_ptr *tvar;
   
    switch(prop_1) {
      case Pdu_prop_1_ndf_mb:
      {
	  try {	      
	      DynamicAny::DynAny_ptr da = get_DAny(DAnyTest::_tc_TestUnionA);    
	      
	      DynamicAny::DynUnion_ptr du = DynamicAny::DynUnion::_narrow(da);
	      du->set_to_no_active_member();
	      tvar = new DynamicAny::DynUnion_ptr(du);	     
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
      case Pdu_prop_1_df_mb:
      {
	  try {      
	      DynamicAny::DynAny_ptr da = get_DAny(DAnyTest::_tc_TUnionD);
	      
	      DynamicAny::DynUnion_ptr du = DynamicAny::DynUnion::_narrow(da);
	      du->set_to_default_member();
	      tvar = new DynamicAny::DynUnion_ptr(du);	     
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
    }

    *testvar = (VSDynUnion_ptr *) tvar;
    return TRUE;
}

int relinquish_du (VSDynUnion_ptr **testvar, enum prop_du_prop_1 prop_1)
{
    DynamicAny::DynUnion_ptr *tvar = (DynamicAny::DynUnion_ptr *) *testvar;
    switch(prop_1) {
      case Pdu_prop_1_ndf_mb:
      case Pdu_prop_1_df_mb:
	     (*tvar)->destroy();
	     break;
     }
    return TRUE;
}



} // end
