/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* Interface DynValue                             */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include <t1_aux.h>

//-----------------------------------------------------------

int valid_current_position(VSLong p_position)
{
    CORBA::Long position = (CORBA::Long) p_position;
    
    if (position == -1)
	return FALSE;
    else
	return TRUE;
}

int is_empty_value(VSDynValue_ptr *p_dVal)
{
    DynamicAny::DynValue_ptr *dVal = (DynamicAny::DynValue_ptr *) p_dVal;

    CORBA::ULong count = (*dVal)->component_count();
    
    if (count == 0)
	return TRUE;
    else
	return FALSE;
}

int is_correct_FieldName(VSString p_name)
{
    char *name = (char *) p_name;    
    
    if (strcmp(name, "x") == 0)
	return TRUE;
    else
	return FALSE;
}

int is_correct_TCKind(VSTCKind p_tcKind)
{
    CORBA::TCKind tcKind = (CORBA::TCKind) p_tcKind;   
   
    if (tcKind == CORBA::tk_short)
	return TRUE;
    else
	return FALSE;  
}

int valid_type_NameValuePairSeq(VSDynValue_ptr *p_dVal, VSNameValuePairSeq_ptr p_nv)
{
    DynamicAny::DynValue_ptr *dVal = (DynamicAny::DynValue_ptr *) p_dVal;
    DynamicAny::NameValuePairSeq *nv = (DynamicAny::NameValuePairSeq *) p_nv;

    if(!dVal || !nv) 
	return FALSE;    

    CORBA::ULong count = (*dVal)->component_count();
    CORBA::ULong len = nv->length();
    CORBA::ULong i = 0;
    if (count != len)
	return FALSE;

    while (i < count)
    {
	CORBA::Any a;
	a = (*nv)[i].value;
	CORBA::Any *nvA = new CORBA::Any(a);
	CORBA::TypeCode_var tc = nvA->type();
	    
	(*dVal)->seek(i);

	if (tc->kind() != (*dVal)->current_member_kind())
	    return FALSE;
	i++;
    }
    
    return TRUE;   
}


int valid_num_NameValuePairSeq(VSDynValue_ptr *p_dVal, VSNameValuePairSeq_ptr p_nv)
{
    DynamicAny::DynValue_ptr *dVal = (DynamicAny::DynValue_ptr *) p_dVal;
    DynamicAny::NameValuePairSeq *nv = (DynamicAny::NameValuePairSeq *) p_nv;

    if(!dVal || !nv) 
	return FALSE;    

    CORBA::ULong count = (*dVal)->component_count();
    CORBA::ULong len = nv->length();

    if (count == len)
	return TRUE;
    else
	return FALSE;
    
}

int is_correct_NameValuePairSeq(VSDynValue_ptr *p_dVal, VSNameValuePairSeq_ptr p_nv)
{
    DynamicAny::DynValue_ptr *dVal = (DynamicAny::DynValue_ptr *) p_dVal;
    DynamicAny::NameValuePairSeq *nv = (DynamicAny::NameValuePairSeq *) p_nv;

    if(!dVal || !nv) 
	return FALSE;    

    CORBA::ULong count = (*dVal)->component_count();
    CORBA::ULong len = nv->length();
    CORBA::ULong i = 0;
    if (count != len)
	return FALSE;
    
    while (i < count)
    {
	CORBA::Any a;
	char * NVid;
	a = (*nv)[i].value;
	CORBA::Any *nvA = new CORBA::Any(a);
	CORBA::TypeCode_var tc = nvA->type();
	NVid = (*nv)[i].id;
	    
	(*dVal)->seek(i);

	 if ((tc->kind() != (*dVal)->current_member_kind()) && strcmp(NVid, (*dVal)->current_member_name()) !=0)
	    return FALSE;
	i++;
    }
    
    return TRUE;  
}


int valid_type_NameDynAnyPairSeq(VSDynValue_ptr *p_dVal, VSNameDynAnyPairSeq_ptr p_nda)
{
    DynamicAny::DynValue_ptr *dVal = (DynamicAny::DynValue_ptr *) p_dVal;
    DynamicAny::NameDynAnyPairSeq *nda = (DynamicAny::NameDynAnyPairSeq *) p_nda;

    if(!dVal || !nda) 
	return FALSE;    

    CORBA::ULong count = (*dVal)->component_count();
    CORBA::ULong len = nda->length();
    CORBA::ULong i = 0;
    if (count != len)
	return FALSE;

    while (i < count)
    {
	DynamicAny::DynAny *da;
	da = (*nda)[i].value;	
	CORBA::TypeCode_var tc = da->type();
	    
	(*dVal)->seek(i);

	if (tc->kind() != (*dVal)->current_member_kind())
	    return FALSE;
	i++;
    }
    
    return TRUE;   
}


int valid_num_NameDynAnyPairSeq(VSDynValue_ptr *p_dVal, VSNameDynAnyPairSeq_ptr p_nda)
{
    DynamicAny::DynValue_ptr *dVal = (DynamicAny::DynValue_ptr *) p_dVal;
    DynamicAny::NameDynAnyPairSeq *nda = (DynamicAny::NameDynAnyPairSeq *) p_nda;

    if(!dVal || !nda) 
	return FALSE;    

    CORBA::ULong count = (*dVal)->component_count();
    CORBA::ULong len = nda->length();

    if (count == len)
	return TRUE;
    else
	return FALSE;
    
}

int is_correct_NameDynAnyPairSeq(VSDynValue_ptr *p_dVal, VSNameDynAnyPairSeq_ptr p_nda)
{
    DynamicAny::DynValue_ptr *dVal = (DynamicAny::DynValue_ptr *) p_dVal;
    DynamicAny::NameDynAnyPairSeq *nda = (DynamicAny::NameDynAnyPairSeq *) p_nda;

    if(!dVal || !nda) 
	return FALSE;    

    CORBA::ULong count = (*dVal)->component_count();
    CORBA::ULong len = nda->length();
    CORBA::ULong i = 0;
    if (count != len)
	return FALSE;
    
    while (i < count)
    {
	DynamicAny::DynAny *da;	
	char * NDAid;
	da = (*nda)[i].value;	
	CORBA::TypeCode_var tc = da->type();
	NDAid = (*nda)[i].id;
	    
	(*dVal)->seek(i);

	 if ((tc->kind() != (*dVal)->current_member_kind()) && strcmp(NDAid, (*dVal)->current_member_name()) !=0)
	    return FALSE;
	i++;
    }
    
    return TRUE;  
}


} // end 
