/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* interface DynAnyFactory                         */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {
#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>


/*------------------------------------------------------*/
/* resolve_initial_references() for DynAnyFactory*/

VSDynAnyFactory_ptr *VSget_DynAnyFactory(VSObjectId *id, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::ORB::ObjectId *oid = (CORBA::ORB::ObjectId *) id;
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    
    DynamicAny::DynAnyFactory_ptr factory;
    try {
	CORBA::Object_ptr optr = orb->resolve_initial_references(*oid);
	factory = DynamicAny::DynAnyFactory::_narrow(optr);
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSDynAnyFactory_ptr *) 0;
    }
    catch (...) {
	return (VSDynAnyFactory_ptr *) 0;
    }
    
    DynamicAny::DynAnyFactory_ptr *rfactory = new DynamicAny::DynAnyFactory_ptr(factory);
    return (VSDynAnyFactory_ptr *) rfactory;		
}

/*------------------------------------------------------*/
/* DynAnyFactory::create_dyn_any */

VSDynAny_ptr *VScreate_dyn_any(VSDynAnyFactory_ptr *factory, VSAny inany, VSEnvironment* ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    DynamicAny::DynAnyFactory_ptr *daf = (DynamicAny::DynAnyFactory_ptr *) factory;
    CORBA::Any *cany = (CORBA::Any *) inany;
    
    DynamicAny::DynAny_ptr da;

    try {
	da = (*daf)->create_dyn_any(*cany);
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSDynAny_ptr *) 0;
    }
    catch (...) {
	return (VSDynAny_ptr *) 0;
    }
    DynamicAny::DynAny_ptr *rany = new DynamicAny::DynAny_ptr(da);
    return (VSDynAny_ptr *) rany;
}


/*------------------------------------------------------*/
/* DynAnyFactory::create_dyn_any_from_type_code */

VSDynAny_ptr *VScreate_dyn_any_from_type_code(VSDynAnyFactory_ptr *factory, VSTypeCode_ptr *p_tc, VSEnvironment* p_ev)
{
    CORBA::Environment *env = (CORBA::Environment *) p_ev;
    DynamicAny::DynAnyFactory_ptr *daf = (DynamicAny::DynAnyFactory_ptr *) factory;
     CORBA::TypeCode_ptr *tc = (CORBA::TypeCode_ptr *) p_tc;
    
    DynamicAny::DynAny_ptr da;

    try {
	da = (*daf)->create_dyn_any_from_type_code(*tc);
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSDynAny_ptr *) 0;
    }
    catch (...) {
	return (VSDynAny_ptr *) 0;
    }
    DynamicAny::DynAny_ptr *rany = new DynamicAny::DynAny_ptr(da);
    return (VSDynAny_ptr *) rany;
}


}
