/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* Interface DynAnyFactory                         */
/***************************************************/


#include <corba_supp.hh>
#include <tserver.hh>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {
#include <t1i_tdi.h>
#include <corba_defs.h>
/*------------------------------------------------------------------*/
/* test variable VSObjectId objectid(anonymous prop) */
int provide_objectid (VSObjectId **testvar, enum prop_objectid_prop_1 prop_1)
{
    CORBA::ORB::ObjectId tvar;
    switch(prop_1){
      case Pobjectid_prop_1_dyn_factoryid:
	    tvar = CORBA::string_dup("DynAnyFactory");
	    break;
    }

    CORBA::ORB::ObjectId *tmp = new CORBA::ORB::ObjectId();
    *tmp = tvar;
    *testvar = (VSObjectId *) tmp;
    return TRUE;
}         

int relinquish_objectid (VSObjectId **testvar, enum prop_objectid_prop_1 prop_1)
{
    CORBA::ORB::ObjectId *tvar = (CORBA::ORB::ObjectId *) *testvar;
    switch(prop_1){
      case Pobjectid_prop_1_dyn_factoryid:
	    CORBA::string_free(*tvar);
	    break;
    }
    return TRUE;
}
                                         


/*------------------------------------------------------------------*/
/* test variable VSDynAnyFactory_ptr dany_factory(anonymous prop) */
int provide_dany_factory (VSDynAnyFactory_ptr **testvar, enum prop_dany_factory_prop_1 prop_1)
{
     DynamicAny::DynAnyFactory_ptr *tvar;
    switch(prop_1){
      case Pdany_factory_prop_1_valid:
      {
	    CORBA::ORB_ptr orb = glob_TS.m_corb->orb(); 
	    CORBA::Object_ptr obj = orb->resolve_initial_references("DynAnyFactory");
	    DynamicAny::DynAnyFactory_var tmp = DynamicAny::DynAnyFactory::_narrow(obj);
	    tvar = new DynamicAny::DynAnyFactory_ptr(tmp); 
      }
      break;
    }

      *testvar = (VSObject_ptr *) tvar;
      return TRUE;
}
                               
int relinquish_dany_factory (VSDynAnyFactory_ptr **testvar, enum prop_dany_factory_prop_1 prop_1)
{
    DynamicAny::DynAnyFactory_ptr *tvar = (DynamicAny::DynAnyFactory_ptr *) *testvar;
    switch(prop_1) {	
      case Pdany_factory_prop_1_valid:
	    break;
    }
    return TRUE;
}


/*------------------------------------------------------------------*/
/* test variable VSAny_ptr inany(anonymous prop) */
int provide_inany (VSAny *testvar, enum prop_inany_prop_1 prop_1)
{
    CORBA::Any *tvar = new CORBA::Any; 
   

    switch(prop_1) {
      case Pinany_prop_1_valid:
      {
	  try {	
	      CORBA::Short tmp = 2;
	      *tvar <<= tmp;
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
      case Pinany_prop_1_invalid:
      {
	  try {
	   
	      TestInterface_impl *impl = new TestInterface_impl;
	      DAnyTest::TestAbstrInterface_ptr obj = impl->_this();
	      
	      *tvar <<= &obj;
	  }	
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
    }

    *testvar = (VSAny) tvar;    
    return TRUE;
}
	    

int relinquish_inany (VSAny *testvar, enum prop_inany_prop_1 prop_1)
{
    CORBA::Any *tvar = (CORBA::Any *) testvar;
    switch(prop_1) {     
      case Pinany_prop_1_valid:
      case Pinany_prop_1_invalid: 
	    break;     
    }
    return TRUE;
}


/*------------------------------------------------------------------*/
/* test variable VSTypeCode_ptr * typecd(anonymous prop) */
int provide_typecd (VSTypeCode_ptr **testvar, enum prop_typecd_prop_1 prop_1)
{
    CORBA::TypeCode_ptr	*tvar;    
    
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb(); 
    switch(prop_1) {     
      case Ptypecd_prop_1_shorttc:
      {
	  try {
	      CORBA::TypeCode_ptr tmp = CORBA::_tc_short;
	      tvar = new CORBA::TypeCode_ptr(tmp);
	  }
	  catch (...) {
	      return FALSE;
	      
	  }
      }
      break;
      case Ptypecd_prop_1_seqtc:
      {
	  try {
	      VSTypeCode_ptr *testvar;
	      provide_typecd(&testvar, Ptypecd_prop_1_shorttc);
	      CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) testvar;
	      CORBA::TypeCode_ptr tmp = orb->create_sequence_tc(1,*tptr);
	      tvar = new CORBA::TypeCode_ptr(tmp);
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
      case Ptypecd_prop_1_tknative:
      {
	  try {
	      CORBA::TypeCode_ptr tmp = DAnyTest::_tc_TestAbstrInterface;
	      tvar = new CORBA::TypeCode_ptr(tmp);
	  }
	  catch (...) {
	      return FALSE;      
	  }
      }
      break;
    }	     
    *testvar = (VSTypeCode_ptr *) tvar;
    return TRUE;
}	

int relinquish_typecd (VSTypeCode_ptr **testvar, enum prop_typecd_prop_1 prop_1)
{
    switch(prop_1) {     
      case Ptypecd_prop_1_shorttc:
      case Ptypecd_prop_1_seqtc:
      case Ptypecd_prop_1_tknative:
	    break;
    }	
    return TRUE;
}	 


}
