//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/DII/Context/t1_aux.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1_aux.cpp,v $
// Revision 1.3  2001/01/25 12:58:28  mli
// Update sets_single_value.
//
// Revision 1.2  2001/01/10 15:53:21  mli
// Remove value nil for variable name.
//
// Revision 1.1.1.1  2000/02/09 17:13:08  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.13  1999/05/11 13:46:06  cwatson
// Added is_good_context_ptr() to the semantics.
//
// Revision 1.12  1999/05/11 11:27:43  cwatson
// Added is_nil_context_ptr() check.
//
// Revision 1.11  1999/05/10 13:19:50  cwatson
// Added invalid and nil ptr checks().
//
// Revision 1.10  1999/04/28 14:25:41  cwatson
// SR100 text revision change.
//
// Revision 1.9  1999/04/27 13:46:48  cwatson
// SR100 Added nil pointer check.
//
// Revision 1.8  1999/04/26 14:48:32  james
// Changed CORBA::Char* to char*
//
// Revision 1.7  1997/12/05 16:28:00  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.6  1997/10/09 10:52:43  tet
// Removed undeeded #define.
//
// Revision 1.5  1997/10/04 16:42:01  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.4  1997/10/03 12:20:43  ian
// Added set_one value.
//
// Revision 1.2  1997/09/12 13:00:36  cwatson
// Moved all _ptr provide and relinquish functions to corba/common/ptr_prov.cpp.
//
// Revision 1.1  1997/08/19 14:57:03  cwatson
// *** empty log message ***
//
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>
#include <corba_defs_tdd.h>

//-----------------------------------------------------------
// General Context.

int is_same_context_ptr(VSContext_ptr *o1, VSContext_ptr *o2)
{
		if(is_nil_context_ptr(o1) && is_nil_context_ptr(o2))
				return TRUE;
		
		return TRUE;
}

int cleanup_context_ptr(VSContext_ptr *o1)
{
		CORBA::Context_ptr *tc = (CORBA::Context_ptr *) o1;
		if(!tc)
				return TRUE;

		delete tc;
		return TRUE;
}

int is_invalid_identifier(VSIdentifier s)
{
	char *ident = (char *)s;

	return ident ? FALSE : TRUE;
}

int is_invalid_any(VSAny *any)
{
	CORBA::Any *pAny = (CORBA::Any *)any;

	if (!pAny)
		return TRUE;

	CORBA::TypeCode_var ptc = pAny->type();

	if (ptc->kind() != CORBA::tk_string)
		return TRUE;

	return FALSE;
}

int is_good_context_ptr(VSContext_ptr *ctx)
{
	return (!is_nil_context_ptr(ctx) && !is_invalid_context_ptr(ctx));
}

int sets_single_value(VSContext_ptr *ctx)
{
  int nReturn = TRUE;
  
  if (!ctx || is_invalid_context_ptr(ctx) || is_nil_context_ptr(ctx))
    return nReturn;
  
  CORBA::NVList_ptr nvList(0);
  
  CORBA::Context_ptr *tc = (CORBA::Context_ptr *)ctx;
  if(!*tc)
    return nReturn;
  
  try {
    (*tc)->get_values("", CORBA::CTX_RESTRICT_SCOPE, "*", nvList);
    
    CORBA::Long nCount = nvList->count();

    if (nCount != 1) // Assert only one value set
      return FALSE;
    
    CORBA::NamedValue_ptr value = nvList->item(0);
    
    switch (glob_TS.testcase(VSPROVIDE_NAME))
      {
      case Pname_v_good:
	if (::strcmp(value->name(), "ApTest"))
	  nReturn = FALSE;
	break;
      case Pname_v_empty:
	if (::strcmp(value->name(), ""))
	  nReturn = FALSE;
	break;
	
      }
    
  }
  catch (CORBA::Exception& ex) {
    
    nReturn = FALSE;
  }
  
  return nReturn;
}


}
