//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include	<corba_supp.hh>
#include	<tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/

VSContextList_ptr *VSnil(VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ContextList_ptr ttc;

		try {
				ttc = CORBA::ContextList::_nil();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSContextList_ptr *) 0;
		}
		catch (...) {
				return (VSContextList_ptr *) 0;
		}

		CORBA::ContextList_ptr *rtc = new CORBA::ContextList_ptr(ttc);
		return (VSContextList_ptr *) rtc;
}

VSContextList_ptr *VSduplicate(VSContextList_ptr *o1, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ContextList_ptr *tc = (CORBA::ContextList_ptr *) o1;
		CORBA::ContextList_ptr ttc;

		try {
				ttc = CORBA::ContextList::_duplicate(tc ? *tc : (CORBA::ContextList_ptr) 0);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSContextList_ptr *) 0;
		}
		catch (...) {
				return (VSContextList_ptr *) 0;
		}

		CORBA::ContextList_ptr *rtc = new CORBA::ContextList_ptr(ttc);
		return (VSContextList_ptr *) rtc;
}

/*------------------------------------------------------*/

VSULong VScount(VSContextList_ptr *cp, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
		CORBA::ULong ret = 0;

		if(!cptr || !*cptr ||
			is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
				return (VSULong) 0;

		try {
				ret = (*cptr)->count();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSULong) 0;
		}
		catch (...) {
				return (VSULong) 0;
		}

		return (VSULong) ret;
}

/*------------------------------------------------------*/

void VSadd(VSContextList_ptr *cp, VSString t, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
		char *str = (char *) t;

		if(!cptr || !*cptr ||
			is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
				return;

		try {
				(*cptr)->add(t);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return;
		}
		catch (...) {
				return;
		}

		return;
}

/*------------------------------------------------------*/

void VSadd_consume(VSContextList_ptr *cp, VSString t, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
		char *str = (char *) t;

		if(!cptr || !*cptr ||
			is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
				return;

		try {
			(*cptr)->add_consume(str ? CORBA::string_dup(str) : str);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return;
		}
		catch (...) {
				return;
		}

		return;
}

/*------------------------------------------------------*/

VSString VSitem(VSContextList_ptr *cp,VSULong ind, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
		CORBA::ULong i = (CORBA::ULong) ind;
		const char *str;

		if(!cptr || !*cptr ||
			is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
				return (VSString) 0;

		try {
				str = (*cptr)->item(i);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSString) 0;
		}
		catch (...) {
				return (VSString) 0;
		}

		return (VSString) str;
}

/*------------------------------------------------------*/

VSStatus VSremove(VSContextList_ptr *cp,VSULong ind, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
		CORBA::ULong i = (CORBA::ULong) ind;

		if(!cptr || !*cptr ||
			is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
				return (VSStatus) 0;

		try {
				(*cptr)->remove(i);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSStatus) 0;
		}
		catch (...) {
				return (VSStatus) 0;
		}

		return (VSStatus) 0;
}

}

