//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <corba_defs_tdd.h>
#include <t1_aux.h>

//-----------------------------------------------------------
// General ContextList.

int is_same_ctxlist_ptr(VSContextList_ptr *o1, VSContextList_ptr *o2)
{
		if(is_nil_ctxlist_ptr(o1) && is_nil_ctxlist_ptr(o2))
				return TRUE;
		
		return TRUE;
}

int cleanup_ctxlist_ptr(VSContextList_ptr *o1)
{
		CORBA::ContextList_ptr *tc = (CORBA::ContextList_ptr *) o1;
		if(!tc)
				return TRUE;

		delete tc;

		return TRUE;
}

int out_of_bounds_for_remove(VSContextList_ptr *cp, VSULong i)
{
    CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
    CORBA::ULong ind = (CORBA::ULong) i;
    CORBA::ULong cnt = 0;

	if(!cptr || is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
        return FALSE;

    try {
        cnt = (*cptr)->count();
    } catch (CORBA::Exception &) {
        // invalid ContextList
    	return FALSE;
    }
    // we are called after the remove
    return (ind+1 > cnt+1) ? TRUE : FALSE;
}

int out_of_bounds(VSContextList_ptr *cp, VSULong i)
{
	CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
	CORBA::ULong ind = (CORBA::ULong) i;
	CORBA::ULong cnt = 0;
 
	if(!cptr || is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
        return FALSE;

	try {
    	cnt = (*cptr)->count();
	}
	catch (CORBA::Exception &cex) {
        return FALSE;
	}
    return (ind+1 > cnt) ? TRUE : FALSE;
}

int has_been_removed(VSContextList_ptr *cp)
{
	CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
	CORBA::ULong cnt = 0;

	if(!cptr || is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
		return TRUE;
 
	try {
    	cnt = (*cptr)->count();
	}
	catch (CORBA::Exception &cex) {
        return TRUE;
	}
    long v = glob_TS.testcase(VSPROVIDE_CTXLISTPTR);
    switch(v){
        case Pctxlist_ptr_v_valid1:
            return (cnt == 0) ? TRUE : FALSE;
    }
 
    return FALSE;
}

unsigned long count(VSContextList_ptr *cp)
{
		CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
		CORBA::ULong cnt = 0;
 
	if(!cptr || is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
        return (unsigned long) 0;
 
	try {
    	cnt = (*cptr)->count();
	}
	catch (CORBA::Exception &cex) {
        return (unsigned long) 0;
	}
 
    return (unsigned long) cnt;
}

int item_count_is_good(VSContextList_ptr *cp)
{
    CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
    CORBA::ULong cnt = 0;

	if(!cptr || is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
		return TRUE;
 
	try {
    	cnt = (*cptr)->count();
	}
	catch (CORBA::Exception &cex) {
		return TRUE;
	}
 
    long v = glob_TS.testcase(VSPROVIDE_CTXLISTPTR);
    switch(v){
        case Pctxlist_ptr_v_valid:
            return (cnt == 0) ? TRUE : FALSE;
        case Pctxlist_ptr_v_valid1:
            return (cnt == 1) ? TRUE : FALSE;
    }
 
    return FALSE;
}

int count_is_good(VSContextList_ptr *cp)
{
    CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
    CORBA::ULong cnt = 0;

	if(!cptr || is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
		return TRUE;
 
	try {
    	cnt = (*cptr)->count();
	}
	catch (CORBA::Exception &cex) {
		return TRUE;
	}
 
    long v = glob_TS.testcase(VSPROVIDE_CTXLISTPTR);
    switch(v){
        case Pctxlist_ptr_v_valid:
            return (cnt == 1) ? TRUE : FALSE;
        case Pctxlist_ptr_v_valid1:
            return (cnt == 2) ? TRUE : FALSE;
    }
 
    return FALSE;
}

int string_is_good(VSString t)
{
		char *str = (char *) t;
		if(!str)
		    return TRUE;
}

int ctxlist_is_good(VSContextList_ptr *cp)
{
    CORBA::ContextList_ptr *cptr = (CORBA::ContextList_ptr *) cp;
    CORBA::ULong index = 0;
    const char *str;

	if(!cptr || is_invalid_ctxlist_ptr(cp) || is_nil_ctxlist_ptr(cp))
		return TRUE;

		long v = glob_TS.testcase(VSPROVIDE_CTXLISTPTR);
    switch(v){
        case Pctxlist_ptr_v_valid:
            index = 0;
            break;
        case Pctxlist_ptr_v_valid1:
            index = 1;
            break;
    }
 
    try {
        str = (*cptr)->item(index);
    }
    catch(...){
        return FALSE;
    }

		return TRUE;
}

}
