//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/DII/ExceptionList/t1.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1.cpp,v $
// Revision 1.2  2001/01/25 12:29:16  uwes
// Removed obsolete defines
//
// Revision 1.1.1.1  2000/02/09 17:13:16  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.8  1999/05/06 16:07:47  cwatson
// SR# 115 Added invalid and nil checks for calling method.
//
// Revision 1.7  1997/12/05 16:28:02  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.6  1997/11/11 13:13:54  cwatson
// Changed the exception handling by removing VSExceptionWrap.
//
// Revision 1.5  1997/10/04 16:42:07  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.4  1997/10/01 15:59:37  cwatson
// Added count(), item(), remove(), add() and add_consume() tests.
// Makefile changes.
//
// Revision 1.3  1997/09/15 10:57:17  cwatson
// Added the VSNOT_IMPLEMENTED macro and we now check for the NOT_IMPLEMENTED
// User Exception.
//
// Revision 1.2  1997/08/22 10:30:27  stephen
// Removed debugging printf calls.
//
// Revision 1.1  1997/08/19 14:57:13  cwatson
// *** empty log message ***
//
//

#include	<corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/

VSExceptionList_ptr *VSnil(VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ExceptionList_ptr ttc;

		try {
				ttc = CORBA::ExceptionList::_nil();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSExceptionList_ptr *) 0;
		}
		catch (...) {
				return (VSExceptionList_ptr *) 0;
		}

		CORBA::ExceptionList_ptr *rtc = new CORBA::ExceptionList_ptr(ttc);
		return (VSExceptionList_ptr *) rtc;
}

VSExceptionList_ptr *VSduplicate(VSExceptionList_ptr *o1, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ExceptionList_ptr *tc = (CORBA::ExceptionList_ptr *) o1;
		CORBA::ExceptionList_ptr ttc;

		try {
				ttc = CORBA::ExceptionList::_duplicate(tc ? *tc : (CORBA::ExceptionList_ptr) 0);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSExceptionList_ptr *) 0;
		}
		catch (...) {
				return (VSExceptionList_ptr *) 0;
		}

		CORBA::ExceptionList_ptr *rtc = new CORBA::ExceptionList_ptr(ttc);
		return (VSExceptionList_ptr *) rtc;
}


/*------------------------------------------------------*/

VSULong VScount(VSExceptionList_ptr *e, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
		CORBA::ULong ret = 0;

		if(!ex || !*ex ||
			is_invalid_exlist_ptr(e) || is_nil_exlist_ptr(e))
				return ret;

		try {
				ret = (*ex)->count();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSULong) 0;
		}
		catch (...) {
				return (VSULong) 0;
		}

		return (VSULong) ret;
}


/*------------------------------------------------------*/

void VSadd(VSExceptionList_ptr *e, VSTypeCode_ptr *t, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
		CORBA::TypeCode_ptr *tc = (CORBA::TypeCode_ptr *) t;

		if(!ex || !*ex ||
			is_invalid_exlist_ptr(e) || is_nil_exlist_ptr(e))
				return;

		try {
				(*ex)->add(tc ? *tc : (CORBA::TypeCode_ptr) 0);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return;
		}
		catch (...) {
				return;
		}

		return;
}


/*------------------------------------------------------*/

void VSadd_consume(VSExceptionList_ptr *e, VSTypeCode_ptr *t, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
		CORBA::TypeCode_ptr *tc = (CORBA::TypeCode_ptr *) t;

		if(!ex || !*ex ||
			is_invalid_exlist_ptr(e) || is_nil_exlist_ptr(e))
				return;

		try {
				(*ex)->add_consume(tc ? *tc : (CORBA::TypeCode_ptr) 0);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return;
		}
		catch (...) {
				return;
		}

		return;
}


/*------------------------------------------------------*/

VSTypeCode_ptr *VSitem(VSExceptionList_ptr *e,VSULong ind, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
		CORBA::ULong i = (CORBA::ULong) ind;
		CORBA::TypeCode_ptr ttc;

		if(!ex || !*ex ||
			is_invalid_exlist_ptr(e) || is_nil_exlist_ptr(e))
				return (VSTypeCode_ptr *) 0;

		try {
				ttc = (*ex)->item(i);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSTypeCode_ptr *) 0;
		}
		catch (...) {
				return (VSTypeCode_ptr *) 0;
		}

		CORBA::TypeCode_ptr *rtc = new CORBA::TypeCode_ptr(ttc);
		return (VSTypeCode_ptr *) rtc;
}


/*------------------------------------------------------*/

VSStatus VSremove(VSExceptionList_ptr *e,VSULong ind, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
		CORBA::ULong i = (CORBA::ULong) ind;

		if(!ex || !*ex ||
			is_invalid_exlist_ptr(e) || is_nil_exlist_ptr(e))
				return (VSStatus) 0;

		try {
				(*ex)->remove(i);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSStatus) 0;
		}
		catch (...) {
				return (VSStatus) 0;
		}

		return (VSStatus) 0;
}

}

