//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <corba_defs_tdd.h>
#include <t1_aux.h>

//-----------------------------------------------------------
// General ExceptionList.

int is_same_exlist_ptr(VSExceptionList_ptr *o1, VSExceptionList_ptr *o2)
{
		if(is_nil_exlist_ptr(o1) && is_nil_exlist_ptr(o2))
				return TRUE;
		
		return TRUE;
}

int cleanup_exlist_ptr(VSExceptionList_ptr *o1)
{
		CORBA::ExceptionList_ptr *tc = (CORBA::ExceptionList_ptr *) o1;
		if(!tc)
				return TRUE;

		delete tc;

		return TRUE;
}

int out_of_bounds_for_remove(VSExceptionList_ptr *e, VSULong i)
{
    CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
    CORBA::ULong ind = (CORBA::ULong) i;
    CORBA::ULong cnt = 0;

    if(!ex || is_nil_exlist_ptr(e) || is_invalid_exlist_ptr(e))
        return FALSE;

    try {
        cnt = (*ex)->count();
    } catch (CORBA::Exception &) {
      // we were passed an invalid ExceptionList
      return FALSE;
    }
    // we are called after the remove, so count()!
    return (ind+1 > cnt+1) ? TRUE : FALSE;
}

int out_of_bounds(VSExceptionList_ptr *e, VSULong i)
{
		CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
 		CORBA::ULong ind = (CORBA::ULong) i;
    CORBA::ULong cnt = 0;
 
    if(!ex || is_nil_exlist_ptr(e) || is_invalid_exlist_ptr(e))
        return FALSE;
 
	try {
    	cnt = (*ex)->count();
	}
	catch (CORBA::Exception &cex) {
        return FALSE;
	}
    return (ind+1 > cnt) ? TRUE : FALSE;
}

int has_been_removed(VSExceptionList_ptr *e)
{
		CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
		CORBA::ULong cnt = 0;
 
    if(!ex || is_nil_exlist_ptr(e) || is_invalid_exlist_ptr(e))
        return TRUE;
 
	try {
    	cnt = (*ex)->count();
	}
	catch (CORBA::Exception &cex) {
        return FALSE;
	}

    long v = glob_TS.testcase(VSPROVIDE_EXLISTPTR);
    switch(v){
        case Pexlist_ptr_v_valid1:
            return (cnt == 0) ? TRUE : FALSE;
    }
 
    return FALSE;
}

unsigned long count(VSExceptionList_ptr *e)
{
		CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
		CORBA::ULong cnt = 0;
 
    if(!ex || is_nil_exlist_ptr(e) || is_invalid_exlist_ptr(e))
        return (unsigned long) 0;
 
	try {
    	cnt = (*ex)->count();
	}
	catch (CORBA::Exception &cex) {
        return (unsigned long) 0;
	}
 
    return (unsigned long) cnt;
}

int item_count_is_good(VSExceptionList_ptr *e)
{
		CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
		CORBA::ULong cnt = 0;
 
    if(!ex || is_nil_exlist_ptr(e) || is_invalid_exlist_ptr(e))
        return TRUE;
 
	try {
    	cnt = (*ex)->count();
	}
	catch (CORBA::Exception &cex) {
        return TRUE;
	}
 
    long v = glob_TS.testcase(VSPROVIDE_EXLISTPTR);
    switch(v){
        case Pexlist_ptr_v_valid:
            return (cnt == 0) ? TRUE : FALSE;
        case Pexlist_ptr_v_valid1:
            return (cnt == 1) ? TRUE : FALSE;
    }
 
    return FALSE;
}

int count_is_good(VSExceptionList_ptr *e)
{
		CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
		CORBA::ULong cnt = 0;
 
    if(!ex || is_nil_exlist_ptr(e) || is_invalid_exlist_ptr(e))
        return TRUE;
 
	try {
    	cnt = (*ex)->count();
	}
	catch (CORBA::Exception &cex) {
        return TRUE;
	}
 
    long v = glob_TS.testcase(VSPROVIDE_EXLISTPTR);
    switch(v){
        case Pexlist_ptr_v_valid:
            return (cnt == 1) ? TRUE : FALSE;
        case Pexlist_ptr_v_valid1:
            return (cnt == 2) ? TRUE : FALSE;
    }
 
    return FALSE;
}

int typecode_is_good(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tc = (CORBA::TypeCode_ptr *) t;
    	if(!tc || is_nil_typecode_ptr(t) || is_invalid_typecode_ptr(t))
				return TRUE;

		return ((*tc)->kind() == CORBA::tk_short) ? TRUE : FALSE;
}

int exlist_is_good(VSExceptionList_ptr *e)
{
		CORBA::ExceptionList_ptr *ex = (CORBA::ExceptionList_ptr *) e;
		CORBA::TypeCode_ptr tc;
		CORBA::ULong index = 0;

    if(!ex || is_nil_exlist_ptr(e) || is_invalid_exlist_ptr(e))
        return TRUE;
 
    long v = glob_TS.testcase(VSPROVIDE_EXLISTPTR);
    switch(v){
        case Pexlist_ptr_v_valid:
						index = 0;
						break;
        case Pexlist_ptr_v_valid1:
						index = 1;
						break;
    }

		try {
				tc = (*ex)->item(index);
		}
		catch(...){
				return FALSE;
		}
		CORBA::TCKind r = tc->kind();

		return TRUE;
}

}

