//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include	<corba_supp.hh>
#include	<tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/

VSNVList_ptr *VSnil(VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::NVList_ptr ttc;

		try {
				ttc = CORBA::NVList::_nil();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSNVList_ptr *) 0;
		}
		catch (...) {
				return (VSNVList_ptr *) 0;
		}

		CORBA::NVList_ptr *rtc = new CORBA::NVList_ptr(ttc);
		return (VSNVList_ptr *) rtc;
}

VSNVList_ptr *VSduplicate(VSNVList_ptr *o1, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::NVList_ptr *tc = (CORBA::NVList_ptr *) o1;
		CORBA::NVList_ptr ttc;

		try {
				ttc = CORBA::NVList::_duplicate(tc ? *tc : (CORBA::NVList_ptr) 0);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSNVList_ptr *) 0;
		}
		catch (...) {
				return (VSNVList_ptr *) 0;
		}

		CORBA::NVList_ptr *rtc = new CORBA::NVList_ptr(ttc);
		return (VSNVList_ptr *) rtc;
}


/*------------------------------------------------------*/

VSULong VScount(VSNVList_ptr *o1, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::NVList_ptr *tc = (CORBA::NVList_ptr *) o1;
		CORBA::ULong ret;

		if(!tc || !*tc ||
			is_invalid_nvlist_ptr(o1) || is_nil_nvlist_ptr(o1))
				return (VSULong) 0;

		try {
				ret = (*tc)->count();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSULong) 0;
		}
		catch (...) {
				return (VSULong) 0;
		}

		return (VSULong) ret;
}

/*------------------------------------------------------*/

VSNamedValue_ptr *VSadd(VSNVList_ptr *o1, VSFlags *f, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::NVList_ptr *tc = (CORBA::NVList_ptr *) o1;
		CORBA::Flags *flags = (CORBA::Flags *) f;
		CORBA::NamedValue_ptr nptr;

		if(!tc || !*tc ||
			is_invalid_nvlist_ptr(o1) || is_nil_nvlist_ptr(o1))
				return (VSNamedValue_ptr *) 0;

		try {
				nptr = (*tc)->add(*flags);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSNamedValue_ptr *) 0;
		}
		catch (...) {
				return (VSNamedValue_ptr *) 0;
		}

		CORBA::NamedValue_ptr *rptr = new CORBA::NamedValue_ptr(nptr);
		return (VSNamedValue_ptr *) rptr;
}

/*------------------------------------------------------*/

VSNamedValue_ptr *VSadd_item(VSNVList_ptr *o1, VSString s, VSFlags *f, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::NVList_ptr *tc = (CORBA::NVList_ptr *) o1;
		char *name = (char *) s;
		CORBA::Flags *flags = (CORBA::Flags *) f;
		CORBA::NamedValue_ptr nptr;

		if(!tc || !*tc ||
			is_invalid_nvlist_ptr(o1) || is_nil_nvlist_ptr(o1))
				return (VSNamedValue_ptr *) 0;

		try {
				nptr = (*tc)->add_item(name,*flags);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSNamedValue_ptr *) 0;
		}
		catch (...) {
				return (VSNamedValue_ptr *) 0;
		}

		CORBA::NamedValue_ptr *rptr = new CORBA::NamedValue_ptr(nptr);
		return (VSNamedValue_ptr *) rptr;
}

/*------------------------------------------------------*/

VSNamedValue_ptr *VSadd_item_consume(VSNVList_ptr *o1, VSString s, VSFlags *f, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::NVList_ptr *tc = (CORBA::NVList_ptr *) o1;
		char *name = (char *) s;
		CORBA::Flags *flags = (CORBA::Flags *) f;
		CORBA::NamedValue_ptr nptr;

		if(!tc || !*tc ||
			is_invalid_nvlist_ptr(o1) || is_nil_nvlist_ptr(o1))
				return (VSNamedValue_ptr *) 0;

		try {
				nptr = (*tc)->add_item_consume(name,*flags);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSNamedValue_ptr *) 0;
		}
		catch (...) {
				return (VSNamedValue_ptr *) 0;
		}

		CORBA::NamedValue_ptr *rptr = new CORBA::NamedValue_ptr(nptr);
		return (VSNamedValue_ptr *) rptr;
}

/*------------------------------------------------------*/

VSNamedValue_ptr *VSitem(VSNVList_ptr *o1, VSULong i, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::NVList_ptr *tc = (CORBA::NVList_ptr *) o1;
		CORBA::ULong ind = (CORBA::ULong) i;
		CORBA::NamedValue_ptr nptr;

		if(!tc || !*tc ||
			is_invalid_nvlist_ptr(o1) || is_nil_nvlist_ptr(o1))
				return (VSNamedValue_ptr *) 0;

		try {
				nptr = (*tc)->item(ind);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSNamedValue_ptr *) 0;
		}
		catch (...) {
				return (VSNamedValue_ptr *) 0;
		}

		CORBA::NamedValue_ptr *rptr = new CORBA::NamedValue_ptr(nptr);
		return (VSNamedValue_ptr *) rptr;
}

/*------------------------------------------------------*/

VSStatus VSremove(VSNVList_ptr *o1, VSULong i, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::NVList_ptr *tc = (CORBA::NVList_ptr *) o1;
		CORBA::ULong ind = (CORBA::ULong) i;

		if(!tc || !*tc ||
			is_invalid_nvlist_ptr(o1) || is_nil_nvlist_ptr(o1))
				return (VSStatus) 0;

		try {
				(*tc)->remove(ind);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSStatus) 0;
		}
		catch (...) {
				return (VSStatus) 0;
		}

		return (VSStatus) 0;
}

}

