//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/DII/NVList/t1_aux.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1_aux.cpp,v $
// Revision 1.2  2001/01/11 13:25:26  mli
// Remove value nul from variable lname.
//
// Revision 1.1.1.1  2000/02/09 17:13:21  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.11  1999/05/10 13:19:55  cwatson
// Added invalid and nil ptr checks().
//
// Revision 1.10  1999/04/30 09:59:17  cwatson
// SR100 Added out_of_bounds_for_remove() to check bounds after a remove.
//
// Revision 1.9  1999/04/28 14:24:19  cwatson
// SR100 text revision change.
//
// Revision 1.8  1999/04/27 12:43:48  cwatson
// SR100 Added try catch around all count() calls.
//
// Revision 1.7  1999/04/26 14:48:42  james
// Changed CORBA::Char* to char*
//
// Revision 1.6  1997/12/05 16:28:04  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.5  1997/10/09 10:52:46  tet
// Removed undeeded #define.
//
// Revision 1.4  1997/10/04 16:42:10  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.3  1997/10/01 16:01:01  cwatson
// Added count(), item(), remove(), add(), add_item() and add_item_consume() tests.
// Makefile changes.
//
// Revision 1.2  1997/09/12 13:01:17  cwatson
// Moved all _ptr provide and relinquish functions to corba/common/ptr_prov.cpp.
//
// Revision 1.1  1997/08/19 14:53:55  cwatson
// *** empty log message ***
//
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include	<corba_defs_tdd.h>
#include	<t1i_tdi.h>
#include	<t1_aux.h>

//-----------------------------------------------------------
// General NVList.

int is_same_nvlist_ptr(VSNVList_ptr *o1, VSNVList_ptr *o2)
{
		if(is_nil_nvlist_ptr(o1) && is_nil_nvlist_ptr(o2))
				return TRUE;
		
		return TRUE;
}

int cleanup_nvlist_ptr(VSNVList_ptr *o1)
{
		CORBA::NVList_ptr *tc = (CORBA::NVList_ptr *) o1;
		if(!tc)
				return TRUE;

		delete tc;
		return TRUE;
}

int out_of_bounds_for_remove(VSNVList_ptr *o1, VSULong i)
{
	CORBA::NVList_ptr *nv = (CORBA::NVList_ptr *) o1;
	CORBA::ULong ind = (CORBA::ULong) i;
	CORBA::ULong cnt = 0;

	if(!nv || is_nil_nvlist_ptr(o1) || is_invalid_nvlist_ptr(o1))
		return FALSE;

	try {
	    cnt = (*nv)->count();
	} catch (CORBA::Exception &) {
	    // invalid NVList
	    return FALSE;
	}

	// we are called after the remove
	return (ind+1 > cnt+1) ? TRUE : FALSE;
}

int out_of_bounds(VSNVList_ptr *o1, VSULong i)
{
		CORBA::NVList_ptr *nv = (CORBA::NVList_ptr *) o1;
		CORBA::ULong ind = (CORBA::ULong) i;
		CORBA::ULong cnt = 0;

		if(!nv || is_nil_nvlist_ptr(o1) || is_invalid_nvlist_ptr(o1))
			return FALSE;

		try {
			cnt = (*nv)->count();
		}
		catch (CORBA::Exception &cex) {
			return FALSE;
		}

		return (ind+1 > cnt) ? TRUE : FALSE;
}

int has_been_removed(VSNVList_ptr *o1)
{
		CORBA::NVList_ptr *nv = (CORBA::NVList_ptr *) o1;
		CORBA::ULong cnt = 0;

		if(!nv || is_nil_nvlist_ptr(o1) || is_invalid_nvlist_ptr(o1))
				return TRUE;

		try {
			cnt = (*nv)->count();
		}
		catch (CORBA::Exception &cex) {
			return TRUE;
		}
		long v = glob_TS.testcase(VSPROVIDE_NVLISTPTR);
		switch(v){
				case Pnvlist_ptr_v_valid1:
						return (cnt == 0) ? TRUE : FALSE;
		}

		return FALSE;
}

unsigned long count(VSNVList_ptr *o1)
{
		CORBA::NVList_ptr *nv = (CORBA::NVList_ptr *) o1;
		CORBA::ULong cnt = 0;

		if(!nv || is_nil_nvlist_ptr(o1) || is_invalid_nvlist_ptr(o1))
			return (unsigned long) 0;

		try {
			cnt = (*nv)->count();
		}
		catch (CORBA::Exception &cex) {
			return (unsigned long) 0;
		}

		return (unsigned long) cnt;
}

int item_count_is_good(VSNVList_ptr *o1)
{
		CORBA::NVList_ptr *nv = (CORBA::NVList_ptr *) o1;
		CORBA::ULong cnt = 0;

		if(!nv || is_nil_nvlist_ptr(o1) || is_invalid_nvlist_ptr(o1))
			return TRUE;

		try {
			cnt = (*nv)->count();
		}
		catch (CORBA::Exception &cex) {
			return TRUE;
		}

		long v = glob_TS.testcase(VSPROVIDE_NVLISTPTR);
		switch(v){
				case Pnvlist_ptr_v_valid:
						return (cnt == 0) ? TRUE : FALSE;
				case Pnvlist_ptr_v_valid1:
						return (cnt == 1) ? TRUE : FALSE;
		}

		return FALSE;
}

int count_is_good(VSNVList_ptr *o1)
{
		CORBA::NVList_ptr *nv = (CORBA::NVList_ptr *) o1;
		CORBA::ULong cnt = 0;

		if(!nv || is_nil_nvlist_ptr(o1) || is_invalid_nvlist_ptr(o1))
			return TRUE;

		try {
			cnt = (*nv)->count();
		}
		catch (CORBA::Exception &cex) {
			return TRUE;
		}

		long v = glob_TS.testcase(VSPROVIDE_NVLISTPTR);
		switch(v){
				case Pnvlist_ptr_v_valid:
						return (cnt == 1) ? TRUE : FALSE;
				case Pnvlist_ptr_v_valid1:
						return (cnt == 2) ? TRUE : FALSE;
		}

		return FALSE;
}

int name_is_good(VSNamedValue_ptr *o1)
{
		CORBA::NamedValue_ptr *nv = (CORBA::NamedValue_ptr *) o1;

		if(!nv || is_nil_nvlist_ptr(o1) || is_invalid_nvlist_ptr(o1))
				return TRUE;
		
		const char *n = (*nv)->name();

		long v = glob_TS.testcase(VSPROVIDE_NVLNAME);
		switch(v){
				case Plname_v_valid:
						return (strcmp(n,"ApTest") == 0) ? TRUE : FALSE;
				case Plname_v_nil:
						return (strcmp(n,"") == 0) ? TRUE : FALSE;
		}

		return FALSE;
}

int flags_are_good(VSNamedValue_ptr *o1)
{
		CORBA::NamedValue_ptr *nv = (CORBA::NamedValue_ptr *) o1;

		if(!nv || is_nil_nvlist_ptr(o1) || is_invalid_nvlist_ptr(o1))
				return TRUE;

		CORBA::Flags f = (*nv)->flags();

		long v = glob_TS.testcase(VSPROVIDE_NVLFLAGS);
		switch(v){
				case Pflags_ptr_v_in:
						return (f == CORBA::ARG_IN) ? TRUE : FALSE;
				case Pflags_ptr_v_out:
						return (f == CORBA::ARG_OUT) ? TRUE : FALSE;
				case Pflags_ptr_v_inout:
						return (f == CORBA::ARG_INOUT) ? TRUE : FALSE;
		}

		return FALSE;
}

int nvalue_is_good(VSNamedValue_ptr *o1)
{
		CORBA::NamedValue_ptr *nv = (CORBA::NamedValue_ptr *) o1;

		if(!nv || is_nil_nvlist_ptr(o1) || is_invalid_nvlist_ptr(o1))
				return TRUE;

		CORBA::Flags f = (*nv)->flags();
		const char *n = (*nv)->name();
		if(f != CORBA::ARG_IN)
				return FALSE;
		if(!n)
				return FALSE;

		return (strcmp(n,"ApTest") == 0) ? TRUE : FALSE;
}

}
