//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/DII/ORB/t1.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1.cpp,v $
// Revision 1.3  2001/01/25 17:29:35  mli
// Update.
//
// Revision 1.1.1.1  2000/02/09 17:13:28  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.9  1997/12/05 16:28:07  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.8  1997/11/11 13:13:55  cwatson
// Changed the exception handling by removing VSExceptionWrap.
//
// Revision 1.7  1997/10/04 16:42:13  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.6  1997/10/01 16:03:02  cwatson
// Makefile changes.
//
// Revision 1.5  1997/09/15 10:57:33  cwatson
// Added the VSNOT_IMPLEMENTED macro and we now check for the NOT_IMPLEMENTED
// User Exception.
//
// Revision 1.4  1997/09/12 13:01:39  cwatson
// Moved all _ptr provide and relinquish functions to corba/common/ptr_prov.cpp.
//
// Revision 1.3  1997/08/22 11:08:44  cwatson
// Changed VSEnvironment_ptr to VSEnvironment *.
//
// Revision 1.2  1997/08/22 10:30:35  stephen
// Removed debugging printf calls.
//
// Revision 1.1  1997/08/06 14:36:34  cwatson
// *** empty log message ***
//
//

#include	<corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>


/*------------------------------------------------------*/

VSStatus VScreate_list(VSLong count, VSNVList_ptr *newlist,
		       VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
  CORBA::NVList_ptr *nvl = (CORBA::NVList_ptr *) newlist;
  try {
    orb->create_list(count,*nvl);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
    return (VSStatus) 0;
  }
  catch (...) {
    return (VSStatus) 0;
  }
  
  newlist = (VSNVList_ptr *) &nvl;
  return (VSStatus) 0;
}

/*------------------------------------------------------*/

VSStatus VScreate_operation_list(VSOperationDef_ptr *o,VSNVList_ptr *newlist,
				 VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
  CORBA::NVList_ptr *nvl = (CORBA::NVList_ptr *) newlist;
  CORBA::OperationDef_ptr *optr = (CORBA::OperationDef_ptr *) o;
  
  try {
    orb->create_operation_list(*optr, *nvl);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
    return (VSStatus) 0;
  }
  catch (...) {
    return (VSStatus) 0;
  }
  newlist = (VSNVList_ptr *) &nvl;
  return (VSStatus) 0;
}

/*------------------------------------------------------*/

VSStatus VScreate_named_value(VSNamedValue_ptr *nv,
															VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
#ifndef VSORB_NO_ORB_CREATE_NAMED_VALUE
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::NamedValue_ptr *nptr = (CORBA::NamedValue_ptr *) nv;

		try {
				orb->create_named_value(*nptr);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSStatus) 0;
		}
		catch (...) {
				return (VSStatus) 0;
		}

		*nv = (VSNamedValue_ptr) *nptr;
#else
		VSNOT_IMPLEMENTED;
#endif
		return (VSStatus) 0;
}

/*------------------------------------------------------*/

VSStatus VScreate_exception_list(VSExceptionList_ptr *el,
															VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
#ifndef VSORB_NO_ORB_CREATE_EXCEPTION_LIST
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::ExceptionList_ptr eptr = (CORBA::ExceptionList_ptr) *el;

		try {
				orb->create_exception_list(eptr);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSStatus) 0;
		}
		catch (...) {
				return (VSStatus) 0;
		}

		*el = (VSExceptionList_ptr) eptr;
#else
		VSNOT_IMPLEMENTED;
#endif
		return (VSStatus) 0;
}

/*------------------------------------------------------*/

VSStatus VScreate_context_list(VSContextList_ptr *cl,
															VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
#ifndef VSORB_NO_ORB_CREATE_CONTEXT_LIST
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::ContextList_ptr *cltmp = (CORBA::ContextList_ptr *) cl;
		CORBA::ContextList_ptr clist = cltmp ? *cltmp : NULL;

		try {
				orb->create_context_list(clist);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSStatus) 0;
		}
		catch (...) {
				return (VSStatus) 0;
		}

		*cl = (VSContextList_ptr) clist;
#else
		VSNOT_IMPLEMENTED;
#endif
		return (VSStatus) 0;
}

VSStatus VSget_default_context(VSContext_ptr *ct,
															VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
#ifndef VSORB_NO_ORB_GET_DEFAULT_CONTEXT
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::Context_ptr ctx = (CORBA::Context_ptr) *ct;

		try {
				orb->get_default_context(ctx);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSStatus) 0;
		}
		catch (...) {
				return (VSStatus) 0;
		}

		*ct = (VSContext_ptr) ctx;
#else
		VSNOT_IMPLEMENTED;
#endif
		return (VSStatus) 0;
}

}

