//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/DII/ORB/t1_aux.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1_aux.cpp,v $
// Revision 1.3  2001/01/25 17:30:49  mli
// Update semantics def. and variable impl. for
// create_list and create_operation_list.
//
// Revision 1.1.1.1  2000/02/09 17:13:29  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.9  1999/05/10 13:17:23  cwatson
// Added nil ptr check and do not check contents of nvlist.
//
// Revision 1.8  1999/05/06 12:01:37  ian
// SR#129 Changed NamedValue checking in is_initial_NamedValue()
//
// Revision 1.7  1999/04/26 14:48:43  james
// Changed CORBA::Char* to char*
//
// Revision 1.6  1997/12/05 16:28:07  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.5  1997/10/09 10:52:48  tet
// Removed undeeded #define.
//
// Revision 1.4  1997/10/04 16:42:13  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.3  1997/10/01 16:03:03  cwatson
// Makefile changes.
//
// Revision 1.2  1997/09/12 13:01:42  cwatson
// Moved all _ptr provide and relinquish functions to corba/common/ptr_prov.cpp.
//
// Revision 1.1  1997/08/06 14:36:36  cwatson
// *** empty log message ***
//
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>

int is_initial_NVList(VSLong count, VSNVList_ptr *nl)
{
  CORBA::NVList_ptr *nvtmp = (CORBA::NVList_ptr *) nl;
  CORBA::NVList_ptr nvl = *nvtmp;
  CORBA::NamedValue_ptr nvptr;
  
  if(!nvl || is_nil_nvlist_ptr(nl))
    return FALSE;

  try {
    for(long i=0; i<count; i++){
      
      nvptr = nvl -> add(CORBA::ARG_IN);
    }
    if(count == nvl->count())
      return TRUE;
  }
  catch(CORBA::SystemException &ex){
    return FALSE;
  }
  return FALSE;
}

int is_initial_opList(VSOperationDef_ptr *o, VSNVList_ptr *nl)
{
  CORBA::NVList_ptr *nvtmp = (CORBA::NVList_ptr *) nl;
  CORBA::NVList_ptr nvl = *nvtmp;
  CORBA::OperationDef_ptr *optr = (CORBA::OperationDef_ptr *) o;
  CORBA::OperationDef_ptr op = *optr;
  try{
    if(nvl->count() == op->params()->length())
      return TRUE;
  }catch(CORBA::Exception &ex) {
    return FALSE;
  }

  return FALSE;
}

int is_initial_NamedValue(VSNamedValue_ptr *pnv)
{
		CORBA::NamedValue_ptr *nvtmp = (CORBA::NamedValue_ptr *) pnv;
		CORBA::NamedValue_ptr nv = *nvtmp;

/*
		if(!nv)
				return FALSE;
*/

		// Spec. says nothing about what NV elements should be initialised to,
		// so we just check that each member fucntion can be safely called.
		try
		{
			nv->name();
			nv->value();
			nv->flags();
		}
		catch (...)
		{
			return FALSE;
		}

		return TRUE;
}

int is_initial_ExList(VSExceptionList_ptr *pel)
{
#ifndef VSORB_NO_EXCEPTIONLIST
		CORBA::ExceptionList_ptr *eltmp = (CORBA::ExceptionList_ptr *) pel;
		CORBA::ExceptionList_ptr el = *eltmp;

		if(!el)
				return FALSE;

		CORBA::ULong cnt = el->count();
		if(cnt != 0)
				return FALSE;

		CORBA::TypeCode_ptr it = 0;
		CORBA::ULong ind = 0;
		try{
				it = el->item(ind);
		}
		catch(CORBA::Exception &ex){
				return TRUE;
		}
		if(it != 0)
				return FALSE;

		return TRUE;
#else
		return FALSE;
#endif
}

int is_initial_ContextList(VSContextList_ptr *pcl)
{
#ifndef VSORB_NO_CONTEXTLIST
		CORBA::ContextList_ptr *cltmp = (CORBA::ContextList_ptr *) pcl;
		CORBA::ContextList_ptr cl = *cltmp;

		if(!cl)
				return FALSE;

		CORBA::ULong cnt = cl->count();
		if(cnt != 0)
				return FALSE;

		const char *it = 0;
		CORBA::ULong ind = 0;
		try{
				it = cl->item(ind);
		}
		catch(CORBA::Exception &ex){
				return TRUE;
		}
		if(it != 0)
				return FALSE;

		return TRUE;
#else
		return FALSE;
#endif
}

int is_default_Context(VSContext_ptr *cl)
{
		CORBA::Context_ptr *ctxtmp = (CORBA::Context_ptr *) cl;
		CORBA::Context_ptr ctx = *ctxtmp;
		if(!ctx)
				return FALSE;

		const char *nm = 0;
#ifndef VSORB_NO_CONTEXT_CONTEXT_NAME
		try {
				nm = ctx->context_name();
		}
		catch(CORBA::SystemException &ex){
				return FALSE;
		}
#else
		return FALSE;
#endif

		return TRUE;
}

}
