//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/DII/ORB/t1i_tdi.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1i_tdi.cpp,v $
// Revision 1.3  2001/01/25 17:30:49  mli
// Update semantics def. and variable impl. for
// create_list and create_operation_list.
//
// Revision 1.1.1.1  2000/02/09 17:13:29  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.7  1997/12/05 16:28:08  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.6  1997/11/11 13:13:56  cwatson
// Changed the exception handling by removing VSExceptionWrap.
//
// Revision 1.5  1997/10/04 16:42:14  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.4  1997/10/01 16:03:03  cwatson
// Makefile changes.
//
// Revision 1.3  1997/09/19 15:48:48  ian
// Windows NT version.
//
// Revision 1.2  1997/09/12 13:01:44  cwatson
// Moved all _ptr provide and relinquish functions to corba/common/ptr_prov.cpp.
//
// Revision 1.1  1997/08/06 14:36:38  cwatson
// *** empty log message ***
//
//
/*
 * ./t1_tdi.c
 */

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

/* ---- global property declarations ---- */
/* ---- test variables and local properties ---- */

int provide_operationDef_ptr(VSOperationDef_ptr **testvar,
			     enum prop_operationDef_ptr_v v)
{
  CORBA::OperationDef_ptr *tvar;
  
  switch(v){
  case PoperationDef_ptr_v_valid:
    {
      IRSupport *irs = glob_TS.repository();
      CORBA::Repository_var IR_ptr = irs->get_repository();
      CORBA::Contained_var ctn = irs->get_contained(IR_ptr, "M_OperationDef::I1");
      CORBA::InterfaceDef_var intf = CORBA::InterfaceDef::_narrow(ctn);
      assert(!CORBA::is_nil(intf)); 
      CORBA::Contained_var ct = intf->lookup("op_params");         
      assert(!CORBA::is_nil(ct));         
      CORBA::OperationDef_ptr op = CORBA::OperationDef::_narrow(ct); 
      assert(!CORBA::is_nil(op));
     
      tvar = new CORBA::OperationDef_ptr(op);
    }
    break;
  }
  
  *testvar = (VSOperationDef_ptr *) tvar;
  return TRUE;
}
  
  int relinquish_operationDef_ptr(VSOperationDef_ptr **testvar,
              enum prop_operationDef_ptr_v v)
{
    return FALSE;
}

/* ------------------------------------------------------------ */
/* test variable onvalue_ptr() */
 
int provide_onvalue_ptr(VSNamedValue_ptr **testvar,
              enum prop_onvalue_ptr_v v)
{
    CORBA::NamedValue_ptr *tvar;
    CORBA::NamedValue_var tmp;
 
    switch(v){
        case Ponvalue_ptr_v_nil:
            tvar = new CORBA::NamedValue_ptr(tmp);
            break;
    }
 
    *testvar = (VSNamedValue_ptr *) tvar;
    return TRUE;
}

int relinquish_onvalue_ptr(VSNamedValue_ptr **testvar,
              enum prop_onvalue_ptr_v v)
{
    CORBA::NamedValue_ptr  *tvar = (CORBA::NamedValue_ptr *) *testvar;
 
    switch(v){
        case Ponvalue_ptr_v_nil:
						CORBA::release(*tvar);
            break;
    }
    return TRUE;
}

/* ------------------------------------------------------------ */
/* test variable onvlist_ptr() */
 
int provide_onvlist_ptr(VSNVList_ptr **testvar,
              enum prop_onvlist_ptr_v v)
{
    CORBA::NVList_ptr *tvar;
    CORBA::NVList_var tmp;
 
    switch(v){
        case Ponvlist_ptr_v_nil:
            tvar = new CORBA::NVList_ptr(tmp);
            break;
    }
 
    *testvar = (VSNVList_ptr *) tvar;
    return TRUE;
}

int relinquish_onvlist_ptr(VSNVList_ptr **testvar,
              enum prop_onvlist_ptr_v v)
{
    CORBA::NVList_ptr  *tvar = (CORBA::NVList_ptr *) *testvar;
 
    switch(v){
        case Ponvlist_ptr_v_nil:
						CORBA::release(*tvar);
            break;
    }
    return TRUE;
}

/* ------------------------------------------------------------ */
/* test variable ocontext_ptr() */
 
int provide_ocontext_ptr(VSContext_ptr **testvar,
              enum prop_ocontext_ptr_v v)
{
    CORBA::Context_ptr *tvar;
    CORBA::Context_var tmp;
 
    switch(v){
        case Pocontext_ptr_v_nil:
            tvar = new CORBA::Context_ptr(tmp);
            break;
    }
 
    *testvar = (VSContext_ptr *) tvar;
    return TRUE;
}

int relinquish_ocontext_ptr(VSContext_ptr **testvar,
              enum prop_ocontext_ptr_v v)
{
    CORBA::Context_ptr  *tvar = (CORBA::Context_ptr *) *testvar;
 
    switch(v){
        case Pocontext_ptr_v_nil:
						CORBA::release(*tvar);
            break;
    }
    return TRUE;
}

/* ------------------------------------------------------------ */
/* test variable oexlist_ptr() */
 
int provide_oexlist_ptr(VSExceptionList_ptr **testvar,
              enum prop_oexlist_ptr_v v)
{
#ifndef VSORB_NO_EXCEPTIONLIST
    CORBA::ExceptionList_ptr *tvar;
    CORBA::ExceptionList_var tmp;
 
    switch(v){
        case Poexlist_ptr_v_nil:
            tvar = new CORBA::ExceptionList_ptr(tmp);
            break;
    }
 
    *testvar = (VSExceptionList_ptr *) tvar;
    return TRUE;
#else
    return FALSE;
#endif
}

int relinquish_oexlist_ptr(VSExceptionList_ptr **testvar,
              enum prop_oexlist_ptr_v v)
{
#ifndef VSORB_NO_EXCEPTIONLIST
    CORBA::ExceptionList_ptr  *tvar = (CORBA::ExceptionList_ptr *) *testvar;
 
    switch(v){
        case Poexlist_ptr_v_nil:
            break;
    }
    return TRUE;
#else
    return FALSE;
#endif
}

/* ------------------------------------------------------------ */
/* test variable octxlist_ptr() */
 
int provide_octxlist_ptr(VSContextList_ptr **testvar,
              enum prop_octxlist_ptr_v v)
{
#ifndef VSORB_NO_CONTEXTLIST
    CORBA::ContextList_ptr *tvar;
    CORBA::ContextList_var tmp;
 
    switch(v){
        case Poctxlist_ptr_v_nil:
            tvar = new CORBA::ContextList_ptr(tmp);
            break;
    }
 
    *testvar = (VSContextList_ptr *) tvar;
    return TRUE;
#else
    return FALSE;
#endif
}

int relinquish_octxlist_ptr(VSContextList_ptr **testvar,
              enum prop_octxlist_ptr_v v)
{
#ifndef VSORB_NO_CONTEXTLIST
    CORBA::ContextList_ptr  *tvar = (CORBA::ContextList_ptr *) *testvar;
 
    switch(v){
        case Poctxlist_ptr_v_nil:
            break;
    }
    return TRUE;
#else
    return FALSE;
#endif
}

}

