//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/DII/Object/t1_aux.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1_aux.cpp,v $
// Revision 1.1.1.1  2000/02/09 17:13:32  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.9  1999/05/10 13:19:57  cwatson
// Added invalid and nil ptr checks().
//
// Revision 1.8  1999/04/28 14:25:03  cwatson
// SR100 text revision change.
//
// Revision 1.7  1999/04/27 13:43:13  cwatson
// SR100 is_equivalent() changed to _is_equivalent().
//
// Revision 1.6  1997/12/05 16:28:10  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.5  1997/11/11 13:13:56  cwatson
// Changed the exception handling by removing VSExceptionWrap.
//
// Revision 1.4  1997/10/09 11:16:44  ian
// Fixed CORBA scope problems.
//
// Revision 1.3  1997/10/09 10:52:49  tet
// Removed undeeded #define.
//
// Revision 1.2  1997/10/04 16:42:15  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.1  1997/10/03 16:05:55  cwatson
// Initial version.
//
//


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>

//-----------------------------------------------------------
// General Object.

int is_valid_object_ptr(VSObject_ptr *o1)
{
		CORBA::Object_ptr *optr1 = (CORBA::Object_ptr *) o1;

		CORBA::Object_ptr optr2 = glob_TS.m_tobj;
		return (*optr1 == optr2);
}

int is_same_object_ptr(VSObject_ptr *o1, VSObject_ptr *o2)
{
		CORBA::Object_ptr *optr1 = (CORBA::Object_ptr *) o1;
		CORBA::Object_ptr *optr2 = (CORBA::Object_ptr *) o2;

		if(is_nil_object_ptr(o1) && is_nil_object_ptr(o2))
				return TRUE;

#ifndef VSORB_NO_OBJECT_IS_EQUIVALENT
		return (*optr1)->_is_equivalent(*optr2);
#else
		return FALSE;
#endif
}

int cleanup_object_ptr(VSObject_ptr *o1)
{
		CORBA::Object_ptr *optr1 = (CORBA::Object_ptr *) o1;
		if(!optr1)
				return TRUE;

		CORBA::release(*optr1);
		delete optr1;
		return TRUE;
}

int hash_value_is_good(VSULong ph,VSULong mx)
{
		CORBA::ULong h = (CORBA::ULong) ph;
		CORBA::ULong maxim = (CORBA::ULong) mx;

		return (h <= maxim) ? TRUE : FALSE;
}

int is_valid_intdef(VSInterfaceDef_ptr *i)
{
		CORBA::InterfaceDef_ptr *idef = (CORBA::InterfaceDef_ptr *) i;
		if(!idef || !*idef)
				return FALSE;

		CORBA::InterfaceDef::FullInterfaceDescription *fd = 		
								(*idef)->describe_interface();
		char *n = fd->name;
		if(n && (strcmp(n,"apiServer") == 0))
				return TRUE;

		return TRUE;
}

int result_is_ok(VSNamedValue_ptr *pn)
{
		CORBA::NamedValue_ptr *n = (CORBA::NamedValue_ptr *) pn;
		if(!n)
				return FALSE;

		printf("NamedValue name = %s:\n",(*n)->name());

		return TRUE;
}

int request_is_ok(VSRequest_ptr *pr)
{
		CORBA::Request_ptr *r = (CORBA::Request_ptr *) pr;
		if(!r || is_nil_request_ptr(pr))
				return FALSE;

		const char *opname = (*r)->operation();
		if(!opname)
				return FALSE;

		if(strcmp(opname,"increment"))
				return FALSE;

		return TRUE;
}

}
