//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include	<corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/

extern CORBA::NVList_ptr argList;

VSRequest_ptr *VSadd_arg(VSRequest_ptr *preq, VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *)ev;

  CORBA::Request_ptr *req = (CORBA::Request_ptr *)preq;

  if (!req || !*req)
    return (VSRequest_ptr *)0;

  try {
		const char *name0 = argList->item(0)->name();

		if (name0)
			(*req)->add_in_arg(name0) = *(argList->item(0)->value());
		else
			(*req)->add_in_arg() = *(argList->item(0)->value());

		const char *name1 = argList->item(1)->name();

		if (name1)
			(*req)->add_inout_arg(name1) = *(argList->item(1)->value());
		else
			(*req)->add_inout_arg() = *(argList->item(1)->value());

		const char *name2 = argList->item(2)->name();
		CORBA::TypeCode_var tc = argList->item(2)->value()->type();

		if (name2)
			(*req)->add_out_arg(name2).replace(tc,0);
		else
			(*req)->add_out_arg().replace(tc,0);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);


    if (env)
      env->exception(t);

    return (VSRequest_ptr *)0;
  }
  catch (...) {
		return (VSRequest_ptr *)0;
	}

  return preq;
}

VSRequest_ptr *VSsend_deferred(VSRequest_ptr *preq, VSEnvironment *ev)
{
	CORBA::Environment *env = (CORBA::Environment *)ev;

	CORBA::Request_ptr *req = (CORBA::Request_ptr *)preq;

	if (!req || !*req)
		return (VSRequest_ptr *)0;
	
	try {
		(*req)->send_deferred();
	}
	catch (CORBA::Exception &ex) {
		CORBA::Exception *t = exception_convert(&ex);


		if (env)
			env->exception(t);

		return (VSRequest_ptr *)0;
	}
	catch (...) {
		return (VSRequest_ptr *)0;
	}	
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	orb->perform_work();

	return preq;
}

VSRequest_ptr *VSsend_oneway(VSRequest_ptr *preq, VSEnvironment *ev)
{
	CORBA::Environment *env = (CORBA::Environment *)ev;

	CORBA::Request_ptr *req = (CORBA::Request_ptr *)preq;

	if (!req || !*req)
		return (VSRequest_ptr *)0;
	
	try {
		(*req)->send_oneway();
	}
	catch (CORBA::Exception &ex) {
		CORBA::Exception *t = exception_convert(&ex);


		if (env)
			env->exception(t);

		return (VSRequest_ptr *)0;
	}
	catch (...) {
		return (VSRequest_ptr *)0;
	}	

	return preq;
}

VSRequest_ptr *VSget_response(VSRequest_ptr *preq, VSEnvironment *ev)
{
	CORBA::Environment *env = (CORBA::Environment *)ev;

	CORBA::Request_ptr *req = (CORBA::Request_ptr *)preq;

	if (!req || !*req)
		return (VSRequest_ptr *)0;
	
	try {
		(*req)->get_response();
	}
	catch (CORBA::Exception &ex) {
		CORBA::Exception *t = exception_convert(&ex);


		if (env)
			env->exception(t);

		return (VSRequest_ptr *)0;
	}
	catch (...) {
    return (VSRequest_ptr *)0;
  }

  return preq;
}

VSRequest_ptr *VSpoll_response(VSRequest_ptr *preq, VSEnvironment *ev)
{
	CORBA::Environment *env = (CORBA::Environment *)ev;

	CORBA::Request_ptr *req = (CORBA::Request_ptr *)preq;

	if (!req || !*req)
		return (VSRequest_ptr *)0;
	
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	orb->perform_work();
	try {
		while (!(*req)->poll_response())
			;
	}
	catch (CORBA::Exception &ex) {
		CORBA::Exception *t = exception_convert(&ex);


		if (env)
			env->exception(t);

		return (VSRequest_ptr *)0;
	}
	catch (...) {
    return (VSRequest_ptr *)0;
  }

  return preq;
}

VSStatus VSinvoke(VSRequest_ptr *preq, VSEnvironment *ev)
{
		VSStatus ret = 0;
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::Request_ptr *req = (CORBA::Request_ptr *) preq;

		CORBA::Request_ptr rreq = (req ? *req : 0);

		try {
			(*req)->invoke();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if (env)
						env->exception(t);
				return (VSStatus) 0;
		}
		catch (...) {
				return (VSStatus) 0;
		}

		*preq = (VSRequest_ptr) rreq;

		return (VSStatus) ret;
}

VSORB_ptr *VSsend_multiple_oneway(
										VSRequestSeq *preqs, 
										VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *)ev;

  CORBA::ORB_ptr orb = glob_TS.m_corb->orb();

  CORBA::ORB::RequestSeq *reqs = (CORBA::ORB::RequestSeq *)preqs;

  if (!reqs)
    return (VSORB_ptr *)0;

  try {
		orb->send_multiple_requests_oneway(*reqs);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);


    if (env)
      env->exception(t);

    return (VSORB_ptr *)0;
  }
  catch (...) {
    return (VSORB_ptr *)0;
  }

  VSORB_ptr *porb = new VSORB_ptr(orb);

  return porb;	
}

VSRequest_ptr *VSnil(VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::Request_ptr ttc;

		try {
				ttc = CORBA::Request::_nil();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if (env)
						env->exception(t);
				return (VSRequest_ptr *)0;
		}
		catch (...) {
				return (VSRequest_ptr *)0;
		}

		CORBA::Request_ptr *rtc = new CORBA::Request_ptr(ttc);
		return (VSRequest_ptr *) rtc;
}

VSRequest_ptr *VSduplicate(VSRequest_ptr *o1, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::Request_ptr *tc = (CORBA::Request_ptr *) o1;
		CORBA::Request_ptr ttc;

		try {
				ttc = CORBA::Request::_duplicate(tc ? *tc : (CORBA::Request_ptr) 0);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if (env)
						env->exception(t);
				return (VSRequest_ptr *)0;
		}
		catch (...) {
				return (VSRequest_ptr *)0;
		}

		CORBA::Request_ptr *rtc = new CORBA::Request_ptr(ttc);
		return (VSRequest_ptr *) rtc;
}

}

