//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/DII/Request/t1_aux.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1_aux.cpp,v $
// Revision 1.3  2001/01/30 22:09:55  arnop
// Give ORB the chance to process events by calling ORB::perform_work()
// for send_deferred() and poll_response()
//
// Revision 1.2  2001/01/25 17:22:30  mli
// Update semantics def. and test variable impl. for add_arg,
// send_oneway and send_multiple_oneway.
//
// Revision 1.1.1.1  2000/02/09 17:13:35  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.14  1999/05/11 11:36:03  stephen
// SR# 139 Added checks for BAD_OP exception.
//
// Revision 1.13  1999/05/11 10:45:42  cwatson
// SR# 141 Must catch BAD_OPERATION for invalid request.
//
// Revision 1.12  1999/05/10 14:38:11  ian
// Added flag to ensure BAD_OPERATION is checked for once only
//
// Revision 1.11  1999/04/29 14:47:28  cwatson
// SR100 Fixed string bounds and catch BAD_OPERATION exception.
//
// Revision 1.10  1999/04/26 14:48:46  james
// Changed CORBA::Char* to char*
//
// Revision 1.9  1997/12/05 16:28:12  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.8  1997/11/11 13:13:57  cwatson
// Changed the exception handling by removing VSExceptionWrap.
//
// Revision 1.7  1997/10/22 14:25:58  ian
// Added tests for send_oneway, send_deferred, get_response, poll_response, send_multiple_oneway and add_arg.
//
// Revision 1.6  1997/10/09 10:52:50  tet
// Removed undeeded #define.
//
// Revision 1.5  1997/10/06 17:57:09  kevin
// fixed Request() ..Result() is untestable with current ORBs
//
// Revision 1.4  1997/10/04 16:42:18  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.3  1997/10/01 16:03:15  cwatson
// Makefile changes.
//
// Revision 1.2  1997/09/12 13:01:52  cwatson
// Moved all _ptr provide and relinquish functions to corba/common/ptr_prov.cpp.
//
// Revision 1.1  1997/08/19 14:57:40  cwatson
// *** empty log message ***
//
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>
#include <t1i_tdi.h>

//-----------------------------------------------------------
// General Request.

int is_same_request_ptr(VSRequest_ptr *o1, VSRequest_ptr *o2)
{
		if(is_nil_request_ptr(o1) && is_nil_request_ptr(o2))
				return TRUE;
		
		return TRUE;
}

int cleanup_request_ptr(VSRequest_ptr *o1)
{
		CORBA::Request_ptr *tc = (CORBA::Request_ptr *) o1;
		if(!tc)
				return TRUE;

		delete tc;
		return TRUE;
}

//-----------------------------------------------------------

extern CORBA::Boolean g_fInvalidRequest;
extern prop_ops g_OpType;

int is_invalid_request(VSRequest_ptr *preq)
{
	CORBA::Request_ptr *req = (CORBA::Request_ptr *)preq;

	if (CORBA::is_nil(*req))
		return TRUE;

	return (g_fInvalidRequest == TRUE);
}

// This flag indicates if we got a BAD_OPERATION after calling invoke().
// If true then we don't expect a further one in check_request_executed(),
// i.e. thrown by server
//
CORBA::Boolean g_gotBadOp = FALSE;

int execute_request(VSRequest_ptr *preq)
{
	int nRet = TRUE;
	g_gotBadOp = FALSE;

    CORBA::Request_ptr *req = (CORBA::Request_ptr *)preq;

    if (!req || !*req)
		nRet = FALSE;
	else
		try
		{
			(*req)->invoke();
		}
	    catch (CORBA::BAD_OPERATION &)
        {
            g_gotBadOp = nRet = (g_OpType == Pops_Invalid);
        }
	    catch (CORBA::Exception &ex)
        {
            nRet = FALSE;
        }

    return nRet;
}

int check_request_sent(VSRequest_ptr *preq)
{
	int nRet = TRUE;

	CORBA::Request_ptr *req = (CORBA::Request_ptr *)preq;

	if (!req || CORBA::is_nil(*req))
		nRet = FALSE;
	else
		try 
		{
			while (!(*req)->poll_response())
				;

			(*req)->get_response();

			// Now args are valid etc.
		}
	    catch (CORBA::BAD_OPERATION &)
        {
            g_gotBadOp = nRet = (g_OpType == Pops_Invalid);
        }
		catch (CORBA::Exception &ex)
  	{
    	nRet = FALSE;
  	}

	return nRet;
}

int get_response(VSRequest_ptr *preq)
{
	int nRet = TRUE;

	CORBA::Request_ptr *req = (CORBA::Request_ptr *)preq;

	if (!req || CORBA::is_nil(*req))
		nRet = FALSE;
	else
		try 
		{
			(*req)->get_response();
		}
	    catch (CORBA::BAD_OPERATION &)
        {
            g_gotBadOp = nRet = (g_OpType == Pops_Invalid);
        }
		catch (CORBA::Exception &ex)
  	{
    	nRet = FALSE;
  	}

	return nRet;
}


extern CORBA::Any g_Any;

int check_request_executed(VSRequest_ptr *preq)
{
  int nRet = TRUE;

  CORBA::Request_ptr *req = (CORBA::Request_ptr *)preq;

  if (!req || CORBA::is_nil(*req))
    nRet = FALSE;
  else
		try
		{
			if (g_OpType == Pops_Invalid)
				if (g_gotBadOp)
				{
					g_gotBadOp = FALSE;
					return TRUE;
                }

 			CORBA::NVList_ptr args = (*req)->arguments();
			CORBA:: NamedValue_ptr nv = (*req)->result();
 			CORBA::Any *aval = nv->value();

   			switch (g_OpType)
   			{
    		case Pops_Invalid:
				{
					if (g_gotBadOp) 
					{
						g_gotBadOp = FALSE;
				
						return TRUE;
					}

					CORBA::Environment_ptr ep = (*req)->env();
					CORBA::Environment *lep = ep;
					nRet = system_exception_is("BAD_OPERATION",(VSEnvironment *)lep);
					break;
				}
				case Pops_CharOp:
				{
					CORBA::Char c1, c2;
					CORBA::Any::to_char s1(c1);
					CORBA::Any::to_char s2(c2);
					
					if ((*aval >>= s1) && (g_Any >>= s2) && (c1 == c2))
						return TRUE;		
					else
						return FALSE;

					break;	
				}
				case Pops_OctetOp:
				{
					CORBA::Octet c1, c2;
					CORBA::Any::to_octet s1(c1);
					CORBA::Any::to_octet s2(c2);
					
					if ((*aval >>= s1) && (g_Any >>= s2) && (c1 == c2))
						return TRUE;		
					else
						return FALSE;

					break;	
				}
				case Pops_BooleanOp:
				{
					CORBA::Boolean c1, c2;
					CORBA::Any::to_boolean s1(c1);
					CORBA::Any::to_boolean s2(c2);
					
					if ((*aval >>= s1) && (g_Any >>= s2) && (c1 == c2))
						return TRUE;		
					else
						return FALSE;

					break;	
				}
				case Pops_ShortOp:
				{
					CORBA::Short s1, s2;
					
					if ((*aval >>= s1) && (g_Any >>= s2) && (s1 == s2))
						return TRUE;		
					else
						return FALSE;

					break;	
				}
				case Pops_LongOp:
				{
					CORBA::Long l1, l2;
					
					if ((*aval >>= l1) && (g_Any >>= l2) && (l1 == l2))
						return TRUE;		
					else
						return FALSE;

					break;	
				}
				case Pops_FloatOp:
				{
					CORBA::Float f1, f2;
					
					if ((*aval >>= f1) && (g_Any >>= f2) && (f1 == f2))
						return TRUE;		
					else
						return FALSE;

					break;	
				}
				case Pops_DoubleOp:
				{
					CORBA::Double d1, d2;
					
					if ((*aval >>= d1) && (g_Any >>= d2) && (d1 == d2))
						return TRUE;		
					else
						return FALSE;

					break;	
				}
				case Pops_StringOp:
				{
					const char *c1, *c2;
					CORBA::Any::to_string s1(c1, 0);
					CORBA::Any::to_string s2(c2, 0);
					
					if ((*aval >>= s1) && (g_Any >>= s2) && !::strcmp(c1, c2))
						return TRUE;		
					else
						return FALSE;
					break;	
				}
				case Pops_UShortOp:
				{
					CORBA::UShort us1, us2;
					
					if ((*aval >>= us1) && (g_Any >>= us2) && (us1 == us2))
						return TRUE;		
					else
						return FALSE;

					break;	
				}
				case Pops_ULongOp: 
				{
					CORBA::ULong ul1, ul2;
					
					if ((*aval >>= ul1) && (g_Any >>= ul2) && (ul1 == ul2))
						return TRUE;		
					else
						return FALSE;

					break;	
				}
			}
		}
		catch (CORBA::Exception &ex)
    {
      nRet = FALSE;
    }

  return nRet;
}

int check_requests_sent(VSRequestSeq *s)
{
  CORBA::ORB::RequestSeq *reqs = (CORBA::ORB::RequestSeq *)s;
  CORBA::ORB::RequestSeq rsq = *reqs;
  CORBA::Request_var r;

  for(int i=0; i< rsq.length(); i++)
    {
      r = rsq[i];
      try{
	r->poll_response();
      }catch(CORBA::Exception &ex){
	return false;
      }
    }
  return true;
}

}
