//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/Exceptions/sysex/t1.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1.cpp,v $
// Revision 1.4  2001/01/25 11:48:21  mli
// Update for VSSystemEx_completed.
//
// Revision 1.3  2000/12/15 13:56:03  mli
// Replace instantiation of CORBA::SystemException and CORBA::UserExeption.
//
// Revision 1.2  2000/02/18 11:03:28  mli
//   o     _narrow (CORBA 2.1) -> _downcast (CORBA 2.3) (2x).
//   o     Corrected an error in method 'VSSystemEx_completed'.
//
// Revision 1.1.1.1  2000/02/09 17:13:40  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.9  1999/04/28 14:26:30  cwatson
// SR100 text revision change.
//
// Revision 1.8  1999/04/27 14:03:25  cwatson
// SR100 Removed unrequired indirection.
//
// Revision 1.7  1997/12/05 16:28:13  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.6  1997/11/11 13:13:58  cwatson
// Changed the exception handling by removing VSExceptionWrap.
//
// Revision 1.5  1997/10/04 16:42:19  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.4  1997/10/01 16:03:58  cwatson
// Makefile changes.
//
// Revision 1.3  1997/09/19 15:44:24  ian
// Windows NT version.
//
// Revision 1.2  1997/09/15 10:57:43  cwatson
// Added the VSNOT_IMPLEMENTED macro and we now check for the NOT_IMPLEMENTED
// User Exception.
//
// Revision 1.1  1997/08/19 15:03:19  cwatson
// *** empty log message ***
//
//

#include	<corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/

VSSystemException *VSSystemEx_narrow(VSException *pex, VSEnvironment *ev)
{
	CORBA::Environment *env = (CORBA::Environment *) ev;
#ifndef VSORB_NO_SYSTEMEXCEPTION_NARROW
	CORBA::Exception *tex = (CORBA::Exception *) pex;
	CORBA::SystemException *lex;

	try {
		lex = CORBA::SystemException::_downcast(tex);
	}
	catch (CORBA::Exception &ex) {
		CORBA::Exception *t = exception_convert(&ex);
	if(env)
		env->exception(t);
	}
	catch (...) {
		return (VSSystemException *) 0;
	}

	return (VSSystemException *) lex;
#else
	VSNOT_IMPLEMENTED;
	return (VSSystemException *) 0;
#endif
}

VSSystemException *VSSystemEx_assign(VSSystemException *pex, VSEnvironment *ev)
{
	CORBA::Environment *env = (CORBA::Environment *) ev;
#ifndef VSORB_NO_SYSTEMEXCEPTION_ASSIGN
	CORBA::SystemException *tex = (CORBA::SystemException *) pex;
	CORBA::SystemException *lex;

	try {
		lex = tex;
	}
	catch (CORBA::Exception &ex) {
		printf("system exception caught.\n");
		CORBA::Exception *t = exception_convert(&ex);
		if(env)
			env->exception(t);
		return (VSSystemException *) 0;
	}
	catch (...) {
		return (VSSystemException *) 0;
	}

	return (VSSystemException *) lex;
		
#else
	VSNOT_IMPLEMENTED;
	return (VSSystemException *) 0;
#endif
}

VSULong VSSystemEx_minor(VSSystemException *pex, VSULong m, VSEnvironment *ev)
{
	CORBA::Environment *env = (CORBA::Environment *) ev;
#ifndef VSORB_NO_SYSTEMEXCEPTION_MINOR
	CORBA::SystemException *tex = (CORBA::SystemException *) pex;
	CORBA::ULong mn = (CORBA::ULong) m;

	try {
		tex->minor(mn);
	}
	catch (CORBA::Exception &ex) {
		CORBA::Exception *t = exception_convert(&ex);
		if(env)
			env->exception(t);
		return (VSULong) 0;
	}
	catch (...) {
		return (VSULong) 0;
	}

	return (VSULong) mn;
#else
	VSNOT_IMPLEMENTED;
	return (VSULong) 0;
#endif
}

VSCompleted VSSystemEx_completed(VSSystemException *pex, VSCompleted m, VSEnvironment *ev)
{
	CORBA::Environment *env = (CORBA::Environment *) ev;
#ifndef VSORB_NO_SYSTEMEXCEPTION_COMPLETED
	CORBA::SystemException *tex = (CORBA::SystemException *) pex;
	CORBA::CompletionStatus *mn = (CORBA::CompletionStatus *) m;

	try {
		tex->completed(*mn);
	}
	catch (CORBA::Exception &ex) {
		CORBA::Exception *t = exception_convert(&ex);
		if(env)
			env->exception(t);
		return (VSCompleted) 0;
	}
	catch (...) {
		return (VSCompleted) 0;
	}

	return (VSCompleted) mn;
#else
	VSNOT_IMPLEMENTED;
	return (VSCompleted) 0;
#endif
}

/*------------------------------------------------------*/

VSUserException *VSUserEx_narrow(VSException *pex, VSEnvironment *ev)
{
	CORBA::Environment *env = (CORBA::Environment *) ev;
#ifndef VSORB_NO_USEREXCEPTION_NARROW
	CORBA::Exception *tex = (CORBA::Exception *) pex;
	CORBA::UserException *lex;

	try {
		lex = CORBA::UserException::_downcast(tex);
	}
	catch (CORBA::Exception &ex) {
		CORBA::Exception *t = exception_convert(&ex);
		if(env)
			env->exception(t);
		return (VSUserException *) 0;
	}
	catch (...) {
		return (VSUserException *) 0;
	}

	return (VSUserException *) lex;
#else
	VSNOT_IMPLEMENTED;
	return (VSUserException *) 0;
#endif
}

VSUserException *VSUserEx_assign(VSUserException *pex, VSEnvironment *ev)
{
	CORBA::Environment *env = (CORBA::Environment *) ev;
#ifndef VSORB_NO_USEREXCEPTION_ASSIGN
	CORBA::UserException *tex = (CORBA::UserException *) pex;
	CORBA::UserException *lex;

	try {
		lex = tex;
	}
	catch (CORBA::Exception &ex) {
		CORBA::Exception *t = exception_convert(&ex);
		if(env)
			env->exception(t);
		return (VSUserException *) 0;
	}
	catch (...) {
		return (VSUserException *) 0;
	}

	return (VSUserException *) lex;
#else
	VSNOT_IMPLEMENTED;
	return (VSUserException *) 0;
#endif
}

}

