//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

/* ---- global property declarations ---- */
/* ---- test variables and local properties ---- */

/* ------------------------------------------------------------ */
/* test variable sysex_ptr() */

static int provide_systemex_ptr(VSException **testvar,
	enum prop_all_sysex sysex_value)
{
	CORBA::Exception *tvar;
	CORBA::ULong mid = 0;
	CORBA::CompletionStatus stat = CORBA::COMPLETED_NO;

	switch(sysex_value){
	case Pall_sysex_unknown:
		tvar = new CORBA::UNKNOWN(mid,stat);
		break;
        case Pall_sysex_bad_param:
	        tvar = new CORBA::BAD_PARAM(mid,stat);
		break;
        case Pall_sysex_no_memory:
	        tvar = new CORBA::NO_MEMORY(mid,stat);
		break;
        case Pall_sysex_imp_limit:
	        tvar = new CORBA::IMP_LIMIT(mid,stat);
		break;
        case Pall_sysex_comm_failure:
	        tvar = new CORBA::COMM_FAILURE(mid,stat);
		break;
        case Pall_sysex_inv_objref:
	        tvar = new CORBA::INV_OBJREF(mid,stat);
		break;
        case Pall_sysex_no_permission:
	        tvar = new CORBA::NO_PERMISSION(mid,stat);
		break;
        case Pall_sysex_internal:
	        tvar = new CORBA::INTERNAL(mid,stat);
		break;
        case Pall_sysex_marshal:
	        tvar = new CORBA::MARSHAL(mid,stat);
		break;
        case Pall_sysex_initialize:
	        tvar = new CORBA::INITIALIZE(mid,stat);
		break;
        case Pall_sysex_no_implement:
	        tvar = new CORBA::NO_IMPLEMENT(mid,stat);
		break;
        case Pall_sysex_bad_typecode:
	        tvar = new CORBA::BAD_TYPECODE(mid,stat);
		break;
        case Pall_sysex_bad_operation:
	        tvar = new CORBA::BAD_OPERATION(mid,stat);
		break;
        case Pall_sysex_no_resources:
	        tvar = new CORBA::NO_RESOURCES(mid,stat);
		break;
        case Pall_sysex_no_response:
	        tvar = new CORBA::NO_RESPONSE(mid,stat);
		break;
        case Pall_sysex_persist_store:
	        tvar = new CORBA::PERSIST_STORE(mid,stat);
		break;
        case Pall_sysex_bad_inv_order:
	        tvar = new CORBA::BAD_INV_ORDER(mid,stat);
		break;
        case Pall_sysex_transient:
	        tvar = new CORBA::TRANSIENT(mid,stat);
		break;
        case Pall_sysex_free_mem:
	        tvar = new CORBA::FREE_MEM(mid,stat);
		break;
        case Pall_sysex_inv_ident:
	        tvar = new CORBA::INV_IDENT(mid,stat);
		break;
        case Pall_sysex_inv_flag:
	        tvar = new CORBA::INV_FLAG(mid,stat);
		break;
        case Pall_sysex_intf_repos:
	        tvar = new CORBA::INTF_REPOS(mid,stat);
		break;
        case Pall_sysex_bad_context:
	        tvar = new CORBA::BAD_CONTEXT(mid,stat);
		break;
        case Pall_sysex_obj_adapter:
	        tvar = new CORBA::OBJ_ADAPTER(mid,stat);
		break;
        case Pall_sysex_data_conversion:
	        tvar = new CORBA::DATA_CONVERSION(mid,stat);
		break;
        case Pall_sysex_object_not_exist:
	        tvar = new CORBA::OBJECT_NOT_EXIST(mid,stat);
		return FALSE;
		break;
	}

	*testvar = (VSException *) tvar;
	return TRUE;
}

static int relinquish_systemex_ptr(VSException **testvar,
	enum prop_all_sysex sysex_value)
{
	CORBA::Exception *tvar = (CORBA::Exception *) *testvar;

	switch(sysex_value){
	case Pall_sysex_unknown:
        case Pall_sysex_bad_param:
        case Pall_sysex_no_memory:
        case Pall_sysex_imp_limit:
        case Pall_sysex_comm_failure:
        case Pall_sysex_inv_objref:
        case Pall_sysex_no_permission:
        case Pall_sysex_internal:
        case Pall_sysex_marshal:
        case Pall_sysex_initialize:
        case Pall_sysex_no_implement:
        case Pall_sysex_bad_typecode:
        case Pall_sysex_bad_operation:
        case Pall_sysex_no_resources:
        case Pall_sysex_no_response:
        case Pall_sysex_persist_store:
        case Pall_sysex_bad_inv_order:
        case Pall_sysex_transient:
        case Pall_sysex_free_mem:
        case Pall_sysex_inv_ident:
        case Pall_sysex_inv_flag:
        case Pall_sysex_intf_repos:
        case Pall_sysex_bad_context:
        case Pall_sysex_obj_adapter:
        case Pall_sysex_data_conversion:
		delete tvar;
		break;
        case Pall_sysex_object_not_exist:
		delete tvar;
		break;
	}
	return TRUE;
}

/* ------------------------------------------------------------ */
/* test variable ex_ptr() */

int provide_ex_ptr(VSException **testvar, enum prop_all_ex value)
{
    if(value <= Pall_ex_object_not_exist)
	return provide_systemex_ptr(testvar,(prop_all_sysex) value);

	CORBA::Exception *tvar;

	switch(value){
	case Pall_ex_sysex:
		tvar = new CORBA::UNKNOWN();
		break;
	case Pall_ex_userex:
		tvar = new not_implemented();
		break;
	}

	*testvar = (VSException *) tvar;
	return TRUE;
}

int relinquish_ex_ptr(VSException **testvar, enum prop_all_ex value)
{
    if(value <= Pall_ex_object_not_exist)
	return relinquish_systemex_ptr(testvar,(prop_all_sysex) value);

	CORBA::Exception *tvar = (CORBA::Exception *) *testvar;

	switch(value){
	case Pall_ex_sysex:
	case Pall_ex_userex:
		delete tvar;
		break;
	}

	return TRUE;
}

/* ------------------------------------------------------------ */
/* test variable sysex_ptr() */

int provide_sysex_ptr(VSException **testvar,
	enum prop_all_sysex sysex_value)
{
    if(sysex_value <= Pall_sysex_object_not_exist)
	return provide_systemex_ptr(testvar,sysex_value);

	CORBA::Exception *tvar;
	CORBA::ULong mid = 0;
	CORBA::CompletionStatus stat = CORBA::COMPLETED_NO;

	switch(sysex_value){
	case Pall_sysex_valid:
		tvar = new CORBA::UNKNOWN();
		break;
	}

	*testvar = (VSException *) tvar;
	return TRUE;
}

int relinquish_sysex_ptr(VSException **testvar,
	enum prop_all_sysex sysex_value)
{
    if(sysex_value <= Pall_sysex_object_not_exist)
	return relinquish_systemex_ptr(testvar,sysex_value);

	CORBA::Exception *tvar = (CORBA::Exception *) *testvar;

	switch(sysex_value){
	case Pall_sysex_valid:
		delete tvar;
		break;
	}
	return TRUE;
}

/* ------------------------------------------------------------ */
/* test variable completed() */

int provide_completed(VSCompleted *testvar, enum prop_completed_prop_1 completed_value)
{
	CORBA::CompletionStatus tvar = CORBA::COMPLETED_NO;

	switch(completed_value){
	case Pcompleted_prop_1_yes:
		tvar = CORBA::COMPLETED_YES;
		break;
	case Pcompleted_prop_1_no:
		tvar = CORBA::COMPLETED_NO;
		break;
	case Pcompleted_prop_1_maybe:
		tvar = CORBA::COMPLETED_MAYBE;
		break;
	}
		
	*testvar = (VSCompleted) new CORBA::CompletionStatus(tvar);
	return TRUE;
}

int relinquish_completed(VSCompleted *testvar, enum prop_completed_prop_1 completed_value)
{
	switch(completed_value){
	case Pcompleted_prop_1_yes:
		break;
	case Pcompleted_prop_1_no:
		break;
	case Pcompleted_prop_1_maybe:
		break;
	}
		
	return TRUE;
}

/* ------------------------------------------------------------ */
/* test variable userex_ptr() */

int provide_userex_ptr(VSException **testvar,
	enum prop_all_userex userex_value)
{
	CORBA::Exception *tvar;
	CORBA::ULong mid = 0;
	CORBA::CompletionStatus stat = CORBA::COMPLETED_NO;

	switch(userex_value){
	case Pall_userex_valid:
		tvar = new not_implemented();
		break;
	case Pall_userex_invalid_name:
		tvar = new CORBA::ORB::InvalidName();
		break;
	}

	*testvar = (VSException *) tvar;
	return TRUE;
}

int relinquish_userex_ptr(VSException **testvar,
	enum prop_all_userex userex_value)
{
	CORBA::Exception *tvar = (CORBA::Exception *) *testvar;

	switch(userex_value){
	case Pall_userex_valid:
	case Pall_userex_invalid_name:
		delete tvar;
		break;
	}
	return TRUE;
}

}

