//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>

/*------------------------------------------------------*/
int
is_correct_type(VSTypeCode_ptr* vs_tc_ptr)
{
	CORBA::TypeCode_ptr* tc_ptr = (CORBA::TypeCode_ptr *) vs_tc_ptr;

  return((*tc_ptr)->kind() == CORBA::tk_string); 
}

/*------------------------------------------------------*/
int
is_correct_type_def(VSIDLType_ptr vs_idltype_ptr)
{
	CORBA::IDLType_ptr idltype_ptr = (CORBA::IDLType_ptr) vs_idltype_ptr;
	CORBA::TCKind tk = idltype_ptr->type()->kind();
	 
	return (tk == CORBA::tk_string);
}



/*------------------------------------------------------*/
int
is_correct_mode(VSAttributeMode vs_mode, VSAttributeDef_ptr vs_attr_ptr)
{
	CORBA::AttributeMode attMode = (CORBA::AttributeMode) vs_mode;
	CORBA::AttributeDef_ptr attribute_ptr = (CORBA::AttributeDef_ptr) vs_attr_ptr;

	char* attName = (char *) attribute_ptr->name();
	if(strstr(attName, "Att_ro"))
			return (attMode == CORBA::ATTR_READONLY);
	if(strstr(attName, "Att_rw"))
			return (attMode == CORBA::ATTR_NORMAL);

	//one of above statements should alway be true
	return FALSE;
}

/*------------------------------------------------------*/
int
is_nil_attributedef_ptr(VSAttributeDef_ptr *o1)
{
		CORBA::AttributeDef_ptr *tc = (CORBA::AttributeDef_ptr *) o1;
		if(!tc)
				return TRUE;
								 
		return CORBA::is_nil(*tc);
}
	
int is_nil_object(VSObject_ptr o1)
{
	CORBA::Object_ptr obj1 = (CORBA::Object_ptr) o1;
			 
	if(!obj1)
		return TRUE;
	return CORBA::is_nil(obj1);
}

int 
cleanup_attributedef_ptr(VSAttributeDef_ptr *o1)
{
		CORBA::AttributeDef_ptr *tc = (CORBA::AttributeDef_ptr *) o1;
		if(!tc)
				return TRUE;
		delete tc;
		return TRUE;
}

/*------------------------------------------------------*/

int
is_correct_mode_write(VSAttributeMode vs_att_mode)
{
		CORBA::AttributeMode att_mode = (CORBA::AttributeMode) vs_att_mode;
		CORBA::Repository_ptr IR_ptr;
		CORBA::Contained_ptr   ctnD_ptr;
		CORBA::AttributeDef_ptr attDef_ptr;
		IRSupport *irs = glob_TS.repository();

		try {
				IR_ptr = irs->get_repository();
				ctnD_ptr = irs->get_contained(IR_ptr,"M_AttributeDef::I1::Att_ro");
				attDef_ptr = CORBA::AttributeDef::_narrow(ctnD_ptr);
				return (attDef_ptr->mode() == att_mode);
		}
		catch(...) {
				return FALSE;
		}
		return FALSE;
}

int
is_same_type_def(VSIDLType_ptr vs_idl1_ptr, VSIDLType_ptr vs_idl2_ptr)
{
		CORBA::IDLType_ptr idl1_ptr = (CORBA::IDLType_ptr) vs_idl1_ptr;
		CORBA::IDLType_ptr idl2_ptr = (CORBA::IDLType_ptr) vs_idl2_ptr;
		CORBA::TypeCode_ptr t1 = idl1_ptr->type();
		CORBA::TypeCode_ptr t2 = idl2_ptr->type();

		return t1->equal(t2);
}

/*------------------------------------------------------*/
/* aux funcs for redefined functionality */
/*------------------------------------------------------*/
int
is_correct_def_kind(VSDefinitionKind vs_dk)
{
		CORBA::DefinitionKind dk = (CORBA::DefinitionKind) vs_dk;
		return (dk == CORBA::dk_Attribute);
}

/*------------------------------------------------------*/
int
is_correct_description(VSDescription_ptr vs_desc_ptr)
{
		CORBA::Contained::Description* desc_ptr = 
				(CORBA::Contained::Description *) vs_desc_ptr;
		if (!desc_ptr)
				return FALSE;

		if (desc_ptr->kind != CORBA::dk_Attribute)
				return FALSE;

		const CORBA::AttributeDescription* attDesc_ptr;
		if (desc_ptr->value >>= attDesc_ptr) {
				if(!attDesc_ptr)
						return FALSE;
				if(!attDesc_ptr->name)
						return FALSE;
				return ((attDesc_ptr->mode == CORBA::ATTR_NORMAL) &&
						(strcmp( (const char*)attDesc_ptr->name, "Att_rw") == 0));
		}
		return FALSE;
}

/*------------------------------------------------------*/
}
