//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

/* ------------------------------------------------------------ */
/* calling_att_ptr is provided for the VSmode_read() test which
	 reads the mode for the two types of attributes: read-only and 
	 read-write. */
int
provide_calling_att_ptr (VSAttributeDef_ptr *testvar,
			 enum prop_calling_att_ptr_attValue attValue)
{
  CORBA::AttributeDef_ptr* tvar = (CORBA::AttributeDef_ptr *) testvar; 
  CORBA::Object_ptr      obj_ptr;
  CORBA::Repository_ptr  IR_ptr;
  CORBA::Contained_ptr		ctnD_ptr;
  IRSupport *irs = glob_TS.repository();
  IR_ptr = irs->get_repository();
  
  switch (attValue)  {
  case Pcalling_att_ptr_attValue_readonly:
    ctnD_ptr = irs->get_contained(IR_ptr,"M_AttributeDef::I1::Att_ro");
    break;
  case Pcalling_att_ptr_attValue_readwrite:
    ctnD_ptr = irs->get_contained(IR_ptr,"M_AttributeDef::I1::Att_rw");
    break;
  }
  if (CORBA::is_nil(ctnD_ptr))
    return FALSE;
  
  *tvar = CORBA::AttributeDef::_narrow(ctnD_ptr);
  return TRUE;
}
  
  int
  relinquish_calling_att_ptr (VSAttributeDef_ptr *testvar,
			      enum prop_calling_att_ptr_attValue attValue)
  {
    //	AttributeDef_ptr* tvar = (AttributeDef_ptr *) testvar;
    
    switch (attValue)  {
    case Pcalling_att_ptr_attValue_readonly:
    case Pcalling_att_ptr_attValue_readwrite:
      //						delete tvar;
      break;
    }
    return TRUE;
  }
  /* ------------------------------------------------------------ */
  /* object_ptr is provided for the VSnarrow test */
  int
  provide_object_ptr (VSObject_ptr **testvar,
		      enum prop_object_ptr_objValue objValue)
  {
    CORBA::Object_ptr tvar;
    CORBA::Object_ptr* rptr;
    CORBA::Object_ptr obj_ptr;
    CORBA::Repository_ptr IR_ptr;
    IRSupport *irs = glob_TS.repository();
    
    switch (objValue)  {
    case Pobject_ptr_objValue_valid:
      IR_ptr = irs->get_repository();
      tvar = irs->get_contained(IR_ptr,"M_AttributeDef::I1::Att_rw");
      break;
    case Pobject_ptr_objValue_null:
      tvar = NULL;
      break;
    }
    rptr = new CORBA::Object_ptr(tvar);
    *testvar = (VSObject_ptr *) rptr;
    return TRUE;
  }
  
  int
  relinquish_object_ptr (VSObject_ptr **testvar,
			 enum prop_object_ptr_objValue objValue)
  {
    CORBA::Object_ptr* tvar = (CORBA::Object_ptr *) *testvar;
    
    switch (objValue)  {
    case Pobject_ptr_objValue_valid:
						delete tvar;
						break;
    case Pobject_ptr_objValue_null:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* attributedef_ptr is provided for the VSduplicate() test. */
  int
  provide_attributedef_ptr (VSAttributeDef_ptr **testvar,
			    enum prop_attributedef_ptr_attdefValue attdefValue)
  {
    CORBA::AttributeDef_ptr tvar;
    CORBA::AttributeDef_ptr* rptr;
    
    CORBA::Object_ptr obj_ptr;
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr ctnD_ptr;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    
    switch (attdefValue)  {
    case Pattributedef_ptr_attdefValue_valid:
      IR_ptr = irs->get_repository();
      ctnD_ptr = irs->get_contained(IR_ptr,"M_AttributeDef::I1::Att_rw");
      tvar = CORBA::AttributeDef::_narrow(ctnD_ptr);
      break;
    case Pattributedef_ptr_attdefValue_null:
      tvar = NULL;
      break;
    }
    rptr = new CORBA::AttributeDef_ptr(tvar);
    *testvar = (VSAttributeDef_ptr *) rptr;
    return TRUE;
  }
  
  int
  relinquish_attributedef_ptr (VSAttributeDef_ptr **testvar,
			       enum prop_attributedef_ptr_attdefValue attdefValue)
  {
    CORBA::AttributeDef_ptr* tvar = (CORBA::AttributeDef_ptr *) *testvar;
    
    switch (attdefValue)  {
    case Pattributedef_ptr_attdefValue_valid:
      delete tvar;
      break;
    case Pattributedef_ptr_attdefValue_null:
      break;
    }
    return TRUE;
  }
  
  
  /* ------------------------------------------------------------ */
  /* mode is provided for the VSmode_write() test which changes the
     mode of a read-only attribute to be read-write. */
  int
  provide_mode (VSAttributeMode *testvar)
  {
    CORBA::AttributeMode* tvar = (CORBA::AttributeMode *) testvar;
    *tvar = CORBA::ATTR_NORMAL;
    return TRUE;
  }
  
  int
  relinquish_mode (VSAttributeMode *testvar)
  {
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* type is provided for the VStype_def_write() test. */
  int
  provide_type (VSIDLType_ptr *testvar)
  {
    CORBA::IDLType_ptr *tvar = (CORBA::IDLType_ptr *) testvar;
    CORBA::Repository_ptr  IR_ptr;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    try {
      *tvar = IR_ptr->get_primitive(CORBA::pk_short);
    }
    catch (...) {
      return FALSE;
    }
    return TRUE;
  }
  
  int
  relinquish_type (VSIDLType_ptr *testvar)
  {
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  
}
