//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>


/*------------------------------------------------------*/
/* In the sample IR our const value has been set to 3 */
/* depending on the order of running the tests VStype_def_write()
	 and VSvalue_write() we need to check the type of the value
	 stored in the Any. */
int
is_correct_value(VSAny_ptr vs_value, int x)
{
	CORBA::Any* constValue = (CORBA::Any *) vs_value;
	CORBA::UShort sval;
	if(*constValue >>= sval)
			return(sval == x);
	CORBA::ULong lval;
	if(*constValue >>= lval)
			return(lval == x);
	
	return FALSE;
}


/*------------------------------------------------------*/
int
is_correct_type(VSTypeCode_ptr* vs_type_ptr)
{
	CORBA::TypeCode_ptr* type_ptr = (CORBA::TypeCode_ptr *) vs_type_ptr;
	//TypeCode* type_ptr = (TypeCode *) vs_type_ptr;

	//CORBA::TCKind k = type_ptr->kind();
	//char* kid = (char *) type_ptr->id();

	return((*type_ptr)->kind() == CORBA::tk_ushort); 
}

/*------------------------------------------------------*/
int
is_type_def_ulong(VSIDLType_ptr vs_idltype_ptr)
{
	CORBA::IDLType_ptr idltype_ptr = (CORBA::IDLType_ptr) vs_idltype_ptr;

	CORBA::TCKind k = idltype_ptr->type()->kind();
	
	return (idltype_ptr->type()->kind() == CORBA::tk_ulong);
}
/*------------------------------------------------------*/
int
is_type_def_ushort(VSIDLType_ptr vs_idltype_ptr)
{
	CORBA::IDLType_ptr idltype_ptr = (CORBA::IDLType_ptr) vs_idltype_ptr;

	CORBA::TCKind k = idltype_ptr->type()->kind();
	
	return (idltype_ptr->type()->kind() == CORBA::tk_ushort); 
}

/*------------------------------------------------------*/
int
is_val_type_ulong(const VSAny_ptr vs_value)
{
	CORBA::Any* constValue = (CORBA::Any *) vs_value;
	CORBA::ULong val;
	if(*constValue >>= val)
			return TRUE;
	
	return FALSE;
}

/*------------------------------------------------------*/
int is_nil_object(VSObject_ptr o1)
{
		CORBA::Object_ptr obj1 = (CORBA::Object_ptr) o1;
		 
		if(!obj1)
			return TRUE;
		return CORBA::is_nil(obj1);
}

int
is_nil_constantdef_ptr(VSConstantDef_ptr *o1)
{
		CORBA::ConstantDef_ptr *tc = (CORBA::ConstantDef_ptr *) o1;
		if(!tc)
				return TRUE;
								 
		return CORBA::is_nil(*tc);
}
											 
int 
cleanup_constantdef_ptr(VSConstantDef_ptr *o1)
{
		CORBA::ConstantDef_ptr *tc = (CORBA::ConstantDef_ptr *) o1;
		if(!tc)
				return TRUE;
		delete tc;
		return TRUE;
		
}

/*------------------------------------------------------*/
/* aux funcs for redefined functionality */
/*------------------------------------------------------*/
int
is_correct_def_kind(VSDefinitionKind vs_dk)
{
		CORBA::DefinitionKind dk = (CORBA::DefinitionKind) vs_dk;
		return (dk == CORBA::dk_Constant);
}

/*------------------------------------------------------*/
/* depending on the order of running the tests we must check
	 for two different types of the const (in case 
	 type_def_write() has previously been called). */
int
is_correct_description(VSDescription_ptr vs_desc_ptr)
{
  CORBA::Contained::Description* desc_ptr = 
    (CORBA::Contained::Description *) vs_desc_ptr;
  
  if (!desc_ptr)
    return FALSE;
  
  if (desc_ptr->kind != CORBA::dk_Constant)
    return FALSE;
  
  const CORBA::ConstantDescription* constDesc_ptr;
  if (desc_ptr->value >>= constDesc_ptr) {
    
    if (!constDesc_ptr)
      return FALSE;
    
    if (!constDesc_ptr->name)
      return FALSE;
    
    CORBA::TypeCode_var t = constDesc_ptr->type;
    if(t->kind() == CORBA::tk_ushort)
      {
	CORBA::UShort constVal;
	if (constDesc_ptr->value >>= constVal)
	  return ((constVal == 88) &&
		  (strcmp( (const char*)constDesc_ptr->name, "C1") == 0));
      }
    if(t->kind() == CORBA::tk_ulong)
      {
	CORBA::ULong constVal;
	if (constDesc_ptr->value >>= constVal)
	  return ((constVal == 999) &&
		  (strcmp( (const char*)constDesc_ptr->name, "C1") == 0));
      }
  }
  return FALSE;
}


int same_type(VSConstantDef_ptr *o1, VSAny_ptr vs_value)
{
  CORBA::ConstantDef_ptr *tc = (CORBA::ConstantDef_ptr *) o1;
  CORBA::Any* av = (CORBA::Any *) vs_value;
  CORBA::TypeCode_var t1 = (*tc)->type();
  CORBA::TypeCode_var t2 = av->type();
  int res = t1->equal(t2);
  return res;
}

int value_correct(VSConstantDef_ptr *o1, VSAny_ptr vs_value)
{
  CORBA::ConstantDef_ptr *tc = (CORBA::ConstantDef_ptr *) o1;
  CORBA::Any *tav = (*tc)->value();
  CORBA::Any *av = (CORBA::Any *) vs_value;
  CORBA::TypeCode_var t1 = tav->type();
  CORBA::TypeCode_var t2 = av->type();
  int res = false;
  if(t1->equal(t2))
    {
      CORBA::UShort sh1, sh2;
      if((*av >>= sh1) && (*tav >>= sh2))
	if (sh1 == sh2)
	  res = true;
    }
  return res;
}

int value_unchanged(VSConstantDef_ptr *o1, VSConstantDef_ptr *o2)
{
  CORBA::ConstantDef_ptr *tc1 = (CORBA::ConstantDef_ptr *) o1;
  CORBA::Any *av1 = (*tc1)->value();
  CORBA::TypeCode_var t1 = av1->type();

  CORBA::ConstantDef_ptr *tc2 = (CORBA::ConstantDef_ptr *) o2;
  CORBA::Any *av2 = (*tc2)->value();
  CORBA::TypeCode_var t2 = av2->type();

  int res = false;
  if(t1->equal(t2))
    {
      CORBA::UShort sh1, sh2;
      if((*av1 >>= sh1) && (*av2 >>= sh2))
	if (sh1 == sh2)
	  res = true;
    }
  return res;
}

/*------------------------------------------------------*/
}
