//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

/* ------------------------------------------------------------ */
/* object_ptr is provided for the VSnarrow() test */
int
provide_object_ptr (VSObject_ptr **testvar,
		    enum prop_object_ptr_objValue objValue)
{
  CORBA::Object_ptr tvar;
  CORBA::Object_ptr *rptr;
  CORBA::Repository_ptr IR_ptr;
  IRSupport *irs = glob_TS.repository();
  
  switch (objValue)  {
  case Pobject_ptr_objValue_valid:
    IR_ptr = irs->get_repository();
    tvar = irs->get_contained(IR_ptr,"M_Container::M_ConstantDef::C1");
    if (!tvar)
      return FALSE;
    break;
  case Pobject_ptr_objValue_null:
    tvar = NULL;
    break;
  }
  rptr = new CORBA::Object_ptr(tvar);
  *testvar = (VSObject_ptr *) rptr;
  return TRUE;
}
  
  int
  relinquish_object_ptr (VSObject_ptr **testvar,
			 enum prop_object_ptr_objValue objValue)
  {
    CORBA::Object_ptr* tvar = (CORBA::Object_ptr *) *testvar;
    switch (objValue)  {
    case Pobject_ptr_objValue_valid:
      delete tvar;
    case Pobject_ptr_objValue_null:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* constantdef_ptr is provided for the VSduplicate() test. */
  int
  provide_constantdef_ptr (VSConstantDef_ptr **testvar,
			   enum prop_constantdef_ptr_constdefValue constdefValue)
  {
    CORBA::ConstantDef_ptr tvar;
    CORBA::ConstantDef_ptr *rptr;
    
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr ctnD_ptr;
    IRSupport *irs = glob_TS.repository();
    
    switch (constdefValue)  {
    case Pconstantdef_ptr_constdefValue_valid:
      IR_ptr = irs->get_repository();
      ctnD_ptr = irs->get_contained(IR_ptr,"M_ConstantDef::C1");
      if (!ctnD_ptr)
	return FALSE;
      tvar = CORBA::ConstantDef::_narrow(ctnD_ptr);
      break;
    case Pconstantdef_ptr_constdefValue_null:
      tvar = NULL;
      break;
    }
    rptr = new CORBA::ConstantDef_ptr(tvar);
    *testvar = (VSConstantDef_ptr *) rptr;
    return TRUE;
  }
  
  int
  relinquish_constantdef_ptr (VSConstantDef_ptr **testvar,
			      enum prop_constantdef_ptr_constdefValue constdefValue)
  {
    CORBA::ConstantDef_ptr* tvar = (CORBA::ConstantDef_ptr *) *testvar;
    switch (constdefValue)  {
    case Pconstantdef_ptr_constdefValue_valid:
      delete tvar;
    case Pconstantdef_ptr_constdefValue_null:
      break;
    }
    return TRUE;
  }
  
  
  /* ------------------------------------------------------------ */
  /* const_value is provided for the VSvalue_write() test. */
  int
  provide_const_value (VSAny_ptr *testvar, 
		       enum prop_const_value_val valType)
  {
    CORBA::Any* rptr;
    CORBA::Any toPass;
    
    
    switch (valType) {
    case Pconst_value_val_same_type:
      CORBA::UShort s;
      s = 88;
      toPass <<= s;
      rptr = new CORBA::Any(toPass);
      break;
    case Pconst_value_val_diff_type:
      CORBA::ULong l;
      l = 999;
      toPass <<= l;
      rptr = new CORBA::Any(toPass);
      break;
    }
    *testvar = (VSAny_ptr) rptr;
    return TRUE;
  }
  
  int
  relinquish_const_value (VSAny_ptr *testvar, 
			  enum prop_const_value_val val)
  {
    CORBA::Any* tvar = (CORBA::Any *) *testvar;
    switch (val) {
    case Pconst_value_val_same_type:
    case Pconst_value_val_diff_type:
      delete tvar;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* type is provided for the VStype_def_write() test. */
  int
  provide_type (VSIDLType_ptr *testvar)
  {
    CORBA::IDLType_ptr *tvar = (CORBA::IDLType_ptr *) testvar;
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr ctnD_ptr;
    
    IRSupport *irs = glob_TS.repository();
    
    try {
      IR_ptr = irs->get_repository();
      *tvar = IR_ptr->get_primitive(CORBA::pk_ulong);
    }
    catch (...) {
      return FALSE;
    }
    
    return TRUE;
  }
  
  int
  relinquish_type (VSIDLType_ptr *testvar)
  {
    return TRUE;
  }

/* ------------------------------------------------------------ */

}
