
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include	<corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/
/* READ INTERFACE */
/*------------------------------------------------------*/
/* id() - read */

VSRepositoryId 
VSid_read(VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;    
    CORBA::Repository_ptr	IR_ptr;
    CORBA::Contained_ptr		ctnD_ptr;
    CORBA::RepositoryId		repId;
		IRSupport *irs = glob_TS.repository();
    
    try {
        IR_ptr = irs->get_repository();
        ctnD_ptr = irs->get_contained(IR_ptr,"M_Contained::M_id");
				if (!ctnD_ptr)
						return (VSRepositoryId) 0;
				//read the repository id.	
				repId = ctnD_ptr->id();
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
    }
		return repId;
}

/*------------------------------------------------------*/
/* name() - read */
/* name attribute identifies the object uniquely within the containing object */

VSIdentifier 
VSname_read(VSEnvironment *ev, VSString vs_contained_name)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::Contained_ptr		ctnD_ptr;
		CORBA::Repository_ptr IR_ptr;
    CORBA::Identifier objName;	
		char* str_contained_name = (char *) vs_contained_name;
		
		IRSupport *irs = glob_TS.repository();
    
    try {
				IR_ptr = irs->get_repository();
				ctnD_ptr = irs->get_contained(IR_ptr, str_contained_name);
				if (!ctnD_ptr)
						return (VSIdentifier) 0;
				objName = ctnD_ptr->name();
    }
    catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
	    			env->exception(t);
    }
    catch (...) {
				return (VSIdentifier) 0;
    }
    return (VSIdentifier) objName;
}

/*------------------------------------------------------*/
/* version() - read */
/* version attribute distinguishes object from other versioned objects with the same name */

VSVersionSpec 
VSversion_read(VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::Object_ptr			obj_ptr;
    CORBA::Repository_ptr	IR_ptr;
    CORBA::Contained_ptr		ctnD_ptr;
		IRSupport *irs = glob_TS.repository();
    CORBA::VersionSpec objVersion;	
    
    try {
        IR_ptr = irs->get_repository();
        ctnD_ptr = irs->get_contained(IR_ptr,"M_Contained::M_name");
				if (!ctnD_ptr)
						return (VSVersionSpec) 0;
				objVersion = ctnD_ptr->version();
    }
		catch (CORBA::Exception &ex) {
		    CORBA::Exception *t = exception_convert(&ex);
		    if(env)
						env->exception(t);
		}
		catch (...) {
		    return (VSVersionSpec) 0;
		}
    
    return (VSVersionSpec) objVersion;
}

/*------------------------------------------------------*/
/* defined_in() - read */
/* defined_in attribute identifies the container within which the object is defined */
/* This test will only work if the get_contained() function uses lookup_name() rather
	 than lookup(). The later only looks up definitions and so the call to
	 get_contained() would fail on our object. */

VSContainer_ptr 
VSdefined_in(VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;    
    CORBA::Object_ptr			obj_ptr;    
    CORBA::Repository_ptr	IR_ptr;
    CORBA::Contained_ptr		ctnD_ptr;
    CORBA::Container_ptr cntr_ptr;	
    IRSupport *irs = glob_TS.repository();
    
    try {
        IR_ptr = irs->get_repository();
        ctnD_ptr = irs->get_contained(IR_ptr,"M_Contained::M_move::I1::C1");
				if (!ctnD_ptr)
						return (VSContainer_ptr) 0;
				cntr_ptr = ctnD_ptr->defined_in();
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
    }
    catch (...) {
	return (VSContainer_ptr) 0;
    }
    
    return (VSContainer_ptr) cntr_ptr;
}

/*------------------------------------------------------*/
/* absolute_name() - read */
/* absolute_name attribute is an absolute ScopedName that identifies a Contained object uniquely */
/* within its enclosing Repository */

VSScopedName 
VSabsolute_name(VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::Object_ptr			obj_ptr;
    CORBA::Repository_ptr	IR_ptr;
    CORBA::Contained_ptr		ctnD_ptr;
		IRSupport *irs = glob_TS.repository();
    CORBA::ScopedName absScope;	
    
    try {
        IR_ptr = irs->get_repository();
        ctnD_ptr = irs->get_contained(IR_ptr,"M_Contained::M_move::I3::C2");
				if(!ctnD_ptr)
						return (VSScopedName) 0;
				absScope = ctnD_ptr->absolute_name();
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
    }
    catch (...) {
	return (VSScopedName) 0;
    }

    return (VSScopedName) absScope;
}

/*------------------------------------------------------*/
/* containing_repository() - read */
/* containing_repository identifies the Repository that is eventually reached by recursively */
/* following the object's defined_in attribute */

VSRepository_ptr 
VScontaining_repository(VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::Object_ptr			obj_ptr;
    CORBA::Repository_ptr	IR_ptr;
    CORBA::Contained_ptr		ctnD_ptr;
		IRSupport *irs = glob_TS.repository();
    CORBA::Repository_ptr containingRep_ptr;	
    
    try {
        IR_ptr = irs->get_repository();
        ctnD_ptr = irs->get_contained(IR_ptr,"M_Contained::M_move::I3::C2");
				if(!ctnD_ptr)
						return (VSRepository_ptr) 0;
				containingRep_ptr = ctnD_ptr->containing_repository();
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
    }
    catch (...) {
	return (VSRepository_ptr) 0;
    }

    return (VSRepository_ptr) containingRep_ptr;
}

/*------------------------------------------------------*/
/* describe() - read */
/* returns a structure containing information about the interface */

VSDescription_ptr 
VSdescribe(VSEnvironment *ev) 
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::Object_ptr			obj_ptr;
    CORBA::Repository_ptr	IR_ptr;
    CORBA::Contained_ptr		cPtr;
		IRSupport *irs = glob_TS.repository();
    CORBA::Contained::Description* desc_ptr;
    
    try {
        IR_ptr = irs->get_repository();
        cPtr = irs->get_contained(IR_ptr,"M_Contained::M_move");
				if(!cPtr)
						return (VSDescription_ptr) 0;
				desc_ptr = cPtr->describe();
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
    }
    catch (...) {
	return (VSDescription_ptr) 0;
    }

    return (VSDescription_ptr) desc_ptr;
} 

/*------------------------------------------------------*/
VSContained_ptr*
VSnil(VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::Contained_ptr	ttc;
				 
		try {
        ttc = CORBA::Contained::_nil();
    }
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
            env->exception(t);
    }
		catch (...) {
        return (VSContained_ptr *) 0;
    }
		 
		CORBA::Contained_ptr *rtc = new CORBA::Contained_ptr(ttc);
		return (VSContained_ptr *) rtc;
}


/*------------------------------------------------------*/
VSContained_ptr*
VSduplicate(VSContained_ptr *o1, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::Contained_ptr *tc = (CORBA::Contained_ptr *) o1;
		CORBA::Contained_ptr ttc;
						 
		try {
        ttc = CORBA::Contained::_duplicate(tc ? *tc : (CORBA::Contained_ptr) 0);
    }
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
            env->exception(t);
    }
		catch (...) {
        return (VSContained_ptr *) 0;
    }
		 
		CORBA::Contained_ptr *rtc = new CORBA::Contained_ptr(ttc);
		return (VSContained_ptr *) rtc;
}

/*------------------------------------------------------*/
VSContained_ptr*
VSnarrow(VSObject_ptr *o1,  VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::Object_ptr *tc = (CORBA::Object_ptr *) o1;
  CORBA::Contained_ptr ttc;
  
  try {
    ttc = CORBA::Contained::_narrow(*tc);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  catch (...) {
    return (VSContained_ptr *) 0;
  }
  
  CORBA::Contained_ptr *rtc = new CORBA::Contained_ptr(ttc);
  return (VSContained_ptr *) rtc;
}

/*------------------------------------------------------*/
/* WRITE INTERFACE */
/*------------------------------------------------------*/
/* id() - write */
/* setting the id of a Contained object. This is called
	 twice - once with a new unique id and once with an id
	 that already exists in the Repository */
void 
VSid_write(const VSRepositoryId rId, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::Object_ptr			obj_ptr;
    CORBA::Repository_ptr	IR_ptr;
    CORBA::Contained_ptr		ctnD_ptr;
		IRSupport *irs = glob_TS.repository();
    CORBA::RepositoryId repId = (CORBA::RepositoryId) rId;

    try {
        IR_ptr = irs->get_repository();
        ctnD_ptr = irs->get_contained(IR_ptr,"M_Contained::M_id");
				if (!ctnD_ptr)
						return;
				//assign the id to the new Repository Id.	
				ctnD_ptr->id(repId);
		}
    catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
	    			env->exception(t);
    }
}


/*------------------------------------------------------*/
/* name() - write */

void 
VSname_write(const VSIdentifier name, VSEnvironment *ev, VSString vs_contained_name)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::Contained_ptr		ctnD_ptr;
		CORBA::Repository_ptr IR_ptr;
    CORBA::Identifier new_name = (CORBA::Identifier) name;
		char* str_contained_name = (char *) vs_contained_name;

		IRSupport *irs = glob_TS.repository();
    
    try {
				IR_ptr = irs->get_repository();
				ctnD_ptr = irs->get_contained(IR_ptr, str_contained_name);
				if (!ctnD_ptr)
						return;
				ctnD_ptr->name(new_name);
    }
    catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
	    			env->exception(t);
    }
}


/*------------------------------------------------------*/
/* version() - write */

void 
VSversion_write(const VSVersionSpec spec, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::Object_ptr			obj_ptr;
    CORBA::Repository_ptr	IR_ptr;
    CORBA::Contained_ptr		ctnD_ptr;
		IRSupport *irs = glob_TS.repository();
    CORBA::VersionSpec vspec = (CORBA::VersionSpec) spec;
    
    try {
        IR_ptr = irs->get_repository();
        ctnD_ptr = irs->get_contained(IR_ptr,"M_Contained::M_name");
				if (! ctnD_ptr)
						return;
				ctnD_ptr->version(vspec);
    }
    catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
	   			 env->exception(t);
    }
}


/*------------------------------------------------------*/
/* move() - write */
/* removes an object from its current Container and adds it to 
	 the one denoted by new_container.
	 Here we are attempting to move an OperationDef to three different
	 locations. This move operation cannot be invoked on the same source object twice
	 as it will no longer exist at the original location. So on each invocation of 
	 VSmove() we update the name of the source and so are moving 3 different source
	 objects: O1, O2, and O3. (this is a way to get around the fact that adl would 
	 normally try to process the X-product of our 3 source and 3 destinations - so 
	 invoking move() on a source object more than once) */

void 
VSmove(const VSContainer_ptr container, 
				const VSIdentifier name, 
				const VSVersionSpec version,
				VSEnvironment *ev,
				VSString moveObj)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::Object_ptr			obj_ptr;
    CORBA::Repository_ptr	IR_ptr;
    CORBA::Contained_ptr		cPtrOld;
		IRSupport *irs = glob_TS.repository();
    CORBA::Container_ptr new_container = (CORBA::Container_ptr) container;
    CORBA::Identifier new_name = (CORBA::Identifier) name;
    CORBA::VersionSpec new_version = (CORBA::VersionSpec) version;
		static int i = 0;
    
    try {
        IR_ptr = irs->get_repository();
				char opname[50];
				strcpy(opname, (char *) moveObj);
				opname[strlen(opname) - 1] += i++;
        cPtrOld = irs->get_contained(IR_ptr,opname);
				if (!cPtrOld)
						return;
				char movedName[20];
				new_name = cPtrOld->name();
				cPtrOld->move(new_container, new_name, new_version);
    }
	catch (CORBA::Exception &ex) {
	    CORBA::Exception *t = exception_convert(&ex);
	    if(env)
					env->exception(t);
	}
	catch (...) {
			return;
	}
}


/*------------------------------------------------------*/

}

