//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>


int is_same_string(VSString s1, VSString s2)
{
		if((s1 == NULL) && (s2 == NULL)) return TRUE;
		if((s1 == NULL) || (s2 == NULL)) return FALSE;
		return (strcmp(s1,s2) == 0) ? TRUE : FALSE;
}

/*------------------------------------------------------*/
int is_nil_object(VSObject_ptr o1)
{
		CORBA::Object_ptr obj1 = (CORBA::Object_ptr) o1;
		 
		if(!obj1)
				return TRUE;
		return CORBA::is_nil(obj1);
}

/*------------------------------------------------------*/
int
is_nil_contained_ptr(VSContained_ptr *o1)
{
		CORBA::Contained_ptr *tc = (CORBA::Contained_ptr *) o1;
		if(!tc)
        return TRUE;
		 return CORBA::is_nil(*tc);
}

/*------------------------------------------------------*/
int cleanup_contained_ptr(VSContained_ptr *o1)
{
		CORBA::Container_ptr *tc = (CORBA::Container_ptr *) o1;
		if(!tc)
			return TRUE;
		delete tc;
		return TRUE;
}

/*------------------------------------------------------*/
//!! finish this
int
is_same_contained_ptr(VSContained_ptr *o1, VSContained_ptr *o2)
{
    if(is_nil_contained_ptr(o1) && is_nil_contained_ptr(o2))
						return TRUE;
		return TRUE;
}

/*------------------------------------------------------*/
int
id_exists(VSRepositoryId vs_id)
{
	if (strstr((char *)vs_id, "new"))
			return FALSE;
	return TRUE;
}

/*------------------------------------------------------*/
int
nameclash(VSContainer_ptr vs_ctnR)
{
		CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) vs_ctnR;
		CORBA::IRObject_ptr obj_ptr = ctnR_ptr;
		CORBA::Contained_ptr ctnD_ptr = CORBA::Contained::_narrow(obj_ptr);
		if(!ctnD_ptr)
				return FALSE;
		VSIdentifier name = ctnD_ptr->name();
		if(!name)
				return FALSE;
		if (strcmp((char *) name, "Iname_clash") == 0)
				return TRUE;
		return FALSE;
}

/*------------------------------------------------------*/
int
inherited_obj(VSString objName)
{
		if(strstr((char *) objName, "C1"))
				return TRUE;
		return FALSE;
}

/*------------------------------------------------------*/
/* a module is not allowed contain an operation according
	 to the containment rules */
int
violates_containment(VSContainer_ptr vs_ctnR)
{
		CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) vs_ctnR;
		CORBA::IRObject_ptr obj_ptr = ctnR_ptr;
		CORBA::Contained_ptr ctnD_ptr = CORBA::Contained::_narrow(obj_ptr);
		VSIdentifier name = ctnD_ptr->name();

		if(!name)
				return FALSE;
		if(strcmp((char *) name, "M_move") == 0) 
				return TRUE;
		return FALSE;
}

/*------------------------------------------------------*/
/* We are testing the describe() function on the subtype
	 ModuleDef (see t1.cpp) */

int is_correct_description(VSDescription_ptr vs_desc_ptr)
{
		CORBA::Contained::Description* desc_ptr =
				(CORBA::Contained::Description *) vs_desc_ptr;
		if (!desc_ptr)
				return FALSE;

		if (desc_ptr->kind != CORBA::dk_Module)
				return FALSE;

		const CORBA::ModuleDescription* modDesc_ptr;
		if (desc_ptr->value >>= modDesc_ptr) {
				if (!modDesc_ptr)
						return FALSE;
				if (!modDesc_ptr->name)
						return FALSE;
				return (strcmp( (char*)modDesc_ptr->name, "M_move") == 0);
		}
		return FALSE;
}

/*------------------------------------------------------*/
int
is_correct_name(VSIdentifier vs_id, VSString vs_name)
{
		char* id = (char *) vs_id;
		char *name = (char *) vs_name;
		char* tmp = strtok(name, "::");
		char* lastToken = tmp;
		while (tmp != NULL) {
				lastToken = tmp;
				tmp = strtok(NULL, "::");
		}
		if(!id)
				return 0;
		return (strcmp(id, lastToken) ==0);
}

/*------------------------------------------------------*/
/* In our sample idl we know the Container in which our
	 test interface is defined. */

int
is_correct_defined_in(VSContainer_ptr vs_ctnR_ptr)
{
		CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) vs_ctnR_ptr;
		CORBA::IRObject_ptr irobj_ptr = ctnR_ptr;
		CORBA::Contained_ptr ctnD_ptr;
		if (ctnD_ptr = CORBA::Contained::_narrow(irobj_ptr)) {
				if(!ctnD_ptr)
						return FALSE;
				char* s = (char *) ctnD_ptr->name();
				if(!s)
						return FALSE;
				return (strcmp(s, "I1") == 0);
		}
		return FALSE;
}

/*------------------------------------------------------*/
int
is_correct_absolute_name(VSScopedName vs_scoped_name)
{	
		char* scoped_name = (char *) vs_scoped_name;
		if(!scoped_name)
				return 0;
		return (strcmp(scoped_name,"::M_Contained::M_move::I3::C2") == 0);
}

/*------------------------------------------------------*/
int 
is_correct_containing_repository(VSRepository_ptr vs_ir_ptr)
{
		CORBA::Repository_ptr IR1_ptr = (CORBA::Repository_ptr) vs_ir_ptr;
		IRSupport *irs = glob_TS.repository();
		CORBA::Repository_ptr IR2_ptr = irs->get_repository();

		return IR1_ptr->_is_equivalent(IR2_ptr);
}

/*------------------------------------------------------*/
}
