//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

/* ------------------------------------------------------------ */
/* globId is provided for the VSid_write() test. We provide a unique
	 id and also an id which already exists in the repository. */

int 
provide_globId (VSRepositoryId *testvar, 
		enum prop_globId_globId_value globId_value)

{
    CORBA::RepositoryId *tvar = (CORBA::RepositoryId *) testvar;

		CORBA::Repository_ptr  IR_ptr;
		CORBA::Contained_ptr   ctnD_ptr;
		IRSupport *irs = glob_TS.repository();
		IR_ptr = irs->get_repository();
    
    switch(globId_value) {
				case PglobId_globId_value_already_exists:
				    ctnD_ptr = irs->get_contained(IR_ptr,"M_Contained::M_id::C1");
						if (!ctnD_ptr)
								return FALSE;
						*tvar = ctnD_ptr->id();
	   			  break;
				case PglobId_globId_value_not_exists:
						*tvar = (CORBA::RepositoryId) "new_contained_id";
				    break;
		}
    return TRUE;
}

int 
relinquish_globId (VSRepositoryId *testvar, 
		   enum prop_globId_globId_value globId_value)
{
    switch(globId_value) {
	case PglobId_globId_value_not_exists:
	    case PglobId_globId_value_already_exists:
		break;
	    }
    return TRUE;
    
}


/* ------------------------------------------------------------ */
/* moveObjName provides the name of the object we are moving in the
	 VSmove() operation. We provide an OperationDef to be moved.
	 See comment on VSmove() in t1.cpp on how the VSmove() is tested
	 over three different OperationDefs in our Repository */

int 
provide_moveObjName (VSString *testvar)
{
    CORBA::Identifier *tvar = (CORBA::Identifier *) testvar;
    
		*tvar = CORBA::string_dup("M_Contained::M_move::I3::O1");
    return TRUE;
}


int 
relinquish_moveObjName (VSString *testvar)
{
  return TRUE;
}

/* ------------------------------------------------------------ */
/* name_write_contained provides the name of the object we are 
	 writing in the VSname_write() operation. */

int 
provide_name_write_contained (VSString *testvar)
{
    CORBA::Identifier *tvar = (CORBA::Identifier *) testvar;
    
		*tvar = CORBA::string_dup("M_Contained::M_name::C_to_change");
    return TRUE;
}


int 
relinquish_name_write_contained (VSString *testvar)
{
  return TRUE;
}
/* ------------------------------------------------------------ */
/* name_read_contained provides the name of the object we are 
	 reading in the VSname_read() operation. */

int 
provide_name_read_contained (VSString *testvar)
{
    CORBA::Identifier *tvar = (CORBA::Identifier *) testvar;
    
		*tvar = CORBA::string_dup("M_Contained::M_name");
    return TRUE;
}


int 
relinquish_name_read_contained (VSString *testvar)
{
  return TRUE;
}

/* ------------------------------------------------------------ */
/* container_ptr is provided to test the VSmove() operation. We provide
	 a new container for the object we are moving. Three different 
	 containers are provided - one which can hold an OperationDef (it is 
	 an OperationDef which we are moving), one which cannot hold an OperationDef,
	 and one which already contains an object of the same name as the OperationDef
	 we are moving. */
int 
provide_container_ptr (VSContainer_ptr *testvar, 
		       enum prop_container_ptr_container_value container_value)
{
    CORBA::Container_ptr *tvar = (CORBA::Container_ptr *) testvar;
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Object_ptr      obj_ptr;
    
		IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();

    switch(container_value) {
			case Pcontainer_ptr_container_value_canhold_oper:
      		obj_ptr = irs->get_contained(IR_ptr,"M_Contained::M_move::I2");
	    		break;
	    case Pcontainer_ptr_container_value_cannothold_oper:
      		obj_ptr = irs->get_contained(IR_ptr,"M_Contained::M_move");
	    		break;
	    case Pcontainer_ptr_container_value_nameclash:
      		obj_ptr = irs->get_contained(IR_ptr,"M_Contained::M_move::Iname_clash");
	    		break;
		}
		if(!obj_ptr)
				return FALSE;
		*tvar = CORBA::Container::_narrow(obj_ptr);
    return TRUE;
}


int 
relinquish_container_ptr (VSContainer_ptr *testvar, 
			  enum prop_container_ptr_container_value container_value)
{
    switch(container_value) {
			case Pcontainer_ptr_container_value_canhold_oper:
	    case Pcontainer_ptr_container_value_cannothold_oper:
			case Pcontainer_ptr_container_value_nameclash:
		    break;
		}
    return TRUE;
    
}


/* ------------------------------------------------------------ */
/* test variable VSContained_ptr contained_ptr(prop cdValue) 
	 used for the VSduplicate() test */
int 
provide_contained_ptr (VSContained_ptr **testvar, 
		       enum prop_contained_ptr_cdValue cdValue)
{
    CORBA::Contained_ptr tvar;
    CORBA::Contained_ptr *rptr;
    CORBA::Object_ptr      obj_ptr;
    CORBA::Repository_ptr  IR_ptr;
		IRSupport *irs = glob_TS.repository();

    switch (cdValue) {
				case Pcontained_ptr_cdValue_valid:
        		IR_ptr = irs->get_repository();
        		tvar = irs->get_contained(IR_ptr,"M_Contained");
						if(! tvar)
								return FALSE;
	    			break;
	    		case Pcontained_ptr_cdValue_null:
						tvar = NULL;
	    			break;
		}
		rptr = new CORBA::Contained_ptr(tvar);
    *testvar = (VSContained_ptr *) rptr;
    return TRUE;
}

int 
relinquish_contained_ptr (VSContained_ptr **testvar, 
													enum prop_contained_ptr_cdValue cdValue)
{
		CORBA::Contained_ptr* tvar = (CORBA::Contained_ptr *) *testvar;
		switch (cdValue) {
				case Pcontained_ptr_cdValue_valid:
				case Pcontained_ptr_cdValue_null:
				break;
		}
		return TRUE;
}
	
	 
/* ------------------------------------------------------------ */
/* test variable VSObject_ptr object_ptr(prop objValue) 
	 used for the VSnarrow() test. */
int 
provide_object_ptr (VSObject_ptr **testvar, 
		    enum prop_object_ptr_objValue objValue)
{
  CORBA::Object_ptr *tvar;
  CORBA::Object_ptr obj_ptr;
  CORBA::Repository_ptr IR_ptr;
  IRSupport *irs = glob_TS.repository();
  
  switch (objValue)  {
  case Pobject_ptr_objValue_valid:
    IR_ptr = irs->get_repository();
    obj_ptr = irs->get_contained(IR_ptr,"M_Contained::M_move::M1::I1");
    tvar = new CORBA::Object_ptr(obj_ptr);
    if(CORBA::is_nil(*tvar))
      return FALSE;
    break;
    
  case Pobject_ptr_objValue_null:
    CORBA::Object_ptr p = CORBA::Object::_nil();
    tvar = new CORBA::Object_ptr(p);
    break;
  }
  
  *testvar = (VSObject_ptr *) tvar;
  return TRUE;
}

int 
relinquish_object_ptr (VSObject_ptr **testvar, 
												 enum prop_object_ptr_objValue objValue)
{
		switch (objValue)  {
				case Pobject_ptr_objValue_valid:
				case Pobject_ptr_objValue_null: 
						break;
		}
		return TRUE;
}




/* ------------------------------------------------------------ */

}







