/*
 Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
 Copyright (c) 2001 Object Oriented Concepts GmbH.

 All rights reserved. No part of this source code may be reproduced,
 stored in a retrieval system, or transmitted, in any form or by any
 means, electronic, mechanical, photocopying, recording or otherwise,
 except as stated in the end-user licence agreement, without the prior
 permission of the copyright owners.

 Developed for X/Open by ApTest Ireland Limited.

 Project: VSORB

 * Copyright (c) 2001                                               
 * GMD FOKUS                                                        
 * Project: CORVAL2

*/


#include	<corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/
/* READ INTERFACE */
/*------------------------------------------------------*/

/* lookup() */

VSContained_ptr 
VSlookup(const VSScopedName search_name, VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::Repository_ptr	IR_ptr;
  CORBA::Contained_ptr		ctnD_ptr;
  
  CORBA::ScopedName searchName = (CORBA::ScopedName ) search_name;
  IRSupport *irs = glob_TS.repository();
  
  try {
    IR_ptr = irs->get_repository();
    ctnD_ptr = IR_ptr->lookup(searchName);
  }
  catch (CORBA::Exception &ex) {
    cout << "VSlookup: "<< ex <<" raised for search name '"
	 <<searchName<<"'."<<endl;
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSContained_ptr) ctnD_ptr;
}
  
/*------------------------------------------------------*/
/* contents() */

VSContainedSeq_ptr
VScontents(VSDefinitionKind limit_type, VSBoolean exclude_inherited, VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  
  CORBA::Object_ptr			obj_ptr;
  CORBA::Repository_ptr	IR_ptr;
  CORBA::Container_ptr		ctnR_ptr;
  CORBA::Contained_ptr		ctnD_ptr;
  CORBA::InterfaceDef_ptr	IntFace_ptr;
  
  CORBA::DefinitionKind limitType = (CORBA::DefinitionKind) limit_type;
  CORBA::Boolean excInherited = (CORBA::Boolean) exclude_inherited;
  CORBA::ContainedSeq* contents_ptr;	
  IRSupport *irs = glob_TS.repository();
  
  try {
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_contents::I2");
    if(!ctnD_ptr) 
      return (VSContained_ptr) 0;
    IntFace_ptr = CORBA::InterfaceDef::_narrow(ctnD_ptr);
    ctnR_ptr = IntFace_ptr;
    contents_ptr = ctnR_ptr->contents(limitType, excInherited);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSContainedSeq_ptr) contents_ptr;
}

/*------------------------------------------------------*/
/* lookup_name() */

VSContainedSeq_ptr 
VSlookup_name(const VSIdentifier search_name, 
	      VSLong levels_to_search, 
	      VSDefinitionKind limit_type,
	      VSBoolean exclude_inherited,
	      VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  
  CORBA::Object_ptr			obj_ptr;
  CORBA::Repository_ptr	IR_ptr;
  CORBA::Contained_ptr 	ctnD_ptr;
  CORBA::ModuleDef_ptr		Module_ptr;
  CORBA::Container_ptr		ctnR_ptr;
  
  CORBA::Identifier searchName = (CORBA::Identifier) search_name;
  CORBA::DefinitionKind limitType = (CORBA::DefinitionKind) limit_type;
  CORBA::Long levelsToSearch = (CORBA::Long) levels_to_search;
  CORBA::Boolean excInherited = (CORBA::Boolean) exclude_inherited;
  CORBA::Identifier objName;	
  CORBA::ContainedSeq* matchingObjects_ptr;		
  IRSupport *irs = glob_TS.repository();

  try {
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_lookup_name::M1::M_search_from");
    if(!ctnD_ptr) 
      return (VSContainedSeq_ptr) 0;
    Module_ptr = CORBA::ModuleDef::_narrow(ctnD_ptr);
    ctnR_ptr = Module_ptr;
    matchingObjects_ptr = ctnR_ptr->lookup_name(searchName, levelsToSearch, limitType, excInherited);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSContainedSeq_ptr) matchingObjects_ptr;
}

/*------------------------------------------------------*/
/* describe_contents() */

VSDescriptionSeq_ptr 
VSdescribe_contents(VSDefinitionKind vs_limit_type,
		    VSBoolean vs_exclude_inherited,
		    VSLong vs_max_returned_objs,
		    VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  
  CORBA::Object_ptr			obj_ptr;
  CORBA::Repository_ptr	IR_ptr;
  CORBA::Container_ptr    ctnR_ptr;
  CORBA::Contained_ptr    ctnD_ptr;
  CORBA::InterfaceDef_ptr IntFace_ptr;
  
  CORBA::DefinitionKind limitType = (CORBA::DefinitionKind) vs_limit_type;
  CORBA::Long maxReturnedObjects = (CORBA::Long) vs_max_returned_objs;
  CORBA::Boolean excInherited = (CORBA::Boolean) vs_exclude_inherited;
  VSDescriptionSeq_ptr contentsDescription_ptr;
  IRSupport *irs = glob_TS.repository();
  
  try {
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_contents::I2");
    if(!ctnD_ptr)
      return (VSContained_ptr) 0;
    IntFace_ptr = CORBA::InterfaceDef::_narrow(ctnD_ptr);
    ctnR_ptr = IntFace_ptr;
    contentsDescription_ptr = ctnR_ptr->describe_contents(limitType,
							  excInherited, maxReturnedObjects);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSDescriptionSeq_ptr) contentsDescription_ptr;
}


/*------------------------------------------------------*/
VSContainer_ptr*
VSnil(VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::Container_ptr ttc;
  
  try {
    ttc = CORBA::Container::_nil();
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  catch (...) {
    return (VSContainer_ptr *) 0;
  }
  CORBA::Container_ptr *rtc = new CORBA::Container_ptr(ttc);
  return (VSContainer_ptr *) rtc;
}

/*------------------------------------------------------*/
VSContainer_ptr*
VSduplicate(VSContainer_ptr *o1, VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  
  CORBA::Container_ptr *tc = (CORBA::Container_ptr *) o1;
  CORBA::Container_ptr ttc;
  
  try {
    ttc = CORBA::Container::_duplicate(tc ? *tc : (CORBA::Container_ptr) 0);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  catch (...) {
    return (VSContainer_ptr *) 0;
  }
  
  CORBA::Container_ptr *rtc = new CORBA::Container_ptr(ttc);
  return (VSContainer_ptr *) rtc;
}

/*------------------------------------------------------*/
VSContainer_ptr*
VSnarrow(VSObject_ptr *o1, VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;

  CORBA::Object_ptr *tc = (CORBA::Object_ptr *) o1;
  CORBA::Container_ptr ttc;
  
  try {
    ttc = CORBA::Container::_narrow(tc ? *tc : (CORBA::Container_ptr) 0);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  catch (...) {
    return (VSContainer_ptr *) 0;
  }
  CORBA::Container_ptr *rtc = new CORBA::Container_ptr(ttc);
  return (VSContainer_ptr *) rtc;
}

/*------------------------------------------------------*/
/* WRITE INTERFACE */
/*------------------------------------------------------*/

VSModuleDef_ptr
VScreate_module(const VSRepositoryId vs_id, 
		const VSIdentifier vs_name,  
		const VSVersionSpec vs_vspec,
		VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::RepositoryId id = (CORBA::RepositoryId) vs_id;
  CORBA::Identifier name = (CORBA::Identifier) vs_name;
  CORBA::VersionSpec vspec = (CORBA::VersionSpec) vs_vspec;
  
  IRSupport *irs = glob_TS.repository();
  CORBA::ModuleDef_ptr new_module_ptr;
  
  try {
    CORBA::Repository_ptr IR_ptr = irs->get_repository();
    CORBA::IRObject_ptr irObj_ptr = irs->get_contained(IR_ptr,"M_Container::M_write");
    if (!irObj_ptr)
      return (VSModuleDef_ptr) 0;
    CORBA::Container_ptr ctnR_ptr = CORBA::Container::_narrow(irObj_ptr);
    new_module_ptr = ctnR_ptr->create_module(id, name, vspec);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSModuleDef_ptr) new_module_ptr;
}

/*------------------------------------------------------*/
VSConstantDef_ptr
VScreate_constant(const VSRepositoryId vs_id,
		  const VSIdentifier vs_name,
		  const VSVersionSpec vs_vspec,
		  const VSIDLType_ptr vs_type,
		  const VSAny_ptr vs_value,
		  VSEnvironment *ev,
		  VSContainer_ptr calling_container_ptr)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::RepositoryId id = (CORBA::RepositoryId) vs_id;
  CORBA::Identifier name = (CORBA::Identifier) vs_name;
  CORBA::VersionSpec vspec = (CORBA::VersionSpec) vs_vspec;
  CORBA::IDLType_ptr type = (CORBA::IDLType_ptr) vs_type;
  CORBA::Any* value = (CORBA::Any *) vs_value;
  
  CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) calling_container_ptr;
  
  CORBA::ConstantDef_ptr new_constant_ptr;
  
  try {
    new_constant_ptr = ctnR_ptr->create_constant(id, name, vspec, type, *value);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSConstantDef_ptr) new_constant_ptr;
}

/*------------------------------------------------------*/
VSStructDef_ptr
VScreate_struct(const VSRepositoryId vs_id,
		const VSIdentifier vs_name,
		const VSVersionSpec vs_vspec,
		const VSStructMemberSeq_ptr vs_members,
		VSEnvironment *ev,
		VSContainer_ptr calling_container_ptr)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::RepositoryId id = (CORBA::RepositoryId) vs_id;
  CORBA::Identifier name = (CORBA::Identifier) vs_name;
  CORBA::VersionSpec vspec = (CORBA::VersionSpec) vs_vspec;
  CORBA::StructMemberSeq* members = (CORBA::StructMemberSeq *) vs_members;
  
  CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) calling_container_ptr;
  
  CORBA::StructDef_ptr new_struct_ptr;
  
  try {
    new_struct_ptr = ctnR_ptr->create_struct(id, name, vspec, *members);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSStructDef_ptr) new_struct_ptr;
}

/*------------------------------------------------------*/
VSUnionDef_ptr
VScreate_union(const VSRepositoryId vs_id,
	       const VSIdentifier vs_name,
	       const VSVersionSpec vs_vspec,
	       const VSIDLType_ptr vs_discriminator_type,
	       const VSUnionMemberSeq_ptr vs_members,
	       VSEnvironment *ev,
	       VSContainer_ptr calling_container_ptr)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::RepositoryId id = (CORBA::RepositoryId) vs_id;
  CORBA::Identifier name = (CORBA::Identifier) vs_name;
  CORBA::VersionSpec vspec = (CORBA::VersionSpec) vs_vspec;
  CORBA::IDLType_ptr discriminator_type = (CORBA::IDLType_ptr) vs_discriminator_type;
  CORBA::UnionMemberSeq* members = (CORBA::UnionMemberSeq *) vs_members;
  
  CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) calling_container_ptr;
  
  CORBA::UnionDef_ptr new_union_ptr;
  
  try {
    new_union_ptr = ctnR_ptr->create_union(id, name, vspec, discriminator_type, *members);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSUnionDef_ptr) new_union_ptr;
}

/*------------------------------------------------------*/
VSEnumDef_ptr
VScreate_enum(const VSRepositoryId vs_id,
	      const VSIdentifier vs_name,
	      const VSVersionSpec vs_vspec,
	      const VSEnumMemberSeq_ptr vs_members,
	      VSEnvironment *ev,
	      VSContainer_ptr calling_container_ptr)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::RepositoryId id = (CORBA::RepositoryId) vs_id;
  CORBA::Identifier name = (CORBA::Identifier) vs_name;
  CORBA::VersionSpec vspec = (CORBA::VersionSpec) vs_vspec;
  CORBA::EnumMemberSeq* members = (CORBA::EnumMemberSeq *) vs_members;
  
  CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) calling_container_ptr;
  
  CORBA::EnumDef_ptr new_enum_ptr;
  
  try {
    new_enum_ptr = ctnR_ptr->create_enum(id, name, vspec, *members);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSEnumDef_ptr) new_enum_ptr;
}

/*------------------------------------------------------*/
VSAliasDef_ptr
VScreate_alias(const VSRepositoryId vs_id,
	       const VSIdentifier vs_name,
	       const VSVersionSpec vs_vspec,
	       const VSIDLType_ptr vs_original_type,
	       VSEnvironment *ev, 
	       VSContainer_ptr calling_container_ptr)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::RepositoryId id = (CORBA::RepositoryId) vs_id;
  CORBA::Identifier name = (CORBA::Identifier) vs_name;
  CORBA::VersionSpec vspec = (CORBA::VersionSpec) vs_vspec;
  CORBA::IDLType_ptr original_type = (CORBA::IDLType_ptr) vs_original_type;
  
  CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) calling_container_ptr;
  
  CORBA::AliasDef_ptr new_alias_ptr;
  
  try {
    new_alias_ptr = ctnR_ptr->create_alias(id, name, vspec, original_type);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSAliasDef_ptr) new_alias_ptr;
}

/*------------------------------------------------------*/
VSInterfaceDef_ptr
VScreate_interface(const VSRepositoryId vs_id,
		   const VSIdentifier vs_name,
		   const VSVersionSpec vs_vspec,
		   const VSInterfaceDefSeq_ptr vs_base_interfaces,
		   VSEnvironment *ev, 
		   VSContainer_ptr calling_container_ptr)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::RepositoryId id = (CORBA::RepositoryId) vs_id;
  CORBA::Identifier name = (CORBA::Identifier) vs_name;
  CORBA::VersionSpec vspec = (CORBA::VersionSpec) vs_vspec;
  CORBA::InterfaceDefSeq* base_interfaces = (CORBA::InterfaceDefSeq *) vs_base_interfaces;
  
  CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) calling_container_ptr;
  
  CORBA::InterfaceDef_ptr new_interface_ptr;
  
  try {
    new_interface_ptr = ctnR_ptr->create_interface(id, name, vspec, *base_interfaces);
  }
  catch(CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSInterfaceDef_ptr) new_interface_ptr;
}
/*------------------------------------------------------*/
VSExceptionDef_ptr
VScreate_exception(const VSRepositoryId vs_id,
		   const VSIdentifier vs_name,
		   const VSVersionSpec vs_vspec,
		   const VSStructMemberSeq_ptr vs_members,
		   VSEnvironment *ev, 
		   VSContainer_ptr calling_container_ptr)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::RepositoryId id = (CORBA::RepositoryId) vs_id;
  CORBA::Identifier name = (CORBA::Identifier) vs_name;
  CORBA::VersionSpec vspec = (CORBA::VersionSpec) vs_vspec;
  CORBA::StructMemberSeq* members = (CORBA::StructMemberSeq *) vs_members;
  
  CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) calling_container_ptr;
  
  CORBA::ExceptionDef_ptr new_exception_ptr;
  
  try {
    new_exception_ptr = ctnR_ptr->create_exception(id, name, vspec, *members);
  }
  catch(CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSExceptionDef_ptr) new_exception_ptr;
}

/*------------------------------------------------------*/
VSValueDef_ptr
VScreate_value(const VSRepositoryId vs_id,
	       const VSIdentifier vs_name,
	       const VSVersionSpec vs_vspec,
	       const VSBoolean is_custom,
	       const VSBoolean is_abstract,
	       const VSValueDef_ptr base_value,
	       const VSBoolean is_truncatable,
	       const VSValueDefSeq_ptr abstract_base_values,
	       const VSInterfaceDefSeq_ptr supported_interfaces,
	       const VSInitializerSeq_ptr initializers,
	       VSEnvironment *ev, 
	       VSContainer_ptr calling_container_ptr)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::RepositoryId id = (CORBA::RepositoryId) vs_id;
  CORBA::Identifier name = (CORBA::Identifier) vs_name;
  CORBA::VersionSpec vspec = (CORBA::VersionSpec) vs_vspec;
  CORBA::Boolean ic = (CORBA::Boolean) is_custom;
  CORBA::Boolean ia = (CORBA::Boolean) is_abstract;
  CORBA::ValueDef_ptr *bv = (CORBA::ValueDef_ptr *) base_value;
  CORBA::Boolean it = (CORBA::Boolean) is_truncatable;
  CORBA::ValueDefSeq *abv = (CORBA::ValueDefSeq *) abstract_base_values;
  CORBA::InterfaceDefSeq *sif = 
    (CORBA::InterfaceDefSeq *) supported_interfaces;
  CORBA::InitializerSeq *il = (CORBA::InitializerSeq *) initializers;
  CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) calling_container_ptr;
  
  CORBA::ValueDef_ptr new_value_ptr;
  
  try {
    new_value_ptr = ctnR_ptr->create_value(id, name, vspec, ic, ia, *bv,
					   it, *abv, *sif, *il);
  }
  catch(CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSValueDef_ptr) new_value_ptr;
}
  
/*----------------------------------------------------------------*/

VSValueBoxDef_ptr
VScreate_value_box(const VSRepositoryId vs_id,
		   const VSIdentifier vs_name,
		   const VSVersionSpec vs_vspec,
		   const VSIDLType_ptr original_type_def,
		   VSEnvironment *ev, 
		   VSContainer_ptr calling_container_ptr)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::RepositoryId id = (CORBA::RepositoryId) vs_id;
  CORBA::Identifier name = (CORBA::Identifier) vs_name;
  CORBA::VersionSpec vspec = (CORBA::VersionSpec) vs_vspec;
  CORBA::IDLType_ptr *otd = (CORBA::IDLType_ptr *) original_type_def;
  
  CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) calling_container_ptr;
  
  CORBA::ValueBoxDef_ptr new_value_box_ptr;
  
  try {
    new_value_box_ptr = ctnR_ptr->create_value_box(id, name, vspec, *otd);
  }
  catch(CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSValueBoxDef_ptr) new_value_box_ptr;
}
  
/*----------------------------------------------------------------*/
VSNativeDef_ptr
VScreate_native(const VSRepositoryId vs_id,
		const VSIdentifier vs_name,
		const VSVersionSpec vs_vspec,
		VSEnvironment *ev,
		VSContainer_ptr calling_container_ptr)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::RepositoryId id = (CORBA::RepositoryId) vs_id;
  CORBA::Identifier name = (CORBA::Identifier) vs_name;
  CORBA::VersionSpec vspec = (CORBA::VersionSpec) vs_vspec;
  CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) calling_container_ptr;
  
  CORBA::NativeDef_ptr new_native_ptr;
  
  try {
    new_native_ptr = ctnR_ptr->create_native(id, name, vspec);
  }
  catch(CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSValueBoxDef_ptr) new_native_ptr;
}
  
/*------------------------------------------------------*/


}

