//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>

/*------------------------------------------------------*/
int is_same_string(VSString s1, VSString s2)
{
		if((s1 == NULL) && (s2 == NULL)) return TRUE;
		if((s1 == NULL) || (s2 == NULL)) return FALSE;
		return (strcmp(s1,s2) == 0) ? TRUE : FALSE;
}

//-----------------------------------------------------------
int is_empty_string(VSString s)
{
		if(s == NULL) return FALSE;
				return (strcmp(s,"") == 0) ? TRUE : FALSE;
}
				 
//-----------------------------------------------------------
int is_invalid_string(VSString s)
{
		 if(s == NULL) return FALSE;
				 return (strcmp(s,"invalid") == 0) ? TRUE : FALSE;
}
									
//-----------------------------------------------------------
int is_nil_object(VSObject_ptr o1)
{
	CORBA::Object_ptr obj1 = (CORBA::Object_ptr) o1;
	 
		if(!obj1)
				return TRUE;
		return CORBA::is_nil(obj1);
}
					 
//-----------------------------------------------------------
int
is_nil_contained_ptr(VSContained_ptr o1)
{
		CORBA::Contained_ptr tc = (CORBA::Contained_ptr) o1;
		if(!tc)
			return TRUE;
								 
		return CORBA::is_nil(tc);
}

//-----------------------------------------------------------
int contents_contain( VSContainedSeq_ptr c_ptr, VSString str)
{

	CORBA::ULong i = 0;

		char * s = (char *) str;
		CORBA::ContainedSeq *contents_ptr = (CORBA::ContainedSeq *) c_ptr;
		if(!contents_ptr)
				return FALSE;
		
		while (i < contents_ptr->length() ) {
				if (is_same_string((*contents_ptr)[i]->absolute_name(), s))
					return TRUE;
				i++;
		}
		
		return FALSE;
}
//-----------------------------------------------------------
int descriptionSeqContains( VSDescriptionSeq_ptr ds_ptr, VSString str)
{

	CORBA::ULong i = 0;

		char * s = (char *) str;
		CORBA::Container::DescriptionSeq *descriptionSeq_ptr = 
						(CORBA::Container::DescriptionSeq *) ds_ptr;
		if(!descriptionSeq_ptr)
				return FALSE;
		
		while (i < descriptionSeq_ptr->length() ) {
				if (is_same_string((*descriptionSeq_ptr)[i].contained_object->absolute_name(), s))
					return TRUE;
				i++;
		}
		return FALSE;
}

//-----------------------------------------------------------
int
descriptionSeqExceedsSize(VSDescriptionSeq_ptr vs_ds_ptr, VSLong vs_max_returned_objs)
{
		CORBA::Container::DescriptionSeq *descriptionSeq_ptr = 
						(CORBA::Container::DescriptionSeq *) vs_ds_ptr;
		CORBA::ULong sizeLimit = (CORBA::ULong) vs_max_returned_objs;
		if(!descriptionSeq_ptr)
				return TRUE;

		if((sizeLimit != -1) && descriptionSeq_ptr->length() > sizeLimit)
				return TRUE;
		return FALSE;
}

//-----------------------------------------------------------
// we're being passed in a string with an absolute scopename
int type_limited(VSString s, VSDefinitionKind def_kind)
{
		CORBA::DefinitionKind dk = (CORBA::DefinitionKind) def_kind;	
		IRSupport *irs = glob_TS.repository();
		CORBA::Repository_ptr IR_ptr;
		CORBA::Contained_ptr ctnD_ptr;

		if(dk == CORBA::dk_all)
			return FALSE;
		
    IR_ptr = irs->get_repository();
    CORBA::Identifier_var id = s;
    ctnD_ptr = irs->get_contained(IR_ptr,id);
		if(!ctnD_ptr)
				return FALSE;
		if (ctnD_ptr->describe()->kind == dk)
				return FALSE;
		return TRUE;
}


//-----------------------------------------------------------
/* we calculate the level of the object by counting the number
	of occurrences of the string "::" in its absolute name.
	(need to add 4 to the depth as in our sample IR this is where
	we begin searching from). */

int level_limited(VSString s, long depth) 
{
		if (numOccurrences(s, "::") > depth + 4)
				return TRUE;
		return FALSE;
}

//-----------------------------------------------------------
//an auxiliary auxiliary function
int numOccurrences(VSString in, VSString of)
{
VSString tmp_ptr;
char tmp[100];
int i = 0;

	strcpy(tmp, in);
	tmp_ptr = strtok(tmp, of);
	while (tmp_ptr != NULL)
	{
			i++;
			tmp_ptr = strtok(NULL, of);
	}
	return i;
}		

int
is_nil_container_ptr(VSContainer_ptr *o1)
{
		CORBA::Container_ptr *tc = (CORBA::Container_ptr *) o1;
		if(!tc)
				return TRUE;
								 
		return CORBA::is_nil(*tc);
}
											 
int 
cleanup_container_ptr(VSContainer_ptr *o1)
{

		CORBA::Container_ptr *tc = (CORBA::Container_ptr *) o1;
		if(!tc)
				return TRUE;
		delete tc;
		return TRUE;
}

/*------------------------------------------------------*/
int
exists_definition(VSRepositoryId def)
{
		CORBA::Repository_ptr IR_ptr;
		CORBA::Contained_ptr ctnD_ptr;
		IRSupport *irs = glob_TS.repository();
		try {
				IR_ptr = irs->get_repository();
				ctnD_ptr = irs->get_contained(IR_ptr, def);
		}
		catch(...)
		{
				return FALSE; //an error has occurred ...return FALSE
		}
		return TRUE;
}

/*------------------------------------------------------*/
int
container_is_interface(VSContainer_ptr vs_cntR_ptr)
{
		CORBA::Container_ptr ctnR_ptr = (CORBA::Container_ptr) vs_cntR_ptr;
		CORBA::InterfaceDef_ptr iface_ptr;
		try {
				if (iface_ptr = CORBA::InterfaceDef::_narrow(ctnR_ptr))
						return TRUE;
				return FALSE;
		}
		catch (...) {
				return FALSE;
		}
		return FALSE;
}

/*------------------------------------------------------*/
int
id_exists(VSRepositoryId vs_id)
{
  CORBA::Repository_ptr IR_ptr;
  CORBA::Contained_ptr ctnD_ptr;
  IRSupport *irs = glob_TS.repository();
  try {
    IR_ptr = irs->get_repository();
    ctnD_ptr = IR_ptr -> lookup_id((CORBA::RepositoryId) vs_id);
    if (!CORBA::is_nil(ctnD_ptr))
      return TRUE;
  }
  catch (...) {
    return FALSE;
  }
  return FALSE;
}

/*------------------------------------------------------*/
}
