/*
 Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
 Copyright (c) 2001 Object Oriented Concepts GmbH.

 All rights reserved. No part of this source code may be reproduced,
 stored in a retrieval system, or transmitted, in any form or by any
 means, electronic, mechanical, photocopying, recording or otherwise,
 except as stated in the end-user licence agreement, without the prior
 permission of the copyright owners.

 Developed for X/Open by ApTest Ireland Limited.

 Project: VSORB

 * Copyright (c) 2001                                               
 * GMD FOKUS                                                        
 * Project: CORVAL2

*/


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

				 
/* ------------------------------------------------------------ */
/* max_returned_objs is provided for the VSdescribe_contents() test
	 to limit the number of objects which can be returned. */

int 
provide_max_returned_objs (VSLong *testvar, 
			   enum prop_max_returned_objs_maxObjs maxObjs)
{
  CORBA::Long* tvar = (CORBA::Long *) testvar;
  
  switch(maxObjs) {
  case Pmax_returned_objs_maxObjs_limiting:
    *tvar = 2;
    break;
  case Pmax_returned_objs_maxObjs_not_limiting:
    *tvar = -1;
    break;
  }
  return TRUE;
}
  
  
  int 
  relinquish_max_returned_objs (VSLong *testvar, 
				enum prop_max_returned_objs_maxObjs maxObjs)
  {
    switch(maxObjs) {
    case Pmax_returned_objs_maxObjs_limiting:
    case Pmax_returned_objs_maxObjs_not_limiting:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* lookup_name is provided for the VSlookup_name() test. It specifies
     the name of the object to be searched for. */
  
  int 
  provide_lookup_name (VSScopedName *testvar, 
		       enum prop_lookup_name_sName sName)
  {
    CORBA::ScopedName* tvar = (CORBA::ScopedName *) testvar;
    
    switch(sName) {
    case Plookup_name_sName_exists:
      *tvar = (CORBA::ScopedName) "MATCH";
      break;
    case Plookup_name_sName_not_exists:
      *tvar = (CORBA::ScopedName) "invalid";
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_lookup_name (VSScopedName *testvar, 
			  enum prop_lookup_name_sName sName)
  {
    switch(sName) {
    case Plookup_name_sName_exists:
    case Plookup_name_sName_not_exists:
      break;
    }
    return TRUE;
  }
  /* ------------------------------------------------------------ */
  /* search_name is provided for the VSlookup() test. It specifies
     the name of the object to be searched for. */
  
  int 
  provide_search_name (VSScopedName *testvar, 
		       enum prop_search_name_sName sName)
  {
    CORBA::ScopedName* tvar = (CORBA::ScopedName *) testvar;
    
    switch(sName) {
    case Psearch_name_sName_exists:
      *tvar = (CORBA::ScopedName) "M_Container::M_contents::E1";
      break;
    case Psearch_name_sName_not_exists:
      *tvar = (CORBA::ScopedName) "invalid";
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_search_name (VSScopedName *testvar, 
			  enum prop_search_name_sName sName)
  {
    switch(sName) {
    case Psearch_name_sName_exists:
    case Psearch_name_sName_not_exists:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* limit_type is provided for the: VScontents(), VSlookup_name(), and
     VSdescribe_contents() tests. The sequences returned by these 
     operations are limited to this limit_type. */
  int 
  provide_limit_type (VSDefinitionKind *testvar, 						enum prop_limit_type_limType limType)
  {
    CORBA::DefinitionKind* tvar = (CORBA::DefinitionKind *) testvar;
    
    switch(limType)
      {
      case Plimit_type_limType_dkAll:
	*tvar = CORBA::dk_all;
	break;
      case Plimit_type_limType_dkOperation:
	*tvar = CORBA::dk_Operation;
	break;
      case Plimit_type_limType_dkInterface:
	*tvar = CORBA::dk_Interface;
	break;
      case Plimit_type_limType_dkConstant:
	*tvar = CORBA::dk_Constant;
	break;
      }
      return TRUE;
  }
  
  int 
  relinquish_limit_type (VSDefinitionKind *testvar, 
			 enum prop_limit_type_limType limType)
  {
    switch(limType)
      {
      case Plimit_type_limType_dkAll:
      case Plimit_type_limType_dkOperation:
      case Plimit_type_limType_dkInterface:
      case Plimit_type_limType_dkConstant:
	break;
      }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* levels_to_search is provided for the VSlookup_name() test. It can 
     limit the search to the current object or to an arbitrary depth of
     contained objects. */
  
  int 
  provide_levels_to_search (VSLong *testvar, 
			    enum prop_levels_to_search_numLevels numLevels)
  {
    CORBA::Long* tvar = (CORBA::Long *) testvar;
    
    switch(numLevels) {
    case Plevels_to_search_numLevels_limiting:
      *tvar = 1;
      break;
    case Plevels_to_search_numLevels_not_limiting:
      *tvar = -1;
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_levels_to_search (VSLong *testvar, 
			       enum prop_levels_to_search_numLevels numLevels)
  {
    switch(numLevels) {
    case Plevels_to_search_numLevels_limiting:
    case Plevels_to_search_numLevels_not_limiting:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* exclude_inherited is provided for the: VScontents(), VSlookup_name()
     and VSdescribe_contents() tests. It include inherited objects in the
     returned sequence, or limit it to objects which are defined in the 
     search space. */
  
  int 
  provide_exclude_inherited (VSBoolean* testvar, 
			     enum prop_exclude_inherited_excInherited excInherited)
  {
    CORBA::Boolean* tvar = (CORBA::Boolean *) testvar;
    
    switch(excInherited) {
    case Pexclude_inherited_excInherited_true:
      *tvar = TRUE;
      break;
    case Pexclude_inherited_excInherited_false:
      *tvar = FALSE;
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_exclude_inherited (VSBoolean* testvar, 
				enum prop_exclude_inherited_excInherited excInherited)
  {
    switch(excInherited) {
    case Pexclude_inherited_excInherited_true:
    case Pexclude_inherited_excInherited_false:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* object_ptr is provided for the VSnarrow() test. */
  
  int
  provide_object_ptr (VSObject_ptr **testvar,
		      enum prop_object_ptr_objValue objValue)
  {
    CORBA::Object_ptr tvar;
    CORBA::Object_ptr* rptr;
    IRSupport *irs = glob_TS.repository();
    
    switch (objValue)  {
    case Pobject_ptr_objValue_valid:
      tvar = irs->get_repository();
      break;
    case Pobject_ptr_objValue_null:
      tvar = NULL;
      break;
    }
    rptr = new CORBA::Object_ptr(tvar);
    *testvar = (VSObject_ptr *) rptr;
    return TRUE;
  }
  
  int
  relinquish_object_ptr (VSObject_ptr **testvar,
			 enum prop_object_ptr_objValue objValue)
  {
    CORBA::Object_ptr* tvar = (CORBA::Object_ptr *) *testvar;
    
    switch (objValue)  {
    case Pobject_ptr_objValue_valid:
      delete tvar;
    case Pobject_ptr_objValue_null:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* container_ptr is provided for the VSduplicate() test. */
  
  int
  provide_container_ptr (VSContainer_ptr **testvar,
			 enum prop_container_ptr_ctnrValue ctnrValue)
  {
    CORBA::Container_ptr tvar;
    CORBA::Container_ptr* rptr;
    
    IRSupport *irs = glob_TS.repository();
    
    switch (ctnrValue)  {
    case Pcontainer_ptr_ctnrValue_valid:
      tvar = irs->get_repository();
      break;
    case Pcontainer_ptr_ctnrValue_null:
      tvar = NULL;
      break;
    }
    rptr = new CORBA::Container_ptr(tvar);
    *testvar = (VSContainer_ptr *) rptr;
    return TRUE;
  }
  
  int
  relinquish_container_ptr (VSContainer_ptr **testvar,
			    enum prop_container_ptr_ctnrValue ctnrValue)
  {
    CORBA::Container_ptr* tvar = (CORBA::Container_ptr *) *testvar;
    switch (ctnrValue)  {
    case Pcontainer_ptr_ctnrValue_valid:
      delete tvar;
    case Pcontainer_ptr_ctnrValue_null:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* PROVIDES FOR THE WRITE INTERFACE														  */
  /* ------------------------------------------------------------ */
  /* provides a new module id for the VScreate_module() test. */
  
  int 
  provide_new_module_id (VSRepositoryId *testvar, 
			 enum prop_new_module_id_defId defId)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    
    CORBA::RepositoryId* tvar = (CORBA::RepositoryId *) testvar;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    switch(defId) {
      case Pnew_module_id_defId_exists:
      ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write");
      if (!ctnD_ptr)
	return FALSE;
      *tvar = ctnD_ptr->id();
      break;
    case Pnew_module_id_defId_not_exists:
      *tvar = (CORBA::RepositoryId) "new_module_id";
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_new_module_id (VSRepositoryId *testvar, 
			    enum prop_new_module_id_defId defId)
  {
    switch(defId) {
    case Pnew_module_id_defId_exists:
    case Pnew_module_id_defId_not_exists:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* Provide a constant id for the VScreate_constant() test.
     provide an id that doesn't yet exist in the Repository and one
     that already exists (the latter doesn't have to be a const) */
  int 
  provide_new_const_id (VSRepositoryId *testvar, 
			enum prop_new_const_id_defId defId)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    
    CORBA::RepositoryId* tvar = (CORBA::RepositoryId *) testvar;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    switch(defId) {
    case Pnew_const_id_defId_exists:
      ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write");
      if (!ctnD_ptr)
	return FALSE;
      *tvar = ctnD_ptr->id();
      break;
    case Pnew_const_id_defId_not_exists:
      *tvar = (CORBA::RepositoryId) "new_const_id";
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_new_const_id (VSRepositoryId *testvar, 
			   enum prop_new_const_id_defId defId)
  {
    switch(defId) {
    case Pnew_const_id_defId_exists:
    case Pnew_const_id_defId_not_exists:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* Provide a struct id for the VScreate_struct() test. */
  
  int 
  provide_new_struct_id (VSRepositoryId *testvar, 
			 enum prop_new_struct_id_defId defId)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    
    CORBA::RepositoryId* tvar = (CORBA::RepositoryId *) testvar;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    switch(defId) {
    case Pnew_struct_id_defId_exists:
      ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write");
      if (!ctnD_ptr)
	return FALSE;
      *tvar = ctnD_ptr->id();
      break;
    case Pnew_struct_id_defId_not_exists:
      *tvar = (CORBA::RepositoryId) "new_struct_id";
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_new_struct_id (VSRepositoryId *testvar, 
			    enum prop_new_struct_id_defId defId)
  {
    switch(defId) {
    case Pnew_struct_id_defId_exists:
    case Pnew_struct_id_defId_not_exists:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* Provide a union id for the VScreate_union() test. */
  
  int 
  provide_new_union_id (VSRepositoryId *testvar, 
			enum prop_new_union_id_defId defId)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    
    CORBA::RepositoryId* tvar = (CORBA::RepositoryId *) testvar;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    switch(defId) {
    case Pnew_union_id_defId_exists:
      ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write");
      if (!ctnD_ptr)
	return FALSE;
      *tvar = ctnD_ptr->id();
      break;
    case Pnew_union_id_defId_not_exists:
      *tvar = (CORBA::RepositoryId) "new_union_id";
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_new_union_id (VSRepositoryId *testvar, 
			   enum prop_new_union_id_defId defId)
  {
    switch(defId) {
    case Pnew_union_id_defId_exists:
    case Pnew_union_id_defId_not_exists:
      break;
    }
    return TRUE;
  }

  /* ------------------------------------------------------------ */
  /* Provide a enum id for the VScreate_enum() test. */
  
  int 
  provide_new_enum_id (VSRepositoryId *testvar, 
		       enum prop_new_enum_id_defId defId)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    
    CORBA::RepositoryId* tvar = (CORBA::RepositoryId *) testvar;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    switch(defId) {
    case Pnew_enum_id_defId_exists:
      ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write");
      if (!ctnD_ptr)
	return FALSE;
      *tvar = ctnD_ptr->id();
      break;
    case Pnew_enum_id_defId_not_exists:
      *tvar = (CORBA::RepositoryId) "new_enum_id";
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_new_enum_id (VSRepositoryId *testvar, 
			  enum prop_new_enum_id_defId defId)
  {
    switch(defId) {
    case Pnew_enum_id_defId_exists:
    case Pnew_enum_id_defId_not_exists:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* Provide a alias id for the VScreate_alias() test. */
  
  int 
  provide_new_alias_id (VSRepositoryId *testvar, 
			enum prop_new_alias_id_defId defId)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    
    CORBA::RepositoryId* tvar = (CORBA::RepositoryId *) testvar;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    switch(defId) {
    case Pnew_alias_id_defId_exists:
      ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write");
      if (!ctnD_ptr)
	return FALSE;
      *tvar = ctnD_ptr->id();
      break;
    case Pnew_alias_id_defId_not_exists:
      *tvar = (CORBA::RepositoryId) "new_alias_id";
      break;
    }
    return TRUE;
  }

  int 
  relinquish_new_alias_id (VSRepositoryId *testvar, 
			   enum prop_new_alias_id_defId defId)
  {
    switch(defId) {
    case Pnew_alias_id_defId_exists:
    case Pnew_alias_id_defId_not_exists:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* Provide a interface id for the VScreate_interface() test. */
  
  int 
  provide_new_interface_id (VSRepositoryId *testvar, 
			    enum prop_new_interface_id_defId defId)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    
    CORBA::RepositoryId* tvar = (CORBA::RepositoryId *) testvar;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    switch(defId) {
    case Pnew_interface_id_defId_exists:
      ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write");
      if (!ctnD_ptr)
	return FALSE;
      *tvar = ctnD_ptr->id();
      break;
    case Pnew_interface_id_defId_not_exists:
      *tvar = (CORBA::RepositoryId) "new_interface_id";
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_new_interface_id (VSRepositoryId *testvar, 
			       enum prop_new_interface_id_defId defId)
  {
    switch(defId) {
    case Pnew_interface_id_defId_exists:
    case Pnew_interface_id_defId_not_exists:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* Provide a exception id for the VScreate_exception() test. */
  
  int 
  provide_new_exception_id (VSRepositoryId *testvar, 
			    enum prop_new_exception_id_defId defId)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    
    CORBA::RepositoryId* tvar = (CORBA::RepositoryId *) testvar;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    switch(defId) {
    case Pnew_exception_id_defId_exists:
      ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write");
      if (!ctnD_ptr)
	return FALSE;
      *tvar = ctnD_ptr->id();
      break;
    case Pnew_exception_id_defId_not_exists:
      *tvar = (CORBA::RepositoryId) "new_exception_id";
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_new_exception_id (VSRepositoryId *testvar, 
			       enum prop_new_exception_id_defId defId)
  {
    switch(defId) {
    case Pnew_exception_id_defId_exists:
    case Pnew_exception_id_defId_not_exists:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* type is used by create_constant(), create_union() and create_alias() */
  int 
  provide_type (VSIDLType_ptr *testvar)
  {
    CORBA::IDLType_ptr *tvar = (CORBA::IDLType_ptr *) testvar;
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr ctnD_ptr;
    
    IRSupport *irs = glob_TS.repository();
    
    try {
      IR_ptr = irs->get_repository();
      *tvar = IR_ptr->get_primitive(CORBA::pk_short);
    }
    catch (...) {
      return FALSE;
    }
    
    return TRUE;
  }
  
  int 
  relinquish_type (VSIDLType_ptr *testvar)
  {
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* const_value is used by create_constant() */
  int 
  provide_const_value (VSAny_ptr *testvar)
  {
    CORBA::Any* rptr;
    CORBA::Any toPass;
    CORBA::Short val;
    
    val = 4;
    toPass <<= val;
    rptr = new CORBA::Any(toPass);
    *testvar = (VSAny_ptr) rptr;
    return TRUE;
  }
  
  int 
  relinquish_const_value (VSAny_ptr *testvar)
  {
    CORBA::Any* tvar = (CORBA::Any *) *testvar;
    delete tvar;
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* calling_container_ptr is used by all the create_ functions except
     create_module() It can provide containers of type interface or of
     type module. */
  /* This has been changed to only provide a container of type Repository.
     It was originally designed to provide 2 different types of container
     and in this way test that containment rules are obeyed in creation of
     different interface types (eg. a module cannot contain an attribute).
     However, as adl executes the X-product of all test variables, create_
     operations would be called twice using the same RepositoryId (one of
     the provide variables) and this would not do ....at all at all. */
  
  int 
  provide_calling_container_ptr (VSContainer_ptr *testvar/*,
							   enum prop_calling_container_ptr_ctnrType ctnrType*/)
  {
    CORBA::Container_ptr* tvar = (CORBA::Container_ptr *) testvar;
    CORBA::IRObject_ptr IRobj_ptr;
    
    IRSupport *irs = glob_TS.repository();
    CORBA::Repository_ptr IR_ptr = irs->get_repository();
    
    /*switch (ctnrType)  {
      case Pcalling_container_ptr_ctnrType_interface_type:*/
    IRobj_ptr = irs->get_contained(IR_ptr,"M_Container::I_write");
    if (!IRobj_ptr)
      return FALSE;
    /*break;
      case Pcalling_container_ptr_ctnrType_module_type:
      return FALSE;
      //IRobj_ptr = irs->get_contained(IR_ptr,"M_Container::M_write");
      break;
      }*/
    *tvar = CORBA::Container::_narrow(IRobj_ptr);
    return TRUE;
  }
  
  int 
  relinquish_calling_container_ptr (VSContainer_ptr *testvar/*,
							      enum prop_calling_container_ptr_ctnrType ctnrType*/)
  {
    /*switch (ctnrType) {
      case Pcalling_container_ptr_ctnrType_interface_type:
      case Pcalling_container_ptr_ctnrType_module_type:
      break;
      }*/
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* used by VScreate_struct() and VScreate_exception() */
  int 
  provide_struct_members (VSStructMemberSeq_ptr *testvar)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    CORBA::StructMemberSeq* tvar;
    CORBA::StructMemberSeq* rptr;
    
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::I_copy::S1");
    if (!ctnD_ptr)
      return FALSE;
    CORBA::StructDef_ptr sDef_ptr = CORBA::StructDef::_narrow(ctnD_ptr);
    tvar = sDef_ptr->members();
    rptr = new CORBA::StructMemberSeq(*tvar);
    *testvar = (VSStructMemberSeq_ptr) rptr;
    return TRUE;
  }
  
  int 
  relinquish_struct_members (VSStructMemberSeq_ptr *testvar)
  {
    CORBA::StructMemberSeq* tvar = (CORBA::StructMemberSeq *) *testvar;
    delete tvar;
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* used by VScreate_union() */
  int 
  provide_union_members (VSUnionMemberSeq_ptr *testvar)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    CORBA::UnionMemberSeq* tvar;
    CORBA::UnionMemberSeq* rptr;
    
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::I_copy::U1");
    if (!ctnD_ptr)
      return FALSE;
    CORBA::UnionDef_ptr uDef_ptr = CORBA::UnionDef::_narrow(ctnD_ptr);
    tvar = uDef_ptr->members();
    rptr = new CORBA::UnionMemberSeq(*tvar);
    *testvar = (VSUnionMemberSeq_ptr) rptr;
    return TRUE;
  }
  
  int 
  relinquish_union_members (VSUnionMemberSeq_ptr *testvar)
  {
    CORBA::UnionMemberSeq* tvar = (CORBA::UnionMemberSeq *) *testvar;
    delete tvar;
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* used by VScreate_enum() */
  int 
  provide_enum_members (VSEnumMemberSeq_ptr *testvar)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    CORBA::EnumMemberSeq* tvar;
    CORBA::EnumMemberSeq* rptr;
    
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::I_copy::E1");
    if (!ctnD_ptr)
      return FALSE;
    CORBA::EnumDef_ptr eDef_ptr = CORBA::EnumDef::_narrow(ctnD_ptr);
    tvar = eDef_ptr->members();
    rptr = new CORBA::EnumMemberSeq(*tvar);
    *testvar = (VSEnumMemberSeq_ptr) rptr;
    return TRUE;
  }
  
  int 
  relinquish_enum_members (VSEnumMemberSeq_ptr *testvar)
  {
    CORBA::EnumMemberSeq* tvar = (CORBA::EnumMemberSeq *) *testvar;
    delete tvar;
    return TRUE;
  }
  
  
  /* ------------------------------------------------------------ */
  /* used by VScreate_interface() */
  int 
  provide_base_interfaces (VSInterfaceDefSeq_ptr *testvar)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    CORBA::InterfaceDefSeq* tvar;
    CORBA::InterfaceDefSeq* rptr;
    
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::I_write");
    if (!ctnD_ptr)
      return FALSE;
    CORBA::InterfaceDef_ptr iDef_ptr = CORBA::InterfaceDef::_narrow(ctnD_ptr);
    tvar = iDef_ptr->base_interfaces();
    rptr = new CORBA::InterfaceDefSeq(*tvar);
    *testvar = (VSInterfaceDefSeq_ptr) rptr;
    return TRUE;
  }
  
  int 
  relinquish_base_interfaces (VSInterfaceDefSeq_ptr *testvar)
  {
    CORBA::InterfaceDefSeq* tvar = (CORBA::InterfaceDefSeq *) *testvar;
    delete tvar;
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */

  int 
  provide_new_value_id (VSRepositoryId *testvar, 
			 enum prop_new_value_id_defId defId)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    
    CORBA::RepositoryId* tvar = (CORBA::RepositoryId *) testvar;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    switch(defId) {
      case Pnew_value_id_defId_exists:
      ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write::TestValueB");
      if (!ctnD_ptr)
	return FALSE;
      *tvar = ctnD_ptr->id();
      break;
    case Pnew_value_id_defId_not_exists:
      *tvar = (CORBA::RepositoryId) "new_value_id";
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_new_value_id (VSRepositoryId *testvar, 
			    enum prop_new_value_id_defId defId)
  {
    switch(defId) {
    case Pnew_value_id_defId_exists:
    case Pnew_value_id_defId_not_exists:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */

  int 
  provide_new_value_box_id (VSRepositoryId *testvar, 
			 enum prop_new_value_box_id_defId defId)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    
    CORBA::RepositoryId* tvar = (CORBA::RepositoryId *) testvar;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    switch(defId) {
      case Pnew_value_box_id_defId_exists:
      ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write::TestShortBox");
      if (!ctnD_ptr)
	return FALSE;
      *tvar = ctnD_ptr->id();
      break;
      /*    case Pnew_value_box_id_defId_not_exists:
      *tvar = (CORBA::RepositoryId) "new_value_box_id";
      break;*/
    }
    return TRUE;
  }
  
  int 
  relinquish_new_value_box_id (VSRepositoryId *testvar, 
			    enum prop_new_value_box_id_defId defId)
  {
    switch(defId) {
    case Pnew_value_box_id_defId_exists:
      //    case Pnew_value_box_id_defId_not_exists:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */

  int 
  provide_new_native_id (VSRepositoryId *testvar, 
			 enum prop_new_native_id_defId defId)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    
    CORBA::RepositoryId* tvar = (CORBA::RepositoryId *) testvar;
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    
    switch(defId) {
      case Pnew_native_id_defId_exists:
      ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write::TestNative");
      if (!ctnD_ptr)
	return FALSE;
      *tvar = ctnD_ptr->id();
      break;
    case Pnew_native_id_defId_not_exists:
      *tvar = (CORBA::RepositoryId) "new_native_id";
      break;
    }
    return TRUE;
  }
  
  int 
  relinquish_new_native_id (VSRepositoryId *testvar, 
			    enum prop_new_native_id_defId defId)
  {
    switch(defId) {
    case Pnew_native_id_defId_exists:
    case Pnew_native_id_defId_not_exists:
      break;
    }
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */

  int
  provide_base_value(VSValueDef_ptr *testvar)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    CORBA::ValueDef_ptr tvar;
    
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write::TestValueB");
    if (!ctnD_ptr)
      return FALSE;
    tvar = CORBA::ValueDef::_narrow(ctnD_ptr);
    *testvar = (VSValueDef_ptr) tvar;
    return TRUE;
  }
  
  int
  relinquish_base_value(VSValueDef_ptr *testvar)
  {
    CORBA::ValueDef *tvar = (CORBA::ValueDef *) *testvar;
    delete tvar;
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */


  int
  provide_abstract_base_values(VSValueDefSeq_ptr *testvar)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    CORBA::ValueDefSeq* tvar;
    
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write::TestValueB");
    if (!ctnD_ptr)
      return FALSE;
    CORBA::ValueDef_var t = CORBA::ValueDef::_narrow(ctnD_ptr);
    tvar = t->abstract_base_values();
    *testvar = (VSValueDefSeq_ptr) tvar;
    return TRUE;
  }
  
  int
  relinquish_abstract_base_values(VSValueDefSeq_ptr *testvar)
  {
    CORBA::ValueDefSeq *tvar = (CORBA::ValueDefSeq *) *testvar;
    delete tvar;
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */

  int
  provide_supported_interfaces(VSInterfaceDefSeq_ptr *testvar)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    CORBA::InterfaceDefSeq* tvar;
    
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write::TestValueB");
    if (!ctnD_ptr)
      return FALSE;
    CORBA::ValueDef_var t = CORBA::ValueDef::_narrow(ctnD_ptr);
    tvar = t->supported_interfaces();
    *testvar = (VSInterfaceDefSeq_ptr) tvar;
    return TRUE;
  }
  

  int
  relinquish_supported_interfaces(VSInterfaceDefSeq_ptr *testvar)
  {
    CORBA::InterfaceDefSeq* tvar = (CORBA::InterfaceDefSeq *) *testvar;
    delete tvar;
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */

  int
  provide_initializers(VSInitializerSeq_ptr *testvar)
  {
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    CORBA::InitializerSeq* tvar;
    
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write::TestValueB");
    if (!ctnD_ptr)
      return FALSE;
    CORBA::ValueDef_var t = CORBA::ValueDef::_narrow(ctnD_ptr);
    tvar = t->initializers();
    *testvar = (VSInitializerSeq_ptr) tvar;
    return TRUE;
  }

  int
  relinquish_initializers(VSInitializerSeq_ptr *testvar)
  {
    CORBA::InitializerSeq* tvar = (CORBA::InitializerSeq *) *testvar;
    delete tvar;
    return TRUE;
  }

  /* ------------------------------------------------------------ */

  int
  provide_original_type_def(VSIDLType_ptr *testvar)
  {
    IRSupport *irs = glob_TS.repository();
    CORBA::Repository_ptr IR_ptr = irs->get_repository();
    CORBA::PrimitiveDef_ptr tvar = IR_ptr->get_primitive(CORBA::pk_ulong);
    CORBA::IDLType_ptr tmp = CORBA::PrimitiveDef::_duplicate(tvar);
    *testvar = (VSIDLType_ptr) tmp;
    return TRUE;
    /*
    CORBA::Repository_ptr  IR_ptr;
    CORBA::Contained_ptr   ctnD_ptr;
    CORBA::IDLType_ptr tvar;
    
    IRSupport *irs = glob_TS.repository();
    IR_ptr = irs->get_repository();
    ctnD_ptr = irs->get_contained(IR_ptr,"M_Container::M_write::TestShortBox");
    if (!ctnD_ptr)
      return FALSE;
    CORBA::ValueBoxDef_var t = CORBA::ValueBoxDef::_narrow(ctnD_ptr);
    tvar = t->original_type_def();

    *testvar = (VSIDLType_ptr) tvar;
    return TRUE;*/
  }

  int
  relinquish_original_type_def(VSIDLType_ptr *testvar)
  {
    CORBA::IDLType_ptr tvar = (CORBA::IDLType_ptr ) *testvar;
    delete tvar;
    return TRUE;
  }
  /* ------------------------------------------------------------ */
}
