//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>

/*------------------------------------------------------*/
int
is_correct_members(VSStructMemberSeq_ptr vs_members_seq)
{
 	CORBA::StructMemberSeq* memberSeq_ptr = (CORBA::StructMemberSeq *) vs_members_seq;
	if(!memberSeq_ptr)
			return FALSE;

	return((memberSeq_ptr->length() == 2) &&
				 (strcmp("s1", (char *) (*memberSeq_ptr)[0].name) == 0) &&
				 (strcmp("f1", (char *) (*memberSeq_ptr)[1].name) == 0));
}

/*------------------------------------------------------*/
int
is_correct_type(VSTypeCode_ptr* vs_tc_ptr)
{
	CORBA::TypeCode_ptr* tc_ptr = (CORBA::TypeCode_ptr *) vs_tc_ptr;
	CORBA::TCKind tk = (*tc_ptr)->kind();
	 
	return (tk == CORBA::tk_except); 
}
							 


/*------------------------------------------------------*/
int is_nil_object(VSObject_ptr o1)
{
	CORBA::Object_ptr obj1 = (CORBA::Object_ptr) o1;
		 
	if(!obj1)
			return TRUE;
	return CORBA::is_nil(obj1);
}

/*------------------------------------------------------*/
int
is_nil_exceptiondef_ptr(VSExceptionDef_ptr *o1)
{
		CORBA::ExceptionDef_ptr *tc = (CORBA::ExceptionDef_ptr *) o1;
		if(!tc)
				return TRUE;
								 
		return CORBA::is_nil(*tc);
}
											 
int 
cleanup_exceptiondef_ptr(VSExceptionDef_ptr *o1)
{
		CORBA::ExceptionDef_ptr *tc = (CORBA::ExceptionDef_ptr *) o1;
		if(!tc)
				return TRUE;
		delete tc;
		return TRUE;
}

/*------------------------------------------------------*/
/* aux functions for write tests */
/*------------------------------------------------------*/
int
is_same_members(VSStructMemberSeq_ptr vs_m1, VSStructMemberSeq_ptr vs_m2)
{
		CORBA::StructMemberSeq* m1Seq_ptr = (CORBA::StructMemberSeq *) vs_m1;
		CORBA::StructMemberSeq* m2Seq_ptr = (CORBA::StructMemberSeq *) vs_m2;

		if(!m1Seq_ptr || !m2Seq_ptr)
				return FALSE;

		if (m1Seq_ptr->length() != m2Seq_ptr->length())
				return FALSE;
		CORBA::ULong i = 0;
		while (i < m1Seq_ptr->length()) {
				if (strcmp( (char *)(*m1Seq_ptr)[i].name, (char *)(*m2Seq_ptr)[i].name) != 0)
						return FALSE;
				i++;
		}
		return TRUE;
}

/*------------------------------------------------------*/
/* aux funcs for redefined functionality */
/*------------------------------------------------------*/
int
is_correct_def_kind(VSDefinitionKind vs_dk)
{
		CORBA::DefinitionKind dk = (CORBA::DefinitionKind) vs_dk;
		return (dk == CORBA::dk_Exception);
}
/*------------------------------------------------------*/
int
is_correct_description(VSDescription_ptr vs_desc_ptr)
{
		CORBA::Contained::Description* desc_ptr = 
				(CORBA::Contained::Description *) vs_desc_ptr;
		if (!desc_ptr)
				return FALSE;

		if (desc_ptr->kind != CORBA::dk_Exception)
				return FALSE;

		const CORBA::ExceptionDescription* exceptDesc_ptr;
		if (desc_ptr->value >>= exceptDesc_ptr) 
				if (!exceptDesc_ptr)
						return FALSE;
				if (!exceptDesc_ptr->name)
						return FALSE;
				return (strcmp( (char*)exceptDesc_ptr->name, "E1") == 0); 
		return FALSE;
}
/*------------------------------------------------------*/
}

