//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <corba_supp.hh>

#include <tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/
/* READ INTERFACE */
/*------------------------------------------------------*/

VSDefinitionKind VSdef_kind(VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::Object_ptr     obj_ptr;
		CORBA::Repository_ptr IR_ptr;
		CORBA::Contained_ptr    ctnD_ptr;
		CORBA::DefinitionKind dk;
		CORBA::IRObject_var myObjRef;
		IRSupport *irs = glob_TS.repository();

	  try {
        IR_ptr = irs->get_repository();
        ctnD_ptr = irs->get_contained(IR_ptr,"M_IRObject::I");
				if (!ctnD_ptr)
						return (VSDefinitionKind) 0;
				//narrow this to give a IRObject reference
				myObjRef = CORBA::IRObject::_narrow(ctnD_ptr);
				
				//now call the function under test
				dk = myObjRef->def_kind();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		}

		return (VSDefinitionKind) dk;
}

/*------------------------------------------------------*/

VSIRObject_ptr *VSnil(VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::IRObject_ptr ttc;

		try {
				ttc = CORBA::IRObject::_nil();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		}
		catch (...) {
				return (VSIRObject_ptr *) 0;
		}

	  CORBA::IRObject_ptr *rtc = new CORBA::IRObject_ptr(ttc);
		return (VSIRObject_ptr *) rtc;
}

/*------------------------------------------------------*/
VSIRObject_ptr*
VSduplicate(VSIRObject_ptr *o1, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::IRObject_ptr *tc = (CORBA::IRObject_ptr *) o1;
		CORBA::IRObject_ptr ttc;

		try {
				ttc = CORBA::IRObject::_duplicate(tc ? *tc : (CORBA::IRObject_ptr) 0);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		}
		catch (...) {
				return (VSIRObject_ptr *) 0;
		}

		CORBA::IRObject_ptr *rtc = new CORBA::IRObject_ptr(ttc);
		return (VSIRObject_ptr *) rtc;
}

/*------------------------------------------------------*/
VSIRObject_ptr*
VSnarrow(VSObject_ptr *o1, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::Object_ptr *tc = (CORBA::Object_ptr *) o1;
		CORBA::IRObject_ptr ttc;

		try {
				ttc = CORBA::IRObject::_narrow(tc ? *tc : (CORBA::IRObject_ptr) 0);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		}
		catch (...) {
				return (VSIRObject_ptr *) 0;
		}

		CORBA::IRObject_ptr *rtc = new CORBA::IRObject_ptr(ttc);
		return (VSIRObject_ptr *) rtc;
}

/*------------------------------------------------------*/
/* WRITE INTERFACE */
/*------------------------------------------------------*/
void VSdestroy(VSEnvironment *ev, VSIRObject_ptr vs_irObj_ptr)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::IRObject_ptr irObj_ptr = (CORBA::IRObject_ptr) vs_irObj_ptr;

		try {
				irObj_ptr->destroy();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		}
}

/*------------------------------------------------------*/
}

