/*
 Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
 Copyright (c) 2001 Object Oriented Concepts GmbH.

 All rights reserved. No part of this source code may be reproduced,
 stored in a retrieval system, or transmitted, in any form or by any
 means, electronic, mechanical, photocopying, recording or otherwise,
 except as stated in the end-user licence agreement, without the prior
 permission of the copyright owners.

 Developed for X/Open by ApTest Ireland Limited.

 Project: CORVAL2
*/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

/* ---- global property declarations ---- */
/* ---- test variables and local properties ---- */

/* ------------------------------------------------------------ */
/* irObj is used to test the destroy() function. It is tested for
	 three cases: Repository, PrimitiveDef and InterfaceDef. The
	 first two should return an error. Calling destroy on an
	 InterfaceDef should also destroy its contents. */

int
provide_irObj (VSIRObject_ptr *testvar, 
							 enum prop_irObj_irObj_value irObj_value)
{
      CORBA::IRObject_ptr* tvar = (CORBA::IRObject_ptr *) testvar;

			IRSupport *irs = glob_TS.repository();
			CORBA::Repository_ptr IR_ptr = irs->get_repository();

			switch(irObj_value){
      		case PirObj_irObj_value_Repository:
						*tvar = IR_ptr;
						break;
      		case PirObj_irObj_value_PrimitiveDef:
						try {
								*tvar = IR_ptr->get_primitive(CORBA::pk_string);
						}
						catch (...) {
								return FALSE;
						}
						break;
      		case PirObj_irObj_value_valid:
						*tvar = irs->get_contained(IR_ptr,"M_IRObject::I");
						break;
      }
			if(!tvar || !*tvar)
					return FALSE;
      return TRUE;
}

int
relinquish_irObj (VSIRObject_ptr *testvar, 
									enum prop_irObj_irObj_value irObj_value)
{
		switch(irObj_value){
				case PirObj_irObj_value_Repository:
				case PirObj_irObj_value_PrimitiveDef:
				case PirObj_irObj_value_valid:
						break;	
		}
		return TRUE;
}

/* ------------------------------------------------------------ */
/* test variable VSObject_ptr object_ptr(prop objValue) 
	 object_ptr is used to test the VSnarrow function. */
int
provide_object_ptr (VSObject_ptr **testvar,
											enum prop_object_ptr_objValue objValue)
{
	CORBA::Object_ptr tvar;
	CORBA::Object_ptr* rptr;
	CORBA::Object_ptr obj_ptr;
	IRSupport *irs = glob_TS.repository();

    switch (objValue)  {
    case Pobject_ptr_objValue_valid:
        tvar = irs->get_repository();
      break;
       case Pobject_ptr_objValue_null:
            tvar = NULL;
						break;
    }
    rptr = new CORBA::Object_ptr(tvar);
    *testvar = (VSObject_ptr *) rptr;
    return TRUE;
}

 
int
relinquish_object_ptr (VSObject_ptr **testvar,
		       enum prop_object_ptr_objValue objValue)
{
	CORBA::Object_ptr* tvar = (CORBA::Object_ptr *) *testvar;

    switch (objValue)  {
        case Pobject_ptr_objValue_valid:
	  delete tvar;
	  break;
    case Pobject_ptr_objValue_null:
      break;
    }
    return TRUE;
}

/* ------------------------------------------------------------ */
/* test variable VSIRObject_ptr object_ptr(prop objValue) 
	 irobject_ptr is used to test the VSduplicate function. */
int
provide_irobject_ptr (VSIRObject_ptr **testvar,
		      enum prop_irobject_ptr_irobjValue irobjValue)
{
  CORBA::IRObject_ptr tvar;
  CORBA::IRObject_ptr* rptr;
  CORBA::Object_ptr obj_ptr;
	IRSupport *irs = glob_TS.repository();
  
    switch (irobjValue)  {
    case Pirobject_ptr_irobjValue_valid:
      tvar = irs->get_repository();
      break;
    case Pirobject_ptr_irobjValue_null:
      tvar = NULL;
      break;
    }
    rptr = new CORBA::IRObject_ptr(tvar);
    *testvar = (VSIRObject_ptr *) rptr;
    return TRUE;
}

 
int
relinquish_irobject_ptr (VSIRObject_ptr **testvar,
			 enum prop_irobject_ptr_irobjValue irobjValue)
{
  CORBA::IRObject_ptr* tvar = (CORBA::IRObject_ptr *) *testvar;
  switch (irobjValue)  {
  case Pirobject_ptr_irobjValue_valid:
    delete tvar;
						break;
  case Pirobject_ptr_irobjValue_null:
    break;
    }
  return TRUE;
}

/* ------------------------------------------------------------ */

}


