//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>

/*----------------------------------------------------------------*/
/* FullInterfaceDescription returns a structure with 8 fields - 6 of 
	 which are defined elsewhere:
	 name(), id(), defined_in(), version() - defined in Contained
	 type() - defined in IDLType
	 base_interfaces - defined in InterfaceDef.
	 So these are tested for equality to these inherited functions.
	 There remains 2: operations and attributes which are tested 
	 independently here */

int
correct_full_description_name(VSFullInterfaceDescription_ptr vsfull_desc, 
																	  VSInterfaceDef_ptr calling_intface_ptr)
{
 CORBA::InterfaceDef::FullInterfaceDescription* fullDesc_ptr = (CORBA::InterfaceDef::FullInterfaceDescription *) vsfull_desc;
 CORBA::InterfaceDef_ptr cIntFace_ptr = (CORBA::InterfaceDef_ptr) calling_intface_ptr;
 CORBA::Contained_ptr ctnD_ptr;

 //implicit widening of the InterfaceDef ptr.
 ctnD_ptr = cIntFace_ptr;

 char* cStr = (char *) ctnD_ptr->name();
 char *iStr = (char *) fullDesc_ptr->name;
 if(strcmp(iStr, cStr) != 0)
		 return FALSE;

	return TRUE;
}

/*----------------------------------------------------------------*/
int
correct_full_description_id(VSFullInterfaceDescription_ptr vsfull_desc, 
																	  VSInterfaceDef_ptr calling_intface_ptr)
{
 CORBA::InterfaceDef::FullInterfaceDescription* fullDesc_ptr = (CORBA::InterfaceDef::FullInterfaceDescription *) vsfull_desc;
 CORBA::InterfaceDef_ptr cIntFace_ptr = (CORBA::InterfaceDef_ptr) calling_intface_ptr;
 CORBA::Contained_ptr ctnD_ptr;

 //implicit widening of the InterfaceDef ptr.
 ctnD_ptr = cIntFace_ptr;

  char* cStr = (char *) ctnD_ptr->id();
	char* iStr = (char *) fullDesc_ptr->id;
	if(strcmp(iStr, cStr) != 0)
			return FALSE;

	return TRUE;
}

/*----------------------------------------------------------------*/
int
correct_full_description_defined_in(VSFullInterfaceDescription_ptr vsfull_desc, 
																	  VSInterfaceDef_ptr calling_intface_ptr)
{
 CORBA::InterfaceDef::FullInterfaceDescription* fullDesc_ptr = (CORBA::InterfaceDef::FullInterfaceDescription *) vsfull_desc;
 CORBA::InterfaceDef_ptr cIntFace_ptr = (CORBA::InterfaceDef_ptr) calling_intface_ptr;
 CORBA::Contained_ptr ctnD_ptr;

 //implicit widening of the InterfaceDef ptr.
 ctnD_ptr = cIntFace_ptr;

	try {
	CORBA::Container_ptr ctnR_ptr = ctnD_ptr->defined_in();
	CORBA::ModuleDef_ptr mod_ptr = CORBA::ModuleDef::_narrow(ctnR_ptr); //narrow to a ModuleDef
	CORBA::Contained_ptr defining_ctnD_ptr = mod_ptr;   //and widen to a Contained.
	char* cStr = (char *) defining_ctnD_ptr->id();
	char* iStr = (char *) fullDesc_ptr->defined_in;
	if(strcmp(iStr, cStr) != 0)
			return FALSE;
	}
	catch(...) {
			return FALSE;
	}

	return TRUE;
}

/*----------------------------------------------------------------*/
int
correct_full_description_version(VSFullInterfaceDescription_ptr vsfull_desc, 
																	  VSInterfaceDef_ptr calling_intface_ptr)
{
 CORBA::InterfaceDef::FullInterfaceDescription* fullDesc_ptr = (CORBA::InterfaceDef::FullInterfaceDescription *) vsfull_desc;
 CORBA::InterfaceDef_ptr cIntFace_ptr = (CORBA::InterfaceDef_ptr) calling_intface_ptr;
 CORBA::Contained_ptr ctnD_ptr;

 ctnD_ptr = cIntFace_ptr;

	char* cStr = (char *) ctnD_ptr->version();
	char* iStr = (char *) fullDesc_ptr->version;
	if(strcmp(iStr, cStr) != 0)
			return FALSE;

	return TRUE;
}

int
/*----------------------------------------------------------------*/
correct_full_description_type(VSFullInterfaceDescription_ptr vsfull_desc, 
																	  VSInterfaceDef_ptr calling_intface_ptr)
{
 CORBA::InterfaceDef::FullInterfaceDescription* fullDesc_ptr = (CORBA::InterfaceDef::FullInterfaceDescription *) vsfull_desc;
 CORBA::InterfaceDef_ptr cIntFace_ptr = (CORBA::InterfaceDef_ptr) calling_intface_ptr;
 CORBA::IDLType_ptr idlType_ptr;

  idlType_ptr = cIntFace_ptr;
	if (!(idlType_ptr->type()->equal(fullDesc_ptr->type)))
			return FALSE;

	return TRUE;
}

/*----------------------------------------------------------------*/
//compare a seq. of RepositoryIds to a seq. of InterfaceDefs
int
correct_full_description_base_interfaces(VSFullInterfaceDescription_ptr vsfull_desc, 
																	  VSInterfaceDef_ptr calling_intface_ptr)
{
 CORBA::InterfaceDef::FullInterfaceDescription* fullDesc_ptr = (CORBA::InterfaceDef::FullInterfaceDescription *) vsfull_desc;
 CORBA::InterfaceDef_ptr cIntFace_ptr = (CORBA::InterfaceDef_ptr) calling_intface_ptr;

	if(fullDesc_ptr->base_interfaces.length() != cIntFace_ptr->base_interfaces()->length())
			return FALSE;
	CORBA::ULong i;
	for (i = 0; i < fullDesc_ptr->base_interfaces.length(); i++) {
			CORBA::InterfaceDefSeq* iFaceBaseSeq = cIntFace_ptr->base_interfaces();
			if (fullDesc_ptr->base_interfaces[i] != (*iFaceBaseSeq)[i]->id())
					return FALSE;
	}

	return TRUE;
}

/*----------------------------------------------------------------*/
/* This could be tested by calling the Container::contents() operation with limit_type
	 set to dk_Operation and then comparing the results. However it's probably a good
	 idea to limit interdependencies between tests (???) so the returned sequence here is
	 checked against the actual expected values. */

int
correct_full_description_operations(VSFullInterfaceDescription_ptr vsfull_desc)
{
 CORBA::InterfaceDef::FullInterfaceDescription* fullDesc_ptr = (CORBA::InterfaceDef::FullInterfaceDescription *) vsfull_desc;

	if(fullDesc_ptr->operations.length() != 3)
			return FALSE;

	VSOpDescriptionSeq_ptr vs_ops = &(fullDesc_ptr->operations);
	return (contains_operations_seq(vs_ops, "Op1") && 
					contains_operations_seq(vs_ops, "Op2") && 
					contains_operations_seq(vs_ops, "Op3"));

	return FALSE;
}

/*----------------------------------------------------------------*/
int
contains_operations_seq(VSOpDescriptionSeq_ptr op_seq, char* opName)
{
 CORBA::OpDescriptionSeq* opSeq_ptr = (CORBA::OpDescriptionSeq *) op_seq;
 CORBA::ULong i;
	
	for (i=0; i < opSeq_ptr->length(); i++) {
		if (strstr((*opSeq_ptr)[i].name, opName))
				return TRUE;
	}
	return FALSE;
}

/*----------------------------------------------------------------*/
int
correct_full_description_attributes(VSFullInterfaceDescription_ptr vsfull_desc)
{
 CORBA::InterfaceDef::FullInterfaceDescription* fullDesc_ptr = (CORBA::InterfaceDef::FullInterfaceDescription *) vsfull_desc;

 if(fullDesc_ptr->attributes.length() != 2)
		return FALSE;

	VSAttrDescriptionSeq_ptr vs_atts = &(fullDesc_ptr->attributes);
	return (contains_attributes_seq(vs_atts, "A1") && 
					contains_attributes_seq(vs_atts, "A2"));

	return FALSE;
}

/*----------------------------------------------------------------*/
int
contains_attributes_seq(VSAttrDescriptionSeq_ptr at_seq, char* atName)
{
 CORBA::AttrDescriptionSeq* atSeq_ptr = (CORBA::AttrDescriptionSeq *) at_seq;
	 CORBA::ULong i;
		
		for (i=0; i < atSeq_ptr->length(); i++) {
		if (strstr((*atSeq_ptr)[i].name, atName))
				return TRUE;
		}
		return FALSE;
}


/*----------------------------------------------------------------*/
/* For the three sample InterfaceDefs (from the sample IR) over 
	 we call this function, we test the correct set of base interfaces
	 is returned. */

int
correct_base_interface_set(VSInterfaceDefSeq_ptr base_seq, VSInterfaceDef_ptr calling_intface_ptr)
{
 CORBA::InterfaceDefSeq* baseSeq_ptr = (CORBA::InterfaceDefSeq *) base_seq;
 CORBA::InterfaceDef_ptr cIntFace_ptr = (CORBA::InterfaceDef_ptr) calling_intface_ptr;
 CORBA::ULong i;

	try {
	if(strstr(cIntFace_ptr->absolute_name(), "Ibase")) 
		return (baseSeq_ptr->length() == 0);
	else if (strstr(cIntFace_ptr->absolute_name(), "Isingle"))
		return ((baseSeq_ptr->length() == 1) && 
						(strstr((*baseSeq_ptr)[0]->absolute_name(), "I2")));
	else if (strstr(cIntFace_ptr->absolute_name(), "Imultiple"))
		return ((baseSeq_ptr->length() == 2) && 
						(contains_base_interface_set(base_seq, "I1")) &&
						(contains_base_interface_set(base_seq, "I4"))); 
	}
	catch(...) {
		return FALSE;
	}

	//one of the above expressions should always evaluate to TRUE
	return FALSE;
}

/*----------------------------------------------------------------*/
int
contains_base_interface_set(VSInterfaceDefSeq_ptr base_seq, char* iface)
{
 CORBA::InterfaceDefSeq* baseSeq_ptr = (CORBA::InterfaceDefSeq *) base_seq;
 CORBA::ULong i;
	
	for (i=0; i < baseSeq_ptr->length(); i++) {
		if (strstr((*baseSeq_ptr)[i]->absolute_name(), iface))
				return TRUE;
	}
	return FALSE;
}

/*----------------------------------------------------------------*/
// M_InterfaceDef::is_a::I3 is our calling interface. According 
// to the way the sample IR is set up, this function should return 
// true if the id contains the substrings: "I1", "I2", or "I3".
int
is_derived_from(VSRepositoryId iface_id)
{
	char *strId = (char *) iface_id;

	return (strstr(strId, "I1") || strstr(strId, "I2") || strstr(strId, "I3"));
}


/*----------------------------------------------------------------*/
int is_nil_object(VSObject_ptr o1)
{
		CORBA::Object_ptr obj1 = (CORBA::Object_ptr) o1;
		 
		if(!obj1)
				return TRUE;
		return CORBA::is_nil(obj1);
}
										
/*----------------------------------------------------------------*/
int
is_nil_interfacedef_ptr(VSInterfaceDef_ptr *o1)
{
		CORBA::InterfaceDef_ptr *tc = (CORBA::InterfaceDef_ptr *) o1;
		if(!tc)
				return TRUE;
								 
		return CORBA::is_nil(*tc);
}
											 
/*----------------------------------------------------------------*/
int 
cleanup_interfacedef_ptr(VSInterfaceDef_ptr *o1)
{
		CORBA::InterfaceDef_ptr *tc = (CORBA::InterfaceDef_ptr *) o1;
		if(!tc)
				return TRUE;
		delete tc;
		return TRUE;
}

/*----------------------------------------------------------------*/
/* auxiliary functions for write function tests */
/*----------------------------------------------------------------*/
/* vs_intDefSeq holds the set of base interfaces which we attempted
	 to assign to our interface. we can check the result by comparing
	 this set to the new base interfaces of our interface. */
int
is_correct_base_interfaces(VSInterfaceDefSeq_ptr vs_intDefSeq)
{
		CORBA::Repository_ptr IR_ptr;
		CORBA::InterfaceDef_ptr ifDef_ptr;
		CORBA::Contained_ptr ctnD_ptr;

		IRSupport *irs = glob_TS.repository();
		IR_ptr = irs->get_repository();
		ctnD_ptr = irs->get_contained(IR_ptr, "M_InterfaceDef::I_write");
		if (!ctnD_ptr)
				return FALSE;
		ifDef_ptr = CORBA::InterfaceDef::_narrow(ctnD_ptr);
		CORBA::InterfaceDefSeq* newBIs_ptr = ifDef_ptr->base_interfaces();
		CORBA::ULong i;
		for(i=0; i < newBIs_ptr->length(); i++) {
				if (!contains_base_interface_set(vs_intDefSeq, (*newBIs_ptr)[i]->name()))
						return FALSE;
		}
		return TRUE;
}

/*----------------------------------------------------------------*/
/* get the defined_in interface of the passed repository id and
	 compare its id to that of the interface in which we are defining these 
	 objects */
int
att_defined_in_correct_interface(VSAttributeDef_ptr vs_attDef_ptr)
{
		CORBA::AttributeDef_ptr attDef_ptr = (CORBA::AttributeDef_ptr) vs_attDef_ptr;
		IRSupport *irs = glob_TS.repository();
		CORBA::Repository_ptr IR_ptr;
		CORBA::IRObject_ptr 	irObj_ptr;
		CORBA::Contained_ptr if1_ptr;
		CORBA::Contained_ptr if2_ptr;

		IR_ptr = irs->get_repository();
		irObj_ptr = attDef_ptr->defined_in();
		if1_ptr = CORBA::Contained::_narrow(irObj_ptr);

		if2_ptr = irs->get_contained(IR_ptr,"M_InterfaceDef::I_write");
		if (!if2_ptr)
				return FALSE;

		return(is_same_string(if1_ptr->id(), if2_ptr->id()));
}

/*------------------------------------------------------*/
int
op_defined_in_correct_interface(VSOperationDef_ptr vs_opDef_ptr)
{
		CORBA::OperationDef_ptr opDef_ptr = (CORBA::OperationDef_ptr) vs_opDef_ptr;
		CORBA::Repository_ptr IR_ptr;
		CORBA::IRObject_ptr 	irObj_ptr;
		CORBA::Contained_ptr if1_ptr;
		CORBA::Contained_ptr if2_ptr;
		IRSupport *irs = glob_TS.repository();

		IR_ptr = irs->get_repository();
		irObj_ptr = opDef_ptr->defined_in();
		if1_ptr = CORBA::Contained::_narrow(irObj_ptr);

		if2_ptr = irs->get_contained(IR_ptr,"M_InterfaceDef::I_write");
		if (!if2_ptr)
				return FALSE;

		return(is_same_string(if1_ptr->id(), if2_ptr->id()));
}
/*------------------------------------------------------*/
int is_same_string(VSString s1, VSString s2)
{
		if((s1 == NULL) && (s2 == NULL)) return TRUE;
		if((s1 == NULL) || (s2 == NULL)) return FALSE;
		return (strcmp(s1,s2) == 0) ? TRUE : FALSE;
}

/*------------------------------------------------------*/
int
is_correct_description(VSDescription_ptr vs_desc_ptr)
{
		CORBA::Contained::Description* desc_ptr = 
				(CORBA::Contained::Description *) vs_desc_ptr;
		if (!desc_ptr)
				return FALSE;

		if (desc_ptr->kind != CORBA::dk_Interface)
				return FALSE;

		const CORBA::InterfaceDescription* ifaceDesc_ptr;
		if (desc_ptr->value >>= ifaceDesc_ptr) {
				if (!ifaceDesc_ptr)
						return FALSE;
				if (!ifaceDesc_ptr->name)
						return FALSE;
				return (strcmp( (char*)ifaceDesc_ptr->name, "I_write") == 0);
		}
		return FALSE;
}

/*------------------------------------------------------*/
int
not_existingid(VSRepositoryId vs_att_id)
{
		char* att_id = (char *) vs_att_id;
		return (strcmp(att_id, "new_att_id") == 0);
}

/*------------------------------------------------------*/
int
not_existing_opid(VSRepositoryId vs_id)
{
		char* op_id = (char *) vs_id;
		return (strcmp(op_id, "new_op_id") == 0);
}

/*------------------------------------------------------*/
int
not_existing_opname(VSIdentifier vs_name)
{
		char* op_name = (char *) vs_name;
		return (strcmp(op_name, "Op0") == 0);
}

/*------------------------------------------------------*/
/*
int
is_abstract_intf(VSInterfaceDef_ptr * intf_ptr)
{
  CORBA::InterfaceDef_ptr aif = (CORBA::InterfaceDef_ptr) intf_ptr;
  CORBA::FullInterfaceDescription_var des = aif->describe_interface();
  if (strcmp(des->id, "TestAbstrInterfaceA")==0)
    return true;
  else (strcmp(des->id, "Ibase")==0)
    return false;
  return false;
}
*/
/*------------------------------------------------------*/
/*
int
is_abstract_intf_set(VSBoolean abs, VSInterfaceDef_ptr * intf_ptr)
{
  CORBA::InterfaceDef_ptr aif = (CORBA::InterfaceDef_ptr) intf_ptr;
  bool res;
  try
    {
      res = aif->is_abstract();
      return (res == (bool) abs);
    }
  catch(CORBA::Exception &ex)
    {
      cout <<"is_abstract_intf: exception."<<ex<<endl;
      return FALSE;
    }
}
*/
/*------------------------------------------------------*/
}
