//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>

/*------------------------------------------------------*/
int
is_correct_contexts(VSContextIdSeq_ptr vs_ctxt_id_seq)
{
 CORBA::ContextIdSeq* contextIdSeq_ptr = (CORBA::ContextIdSeq*) vs_ctxt_id_seq;

	return((contextIdSeq_ptr->length() == 2) &&
				 (strcmp("one", (char *) (*contextIdSeq_ptr)[0]) == 0) &&
				 (strcmp("two", (char *) (*contextIdSeq_ptr)[1]) == 0));
	return FALSE;
}

/*------------------------------------------------------*/
/* For the moment we just check the ids of the exceptions
	 in the list and ensure that these are correct. It is 
	 assumed then that the other fields in the ExceptionDescription
	 are set correctly - the reading of these fields is tested
	 under the ExceptionDef test suite. */
int
is_correct_exception_set(VSExceptionDefSeq_ptr vs_except_def_seq)
{
 CORBA::ExceptionDefSeq* exceptionDefSeq_ptr = (CORBA::ExceptionDefSeq *) vs_except_def_seq;

	return((exceptionDefSeq_ptr->length() == 2) &&
				 (exception_seq_contains(vs_except_def_seq, "E1")) &&
				 (exception_seq_contains(vs_except_def_seq, "E2")));
		return FALSE;
}

int
exception_seq_contains(VSExceptionDefSeq_ptr vs_except_def_seq, char* exception_name)
{
 CORBA::ExceptionDefSeq* exceptDefSeq_ptr = (CORBA::ExceptionDefSeq *) vs_except_def_seq;
 CORBA::ULong i;
 CORBA::Object_ptr      obj_ptr;
 CORBA::Repository_ptr  IR_ptr;
 CORBA::Contained_ptr   ctnD_ptr;
 IRSupport *irs = glob_TS.repository();
 IR_ptr = irs->get_repository();

		char lookupStr[50];
		strcpy(lookupStr, "M_OperationDef::I1::");
		strcat(lookupStr, exception_name);
    ctnD_ptr = irs->get_contained(IR_ptr,lookupStr);

		CORBA::RepositoryId repId = ctnD_ptr->id();
		char* repId_str = (char *) repId;

	for(i = 0; i < exceptDefSeq_ptr->length(); i++) {
		char* exceptionId_str = (char *) (*exceptDefSeq_ptr)[i]->id();
		if (strcmp(exceptionId_str, repId_str) == 0)
			return TRUE;
	}
	return FALSE;
}


/*------------------------------------------------------*/
int
is_correct_param_sequence(VSParDescriptionSeq_ptr vs_par_desc_seq)
{
 CORBA::ParDescriptionSeq* parDescSeq_ptr = (CORBA::ParDescriptionSeq *) vs_par_desc_seq;

	if(parDescSeq_ptr->length() != 3)
			return FALSE;
	return((strcmp((*parDescSeq_ptr)[0].name, "s") == 0) &&
				 (strcmp((*parDescSeq_ptr)[1].name, "f") == 0) &&
				 (strcmp((*parDescSeq_ptr)[2].name, "i") == 0));
		return FALSE;
}

/*------------------------------------------------------*/
int
is_correct_param_modes(VSParDescriptionSeq_ptr vs_par_desc_seq)
{
 CORBA::ParDescriptionSeq* parDescSeq_ptr = (CORBA::ParDescriptionSeq *) vs_par_desc_seq;

	return(((*parDescSeq_ptr)[0].mode == CORBA::PARAM_IN) &&
				 ((*parDescSeq_ptr)[1].mode == CORBA::PARAM_OUT) &&
				 ((*parDescSeq_ptr)[2].mode == CORBA::PARAM_INOUT)); 
		return FALSE;
}

/*------------------------------------------------------*/
int
is_correct_result(VSTypeCode_ptr* vs_tc_ptr)
{
	CORBA::TypeCode_ptr* tc_ptr = (CORBA::TypeCode_ptr *) vs_tc_ptr;
	 
	return((*tc_ptr)->kind() == CORBA::tk_float); 
}
							 
/*------------------------------------------------------*/
int
is_correct_result_def(VSIDLType_ptr vs_idltype_ptr)
{
	CORBA::IDLType_ptr idltype_ptr = (CORBA::IDLType_ptr) vs_idltype_ptr;
									
	return(idltype_ptr->type()->kind() == CORBA::tk_float);
		return FALSE;
}
															

/*------------------------------------------------------*/
int
is_correct_mode(VSOperationMode vs_op_mode, VSOperationDef_ptr vs_op_ptr)
{
 CORBA::OperationMode opMode = (CORBA::OperationMode) vs_op_mode;
 CORBA::OperationDef_ptr op_ptr = (CORBA::OperationDef_ptr) vs_op_ptr;

	char* opName = (char *) op_ptr->name();
	if(strstr(opName, "op_oneway"))
			return (opMode == CORBA::OP_ONEWAY);
	if(strstr(opName, "op_normal"))
			return (opMode == CORBA::OP_NORMAL);

  //one of above statements should alway be true
	return FALSE;
}


/*------------------------------------------------------*/
int
is_nil_operationdef_ptr(VSOperationDef_ptr *o1)
{
		CORBA::OperationDef_ptr *tc = (CORBA::OperationDef_ptr *) o1;
		if(!tc)
				return TRUE;
								 
		return CORBA::is_nil(*tc);
}
	
int is_nil_object(VSObject_ptr o1)
{
	CORBA::Object_ptr obj1 = (CORBA::Object_ptr) o1;
	 
		if(!obj1)
				return TRUE;
		return CORBA::is_nil(obj1);
}

int 
cleanup_operationdef_ptr(VSOperationDef_ptr *o1)
{
		CORBA::OperationDef_ptr *tc = (CORBA::OperationDef_ptr *) o1;
		if(!tc)
				return TRUE;
		delete tc;
		return TRUE;
}

/*------------------------------------------------------*/
/* aux functions for the write tests */
/*------------------------------------------------------*/

int 
is_same_type_def(VSIDLType_ptr vs_idl1_ptr, VSIDLType_ptr vs_idl2_ptr)
{
    CORBA::IDLType_ptr idl1_ptr = (CORBA::IDLType_ptr) vs_idl1_ptr;
		CORBA::IDLType_ptr idl2_ptr = (CORBA::IDLType_ptr) vs_idl2_ptr;
		return (idl1_ptr->type()->equal(idl2_ptr->type()));
}

/*------------------------------------------------------*/
int 
is_same_params(VSParDescriptionSeq_ptr vs_pseq1, VSParDescriptionSeq_ptr vs_pseq2)
{
		CORBA::ParDescriptionSeq* pSeq1_ptr = (CORBA::ParDescriptionSeq *) vs_pseq1;
		CORBA::ParDescriptionSeq* pSeq2_ptr = (CORBA::ParDescriptionSeq *) vs_pseq2;
		if (pSeq1_ptr->length() != pSeq2_ptr->length())
				return FALSE;
		CORBA::ULong i = 0;
		while (i < pSeq1_ptr->length()) {
				if (strcmp( (char *)(*pSeq1_ptr)[i].name, (char *)(*pSeq2_ptr)[i].name) != 0)
						return FALSE;
				i++;
		}
		return TRUE;
}

/*------------------------------------------------------*/
int 
is_same_mode(VSOperationMode vs_m1, VSOperationMode vs_m2)
{
		CORBA::OperationMode m1 = (CORBA::OperationMode) vs_m1;
		CORBA::OperationMode m2 = (CORBA::OperationMode) vs_m2;
		return (m1 == m2);
}

/*------------------------------------------------------*/
int 
is_same_contexts(VSContextIdSeq_ptr vs_ctxtSeq1, VSContextIdSeq_ptr vs_ctxtSeq2)
{
		CORBA::ContextIdSeq* ctxtSeq1_ptr = (CORBA::ContextIdSeq *) vs_ctxtSeq1;
		CORBA::ContextIdSeq* ctxtSeq2_ptr = (CORBA::ContextIdSeq *) vs_ctxtSeq2;
		if (ctxtSeq1_ptr->length() != ctxtSeq2_ptr->length())
				return FALSE;
		CORBA::ULong i = 0;
		while (i < ctxtSeq1_ptr->length()) {
				if(strcmp( (char *)(*ctxtSeq1_ptr)[i], (char *)(*ctxtSeq2_ptr)[i]) != 0)
						return FALSE;
				i++;
		}
		return TRUE;
}

/*------------------------------------------------------*/

int
is_same_exceptions(VSExceptionDefSeq_ptr vs_exDefSeq1, VSExceptionDefSeq_ptr vs_exDefSeq2)
{
		CORBA::ExceptionDefSeq* exSeq1_ptr = (CORBA::ExceptionDefSeq *) vs_exDefSeq1;
		CORBA::ExceptionDefSeq* exSeq2_ptr = (CORBA::ExceptionDefSeq *) vs_exDefSeq2;
		if (exSeq1_ptr->length() != exSeq2_ptr->length())
				return FALSE;
		CORBA::ULong i = 0;
		const CORBA::ExceptionDescription *exc_desc1;
		const CORBA::ExceptionDescription *exc_desc2;
		CORBA::Contained::Description* desc1;
		CORBA::Contained::Description* desc2;

		while (i < exSeq1_ptr->length()) {
				CORBA::ExceptionDef_ptr ex1_ptr = (CORBA::ExceptionDef_ptr) (*exSeq1_ptr)[i];
				CORBA::ExceptionDef_ptr ex2_ptr = (CORBA::ExceptionDef_ptr) (*exSeq2_ptr)[i];
				desc1 = ex1_ptr->describe();
				desc2 = ex2_ptr->describe();
				if( !(desc1->value >>= exc_desc1) )
					return FALSE;
				if( !(desc2->value >>= exc_desc2) )
					return FALSE;
				if (strcmp( (char *)exc_desc1->name, 
										(char *)exc_desc2->name) != 0)
						return FALSE;
				i++;
		}
		return TRUE;
}

/*------------------------------------------------------*/
/* the idl is set up that the operation named "mode_write_valid"
	 is the only valid one for the mode_write operation */

int 
is_invalid_op(VSOperationDef_ptr vs_opDef_ptr)
{
		CORBA::OperationDef_ptr opDef_ptr = (CORBA::OperationDef_ptr) vs_opDef_ptr;
		if (strcmp( (char*)opDef_ptr->name(), "mode_write_valid") == 0)
				return TRUE;
		return FALSE;
}

/*------------------------------------------------------*/
/* aux funcs for redefined functionality */
/*------------------------------------------------------*/
int
is_correct_def_kind(VSDefinitionKind vs_dk)
{
		CORBA::DefinitionKind dk = (CORBA::DefinitionKind) vs_dk;
		return (dk == CORBA::dk_Operation);
}
/*------------------------------------------------------*/
int
is_correct_description(VSDescription_ptr vs_desc_ptr)
{
		CORBA::Contained::Description* desc_ptr = 
				(CORBA::Contained::Description *) vs_desc_ptr;
		if (desc_ptr->kind != CORBA::dk_Operation)
				return FALSE;
		const CORBA::OperationDescription* opDesc_ptr;
		if (desc_ptr->value >>= opDesc_ptr) {
		return ((opDesc_ptr->mode == CORBA::OP_ONEWAY) &&
						(strcmp( (char*)opDesc_ptr->name, "op_oneway") == 0));
		}
		return FALSE;
}
/*------------------------------------------------------*/
}
