//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

				 
/* ------------------------------------------------------------ */
/* calling_op_ptr is provided for the VSmode_read() test.
	 Here we provide the two modes of operations. */
int
provide_calling_op_ptr (VSOperationDef_ptr *testvar,
											enum prop_calling_op_ptr_opValue opValue)
{
	CORBA::OperationDef_ptr *tvar = (CORBA::OperationDef_ptr *) testvar;
	CORBA::Object_ptr      obj_ptr;
	CORBA::Repository_ptr  IR_ptr;
	CORBA::Contained_ptr   ctnD_ptr;
	IRSupport *irs = glob_TS.repository();

    IR_ptr = irs->get_repository();

    switch (opValue)  {
        case Pcalling_op_ptr_opValue_oneway:
        		ctnD_ptr = irs->get_contained(IR_ptr,"M_OperationDef::I1::op_oneway");
						break;
       case Pcalling_op_ptr_opValue_normal:
        		ctnD_ptr = irs->get_contained(IR_ptr,"M_OperationDef::I1::op_normal");
						break;
    }
		*tvar = CORBA::OperationDef::_narrow(ctnD_ptr);
		return TRUE;
}

int
relinquish_calling_op_ptr (VSOperationDef_ptr *testvar,
											enum prop_calling_op_ptr_opValue opValue)
{
    switch (opValue)  {
        case Pcalling_op_ptr_opValue_oneway:
				case Pcalling_op_ptr_opValue_normal:
        break;
    }
		return TRUE;
}
/* ------------------------------------------------------------ */
/* object_ptr is provided for the VSnarrow() test. */
int
provide_object_ptr (VSObject_ptr **testvar,
											enum prop_object_ptr_objValue objValue)
{
	CORBA::Object_ptr tvar ;
	CORBA::Object_ptr* rptr;
	CORBA::Object_ptr obj_ptr;
	CORBA::Repository_ptr IR_ptr;
	IRSupport *irs = glob_TS.repository();
  IR_ptr = irs->get_repository();

    switch (objValue)  {
        case Pobject_ptr_objValue_valid:
    				IR_ptr = irs->get_repository();
        		tvar = irs->get_contained(IR_ptr,"M_OperationDef::I1::op_normal");
						break;
       case Pobject_ptr_objValue_null:
            tvar = NULL;
						break;
    }
		rptr = new CORBA::Object_ptr(tvar);
		*testvar = (VSObject_ptr *) rptr;
		return TRUE;
}

int
relinquish_object_ptr (VSObject_ptr **testvar,
													enum prop_object_ptr_objValue objValue)
{
	CORBA::Object_ptr* tvar = (CORBA::Object_ptr *) *testvar;

    switch (objValue)  {
        case Pobject_ptr_objValue_valid:
						delete tvar;
						break;
				case Pobject_ptr_objValue_null:
        		break;
    }
		return TRUE;
}

/* ------------------------------------------------------------ */
/* operationdef_ptr is provided for the VSduplicate() test */
int
provide_operationdef_ptr (VSOperationDef_ptr **testvar,
											enum prop_operationdef_ptr_opdefValue opdefValue)
{
	CORBA::OperationDef_ptr tvar;
	CORBA::OperationDef_ptr* rptr;
	 
	CORBA::Object_ptr obj_ptr;
  CORBA::Repository_ptr  IR_ptr;
	CORBA::Contained_ptr ctnD_ptr;
	IRSupport *irs = glob_TS.repository();
  IR_ptr = irs->get_repository();


		switch (opdefValue)  {
        case Poperationdef_ptr_opdefValue_valid:
    				IR_ptr = irs->get_repository();
        		ctnD_ptr = irs->get_contained(IR_ptr,"M_OperationDef");
						tvar = CORBA::OperationDef::_narrow(ctnD_ptr);
						break;
       case Poperationdef_ptr_opdefValue_null:
            tvar = NULL;
						break;
    }
		rptr = new CORBA::OperationDef_ptr(tvar);
		*testvar = (VSOperationDef_ptr *) rptr;
		return TRUE;
}

int
relinquish_operationdef_ptr (VSOperationDef_ptr **testvar,
											enum prop_operationdef_ptr_opdefValue opdefValue)
{
		CORBA::OperationDef_ptr* tvar = (CORBA::OperationDef_ptr *) *testvar;

		switch (opdefValue)  {
        case Poperationdef_ptr_opdefValue_valid:
						delete tvar;
						break;
				case Poperationdef_ptr_opdefValue_null:
						break;
    }
		return TRUE;
}

/* ------------------------------------------------------------ */
/* provide functions for the write tests */
/* ------------------------------------------------------------ */
/* type is provided for the VSresult_def_write() test. */
int 
provide_type (VSIDLType_ptr *testvar)
{
    CORBA::IDLType_ptr *tvar = (CORBA::IDLType_ptr *) testvar;
		CORBA::Repository_ptr  IR_ptr;
		IRSupport *irs = glob_TS.repository();
		IR_ptr = irs->get_repository();
		try {
				*tvar = IR_ptr->get_primitive(CORBA::pk_short);
		}
		catch (...) {
				return FALSE;
		}
		return TRUE;
}

int 
relinquish_type (VSIDLType_ptr *testvar)
{
		return TRUE;
}

/* ------------------------------------------------------------ */
/* params is provided for the VSparams_write() test. 
	 We provide it by reading the paramaters of an existing operation. */
int 
provide_params (VSParDescriptionSeq_ptr *testvar)
{
		CORBA::Repository_ptr  IR_ptr;
		CORBA::Contained_ptr   ctnD_ptr;
		CORBA::ParDescriptionSeq* tvar;
		CORBA::ParDescriptionSeq* rptr;

		IRSupport *irs = glob_TS.repository();
		IR_ptr = irs->get_repository();
		ctnD_ptr = irs->get_contained(IR_ptr,"M_OperationDef::I1::op_params");
		if (!ctnD_ptr)
				return FALSE;
		CORBA::OperationDef_ptr opDef_ptr = CORBA::OperationDef::_narrow(ctnD_ptr);
		tvar = opDef_ptr->params();
		rptr = new CORBA::ParDescriptionSeq(*tvar);
		*testvar = (VSParDescriptionSeq_ptr) rptr;
		return TRUE;
}

int 
relinquish_params (VSParDescriptionSeq_ptr *testvar)
{
		CORBA::ParDescriptionSeq* tvar = (CORBA::ParDescriptionSeq *) *testvar;
		delete tvar;
		return TRUE;
}

/* ------------------------------------------------------------ */
/* mode is provided for the VSmode_write() test. */
int 
provide_mode (VSOperationMode *testvar)
{
		CORBA::OperationMode* tvar = (CORBA::OperationMode *) testvar;
		*tvar = CORBA::OP_NORMAL;
		return TRUE;
}

int 
relinquish_mode (VSOperationMode *testvar)
{
		return TRUE;
}

/* ------------------------------------------------------------ */
/* contexts is provided for the VScontexts_write() test. 
	 We provide it by reading the context of an existing operation. */
int 
provide_contexts (VSContextIdSeq_ptr *testvar)
{
		CORBA::Repository_ptr  IR_ptr;
		CORBA::Contained_ptr   ctnD_ptr;
		CORBA::ContextIdSeq* tvar;
		CORBA::ContextIdSeq* rptr;

		IRSupport *irs = glob_TS.repository();
		IR_ptr = irs->get_repository();
		ctnD_ptr = irs->get_contained(IR_ptr,"M_OperationDef::I1::op_context");
		if (!ctnD_ptr)
				return FALSE;
		CORBA::OperationDef_ptr opDef_ptr = CORBA::OperationDef::_narrow(ctnD_ptr);
		tvar = opDef_ptr->contexts();
		rptr = new CORBA::ContextIdSeq(*tvar);
		*testvar = (VSContextIdSeq_ptr) rptr;
		return TRUE;
}

int 
relinquish_contexts (VSContextIdSeq_ptr *testvar)
{
		return TRUE;
}

/* ------------------------------------------------------------ */
/* exceptions is provided for the VSexceptions_write() test. 
	 We provide it by reading the exceptions of an existing operation. */
int
provide_exceptions (VSExceptionDefSeq_ptr *testvar)
{
		CORBA::Repository_ptr  IR_ptr;
		CORBA::Contained_ptr   ctnD_ptr;
		CORBA::ExceptionDefSeq* tvar;
		CORBA::ExceptionDefSeq* rptr;

		IRSupport *irs = glob_TS.repository();
		IR_ptr = irs->get_repository();    
		ctnD_ptr = irs->get_contained(IR_ptr,"M_OperationDef::I1::op_exception");    
		if (!ctnD_ptr)
				return FALSE;
		CORBA::OperationDef_ptr opDef_ptr = CORBA::OperationDef::_narrow(ctnD_ptr);
		tvar = opDef_ptr->exceptions();
		rptr = new CORBA::ExceptionDefSeq(*tvar);
		*testvar = (VSExceptionDefSeq_ptr) rptr;
		return TRUE;
}

int 
relinquish_exceptions (VSExceptionDefSeq_ptr *testvar)
{
		CORBA::ExceptionDefSeq* tvar = (CORBA::ExceptionDefSeq *) *testvar;
		delete tvar;
		return TRUE;
}

/* ------------------------------------------------------------ */
/* mode_write_op_ptr is provided for the VSmode_write() test. 
	 We provide various operations as the mode attribute can only be 
	 set to OP_ONEWAY if the result is TC_void and all elements of 
	 params have a mode of PARAM_IN. So we provide an operation with 
	 parameters, one with a non-void result, and one which should work. */
int 
provide_mode_write_op_ptr (VSOperationDef_ptr *testvar, 
													 enum prop_mode_write_op_ptr_opValue opValue)
{
		CORBA::OperationDef_ptr *tvar = (CORBA::OperationDef_ptr *) testvar;
		CORBA::Object_ptr      obj_ptr;
		CORBA::Repository_ptr  IR_ptr;
		CORBA::Contained_ptr   ctnD_ptr;
		IRSupport *irs = glob_TS.repository();
		
		IR_ptr = irs->get_repository();
		
		switch (opValue) {
				case Pmode_write_op_ptr_opValue_invalid_result:
						ctnD_ptr = irs->get_contained(IR_ptr,"M_OperationDef::I1::op_float");
						break;
				case Pmode_write_op_ptr_opValue_invalid_params:
						ctnD_ptr = irs->get_contained(IR_ptr,"M_OperationDef::I1::op_params");
						break;
				case Pmode_write_op_ptr_opValue_valid:
						ctnD_ptr = irs->get_contained(IR_ptr,"M_OperationDef::I1::op_normal");
						break;
		}
		*tvar = CORBA::OperationDef::_narrow(ctnD_ptr);
		return TRUE;
}

int
relinquish_mode_write_op_ptr (VSOperationDef_ptr *testvar, 
															enum prop_mode_write_op_ptr_opValue opValue)
{
		switch (opValue) {
				case Pmode_write_op_ptr_opValue_invalid_result:
				case Pmode_write_op_ptr_opValue_invalid_params:
				case Pmode_write_op_ptr_opValue_valid:
						break;
		}
		return TRUE;
}

/* ------------------------------------------------------------ */

}
