/*
 Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
 Copyright (c) 2001 Object Oriented Concepts GmbH.

 All rights reserved. No part of this source code may be reproduced,
 stored in a retrieval system, or transmitted, in any form or by any
 means, electronic, mechanical, photocopying, recording or otherwise,
 except as stated in the end-user licence agreement, without the prior
 permission of the copyright owners.

 Developed for X/Open by ApTest Ireland Limited.

 Project: VSORB

 * Copyright (c) 2001                                               
 * GMD FOKUS                                                        
 * Project: CORVAL2
*/

#include	<corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/
/* READ INTERFACE */
/*------------------------------------------------------*/

/* lookup_id() */

VSContained_ptr 
VSlookup_id(const VSRepositoryId search_id, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;

		CORBA::Object_ptr			obj_ptr;
		CORBA::Repository_ptr	IR_ptr;
		CORBA::Contained_ptr		ctnd_ptr;
		CORBA::RepositoryId searchId = (CORBA::RepositoryId) search_id;
		IRSupport *irs = glob_TS.repository();

		try {
				IR_ptr = irs->get_repository();
				ctnd_ptr = IR_ptr->lookup_id(searchId);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		}
		return (VSContained_ptr) ctnd_ptr;
}

/*------------------------------------------------------*/
/* get_primitive() */

VSPrimitiveDef_ptr
VSget_primitive(VSPrimitiveKind kind, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;

		CORBA::Object_ptr			obj_ptr;
		CORBA::Repository_ptr	IR_ptr;

		CORBA::PrimitiveKind pKind = (CORBA::PrimitiveKind) kind;
		IRSupport *irs = glob_TS.repository();
		CORBA::PrimitiveDef_ptr pDef_ptr;
		
		try {
				IR_ptr = irs->get_repository();
				pDef_ptr = IR_ptr->get_primitive(pKind);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		}
		return (VSPrimitiveDef_ptr) pDef_ptr;
}

/*------------------------------------------------------*/
VSRepository_ptr*
VSnil(VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;

		CORBA::Repository_ptr ttc;
				 
		try {
				ttc = CORBA::Repository::_nil();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				}
				catch (...) {
						return (VSRepository_ptr *) 0;
		}
		CORBA::Repository_ptr *rtc = new CORBA::Repository_ptr(ttc);
		return (VSRepository_ptr *) rtc;
}

/*------------------------------------------------------*/
VSRepository_ptr*
VSduplicate(VSRepository_ptr *o1, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;

		CORBA::Repository_ptr *tc = (CORBA::Repository_ptr *) o1;
		CORBA::Repository_ptr ttc;
						 
		try {
       ttc = CORBA::Repository::_duplicate(tc ? *tc : (CORBA::Repository_ptr) 0);
    }
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
            env->exception(t);
    }
		catch (...) {
        return (VSRepository_ptr *) 0;
    }
		 
		CORBA::Repository_ptr *rtc = new CORBA::Repository_ptr(ttc);
		return (VSRepository_ptr *) rtc;
}
/*------------------------------------------------------*/

VSRepository_ptr*
VSnarrow(VSObject_ptr *o1,  VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		
		CORBA::Object_ptr *tc = (CORBA::Object_ptr *) o1;
		CORBA::Repository_ptr ttc;
    try {
        ttc = CORBA::Repository::_narrow(tc ? *tc : (CORBA::Repository_ptr) 0);
    }
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
            env->exception(t);
    }
		catch (...) {
        return (VSRepository_ptr *) 0;
    }
		 
		CORBA::Repository_ptr *rtc = new CORBA::Repository_ptr(ttc);
		return (VSRepository_ptr *) rtc;
}


/*------------------------------------------------------*/
/* WRITE INTERFACE */
/*------------------------------------------------------*/

VSStringDef_ptr
VScreate_string(VSULong vs_bound, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ULong bound = (CORBA::ULong) vs_bound;
		CORBA::Repository_ptr IR_ptr;
		CORBA::StringDef_ptr sDef_ptr;

		IRSupport *irs = glob_TS.repository();
		try {
				IR_ptr = irs->get_repository();
				sDef_ptr = IR_ptr->create_string(bound);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		}
		return (VSStringDef_ptr) sDef_ptr;
}

/*------------------------------------------------------*/
VSSequenceDef_ptr
VScreate_sequence(VSULong vs_bound, 
									const VSIDLType_ptr vs_element_type, 
									VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ULong bound = (CORBA::ULong) vs_bound;
		CORBA::IDLType_ptr element_type = (CORBA::IDLType_ptr) vs_element_type;
		CORBA::Repository_ptr IR_ptr;    
		CORBA::SequenceDef_ptr seqDef_ptr;

		IRSupport *irs = glob_TS.repository();
		try {
				IR_ptr = irs->get_repository();
				seqDef_ptr = IR_ptr->create_sequence(bound, element_type);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		}
		return (VSSequenceDef_ptr) seqDef_ptr;
}

/*------------------------------------------------------*/
VSArrayDef_ptr
VScreate_array(VSULong vs_length,
							 const VSIDLType_ptr vs_element_type, 
							 VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ULong length = (CORBA::ULong) vs_length;
		CORBA::IDLType_ptr element_type = (CORBA::IDLType_ptr) vs_element_type;
		CORBA::Repository_ptr IR_ptr;    
		CORBA::ArrayDef_ptr arDef_ptr;

		IRSupport *irs = glob_TS.repository();
		try {
				IR_ptr = irs->get_repository();
				arDef_ptr = IR_ptr->create_array(length, element_type);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		}
		return (VSArrayDef_ptr) arDef_ptr;
}

/*------------------------------------------------------*/
/* redefined functions */
/*------------------------------------------------------*/
VSDefinitionKind VSdef_kind(VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::Repository_ptr IR_ptr;
		CORBA::DefinitionKind dk;
		IRSupport *irs = glob_TS.repository();
    try {
				IR_ptr = irs->get_repository();
				dk = IR_ptr->def_kind();
			}
    catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		}
    return (VSDefinitionKind) dk;
}
 /*------------------------------------------------------*/

VSTypeCode_ptr VSget_canonical_typecode(const VSTypeCode_ptr vs_tc, 
					VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::Repository_ptr IR_ptr;
  CORBA::TypeCode_ptr rtc;
  CORBA::TypeCode_ptr otc = (CORBA::TypeCode_ptr) vs_tc;
  
  IRSupport *irs = glob_TS.repository();
  try {
    IR_ptr = irs->get_repository();
    rtc = IR_ptr->get_canonical_typecode(otc);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSTypeCode_ptr) rtc;
}

 /*------------------------------------------------------*/

VSWstringDef_ptr
VScreate_wstring(VSULong vs_bound, VSEnvironment *ev)
{
  CORBA::Environment *env = (CORBA::Environment *) ev;
  CORBA::ULong bound = (CORBA::ULong) vs_bound;
  CORBA::Repository_ptr IR_ptr;
  CORBA::WstringDef_ptr sDef_ptr;
  
  IRSupport *irs = glob_TS.repository();
  try {
    IR_ptr = irs->get_repository();
    sDef_ptr = IR_ptr->create_wstring(bound);
  }
  catch (CORBA::Exception &ex) {
    CORBA::Exception *t = exception_convert(&ex);
    if(env)
      env->exception(t);
  }
  return (VSWstringDef_ptr) sDef_ptr;
}

 /*------------------------------------------------------*/
}

