/*
   Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
   Copyright (c) 2001 Object Oriented Concepts GmbH.

   All rights reserved. No part of this source code may be reproduced,
   stored in a retrieval system, or transmitted, in any form or by any
   means, electronic, mechanical, photocopying, recording or otherwise,
   except as stated in the end-user licence agreement, without the prior
   permission of the copyright owners.
  
   Developed for X/Open by ApTest Ireland Limited.
  
   Project: CORVAL2
*/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

/* ------------------------------------------------------------ */
/* object_ptr is provided for the VSnarrow() test */
int
provide_object_ptr (VSObject_ptr **testvar,
											enum prop_object_ptr_objValue objValue)
{
	CORBA::Object_ptr tvar;
	CORBA::Object_ptr *rptr;
	CORBA::Repository_ptr IR_ptr;
	CORBA::Contained_ptr ctnD_ptr;
	CORBA::StructDef_ptr structDef_ptr;
	CORBA::StructMemberSeq* memberSeq_ptr;
	IRSupport *irs = glob_TS.repository();

    switch (objValue)  {
        case Pobject_ptr_objValue_valid:
        		IR_ptr = irs->get_repository();
						ctnD_ptr = irs->get_contained(IR_ptr,"M_SequenceDef::S1");
						structDef_ptr = CORBA::StructDef::_narrow(ctnD_ptr);
						memberSeq_ptr = structDef_ptr->members();
						tvar = (*memberSeq_ptr)[0].type_def;
						break;
       case Pobject_ptr_objValue_null:
            tvar = NULL;
						break;
    }
		rptr = new CORBA::Object_ptr(tvar);
		*testvar = (VSObject_ptr *) rptr;
		return TRUE;
}

int
relinquish_object_ptr (VSObject_ptr **testvar,
													enum prop_object_ptr_objValue objValue)
{
    CORBA::Object_ptr* tvar = (CORBA::Object_ptr *) *testvar;

    switch (objValue)  {
        case Pobject_ptr_objValue_valid:
						delete tvar;
						break;
				case Pobject_ptr_objValue_null:
        		break;
    }
		return TRUE;
}

/* ------------------------------------------------------------ */
/* sequencedef_ptr is provided for the VSduplicate() test */
int
provide_sequencedef_ptr (VSSequenceDef_ptr **testvar,
											enum prop_sequencedef_ptr_sequenceValue sequenceValue)
{
	CORBA::SequenceDef_ptr tvar;
	CORBA::SequenceDef_ptr* rptr;
	 
  CORBA::Repository_ptr  IR_ptr;
	CORBA::Contained_ptr ctnD_ptr;
	IRSupport *irs = glob_TS.repository();
	CORBA::StructDef_ptr structDef_ptr;
	CORBA::StructMemberSeq* memberSeq_ptr;
	CORBA::IDLType_ptr idlobj_ptr;

		switch (sequenceValue)  {
        case Psequencedef_ptr_sequenceValue_valid:
        		IR_ptr = irs->get_repository();
        		ctnD_ptr = irs->get_contained(IR_ptr,"M_SequenceDef::S1");
						structDef_ptr = CORBA::StructDef::_narrow(ctnD_ptr);
						memberSeq_ptr = structDef_ptr->members();
						idlobj_ptr = (CORBA::IDLType_ptr)(*memberSeq_ptr)[0].type_def;
						tvar = CORBA::SequenceDef::_narrow(idlobj_ptr);
						break;
       case Psequencedef_ptr_sequenceValue_null:
            tvar = NULL;
						break;
    }
		rptr = new CORBA::SequenceDef_ptr(tvar);
		*testvar = (VSSequenceDef_ptr *) rptr;
		return TRUE;
}

int
relinquish_sequencedef_ptr (VSSequenceDef_ptr **testvar,
													enum prop_sequencedef_ptr_sequenceValue sequenceValue)
{
	CORBA::SequenceDef_ptr* tvar = (CORBA::SequenceDef_ptr *) *testvar;

		switch (sequenceValue)  {
        case Psequencedef_ptr_sequenceValue_valid:
						delete tvar;
						break;
				case Psequencedef_ptr_sequenceValue_null:
						break;
    }
		return TRUE;
}

/* ------------------------------------------------------------ */
/* type is provided for the VSelement_type_def_write() test */
int
provide_type (VSIDLType_ptr *testvar)
{
    CORBA::IDLType_ptr *tvar = (CORBA::IDLType_ptr *) testvar;
		CORBA::Repository_ptr  IR_ptr;
		IRSupport *irs = glob_TS.repository();
		IR_ptr = irs->get_repository();
		try {
				*tvar = IR_ptr->get_primitive(CORBA::pk_short);
		}
		catch (...) {
				return FALSE;
		}
		return TRUE;
}

int 
relinquish_type (VSIDLType_ptr *testvar)
{
		return TRUE;
}
/* ------------------------------------------------------------ */

}
