/*
 Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
 Copyright (c) 2001 Object Oriented Concepts GmbH.

 All rights reserved. No part of this source code may be reproduced,
 stored in a retrieval system, or transmitted, in any form or by any
 means, electronic, mechanical, photocopying, recording or otherwise,
 except as stated in the end-user licence agreement, without the prior
 permission of the copyright owners.

 Developed for X/Open by ApTest Ireland Limited.

 Project: CORVAL2
*/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

/* ------------------------------------------------------------ */
/* object_ptr is provided for the VSnarrow() test. */
int
provide_object_ptr (VSObject_ptr **testvar,
											enum prop_object_ptr_objValue objValue)
{
	CORBA::Object_ptr tvar;
	CORBA::Object_ptr *rptr;
	CORBA::Object_ptr obj_ptr;
	CORBA::Repository_ptr IR_ptr;
	IRSupport *irs = glob_TS.repository();

    switch (objValue)  {
        case Pobject_ptr_objValue_valid:
        		IR_ptr = irs->get_repository();
        		tvar = irs->get_contained(IR_ptr,"M_StructDef::S1");
						break;
       case Pobject_ptr_objValue_null:
            tvar = NULL;
						break;
    }
		rptr = new CORBA::Object_ptr(tvar);
		*testvar = (VSObject_ptr *) rptr;
		return TRUE;
}

int
relinquish_object_ptr (VSObject_ptr **testvar,
													enum prop_object_ptr_objValue objValue)
{
	CORBA::Object_ptr* tvar = (CORBA::Object_ptr *) *testvar;

    switch (objValue)  {
        case Pobject_ptr_objValue_valid:
						delete tvar;
						break;
				case Pobject_ptr_objValue_null:
        		break;
    }
		return TRUE;
}

/* ------------------------------------------------------------ */
/* structdef_ptr is provided for the VSduplicate() test. */
int
provide_structdef_ptr (VSStructDef_ptr **testvar,
											enum prop_structdef_ptr_structValue structValue)
{
	CORBA::StructDef_ptr tvar;
	CORBA::StructDef_ptr *rptr;
	 
	CORBA::Object_ptr obj_ptr;
  CORBA::Repository_ptr  IR_ptr;
	CORBA::Contained_ptr ctnD_ptr;
	IRSupport *irs = glob_TS.repository();

		switch (structValue)  {
        case Pstructdef_ptr_structValue_valid:
        		IR_ptr = irs->get_repository();
        		ctnD_ptr = irs->get_contained(IR_ptr,"M_StructDef::S1");
						tvar = CORBA::StructDef::_narrow(ctnD_ptr);
						break;
       case Pstructdef_ptr_structValue_null:
            tvar = NULL;
						break;
    }
		rptr = new CORBA::StructDef_ptr(tvar);
		*testvar = (VSStructDef_ptr *) rptr;
		return TRUE;
}

int
relinquish_structdef_ptr (VSStructDef_ptr **testvar,
													enum prop_structdef_ptr_structValue structValue)
{
		CORBA::StructDef_ptr* tvar = (CORBA::StructDef_ptr *) *testvar;

		switch (structValue)  {
        case Pstructdef_ptr_structValue_valid:
						delete tvar;
						break;
				case Pstructdef_ptr_structValue_null:
						break;
    }
		return TRUE;
}

				 
/* ------------------------------------------------------------ */
/* new_members is provided for the VSmembers_write() test. */
int 
provide_new_members (VSStructMemberSeq_ptr *testvar)
{
		CORBA::Repository_ptr  IR_ptr;
		CORBA::Contained_ptr   ctnD_ptr;
		CORBA::StructMemberSeq* tvar;
		CORBA::StructMemberSeq* rptr;
								 
		IRSupport *irs = glob_TS.repository();
		IR_ptr = irs->get_repository();
		ctnD_ptr = irs->get_contained(IR_ptr,"M_StructDef::SCopy");
		if (!ctnD_ptr)
				return FALSE;
		CORBA::StructDef_ptr uDef_ptr = CORBA::StructDef::_narrow(ctnD_ptr);
		tvar = uDef_ptr->members();
		rptr = new CORBA::StructMemberSeq(*tvar);
		*testvar = (VSStructMemberSeq_ptr) rptr;
		return TRUE;
}


int 
relinquish_new_members (VSStructMemberSeq_ptr *testvar)
{
		CORBA::StructMemberSeq* tvar = (CORBA::StructMemberSeq *) *testvar;
		delete tvar;
		return TRUE;
}


/* ------------------------------------------------------------ */

}
