/*
 Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
 Copyright (c) 2001 Object Oriented Concepts GmbH.

 All rights reserved. No part of this source code may be reproduced,
 stored in a retrieval system, or transmitted, in any form or by any
 means, electronic, mechanical, photocopying, recording or otherwise,
 except as stated in the end-user licence agreement, without the prior
 permission of the copyright owners.

 Developed for X/Open by ApTest Ireland Limited.

 Project: CORVAL2
*/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

/* ------------------------------------------------------------ */
/* object_ptr is provided for the VSnarrow() test */
int
provide_object_ptr (VSObject_ptr **testvar,
											enum prop_object_ptr_objValue objValue)
{
	CORBA::Object_ptr tvar;
	CORBA::Object_ptr* rptr;
	CORBA::Object_ptr obj_ptr;
	CORBA::Repository_ptr IR_ptr;
	IRSupport *irs = glob_TS.repository();

    switch (objValue)  {
        case Pobject_ptr_objValue_valid:
        		IR_ptr = irs->get_repository();
        		tvar = irs->get_contained(IR_ptr,"M_UnionDef::U1");
						break;
       case Pobject_ptr_objValue_null:
            tvar = NULL;
						break;
    }
		rptr = new CORBA::Object_ptr(tvar);
		*testvar = (VSObject_ptr *) rptr;
		return TRUE;
}

int
relinquish_object_ptr (VSObject_ptr **testvar,
													enum prop_object_ptr_objValue objValue)
{
	CORBA::Object_ptr* tvar = (CORBA::Object_ptr *) *testvar;

    switch (objValue)  {
        case Pobject_ptr_objValue_valid:
						delete tvar;
						break;
				case Pobject_ptr_objValue_null:
        		break;
    }
		return TRUE;
}

/* ------------------------------------------------------------ */
/* uniondef_ptr is provided for the VSduplicate() test. */
int
provide_uniondef_ptr (VSUnionDef_ptr **testvar,
											enum prop_uniondef_ptr_unionValue unionValue)
{
	CORBA::UnionDef_ptr tvar;
	CORBA::UnionDef_ptr* rptr;
	 
	CORBA::Object_ptr obj_ptr;
  CORBA::Repository_ptr  IR_ptr;
	CORBA::Contained_ptr ctnD_ptr;
	IRSupport *irs = glob_TS.repository();

		switch (unionValue)  {
        case Puniondef_ptr_unionValue_valid:
        		IR_ptr = irs->get_repository();
        		ctnD_ptr  = irs->get_contained(IR_ptr,"M_UnionDef::U1");
						tvar = CORBA::UnionDef::_narrow(ctnD_ptr);
						break;
       case Puniondef_ptr_unionValue_null:
            tvar = NULL;
						break;
    }
		rptr = new CORBA::UnionDef_ptr(tvar);
		*testvar = (VSUnionDef_ptr *) rptr;
		return TRUE;
}

int
relinquish_uniondef_ptr (VSUnionDef_ptr **testvar,
													enum prop_uniondef_ptr_unionValue unionValue)
{
		CORBA::UnionDef_ptr* tvar = (CORBA::UnionDef_ptr *) *testvar;

		switch (unionValue)  {
        case Puniondef_ptr_unionValue_valid:
						delete tvar;
						break;
				case Puniondef_ptr_unionValue_null:
						break;
    }
		return TRUE;
}

/* ------------------------------------------------------------ */
/* new_members is provided for the VSmembers_write test. 
	 We provide them by reading the members of an existing Struct */
int 
provide_new_members (VSUnionMemberSeq_ptr *testvar)
{
		CORBA::Repository_ptr  IR_ptr;
		CORBA::Contained_ptr   ctnD_ptr;
		CORBA::UnionMemberSeq* tvar;
		CORBA::UnionMemberSeq* rptr;
								 
		IRSupport *irs = glob_TS.repository();
		IR_ptr = irs->get_repository();
		ctnD_ptr = irs->get_contained(IR_ptr,"M_UnionDef::UCopy");
		if (!ctnD_ptr)
				return FALSE;
		CORBA::UnionDef_ptr uDef_ptr = CORBA::UnionDef::_narrow(ctnD_ptr);
		tvar = uDef_ptr->members();
		rptr = new CORBA::UnionMemberSeq(*tvar);
		*testvar = (VSUnionMemberSeq_ptr) rptr;
		return TRUE;
}

int 
relinquish_new_members (VSUnionMemberSeq_ptr *testvar)
{
		CORBA::UnionMemberSeq* tvar = (CORBA::UnionMemberSeq *) *testvar;
		delete tvar;
		return TRUE;
}

/* ------------------------------------------------------------ */
/* type is provided for the VSdiscriminator_type_def_write() test */
int 
provide_type (VSIDLType_ptr *testvar)
{
    CORBA::IDLType_ptr *tvar = (CORBA::IDLType_ptr *) testvar;
		CORBA::Repository_ptr  IR_ptr;
		IRSupport *irs = glob_TS.repository();
		IR_ptr = irs->get_repository();
		try {
				*tvar = IR_ptr->get_primitive(CORBA::pk_short);
		}
		catch (...) {
				return FALSE;
		}
		return TRUE;
}

int 
relinquish_type (VSIDLType_ptr *testvar)
{
		return TRUE;
}
				 
/* ------------------------------------------------------------ */
}
