/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/***************************************************/
/* POA attributes                                  */
/***************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <POAtest_impl.h>

extern "C" {
#include <t1.h>
#include <t1_aux.h>
#include <t1i_tdi.h>

  PortableServer::POA_ptr rootPOA;
  PortableServer::POAManager_ptr poa_mgr;
  void get_poa()
  {   
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
    rootPOA = PortableServer::POA::_narrow(o);
    poa_mgr = rootPOA->the_POAManager();
  }
  
  /*------------------------------------------------------------------*/
  /* VSread_the_name() */
  VSString VSread_the_name(VSPOA_ptr *p_poa, VSEnvironment *ev)
  {
    PortableServer::POA_ptr *poa = (PortableServer::POA_ptr *) p_poa;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    char *poa_name;    
    try {
      poa_name = (*poa)->the_name();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSString) 0;
    }
    catch (...) {
      return (VSString) 0;
    }

    return (VSString) poa_name;
  }

  /*------------------------------------------------------------------*/
  /* VSread_the_parent() */
  VSPOA_ptr *VSread_the_parent(VSPOA_ptr *p_poa, VSEnvironment *ev)
  {
    PortableServer::POA_ptr *poa = (PortableServer::POA_ptr *) p_poa;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    PortableServer::POA_ptr parentPOA;  
    try {
      parentPOA = (*poa)->the_parent();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSPOA_ptr *) 0;
    }
    catch (...) {
      return (VSPOA_ptr *) 0;
    }
    
    PortableServer::POA_ptr *rpoa = new PortableServer::POA_ptr(parentPOA);
    return (VSPOA_ptr *) rpoa;
  }

  /*------------------------------------------------------------------*/
  /* VSread_the_POAManager() */
  VSPOAManager_ptr *VSread_the_POAManager(VSPOA_ptr *p_poa, VSEnvironment *ev)
  {
    PortableServer::POA_ptr *poa = (PortableServer::POA_ptr *) p_poa;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    PortableServer::POAManager_ptr poaMgr;
    try {
      poaMgr = (*poa)->the_POAManager();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSPOAManager_ptr *) 0;
    }
    catch (...) {
      return (VSPOAManager_ptr *) 0;
    }
    
    PortableServer::POAManager_ptr *rpoaMgr = new PortableServer::POAManager_ptr(poaMgr);
    return (VSPOAManager_ptr *) rpoaMgr;
  }

  /*------------------------------------------------------------------*/
  /* VSread_the_children() */
  VSPOAList *VSread_the_children(VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;
    
    PortableServer::POA_ptr rootPOA;
    PortableServer::POAManager_ptr poa_mgr;
    try {  
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
      poa_mgr = rootPOA->the_POAManager();
    }
    catch (...) {
      return (VSPOAList *) 0;
    }
   
    PortableServer::POAList* childPOAList;
    try {
      CORBA::PolicyList polList;	  
      PortableServer::POA_ptr test_POA = rootPOA->create_POA("childPOA", poa_mgr, polList);
      
      childPOAList = rootPOA->the_children();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSPOAList *) 0;
    }
    catch (...) {
      return (VSPOAList *) 0;
    }    

    PortableServer::POAList *rlist = new PortableServer::POAList(*childPOAList);
    return (VSPOAList *) rlist;
  }

  /*------------------------------------------------------------------*/
  /* VSwrite_the_activator() */
  VSBoolean VSwrite_the_activator(VSPOA_ptr *p_poa, VSEnvironment *ev)
  {
    PortableServer::POA_ptr *poa = (PortableServer::POA_ptr *) p_poa;
    CORBA::Environment *env = (CORBA::Environment *) ev;

     adpActivator* adapter_actSvt = new adpActivator;
     PortableServer::AdapterActivator_ptr adapter_act = adapter_actSvt->_this();
    try {
      (*poa)->the_activator(adapter_act);
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return FALSE;
    }
    catch (...) {
      return FALSE;
    }
   
    return TRUE;
  }


  /*------------------------------------------------------------------*/
  /* VSread_the_activator() */
  VSAdapterActivator_ptr *VSread_the_activator(VSPOA_ptr *p_poa, VSEnvironment *ev)
  {
    PortableServer::POA_ptr *poa = (PortableServer::POA_ptr *) p_poa;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    PortableServer::AdapterActivator_ptr activator;
    try {
      activator = (*poa)->the_activator();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSAdapterActivator_ptr *) 0;
    }
    catch (...) {
      return (VSAdapterActivator_ptr *) 0;
    }
    
    PortableServer::AdapterActivator_ptr *ract = new PortableServer::AdapterActivator_ptr(activator);
    return (VSAdapterActivator_ptr *) ract;
  }

}
