/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/


/********************************************************/
/* Operations for the USE_DEFAULT_SERVANT policy        */
/********************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <POAtest_impl.h>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

  PortableServer::POA_ptr rootPOA;
  PortableServer::POAManager_ptr poa_mgr;
  void get_poa()
  {   
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
    rootPOA = PortableServer::POA::_narrow(o);
    poa_mgr = rootPOA->the_POAManager();
  }

  /********************************************************/
  /* test variable VSPOA_ptr * setSvtPOA(prop value) */

  int provide_setSvtPOA (VSPOA_ptr **testvar, enum prop_setSvtPOA_value value)
  {
    PortableServer::POA_ptr *tvar;
    
    get_poa();

    switch (value) {
    case PsetSvtPOA_value_defaltSvt:
       {
	try {
	  PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_DEFAULT_SERVANT);
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);	
	  
	  CORBA::PolicyList polList;
	  polList.length(3);	 
	  
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  polList[2] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("defaultSvt", poa_mgr, polList);	

	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	// without MULTIPLE_ID policy ->  error!!!
	catch (PortableServer::POA::InvalidPolicy &ex) {
	  return FALSE;
	}      
	catch (...) {
	  return FALSE;
	}  	            
	break;
      }        
    case PsetSvtPOA_value_noDefaultSvt:
      {
	try {
	  CORBA::PolicyList polList;
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("noDefaultSvt", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }     
    }

    *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }

  int relinquish_setSvtPOA (VSPOA_ptr **testvar, enum prop_setSvtPOA_value value)
  {
    PortableServer::POA_ptr *tvar = (PortableServer::POA_ptr *) *testvar;
    switch (value) {
    case PsetSvtPOA_value_defaltSvt:
    case PsetSvtPOA_value_noDefaultSvt:
      (*tvar)->destroy(TRUE, TRUE);
      break;
    }
  }

  /*****************************************************/
  /* test variable VSServant * svt(prop value) */

  int provide_svt (VSServant **testvar, enum prop_svt_value value)
  {
    PortableServer::Servant *svt;
   
    switch (value) {
    case Psvt_value_valid:
      {
        PortableServer::Servant test_svt;
        try {
          test_svt = new I_impl::I_impl(1);
        }
        catch (...) {
          return FALSE;
        }
        svt = new PortableServer::Servant(test_svt);
      }
      break;
    }

    *testvar = (VSServant *) svt;

    return TRUE;
  }
  
  int relinquish_svt (void*** testvar, enum prop_svt_value value)
  {
    PortableServer::Servant *tvar = (PortableServer::Servant *) *testvar; 

    switch (value) {
    case Psvt_value_valid:
      // delete *tvar; ?! error
      break;   
    }
    
    return TRUE;
  }

  /************************************************************/
  /* test variable VSPOA_ptr * getSvtPOA(prop value) */

  int provide_getSvtPOA (void*** testvar, enum prop_getSvtPOA_value value)
  {
    PortableServer::POA_ptr *tvar;
    
    get_poa(); 

    switch (value) {
    case PgetSvtPOA_value_defaltSvt:
      {
	try {
	  PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_DEFAULT_SERVANT);
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);	
	  
	  CORBA::PolicyList polList;
	  polList.length(3);	 
	  
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  polList[2] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("defaltSvt", poa_mgr, polList);

	  PortableServer::Servant svt = new I_impl::I_impl(1);
	  test_POA->set_servant(svt);

	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	// without MULTIPLE_ID policy ->  error!!!
	catch (PortableServer::POA::InvalidPolicy &ex) {
	  return FALSE;
	}      
	catch (...) {
	  return FALSE;
	}  	            
	break;
      }       
    case PgetSvtPOA_value_noDefaultSvt:
      {
	try {
	  CORBA::PolicyList polList;
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("noDefaultSvt", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }   
    case PgetSvtPOA_value_nosetSvt:
     {
	try {
	  PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_DEFAULT_SERVANT);
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);	
	  
	  CORBA::PolicyList polList;
	  polList.length(3);	 
	  
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  polList[2] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("nosetSvt", poa_mgr, polList);
	 
	 tvar = new PortableServer::POA_ptr(test_POA);
	}
	// without MULTIPLE_ID policy ->  error!!!
	catch (PortableServer::POA::InvalidPolicy &ex) {
	  return FALSE;
	}      
	catch (...) {
	  return FALSE;
	}  	            
	break;
      }     
    }
    
    *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }  
  
  int relinquish_getSvtPOA (void*** testvar, enum prop_getSvtPOA_value value)
  {
    PortableServer::POA_ptr *tvar = (PortableServer::POA_ptr *) *testvar;
    switch (value) {
    case PgetSvtPOA_value_defaltSvt:
    case PgetSvtPOA_value_noDefaultSvt:
    case PgetSvtPOA_value_nosetSvt:
      (*tvar)->destroy(TRUE, TRUE);
      break;
    }
    return TRUE;
  }

}

