/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/**************************************************/
/* Identity mapping operations                    */
/**************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <POAtest_impl.h>

extern "C" {
#include <t1.h>
#include <t1_aux.h>
#include <t1i_tdi.h>


  /*------------------------------------------------------------------*/
  /* reference_to_id(..) */
  VSObjectId_var *VSreference_to_id(VSString p_str, VSPOA_ptr *thePOA, VSEnvironment *ev)
  {
    char *str = (char *) p_str;
    PortableServer::POA_ptr *test_poa = (PortableServer::POA_ptr *) thePOA; 
    CORBA::Environment *env = (CORBA::Environment *) ev;

   
    PortableServer::POA_ptr rootPOA;   
    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);      
    }
    catch (...) {
      return (VSObjectId_var *) 0;
    }       

    CORBA::Object_ptr objref;
   
    if (strcmp(str, "objinPOA") == 0)
      {
       	try {
	  PortableServer::ObjectId_var oid = PortableServer::string_to_ObjectId(str);
	  objref = (*test_poa)->create_reference_with_id(oid, "M_POAtest::I");
	}       
	catch (...) {
	  return (VSObjectId_var *) 0;
	}
      }
    if (strcmp(str, "objNotinPOA") == 0)
      {
	try {      
	  objref = rootPOA->create_reference("M_POAtest::I");
	}       
	catch (...) {
	  return (VSObjectId_var *) 0;
	}
      }

   PortableServer::ObjectId_var objId;
   try {     
     objId = (*test_poa)->reference_to_id(objref);
   }
   catch (CORBA::Exception &ex) {
     CORBA::Exception *t = exception_convert(&ex);
    
     if(env){
       env->exception(t);
     }
     return (VSObjectId_var *) 0;
   }
   catch (...) {
     return (VSObjectId_var *) 0;
   }
   
   PortableServer::ObjectId_var *roid = new PortableServer::ObjectId_var(objId);
   return (VSObjectId_var *) roid;

  }
 
  /*------------------------------------------------------------------*/
  /* id_to_reference(..) */

  VSObject_ptr *VSid_to_reference(VSObjectId_var *p_oid, VSPOA_ptr *poa, VSEnvironment *ev)
  {
    PortableServer::ObjectId_var *oid = (PortableServer::ObjectId_var *) p_oid;
    PortableServer::POA_ptr *test_poa = (PortableServer::POA_ptr *) poa;
    CORBA::Environment *env = (CORBA::Environment *) ev;
 
    char *poa_name, *oid_str;   
    try {
      poa_name = (*test_poa)->the_name();
      oid_str = PortableServer::ObjectId_to_string(*oid);
    }
    catch (...) {
      return (VSObject_ptr *) 0;
    } 

    CORBA::Object_ptr obj;  
    
    if((strcmp(poa_name, "retainPOA") || strcmp(oid_str, "actRetain")) == 0)
      {
	try {
	  PortableServer::Servant svt = new I_impl::I_impl(1);
	  (*test_poa)->activate_object_with_id(*oid, svt);	 
	}
	catch (...) {
	  return (VSObject_ptr *) 0;
	    }
	  
	try {
	  obj =(*test_poa)->id_to_reference(*oid);
	}
	catch (CORBA::Exception &ex) {
	  CORBA::Exception *t = exception_convert(&ex);
	  if(env)
	    env->exception(t);
	  return (VSObject_ptr *) 0;
	}
      }	  
    else
      {
	if ((strcmp(poa_name, "retainPOA") && strcmp(oid_str, "refRetain")) == 0)
	  { 
	    try {
	      CORBA::Object_ptr refRetainobj = (*test_poa)->create_reference_with_id(*oid, "M_POAtest::I");
	    }
	    catch (...) {
	      return (VSObject_ptr *) 0;
	    }
	  
	    try {
	      obj =(*test_poa)->id_to_reference(*oid);
	    }
	    catch (CORBA::Exception &ex) {
	      CORBA::Exception *t = exception_convert(&ex);
	      if(env)
		env->exception(t);
	      return (VSObject_ptr *) 0;
	    }	  
	  }
	else
	  {	    
	    if((strcmp(poa_name, "nonRetainPOA" ) && strcmp(oid_str, "refNonRetain"))== 0)
	      {
		try {
		  CORBA::Object_ptr refRetainobj = (*test_poa)->create_reference_with_id(*oid, "M_POAtest::I");
	  	}
		catch (...) {
		  return (VSObject_ptr *) 0;
		}
	  
		try {
		  obj =(*test_poa)->id_to_reference(*oid);
		}
		catch (CORBA::Exception &ex) {
		  CORBA::Exception *t = exception_convert(&ex);
		  if(env)
		    env->exception(t);
		  return (VSObject_ptr *) 0;
		}
	      }
	    else
	      return (VSObject_ptr *) 0;
	  }
      }

    CORBA::Object_ptr *robj = new CORBA::Object_ptr(obj);
    return (VSObject_ptr *) robj;
  }

}
