/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/**************************************************/
/* Identity mapping operations                    */
/**************************************************/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

  /*-----------------------------------------------------------*/
  /* test variable VSString * refToid(prop value) */
  int provide_refToid (VSString *testvar, enum prop_refToid_value value)
  {
    char **tvar = (char **) testvar;

    switch(value) {
    case PrefToid_value_inPOA:
      {
	try {
	  *tvar = CORBA::string_dup("objinPOA");
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
    case PrefToid_value_notinPOA:    
      {
	try {
	  *tvar = CORBA::string_dup("objNotinPOA");
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
    }
    
    return TRUE;  
  }

  int relinquish_refToid (VSString *testvar, enum prop_refToid_value value)
  {
    char **tvar = (char **) testvar;

    switch(value) {    
    case PrefToid_value_inPOA: 
    case PrefToid_value_notinPOA:
      CORBA::string_free(*tvar);
      break;
    }
  
    return TRUE;
  }


  /*-------------------------------------------------------------*/
  /* test variable VSPOA_ptr * refToid_POA(prop value) */
  int provide_refToid_POA (VSPOA_ptr **testvar, enum prop_refToid_POA_value value)
  {
    PortableServer::POA_ptr *tvar;

    CORBA::PolicyList polList;

    PortableServer::POA_ptr rootPOA;
    PortableServer::POAManager_ptr poa_mgr;
  
    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
      poa_mgr = rootPOA->the_POAManager();
    }
    catch (...) {
      return FALSE;
    }       

    switch (value) {
    case PrefToid_POA_value_testPOA:
      {
	try {
	  PortableServer::IdAssignmentPolicy_var assignmentPol = rootPOA->create_id_assignment_policy(PortableServer::USER_ID); 

	  polList.length(1);
	  polList[0] = PortableServer::IdAssignmentPolicy::_duplicate(assignmentPol);

	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("testPOA", poa_mgr, polList);
	  assignmentPol->destroy();
 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}
      
	break;
      }

    }
    *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }

  int relinquish_refToid_POA (VSPOA_ptr **testvar, enum prop_refToid_POA_value value)
  {
    PortableServer::POA_ptr  *tvar = (PortableServer::POA_ptr *) *testvar;
    switch (value) {
    case PrefToid_POA_value_testPOA:
      (*tvar)->destroy(TRUE, TRUE);
      break;
    }

    return TRUE;
  } 


  /*------------------------------------------------------------*/
  /* test variable VSObjectId_var * idToref(prop value) */
  int provide_idToref (VSObjectId_var **testvar, enum prop_idToref_value value)
  {
    PortableServer::ObjectId_var tvar;

    PortableServer::POA_ptr rootPOA;
    PortableServer::POAManager_ptr poa_mgr;
  
    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
      poa_mgr = rootPOA->the_POAManager();      
    }
    catch (...) {
      return FALSE;
    } 
         
     

    switch (value) {
    case PidToref_value_actRetain:
      {
	try {	 
	  tvar = PortableServer::string_to_ObjectId("actRetain");	
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
    case  PidToref_value_refRetain:
      {	
	try {	   	
	  tvar = PortableServer::string_to_ObjectId("refRetain");	 
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
    case  PidToref_value_refNonRetain:
      {
	try {	
	  tvar = PortableServer::string_to_ObjectId("refNonRetain");	  
	}
	catch (...) {
	  return FALSE;
	}     
      break;
      }
    }   
  
    try {
      PortableServer::ObjectId_var *tmp = new PortableServer::ObjectId_var();
      *tmp = tvar;
      *testvar = (VSObjectId_var *) tmp;
    }
    catch (...) {
      return FALSE;
    }
    
    return TRUE; 

  }

  int relinquish_idToref (void*** testvar, enum prop_idToref_value value)
  {
    PortableServer::ObjectId_var *tvar = (PortableServer::ObjectId_var *) *testvar;

    switch (value) {
    case PidToref_value_actRetain:
    case PidToref_value_refRetain:
    case PidToref_value_refNonRetain:
      break;
    }     
  }


  /*-------------------------------------------------------------*/
  /* test variable VSPOA_ptr * idToref_POA(prop value) */
  int provide_idToref_POA (VSPOA_ptr **testvar, enum prop_idToref_POA_value value)
  {
    PortableServer::POA_ptr *tvar;   

    PortableServer::POA_ptr rootPOA;
    PortableServer::POAManager_ptr poa_mgr;
  
    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
      poa_mgr = rootPOA->the_POAManager();
    }
    catch (...) {
      return FALSE;
    }
    CORBA::PolicyList polList_Retain, polList_NonRetain;   
  
    PortableServer::IdAssignmentPolicy_var assignmentPol = rootPOA->create_id_assignment_policy(PortableServer::USER_ID);
    PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);	
    PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
    

    switch (value) {
    case PidToref_POA_value_retainPOA:
      {
	try {
	  polList_Retain.length(1);
	  polList_Retain[0] = PortableServer::IdAssignmentPolicy::_duplicate(assignmentPol);

	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("retainPOA", poa_mgr, polList_Retain); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
    case PidToref_POA_value_nonRetainPOA:
      {
	try {
	  polList_NonRetain.length(3);
	  polList_NonRetain[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  polList_NonRetain[1] = PortableServer::IdAssignmentPolicy::_duplicate(assignmentPol);
	  polList_NonRetain[2] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);

	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("nonRetainPOA", poa_mgr, polList_NonRetain); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      } 

    }
    *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }

  int relinquish_idToref_POA (VSPOA_ptr **testvar, enum prop_idToref_POA_value value)
  {
    PortableServer::POA_ptr  *tvar = (PortableServer::POA_ptr *) *testvar;
    switch (value) {
    case PidToref_POA_value_retainPOA:
    case PidToref_POA_value_nonRetainPOA:
      (*tvar)->destroy(TRUE, TRUE);
      break;
    }

    return TRUE;
  }

}
