/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/**************************************************/
/* Identity mapping operations                    */
/**************************************************/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

  PortableServer::POA_ptr rootPOA;
  PortableServer::POAManager_ptr poa_mgr;
  void get_poa()
  {   
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
    rootPOA = PortableServer::POA::_narrow(o);
    poa_mgr = rootPOA->the_POAManager();
  }

  /*-----------------------------------------------------------*/
  /* test variable VSString *objStr(prop value) */

  int provide_objStr (VSString *testvar, enum prop_objStr_value value)
  {
    char **tvar = (char **) testvar;

    switch (value) {
    case PobjStr_value_inAOM:
      {
	try {	 
	  *tvar = CORBA::string_dup("inAOM"); 
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
    case PobjStr_value_notinAOM:
      {
	try {	 
	  *tvar = CORBA::string_dup("notinAOM");
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
    case PobjStr_value_inotherPOA:
      {
	try {	 
	  *tvar = CORBA::string_dup("inotherPOA");
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }     
    }
   
    return TRUE; 
  }

  int relinquish_objStr (VSString *testvar, enum prop_objStr_value value)
  {
    char **tvar = (char **) testvar; 
    switch (value) {
    case PobjStr_value_inAOM:
    case PobjStr_value_notinAOM:
    case PobjStr_value_inotherPOA:
      CORBA::string_free(*tvar);     
      break; 
    }
    return TRUE;
  }

  /*-----------------------------------------------------------*/
  /* test variable VSPOA_ptr * refTosvt_POA(prop value) */

  int provide_refTosvt_POA (VSPOA_ptr **testvar, enum prop_refTosvt_POA_value value)
  {
    PortableServer::POA_ptr *tvar;
    
    get_poa();    
   
    PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);   
    
    switch (value) {
    case PrefTosvt_POA_value_poaRetain:
      {
	try {
	  CORBA::PolicyList polList;
	 
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaRetain", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
    case PrefTosvt_POA_value_poaDefaultSvt:
      {
	try {
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_DEFAULT_SERVANT);
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);	
	  
	  CORBA::PolicyList polList;
	  polList.length(3);	 
	  
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  polList[2] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaDefaultSvt", poa_mgr, polList);	

	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	// without MULTIPLE_ID policy ->  error!!!
	catch (PortableServer::POA::InvalidPolicy &ex) {
	  return FALSE;
	}      
	catch (...) {
	  return FALSE;
	}  	            
	break;
      }   
    case PrefTosvt_POA_value_poaSvtMgr:
      {
	try {
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
	  
	  
	  CORBA::PolicyList polList;
	  polList.length(2);
	    
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaSvtMgr", poa_mgr, polList); 
	  
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
     
    }
    
    retentionPol->destroy();

    *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }


  int relinquish_refTosvt_POA (VSPOA_ptr **testvar, enum prop_refTosvt_POA_value value)
  {
    PortableServer::POA_ptr *tvar = (PortableServer::POA_ptr *) *testvar;

    switch (value) {
    case PrefTosvt_POA_value_poaRetain:
    case PrefTosvt_POA_value_poaDefaultSvt:
    case PrefTosvt_POA_value_poaSvtMgr:
      (*tvar)->destroy(TRUE, TRUE);
      break;
    }
  }


  /*-----------------------------------------------------------*/
  /* test variable VSPOA_ptr * svtToref_POA(prop value) */

  int provide_svtToref_POA (VSPOA_ptr **testvar, enum prop_svtToref_POA_value value)
  {
    PortableServer::POA_ptr *tvar;
    
    get_poa(); 

    PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);
    PortableServer::ImplicitActivationPolicy_var activePol = rootPOA->create_implicit_activation_policy(PortableServer::IMPLICIT_ACTIVATION);
   
    switch (value) {
    case PsvtToref_POA_value_poaUniqueId:
      {
	try {
	  CORBA::PolicyList polList;	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaUniqueId", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
      case PsvtToref_POA_value_poaMultId:
      {
	try {
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);
	  
	  CORBA::PolicyList polList;
	  polList.length(2);
	  polList[0] = PortableServer::ImplicitActivationPolicy::_duplicate(activePol);
	  polList[1] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaMultId", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
      case PsvtToref_POA_value_poaImplicit:
      {
	try {
	  CORBA::PolicyList polList;
	  polList.length(1);

	  polList[0] = PortableServer::ImplicitActivationPolicy::_duplicate(activePol);
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaImplicit", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
      case PsvtToref_POA_value_poaDefaultSvt:
      {
	try {
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_DEFAULT_SERVANT);
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);	

	  CORBA::PolicyList polList;
	  polList.length(3);	 
	  
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  polList[2] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaDefault", poa_mgr, polList);	

	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
      case PsvtToref_POA_value_poaOther:  
      {       
	try {
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);

	  CORBA::PolicyList polList;

	  polList.length(2);
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol); 
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaOther", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
    }   

    *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }  
 
  int relinquish_svtToref_POA (VSPOA_ptr **testvar, enum prop_svtToref_POA_value value)
  {
    PortableServer::POA_ptr *tvar = (PortableServer::POA_ptr *) *testvar;
    switch (value) {
    case PsvtToref_POA_value_poaUniqueId:
    case PsvtToref_POA_value_poaMultId:
    case PsvtToref_POA_value_poaImplicit:
    case PsvtToref_POA_value_poaDefaultSvt:
    case PsvtToref_POA_value_poaOther:         
      (*tvar)->destroy(TRUE, TRUE);
      break;
    }
    return TRUE;
  }

}
