/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/**************************************************/
/* Identity mapping operations                    */
/**************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <POAtest_impl.h>

extern "C" {
#include <t1.h>
#include <t1_aux.h>
#include <t1i_tdi.h>

  /*------------------------------------------------------------------*/
  /* id_to_servant(..) */

  VSServant *VSid_to_servant(VSObjectId_var *p_oid, VSPOA_ptr *thePOA, VSEnvironment *ev)
  {
    PortableServer::ObjectId_var *oid = (PortableServer::ObjectId_var *) p_oid;
    PortableServer::POA_ptr *test_poa = (PortableServer::POA_ptr *) thePOA; 
    CORBA::Environment *env = (CORBA::Environment *) ev;
   
    char *poa_name, *oid_str;   
    try {
      poa_name = (*test_poa)->the_name();
      oid_str = PortableServer::ObjectId_to_string(*oid);
    }
    catch (...) {
      return (VSServant *) 0;
    } 

    PortableServer::Servant svt; 
    PortableServer::Servant test_svt = new I_impl::I_impl(1);
    
    if(strcmp(poa_name, "poaRetain") == 0)
      {	
	if (strcmp(oid_str, "inAOM") == 0)
	  {
	    try {
	      (*test_poa)->activate_object_with_id(*oid, test_svt);
	    }
	    catch (...) {
	      return (VSServant *) 0;	
	    }
	  }
	else
	  {	    
	    try {
	      (*test_poa)->create_reference_with_id(*oid, "M_POAtest::I");
	    }
	    catch (...) {
	      return (VSObjectId_var *) 0;	
	    }
	  }
      } 

    if(strcmp(poa_name, "poaSvtMgr") == 0)
      {
	try {
	  CORBA::Object_ptr obj = (*test_poa)->create_reference_with_id(*oid, "M_POAtest::I");
	}
	catch (...) {
	  return (VSObjectId_var *) 0;	
	}
      }

    if (strcmp(poa_name, "poaDefaultSvt") == 0)	
      {	
	CORBA::Object_ptr defaultsvt_obj;
	try {
	  (*test_poa)->set_servant(test_svt);
	  defaultsvt_obj = (*test_poa)->create_reference_with_id(*oid, "M_POAtest::I");
	  test_svt->_remove_ref();
	}
	catch (...) {
	  return (VSObjectId_var *) 0;	
	}
	
	try {
	  PortableServer::ObjectId_var oid = (*test_poa)->reference_to_id(defaultsvt_obj);
	  svt = (*test_poa)->id_to_servant(oid);
	}
	catch (CORBA::Exception &ex) {
	  CORBA::Exception *t = exception_convert(&ex);
	  if(env)
	    env->exception(t);
	  return (VSServant *) 0;
	} 
      }
    else {
      try {
	  svt = (*test_poa)->id_to_servant(*oid);
	}
	catch (CORBA::Exception &ex) {
	  CORBA::Exception *t = exception_convert(&ex);
	  if(env)
	    env->exception(t);
	  return (VSServant *) 0;
	} 
    }

    
	// return (VSServant) svt;
	PortableServer::Servant *rsvt = new PortableServer::Servant(svt);
	return (VSServant *) rsvt;

  }  


  /*------------------------------------------------------------------*/
  /* servant_to_id(..) */

  VSObjectId_var *VSservant_to_id(VSULong p_svt, VSPOA_ptr *thePOA, VSEnvironment *ev)
  {
    CORBA::ULong svt = p_svt;
    PortableServer::POA_ptr *test_poa = (PortableServer::POA_ptr *) thePOA; 
    CORBA::Environment *env = (CORBA::Environment *) ev;

    char *poa_name;   
    try {
      poa_name = (*test_poa)->the_name();
    }
    catch (...) {
      return (VSObjectId_var *) 0;
    }

    PortableServer::ObjectId_var objId;
    PortableServer::Servant test_svt = new I_impl::I_impl(1);
    
    if (svt==1)
      {    
	if (strcmp(poa_name, "poaUniqueId") == 0) 
	  {
	    try {	     
	      PortableServer::ObjectId_var oid = (*test_poa)->activate_object(test_svt); 
	    }
	    catch (...) {
	      return (VSObjectId_var *) 0;
	    }	 	
	  }	
	if (strcmp(poa_name, "poaMultId") == 0)
	  { 
	    try {	      
	     PortableServer::ObjectId_var oid = (*test_poa)->activate_object(test_svt); 
	    }
	    catch (...) {
	      return (VSObjectId_var *) 0;
	    }
	  }
	if (strcmp(poa_name, "poaImplicit") == 0)
	  { 
	    try {	      
	      PortableServer::ObjectId_var oid = (*test_poa)->activate_object(test_svt); 
	    }
	    catch (...) {
	      return (VSObjectId_var *) 0;
	    }
	  }
      }

    if (strcmp(poa_name, "poaDefaultSvt") == 0)	
      {	
	
	try {
	  (*test_poa)->set_servant(test_svt);	 
	  test_svt->_remove_ref();
	}
	catch (...) {
	  return (VSObjectId_var *) 0;	
	}
      }

    try {
      objId = (*test_poa)->servant_to_id(test_svt);
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSObjectId_var *) 0;
    }

    PortableServer::ObjectId_var *roid = new PortableServer::ObjectId_var(objId);
    return (VSObjectId_var *) roid;
  }
}
