/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/**************************************************/
/* Identity mapping operations                    */
/**************************************************/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

  PortableServer::POA_ptr rootPOA;
  PortableServer::POAManager_ptr poa_mgr;
  void get_poa()
  {   
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
    rootPOA = PortableServer::POA::_narrow(o);
    poa_mgr = rootPOA->the_POAManager();
  }

  /*-----------------------------------------------------------*/
  /* test variable VSObjectId_var * oid(prop value) */

  int provide_oid (VSObjectId_var **testvar, enum prop_oid_value value)
  {
    PortableServer::ObjectId_var tvar;

    switch (value) {
    case Poid_value_inAOM:
      {
	try {	 
	  tvar = PortableServer::string_to_ObjectId("inAOM");	
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
       case Poid_value_notinAOM:
      {
	try {	 
	  tvar = PortableServer::string_to_ObjectId("notinAOM");	
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
    } 
   
     try {
      PortableServer::ObjectId_var *tmp = new PortableServer::ObjectId_var();
      *tmp = tvar;
      *testvar = (VSObjectId_var *) tmp;
    }
    catch (...) {
      return FALSE;
    }    
    return TRUE; 
  }

  int relinquish_oid (VSObjectId_var **testvar, enum prop_oid_value value)
  {
    PortableServer::ObjectId_var *tvar = (PortableServer::ObjectId_var *) *testvar; 
    switch (value) {
    case Poid_value_inAOM:

      break; 
    }
  }

  /*-----------------------------------------------------------*/
  /* test variable VSPOA_ptr * idTosvt_POA(prop value) */

  int provide_idTosvt_POA (VSPOA_ptr **testvar, enum prop_idTosvt_POA_value value)
  {
    PortableServer::POA_ptr *tvar;
    
    get_poa();  
  
   
    PortableServer::IdAssignmentPolicy_var assignmentPol = rootPOA->create_id_assignment_policy(PortableServer::USER_ID);
    PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);   
    
    switch (value) {
    case PidTosvt_POA_value_poaRetain:
      {
	try {
	  CORBA::PolicyList polList;
	  polList.length(1);	 
	  polList[0] = PortableServer::IdAssignmentPolicy::_duplicate(assignmentPol);

	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaRetain", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
      case PidTosvt_POA_value_poaDefaultSvt:
      {
	try {
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_DEFAULT_SERVANT);
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);
	

	  CORBA::PolicyList polList;
	  polList.length(4);	 
	  polList[0] = PortableServer::IdAssignmentPolicy::_duplicate(assignmentPol);
	  polList[1] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  polList[2] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);	  
	  polList[3] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaDefault", poa_mgr, polList); 
	

	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }      
      case PidTosvt_POA_value_poaSvtMgr:
      {
	try {
	   PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);

	   CORBA::PolicyList polList;
	   polList.length(3);
	    
	   polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	   polList[1] = PortableServer::IdAssignmentPolicy::_duplicate(assignmentPol);
	   polList[2] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  
	   PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaSvtMgr", poa_mgr, polList); 
	 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
    }
    
    assignmentPol->destroy();
    retentionPol->destroy();

    *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }


  int relinquish_idTosvt_POA (VSPOA_ptr **testvar, enum prop_idTosvt_POA_value value)
  {
    PortableServer::POA_ptr *tvar = (PortableServer::POA_ptr *) *testvar;

    switch (value) {
    case PidTosvt_POA_value_poaRetain:
    case PidTosvt_POA_value_poaDefaultSvt:
    case PidTosvt_POA_value_poaSvtMgr:    
      (*tvar)->destroy(TRUE, TRUE);
      break;
    }
  }


  /*-----------------------------------------------------------*/
  /* test variable VSPOA_ptr * svtToid_POA(prop value) */

  int provide_svtToid_POA (VSPOA_ptr **testvar, enum prop_svtToid_POA_value value)
  {
    PortableServer::POA_ptr *tvar;
    
    get_poa(); 

    PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);
    PortableServer::ImplicitActivationPolicy_var activePol = rootPOA->create_implicit_activation_policy(PortableServer::IMPLICIT_ACTIVATION);
   
    switch (value) {
    case PsvtToid_POA_value_poaUniqueId:
      {
	try {
	  CORBA::PolicyList polList;	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaUniqueId", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
      case PsvtToid_POA_value_poaMultId:
      {
	try {
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);
	  
	  CORBA::PolicyList polList;
	  polList.length(2);
	  polList[0] = PortableServer::ImplicitActivationPolicy::_duplicate(activePol);
	  polList[1] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaMultId", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
      case PsvtToid_POA_value_poaImplicit:
      {
	try {
	  CORBA::PolicyList polList;
	  polList.length(1);

	  polList[0] = PortableServer::ImplicitActivationPolicy::_duplicate(activePol);
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaImplicit", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
      case PsvtToid_POA_value_poaDefaultSvt:
      {
	try {
	   PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_DEFAULT_SERVANT);
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);	

	  CORBA::PolicyList polList;
	  polList.length(3);	 
	  
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  polList[2] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaDefault", poa_mgr, polList);	

	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
      case PsvtToid_POA_value_poaOther:  
      {       
	try {
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);

	  CORBA::PolicyList polList;

	  polList.length(2);
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol); 
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("poaOther", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}      
	break;
      }
    }   

    *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }  
 
  int relinquish_svtToid_POA (VSPOA_ptr **testvar, enum prop_svtToid_POA_value value)
  {
    PortableServer::POA_ptr *tvar = (PortableServer::POA_ptr *) *testvar;
    switch (value) {
    case PsvtToid_POA_value_poaUniqueId:
    case PsvtToid_POA_value_poaMultId:
    case PsvtToid_POA_value_poaImplicit:
    case PsvtToid_POA_value_poaDefaultSvt:
    case PsvtToid_POA_value_poaOther:         
      (*tvar)->destroy(TRUE, TRUE);
      break;
    }
    return TRUE;
  }

}
