/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/*************************************************/
/* Object activation and deactivation            */
/*************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <POAtest_impl.h>

extern "C" {
#include <t1.h>
#include <t1_aux.h>
#include <t1i_tdi.h>


  /*------------------------------------------------------*/
  /* VSactivate_object() */

  VSObjectId* VSactivate_object(VSServant *svt, VSPOA_ptr *testpoa, VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;
    PortableServer::POA_ptr *tpoa = (PortableServer::POA_ptr *) testpoa;
    PortableServer::Servant *tsvt = (PortableServer::Servant *) svt;
    
    PortableServer::ObjectId_var oid;

    char* poa_name;
  
    try {
      poa_name = (*tpoa)->the_name();
    }
    catch (...) {
      return (VSObjectId *) 0;
    }
  
    if (strcmp(poa_name, "poaUniqueId") == 0) 
      {
	try {
	  PortableServer::ObjectId_var oid1 = (*tpoa)->activate_object(*tsvt);
	  oid = (*tpoa)->activate_object(*tsvt);	 
	}
	catch (CORBA::Exception &ex) {
	  CORBA::Exception *t = exception_convert(&ex);
	  if(env)
	    env->exception(t);
	  return (VSObjectId *) 0;
	}
	catch (...) {
	  return (VSObjectId *) 0;
	}
      }
  
    else {	
      try {
	oid = (*tpoa)->activate_object(*tsvt);
      }
      catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	  env->exception(t);
	return (VSObjectId *) 0;
      }
      catch (...) {
	return (VSObjectId *) 0;
      }
    }

    PortableServer::ObjectId_var *objId = new PortableServer::ObjectId_var(oid);
    return (VSObjectId *) objId;
  }

  /*------------------------------------------------------*/
  /* VSactivate_object_with_id() */

  VSBoolean VSactivate_object_with_id(VSServant *svt, VSObjectId_var *oid, VSPOA_ptr *testpoa, VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;
    PortableServer::POA_ptr *tpoa = (PortableServer::POA_ptr *) testpoa;
    PortableServer::Servant *tsvt = (PortableServer::Servant *) svt;
    PortableServer::ObjectId_var *objid = (PortableServer::ObjectId_var *) oid;

    char *poa_name, *oid_str;
    try {
      poa_name = (*tpoa)->the_name();
      oid_str = PortableServer::ObjectId_to_string(*objid);
    }
    catch (...) {
      return FALSE;
    }   
 
    if (strcmp(poa_name, "poaUniqueId") == 0) 
    {
      try {
	PortableServer::ObjectId_var  test_oid = PortableServer::string_to_ObjectId("test");
	(*tpoa)->activate_object_with_id(test_oid, *tsvt);
	(*tpoa)->activate_object_with_id(*objid, *tsvt);
      }
      catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	  env->exception(t);
	return FALSE;
      }     
    }  
    else {
      if (strcmp(oid_str, "active") == 0)
      {
	try {
	  PortableServer::Servant new_svt = new I_impl::I_impl(2);
	  (*tpoa)->activate_object_with_id(*objid, *tsvt);
	  (*tpoa)->activate_object_with_id(*objid, new_svt);
	}
	catch (CORBA::Exception &ex) {
	  CORBA::Exception *t = exception_convert(&ex);
	  if(env)
	    env->exception(t);
	  return FALSE;
	}
      }
    else
      {    
	try {
	  (*tpoa)->activate_object_with_id(*objid, *tsvt);
	}
	catch (CORBA::Exception &ex) {
	  CORBA::Exception *t = exception_convert(&ex);
	  if(env)
	    env->exception(t);
	  return FALSE;
	}
      }
  }
    
    return TRUE;

  }  

  /*------------------------------------------------------*/
  /* VSdeactivate_objec() */
  VSBoolean VSdeactivate_object(VSObjectId_var *oid, VSPOA_ptr *testpoa, VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;
    PortableServer::POA_ptr *tpoa = (PortableServer::POA_ptr *) testpoa;    
    PortableServer::ObjectId_var *objid = (PortableServer::ObjectId_var *) oid;

    char *poa_name, *oid_str;
    try {
      poa_name = (*tpoa)->the_name();
      oid_str = PortableServer::ObjectId_to_string(*objid);
    }
    catch (...) {
      return FALSE;
    }   
 
    PortableServer::Servant svt = new I_impl::I_impl(1);
    if (strcmp(poa_name, "poaAOM") == 0) 
      {
	if (strcmp(oid_str, "active") == 0) 
	  {      
	    try {
	      (*tpoa)->activate_object_with_id(*objid, svt);
	    }
	    catch (...){
	      return FALSE;
	    } 
	  }
	else
	  try {
	    (*tpoa)->create_reference_with_id(*objid, "M_POAtest::I");
	  }
	  catch (...) {
	    return FALSE;
	  }
      }
    else
      {
      	try {
	  (*tpoa)->create_reference_with_id(*objid, "M_POAtest::I");
	}
	catch (...) {
	  return FALSE;
	}
      }
    
    try {
      (*tpoa)->deactivate_object(*objid);
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return FALSE;
    }
    
    return TRUE;    
}

}
