/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/*************************************************/
/* Object activation and deactivation            */
/*************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <POAtest_impl.h>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>  
 
  /*------------------------------------------------*/
  /* test variable p_svt(prop value)  */
  int provide_p_svt (VSServant **testvar, enum prop_p_svt_value value)
  {
    PortableServer::Servant *tsvt;
 
    switch(value) {
    case Pp_svt_value_valid:
      {  
	PortableServer::Servant svt;
	try {
	  svt = new I_impl::I_impl(1);
	}
	catch (...) {	
	  return FALSE;
	}      
	tsvt = new PortableServer::Servant(svt);
      }
      break;
    }
  
    *testvar = (VSServant *) tsvt;
 
    return TRUE;
  }

  int relinquish_p_svt (VSServant **testvar, enum prop_p_svt_value value)
  {
    PortableServer::Servant *tvar = (PortableServer::Servant *) *testvar;
  
    switch(value) {
    case Pp_svt_value_valid:
      // delete *tvar; ?! error
      break;
    }
    
    return TRUE;
  }

  /*------------------------------------------------------------*/
  /* test variable VSPOA_ptr * testpoa(prop value) */
  
  int provide_testpoa (VSPOA_ptr **testvar, enum prop_testpoa_value value)
  {
    PortableServer::POA_ptr *tvar;

    PortableServer::POA_ptr rootPOA;
    PortableServer::POAManager_var poa_mgr;

    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
      poa_mgr = rootPOA->the_POAManager();
    }
    catch (...) {
      return FALSE;
    }
   

    CORBA::PolicyList polList;
    
    switch(value) {

    case Ptestpoa_value_poa_AOM:
      {
	try {	  
	  PortableServer::POA_ptr test_poa = rootPOA->create_POA("poaAOM", poa_mgr, polList);
	  tvar = new PortableServer::POA_ptr(test_poa);
	}
	catch (...) {
	  return FALSE;
	}       
      break;
    }
    
    case Ptestpoa_value_poa_nonRetain:
      {
	try {
	  PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER); 
           
	  polList.length(2);
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  // !!!
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  PortableServer::POA_ptr test_poa = rootPOA->create_POA("poanonRetain", poa_mgr, polList);
	  retentionPol->destroy();
	  processingPol->destroy();

	  tvar = new PortableServer::POA_ptr(test_poa);
	}
	catch (...) {
	  return FALSE;
	}	
	break;
      }

    case Ptestpoa_value_poa_nonSysId:
      {
	try {
	  PortableServer::IdAssignmentPolicy_var IdAssignPol = rootPOA->create_id_assignment_policy(PortableServer::USER_ID); 
           
	  polList.length(1);
	  polList[0] = PortableServer::IdAssignmentPolicy::_duplicate(IdAssignPol);
	
	  PortableServer::POA_ptr test_poa = rootPOA->create_POA("poanonSysId", poa_mgr, polList);
	  IdAssignPol->destroy();

	  tvar = new PortableServer::POA_ptr(test_poa);
	}
	catch (...) {
	  return FALSE;
	}	
	break;
      }
      
    case Ptestpoa_value_poa_UniqueId:
      {
	try {
	  PortableServer::IdUniquenessPolicy_var uniqueIdPol = rootPOA->create_id_uniqueness_policy(PortableServer::UNIQUE_ID); 
           
	  polList.length(1);
	  polList[0] = PortableServer::IdUniquenessPolicy::_duplicate(uniqueIdPol);
	
	  PortableServer::POA_ptr test_poa = rootPOA->create_POA("poaUniqueId", poa_mgr, polList);
	  uniqueIdPol->destroy();

	  tvar = new PortableServer::POA_ptr(test_poa);
	}
	catch (...) {
	  return FALSE;
	}	
	break;
      } 
    }   
    
    *testvar = (VSPOA_ptr *) tvar;
    return TRUE;
  }

  int relinquish_testpoa (VSPOA_ptr **testvar, enum prop_testpoa_value value)
  {
    PortableServer::POA_ptr *tvar = (PortableServer::POA_ptr *) *testvar;

    switch(value) {
    case Ptestpoa_value_poa_AOM:
    case Ptestpoa_value_poa_nonRetain:
    case Ptestpoa_value_poa_nonSysId: 
    case Ptestpoa_value_poa_UniqueId:
      (*tvar)->destroy(TRUE, TRUE); 
      break;      
    }
    
    return TRUE;
  }

  /*------------------------------------------------------*/
  /* test variable VSPOA_ptr * testpoa_id(prop value) */

  int provide_testpoa_id (VSPOA_ptr **testvar, enum prop_testpoa_id_value value)
  {
    PortableServer::POA_ptr *tvar;

    PortableServer::POA_ptr rootPOA;
    PortableServer::POAManager_var poa_mgr;

    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
      poa_mgr = rootPOA->the_POAManager();
    }
    catch (...) {
      return FALSE;
    }

    CORBA::PolicyList polList;

    switch(value) {
    case Ptestpoa_id_value_poa_Retain:
      {
	try {
	  PortableServer::IdAssignmentPolicy_var assignmentPol = rootPOA->create_id_assignment_policy(PortableServer::USER_ID); 
	  polList.length(1);
	  
	  polList[0] = PortableServer::IdAssignmentPolicy::_duplicate(assignmentPol);
	
	  PortableServer::POA_ptr test_poa = rootPOA->create_POA("poaRetain", poa_mgr, polList);
	  assignmentPol->destroy();
	 
	  tvar = new PortableServer::POA_ptr(test_poa);
	}
	catch (...) {
	  return FALSE;
	}	
	break;
      }
      
    case Ptestpoa_id_value_poa_nonRetain:
      {
	try {
	   PortableServer::IdAssignmentPolicy_var assignmentPol = rootPOA->create_id_assignment_policy(PortableServer::USER_ID); 
	  PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER); 
           
	  polList.length(3);
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  // !!!
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  // !!!
	  polList[2] = PortableServer::IdAssignmentPolicy::_duplicate(assignmentPol);

	  PortableServer::POA_ptr test_poa = rootPOA->create_POA("poanonRetain", poa_mgr, polList);
	  retentionPol->destroy();
	  processingPol->destroy();
	  assignmentPol->destroy();

	  tvar = new PortableServer::POA_ptr(test_poa);
	}
	catch (...) {
	  return FALSE;
	}	
	break;
      }
      
    case Ptestpoa_id_value_poa_SysId:
      {	 
	try {	  
	  PortableServer::POA_ptr test_poa = rootPOA->create_POA("poaAOM", poa_mgr, polList);
	  tvar = new PortableServer::POA_ptr(test_poa);
	}
	catch (...) {
	  return FALSE;
	}       
      break; 
      }      

    case Ptestpoa_id_value_poa_UniqueId:
      {
	try {
	  PortableServer::IdAssignmentPolicy_var assignmentPol = rootPOA->create_id_assignment_policy(PortableServer::USER_ID);
	  PortableServer::IdUniquenessPolicy_var uniqueIdPol = rootPOA->create_id_uniqueness_policy(PortableServer::UNIQUE_ID); 
           
	  polList.length(2);
	  polList[0] = PortableServer::IdUniquenessPolicy::_duplicate(uniqueIdPol);
	  polList[1] = PortableServer::IdAssignmentPolicy::_duplicate(assignmentPol);

	  PortableServer::POA_ptr test_poa = rootPOA->create_POA("poaUniqueId", poa_mgr, polList);
	  uniqueIdPol->destroy();
	  assignmentPol->destroy();

	  tvar = new PortableServer::POA_ptr(test_poa);
	}
	catch (...) {
	  return FALSE;
	}	
	break;
	} 
   
    }
      
    *testvar = (VSPOA_ptr *) tvar;
    return TRUE;
  }
    
  
  int relinquish_testpoa_id (VSPOA_ptr **testvar, enum prop_testpoa_id_value value)
  {
    PortableServer::POA_ptr *tvar = (PortableServer::POA_ptr *) *testvar;

    switch(value) {
    case Ptestpoa_id_value_poa_Retain: 
    case Ptestpoa_id_value_poa_nonRetain:
    case Ptestpoa_id_value_poa_SysId:
    case Ptestpoa_id_value_poa_UniqueId:
      (*tvar)->destroy(TRUE, TRUE);
      break;      
    }
    return TRUE;
  }


  /*----------------------------------------------------- */ 
  /* test variable VSObjectId_var * oid(prop value) */

  int provide_oid (VSObjectId_var **testvar, enum prop_oid_value value)
  {
    PortableServer::ObjectId_var tvar;

    switch(value) {
    case Poid_value_new: 
      {
	try {
	  tvar = PortableServer::string_to_ObjectId("new");	 
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }

    case Poid_value_active:
      {
	try {
	  tvar = PortableServer::string_to_ObjectId("active");	  
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }

    }
     
    try {
      PortableServer::ObjectId_var *tmp = new PortableServer::ObjectId_var();
      *tmp = tvar;
      *testvar = (VSObjectId_var *) tmp;
    }
    catch (...) {
      return FALSE;
    }
    
    return TRUE; 
  }

  int relinquish_oid (VSObjectId_var **testvar, enum prop_oid_value value)
  {    
    PortableServer::ObjectId_var *tvar = (PortableServer::ObjectId_var *) *testvar;

    switch(value) {
    case Poid_value_new:
    case Poid_value_active:
      break;   
    }
    return TRUE;
  }


  /*------------------------------------------------------------*/
  /* test variable VSPOA_ptr * testpoa_deact(prop value) */
  int provide_testpoa_deact(VSPOA_ptr **testvar, enum prop_testpoa_deact_value value)
  {
    PortableServer::POA_ptr *tvar;

    PortableServer::POA_ptr rootPOA;
    PortableServer::POAManager_var poa_mgr;

    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
      poa_mgr = rootPOA->the_POAManager();
    }
    catch (...) {
      return FALSE;
    }

    PortableServer::IdAssignmentPolicy_var IdAssignPol = rootPOA->create_id_assignment_policy(PortableServer::USER_ID);   

    CORBA::PolicyList polList;
    
    switch(value) {
    case Ptestpoa_deact_value_poa_AOM:
      {
	try {
	   polList.length(1);
	  polList[0] = PortableServer::IdAssignmentPolicy::_duplicate(IdAssignPol);
	  PortableServer::POA_ptr test_poa = rootPOA->create_POA("poaAOM", poa_mgr, polList);
	  tvar = new PortableServer::POA_ptr(test_poa);
	}
	catch (...) {
	  return FALSE;
	}       
      break;
    }
    
    case Ptestpoa_deact_value_poa_nonRetain:
      {
	try {
	  PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER); 
           
	  polList.length(3);
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  // !!!
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  polList[2] = PortableServer::IdAssignmentPolicy::_duplicate(IdAssignPol);
	  PortableServer::POA_ptr test_poa = rootPOA->create_POA("poanonRetain", poa_mgr, polList);	 

	  tvar = new PortableServer::POA_ptr(test_poa);
	}
	catch (...) {
	  return FALSE;
	}	
	break;
      }
    }   
    
    *testvar = (VSPOA_ptr *) tvar;
    return TRUE;
  }

  int relinquish_testpoa_deact(VSPOA_ptr **testvar, enum prop_testpoa_deact_value value)
{
    PortableServer::POA_ptr *tvar = (PortableServer::POA_ptr *) *testvar;

    switch(value) {
    case Ptestpoa_deact_value_poa_AOM:
    case Ptestpoa_deact_value_poa_nonRetain:   
      (*tvar)->destroy(TRUE, TRUE); 
      break;      
    }
    
    return TRUE;
  }


 
}

