/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/**************************************************/
/* POA creation und destruction                   */
/**************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <POAtest_impl.h>

extern "C" {
#include <t1.h>
#include <t1_aux.h>
#include <t1i_tdi.h>
  
  /*------------------------------------------------------------------*/
  /* Test the function create_POA()                                   */ 

  VSPOA_ptr *VScreate_POA(VSAdapter adpt, VSPOAManager_ptr *mgr, VSULong pol, VSEnvironment *ev)
  {
    char *adapter = (char *) adpt;
    PortableServer::POAManager_ptr *POA_mgr = (PortableServer::POAManager_ptr *) mgr;
    CORBA::ULong policy = (CORBA::ULong) pol;
    CORBA::Environment *env = (CORBA::Environment *) ev;
    
    PortableServer::POA_ptr rootPOA;
    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
    }
    catch (...) {
      return (VSPOA_ptr *) 0;
    }

    CORBA::PolicyList polList;
    PortableServer::POA_ptr childnewPOA, childexistPOA = 0; // the childexistPOA = 0 !!!

    if (policy) {
      try { 
	PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::RETAIN); 
	polList.length(1);
	polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
      }
      catch (...) {
	return (VSPOA_ptr *) 0;
      }
      
      if (strcmp(adapter, "existPOA") == 0) 
	{
	  try {
	    childexistPOA = rootPOA->create_POA(adapter, *POA_mgr, polList);
	  } 
	  catch (...) {
	    return (VSPOA_ptr *) 0;
	  }       
	}     
      }     
    else {
      try {      
	PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::RETAIN); 
	PortableServer::ServantRetentionPolicy_var nonretentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);
	
	polList.length(2);
	polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	polList[1] = PortableServer::ServantRetentionPolicy::_duplicate(nonretentionPol);    
      }
      catch (...) {
	return (VSPOA_ptr *) 0;
      }     
    }

    try {
      childnewPOA = rootPOA->create_POA(adapter, *POA_mgr, polList);      
    }    
    catch (CORBA::Exception &ex) {	
      CORBA::Exception *t = exception_convert(&ex);	 
      if(env)
	env->exception(t);  
      if(childexistPOA)
	childexistPOA->destroy(TRUE, TRUE);
      return (VSPOA_ptr *) 0;
    }
    catch (...) {
      return (VSPOA_ptr *) 0;
    } 
 
    PortableServer::POA_ptr *rpoa = new PortableServer::POA_ptr(childnewPOA);   
    return (VSPOA_ptr *) rpoa ; 
  }
    
      
  

  /*------------------------------------------------------------------*/
  /* destroy(..) */ 
  VSBoolean VSdestroy(VSPOA_ptr *p_poa, VSBoolean p_etherealize, VSBoolean p_wait, VSEnvironment *ev)
  {
     PortableServer::POA_ptr *poa = (PortableServer::POA_ptr *) p_poa;
    CORBA::ULong wait = (CORBA::ULong) p_wait;
    CORBA::ULong etherealize = (CORBA::ULong) p_etherealize;
    CORBA::Environment *env = (CORBA::Environment *) ev;
    
     try {
      (*poa)->destroy(wait, etherealize);   
    }
    catch (CORBA::Exception &ex) {   
      CORBA::Exception *t = exception_convert(&ex);    
      if(env){
	env->exception(t);     
      }    
      return FALSE;
    } 
    catch (...) {
      return FALSE;
    }
    return TRUE;
  }

  VSPOA_ptr *VSfind_POA(VSString name, VSBoolean p_activate, VSPOA_ptr *p_poa, VSEnvironment *ev)
  {
    char *adapter = (char *) name;
    PortableServer::POA_ptr *parentPOA = (PortableServer::POA_ptr *) p_poa;
    CORBA::ULong activate = (CORBA::ULong) p_activate;    
    CORBA::Environment *env = (CORBA::Environment *) ev;

    PortableServer::POA_ptr existPOA;
    if (strcmp(adapter, "existPOA") == 0)
      {
	try {
	  CORBA::PolicyList polList;
	  existPOA = (*parentPOA)->create_POA(adapter, PortableServer::POAManager::_nil(), polList);
	}
	catch (...) {
	  return (VSPOA_ptr *) 0;
	}
      }

    PortableServer::POAManager_ptr poa_mgr;
    try { 
      poa_mgr = (*parentPOA)->the_POAManager();
      poa_mgr->activate();
    }
    catch (...) {
      return (VSPOA_ptr *) 0;
    }

   

    PortableServer::POA_ptr childPOA;
    try {
      childPOA = (*parentPOA)->find_POA(adapter, activate);
    }
    catch (CORBA::Exception &ex) {	
      CORBA::Exception *t = exception_convert(&ex);	 
      if(env)
	env->exception(t);
      // Error !?
      // if (existPOA)
      // existPOA->destroy(TRUE, TRUE);
      return (VSPOA_ptr *) 0;
    }
    catch (...) {
      return (VSPOA_ptr *) 0;
    }    
  
    PortableServer::POA_ptr *rpoa = new PortableServer::POA_ptr(childPOA);   
    return (VSPOA_ptr *) rpoa ;
  }

}



