/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/**************************************************/
/* POA creation und destruction                   */
/**************************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <POAtest_impl.h>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

   
  /* ------------------------------------------------------------ */
  /* test variable VSAdapter adapter(prop Name) */

  int provide_adapter (VSAdapter *testvar, enum prop_adapter_Name Name)
  {
    char **tvar = (char **) testvar; 

    switch(Name) {
    case Padapter_Name_nil:
      {
	try {
	  *tvar = CORBA::string_dup("newPOA");
	}
	catch (...) {
	  return FALSE;
	}
	break;
      } 
    case Padapter_Name_exist:
      {
	try {
	  *tvar = CORBA::string_dup("existPOA");
	}
	catch (...) {
	  return FALSE;
	}
	break;
      } 
    }  
    return TRUE;
  }

  int relinquish_adapter (VSAdapter *testvar, enum prop_adapter_Name Name)
  {
    char **tvar = (char **) testvar;
    
    switch(Name) {
    case Padapter_Name_nil:
    case Padapter_Name_exist:
      CORBA::string_free(*tvar);
      break;
    }
    return TRUE;
  }

  /* ------------------------------------------------------------ */
  /* test variable VSPOAManager_ptr * POAmgr(prop Value) */
  int provide_POAmgr (VSPOAManager_ptr **testvar, enum prop_POAmgr_Value Value)
  {
    PortableServer::POAManager_ptr *tvar;
    PortableServer::POA_var rootPOA;
    try {  
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_var o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
    }
    catch (...) {
      return FALSE;
    }

    switch(Value) {
    case PPOAmgr_Value_nil: 
      {
	try {
	  PortableServer::POAManager_ptr poa_mgr = rootPOA->the_POAManager();
	  tvar = new PortableServer::POAManager_ptr(poa_mgr);
	}
	catch (...) {
	  return FALSE;
	}
      }
    case PPOAmgr_Value_same:{
      try {
	PortableServer::POAManager_ptr poa_mgr = PortableServer::POAManager::_nil();
	tvar = new PortableServer::POAManager_ptr(poa_mgr);
      }
      catch (...) {
	return FALSE;
      }
    }

    *testvar = (VSPOAManager_ptr *) tvar;
    return TRUE;
    }
  }


  int relinquish_POAmgr (VSPOAManager_ptr **testvar, enum prop_POAmgr_Value Value)
  {
    PortableServer::POAManager_ptr  *tvar = (PortableServer::POAManager_ptr *) *testvar; 
    switch(Value) {
    case PPOAmgr_Value_nil:
    case PPOAmgr_Value_same:
      CORBA::release(*tvar);
      break;
    }
  }


  /* ------------------------------------------------------------ */
  /* test variable VSPOA_ptr * testPOA(prop value) */
  int provide_testPOA (VSPOA_ptr **testvar, enum prop_testPOA_value value)
  {
    PortableServer::POA_ptr *tvar;
    
    CORBA::PolicyList polList;

    PortableServer::POA_ptr rootPOA;
    PortableServer::POAManager_ptr poa_mgr;
  
    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
      poa_mgr = rootPOA->the_POAManager();
    }
    catch (...) {
      return FALSE;
    }   

    switch (value) {
    case PtestPOA_value_valid:
      {
	try {
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("testPOA", poa_mgr, polList);
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	} 
	break;
      }
    }

    *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }


int relinquish_testPOA (VSPOA_ptr **testvar, enum prop_testPOA_value value)
{
  PortableServer::POA_ptr  *tvar = (PortableServer::POA_ptr *) *testvar;
  switch (value) {
  case PtestPOA_value_valid:
    //   (*tvar)->destroy(TRUE, TRUE);
      break;
  }
}


  /* ------------------------------------------------------------ */
  /* test variable VSString childPOA_name(prop value) */
  int provide_childPOA_name (VSString *testvar, enum prop_childPOA_name_value value)
  {
    char **tvar = (char **) testvar; 
    
    switch (value) {
    case PchildPOA_name_value_existPOA:
      {
	try {
	  *tvar = CORBA::string_dup("existPOA");
	}
	catch (...) {
	  return FALSE;
	}
	break;
      } 
    case PchildPOA_name_value_childPOA:
      {
	try {
	  *tvar = CORBA::string_dup("childPOA");
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
    case PchildPOA_name_value_otherPOA: 
       {
	try {
	  *tvar = CORBA::string_dup("otherPOA");
	}
	catch (...) {
	  return FALSE;
	}
	break;
       }
    }   
    return TRUE;
  }

  int relinquish_childPOA_name (VSString *testvar, enum prop_childPOA_name_value value)
  {
    char **tvar = (char **) testvar; 
    
    switch (value) {
    case PchildPOA_name_value_existPOA:
    case PchildPOA_name_value_childPOA:
    case PchildPOA_name_value_otherPOA:
      CORBA::string_free(*tvar);
      break;
    }
    return TRUE;
  }

  /* ------------------------------------------------------------ */
  /* test variable VSPOA_ptr * parentPOA(prop value) */
  int provide_parentPOA (VSPOA_ptr **testvar, enum prop_parentPOA_value value)
  {   
    PortableServer::POA_ptr *tvar;
    
    CORBA::PolicyList polList;

    PortableServer::POA_ptr rootPOA;
    PortableServer::POAManager_ptr poa_mgr;
  
    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
      poa_mgr = rootPOA->the_POAManager();
    }
    catch (...) {
      return FALSE;
    }  

    switch (value) {
    case PparentPOA_value_withAct:
      {
	try {
	  PortableServer::POA_ptr adpActPOA = rootPOA->create_POA("actPOA", poa_mgr, polList);	  
	   
	  adpActivator* adapter_actSvt = new adpActivator;
	  PortableServer::AdapterActivator_ptr adapter_act = adapter_actSvt->_this();
	  
	  adpActPOA->the_activator(adapter_act);
	  tvar = new PortableServer::POA_ptr(adpActPOA);
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
    case PparentPOA_value_noAct:
      {
	try {
	  PortableServer::POA_ptr adpnoActPOA = rootPOA->create_POA("noActPOA", poa_mgr, polList);
	  tvar = new PortableServer::POA_ptr(adpnoActPOA);
	}
	catch (...) {
	  return FALSE;
	}
	break;	
      }
    }

    *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }

  int relinquish_parentPOA (VSPOA_ptr **testvar, enum prop_parentPOA_value value)
  {
    PortableServer::POA_ptr  *tvar = (PortableServer::POA_ptr *) *testvar;
 
    switch (value) {
   
    case PparentPOA_value_withAct:
    case PparentPOA_value_noAct:
     (*tvar)->destroy(TRUE, TRUE);
      break;
    }

    return TRUE;  
  }

}
