/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/**********************************************************/
/* Policy interfaces and factories for the policy objects */
/**********************************************************/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {
#include <t1.h>
#include <t1_aux.h>
#include <t1i_tdi.h>

  PortableServer::POA_ptr rootPOA;
  PortableServer::POAManager_ptr poa_mgr;
  void get_poa()
  {   
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
    rootPOA = PortableServer::POA::_narrow(o);
    poa_mgr = rootPOA->the_POAManager();
  }

  /*------------------------------------------------------------------*/
  /* create_thread_policy() */

  VSThreadPolicy_var *VScreate_thread_policy(VSThreadPolicyValue p_polVal, VSEnvironment *ev)
  {
    PortableServer::ThreadPolicyValue polVal = (PortableServer::ThreadPolicyValue) p_polVal;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::ThreadPolicy_var policy;
    try {
      policy = rootPOA->create_thread_policy(polVal);
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSThreadPolicy_var *) 0;
    }
    catch (...) {
      return (VSThreadPolicy_var *) 0;
    }

    PortableServer::ThreadPolicy_var *rpol = new PortableServer::ThreadPolicy_var(policy);
    return (VSThreadPolicy_var *) rpol;
  }
  
  /*------------------------------------------------------------------*/
  /* value() */
  VSThreadPolicyValue VSThread_value(VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::ThreadPolicy_var policy = rootPOA->create_thread_policy(PortableServer::ORB_CTRL_MODEL);

    PortableServer::ThreadPolicyValue polVal;
    
    try {
      polVal = policy->value();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSThreadPolicyValue) 0;
    }
    catch (...) {
      return (VSThreadPolicyValue) 0;
    }

    return (VSThreadPolicyValue) polVal;
  }

  /*------------------------------------------------------------------*/
  /* create_lifespan_policy */

  VSLifespanPolicy_var *VScreate_lifespan_policy(VSLifespanPolicyValue p_polVal, VSEnvironment *ev)
  {
    PortableServer::LifespanPolicyValue polVal = (PortableServer::LifespanPolicyValue) p_polVal;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::LifespanPolicy_var policy;
    try {
      policy = rootPOA->create_lifespan_policy(polVal);
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSLifespanPolicy_var *) 0;
    }
    catch (...) {
      return (VSLifespanPolicy_var *) 0;
    }

    PortableServer::LifespanPolicy_var *rpol = new PortableServer::LifespanPolicy_var(policy);
    return (VSLifespanPolicy_var *) rpol;
  }
  
  /*------------------------------------------------------------------*/
  /* value() */
  VSLifespanPolicyValue VSLifespan_value(VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::LifespanPolicy_var policy = rootPOA->create_lifespan_policy(PortableServer::PERSISTENT);

    PortableServer::LifespanPolicyValue polVal;
    
    try {
      polVal = policy->value();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSLifespanPolicyValue) 0;
    }
    catch (...) {
      return (VSLifespanPolicyValue) 0;
    }

    return (VSLifespanPolicyValue) polVal;
  }

  /*------------------------------------------------------------------*/
  /* create_id_assignment_policy  */

  VSIdUniquenessPolicy_var *VScreate_id_uniqueness_policy(VSIdUniquenessPolicyValue p_polVal, VSEnvironment *ev)
  {
    PortableServer::IdUniquenessPolicyValue polVal = (PortableServer::IdUniquenessPolicyValue) p_polVal;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::IdUniquenessPolicy_var policy;
    try {
      policy = rootPOA->create_id_uniqueness_policy(polVal);
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSIdUniquenessPolicy_var *) 0;
    }
    catch (...) {
      return (VSIdUniquenessPolicy_var *) 0;
    }

    PortableServer::IdUniquenessPolicy_var *rpol = new PortableServer::IdUniquenessPolicy_var(policy);
    return (VSIdUniquenessPolicy_var *) rpol;
  }
  
  /*------------------------------------------------------------------*/
  /* value() */
  VSIdUniquenessPolicyValue VSIdUniqueness_value(VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::IdUniquenessPolicy_var policy = rootPOA->create_id_uniqueness_policy(PortableServer::UNIQUE_ID);

    PortableServer::IdUniquenessPolicyValue polVal;
    
    try {
      polVal = policy->value();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSIdUniquenessPolicyValue) 0;
    }
    catch (...) {
      return (VSIdUniquenessPolicyValue) 0;
    }

    return (VSIdUniquenessPolicyValue) polVal;
  }

  /*------------------------------------------------------------------*/
  /* create_id_assignment_policy */

  VSIdAssignmentPolicy_var *VScreate_id_assignment_policy(VSIdAssignmentPolicyValue p_polVal, VSEnvironment *ev)
  {
    PortableServer::IdAssignmentPolicyValue polVal = (PortableServer::IdAssignmentPolicyValue) p_polVal;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::IdAssignmentPolicy_var policy;
    try {
      policy = rootPOA->create_id_assignment_policy(polVal);
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSIdAssignmentPolicy_var *) 0;
    }
    catch (...) {
      return (VSIdAssignmentPolicy_var *) 0;
    }

    PortableServer::IdAssignmentPolicy_var *rpol = new PortableServer::IdAssignmentPolicy_var(policy);
    return (VSIdAssignmentPolicy_var *) rpol;
  }
  
  /*------------------------------------------------------------------*/
  /* value() */
  VSIdAssignmentPolicyValue VSIdAssignment_value(VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::IdAssignmentPolicy_var policy = rootPOA->create_id_assignment_policy(PortableServer::USER_ID);

    PortableServer::IdAssignmentPolicyValue polVal;
    
    try {
      polVal = policy->value();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSIdAssignmentPolicyValue) 0;
    }
    catch (...) {
      return (VSIdAssignmentPolicyValue) 0;
    }

    return (VSIdAssignmentPolicyValue) polVal;
  }

 
  /*------------------------------------------------------------------*/
  /* create_implicit_activatio_policy */

  VSImplicitActivationPolicy_var *VScreate_implicit_activation_policy(VSImplicitActivationPolicyValue p_polVal, VSEnvironment *ev)
  {
    PortableServer::ImplicitActivationPolicyValue polVal = (PortableServer::ImplicitActivationPolicyValue) p_polVal;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::ImplicitActivationPolicy_var policy;
    try {
      policy = rootPOA->create_implicit_activation_policy(polVal);
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSImplicitActivationPolicy_var *) 0;
    }
    catch (...) {
      return (VSImplicitActivationPolicy_var *) 0;
    }

    PortableServer::ImplicitActivationPolicy_var *rpol = new PortableServer::ImplicitActivationPolicy_var(policy);
    return (VSImplicitActivationPolicy_var *) rpol;
  }
  
  /*------------------------------------------------------------------*/
  /* value() */
  VSImplicitActivationPolicyValue VSImplicitActivation_value(VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::ImplicitActivationPolicy_var policy = rootPOA->create_implicit_activation_policy(PortableServer::IMPLICIT_ACTIVATION);

    PortableServer::ImplicitActivationPolicyValue polVal;
    
    try {
      polVal = policy->value();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSImplicitActivationPolicyValue) 0;
    }
    catch (...) {
      return (VSImplicitActivationPolicyValue) 0;
    }

    return (VSImplicitActivationPolicyValue) polVal;
  }

  /*------------------------------------------------------------------*/
  /* create_servant_retention_policy */

  VSServantRetentionPolicy_var *VScreate_servant_retention_policy(VSServantRetentionPolicyValue p_polVal, VSEnvironment *ev)
  {
    PortableServer::ServantRetentionPolicyValue polVal = (PortableServer::ServantRetentionPolicyValue) p_polVal;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::ServantRetentionPolicy_var policy;
    try {
      policy = rootPOA->create_servant_retention_policy(polVal);
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSServantRetentionPolicy_var *) 0;
    }
    catch (...) {
      return (VSServantRetentionPolicy_var *) 0;
    }

    PortableServer::ServantRetentionPolicy_var *rpol = new PortableServer::ServantRetentionPolicy_var(policy);
    return (VSServantRetentionPolicy_var *) rpol;
  }
  
  /*------------------------------------------------------------------*/
  /* value() */
  VSServantRetentionPolicyValue VSServantRetention_value(VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::ServantRetentionPolicy_var policy = rootPOA->create_servant_retention_policy(PortableServer::RETAIN);

    PortableServer::ServantRetentionPolicyValue polVal;
    
    try {
      polVal = policy->value();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSServantRetentionPolicyValue) 0;
    }
    catch (...) {
      return (VSServantRetentionPolicyValue) 0;
    }

    return (VSServantRetentionPolicyValue) polVal;
  }

  /*------------------------------------------------------------------*/
  /* create_lifespan_policy */

  VSRequestProcessingPolicy_var *VScreate_request_processing_policy(VSRequestProcessingPolicyValue p_polVal, VSEnvironment *ev)
  {
    PortableServer::RequestProcessingPolicyValue polVal = (PortableServer::RequestProcessingPolicyValue) p_polVal;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::RequestProcessingPolicy_var policy;
    try {
      policy = rootPOA->create_request_processing_policy(polVal);
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSRequestProcessingPolicy_var *) 0;
    }
    catch (...) {
      return (VSRequestProcessingPolicy_var *) 0;
    }

    PortableServer::RequestProcessingPolicy_var *rpol = new PortableServer::RequestProcessingPolicy_var(policy);
    return (VSRequestProcessingPolicy_var *) rpol;
  }
  
  /*------------------------------------------------------------------*/
  /* value() */
  VSRequestProcessingPolicyValue VSRequestProcessing_value(VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;

    get_poa();

    PortableServer::RequestProcessingPolicy_var policy = rootPOA->create_request_processing_policy(PortableServer::USE_ACTIVE_OBJECT_MAP_ONLY);

    PortableServer::RequestProcessingPolicyValue polVal;
    
    try {
      polVal = policy->value();
    }
    catch (CORBA::Exception &ex) {
      CORBA::Exception *t = exception_convert(&ex);
      if(env)
	env->exception(t);
      return (VSRequestProcessingPolicyValue) 0;
    }
    catch (...) {
      return (VSRequestProcessingPolicyValue) 0;
    }

    return (VSRequestProcessingPolicyValue) polVal;
  }
}
