/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/****************************************************/
/* Reference creation operations                    */
/****************************************************/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

  /*-----------------------------------------------------------*/
  /* test variable VSRepositoryId * rep(prop value) */
  int provide_rep (VSRepositoryId **testvar, enum prop_rep_value value)
  {
    char ***tvar = (char ***) testvar; 

    switch(value) {
    
    case Prep_value_valid:
      {
	try {
	  **tvar = CORBA::string_dup("M_POAtest::I");
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
    case Prep_value_null:
      {
	try {
	  **tvar = CORBA::string_dup("");
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }   
    }

    return TRUE;
  }

  int relinquish_rep (VSRepositoryId **testvar, enum prop_rep_value value)
  {
    char ***tvar = (char ***) testvar;

    switch(value) {    
    case Prep_value_valid:
    case Prep_value_null:
      CORBA::string_free(**tvar);
      break;
    }
  
    return TRUE;
  }


  /*----------------------------------------------------*/
  /* test variable VSPOA_ptr * thePOA(prop value) */
  int provide_thePOA (VSPOA_ptr **testvar, enum prop_thePOA_value value)
  {
    PortableServer::POA_ptr *tvar;

    CORBA::PolicyList polList;

    PortableServer::POA_ptr rootPOA;
    PortableServer::POAManager_ptr poa_mgr;
  
    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      rootPOA = PortableServer::POA::_narrow(o);
      poa_mgr = rootPOA->the_POAManager();
    }
    catch (...) {
      return FALSE;
    }       

   switch (value) {
   case PthePOA_value_sysId: 
     {    
       PortableServer::POA_ptr test_POA = rootPOA->create_POA("poasysId", poa_mgr, polList);
       tvar = new PortableServer::POA_ptr(test_POA);
     }
     break;

   case PthePOA_value_userId: 
     {    
       PortableServer::IdAssignmentPolicy_var assignmentPol = rootPOA->create_id_assignment_policy(PortableServer::USER_ID); 

       polList.length(1);
       polList[0] = PortableServer::IdAssignmentPolicy::_duplicate(assignmentPol);

       PortableServer::POA_ptr test_POA = rootPOA->create_POA("poauserId", poa_mgr, polList);
       assignmentPol->destroy();  

       tvar = new PortableServer::POA_ptr(test_POA);
     }
     break;
  }

   *testvar = (VSPOA_ptr *) tvar;  
   return TRUE;
}


  int relinquish_thePOA (VSPOA_ptr **testvar, enum prop_thePOA_value value)
  {
    PortableServer::POA_ptr  *tvar = (PortableServer::POA_ptr *) *testvar; 
    switch(value) {    
    case PthePOA_value_sysId:
    case PthePOA_value_userId:
      (*tvar)->destroy(TRUE, TRUE);
      break;
    }

    return TRUE;
  }

  /*----------------------------------------------------- */
  /* test variable VSObjectId_var * oid(prop value) */

  int provide_oid (VSObjectId_var **testvar, enum prop_oid_value value)
  {
    PortableServer::ObjectId_var tvar;

    switch(value) {
    case Poid_value_valid:
      {
        try {
          tvar = PortableServer::string_to_ObjectId("vaid");
        }
        catch (...) {
          return FALSE;
        }
        break;
      }
    }

    try {
      PortableServer::ObjectId_var *tmp = new PortableServer::ObjectId_var();
      *tmp = tvar;
      *testvar = (VSObjectId_var *) tmp;
    }
    catch (...) {
      return FALSE;
    }

    return TRUE;
  }

  int relinquish_oid (VSObjectId_var **testvar, enum prop_oid_value value)
  {
    PortableServer::ObjectId_var *tvar = (PortableServer::ObjectId_var *) *testvar;

    switch(value) {
    case Poid_value_valid:
      break;
    }
    return TRUE;
  }
               


}


