/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/**********************************************/
/* Servant manager registration               */
/**********************************************/

#include <corba_supp.hh>
#include <tserver.hh>
#include <POAtest_impl.h>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

  PortableServer::POA_ptr rootPOA;
  PortableServer::POAManager_ptr poa_mgr;
  void get_poa()
  {   
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
    rootPOA = PortableServer::POA::_narrow(o);
    poa_mgr = rootPOA->the_POAManager();
  }

  /*******************************************************************/
  /* test variable VSServantManager_ptr * svtMgr(prop value) */

  int provide_svtMgr (VSServantManager_ptr **testvar, enum prop_svtMgr_value value)
  {
    PortableServer::ServantManager_ptr *tvar;
    switch (value) {
    case PsvtMgr_value_valid:
      {
	try {
	  SvtActivator* m_activator = new SvtActivator;
	  PortableServer::ServantManager_ptr svtMgr = m_activator->_this();
	  tvar = new PortableServer::ServantManager_ptr(svtMgr);
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }
    }
    
    *testvar = (VSServantManager_ptr *) tvar;
    return TRUE;
  }

  int relinquish_svtMgr (VSServantManager_ptr **testvar, enum prop_svtMgr_value value)
  {
    PortableServer::ServantManager_ptr *tvar = (PortableServer::ServantManager_ptr *) *testvar;
    
    switch (value) {
    case PsvtMgr_value_valid:
      break;
    }
    
    return TRUE;
  }


  /*******************************************************************/
  /* test variable VSPOA_ptr * setSvtMgrPOA(prop value) */
  int provide_setSvtMgrPOA (void*** testvar, enum prop_setSvtMgrPOA_value value)
  {
    PortableServer::POA_ptr *tvar;
    
    get_poa();

    switch (value) {
    case PsetSvtMgrPOA_value_noSvtMgrPol:
      {
	try {
	  CORBA::PolicyList polList;
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("noSvtMgrPol", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }      
    case PsetSvtMgrPOA_value_svtMgrPol:
      {
	try {	  
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);	
	  
	  CORBA::PolicyList polList;
	  polList.length(2);	  
	  
	  polList[0] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  polList[1] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("svtMgrPol", poa_mgr, polList);	

	  tvar = new PortableServer::POA_ptr(test_POA);
	}	
	catch (...) {
	  return FALSE;
	}  	            
	break;
      }       
    case PsetSvtMgrPOA_value_svtMgrset:
      {
	try {	  
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);	
	  
	  CORBA::PolicyList polList;
	  polList.length(2);	  
	  
	  polList[0] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  polList[1] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("svtMgrset", poa_mgr, polList);	

	  tvar = new PortableServer::POA_ptr(test_POA);
	}	
	catch (...) {
	  return FALSE;
	}  	            
	break;
      }    
    }
     *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }
  
  
  int relinquish_setSvtMgrPOA (void*** testvar, enum prop_setSvtMgrPOA_value value)
  {
    PortableServer::POA_ptr *tvar = (PortableServer::POA_ptr *) *testvar;
    switch (value) {
    case PsetSvtMgrPOA_value_noSvtMgrPol:
    case PsetSvtMgrPOA_value_svtMgrPol:
    case PsetSvtMgrPOA_value_svtMgrset:
      (*tvar)->destroy(TRUE, TRUE);
      break;
    }
    return TRUE;
  }

  
  /*******************************************************************/  
  /* test variable VSPOA_ptr * getSvtMgrPOA(prop value) */
  int provide_getSvtMgrPOA (VSPOA_ptr **testvar, enum prop_getSvtMgrPOA_value value)
  {
    PortableServer::POA_ptr *tvar;
    
    get_poa(); 

    switch (value) {
    case PgetSvtMgrPOA_value_retainPol:
      {
	try {	  
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);	
	  
	  CORBA::PolicyList polList;
	  polList.length(2);	  
	  
	  polList[0] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  polList[1] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("retainPol", poa_mgr, polList);

	  SvtActivator* m_activator = new SvtActivator;
	  PortableServer::ServantManager_ptr actMgr = m_activator->_this();
	  test_POA->set_servant_manager(actMgr);

	  tvar = new PortableServer::POA_ptr(test_POA);
	}	
	catch (...) {
	  return FALSE;
	}  	            
	break;
      }        
    case PgetSvtMgrPOA_value_noRetainPol:
      {	
	try {
	  PortableServer::ServantRetentionPolicy_var retentionPol = rootPOA->create_servant_retention_policy(PortableServer::NON_RETAIN);
	  PortableServer::RequestProcessingPolicy_var processingPol = rootPOA->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
	  PortableServer::IdUniquenessPolicy_var idUniqPol = rootPOA->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);	
	  
	  CORBA::PolicyList polList;
	  polList.length(3);	  
	  
	  polList[0] = PortableServer::ServantRetentionPolicy::_duplicate(retentionPol);
	  polList[1] = PortableServer::RequestProcessingPolicy::_duplicate(processingPol);
	  polList[2] = PortableServer::IdUniquenessPolicy::_duplicate(idUniqPol);
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("noRetainPol", poa_mgr, polList);

	  SvtLocator* m_locator = new SvtLocator;
	  PortableServer::ServantManager_var loc_mgr = m_locator->_this ();
	  test_POA->set_servant_manager(loc_mgr);	

	  tvar = new PortableServer::POA_ptr(test_POA);
	}	
	catch (...) {
	  return FALSE;
	}  	            
	break;
      }
    case PgetSvtMgrPOA_value_noSvtMgrPol:
      {
	try {
	  CORBA::PolicyList polList;
	  
	  PortableServer::POA_ptr test_POA = rootPOA->create_POA("noSvtMgrPol", poa_mgr, polList); 
	  tvar = new PortableServer::POA_ptr(test_POA);
	}
	catch (...) {
	  return FALSE;
	}
	break;
      }     
    }
    *testvar = (VSPOA_ptr *) tvar;  
    return TRUE;
  }  
 
  int relinquish_getSvtMgrPOA (VSPOA_ptr **testvar, enum prop_getSvtMgrPOA_value value)
  {
    PortableServer::POA_ptr *tvar = (PortableServer::POA_ptr *) *testvar;
    switch (value) {
    case PgetSvtMgrPOA_value_retainPol:
    case PgetSvtMgrPOA_value_noRetainPol:
    case PgetSvtMgrPOA_value_noSvtMgrPol:    
      (*tvar)->destroy(TRUE, TRUE);
      break;
    }
    
    return TRUE;
  } 

}  
