/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/*********************************************/
/*  POAManger interface                      */
/*********************************************/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {
#include <t1.h>
#include <t1_aux.h>
#include <t1i_tdi.h>

 
  /*------------------------------------------------------------------*/
  /* Test the function activate()                                   */ 
  VSBoolean VSactivate(VSPOAManager_ptr *mgr, VSEnvironment *ev)
  {
    PortableServer::POAManager_ptr *POAmgr = (PortableServer::POAManager_ptr *) mgr;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    try {
      (*POAmgr)->activate();   
    }
    catch (CORBA::Exception &ex) {   
      CORBA::Exception *t = exception_convert(&ex);    
      if(env){
	env->exception(t);     
      }    
      return FALSE;
    } 
    catch (...) {
      return FALSE;
    }
    return TRUE;
  }
  
  /*------------------------------------------------------------------*/
  /* Test the function hold_requests()                                   */ 
  VSBoolean VShold_requests(VSPOAManager_ptr *mgr, VSBoolean p_wait, VSEnvironment *ev)
  {
    PortableServer::POAManager_ptr *POAmgr = (PortableServer::POAManager_ptr *) mgr;
    CORBA::ULong wait = (CORBA::ULong) p_wait;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    try {
      (*POAmgr)->hold_requests(wait);   
    }
    catch (CORBA::Exception &ex) {   
      CORBA::Exception *t = exception_convert(&ex);    
      if(env){
	env->exception(t);     
      }    
      return FALSE;
    } 
    catch (...) {
      return FALSE;
    }
    return TRUE;
  }

  /*------------------------------------------------------------------*/
  /* Test the function discard_requests()  */ 
  VSBoolean VSdiscard_requests(VSPOAManager_ptr *mgr, VSBoolean p_wait, VSEnvironment *ev)
  {
    PortableServer::POAManager_ptr *POAmgr = (PortableServer::POAManager_ptr *) mgr;
    CORBA::ULong wait = (CORBA::ULong) p_wait;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    try {
      (*POAmgr)->discard_requests(wait);   
    }
    catch (CORBA::Exception &ex) {   
      CORBA::Exception *t = exception_convert(&ex);    
      if(env){
	env->exception(t);     
      }    
      return FALSE;
    } 
    catch (...) {
      return FALSE;
    }
    return TRUE;
  }


  /*------------------------------------------------------------------*/
  /* Test the function deactivate() */ 
  VSBoolean VSdeactivate(VSPOAManager_ptr *mgr, VSBoolean p_wait, VSBoolean p_etherealize, VSEnvironment *ev)
  {
    PortableServer::POAManager_ptr *POAmgr = (PortableServer::POAManager_ptr *) mgr;
    CORBA::ULong wait = (CORBA::ULong) p_wait;
    CORBA::ULong etherealize = (CORBA::ULong) p_etherealize;
    CORBA::Environment *env = (CORBA::Environment *) ev;

    try {
      (*POAmgr)->deactivate(wait, etherealize);   
    }
    catch (CORBA::Exception &ex) {   
      CORBA::Exception *t = exception_convert(&ex);    
      if(env){
	env->exception(t);     
      }    
      return FALSE;
    } 
    catch (...) {
      return FALSE;
    }
    return TRUE;
  }


  /*------------------------------------------------------------------*/
  /* Test the function get_state() */ 
  VSState VSget_state(VSEnvironment *ev)
  {
    CORBA::Environment *env = (CORBA::Environment *) ev;
    
    PortableServer::POAManager_ptr poaMgr;;  
    try {
      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
      CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
      PortableServer::POA_ptr rootPOA = PortableServer::POA::_narrow(o);
      poaMgr = rootPOA->the_POAManager();
      poaMgr->activate();
    }
    catch (...) {
      return (VSState) 0;
    }

    PortableServer::POAManager::State mgrState;
    try {
      mgrState = poaMgr->get_state();   
    }
    catch (CORBA::Exception &ex) {   
      CORBA::Exception *t = exception_convert(&ex);    
      if(env){
	env->exception(t);     
      }    
      return (VSState) 0;
    } 
    catch (...) {
      return (VSState) 0;
    }
    return (VSState) mgrState;
  }
}

