/********************************************************************
 * Copyright (c) 2000
 * GMD FOKUS
 * Kaiserin-Augusta-Allee 31
 * 10589 Berlin
 * Germany
 * Tel: +49-30-3463-7000
 * Fax: +49-30-3463-8000
 *
 * All rights reserved. No part of this source code may be
 * reproduced, stored in a retrieval system, or transmitted,
 * in any form or by any means, electronic, mechanical,
 * photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement,
 * without the prior permission of the copyright owners.             *
 * Project: CORVAL2
 *
 * Contact:
 * Aihong Yin
 * e-mail: yin@fokus.gmd.de
 *
 * Date: Oct. 2000
 ********************************************************************/

/*********************************************/
/*  POAManger interface                      */
/*********************************************/

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

  PortableServer::POA_ptr rootPOA;  
  void get_rootpoa() {    
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    CORBA::Object_ptr o = orb->resolve_initial_references("RootPOA");
    rootPOA = PortableServer::POA::_narrow(o);    
  }       

   
/* ------------------------------------------------------------ */

int provide_poamgr_active (VSPOAManager_ptr **testvar, enum prop_poamgr_active_state state)
{
  PortableServer::POAManager_ptr *tvar;

  get_rootpoa();     
   
  PortableServer::POAManager_var nil_mgr = PortableServer::POAManager::_nil();  
  CORBA::PolicyList polList;

  PortableServer::POA_var POA_hold, POA_discard, POA_inact;

  switch(state) {
 
  case Ppoamgr_active_state_hold:
    {
    try {
      POA_hold = rootPOA->create_POA("POAhold", nil_mgr, polList);
      PortableServer::POAManager_ptr mgr = POA_hold->the_POAManager();
      
      tvar = new PortableServer::POAManager_ptr(mgr);
    }
    catch (...) {
      return FALSE;
    }
    break;
    }    
  case Ppoamgr_active_state_discard:
    {
      try {
	POA_discard = rootPOA->create_POA("POAdiscard", nil_mgr, polList);
	PortableServer::POAManager_ptr mgr = POA_discard->the_POAManager();
	mgr->discard_requests(TRUE);

	tvar = new PortableServer::POAManager_ptr(mgr);
      }
      catch (...) {
      return FALSE;
      }      
      break;
    }    
  case Ppoamgr_active_state_inactive:
    {
      try {
	POA_inact = rootPOA->create_POA("POAinact", nil_mgr, polList);
	PortableServer::POAManager_ptr mgr = POA_inact->the_POAManager();
	mgr->deactivate(TRUE, TRUE);

	tvar = new PortableServer::POAManager_ptr(mgr);
      }
      catch (...) {
      return FALSE;
      }    
    break;
    }
  }
    
  *testvar = (VSPOAManager_ptr *) tvar;  
  return TRUE; 
}

int relinquish_poamgr_active (VSPOAManager_ptr **testvar, enum prop_poamgr_active_state state)
{
  PortableServer::POAManager_ptr *tvar = ( PortableServer::POAManager_ptr *) *testvar;
  switch(state) { 
  case Ppoamgr_active_state_hold:
  case Ppoamgr_active_state_discard:
  case Ppoamgr_active_state_inactive: 
    // CORBA::release(*tvar);
    break;
  }
  
  return TRUE;
}


/* ------------------------------------------------------------ */

int provide_poamgr_hold (VSPOAManager_ptr **testvar, enum prop_poamgr_hold_state state)
{
  PortableServer::POAManager_ptr *tvar;
  
  get_rootpoa();
   
  PortableServer::POAManager_var nil_mgr = PortableServer::POAManager::_nil();
  CORBA::PolicyList polList;

  PortableServer::POA_var POA_active, POA_discard, POA_inact;

  switch(state) { 
  case Ppoamgr_hold_state_active:
    {
    try {
      POA_active = rootPOA->create_POA("POAactive", nil_mgr, polList);
      PortableServer::POAManager_ptr mgr = POA_active->the_POAManager();
      mgr->activate();
      
      tvar = new PortableServer::POAManager_ptr(mgr);
    }
    catch (...) {
      return FALSE;
    }
    break;
    }    
  case Ppoamgr_hold_state_discard:
    {
      try {
	POA_discard = rootPOA->create_POA("POAdiscard", nil_mgr, polList);
	PortableServer::POAManager_ptr mgr = POA_discard->the_POAManager();
	mgr->discard_requests(TRUE);

	tvar = new PortableServer::POAManager_ptr(mgr);
      }
      catch (...) {
      return FALSE;
      }     
      break;
    }    
  case Ppoamgr_hold_state_inactive:
    {
      try {
	POA_inact = rootPOA->create_POA("POAinact", nil_mgr, polList);
	PortableServer::POAManager_ptr mgr = POA_inact->the_POAManager();
	mgr->deactivate(TRUE, TRUE);

	tvar = new PortableServer::POAManager_ptr(mgr);
      }
      catch (...) {
      return FALSE;
      }     
      break;
    }
  } 
    
  *testvar = (VSPOAManager_ptr *) tvar;  
  return TRUE; 
}

int relinquish_poamgr_hold (VSPOAManager_ptr **testvar, enum prop_poamgr_hold_state state)
{
  PortableServer::POAManager_ptr *tvar = ( PortableServer::POAManager_ptr *) *testvar;
  switch(state) { 
  case Ppoamgr_hold_state_active:
  case Ppoamgr_hold_state_discard:
  case Ppoamgr_hold_state_inactive: 
    // CORBA::release(*tvar);
    break;
  }
  
  return TRUE;
}


/* ------------------------------------------------------------ */

int provide_poamgr_discard (VSPOAManager_ptr **testvar, enum prop_poamgr_discard_state state)
{
  PortableServer::POAManager_ptr *tvar;
  
  get_rootpoa();
   
  PortableServer::POAManager_var nil_mgr = PortableServer::POAManager::_nil();
  CORBA::PolicyList polList;

  PortableServer::POA_var POA_active, POA_hold, POA_inact;

  switch(state) { 
  case Ppoamgr_discard_state_active:
    {
    try {
      POA_active = rootPOA->create_POA("POAactive", nil_mgr, polList);
      PortableServer::POAManager_ptr mgr = POA_active->the_POAManager();
      mgr->activate();
      
      tvar = new PortableServer::POAManager_ptr(mgr);
    }
    catch (...) {
      return FALSE;
    }
    break;
    }    
  case Ppoamgr_discard_state_hold:
    {
      try {
	POA_hold = rootPOA->create_POA("POAhold", nil_mgr, polList);
	PortableServer::POAManager_ptr mgr = POA_hold->the_POAManager();
      
	tvar = new PortableServer::POAManager_ptr(mgr);
      }
      catch (...) {
	return FALSE;
      }
      break;
    }    
  case Ppoamgr_discard_state_inactive:
    {
      try {
	POA_inact = rootPOA->create_POA("POAinact", nil_mgr, polList);
	PortableServer::POAManager_ptr mgr = POA_inact->the_POAManager();
	mgr->deactivate(TRUE, TRUE);

	tvar = new PortableServer::POAManager_ptr(mgr);
      }
      catch (...) {
	return FALSE;
      }     
      break;
    }
  } 
    
  *testvar = (VSPOAManager_ptr *) tvar;  
  return TRUE; 
}

int relinquish_poamgr_discard (VSPOAManager_ptr **testvar, enum prop_poamgr_discard_state state)
{
  PortableServer::POAManager_ptr *tvar = ( PortableServer::POAManager_ptr *) *testvar;
  switch(state) { 
  case Ppoamgr_discard_state_active:
  case Ppoamgr_discard_state_hold:
  case Ppoamgr_discard_state_inactive: 
    // CORBA::release(*tvar);
    break;
  }
  
  return TRUE;
}


/* ------------------------------------------------------------ */

int provide_poamgr_inactive (VSPOAManager_ptr **testvar, enum prop_poamgr_inactive_state state)
{
  PortableServer::POAManager_ptr *tvar;
  
  get_rootpoa();
  
  PortableServer::POAManager_var nil_mgr = PortableServer::POAManager::_nil();
  CORBA::PolicyList polList;

  PortableServer::POA_var POA_active, POA_hold, POA_discard;

  switch(state) { 
  case Ppoamgr_inactive_state_active:
    {
    try {
      POA_active = rootPOA->create_POA("POAactive", nil_mgr, polList);
      PortableServer::POAManager_ptr mgr = POA_active->the_POAManager();
      mgr->activate();
      
      tvar = new PortableServer::POAManager_ptr(mgr);
    }
    catch (...) {
      return FALSE;
    }
    break;
    }    
  case Ppoamgr_inactive_state_hold:
    {
      try {
	POA_hold = rootPOA->create_POA("POAhold", nil_mgr, polList);
	PortableServer::POAManager_ptr mgr = POA_hold->the_POAManager();
      
	tvar = new PortableServer::POAManager_ptr(mgr);
      }
      catch (...) {
	return FALSE;
      }
      break;
    } 
  case Ppoamgr_inactive_state_discard: 
    {
       try {
	POA_discard = rootPOA->create_POA("POAdiscard", nil_mgr, polList);
	PortableServer::POAManager_ptr mgr = POA_discard->the_POAManager();
	mgr->discard_requests(TRUE);

	tvar = new PortableServer::POAManager_ptr(mgr);
      }
      catch (...) {
      return FALSE;
      }     
      break;     
    }
  } 
    
  *testvar = (VSPOAManager_ptr *) tvar;  
  return TRUE; 
}

int relinquish_poamgr_inactive (VSPOAManager_ptr **testvar, enum prop_poamgr_inactive_state state)
{
  PortableServer::POAManager_ptr *tvar = ( PortableServer::POAManager_ptr *) *testvar;
  switch(state) { 
  case Ppoamgr_inactive_state_active:
  case Ppoamgr_inactive_state_hold:
  case Ppoamgr_inactive_state_discard: 
    // CORBA::release(*tvar);
    break;
  }
  
  return TRUE;
}



}
