//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <corba_defs_tdd.h>
#include <corba_defs.h>

    /* ---- global property declarations ---- */
    /* ---- test variables and local properties ---- */

    /* ------------------------------------------------------------ */
    /* test variable target_object_ptr() */

    int provide_target_object_ptr(VSObject_ptr **testvar, enum prop_target_object_ptr_v v)
    {
	CORBA::Object_ptr  *tvar;
	glob_TS.testcase(VSPROVIDE_OBJPTR) = v;

	switch(v){
	case Ptarget_object_ptr_v_valid:
	  {
	    CORBA::Object_ptr o = glob_TS.m_tobj->_duplicate(glob_TS.m_tobj);
	    tvar = new CORBA::Object_ptr(o);
	    
	    if(CORBA::is_nil(*tvar))
	      return FALSE; 
	  }
	  break;
	case Ptarget_object_ptr_v_nil:
	  {
	    CORBA::Object *ao = CORBA::Object::_nil();
	    tvar = new CORBA::Object_ptr(ao);

	  }
	  break;
	case Ptarget_object_ptr_v_invalid:
	  tvar = glob_TS.invalid_object_ptr();
	  break;
    }

	*testvar = (VSObject_ptr *) tvar;
	return TRUE;
    }

    int relinquish_target_object_ptr(VSObject_ptr **testvar, enum prop_target_object_ptr_v v)
    {
	CORBA::Object_ptr  *tvar = (CORBA::Object_ptr *) *testvar;

	switch(v){
	case Ptarget_object_ptr_v_valid:
	case Ptarget_object_ptr_v_invalid:
	case Ptarget_object_ptr_v_nil:
	    break;
	}
	return TRUE;
    }

    /* ------------------------------------------------------------ */
    /* test variable nvalue_ptr() */
    
    int provide_nvalue_ptr(VSNamedValue_ptr **testvar,
			   enum prop_nvalue_ptr_v v)
    {
	CORBA::NamedValue_ptr	*tvar;
	CORBA::NamedValue_var	tmp;
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	glob_TS.testcase(VSPROVIDE_NVALPTR) = v;
	
	switch(v){
        case Pnvalue_ptr_v_valid:
	{
	    CORBA::NamedValue_ptr nptr;
	    orb->create_named_value(nptr);
            tvar = new CORBA::NamedValue_ptr(nptr);
	    
	    /* If we have failed to create a valid object, return FALSE */
	    if(CORBA::is_nil(*tvar))
		return FALSE;
	}
	break;
    case Pnvalue_ptr_v_nil:
	tvar = new CORBA::NamedValue_ptr(tmp);
	break;
    case Pnvalue_ptr_v_invalid:
	tvar = glob_TS.invalid_nvalue_ptr();
	break;
    }
	
	*testvar = (VSNamedValue_ptr *) tvar;
	return TRUE;
    }
    
    int relinquish_nvalue_ptr(VSNamedValue_ptr **testvar,
			      enum prop_nvalue_ptr_v v)
    {
	CORBA::NamedValue_ptr  *tvar = (CORBA::NamedValue_ptr *) *testvar;
	
	switch(v){
        case Pnvalue_ptr_v_valid:
            break;
        case Pnvalue_ptr_v_nil:
            break;
        case Pnvalue_ptr_v_invalid:
            break;
	}
	return TRUE;
    }

    /* ------------------------------------------------------------ */
    /* test variable typecode_ptr() */
    
    int provide_typecode_ptr(VSTypeCode_ptr **testvar,
			     enum prop_typecode_ptr_v v)
    {
	CORBA::TypeCode_ptr	*tvar;
	CORBA::TypeCode_ptr	tmp;
	glob_TS.testcase(VSPROVIDE_TCODEPTR) = v;
	
	switch(v){
        case Ptypecode_ptr_v_valid:
	{
           tmp = CORBA::_tc_short;
           tvar = new CORBA::TypeCode_ptr(tmp);
	   /* If we have failed to create a valid object, return FALSE */
	   if(CORBA::is_nil(*tvar))
	     return FALSE;
	}
	break;
    case Ptypecode_ptr_v_nil:
      tmp = CORBA::TypeCode::_nil();
      tvar = new CORBA::TypeCode_ptr(tmp);
      break;
    case Ptypecode_ptr_v_invalid:
	tvar = glob_TS.invalid_typecode_ptr();
	break;
    }
	
	*testvar = (VSTypeCode_ptr *) tvar;
	return TRUE;
    }
    
    int relinquish_typecode_ptr(VSTypeCode_ptr **testvar,
				enum prop_typecode_ptr_v v)
    {
	CORBA::TypeCode_ptr  *tvar = (CORBA::TypeCode_ptr *) *testvar;
	
	switch(v){
        case Ptypecode_ptr_v_valid:
            break;
        case Ptypecode_ptr_v_nil:
            break;
        case Ptypecode_ptr_v_invalid:
            break;
	}
	return TRUE;
    }

    /* ------------------------------------------------------------ */
    /* test variable env_ptr() */
    
    int provide_env_ptr(VSEnvironment_ptr **testvar,
			enum prop_env_ptr_v v)
    {
	CORBA::Environment_ptr	*tvar;
	CORBA::Environment_var	tmp;
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	glob_TS.testcase(VSPROVIDE_ENVPTR) = v;

	switch(v){
        case Penv_ptr_v_valid:
	{
	    CORBA::Environment_ptr e;
	    orb->create_environment(e);
            tvar = new CORBA::Environment_ptr(e);
	    
	    /* If we have failed to create a valid object, return FALSE */
	    if(CORBA::is_nil(*tvar))
		return FALSE;
	}
	break;
    case Penv_ptr_v_nil:
	tvar = new CORBA::Environment_ptr(tmp);
	break;
    case Penv_ptr_v_invalid:
	tvar = glob_TS.invalid_env_ptr();
	break;
    case Penv_ptr_v_null:
	tvar = NULL;
	break;
    }
	
	*testvar = (VSEnvironment_ptr *) tvar;
	return TRUE;
    }
    
    int relinquish_env_ptr(VSEnvironment_ptr **testvar,
			   enum prop_env_ptr_v v)
    {
	CORBA::Environment_ptr  *tvar = (CORBA::Environment_ptr *) *testvar;
	
	switch(v){
        case Penv_ptr_v_valid:
            break;
        case Penv_ptr_v_nil:
            break;
        case Penv_ptr_v_invalid:
            break;
        case Penv_ptr_v_null:
            break;
	}
	return TRUE;
    }

    /* ------------------------------------------------------------ */
    /* test variable nvlist_ptr() */
    
    int provide_nvlist_ptr(VSNVList_ptr **testvar,
			   enum prop_nvlist_ptr_v v)
    {
	CORBA::NVList_ptr	*tvar;
	CORBA::NVList_var	tmp;
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	glob_TS.testcase(VSPROVIDE_NVLISTPTR) = v;
	
	switch(v){
        case Pnvlist_ptr_v_valid:
	{
	    CORBA::NVList_ptr nptr;
	    try {
		orb->create_list(0,nptr);
	    }
	    catch(...) {
		return FALSE;
	    }
            tvar = new CORBA::NVList_ptr(nptr);
	    
	    /* If we have failed to create a valid object, return FALSE */
	    if(CORBA::is_nil(*tvar))
		return FALSE;
	}
	break;

    case Pnvlist_ptr_v_valid1:
    {
	CORBA::NVList_ptr nptr;
	try {
	    orb->create_list(0,nptr);
	    nptr->add_item("ApTest",CORBA::ARG_IN);
	}
	catch(...) {
	    return FALSE;
	}
	tvar = new CORBA::NVList_ptr(nptr);
	
	/* If we have failed to create a valid object, return FALSE */
	if(CORBA::is_nil(*tvar))
	    return FALSE;
    }
	break;

    case Pnvlist_ptr_v_nil:
	tvar = new CORBA::NVList_ptr(tmp);
	break;
    case Pnvlist_ptr_v_invalid:
	tvar = glob_TS.invalid_nvlist_ptr();
	break;
    case Pnvlist_ptr_v_null:
	tvar = NULL;
	break;
    }
	
	*testvar = (VSNVList_ptr *) tvar;
	return TRUE;
    }
    
    int relinquish_nvlist_ptr(VSNVList_ptr **testvar,
			      enum prop_nvlist_ptr_v v)
    {
	CORBA::NVList_ptr  *tvar = (CORBA::NVList_ptr *) *testvar;
	
	switch(v){
        case Pnvlist_ptr_v_valid:
        case Pnvlist_ptr_v_valid1:
            break;
        case Pnvlist_ptr_v_nil:
            break;
        case Pnvlist_ptr_v_invalid:
            break;
        case Pnvlist_ptr_v_null:
            break;
	}
	return TRUE;
    }

    /* ------------------------------------------------------------ */
    /* test variable request_ptr() */
    
    int provide_request_ptr(VSRequest_ptr **testvar,
			    enum prop_request_ptr_v v)
    {
	CORBA::Request_ptr	*tvar;
	CORBA::Request_var	tmp;
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	glob_TS.testcase(VSPROVIDE_REQPTR) = v;
	
	switch(v){
        case Prequest_ptr_v_valid:
	{
	    CORBA::Context_ptr ctx = 0;
	    CORBA::NVList_ptr nptr = 0;
	    CORBA::NamedValue_ptr nval = 0;
	    CORBA::Request_ptr req = 0;
	    CORBA::Flags f(CORBA::ARG_IN);

	    orb->create_named_value(nval);
	    orb-> get_default_context(ctx);
	    
	    try {
		cout<<"create request."<<endl;
		glob_TS.m_tobj->_create_request(ctx,"increment",nptr,nval,req,f		);

	    }
	    catch(CORBA::Exception &ex){
	    }

            tvar = new CORBA::Request_ptr(req);
	    
	    /* If we have failed to create a valid object, return FALSE */
	    if(CORBA::is_nil(*tvar))
		return FALSE;
	}
	break;

    case Prequest_ptr_v_nil:
	tvar = new CORBA::Request_ptr(tmp);
	break;
    case Prequest_ptr_v_invalid:
	tvar = glob_TS.invalid_request_ptr();
	break;
    case Prequest_ptr_v_null:
	tvar = NULL;
	break;
    }
	
	*testvar = (VSRequest_ptr *) tvar;
	return TRUE;
    }
    
    int relinquish_request_ptr(VSRequest_ptr **testvar,
			       enum prop_request_ptr_v v)
    {
	CORBA::Request_ptr  *tvar = (CORBA::Request_ptr *) *testvar;
	
	switch(v){
        case Prequest_ptr_v_valid:
            break;
        case Prequest_ptr_v_nil:
            break;
        case Prequest_ptr_v_invalid:
            break;
        case Prequest_ptr_v_null:
            break;
	}
	return TRUE;
    }

    /* ------------------------------------------------------------ */
    /* test variable context_ptr() */
    
    int provide_context_ptr(VSContext_ptr **testvar,
			    enum prop_context_ptr_v v)
    {
	CORBA::Context_ptr	*tvar;
	CORBA::Context_var	tmp;
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	glob_TS.testcase(VSPROVIDE_CTXPTR) = v;
	
	switch(v){
        case Pcontext_ptr_v_valid:
	{
    CORBA::Context_var def_ctx;
    CORBA::Context_ptr ctx;
    orb->get_default_context(def_ctx);
    def_ctx->create_child("", ctx);
    tvar = new CORBA::Context_ptr(ctx);
	    
	    /* If we have failed to create a valid object, return FALSE */
	    if(CORBA::is_nil(*tvar))
		return FALSE;
	}
	break;
    case Pcontext_ptr_v_nil:
	tvar = new CORBA::Context_ptr(tmp);
	break;
    case Pcontext_ptr_v_invalid:
	tvar = glob_TS.invalid_context_ptr();
	break;
    case Pcontext_ptr_v_null:
	tvar = NULL;
	break;
    }
	
	*testvar = (VSContext_ptr *) tvar;
	return TRUE;
    }
    
    int relinquish_context_ptr(VSContext_ptr **testvar,
			       enum prop_context_ptr_v v)
    {
	CORBA::Context_ptr  *tvar = (CORBA::Context_ptr *) *testvar;
	
	switch(v){
        case Pcontext_ptr_v_valid:
            break;
        case Pcontext_ptr_v_nil:
            break;
        case Pcontext_ptr_v_invalid:
            break;
        case Pcontext_ptr_v_null:
            break;
	}
	return TRUE;
    }

    /* ------------------------------------------------------------ */
    /* test variable orb_ptr() */
    
    int provide_orb_ptr(VSORB_ptr **testvar,
			enum prop_orb_ptr_v v)
    {
	CORBA::ORB_ptr	*tvar;
	CORBA::ORB_var	tmp;
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	glob_TS.testcase(VSPROVIDE_ORBPTR) = v;
	
	switch(v){
        case Porb_ptr_v_valid:
	{
            tvar = new CORBA::ORB_ptr(orb);
	    
	    /* If we have failed to create a valid object, return FALSE */
	    if(CORBA::is_nil(*tvar))
		return FALSE;
	}
	break;
    case Porb_ptr_v_nil:
	tvar = new CORBA::ORB_ptr(tmp);
	break;
    case Porb_ptr_v_invalid:
	tvar = glob_TS.invalid_orb_ptr();
	break;
    case Porb_ptr_v_null:
	tvar = NULL;
	break;
    }
	
	*testvar = (VSORB_ptr *) tvar;
	return TRUE;
    }
    
    int relinquish_orb_ptr(VSORB_ptr **testvar,
			   enum prop_orb_ptr_v v)
    {
	CORBA::ORB_ptr  *tvar = (CORBA::ORB_ptr *) *testvar;
	
	switch(v){
        case Porb_ptr_v_valid:
            break;
        case Porb_ptr_v_nil:
            break;
        case Porb_ptr_v_invalid:
            break;
        case Porb_ptr_v_null:
            break;
	}
	return TRUE;
    }

    /* ------------------------------------------------------------ */
    /* test variable ctxlist_ptr() */
    
    int provide_ctxlist_ptr(VSContextList_ptr **testvar,
			    enum prop_ctxlist_ptr_v v)
    {
	CORBA::ContextList_ptr	*tvar;
	CORBA::ContextList_var	tmp;
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	glob_TS.testcase(VSPROVIDE_CTXLISTPTR) = v;
	
	switch(v){
        case Pctxlist_ptr_v_valid:
	{
	    CORBA::ContextList_ptr ctx;
	    orb->create_context_list(ctx);
            tvar = new CORBA::ContextList_ptr(ctx);
	    
	    /* If we have failed to create a valid object, return FALSE */
	    if(CORBA::is_nil(*tvar))
		return FALSE;
	}
	break;

    case Pctxlist_ptr_v_valid1:
    {
	CORBA::ContextList_ptr ctx;
	orb->create_context_list(ctx);
	tvar = new CORBA::ContextList_ptr(ctx);
	ctx->add("ApTest");
	
	/* If we have failed to create a valid object, return FALSE */
	if(CORBA::is_nil(*tvar))
	    return FALSE;
    }
	break;

    case Pctxlist_ptr_v_nil:
	tvar = new CORBA::ContextList_ptr(tmp);
	break;
    case Pctxlist_ptr_v_invalid:
	tvar = glob_TS.invalid_ctxlist_ptr();
	break;
    case Pctxlist_ptr_v_null:
	tvar = NULL;
	break;
    }
	
	*testvar = (VSContextList_ptr *) tvar;
	return TRUE;
    }
    
    int relinquish_ctxlist_ptr(VSContextList_ptr **testvar,
			       enum prop_ctxlist_ptr_v v)
    {
	CORBA::ContextList_ptr  *tvar = (CORBA::ContextList_ptr *) *testvar;
	
	switch(v){
        case Pctxlist_ptr_v_valid:
            break;
        case Pctxlist_ptr_v_nil:
            break;
        case Pctxlist_ptr_v_invalid:
            break;
        case Pctxlist_ptr_v_null:
            break;
	}
	return TRUE;
    }

    /* ------------------------------------------------------------ */
    /* test variable exlist_ptr() */
    
    int provide_exlist_ptr(VSExceptionList_ptr **testvar,
			   enum prop_exlist_ptr_v v)
    {
	CORBA::ExceptionList_ptr	*tvar;
	CORBA::ExceptionList_var	tmp;
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	glob_TS.testcase(VSPROVIDE_EXLISTPTR) = v;
	
	switch(v){
        case Pexlist_ptr_v_valid:
	{
	    CORBA::ExceptionList_ptr ctx;
	    orb->create_exception_list(ctx);
            tvar = new CORBA::ExceptionList_ptr(ctx);
	    
	    /* If we have failed to create a valid object, return FALSE */
	    if(CORBA::is_nil(*tvar))
		return FALSE;
	}
	break;

    case Pexlist_ptr_v_valid1:
    {
	CORBA::ExceptionList_ptr ctx;
	orb->create_exception_list(ctx);
	tvar = new CORBA::ExceptionList_ptr(ctx);
	CORBA::TypeCode_ptr t = CORBA::_tc_short;
	ctx->add(t);
	
	/* If we have failed to create a valid object, return FALSE */
	if(CORBA::is_nil(*tvar))
	    return FALSE;
    }
	break;

    case Pexlist_ptr_v_nil:
	tvar = new CORBA::ExceptionList_ptr(tmp);
	break;

    case Pexlist_ptr_v_invalid:
	tvar = glob_TS.invalid_exlist_ptr();
	break;

    case Pexlist_ptr_v_null:
	tvar = NULL;
	break;
    }
	
	*testvar = (VSExceptionList_ptr *) tvar;
	return TRUE;
    }
    
    int relinquish_exlist_ptr(VSExceptionList_ptr **testvar,
			      enum prop_exlist_ptr_v v)
    {
	CORBA::ExceptionList_ptr  *tvar = (CORBA::ExceptionList_ptr *) *testvar;
	
	switch(v){
        case Pexlist_ptr_v_valid:
            break;
        case Pexlist_ptr_v_nil:
            break;
        case Pexlist_ptr_v_invalid:
            break;
        case Pexlist_ptr_v_null:
            break;
	}
	return TRUE;
    }
}

