//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <corba_defs_tdd.h>
#include <corba_defs.h>

/* ---- global property declarations ---- */
/* ---- test variables and local properties ---- */

/* ------------------------------------------------------------ */
/* test variable repid() */

int provide_repid(VSRepositoryId *testvar, enum prop_repid_v value)
{
  char *tvar;
  
  switch(value){
  case Prepid_v_empty:
    tvar = CORBA::string_dup("");
    break;
  }
  
  *testvar = (VSRepositoryId) tvar;
  return TRUE;
}
  
  int relinquish_repid(VSRepositoryId *testvar, enum prop_repid_v value)
  {
    char *tvar = (char *) *testvar;
    
    switch(value){
    case Prepid_v_empty:
      CORBA::string_free(tvar);
      break;
    }
    
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* test variable name() */
  
  int provide_name(VSIdentifier *testvar, enum prop_name_v value)
  {
    CORBA::Identifier tvar;
    
    switch(value){
    case Pname_v_good:
      tvar = CORBA::string_dup("ApTest");
      break;
    case Pname_v_empty:
      tvar = CORBA::string_dup("");
      break;
    }
    
    *testvar = (VSIdentifier) tvar;
    glob_TS.testcase(VSPROVIDE_NAME) = value;
    return TRUE;
  }
  
  int relinquish_name(VSIdentifier *testvar, enum prop_name_v value)
  {
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* test variable struct_mseq() */
  
  static int provide_struct_mem(CORBA::StructMember **testvar)
  {
    CORBA::StructMember *s = new CORBA::StructMember();
    CORBA::IDLType_var idlt;
    s->name = CORBA::string_dup("ApTest_mem1");
    s->type = CORBA::TypeCode::_duplicate(CORBA::_tc_short);
    s->type_def = idlt;
    
    *testvar = s;
    return TRUE;
  }
  
  static int relinquish_struct_mem(CORBA::StructMember **testvar)
  {
    CORBA::StructMember *s = *testvar;
    delete s;
    
    return TRUE;
  }
  
  int provide_struct_mseq(VSStructMemberSeq **testvar, enum prop_struct_mseq_v value)
  {
    CORBA::StructMemberSeq *tvar = (CORBA::StructMemberSeq *) *testvar;
    
    switch(value){
    case Pstruct_mseq_v_empty:
      tvar = new CORBA::StructMemberSeq();
      break;
    case Pstruct_mseq_v_good:
      {
	CORBA::StructMember *s;
	provide_struct_mem(&s);
	CORBA::ULong max = 1;
	CORBA::ULong len = 1;
	tvar = new CORBA::StructMemberSeq(len,max,s);
      }
      break;
    }
    
    *testvar = (VSStructMemberSeq *) tvar;
    return TRUE;
  }
  
  int relinquish_struct_mseq(VSStructMemberSeq **testvar, enum prop_struct_mseq_v value)
  {
    CORBA::StructMemberSeq *tvar = (CORBA::StructMemberSeq *) *testvar;
    
    switch(value){
    case Pstruct_mseq_v_good:
      {
	CORBA::StructMember *s = &((*tvar)[0]);
	//relinquish_struct_mem(&s);
      }
    case Pstruct_mseq_v_empty:
      delete tvar;
      break;
    }
    
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* test variable union_mseq() */
  
  static int provide_union_mem(CORBA::UnionMember **testvar)
  {
    CORBA::UnionMember *s = new CORBA::UnionMember();
    CORBA::IDLType_var idlt;
    
    CORBA::Long anyl = 1;
    CORBA::Any a;
    a <<= anyl;
    s->label = a;
    
    s->type_def = idlt;
    s->name = "ApTest_mem1";
    s->type = CORBA::TypeCode::_duplicate(CORBA::_tc_long);
    
    *testvar = s;
    return TRUE;
  }
  
  static int relinquish_union_mem(CORBA::UnionMember **testvar)
  {
    CORBA::UnionMember *s = *testvar;
    delete s;
    
    return TRUE;
  }
  
  int provide_union_mseq(VSUnionMemberSeq **testvar, enum prop_union_mseq_v value)
  {
    CORBA::UnionMemberSeq *tvar = (CORBA::UnionMemberSeq *) *testvar;
    
    switch(value){
    case Punion_mseq_v_empty:
      tvar = new CORBA::UnionMemberSeq();
      break;
    case Punion_mseq_v_good:
      {
	CORBA::UnionMember *s;
	provide_union_mem(&s);
	CORBA::ULong max = 1;
	CORBA::ULong len = 1;
	tvar = new CORBA::UnionMemberSeq(len,max,s);
      }
      break;
    }
    
    *testvar = (VSUnionMemberSeq *) tvar;
    return TRUE;
  }
  
  int relinquish_union_mseq(VSUnionMemberSeq **testvar, enum prop_union_mseq_v value)
  {
    CORBA::UnionMemberSeq *tvar = (CORBA::UnionMemberSeq *) *testvar;
    
    switch(value){
    case Punion_mseq_v_good:
      {
	CORBA::UnionMember *s = &((*tvar)[0]);
      }
    case Punion_mseq_v_empty:
      delete tvar;
      break;
    }
    
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* test variable discriminator_type() */
  
  int provide_discriminator_type(VSTypeCode_ptr **testvar, enum prop_discriminator_type_v value)
  {
    CORBA::TypeCode_ptr *tvar = (CORBA::TypeCode_ptr *) *testvar;
    CORBA::TypeCode_ptr a;
    
    switch(value){
    case Pdiscriminator_type_v_dlong:
      a = CORBA::_tc_long;
      break;
    case Pdiscriminator_type_v_dulong:
      a = CORBA::_tc_ulong;
      break;
    case Pdiscriminator_type_v_dshort:
      a = CORBA::_tc_short;
      break;
    case Pdiscriminator_type_v_dushort:
      a = CORBA::_tc_ushort;
      break;
    case Pdiscriminator_type_v_dchar:
      a = CORBA::_tc_char;
      break;
    case Pdiscriminator_type_v_dboolean:
      a = CORBA::_tc_boolean;
      break;
    }
    
    tvar = new CORBA::TypeCode_ptr(CORBA::TypeCode::_duplicate(a));
    *testvar = (VSTypeCode_ptr *) tvar;
    return TRUE;
  }
  
  int relinquish_discriminator_type(VSTypeCode_ptr **testvar, enum prop_discriminator_type_v value)
  {
    CORBA::TypeCode_ptr *tvar = (CORBA::TypeCode_ptr *) *testvar;

    switch(value){
    case Pdiscriminator_type_v_dlong:
    case Pdiscriminator_type_v_dulong:
    case Pdiscriminator_type_v_dshort:
    case Pdiscriminator_type_v_dushort:
    case Pdiscriminator_type_v_dchar:
    case Pdiscriminator_type_v_dboolean:
      CORBA::release(*tvar);
      delete tvar;
      break;
    }
    
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* test variable enum_mseq() */
  
  static int provide_enum_mem(CORBA::Identifier **testvar)
  {
    CORBA::Identifier *s = new CORBA::Identifier("ApTest_mem1");
    
    *testvar = s;
    return TRUE;
  }
  
  static int relinquish_enum_mem(CORBA::Identifier **testvar)
  {
    CORBA::Identifier *s = *testvar;
    delete [] *s;
    return TRUE;
  }
  
  int provide_enum_mseq(VSEnumMemberSeq **testvar, enum prop_enum_mseq_v value)
  {
    CORBA::EnumMemberSeq *tvar = (CORBA::EnumMemberSeq *) *testvar;
    
    switch(value){
    case Penum_mseq_v_empty:
      tvar = new CORBA::EnumMemberSeq();
      break;
    case Penum_mseq_v_good:
      {
	CORBA::Identifier *s;
	provide_enum_mem(&s);
	CORBA::ULong max = 1;
	CORBA::ULong len = 1;
	tvar = new CORBA::EnumMemberSeq(len,max,s);
      }
      break;
    }
    
    *testvar = (VSEnumMemberSeq *) tvar;
    return TRUE;
  }
  
  int relinquish_enum_mseq(VSEnumMemberSeq **testvar, enum prop_enum_mseq_v value)
  {
    CORBA::EnumMemberSeq *tvar = (CORBA::EnumMemberSeq *) *testvar;
    
    switch(value){
    case Penum_mseq_v_good:
      {
	char *s = (char *) (*tvar)[0];
	CORBA::Identifier *t = &s;
	//						relinquish_enum_mem(&t);
      }
    case Penum_mseq_v_empty:
      delete tvar;
      break;
    }
    
    return TRUE;
  }
  
  /* ------------------------------------------------------------ */
  /* test variable typecode() */
  
  int provide_typecode(VSTypeCode_ptr **testvar, enum prop_typecode_v value)
  {
    CORBA::TypeCode_ptr *tvar;
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    CORBA::TypeCode_ptr a;
    a = CORBA::_tc_short;
    
    switch(value){
    case Ptypecode_v_tvoid:
      a = CORBA::_tc_void;
      break;
    case Ptypecode_v_tshort:
      a = CORBA::_tc_short;
      break;
    case Ptypecode_v_tlong:
      a = CORBA::_tc_long;
      break;
    case Ptypecode_v_tushort:
      a = CORBA::_tc_ushort;
      break;
    case Ptypecode_v_tulong:
      a = CORBA::_tc_ulong;
      break;
    case Ptypecode_v_tfloat:
      a = CORBA::_tc_float;
      break;
    case Ptypecode_v_tdouble:
      a = CORBA::_tc_double;
      break;
    case Ptypecode_v_tboolean:
      a = CORBA::_tc_boolean;
      break;
    case Ptypecode_v_tchar:
      a = CORBA::_tc_char;
      break;
    case Ptypecode_v_toctet:
      a = CORBA::_tc_octet;
      break;
    case Ptypecode_v_tany:
      a = CORBA::_tc_any;
      break;
    case Ptypecode_v_tTypeCode:
      a = CORBA::_tc_TypeCode;
      break;
    case Ptypecode_v_tPrincipal:
      a = CORBA::_tc_Principal;
      break;
    case Ptypecode_v_tobjref:
      a = CORBA::_tc_Object;
      break;
    case Ptypecode_v_tstruct:
      {
	VSStructMemberSeq *testvar;
	provide_struct_mseq(&testvar, Pstruct_mseq_v_good);
	CORBA::StructMemberSeq *sseq = (CORBA::StructMemberSeq *) testvar;
	CORBA::TypeCode_ptr t = orb->create_struct_tc("","ApTest",*sseq);
	a = t;
      }
      break;

    case Ptypecode_v_tunion:
      {
	VSUnionMemberSeq *testvar;
	provide_union_mseq(&testvar, Punion_mseq_v_good);
	CORBA::UnionMemberSeq *sseq = (CORBA::UnionMemberSeq *) testvar;
	CORBA::TypeCode_var dtype = CORBA::TypeCode::_duplicate(CORBA::_tc_long);
	CORBA::TypeCode_ptr t = orb->create_union_tc("","ApTest",dtype,*sseq);
	a = t;
      }
      break;

    case Ptypecode_v_tenum:
      {
	VSEnumMemberSeq *testvar;
	provide_enum_mseq(&testvar, Penum_mseq_v_good);
	CORBA::EnumMemberSeq *eseq = (CORBA::EnumMemberSeq *) testvar;
	CORBA::TypeCode_ptr t = orb->create_enum_tc("","ApTest",*eseq);
	a = t;
      }
      break;

    case Ptypecode_v_tstring:
      a = CORBA::_tc_string;
      break;
    case Ptypecode_v_tsequence:
      {
	VSTypeCode_ptr *testvar;
	provide_typecode(&testvar, Ptypecode_v_tshort);
	CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) testvar;
	CORBA::TypeCode_ptr t = orb->create_sequence_tc(1,*tptr);
	a = t;
      }
      break;

    case Ptypecode_v_tarray:
      {
	VSTypeCode_ptr *testvar;
	provide_typecode(&testvar, Ptypecode_v_tshort);
	CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) testvar;
	CORBA::TypeCode_ptr t = orb->create_array_tc(1,*tptr);
	a = t;
      }
      break;

    case Ptypecode_v_talias:
      {
	VSTypeCode_ptr *testvar;
	provide_typecode(&testvar, Ptypecode_v_tshort);
	CORBA::TypeCode_ptr tptr = CORBA::TypeCode::_duplicate(CORBA::_tc_short);
	CORBA::TypeCode_ptr t = orb->create_alias_tc("","ApTest",tptr);
	a = t;
      }
      break;

    case Ptypecode_v_texcept:
      {
	VSStructMemberSeq *testvar;
	provide_struct_mseq(&testvar, Pstruct_mseq_v_good);
	CORBA::StructMemberSeq *sseq = (CORBA::StructMemberSeq *) testvar;
	CORBA::TypeCode_ptr t = orb->create_exception_tc("","ApTest",*sseq);
	a = t;
      }
      break;

    }
    
    tvar = new CORBA::TypeCode_ptr(CORBA::TypeCode::_duplicate(a));
    *testvar = (VSTypeCode_ptr *) tvar;
    glob_TS.testcase(VSPROVIDE_TC) = value;
    return TRUE;
  }
  
  int relinquish_typecode(VSTypeCode_ptr **testvar, enum prop_typecode_v value)
  {
    CORBA::TypeCode_ptr *tvar = (CORBA::TypeCode_ptr *) *testvar;
    
    switch(value){
    case Ptypecode_v_tvoid:
    case Ptypecode_v_tshort:
    case Ptypecode_v_tlong:
    case Ptypecode_v_tushort:
    case Ptypecode_v_tulong:
    case Ptypecode_v_tfloat:
    case Ptypecode_v_tdouble:
    case Ptypecode_v_tboolean:
    case Ptypecode_v_tchar:
    case Ptypecode_v_toctet:
    case Ptypecode_v_tany:
    case Ptypecode_v_tTypeCode:
    case Ptypecode_v_tPrincipal:
    case Ptypecode_v_tobjref:
      CORBA::release(*tvar);
      delete tvar;
      break;
    case Ptypecode_v_tstruct:
    case Ptypecode_v_tunion:
    case Ptypecode_v_tenum:
    case Ptypecode_v_tstring:
    case Ptypecode_v_tsequence:
    case Ptypecode_v_tarray:
    case Ptypecode_v_talias:
    case Ptypecode_v_texcept:
      CORBA::release(*tvar);
      delete tvar;
      break;
    }
    
    return TRUE;
  }
  
}

