//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include  <corba_supp.hh>
#include	<tserver.hh>

extern "C" {
#include <corba_defs_tdd.h>
}

TS glob_TS;

TS::TS()
{
  m_invalid_orb_ptr = NULL;
  m_invalid_nvalue_ptr = NULL;
  m_invalid_typecode_ptr = NULL;
  m_invalid_context_ptr = NULL;
  m_invalid_request_ptr = NULL;
  m_invalid_env_ptr = NULL;
  m_invalid_object_ptr = NULL;
  m_invalid_nvlist_ptr = NULL;
  m_invalid_ctxlist_ptr = NULL;
  m_invalid_exlist_ptr = NULL;
  m_rep = NULL;
}

TS::~TS()
{
  if(m_nullIOR)
    CORBA::string_free(m_nullIOR);
  if(m_tcases)
    delete [] m_tcases;
  
  VSTypeCode_ptr *t = (VSTypeCode_ptr *) m_invalid_typecode_ptr;
  relinquish_typecode_ptr(&t,Ptypecode_ptr_v_valid);

  if(!CORBA::is_nil(glob_TS.m_tobj))
    CORBA::release(glob_TS.m_tobj);
  /*
    if(m_corb)
  delete m_corb;
  
  if(m_rep)
    delete m_rep;
  */
}

void TS::initialise()
{
  CORBA::Environment_ptr ev;
  CORBA::Object_ptr obj;
  unsigned long i = 0;
  
  int *argc;
  char **argv;
  provide_argc(&argc);
  provide_argv(&argv);
  
  glob_TS.m_corb = new OrbSupport();
  if(!CORBA::is_nil(glob_TS.m_corb->orb())&& glob_TS.m_corb->statusok())
    glob_TS.m_corb->orb()->create_environment(ev);
  else
    glob_TS.m_corb->orbFinish("Failed to initialise ORB", NULL);
  
  try {
    obj = glob_TS.m_corb->readObjref();
    glob_TS.m_tobj = apiServer::_narrow(obj);
    assert(!CORBA::is_nil(glob_TS.m_tobj));
    
    CORBA::release(obj);
  }
  catch (CORBA::Exception &ex) {
    ev->exception(&ex);
    glob_TS.m_corb->orbFinish("Failed to read object reference.", ev);
  }
  
  m_tcases = new CORBA::ULong[TSMAX_TESTCASES];
  for(i=0; i< TSMAX_TESTCASES; i++)
    m_tcases[i] = 0;
  
  m_invalid_orb_ptr = NULL;
  
  void * mptr = (void *) malloc(sizeof(CORBA::ORB));
  memset(mptr,0,sizeof(CORBA::ORB));
  CORBA::ORB_ptr otptr = (CORBA::ORB *)  mptr;
  m_invalid_orb_ptr = new CORBA::ORB_ptr (otptr);
  
  mptr = (void *) malloc(sizeof(CORBA::Environment));
  memset(mptr,0,sizeof(CORBA::Environment));
  CORBA::Environment_ptr etptr = (CORBA::Environment *) mptr;
  m_invalid_env_ptr = new CORBA::Environment_ptr (etptr);

  mptr = (void *) malloc(sizeof(CORBA::Object));
  memset(mptr,0,sizeof(CORBA::Object));
  CORBA::Object_ptr btptr = (CORBA::Object *) mptr;
  m_invalid_object_ptr = new CORBA::Object_ptr (btptr);
  
  mptr = (void *) malloc(sizeof(CORBA::TypeCode));
  memset(mptr,0,sizeof(CORBA::TypeCode));
  CORBA::TypeCode_ptr ttptr = (CORBA::TypeCode *) mptr;
  //	CORBA::TypeCode_ptr ttptr = 0;
  m_invalid_typecode_ptr = new CORBA::TypeCode_ptr (ttptr);

  mptr = (void *) malloc(sizeof(CORBA::Context));
  memset(mptr,0,sizeof(CORBA::Context));
  CORBA::Context_ptr ctptr = (CORBA::Context *) mptr;
  m_invalid_context_ptr = new CORBA::Context_ptr (ctptr);
  
  mptr = (void *) malloc(sizeof(CORBA::Request));
  memset(mptr,0,sizeof(CORBA::Request));
  CORBA::Request_ptr rtptr = (CORBA::Request *) mptr;
  m_invalid_request_ptr = new CORBA::Request_ptr (rtptr);
  
  mptr = (void *) malloc(sizeof(CORBA::NamedValue));
  memset(mptr,0,sizeof(CORBA::NamedValue));
  CORBA::NamedValue_ptr ntptr = (CORBA::NamedValue *) mptr;
  m_invalid_nvalue_ptr = new CORBA::NamedValue_ptr (ntptr);
  
  mptr = (void *) malloc(sizeof(CORBA::NVList));
  memset(mptr,0,sizeof(CORBA::NVList));
  CORBA::NVList_ptr vtptr = (CORBA::NVList *) mptr;
  m_invalid_nvlist_ptr = new CORBA::NVList_ptr (vtptr);
  
  mptr = (void *) malloc(sizeof(CORBA::ContextList));
  memset(mptr,0,sizeof(CORBA::ContextList));
  CORBA::ContextList_ptr ltptr = (CORBA::ContextList *) mptr;
  m_invalid_ctxlist_ptr = new CORBA::ContextList_ptr (ltptr);
  
  mptr = (void *) malloc(sizeof(CORBA::ExceptionList));
  memset(mptr,0,sizeof(CORBA::ExceptionList));
  CORBA::ExceptionList_ptr xtptr = (CORBA::ExceptionList *) mptr;
  m_invalid_exlist_ptr = new CORBA::ExceptionList_ptr (xtptr);
  
  m_rep = new IRSupport();
  
  m_nullIOR = CORBA::string_dup("IOR:00202020000000010020202000000000");
}

char *TS::nullIOR_string()
{
  return m_nullIOR;
}

CORBA::ORB_ptr *TS::invalid_orb_ptr()
{
  // Wait until m_corb is valid and then cache the invalid
  // ORB_ptr.
  
  if((m_invalid_orb_ptr == NULL) && m_corb){
    m_invalid_orb_ptr = new CORBA::ORB_ptr(m_corb->orb());
    CORBA::release(*m_invalid_orb_ptr);
  }
  return m_invalid_orb_ptr;
}

