//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/corba/release/t1_aux.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1_aux.cpp,v $
// Revision 1.2  2001/01/17 17:19:55  corval2
// Update input data and impl. of *_is_released functions.
//
// Revision 1.1.1.1  2000/02/09 17:14:43  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.6  1999/04/30 10:48:33  cwatson
// SR100 object, orb and typecode release modified.
//
// Revision 1.5  1997/12/05 16:28:25  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.4  1997/10/09 10:53:22  tet
// Removed undeeded #define.
//
// Revision 1.3  1997/10/04 16:43:10  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.2  1997/10/01 15:36:41  cwatson
// Makefile changes.
// Changed release calls to duplicate calls in t1_aux.cpp.
//
// Revision 1.1  1997/09/09 11:54:50  cwatson
// The tests for the release(_ptr) functions.
//
//


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>

//-----------------------------------------------------------
// Object.

int object_is_released(VSObject_ptr *o1)
{
  CORBA::Object_ptr *optr1 = (CORBA::Object_ptr *) o1;
  CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
  CORBA::Object_ptr o;
  
  /* Ignore the null and nil cases. */
  if(!optr1 || is_nil_object_ptr(o1))
    return TRUE;
  
  /* TServer still holds a reference to the object, so
     release() will not delete the object */
  return TRUE;
  
  try {
    o = CORBA::Object::_duplicate(*optr1);
    }
  catch (CORBA::Exception &ex) {
    return TRUE;
  }
  catch (...) {
    return TRUE;
  }
  
  return FALSE;
}

//-----------------------------------------------------------
// NamedValue_ptr.

int nvalue_is_released(VSNamedValue_ptr *o1)
{
  CORBA::NamedValue_ptr *optr1 = (CORBA::NamedValue_ptr *) o1;
  
  /* Ignore the null and nil cases. */
  if(!optr1 || is_nil_nvalue_ptr(o1))
    return TRUE;
  
  /* Is is very likely (and CORBA compliant behavior)
     that the ORB code holds references to _tc_*
     typecode constants, so release() will not delete
     the typecode. */
  return TRUE;
  
  try {
    CORBA::NamedValue::_duplicate(*optr1);
  }
  catch (CORBA::Exception &ex) {
    return TRUE;
  }
  catch (...) {
    return TRUE;
  }
  
  return FALSE;
}

//-----------------------------------------------------------
// TypeCode_ptr.

int typecode_is_released(VSTypeCode_ptr *o1)
{
  CORBA::TypeCode_ptr *optr1 = (CORBA::TypeCode_ptr *) o1;
  
  /* Ignore the null and nil cases. */
  if(!optr1 || is_nil_typecode_ptr(o1))
    return TRUE;
  
  /* Is is very likely (and CORBA compliant behavior)
     that the ORB code holds references to _tc_*
     typecode constants, so release() will not delete
     the typecode. */
  return TRUE;
  
  try {
    CORBA::TypeCode::_duplicate(*optr1);
  }
  catch (CORBA::Exception &ex) {
    return TRUE;
  }
  catch (...) {
    return TRUE;
  }
  
  return FALSE;
}

//-----------------------------------------------------------
// Environment_ptr.

int env_is_released(VSEnvironment_ptr *o1)
{
  CORBA::Environment_ptr *optr1 = (CORBA::Environment_ptr *) o1;
  
  /* Ignore the null and nil cases. */
  if(!optr1 || is_nil_env_ptr(o1))
    return TRUE;
  /* Is is very likely (and CORBA compliant behavior)
     that the ORB code holds references to _tc_*
     typecode constants, so release() will not delete
     the typecode. */
  return TRUE;
  
  try {
    CORBA::Environment::_duplicate(*optr1);
  }
  catch (CORBA::Exception &ex) {
    return TRUE;
  }
  catch (...) {
    return TRUE;
  }
  
  return FALSE;
}

//-----------------------------------------------------------
// NVList_ptr.

int nvlist_is_released(VSNVList_ptr *o1)
{
  CORBA::NVList_ptr *optr1 = (CORBA::NVList_ptr *) o1;
  
  /* Ignore the null and nil cases. */
  if(!optr1 || is_nil_nvlist_ptr(o1))
    return TRUE;
  
  /* Is is very likely (and CORBA compliant behavior)
     that the ORB code holds references to _tc_*
     typecode constants, so release() will not delete
     the typecode. */
  return TRUE;
  
  try {
    CORBA::NVList::_duplicate(*optr1);
  }
  catch (CORBA::Exception &ex) {
    return TRUE;
  }
  catch (...) {
    return TRUE;
  }

  return FALSE;
}

//-----------------------------------------------------------
// Context_ptr.

int context_is_released(VSContext_ptr *o1)
{
  CORBA::Context_ptr *optr1 = (CORBA::Context_ptr *) o1;
  
  /* Ignore the null and nil cases. */
  if(!optr1 || is_nil_context_ptr(o1))
    return TRUE;
  
  /* Is is very likely (and CORBA compliant behavior)
     that the ORB code holds references to _tc_*
     typecode constants, so release() will not delete
     the typecode. */
  return TRUE;
  
  try {
    CORBA::Context::_duplicate(*optr1);
  }
  catch (CORBA::Exception &ex) {
    return TRUE;
  }
  catch (...) {
    return TRUE;
  }
  
  return FALSE;
}

//-----------------------------------------------------------
// Request_ptr.

int request_is_released(VSRequest_ptr *o1)
{
  CORBA::Request_ptr *optr1 = (CORBA::Request_ptr *) o1;
  
  /* Ignore the null and nil cases. */
  if(!optr1 || is_nil_request_ptr(o1))
    return TRUE;
  
  /* Is is very likely (and CORBA compliant behavior)
     that the ORB code holds references to _tc_*
     typecode constants, so release() will not delete
     the typecode. */
  return TRUE;
  
  try {
    CORBA::Request::_duplicate(*optr1);
  }
  catch (CORBA::Exception &ex) {
    return TRUE;
  }
  catch (...) {
    return TRUE;
  }
  
		return FALSE;
}

//-----------------------------------------------------------
// ORB_ptr.

int orb_is_released(VSORB_ptr *o1)
{
  CORBA::ORB_ptr *optr1 = (CORBA::ORB_ptr *) o1;

  /* Ignore the null and nil cases. */
  if(!optr1 || is_nil_orb_ptr(o1))
    return TRUE;

  /* TServer still holds a reference to the ORB, so
     release() will not delete the ORB */
  return TRUE;

  try {
    CORBA::ORB::_duplicate(*optr1);
  }
  catch (CORBA::Exception &ex) {
    return TRUE;
  }
  catch (...) {
    return TRUE;
  }
  
  return FALSE;
}

}
