//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include	<corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

/*------------------------------------------------------*/

VSString VSobject_to_string(VSObject_ptr *ob, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		char *str = 0;
		CORBA::Object_ptr *obj = (CORBA::Object_ptr *) ob;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();

		try {
				str = orb->object_to_string(*obj);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSString) 0;
		}

		return (VSString) str;
}

VSObject_ptr *VSstring_to_object(VSString st, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::Object_ptr obj = 0;
		char *str = (char *) st;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();

		try {
				obj = orb->string_to_object(str);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSObject_ptr *) 0;
		}
		catch (...) {
				return (VSObject_ptr *) 0;
		}

		CORBA::Object_ptr *robj = new CORBA::Object_ptr(obj);
		return (VSObject_ptr *) robj;
}

/*------------------------------------------------------*/

VSObjectIdList *VSlist_initial_services(VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB::ObjectIdList *olist;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();

		try {
				olist = orb->list_initial_services();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSObjectIdList *) 0;
		}
		catch (...) {
				return (VSObjectIdList *) 0;
		}

		CORBA::ORB::ObjectIdList *rlist = new CORBA::ORB::ObjectIdList(*olist);
		return (VSObjectIdList *) rlist;
}


VSObject_ptr *VSresolve_initial_references(VSObjectId *o,VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB::ObjectId *oid = (CORBA::ORB::ObjectId *) o;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::Object_ptr optr;

		try {
				optr = orb->resolve_initial_references(*oid);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSObjectIdList *) 0;
		}
		catch (...) {
				return (VSObjectIdList *) 0;
		}

		CORBA::Object_ptr *rptr = new CORBA::Object_ptr(optr);
		return (VSObject_ptr *) rptr;
}

/*------------------------------------------------------*/

VSORB_ptr *VSnil(VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB_ptr ttc;

		try {
				ttc = CORBA::ORB::_nil();
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSORB_ptr *) 0;
		}
		catch (...) {
				return (VSORB_ptr *) 0;
		}

		CORBA::ORB_ptr *rtc = new CORBA::ORB_ptr(ttc);
		return (VSORB_ptr *) rtc;
}

VSORB_ptr *VSduplicate(VSORB_ptr *o1, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB_ptr *tc = (CORBA::ORB_ptr *) o1;
		CORBA::ORB_ptr ttc;

		try {
				ttc = CORBA::ORB::_duplicate(tc ? *tc : (CORBA::ORB_ptr) 0);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
				return (VSORB_ptr *) 0;
		}
		catch (...) {
				return (VSORB_ptr *) 0;
		}

		CORBA::ORB_ptr *rtc = new CORBA::ORB_ptr(ttc);
		return (VSORB_ptr *) rtc;
}

/*------------------------------------------------------*/

VSTypeCode_ptr *VScreate_struct_tc(VSRepositoryId id, VSIdentifier name,
														VSStructMemberSeq* members, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::TypeCode_ptr tptr;
		char *lid = (char *) id;
		char *lname = (char *) name;
		CORBA::StructMemberSeq *lmembers = (CORBA::StructMemberSeq *) members;

		try {
				tptr = orb->create_struct_tc(lid,lname,*lmembers);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		return (VSTypeCode_ptr *) 0;
		}
		catch (...) {
				return (VSTypeCode_ptr *) 0;
		}

		CORBA::TypeCode_ptr *rptr = new CORBA::TypeCode_ptr(tptr);
		return (VSTypeCode_ptr *) rptr;
}

/*------------------------------------------------------*/

VSTypeCode_ptr *VScreate_union_tc(VSRepositoryId id, VSIdentifier name,
														VSTypeCode_ptr *d,
														VSUnionMemberSeq* members, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::TypeCode_ptr tptr;
		char *lid = (char *) id;
		char *lname = (char *) name;
		CORBA::TypeCode_ptr *dtype = (CORBA::TypeCode_ptr *) d;
		CORBA::UnionMemberSeq *lmembers = (CORBA::UnionMemberSeq *) members;

		try {
				tptr = orb->create_union_tc(lid,lname,*dtype,*lmembers);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		return (VSTypeCode_ptr *) 0;
		}
		catch (...) {
				return (VSTypeCode_ptr *) 0;
		}

		CORBA::TypeCode_ptr *rptr = new CORBA::TypeCode_ptr(tptr);
		return (VSTypeCode_ptr *) rptr;
}

/*------------------------------------------------------*/

VSTypeCode_ptr *VScreate_enum_tc(VSRepositoryId id, VSIdentifier name,
														VSEnumMemberSeq* members, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::TypeCode_ptr tptr;
		char *lid = (char *) id;
		char *lname = (char *) name;
		CORBA::EnumMemberSeq *lmembers = (CORBA::EnumMemberSeq *) members;

		try {
				tptr = orb->create_enum_tc(lid,lname,*lmembers);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		return (VSTypeCode_ptr *) 0;
		}
		catch (...) {
				return (VSTypeCode_ptr *) 0;
		}

		CORBA::TypeCode_ptr *rptr = new CORBA::TypeCode_ptr(tptr);
		return (VSTypeCode_ptr *) rptr;
}

/*------------------------------------------------------*/

VSTypeCode_ptr *VScreate_alias_tc(VSRepositoryId id, VSIdentifier name,
														VSTypeCode_ptr *orig, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::TypeCode_ptr tptr;
		CORBA::TypeCode_ptr *lorig = (CORBA::TypeCode_ptr *) orig;
		char *lid = (char *) id;
		char *lname = (char *) name;

		try {
				tptr = orb->create_alias_tc(lid,lname,*lorig);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		return (VSTypeCode_ptr *) 0;
		}
		catch (...) {
				return (VSTypeCode_ptr *) 0;
		}

		CORBA::TypeCode_ptr *rptr = new CORBA::TypeCode_ptr(tptr);
		return (VSTypeCode_ptr *) rptr;
}

/*------------------------------------------------------*/

VSTypeCode_ptr *VScreate_exception_tc(VSRepositoryId id, VSIdentifier name,
														VSStructMemberSeq* members, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::TypeCode_ptr tptr;
		char *lid = (char *) id;
		char *lname = (char *) name;
		CORBA::StructMemberSeq *lmembers = (CORBA::StructMemberSeq *) members;

		try {
				tptr = orb->create_exception_tc(lid,lname,*lmembers);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		return (VSTypeCode_ptr *) 0;
		}
		catch (...) {
				return (VSTypeCode_ptr *) 0;
		}

		CORBA::TypeCode_ptr *rptr = new CORBA::TypeCode_ptr(tptr);
		return (VSTypeCode_ptr *) rptr;
}

/*------------------------------------------------------*/

VSTypeCode_ptr *VScreate_interface_tc(VSRepositoryId id, VSIdentifier name,
														VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::TypeCode_ptr tptr;
		char *lid = (char *) id;
		char *lname = (char *) name;

		try {
				tptr = orb->create_interface_tc(lid,lname);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		return (VSTypeCode_ptr *) 0;
		}
		catch (...) {
				return (VSTypeCode_ptr *) 0;
		}

		CORBA::TypeCode_ptr *rptr = new CORBA::TypeCode_ptr(tptr);
		return (VSTypeCode_ptr *) rptr;
}

/*------------------------------------------------------*/

VSTypeCode_ptr *VScreate_string_tc(VSULong b, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::TypeCode_ptr tptr;
		CORBA::ULong bound = (CORBA::ULong) b;

		try {
				tptr = orb->create_string_tc(bound);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		return (VSTypeCode_ptr *) 0;
		}
		catch (...) {
				return (VSTypeCode_ptr *) 0;
		}

		CORBA::TypeCode_ptr *rptr = new CORBA::TypeCode_ptr(tptr);
		return (VSTypeCode_ptr *) rptr;
}

/*------------------------------------------------------*/

VSTypeCode_ptr *VScreate_sequence_tc(VSULong b, VSTypeCode_ptr *et, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::TypeCode_ptr tptr;
		CORBA::ULong bound = (CORBA::ULong) b;
		CORBA::TypeCode_ptr *etype = (CORBA::TypeCode_ptr *) et;

		try {
				tptr = orb->create_sequence_tc(bound,*etype);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		return (VSTypeCode_ptr *) 0;
		}
		catch (...) {
				return (VSTypeCode_ptr *) 0;
		}

		CORBA::TypeCode_ptr *rptr = new CORBA::TypeCode_ptr(tptr);
		return (VSTypeCode_ptr *) rptr;
}

/*------------------------------------------------------*/

VSTypeCode_ptr *VScreate_array_tc(VSULong b, VSTypeCode_ptr *et, VSEnvironment *ev)
{
		CORBA::Environment *env = (CORBA::Environment *) ev;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
		CORBA::TypeCode_ptr tptr;
		CORBA::ULong bound = (CORBA::ULong) b;
		CORBA::TypeCode_ptr *etype = (CORBA::TypeCode_ptr *) et;

		try {
				tptr = orb->create_array_tc(bound,*etype);
		}
		catch (CORBA::Exception &ex) {
				CORBA::Exception *t = exception_convert(&ex);
				if(env)
						env->exception(t);
		return (VSTypeCode_ptr *) 0;
		}
		catch (...) {
				return (VSTypeCode_ptr *) 0;
		}

		CORBA::TypeCode_ptr *rptr = new CORBA::TypeCode_ptr(tptr);
		return (VSTypeCode_ptr *) rptr;
}

/*------------------------------------------------------*/
/* The Function register_value_factory(..) */

VSValueFactory VSregister_value_factory(VSRepositoryId p_repId, VSValueFactory p_factory, VSEnvironment *ev)
{
   CORBA::Environment *env = (CORBA::Environment *) ev; 
   CORBA::ValueFactoryBase *factory = ( CORBA::ValueFactoryBase *) p_factory;
   char *repId = (char *) p_repId;

   CORBA::ValueFactoryBase *oldFactory;
   try {
       CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
       oldFactory = orb->register_value_factory(repId, factory);
   }
   catch (CORBA::Exception &ex) {
       CORBA::Exception *t = exception_convert(&ex);
       if(env)
	   env->exception(t);
       return (VSValueFactory) 0;
   }
   catch (...) {
       return (VSValueFactory) 0;
   } 

   
   return (VSValueFactory) oldFactory;
}
   
/*------------------------------------------------------*/
/* The Function unregister_value_factory(..) */

VSBoolean VSunregister_value_factory(VSRepositoryId p_repId, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev; 
    char *repId = (char *) p_repId;
    
    try {
       CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
       orb->unregister_value_factory(repId);
   }
   catch (CORBA::Exception &ex) {
       CORBA::Exception *t = exception_convert(&ex);
       if(env)
	   env->exception(t);
       return (VSBoolean) FALSE;
   }
   catch (...) {
       return (VSBoolean) FALSE;
   } 
   
    return TRUE;
}

/*------------------------------------------------------*/
/* The Function lookup_value_factory(..) */

VSValueFactory VSlookup_value_factory(VSValueFactory p_factory, VSRepositoryId p_repId, VSEnvironment *ev)
{
    CORBA::ValueFactoryBase *factory = ( CORBA::ValueFactoryBase *) p_factory;
    CORBA::Environment *env = (CORBA::Environment *) ev;   
    char *repId = (char *) p_repId;

    try {
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	CORBA::ValueFactoryBase *f = orb->register_value_factory(repId, factory);
    }
    catch (...) {
	return (VSValueFactory) 0;
    }

    CORBA::ValueFactoryBase *oldFactory;
    try {
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	oldFactory = orb->lookup_value_factory(repId);
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	   env->exception(t);
	return (VSValueFactory) 0;
    }
    catch (...) {
	return (VSValueFactory) 0;
    } 

    return (VSValueFactory) oldFactory;
}


/*------------------------------------------------------*/
/* create_native_tc() */

VSTypeCode_ptr *VScreate_wstring_tc(VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
   
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    	
    CORBA::TypeCode_ptr tc;
    try {
	tc = orb->create_wstring_tc(10);
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSTypeCode_ptr *) 0;
    }
    catch (...) {
	return (VSTypeCode_ptr *) 0;
    }
    
    CORBA::TypeCode_ptr *rtc = new CORBA::TypeCode_ptr(tc);
    return (VSTypeCode_ptr *) rtc;		
}


/*------------------------------------------------------*/
/* create_value_tc() */

VSTypeCode_ptr *VScreate_value_tc(VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
   
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    CORBA::ValueMemberSeq mems(2);
    mems.length(2);

    mems[0].id = CORBA::string_dup("IDL:DAnyTest::TestValueA:1.0");
    mems[0].name = CORBA::string_dup("x");
    mems[0].type = CORBA::TypeCode::_duplicate(CORBA::_tc_short);
    mems[0].access = CORBA::PUBLIC_MEMBER;

    mems[1].id = CORBA::string_dup("IDL:DAnyTest::TestValueA:1.0");
    mems[1].name = CORBA::string_dup("y");
    mems[1].type = CORBA::TypeCode::_duplicate(CORBA::_tc_long);
    mems[1].access = CORBA::PUBLIC_MEMBER;

	
    CORBA::TypeCode_ptr tc;
    try {
	tc = orb->create_value_tc(
	    "IDL:DAnyTest/TestValueA:1.0",
	    "TestValue",
	    CORBA::VM_NONE,
	    CORBA::TypeCode::_nil(),
	    mems);
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSTypeCode_ptr *) 0;
    }
    catch (...) {
	return (VSTypeCode_ptr *) 0;
    }
    
    CORBA::TypeCode_ptr *rtc = new CORBA::TypeCode_ptr(tc);
    return (VSTypeCode_ptr *) rtc;		
}


/*------------------------------------------------------*/
/* create_value_box_tc() */

VSTypeCode_ptr *VScreate_value_box_tc(VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
   
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    	
    CORBA::TypeCode_ptr tc;
    try {
	tc = orb->create_value_box_tc(
	    "IDL:DAnyTest/TestValueBox:1.0",
	    "TestValueBox",
	     CORBA::TypeCode::_nil());
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSTypeCode_ptr *) 0;
    }
    catch (...) {
	return (VSTypeCode_ptr *) 0;
    }
    
    CORBA::TypeCode_ptr *rtc = new CORBA::TypeCode_ptr(tc);
    return (VSTypeCode_ptr *) rtc;		
}

/*------------------------------------------------------*/
/* create_native_tc() */

VSTypeCode_ptr *VScreate_native_tc(VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
   
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    	
    CORBA::TypeCode_ptr tc;
    try {
	tc = orb->create_native_tc(
	    "IDL:DAnyTest/TestNative:1.0",
	    "TestNative");
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSTypeCode_ptr *) 0;
    }
    catch (...) {
	return (VSTypeCode_ptr *) 0;
    }
    
    CORBA::TypeCode_ptr *rtc = new CORBA::TypeCode_ptr(tc);
    return (VSTypeCode_ptr *) rtc;		
}


/*------------------------------------------------------*/
/* create_abstract_interface_tc() */

VSTypeCode_ptr *VScreate_abstract_interface_tc(VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
   
    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
    	
    CORBA::TypeCode_ptr tc;
    try {
	tc = orb->create_abstract_interface_tc(
	    "IDL:DAnyTest/TestAbstractInterface:1.0",
	    "TestAbstractInterfac");
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSTypeCode_ptr *) 0;
    }
    catch (...) {
	return (VSTypeCode_ptr *) 0;
    }
    
    CORBA::TypeCode_ptr *rtc = new CORBA::TypeCode_ptr(tc);
    return (VSTypeCode_ptr *) rtc;		
}

/*------------------------------------------------------*/
VSBoolean VSwork_pending(VSORB_ptr *p_orb, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::ORB_ptr *orb = (CORBA::ORB_ptr *) p_orb;
    
    CORBA::Boolean boo;
    try {
	boo = (*orb)->work_pending();
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) boo; 
}

/*------------------------------------------------------*/
VSBoolean VSperform_work(VSORB_ptr *p_orb, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::ORB_ptr *orb = (CORBA::ORB_ptr *) p_orb;
    
    CORBA::Boolean boo;
    try {
	(*orb)->perform_work();
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE; 
}   

/*------------------------------------------------------*/
/* The Function run(..) */
VSBoolean VSrun(VSORB_ptr *p_orb, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::ORB_ptr *orb = (CORBA::ORB_ptr *) p_orb;
    
    try {
	(*orb)->run();
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}

/*------------------------------------------------------*/
/* The Function shut_down(..) */
VSBoolean VSshutdown(VSORB_ptr *p_orb, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::ORB_ptr *orb = (CORBA::ORB_ptr *) p_orb;
    
    try {
	(*orb)->shutdown(1);
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return (VSBoolean) TRUE;
}


/*------------------------------------------------------*/
/* The Function destroy(..) */
VSBoolean VSdestroy(VSORB_ptr *p_orb, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;
    CORBA::ORB_ptr *orb = (CORBA::ORB_ptr *) p_orb;
    
    try {
	(*orb)->destroy();
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSBoolean) FALSE;
    }
    catch (...) {
	return (VSBoolean) FALSE;
    }
    
    return TRUE;
}

/*------------------------------------------------------*/
/* create_policy */

VSPolicy_ptr *VScreate_policy(VSPolicyType p_type, VSAny p_value, VSEnvironment *ev)
{
    CORBA::Environment *env = (CORBA::Environment *) ev;    
    CORBA::PolicyType typ = (CORBA::PolicyType) p_type;
    CORBA::Any *val = (CORBA::Any *) p_value;

    CORBA::ORB_ptr orb = glob_TS.m_corb->orb();

    CORBA::Policy_ptr pol;
    try {
	pol = orb->create_policy(typ, *val);
    }
    catch (CORBA::Exception &ex) {
	CORBA::Exception *t = exception_convert(&ex);
	if(env)
	    env->exception(t);
	return (VSPolicy_ptr *) 0;
    }
    catch (...) {
	return (VSPolicy_ptr *) 0;
    }
    
    CORBA::Policy_ptr *rPol = new CORBA::Policy_ptr(pol);
    return (VSPolicy_ptr *) rPol;
}

} // end

