//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/pObjects/ORB/t1_aux.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1_aux.cpp,v $
// Revision 1.3  2001/01/30 16:54:03  ayi
// Add new tests.
//
// Revision 1.2  2001/01/17 16:33:04  corval2
// Update for VSobject_to_string, VSstring_to_object and VScreate_union_tc.
//
// Revision 1.1.1.1  2000/02/09 17:14:47  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.14  1999/04/30 10:42:42  cwatson
// SR100 Modified is_nullIOR_string().
//
// Revision 1.13  1999/04/26 14:48:54  james
// Changed CORBA::Char* to char*
//
// Revision 1.12  1999/04/21 13:38:30  stephen
// Missing leading underscore for is_equivalent test.
//
// Revision 1.11  1997/12/05 16:28:31  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.10  1997/11/26 16:39:34  cwatson
// Removed ORBIX specific code.
//
// Revision 1.9  1997/11/11 13:14:02  cwatson
// Changed the exception handling by removing VSExceptionWrap.
//
// Revision 1.8  1997/10/09 10:53:24  tet
// Removed undeeded #define.
//
// Revision 1.7  1997/10/04 16:43:14  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.6  1997/10/01 15:56:47  cwatson
// Makefile changes.
//
// Revision 1.5  1997/09/19 15:52:38  ian
// Windows NT version.
//
// Revision 1.4  1997/09/12 13:04:57  cwatson
// Moved all the _ptr provide and relinquish functions to corba/common/ptr_prov.cpp.
//
// Revision 1.3  1997/08/22 10:31:52  stephen
// Removed debugging printf calls.
//
// Revision 1.2  1997/08/20 09:12:13  cwatson
// Added list_initial_services, resolve_initial_references.
//
// Revision 1.1  1997/08/06 14:40:26  cwatson
// *** empty log message ***
//
//
/*
 *
 * t1_aux.c -- 
 *
 * This file contains declarations of auxiliary functions
 * in module "t1"
 *
 */


#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>

//-----------------------------------------------------------
// General Object.

int is_same_object(VSObject_ptr *o1, VSObject_ptr *o2)
{
		CORBA::Object_ptr *obj1 = (CORBA::Object_ptr *) o1;
		CORBA::Object_ptr *obj2 = (CORBA::Object_ptr *) o2;

		if(is_nil_object_ptr(o1) && is_nil_object_ptr(o2))
				return TRUE;
		return (*obj1)->_is_equivalent(*obj2);
}

VSObject_ptr *get_obj_from_str(VSString s)
{
		char * str = (char *) s;
		CORBA::Object_ptr obj;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();

		try {
				obj = orb->string_to_object(str);
		}
		catch (CORBA::Exception &sysEx) {
				return (VSObject_ptr *) 0;
		}

		CORBA::Object_ptr *robj = new CORBA::Object_ptr(obj);
		return (VSObject_ptr *) robj;
}

//-----------------------------------------------------------
// General String.

VSString get_str_from_obj(VSObject_ptr *o)
{
		char * str;
		CORBA::Object_ptr *obj = (CORBA::Object_ptr *) o;
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();

		try {
				str = orb->object_to_string(*obj);
		}
		catch (CORBA::Exception &sysEx) {
				return (VSString) 0;
		}
		return (VSString) str;
}

int is_same_string(VSString s1, VSString s2)
{
		if((s1 == NULL) && (s2 == NULL)) return TRUE;
		if((s1 == NULL) || (s2 == NULL)) return FALSE;

		// IOR strings can differ in endianess and
		// the values of alignment gaps. Therefore
		// we cannot just compare the strings. Especially
		// the NIL IOR strings are likeley to differ because
		// one is generated by vsOrb and the other by the
		// ORB under test.

		if (is_nullIOR_string (s1) && is_nullIOR_string (s2))
                return TRUE;
		if (is_nullIOR_string (s1) || is_nullIOR_string (s2))
                return FALSE;

		// Due to paddings, also strings of non-nil objects may differ.
		// There for we can not compare the strings only.
		// We compare the object references.

		// return(strcmp(s1, s2) == 0) ? TRUE : FALSE;
		return(is_same_object(get_obj_from_str(s1), get_obj_from_str(s2)));
}

int is_empty_string(VSString s)
{
		if(s == NULL) return FALSE;
		return (strcmp(s,"") == 0) ? TRUE : FALSE;
}

int is_bad_string(VSString s)
{
		if(s == NULL) return FALSE;
		return (strcmp(s,"bad") == 0) ? TRUE : FALSE;
}

int is_nullIOR_string(VSString s)
{
		if(s == NULL) return FALSE;

		// there are two NIL IORs: one for big and
		// one for little endian

		/* Test for IOR: */
		if(strncmp(s,"IOR:",4))
                return FALSE;
		s += 4;

		/* Check Endianess */
		int is_big_endian = !strncmp(s,"00",2);
		s += 2;

		/* Ignore the padding */
		s += 6;

		/* Test for empty type_id string */
		if(strncmp(s,is_big_endian?"0000000100":"0100000000",10))
                return FALSE;
		s += 10;

		/* Ignore the padding */
		s += 6;

		/* Test for empty profile sequence */
		if(strncmp(s,"00000000",8))
                return FALSE;

		return TRUE;
}

//-----------------------------------------------------------
// General ORB.

int is_same_orb_ptr(VSORB_ptr *o1, VSORB_ptr *o2)
{
		if(is_nil_orb_ptr(o1) && is_nil_orb_ptr(o2))
				return TRUE;
		
		return TRUE;
}

int cleanup_orb_ptr(VSORB_ptr *o1)
{
		CORBA::ORB_ptr *tc = (CORBA::ORB_ptr *) o1;
		if(!tc)
				return TRUE;

		delete tc;
		return TRUE;
}

//-----------------------------------------------------------
// ObjectIdList.

int is_oidlist_valid(VSObjectIdList *o)
{
		CORBA::ORB::ObjectIdList *olist = (CORBA::ORB::ObjectIdList *) o;

		if(!olist)
				return TRUE;

//		printf("oid length = %ld\n",olist->length());
		for(unsigned long i = 0; i<olist->length(); i++){
				CORBA::ORB::ObjectId str;
				str = (*olist)[i];
				if(!str){
						delete olist;
						return FALSE;
				}
//		printf(":%s:\n",str);
		}

		delete olist;
		return TRUE;
}

int cleanup_oidlist(VSObjectIdList *o)
{
		CORBA::ORB::ObjectIdList *olist = (CORBA::ORB::ObjectIdList *) o;
		if(!olist)
				return TRUE;

		delete olist;
		return TRUE;
}

//-----------------------------------------------------------
// ObjectId.

int is_empty_objectid(VSObjectId *id)
{
		CORBA::ORB::ObjectId *oid = (CORBA::ORB::ObjectId *) id;
		if(!oid || !*oid)
				return FALSE;

		if(strcmp(*oid,"") == 0)
				return TRUE;
		return FALSE;
}

int is_bad_objectid(VSObjectId *id)
{
		CORBA::ORB::ObjectId *oid = (CORBA::ORB::ObjectId *) id;
		if(!oid || !*oid)
				return FALSE;

		if(strcmp(*oid,"bad") == 0)
				return TRUE;
		return FALSE;
}

int is_nil_objectid(VSObjectId *id)
{
		CORBA::ORB::ObjectId *oid = (CORBA::ORB::ObjectId *) id;
		if(!oid)
				return TRUE;

		return FALSE;
}

int is_valid_objectptr(VSObject_ptr *o)
{
		CORBA::Object_ptr *obj = (CORBA::Object_ptr *) o;

		int ret = (obj ? TRUE : FALSE);
		if(obj)
				delete obj;

		return ret;
}

//-----------------------------------------------------------

int valuefactory_registered(VSRepositoryId p_repId)
{   
    char *repId = (char *) p_repId;

    CORBA::ValueFactoryBase *factory;
    try {
	CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	factory = orb->lookup_value_factory(repId);
    }
    catch (...) {	
	return FALSE;
    }
    
    return TRUE;
}

int is_return_ValueFactory(VSValueFactory p_factory)
{
    CORBA::ValueFactoryBase *factory = ( CORBA::ValueFactoryBase *) p_factory;

    if (factory == 0)
	return FALSE;
    else
	return TRUE;
}

//-----------------------------------------------------------

int return_correct_tc(VSTypeCode_ptr *p_tc, VSString str)
{
    CORBA::TypeCode_ptr *tc = (CORBA::TypeCode_ptr *) p_tc;
    
    switch ((*tc)->kind()) {
      case CORBA::tk_wstring:
      {
	  if(strcmp(str,"WString") == 0)
	      return TRUE;
	  else
	      return FALSE;
      }
      break;
      case CORBA::tk_value:
      {
	  if(strcmp(str,"Value") == 0)
	      return TRUE;
	  else
	      return FALSE;
      }
      break;	
      case CORBA::tk_value_box:
      {
	  if(strcmp(str,"ValueBox") == 0)
	      return TRUE;
	  else
	      return FALSE;
      }
      break;
      case CORBA::tk_native:
      {
	  if(strcmp(str,"Native") == 0)
	      return TRUE;
	  else
	      return FALSE;
      }
      break;
      case CORBA::tk_abstract_interface:
      {
	  if(strcmp(str,"AbstractInterface") == 0)
	      return TRUE;
	  else
	      return FALSE;
      }
      break;
      default:
	    return FALSE;
	    break;
    }
}




//-----------------------------------------------------------

int is_orb_workpending(VSBoolean p_boo)
{
    CORBA::Boolean boo = (CORBA::Boolean) p_boo;

    return boo;
}


int is_orb_performwork(VSBoolean p_boo)
{
    CORBA::Boolean boo = (CORBA::Boolean) p_boo;

    return boo;
}

int is_orb_schutdown(VSBoolean p_boo)
{
    CORBA::Boolean boo = (CORBA::Boolean) p_boo;

    return boo;
}

int is_orb_run(VSBoolean p_boo)
{
    CORBA::Boolean boo = (CORBA::Boolean) p_boo;

    return boo;
}

int is_orb_destroy(VSBoolean p_boo)
{
    CORBA::Boolean boo = (CORBA::Boolean) p_boo;

    return boo;
}

//-----------------------------------------------------------

int is_return_policy(VSPolicy_ptr *p_pol)
{
    CORBA::Policy_ptr *pol = (CORBA::Policy_ptr *) p_pol;
    
    if (pol)
	return TRUE;
    else
	return FALSE;
}

int valid_policy_parameter(VSPolicyType p_type, VSAny p_a)
{
    CORBA::PolicyType typ = (CORBA::PolicyType) p_type;
    CORBA::Any *a = (CORBA::Any *) p_a;

    PortableServer::ThreadPolicyValue threadVal;

    if ((*a) >>= threadVal) {
	if ((typ == PortableServer::THREAD_POLICY_ID) && (threadVal = PortableServer::SINGLE_THREAD_MODEL))
	    return TRUE;
	else
	    return FALSE;
    }
    else
	return FALSE;
}


} // end
