//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <t1_aux.h>

//-----------------------------------------------------------
// Struct TypeCode_ptr.

int is_valid_struct_tc(VSTypeCode_ptr *t)
{
		int ret = TRUE;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		if(!tptr || !(CORBA::TypeCode *) *tptr)
				return FALSE;

		ret = ((*tptr)->kind() == CORBA::tk_struct) ? TRUE : FALSE;
		CORBA::release(*tptr);
		delete tptr;

		return ret;
}

int struct_tc_empty(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		CORBA::ULong mcount = (*tptr)->member_count();

		return (mcount == 0) ? TRUE : FALSE;
}

int struct_tc_good(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;

		CORBA::ULong mcount = (*tptr)->member_count();
		if(mcount != 1)
				return FALSE;

		const char *mem_nm = (*tptr)->member_name(0);
		if(strcmp(mem_nm,"ApTest_mem1"))
				return FALSE;

		CORBA::TypeCode_ptr mem_type = (*tptr)->member_type(0);
		if(mem_type != CORBA::_tc_short)
				return FALSE;

		return TRUE;
}

int is_empty_sseq(VSStructMemberSeq *s)
{
		CORBA::StructMemberSeq *seq = (CORBA::StructMemberSeq *) s;
		if(!seq)
				return FALSE;

		return seq->length() ? FALSE : TRUE;
}

int is_good_sseq(VSStructMemberSeq *s)
{
		CORBA::StructMemberSeq *seq = (CORBA::StructMemberSeq *) s;
		if(!seq)
				return FALSE;

		return (seq->length() > 0) ? TRUE : FALSE;
}

//-----------------------------------------------------------
// Union TypeCode_ptr.

int is_valid_union_tc(VSTypeCode_ptr *d, VSTypeCode_ptr *t)
{
		int ret = TRUE;
		CORBA::TypeCode_ptr *dis = (CORBA::TypeCode_ptr *) d;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;

		if(!tptr || !(CORBA::TypeCode *) *tptr)
			return FALSE;

		if(!dis || !(CORBA::TypeCode *) *dis)
			return FALSE;

		switch((*dis)->kind())
		{
			case CORBA::tk_ushort:
			case CORBA::tk_short:
			case CORBA::tk_ulong:
			case CORBA::tk_long:
				ret = ((*tptr)->kind() == CORBA::tk_union) ? TRUE : FALSE;
				break;
			default:
				ret = FALSE;
				break;
		}
		CORBA::release(*tptr);
		delete tptr;

		return ret;
}

int union_tc_empty(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;

		if(!tptr || !(CORBA::TypeCode *) *tptr)
				return FALSE;

		CORBA::ULong mcount = (*tptr)->member_count();

		return (mcount == 0) ? TRUE : FALSE;
}

int union_tc_good(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;

		if(!tptr || !(CORBA::TypeCode *) *tptr)
			return FALSE;

		CORBA::ULong mcount = (*tptr)->member_count();
		if(mcount != 1)
				return FALSE;

		const char *mem_nm = (*tptr)->member_name(0);
		if(strcmp(mem_nm,"ApTest_mem1"))
				return FALSE;

		CORBA::TypeCode_ptr mem_type = (*tptr)->member_type(0);
		if(mem_type != CORBA::_tc_long)
				return FALSE;

		CORBA::Any label = *(*tptr)->member_label(0);
		CORBA::TypeCode_ptr a_type = label.type();

		// Even though we have inserted the label as a long
		// the ORB may return the label value back in the same
		// type as the discriminator.
		CORBA::TCKind k1 = a_type->kind();
		switch (k1) {
		case CORBA::tk_short:
		  {
		    CORBA::Short anyl;
		    if (!((label >>= anyl) && anyl == 1))
		      return FALSE;
		  }
		  break;
		case CORBA::tk_ushort:
		  {
		    CORBA::UShort anyl;
		    if (!((label >>= anyl) && anyl == 1))
		      return FALSE;
		  }
		  break;
		case CORBA::tk_long:
		  {
		    CORBA::Long anyl;
		    label >>= anyl;
		    if (!((label >>= anyl) && anyl == 1))
		      return FALSE;
		  }
		  break;
		case CORBA::tk_ulong:
		  {
		    CORBA::ULong anyl;
		    if (!((label >>= anyl) && anyl == 1))
		      return FALSE;
		  }
		  break;
		default:
		  return FALSE;
		}
#if 0

		CORBA::Long anyl;
		label >>= anyl;
		if((k1 != CORBA::tk_long) && (anyl != 1))
				return FALSE;
#endif

		CORBA::TypeCode_ptr d_type = (*tptr)->discriminator_type();
		CORBA::TCKind k = d_type->kind();
		if((k != CORBA::tk_short) && (k != CORBA::tk_ushort) &&
		   (k != CORBA::tk_long) && (k != CORBA::tk_ulong) )
				return FALSE;

		return TRUE;
}

int is_empty_useq(VSUnionMemberSeq *s)
{
		CORBA::UnionMemberSeq *seq = (CORBA::UnionMemberSeq *) s;
		if(!seq)
				return FALSE;

		return seq->length() ? FALSE : TRUE;
}

int is_good_useq(VSUnionMemberSeq *s)
{
		CORBA::UnionMemberSeq *seq = (CORBA::UnionMemberSeq *) s;
		if(!seq)
				return FALSE;

		return (seq->length() > 0) ? TRUE : FALSE;
}

int are_good_params(VSUnionMemberSeq *s, VSTypeCode_ptr *d, VSRepositoryId id, VSIdentifier name)
{
        int ret = TRUE;
		CORBA::TypeCode_ptr *dis = (CORBA::TypeCode_ptr *) d;

		if(!dis || !(CORBA::TypeCode *) *dis)
			return FALSE;

        char *lid = (char *) id;
        char *lname = (char *) name;

        if (!lname || !*lname)
            return FALSE;

        if (!is_good_useq(s))
            return FALSE;

        CORBA::TCKind k = (*dis)->kind();
        if((k != CORBA::tk_short) && (k != CORBA::tk_ushort) &&
           (k != CORBA::tk_long) && (k != CORBA::tk_ulong))
                return FALSE;

        return TRUE;
}


//-----------------------------------------------------------
// Enum TypeCode_ptr.

int is_valid_enum_tc(VSTypeCode_ptr *t)
{
		int ret = TRUE;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		if(!tptr || !(CORBA::TypeCode *) *tptr)
				return FALSE;

		ret = ((*tptr)->kind() == CORBA::tk_enum) ? TRUE : FALSE;
		CORBA::release(*tptr);
		delete tptr;

		return ret;
}

int enum_tc_empty(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		CORBA::ULong mcount = (*tptr)->member_count();

		return (mcount == 0) ? TRUE : FALSE;
}

int enum_tc_good(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;

		CORBA::ULong mcount = (*tptr)->member_count();
		if(mcount != 1)
				return FALSE;

		const char *mem_nm = (*tptr)->member_name(0);
		if(strcmp(mem_nm,"ApTest_mem1"))
				return FALSE;

		return TRUE;
}

int is_empty_eseq(VSEnumMemberSeq *s)
{
		CORBA::EnumMemberSeq *seq = (CORBA::EnumMemberSeq *) s;
		if(!seq)
				return FALSE;

		return seq->length() ? FALSE : TRUE;
}

int is_good_eseq(VSEnumMemberSeq *s)
{
		CORBA::EnumMemberSeq *seq = (CORBA::EnumMemberSeq *) s;
		if(!seq)
				return FALSE;

		return (seq->length() > 0) ? TRUE : FALSE;
}

//-----------------------------------------------------------
// Alias TypeCode_ptr.

int is_valid_alias_tc(VSTypeCode_ptr *t)
{
		int ret = TRUE;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		if(!tptr || !(CORBA::TypeCode *) *tptr)
				return FALSE;

		ret = ((*tptr)->kind() == CORBA::tk_alias) ? TRUE : FALSE;
		CORBA::release(*tptr);
		delete tptr;

		return ret;
}

int content_is_same(VSTypeCode_ptr *t1, VSTypeCode_ptr *t2)
{
		int ret = TRUE;
		unsigned long i = 0;
		CORBA::TypeCode_ptr *tptr1 = (CORBA::TypeCode_ptr *) t1;
		CORBA::TypeCode_ptr *tptr2 = (CORBA::TypeCode_ptr *) t2;
		if(!tptr1 || !(CORBA::TypeCode *) *tptr1)
				return FALSE;
		if(!tptr2 || !(CORBA::TypeCode *) *tptr2)
				return FALSE;

		CORBA::TypeCode_ptr ct = (*tptr1)->content_type();
		CORBA::TCKind k1 = ct->kind();
		CORBA::TCKind k2 = (*tptr2)->kind();
		ret = (k1 == k2) ? TRUE : FALSE;

		return ret;
}

//-----------------------------------------------------------
// Exception TypeCode_ptr.

int is_valid_exception_tc(VSTypeCode_ptr *t)
{
		int ret = TRUE;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		if(!tptr || !(CORBA::TypeCode *) *tptr)
				return FALSE;

		ret = ((*tptr)->kind() == CORBA::tk_except) ? TRUE : FALSE;
		CORBA::release(*tptr);
		delete tptr;

		return ret;
}

//-----------------------------------------------------------
// Interface TypeCode_ptr.

int is_valid_interface_tc(VSTypeCode_ptr *t)
{
		int ret = TRUE;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		if(!tptr || !(CORBA::TypeCode *) *tptr)
				return FALSE;

		ret = ((*tptr)->kind() == CORBA::tk_objref) ? TRUE : FALSE;
		CORBA::release(*tptr);
		delete tptr;

		return ret;
}

//-----------------------------------------------------------
// String TypeCode_ptr.

int is_valid_string_tc(VSTypeCode_ptr *t)
{
		int ret = TRUE;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		if(!tptr || !(CORBA::TypeCode *) *tptr)
				return FALSE;

		ret = ((*tptr)->kind() == CORBA::tk_string) ? TRUE : FALSE;
		CORBA::release(*tptr);
		delete tptr;

		return ret;
}

int is_same_length(VSULong b,VSTypeCode_ptr *t)
{
		int ret = TRUE;
		CORBA::ULong bound  = (CORBA::ULong) b;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		if(!tptr || !(CORBA::TypeCode *) *tptr)
				return FALSE;

		ret = ((*tptr)->length() == bound) ? TRUE : FALSE;

		return ret;
}

//-----------------------------------------------------------
// Sequence TypeCode_ptr.

int is_valid_sequence_tc(VSTypeCode_ptr *t)
{
		int ret = TRUE;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		if(!tptr || !(CORBA::TypeCode *) *tptr)
				return FALSE;

		ret = ((*tptr)->kind() == CORBA::tk_sequence) ? TRUE : FALSE;
		CORBA::release(*tptr);
		delete tptr;

		return ret;
}

//-----------------------------------------------------------
// Array TypeCode_ptr.

int is_valid_array_tc(VSTypeCode_ptr *t)
{
		int ret = TRUE;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		if(!tptr || !(CORBA::TypeCode *) *tptr)
				return FALSE;

		ret = ((*tptr)->kind() == CORBA::tk_array) ? TRUE : FALSE;
		CORBA::release(*tptr);
		delete tptr;

		return ret;
}

}
