//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/pObjects/ORB/t1i_tdi.cpp
//
// Purpose:
//
// Modifications:
// $Log: t1i_tdi.cpp,v $
// Revision 1.3  2001/01/30 16:54:03  ayi
// Add new tests.
//
// Revision 1.2  2001/01/17 16:33:05  corval2
// Update for VSobject_to_string, VSstring_to_object and VScreate_union_tc.
//
// Revision 1.1.1.1  2000/02/09 17:14:47  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.11  1999/04/26 14:48:56  james
// Changed CORBA::Char* to char*
//
// Revision 1.10  1997/12/05 16:28:32  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.9  1997/11/11 13:14:03  cwatson
// Changed the exception handling by removing VSExceptionWrap.
//
// Revision 1.8  1997/10/04 16:43:15  stephen
// Changed include to corba_supp.hh.
//
// Revision 1.7  1997/10/01 15:56:48  cwatson
// Makefile changes.
//
// Revision 1.6  1997/09/22 09:52:00  ian
// Changed CORBA::OBJECT_NIL to CORBA::Object_nil().
//
// Revision 1.5  1997/09/19 15:52:40  ian
// Windows NT version.
//
// Revision 1.4  1997/09/12 13:04:59  cwatson
// Moved all the _ptr provide and relinquish functions to corba/common/ptr_prov.cpp.
//
// Revision 1.3  1997/08/22 10:31:52  stephen
// Removed debugging printf calls.
//
// Revision 1.2  1997/08/20 09:12:15  cwatson
// Added list_initial_services, resolve_initial_references.
//
// Revision 1.1  1997/08/06 14:40:27  cwatson
// *** empty log message ***
//
//
/*
 * ./t1_tdi.c
 */

#include <corba_supp.hh>
#include <tserver.hh>
#include <fstream.h>
#include <DAny.h>
#include <DAny_impl.h>

extern "C" {

#include <t1i_tdi.h>
#include <corba_defs.h>

/* ---- global property declarations ---- */
/* ---- test variables and local properties ---- */

/* ------------------------------------------------------------ */
/* test variable object_ptr() */

int provide_object_ptr(VSObject_ptr **testvar,
							enum prop_object_ptr_obj_value obj_value)
{
		CORBA::Object_ptr  *tvar;

		switch(obj_value){
				case Pobject_ptr_obj_value_valid:
						{
						CORBA::Object_ptr o = glob_TS.m_tobj->_duplicate(glob_TS.m_tobj);
						tvar = new CORBA::Object_ptr(o);
						}
						break;
				case Pobject_ptr_obj_value_invalid:
						tvar = glob_TS.invalid_object_ptr();
						break;
				case Pobject_ptr_obj_value_nil:
						{
            CORBA::Object *ao = CORBA::Object::_nil();
            tvar = new CORBA::Object_ptr(ao);
            }
						break;
		}

		*testvar = (VSObject_ptr *) tvar;
		return TRUE;
}

int relinquish_object_ptr(VSObject_ptr **testvar,
							enum prop_object_ptr_obj_value obj_value)
{
		CORBA::Object_ptr  *tvar = (CORBA::Object_ptr *) *testvar;
		switch(obj_value){
				case Pobject_ptr_obj_value_valid:
						break;
				case Pobject_ptr_obj_value_invalid:
						break;
				case Pobject_ptr_obj_value_nil:
						CORBA::release(*tvar);
						break;
		}
		return TRUE;
}

/* ------------------------------------------------------------ */
/* test variable string() */

int provide_s2o_string (VSString *testvar, enum prop_s2o_string_prop_1 prop_1)
{
  char **tvar = (char **) testvar;
  switch(prop_1){
  case Ps2o_string_prop_1_good:
    *tvar = glob_TS.m_corb->readString();
    break;
  case Ps2o_string_prop_1_bad:
    *tvar = CORBA::string_dup("bad");
    break;
  case Ps2o_string_prop_1_null:
    *tvar = NULL;
    break;
  case Ps2o_string_prop_1_empty:
    *tvar = CORBA::string_dup("");
    break;
  case Ps2o_string_prop_1_nullIOR:
    *tvar = glob_TS.nullIOR_string();
    break;
  }
  return TRUE;
}

int relinquish_s2o_string (VSString *testvar, enum prop_s2o_string_prop_1 prop_1)
{
		char **tvar = (char **) testvar;
		switch(prop_1){
				case Ps2o_string_prop_1_good:
				case Ps2o_string_prop_1_empty:
						CORBA::string_free(*tvar);
						break;
				case Ps2o_string_prop_1_bad:
						break;
				case Ps2o_string_prop_1_null:
						break;
				case Ps2o_string_prop_1_nullIOR:
						break;
		}
		return TRUE;
}

/* ------------------------------------------------------------ */
/* test variable objectid() */

static unsigned long oid_count = 0;
static CORBA::ORB::ObjectIdList *olist = 0;

static void init_olist()
{
		CORBA::ORB_ptr orb = glob_TS.m_corb->orb();

    try {
        olist = orb->list_initial_services();
    }
    catch (...) {
    }
}

static char *get_objectid()
{
		if(!olist)
				init_olist();

    if(!olist)
				return CORBA::string_dup("");

		if(oid_count < olist->maximum()){
        CORBA::ORB::ObjectId str;
        str = (*olist)[oid_count++];
        if(!str)
						return CORBA::string_dup("");

				return CORBA::string_dup(str);
		}

		return CORBA::string_dup("");
}

int provide_objectid (VSObjectId **testvar, enum prop_objectid_prop_1 prop_1)
{
		CORBA::ORB::ObjectId tvar;
		switch(prop_1){
				case Pobjectid_prop_1_good1:
				case Pobjectid_prop_1_good2:
				case Pobjectid_prop_1_good3:
				case Pobjectid_prop_1_good4:
						tvar = get_objectid();
						break;
				case Pobjectid_prop_1_bad:
						tvar = CORBA::string_dup("bad");
						break;
/*
				case Pobjectid_prop_1_null:
						*tvar = NULL;
						break;
*/
				case Pobjectid_prop_1_empty:
						tvar = CORBA::string_dup("");
						break;
		}
		
		CORBA::ORB::ObjectId *tmp = new CORBA::ORB::ObjectId();
		*tmp = tvar;
		*testvar = (VSObjectId *) tmp;
		return TRUE;
}

int relinquish_objectid (VSObjectId **testvar, enum prop_objectid_prop_1 prop_1)
{
		CORBA::ORB::ObjectId *tvar = (CORBA::ORB::ObjectId *) *testvar;
		switch(prop_1){
				case Pobjectid_prop_1_good1:
				case Pobjectid_prop_1_good2:
				case Pobjectid_prop_1_good3:
				case Pobjectid_prop_1_good4:
				case Pobjectid_prop_1_bad:
				case Pobjectid_prop_1_empty:
						CORBA::string_free(*tvar);
						break;
/*
				case Pobjectid_prop_1_null:
						break;
*/
		}
		return TRUE;
}

/*------------------------------------------------------------------*/
/* test variable VSValueFactory * factory(prop value) */

int provide_factory (VSValueFactory *testvar, enum prop_factory_prop_1 prop_1)
{
    CORBA::ValueFactoryBase *tvar;

    switch(prop_1){
      case Pfactory_prop_1_valid:
      {	
	  try {
	      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	      TestValueFactory_impl *fac = new TestValueFactory_impl();
	      tvar = fac;
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;
    }

    *testvar = (VSValueFactory) tvar;
    return TRUE;
}

int relinquish_factory (VSValueFactory *testvar, enum prop_factory_prop_1 prop_1)
{
   CORBA::ValueFactoryBase *tvar = (CORBA::ValueFactoryBase *) *testvar;

   switch(prop_1){
     case Pfactory_prop_1_valid:
	   break;
   } 
    
    return TRUE;
}


/*------------------------------------------------------------------*/
/* test variable VSRepositoryId repId(prop value) */
int provide_repId (VSRepositoryId *testvar, enum prop_repId_prop_1 prop_1)
{
    char **tvar = (char **) testvar; 
    switch(prop_1){
      case PrepId_prop_1_valid:
	    *tvar = CORBA::string_dup("IDL:DAnyTest/TestValue:1.0"); 
	    break;
    }
    return TRUE;
}

int relinquish_repId (VSRepositoryId *testvar, enum prop_repId_prop_1 prop_1)
{
    char **tvar = (char **) testvar;  
    switch(prop_1){
      case PrepId_prop_1_valid:	   
	    break; 
    }

    return TRUE;
}


/*------------------------------------------------------------------*/
/* test variable VSORB * orb(anonymous prop) */

int provide_orb (VSORB_ptr **testvar, enum prop_orb_prop_1 prop_1)
{
   CORBA::ORB_ptr *tvar; 

    switch(prop_1){
      case Porb_prop_1_valid:
      {
	  try {
	      CORBA::ORB_ptr orb = glob_TS.m_corb->orb();
	      CORBA::ORB_ptr tmp =  CORBA::ORB::_duplicate(orb);
	      tvar = new CORBA::ORB_ptr(tmp);
	  }
	  catch (...) {
	      return FALSE;
	  }
      }
      break;	    
    }

    *testvar = (VSORB_ptr *) tvar;
    return TRUE;
}

int relinquish_orb (VSORB_ptr **testvar, enum prop_orb_prop_1 prop_1)
{
    CORBA::ORB_ptr *tvar = (CORBA::ORB_ptr *) *testvar;

    switch(prop_1){
      case Porb_prop_1_valid:
	    (*tvar)->destroy();
	    break;
    }
    return TRUE;
}

/*------------------------------------------------------------------*/
/* test variable VSPolicyType p_type(anonymous prop) */
int provide_p_type (VSPolicyType *testvar, enum prop_p_type_prop_1 prop_1)
{
    CORBA::PolicyType tvar;

     switch(prop_1){
       case Pp_type_prop_1_bad:       
	     tvar = 25;
	     break;       
       case Pp_type_prop_1_good:
	     tvar = PortableServer::THREAD_POLICY_ID;
	     break;
     }

     *testvar = (VSPolicyType) tvar;
     return TRUE;
}

int relinquish_p_type (VSPolicyType *testvar, enum prop_p_type_prop_1 prop_1)
{
    CORBA::PolicyType tvar = (CORBA::PolicyType) *testvar;
    
     switch(prop_1){
       case Pp_type_prop_1_bad:
       case Pp_type_prop_1_good:	    
	     break;
     }
     return TRUE;
}


/*------------------------------------------------------------------*/
/* test variable VSAny p_value(anonymous prop) */
int provide_p_value (VSAny *testvar, enum prop_p_value_prop_1 prop_1)
{
    CORBA::Any *tvar = new CORBA::Any;;
    
    switch(prop_1){
      case Pp_value_prop_1_bad:
      {
	  PortableServer::ServantRetentionPolicyValue val = PortableServer::NON_RETAIN;
	  *tvar <<= val;
      }
      break;
      case Pp_value_prop_1_good:
      {
	  PortableServer::ThreadPolicyValue val = PortableServer::SINGLE_THREAD_MODEL;
	   *tvar <<= val;
      }
      break;
   }
    
    *testvar = (VSAny) tvar;
    return TRUE;
}

int relinquish_p_value (VSAny *testvar, enum prop_p_value_prop_1 prop_1)
{
    CORBA::Any *tvar = (CORBA::Any *) testvar;

    switch(prop_1){
      case Pp_value_prop_1_bad:
      case Pp_value_prop_1_good:	   
	    break;
    }
    
    return TRUE;
}

} // end

