//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include	<corba_supp.hh>
#include	<tserver.hh>

extern "C" {

#include	<t1.h>
#include	<t1_aux.h>
#include	<t1i_tdi.h>

    /*------------------------------------------------------*/

    VSObject_ptr *VSnil(VSEnvironment *ev)
    {
	CORBA::Environment *env = (CORBA::Environment *) ev;
	CORBA::Object_ptr ttc;

	try {
	    ttc = CORBA::Object::_nil();
	}
	catch (CORBA::Exception &ex) {
	    CORBA::Exception *t = exception_convert(&ex);
	    if(env)
		env->exception(t);
	    return (VSObject_ptr *) 0;
	}
	catch (...) {
	    return (VSObject_ptr *) 0;
	}

	CORBA::Object_ptr *rtc = new CORBA::Object_ptr(ttc);
	return (VSObject_ptr *) rtc;
    }

    VSObject_ptr *VSduplicate(VSObject_ptr *o1, VSEnvironment *ev)
    {
	CORBA::Environment *env = (CORBA::Environment *) ev;
	CORBA::Object_ptr *tc = (CORBA::Object_ptr *) o1;
	CORBA::Object_ptr ttc;

	try {
	    ttc = CORBA::Object::_duplicate(tc ? *tc : (CORBA::Object_ptr) 0);
	}
	catch (CORBA::Exception &ex) {
	    CORBA::Exception *t = exception_convert(&ex);
	    if(env)
		env->exception(t);
	    return (VSObject_ptr *) 0;
	}
	catch (...) {
	    return (VSObject_ptr *) 0;
	}

	CORBA::Object_ptr *rtc = new CORBA::Object_ptr(ttc);
	return (VSObject_ptr *) rtc;
    }

    /*------------------------------------------------------*/

    VSInterfaceDef_ptr *VSget_interface(VSObject_ptr *o1, VSEnvironment *ev)
    {
	CORBA::Environment *env = (CORBA::Environment *) ev;
	CORBA::Object_ptr *o = (CORBA::Object_ptr *) o1;
	CORBA::InterfaceDef_ptr ret;

	if(!o || is_invalid_object_ptr(o1) || is_nil_object_ptr(o1))
		return (VSInterfaceDef_ptr *) 0;

	try {
	    ret = (*o)->_get_interface();
	}
	catch (CORBA::Exception &ex) {
	    CORBA::Exception *t = exception_convert(&ex);
	    if(env)
		env->exception(t);
	    return (VSInterfaceDef_ptr *) 0;
	}
	catch (...) {
	    return (VSInterfaceDef_ptr *) 0;
	}

	CORBA::InterfaceDef_ptr *rtf = new CORBA::InterfaceDef_ptr(ret);
	return (VSInterfaceDef_ptr *) rtf;
    }

    /*------------------------------------------------------*/

    VSBoolean VSis_a(VSObject_ptr *o1, VSChar* plid, VSEnvironment *ev)
    {
	CORBA::Boolean ret = 0;
	CORBA::Environment *env = (CORBA::Environment *) ev;
	CORBA::Object_ptr *o = (CORBA::Object_ptr *) o1;
	char *lid = (char *) plid;

	if(!o || is_invalid_object_ptr(o1) || is_nil_object_ptr(o1))
		return (VSBoolean) ret;

	try {
	    ret = (*o)->_is_a(lid);
	}
	catch (CORBA::Exception &ex) {
	    CORBA::Exception *t = exception_convert(&ex);
	    if(env)
		env->exception(t);
	    return (VSBoolean) 0;
	}
	catch (...) {
	    return (VSBoolean) 0;
	}

	return (VSBoolean) ret;
    }

    /*------------------------------------------------------*/

    VSBoolean VSis_equivalent(VSObject_ptr *o1, VSObject_ptr *o2, VSEnvironment *ev)
    {
	CORBA::Boolean ret = 0;
	CORBA::Environment *env = (CORBA::Environment *) ev;
	CORBA::Object_ptr *optr1 = (CORBA::Object_ptr *) o1;
	CORBA::Object_ptr *optr2 = (CORBA::Object_ptr *) o2;

	if(!o1 || is_invalid_object_ptr(o1) || is_nil_object_ptr(o1))
		return (VSBoolean) ret;

	try {
	    ret = (*optr1)->_is_equivalent(optr2 ? *optr2 : (CORBA::Object_ptr) 0);
	}
	catch (CORBA::Exception &ex) {
	    CORBA::Exception *t = exception_convert(&ex);
	    if(env)
		env->exception(t);
	    return (VSBoolean) 0;
	}
	catch (...) {
	    return (VSBoolean) 0;
	}

	return (VSBoolean) ret;
    }


    /*------------------------------------------------------*/

    VSBoolean VSnon_existent(VSObject_ptr *o1, VSEnvironment *ev)
    {
	CORBA::Boolean ret = -1;
	CORBA::Environment *env = (CORBA::Environment *) ev;
	CORBA::Object_ptr *o = (CORBA::Object_ptr *) o1;

	if(!o || is_invalid_object_ptr(o1) || is_nil_object_ptr(o1))
		return (VSBoolean) ret;

	try {
	    ret = (*o)->_non_existent();
	}
	catch (CORBA::Exception &ex) {
	    CORBA::Exception *t = exception_convert(&ex);
	    if(env)
		env->exception(t);
	    return (VSBoolean) 0;
	}
	catch (...) {
	    return (VSBoolean) 0;
	}

	return (VSBoolean) ret;
    }

    /*------------------------------------------------------*/

    VSULong VShash(VSObject_ptr *o1, VSULong mx, VSEnvironment *ev)
    {
	CORBA::ULong ret = 0;
	CORBA::Environment *env = (CORBA::Environment *) ev;
	CORBA::Object_ptr *o = (CORBA::Object_ptr *) o1;
	CORBA::ULong maxim = (CORBA::ULong) mx;

	if(!o || is_invalid_object_ptr(o1) || is_nil_object_ptr(o1))
		return (VSULong) ret;

	try {
	    ret = (*o)->_hash(maxim);
	}
	catch (CORBA::Exception &ex) {
	    CORBA::Exception *t = exception_convert(&ex);
	    if(env)
		env->exception(t);
	    return (VSULong) 0;
	}
	catch (...) {
	    return (VSULong) 0;
	}

	return (VSULong) ret;
    }

}

