//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
// Copyright (c) 2001 Object Oriented Concepts GmbH.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <corba_supp.hh>
#include <tserver.hh>

extern "C" {

#include <corba_defs_tdd.h>
#include <t1_aux.h>

//-----------------------------------------------------------
// General TypeCode.

int is_same_typecode_ptr(VSTypeCode_ptr *o1, VSTypeCode_ptr *o2)
{
		if(is_nil_typecode_ptr(o1) && is_nil_typecode_ptr(o2))
				return TRUE;
		
		return TRUE;
}

int cleanup_typecode_ptr(VSTypeCode_ptr *o1)
{
		CORBA::TypeCode_ptr *tc = (CORBA::TypeCode_ptr *) o1;
		if(!tc)
				return TRUE;

		delete tc;
		return TRUE;
}

int kind_is_correct(VSTCKind pk)
{
		CORBA::TCKind k = (CORBA::TCKind) pk;
		int ret = FALSE;

		CORBA::ULong value = glob_TS.testcase(VSPROVIDE_TC);
		switch(value){
       	case Ptypecode_v_tvoid:
						ret = (k == CORBA::tk_void);
						break;
        case Ptypecode_v_tshort:
						ret = (k == CORBA::tk_short);
						break;
        case Ptypecode_v_tlong:
						ret = (k == CORBA::tk_long);
						break;
        case Ptypecode_v_tushort:
						ret = (k == CORBA::tk_ushort);
						break;
        case Ptypecode_v_tulong:
						ret = (k == CORBA::tk_ulong);
						break;
        case Ptypecode_v_tfloat:
						ret = (k == CORBA::tk_float);
						break;
        case Ptypecode_v_tdouble:
						ret = (k == CORBA::tk_double);
						break;
        case Ptypecode_v_tboolean:
						ret = (k == CORBA::tk_boolean);
						break;
        case Ptypecode_v_tchar:
						ret = (k == CORBA::tk_char);
						break;
        case Ptypecode_v_toctet:
						ret = (k == CORBA::tk_octet);
						break;
        case Ptypecode_v_tany:
						ret = (k == CORBA::tk_any);
						break;
        case Ptypecode_v_tTypeCode:
						ret = (k == CORBA::tk_TypeCode);
						break;
        case Ptypecode_v_tPrincipal:
						ret = (k == CORBA::tk_Principal);
						break;
        case Ptypecode_v_tobjref:
						ret = (k == CORBA::tk_objref);
						break;
        case Ptypecode_v_tstruct:
						ret = (k == CORBA::tk_struct);
						break;
        case Ptypecode_v_tunion:
						ret = (k == CORBA::tk_union);
						break;
        case Ptypecode_v_tenum:
						ret = (k == CORBA::tk_enum);
						break;
        case Ptypecode_v_tstring:
						ret = (k == CORBA::tk_string);
						break;
        case Ptypecode_v_tsequence:
						ret = (k == CORBA::tk_sequence);
						break;
        case Ptypecode_v_tarray:
						ret = (k == CORBA::tk_array);
						break;
        case Ptypecode_v_talias:
						ret = (k == CORBA::tk_alias);
						break;
        case Ptypecode_v_texcept:
						ret = (k == CORBA::tk_except);
						break;
		}

		return ret;
}

int is_equal_kind(VSTypeCode_ptr *t1,VSTypeCode_ptr *t2)
{
		CORBA::TypeCode_ptr *tptr1 = (CORBA::TypeCode_ptr *) t1;
		CORBA::TypeCode_ptr *tptr2 = (CORBA::TypeCode_ptr *) t2;

		if(!tptr1 || !(CORBA::TypeCode *) *tptr1)
				return FALSE;
		if(!tptr2 || !(CORBA::TypeCode *) *tptr2)
				return FALSE;

		return ((*tptr1)->kind() == (*tptr2)->kind()) ? TRUE : FALSE;
}

//-----------------------------------------------------------
//

int id_typecode_is_allowed(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		CORBA::TCKind k = (*tptr)->kind();
		switch(k){
				case CORBA::tk_struct:
				case CORBA::tk_union:
				case CORBA::tk_enum:
				case CORBA::tk_except:
				case CORBA::tk_alias:
				case CORBA::tk_objref:
						return TRUE;
		}

		return FALSE;
}

int id_is_correct(VSChar *pid)
{
		char *id = (char *) pid;

		return TRUE;
}

int name_typecode_is_allowed(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		CORBA::TCKind k = (*tptr)->kind();
		switch(k){
				case CORBA::tk_struct:
				case CORBA::tk_union:
				case CORBA::tk_enum:
				case CORBA::tk_except:
				case CORBA::tk_alias:
				case CORBA::tk_objref:
						return TRUE;
		}

		return FALSE;
}

int name_is_correct(VSChar *pname)
{
		char *name = (char *) pname;

		CORBA::ULong value = glob_TS.testcase(VSPROVIDE_TC);
		switch(value){
        case Ptypecode_v_tstruct:
        case Ptypecode_v_tunion:
        case Ptypecode_v_tenum:
        case Ptypecode_v_texcept:
        case Ptypecode_v_talias:
						{
						if(!name)
								return FALSE;
						return((strcmp(name,"ApTest") == 0) ? TRUE : FALSE);
						}
						break;
        case Ptypecode_v_tobjref:
						break;
				default:
						break;
		}

		return TRUE;
}

//-----------------------------------------------------------

int member_count_typecode_is_allowed(VSTypeCode_ptr *t)
{
		return member_name_typecode_is_allowed(t);
}

int member_count_is_correct(VSULong c)
{
		CORBA::ULong count = (CORBA::ULong) c;

		return (count == 1) ? TRUE : FALSE;
}

int member_name_typecode_is_allowed(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		CORBA::TCKind k = (*tptr)->kind();
		switch(k){
				case CORBA::tk_struct:
				case CORBA::tk_union:
				case CORBA::tk_enum:
				case CORBA::tk_except:
						return TRUE;
		}

		return FALSE;
}

int member_name_is_correct(VSChar *pname)
{
		char *name = (char *) pname;

		CORBA::ULong value = glob_TS.testcase(VSPROVIDE_TC);
		switch(value){
        case Ptypecode_v_tstruct:
        case Ptypecode_v_tunion:
        case Ptypecode_v_tenum:
        case Ptypecode_v_texcept:
						{
						if(!name)
								return FALSE;
						return((strcmp(name,"ApTest_mem1") == 0) ? TRUE : FALSE);
						}
						break;
				default:
						break;
		}

		return TRUE;
}

int name_index_out_of_bounds(VSTypeCode_ptr *t, VSULong i)
{
		CORBA::ULong index = (CORBA::ULong) i;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;

		if(member_count_typecode_is_allowed(t)){
				CORBA::ULong count = (*tptr)->member_count();
				return (index < count) ? FALSE : TRUE;
		}

		return FALSE;
}

//-----------------------------------------------------------

int member_type_typecode_is_allowed(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		CORBA::TCKind k = (*tptr)->kind();
		switch(k){
				case CORBA::tk_struct:
				case CORBA::tk_union:
				case CORBA::tk_except:
						return TRUE;
		}

		return FALSE;
}

int member_type_is_correct(VSTypeCode_ptr *t)
{
  CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
  
  CORBA::ULong value = glob_TS.testcase(VSPROVIDE_TC);
  switch(value){
  case Ptypecode_v_tstruct:
  case Ptypecode_v_texcept:
    {
      if(!tptr || !(CORBA::TypeCode *) *tptr)
	return FALSE;
      CORBA::TCKind k = (*tptr)->kind();
      CORBA::release(*tptr);
      delete tptr;
      return((k == CORBA::tk_short) ? TRUE : FALSE);
      break;
    }
  case Ptypecode_v_tunion:
    {
      if(!tptr || !(CORBA::TypeCode *) *tptr)
	return FALSE;
      CORBA::TCKind k = (*tptr)->kind();
      CORBA::release(*tptr);
      delete tptr;
      return((k == CORBA::tk_long) ? TRUE : FALSE);
    }
    break;
  default:
    break;
  }
  
  return TRUE;
}

int type_index_out_of_bounds(VSTypeCode_ptr *t, VSULong i)
{
		CORBA::ULong index = (CORBA::ULong) i;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;

		if(member_type_typecode_is_allowed(t)){
				CORBA::ULong count = (*tptr)->member_count();
				return (index < count) ? FALSE : TRUE;
		}

		return FALSE;
}

//-----------------------------------------------------------

int member_label_typecode_is_allowed(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		CORBA::TCKind k = (*tptr)->kind();
		switch(k){
				case CORBA::tk_union:
						return TRUE;
		}

		return FALSE;
}

int member_label_is_correct(VSAny *t)
{
		CORBA::Any *aptr = (CORBA::Any *) t;

		CORBA::ULong value = glob_TS.testcase(VSPROVIDE_TC);
		switch(value){
        case Ptypecode_v_tunion:
						{
						if(!aptr)
								return FALSE;
						CORBA::Long al;
						(*aptr) >>= al;
						delete aptr;
						return((al == 1) ? TRUE : FALSE);
						}
						break;
				default:
						break;
		}

		return TRUE;
}

int label_index_out_of_bounds(VSTypeCode_ptr *t, VSULong i)
{
		CORBA::ULong index = (CORBA::ULong) i;
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;

		if(member_label_typecode_is_allowed(t)){
				CORBA::ULong count = (*tptr)->member_count();
				return (index < count) ? FALSE : TRUE;
		}

		return FALSE;
}

//-----------------------------------------------------------

int dt_typecode_is_allowed(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		CORBA::TCKind k = (*tptr)->kind();
		switch(k){
				case CORBA::tk_union:
						return TRUE;
		}

		return FALSE;
}

int dt_is_correct(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;

		CORBA::ULong value = glob_TS.testcase(VSPROVIDE_TC);
		switch(value){
        case Ptypecode_v_tunion:
						{
						if(!tptr || !(CORBA::TypeCode *) *tptr)
								return FALSE;
						CORBA::TCKind k = (*tptr)->kind();
						CORBA::release(*tptr);
						delete tptr;
						return((k == CORBA::tk_long) ? TRUE : FALSE);
						}
						break;
				default:
						break;
		}

		return TRUE;
}

//-----------------------------------------------------------

int dfind_typecode_is_allowed(VSTypeCode_ptr *t)
{
		return dt_typecode_is_allowed(t);
}

int dfind_is_correct(VSLong i)
{
		CORBA::Long index = (CORBA::Long) i;
		CORBA::ULong value = glob_TS.testcase(VSPROVIDE_TC);

		switch(value){
        	case Ptypecode_v_tunion:
				return (index == -1) ? TRUE : FALSE;
		}

		return FALSE;
}

//-----------------------------------------------------------

int length_typecode_is_allowed(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		CORBA::TCKind k = (*tptr)->kind();
		switch(k){
				case CORBA::tk_string:
				case CORBA::tk_sequence:
				case CORBA::tk_array:
						return TRUE;
		}

		return FALSE;
}

int length_is_correct(VSULong c)
{
		CORBA::ULong count = (CORBA::ULong) c;
		CORBA::ULong value = glob_TS.testcase(VSPROVIDE_TC);

		switch(value){
        case Ptypecode_v_tsequence:
        case Ptypecode_v_tarray:
						return (count == 1) ? TRUE : FALSE;
        case Ptypecode_v_tstring:
						return (count == 0) ? TRUE : FALSE;
		}

		return FALSE;
}

//-----------------------------------------------------------

int content_typecode_is_allowed(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;
		CORBA::TCKind k = (*tptr)->kind();
		switch(k){
				case CORBA::tk_sequence:
				case CORBA::tk_array:
				case CORBA::tk_alias:
						return TRUE;
		}

		return FALSE;
}

int content_is_correct(VSTypeCode_ptr *t)
{
		CORBA::TypeCode_ptr *tptr = (CORBA::TypeCode_ptr *) t;

		CORBA::ULong value = glob_TS.testcase(VSPROVIDE_TC);
		switch(value){
        case Ptypecode_v_tsequence:
        case Ptypecode_v_tarray:
        case Ptypecode_v_talias:
						{
						if(!tptr || !(CORBA::TypeCode *) *tptr)
								return FALSE;
						CORBA::TCKind k = (*tptr)->kind();
						CORBA::release(*tptr);
						delete tptr;
						return((k == CORBA::tk_short) ? TRUE : FALSE);
						}
						break;
				default:
						break;
		}

		return TRUE;
}

}

