//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/testserv/tinClt.cpp
//
// Purpose:
//
// Modifications:
// $Log: tinClt.cpp,v $
// Revision 1.3  2001/01/21 18:07:09  arnop
// - use _var helper functions .in() et al at some locations
// - operator>>=( Any, T*&) is deprecated
//
// Revision 1.2  2001/01/10 12:53:51  mli
// Add server.cpp server.hh.
//
// Revision 1.1.1.1  2000/02/09 17:14:57  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.8  1999/04/26 14:49:01  james
// Changed CORBA::Char* to char*
//
// Revision 1.7  1997/12/05 16:23:18  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.6  1997/11/27 12:07:40  cwatson
// Removed Orbix specific code.
//
// Revision 1.5  1997/11/10 11:52:12  stephen
// Updated to uniform naming convention 'apiServer'.
//
// Revision 1.4  1997/10/23 13:34:12  stephen
// Removed parameters to OrbSupport object.
//
// Revision 1.3  1997/10/04 15:46:54  cwatson
// Abstracted common ORB code to a higher level (in orbsupp).
//
// Revision 1.2  1997/10/03 13:52:21  ian
// Simplified test server interface.
//
// Revision 1.1  1997/10/01 16:07:46  cwatson
// Makefile changes.
// Moved the old SII test to here as an API support server.
// Moved ORB specific code to samples dir.
//
//

#include <stdlib.h>
#include <orb_supp.hh>
#include <tin.hh>

int
main( int argc, char *argv[] )
{
    apiServer_ptr tobj;
    CORBA::Long num;
    CORBA::Object_ptr obj;
    CORBA::Environment_ptr ev;

    OrbSupport *corb;
    try {
	corb = new OrbSupport();
	corb->orb()->create_environment(ev);
	corb->client_init();
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed to initialise ORB", ev);
    }
    if(!corb->statusok())
	corb->orbFinish("Failed to initialise ORB", ev);

    try {
	obj = corb->readObjref();
	tobj = apiServer::_narrow(obj);
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed to read object reference.", ev);
    }


    //
    // CORBA::Char test
    //
    char a;
    char b = 'b';
    char c = 'c';
    char d;
    
    try {
	a = tobj->charOp(b, c, d);

	if ( (a == 'b') && (c == 'b') && (d == 'c') )
	    cout << "charOp() success" << endl;
	else
	    cout << "charOp() failed" << endl;
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed on charOp().", ev);
    }
    
    //
    // CORBA::Octet test
    //
    CORBA::Octet o;
    CORBA::Octet o1 = 'x';
    CORBA::Octet o2 = 'y';
    CORBA::Octet o3;
    
    try {
	o = tobj->octetOp(o1, o2, o3);

	if ( (o == 'x') && (o2 == 'x') && (o3 == 'y') )
	    cout << "octetOp() success" << endl;
	else
	    cout << "octetOp() failed" << endl;
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed on octetOp().", ev);
    }
    
    //
    // CORBA::Boolean test
    //
    CORBA::Boolean b0;
    CORBA::Boolean b1 = TRUE;
    CORBA::Boolean b2 = FALSE;
    CORBA::Boolean b3;
    
    try {
	b0 = tobj->booleanOp(b1, b2, b3);

	if ( b0 && b2 && !b3 )
	    cout << "booleanOp() success" << endl;
	else
	    cout << "booleanOp() failed" << endl;
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed on octetOp().", ev);
    }
    
    //
    // CORBA::Short test
    //
    CORBA::Short s;
    CORBA::Short s1 = 32;
    CORBA::Short s2 = -84;
    CORBA::Short s3;
    
    try {
	s = tobj->shortOp(s1, s2, s3);

	if ( (s == 32) && (s2 == 32) && (s3 == -84) )
	    cout << "shortOp() success" << endl;
	else
	    cout << "shortOp() failed" << endl;
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed on shortOp().", ev);
    }

    //
    // CORBA::Long test
    //
    CORBA::Long l;
    CORBA::Long l1 = 6629012;
    CORBA::Long l2 = -6627102;
    CORBA::Long l3;
    
    try {
	l = tobj->longOp(l1, l2, l3);

	if ( (l == 6629012) && (l2 == 6629012) && (l3 == -6627102) )
	    cout << "longOp() success" << endl;
	else
	    cout << "longOp() failed" << endl;
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed on longOp().", ev);
    }

    //
    // CORBA::Float test
    //
    CORBA::Float f;
    CORBA::Float f1 = 32.84;
    CORBA::Float f2 = 9876543.21;
    CORBA::Float f3;
    
    try {
	f = tobj->floatOp(f1, f2, f3);

	if ( ( abs(int (f - 32.84)) > 0.00001 ) ||
	    ( abs(int (f2 - 32.84)) > 0.00001 ) ||
		( abs(int (f3 - 9876543.21)) > 0.00001 ) )
	    cout << "floatOp() failed" << endl;
	else
	    cout << "floatOp() success" << endl;
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed on floatOp().", ev);
    }

    //
    // CORBA::Double test
    //
    CORBA::Double d0;
    CORBA::Double d1 = 408399;
    CORBA::Double d2 = -1234567890;
    CORBA::Double d3;
    
    try {
	d0 = tobj->doubleOp(d1, d2, d3);

	if ( (d0 == 408399) && (d2 == 408399) && (d3 == -1234567890) )
	    cout << "doubleOp() success" << endl;
	else
	    cout << "doubleOp() failed" << endl;
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed on doubleOp().", ev);
    }

    //
    // char* test
    //
    char* str;
    CORBA::String_var str1 = CORBA::string_dup("wheelchair");
    CORBA::String_var str2 = CORBA::string_dup("opticians");
    CORBA::String_var str3;
    
    try {
	str = tobj->stringOp(str1.in(), str2.inout(), str3.out());

	if ( !(strcmp(str, "wheelchair")) &&
	    !(strcmp(str2, "wheelchair")) &&
	    !(strcmp(str3, "opticians")) )
	    cout << "stringOp() success" << endl;
	else
	    cout << "stringOp() failed" << endl;
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed on stringOp().", ev);
    }

    //
    // CORBA::UShort test
    //
    CORBA::UShort us;
    CORBA::UShort us1 = 32;
    CORBA::UShort us2 = 84;
    CORBA::UShort us3;
    
    try {
	us = tobj->ushortOp(us1, us2, us3);

	if ( (us == 32) && (us2 == 32) && (us3 == 84) )
	    cout << "ushortOp() success" << endl;
	else
	    cout << "ushortOp() failed" << endl;
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed on ushortOp().", ev);
    }

    //
    // CORBA::ULong test
    //
    CORBA::ULong ul;
    CORBA::ULong ul1 = 6629012;
    CORBA::ULong ul2 = 6627102;
    CORBA::ULong ul3;
    
    try {
	ul = tobj->ulongOp(ul1, ul2, ul3);

	if ( (ul == 6629012) && (ul2 == 6629012) && (ul3 == 6627102) )
	    cout << "ulongOp() success" << endl;
	else
	    cout << "ulongOp() failed" << endl;
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed on ulongOp().", ev);
    }

    //
    // Increment test
    //
    num = 10;
    try {
	num = tobj->increment(num);
	if (num == 11)
	    cout << "increment() success" << endl;
	else
	    cout << "increment() failed" << endl;
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed to increment.", ev);
    }

    //
    // Release the orb reference
    //
    try {
	CORBA::release(tobj);
    }
    catch (CORBA::Exception &ex) {
	ev->exception(&ex);
	corb->orbFinish("Failed to release.", ev);
    }

    delete corb;

    return 1;
}

