//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/api/testserv/tinSrv.cpp
//
// Purpose:
//
// Modifications:
// $Log: tinSrv.cpp,v $
// Revision 1.3  2001/01/10 12:53:51  mli
// Add server.cpp server.hh.
//
// Revision 1.2  2000/04/11 16:26:27  mli
//   o   use CORBA::Environment_var and
//       CORBA::ORB::create_environment
//   o   use _this() on apiServer_imp
//
// Revision 1.1.1.1  2000/02/09 17:14:57  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.7  1998/04/27 13:16:15  stephen
// Pass server object through to init call.
//
// Revision 1.6  1997/12/05 16:23:18  cwatson
// Now uses .hh for c++ include files.
//
// Revision 1.5  1997/11/27 12:07:40  cwatson
// Removed Orbix specific code.
//
// Revision 1.4  1997/11/10 11:50:57  stephen
// Updated to uniform naming convention 'apiServer'.
//
// Revision 1.3  1997/10/23 13:33:48  stephen
// Removed argc and argv from OrbSupport object.
//
// Revision 1.2  1997/10/04 15:46:54  cwatson
// Abstracted common ORB code to a higher level (in orbsupp).
//
// Revision 1.1  1997/10/01 16:07:47  cwatson
// Makefile changes.
// Moved the old SII test to here as an API support server.
// Moved ORB specific code to samples dir.
//
//

#include <orb_supp.hh>
#include <tin.hh>
#include <server.hh>

/*
************************************************************
*    The Main entry point for the server capsule
*    The main purpose is to create an object reference to
*    the TestServer interface and "export" it so that
*    clients may make invocations upon it and then suspend
*    to wait for such invocations.
************************************************************
*/

int main( int argc, char *argv[] )
{
    CORBA::Environment_var ev;
    apiServer_imp *objref;
    
    OrbSupport *sorb;
    
    sorb = new OrbSupport(argc, argv, "apiServer");
    if(!CORBA::is_nil(sorb->orb()) && sorb->statusok())
      sorb->orb()->create_environment(ev);
    else
      {
	delete sorb;
        return -1;
      }
    
    try {
	objref = new apiServer_imp;
    }
    catch (CORBA::Exception &sysEx) {
	ev->exception(&sysEx);
	delete sorb;
	return -1;
    }
    
    try {
	sorb->server_init(objref);
    }
    catch (CORBA::Exception &sysEx) {
	ev->exception(&sysEx);
	delete sorb;
	return -1;
    }
    
    try {
      CORBA::Object_ptr objptr;
      objptr= objref->_this();
      sorb->writeObjref(objptr);
    }
    catch (CORBA::Exception &sysEx) {
	ev->exception(&sysEx);
	delete sorb;
	return -1;
    }
    
    
    if(!sorb || !sorb->statusok()){
	delete sorb;
	return -1;
    }
    
    sorb->orb()->run();
    delete sorb;
    
    return 1;
}    

