//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/28 18:39:44  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:15:46  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.6  1998/03/27 14:57:10  stephen
// Comment updated for previous fix.
//
// Revision 1.5  1998/03/27 14:56:21  stephen
// Apple initialised with incorrect value.
//
// Revision 1.4  1997/12/09 11:24:59  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

  if (::Apple != 0)
  {
    ::tet_infoline("The constant ::Apple has incorrect value");
    ::tet_printf("Expected 0, but found %d",::Apple);
    ::tet_result(TET_FAIL);
  }
  if (::Banana != 253)
  {
    ::tet_infoline("The constant ::Banana has incorrect value");
    ::tet_printf("Expected 253, but found %d",::Banana);
    ::tet_result(TET_FAIL);
  }
  if (::Carrots != 94873)
  {
    ::tet_infoline("The constant ::Carrots has incorrect value");
    ::tet_printf("Expected 94873, but found %d",::Carrots);
    ::tet_result(TET_FAIL);
  }
  if (::Lemon != 4294967295)
  {
    ::tet_infoline("The constant ::Lemon has incorrect value");
    ::tet_printf("Expected 4294967295, but found %U",::Lemon);
    ::tet_result(TET_FAIL);
  }
  if (::BlackCherry != 0)
  {
    ::tet_infoline("The constant ::BlackCherry has incorrect value");
    ::tet_printf("Expected 0, but found %d",::BlackCherry);
    ::tet_result(TET_FAIL);
  }
  if (::Starfruit != 10)
  {
    ::tet_infoline("The constant ::Starfruit has incorrect value");
    ::tet_printf("Expected 10, but found %d",::Starfruit);
    ::tet_result(TET_FAIL);
  }
  if (::PassionFruit != 342391)
  {
    ::tet_infoline("The constant ::PassionFruit has incorrect value");
    ::tet_printf("Expected 342391, but found %d",::PassionFruit);
    ::tet_result(TET_FAIL);
  }
  if (::Orange != 4294967295)
  {
    ::tet_infoline("The constant ::Orange has incorrect value");
    ::tet_printf("Expected 4294967295, but found %U",::Orange);
    ::tet_result(TET_FAIL);
  }
  if (::Dates != 74565)
  {
    ::tet_infoline("The constant ::Dates has incorrect value");
    ::tet_printf("Expected 74565, but found %d",::Dates);
    ::tet_result(TET_FAIL);
  }
  if (::Pumpkin != 11259375)
  {
    ::tet_infoline("The constant ::Pumpkin has incorrect value");
    ::tet_printf("Expected 11259375, but found %d",::Pumpkin);
    ::tet_result(TET_FAIL);
  }
  if (::Mango != 11259375)
  {
    ::tet_infoline("The constant ::Mango has incorrect value");
    ::tet_printf("Expected 11259375, but found %d",::Mango);
    ::tet_result(TET_FAIL);
  }
  if (::WaterMelon != 1000418)
  {
    ::tet_infoline("The constant ::WaterMelon has incorrect value");
    ::tet_printf("Expected 1000418, but found %d",::WaterMelon);
    ::tet_result(TET_FAIL);
  }
  if (::Lime != 123051)
  {
    ::tet_infoline("The constant ::Lime has incorrect value");
    ::tet_printf("Expected 123051, but found %d",::Lime);
    ::tet_result(TET_FAIL);
  }
  if (::Coconut != 4294967295)
  {
    ::tet_infoline("The constant ::Coconut has incorrect value");
    ::tet_printf("Expected 4294967295, but found %U",::Coconut);
    ::tet_result(TET_FAIL);
  }
::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerLychees(void)
{
  
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Lychees()");
#endif
  
  ernie.seed("LycheesCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LycheesReturn");
  try {
     tobj->Lychees();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Lychees");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


