//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 16:53:52  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:15:49  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.6  1999/05/04 12:05:49  james
// SR# 101 Changed CORBA::Char* to char*
//
// Revision 1.5  1999/04/29 14:12:12  ian
// SR111 Changed "\A""B" to octal "\012B"
//
// Revision 1.4  1997/12/09 11:25:13  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

  if (strcmp(::Apple,"abcdefghijklmnopqrstuvwxyz"))
  {
    ::tet_infoline("The constant ::Apple has incorrect value");
    strcpy(buf1,StringToString("abcdefghijklmnopqrstuvwxyz"));
    ::tet_printf("Expected %s, but found %s",buf1,
		 StringToString((char*)::Apple));
    ::tet_result(TET_FAIL);
  }
  if (strcmp(::Banana,"ABCDEFGHIJKLMNOPQRSTUVWXYZ"))
  {
    ::tet_infoline("The constant ::Banana has incorrect value");
    strcpy(buf1,StringToString("ABCDEFGHIJKLMNOPQRSTUVWXYZ"));
    ::tet_printf("Expected %s, but found %s",buf1,
		 StringToString((char*)::Banana));
    ::tet_result(TET_FAIL);
  }
  if (strcmp(::Carrots,"0123456789"))
  {
    ::tet_infoline("The constant ::Carrots has incorrect value");
    strcpy(buf1,StringToString("0123456789"));
    ::tet_printf("Expected %s, but found %s",buf1,
		 StringToString((char*)::Carrots));
    ::tet_result(TET_FAIL);
  }
  if (strcmp(::Lemon,"!#$%&()*+,-./:;<;>@[]^ _`{|}~"))
  {
    ::tet_infoline("The constant ::Lemon has incorrect value");
    strcpy(buf1,StringToString("!#$%&()*+,-./:;<;>@[]^ _`{|}~"));
    ::tet_printf("Expected %s, but found %s",buf1,
		 StringToString((char*)::Lemon));
    ::tet_result(TET_FAIL);
  }
  if (strcmp(::BlackCherry,"\n\t\v\b\r\f\a\\\?\'\""))
  {
    ::tet_infoline("The constant ::BlackCherry has incorrect value");
    strcpy(buf1,StringToString("\n\t\v\b\r\f\a\\\?\'\""));
    ::tet_printf("Expected %s, but found %s",buf1,
		 StringToString((char*)::BlackCherry));
    ::tet_result(TET_FAIL);
  }
  if (strcmp(::Starfruit,"\001\74\321\377"))
  {
    ::tet_infoline("The constant ::Starfruit has incorrect value");
    strcpy(buf1,StringToString("\001\74\321\377"));
    ::tet_printf("Expected %s, but found %s",buf1,
		 StringToString((char*)::Starfruit));
    ::tet_result(TET_FAIL);
  }
  if (strcmp(::PassionFruit,"\x1\xA\xe\x32\xAf\xff"))
  {
    ::tet_infoline("The constant ::PassionFruit has incorrect value");
    strcpy(buf1,StringToString("\x1\xA\xe\x32\xAf\xff"));
    ::tet_printf("Expected %s, but found %s",buf1,
		 StringToString((char*)::PassionFruit));
    ::tet_result(TET_FAIL);
  }
  if (strcmp(::Orange,"abcdefghijk""lmnopqrstuvwxyz"))
  {
    ::tet_infoline("The constant ::Orange has incorrect value");
    strcpy(buf1,StringToString("abcdefghijk""lmnopqrstuvwxyz"));
    ::tet_printf("Expected %s, but found %s",buf1,
		 StringToString((char*)::Orange));
    ::tet_result(TET_FAIL);
  }
  if (strcmp(::Dates,"\012B"))
  {
    ::tet_infoline("The constant ::Dates has incorrect value");
    strcpy(buf1,StringToString("\012B"));
    ::tet_printf("Expected %s, but found %s",buf1,
		 StringToString((char*)::Dates));
    ::tet_result(TET_FAIL);
  }
::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerPumpkin(void)
{
  
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Pumpkin()");
#endif
  
  ernie.seed("PumpkinCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PumpkinReturn");
  try {
     tobj->Pumpkin();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Pumpkin");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


