//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 16:54:24  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:15:51  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.6  1999/04/26 13:49:59  james
// Changed CORBA::Char* to char*
//
// Revision 1.5  1998/08/24 11:45:39  james
// ENV updates
//
// Revision 1.4  1997/12/09 11:25:17  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

  if (::Cucumber != TRUE)
  {
    ::tet_infoline("The constant ::Cucumber has incorrect value");
    ::tet_printf("Expected TRUE, but found %s",BooleanToString(::Cucumber));
    ::tet_result(TET_FAIL);
  }
  if (::Ginger != FALSE)
  {
    ::tet_infoline("The constant ::Ginger has incorrect value");
    ::tet_printf("Expected FALSE, but found %s",BooleanToString(::Ginger));
    ::tet_result(TET_FAIL);
  }
::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerMint(void)
{
  
  char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function char * ::idlServer::Mint()");
#endif
  
  ernie.seed("MintCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MintReturn");
  try {
    ReturnValue = tobj->Mint();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Mint");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGardenPeas(void)
{
  
  ::BlackEyedBeans* p1;
  ::BlackEyedBeans p2;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::GardenPeas(::BlackEyedBeans *&p1,::BlackEyedBeans &p2)");
#endif
  
  ernie.seed("GardenPeasCall");
  GetBlackEyedBeans(p2);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GardenPeasReturn");
  try {
     tobj->GardenPeas(p1,p2);
  }
  catch (::idlServer::Asparagus &ex){
    if (!CheckidlServerAsparagus(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GardenPeas");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("p1");
  if (!CheckBlackEyedBeans(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBlackEyedBeans(p2))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBanana(void)
{
  
  ::BlackEyedBeans p1;
  CORBA::Context_ptr ctx;
  CORBA::Any ctxAny;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Banana(const ::BlackEyedBeans &p1,CORBA::Context_ptr ctx)");
#endif
  
  ernie.seed("BananaCall");
  GetBlackEyedBeans(p1);
  orb->orb()->get_default_context(ctx);
  ctxAny <<= GetString();
  ctx->set_one_value("Orange",ctxAny);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BananaReturn");
  try {
     tobj->Banana(p1,ctx);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Banana");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


