//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 16:56:33  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:15:57  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.6  1997/12/10 20:20:57  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testApplePeanuts(void)
{
  
  ::Banana_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Banana_ptr ::Apple::Peanuts()");
#endif
  
  ernie.seed("PeanutsCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PeanutsReturn");
  try {
    ReturnValue = tobj->Peanuts();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::Apple::Peanuts");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!Checkobject(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testApplePeanutsW(void)
{
  
  ::Banana_ptr _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::Apple::Peanuts(::Banana_ptr _attr)");
#endif
  
  ernie.seed("PeanutsCall");
  _attr = Banana::_narrow(Getobject());

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PeanutsReturn");
  try {
     tobj->Peanuts(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::Apple::Peanuts");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testBananaOrange(void)
{
  
  ::Banana_ptr p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::Banana::Orange(::Banana_ptr p1)");
#endif
  
  ernie.seed("OrangeCall");
  p1 = Banana::_narrow(Getobject());

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("OrangeReturn");
  try {
     tobj->Orange(p1);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::Banana::Orange");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testBananaBasil(void)
{
  
  ::Banana_ptr p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::Banana::Basil(::Banana_ptr &p1)");
#endif
  
  ernie.seed("BasilCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BasilReturn");
  try {
     tobj->Basil(p1);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::Banana::Basil");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("p1");
  if (!Checkobject(p1))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBananaVineTomatoes(void)
{
  
  ::Banana_ptr p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::Banana::VineTomatoes(::Banana_ptr &p1)");
#endif
  
  ernie.seed("VineTomatoesCall");
  p1 = Banana::_narrow(Getobject());

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("VineTomatoesReturn");
  try {
     tobj->VineTomatoes(p1);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::Banana::VineTomatoes");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("p1");
  if (!Checkobject(p1))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLemon(void)
{
  
  ::idlServer_ptr ReturnValue;
  ::Corn p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::idlServer_ptr ::idlServer::Lemon(const ::Corn &p1)");
#endif
  
  ernie.seed("LemonCall");
  GetCorn(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LemonReturn");
  try {
    ReturnValue = tobj->Lemon(p1);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Lemon");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!Checkobject(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


