//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:00:32  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:15:59  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.5  1997/12/09 11:25:49  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

  if (::Apple::Lemon != 3)
  {
    ::tet_infoline("The constant ::Apple::Lemon has incorrect value");
    ::tet_printf("Expected 3, but found %d",::Apple::Lemon);
    ::tet_result(TET_FAIL);
  }
  if (::Banana::Lemon != 3)
  {
    ::tet_infoline("The constant ::Banana::Lemon has incorrect value");
    ::tet_printf("Expected 3, but found %d",::Banana::Lemon);
    ::tet_result(TET_FAIL);
  }
  if (!CompareFloats(::Carrots::Lemon,4.5))
  {
    ::tet_infoline("The constant ::Carrots::Lemon has incorrect value");
    ::tet_printf("Expected %g, but found %g",(float)4.5,(float)::Carrots::Lemon);
    ::tet_result(TET_FAIL);
  }
  if (::Dates::Lemon != (CORBA::Char)54)
  {
    ::tet_infoline("The constant ::Dates::Lemon has incorrect value");
    strcpy(buf1,CharToString((unsigned char)54));
    ::tet_printf("Expected %s, but found %s", buf1,CharToString(::Dates::Lemon));
    ::tet_result(TET_FAIL);
  }
::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerLemon(void)
{
  
  ::idlServer::Melon p1;
  ::Apple::Pumpkin p2;
  ::Banana::Pumpkin p3;
  ::Carrots::Pumpkin p4;
  ::Dates::Pumpkin p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Lemon(const ::idlServer::Melon p1,::Apple::Pumpkin &p2,::Banana::Pumpkin &p3,::Carrots::Pumpkin &p4,::Dates::Pumpkin &p5)");
#endif
  
  ernie.seed("LemonCall");
  GetidlServerMelon(p1);
  GetApplePumpkin(p2);
  GetBananaPumpkin(p3);
  GetCarrotsPumpkin(p4);
  GetDatesPumpkin(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LemonReturn");
  try {
     tobj->Lemon(p1,p2,p3,p4,p5);
  }
  catch (::Carrots::Oregano &ex){
    if (!CheckCarrotsOregano(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Lemon");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("p2");
  if (!CheckApplePumpkin(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBananaPumpkin(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p4");
  if (!CheckCarrotsPumpkin(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p5");
  if (!CheckDatesPumpkin(p5))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


