//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:01:24  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:02  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.7  1999/04/26 13:50:23  james
// Changed CORBA::Char* to char*
//
// Revision 1.6  1998/08/24 11:45:56  james
// ENV updates
//
// Revision 1.5  1997/12/09 11:26:00  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerBayleaf(void)
{
  
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Bayleaf()");
#endif
  
  ernie.seed("BayleafCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BayleafReturn");
  try {
     tobj->Bayleaf();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Bayleaf");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBlackCherry(void)
{
  
  CORBA::Float ReturnValue;
  CORBA::Float p1;
  CORBA::Float p2;
  CORBA::Float p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Float ::idlServer::BlackCherry(CORBA::Float p1,CORBA::Float &p2,CORBA::Float &p3)");
#endif
  
  ernie.seed("BlackCherryCall");
  GetFloat(p1);
  GetFloat(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackCherryReturn");
  try {
    ReturnValue = tobj->BlackCherry(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BlackCherry");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFloat(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckFloat(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckFloat(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerStarfruit(void)
{
  
  CORBA::Double ReturnValue;
  CORBA::Double p1;
  CORBA::Double p2;
  CORBA::Double p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Double ::idlServer::Starfruit(CORBA::Double p1,CORBA::Double &p2,CORBA::Double &p3)");
#endif
  
  ernie.seed("StarfruitCall");
  GetDouble(p1);
  GetDouble(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("StarfruitReturn");
  try {
    ReturnValue = tobj->Starfruit(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Starfruit");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDouble(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckDouble(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckDouble(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRhubarb(void)
{
  
  CORBA::Long ReturnValue;
  CORBA::Long p1;
  CORBA::Long p2;
  CORBA::Long p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::idlServer::Rhubarb(CORBA::Long p1,CORBA::Long &p2,CORBA::Long &p3)");
#endif
  
  ernie.seed("RhubarbCall");
  GetLong(p1);
  GetLong(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RhubarbReturn");
  try {
    ReturnValue = tobj->Rhubarb(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Rhubarb");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckLong(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckLong(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPassionFruit(void)
{
  
  CORBA::Short ReturnValue;
  CORBA::Short p1;
  CORBA::Short p2;
  CORBA::Short p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Short ::idlServer::PassionFruit(CORBA::Short p1,CORBA::Short &p2,CORBA::Short &p3)");
#endif
  
  ernie.seed("PassionFruitCall");
  GetShort(p1);
  GetShort(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PassionFruitReturn");
  try {
    ReturnValue = tobj->PassionFruit(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PassionFruit");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckShort(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckShort(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPumpkin(void)
{
  
  CORBA::ULong ReturnValue;
  CORBA::ULong p1;
  CORBA::ULong p2;
  CORBA::ULong p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::ULong ::idlServer::Pumpkin(CORBA::ULong p1,CORBA::ULong &p2,CORBA::ULong &p3)");
#endif
  
  ernie.seed("PumpkinCall");
  GetULong(p1);
  GetULong(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PumpkinReturn");
  try {
    ReturnValue = tobj->Pumpkin(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Pumpkin");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckULong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckULong(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckULong(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMango(void)
{
  
  CORBA::UShort ReturnValue;
  CORBA::UShort p1;
  CORBA::UShort p2;
  CORBA::UShort p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::idlServer::Mango(CORBA::UShort p1,CORBA::UShort &p2,CORBA::UShort &p3)");
#endif
  
  ernie.seed("MangoCall");
  GetUShort(p1);
  GetUShort(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MangoReturn");
  try {
    ReturnValue = tobj->Mango(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Mango");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckUShort(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckUShort(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerWaterMelon(void)
{
  
  CORBA::Char ReturnValue;
  CORBA::Char p1;
  CORBA::Char p2;
  CORBA::Char p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char ::idlServer::WaterMelon(CORBA::Char p1,CORBA::Char &p2,CORBA::Char &p3)");
#endif
  
  ernie.seed("WaterMelonCall");
  GetChar(p1);
  GetChar(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("WaterMelonReturn");
  try {
    ReturnValue = tobj->WaterMelon(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::WaterMelon");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChar(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckChar(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckChar(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLime(void)
{
  
  CORBA::Boolean ReturnValue;
  CORBA::Boolean p1;
  CORBA::Boolean p2;
  CORBA::Boolean p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Boolean ::idlServer::Lime(CORBA::Boolean p1,CORBA::Boolean &p2,CORBA::Boolean &p3)");
#endif
  
  ernie.seed("LimeCall");
  GetBoolean(p1);
  GetBoolean(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LimeReturn");
  try {
    ReturnValue = tobj->Lime(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Lime");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBoolean(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBoolean(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBoolean(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerDates(void)
{
  
  CORBA::Octet ReturnValue;
  CORBA::Octet p1;
  CORBA::Octet p2;
  CORBA::Octet p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Octet ::idlServer::Dates(CORBA::Octet p1,CORBA::Octet &p2,CORBA::Octet &p3)");
#endif
  
  ernie.seed("DatesCall");
  GetOctet(p1);
  GetOctet(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DatesReturn");
  try {
    ReturnValue = tobj->Dates(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Dates");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOctet(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckOctet(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckOctet(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCoconut(void)
{
  
  CORBA::Object_ptr ReturnValue;
  CORBA::Object_ptr p1;
  CORBA::Object_ptr p2;
  CORBA::Object_ptr p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Object_ptr ::idlServer::Coconut(CORBA::Object_ptr p1,CORBA::Object_ptr &p2,CORBA::Object_ptr &p3)");
#endif
  
  ernie.seed("CoconutCall");
  p1 = Getobject();
  p3 = Getobject();

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CoconutReturn");
  try {
    ReturnValue = tobj->Coconut(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Coconut");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!Checkobject(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!Checkobject(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!Checkobject(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLychees(void)
{
  
  char * ReturnValue;
  char * p1;
  char * p2;
  char * p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function char * ::idlServer::Lychees(const char * p1,char * &p2,char * &p3)");
#endif
  
  ernie.seed("LycheesCall");
  p1 = GetString();
  p3 = GetString();

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LycheesReturn");
  try {
    ReturnValue = tobj->Lychees(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Lychees");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckString(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckString(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCabbage(void)
{
  
  char * ReturnValue;
  char * p1;
  char * p2;
  char * p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function char * ::idlServer::Cabbage(const char * p1,char * &p2,char * &p3)");
#endif
  
  ernie.seed("CabbageCall");
  p1 = GetString(23);
  p3 = GetString(44);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CabbageReturn");
  try {
    ReturnValue = tobj->Cabbage(p1,p2,p3);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Cabbage");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue,20))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckString(p2,38))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckString(p3,44))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


