//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:02:35  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:08  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.7  1998/08/24 11:46:03  james
// ENV updates
//
// Revision 1.6  1997/12/11 16:26:33  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerJuniperBerries(void)
{
  
  CORBA::Long p1;
  
  ::Sweetcorn p2;
  CORBA::Octet p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::JuniperBerries(CORBA::Long p1,::Sweetcorn &p2,CORBA::Octet &p3)");
#endif
  
  ernie.seed("JuniperBerriesCall");
  GetLong(p1);
  GetOctet(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("JuniperBerriesReturn");
  try {
     tobj->JuniperBerries(p1,p2,p3);
  }
  catch (::Mangetout &ex){
    if (!CheckMangetout(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::JuniperBerries");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("p2");
  if (!CheckSweetcorn(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckOctet(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerDaikan(void)
{
  
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Daikan()");
#endif
  
  ernie.seed("DaikanCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DaikanReturn");
  try {
     tobj->Daikan();
  }
  catch (::Cress &ex){
    if (ernie.getULong() % 2 != 0)
    {
        ::tet_infoline("Unexpected user exception (::Cress) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCress(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::idlServer::RedGrapes &ex){
    if (ernie.getULong() % 2 != 1)
    {
        ::tet_infoline("Unexpected user exception (::idlServer::RedGrapes) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckidlServerRedGrapes(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Daikan");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerThyme(void)
{
  
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Thyme()");
#endif
  
  ernie.seed("ThymeCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ThymeReturn");
  try {
     tobj->Thyme();
  }
  catch (::Cress &ex){
    if (ernie.getULong() % 2 != 0)
    {
        ::tet_infoline("Unexpected user exception (::Cress) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCress(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::idlServer::RedGrapes &ex){
    if (ernie.getULong() % 2 != 1)
    {
        ::tet_infoline("Unexpected user exception (::idlServer::RedGrapes) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckidlServerRedGrapes(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Thyme");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLettuce(void)
{
  
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Lettuce()");
#endif
  
  ernie.seed("LettuceCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LettuceReturn");
  try {
     tobj->Lettuce();
  }
  catch (::Mangetout &ex){
    if (ernie.getULong() % 6 != 0)
    {
        ::tet_infoline("Unexpected user exception (::Mangetout) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckMangetout(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Cress &ex){
    if (ernie.getULong() % 6 != 1)
    {
        ::tet_infoline("Unexpected user exception (::Cress) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCress(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::BambooShoot &ex){
    if (ernie.getULong() % 6 != 2)
    {
        ::tet_infoline("Unexpected user exception (::BambooShoot) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckBambooShoot(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Cassava &ex){
    if (ernie.getULong() % 6 != 3)
    {
        ::tet_infoline("Unexpected user exception (::Cassava) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCassava(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Melon &ex){
    if (ernie.getULong() % 6 != 4)
    {
        ::tet_infoline("Unexpected user exception (::Melon) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckMelon(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::idlServer::RedGrapes &ex){
    if (ernie.getULong() % 6 != 5)
    {
        ::tet_infoline("Unexpected user exception (::idlServer::RedGrapes) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckidlServerRedGrapes(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Lettuce");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


