//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:03:02  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:10  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.7  1999/04/26 13:50:43  james
// Changed CORBA::Char* to char*
//
// Revision 1.6  1998/08/24 11:46:08  james
// ENV updates
//
// Revision 1.5  1997/12/11 17:17:33  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
siiServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = siiServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testsiiServerCucumber(void)
{
  
  CORBA::Float ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Float ::siiServer::Cucumber()");
#endif
  
  ernie.seed("CucumberCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CucumberReturn");
  try {
    ReturnValue = tobj->Cucumber();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Cucumber");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFloat(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerCucumberW(void)
{
  
  CORBA::Float _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Cucumber(CORBA::Float _attr)");
#endif
  
  ernie.seed("CucumberCall");
  GetFloat(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CucumberReturn");
  try {
     tobj->Cucumber(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Cucumber");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGinger(void)
{
  
  CORBA::Double ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Double ::siiServer::Ginger()");
#endif
  
  ernie.seed("GingerCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GingerReturn");
  try {
    ReturnValue = tobj->Ginger();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Ginger");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDouble(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGingerW(void)
{
  
  CORBA::Double _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Ginger(CORBA::Double _attr)");
#endif
  
  ernie.seed("GingerCall");
  GetDouble(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GingerReturn");
  try {
     tobj->Ginger(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Ginger");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerApricots(void)
{
  
  CORBA::Long ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::siiServer::Apricots()");
#endif
  
  ernie.seed("ApricotsCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ApricotsReturn");
  try {
    ReturnValue = tobj->Apricots();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Apricots");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerApricotsW(void)
{
  
  CORBA::Long _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Apricots(CORBA::Long _attr)");
#endif
  
  ernie.seed("ApricotsCall");
  GetLong(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ApricotsReturn");
  try {
     tobj->Apricots(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Apricots");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerBlackEyedBeans(void)
{
  
  CORBA::ULong ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::ULong ::siiServer::BlackEyedBeans()");
#endif
  
  ernie.seed("BlackEyedBeansCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackEyedBeansReturn");
  try {
    ReturnValue = tobj->BlackEyedBeans();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::BlackEyedBeans");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckULong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerBlackEyedBeansW(void)
{
  
  CORBA::ULong _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::BlackEyedBeans(CORBA::ULong _attr)");
#endif
  
  ernie.seed("BlackEyedBeansCall");
  GetULong(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackEyedBeansReturn");
  try {
     tobj->BlackEyedBeans(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::BlackEyedBeans");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerLeek(void)
{
  
  CORBA::Short ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Short ::siiServer::Leek()");
#endif
  
  ernie.seed("LeekCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LeekReturn");
  try {
    ReturnValue = tobj->Leek();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Leek");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerLeekW(void)
{
  
  CORBA::Short _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Leek(CORBA::Short _attr)");
#endif
  
  ernie.seed("LeekCall");
  GetShort(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LeekReturn");
  try {
     tobj->Leek(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Leek");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerMint(void)
{
  
  CORBA::UShort ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::siiServer::Mint()");
#endif
  
  ernie.seed("MintCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MintReturn");
  try {
    ReturnValue = tobj->Mint();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Mint");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerMintW(void)
{
  
  CORBA::UShort _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Mint(CORBA::UShort _attr)");
#endif
  
  ernie.seed("MintCall");
  GetUShort(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MintReturn");
  try {
     tobj->Mint(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Mint");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGardenPeas(void)
{
  
  CORBA::Char ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char ::siiServer::GardenPeas()");
#endif
  
  ernie.seed("GardenPeasCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GardenPeasReturn");
  try {
    ReturnValue = tobj->GardenPeas();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::GardenPeas");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChar(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGardenPeasW(void)
{
  
  CORBA::Char _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::GardenPeas(CORBA::Char _attr)");
#endif
  
  ernie.seed("GardenPeasCall");
  GetChar(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GardenPeasReturn");
  try {
     tobj->GardenPeas(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::GardenPeas");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerFig(void)
{
  
  CORBA::Boolean ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Boolean ::siiServer::Fig()");
#endif
  
  ernie.seed("FigCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("FigReturn");
  try {
    ReturnValue = tobj->Fig();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Fig");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBoolean(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerFigW(void)
{
  
  CORBA::Boolean _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Fig(CORBA::Boolean _attr)");
#endif
  
  ernie.seed("FigCall");
  GetBoolean(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("FigReturn");
  try {
     tobj->Fig(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Fig");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerAsparagus(void)
{
  
  CORBA::Octet ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Octet ::siiServer::Asparagus()");
#endif
  
  ernie.seed("AsparagusCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AsparagusReturn");
  try {
    ReturnValue = tobj->Asparagus();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Asparagus");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOctet(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerAsparagusW(void)
{
  
  CORBA::Octet _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Asparagus(CORBA::Octet _attr)");
#endif
  
  ernie.seed("AsparagusCall");
  GetOctet(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AsparagusReturn");
  try {
     tobj->Asparagus(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Asparagus");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGaliaMelon(void)
{
  
  char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function char * ::siiServer::GaliaMelon()");
#endif
  
  ernie.seed("GaliaMelonCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GaliaMelonReturn");
  try {
    ReturnValue = tobj->GaliaMelon();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::GaliaMelon");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGaliaMelonW(void)
{
  
  char * _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::GaliaMelon(const char * _attr)");
#endif
  
  ernie.seed("GaliaMelonCall");
  _attr = GetString();

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GaliaMelonReturn");
  try {
     tobj->GaliaMelon(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::GaliaMelon");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerSultana(void)
{
  
  char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function char * ::siiServer::Sultana()");
#endif
  
  ernie.seed("SultanaCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SultanaReturn");
  try {
    ReturnValue = tobj->Sultana();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Sultana");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue,23))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerSultanaW(void)
{
  
  char * _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Sultana(const char * _attr)");
#endif
  
  ernie.seed("SultanaCall");
  _attr = GetString(23);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SultanaReturn");
  try {
     tobj->Sultana(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Sultana");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerTurnip(void)
{
  
  CORBA::Object_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Object_ptr ::siiServer::Turnip()");
#endif
  
  ernie.seed("TurnipCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TurnipReturn");
  try {
    ReturnValue = tobj->Turnip();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Turnip");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!Checkobject(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerTurnipW(void)
{
  
  CORBA::Object_ptr _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Turnip(CORBA::Object_ptr _attr)");
#endif
  
  ernie.seed("TurnipCall");
  _attr = Getobject();

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TurnipReturn");
  try {
     tobj->Turnip(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Turnip");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerOrange(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::siiServer::Orange()");
#endif
  
  ernie.seed("OrangeCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("OrangeReturn");
  try {
    ReturnValue = tobj->Orange();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Orange");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerOrangeW(void)
{
  
  CORBA::Any _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Orange(const CORBA::Any &_attr)");
#endif
  
  ernie.seed("OrangeCall");
  GetAny(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("OrangeReturn");
  try {
     tobj->Orange(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Orange");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerLoganBerry(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::siiServer::LoganBerry()");
#endif
  
  ernie.seed("LoganBerryCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerryReturn");
  try {
    ReturnValue = tobj->LoganBerry();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::LoganBerry");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerLoganBerryW(void)
{
  
  CORBA::TypeCode_ptr _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::LoganBerry(CORBA::TypeCode_ptr _attr)");
#endif
  
  ernie.seed("LoganBerryCall");
  GetCORBATypeCode_ptr(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerryReturn");
  try {
     tobj->LoganBerry(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::LoganBerry");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerParsnip(void)
{
  
  CORBA::Float ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Float ::siiServer::Parsnip()");
#endif
  
  ernie.seed("ParsnipCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ParsnipReturn");
  try {
    ReturnValue = tobj->Parsnip();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Parsnip");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFloat(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerCelery(void)
{
  
  CORBA::Double ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Double ::siiServer::Celery()");
#endif
  
  ernie.seed("CeleryCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CeleryReturn");
  try {
    ReturnValue = tobj->Celery();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Celery");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDouble(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerWhiteCabbage(void)
{
  
  CORBA::Long ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::siiServer::WhiteCabbage()");
#endif
  
  ernie.seed("WhiteCabbageCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("WhiteCabbageReturn");
  try {
    ReturnValue = tobj->WhiteCabbage();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::WhiteCabbage");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerAniseed(void)
{
  
  CORBA::ULong ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::ULong ::siiServer::Aniseed()");
#endif
  
  ernie.seed("AniseedCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AniseedReturn");
  try {
    ReturnValue = tobj->Aniseed();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Aniseed");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckULong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerBayleaf(void)
{
  
  CORBA::Short ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Short ::siiServer::Bayleaf()");
#endif
  
  ernie.seed("BayleafCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BayleafReturn");
  try {
    ReturnValue = tobj->Bayleaf();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Bayleaf");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerDill(void)
{
  
  CORBA::UShort ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::siiServer::Dill()");
#endif
  
  ernie.seed("DillCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DillReturn");
  try {
    ReturnValue = tobj->Dill();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Dill");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerOregano(void)
{
  
  CORBA::Char ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char ::siiServer::Oregano()");
#endif
  
  ernie.seed("OreganoCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("OreganoReturn");
  try {
    ReturnValue = tobj->Oregano();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Oregano");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChar(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerBeansprout(void)
{
  
  CORBA::Boolean ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Boolean ::siiServer::Beansprout()");
#endif
  
  ernie.seed("BeansproutCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BeansproutReturn");
  try {
    ReturnValue = tobj->Beansprout();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Beansprout");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBoolean(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerAlmonds(void)
{
  
  CORBA::Octet ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Octet ::siiServer::Almonds()");
#endif
  
  ernie.seed("AlmondsCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AlmondsReturn");
  try {
    ReturnValue = tobj->Almonds();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Almonds");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOctet(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGuava(void)
{
  
  char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function char * ::siiServer::Guava()");
#endif
  
  ernie.seed("GuavaCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GuavaReturn");
  try {
    ReturnValue = tobj->Guava();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Guava");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerMushroom(void)
{
  
  char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function char * ::siiServer::Mushroom()");
#endif
  
  ernie.seed("MushroomCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MushroomReturn");
  try {
    ReturnValue = tobj->Mushroom();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Mushroom");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue,25))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerRunnerbean(void)
{
  
  CORBA::Object_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Object_ptr ::siiServer::Runnerbean()");
#endif
  
  ernie.seed("RunnerbeanCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RunnerbeanReturn");
  try {
    ReturnValue = tobj->Runnerbean();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Runnerbean");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!Checkobject(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerTofu(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::siiServer::Tofu()");
#endif
  
  ernie.seed("TofuCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TofuReturn");
  try {
    ReturnValue = tobj->Tofu();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Tofu");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerPrunes(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::siiServer::Prunes()");
#endif
  
  ernie.seed("PrunesCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PrunesReturn");
  try {
    ReturnValue = tobj->Prunes();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Prunes");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGrannySmith(void)
{
  
  
  ::EggPlant ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::EggPlant ::siiServer::GrannySmith()");
#endif
  
  ernie.seed("GrannySmithCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GrannySmithReturn");
  try {
    ReturnValue = tobj->GrannySmith();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::GrannySmith");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckEggPlant(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGrannySmithW(void)
{
  
  
  ::EggPlant _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::GrannySmith(::EggPlant _attr)");
#endif
  
  ernie.seed("GrannySmithCall");
  GetEggPlant(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GrannySmithReturn");
  try {
     tobj->GrannySmith(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::GrannySmith");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerSweetPotato(void)
{
  
  ::Plum_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Plum_slice * ::siiServer::SweetPotato()");
#endif
  
  ernie.seed("SweetPotatoCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetPotatoReturn");
  try {
    ReturnValue = tobj->SweetPotato();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::SweetPotato");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPlum(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerSweetPotatoW(void)
{
  
  ::Plum _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::SweetPotato(const ::Plum _attr)");
#endif
  
  ernie.seed("SweetPotatoCall");
  GetPlum(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetPotatoReturn");
  try {
     tobj->SweetPotato(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::SweetPotato");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerBillberries(void)
{
  
  ::Zucchini ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Zucchini ::siiServer::Billberries()");
#endif
  
  ernie.seed("BillberriesCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BillberriesReturn");
  try {
    ReturnValue = tobj->Billberries();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Billberries");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckZucchini(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerBillberriesW(void)
{
  
  ::Zucchini _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Billberries(const ::Zucchini &_attr)");
#endif
  
  ernie.seed("BillberriesCall");
  GetZucchini(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BillberriesReturn");
  try {
     tobj->Billberries(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Billberries");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerLolloRossa(void)
{
  
  ::Avocado ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado ::siiServer::LolloRossa()");
#endif
  
  ernie.seed("LolloRossaCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LolloRossaReturn");
  try {
    ReturnValue = tobj->LolloRossa();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::LolloRossa");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocado(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerLolloRossaW(void)
{
  
  ::Avocado _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::LolloRossa(const ::Avocado &_attr)");
#endif
  
  ernie.seed("LolloRossaCall");
  GetAvocado(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LolloRossaReturn");
  try {
     tobj->LolloRossa(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::LolloRossa");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerSweetcorn(void)
{
  
  
  ::Lentil ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lentil ::siiServer::Sweetcorn()");
#endif
  
  ernie.seed("SweetcornCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetcornReturn");
  try {
    ReturnValue = tobj->Sweetcorn();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Sweetcorn");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLentil(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerSweetcornW(void)
{
  
  
  ::Lentil _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Sweetcorn(const ::Lentil &_attr)");
#endif
  
  ernie.seed("SweetcornCall");
  GetLentil(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetcornReturn");
  try {
     tobj->Sweetcorn(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Sweetcorn");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerShallots(void)
{
  
  
  ::EggPlant ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::EggPlant ::siiServer::Shallots()");
#endif
  
  ernie.seed("ShallotsCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ShallotsReturn");
  try {
    ReturnValue = tobj->Shallots();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Shallots");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckEggPlant(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerKidneyBean(void)
{
  
  ::Plum_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Plum_slice * ::siiServer::KidneyBean()");
#endif
  
  ernie.seed("KidneyBeanCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KidneyBeanReturn");
  try {
    ReturnValue = tobj->KidneyBean();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::KidneyBean");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPlum(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerButterBean(void)
{
  
  ::Zucchini ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Zucchini ::siiServer::ButterBean()");
#endif
  
  ernie.seed("ButterBeanCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ButterBeanReturn");
  try {
    ReturnValue = tobj->ButterBean();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::ButterBean");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckZucchini(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGoldenDelicous(void)
{
  
  ::Avocado ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado ::siiServer::GoldenDelicous()");
#endif
  
  ernie.seed("GoldenDelicousCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GoldenDelicousReturn");
  try {
    ReturnValue = tobj->GoldenDelicous();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::GoldenDelicous");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocado(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerCrabApple(void)
{
  
  
  ::Lentil ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lentil ::siiServer::CrabApple()");
#endif
  
  ernie.seed("CrabAppleCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CrabAppleReturn");
  try {
    ReturnValue = tobj->CrabApple();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::CrabApple");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLentil(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerRedOnion(void)
{
  
  CORBA::Long ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::siiServer::RedOnion()");
#endif
  
  ernie.seed("RedOnionCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedOnionReturn");
  try {
    ReturnValue = tobj->RedOnion();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::RedOnion");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerRedOnionW(void)
{
  
  CORBA::Long _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::RedOnion(CORBA::Long _attr)");
#endif
  
  ernie.seed("RedOnionCall");
  GetLong(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedOnionReturn");
  try {
     tobj->RedOnion(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::RedOnion");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerHazelNut(void)
{
  
  CORBA::Long ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::siiServer::HazelNut()");
#endif
  
  ernie.seed("HazelNutCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("HazelNutReturn");
  try {
    ReturnValue = tobj->HazelNut();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::HazelNut");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerHazelNutW(void)
{
  
  CORBA::Long _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::HazelNut(CORBA::Long _attr)");
#endif
  
  ernie.seed("HazelNutCall");
  GetLong(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("HazelNutReturn");
  try {
     tobj->HazelNut(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::HazelNut");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerCinnamon(void)
{
  
  CORBA::Long ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::siiServer::Cinnamon()");
#endif
  
  ernie.seed("CinnamonCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CinnamonReturn");
  try {
    ReturnValue = tobj->Cinnamon();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Cinnamon");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerCinnamonW(void)
{
  
  CORBA::Long _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Cinnamon(CORBA::Long _attr)");
#endif
  
  ernie.seed("CinnamonCall");
  GetLong(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CinnamonReturn");
  try {
     tobj->Cinnamon(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Cinnamon");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGooseBerry(void)
{
  
  CORBA::Long ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::siiServer::GooseBerry()");
#endif
  
  ernie.seed("GooseBerryCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GooseBerryReturn");
  try {
    ReturnValue = tobj->GooseBerry();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::GooseBerry");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerGooseBerryW(void)
{
  
  CORBA::Long _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::GooseBerry(CORBA::Long _attr)");
#endif
  
  ernie.seed("GooseBerryCall");
  GetLong(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GooseBerryReturn");
  try {
     tobj->GooseBerry(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::GooseBerry");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerJuniperBerries(void)
{
  
  
  ::EggPlant ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::EggPlant ::siiServer::JuniperBerries()");
#endif
  
  ernie.seed("JuniperBerriesCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("JuniperBerriesReturn");
  try {
    ReturnValue = tobj->JuniperBerries();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::JuniperBerries");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckEggPlant(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerJuniperBerriesW(void)
{
  
  
  ::EggPlant _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::JuniperBerries(::EggPlant _attr)");
#endif
  
  ernie.seed("JuniperBerriesCall");
  GetEggPlant(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("JuniperBerriesReturn");
  try {
     tobj->JuniperBerries(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::JuniperBerries");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerCloves(void)
{
  
  
  ::EggPlant ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::EggPlant ::siiServer::Cloves()");
#endif
  
  ernie.seed("ClovesCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ClovesReturn");
  try {
    ReturnValue = tobj->Cloves();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Cloves");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckEggPlant(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerClovesW(void)
{
  
  
  ::EggPlant _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Cloves(::EggPlant _attr)");
#endif
  
  ernie.seed("ClovesCall");
  GetEggPlant(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ClovesReturn");
  try {
     tobj->Cloves(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Cloves");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerDaikan(void)
{
  
  
  ::EggPlant ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::EggPlant ::siiServer::Daikan()");
#endif
  
  ernie.seed("DaikanCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DaikanReturn");
  try {
    ReturnValue = tobj->Daikan();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Daikan");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckEggPlant(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerDaikanW(void)
{
  
  
  ::EggPlant _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Daikan(::EggPlant _attr)");
#endif
  
  ernie.seed("DaikanCall");
  GetEggPlant(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DaikanReturn");
  try {
     tobj->Daikan(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Daikan");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerMustard(void)
{
  
  
  ::EggPlant ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::EggPlant ::siiServer::Mustard()");
#endif
  
  ernie.seed("MustardCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MustardReturn");
  try {
    ReturnValue = tobj->Mustard();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Mustard");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckEggPlant(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testsiiServerMustardW(void)
{
  
  
  ::EggPlant _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::siiServer::Mustard(::EggPlant _attr)");
#endif
  
  ernie.seed("MustardCall");
  GetEggPlant(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MustardReturn");
  try {
     tobj->Mustard(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::siiServer::Mustard");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


