//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/cmain.cpp
//
// Modifications:
// $Log: cmain.cpp,v $
// Revision 1.2  2000/03/28 18:19:38  mli
//   o   remove 'CORBA::Environment ev;'
//   o   void (*tet_startup)(void) -> void (*tet_startup)()
//   o   void (*tet_cleanup)(void) -> void (*tet_cleanup)()
//
// Revision 1.1.1.1  2000/02/09 17:16:11  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.8  1998/08/24 11:46:09  james
// ENV updates
//
// Revision 1.7  1997/12/11 17:17:34  james
// Code Regenerated
//
//
//

#include <orb_supp.hh>
#include <syncinfo.hh>
#include <stdlib.h>

#ifndef WIN32
#include <unistd.h>
#else
#include <process.h>
#endif

extern "C" {
#include <tet_api.h>
}

#include "cmain.hh"

//
// This global defines the systems involved in a TET sync event
// Currently the client sync with the proxy to receive the IOR
//
int remSystem[] = { CSW };	

// Our handle to the ORB under test
OrbSupport *orb;

// The stringified IOR passed to us by the proxy
char *stringifiedIOR = 0;

void
clientStartUp()
{
  // Initialise the ORB under test


  try {
      orb = new OrbSupport();
      orb->client_init();
  }
  catch (CORBA::Exception &ex) {
      tet_infoline("client: failed to initialise Orb");
      return;
  }
  
  // Provide space to receive proxified IOR
  struct tet_synmsg msg;
  char rcvbuf[2048];
  
  // Indicate receive mode to TET
  msg.tsm_flags = TET_SMRCVMSG;
  msg.tsm_dlen = sizeof(rcvbuf);
  msg.tsm_data = rcvbuf;
  
  if (tet_remsync(CEVENT, remSystem, 1, VS_TIMEOUT, TET_SV_YES, &msg) != 0)
  {
      tet_printf("client: sync point %d failed", CEVENT);
      return;
  }

  // Cache stringified IOR for method calls to use
  stringifiedIOR = strdup(rcvbuf);

#ifdef DEBUG  
  tet_printf("client: sync point %d OK", CEVENT);
#endif
}

void
clientCleanUp()
{
    delete orb;
    return;
}

#ifdef VSORB_IDL_TESTS
 
extern "C" {
void (*tet_startup)() = 0,
(*tet_cleanup)() = 0;
struct tet_testlist tet_testlist[] = {
      {testIDL,1},
      {NULL,0} };
}

#else

extern "C" {
  void (*tet_startup)() = clientStartUp,
  (*tet_cleanup)() = clientCleanUp;
  struct tet_testlist tet_testlist[] = {
    {testsiiServerCucumber,1},
    {testsiiServerCucumberW,2},
    {testsiiServerGinger,3},
    {testsiiServerGingerW,4},
    {testsiiServerApricots,5},
    {testsiiServerApricotsW,6},
    {testsiiServerBlackEyedBeans,7},
    {testsiiServerBlackEyedBeansW,8},
    {testsiiServerLeek,9},
    {testsiiServerLeekW,10},
    {testsiiServerMint,11},
    {testsiiServerMintW,12},
    {testsiiServerGardenPeas,13},
    {testsiiServerGardenPeasW,14},
    {testsiiServerFig,15},
    {testsiiServerFigW,16},
    {testsiiServerAsparagus,17},
    {testsiiServerAsparagusW,18},
    {testsiiServerGaliaMelon,19},
    {testsiiServerGaliaMelonW,20},
    {testsiiServerSultana,21},
    {testsiiServerSultanaW,22},
    {testsiiServerTurnip,23},
    {testsiiServerTurnipW,24},
    {testsiiServerOrange,25},
    {testsiiServerOrangeW,26},
    {testsiiServerLoganBerry,27},
    {testsiiServerLoganBerryW,28},
    {testsiiServerParsnip,29},
    {testsiiServerCelery,30},
    {testsiiServerWhiteCabbage,31},
    {testsiiServerAniseed,32},
    {testsiiServerBayleaf,33},
    {testsiiServerDill,34},
    {testsiiServerOregano,35},
    {testsiiServerBeansprout,36},
    {testsiiServerAlmonds,37},
    {testsiiServerGuava,38},
    {testsiiServerMushroom,39},
    {testsiiServerRunnerbean,40},
    {testsiiServerTofu,41},
    {testsiiServerPrunes,42},
    {testsiiServerGrannySmith,43},
    {testsiiServerGrannySmithW,44},
    {testsiiServerSweetPotato,45},
    {testsiiServerSweetPotatoW,46},
    {testsiiServerBillberries,47},
    {testsiiServerBillberriesW,48},
    {testsiiServerLolloRossa,49},
    {testsiiServerLolloRossaW,50},
    {testsiiServerSweetcorn,51},
    {testsiiServerSweetcornW,52},
    {testsiiServerShallots,53},
    {testsiiServerKidneyBean,54},
    {testsiiServerButterBean,55},
    {testsiiServerGoldenDelicous,56},
    {testsiiServerCrabApple,57},
    {testsiiServerRedOnion,58},
    {testsiiServerRedOnionW,59},
    {testsiiServerHazelNut,60},
    {testsiiServerHazelNutW,61},
    {testsiiServerCinnamon,62},
    {testsiiServerCinnamonW,63},
    {testsiiServerGooseBerry,64},
    {testsiiServerGooseBerryW,65},
    {testsiiServerJuniperBerries,66},
    {testsiiServerJuniperBerriesW,67},
    {testsiiServerCloves,68},
    {testsiiServerClovesW,69},
    {testsiiServerDaikan,70},
    {testsiiServerDaikanW,71},
    {testsiiServerMustard,72},
    {testsiiServerMustardW,73},
    {NULL,0} };
}

#endif
