//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:03:23  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:12  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.5  1997/12/11 18:15:38  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testLoganBerryBananaSultana(void)
{
  
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::LoganBerry::Banana::Sultana()");
#endif
  
  ernie.seed("SultanaCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SultanaReturn");
  try {
     tobj->Sultana();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::LoganBerry::Banana::Sultana");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAniseed(void)
{
  
  ::LoganBerry::Parsnip::Celery::WhiteCabbage ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::LoganBerry::Parsnip::Celery::WhiteCabbage ::idlServer::Aniseed()");
#endif
  
  ernie.seed("AniseedCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AniseedReturn");
  try {
    ReturnValue = tobj->Aniseed();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Aniseed");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLoganBerryParsnipCeleryWhiteCabbage(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAniseedW(void)
{
  
  ::LoganBerry::Parsnip::Celery::WhiteCabbage _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Aniseed(const ::LoganBerry::Parsnip::Celery::WhiteCabbage &_attr)");
#endif
  
  ernie.seed("AniseedCall");
  GetLoganBerryParsnipCeleryWhiteCabbage(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AniseedReturn");
  try {
     tobj->Aniseed(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Aniseed");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBayleaf(void)
{
  
  ::LoganBerry::Banana::GaliaMelon p1;
  CORBA::Long Aniseed;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Bayleaf(::LoganBerry::Banana::GaliaMelon p1,CORBA::Long &Aniseed)");
#endif
  
  ernie.seed("BayleafCall");
  GetLoganBerryBananaGaliaMelon(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BayleafReturn");
  try {
     tobj->Bayleaf(p1,Aniseed);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Bayleaf");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("Aniseed");
  if (!CheckLong(Aniseed))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


