//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:03:45  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:14  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:26:29  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerLoganBerryAttr(void)
{
  
  ::LoganBerry* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::LoganBerry* ::idlServer::LoganBerryAttr()");
#endif
  
  ernie.seed("LoganBerryAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerryAttrReturn");
  try {
    ReturnValue = tobj->LoganBerryAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LoganBerryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLoganBerry(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLoganBerryAttrW(void)
{
  
  ::LoganBerry _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::LoganBerryAttr(const ::LoganBerry &_attr)");
#endif
  
  ernie.seed("LoganBerryAttrCall");
  GetLoganBerry(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerryAttrReturn");
  try {
     tobj->LoganBerryAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LoganBerryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLoganBerryOp(void)
{
  
  ::LoganBerry* ReturnValue;
  ::LoganBerry p1;
  ::LoganBerry* p2;
  ::LoganBerry p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::LoganBerry* ::idlServer::LoganBerryOp(const ::LoganBerry &p1,::LoganBerry *&p2,::LoganBerry &p3)");
#endif
  
  ernie.seed("LoganBerryOpCall");
  GetLoganBerry(p1);
  GetLoganBerry(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerryOpReturn");
  try {
    ReturnValue = tobj->LoganBerryOp(p1,p2,p3);
  }
  catch (::idlServer::LoganBerryExcpt &ex){
    if (!CheckidlServerLoganBerryExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LoganBerryOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLoganBerry(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckLoganBerry(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckLoganBerry(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerParsnipAttr(void)
{
  
  ::Parsnip_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Parsnip_slice * ::idlServer::ParsnipAttr()");
#endif
  
  ernie.seed("ParsnipAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ParsnipAttrReturn");
  try {
    ReturnValue = tobj->ParsnipAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ParsnipAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckParsnip(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerParsnipAttrW(void)
{
  
  ::Parsnip _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::ParsnipAttr(const ::Parsnip _attr)");
#endif
  
  ernie.seed("ParsnipAttrCall");
  GetParsnip(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ParsnipAttrReturn");
  try {
     tobj->ParsnipAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ParsnipAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerParsnipOp(void)
{
  
  ::Parsnip_slice * ReturnValue;
  ::Parsnip p1;
  ::Parsnip_slice * p2;
  ::Parsnip p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Parsnip_slice * ::idlServer::ParsnipOp(const ::Parsnip p1,::Parsnip_slice *&p2,::Parsnip p3)");
#endif
  
  ernie.seed("ParsnipOpCall");
  GetParsnip(p1);
  GetParsnip(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ParsnipOpReturn");
  try {
    ReturnValue = tobj->ParsnipOp(p1,p2,p3);
  }
  catch (::idlServer::ParsnipExcpt &ex){
    if (!CheckidlServerParsnipExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ParsnipOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckParsnip(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckParsnip(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckParsnip(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCeleryAttr(void)
{
  
  ::Celery* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Celery* ::idlServer::CeleryAttr()");
#endif
  
  ernie.seed("CeleryAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CeleryAttrReturn");
  try {
    ReturnValue = tobj->CeleryAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CeleryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCelery(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCeleryAttrW(void)
{
  
  ::Celery _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::CeleryAttr(const ::Celery &_attr)");
#endif
  
  ernie.seed("CeleryAttrCall");
  GetCelery(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CeleryAttrReturn");
  try {
     tobj->CeleryAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CeleryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCeleryOp(void)
{
  
  ::Celery* ReturnValue;
  ::Celery p1;
  ::Celery* p2;
  ::Celery p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Celery* ::idlServer::CeleryOp(const ::Celery &p1,::Celery *&p2,::Celery &p3)");
#endif
  
  ernie.seed("CeleryOpCall");
  GetCelery(p1);
  GetCelery(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CeleryOpReturn");
  try {
    ReturnValue = tobj->CeleryOp(p1,p2,p3);
  }
  catch (::idlServer::CeleryExcpt &ex){
    if (!CheckidlServerCeleryExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CeleryOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCelery(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckCelery(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckCelery(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerWhiteCabbageAttr(void)
{
  
  ::WhiteCabbage_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::WhiteCabbage_slice * ::idlServer::WhiteCabbageAttr()");
#endif
  
  ernie.seed("WhiteCabbageAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("WhiteCabbageAttrReturn");
  try {
    ReturnValue = tobj->WhiteCabbageAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::WhiteCabbageAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckWhiteCabbage(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerWhiteCabbageAttrW(void)
{
  
  ::WhiteCabbage _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::WhiteCabbageAttr(const ::WhiteCabbage _attr)");
#endif
  
  ernie.seed("WhiteCabbageAttrCall");
  GetWhiteCabbage(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("WhiteCabbageAttrReturn");
  try {
     tobj->WhiteCabbageAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::WhiteCabbageAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerWhiteCabbageOp(void)
{
  
  ::WhiteCabbage_slice * ReturnValue;
  ::WhiteCabbage p1;
  ::WhiteCabbage_slice * p2;
  ::WhiteCabbage p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::WhiteCabbage_slice * ::idlServer::WhiteCabbageOp(const ::WhiteCabbage p1,::WhiteCabbage_slice *&p2,::WhiteCabbage p3)");
#endif
  
  ernie.seed("WhiteCabbageOpCall");
  GetWhiteCabbage(p1);
  GetWhiteCabbage(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("WhiteCabbageOpReturn");
  try {
    ReturnValue = tobj->WhiteCabbageOp(p1,p2,p3);
  }
  catch (::idlServer::WhiteCabbageExcpt &ex){
    if (!CheckidlServerWhiteCabbageExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::WhiteCabbageOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckWhiteCabbage(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckWhiteCabbage(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckWhiteCabbage(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAniseedAttr(void)
{
  
  ::Aniseed* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Aniseed* ::idlServer::AniseedAttr()");
#endif
  
  ernie.seed("AniseedAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AniseedAttrReturn");
  try {
    ReturnValue = tobj->AniseedAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AniseedAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAniseed(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAniseedAttrW(void)
{
  
  ::Aniseed _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::AniseedAttr(const ::Aniseed &_attr)");
#endif
  
  ernie.seed("AniseedAttrCall");
  GetAniseed(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AniseedAttrReturn");
  try {
     tobj->AniseedAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AniseedAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAniseedOp(void)
{
  
  ::Aniseed* ReturnValue;
  ::Aniseed p1;
  ::Aniseed* p2;
  ::Aniseed p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Aniseed* ::idlServer::AniseedOp(const ::Aniseed &p1,::Aniseed *&p2,::Aniseed &p3)");
#endif
  
  ernie.seed("AniseedOpCall");
  GetAniseed(p1);
  GetAniseed(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AniseedOpReturn");
  try {
    ReturnValue = tobj->AniseedOp(p1,p2,p3);
  }
  catch (::idlServer::AniseedExcpt &ex){
    if (!CheckidlServerAniseedExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AniseedOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAniseed(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAniseed(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAniseed(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBayleafAttr(void)
{
  
  ::Bayleaf_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Bayleaf_slice * ::idlServer::BayleafAttr()");
#endif
  
  ernie.seed("BayleafAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BayleafAttrReturn");
  try {
    ReturnValue = tobj->BayleafAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BayleafAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBayleaf(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBayleafAttrW(void)
{
  
  ::Bayleaf _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BayleafAttr(const ::Bayleaf _attr)");
#endif
  
  ernie.seed("BayleafAttrCall");
  GetBayleaf(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BayleafAttrReturn");
  try {
     tobj->BayleafAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BayleafAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBayleafOp(void)
{
  
  ::Bayleaf_slice * ReturnValue;
  ::Bayleaf p1;
  ::Bayleaf_slice * p2;
  ::Bayleaf p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Bayleaf_slice * ::idlServer::BayleafOp(const ::Bayleaf p1,::Bayleaf_slice *&p2,::Bayleaf p3)");
#endif
  
  ernie.seed("BayleafOpCall");
  GetBayleaf(p1);
  GetBayleaf(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BayleafOpReturn");
  try {
    ReturnValue = tobj->BayleafOp(p1,p2,p3);
  }
  catch (::idlServer::BayleafExcpt &ex){
    if (!CheckidlServerBayleafExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BayleafOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBayleaf(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBayleaf(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBayleaf(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerDillAttr(void)
{
  
  ::Dill* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Dill* ::idlServer::DillAttr()");
#endif
  
  ernie.seed("DillAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DillAttrReturn");
  try {
    ReturnValue = tobj->DillAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::DillAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDill(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerDillAttrW(void)
{
  
  ::Dill _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::DillAttr(const ::Dill &_attr)");
#endif
  
  ernie.seed("DillAttrCall");
  GetDill(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DillAttrReturn");
  try {
     tobj->DillAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::DillAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerDillOp(void)
{
  
  ::Dill* ReturnValue;
  ::Dill p1;
  ::Dill* p2;
  ::Dill p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Dill* ::idlServer::DillOp(const ::Dill &p1,::Dill *&p2,::Dill &p3)");
#endif
  
  ernie.seed("DillOpCall");
  GetDill(p1);
  GetDill(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DillOpReturn");
  try {
    ReturnValue = tobj->DillOp(p1,p2,p3);
  }
  catch (::idlServer::DillExcpt &ex){
    if (!CheckidlServerDillExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::DillOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDill(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckDill(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckDill(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerOreganoAttr(void)
{
  
  ::Oregano_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Oregano_slice * ::idlServer::OreganoAttr()");
#endif
  
  ernie.seed("OreganoAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("OreganoAttrReturn");
  try {
    ReturnValue = tobj->OreganoAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::OreganoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOregano(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerOreganoAttrW(void)
{
  
  ::Oregano _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::OreganoAttr(const ::Oregano _attr)");
#endif
  
  ernie.seed("OreganoAttrCall");
  GetOregano(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("OreganoAttrReturn");
  try {
     tobj->OreganoAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::OreganoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerOreganoOp(void)
{
  
  ::Oregano_slice * ReturnValue;
  ::Oregano p1;
  ::Oregano_slice * p2;
  ::Oregano p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Oregano_slice * ::idlServer::OreganoOp(const ::Oregano p1,::Oregano_slice *&p2,::Oregano p3)");
#endif
  
  ernie.seed("OreganoOpCall");
  GetOregano(p1);
  GetOregano(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("OreganoOpReturn");
  try {
    ReturnValue = tobj->OreganoOp(p1,p2,p3);
  }
  catch (::idlServer::OreganoExcpt &ex){
    if (!CheckidlServerOreganoExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::OreganoOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOregano(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckOregano(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckOregano(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


