//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:05:34  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:15  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:26:36  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerBeansproutAttr(void)
{
  
  ::Beansprout* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Beansprout* ::idlServer::BeansproutAttr()");
#endif
  
  ernie.seed("BeansproutAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BeansproutAttrReturn");
  try {
    ReturnValue = tobj->BeansproutAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BeansproutAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBeansprout(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBeansproutAttrW(void)
{
  
  ::Beansprout _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BeansproutAttr(const ::Beansprout &_attr)");
#endif
  
  ernie.seed("BeansproutAttrCall");
  GetBeansprout(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BeansproutAttrReturn");
  try {
     tobj->BeansproutAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BeansproutAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBeansproutOp(void)
{
  
  ::Beansprout* ReturnValue;
  ::Beansprout p1;
  ::Beansprout* p2;
  ::Beansprout p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Beansprout* ::idlServer::BeansproutOp(const ::Beansprout &p1,::Beansprout *&p2,::Beansprout &p3)");
#endif
  
  ernie.seed("BeansproutOpCall");
  GetBeansprout(p1);
  GetBeansprout(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BeansproutOpReturn");
  try {
    ReturnValue = tobj->BeansproutOp(p1,p2,p3);
  }
  catch (::idlServer::BeansproutExcpt &ex){
    if (!CheckidlServerBeansproutExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BeansproutOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBeansprout(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBeansprout(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBeansprout(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAlmondsAttr(void)
{
  
  ::Almonds_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Almonds_slice * ::idlServer::AlmondsAttr()");
#endif
  
  ernie.seed("AlmondsAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AlmondsAttrReturn");
  try {
    ReturnValue = tobj->AlmondsAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AlmondsAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAlmonds(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAlmondsAttrW(void)
{
  
  ::Almonds _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::AlmondsAttr(const ::Almonds _attr)");
#endif
  
  ernie.seed("AlmondsAttrCall");
  GetAlmonds(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AlmondsAttrReturn");
  try {
     tobj->AlmondsAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AlmondsAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAlmondsOp(void)
{
  
  ::Almonds_slice * ReturnValue;
  ::Almonds p1;
  ::Almonds_slice * p2;
  ::Almonds p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Almonds_slice * ::idlServer::AlmondsOp(const ::Almonds p1,::Almonds_slice *&p2,::Almonds p3)");
#endif
  
  ernie.seed("AlmondsOpCall");
  GetAlmonds(p1);
  GetAlmonds(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AlmondsOpReturn");
  try {
    ReturnValue = tobj->AlmondsOp(p1,p2,p3);
  }
  catch (::idlServer::AlmondsExcpt &ex){
    if (!CheckidlServerAlmondsExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AlmondsOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAlmonds(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAlmonds(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAlmonds(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGuavaAttr(void)
{
  
  ::Guava* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Guava* ::idlServer::GuavaAttr()");
#endif
  
  ernie.seed("GuavaAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GuavaAttrReturn");
  try {
    ReturnValue = tobj->GuavaAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GuavaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGuava(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGuavaAttrW(void)
{
  
  ::Guava _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::GuavaAttr(const ::Guava &_attr)");
#endif
  
  ernie.seed("GuavaAttrCall");
  GetGuava(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GuavaAttrReturn");
  try {
     tobj->GuavaAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GuavaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGuavaOp(void)
{
  
  ::Guava* ReturnValue;
  ::Guava p1;
  ::Guava* p2;
  ::Guava p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Guava* ::idlServer::GuavaOp(const ::Guava &p1,::Guava *&p2,::Guava &p3)");
#endif
  
  ernie.seed("GuavaOpCall");
  GetGuava(p1);
  GetGuava(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GuavaOpReturn");
  try {
    ReturnValue = tobj->GuavaOp(p1,p2,p3);
  }
  catch (::idlServer::GuavaExcpt &ex){
    if (!CheckidlServerGuavaExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GuavaOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGuava(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckGuava(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckGuava(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMushroomAttr(void)
{
  
  ::Mushroom_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mushroom_slice * ::idlServer::MushroomAttr()");
#endif
  
  ernie.seed("MushroomAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MushroomAttrReturn");
  try {
    ReturnValue = tobj->MushroomAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MushroomAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMushroom(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMushroomAttrW(void)
{
  
  ::Mushroom _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::MushroomAttr(const ::Mushroom _attr)");
#endif
  
  ernie.seed("MushroomAttrCall");
  GetMushroom(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MushroomAttrReturn");
  try {
     tobj->MushroomAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MushroomAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMushroomOp(void)
{
  
  ::Mushroom_slice * ReturnValue;
  ::Mushroom p1;
  ::Mushroom_slice * p2;
  ::Mushroom p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mushroom_slice * ::idlServer::MushroomOp(const ::Mushroom p1,::Mushroom_slice *&p2,::Mushroom p3)");
#endif
  
  ernie.seed("MushroomOpCall");
  GetMushroom(p1);
  GetMushroom(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MushroomOpReturn");
  try {
    ReturnValue = tobj->MushroomOp(p1,p2,p3);
  }
  catch (::idlServer::MushroomExcpt &ex){
    if (!CheckidlServerMushroomExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MushroomOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMushroom(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckMushroom(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckMushroom(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRunnerbeanAttr(void)
{
  
  ::Runnerbean* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Runnerbean* ::idlServer::RunnerbeanAttr()");
#endif
  
  ernie.seed("RunnerbeanAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RunnerbeanAttrReturn");
  try {
    ReturnValue = tobj->RunnerbeanAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RunnerbeanAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRunnerbean(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRunnerbeanAttrW(void)
{
  
  ::Runnerbean _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::RunnerbeanAttr(const ::Runnerbean &_attr)");
#endif
  
  ernie.seed("RunnerbeanAttrCall");
  GetRunnerbean(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RunnerbeanAttrReturn");
  try {
     tobj->RunnerbeanAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RunnerbeanAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRunnerbeanOp(void)
{
  
  ::Runnerbean* ReturnValue;
  ::Runnerbean p1;
  ::Runnerbean* p2;
  ::Runnerbean p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Runnerbean* ::idlServer::RunnerbeanOp(const ::Runnerbean &p1,::Runnerbean *&p2,::Runnerbean &p3)");
#endif
  
  ernie.seed("RunnerbeanOpCall");
  GetRunnerbean(p1);
  GetRunnerbean(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RunnerbeanOpReturn");
  try {
    ReturnValue = tobj->RunnerbeanOp(p1,p2,p3);
  }
  catch (::idlServer::RunnerbeanExcpt &ex){
    if (!CheckidlServerRunnerbeanExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RunnerbeanOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRunnerbean(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckRunnerbean(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckRunnerbean(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerTofuAttr(void)
{
  
  ::Tofu_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Tofu_slice * ::idlServer::TofuAttr()");
#endif
  
  ernie.seed("TofuAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TofuAttrReturn");
  try {
    ReturnValue = tobj->TofuAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::TofuAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTofu(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerTofuAttrW(void)
{
  
  ::Tofu _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::TofuAttr(const ::Tofu _attr)");
#endif
  
  ernie.seed("TofuAttrCall");
  GetTofu(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TofuAttrReturn");
  try {
     tobj->TofuAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::TofuAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerTofuOp(void)
{
  
  ::Tofu_slice * ReturnValue;
  ::Tofu p1;
  ::Tofu_slice * p2;
  ::Tofu p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Tofu_slice * ::idlServer::TofuOp(const ::Tofu p1,::Tofu_slice *&p2,::Tofu p3)");
#endif
  
  ernie.seed("TofuOpCall");
  GetTofu(p1);
  GetTofu(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TofuOpReturn");
  try {
    ReturnValue = tobj->TofuOp(p1,p2,p3);
  }
  catch (::idlServer::TofuExcpt &ex){
    if (!CheckidlServerTofuExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::TofuOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTofu(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckTofu(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckTofu(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPrunesAttr(void)
{
  
  ::Prunes* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Prunes* ::idlServer::PrunesAttr()");
#endif
  
  ernie.seed("PrunesAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PrunesAttrReturn");
  try {
    ReturnValue = tobj->PrunesAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PrunesAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPrunes(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPrunesAttrW(void)
{
  
  ::Prunes _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::PrunesAttr(const ::Prunes &_attr)");
#endif
  
  ernie.seed("PrunesAttrCall");
  GetPrunes(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PrunesAttrReturn");
  try {
     tobj->PrunesAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PrunesAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPrunesOp(void)
{
  
  ::Prunes* ReturnValue;
  ::Prunes p1;
  ::Prunes* p2;
  ::Prunes p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Prunes* ::idlServer::PrunesOp(const ::Prunes &p1,::Prunes *&p2,::Prunes &p3)");
#endif
  
  ernie.seed("PrunesOpCall");
  GetPrunes(p1);
  GetPrunes(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PrunesOpReturn");
  try {
    ReturnValue = tobj->PrunesOp(p1,p2,p3);
  }
  catch (::idlServer::PrunesExcpt &ex){
    if (!CheckidlServerPrunesExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PrunesOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPrunes(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckPrunes(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckPrunes(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBrocoliAttr(void)
{
  
  ::Brocoli_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Brocoli_slice * ::idlServer::BrocoliAttr()");
#endif
  
  ernie.seed("BrocoliAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BrocoliAttrReturn");
  try {
    ReturnValue = tobj->BrocoliAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BrocoliAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBrocoli(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBrocoliAttrW(void)
{
  
  ::Brocoli _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BrocoliAttr(const ::Brocoli _attr)");
#endif
  
  ernie.seed("BrocoliAttrCall");
  GetBrocoli(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BrocoliAttrReturn");
  try {
     tobj->BrocoliAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BrocoliAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBrocoliOp(void)
{
  
  ::Brocoli_slice * ReturnValue;
  ::Brocoli p1;
  ::Brocoli_slice * p2;
  ::Brocoli p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Brocoli_slice * ::idlServer::BrocoliOp(const ::Brocoli p1,::Brocoli_slice *&p2,::Brocoli p3)");
#endif
  
  ernie.seed("BrocoliOpCall");
  GetBrocoli(p1);
  GetBrocoli(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BrocoliOpReturn");
  try {
    ReturnValue = tobj->BrocoliOp(p1,p2,p3);
  }
  catch (::idlServer::BrocoliExcpt &ex){
    if (!CheckidlServerBrocoliExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BrocoliOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBrocoli(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBrocoli(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBrocoli(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


