//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:06:06  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:17  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:26:41  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerGrannySmithAttr(void)
{
  
  ::GrannySmith* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GrannySmith* ::idlServer::GrannySmithAttr()");
#endif
  
  ernie.seed("GrannySmithAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GrannySmithAttrReturn");
  try {
    ReturnValue = tobj->GrannySmithAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GrannySmithAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGrannySmith(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGrannySmithAttrW(void)
{
  
  ::GrannySmith _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::GrannySmithAttr(const ::GrannySmith &_attr)");
#endif
  
  ernie.seed("GrannySmithAttrCall");
  GetGrannySmith(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GrannySmithAttrReturn");
  try {
     tobj->GrannySmithAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GrannySmithAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGrannySmithOp(void)
{
  
  ::GrannySmith* ReturnValue;
  ::GrannySmith p1;
  ::GrannySmith* p2;
  ::GrannySmith p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GrannySmith* ::idlServer::GrannySmithOp(const ::GrannySmith &p1,::GrannySmith *&p2,::GrannySmith &p3)");
#endif
  
  ernie.seed("GrannySmithOpCall");
  GetGrannySmith(p1);
  GetGrannySmith(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GrannySmithOpReturn");
  try {
    ReturnValue = tobj->GrannySmithOp(p1,p2,p3);
  }
  catch (::idlServer::GrannySmithExcpt &ex){
    if (!CheckidlServerGrannySmithExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GrannySmithOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGrannySmith(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckGrannySmith(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckGrannySmith(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBrazilNutAttr(void)
{
  
  ::BrazilNut_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BrazilNut_slice * ::idlServer::BrazilNutAttr()");
#endif
  
  ernie.seed("BrazilNutAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BrazilNutAttrReturn");
  try {
    ReturnValue = tobj->BrazilNutAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BrazilNutAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBrazilNut(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBrazilNutAttrW(void)
{
  
  ::BrazilNut _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BrazilNutAttr(const ::BrazilNut _attr)");
#endif
  
  ernie.seed("BrazilNutAttrCall");
  GetBrazilNut(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BrazilNutAttrReturn");
  try {
     tobj->BrazilNutAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BrazilNutAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBrazilNutOp(void)
{
  
  ::BrazilNut_slice * ReturnValue;
  ::BrazilNut p1;
  ::BrazilNut_slice * p2;
  ::BrazilNut p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BrazilNut_slice * ::idlServer::BrazilNutOp(const ::BrazilNut p1,::BrazilNut_slice *&p2,::BrazilNut p3)");
#endif
  
  ernie.seed("BrazilNutOpCall");
  GetBrazilNut(p1);
  GetBrazilNut(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BrazilNutOpReturn");
  try {
    ReturnValue = tobj->BrazilNutOp(p1,p2,p3);
  }
  catch (::idlServer::BrazilNutExcpt &ex){
    if (!CheckidlServerBrazilNutExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BrazilNutOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBrazilNut(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBrazilNut(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBrazilNut(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSweetPotatoAttr(void)
{
  
  ::SweetPotato* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::SweetPotato* ::idlServer::SweetPotatoAttr()");
#endif
  
  ernie.seed("SweetPotatoAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetPotatoAttrReturn");
  try {
    ReturnValue = tobj->SweetPotatoAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SweetPotatoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSweetPotato(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSweetPotatoAttrW(void)
{
  
  ::SweetPotato _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::SweetPotatoAttr(const ::SweetPotato &_attr)");
#endif
  
  ernie.seed("SweetPotatoAttrCall");
  GetSweetPotato(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetPotatoAttrReturn");
  try {
     tobj->SweetPotatoAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SweetPotatoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSweetPotatoOp(void)
{
  
  ::SweetPotato* ReturnValue;
  ::SweetPotato p1;
  ::SweetPotato* p2;
  ::SweetPotato p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::SweetPotato* ::idlServer::SweetPotatoOp(const ::SweetPotato &p1,::SweetPotato *&p2,::SweetPotato &p3)");
#endif
  
  ernie.seed("SweetPotatoOpCall");
  GetSweetPotato(p1);
  GetSweetPotato(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetPotatoOpReturn");
  try {
    ReturnValue = tobj->SweetPotatoOp(p1,p2,p3);
  }
  catch (::idlServer::SweetPotatoExcpt &ex){
    if (!CheckidlServerSweetPotatoExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SweetPotatoOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSweetPotato(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckSweetPotato(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckSweetPotato(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerTangarineAttr(void)
{
  
  ::Tangarine_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Tangarine_slice * ::idlServer::TangarineAttr()");
#endif
  
  ernie.seed("TangarineAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TangarineAttrReturn");
  try {
    ReturnValue = tobj->TangarineAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::TangarineAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTangarine(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerTangarineAttrW(void)
{
  
  ::Tangarine _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::TangarineAttr(const ::Tangarine _attr)");
#endif
  
  ernie.seed("TangarineAttrCall");
  GetTangarine(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TangarineAttrReturn");
  try {
     tobj->TangarineAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::TangarineAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerTangarineOp(void)
{
  
  ::Tangarine_slice * ReturnValue;
  ::Tangarine p1;
  ::Tangarine_slice * p2;
  ::Tangarine p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Tangarine_slice * ::idlServer::TangarineOp(const ::Tangarine p1,::Tangarine_slice *&p2,::Tangarine p3)");
#endif
  
  ernie.seed("TangarineOpCall");
  GetTangarine(p1);
  GetTangarine(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TangarineOpReturn");
  try {
    ReturnValue = tobj->TangarineOp(p1,p2,p3);
  }
  catch (::idlServer::TangarineExcpt &ex){
    if (!CheckidlServerTangarineExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::TangarineOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTangarine(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckTangarine(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckTangarine(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBillberriesAttr(void)
{
  
  ::Billberries* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Billberries* ::idlServer::BillberriesAttr()");
#endif
  
  ernie.seed("BillberriesAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BillberriesAttrReturn");
  try {
    ReturnValue = tobj->BillberriesAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BillberriesAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBillberries(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBillberriesAttrW(void)
{
  
  ::Billberries _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BillberriesAttr(const ::Billberries &_attr)");
#endif
  
  ernie.seed("BillberriesAttrCall");
  GetBillberries(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BillberriesAttrReturn");
  try {
     tobj->BillberriesAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BillberriesAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBillberriesOp(void)
{
  
  ::Billberries* ReturnValue;
  ::Billberries p1;
  ::Billberries* p2;
  ::Billberries p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Billberries* ::idlServer::BillberriesOp(const ::Billberries &p1,::Billberries *&p2,::Billberries &p3)");
#endif
  
  ernie.seed("BillberriesOpCall");
  GetBillberries(p1);
  GetBillberries(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BillberriesOpReturn");
  try {
    ReturnValue = tobj->BillberriesOp(p1,p2,p3);
  }
  catch (::idlServer::BillberriesExcpt &ex){
    if (!CheckidlServerBillberriesExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BillberriesOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBillberries(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBillberries(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBillberries(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCantelopeMelonAttr(void)
{
  
  ::CantelopeMelon_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::CantelopeMelon_slice * ::idlServer::CantelopeMelonAttr()");
#endif
  
  ernie.seed("CantelopeMelonAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CantelopeMelonAttrReturn");
  try {
    ReturnValue = tobj->CantelopeMelonAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CantelopeMelonAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCantelopeMelon(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCantelopeMelonAttrW(void)
{
  
  ::CantelopeMelon _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::CantelopeMelonAttr(const ::CantelopeMelon _attr)");
#endif
  
  ernie.seed("CantelopeMelonAttrCall");
  GetCantelopeMelon(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CantelopeMelonAttrReturn");
  try {
     tobj->CantelopeMelonAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CantelopeMelonAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCantelopeMelonOp(void)
{
  
  ::CantelopeMelon_slice * ReturnValue;
  ::CantelopeMelon p1;
  ::CantelopeMelon_slice * p2;
  ::CantelopeMelon p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::CantelopeMelon_slice * ::idlServer::CantelopeMelonOp(const ::CantelopeMelon p1,::CantelopeMelon_slice *&p2,::CantelopeMelon p3)");
#endif
  
  ernie.seed("CantelopeMelonOpCall");
  GetCantelopeMelon(p1);
  GetCantelopeMelon(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CantelopeMelonOpReturn");
  try {
    ReturnValue = tobj->CantelopeMelonOp(p1,p2,p3);
  }
  catch (::idlServer::CantelopeMelonExcpt &ex){
    if (!CheckidlServerCantelopeMelonExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CantelopeMelonOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCantelopeMelon(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckCantelopeMelon(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckCantelopeMelon(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLolloRossaAttr(void)
{
  
  ::LolloRossa* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::LolloRossa* ::idlServer::LolloRossaAttr()");
#endif
  
  ernie.seed("LolloRossaAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LolloRossaAttrReturn");
  try {
    ReturnValue = tobj->LolloRossaAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LolloRossaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLolloRossa(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLolloRossaAttrW(void)
{
  
  ::LolloRossa _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::LolloRossaAttr(const ::LolloRossa &_attr)");
#endif
  
  ernie.seed("LolloRossaAttrCall");
  GetLolloRossa(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LolloRossaAttrReturn");
  try {
     tobj->LolloRossaAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LolloRossaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLolloRossaOp(void)
{
  
  ::LolloRossa* ReturnValue;
  ::LolloRossa p1;
  ::LolloRossa* p2;
  ::LolloRossa p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::LolloRossa* ::idlServer::LolloRossaOp(const ::LolloRossa &p1,::LolloRossa *&p2,::LolloRossa &p3)");
#endif
  
  ernie.seed("LolloRossaOpCall");
  GetLolloRossa(p1);
  GetLolloRossa(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LolloRossaOpReturn");
  try {
    ReturnValue = tobj->LolloRossaOp(p1,p2,p3);
  }
  catch (::idlServer::LolloRossaExcpt &ex){
    if (!CheckidlServerLolloRossaExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LolloRossaOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLolloRossa(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckLolloRossa(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckLolloRossa(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRowanBerryAttr(void)
{
  
  ::RowanBerry_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::RowanBerry_slice * ::idlServer::RowanBerryAttr()");
#endif
  
  ernie.seed("RowanBerryAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RowanBerryAttrReturn");
  try {
    ReturnValue = tobj->RowanBerryAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RowanBerryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRowanBerry(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRowanBerryAttrW(void)
{
  
  ::RowanBerry _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::RowanBerryAttr(const ::RowanBerry _attr)");
#endif
  
  ernie.seed("RowanBerryAttrCall");
  GetRowanBerry(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RowanBerryAttrReturn");
  try {
     tobj->RowanBerryAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RowanBerryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRowanBerryOp(void)
{
  
  ::RowanBerry_slice * ReturnValue;
  ::RowanBerry p1;
  ::RowanBerry_slice * p2;
  ::RowanBerry p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::RowanBerry_slice * ::idlServer::RowanBerryOp(const ::RowanBerry p1,::RowanBerry_slice *&p2,::RowanBerry p3)");
#endif
  
  ernie.seed("RowanBerryOpCall");
  GetRowanBerry(p1);
  GetRowanBerry(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RowanBerryOpReturn");
  try {
    ReturnValue = tobj->RowanBerryOp(p1,p2,p3);
  }
  catch (::idlServer::RowanBerryExcpt &ex){
    if (!CheckidlServerRowanBerryExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RowanBerryOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRowanBerry(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckRowanBerry(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckRowanBerry(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


