//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:06:29  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:18  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:26:46  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerSweetcornAttr(void)
{
  
  ::Sweetcorn* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Sweetcorn* ::idlServer::SweetcornAttr()");
#endif
  
  ernie.seed("SweetcornAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetcornAttrReturn");
  try {
    ReturnValue = tobj->SweetcornAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SweetcornAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSweetcorn(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSweetcornAttrW(void)
{
  
  ::Sweetcorn _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::SweetcornAttr(const ::Sweetcorn &_attr)");
#endif
  
  ernie.seed("SweetcornAttrCall");
  GetSweetcorn(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetcornAttrReturn");
  try {
     tobj->SweetcornAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SweetcornAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSweetcornOp(void)
{
  
  ::Sweetcorn* ReturnValue;
  ::Sweetcorn p1;
  ::Sweetcorn* p2;
  ::Sweetcorn p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Sweetcorn* ::idlServer::SweetcornOp(const ::Sweetcorn &p1,::Sweetcorn *&p2,::Sweetcorn &p3)");
#endif
  
  ernie.seed("SweetcornOpCall");
  GetSweetcorn(p1);
  GetSweetcorn(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetcornOpReturn");
  try {
    ReturnValue = tobj->SweetcornOp(p1,p2,p3);
  }
  catch (::idlServer::SweetcornExcpt &ex){
    if (!CheckidlServerSweetcornExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SweetcornOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSweetcorn(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckSweetcorn(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckSweetcorn(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerOnionAttr(void)
{
  
  ::Onion_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Onion_slice * ::idlServer::OnionAttr()");
#endif
  
  ernie.seed("OnionAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("OnionAttrReturn");
  try {
    ReturnValue = tobj->OnionAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::OnionAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOnion(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerOnionAttrW(void)
{
  
  ::Onion _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::OnionAttr(const ::Onion _attr)");
#endif
  
  ernie.seed("OnionAttrCall");
  GetOnion(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("OnionAttrReturn");
  try {
     tobj->OnionAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::OnionAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerOnionOp(void)
{
  
  ::Onion_slice * ReturnValue;
  ::Onion p1;
  ::Onion_slice * p2;
  ::Onion p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Onion_slice * ::idlServer::OnionOp(const ::Onion p1,::Onion_slice *&p2,::Onion p3)");
#endif
  
  ernie.seed("OnionOpCall");
  GetOnion(p1);
  GetOnion(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("OnionOpReturn");
  try {
    ReturnValue = tobj->OnionOp(p1,p2,p3);
  }
  catch (::idlServer::OnionExcpt &ex){
    if (!CheckidlServerOnionExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::OnionOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOnion(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckOnion(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckOnion(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerShallotsAttr(void)
{
  
  ::Shallots* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Shallots* ::idlServer::ShallotsAttr()");
#endif
  
  ernie.seed("ShallotsAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ShallotsAttrReturn");
  try {
    ReturnValue = tobj->ShallotsAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ShallotsAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShallots(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerShallotsAttrW(void)
{
  
  ::Shallots _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::ShallotsAttr(const ::Shallots &_attr)");
#endif
  
  ernie.seed("ShallotsAttrCall");
  GetShallots(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ShallotsAttrReturn");
  try {
     tobj->ShallotsAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ShallotsAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerShallotsOp(void)
{
  
  ::Shallots* ReturnValue;
  ::Shallots p1;
  ::Shallots* p2;
  ::Shallots p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Shallots* ::idlServer::ShallotsOp(const ::Shallots &p1,::Shallots *&p2,::Shallots &p3)");
#endif
  
  ernie.seed("ShallotsOpCall");
  GetShallots(p1);
  GetShallots(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ShallotsOpReturn");
  try {
    ReturnValue = tobj->ShallotsOp(p1,p2,p3);
  }
  catch (::idlServer::ShallotsExcpt &ex){
    if (!CheckidlServerShallotsExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ShallotsOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShallots(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckShallots(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckShallots(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPlumAttr(void)
{
  
  ::Plum_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Plum_slice * ::idlServer::PlumAttr()");
#endif
  
  ernie.seed("PlumAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PlumAttrReturn");
  try {
    ReturnValue = tobj->PlumAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PlumAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPlum(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPlumAttrW(void)
{
  
  ::Plum _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::PlumAttr(const ::Plum _attr)");
#endif
  
  ernie.seed("PlumAttrCall");
  GetPlum(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PlumAttrReturn");
  try {
     tobj->PlumAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PlumAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPlumOp(void)
{
  
  ::Plum_slice * ReturnValue;
  ::Plum p1;
  ::Plum_slice * p2;
  ::Plum p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Plum_slice * ::idlServer::PlumOp(const ::Plum p1,::Plum_slice *&p2,::Plum p3)");
#endif
  
  ernie.seed("PlumOpCall");
  GetPlum(p1);
  GetPlum(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PlumOpReturn");
  try {
    ReturnValue = tobj->PlumOp(p1,p2,p3);
  }
  catch (::idlServer::PlumExcpt &ex){
    if (!CheckidlServerPlumExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PlumOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPlum(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckPlum(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckPlum(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKidneyBeanAttr(void)
{
  
  ::KidneyBean* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::KidneyBean* ::idlServer::KidneyBeanAttr()");
#endif
  
  ernie.seed("KidneyBeanAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KidneyBeanAttrReturn");
  try {
    ReturnValue = tobj->KidneyBeanAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KidneyBeanAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKidneyBean(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKidneyBeanAttrW(void)
{
  
  ::KidneyBean _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::KidneyBeanAttr(const ::KidneyBean &_attr)");
#endif
  
  ernie.seed("KidneyBeanAttrCall");
  GetKidneyBean(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KidneyBeanAttrReturn");
  try {
     tobj->KidneyBeanAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KidneyBeanAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKidneyBeanOp(void)
{
  
  ::KidneyBean* ReturnValue;
  ::KidneyBean p1;
  ::KidneyBean* p2;
  ::KidneyBean p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::KidneyBean* ::idlServer::KidneyBeanOp(const ::KidneyBean &p1,::KidneyBean *&p2,::KidneyBean &p3)");
#endif
  
  ernie.seed("KidneyBeanOpCall");
  GetKidneyBean(p1);
  GetKidneyBean(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KidneyBeanOpReturn");
  try {
    ReturnValue = tobj->KidneyBeanOp(p1,p2,p3);
  }
  catch (::idlServer::KidneyBeanExcpt &ex){
    if (!CheckidlServerKidneyBeanExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KidneyBeanOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKidneyBean(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckKidneyBean(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckKidneyBean(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerZucchiniAttr(void)
{
  
  ::Zucchini_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Zucchini_slice * ::idlServer::ZucchiniAttr()");
#endif
  
  ernie.seed("ZucchiniAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ZucchiniAttrReturn");
  try {
    ReturnValue = tobj->ZucchiniAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ZucchiniAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckZucchini(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerZucchiniAttrW(void)
{
  
  ::Zucchini _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::ZucchiniAttr(const ::Zucchini _attr)");
#endif
  
  ernie.seed("ZucchiniAttrCall");
  GetZucchini(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ZucchiniAttrReturn");
  try {
     tobj->ZucchiniAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ZucchiniAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerZucchiniOp(void)
{
  
  ::Zucchini_slice * ReturnValue;
  ::Zucchini p1;
  ::Zucchini_slice * p2;
  ::Zucchini p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Zucchini_slice * ::idlServer::ZucchiniOp(const ::Zucchini p1,::Zucchini_slice *&p2,::Zucchini p3)");
#endif
  
  ernie.seed("ZucchiniOpCall");
  GetZucchini(p1);
  GetZucchini(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ZucchiniOpReturn");
  try {
    ReturnValue = tobj->ZucchiniOp(p1,p2,p3);
  }
  catch (::idlServer::ZucchiniExcpt &ex){
    if (!CheckidlServerZucchiniExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ZucchiniOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckZucchini(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckZucchini(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckZucchini(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerButterBeanAttr(void)
{
  
  ::ButterBean* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::ButterBean* ::idlServer::ButterBeanAttr()");
#endif
  
  ernie.seed("ButterBeanAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ButterBeanAttrReturn");
  try {
    ReturnValue = tobj->ButterBeanAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ButterBeanAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckButterBean(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerButterBeanAttrW(void)
{
  
  ::ButterBean _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::ButterBeanAttr(const ::ButterBean &_attr)");
#endif
  
  ernie.seed("ButterBeanAttrCall");
  GetButterBean(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ButterBeanAttrReturn");
  try {
     tobj->ButterBeanAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ButterBeanAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerButterBeanOp(void)
{
  
  ::ButterBean* ReturnValue;
  ::ButterBean p1;
  ::ButterBean* p2;
  ::ButterBean p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::ButterBean* ::idlServer::ButterBeanOp(const ::ButterBean &p1,::ButterBean *&p2,::ButterBean &p3)");
#endif
  
  ernie.seed("ButterBeanOpCall");
  GetButterBean(p1);
  GetButterBean(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ButterBeanOpReturn");
  try {
    ReturnValue = tobj->ButterBeanOp(p1,p2,p3);
  }
  catch (::idlServer::ButterBeanExcpt &ex){
    if (!CheckidlServerButterBeanExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ButterBeanOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckButterBean(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckButterBean(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckButterBean(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoAttr(void)
{
  
  ::Avocado_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado_slice * ::idlServer::AvocadoAttr()");
#endif
  
  ernie.seed("AvocadoAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoAttrReturn");
  try {
    ReturnValue = tobj->AvocadoAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocado(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoAttrW(void)
{
  
  ::Avocado _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::AvocadoAttr(const ::Avocado _attr)");
#endif
  
  ernie.seed("AvocadoAttrCall");
  GetAvocado(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoAttrReturn");
  try {
     tobj->AvocadoAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoOp(void)
{
  
  ::Avocado_slice * ReturnValue;
  ::Avocado p1;
  ::Avocado_slice * p2;
  ::Avocado p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado_slice * ::idlServer::AvocadoOp(const ::Avocado p1,::Avocado_slice *&p2,::Avocado p3)");
#endif
  
  ernie.seed("AvocadoOpCall");
  GetAvocado(p1);
  GetAvocado(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoOpReturn");
  try {
    ReturnValue = tobj->AvocadoOp(p1,p2,p3);
  }
  catch (::idlServer::AvocadoExcpt &ex){
    if (!CheckidlServerAvocadoExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocado(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocado(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocado(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


