//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:06:51  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:20  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:26:51  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerGoldenDelicousAttr(void)
{
  
  ::GoldenDelicous* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GoldenDelicous* ::idlServer::GoldenDelicousAttr()");
#endif
  
  ernie.seed("GoldenDelicousAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GoldenDelicousAttrReturn");
  try {
    ReturnValue = tobj->GoldenDelicousAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GoldenDelicousAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGoldenDelicous(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGoldenDelicousAttrW(void)
{
  
  ::GoldenDelicous _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::GoldenDelicousAttr(const ::GoldenDelicous &_attr)");
#endif
  
  ernie.seed("GoldenDelicousAttrCall");
  GetGoldenDelicous(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GoldenDelicousAttrReturn");
  try {
     tobj->GoldenDelicousAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GoldenDelicousAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGoldenDelicousOp(void)
{
  
  ::GoldenDelicous* ReturnValue;
  ::GoldenDelicous p1;
  ::GoldenDelicous* p2;
  ::GoldenDelicous p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GoldenDelicous* ::idlServer::GoldenDelicousOp(const ::GoldenDelicous &p1,::GoldenDelicous *&p2,::GoldenDelicous &p3)");
#endif
  
  ernie.seed("GoldenDelicousOpCall");
  GetGoldenDelicous(p1);
  GetGoldenDelicous(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GoldenDelicousOpReturn");
  try {
    ReturnValue = tobj->GoldenDelicousOp(p1,p2,p3);
  }
  catch (::idlServer::GoldenDelicousExcpt &ex){
    if (!CheckidlServerGoldenDelicousExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GoldenDelicousOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGoldenDelicous(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckGoldenDelicous(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckGoldenDelicous(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCrabAppleAttr(void)
{
  
  ::CrabApple_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::CrabApple_slice * ::idlServer::CrabAppleAttr()");
#endif
  
  ernie.seed("CrabAppleAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CrabAppleAttrReturn");
  try {
    ReturnValue = tobj->CrabAppleAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CrabAppleAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCrabApple(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCrabAppleAttrW(void)
{
  
  ::CrabApple _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::CrabAppleAttr(const ::CrabApple _attr)");
#endif
  
  ernie.seed("CrabAppleAttrCall");
  GetCrabApple(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CrabAppleAttrReturn");
  try {
     tobj->CrabAppleAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CrabAppleAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCrabAppleOp(void)
{
  
  ::CrabApple_slice * ReturnValue;
  ::CrabApple p1;
  ::CrabApple_slice * p2;
  ::CrabApple p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::CrabApple_slice * ::idlServer::CrabAppleOp(const ::CrabApple p1,::CrabApple_slice *&p2,::CrabApple p3)");
#endif
  
  ernie.seed("CrabAppleOpCall");
  GetCrabApple(p1);
  GetCrabApple(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CrabAppleOpReturn");
  try {
    ReturnValue = tobj->CrabAppleOp(p1,p2,p3);
  }
  catch (::idlServer::CrabAppleExcpt &ex){
    if (!CheckidlServerCrabAppleExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CrabAppleOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCrabApple(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckCrabApple(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckCrabApple(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCinnamonAttr(void)
{
  
  ::Cinnamon* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Cinnamon* ::idlServer::CinnamonAttr()");
#endif
  
  ernie.seed("CinnamonAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CinnamonAttrReturn");
  try {
    ReturnValue = tobj->CinnamonAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CinnamonAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCinnamon(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCinnamonAttrW(void)
{
  
  ::Cinnamon _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::CinnamonAttr(const ::Cinnamon &_attr)");
#endif
  
  ernie.seed("CinnamonAttrCall");
  GetCinnamon(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CinnamonAttrReturn");
  try {
     tobj->CinnamonAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CinnamonAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCinnamonOp(void)
{
  
  ::Cinnamon* ReturnValue;
  ::Cinnamon p1;
  ::Cinnamon* p2;
  ::Cinnamon p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Cinnamon* ::idlServer::CinnamonOp(const ::Cinnamon &p1,::Cinnamon *&p2,::Cinnamon &p3)");
#endif
  
  ernie.seed("CinnamonOpCall");
  GetCinnamon(p1);
  GetCinnamon(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CinnamonOpReturn");
  try {
    ReturnValue = tobj->CinnamonOp(p1,p2,p3);
  }
  catch (::idlServer::CinnamonExcpt &ex){
    if (!CheckidlServerCinnamonExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CinnamonOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCinnamon(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckCinnamon(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckCinnamon(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerJuniperBerriesAttr(void)
{
  
  ::JuniperBerries_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::JuniperBerries_slice * ::idlServer::JuniperBerriesAttr()");
#endif
  
  ernie.seed("JuniperBerriesAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("JuniperBerriesAttrReturn");
  try {
    ReturnValue = tobj->JuniperBerriesAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::JuniperBerriesAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckJuniperBerries(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerJuniperBerriesAttrW(void)
{
  
  ::JuniperBerries _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::JuniperBerriesAttr(const ::JuniperBerries _attr)");
#endif
  
  ernie.seed("JuniperBerriesAttrCall");
  GetJuniperBerries(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("JuniperBerriesAttrReturn");
  try {
     tobj->JuniperBerriesAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::JuniperBerriesAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerJuniperBerriesOp(void)
{
  
  ::JuniperBerries_slice * ReturnValue;
  ::JuniperBerries p1;
  ::JuniperBerries_slice * p2;
  ::JuniperBerries p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::JuniperBerries_slice * ::idlServer::JuniperBerriesOp(const ::JuniperBerries p1,::JuniperBerries_slice *&p2,::JuniperBerries p3)");
#endif
  
  ernie.seed("JuniperBerriesOpCall");
  GetJuniperBerries(p1);
  GetJuniperBerries(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("JuniperBerriesOpReturn");
  try {
    ReturnValue = tobj->JuniperBerriesOp(p1,p2,p3);
  }
  catch (::idlServer::JuniperBerriesExcpt &ex){
    if (!CheckidlServerJuniperBerriesExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::JuniperBerriesOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckJuniperBerries(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckJuniperBerries(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckJuniperBerries(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMustardAttr(void)
{
  
  ::Mustard* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mustard* ::idlServer::MustardAttr()");
#endif
  
  ernie.seed("MustardAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MustardAttrReturn");
  try {
    ReturnValue = tobj->MustardAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MustardAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMustard(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMustardAttrW(void)
{
  
  ::Mustard _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::MustardAttr(const ::Mustard &_attr)");
#endif
  
  ernie.seed("MustardAttrCall");
  GetMustard(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MustardAttrReturn");
  try {
     tobj->MustardAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MustardAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMustardOp(void)
{
  
  ::Mustard* ReturnValue;
  ::Mustard p1;
  ::Mustard* p2;
  ::Mustard p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mustard* ::idlServer::MustardOp(const ::Mustard &p1,::Mustard *&p2,::Mustard &p3)");
#endif
  
  ernie.seed("MustardOpCall");
  GetMustard(p1);
  GetMustard(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MustardOpReturn");
  try {
    ReturnValue = tobj->MustardOp(p1,p2,p3);
  }
  catch (::idlServer::MustardExcpt &ex){
    if (!CheckidlServerMustardExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MustardOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMustard(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckMustard(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckMustard(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerThymeAttr(void)
{
  
  ::Thyme_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Thyme_slice * ::idlServer::ThymeAttr()");
#endif
  
  ernie.seed("ThymeAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ThymeAttrReturn");
  try {
    ReturnValue = tobj->ThymeAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ThymeAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckThyme(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerThymeAttrW(void)
{
  
  ::Thyme _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::ThymeAttr(const ::Thyme _attr)");
#endif
  
  ernie.seed("ThymeAttrCall");
  GetThyme(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ThymeAttrReturn");
  try {
     tobj->ThymeAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ThymeAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerThymeOp(void)
{
  
  ::Thyme_slice * ReturnValue;
  ::Thyme p1;
  ::Thyme_slice * p2;
  ::Thyme p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Thyme_slice * ::idlServer::ThymeOp(const ::Thyme p1,::Thyme_slice *&p2,::Thyme p3)");
#endif
  
  ernie.seed("ThymeOpCall");
  GetThyme(p1);
  GetThyme(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ThymeOpReturn");
  try {
    ReturnValue = tobj->ThymeOp(p1,p2,p3);
  }
  catch (::idlServer::ThymeExcpt &ex){
    if (!CheckidlServerThymeExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ThymeOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckThyme(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckThyme(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckThyme(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerTarragonAttr(void)
{
  
  ::Tarragon* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Tarragon* ::idlServer::TarragonAttr()");
#endif
  
  ernie.seed("TarragonAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TarragonAttrReturn");
  try {
    ReturnValue = tobj->TarragonAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::TarragonAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTarragon(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerTarragonAttrW(void)
{
  
  ::Tarragon _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::TarragonAttr(const ::Tarragon &_attr)");
#endif
  
  ernie.seed("TarragonAttrCall");
  GetTarragon(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TarragonAttrReturn");
  try {
     tobj->TarragonAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::TarragonAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerTarragonOp(void)
{
  
  ::Tarragon* ReturnValue;
  ::Tarragon p1;
  ::Tarragon* p2;
  ::Tarragon p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Tarragon* ::idlServer::TarragonOp(const ::Tarragon &p1,::Tarragon *&p2,::Tarragon &p3)");
#endif
  
  ernie.seed("TarragonOpCall");
  GetTarragon(p1);
  GetTarragon(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("TarragonOpReturn");
  try {
    ReturnValue = tobj->TarragonOp(p1,p2,p3);
  }
  catch (::idlServer::TarragonExcpt &ex){
    if (!CheckidlServerTarragonExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::TarragonOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTarragon(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckTarragon(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckTarragon(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCressAttr(void)
{
  
  ::Cress_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Cress_slice * ::idlServer::CressAttr()");
#endif
  
  ernie.seed("CressAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CressAttrReturn");
  try {
    ReturnValue = tobj->CressAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CressAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCress(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCressAttrW(void)
{
  
  ::Cress _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::CressAttr(const ::Cress _attr)");
#endif
  
  ernie.seed("CressAttrCall");
  GetCress(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CressAttrReturn");
  try {
     tobj->CressAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CressAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCressOp(void)
{
  
  ::Cress_slice * ReturnValue;
  ::Cress p1;
  ::Cress_slice * p2;
  ::Cress p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Cress_slice * ::idlServer::CressOp(const ::Cress p1,::Cress_slice *&p2,::Cress p3)");
#endif
  
  ernie.seed("CressOpCall");
  GetCress(p1);
  GetCress(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CressOpReturn");
  try {
    ReturnValue = tobj->CressOp(p1,p2,p3);
  }
  catch (::idlServer::CressExcpt &ex){
    if (!CheckidlServerCressExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CressOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCress(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckCress(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckCress(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


