//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:07:14  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:21  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:26:56  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerMelonAttr(void)
{
  
  ::Melon* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Melon* ::idlServer::MelonAttr()");
#endif
  
  ernie.seed("MelonAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MelonAttrReturn");
  try {
    ReturnValue = tobj->MelonAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MelonAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMelon(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMelonAttrW(void)
{
  
  ::Melon _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::MelonAttr(const ::Melon &_attr)");
#endif
  
  ernie.seed("MelonAttrCall");
  GetMelon(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MelonAttrReturn");
  try {
     tobj->MelonAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MelonAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMelonOp(void)
{
  
  ::Melon* ReturnValue;
  ::Melon p1;
  ::Melon* p2;
  ::Melon p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Melon* ::idlServer::MelonOp(const ::Melon &p1,::Melon *&p2,::Melon &p3)");
#endif
  
  ernie.seed("MelonOpCall");
  GetMelon(p1);
  GetMelon(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MelonOpReturn");
  try {
    ReturnValue = tobj->MelonOp(p1,p2,p3);
  }
  catch (::idlServer::MelonExcpt &ex){
    if (!CheckidlServerMelonExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MelonOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMelon(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckMelon(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckMelon(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerElderberryAttr(void)
{
  
  ::Elderberry_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Elderberry_slice * ::idlServer::ElderberryAttr()");
#endif
  
  ernie.seed("ElderberryAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ElderberryAttrReturn");
  try {
    ReturnValue = tobj->ElderberryAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ElderberryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckElderberry(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerElderberryAttrW(void)
{
  
  ::Elderberry _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::ElderberryAttr(const ::Elderberry _attr)");
#endif
  
  ernie.seed("ElderberryAttrCall");
  GetElderberry(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ElderberryAttrReturn");
  try {
     tobj->ElderberryAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ElderberryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerElderberryOp(void)
{
  
  ::Elderberry_slice * ReturnValue;
  ::Elderberry p1;
  ::Elderberry_slice * p2;
  ::Elderberry p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Elderberry_slice * ::idlServer::ElderberryOp(const ::Elderberry p1,::Elderberry_slice *&p2,::Elderberry p3)");
#endif
  
  ernie.seed("ElderberryOpCall");
  GetElderberry(p1);
  GetElderberry(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ElderberryOpReturn");
  try {
    ReturnValue = tobj->ElderberryOp(p1,p2,p3);
  }
  catch (::idlServer::ElderberryExcpt &ex){
    if (!CheckidlServerElderberryExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ElderberryOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckElderberry(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckElderberry(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckElderberry(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerFennellAttr(void)
{
  
  ::Fennell* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Fennell* ::idlServer::FennellAttr()");
#endif
  
  ernie.seed("FennellAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("FennellAttrReturn");
  try {
    ReturnValue = tobj->FennellAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::FennellAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFennell(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerFennellAttrW(void)
{
  
  ::Fennell _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::FennellAttr(const ::Fennell &_attr)");
#endif
  
  ernie.seed("FennellAttrCall");
  GetFennell(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("FennellAttrReturn");
  try {
     tobj->FennellAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::FennellAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerFennellOp(void)
{
  
  ::Fennell* ReturnValue;
  ::Fennell p1;
  ::Fennell* p2;
  ::Fennell p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Fennell* ::idlServer::FennellOp(const ::Fennell &p1,::Fennell *&p2,::Fennell &p3)");
#endif
  
  ernie.seed("FennellOpCall");
  GetFennell(p1);
  GetFennell(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("FennellOpReturn");
  try {
    ReturnValue = tobj->FennellOp(p1,p2,p3);
  }
  catch (::idlServer::FennellExcpt &ex){
    if (!CheckidlServerFennellExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::FennellOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFennell(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckFennell(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckFennell(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerChickpeaAttr(void)
{
  
  ::Chickpea_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Chickpea_slice * ::idlServer::ChickpeaAttr()");
#endif
  
  ernie.seed("ChickpeaAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ChickpeaAttrReturn");
  try {
    ReturnValue = tobj->ChickpeaAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ChickpeaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChickpea(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerChickpeaAttrW(void)
{
  
  ::Chickpea _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::ChickpeaAttr(const ::Chickpea _attr)");
#endif
  
  ernie.seed("ChickpeaAttrCall");
  GetChickpea(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ChickpeaAttrReturn");
  try {
     tobj->ChickpeaAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ChickpeaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerChickpeaOp(void)
{
  
  ::Chickpea_slice * ReturnValue;
  ::Chickpea p1;
  ::Chickpea_slice * p2;
  ::Chickpea p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Chickpea_slice * ::idlServer::ChickpeaOp(const ::Chickpea p1,::Chickpea_slice *&p2,::Chickpea p3)");
#endif
  
  ernie.seed("ChickpeaOpCall");
  GetChickpea(p1);
  GetChickpea(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ChickpeaOpReturn");
  try {
    ReturnValue = tobj->ChickpeaOp(p1,p2,p3);
  }
  catch (::idlServer::ChickpeaExcpt &ex){
    if (!CheckidlServerChickpeaExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ChickpeaOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChickpea(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckChickpea(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckChickpea(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPlantainAttr(void)
{
  
  ::Plantain* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Plantain* ::idlServer::PlantainAttr()");
#endif
  
  ernie.seed("PlantainAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PlantainAttrReturn");
  try {
    ReturnValue = tobj->PlantainAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PlantainAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPlantain(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPlantainAttrW(void)
{
  
  ::Plantain _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::PlantainAttr(const ::Plantain &_attr)");
#endif
  
  ernie.seed("PlantainAttrCall");
  GetPlantain(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PlantainAttrReturn");
  try {
     tobj->PlantainAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PlantainAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPlantainOp(void)
{
  
  ::Plantain* ReturnValue;
  ::Plantain p1;
  ::Plantain* p2;
  ::Plantain p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Plantain* ::idlServer::PlantainOp(const ::Plantain &p1,::Plantain *&p2,::Plantain &p3)");
#endif
  
  ernie.seed("PlantainOpCall");
  GetPlantain(p1);
  GetPlantain(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PlantainOpReturn");
  try {
    ReturnValue = tobj->PlantainOp(p1,p2,p3);
  }
  catch (::idlServer::PlantainExcpt &ex){
    if (!CheckidlServerPlantainExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PlantainOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPlantain(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckPlantain(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckPlantain(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLimaBeanAttr(void)
{
  
  ::LimaBean_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::LimaBean_slice * ::idlServer::LimaBeanAttr()");
#endif
  
  ernie.seed("LimaBeanAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LimaBeanAttrReturn");
  try {
    ReturnValue = tobj->LimaBeanAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LimaBeanAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLimaBean(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLimaBeanAttrW(void)
{
  
  ::LimaBean _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::LimaBeanAttr(const ::LimaBean _attr)");
#endif
  
  ernie.seed("LimaBeanAttrCall");
  GetLimaBean(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LimaBeanAttrReturn");
  try {
     tobj->LimaBeanAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LimaBeanAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLimaBeanOp(void)
{
  
  ::LimaBean_slice * ReturnValue;
  ::LimaBean p1;
  ::LimaBean_slice * p2;
  ::LimaBean p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::LimaBean_slice * ::idlServer::LimaBeanOp(const ::LimaBean p1,::LimaBean_slice *&p2,::LimaBean p3)");
#endif
  
  ernie.seed("LimaBeanOpCall");
  GetLimaBean(p1);
  GetLimaBean(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LimaBeanOpReturn");
  try {
    ReturnValue = tobj->LimaBeanOp(p1,p2,p3);
  }
  catch (::idlServer::LimaBeanExcpt &ex){
    if (!CheckidlServerLimaBeanExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LimaBeanOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLimaBean(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckLimaBean(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckLimaBean(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRedPepperAttr(void)
{
  
  ::RedPepper* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::RedPepper* ::idlServer::RedPepperAttr()");
#endif
  
  ernie.seed("RedPepperAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedPepperAttrReturn");
  try {
    ReturnValue = tobj->RedPepperAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RedPepperAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRedPepper(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRedPepperAttrW(void)
{
  
  ::RedPepper _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::RedPepperAttr(const ::RedPepper &_attr)");
#endif
  
  ernie.seed("RedPepperAttrCall");
  GetRedPepper(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedPepperAttrReturn");
  try {
     tobj->RedPepperAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RedPepperAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRedPepperOp(void)
{
  
  ::RedPepper* ReturnValue;
  ::RedPepper p1;
  ::RedPepper* p2;
  ::RedPepper p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::RedPepper* ::idlServer::RedPepperOp(const ::RedPepper &p1,::RedPepper *&p2,::RedPepper &p3)");
#endif
  
  ernie.seed("RedPepperOpCall");
  GetRedPepper(p1);
  GetRedPepper(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedPepperOpReturn");
  try {
    ReturnValue = tobj->RedPepperOp(p1,p2,p3);
  }
  catch (::idlServer::RedPepperExcpt &ex){
    if (!CheckidlServerRedPepperExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RedPepperOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRedPepper(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckRedPepper(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckRedPepper(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSharanfruitAttr(void)
{
  
  ::Sharanfruit_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Sharanfruit_slice * ::idlServer::SharanfruitAttr()");
#endif
  
  ernie.seed("SharanfruitAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SharanfruitAttrReturn");
  try {
    ReturnValue = tobj->SharanfruitAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SharanfruitAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSharanfruit(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSharanfruitAttrW(void)
{
  
  ::Sharanfruit _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::SharanfruitAttr(const ::Sharanfruit _attr)");
#endif
  
  ernie.seed("SharanfruitAttrCall");
  GetSharanfruit(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SharanfruitAttrReturn");
  try {
     tobj->SharanfruitAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SharanfruitAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSharanfruitOp(void)
{
  
  ::Sharanfruit_slice * ReturnValue;
  ::Sharanfruit p1;
  ::Sharanfruit_slice * p2;
  ::Sharanfruit p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Sharanfruit_slice * ::idlServer::SharanfruitOp(const ::Sharanfruit p1,::Sharanfruit_slice *&p2,::Sharanfruit p3)");
#endif
  
  ernie.seed("SharanfruitOpCall");
  GetSharanfruit(p1);
  GetSharanfruit(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SharanfruitOpReturn");
  try {
    ReturnValue = tobj->SharanfruitOp(p1,p2,p3);
  }
  catch (::idlServer::SharanfruitExcpt &ex){
    if (!CheckidlServerSharanfruitExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SharanfruitOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSharanfruit(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckSharanfruit(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckSharanfruit(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


