//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:07:36  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:23  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:27:01  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerPapayaAttr(void)
{
  
  ::Papaya* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Papaya* ::idlServer::PapayaAttr()");
#endif
  
  ernie.seed("PapayaAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PapayaAttrReturn");
  try {
    ReturnValue = tobj->PapayaAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PapayaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPapaya(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPapayaAttrW(void)
{
  
  ::Papaya _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::PapayaAttr(const ::Papaya &_attr)");
#endif
  
  ernie.seed("PapayaAttrCall");
  GetPapaya(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PapayaAttrReturn");
  try {
     tobj->PapayaAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PapayaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPapayaOp(void)
{
  
  ::Papaya* ReturnValue;
  ::Papaya p1;
  ::Papaya* p2;
  ::Papaya p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Papaya* ::idlServer::PapayaOp(const ::Papaya &p1,::Papaya *&p2,::Papaya &p3)");
#endif
  
  ernie.seed("PapayaOpCall");
  GetPapaya(p1);
  GetPapaya(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PapayaOpReturn");
  try {
    ReturnValue = tobj->PapayaOp(p1,p2,p3);
  }
  catch (::idlServer::PapayaExcpt &ex){
    if (!CheckidlServerPapayaExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PapayaOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPapaya(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckPapaya(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckPapaya(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRaspberryAttr(void)
{
  
  ::Raspberry_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Raspberry_slice * ::idlServer::RaspberryAttr()");
#endif
  
  ernie.seed("RaspberryAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RaspberryAttrReturn");
  try {
    ReturnValue = tobj->RaspberryAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RaspberryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRaspberry(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRaspberryAttrW(void)
{
  
  ::Raspberry _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::RaspberryAttr(const ::Raspberry _attr)");
#endif
  
  ernie.seed("RaspberryAttrCall");
  GetRaspberry(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RaspberryAttrReturn");
  try {
     tobj->RaspberryAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RaspberryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRaspberryOp(void)
{
  
  ::Raspberry_slice * ReturnValue;
  ::Raspberry p1;
  ::Raspberry_slice * p2;
  ::Raspberry p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Raspberry_slice * ::idlServer::RaspberryOp(const ::Raspberry p1,::Raspberry_slice *&p2,::Raspberry p3)");
#endif
  
  ernie.seed("RaspberryOpCall");
  GetRaspberry(p1);
  GetRaspberry(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RaspberryOpReturn");
  try {
    ReturnValue = tobj->RaspberryOp(p1,p2,p3);
  }
  catch (::idlServer::RaspberryExcpt &ex){
    if (!CheckidlServerRaspberryExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RaspberryOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRaspberry(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckRaspberry(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckRaspberry(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerDamsonsAttr(void)
{
  
  ::Damsons* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Damsons* ::idlServer::DamsonsAttr()");
#endif
  
  ernie.seed("DamsonsAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DamsonsAttrReturn");
  try {
    ReturnValue = tobj->DamsonsAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::DamsonsAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDamsons(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerDamsonsAttrW(void)
{
  
  ::Damsons _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::DamsonsAttr(const ::Damsons &_attr)");
#endif
  
  ernie.seed("DamsonsAttrCall");
  GetDamsons(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DamsonsAttrReturn");
  try {
     tobj->DamsonsAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::DamsonsAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerDamsonsOp(void)
{
  
  ::Damsons* ReturnValue;
  ::Damsons p1;
  ::Damsons* p2;
  ::Damsons p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Damsons* ::idlServer::DamsonsOp(const ::Damsons &p1,::Damsons *&p2,::Damsons &p3)");
#endif
  
  ernie.seed("DamsonsOpCall");
  GetDamsons(p1);
  GetDamsons(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DamsonsOpReturn");
  try {
    ReturnValue = tobj->DamsonsOp(p1,p2,p3);
  }
  catch (::idlServer::DamsonsExcpt &ex){
    if (!CheckidlServerDamsonsExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::DamsonsOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDamsons(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckDamsons(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckDamsons(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerArtichokesAttr(void)
{
  
  ::Artichokes_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Artichokes_slice * ::idlServer::ArtichokesAttr()");
#endif
  
  ernie.seed("ArtichokesAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ArtichokesAttrReturn");
  try {
    ReturnValue = tobj->ArtichokesAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ArtichokesAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckArtichokes(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerArtichokesAttrW(void)
{
  
  ::Artichokes _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::ArtichokesAttr(const ::Artichokes _attr)");
#endif
  
  ernie.seed("ArtichokesAttrCall");
  GetArtichokes(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ArtichokesAttrReturn");
  try {
     tobj->ArtichokesAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ArtichokesAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerArtichokesOp(void)
{
  
  ::Artichokes_slice * ReturnValue;
  ::Artichokes p1;
  ::Artichokes_slice * p2;
  ::Artichokes p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Artichokes_slice * ::idlServer::ArtichokesOp(const ::Artichokes p1,::Artichokes_slice *&p2,::Artichokes p3)");
#endif
  
  ernie.seed("ArtichokesOpCall");
  GetArtichokes(p1);
  GetArtichokes(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ArtichokesOpReturn");
  try {
    ReturnValue = tobj->ArtichokesOp(p1,p2,p3);
  }
  catch (::idlServer::ArtichokesExcpt &ex){
    if (!CheckidlServerArtichokesExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ArtichokesOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckArtichokes(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckArtichokes(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckArtichokes(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCookingAppleAttr(void)
{
  
  ::CookingApple* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::CookingApple* ::idlServer::CookingAppleAttr()");
#endif
  
  ernie.seed("CookingAppleAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CookingAppleAttrReturn");
  try {
    ReturnValue = tobj->CookingAppleAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CookingAppleAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCookingApple(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCookingAppleAttrW(void)
{
  
  ::CookingApple _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::CookingAppleAttr(const ::CookingApple &_attr)");
#endif
  
  ernie.seed("CookingAppleAttrCall");
  GetCookingApple(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CookingAppleAttrReturn");
  try {
     tobj->CookingAppleAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CookingAppleAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCookingAppleOp(void)
{
  
  ::CookingApple* ReturnValue;
  ::CookingApple p1;
  ::CookingApple* p2;
  ::CookingApple p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::CookingApple* ::idlServer::CookingAppleOp(const ::CookingApple &p1,::CookingApple *&p2,::CookingApple &p3)");
#endif
  
  ernie.seed("CookingAppleOpCall");
  GetCookingApple(p1);
  GetCookingApple(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CookingAppleOpReturn");
  try {
    ReturnValue = tobj->CookingAppleOp(p1,p2,p3);
  }
  catch (::idlServer::CookingAppleExcpt &ex){
    if (!CheckidlServerCookingAppleExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CookingAppleOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCookingApple(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckCookingApple(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckCookingApple(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGreenChillyAttr(void)
{
  
  ::GreenChilly_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GreenChilly_slice * ::idlServer::GreenChillyAttr()");
#endif
  
  ernie.seed("GreenChillyAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GreenChillyAttrReturn");
  try {
    ReturnValue = tobj->GreenChillyAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GreenChillyAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGreenChilly(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGreenChillyAttrW(void)
{
  
  ::GreenChilly _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::GreenChillyAttr(const ::GreenChilly _attr)");
#endif
  
  ernie.seed("GreenChillyAttrCall");
  GetGreenChilly(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GreenChillyAttrReturn");
  try {
     tobj->GreenChillyAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GreenChillyAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGreenChillyOp(void)
{
  
  ::GreenChilly_slice * ReturnValue;
  ::GreenChilly p1;
  ::GreenChilly_slice * p2;
  ::GreenChilly p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GreenChilly_slice * ::idlServer::GreenChillyOp(const ::GreenChilly p1,::GreenChilly_slice *&p2,::GreenChilly p3)");
#endif
  
  ernie.seed("GreenChillyOpCall");
  GetGreenChilly(p1);
  GetGreenChilly(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GreenChillyOpReturn");
  try {
    ReturnValue = tobj->GreenChillyOp(p1,p2,p3);
  }
  catch (::idlServer::GreenChillyExcpt &ex){
    if (!CheckidlServerGreenChillyExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GreenChillyOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGreenChilly(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckGreenChilly(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckGreenChilly(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerYellowPepperAttr(void)
{
  
  ::YellowPepper* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::YellowPepper* ::idlServer::YellowPepperAttr()");
#endif
  
  ernie.seed("YellowPepperAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("YellowPepperAttrReturn");
  try {
    ReturnValue = tobj->YellowPepperAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::YellowPepperAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckYellowPepper(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerYellowPepperAttrW(void)
{
  
  ::YellowPepper _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::YellowPepperAttr(const ::YellowPepper &_attr)");
#endif
  
  ernie.seed("YellowPepperAttrCall");
  GetYellowPepper(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("YellowPepperAttrReturn");
  try {
     tobj->YellowPepperAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::YellowPepperAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerYellowPepperOp(void)
{
  
  ::YellowPepper* ReturnValue;
  ::YellowPepper p1;
  ::YellowPepper* p2;
  ::YellowPepper p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::YellowPepper* ::idlServer::YellowPepperOp(const ::YellowPepper &p1,::YellowPepper *&p2,::YellowPepper &p3)");
#endif
  
  ernie.seed("YellowPepperOpCall");
  GetYellowPepper(p1);
  GetYellowPepper(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("YellowPepperOpReturn");
  try {
    ReturnValue = tobj->YellowPepperOp(p1,p2,p3);
  }
  catch (::idlServer::YellowPepperExcpt &ex){
    if (!CheckidlServerYellowPepperExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::YellowPepperOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckYellowPepper(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckYellowPepper(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckYellowPepper(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBlueberryAttr(void)
{
  
  ::Blueberry_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Blueberry_slice * ::idlServer::BlueberryAttr()");
#endif
  
  ernie.seed("BlueberryAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlueberryAttrReturn");
  try {
    ReturnValue = tobj->BlueberryAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BlueberryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBlueberry(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBlueberryAttrW(void)
{
  
  ::Blueberry _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BlueberryAttr(const ::Blueberry _attr)");
#endif
  
  ernie.seed("BlueberryAttrCall");
  GetBlueberry(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlueberryAttrReturn");
  try {
     tobj->BlueberryAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BlueberryAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBlueberryOp(void)
{
  
  ::Blueberry_slice * ReturnValue;
  ::Blueberry p1;
  ::Blueberry_slice * p2;
  ::Blueberry p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Blueberry_slice * ::idlServer::BlueberryOp(const ::Blueberry p1,::Blueberry_slice *&p2,::Blueberry p3)");
#endif
  
  ernie.seed("BlueberryOpCall");
  GetBlueberry(p1);
  GetBlueberry(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlueberryOpReturn");
  try {
    ReturnValue = tobj->BlueberryOp(p1,p2,p3);
  }
  catch (::idlServer::BlueberryExcpt &ex){
    if (!CheckidlServerBlueberryExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BlueberryOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBlueberry(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBlueberry(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBlueberry(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


