//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:07:55  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:24  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:27:06  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerScallionAttr(void)
{
  
  ::Scallion* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Scallion* ::idlServer::ScallionAttr()");
#endif
  
  ernie.seed("ScallionAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ScallionAttrReturn");
  try {
    ReturnValue = tobj->ScallionAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ScallionAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckScallion(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerScallionAttrW(void)
{
  
  ::Scallion _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::ScallionAttr(const ::Scallion &_attr)");
#endif
  
  ernie.seed("ScallionAttrCall");
  GetScallion(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ScallionAttrReturn");
  try {
     tobj->ScallionAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ScallionAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerScallionOp(void)
{
  
  ::Scallion* ReturnValue;
  ::Scallion p1;
  ::Scallion* p2;
  ::Scallion p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Scallion* ::idlServer::ScallionOp(const ::Scallion &p1,::Scallion *&p2,::Scallion &p3)");
#endif
  
  ernie.seed("ScallionOpCall");
  GetScallion(p1);
  GetScallion(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ScallionOpReturn");
  try {
    ReturnValue = tobj->ScallionOp(p1,p2,p3);
  }
  catch (::idlServer::ScallionExcpt &ex){
    if (!CheckidlServerScallionExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ScallionOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckScallion(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckScallion(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckScallion(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKiwanoMelonAttr(void)
{
  
  ::KiwanoMelon_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::KiwanoMelon_slice * ::idlServer::KiwanoMelonAttr()");
#endif
  
  ernie.seed("KiwanoMelonAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KiwanoMelonAttrReturn");
  try {
    ReturnValue = tobj->KiwanoMelonAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KiwanoMelonAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKiwanoMelon(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKiwanoMelonAttrW(void)
{
  
  ::KiwanoMelon _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::KiwanoMelonAttr(const ::KiwanoMelon _attr)");
#endif
  
  ernie.seed("KiwanoMelonAttrCall");
  GetKiwanoMelon(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KiwanoMelonAttrReturn");
  try {
     tobj->KiwanoMelonAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KiwanoMelonAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKiwanoMelonOp(void)
{
  
  ::KiwanoMelon_slice * ReturnValue;
  ::KiwanoMelon p1;
  ::KiwanoMelon_slice * p2;
  ::KiwanoMelon p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::KiwanoMelon_slice * ::idlServer::KiwanoMelonOp(const ::KiwanoMelon p1,::KiwanoMelon_slice *&p2,::KiwanoMelon p3)");
#endif
  
  ernie.seed("KiwanoMelonOpCall");
  GetKiwanoMelon(p1);
  GetKiwanoMelon(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KiwanoMelonOpReturn");
  try {
    ReturnValue = tobj->KiwanoMelonOp(p1,p2,p3);
  }
  catch (::idlServer::KiwanoMelonExcpt &ex){
    if (!CheckidlServerKiwanoMelonExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KiwanoMelonOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKiwanoMelon(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckKiwanoMelon(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckKiwanoMelon(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCeleriacAttr(void)
{
  
  ::Celeriac* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Celeriac* ::idlServer::CeleriacAttr()");
#endif
  
  ernie.seed("CeleriacAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CeleriacAttrReturn");
  try {
    ReturnValue = tobj->CeleriacAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CeleriacAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCeleriac(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCeleriacAttrW(void)
{
  
  ::Celeriac _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::CeleriacAttr(const ::Celeriac &_attr)");
#endif
  
  ernie.seed("CeleriacAttrCall");
  GetCeleriac(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CeleriacAttrReturn");
  try {
     tobj->CeleriacAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CeleriacAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCeleriacOp(void)
{
  
  ::Celeriac* ReturnValue;
  ::Celeriac p1;
  ::Celeriac* p2;
  ::Celeriac p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Celeriac* ::idlServer::CeleriacOp(const ::Celeriac &p1,::Celeriac *&p2,::Celeriac &p3)");
#endif
  
  ernie.seed("CeleriacOpCall");
  GetCeleriac(p1);
  GetCeleriac(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CeleriacOpReturn");
  try {
    ReturnValue = tobj->CeleriacOp(p1,p2,p3);
  }
  catch (::idlServer::CeleriacExcpt &ex){
    if (!CheckidlServerCeleriacExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CeleriacOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCeleriac(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckCeleriac(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckCeleriac(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerClementineAttr(void)
{
  
  ::Clementine_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Clementine_slice * ::idlServer::ClementineAttr()");
#endif
  
  ernie.seed("ClementineAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ClementineAttrReturn");
  try {
    ReturnValue = tobj->ClementineAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ClementineAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckClementine(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerClementineAttrW(void)
{
  
  ::Clementine _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::ClementineAttr(const ::Clementine _attr)");
#endif
  
  ernie.seed("ClementineAttrCall");
  GetClementine(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ClementineAttrReturn");
  try {
     tobj->ClementineAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ClementineAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerClementineOp(void)
{
  
  ::Clementine_slice * ReturnValue;
  ::Clementine p1;
  ::Clementine_slice * p2;
  ::Clementine p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Clementine_slice * ::idlServer::ClementineOp(const ::Clementine p1,::Clementine_slice *&p2,::Clementine p3)");
#endif
  
  ernie.seed("ClementineOpCall");
  GetClementine(p1);
  GetClementine(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ClementineOpReturn");
  try {
    ReturnValue = tobj->ClementineOp(p1,p2,p3);
  }
  catch (::idlServer::ClementineExcpt &ex){
    if (!CheckidlServerClementineExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::ClementineOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckClementine(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckClementine(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckClementine(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRecordAttr(void)
{
  
  ::Record* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Record* ::idlServer::RecordAttr()");
#endif
  
  ernie.seed("RecordAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RecordAttrReturn");
  try {
    ReturnValue = tobj->RecordAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RecordAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRecord(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRecordAttrW(void)
{
  
  ::Record _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::RecordAttr(const ::Record &_attr)");
#endif
  
  ernie.seed("RecordAttrCall");
  GetRecord(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RecordAttrReturn");
  try {
     tobj->RecordAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RecordAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRecordOp(void)
{
  
  ::Record* ReturnValue;
  ::Record p1;
  ::Record* p2;
  ::Record p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Record* ::idlServer::RecordOp(const ::Record &p1,::Record *&p2,::Record &p3)");
#endif
  
  ernie.seed("RecordOpCall");
  GetRecord(p1);
  GetRecord(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RecordOpReturn");
  try {
    ReturnValue = tobj->RecordOp(p1,p2,p3);
  }
  catch (::idlServer::RecordExcpt &ex){
    if (!CheckidlServerRecordExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RecordOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRecord(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckRecord(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckRecord(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKerrsPinksAttr(void)
{
  
  ::KerrsPinks_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::KerrsPinks_slice * ::idlServer::KerrsPinksAttr()");
#endif
  
  ernie.seed("KerrsPinksAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KerrsPinksAttrReturn");
  try {
    ReturnValue = tobj->KerrsPinksAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KerrsPinksAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKerrsPinks(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKerrsPinksAttrW(void)
{
  
  ::KerrsPinks _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::KerrsPinksAttr(const ::KerrsPinks _attr)");
#endif
  
  ernie.seed("KerrsPinksAttrCall");
  GetKerrsPinks(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KerrsPinksAttrReturn");
  try {
     tobj->KerrsPinksAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KerrsPinksAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKerrsPinksOp(void)
{
  
  ::KerrsPinks_slice * ReturnValue;
  ::KerrsPinks p1;
  ::KerrsPinks_slice * p2;
  ::KerrsPinks p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::KerrsPinks_slice * ::idlServer::KerrsPinksOp(const ::KerrsPinks p1,::KerrsPinks_slice *&p2,::KerrsPinks p3)");
#endif
  
  ernie.seed("KerrsPinksOpCall");
  GetKerrsPinks(p1);
  GetKerrsPinks(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KerrsPinksOpReturn");
  try {
    ReturnValue = tobj->KerrsPinksOp(p1,p2,p3);
  }
  catch (::idlServer::KerrsPinksExcpt &ex){
    if (!CheckidlServerKerrsPinksExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KerrsPinksOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKerrsPinks(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckKerrsPinks(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckKerrsPinks(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMarisPiperAttr(void)
{
  
  ::MarisPiper* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::MarisPiper* ::idlServer::MarisPiperAttr()");
#endif
  
  ernie.seed("MarisPiperAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MarisPiperAttrReturn");
  try {
    ReturnValue = tobj->MarisPiperAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MarisPiperAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMarisPiper(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMarisPiperAttrW(void)
{
  
  ::MarisPiper _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::MarisPiperAttr(const ::MarisPiper &_attr)");
#endif
  
  ernie.seed("MarisPiperAttrCall");
  GetMarisPiper(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MarisPiperAttrReturn");
  try {
     tobj->MarisPiperAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MarisPiperAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMarisPiperOp(void)
{
  
  ::MarisPiper* ReturnValue;
  ::MarisPiper p1;
  ::MarisPiper* p2;
  ::MarisPiper p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::MarisPiper* ::idlServer::MarisPiperOp(const ::MarisPiper &p1,::MarisPiper *&p2,::MarisPiper &p3)");
#endif
  
  ernie.seed("MarisPiperOpCall");
  GetMarisPiper(p1);
  GetMarisPiper(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MarisPiperOpReturn");
  try {
    ReturnValue = tobj->MarisPiperOp(p1,p2,p3);
  }
  catch (::idlServer::MarisPiperExcpt &ex){
    if (!CheckidlServerMarisPiperExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MarisPiperOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMarisPiper(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckMarisPiper(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckMarisPiper(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBanana1Attr(void)
{
  
  ::Banana1_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Banana1_slice * ::idlServer::Banana1Attr()");
#endif
  
  ernie.seed("Banana1AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Banana1AttrReturn");
  try {
    ReturnValue = tobj->Banana1Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Banana1Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBanana1(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBanana1AttrW(void)
{
  
  ::Banana1 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Banana1Attr(const ::Banana1 _attr)");
#endif
  
  ernie.seed("Banana1AttrCall");
  GetBanana1(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Banana1AttrReturn");
  try {
     tobj->Banana1Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Banana1Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBanana1Op(void)
{
  
  ::Banana1_slice * ReturnValue;
  ::Banana1 p1;
  ::Banana1_slice * p2;
  ::Banana1 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Banana1_slice * ::idlServer::Banana1Op(const ::Banana1 p1,::Banana1_slice *&p2,::Banana1 p3)");
#endif
  
  ernie.seed("Banana1OpCall");
  GetBanana1(p1);
  GetBanana1(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Banana1OpReturn");
  try {
    ReturnValue = tobj->Banana1Op(p1,p2,p3);
  }
  catch (::idlServer::Banana1Excpt &ex){
    if (!CheckidlServerBanana1Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Banana1Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBanana1(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBanana1(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBanana1(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


