//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/cmain.cpp
//
// Modifications:
// $Log: cmain.cpp,v $
// Revision 1.2  2000/03/28 18:26:17  mli
//   o   remove 'CORBA::Environment ev;'
//   o   void (*tet_startup)(void) -> void (*tet_startup)()
//   o   void (*tet_cleanup)(void) -> void (*tet_cleanup)()
//
// Revision 1.1.1.1  2000/02/09 17:16:26  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.7  1998/08/24 11:46:31  james
// ENV updates
//
// Revision 1.6  1997/12/09 11:27:13  james
// Code Regenerated
//
//
//

#include <orb_supp.hh>
#include <syncinfo.hh>
#include <stdlib.h>

#ifndef WIN32
#include <unistd.h>
#else
#include <process.h>
#endif

extern "C" {
#include <tet_api.h>
}

#include "cmain.hh"

//
// This global defines the systems involved in a TET sync event
// Currently the client sync with the proxy to receive the IOR
//
int remSystem[] = { CSW };	

// Our handle to the ORB under test
OrbSupport *orb;

// The stringified IOR passed to us by the proxy
char *stringifiedIOR = 0;

void
clientStartUp()
{
  // Initialise the ORB under test

  try {
      orb = new OrbSupport();
      orb->client_init();
  }
  catch (CORBA::Exception &ex) {
      tet_infoline("client: failed to initialise Orb");
      return;
  }
  
  // Provide space to receive proxified IOR
  struct tet_synmsg msg;
  char rcvbuf[2048];
  
  // Indicate receive mode to TET
  msg.tsm_flags = TET_SMRCVMSG;
  msg.tsm_dlen = sizeof(rcvbuf);
  msg.tsm_data = rcvbuf;
  
  if (tet_remsync(CEVENT, remSystem, 1, VS_TIMEOUT, TET_SV_YES, &msg) != 0)
  {
      tet_printf("client: sync point %d failed", CEVENT);
      return;
  }

  // Cache stringified IOR for method calls to use
  stringifiedIOR = strdup(rcvbuf);

#ifdef DEBUG  
  tet_printf("client: sync point %d OK", CEVENT);
#endif
}

void
clientCleanUp()
{
    delete orb;
    return;
}

#ifdef VSORB_IDL_TESTS
 
extern "C" {
void (*tet_startup)() = 0,
(*tet_cleanup)() = 0;
struct tet_testlist tet_testlist[] = {
      {testIDL,1},
      {NULL,0} };
}

#else

extern "C" {
  void (*tet_startup)() = clientStartUp,
  (*tet_cleanup)() = clientCleanUp;
  struct tet_testlist tet_testlist[] = {
    {testidlServerZucchini1Attr,1},
    {testidlServerZucchini1AttrW,2},
    {testidlServerZucchini1Op,3},
    {testidlServerRedOnion1Attr,4},
    {testidlServerRedOnion1AttrW,5},
    {testidlServerRedOnion1Op,6},
    {testidlServerJuniperBerries1Attr,7},
    {testidlServerJuniperBerries1AttrW,8},
    {testidlServerJuniperBerries1Op,9},
    {testidlServerTarragon1Attr,10},
    {testidlServerTarragon1AttrW,11},
    {testidlServerTarragon1Op,12},
    {testidlServerElderberry1Attr,13},
    {testidlServerElderberry1AttrW,14},
    {testidlServerElderberry1Op,15},
    {testidlServerPlantain1Attr,16},
    {testidlServerPlantain1AttrW,17},
    {testidlServerPlantain1Op,18},
    {testidlServerSharanfruit1Attr,19},
    {testidlServerSharanfruit1AttrW,20},
    {testidlServerSharanfruit1Op,21},
    {testidlServerDamsons1Attr,22},
    {testidlServerDamsons1AttrW,23},
    {testidlServerDamsons1Op,24},
    {testidlServerEddo1Attr,25},
    {testidlServerEddo1AttrW,26},
    {testidlServerEddo1Op,27},
    {testidlServerCoxPippin1Attr,28},
    {testidlServerCoxPippin1AttrW,29},
    {testidlServerCoxPippin1Op,30},
    {testidlServerKiwanoMelon1Attr,31},
    {testidlServerKiwanoMelon1AttrW,32},
    {testidlServerKiwanoMelon1Op,33},
    {testidlServerRedCabbage1Attr,34},
    {testidlServerRedCabbage1AttrW,35},
    {testidlServerRedCabbage1Op,36},
    {testidlServerPimpernel1Attr,37},
    {testidlServerPimpernel1AttrW,38},
    {testidlServerPimpernel1Op,39},
    {testidlServerBlackCherry2Attr,40},
    {testidlServerBlackCherry2AttrW,41},
    {testidlServerBlackCherry2Op,42},
    {testidlServerCucumber2Attr,43},
    {testidlServerCucumber2AttrW,44},
    {testidlServerCucumber2Op,45},
    {NULL,0} };
}

#endif
