//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:08:42  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:27  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:27:18  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerParsnip2Attr(void)
{
  
  ::Parsnip2_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Parsnip2_slice * ::idlServer::Parsnip2Attr()");
#endif
  
  ernie.seed("Parsnip2AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsnip2AttrReturn");
  try {
    ReturnValue = tobj->Parsnip2Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Parsnip2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckParsnip2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerParsnip2AttrW(void)
{
  
  ::Parsnip2 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Parsnip2Attr(const ::Parsnip2 _attr)");
#endif
  
  ernie.seed("Parsnip2AttrCall");
  GetParsnip2(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsnip2AttrReturn");
  try {
     tobj->Parsnip2Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Parsnip2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerParsnip2Op(void)
{
  
  ::Parsnip2_slice * ReturnValue;
  ::Parsnip2 p1;
  ::Parsnip2_slice * p2;
  ::Parsnip2 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Parsnip2_slice * ::idlServer::Parsnip2Op(const ::Parsnip2 p1,::Parsnip2_slice *&p2,::Parsnip2 p3)");
#endif
  
  ernie.seed("Parsnip2OpCall");
  GetParsnip2(p1);
  GetParsnip2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsnip2OpReturn");
  try {
    ReturnValue = tobj->Parsnip2Op(p1,p2,p3);
  }
  catch (::idlServer::Parsnip2Excpt &ex){
    if (!CheckidlServerParsnip2Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Parsnip2Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckParsnip2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckParsnip2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckParsnip2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPrunes2Attr(void)
{
  
  ::Prunes2* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Prunes2* ::idlServer::Prunes2Attr()");
#endif
  
  ernie.seed("Prunes2AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Prunes2AttrReturn");
  try {
    ReturnValue = tobj->Prunes2Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Prunes2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPrunes2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPrunes2AttrW(void)
{
  
  ::Prunes2 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Prunes2Attr(const ::Prunes2 &_attr)");
#endif
  
  ernie.seed("Prunes2AttrCall");
  GetPrunes2(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Prunes2AttrReturn");
  try {
     tobj->Prunes2Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Prunes2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPrunes2Op(void)
{
  
  ::Prunes2* ReturnValue;
  ::Prunes2 p1;
  ::Prunes2* p2;
  ::Prunes2 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Prunes2* ::idlServer::Prunes2Op(const ::Prunes2 &p1,::Prunes2 *&p2,::Prunes2 &p3)");
#endif
  
  ernie.seed("Prunes2OpCall");
  GetPrunes2(p1);
  GetPrunes2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Prunes2OpReturn");
  try {
    ReturnValue = tobj->Prunes2Op(p1,p2,p3);
  }
  catch (::idlServer::Prunes2Excpt &ex){
    if (!CheckidlServerPrunes2Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Prunes2Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPrunes2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckPrunes2(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckPrunes2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPlum2Attr(void)
{
  
  ::Plum2_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Plum2_slice * ::idlServer::Plum2Attr()");
#endif
  
  ernie.seed("Plum2AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Plum2AttrReturn");
  try {
    ReturnValue = tobj->Plum2Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Plum2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPlum2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPlum2AttrW(void)
{
  
  ::Plum2 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Plum2Attr(const ::Plum2 _attr)");
#endif
  
  ernie.seed("Plum2AttrCall");
  GetPlum2(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Plum2AttrReturn");
  try {
     tobj->Plum2Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Plum2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPlum2Op(void)
{
  
  ::Plum2_slice * ReturnValue;
  ::Plum2 p1;
  ::Plum2_slice * p2;
  ::Plum2 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Plum2_slice * ::idlServer::Plum2Op(const ::Plum2 p1,::Plum2_slice *&p2,::Plum2 p3)");
#endif
  
  ernie.seed("Plum2OpCall");
  GetPlum2(p1);
  GetPlum2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Plum2OpReturn");
  try {
    ReturnValue = tobj->Plum2Op(p1,p2,p3);
  }
  catch (::idlServer::Plum2Excpt &ex){
    if (!CheckidlServerPlum2Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Plum2Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPlum2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckPlum2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckPlum2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerEggPlant2Attr(void)
{
  
  ::EggPlant2* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::EggPlant2* ::idlServer::EggPlant2Attr()");
#endif
  
  ernie.seed("EggPlant2AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("EggPlant2AttrReturn");
  try {
    ReturnValue = tobj->EggPlant2Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::EggPlant2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckEggPlant2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerEggPlant2AttrW(void)
{
  
  ::EggPlant2 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::EggPlant2Attr(const ::EggPlant2 &_attr)");
#endif
  
  ernie.seed("EggPlant2AttrCall");
  GetEggPlant2(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("EggPlant2AttrReturn");
  try {
     tobj->EggPlant2Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::EggPlant2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerEggPlant2Op(void)
{
  
  ::EggPlant2* ReturnValue;
  ::EggPlant2 p1;
  ::EggPlant2* p2;
  ::EggPlant2 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::EggPlant2* ::idlServer::EggPlant2Op(const ::EggPlant2 &p1,::EggPlant2 *&p2,::EggPlant2 &p3)");
#endif
  
  ernie.seed("EggPlant2OpCall");
  GetEggPlant2(p1);
  GetEggPlant2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("EggPlant2OpReturn");
  try {
    ReturnValue = tobj->EggPlant2Op(p1,p2,p3);
  }
  catch (::idlServer::EggPlant2Excpt &ex){
    if (!CheckidlServerEggPlant2Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::EggPlant2Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckEggPlant2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckEggPlant2(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckEggPlant2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMustard2Attr(void)
{
  
  ::Mustard2* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mustard2* ::idlServer::Mustard2Attr()");
#endif
  
  ernie.seed("Mustard2AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mustard2AttrReturn");
  try {
    ReturnValue = tobj->Mustard2Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Mustard2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMustard2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMustard2AttrW(void)
{
  
  ::Mustard2 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Mustard2Attr(const ::Mustard2 &_attr)");
#endif
  
  ernie.seed("Mustard2AttrCall");
  GetMustard2(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mustard2AttrReturn");
  try {
     tobj->Mustard2Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Mustard2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMustard2Op(void)
{
  
  ::Mustard2* ReturnValue;
  ::Mustard2 p1;
  ::Mustard2* p2;
  ::Mustard2 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mustard2* ::idlServer::Mustard2Op(const ::Mustard2 &p1,::Mustard2 *&p2,::Mustard2 &p3)");
#endif
  
  ernie.seed("Mustard2OpCall");
  GetMustard2(p1);
  GetMustard2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mustard2OpReturn");
  try {
    ReturnValue = tobj->Mustard2Op(p1,p2,p3);
  }
  catch (::idlServer::Mustard2Excpt &ex){
    if (!CheckidlServerMustard2Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Mustard2Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMustard2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckMustard2(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckMustard2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerElderberry2Attr(void)
{
  
  ::Elderberry2_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Elderberry2_slice * ::idlServer::Elderberry2Attr()");
#endif
  
  ernie.seed("Elderberry2AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Elderberry2AttrReturn");
  try {
    ReturnValue = tobj->Elderberry2Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Elderberry2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckElderberry2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerElderberry2AttrW(void)
{
  
  ::Elderberry2 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Elderberry2Attr(const ::Elderberry2 _attr)");
#endif
  
  ernie.seed("Elderberry2AttrCall");
  GetElderberry2(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Elderberry2AttrReturn");
  try {
     tobj->Elderberry2Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Elderberry2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerElderberry2Op(void)
{
  
  ::Elderberry2_slice * ReturnValue;
  ::Elderberry2 p1;
  ::Elderberry2_slice * p2;
  ::Elderberry2 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Elderberry2_slice * ::idlServer::Elderberry2Op(const ::Elderberry2 p1,::Elderberry2_slice *&p2,::Elderberry2 p3)");
#endif
  
  ernie.seed("Elderberry2OpCall");
  GetElderberry2(p1);
  GetElderberry2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Elderberry2OpReturn");
  try {
    ReturnValue = tobj->Elderberry2Op(p1,p2,p3);
  }
  catch (::idlServer::Elderberry2Excpt &ex){
    if (!CheckidlServerElderberry2Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Elderberry2Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckElderberry2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckElderberry2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckElderberry2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRedPepper2Attr(void)
{
  
  ::RedPepper2* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::RedPepper2* ::idlServer::RedPepper2Attr()");
#endif
  
  ernie.seed("RedPepper2AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedPepper2AttrReturn");
  try {
    ReturnValue = tobj->RedPepper2Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RedPepper2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRedPepper2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRedPepper2AttrW(void)
{
  
  ::RedPepper2 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::RedPepper2Attr(const ::RedPepper2 &_attr)");
#endif
  
  ernie.seed("RedPepper2AttrCall");
  GetRedPepper2(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedPepper2AttrReturn");
  try {
     tobj->RedPepper2Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RedPepper2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRedPepper2Op(void)
{
  
  ::RedPepper2* ReturnValue;
  ::RedPepper2 p1;
  ::RedPepper2* p2;
  ::RedPepper2 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::RedPepper2* ::idlServer::RedPepper2Op(const ::RedPepper2 &p1,::RedPepper2 *&p2,::RedPepper2 &p3)");
#endif
  
  ernie.seed("RedPepper2OpCall");
  GetRedPepper2(p1);
  GetRedPepper2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedPepper2OpReturn");
  try {
    ReturnValue = tobj->RedPepper2Op(p1,p2,p3);
  }
  catch (::idlServer::RedPepper2Excpt &ex){
    if (!CheckidlServerRedPepper2Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RedPepper2Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRedPepper2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckRedPepper2(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckRedPepper2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerArtichokes2Attr(void)
{
  
  ::Artichokes2_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Artichokes2_slice * ::idlServer::Artichokes2Attr()");
#endif
  
  ernie.seed("Artichokes2AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Artichokes2AttrReturn");
  try {
    ReturnValue = tobj->Artichokes2Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Artichokes2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckArtichokes2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerArtichokes2AttrW(void)
{
  
  ::Artichokes2 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Artichokes2Attr(const ::Artichokes2 _attr)");
#endif
  
  ernie.seed("Artichokes2AttrCall");
  GetArtichokes2(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Artichokes2AttrReturn");
  try {
     tobj->Artichokes2Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Artichokes2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerArtichokes2Op(void)
{
  
  ::Artichokes2_slice * ReturnValue;
  ::Artichokes2 p1;
  ::Artichokes2_slice * p2;
  ::Artichokes2 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Artichokes2_slice * ::idlServer::Artichokes2Op(const ::Artichokes2 p1,::Artichokes2_slice *&p2,::Artichokes2 p3)");
#endif
  
  ernie.seed("Artichokes2OpCall");
  GetArtichokes2(p1);
  GetArtichokes2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Artichokes2OpReturn");
  try {
    ReturnValue = tobj->Artichokes2Op(p1,p2,p3);
  }
  catch (::idlServer::Artichokes2Excpt &ex){
    if (!CheckidlServerArtichokes2Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Artichokes2Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckArtichokes2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckArtichokes2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckArtichokes2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKiwanoMelon2Attr(void)
{
  
  ::KiwanoMelon2_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::KiwanoMelon2_slice * ::idlServer::KiwanoMelon2Attr()");
#endif
  
  ernie.seed("KiwanoMelon2AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KiwanoMelon2AttrReturn");
  try {
    ReturnValue = tobj->KiwanoMelon2Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KiwanoMelon2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKiwanoMelon2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKiwanoMelon2AttrW(void)
{
  
  ::KiwanoMelon2 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::KiwanoMelon2Attr(const ::KiwanoMelon2 _attr)");
#endif
  
  ernie.seed("KiwanoMelon2AttrCall");
  GetKiwanoMelon2(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KiwanoMelon2AttrReturn");
  try {
     tobj->KiwanoMelon2Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KiwanoMelon2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKiwanoMelon2Op(void)
{
  
  ::KiwanoMelon2_slice * ReturnValue;
  ::KiwanoMelon2 p1;
  ::KiwanoMelon2_slice * p2;
  ::KiwanoMelon2 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::KiwanoMelon2_slice * ::idlServer::KiwanoMelon2Op(const ::KiwanoMelon2 p1,::KiwanoMelon2_slice *&p2,::KiwanoMelon2 p3)");
#endif
  
  ernie.seed("KiwanoMelon2OpCall");
  GetKiwanoMelon2(p1);
  GetKiwanoMelon2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KiwanoMelon2OpReturn");
  try {
    ReturnValue = tobj->KiwanoMelon2Op(p1,p2,p3);
  }
  catch (::idlServer::KiwanoMelon2Excpt &ex){
    if (!CheckidlServerKiwanoMelon2Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KiwanoMelon2Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKiwanoMelon2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckKiwanoMelon2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckKiwanoMelon2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBritishQueens2Attr(void)
{
  
  ::BritishQueens2* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BritishQueens2* ::idlServer::BritishQueens2Attr()");
#endif
  
  ernie.seed("BritishQueens2AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BritishQueens2AttrReturn");
  try {
    ReturnValue = tobj->BritishQueens2Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BritishQueens2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBritishQueens2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBritishQueens2AttrW(void)
{
  
  ::BritishQueens2 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BritishQueens2Attr(const ::BritishQueens2 &_attr)");
#endif
  
  ernie.seed("BritishQueens2AttrCall");
  GetBritishQueens2(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BritishQueens2AttrReturn");
  try {
     tobj->BritishQueens2Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BritishQueens2Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBritishQueens2Op(void)
{
  
  ::BritishQueens2* ReturnValue;
  ::BritishQueens2 p1;
  ::BritishQueens2* p2;
  ::BritishQueens2 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BritishQueens2* ::idlServer::BritishQueens2Op(const ::BritishQueens2 &p1,::BritishQueens2 *&p2,::BritishQueens2 &p3)");
#endif
  
  ernie.seed("BritishQueens2OpCall");
  GetBritishQueens2(p1);
  GetBritishQueens2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BritishQueens2OpReturn");
  try {
    ReturnValue = tobj->BritishQueens2Op(p1,p2,p3);
  }
  catch (::idlServer::BritishQueens2Excpt &ex){
    if (!CheckidlServerBritishQueens2Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BritishQueens2Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBritishQueens2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBritishQueens2(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBritishQueens2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerStarfruit3Attr(void)
{
  
  ::Starfruit3_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Starfruit3_slice * ::idlServer::Starfruit3Attr()");
#endif
  
  ernie.seed("Starfruit3AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Starfruit3AttrReturn");
  try {
    ReturnValue = tobj->Starfruit3Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Starfruit3Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckStarfruit3(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerStarfruit3AttrW(void)
{
  
  ::Starfruit3 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Starfruit3Attr(const ::Starfruit3 _attr)");
#endif
  
  ernie.seed("Starfruit3AttrCall");
  GetStarfruit3(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Starfruit3AttrReturn");
  try {
     tobj->Starfruit3Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Starfruit3Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerStarfruit3Op(void)
{
  
  ::Starfruit3_slice * ReturnValue;
  ::Starfruit3 p1;
  ::Starfruit3_slice * p2;
  ::Starfruit3 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Starfruit3_slice * ::idlServer::Starfruit3Op(const ::Starfruit3 p1,::Starfruit3_slice *&p2,::Starfruit3 p3)");
#endif
  
  ernie.seed("Starfruit3OpCall");
  GetStarfruit3(p1);
  GetStarfruit3(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Starfruit3OpReturn");
  try {
    ReturnValue = tobj->Starfruit3Op(p1,p2,p3);
  }
  catch (::idlServer::Starfruit3Excpt &ex){
    if (!CheckidlServerStarfruit3Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Starfruit3Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckStarfruit3(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckStarfruit3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckStarfruit3(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerParsnip3Attr(void)
{
  
  ::Parsnip3_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Parsnip3_slice * ::idlServer::Parsnip3Attr()");
#endif
  
  ernie.seed("Parsnip3AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsnip3AttrReturn");
  try {
    ReturnValue = tobj->Parsnip3Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Parsnip3Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckParsnip3(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerParsnip3AttrW(void)
{
  
  ::Parsnip3 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Parsnip3Attr(const ::Parsnip3 _attr)");
#endif
  
  ernie.seed("Parsnip3AttrCall");
  GetParsnip3(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsnip3AttrReturn");
  try {
     tobj->Parsnip3Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Parsnip3Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerParsnip3Op(void)
{
  
  ::Parsnip3_slice * ReturnValue;
  ::Parsnip3 p1;
  ::Parsnip3_slice * p2;
  ::Parsnip3 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Parsnip3_slice * ::idlServer::Parsnip3Op(const ::Parsnip3 p1,::Parsnip3_slice *&p2,::Parsnip3 p3)");
#endif
  
  ernie.seed("Parsnip3OpCall");
  GetParsnip3(p1);
  GetParsnip3(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsnip3OpReturn");
  try {
    ReturnValue = tobj->Parsnip3Op(p1,p2,p3);
  }
  catch (::idlServer::Parsnip3Excpt &ex){
    if (!CheckidlServerParsnip3Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Parsnip3Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckParsnip3(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckParsnip3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckParsnip3(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBillberries3Attr(void)
{
  
  ::Billberries3* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Billberries3* ::idlServer::Billberries3Attr()");
#endif
  
  ernie.seed("Billberries3AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Billberries3AttrReturn");
  try {
    ReturnValue = tobj->Billberries3Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Billberries3Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBillberries3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBillberries3AttrW(void)
{
  
  ::Billberries3 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Billberries3Attr(const ::Billberries3 &_attr)");
#endif
  
  ernie.seed("Billberries3AttrCall");
  GetBillberries3(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Billberries3AttrReturn");
  try {
     tobj->Billberries3Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Billberries3Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBillberries3Op(void)
{
  
  ::Billberries3* ReturnValue;
  ::Billberries3 p1;
  ::Billberries3* p2;
  ::Billberries3 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Billberries3* ::idlServer::Billberries3Op(const ::Billberries3 &p1,::Billberries3 *&p2,::Billberries3 &p3)");
#endif
  
  ernie.seed("Billberries3OpCall");
  GetBillberries3(p1);
  GetBillberries3(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Billberries3OpReturn");
  try {
    ReturnValue = tobj->Billberries3Op(p1,p2,p3);
  }
  catch (::idlServer::Billberries3Excpt &ex){
    if (!CheckidlServerBillberries3Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Billberries3Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBillberries3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBillberries3(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBillberries3(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGooseBerry3Attr(void)
{
  
  ::GooseBerry3_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GooseBerry3_slice * ::idlServer::GooseBerry3Attr()");
#endif
  
  ernie.seed("GooseBerry3AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GooseBerry3AttrReturn");
  try {
    ReturnValue = tobj->GooseBerry3Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GooseBerry3Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGooseBerry3(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGooseBerry3AttrW(void)
{
  
  ::GooseBerry3 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::GooseBerry3Attr(const ::GooseBerry3 _attr)");
#endif
  
  ernie.seed("GooseBerry3AttrCall");
  GetGooseBerry3(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GooseBerry3AttrReturn");
  try {
     tobj->GooseBerry3Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GooseBerry3Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGooseBerry3Op(void)
{
  
  ::GooseBerry3_slice * ReturnValue;
  ::GooseBerry3 p1;
  ::GooseBerry3_slice * p2;
  ::GooseBerry3 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GooseBerry3_slice * ::idlServer::GooseBerry3Op(const ::GooseBerry3 p1,::GooseBerry3_slice *&p2,::GooseBerry3 p3)");
#endif
  
  ernie.seed("GooseBerry3OpCall");
  GetGooseBerry3(p1);
  GetGooseBerry3(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GooseBerry3OpReturn");
  try {
    ReturnValue = tobj->GooseBerry3Op(p1,p2,p3);
  }
  catch (::idlServer::GooseBerry3Excpt &ex){
    if (!CheckidlServerGooseBerry3Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GooseBerry3Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGooseBerry3(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckGooseBerry3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckGooseBerry3(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerElderberry3Attr(void)
{
  
  ::Elderberry3_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Elderberry3_slice * ::idlServer::Elderberry3Attr()");
#endif
  
  ernie.seed("Elderberry3AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Elderberry3AttrReturn");
  try {
    ReturnValue = tobj->Elderberry3Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Elderberry3Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckElderberry3(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerElderberry3AttrW(void)
{
  
  ::Elderberry3 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Elderberry3Attr(const ::Elderberry3 _attr)");
#endif
  
  ernie.seed("Elderberry3AttrCall");
  GetElderberry3(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Elderberry3AttrReturn");
  try {
     tobj->Elderberry3Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Elderberry3Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerElderberry3Op(void)
{
  
  ::Elderberry3_slice * ReturnValue;
  ::Elderberry3 p1;
  ::Elderberry3_slice * p2;
  ::Elderberry3 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Elderberry3_slice * ::idlServer::Elderberry3Op(const ::Elderberry3 p1,::Elderberry3_slice *&p2,::Elderberry3 p3)");
#endif
  
  ernie.seed("Elderberry3OpCall");
  GetElderberry3(p1);
  GetElderberry3(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Elderberry3OpReturn");
  try {
    ReturnValue = tobj->Elderberry3Op(p1,p2,p3);
  }
  catch (::idlServer::Elderberry3Excpt &ex){
    if (!CheckidlServerElderberry3Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Elderberry3Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckElderberry3(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckElderberry3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckElderberry3(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


