//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:09:04  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:30  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.5  1997/12/12 08:50:59  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
TestServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = TestServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testTestServerLoganBerry19Attr(void)
{
  
  ::LoganBerry19 ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::LoganBerry19 ::TestServer::LoganBerry19Attr()");
#endif
  
  ernie.seed("LoganBerry19AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerry19AttrReturn");
  try {
    ReturnValue = tobj->LoganBerry19Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::LoganBerry19Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLoganBerry19(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerLoganBerry19AttrW(void)
{
  
  ::LoganBerry19 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::LoganBerry19Attr(const ::LoganBerry19 &_attr)");
#endif
  
  ernie.seed("LoganBerry19AttrCall");
  GetLoganBerry19(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerry19AttrReturn");
  try {
     tobj->LoganBerry19Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::LoganBerry19Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerLoganBerry19Op(void)
{
  
  ::LoganBerry19 ReturnValue;
  ::LoganBerry19 p1;
  ::LoganBerry19 p2;
  ::LoganBerry19 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::LoganBerry19 ::TestServer::LoganBerry19Op(const ::LoganBerry19 &p1,::LoganBerry19 &p2,::LoganBerry19 &p3)");
#endif
  
  ernie.seed("LoganBerry19OpCall");
  GetLoganBerry19(p1);
  GetLoganBerry19(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerry19OpReturn");
  try {
    ReturnValue = tobj->LoganBerry19Op(p1,p2,p3);
  }
  catch (::TestServer::LoganBerry19Excpt &ex){
    if (!CheckTestServerLoganBerry19Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::LoganBerry19Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLoganBerry19(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckLoganBerry19(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckLoganBerry19(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerOrange19Attr(void)
{
  
  ::Orange19 ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Orange19 ::TestServer::Orange19Attr()");
#endif
  
  ernie.seed("Orange19AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Orange19AttrReturn");
  try {
    ReturnValue = tobj->Orange19Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Orange19Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOrange19(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerOrange19AttrW(void)
{
  
  ::Orange19 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::Orange19Attr(const ::Orange19 &_attr)");
#endif
  
  ernie.seed("Orange19AttrCall");
  GetOrange19(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Orange19AttrReturn");
  try {
     tobj->Orange19Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Orange19Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerOrange19Op(void)
{
  
  ::Orange19 ReturnValue;
  ::Orange19 p1;
  ::Orange19 p2;
  ::Orange19 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Orange19 ::TestServer::Orange19Op(const ::Orange19 &p1,::Orange19 &p2,::Orange19 &p3)");
#endif
  
  ernie.seed("Orange19OpCall");
  GetOrange19(p1);
  GetOrange19(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Orange19OpReturn");
  try {
    ReturnValue = tobj->Orange19Op(p1,p2,p3);
  }
  catch (::TestServer::Orange19Excpt &ex){
    if (!CheckTestServerOrange19Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Orange19Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOrange19(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckOrange19(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckOrange19(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerBlackCherry20Attr(void)
{
  
  ::BlackCherry20 ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BlackCherry20 ::TestServer::BlackCherry20Attr()");
#endif
  
  ernie.seed("BlackCherry20AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackCherry20AttrReturn");
  try {
    ReturnValue = tobj->BlackCherry20Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::BlackCherry20Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBlackCherry20(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerBlackCherry20AttrW(void)
{
  
  ::BlackCherry20 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::BlackCherry20Attr(const ::BlackCherry20 &_attr)");
#endif
  
  ernie.seed("BlackCherry20AttrCall");
  GetBlackCherry20(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackCherry20AttrReturn");
  try {
     tobj->BlackCherry20Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::BlackCherry20Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerBlackCherry20Op(void)
{
  
  ::BlackCherry20 ReturnValue;
  ::BlackCherry20 p1;
  ::BlackCherry20 p2;
  ::BlackCherry20 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BlackCherry20 ::TestServer::BlackCherry20Op(const ::BlackCherry20 &p1,::BlackCherry20 &p2,::BlackCherry20 &p3)");
#endif
  
  ernie.seed("BlackCherry20OpCall");
  GetBlackCherry20(p1);
  GetBlackCherry20(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackCherry20OpReturn");
  try {
    ReturnValue = tobj->BlackCherry20Op(p1,p2,p3);
  }
  catch (::TestServer::BlackCherry20Excpt &ex){
    if (!CheckTestServerBlackCherry20Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::BlackCherry20Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBlackCherry20(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBlackCherry20(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBlackCherry20(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerLemon20Attr(void)
{
  
  ::Lemon20 ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lemon20 ::TestServer::Lemon20Attr()");
#endif
  
  ernie.seed("Lemon20AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lemon20AttrReturn");
  try {
    ReturnValue = tobj->Lemon20Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Lemon20Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLemon20(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerLemon20AttrW(void)
{
  
  ::Lemon20 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::Lemon20Attr(const ::Lemon20 &_attr)");
#endif
  
  ernie.seed("Lemon20AttrCall");
  GetLemon20(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lemon20AttrReturn");
  try {
     tobj->Lemon20Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Lemon20Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerLemon20Op(void)
{
  
  ::Lemon20 ReturnValue;
  ::Lemon20 p1;
  ::Lemon20 p2;
  ::Lemon20 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lemon20 ::TestServer::Lemon20Op(const ::Lemon20 &p1,::Lemon20 &p2,::Lemon20 &p3)");
#endif
  
  ernie.seed("Lemon20OpCall");
  GetLemon20(p1);
  GetLemon20(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lemon20OpReturn");
  try {
    ReturnValue = tobj->Lemon20Op(p1,p2,p3);
  }
  catch (::TestServer::Lemon20Excpt &ex){
    if (!CheckTestServerLemon20Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Lemon20Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLemon20(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckLemon20(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckLemon20(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerCoconut6Attr(void)
{
  
  ::Coconut6_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Coconut6_slice * ::TestServer::Coconut6Attr()");
#endif
  
  ernie.seed("Coconut6AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Coconut6AttrReturn");
  try {
    ReturnValue = tobj->Coconut6Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Coconut6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCoconut6(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerCoconut6AttrW(void)
{
  
  ::Coconut6 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::Coconut6Attr(const ::Coconut6 _attr)");
#endif
  
  ernie.seed("Coconut6AttrCall");
  GetCoconut6(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Coconut6AttrReturn");
  try {
     tobj->Coconut6Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Coconut6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerCoconut6Op(void)
{
  
  ::Coconut6_slice * ReturnValue;
  ::Coconut6 p1;
  ::Coconut6 p2;
  ::Coconut6 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Coconut6_slice * ::TestServer::Coconut6Op(const ::Coconut6 p1,::Coconut6 p2,::Coconut6 p3)");
#endif
  
  ernie.seed("Coconut6OpCall");
  GetCoconut6(p1);
  GetCoconut6(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Coconut6OpReturn");
  try {
    ReturnValue = tobj->Coconut6Op(p1,p2,p3);
  }
  catch (::TestServer::Coconut6Excpt &ex){
    if (!CheckTestServerCoconut6Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Coconut6Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCoconut6(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckCoconut6(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckCoconut6(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerLime6Attr(void)
{
  
  ::Lime6 ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lime6 ::TestServer::Lime6Attr()");
#endif
  
  ernie.seed("Lime6AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lime6AttrReturn");
  try {
    ReturnValue = tobj->Lime6Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Lime6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLime6(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerLime6AttrW(void)
{
  
  ::Lime6 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::Lime6Attr(const ::Lime6 &_attr)");
#endif
  
  ernie.seed("Lime6AttrCall");
  GetLime6(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lime6AttrReturn");
  try {
     tobj->Lime6Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Lime6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerLime6Op(void)
{
  
  ::Lime6 ReturnValue;
  ::Lime6 p1;
  ::Lime6 p2;
  ::Lime6 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lime6 ::TestServer::Lime6Op(const ::Lime6 &p1,::Lime6 &p2,::Lime6 &p3)");
#endif
  
  ernie.seed("Lime6OpCall");
  GetLime6(p1);
  GetLime6(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lime6OpReturn");
  try {
    ReturnValue = tobj->Lime6Op(p1,p2,p3);
  }
  catch (::TestServer::Lime6Excpt &ex){
    if (!CheckTestServerLime6Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Lime6Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLime6(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckLime6(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckLime6(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerMint6Attr(void)
{
  
  ::Mint6_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mint6_slice * ::TestServer::Mint6Attr()");
#endif
  
  ernie.seed("Mint6AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mint6AttrReturn");
  try {
    ReturnValue = tobj->Mint6Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Mint6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMint6(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerMint6AttrW(void)
{
  
  ::Mint6 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::Mint6Attr(const ::Mint6 _attr)");
#endif
  
  ernie.seed("Mint6AttrCall");
  GetMint6(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mint6AttrReturn");
  try {
     tobj->Mint6Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Mint6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerMint6Op(void)
{
  
  ::Mint6_slice * ReturnValue;
  ::Mint6 p1;
  ::Mint6_slice * p2;
  ::Mint6 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mint6_slice * ::TestServer::Mint6Op(const ::Mint6 p1,::Mint6_slice *&p2,::Mint6 p3)");
#endif
  
  ernie.seed("Mint6OpCall");
  GetMint6(p1);
  GetMint6(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mint6OpReturn");
  try {
    ReturnValue = tobj->Mint6Op(p1,p2,p3);
  }
  catch (::TestServer::Mint6Excpt &ex){
    if (!CheckTestServerMint6Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Mint6Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMint6(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckMint6(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckMint6(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerBlackEyedBeans6Attr(void)
{
  
  ::BlackEyedBeans6* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BlackEyedBeans6* ::TestServer::BlackEyedBeans6Attr()");
#endif
  
  ernie.seed("BlackEyedBeans6AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackEyedBeans6AttrReturn");
  try {
    ReturnValue = tobj->BlackEyedBeans6Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::BlackEyedBeans6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBlackEyedBeans6(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerBlackEyedBeans6AttrW(void)
{
  
  ::BlackEyedBeans6 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::BlackEyedBeans6Attr(const ::BlackEyedBeans6 &_attr)");
#endif
  
  ernie.seed("BlackEyedBeans6AttrCall");
  GetBlackEyedBeans6(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackEyedBeans6AttrReturn");
  try {
     tobj->BlackEyedBeans6Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::BlackEyedBeans6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerBlackEyedBeans6Op(void)
{
  
  ::BlackEyedBeans6* ReturnValue;
  ::BlackEyedBeans6 p1;
  ::BlackEyedBeans6* p2;
  ::BlackEyedBeans6 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BlackEyedBeans6* ::TestServer::BlackEyedBeans6Op(const ::BlackEyedBeans6 &p1,::BlackEyedBeans6 *&p2,::BlackEyedBeans6 &p3)");
#endif
  
  ernie.seed("BlackEyedBeans6OpCall");
  GetBlackEyedBeans6(p1);
  GetBlackEyedBeans6(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackEyedBeans6OpReturn");
  try {
    ReturnValue = tobj->BlackEyedBeans6Op(p1,p2,p3);
  }
  catch (::TestServer::BlackEyedBeans6Excpt &ex){
    if (!CheckTestServerBlackEyedBeans6Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::BlackEyedBeans6Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBlackEyedBeans6(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBlackEyedBeans6(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBlackEyedBeans6(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerMangetout6Attr(void)
{
  
  ::Mangetout6* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mangetout6* ::TestServer::Mangetout6Attr()");
#endif
  
  ernie.seed("Mangetout6AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mangetout6AttrReturn");
  try {
    ReturnValue = tobj->Mangetout6Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Mangetout6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMangetout6(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerMangetout6AttrW(void)
{
  
  ::Mangetout6 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::Mangetout6Attr(const ::Mangetout6 &_attr)");
#endif
  
  ernie.seed("Mangetout6AttrCall");
  GetMangetout6(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mangetout6AttrReturn");
  try {
     tobj->Mangetout6Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Mangetout6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerMangetout6Op(void)
{
  
  ::Mangetout6* ReturnValue;
  ::Mangetout6 p1;
  ::Mangetout6* p2;
  ::Mangetout6 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mangetout6* ::TestServer::Mangetout6Op(const ::Mangetout6 &p1,::Mangetout6 *&p2,::Mangetout6 &p3)");
#endif
  
  ernie.seed("Mangetout6OpCall");
  GetMangetout6(p1);
  GetMangetout6(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mangetout6OpReturn");
  try {
    ReturnValue = tobj->Mangetout6Op(p1,p2,p3);
  }
  catch (::TestServer::Mangetout6Excpt &ex){
    if (!CheckTestServerMangetout6Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Mangetout6Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMangetout6(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckMangetout6(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckMangetout6(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerLettuce6Attr(void)
{
  
  ::Lettuce6* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lettuce6* ::TestServer::Lettuce6Attr()");
#endif
  
  ernie.seed("Lettuce6AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lettuce6AttrReturn");
  try {
    ReturnValue = tobj->Lettuce6Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Lettuce6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLettuce6(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerLettuce6AttrW(void)
{
  
  ::Lettuce6 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::Lettuce6Attr(const ::Lettuce6 &_attr)");
#endif
  
  ernie.seed("Lettuce6AttrCall");
  GetLettuce6(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lettuce6AttrReturn");
  try {
     tobj->Lettuce6Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Lettuce6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerLettuce6Op(void)
{
  
  ::Lettuce6* ReturnValue;
  ::Lettuce6 p1;
  ::Lettuce6* p2;
  ::Lettuce6 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lettuce6* ::TestServer::Lettuce6Op(const ::Lettuce6 &p1,::Lettuce6 *&p2,::Lettuce6 &p3)");
#endif
  
  ernie.seed("Lettuce6OpCall");
  GetLettuce6(p1);
  GetLettuce6(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lettuce6OpReturn");
  try {
    ReturnValue = tobj->Lettuce6Op(p1,p2,p3);
  }
  catch (::TestServer::Lettuce6Excpt &ex){
    if (!CheckTestServerLettuce6Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Lettuce6Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLettuce6(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckLettuce6(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckLettuce6(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerChervil6Attr(void)
{
  
  ::Chervil6_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Chervil6_slice * ::TestServer::Chervil6Attr()");
#endif
  
  ernie.seed("Chervil6AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Chervil6AttrReturn");
  try {
    ReturnValue = tobj->Chervil6Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Chervil6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChervil6(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerChervil6AttrW(void)
{
  
  ::Chervil6 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::Chervil6Attr(const ::Chervil6 _attr)");
#endif
  
  ernie.seed("Chervil6AttrCall");
  GetChervil6(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Chervil6AttrReturn");
  try {
     tobj->Chervil6Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Chervil6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerChervil6Op(void)
{
  
  ::Chervil6_slice * ReturnValue;
  ::Chervil6 p1;
  ::Chervil6_slice * p2;
  ::Chervil6 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Chervil6_slice * ::TestServer::Chervil6Op(const ::Chervil6 p1,::Chervil6_slice *&p2,::Chervil6 p3)");
#endif
  
  ernie.seed("Chervil6OpCall");
  GetChervil6(p1);
  GetChervil6(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Chervil6OpReturn");
  try {
    ReturnValue = tobj->Chervil6Op(p1,p2,p3);
  }
  catch (::TestServer::Chervil6Excpt &ex){
    if (!CheckTestServerChervil6Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Chervil6Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChervil6(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckChervil6(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckChervil6(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerBlueberry6Attr(void)
{
  
  ::Blueberry6* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Blueberry6* ::TestServer::Blueberry6Attr()");
#endif
  
  ernie.seed("Blueberry6AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Blueberry6AttrReturn");
  try {
    ReturnValue = tobj->Blueberry6Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Blueberry6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBlueberry6(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerBlueberry6AttrW(void)
{
  
  ::Blueberry6 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::Blueberry6Attr(const ::Blueberry6 &_attr)");
#endif
  
  ernie.seed("Blueberry6AttrCall");
  GetBlueberry6(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Blueberry6AttrReturn");
  try {
     tobj->Blueberry6Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Blueberry6Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerBlueberry6Op(void)
{
  
  ::Blueberry6* ReturnValue;
  ::Blueberry6 p1;
  ::Blueberry6* p2;
  ::Blueberry6 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Blueberry6* ::TestServer::Blueberry6Op(const ::Blueberry6 &p1,::Blueberry6 *&p2,::Blueberry6 &p3)");
#endif
  
  ernie.seed("Blueberry6OpCall");
  GetBlueberry6(p1);
  GetBlueberry6(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Blueberry6OpReturn");
  try {
    ReturnValue = tobj->Blueberry6Op(p1,p2,p3);
  }
  catch (::TestServer::Blueberry6Excpt &ex){
    if (!CheckTestServerBlueberry6Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Blueberry6Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBlueberry6(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBlueberry6(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBlueberry6(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerTurnip7Attr(void)
{
  
  ::Turnip7* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Turnip7* ::TestServer::Turnip7Attr()");
#endif
  
  ernie.seed("Turnip7AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Turnip7AttrReturn");
  try {
    ReturnValue = tobj->Turnip7Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Turnip7Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTurnip7(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerTurnip7AttrW(void)
{
  
  ::Turnip7 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::Turnip7Attr(const ::Turnip7 &_attr)");
#endif
  
  ernie.seed("Turnip7AttrCall");
  GetTurnip7(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Turnip7AttrReturn");
  try {
     tobj->Turnip7Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Turnip7Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerTurnip7Op(void)
{
  
  ::Turnip7* ReturnValue;
  ::Turnip7 p1;
  ::Turnip7* p2;
  ::Turnip7 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Turnip7* ::TestServer::Turnip7Op(const ::Turnip7 &p1,::Turnip7 *&p2,::Turnip7 &p3)");
#endif
  
  ernie.seed("Turnip7OpCall");
  GetTurnip7(p1);
  GetTurnip7(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Turnip7OpReturn");
  try {
    ReturnValue = tobj->Turnip7Op(p1,p2,p3);
  }
  catch (::TestServer::Turnip7Excpt &ex){
    if (!CheckTestServerTurnip7Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Turnip7Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTurnip7(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckTurnip7(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckTurnip7(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerSultana7Attr(void)
{
  
  ::Sultana7* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Sultana7* ::TestServer::Sultana7Attr()");
#endif
  
  ernie.seed("Sultana7AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sultana7AttrReturn");
  try {
    ReturnValue = tobj->Sultana7Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Sultana7Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSultana7(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerSultana7AttrW(void)
{
  
  ::Sultana7 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TestServer::Sultana7Attr(const ::Sultana7 &_attr)");
#endif
  
  ernie.seed("Sultana7AttrCall");
  GetSultana7(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sultana7AttrReturn");
  try {
     tobj->Sultana7Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Sultana7Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTestServerSultana7Op(void)
{
  
  ::Sultana7* ReturnValue;
  ::Sultana7 p1;
  ::Sultana7* p2;
  ::Sultana7 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Sultana7* ::TestServer::Sultana7Op(const ::Sultana7 &p1,::Sultana7 *&p2,::Sultana7 &p3)");
#endif
  
  ernie.seed("Sultana7OpCall");
  GetSultana7(p1);
  GetSultana7(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sultana7OpReturn");
  try {
    ReturnValue = tobj->Sultana7Op(p1,p2,p3);
  }
  catch (::TestServer::Sultana7Excpt &ex){
    if (!CheckTestServerSultana7Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TestServer::Sultana7Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSultana7(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckSultana7(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckSultana7(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


