//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/cmain.cpp
//
// Modifications:
// $Log: cmain.cpp,v $
// Revision 1.2  2000/03/28 18:27:09  mli
//   o   remove 'CORBA::Environment ev;'
//   o   void (*tet_startup)(void) -> void (*tet_startup)()
//   o   void (*tet_cleanup)(void) -> void (*tet_cleanup)()
//
// Revision 1.1.1.1  2000/02/09 17:16:30  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.8  1998/08/24 11:46:35  james
// ENV updates
//
// Revision 1.7  1997/12/12 08:51:01  james
// Code Regenerated
//
//
//

#include <orb_supp.hh>
#include <syncinfo.hh>
#include <stdlib.h>

#ifndef WIN32
#include <unistd.h>
#else
#include <process.h>
#endif

extern "C" {
#include <tet_api.h>
}

#include "cmain.hh"

//
// This global defines the systems involved in a TET sync event
// Currently the client sync with the proxy to receive the IOR
//
int remSystem[] = { CSW };	

// Our handle to the ORB under test
OrbSupport *orb;

// The stringified IOR passed to us by the proxy
char *stringifiedIOR = 0;

void
clientStartUp()
{
  // Initialise the ORB under test

  try {
      orb = new OrbSupport();
      orb->client_init();
  }
  catch (CORBA::Exception &ex) {
      tet_infoline("client: failed to initialise Orb");
      return;
  }
  
  // Provide space to receive proxified IOR
  struct tet_synmsg msg;
  char rcvbuf[2048];
  
  // Indicate receive mode to TET
  msg.tsm_flags = TET_SMRCVMSG;
  msg.tsm_dlen = sizeof(rcvbuf);
  msg.tsm_data = rcvbuf;
  
  if (tet_remsync(CEVENT, remSystem, 1, VS_TIMEOUT, TET_SV_YES, &msg) != 0)
  {
      tet_printf("client: sync point %d failed", CEVENT);
      return;
  }

  // Cache stringified IOR for method calls to use
  stringifiedIOR = strdup(rcvbuf);

#ifdef DEBUG  
  tet_printf("client: sync point %d OK", CEVENT);
#endif
}

void
clientCleanUp()
{
    delete orb;
    return;
}

#ifdef VSORB_IDL_TESTS
 
extern "C" {
void (*tet_startup)() = 0,
(*tet_cleanup)() = 0;
struct tet_testlist tet_testlist[] = {
      {testIDL,1},
      {NULL,0} };
}

#else

extern "C" {
  void (*tet_startup)() = clientStartUp,
  (*tet_cleanup)() = clientCleanUp;
  struct tet_testlist tet_testlist[] = {
    {testTestServerLoganBerry19Attr,1},
    {testTestServerLoganBerry19AttrW,2},
    {testTestServerLoganBerry19Op,3},
    {testTestServerOrange19Attr,4},
    {testTestServerOrange19AttrW,5},
    {testTestServerOrange19Op,6},
    {testTestServerBlackCherry20Attr,7},
    {testTestServerBlackCherry20AttrW,8},
    {testTestServerBlackCherry20Op,9},
    {testTestServerLemon20Attr,10},
    {testTestServerLemon20AttrW,11},
    {testTestServerLemon20Op,12},
    {testTestServerCoconut6Attr,13},
    {testTestServerCoconut6AttrW,14},
    {testTestServerCoconut6Op,15},
    {testTestServerLime6Attr,16},
    {testTestServerLime6AttrW,17},
    {testTestServerLime6Op,18},
    {testTestServerMint6Attr,19},
    {testTestServerMint6AttrW,20},
    {testTestServerMint6Op,21},
    {testTestServerBlackEyedBeans6Attr,22},
    {testTestServerBlackEyedBeans6AttrW,23},
    {testTestServerBlackEyedBeans6Op,24},
    {testTestServerMangetout6Attr,25},
    {testTestServerMangetout6AttrW,26},
    {testTestServerMangetout6Op,27},
    {testTestServerLettuce6Attr,28},
    {testTestServerLettuce6AttrW,29},
    {testTestServerLettuce6Op,30},
    {testTestServerChervil6Attr,31},
    {testTestServerChervil6AttrW,32},
    {testTestServerChervil6Op,33},
    {testTestServerBlueberry6Attr,34},
    {testTestServerBlueberry6AttrW,35},
    {testTestServerBlueberry6Op,36},
    {testTestServerTurnip7Attr,37},
    {testTestServerTurnip7AttrW,38},
    {testTestServerTurnip7Op,39},
    {testTestServerSultana7Attr,40},
    {testTestServerSultana7AttrW,41},
    {testTestServerSultana7Op,42},
    {NULL,0} };
}

#endif
