//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:09:43  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:34  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.5  1997/12/12 08:59:32  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerLoganBerry44Attr(void)
{
  
  ::LoganBerry44* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::LoganBerry44* ::idlServer::LoganBerry44Attr()");
#endif
  
  ernie.seed("LoganBerry44AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerry44AttrReturn");
  try {
    ReturnValue = tobj->LoganBerry44Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LoganBerry44Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLoganBerry44(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLoganBerry44AttrW(void)
{
  
  ::LoganBerry44 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::LoganBerry44Attr(const ::LoganBerry44 &_attr)");
#endif
  
  ernie.seed("LoganBerry44AttrCall");
  GetLoganBerry44(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerry44AttrReturn");
  try {
     tobj->LoganBerry44Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LoganBerry44Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLoganBerry44Op(void)
{
  
  ::LoganBerry44* ReturnValue;
  ::LoganBerry44 p1;
  ::LoganBerry44* p2;
  ::LoganBerry44 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::LoganBerry44* ::idlServer::LoganBerry44Op(const ::LoganBerry44 &p1,::LoganBerry44 *&p2,::LoganBerry44 &p3)");
#endif
  
  ernie.seed("LoganBerry44OpCall");
  GetLoganBerry44(p1);
  GetLoganBerry44(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerry44OpReturn");
  try {
    ReturnValue = tobj->LoganBerry44Op(p1,p2,p3);
  }
  catch (::idlServer::LoganBerry44Excpt &ex){
    if (!CheckidlServerLoganBerry44Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LoganBerry44Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLoganBerry44(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckLoganBerry44(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckLoganBerry44(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerOrange44Attr(void)
{
  
  ::Orange44* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Orange44* ::idlServer::Orange44Attr()");
#endif
  
  ernie.seed("Orange44AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Orange44AttrReturn");
  try {
    ReturnValue = tobj->Orange44Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Orange44Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOrange44(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerOrange44AttrW(void)
{
  
  ::Orange44 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Orange44Attr(const ::Orange44 &_attr)");
#endif
  
  ernie.seed("Orange44AttrCall");
  GetOrange44(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Orange44AttrReturn");
  try {
     tobj->Orange44Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Orange44Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerOrange44Op(void)
{
  
  ::Orange44* ReturnValue;
  ::Orange44 p1;
  ::Orange44* p2;
  ::Orange44 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Orange44* ::idlServer::Orange44Op(const ::Orange44 &p1,::Orange44 *&p2,::Orange44 &p3)");
#endif
  
  ernie.seed("Orange44OpCall");
  GetOrange44(p1);
  GetOrange44(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Orange44OpReturn");
  try {
    ReturnValue = tobj->Orange44Op(p1,p2,p3);
  }
  catch (::idlServer::Orange44Excpt &ex){
    if (!CheckidlServerOrange44Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Orange44Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOrange44(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckOrange44(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckOrange44(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSultana44Attr(void)
{
  
  ::Orange44::Sultana44* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Orange44::Sultana44* ::idlServer::Sultana44Attr()");
#endif
  
  ernie.seed("Sultana44AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sultana44AttrReturn");
  try {
    ReturnValue = tobj->Sultana44Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Sultana44Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOrange44Sultana44(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSultana44AttrW(void)
{
  
  ::Orange44::Sultana44 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Sultana44Attr(const ::Orange44::Sultana44 &_attr)");
#endif
  
  ernie.seed("Sultana44AttrCall");
  GetOrange44Sultana44(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sultana44AttrReturn");
  try {
     tobj->Sultana44Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Sultana44Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSultana44Op(void)
{
  
  ::Orange44::Sultana44* ReturnValue;
  ::Orange44::Sultana44 p1;
  ::Orange44::Sultana44* p2;
  ::Orange44::Sultana44 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Orange44::Sultana44* ::idlServer::Sultana44Op(const ::Orange44::Sultana44 &p1,::Orange44::Sultana44 *&p2,::Orange44::Sultana44 &p3)");
#endif
  
  ernie.seed("Sultana44OpCall");
  GetOrange44Sultana44(p1);
  GetOrange44Sultana44(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sultana44OpReturn");
  try {
    ReturnValue = tobj->Sultana44Op(p1,p2,p3);
  }
  catch (::idlServer::Sultana44Excpt &ex){
    if (!CheckidlServerSultana44Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Sultana44Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOrange44Sultana44(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckOrange44Sultana44(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckOrange44Sultana44(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBlackCherry45Attr(void)
{
  
  ::BlackCherry45* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BlackCherry45* ::idlServer::BlackCherry45Attr()");
#endif
  
  ernie.seed("BlackCherry45AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackCherry45AttrReturn");
  try {
    ReturnValue = tobj->BlackCherry45Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BlackCherry45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBlackCherry45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBlackCherry45AttrW(void)
{
  
  ::BlackCherry45 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BlackCherry45Attr(const ::BlackCherry45 &_attr)");
#endif
  
  ernie.seed("BlackCherry45AttrCall");
  GetBlackCherry45(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackCherry45AttrReturn");
  try {
     tobj->BlackCherry45Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BlackCherry45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBlackCherry45Op(void)
{
  
  ::BlackCherry45* ReturnValue;
  ::BlackCherry45 p1;
  ::BlackCherry45* p2;
  ::BlackCherry45 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BlackCherry45* ::idlServer::BlackCherry45Op(const ::BlackCherry45 &p1,::BlackCherry45 *&p2,::BlackCherry45 &p3)");
#endif
  
  ernie.seed("BlackCherry45OpCall");
  GetBlackCherry45(p1);
  GetBlackCherry45(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackCherry45OpReturn");
  try {
    ReturnValue = tobj->BlackCherry45Op(p1,p2,p3);
  }
  catch (::idlServer::BlackCherry45Excpt &ex){
    if (!CheckidlServerBlackCherry45Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BlackCherry45Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBlackCherry45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBlackCherry45(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBlackCherry45(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLemon45Attr(void)
{
  
  ::Lemon45* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lemon45* ::idlServer::Lemon45Attr()");
#endif
  
  ernie.seed("Lemon45AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lemon45AttrReturn");
  try {
    ReturnValue = tobj->Lemon45Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Lemon45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLemon45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLemon45AttrW(void)
{
  
  ::Lemon45 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Lemon45Attr(const ::Lemon45 &_attr)");
#endif
  
  ernie.seed("Lemon45AttrCall");
  GetLemon45(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lemon45AttrReturn");
  try {
     tobj->Lemon45Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Lemon45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLemon45Op(void)
{
  
  ::Lemon45* ReturnValue;
  ::Lemon45 p1;
  ::Lemon45* p2;
  ::Lemon45 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lemon45* ::idlServer::Lemon45Op(const ::Lemon45 &p1,::Lemon45 *&p2,::Lemon45 &p3)");
#endif
  
  ernie.seed("Lemon45OpCall");
  GetLemon45(p1);
  GetLemon45(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lemon45OpReturn");
  try {
    ReturnValue = tobj->Lemon45Op(p1,p2,p3);
  }
  catch (::idlServer::Lemon45Excpt &ex){
    if (!CheckidlServerLemon45Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Lemon45Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLemon45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckLemon45(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckLemon45(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBanana45Attr(void)
{
  
  ::Lemon45::Banana45* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lemon45::Banana45* ::idlServer::Banana45Attr()");
#endif
  
  ernie.seed("Banana45AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Banana45AttrReturn");
  try {
    ReturnValue = tobj->Banana45Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Banana45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLemon45Banana45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBanana45AttrW(void)
{
  
  ::Lemon45::Banana45 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Banana45Attr(const ::Lemon45::Banana45 &_attr)");
#endif
  
  ernie.seed("Banana45AttrCall");
  GetLemon45Banana45(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Banana45AttrReturn");
  try {
     tobj->Banana45Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Banana45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBanana45Op(void)
{
  
  ::Lemon45::Banana45* ReturnValue;
  ::Lemon45::Banana45 p1;
  ::Lemon45::Banana45* p2;
  ::Lemon45::Banana45 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lemon45::Banana45* ::idlServer::Banana45Op(const ::Lemon45::Banana45 &p1,::Lemon45::Banana45 *&p2,::Lemon45::Banana45 &p3)");
#endif
  
  ernie.seed("Banana45OpCall");
  GetLemon45Banana45(p1);
  GetLemon45Banana45(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Banana45OpReturn");
  try {
    ReturnValue = tobj->Banana45Op(p1,p2,p3);
  }
  catch (::idlServer::Banana45Excpt &ex){
    if (!CheckidlServerBanana45Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Banana45Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLemon45Banana45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckLemon45Banana45(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckLemon45Banana45(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCorriander45Attr(void)
{
  
  ::Corriander45* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Corriander45* ::idlServer::Corriander45Attr()");
#endif
  
  ernie.seed("Corriander45AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Corriander45AttrReturn");
  try {
    ReturnValue = tobj->Corriander45Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Corriander45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCorriander45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCorriander45AttrW(void)
{
  
  ::Corriander45 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Corriander45Attr(const ::Corriander45 &_attr)");
#endif
  
  ernie.seed("Corriander45AttrCall");
  GetCorriander45(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Corriander45AttrReturn");
  try {
     tobj->Corriander45Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Corriander45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCorriander45Op(void)
{
  
  ::Corriander45* ReturnValue;
  ::Corriander45 p1;
  ::Corriander45* p2;
  ::Corriander45 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Corriander45* ::idlServer::Corriander45Op(const ::Corriander45 &p1,::Corriander45 *&p2,::Corriander45 &p3)");
#endif
  
  ernie.seed("Corriander45OpCall");
  GetCorriander45(p1);
  GetCorriander45(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Corriander45OpReturn");
  try {
    ReturnValue = tobj->Corriander45Op(p1,p2,p3);
  }
  catch (::idlServer::Corriander45Excpt &ex){
    if (!CheckidlServerCorriander45Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Corriander45Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCorriander45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckCorriander45(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckCorriander45(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKiwi45Attr(void)
{
  
  ::Kiwi45* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Kiwi45* ::idlServer::Kiwi45Attr()");
#endif
  
  ernie.seed("Kiwi45AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Kiwi45AttrReturn");
  try {
    ReturnValue = tobj->Kiwi45Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Kiwi45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKiwi45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKiwi45AttrW(void)
{
  
  ::Kiwi45 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Kiwi45Attr(const ::Kiwi45 &_attr)");
#endif
  
  ernie.seed("Kiwi45AttrCall");
  GetKiwi45(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Kiwi45AttrReturn");
  try {
     tobj->Kiwi45Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Kiwi45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKiwi45Op(void)
{
  
  ::Kiwi45* ReturnValue;
  ::Kiwi45 p1;
  ::Kiwi45* p2;
  ::Kiwi45 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Kiwi45* ::idlServer::Kiwi45Op(const ::Kiwi45 &p1,::Kiwi45 *&p2,::Kiwi45 &p3)");
#endif
  
  ernie.seed("Kiwi45OpCall");
  GetKiwi45(p1);
  GetKiwi45(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Kiwi45OpReturn");
  try {
    ReturnValue = tobj->Kiwi45Op(p1,p2,p3);
  }
  catch (::idlServer::Kiwi45Excpt &ex){
    if (!CheckidlServerKiwi45Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Kiwi45Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKiwi45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckKiwi45(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckKiwi45(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGalaApple45Attr(void)
{
  
  ::Kiwi45::GalaApple45* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Kiwi45::GalaApple45* ::idlServer::GalaApple45Attr()");
#endif
  
  ernie.seed("GalaApple45AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GalaApple45AttrReturn");
  try {
    ReturnValue = tobj->GalaApple45Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GalaApple45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKiwi45GalaApple45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGalaApple45AttrW(void)
{
  
  ::Kiwi45::GalaApple45 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::GalaApple45Attr(const ::Kiwi45::GalaApple45 &_attr)");
#endif
  
  ernie.seed("GalaApple45AttrCall");
  GetKiwi45GalaApple45(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GalaApple45AttrReturn");
  try {
     tobj->GalaApple45Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GalaApple45Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGalaApple45Op(void)
{
  
  ::Kiwi45::GalaApple45* ReturnValue;
  ::Kiwi45::GalaApple45 p1;
  ::Kiwi45::GalaApple45* p2;
  ::Kiwi45::GalaApple45 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Kiwi45::GalaApple45* ::idlServer::GalaApple45Op(const ::Kiwi45::GalaApple45 &p1,::Kiwi45::GalaApple45 *&p2,::Kiwi45::GalaApple45 &p3)");
#endif
  
  ernie.seed("GalaApple45OpCall");
  GetKiwi45GalaApple45(p1);
  GetKiwi45GalaApple45(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GalaApple45OpReturn");
  try {
    ReturnValue = tobj->GalaApple45Op(p1,p2,p3);
  }
  catch (::idlServer::GalaApple45Excpt &ex){
    if (!CheckidlServerGalaApple45Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GalaApple45Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckKiwi45GalaApple45(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckKiwi45GalaApple45(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckKiwi45GalaApple45(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerYellowPepper46Attr(void)
{
  
  ::YellowPepper46* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::YellowPepper46* ::idlServer::YellowPepper46Attr()");
#endif
  
  ernie.seed("YellowPepper46AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("YellowPepper46AttrReturn");
  try {
    ReturnValue = tobj->YellowPepper46Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::YellowPepper46Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckYellowPepper46(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerYellowPepper46AttrW(void)
{
  
  ::YellowPepper46 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::YellowPepper46Attr(const ::YellowPepper46 &_attr)");
#endif
  
  ernie.seed("YellowPepper46AttrCall");
  GetYellowPepper46(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("YellowPepper46AttrReturn");
  try {
     tobj->YellowPepper46Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::YellowPepper46Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerYellowPepper46Op(void)
{
  
  ::YellowPepper46* ReturnValue;
  ::YellowPepper46 p1;
  ::YellowPepper46* p2;
  ::YellowPepper46 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::YellowPepper46* ::idlServer::YellowPepper46Op(const ::YellowPepper46 &p1,::YellowPepper46 *&p2,::YellowPepper46 &p3)");
#endif
  
  ernie.seed("YellowPepper46OpCall");
  GetYellowPepper46(p1);
  GetYellowPepper46(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("YellowPepper46OpReturn");
  try {
    ReturnValue = tobj->YellowPepper46Op(p1,p2,p3);
  }
  catch (::idlServer::YellowPepper46Excpt &ex){
    if (!CheckidlServerYellowPepper46Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::YellowPepper46Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckYellowPepper46(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckYellowPepper46(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckYellowPepper46(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerParsley46Attr(void)
{
  
  ::Parsley46* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Parsley46* ::idlServer::Parsley46Attr()");
#endif
  
  ernie.seed("Parsley46AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsley46AttrReturn");
  try {
    ReturnValue = tobj->Parsley46Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Parsley46Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckParsley46(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerParsley46AttrW(void)
{
  
  ::Parsley46 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Parsley46Attr(const ::Parsley46 &_attr)");
#endif
  
  ernie.seed("Parsley46AttrCall");
  GetParsley46(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsley46AttrReturn");
  try {
     tobj->Parsley46Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Parsley46Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerParsley46Op(void)
{
  
  ::Parsley46* ReturnValue;
  ::Parsley46 p1;
  ::Parsley46* p2;
  ::Parsley46 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Parsley46* ::idlServer::Parsley46Op(const ::Parsley46 &p1,::Parsley46 *&p2,::Parsley46 &p3)");
#endif
  
  ernie.seed("Parsley46OpCall");
  GetParsley46(p1);
  GetParsley46(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsley46OpReturn");
  try {
    ReturnValue = tobj->Parsley46Op(p1,p2,p3);
  }
  catch (::idlServer::Parsley46Excpt &ex){
    if (!CheckidlServerParsley46Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Parsley46Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckParsley46(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckParsley46(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckParsley46(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPeach46Attr(void)
{
  
  ::Parsley46::Peach46* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Parsley46::Peach46* ::idlServer::Peach46Attr()");
#endif
  
  ernie.seed("Peach46AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Peach46AttrReturn");
  try {
    ReturnValue = tobj->Peach46Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Peach46Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckParsley46Peach46(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPeach46AttrW(void)
{
  
  ::Parsley46::Peach46 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Peach46Attr(const ::Parsley46::Peach46 &_attr)");
#endif
  
  ernie.seed("Peach46AttrCall");
  GetParsley46Peach46(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Peach46AttrReturn");
  try {
     tobj->Peach46Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Peach46Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPeach46Op(void)
{
  
  ::Parsley46::Peach46* ReturnValue;
  ::Parsley46::Peach46 p1;
  ::Parsley46::Peach46* p2;
  ::Parsley46::Peach46 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Parsley46::Peach46* ::idlServer::Peach46Op(const ::Parsley46::Peach46 &p1,::Parsley46::Peach46 *&p2,::Parsley46::Peach46 &p3)");
#endif
  
  ernie.seed("Peach46OpCall");
  GetParsley46Peach46(p1);
  GetParsley46Peach46(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Peach46OpReturn");
  try {
    ReturnValue = tobj->Peach46Op(p1,p2,p3);
  }
  catch (::idlServer::Peach46Excpt &ex){
    if (!CheckidlServerPeach46Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Peach46Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckParsley46Peach46(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckParsley46Peach46(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckParsley46Peach46(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRadish47Attr(void)
{
  
  ::Radish47* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Radish47* ::idlServer::Radish47Attr()");
#endif
  
  ernie.seed("Radish47AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Radish47AttrReturn");
  try {
    ReturnValue = tobj->Radish47Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Radish47Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRadish47(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRadish47AttrW(void)
{
  
  ::Radish47 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Radish47Attr(const ::Radish47 &_attr)");
#endif
  
  ernie.seed("Radish47AttrCall");
  GetRadish47(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Radish47AttrReturn");
  try {
     tobj->Radish47Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Radish47Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRadish47Op(void)
{
  
  ::Radish47* ReturnValue;
  ::Radish47 p1;
  ::Radish47* p2;
  ::Radish47 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Radish47* ::idlServer::Radish47Op(const ::Radish47 &p1,::Radish47 *&p2,::Radish47 &p3)");
#endif
  
  ernie.seed("Radish47OpCall");
  GetRadish47(p1);
  GetRadish47(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Radish47OpReturn");
  try {
    ReturnValue = tobj->Radish47Op(p1,p2,p3);
  }
  catch (::idlServer::Radish47Excpt &ex){
    if (!CheckidlServerRadish47Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Radish47Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRadish47(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckRadish47(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckRadish47(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSharanfruit47Attr(void)
{
  
  ::Sharanfruit47* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Sharanfruit47* ::idlServer::Sharanfruit47Attr()");
#endif
  
  ernie.seed("Sharanfruit47AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sharanfruit47AttrReturn");
  try {
    ReturnValue = tobj->Sharanfruit47Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Sharanfruit47Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSharanfruit47(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSharanfruit47AttrW(void)
{
  
  ::Sharanfruit47 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Sharanfruit47Attr(const ::Sharanfruit47 &_attr)");
#endif
  
  ernie.seed("Sharanfruit47AttrCall");
  GetSharanfruit47(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sharanfruit47AttrReturn");
  try {
     tobj->Sharanfruit47Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Sharanfruit47Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSharanfruit47Op(void)
{
  
  ::Sharanfruit47* ReturnValue;
  ::Sharanfruit47 p1;
  ::Sharanfruit47* p2;
  ::Sharanfruit47 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Sharanfruit47* ::idlServer::Sharanfruit47Op(const ::Sharanfruit47 &p1,::Sharanfruit47 *&p2,::Sharanfruit47 &p3)");
#endif
  
  ernie.seed("Sharanfruit47OpCall");
  GetSharanfruit47(p1);
  GetSharanfruit47(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sharanfruit47OpReturn");
  try {
    ReturnValue = tobj->Sharanfruit47Op(p1,p2,p3);
  }
  catch (::idlServer::Sharanfruit47Excpt &ex){
    if (!CheckidlServerSharanfruit47Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Sharanfruit47Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSharanfruit47(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckSharanfruit47(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckSharanfruit47(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBeefTomatoes47Attr(void)
{
  
  ::Sharanfruit47::BeefTomatoes47* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Sharanfruit47::BeefTomatoes47* ::idlServer::BeefTomatoes47Attr()");
#endif
  
  ernie.seed("BeefTomatoes47AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BeefTomatoes47AttrReturn");
  try {
    ReturnValue = tobj->BeefTomatoes47Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BeefTomatoes47Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSharanfruit47BeefTomatoes47(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBeefTomatoes47AttrW(void)
{
  
  ::Sharanfruit47::BeefTomatoes47 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BeefTomatoes47Attr(const ::Sharanfruit47::BeefTomatoes47 &_attr)");
#endif
  
  ernie.seed("BeefTomatoes47AttrCall");
  GetSharanfruit47BeefTomatoes47(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BeefTomatoes47AttrReturn");
  try {
     tobj->BeefTomatoes47Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BeefTomatoes47Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBeefTomatoes47Op(void)
{
  
  ::Sharanfruit47::BeefTomatoes47* ReturnValue;
  ::Sharanfruit47::BeefTomatoes47 p1;
  ::Sharanfruit47::BeefTomatoes47* p2;
  ::Sharanfruit47::BeefTomatoes47 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Sharanfruit47::BeefTomatoes47* ::idlServer::BeefTomatoes47Op(const ::Sharanfruit47::BeefTomatoes47 &p1,::Sharanfruit47::BeefTomatoes47 *&p2,::Sharanfruit47::BeefTomatoes47 &p3)");
#endif
  
  ernie.seed("BeefTomatoes47OpCall");
  GetSharanfruit47BeefTomatoes47(p1);
  GetSharanfruit47BeefTomatoes47(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BeefTomatoes47OpReturn");
  try {
    ReturnValue = tobj->BeefTomatoes47Op(p1,p2,p3);
  }
  catch (::idlServer::BeefTomatoes47Excpt &ex){
    if (!CheckidlServerBeefTomatoes47Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BeefTomatoes47Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSharanfruit47BeefTomatoes47(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckSharanfruit47BeefTomatoes47(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckSharanfruit47BeefTomatoes47(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBambooShoot48Attr(void)
{
  
  ::BambooShoot48* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BambooShoot48* ::idlServer::BambooShoot48Attr()");
#endif
  
  ernie.seed("BambooShoot48AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BambooShoot48AttrReturn");
  try {
    ReturnValue = tobj->BambooShoot48Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BambooShoot48Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBambooShoot48(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBambooShoot48AttrW(void)
{
  
  ::BambooShoot48 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BambooShoot48Attr(const ::BambooShoot48 &_attr)");
#endif
  
  ernie.seed("BambooShoot48AttrCall");
  GetBambooShoot48(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BambooShoot48AttrReturn");
  try {
     tobj->BambooShoot48Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BambooShoot48Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBambooShoot48Op(void)
{
  
  ::BambooShoot48* ReturnValue;
  ::BambooShoot48 p1;
  ::BambooShoot48* p2;
  ::BambooShoot48 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BambooShoot48* ::idlServer::BambooShoot48Op(const ::BambooShoot48 &p1,::BambooShoot48 *&p2,::BambooShoot48 &p3)");
#endif
  
  ernie.seed("BambooShoot48OpCall");
  GetBambooShoot48(p1);
  GetBambooShoot48(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BambooShoot48OpReturn");
  try {
    ReturnValue = tobj->BambooShoot48Op(p1,p2,p3);
  }
  catch (::idlServer::BambooShoot48Excpt &ex){
    if (!CheckidlServerBambooShoot48Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BambooShoot48Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBambooShoot48(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBambooShoot48(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBambooShoot48(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


