//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/cmain.cpp
//
// Modifications:
// $Log: cmain.cpp,v $
// Revision 1.2  2000/03/28 18:27:59  mli
//   o   remove 'CORBA::Environment ev;'
//   o   void (*tet_startup)(void) -> void (*tet_startup)()
//   o   void (*tet_cleanup)(void) -> void (*tet_cleanup)()
//
// Revision 1.1.1.1  2000/02/09 17:16:34  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.8  1998/08/24 11:46:43  james
// ENV updates
//
// Revision 1.7  1997/12/12 08:59:34  james
// Code Regenerated
//
//
//

#include <orb_supp.hh>
#include <syncinfo.hh>
#include <stdlib.h>

#ifndef WIN32
#include <unistd.h>
#else
#include <process.h>
#endif

extern "C" {
#include <tet_api.h>
}

#include "cmain.hh"

//
// This global defines the systems involved in a TET sync event
// Currently the client sync with the proxy to receive the IOR
//
int remSystem[] = { CSW };	

// Our handle to the ORB under test
OrbSupport *orb;

// The stringified IOR passed to us by the proxy
char *stringifiedIOR = 0;

void
clientStartUp()
{
  // Initialise the ORB under test

  try {
      orb = new OrbSupport();
      orb->client_init();
  }
  catch (CORBA::Exception &ex) {
      tet_infoline("client: failed to initialise Orb");
      return;
  }
  
  // Provide space to receive proxified IOR
  struct tet_synmsg msg;
  char rcvbuf[2048];
  
  // Indicate receive mode to TET
  msg.tsm_flags = TET_SMRCVMSG;
  msg.tsm_dlen = sizeof(rcvbuf);
  msg.tsm_data = rcvbuf;
  
  if (tet_remsync(CEVENT, remSystem, 1, VS_TIMEOUT, TET_SV_YES, &msg) != 0)
  {
      tet_printf("client: sync point %d failed", CEVENT);
      return;
  }

  // Cache stringified IOR for method calls to use
  stringifiedIOR = strdup(rcvbuf);

#ifdef DEBUG  
  tet_printf("client: sync point %d OK", CEVENT);
#endif
}

void
clientCleanUp()
{
    delete orb;
    return;
}

#ifdef VSORB_IDL_TESTS
 
extern "C" {
void (*tet_startup)() = 0,
(*tet_cleanup)() = 0;
struct tet_testlist tet_testlist[] = {
      {testIDL,1},
      {NULL,0} };
}

#else

extern "C" {
  void (*tet_startup)() = clientStartUp,
  (*tet_cleanup)() = clientCleanUp;
  struct tet_testlist tet_testlist[] = {
    {testidlServerLoganBerry44Attr,1},
    {testidlServerLoganBerry44AttrW,2},
    {testidlServerLoganBerry44Op,3},
    {testidlServerOrange44Attr,4},
    {testidlServerOrange44AttrW,5},
    {testidlServerOrange44Op,6},
    {testidlServerSultana44Attr,7},
    {testidlServerSultana44AttrW,8},
    {testidlServerSultana44Op,9},
    {testidlServerBlackCherry45Attr,10},
    {testidlServerBlackCherry45AttrW,11},
    {testidlServerBlackCherry45Op,12},
    {testidlServerLemon45Attr,13},
    {testidlServerLemon45AttrW,14},
    {testidlServerLemon45Op,15},
    {testidlServerBanana45Attr,16},
    {testidlServerBanana45AttrW,17},
    {testidlServerBanana45Op,18},
    {testidlServerCorriander45Attr,19},
    {testidlServerCorriander45AttrW,20},
    {testidlServerCorriander45Op,21},
    {testidlServerKiwi45Attr,22},
    {testidlServerKiwi45AttrW,23},
    {testidlServerKiwi45Op,24},
    {testidlServerGalaApple45Attr,25},
    {testidlServerGalaApple45AttrW,26},
    {testidlServerGalaApple45Op,27},
    {testidlServerYellowPepper46Attr,28},
    {testidlServerYellowPepper46AttrW,29},
    {testidlServerYellowPepper46Op,30},
    {testidlServerParsley46Attr,31},
    {testidlServerParsley46AttrW,32},
    {testidlServerParsley46Op,33},
    {testidlServerPeach46Attr,34},
    {testidlServerPeach46AttrW,35},
    {testidlServerPeach46Op,36},
    {testidlServerRadish47Attr,37},
    {testidlServerRadish47AttrW,38},
    {testidlServerRadish47Op,39},
    {testidlServerSharanfruit47Attr,40},
    {testidlServerSharanfruit47AttrW,41},
    {testidlServerSharanfruit47Op,42},
    {testidlServerBeefTomatoes47Attr,43},
    {testidlServerBeefTomatoes47AttrW,44},
    {testidlServerBeefTomatoes47Op,45},
    {testidlServerBambooShoot48Attr,46},
    {testidlServerBambooShoot48AttrW,47},
    {testidlServerBambooShoot48Op,48},
    {NULL,0} };
}

#endif
