//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:10:05  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:36  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.5  1997/12/12 09:00:22  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerCultra66Attr(void)
{
  
  ::Cultra66* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Cultra66* ::idlServer::Cultra66Attr()");
#endif
  
  ernie.seed("Cultra66AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cultra66AttrReturn");
  try {
    ReturnValue = tobj->Cultra66Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Cultra66Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCultra66(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCultra66AttrW(void)
{
  
  ::Cultra66 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Cultra66Attr(const ::Cultra66 &_attr)");
#endif
  
  ernie.seed("Cultra66AttrCall");
  GetCultra66(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cultra66AttrReturn");
  try {
     tobj->Cultra66Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Cultra66Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCultra66Op(void)
{
  
  ::Cultra66* ReturnValue;
  ::Cultra66 p1;
  ::Cultra66* p2;
  ::Cultra66 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Cultra66* ::idlServer::Cultra66Op(const ::Cultra66 &p1,::Cultra66 *&p2,::Cultra66 &p3)");
#endif
  
  ernie.seed("Cultra66OpCall");
  GetCultra66(p1);
  GetCultra66(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cultra66OpReturn");
  try {
    ReturnValue = tobj->Cultra66Op(p1,p2,p3);
  }
  catch (::idlServer::Cultra66Excpt &ex){
    if (!CheckidlServerCultra66Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Cultra66Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCultra66(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckCultra66(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckCultra66(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPimpernel66Attr(void)
{
  
  ::Pimpernel66* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Pimpernel66* ::idlServer::Pimpernel66Attr()");
#endif
  
  ernie.seed("Pimpernel66AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pimpernel66AttrReturn");
  try {
    ReturnValue = tobj->Pimpernel66Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Pimpernel66Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPimpernel66(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPimpernel66AttrW(void)
{
  
  ::Pimpernel66 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Pimpernel66Attr(const ::Pimpernel66 &_attr)");
#endif
  
  ernie.seed("Pimpernel66AttrCall");
  GetPimpernel66(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pimpernel66AttrReturn");
  try {
     tobj->Pimpernel66Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Pimpernel66Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPimpernel66Op(void)
{
  
  ::Pimpernel66* ReturnValue;
  ::Pimpernel66 p1;
  ::Pimpernel66* p2;
  ::Pimpernel66 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Pimpernel66* ::idlServer::Pimpernel66Op(const ::Pimpernel66 &p1,::Pimpernel66 *&p2,::Pimpernel66 &p3)");
#endif
  
  ernie.seed("Pimpernel66OpCall");
  GetPimpernel66(p1);
  GetPimpernel66(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pimpernel66OpReturn");
  try {
    ReturnValue = tobj->Pimpernel66Op(p1,p2,p3);
  }
  catch (::idlServer::Pimpernel66Excpt &ex){
    if (!CheckidlServerPimpernel66Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Pimpernel66Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPimpernel66(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckPimpernel66(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckPimpernel66(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKerrsPinks66Attr(void)
{
  
  ::Pimpernel66::KerrsPinks66* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Pimpernel66::KerrsPinks66* ::idlServer::KerrsPinks66Attr()");
#endif
  
  ernie.seed("KerrsPinks66AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KerrsPinks66AttrReturn");
  try {
    ReturnValue = tobj->KerrsPinks66Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KerrsPinks66Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPimpernel66KerrsPinks66(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKerrsPinks66AttrW(void)
{
  
  ::Pimpernel66::KerrsPinks66 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::KerrsPinks66Attr(const ::Pimpernel66::KerrsPinks66 &_attr)");
#endif
  
  ernie.seed("KerrsPinks66AttrCall");
  GetPimpernel66KerrsPinks66(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KerrsPinks66AttrReturn");
  try {
     tobj->KerrsPinks66Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KerrsPinks66Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerKerrsPinks66Op(void)
{
  
  ::Pimpernel66::KerrsPinks66* ReturnValue;
  ::Pimpernel66::KerrsPinks66 p1;
  ::Pimpernel66::KerrsPinks66* p2;
  ::Pimpernel66::KerrsPinks66 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Pimpernel66::KerrsPinks66* ::idlServer::KerrsPinks66Op(const ::Pimpernel66::KerrsPinks66 &p1,::Pimpernel66::KerrsPinks66 *&p2,::Pimpernel66::KerrsPinks66 &p3)");
#endif
  
  ernie.seed("KerrsPinks66OpCall");
  GetPimpernel66KerrsPinks66(p1);
  GetPimpernel66KerrsPinks66(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KerrsPinks66OpReturn");
  try {
    ReturnValue = tobj->KerrsPinks66Op(p1,p2,p3);
  }
  catch (::idlServer::KerrsPinks66Excpt &ex){
    if (!CheckidlServerKerrsPinks66Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::KerrsPinks66Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPimpernel66KerrsPinks66(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckPimpernel66KerrsPinks66(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckPimpernel66KerrsPinks66(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerWilja66Attr(void)
{
  
  ::Pimpernel66::KerrsPinks66::Wilja66* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Pimpernel66::KerrsPinks66::Wilja66* ::idlServer::Wilja66Attr()");
#endif
  
  ernie.seed("Wilja66AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Wilja66AttrReturn");
  try {
    ReturnValue = tobj->Wilja66Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Wilja66Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPimpernel66KerrsPinks66Wilja66(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerWilja66AttrW(void)
{
  
  ::Pimpernel66::KerrsPinks66::Wilja66 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Wilja66Attr(const ::Pimpernel66::KerrsPinks66::Wilja66 &_attr)");
#endif
  
  ernie.seed("Wilja66AttrCall");
  GetPimpernel66KerrsPinks66Wilja66(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Wilja66AttrReturn");
  try {
     tobj->Wilja66Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Wilja66Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerWilja66Op(void)
{
  
  ::Pimpernel66::KerrsPinks66::Wilja66* ReturnValue;
  ::Pimpernel66::KerrsPinks66::Wilja66 p1;
  ::Pimpernel66::KerrsPinks66::Wilja66* p2;
  ::Pimpernel66::KerrsPinks66::Wilja66 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Pimpernel66::KerrsPinks66::Wilja66* ::idlServer::Wilja66Op(const ::Pimpernel66::KerrsPinks66::Wilja66 &p1,::Pimpernel66::KerrsPinks66::Wilja66 *&p2,::Pimpernel66::KerrsPinks66::Wilja66 &p3)");
#endif
  
  ernie.seed("Wilja66OpCall");
  GetPimpernel66KerrsPinks66Wilja66(p1);
  GetPimpernel66KerrsPinks66Wilja66(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Wilja66OpReturn");
  try {
    ReturnValue = tobj->Wilja66Op(p1,p2,p3);
  }
  catch (::idlServer::Wilja66Excpt &ex){
    if (!CheckidlServerWilja66Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Wilja66Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPimpernel66KerrsPinks66Wilja66(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckPimpernel66KerrsPinks66Wilja66(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckPimpernel66KerrsPinks66Wilja66(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBroadbeans67Attr(void)
{
  
  ::Broadbeans67* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Broadbeans67* ::idlServer::Broadbeans67Attr()");
#endif
  
  ernie.seed("Broadbeans67AttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Broadbeans67AttrReturn");
  try {
    ReturnValue = tobj->Broadbeans67Attr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Broadbeans67Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBroadbeans67(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBroadbeans67AttrW(void)
{
  
  ::Broadbeans67 _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::Broadbeans67Attr(const ::Broadbeans67 &_attr)");
#endif
  
  ernie.seed("Broadbeans67AttrCall");
  GetBroadbeans67(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Broadbeans67AttrReturn");
  try {
     tobj->Broadbeans67Attr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Broadbeans67Attr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBroadbeans67Op(void)
{
  
  ::Broadbeans67* ReturnValue;
  ::Broadbeans67 p1;
  ::Broadbeans67* p2;
  ::Broadbeans67 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Broadbeans67* ::idlServer::Broadbeans67Op(const ::Broadbeans67 &p1,::Broadbeans67 *&p2,::Broadbeans67 &p3)");
#endif
  
  ernie.seed("Broadbeans67OpCall");
  GetBroadbeans67(p1);
  GetBroadbeans67(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Broadbeans67OpReturn");
  try {
    ReturnValue = tobj->Broadbeans67Op(p1,p2,p3);
  }
  catch (::idlServer::Broadbeans67Excpt &ex){
    if (!CheckidlServerBroadbeans67Excpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::Broadbeans67Op");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBroadbeans67(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBroadbeans67(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBroadbeans67(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


