//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:13:13  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:54  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:29:16  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerAppleAttr(void)
{
  
  ::Apple ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Apple ::idlServer::AppleAttr()");
#endif
  
  ernie.seed("AppleAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AppleAttrReturn");
  try {
    ReturnValue = tobj->AppleAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AppleAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckApple(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAppleAttrW(void)
{
  
  ::Apple _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::AppleAttr(const ::Apple &_attr)");
#endif
  
  ernie.seed("AppleAttrCall");
  GetApple(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AppleAttrReturn");
  try {
     tobj->AppleAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AppleAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAppleOp(void)
{
  
  ::Apple ReturnValue;
  ::Apple p1;
  ::Apple p2;
  ::Apple p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Apple ::idlServer::AppleOp(const ::Apple &p1,::Apple &p2,::Apple &p3)");
#endif
  
  ernie.seed("AppleOpCall");
  GetApple(p1);
  GetApple(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AppleOpReturn");
  try {
    ReturnValue = tobj->AppleOp(p1,p2,p3);
  }
  catch (::idlServer::AppleExcpt &ex){
    if (!CheckidlServerAppleExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AppleOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckApple(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckApple(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckApple(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoAttr(void)
{
  
  ::Avocado ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado ::idlServer::AvocadoAttr()");
#endif
  
  ernie.seed("AvocadoAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoAttrReturn");
  try {
    ReturnValue = tobj->AvocadoAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocado(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoAttrW(void)
{
  
  ::Avocado _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::AvocadoAttr(const ::Avocado &_attr)");
#endif
  
  ernie.seed("AvocadoAttrCall");
  GetAvocado(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoAttrReturn");
  try {
     tobj->AvocadoAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoOp(void)
{
  
  ::Avocado ReturnValue;
  ::Avocado p1;
  ::Avocado p2;
  ::Avocado p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado ::idlServer::AvocadoOp(const ::Avocado &p1,::Avocado &p2,::Avocado &p3)");
#endif
  
  ernie.seed("AvocadoOpCall");
  GetAvocado(p1);
  GetAvocado(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoOpReturn");
  try {
    ReturnValue = tobj->AvocadoOp(p1,p2,p3);
  }
  catch (::idlServer::AvocadoExcpt &ex){
    if (!CheckidlServerAvocadoExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocado(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocado(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocado(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPeanutAttr(void)
{
  
  ::Peanut ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Peanut ::idlServer::PeanutAttr()");
#endif
  
  ernie.seed("PeanutAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PeanutAttrReturn");
  try {
    ReturnValue = tobj->PeanutAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PeanutAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPeanut(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPeanutAttrW(void)
{
  
  ::Peanut _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::PeanutAttr(const ::Peanut &_attr)");
#endif
  
  ernie.seed("PeanutAttrCall");
  GetPeanut(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PeanutAttrReturn");
  try {
     tobj->PeanutAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PeanutAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerPeanutOp(void)
{
  
  ::Peanut ReturnValue;
  ::Peanut p1;
  ::Peanut p2;
  ::Peanut p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Peanut ::idlServer::PeanutOp(const ::Peanut &p1,::Peanut &p2,::Peanut &p3)");
#endif
  
  ernie.seed("PeanutOpCall");
  GetPeanut(p1);
  GetPeanut(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PeanutOpReturn");
  try {
    ReturnValue = tobj->PeanutOp(p1,p2,p3);
  }
  catch (::idlServer::PeanutExcpt &ex){
    if (!CheckidlServerPeanutExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::PeanutOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPeanut(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckPeanut(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckPeanut(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSatsumaAttr(void)
{
  
  ::Satsuma_slice * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Satsuma_slice * ::idlServer::SatsumaAttr()");
#endif
  
  ernie.seed("SatsumaAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SatsumaAttrReturn");
  try {
    ReturnValue = tobj->SatsumaAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SatsumaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSatsuma(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSatsumaAttrW(void)
{
  
  ::Satsuma _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::SatsumaAttr(const ::Satsuma _attr)");
#endif
  
  ernie.seed("SatsumaAttrCall");
  GetSatsuma(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SatsumaAttrReturn");
  try {
     tobj->SatsumaAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SatsumaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSatsumaOp(void)
{
  
  ::Satsuma_slice * ReturnValue;
  ::Satsuma p1;
  ::Satsuma p2;
  ::Satsuma p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Satsuma_slice * ::idlServer::SatsumaOp(const ::Satsuma p1,::Satsuma p2,::Satsuma p3)");
#endif
  
  ernie.seed("SatsumaOpCall");
  GetSatsuma(p1);
  GetSatsuma(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SatsumaOpReturn");
  try {
    ReturnValue = tobj->SatsumaOp(p1,p2,p3);
  }
  catch (::idlServer::SatsumaExcpt &ex){
    if (!CheckidlServerSatsumaExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SatsumaOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSatsuma(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckSatsuma(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckSatsuma(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRedChillyAttr(void)
{
  
  ::RedChilly ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::RedChilly ::idlServer::RedChillyAttr()");
#endif
  
  ernie.seed("RedChillyAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedChillyAttrReturn");
  try {
    ReturnValue = tobj->RedChillyAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RedChillyAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRedChilly(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRedChillyAttrW(void)
{
  
  ::RedChilly _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::RedChillyAttr(const ::RedChilly &_attr)");
#endif
  
  ernie.seed("RedChillyAttrCall");
  GetRedChilly(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedChillyAttrReturn");
  try {
     tobj->RedChillyAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RedChillyAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerRedChillyOp(void)
{
  
  ::RedChilly ReturnValue;
  ::RedChilly p1;
  ::RedChilly p2;
  ::RedChilly p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::RedChilly ::idlServer::RedChillyOp(const ::RedChilly &p1,::RedChilly &p2,::RedChilly &p3)");
#endif
  
  ernie.seed("RedChillyOpCall");
  GetRedChilly(p1);
  GetRedChilly(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedChillyOpReturn");
  try {
    ReturnValue = tobj->RedChillyOp(p1,p2,p3);
  }
  catch (::idlServer::RedChillyExcpt &ex){
    if (!CheckidlServerRedChillyExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::RedChillyOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRedChilly(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckRedChilly(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckRedChilly(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCeleriacAttr(void)
{
  
  ::Celeriac ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Celeriac ::idlServer::CeleriacAttr()");
#endif
  
  ernie.seed("CeleriacAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CeleriacAttrReturn");
  try {
    ReturnValue = tobj->CeleriacAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CeleriacAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCeleriac(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCeleriacAttrW(void)
{
  
  ::Celeriac _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::CeleriacAttr(const ::Celeriac &_attr)");
#endif
  
  ernie.seed("CeleriacAttrCall");
  GetCeleriac(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CeleriacAttrReturn");
  try {
     tobj->CeleriacAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CeleriacAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerCeleriacOp(void)
{
  
  ::Celeriac ReturnValue;
  ::Celeriac p1;
  ::Celeriac p2;
  ::Celeriac p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Celeriac ::idlServer::CeleriacOp(const ::Celeriac &p1,::Celeriac &p2,::Celeriac &p3)");
#endif
  
  ernie.seed("CeleriacOpCall");
  GetCeleriac(p1);
  GetCeleriac(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CeleriacOpReturn");
  try {
    ReturnValue = tobj->CeleriacOp(p1,p2,p3);
  }
  catch (::idlServer::CeleriacExcpt &ex){
    if (!CheckidlServerCeleriacExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::CeleriacOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCeleriac(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckCeleriac(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckCeleriac(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBananaAttr(void)
{
  
  ::Banana ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Banana ::idlServer::BananaAttr()");
#endif
  
  ernie.seed("BananaAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BananaAttrReturn");
  try {
    ReturnValue = tobj->BananaAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BananaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBanana(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBananaAttrW(void)
{
  
  ::Banana _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BananaAttr(const ::Banana &_attr)");
#endif
  
  ernie.seed("BananaAttrCall");
  GetBanana(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BananaAttrReturn");
  try {
     tobj->BananaAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BananaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBananaOp(void)
{
  
  ::Banana ReturnValue;
  ::Banana p1;
  ::Banana p2;
  ::Banana p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Banana ::idlServer::BananaOp(const ::Banana &p1,::Banana &p2,::Banana &p3)");
#endif
  
  ernie.seed("BananaOpCall");
  GetBanana(p1);
  GetBanana(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BananaOpReturn");
  try {
    ReturnValue = tobj->BananaOp(p1,p2,p3);
  }
  catch (::idlServer::BananaExcpt &ex){
    if (!CheckidlServerBananaExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BananaOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBanana(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBanana(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBanana(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


