//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:13:53  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:16:57  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:29:26  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerBananaAttr(void)
{
  
  ::Banana ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Banana ::idlServer::BananaAttr()");
#endif
  
  ernie.seed("BananaAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BananaAttrReturn");
  try {
    ReturnValue = tobj->BananaAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BananaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBanana(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBananaAttrW(void)
{
  
  ::Banana _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BananaAttr(const ::Banana &_attr)");
#endif
  
  ernie.seed("BananaAttrCall");
  GetBanana(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BananaAttrReturn");
  try {
     tobj->BananaAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BananaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBananaOp(void)
{
  
  ::Banana ReturnValue;
  ::Banana p1;
  ::Banana p2;
  ::Banana p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Banana ::idlServer::BananaOp(const ::Banana &p1,::Banana &p2,::Banana &p3)");
#endif
  
  ernie.seed("BananaOpCall");
  GetBanana(p1);
  GetBanana(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BananaOpReturn");
  try {
    ReturnValue = tobj->BananaOp(p1,p2,p3);
  }
  catch (::idlServer::BananaExcpt &ex){
    if (!CheckidlServerBananaExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BananaOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBanana(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBanana(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBanana(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoAttr(void)
{
  
  ::Avocado ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado ::idlServer::AvocadoAttr()");
#endif
  
  ernie.seed("AvocadoAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoAttrReturn");
  try {
    ReturnValue = tobj->AvocadoAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocado(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoAttrW(void)
{
  
  ::Avocado _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::AvocadoAttr(const ::Avocado &_attr)");
#endif
  
  ernie.seed("AvocadoAttrCall");
  GetAvocado(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoAttrReturn");
  try {
     tobj->AvocadoAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoOp(void)
{
  
  ::Avocado ReturnValue;
  ::Avocado p1;
  ::Avocado p2;
  ::Avocado p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado ::idlServer::AvocadoOp(const ::Avocado &p1,::Avocado &p2,::Avocado &p3)");
#endif
  
  ernie.seed("AvocadoOpCall");
  GetAvocado(p1);
  GetAvocado(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoOpReturn");
  try {
    ReturnValue = tobj->AvocadoOp(p1,p2,p3);
  }
  catch (::idlServer::AvocadoExcpt &ex){
    if (!CheckidlServerAvocadoExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocado(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocado(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocado(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMangoAttr(void)
{
  
  ::Avocado::Mango ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango ::idlServer::MangoAttr()");
#endif
  
  ernie.seed("MangoAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MangoAttrReturn");
  try {
    ReturnValue = tobj->MangoAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MangoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMango(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMangoAttrW(void)
{
  
  ::Avocado::Mango _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::MangoAttr(const ::Avocado::Mango &_attr)");
#endif
  
  ernie.seed("MangoAttrCall");
  GetAvocadoMango(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MangoAttrReturn");
  try {
     tobj->MangoAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MangoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMangoOp(void)
{
  
  ::Avocado::Mango ReturnValue;
  ::Avocado::Mango p1;
  ::Avocado::Mango p2;
  ::Avocado::Mango p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango ::idlServer::MangoOp(const ::Avocado::Mango &p1,::Avocado::Mango &p2,::Avocado::Mango &p3)");
#endif
  
  ernie.seed("MangoOpCall");
  GetAvocadoMango(p1);
  GetAvocadoMango(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MangoOpReturn");
  try {
    ReturnValue = tobj->MangoOp(p1,p2,p3);
  }
  catch (::idlServer::MangoExcpt &ex){
    if (!CheckidlServerMangoExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MangoOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMango(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocadoMango(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocadoMango(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerStarfruitAttr(void)
{
  
  ::Avocado::Mango::Starfruit ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit ::idlServer::StarfruitAttr()");
#endif
  
  ernie.seed("StarfruitAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("StarfruitAttrReturn");
  try {
    ReturnValue = tobj->StarfruitAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::StarfruitAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruit(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerStarfruitAttrW(void)
{
  
  ::Avocado::Mango::Starfruit _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::StarfruitAttr(const ::Avocado::Mango::Starfruit &_attr)");
#endif
  
  ernie.seed("StarfruitAttrCall");
  GetAvocadoMangoStarfruit(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("StarfruitAttrReturn");
  try {
     tobj->StarfruitAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::StarfruitAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerStarfruitOp(void)
{
  
  ::Avocado::Mango::Starfruit ReturnValue;
  ::Avocado::Mango::Starfruit p1;
  ::Avocado::Mango::Starfruit p2;
  ::Avocado::Mango::Starfruit p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit ::idlServer::StarfruitOp(const ::Avocado::Mango::Starfruit &p1,::Avocado::Mango::Starfruit &p2,::Avocado::Mango::Starfruit &p3)");
#endif
  
  ernie.seed("StarfruitOpCall");
  GetAvocadoMangoStarfruit(p1);
  GetAvocadoMangoStarfruit(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("StarfruitOpReturn");
  try {
    ReturnValue = tobj->StarfruitOp(p1,p2,p3);
  }
  catch (::idlServer::StarfruitExcpt &ex){
    if (!CheckidlServerStarfruitExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::StarfruitOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruit(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocadoMangoStarfruit(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocadoMangoStarfruit(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerDatesAttr(void)
{
  
  ::Avocado::Mango::Starfruit::Dates ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates ::idlServer::DatesAttr()");
#endif
  
  ernie.seed("DatesAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DatesAttrReturn");
  try {
    ReturnValue = tobj->DatesAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::DatesAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDates(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerDatesAttrW(void)
{
  
  ::Avocado::Mango::Starfruit::Dates _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::DatesAttr(const ::Avocado::Mango::Starfruit::Dates &_attr)");
#endif
  
  ernie.seed("DatesAttrCall");
  GetAvocadoMangoStarfruitDates(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DatesAttrReturn");
  try {
     tobj->DatesAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::DatesAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerDatesOp(void)
{
  
  ::Avocado::Mango::Starfruit::Dates ReturnValue;
  ::Avocado::Mango::Starfruit::Dates p1;
  ::Avocado::Mango::Starfruit::Dates p2;
  ::Avocado::Mango::Starfruit::Dates p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates ::idlServer::DatesOp(const ::Avocado::Mango::Starfruit::Dates &p1,::Avocado::Mango::Starfruit::Dates &p2,::Avocado::Mango::Starfruit::Dates &p3)");
#endif
  
  ernie.seed("DatesOpCall");
  GetAvocadoMangoStarfruitDates(p1);
  GetAvocadoMangoStarfruitDates(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("DatesOpReturn");
  try {
    ReturnValue = tobj->DatesOp(p1,p2,p3);
  }
  catch (::idlServer::DatesExcpt &ex){
    if (!CheckidlServerDatesExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::DatesOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDates(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocadoMangoStarfruitDates(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocadoMangoStarfruitDates(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLimeAttr(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime ::idlServer::LimeAttr()");
#endif
  
  ernie.seed("LimeAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LimeAttrReturn");
  try {
    ReturnValue = tobj->LimeAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LimeAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLime(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLimeAttrW(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::LimeAttr(const ::Avocado::Mango::Starfruit::Dates::Lime &_attr)");
#endif
  
  ernie.seed("LimeAttrCall");
  GetAvocadoMangoStarfruitDatesLime(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LimeAttrReturn");
  try {
     tobj->LimeAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LimeAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLimeOp(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime ReturnValue;
  ::Avocado::Mango::Starfruit::Dates::Lime p1;
  ::Avocado::Mango::Starfruit::Dates::Lime p2;
  ::Avocado::Mango::Starfruit::Dates::Lime p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime ::idlServer::LimeOp(const ::Avocado::Mango::Starfruit::Dates::Lime &p1,::Avocado::Mango::Starfruit::Dates::Lime &p2,::Avocado::Mango::Starfruit::Dates::Lime &p3)");
#endif
  
  ernie.seed("LimeOpCall");
  GetAvocadoMangoStarfruitDatesLime(p1);
  GetAvocadoMangoStarfruitDatesLime(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LimeOpReturn");
  try {
    ReturnValue = tobj->LimeOp(p1,p2,p3);
  }
  catch (::idlServer::LimeExcpt &ex){
    if (!CheckidlServerLimeExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LimeOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLime(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocadoMangoStarfruitDatesLime(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocadoMangoStarfruitDatesLime(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLeekAttr(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime::Leek ::idlServer::LeekAttr()");
#endif
  
  ernie.seed("LeekAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LeekAttrReturn");
  try {
    ReturnValue = tobj->LeekAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LeekAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeek(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLeekAttrW(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::LeekAttr(const ::Avocado::Mango::Starfruit::Dates::Lime::Leek &_attr)");
#endif
  
  ernie.seed("LeekAttrCall");
  GetAvocadoMangoStarfruitDatesLimeLeek(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LeekAttrReturn");
  try {
     tobj->LeekAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LeekAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerLeekOp(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek ReturnValue;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek p1;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek p2;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime::Leek ::idlServer::LeekOp(const ::Avocado::Mango::Starfruit::Dates::Lime::Leek &p1,::Avocado::Mango::Starfruit::Dates::Lime::Leek &p2,::Avocado::Mango::Starfruit::Dates::Lime::Leek &p3)");
#endif
  
  ernie.seed("LeekOpCall");
  GetAvocadoMangoStarfruitDatesLimeLeek(p1);
  GetAvocadoMangoStarfruitDatesLimeLeek(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LeekOpReturn");
  try {
    ReturnValue = tobj->LeekOp(p1,p2,p3);
  }
  catch (::idlServer::LeekExcpt &ex){
    if (!CheckidlServerLeekExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::LeekOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeek(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeek(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeek(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGingerAttr(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger ::idlServer::GingerAttr()");
#endif
  
  ernie.seed("GingerAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GingerAttrReturn");
  try {
    ReturnValue = tobj->GingerAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GingerAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGinger(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGingerAttrW(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::GingerAttr(const ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger &_attr)");
#endif
  
  ernie.seed("GingerAttrCall");
  GetAvocadoMangoStarfruitDatesLimeLeekGinger(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GingerAttrReturn");
  try {
     tobj->GingerAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GingerAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGingerOp(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger ReturnValue;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger p1;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger p2;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger ::idlServer::GingerOp(const ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger &p1,::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger &p2,::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger &p3)");
#endif
  
  ernie.seed("GingerOpCall");
  GetAvocadoMangoStarfruitDatesLimeLeekGinger(p1);
  GetAvocadoMangoStarfruitDatesLimeLeekGinger(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GingerOpReturn");
  try {
    ReturnValue = tobj->GingerOp(p1,p2,p3);
  }
  catch (::idlServer::GingerExcpt &ex){
    if (!CheckidlServerGingerExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GingerOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGinger(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGinger(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGinger(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMintAttr(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint ::idlServer::MintAttr()");
#endif
  
  ernie.seed("MintAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MintAttrReturn");
  try {
    ReturnValue = tobj->MintAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MintAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMint(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMintAttrW(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::MintAttr(const ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint &_attr)");
#endif
  
  ernie.seed("MintAttrCall");
  GetAvocadoMangoStarfruitDatesLimeLeekGingerMint(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MintAttrReturn");
  try {
     tobj->MintAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MintAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerMintOp(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint ReturnValue;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint p1;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint p2;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint ::idlServer::MintOp(const ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint &p1,::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint &p2,::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint &p3)");
#endif
  
  ernie.seed("MintOpCall");
  GetAvocadoMangoStarfruitDatesLimeLeekGingerMint(p1);
  GetAvocadoMangoStarfruitDatesLimeLeekGingerMint(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MintOpReturn");
  try {
    ReturnValue = tobj->MintOp(p1,p2,p3);
  }
  catch (::idlServer::MintExcpt &ex){
    if (!CheckidlServerMintExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::MintOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMint(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMint(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMint(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGardenPeasAttr(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas ::idlServer::GardenPeasAttr()");
#endif
  
  ernie.seed("GardenPeasAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GardenPeasAttrReturn");
  try {
    ReturnValue = tobj->GardenPeasAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GardenPeasAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeas(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGardenPeasAttrW(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::GardenPeasAttr(const ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas &_attr)");
#endif
  
  ernie.seed("GardenPeasAttrCall");
  GetAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeas(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GardenPeasAttrReturn");
  try {
     tobj->GardenPeasAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GardenPeasAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGardenPeasOp(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas ReturnValue;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas p1;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas p2;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas ::idlServer::GardenPeasOp(const ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas &p1,::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas &p2,::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas &p3)");
#endif
  
  ernie.seed("GardenPeasOpCall");
  GetAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeas(p1);
  GetAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeas(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GardenPeasOpReturn");
  try {
    ReturnValue = tobj->GardenPeasOp(p1,p2,p3);
  }
  catch (::idlServer::GardenPeasExcpt &ex){
    if (!CheckidlServerGardenPeasExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GardenPeasOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeas(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeas(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeas(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSultanaAttr(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana ::idlServer::SultanaAttr()");
#endif
  
  ernie.seed("SultanaAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SultanaAttrReturn");
  try {
    ReturnValue = tobj->SultanaAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SultanaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeasSultana(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSultanaAttrW(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::SultanaAttr(const ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana &_attr)");
#endif
  
  ernie.seed("SultanaAttrCall");
  GetAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeasSultana(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SultanaAttrReturn");
  try {
     tobj->SultanaAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SultanaAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerSultanaOp(void)
{
  
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana ReturnValue;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana p1;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana p2;
  ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana ::idlServer::SultanaOp(const ::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana &p1,::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana &p2,::Avocado::Mango::Starfruit::Dates::Lime::Leek::Ginger::Mint::GardenPeas::Sultana &p3)");
#endif
  
  ernie.seed("SultanaOpCall");
  GetAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeasSultana(p1);
  GetAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeasSultana(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SultanaOpReturn");
  try {
    ReturnValue = tobj->SultanaOp(p1,p2,p3);
  }
  catch (::idlServer::SultanaExcpt &ex){
    if (!CheckidlServerSultanaExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::SultanaOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeasSultana(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeasSultana(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckAvocadoMangoStarfruitDatesLimeLeekGingerMintGardenPeasSultana(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


