//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/client.cpp
//
// Modifications:
// $Log: client.cpp,v $
// Revision 1.2  2000/03/29 17:16:51  mli
//   o   remove 'CORBA::Environment ev;'
//   o   int initialise
//
// Revision 1.1.1.1  2000/02/09 17:17:07  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.4  1997/12/09 11:30:08  james
// Code Regenerated
//
//
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
idlServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {

#ifdef DEBUG
	  ::tet_infoline("client: connecting to proxy");
	  ::tet_printf("client: using %s", stringifiedIOR);
#endif
	  
	  try {
	      obj = orb->getObject(stringifiedIOR);
	  }
	  catch (CORBA::SystemException &ex) {
	      ::tet_infoline("client: getObj failed");
	      ::tet_printf("client: minor code %d", ex.minor());
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  if (obj == 0)
	  {
	      ::tet_infoline("client: failed connecting to proxy");
	      ::tet_result(TET_FAIL);
	      return;
	  }
	  
	  try {
	      tobj = idlServer::_narrow(obj);
	  }
	  catch (CORBA::Exception &ex) {
	      ::tet_infoline("client: failed to narrow object reference.");
	  }
	  initialised = 1;

	  char * tout = 0;
	  tout = getenv("VSORB_EVENT_TIMEOUT");

	  unsigned long timeout = (tout) ? atoi(tout) : 15;
	  orb->setTimeout(tobj, timeout);
      }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testidlServerBroadbeansAttr(void)
{
  
  
  ::Broadbeans ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Broadbeans ::idlServer::BroadbeansAttr()");
#endif
  
  ernie.seed("BroadbeansAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BroadbeansAttrReturn");
  try {
    ReturnValue = tobj->BroadbeansAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BroadbeansAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBroadbeans(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBroadbeansAttrW(void)
{
  
  
  ::Broadbeans _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::BroadbeansAttr(const ::Broadbeans &_attr)");
#endif
  
  ernie.seed("BroadbeansAttrCall");
  GetBroadbeans(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BroadbeansAttrReturn");
  try {
     tobj->BroadbeansAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BroadbeansAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerBroadbeansOp(void)
{
  
  
  ::Broadbeans ReturnValue;
  
  ::Broadbeans p1;
  
  ::Broadbeans p2;
  
  ::Broadbeans p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Broadbeans ::idlServer::BroadbeansOp(const ::Broadbeans &p1,::Broadbeans &p2,::Broadbeans &p3)");
#endif
  
  ernie.seed("BroadbeansOpCall");
  GetBroadbeans(p1);
  GetBroadbeans(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BroadbeansOpReturn");
  try {
    ReturnValue = tobj->BroadbeansOp(p1,p2,p3);
  }
  catch (::idlServer::BroadbeansExcpt &ex){
    if (!CheckidlServerBroadbeansExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::BroadbeansOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBroadbeans(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBroadbeans(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBroadbeans(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoAttr(void)
{
  
  ::Broadbeans::Avocado ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Broadbeans::Avocado ::idlServer::AvocadoAttr()");
#endif
  
  ernie.seed("AvocadoAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoAttrReturn");
  try {
    ReturnValue = tobj->AvocadoAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBroadbeansAvocado(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoAttrW(void)
{
  
  ::Broadbeans::Avocado _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::AvocadoAttr(const ::Broadbeans::Avocado &_attr)");
#endif
  
  ernie.seed("AvocadoAttrCall");
  GetBroadbeansAvocado(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoAttrReturn");
  try {
     tobj->AvocadoAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerAvocadoOp(void)
{
  
  ::Broadbeans::Avocado ReturnValue;
  ::Broadbeans::Avocado p1;
  ::Broadbeans::Avocado p2;
  ::Broadbeans::Avocado p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Broadbeans::Avocado ::idlServer::AvocadoOp(const ::Broadbeans::Avocado &p1,::Broadbeans::Avocado &p2,::Broadbeans::Avocado &p3)");
#endif
  
  ernie.seed("AvocadoOpCall");
  GetBroadbeansAvocado(p1);
  GetBroadbeansAvocado(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("AvocadoOpReturn");
  try {
    ReturnValue = tobj->AvocadoOp(p1,p2,p3);
  }
  catch (::idlServer::AvocadoExcpt &ex){
    if (!CheckidlServerAvocadoExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::AvocadoOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBroadbeansAvocado(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBroadbeansAvocado(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBroadbeansAvocado(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGalaAppleAttr(void)
{
  
  ::Broadbeans::GalaApple ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Broadbeans::GalaApple ::idlServer::GalaAppleAttr()");
#endif
  
  ernie.seed("GalaAppleAttrCall");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GalaAppleAttrReturn");
  try {
    ReturnValue = tobj->GalaAppleAttr();
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GalaAppleAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBroadbeansGalaApple(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGalaAppleAttrW(void)
{
  
  ::Broadbeans::GalaApple _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::idlServer::GalaAppleAttr(const ::Broadbeans::GalaApple &_attr)");
#endif
  
  ernie.seed("GalaAppleAttrCall");
  GetBroadbeansGalaApple(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GalaAppleAttrReturn");
  try {
     tobj->GalaAppleAttr(_attr);
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GalaAppleAttr");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testidlServerGalaAppleOp(void)
{
  
  ::Broadbeans::GalaApple ReturnValue;
  ::Broadbeans::GalaApple p1;
  ::Broadbeans::GalaApple p2;
  ::Broadbeans::GalaApple p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Broadbeans::GalaApple ::idlServer::GalaAppleOp(const ::Broadbeans::GalaApple &p1,::Broadbeans::GalaApple &p2,::Broadbeans::GalaApple &p3)");
#endif
  
  ernie.seed("GalaAppleOpCall");
  GetBroadbeansGalaApple(p1);
  GetBroadbeansGalaApple(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GalaAppleOpReturn");
  try {
    ReturnValue = tobj->GalaAppleOp(p1,p2,p3);
  }
  catch (::idlServer::GalaAppleExcpt &ex){
    if (!CheckidlServerGalaAppleExcpt(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::idlServer::GalaAppleOp");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBroadbeansGalaApple(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p2");
  if (!CheckBroadbeansGalaApple(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  AddName("p3");
  if (!CheckBroadbeansGalaApple(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


