//
//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/iiop/proxy/proxy.cpp
//
// Purpose:
//
// Modifications:
// $Log: proxy.cpp,v $
// Revision 1.5  2001/02/16 17:56:07  mli
// Update path to tin.idl.
//
// Revision 1.4  2000/09/21 12:45:52  corval2
// Adaptation for the location of tin.idl.
//
// Revision 1.3  2000/02/11 16:12:58  mli
//   o    void (*tet_startup)(void) -> void (*tet_startup)()
//   o    void (*tet_cleanup)(void) -> void (*tet_cleanup)()
//
// Revision 1.2  2000/02/11 16:09:39  mli
// o  Decl. of index i extracted out of the 'for' loop (2x)
// o  Return type  CORBA::ULong -> long for:
//         processAttribute, processOperation, processException
//
// Revision 1.19  1998/08/28 15:42:54  james
// RequestRecord added
//
// Revision 1.18  1998/06/04 15:01:10  stephen
// Updated locateObject support for forwarded object references.
//
// Revision 1.17  1998/05/11 14:20:52  stephen
// Node reference semantic changes propagated.
//
// Revision 1.16  1997/12/15 11:14:43  kevin
// Final GA version with correct op count.
//
// Revision 1.15  1997/12/14 15:55:23  stephen
// Added TRANSPORT_ERROR return code.
//
// Revision 1.14  1997/12/12 15:18:48  stephen
// Updated to reflect number of functions under test.
//
// Revision 1.13  1997/12/05 15:41:03  stephen
// Tidying up.
//
// Revision 1.12  1997/12/04 14:22:13  stephen
// Better debug info, small leak fixed, and updated
// to use vsOrbString class.
//
// Revision 1.11  1997/11/27 12:24:35  cwatson
// Removed Orbix specific code.
//
// Revision 1.10  1997/11/21 13:03:49  stephen
// Consistent error reporting.
//
// Revision 1.9  1997/11/17 15:29:13  stephen
// Added exception support.
//
// Revision 1.8  1997/11/02 12:14:53  stephen
// Sync points clarified.
//
// Revision 1.7  1997/11/01 16:13:08  stephen
// Added check to locate Object.
//
// Revision 1.6  1997/10/31 16:24:17  stephen
// Undeeded VS_TIMEOUT param.
//
// Revision 1.5  1997/10/31 13:15:36  stephen
// Updated systems used in dTest.
//
// Revision 1.4  1997/10/30 19:34:00  stephen
// Even more enhanced error handling.
//
// Revision 1.3  1997/10/30 16:06:34  james
// Enhanced error reporting
//
// Revision 1.2  1997/10/28 15:37:32  stephen
// Added timeout to iiopProxy() function.
//
// Revision 1.1  1997/10/22 13:37:57  stephen
// Initial IIOP Proxy configured to run under TET.
//
//


#include <proxy.hh>

extern "C" {
#include <tet_api.h>
}

int sysServer[1];	
int sysClient[1];	
int *remSystems;
int numSystems;

// Global variables
char ebuf[1024];
Transport * transport;
Specification specification;
// Cached ProfileBody for server's IOR
ProfileBody * sBody;
// Cached ProfileBody for proxy's IOR
ProfileBody * pBody;
unsigned char Connected = 0;

extern FILE * yyin;
extern Ernie ernie;

IOR * sIOR = 0;
IOR * pIOR = 0;
char * proxyStr = 0;
unsigned long timeout = 0;
Node** tcIndirect = 0;
RequestRecord requestRecord;



CORBA::ULong requestIds[MAX_REQUESTS];
CORBA::Char *requestNames[MAX_REQUESTS];


RequestRecord::RequestRecord()
{
        memset(requestIds,0,sizeof(requestIds));
        memset(requestNames,0,sizeof(requestNames));
        expectedRequests=0;
        expectedReplies=0;
        registeredRequests = 0;
}

void
RequestRecord::Initialise(int num)
{
        for (int i =0; i < MAX_REQUESTS; ++i)
                if (requestNames[i] != 0)
                        free(requestNames[i]);
        memset(requestIds,0,sizeof(requestIds));
        memset(requestNames,0,sizeof(requestNames));

        expectedRequests=num;
        expectedReplies=num;
        registeredRequests = 0;
}
    
void
RequestRecord::AddRequest(CORBA::ULong id, CORBA::Char *name, CORBA::Boolean responseExpected)
{
        if (name[0] != '_'
	 || !strncmp(name, "_get", 4)
	 || !strncmp(name, "_set", 4))
        {
                --expectedRequests;
                if (!responseExpected)
                        --expectedReplies;
        }

        if (registeredRequests < MAX_REQUESTS)
        {
                requestIds[registeredRequests] = id;
                requestNames[registeredRequests] = strdup(name);
                ++registeredRequests;
        }
        else
        {
                tet_infoline("proxy: Error: Too many requests per test increase the MAX_REQUESTS value in proxy.hh and recompile.");
                tet_result(TET_FAIL);
        }
}

void
RequestRecord::RegisterReply(CORBA::ULong id)
{
  int i;
  for (i =0; i < registeredRequests && requestIds[i] != id; ++i);
        if (i == registeredRequests)
        {
                tet_printf("proxy: Error: Received reply containing unknown request id %d", id);
                tet_result(TET_FAIL);
        }
        else
        {
        if (requestNames[i][0] != '_'
	 || !strncmp(requestNames[i], "_get", 4)
	 || !strncmp(requestNames[i], "_set", 4))
                --expectedReplies;
        }
}

CORBA::Char*
RequestRecord::GetName(CORBA::ULong id)
{
  int i;
  for (i =0; i < registeredRequests && requestIds[i] != id; ++i);
        if (i == registeredRequests)
        {
                tet_printf("proxy: Error: Received reply containing unknown request id %d", id);
                tet_result(TET_FAIL);
                return 0;
        }
        else
                return requestNames[i];
}

CORBA::Char*
RequestRecord::GetSeed(CORBA::ULong id, CORBA::Boolean isARequest)
{
        CORBA::Char *name;
        CORBA::Char *seed;

        if ((name = GetName(id)) == 0)
                return 0;

        seed = new char [strlen(name) + 10];
        strcpy(seed,(name[0] == '_' && strlen(name) > 5)?name+5:name);
        strcat(seed,(isARequest)?"Call":"Return");

        return seed;
}

CORBA::Boolean
RequestRecord::notFinished()
{
        return (expectedRequests != 0 || expectedReplies !=0);
}

void
proxyStartUp( void ) 
{
    // Find out which other systems are taking part in this test.

    if ((numSystems=tet_remgetlist(&remSystems)) != 2)
    {
      tet_printf("proxy: Error tet_remgetlist reports that there are %d other systems  participating in this test.",numSystems);
	    tet_result(TET_FAIL);
	    return;
    }

    if (!(remSystems[0] %2))
      sysServer[0] = remSystems[0];
    else if (!(remSystems[1] %2))
      sysServer[0] = remSystems[1];
    else
    {
      tet_infoline("proxy: Error unable to identify the server from system id list.");
      tet_infoline("proxy: The server must be configured in tetdist.cfg as having an even number for its system id.");
	    tet_result(TET_FAIL);
	    return;
    }

    if (remSystems[0] %2)
      sysClient[0] = remSystems[0];
    else if (remSystems[1] %2)
      sysClient[0] = remSystems[1];
    else
    {
      tet_infoline("proxy: Error unable to identify the client from system id list.");
      tet_infoline("proxy: The client must be configured in tetdist.cfg as having an odd number for its system id.");
	    tet_result(TET_FAIL);
	    return;
    }

    // Initialise Transport instance
    transport = Transport::Instance();
    if (!transport->initialise())
    {
	tet_infoline("proxy: unable to initialise Transport");
	tet_result(TET_FAIL);
	return;
    }

    // Open the IDL file
    if ( (yyin = fopen("../../../idl/iiop/tin.idl", "r")) == 0)
    {
	tet_infoline("proxy: unable to open IDL input file");
	tet_result(100);
	
	return;
    }

    // Initialise the parse tree 
    if (yyparse() != 0)
    {
	tet_infoline("proxy: unable to parse IDL input file");
	tet_result(101);

	return;
    }

    fclose(yyin);
    yyin = 0;
    

    char rcvbuf[2048];
    
    struct tet_synmsg msg;

    msg.tsm_flags = TET_SMRCVMSG;
    msg.tsm_dlen = sizeof(rcvbuf);
    msg.tsm_data = rcvbuf;

    memset(rcvbuf,0,2048);
    if (tet_remsync(SERVER_TO_PROXY, sysServer, 1, VS_TIMEOUT, TET_SV_YES, &msg) != 0) {
	tet_infoline("proxy: failed to sync with server");
	tet_result(TET_FAIL);
	return;
    }
#ifdef DEBUG
    tet_infoline("proxy: sync point 1000 OK");
    tet_infoline("proxy: received following IOR");
    tet_printf("%s", rcvbuf);

#endif
    
    // Unmarshall server IOR
    sIOR = new IOR(rcvbuf);

    // Cache the servers ProfileBody
    sBody = sIOR->extractInternetTag();

    // Create and externalise the proxified IOR
    pIOR = createProxyIOR(sIOR);
    proxyStr = pIOR->toString();

    msg.tsm_flags = TET_SMSNDMSG;
    msg.tsm_dlen = strlen(proxyStr) + 1;
    msg.tsm_data = proxyStr;

#ifdef DEBUG
    tet_infoline("proxy: sending following IOR");
    tet_printf("%s", proxyStr);
#endif

    if (tet_remsync(PROXY_TO_CLIENT, sysClient, 1, VS_TIMEOUT, TET_SV_YES, &msg) != 0) {
	tet_infoline("proxy: failed to sync with client");
	return;
    }
    
#ifdef DEBUG
    tet_infoline("proxy: sync point 2000 OK");
#endif

    char * tout = 0;
    tout = getenv("VSORB_EVENT_TIMEOUT");

    timeout = (tout) ? atoi(tout) : 15;
}

void
proxyCleanUp ( void )
{
    if (sIOR)
	delete sIOR;

    if (pIOR)
	delete pIOR;

    //if (proxyStr)
	//delete proxyStr;
    
    sIOR = pIOR = 0;

    if (tcIndirect != 0)
	delete [] tcIndirect;
}


// iiopProxy is called multiple times so that
// the 3 parties (client, server and proxy)
// have the same number of IC's
void
iiopProxy ( void )
{
    CORBA::ULong fd = 0;
    CORBA::Long result = 0;

    requestRecord.Initialise(1);

    while (requestRecord.notFinished())
    {
    	fd = transport->processEvent(timeout);

	    if (fd == 0)
	    {
	      tet_infoline("proxy: timedout on event");
	      tet_result(TET_FAIL);
	      return;
    	}

      if (!Connected)
      {
        LocateStatusType  lst;
#ifdef DEBUG
      	tet_infoline("proxy: connecting to server");
#endif

        if (transport->connectTo(sBody->host(), sBody->port()) == -1)
	      {
	        tet_infoline("proxy: failed connecting to server");
	        tet_result(TET_FAIL);
	        return;
	      }
        while ((lst=locateObject(transport->serverFd(), sBody)) == OBJECT_FORWARD)
        {
          transport->cleanUp();
          if (transport->connectTo(sBody->host(), sBody->port()) == -1)
	        {
	            tet_infoline("proxy: failed connecting to server");
	            tet_result(TET_FAIL);
	            return;
	        }
        }
        if (lst == UNKNOWN_OBJECT)
        {
	        tet_infoline("proxy: failed to locate object");
	        tet_result(TET_FAIL);
	        return;
        }
        Connected = 1;
      }
    
	
  	  result = processEvent(fd);
	  
	    if (result == MARSHALL_ERROR)
                tet_result(TET_FAIL);
            else if (result == CLIENT_EXITED ||
	        result == SERVER_EXITED ||
	        result == CONFIG_ERROR ||
	        result == SEND_REQUEST_ERR ||
	        result == SEND_REPLY_ERR ||
	        result == TRANSPORT_ERROR)
	    {
	        tet_result(TET_FAIL);
	        return;
	    }
    }

    tet_result(TET_PASS);
    return;
};

ProfileBody *
dumpIOR(IOR *ior)
{
    sprintf((char *)&ebuf[0], "   TypeID:        %s\n", ior->typeId());

    ProfileBody *body = ior->extractInternetTag();
    
    sprintf((char *)&ebuf[0], "   Version:       %d.%d\n",
	    (CORBA::ULong)body->ver().major(),
	    (CORBA::ULong)body->ver().minor());
    
    sprintf((char *)&ebuf[0], "   Host:          %s\n",body->host());
    sprintf((char *)&ebuf[0], "   Port:          %d\n",body->port());

    return body;
};

LocateStatusType
locateObject(CORBA::ULong fd, ProfileBody *& body)
{
    IIOPLocateRequestMessage locReq;

    locReq.request_id(0);
    locReq.object_key(body->objectKey());
    locReq.marshall();
    
    CORBA::Long locLen = locReq.getMarshalledBufferLen();
    unsigned char * locBuf = locReq.getMarshalledBuffer();
    
    locReq.updateStreamLength(locLen);
    
    CORBA::Long numBytes = transport->transmit(fd, locBuf, locLen);

    // peek at the fixed size header;
    locBuf = new unsigned char[12];
    
    locLen = transport->receive(fd, locBuf, 12, MSG_PEEK);

    if ( (locLen == 0) || (locLen == -1) )
	return UNKNOWN_OBJECT;
    
    IIOPMessageHeader msgHdr(locBuf, locLen);
    delete locBuf;

    if (msgHdr.unmarshall() == -1)
	return UNKNOWN_OBJECT;
    
    CORBA::ULong msgSize = msgHdr.message_size();
    msgSize +=12;
    
    // read the full message;
    locBuf = new unsigned char[msgSize];
    locLen = transport->receive(fd, locBuf, msgSize);

    // Invalid message length
    if ( (locLen == 0) || (locLen == -1) )
	return UNKNOWN_OBJECT;
    
    IIOPLocateReplyMessage locReply(locBuf, locLen);
    
    locReply.unmarshall();
    
    LocateStatusType locRep;
    locRep = locReply.locate_status();
    
    delete locBuf;
    
#ifdef DEBUG
	tet_printf("proxy: locRep = %d",locRep);
#endif
    switch (locRep)
    {
    case UNKNOWN_OBJECT:
	break;
    case OBJECT_HERE:
#ifdef DEBUG
	tet_infoline("proxy: Object found");
#endif
	break;
    case OBJECT_FORWARD:
	IOR *ior = new IOR();
	
	locReply >> ior;

        ProfileBody * msgBody = ior->extractInternetTag();

        delete body;
        body = msgBody;

#ifdef DEBUG
	tet_infoline("proxy: Object has moved. Trying to locate...");
        tet_printf("%s", ior->toString());
#endif
	break;
    };

    return locRep;
}


IOR*
createProxyIOR(IOR* serverIOR)
{
    //
    // Build an IOR with the same type information
    // and objectKey but our host & port 
    // 
    
    pBody = new ProfileBody;

    pBody->host(transport->IPAddress());
    pBody->port(transport->listeningPort());

    pBody->objectKey(serverIOR->profileBody()->objectKey());

    IOR * pIOR = new IOR;

    pIOR->typeId(serverIOR->typeId());

    pIOR->insertInternetTag(pBody);

    return pIOR;
}

CORBA::Long
processEvent(CORBA::ULong fd)
{
    CORBA::ULong ret = 0;

    // peek at the fixed size header;
    vsOrbString buffer(12);
    CORBA::Long bufLen = 0;
    
    bufLen = transport->receive(fd, buffer, 12, MSG_PEEK);

    if (bufLen == 0)
    {
	if (fd == transport->clientFd()) 
	{
	    tet_infoline("proxy: client has exited");
	    return CLIENT_EXITED;
	}
	else
	{
	    tet_infoline("proxy: server has exited");
	    return SERVER_EXITED;
	}
    }
    
    if (bufLen == -1)
    {
	tet_infoline("proxy: Failed to read");
	return TRANSPORT_ERROR;
    }
    
    if (bufLen != 12)
    {
	tet_infoline("proxy: Incomplete IIOP message header");
  hexdump(buffer, bufLen);
	return TRANSPORT_ERROR;
    }
    
    IIOPMessageHeader msgHdr(buffer, bufLen);
    

    if (msgHdr.unmarshall() == -1)
    {
	tet_infoline("proxy: Malformed IIOP message header");
        hexdump(buffer, bufLen);
	return TRANSPORT_ERROR;
    }
    
    CORBA::ULong msgSize = msgHdr.message_size();
    msgSize +=12;
    
    // read the full message
    buffer.reinit(msgSize);
    
    bufLen = transport->receive(fd, buffer, msgSize);

    if ( (bufLen == 0) || (bufLen == -1) )
	return TRANSPORT_ERROR;
    
    switch (msgHdr.message_type())
    {
    case Request:
    {
	_Identifier opName;
	Node * node;
	CORBA::Long numBytes = 0;

        if (tcIndirect != 0)
	    delete [] tcIndirect;
	
	tcIndirect = new Node *[bufLen];
	memset(tcIndirect, 0, sizeof(Node *)*bufLen);
	
	// Unmarshall the Request
	IIOPRequestMessage inReqMsg(buffer, bufLen);
	inReqMsg.unmarshall();

        requestRecord.AddRequest(inReqMsg.request_id(),inReqMsg.operation(),inReqMsg.response_expected());
        CORBA::Char * seed = requestRecord.GetSeed(inReqMsg.request_id(),TRUE);
        ernie.seed(seed);
				delete [] seed;
				seed = 0;
#ifdef DEBUG
	tet_printf("proxy: request for %s ", inReqMsg.operation());
	hexdump(buffer, bufLen);
#endif
	

	// Attribute methods are processed differently
	// from normal IDL method calls
	if (!strncmp(inReqMsg.operation(), "_get", 4)
	 || !strncmp(inReqMsg.operation(), "_set", 4))
	{
	    // The identifier in the IDL parse
	    // tree won't contain the prefix
	    opName = inReqMsg.operation() + 5;
	    
	    
	    node = findAttrDef(opName, specification);
	    if (node == 0)
	    {
		tet_printf("proxy: Error: operation '%s' not found in IDL file",(char *)opName);
		return CONFIG_ERROR;
	    }
	    
            if (processAttribute(node, inReqMsg, inReqMsg.operation()[1] =='s') != 0)
	    {
		tet_printf("proxy: detected marshalling error processing request to %s", inReqMsg.operation());
#ifndef DEBUG
                hexdump(buffer, bufLen);
#endif

		numBytes = transport->transmit(transport->serverFd(),
					       buffer, bufLen);
#ifdef DEBUG
		tet_printf("proxy: sent %d bytes to server", numBytes);
#endif
		if (numBytes != bufLen)
		    tet_infoline("proxy: failed to send request to server");
		else
	            ret = 0;

		tet_result(TET_FAIL);
		break;
		
	    }
	}
	else if (inReqMsg.operation()[0] != '_') // Don't look for functions such as _is_a in the IDL file.
	{
            // Find the OpDcl in the tree
	    opName = inReqMsg.operation();
	    node= findOpDef(opName, specification);
	    if (node == 0)
	    {
		tet_printf("proxy: Error: operation '%s' not found in IDL file",(char *)opName);
		return CONFIG_ERROR;
	    }
	    
	    
	    if (processOperation(node, inReqMsg) != 0)
	    {
		tet_printf("proxy: detected marshalling error processing request to %s", inReqMsg.operation());
#ifndef DEBUG
                hexdump(buffer, bufLen);
#endif

		numBytes = transport->transmit(transport->serverFd(),
					       buffer, bufLen);
#ifdef DEBUG
		tet_printf("proxy: sent %d bytes to server", numBytes);
#endif
		if (numBytes != bufLen)
		    tet_infoline("proxy: failed to send request to server");
		else
	            ret = 0;

		tet_result(TET_FAIL);
		break;
	    }
	}
	
	numBytes = transport->transmit(transport->serverFd(),
				       buffer, bufLen);
#ifdef DEBUG
	tet_printf("proxy: sent %d bytes to server", numBytes);
#endif
	if (numBytes != bufLen)
	{
	    tet_infoline("proxy: failed to send request to server");
	    ret = SEND_REQUEST_ERR;
	}
	else
            ret = 0;
	
	break;
    }	
    case Reply:
    {
        IIOPReplyMessage inRepMsg(buffer, bufLen);
	inRepMsg.unmarshall();
	CORBA::ULong numBytes = 0;
        CORBA::Char *currentOp;

	if (tcIndirect != 0)
	    delete [] tcIndirect;

	tcIndirect = new Node *[bufLen];
	memset(tcIndirect, 0, sizeof(Node *)*bufLen);

        requestRecord.RegisterReply(inRepMsg.request_id());
        CORBA::Char * seed = requestRecord.GetSeed(inRepMsg.request_id(),FALSE);
        ernie.seed(seed);
				delete [] seed;
				seed = 0;
        currentOp = requestRecord.GetName(inRepMsg.request_id());

#ifdef DEBUG
	tet_printf("proxy: reply to %s ", currentOp);
	hexdump(buffer, bufLen);
#endif

	switch (inRepMsg.reply_status())
	{
	case NO_EXCEPTION:
	{
	    _Identifier opName;
	    Node * node;
	    
	    // Attribute methods are processed differently
	    // from normal IDL method calls
	    if (!strncmp(currentOp, "_get", 4)
	     || !strncmp(currentOp, "_set", 4))
	    {
		// The identifier in the IDL parse
		// tree won't contain the prefix
		opName = currentOp + 5;
		node = findAttrDef(opName, specification);

                if (node == 0)
		{
		    tet_printf("proxy: Error: operation '%s' not found in IDL file",(char *)opName);
		    return CONFIG_ERROR;
		}
		
		if (processAttribute(node, inRepMsg, currentOp[1] =='s') != 0)
		{
		    tet_printf("proxy: detected marshalling error processing reply to %s", currentOp);
#ifndef DEBUG
                    hexdump(buffer, bufLen);
#endif

		    numBytes = transport->transmit(transport->clientFd(),
					   buffer, bufLen);
		    if (numBytes != bufLen)
			tet_infoline("proxy: failed to send reply to client");
#ifdef DEBUG
		    else
			tet_printf("proxy: sent %d bytes to client", numBytes);
#endif

		    return MARSHALL_ERROR;
		}
	    }
            else if (currentOp[0] != '_') // Don't look for functions such as _is_a in the IDL file.
	    {
		// Find the OpDcl in the tree
		opName = currentOp;
		node= findOpDef(opName, specification);
		if (node == 0)
		{
		    tet_printf("proxy: Error: operation '%s' not found in IDL file",(char *)opName);
		    return CONFIG_ERROR;
		}
		
		if (processOperation(node, inRepMsg) != 0)
		{
		    tet_printf("proxy: detected marshalling error processing reply to %s", currentOp);
#ifndef DEBUG
                    hexdump(buffer, bufLen);
#endif
		    numBytes = transport->transmit(transport->clientFd(),
					   buffer, bufLen);
		    if (numBytes != bufLen)
			tet_infoline("proxy: failed to send reply to client");
#ifdef DEBUG
		    else
			tet_printf("proxy: sent %d bytes to client", numBytes);
#endif

		    return MARSHALL_ERROR;
		}
	    }
	    
	    numBytes = transport->transmit(transport->clientFd(),
					   buffer, bufLen);
	    if (numBytes != bufLen)
	    {
		tet_infoline("proxy: failed to send reply to client");
		ret = SEND_REPLY_ERR;
	    }
	    else
	    {
#ifdef DEBUG
		tet_printf("proxy: sent %d bytes to client", numBytes);
#endif
		ret = 0;
	    }
	    
	    currentOp = 0;
	}
	break;

    case USER_EXCEPTION:
    {
	_Identifier opName;
	Node * node;
	char * repid;
	
	inRepMsg >> repid;
#ifdef DEBUG
	tet_printf("proxy: received exception %s", repid);
#endif

	// Find the OpDcl in the tree
	opName = currentOp;
	node = findOpDef(opName, specification);
	if (node == 0)
        {
		tet_printf("proxy: Error: operation '%s' not found in IDL file",(char *)opName);
	    return CONFIG_ERROR;
	}
	
	if (processException(node, inRepMsg) != 0)
	{
	    tet_printf("proxy: detected marshalling error processing exceptional reply to %s", currentOp);
#ifndef DEBUG
            hexdump(buffer, bufLen);
#endif
	    numBytes = transport->transmit(transport->clientFd(),
					   buffer, bufLen);
	    return MARSHALL_ERROR;
	}

	numBytes = transport->transmit(transport->clientFd(),
					   buffer, bufLen);
	if (numBytes != bufLen)
        {
	    tet_infoline("proxy: failed to send reply to client");
	    ret = SEND_REPLY_ERR;
	}
	else
        {
#ifdef DEBUG
	    tet_printf("proxy: sent %d bytes to client", numBytes);
#endif
	    ret = 0;
	}
	currentOp = 0;
    }
	
	break;

    case SYSTEM_EXCEPTION:
	numBytes = transport->transmit(transport->clientFd(),
					   buffer, bufLen);
	if (numBytes != bufLen)
        {
	    tet_infoline("proxy: failed to send SystemException to client");
	    ret = SEND_REPLY_ERR;
	}
	else
        {
#ifdef DEBUG
	    tet_printf("proxy: sent SystemException of %d bytes to client", numBytes);
#endif
	    ret = 0;
	}
	break;

    default:
	break;
    }
    }
	break;

    case CancelRequest:
	//cout << "CancelMessage message received" << endl;
	break;

    case LocateRequest:
	//cout << "LocateRequest message received" << endl;
	break;

    case LocateReply:
	//cout << "LocateReply message received" << endl;
	break;

    case CloseConnection:
	//cout << "CloseConnection message received" << endl;
	break;

    case MessageError:
	//cout << "MessageError message received" << endl;
	break;

    default:
	//cout << "Ooops whats this?" << endl;
	break;
    };

    return ret;
}

long
processAttribute(Node *node, IIOPMessageHeader &msg, CORBA::Boolean set)
{
    
    if ( (set && (msg.message_type() == Request))
	|| (!set && (msg.message_type() == Reply)) )
    {
	// detach a local copy to process on but
	// leave it connected to the parent
	// so outer scopes can be searched for identifiers
	AttrDcl def((const AttrDcl &)*node);

	def.Parent(node->Parent());
	return def.decodeIIOP(msg);
    }
    return 0;
}

long
processOperation(Node * node, IIOPMessageHeader &msg)
{
    // Check we actually found the operation 
    // in the IDL file
    if (node == 0)
	return -1;

    OpDcl def((const OpDcl &)*node);
    def.Parent(node->Parent());

    return def.decodeIIOP(msg);
}

long
processException(Node * opNode, IIOPMessageHeader &msg)
{
    // Looking for the exception declaration(s)
    // associated with this operation
    if (opNode == 0)
	return -1;

    // Ensure the operation actually
    // has a raises clause

    if ( (*opNode)[IdRaisesExpr] == 0 )
    {
	tet_infoline("proxy: operation has no raises clause");
	tet_infoline("proxy: yet server marshalled a USER EXCEPTION");
	    
	return -1;
    }    
    
    // Check whether the operation
    // supportas multiple exceptions

    CORBA::Long index = 0;
    
    if ( (*opNode)[IdRaisesExpr].Length() > 1 )
	index = ernie.getULong() % (*opNode)[IdRaisesExpr].Length();
    
    // Find the associated scopedname
    // and delegate the processing
    Node * def = 0;

    def = FindDefinition(*opNode, (_ScopedName&)(*opNode)[IdRaisesExpr][index], 0);

    if (def == 0)
    {
	tet_infoline("proxy: coundn't find exception definition");
	return -1;
    }
    
    ExceptDcl exDef((const ExceptDcl &)*def);
    exDef.Parent(def->Parent());

    return exDef.decodeIIOP(msg);
}



void hexdump(
    unsigned char *ptr,
    int len)
{
    char  buf[100];
    char  tmp[100];
    int  i,n,x;

    for (i = 0; i < len;i+=16)
    {
	memset(buf,0,100);
	sprintf(buf,"%4.1d: ",i);
	
	for (n = i; n < i + 16; ++n)
	{
	    if (n < len)
	    {
		sprintf(tmp,"%2.2x",(unsigned int)*(ptr+n));
		strcat(buf,tmp);
	    }
	    else
		strcat(buf,"  ");
	    if (n%2)
		strcat(buf," ");
	}
	strcat(buf,"  ");

	for (n = i,x = strlen(buf); n < i + 16 && n < len; ++n)
	    buf[x++] = (isprint(*(ptr+n)) && !iscntrl(*(ptr+n)))? *(ptr+n): '.';

	tet_infoline(buf);
    }
}



extern "C" {
    void (*tet_startup)() = proxyStartUp;
    void (*tet_cleanup)() = proxyCleanUp;
    struct tet_testlist tet_testlist[] = { {iiopProxy,1},
		{iiopProxy,2},
		{iiopProxy,3},
		{iiopProxy,4},
		{iiopProxy,5},
		{iiopProxy,6},
		{iiopProxy,7},
		{iiopProxy,8},
		{iiopProxy,9},
		{iiopProxy,10},
		{iiopProxy,11},
		{iiopProxy,12},
		{iiopProxy,13},
		{iiopProxy,14},
		{iiopProxy,15},
		{iiopProxy,16},
		{iiopProxy,17},
		{iiopProxy,18},
		{iiopProxy,19},
		{iiopProxy,20},
		{iiopProxy,21},
		{iiopProxy,22},
		{iiopProxy,23},
		{iiopProxy,24},
		{iiopProxy,25},
		{iiopProxy,26},
		{iiopProxy,27},
		{iiopProxy,28},
		{iiopProxy,29},
		{iiopProxy,30},
		{iiopProxy,31},
		{iiopProxy,32},
		{iiopProxy,33},
		{iiopProxy,34},
		{iiopProxy,35},
		{iiopProxy,36},
		{iiopProxy,37},
		{iiopProxy,38},
		{iiopProxy,39},
		{iiopProxy,40},
		{iiopProxy,41},
		{iiopProxy,42},
		{iiopProxy,43},
		{iiopProxy,44},
		{iiopProxy,45},
		{iiopProxy,46},
		{iiopProxy,47},
		{iiopProxy,48},
		{iiopProxy,49},
		{iiopProxy,50},
		{iiopProxy,51},
		{iiopProxy,52},
		{iiopProxy,53},
		{iiopProxy,54},
		{iiopProxy,55},
		{iiopProxy,56},
		{iiopProxy,57},
		{iiopProxy,58},
		{iiopProxy,59},
		{iiopProxy,60},
		{iiopProxy,61},
		{iiopProxy,62},
		{iiopProxy,63},
		{iiopProxy,64},
		{iiopProxy,65},
		{iiopProxy,66},
		{iiopProxy,67},
		{iiopProxy,68},
		{iiopProxy,69},
		{iiopProxy,70},
		{iiopProxy,71},
		{iiopProxy,72},
		{iiopProxy,73},
		{iiopProxy,74},
		{iiopProxy,75},
		{iiopProxy,76},
		{iiopProxy,77},
		{iiopProxy,78},
		{iiopProxy,79},
		{iiopProxy,80},
		{iiopProxy,81},
		{iiopProxy,82},
		{iiopProxy,83},
		{iiopProxy,84},
		{iiopProxy,85},
		{iiopProxy,86},
		{iiopProxy,87},
		{iiopProxy,88},
		{iiopProxy,89},
		{iiopProxy,90},
		{iiopProxy,91},
		{iiopProxy,92},
		{iiopProxy,93},
		{iiopProxy,94},
		{iiopProxy,95},
		{iiopProxy,96},
		{iiopProxy,97},
		{iiopProxy,98},
		{iiopProxy,99},
		{iiopProxy,100},
		{iiopProxy,101},
		{iiopProxy,102},
		{iiopProxy,103},
		{iiopProxy,104},
		{iiopProxy,105},
		{iiopProxy,106},
		{iiopProxy,107},
		{iiopProxy,108},
		{iiopProxy,109},
		{iiopProxy,110},
		{iiopProxy,111},
		{iiopProxy,112},
		{iiopProxy,113},
		{iiopProxy,114},
		{iiopProxy,115},
		{iiopProxy,116},
		{iiopProxy,117},
		{iiopProxy,118},
		{iiopProxy,119},
		{iiopProxy,120},
		{iiopProxy,121},
		{iiopProxy,122},
		{iiopProxy,123},
		{iiopProxy,124},
		{iiopProxy,125},
		{iiopProxy,126},
		{iiopProxy,127},
		{iiopProxy,128},
		{iiopProxy,129},
		{iiopProxy,130},
		{iiopProxy,131},
		{iiopProxy,132},
		{iiopProxy,133},
		{iiopProxy,134},
		{iiopProxy,135},
		{iiopProxy,136},
		{iiopProxy,137},
		{iiopProxy,138},
		{iiopProxy,139},
		{iiopProxy,140},
		{iiopProxy,141},
		{iiopProxy,142},
		{iiopProxy,143},
		{iiopProxy,144},
		{iiopProxy,145},
		{iiopProxy,146},
		{iiopProxy,147},
		{iiopProxy,148},
		{iiopProxy,149},
		{iiopProxy,150},
		{iiopProxy,151},
		{iiopProxy,152},
		{iiopProxy,153},
		{iiopProxy,154},
		{iiopProxy,155},
		{iiopProxy,156},
		{iiopProxy,157},
		{iiopProxy,158},
		{iiopProxy,159},
		{iiopProxy,160},
		{iiopProxy,161},
		{iiopProxy,162},
		{iiopProxy,163},
		{iiopProxy,164},
		{iiopProxy,165},
		{iiopProxy,166},
		{iiopProxy,167},
		{iiopProxy,168},
		{iiopProxy,169},
		{iiopProxy,170},
		{iiopProxy,171},
		{iiopProxy,172},
		{iiopProxy,173},
		{iiopProxy,174},
		{iiopProxy,175},
		{iiopProxy,176},
		{iiopProxy,177},
		{iiopProxy,178},
		{iiopProxy,179},
		{iiopProxy,180},
		{iiopProxy,181},
		{iiopProxy,182},
		{iiopProxy,183},
		{iiopProxy,184},
		{iiopProxy,185},
		{iiopProxy,186},
		{iiopProxy,187},
		{iiopProxy,188},
		{iiopProxy,189},
		{iiopProxy,190},
		{iiopProxy,191},
		{iiopProxy,192},
		{iiopProxy,193},
		{iiopProxy,194},
		{iiopProxy,195},
		{iiopProxy,196},
		{iiopProxy,197},
		{iiopProxy,198},
		{iiopProxy,199},
		{iiopProxy,200},
		{iiopProxy,201},
		{iiopProxy,202},
		{iiopProxy,203},
		{iiopProxy,204},
		{iiopProxy,205},
		{iiopProxy,206},
		{iiopProxy,207},
		{iiopProxy,208},
		{iiopProxy,209},
		{iiopProxy,210},
		{iiopProxy,211},
		{iiopProxy,212},
		{iiopProxy,213},
		{iiopProxy,214},
		{iiopProxy,215},
		{iiopProxy,216},
		{iiopProxy,217},
		{iiopProxy,218},
		{iiopProxy,219},
		{iiopProxy,220},
		{iiopProxy,221},
		{iiopProxy,222},
		{iiopProxy,223},
		{iiopProxy,224},
		{iiopProxy,225},
		{iiopProxy,226},
		{iiopProxy,227},
		{iiopProxy,228},
		{iiopProxy,229},
		{iiopProxy,230},
		{iiopProxy,231},
		{iiopProxy,232},
		{iiopProxy,233},
		{iiopProxy,234},
		{iiopProxy,235},
		{iiopProxy,236},
		{iiopProxy,237},
		{iiopProxy,238},
		{iiopProxy,239},
		{iiopProxy,240},
		{iiopProxy,241},
		{iiopProxy,242},
		{iiopProxy,243},
		{iiopProxy,244},
		{iiopProxy,245},
		{iiopProxy,246},
		{iiopProxy,247},
		{iiopProxy,248},
		{iiopProxy,249},
		{iiopProxy,250},
		{iiopProxy,251},
		{iiopProxy,252},
		{iiopProxy,253},
		{iiopProxy,254},
		{iiopProxy,255},
		{iiopProxy,256},
		{iiopProxy,257},
		{iiopProxy,258},
		{iiopProxy,259},
		{iiopProxy,260},
		{iiopProxy,261},
		{iiopProxy,262},
		{iiopProxy,263},
		{iiopProxy,264},
		{iiopProxy,265},
		{iiopProxy,266},
		{iiopProxy,267},
		{iiopProxy,268},
		{iiopProxy,269},
		{iiopProxy,270},
		{iiopProxy,271},
		{iiopProxy,272},
		{iiopProxy,273},
		{iiopProxy,274},
		{iiopProxy,275},
		{iiopProxy,276},
		{iiopProxy,277},
		{iiopProxy,278},
		{iiopProxy,279},
		{iiopProxy,280},
		{iiopProxy,281},
		{iiopProxy,282},
		{iiopProxy,283},
		{iiopProxy,284},
		{iiopProxy,285},
		{iiopProxy,286},
		{iiopProxy,287},
		{iiopProxy,288},
		{iiopProxy,289},
		{iiopProxy,290},
		{iiopProxy,291},
		{iiopProxy,292},
		{iiopProxy,293},
		{iiopProxy,294},
		{iiopProxy,295},
		{iiopProxy,296},
		{iiopProxy,297},
		{iiopProxy,298},
		{iiopProxy,299},
		{iiopProxy,300},
	      {NULL,0} };
}


