//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/iiop/proxy/proxy.hh
//
// Modifications:
// $Log: proxy.hh,v $
// Revision 1.3  2000/09/05 15:04:05  corval2
// Fixed includes for CC 5.1
//
// Revision 1.2  2000/02/11 16:08:47  mli
// Return type CORBA::ULong -> long for the following methods:
//     processAttribute, processOperation, processException
//
// Revision 1.1.1.1  2000/02/09 17:17:11  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.9  1999/04/26 14:23:44  james
// Changed CORBA::Char* to char*
//
// Revision 1.8  1998/08/28 15:42:01  james
// RequestRecord added
//
// Revision 1.7  1998/06/04 15:01:10  stephen
// Updated locateObject support for forwarded object references.
//
// Revision 1.6  1998/05/11 14:20:52  stephen
// Node reference semantic changes propagated.
//
// Revision 1.5  1997/12/15 11:16:36  kevin
// GA version.
//
// Revision 1.4  1997/12/14 15:55:24  stephen
// Added TRANSPORT_ERROR return code.
//
// Revision 1.3  1997/11/17 15:29:13  stephen
// Added exception support.
//
// Revision 1.2  1997/10/30 16:10:51  ian
// Missing WIN32 header.
//
// Revision 1.1  1997/10/22 13:37:58  stephen
// Initial IIOP Proxy configured to run under TET.
//
//

#ifndef VSORB_PROXY_HH
#define VSORB_PROXY_HH

#include <iostream.h>

#ifdef WIN32
#include <fstream.h>
#endif

#include <syncinfo.hh>

#include <stdlib.h>
#include <stdio.h>
#include <iiop.hh>
#include <transport.hh>
#include <ernie.hh>
#include <node.hh>
#include <literals.hh>
#include <grammar.hh>
#include <utilities.hh>
#include <ctype.h>

// Helper routines
int yyparse(void);


ProfileBody* dumpIOR(IOR *);
LocateStatusType locateObject(CORBA::ULong, ProfileBody *&);
IOR* createProxyIOR(IOR* serverIOR);
CORBA::Long processEvent(CORBA::ULong fd);

long processAttribute(Node *, IIOPMessageHeader&, CORBA::Boolean);
long processOperation(Node *, IIOPMessageHeader&);
long processException(Node *, IIOPMessageHeader&);

Node * findAttrDef(const _Identifier &name, Node &ptr);
Node * findOpDef(const _Identifier &name, Node &ptr);

Node * FindElementSpec(Node *ptr, ExprValue exp, Node *& defaultNode);
Node * findTypeCodeDef();
void hexdump(unsigned char *ptr, int len);


class RequestRecord
{
    
private:
#define MAX_REQUESTS 100

    CORBA::ULong requestIds[MAX_REQUESTS];
    char *requestNames[MAX_REQUESTS];
    int expectedRequests;
    int expectedReplies;
    int registeredRequests;

public:

  RequestRecord();
  void Initialise(int numRequests);
    
  void AddRequest(CORBA::ULong id, char *name, CORBA::Boolean responseExpected);  
  void RegisterReply(CORBA::ULong id);
  char* GetName(CORBA::ULong id);
  char *GetSeed(CORBA::ULong id, CORBA::Boolean isARequest);

  CORBA::Boolean notFinished();
};

//
// Normal conditions
//

#define CLIENT_EXITED 1000
#define SERVER_EXITED 1001
#define EVENT_EOF 1002

//
// Error conditions
//

#define MARSHALL_ERROR 1200
#define SEND_REQUEST_ERR 1210
#define SEND_REPLY_ERR 1220
#define CONFIG_ERROR 1230
#define TRANSPORT_ERROR 1240

#endif
