//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <orb_supp.hh>
#include <tet_api.h>

/*
*******************************************************************
*******************************************************************
*/

OrbSupport::OrbSupport(const char *name)
{
    m_oid = CORBA::string_dup("");
    m_name = CORBA::string_dup("");
    
    int argc = 1;
    char arg0[] = { 't', 'i', 'n', 0 };
    char *argv[] = {arg0, 0}; 

    if (ORB_ID != 0)
	m_oid = CORBA::string_dup(ORB_ID);
        
    m_orb = CORBA::ORB_init(argc, argv, m_oid);
    if(CORBA::is_nil(m_orb)){
	m_statusok = FALSE;
	return;
    }
    
    CORBA::Object_var poaobj = m_orb->resolve_initial_references ("RootPOA");
    m_poa = PortableServer::POA::_narrow (poaobj);
    if(CORBA::is_nil(m_poa)){
      m_statusok = FALSE;
      return;
    }
    m_poamgr = m_poa->the_POAManager();
    if(CORBA::is_nil(m_poamgr)){
      m_statusok = FALSE;
      return;
    }
    
    if(name)
	m_name = CORBA::string_dup(name);
    m_io = new IOObjref(m_orb);
    if(!m_io){
	m_statusok = FALSE;
	return;
    }
    m_statusok = TRUE;

}

OrbSupport::OrbSupport(int argc, char* argv[], const char *name)
{
    m_oid = CORBA::string_dup("");
    m_name = CORBA::string_dup("");

    if (ORB_ID != 0)
	m_oid = CORBA::string_dup(ORB_ID);
        
    m_orb = CORBA::ORB_init(argc, argv, m_oid);
    if(CORBA::is_nil(m_orb)){
	m_statusok = FALSE;
	return;
    }
    
    CORBA::Object_var poaobj = m_orb->resolve_initial_references ("RootPOA");
    m_poa = PortableServer::POA::_narrow (poaobj);
    if(CORBA::is_nil(m_poa)){
      m_statusok = FALSE;
      return;
    }
    m_poamgr = m_poa->the_POAManager();
    if(CORBA::is_nil(m_poamgr)){
      m_statusok = FALSE;
      return;
    }
    
    if(name)
	m_name = CORBA::string_dup(name);
    m_io = new IOObjref(m_orb);
    if(!m_io){
	m_statusok = FALSE;
	return;
    }
    m_statusok = TRUE;

}

void OrbSupport::orbFinish(const char *str, CORBA::Environment_ptr ev)
{
    assert(m_orb != CORBA::ORB::_nil());

    if(str && *str)
	cout << str << endl;
    if(ev){
	cout << ev->exception() << endl;
    }

    if(m_name){
	CORBA::string_free(m_name);
	m_name = NULL;
    }
    if(m_oid){
	CORBA::string_free(m_oid);
	m_oid = NULL;
    }
    if(m_io){
	delete m_io;
	m_io = NULL;
    }
    if(!CORBA::is_nil(m_orb)){
        m_orb->destroy();
	//        CORBA::release(m_orb);
        //m_orb = CORBA::ORB::_nil();
    }
}

OrbSupport::~OrbSupport()
{
    orbFinish("",NULL);
}

char * OrbSupport::readString(unsigned char proxy)
{
    return m_io->readString(proxy);
}

CORBA::Object_ptr	OrbSupport::readObjref(unsigned char proxy)
{
    return m_io->readObjref(proxy);
}

int	OrbSupport::writeObjref(CORBA::Object_ptr objref)
{
    return m_io->writeObjref(objref);
}

char * OrbSupport::getObjref(CORBA::Object_ptr objref)
{
    char *ostr;

    ostr = m_orb->object_to_string(objref);
    return ostr;
}

CORBA::Object_ptr
OrbSupport::getObject(const char *  objref)
{
    return m_orb->string_to_object(objref);
}




/*
************************************************************
* will read and write an object reference in string form to
* or from the m_file.
************************************************************
*/

#define OBJ_STR_LEN	4096
#include <stdlib.h>

IOObjref::IOObjref(CORBA::ORB_ptr o)
{
    m_orb = o;
    char *fname = getenv("VSORB_IORFILE");
    m_file = CORBA::string_dup(fname ? fname : "/tmp/vsorb.ior");
}

IOObjref::~IOObjref()
{
    CORBA::string_free(m_file);
}

char * IOObjref::readString(unsigned char proxy)
{
    FILE  *fp;  /* File of object references */
    int		pos;
    char	*obj_str;
    char	temp[OBJ_STR_LEN + 1];

    if (proxy)
    {
	char * fname = getenv("VSORB_PROXY_IOR");
	
	if (fname == 0) {
	    fprintf(stderr,"\nCould not read VSORB_PROXY_IOR\n");
	    fprintf(stderr,"  Please check your environment\n\n");
	    return 0;
	}
	else
	{
	    CORBA::string_free(m_file);
	    m_file = CORBA::string_dup(fname);
	}
	
    }
    
    fp = fopen(m_file,"r");


    if (fp == NULL) {
	fprintf(stderr,"\nCould not read%s\n",m_file);
	fprintf(stderr,"  Please check your access to the file ...\n\n");
	return 0;
    }

    /* Read in the string for the object reference and convert it. */
    obj_str = fgets(temp, OBJ_STR_LEN, fp);
    if(obj_str == NULL) {
	fprintf(stderr, "No object reference to read in %s\n", m_file);
	fclose(fp);
	return 0;
    }

    /* Remove the newline at the end. */
    pos = strlen(temp) - 1;
    if(temp[pos] == '\n')
	temp[pos] = '\0';

    fclose(fp);
    return CORBA::string_dup(temp);
}

CORBA::Object_ptr	IOObjref::readObjref(unsigned char proxy)
{
    char* str = readString(proxy);
    if(str){
	m_objref = m_orb->string_to_object(str);
	CORBA::string_free(str);
    }

    return CORBA::Object::_duplicate(m_objref);
}


int IOObjref::writeObjref(CORBA::Object_ptr objref)
{
    FILE      *fp;  /* File of object references */
    char* ostr;

    fp = fopen(m_file,"w");
    if (fp == NULL) {
	fprintf(stderr,"\nCould not create %s\n",m_file);
	fprintf(stderr,"  Please check your access to the directory...\n\n");
	return 1;
    }

    ostr = m_orb->object_to_string(objref);
    if(!ostr){
	fclose(fp);
	return 1;
    }
    /* Write the stringified object references to the output file */
    if(strlen(ostr) > OBJ_STR_LEN)
	fprintf(stderr,"Warning - writing out a long object reference.\n");
    fprintf(fp,"%s\n",ostr);
    fclose(fp);
    
    /* Free the memory that was allocated.  */
    CORBA::string_free(ostr);

    return 0;
}
