//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/orbsupp/samples/irorbix.cpp
//
// Purpose:
//
// Modifications:
// $Log: irorbix.cpp,v $
// Revision 1.1.1.1  2000/02/09 17:17:14  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.3  1997/12/08 14:20:25  cwatson
// Now uses .hh instead of .h
//
// Revision 1.2  1997/10/22 13:21:12  stephen
// No longer need argc and argv passed in to OrbSupport obj.
//
// Revision 1.1  1997/10/04 15:48:33  cwatson
// Initial version of general ORB support code.
//
//

#include	<ir_support.hh>

IRSupport::IRSupport()
{
    CORBA::Environment ev;
    int argc = 0;
    char **argv = 0;

    try {
        m_osupp = new OrbSupport();
    }
    catch (CORBA::Exception &ex) {
        ev.exception(&ex);
        m_osupp->orbFinish("Failed to initialise ORB", &ev);
    }
    if(!m_osupp->statusok())
        m_osupp->orbFinish("Failed to initialise ORB", &ev);
}

IRSupport::~IRSupport()
{
    delete m_osupp;
}

Repository_ptr IRSupport::get_repository()
{
    Repository_ptr IR_ptr;
    CORBA::Object_ptr     obj_ptr;
    CORBA::ORB_ptr 				orb = m_osupp->orb();

    try {
	obj_ptr = orb->resolve_initial_references("InterfaceRepository");
	IR_ptr = Repository::_narrow(obj_ptr);
    }
    catch (CORBA::Exception &ex) {
        printf("system exception caught.\n");
	return (Repository_ptr) 0;
    }
    catch (...) {
	return (Repository_ptr) 0;
    }

    return IR_ptr;
}

Contained_ptr IRSupport::get_contained(
    Repository_ptr ir, ScopedName sname)
{
    Contained_ptr cptr;

    if(!ir)
	return (Contained_ptr) 0;
    
    try {
	cptr = ir->lookup(sname);
    }
    catch (CORBA::Exception &ex) {
        printf("system exception caught.\n");
	return (Contained_ptr) 0;
    }
    catch (...) {
	return (Contained_ptr) 0;
    }

    return cptr;
}



