//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/cmain.cpp
//
// Modifications:
// $Log: cmain.cpp,v $
// Revision 1.3  2000/05/23 16:59:41  mli
// remove 'CORBA::Environment ev;'
//
// Revision 1.2  2000/02/11 16:15:55  mli
//   o    void (*tet_startup)(void) -> void (*tet_startup)()
//   o    void (*tet_cleanup)(void) -> void (*tet_cleanup)()
//
// Revision 1.1.1.1  2000/02/09 17:17:16  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.7  1998/09/04 15:05:15  james
// tabs removed
//
// Revision 1.6  1997/12/15 12:09:16  james
// Final Code Regeneration
//
//
//

#include <orb_supp.hh>
#include <syncinfo.hh>
#include <stdlib.h>

#ifndef WIN32
#include <unistd.h>
#else
#include <process.h>
#endif

extern "C" {
#include <tet_api.h>
}

#include "cmain.hh"

//
// This global defines the systems involved in a TET sync event
// Currently the client sync with the proxy to receive the IOR
//
int remSystem[] = { CSW };

// Our handle to the ORB under test
OrbSupport *orb;

// The stringified IOR passed to us by the proxy
char *stringifiedIOR = 0;

void
clientStartUp()
{
  // Initialise the ORB under test
  try {
      orb = new OrbSupport();
      orb->client_init();
  }
  catch (CORBA::Exception &ex) {
      tet_infoline("client: failed to initialise Orb");
      return;
  }
  
  // Provide space to receive proxified IOR
  struct tet_synmsg msg;
  char rcvbuf[2048];
  
  // Indicate receive mode to TET
  msg.tsm_flags = TET_SMRCVMSG;
  msg.tsm_dlen = sizeof(rcvbuf);
  msg.tsm_data = rcvbuf;
  
  if (tet_remsync(CEVENT, remSystem, 1, VS_TIMEOUT, TET_SV_YES, &msg) != 0)
  {
      tet_printf("client: sync point %d failed", CEVENT);
      return;
  }

  // Cache stringified IOR for method calls to use
  stringifiedIOR = strdup(rcvbuf);

#ifdef DEBUG  
  tet_printf("client: sync point %d OK", CEVENT);
#endif
}

void
clientCleanUp()
{
    delete orb;
    return;
}

#ifdef VSORB_IDL_TESTS
 
extern "C" {
void (*tet_startup)(void) = 0,
(*tet_cleanup)(void) = 0;
struct tet_testlist tet_testlist[] = {
      {testIDL,1},
      {NULL,0} };
}

#else

extern "C" {
  void (*tet_startup)() = clientStartUp,
  (*tet_cleanup)() = clientCleanUp;
  struct tet_testlist tet_testlist[] = {
    {testBasicTestsPrunes1,1},
    {testBasicTestsBrocoli1,2},
    {testBasicTestsGrannySmith1,3},
    {testBasicTestsBrazilNut1,4},
    {testBasicTestsSweetPotato1,5},
    {testBasicTestsTangarine1,6},
    {testBasicTestsBillberries1,7},
    {testBasicTestsCantelopeMelon1,8},
    {testBasicTestsLolloRossa1,9},
    {testBasicTestsRowanBerry1,10},
    {testBasicTestsSweetcorn1,11},
    {testBasicTestsOnion1,12},
    {testBasicTestsShallots1,13},
    {testBasicTestsPlum1,14},
    {testBasicTestsKidneyBean1,15},
    {testBasicTestsZucchini1,16},
    {testBasicTestsButterBean1,17},
    {testBasicTestsAvocado1,18},
    {testBasicTestsGoldenDelicous1,19},
    {testBasicTestsLentil1,20},
    {testBasicTestsFig1,21},
    {testBasicTestsCrabApple1,22},
    {testBasicTestsRedOnion1,23},
    {testBasicTestsHazelNut1,24},
    {testBasicTestsCinnamon1,25},
    {testBasicTestsGooseBerry1,26},
    {testBasicTestsEggPlant1,27},
    {testBasicTestsJuniperBerries1,28},
    {testBasicTestsCloves1,29},
    {testBasicTestsDaikan1,30},
    {testBasicTestsMustard1,31},
    {testBasicTestsStrawberry1,32},
    {testBasicTestsBreadFruit1,33},
    {testBasicTestsThyme1,34},
    {testBasicTestsSweedes1,35},
    {testBasicTestsPomegranate1,36},
    {testBasicTestsTarragon1,37},
    {testBasicTestsLettuce1,38},
    {testBasicTestsMangetout1,39},
    {testBasicTestsCress1,40},
    {testBasicTestsBambooShoot1,41},
    {testBasicTestsCassava1,42},
    {testBasicTestsMelon1,43},
    {testAttrTestsRedGrapes1,44},
    {testAttrTestsRedGrapes1W,45},
    {testAttrTestsBeechNut1,46},
    {testAttrTestsBeechNut1W,47},
    {testAttrTestsElderberry1,48},
    {testAttrTestsElderberry1W,49},
    {testAttrTestsKumquats1,50},
    {testAttrTestsKumquats1W,51},
    {testAttrTestsWalnuts1,52},
    {testAttrTestsWalnuts1W,53},
    {testAttrTestsFennell1,54},
    {testAttrTestsFennell1W,55},
    {testAttrTestsUglifruit1,56},
    {testAttrTestsUglifruit1W,57},
    {testAttrTestsWaterchestnut1,58},
    {testAttrTestsWaterchestnut1W,59},
    {testAttrTestsChickpea1,60},
    {testAttrTestsChickpea1W,61},
    {testAttrTestsBeetroot1,62},
    {testAttrTestsBeetroot1W,63},
    {testAttrTestsOkra1,64},
    {testAttrTestsOkra1W,65},
    {testAttrTestsPlantain1,66},
    {testAttrTestsPlantain1W,67},
    {testAttrTestsBlackcurrant1,68},
    {testAttrTestsPineapple1,69},
    {testAttrTestsLimaBean1,70},
    {testAttrTestsMangosteen1,71},
    {testAttrTestsPear1,72},
    {testAttrTestsRedPepper1,73},
    {testAttrTestsBeefTomatoes1,74},
    {testAttrTestsFrenchBeans1,75},
    {testAttrTestsSharanfruit1,76},
    {testAttrTestsRadish1,77},
    {testAttrTestsCherry1,78},
    {testAttrTestsPapaya1,79},
    {testConstructedTypeTestsSusander1,80},
    {testConstructedTypeTestsGrapefruit1,81},
    {testConstructedTypeTestsPepinoMelon1,82},
    {testConstructedTypeTestsPeanutsquash1,83},
    {testConstructedTypeTestsArtichokes1,84},
    {testConstructedTypeTestsRasin1,85},
    {testConstructedTypeTestsChives1,86},
    {testConstructedTypeTestsEddo1,87},
    {testConstructedTypeTestsGreenChilly1,88},
    {testConstructedTypeTestsPeach1,89},
    {testConstructedTypeTestsParsley1,90},
    {testConstructedTypeTestsSatsuma1,91},
    {testConstructedTypeTestsBasil1,92},
    {testConstructedTypeTestsCoxPippin1,93},
    {testConstructedTypeTestsBlackberry1,94},
    {testConstructedTypeTestsGreenPepper1,95},
    {testConstructedTypeTestsScallion1,96},
    {testConstructedTypeTestsCurrant1,97},
    {testConstructedTypeTestsIcebergLettuce1,98},
    {testConstructedTypeTestsTomato1,99},
    {testConstructedTypeTestsPippinApple1,100},
    {testConstructedTypeTestsCeleriac1,101},
    {testConstructedTypeTestsCauliflower1,102},
    {testConstructedTypeTestsCorriander1,103},
    {testConstructedTypeTestsRedCabbage1,104},
    {testConstructedTypeTestsChineseLeaves1,105},
    {testConstructedTypeTestsPotato1,106},
    {testConstructedTypeTestsHomeguard1,107},
    {testConstructedTypeTestsWilja1,108},
    {testConstructedTypeTestsKerrsPinks1,109},
    {testTypeCodeTestsGoldenWonder1,110},
    {testTypeCodeTestsPimpernel1,111},
    {testTypeCodeTestsCultra1,112},
    {testTypeCodeTestsRooster1,113},
    {testTypeCodeTestsMarisPiper1,114},
    {testTypeCodeTestsCara1,115},
    {testTypeCodeTestsKingEdward1,116},
    {testTypeCodeTestsPentlandDell1,117},
    {testTypeCodeTestsApple2,118},
    {testTypeCodeTestsBanana2,119},
    {testTypeCodeTestsCarrots2,120},
    {testTypeCodeTestsLemon2,121},
    {testTypeCodeTestsBlackCherry2,122},
    {testTypeCodeTestsStarfruit2,123},
    {testTypeCodeTestsRhubarb2,124},
    {testTypeCodeTestsPassionFruit2,125},
    {testTypeCodeTestsPumpkin2,126},
    {testTypeCodeTestsMango2,127},
    {testTypeCodeTestsWaterMelon2,128},
    {testTypeCodeTestsLime2,129},
    {testTypeCodeTestsDates2,130},
    {testTypeCodeTestsCoconut2,131},
    {testTypeCodeTestsLychees2,132},
    {testTypeCodeTestsCabbage2,133},
    {testTypeCodeTestsCucumber2,134},
    {testTypeCodeTestsGinger2,135},
    {testTypeCodeTestsApricots2,136},
    {testTypeCodeTestsBlackEyedBeans2,137},
    {testTypeCodeTestsLeek2,138},
    {testTypeCodeTestsMint2,139},
    {testTypeCodeTestsGardenPeas2,140},
    {testTypeCodeTestsGardenPeas2W,141},
    {testTypeCodeTestsAsparagus2,142},
    {testTypeCodeTestsAsparagus2W,143},
    {testTypeCodeTestsGaliaMelon2,144},
    {testTypeCodeTestsGaliaMelon2W,145},
    {testTypeCodeTestsSultana2,146},
    {testTypeCodeTestsSultana2W,147},
    {testTypeCodeTestsTurnip2,148},
    {testTypeCodeTestsTurnip2W,149},
    {testTypeCodeTestsOrange2,150},
    {testTypeCodeTestsOrange2W,151},
    {testTypeCodeTestsLoganBerry2,152},
    {testTypeCodeTestsLoganBerry2W,153},
    {testTypeCodeTestsParsnip2,154},
    {testTypeCodeTestsCelery2,155},
    {testTypeCodeTestsWhiteCabbage2,156},
    {testTypeCodeTestsAniseed2,157},
    {testTypeCodeTestsBayleaf2,158},
    {testTypeCodeTestsDill2,159},
    {testTypeCodeTestsOregano2,160},
    {testAnyTestsBeansprout2,161},
    {testAnyTestsAlmonds2,162},
    {testAnyTestsGuava2,163},
    {testAnyTestsMushroom2,164},
    {testAnyTestsRunnerbean2,165},
    {testAnyTestsTofu2,166},
    {testAnyTestsPrunes2,167},
    {testAnyTestsBrocoli2,168},
    {testAnyTestsGrannySmith2,169},
    {testAnyTestsBrazilNut2,170},
    {testAnyTestsSweetPotato2,171},
    {testAnyTestsTangarine2,172},
    {testAnyTestsBillberries2,173},
    {testAnyTestsCantelopeMelon2,174},
    {testAnyTestsLolloRossa2,175},
    {testAnyTestsRowanBerry2,176},
    {testAnyTestsSweetcorn2,177},
    {testAnyTestsOnion2,178},
    {testAnyTestsShallots2,179},
    {testAnyTestsPlum2,180},
    {testAnyTestsKidneyBean2,181},
    {testAnyTestsZucchini2,182},
    {testAnyTestsButterBean2,183},
    {testAnyTestsAvocado2,184},
    {testAnyTestsGoldenDelicous2,185},
    {testAnyTestsLentil2,186},
    {testAnyTestsFig2,187},
    {testAnyTestsCrabApple2,188},
    {testAnyTestsRedOnion2,189},
    {testAnyTestsHazelNut2,190},
    {testAnyTestsCinnamon2,191},
    {testAnyTestsCinnamon2W,192},
    {testAnyTestsGooseBerry2,193},
    {testAnyTestsGooseBerry2W,194},
    {testAnyTestsEggPlant2,195},
    {testAnyTestsEggPlant2W,196},
    {testAnyTestsJuniperBerries2,197},
    {testAnyTestsJuniperBerries2W,198},
    {testAnyTestsCloves2,199},
    {testAnyTestsCloves2W,200},
    {testAnyTestsDaikan2,201},
    {testAnyTestsDaikan2W,202},
    {testAnyTestsMustard2,203},
    {testAnyTestsMustard2W,204},
    {testAnyTestsStrawberry2,205},
    {testAnyTestsBreadFruit2,206},
    {testAnyTestsThyme2,207},
    {testAnyTestsSweedes2,208},
    {testAnyTestsPomegranate2,209},
    {testAnyTestsTarragon2,210},
    {testAnyTestsLettuce2,211},
    {testAllTypesTestsMangetout2,212},
    {testAllTypesTestsBambooShoot2,213},
    {testAllTypesTestsMelon2,214},
    {testAllTypesTestsRedGrapes2,215},
    {testAllTypesTestsElderberry2,216},
    {testAllTypesTestsKumquats2,217},
    {testAllTypesTestsFennell2,218},
    {testAllTypesTestsUglifruit2,219},
    {testAllTypesTestsChickpea2,220},
    {testAllTypesTestsOkra2,221},
    {testAllTypesTestsPlantain2,222},
    {testAllTypesTestsBlackcurrant2,223},
    {testAllTypesTestsPineapple2,224},
    {testAllTypesTestsMangosteen2,225},
    {testAllTypesTestsPear2,226},
    {testAllTypesTestsBeefTomatoes2,227},
    {testAllTypesTestsSharanfruit2,228},
    {testAllTypesTestsCherry2,229},
    {testAllTypesTestsPeppercorn2,230},
    {testAllTypesTestsRaspberry2,231},
    {testAllTypesTestsPepinoMelon2,232},
    {testAllTypesTestsPeanutsquash2,233},
    {testAllTypesTestsArtichokes2,234},
    {testAllTypesTestsGreenGrapes2,235},
    {testAllTypesTestsCookingApple2,236},
    {testAllTypesTestsMullberry2,237},
    {testAllTypesTestsYam2,238},
    {testAllTypesTestsGreenChilly2,239},
    {testAllTypesTestsCorn2,240},
    {testAllTypesTestsPeach2,241},
    {testAllTypesTestsParsley2,242},
    {testAllTypesTestsSatsuma2,243},
    {testAllTypesTestsBasil2,244},
    {testAllTypesTestsBlueberry2,245},
    {testAllTypesTestsChervil2,246},
    {testAllTypesTestsGreenPepper2,247},
    {testAllTypesTestsScallion2,248},
    {testAllTypesTestsPricklyPear2,249},
    {testAllTypesTestsChestnut2,250},
    {testAllTypesTestsKiwanoMelon2,251},
    {testAllTypesTestsChayato2,252},
    {testAllTypesTestsRedChilly2,253},
    {testAllTypesTestsGalaApple2,254},
    {testAllTypesTestsKiwi2,255},
    {testAllTypesTestsCorriander2,256},
    {testAllTypesTestsRedCabbage2,257},
    {testAllTypesTestsBroadbeans2,258},
    {testAllTypesTestsPotato2,259},
    {testAllTypesTestsHomeguard2,260},
    {testAllTypesTestsWilja2,261},
    {testOneWayTestsBalmoral2,262},
    {testOneWayTestsKerrsPinks2,263},
    {testOneWayTestsGoldenWonder2,264},
    {testOneWayTestsPimpernel2,265},
    {testOneWayTestsCultra2,266},
    {testOneWayTestsRooster2,267},
    {testOneWayTestsMarisPiper2,268},
    {testOneWayTestsCara2,269},
    {testOneWayTestsKingEdward2,270},
    {testOneWayTestsPentlandDell2,271},
    {testOneWayTestsApple3,272},
    {testOneWayTestsBanana3,273},
    {testOneWayTestsCarrots3,274},
    {testOneWayTestsLemon3,275},
    {testOneWayTestsBlackCherry3,276},
    {testOneWayTestsStarfruit3,277},
    {testOneWayTestsRhubarb3,278},
    {testOneWayTestsPassionFruit3,279},
    {testOneWayTestsPumpkin3,280},
    {testContextTestsMango3,281},
    {testContextTestsLime3,282},
    {testContextTestsCoconut3,283},
    {testContextTestsCabbage3,284},
    {testContextTestsGinger3,285},
    {testContextTestsBlackEyedBeans3,286},
    {testContextTestsLeek3,287},
    {testContextTestsMint3,288},
    {testContextTestsAsparagus3,289},
    {testContextTestsSultana3,290},
    {testExceptionTestsOrange3,291},
    {testExceptionTestsBeansprout3,292},
    {testExceptionTestsSweetPotato3,293},
    {testExceptionTestsKidneyBean3,294},
    {testExceptionTestsLentil3,295},
    {testExceptionTestsDaikan3,296},
    {testExceptionTestsCassava3,297},
    {testExceptionTestsUglifruit3,298},
    {testExceptionTestsPear3,299},
    {testExceptionTestsSusander3,300},
    {NULL,0} };
}

#endif
