//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: tset/c++/sii/common/checkany.cpp
//
// Modifications:
// $Log: checkany.cpp,v $
// Revision 1.5  2001/01/21 18:07:09  arnop
// - use _var helper functions .in() et al at some locations
// - operator>>=( Any, T*&) is deprecated
//
// Revision 1.4  2000/07/17 09:00:52  corval2
// Fixed _out types
//
// Revision 1.3  2000/07/14 17:10:23  corval2
// Added missing const
//
// Revision 1.2  2000/07/14 15:18:13  corval2
// Added missing const
//
// Revision 1.1.1.1  2000/02/09 17:17:18  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.10  1999/04/26 14:09:32  james
// Changed CORBA::Char* to char*
//
// Revision 1.9  1998/09/04 14:49:32  james
// limit placed on number of anys in a request
//
// Revision 1.8  1998/05/11 14:54:17  stephen
// Better diagnostic information for bounded strings.
//
// Revision 1.5  1998/04/27 11:17:09  stephen
// Added const correctness to bounded string types.
//

#include <stdio.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern char name[];

 
int CheckAny(const CORBA::Any &any)
{
  CORBA::ULong choice;

  ++ernie.anyCount;

  if (ernie.anyCount < 10)
    choice = ernie.getULong()%66;
  else
    choice = ernie.getULong()%11;

  switch(choice)
  {
    case   0: {
                CORBA::ULong  v;
#ifdef DEBUG
                ::tet_infoline("CheckAny expecting CORBA::ULong");
#endif
                if (!(any >>= v))
                {
                  tet_printf("%s did not contain a CORBA::ULong as expected", name);
                  return 0;
                }
                if (!ernie.checkULong(v))
                {
                  tet_printf("Expected %s to contain CORBA::ULong with value %u, but found %u instead.",
                    name,ernie.getULongAgain(),v);
                  return 0;
                }
              }
              break;
    case   1: {
                CORBA::Long  v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting CORBA::Long");
#endif
                if (!(any >>= v))
                {
                  tet_printf("%s did not contain a CORBA::Long as expected",
                      name);
                  return 0;
                }
                if (!ernie.checkLong(v))
                {
                  tet_printf("Expected %s to contain CORBA::Long with value %d, but found %d instead.",
                    name,ernie.getLongAgain(),v);
                  return 0;
                }
              }
              break;
    case   2: {
                CORBA::UShort  v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting CORBA::UShort");
#endif
                if (!(any >>= v))
                {
                  tet_printf("%s did not contain a CORBA::UShort as expected",
                      name);
                  return 0;
                }
                if (!ernie.checkUShort(v))
                {
                  tet_printf("Expected %s to contain CORBA::UShort with value %u, but found %u instead.",
                    name,ernie.getUShortAgain(),v);
                  return 0;
                }
              }
              break;
    case   3: {
                CORBA::Short  v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting CORBA::Short");
#endif
                if (!(any >>= v))
                {
                  tet_printf("%s did not contain a CORBA::Short as expected",
                      name);
                  return 0;
                }
                if (!ernie.checkShort(v))
                {
                  tet_printf("Expected %s to contain CORBA::Short with value %d, but found %d instead.",
                    name,ernie.getShortAgain(),v);
                  return 0;
                }
              }
              break;
    case   4: {
                CORBA::Boolean  v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting CORBA::Boolean");
#endif
                if (!(any >>= CORBA::Any::to_boolean(v)))
                {
                  tet_printf("%s did not contain a CORBA::Boolean as expected",
                      name);
                  return 0;
                }
                if (!ernie.checkBoolean(v))
                {
                  tet_printf("Expected %s to contain CORBA::Boolean with value %s, but found %s instead.",
                    name,
                    BooleanToString(ernie.getBooleanAgain()),
                    BooleanToString(v));
                  return 0;
                }
              }
              break;
    case   5: {
                CORBA::Octet  v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting CORBA::Octet");
#endif
                if (!(any >>= CORBA::Any::to_octet(v)))
                {
                  tet_printf("%s did not contain a CORBA::Octet as expected",
                      name);
                  return 0;
                }
                if (!ernie.checkOctet(v))
                {
                  ::tet_printf("Expected %s to contain CORBA::Octet with value %s, but found %s instead.",
                    name,
                    OctetToString(ernie.getOctetAgain()),
                    OctetToString(v));
                  return 0;
                }
              }
              break;
    case   6: {
                CORBA::Char  v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting CORBA::Char");
#endif
                if (!(any >>= CORBA::Any::to_char(v)))
                {
                  tet_printf("%s did not contain a CORBA::Char as expected",
                      name);
                  return 0;
                }
                if (!ernie.checkChar(v))
                {
                  tet_printf("Expected %s to contain CORBA::Char with value %s, but found %s instead.",
                    name,
                    CharToString(ernie.getCharAgain()),
                    CharToString(v));
                  return 0;
                }
              }
              break;
    case   7: {
                CORBA::Float  v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting CORBA::Float");
#endif
                if (!(any >>= v))
                {
                  tet_printf("%s did not contain a CORBA::Float as expected",
                      name);
                  return 0;
                }
                if (!ernie.checkFloat(v))
                {
                  tet_printf("Expected %s to contain CORBA::Float with value %g, but found %g instead.",
                    name,
                    ernie.getFloatAgain(),
                    v);
                  return 0;
                }
              }
               break;
    case   8: {
                CORBA::Double  v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting CORBA::Double");
#endif
                if (!(any >>= v))
                {
                  tet_printf("%s did not contain a CORBA::Double as expected",
                      name);
                  return 0;
                }
                if (!ernie.checkDouble(v))
                {
                  tet_printf("Expected %s to contain CORBA::Double with value %g, but found %g instead.",
                    name,
                    ernie.getDoubleAgain(),
                    v);
                  return 0;
                }
              }
              break;
    case   9: {
                char *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting char*");
#endif
                if (!(any >>= CORBA::Any::to_string((const char*)v,0)))
                {
                  tet_printf("%s did not contain a char* as expected",
                      name);
                  return 0;
                }
                if (!ernie.checkString(v))
                {
                  tet_printf("Expected %s to contain char* with value %s, but found %s instead.",
                    name,ernie.getStringAgain(),v);
                  return 0;
                }
              }
              break;
    case  10: {
                CORBA::ULong  b;
                char   *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting bounded char*");
#endif
                b = ernie.getULong()%30+1;

                if (!(any >>= CORBA::Any::to_string((const char*)v,b)))
                {
                  tet_printf("%s did not contain a bounded char* as expected",
                      name);
                  return 0;
                }
                if (!ernie.checkString(v,b))
                {
                  tet_printf("Expected %s to contain a bounded char* with value %s, but found %s instead.",
                    name,ernie.getStringAgain(),v);
                  return 0;
                }
              }
              break;
    case   11: {
                CORBA::TypeCode_ptr  v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting CORBA::TypeCode");
#endif
                if (!(any >>= v))
                {
                  tet_printf("%s did not contain a CORBA::TypeCode as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckCORBATypeCode_ptr(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case   12: {
                const CORBA::Any *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expecting CORBA::Any");
#endif
                if (!(any >>= v))
                {
                  tet_printf("%s did not contain a CORBA::Any as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckAny(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  13: {
                ::Record2_forany v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Record2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Record2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckRecord2(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  14: {
                ::Chestnut1_forany v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Chestnut1");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Chestnut1 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckChestnut1(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  15: {
                ::Carrots_forany v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Carrots");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Carrots as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckCarrots(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  16: {
                ::Damsons2_forany v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Damsons2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Damsons2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckDamsons2(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  17: {
                ::Mullberry1_forany v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Mullberry1");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Mullberry1 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckMullberry1(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  18: {
                ::WaterMelon3_forany v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::WaterMelon3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::WaterMelon3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckWaterMelon3(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  19: {
                ::Raspberry1_forany v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Raspberry1");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Raspberry1 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckRaspberry1(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  20: {
                ::Corn1_forany v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Corn1");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Corn1 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckCorn1(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  21: {
                ::Pumpkin_forany v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Pumpkin");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Pumpkin as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckPumpkin(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  22: {
                ::Peanuts1 v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Peanuts1");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Peanuts1 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckPeanuts1(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  23: {
                const ::WaterMelon *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::WaterMelon");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::WaterMelon as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckWaterMelon(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  24: {
                const ::BambooShoot3 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::BambooShoot3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::BambooShoot3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckBambooShoot3(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  25: {
                const ::Mangosteen3 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Mangosteen3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Mangosteen3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckMangosteen3(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  26: {
                const ::Currant2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Currant2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Currant2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckCurrant2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  27: {
                const ::Chives2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Chives2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Chives2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckChives2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  28: {
                const ::GardenPeas3 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::GardenPeas3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::GardenPeas3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckGardenPeas3(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  29: {
                const ::Squash1 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Squash1");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Squash1 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckSquash1(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  30: {
                const ::YellowPepper2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::YellowPepper2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::YellowPepper2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckYellowPepper2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  31: {
                const ::Grapefruit2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Grapefruit2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Grapefruit2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckGrapefruit2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  32: {
                const ::BlackEyedBeans *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::BlackEyedBeans");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::BlackEyedBeans as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckBlackEyedBeans(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  33: {
                const ::GardenPeas1 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::GardenPeas1");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::GardenPeas1 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckGardenPeas1(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  34: {
                const ::Celeriac2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Celeriac2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Celeriac2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckCeleriac2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  35: {
                const ::GoldenDelicous3 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::GoldenDelicous3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::GoldenDelicous3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckGoldenDelicous3(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  36: {
                const ::Asparagus *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Asparagus");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Asparagus as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckAsparagus(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  37: {
                const ::Oregano3 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Oregano3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Oregano3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckOregano3(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  38: {
                const ::BeechNut2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::BeechNut2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::BeechNut2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckBeechNut2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  39: {
                const ::BritishQueens1 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::BritishQueens1");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::BritishQueens1 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckBritishQueens1(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  40: {
                const ::BrazilNut3 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::BrazilNut3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::BrazilNut3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckBrazilNut3(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  41: {
                const ::LoganBerry *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::LoganBerry");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::LoganBerry as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckLoganBerry(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  42: {
                const ::Lychees3 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Lychees3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Lychees3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckLychees3(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  43: {
                const ::Cauliflower2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Cauliflower2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Cauliflower2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckCauliflower2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  44: {
                const ::Beetroot2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Beetroot2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Beetroot2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckBeetroot2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  45: {
                const ::Fig *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Fig");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Fig as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckFig(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  46: {
                const ::IcebergLettuce2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::IcebergLettuce2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::IcebergLettuce2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckIcebergLettuce2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  47: {
                const ::Clementine2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Clementine2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Clementine2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckClementine2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  48: {
                const ::Susander2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Susander2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Susander2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckSusander2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  49: {
                const ::Blackberry2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Blackberry2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Blackberry2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckBlackberry2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  50: {
                const ::Peppercorn *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Peppercorn");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Peppercorn as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckPeppercorn(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  51: {
                const ::Tomato2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Tomato2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Tomato2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckTomato2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  52: {
                const ::Fennell3 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Fennell3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Fennell3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckFennell3(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  53: {
                const ::Rasin2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Rasin2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Rasin2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckRasin2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  54: {
                const ::Dates3 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Dates3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Dates3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckDates3(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  55: {
                const ::Squash2 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Squash2");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Squash2 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckSquash2(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  56: {
                const ::GaliaMelon3 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::GaliaMelon3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::GaliaMelon3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckGaliaMelon3(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  57: {
                const ::Turnip3 *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Turnip3");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Turnip3 as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckTurnip3(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  58: {
                const ::Tomato *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Tomato");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Tomato as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckTomato(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  59: {
                const ::Radish *v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected ::Radish");
#endif
                if (!(any  >>= v))
                {
                  tet_printf("%s did not contain a ::Radish as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!CheckRadish(*v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  60: {
                char* v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected char*<4>");
#endif
                if (!(any >>= CORBA::Any::to_string((const char*)v,4)
))
                {
                  tet_printf("%s did not contain a char*<4> as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!Checkstring_4(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  61: {
                char* v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected char*<20>");
#endif
                if (!(any >>= CORBA::Any::to_string((const char*)v,20)
))
                {
                  tet_printf("%s did not contain a char*<20> as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!Checkstring_20(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  62: {
                char* v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected char*<12>");
#endif
                if (!(any >>= CORBA::Any::to_string((const char*)v,12)
))
                {
                  tet_printf("%s did not contain a char*<12> as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!Checkstring_12(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  63: {
                char* v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected char*<46>");
#endif
                if (!(any >>= CORBA::Any::to_string((const char*)v,46)
))
                {
                  tet_printf("%s did not contain a char*<46> as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!Checkstring_46(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  64: {
                char* v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected char*<43>");
#endif
                if (!(any >>= CORBA::Any::to_string((const char*)v,43)
))
                {
                  tet_printf("%s did not contain a char*<43> as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!Checkstring_43(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;
    case  65: {
                char* v;

#ifdef DEBUG
                ::tet_infoline("CheckAny expected char*<25>");
#endif
                if (!(any >>= CORBA::Any::to_string((const char*)v,25)
))
                {
                  tet_printf("%s did not contain a char*<25> as expected",
                      name);
                  return 0;
                }
                AddName("_value");
                if (!Checkstring_25(v))
                {
                  RemoveName();
                  return 0;
                }
                RemoveName();
              }
              break;

  }
  return 1;
}
