//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: VSORB
//
// File: src/idl/templates/any.cpp
//
// Modifications:
// $Log: types.cpp,v $
// Revision 1.4  2001/01/21 18:07:09  arnop
// - use _var helper functions .in() et al at some locations
// - operator>>=( Any, T*&) is deprecated
//
// Revision 1.3  2000/07/19 09:53:26  corval2
// Changed signatures to const char * to prevent compiler warnings
//
// Revision 1.2  2000/07/17 09:43:21  corval2
// Fixed remaining _out type issues
//
// Revision 1.1.1.1  2000/02/09 17:17:19  mli
// Imported TOG's VSOrb 1.1.1
//
// Revision 1.9  1999/04/26 14:09:35  james
// Changed CORBA::Char* to char*
//
// Revision 1.8  1998/09/04 15:00:55  james
// ernie.getSequenceLength is now used when handling sequences
//
// Revision 1.7  1998/04/27 12:53:35  stephen
// Fixed incorrect bound in getString call.
//
// Revision 1.6  1997/12/15 12:09:28  james
// Final Code Regeneration
//
//
//

#include <stdio.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

Ernie ernie;

extern char *stringifiedIOR;
extern OrbSupport *orb;

char  name[1024]={'\0'};

void AddName(const char *p)
{
  if (strlen(name))
    strcat(name,".");
  strcat(name,p);
}

void RemoveName()
{
  int i;
 
  for (i = strlen(name);
    i > 0
      && name[i] != '.';
    --i)
    ;
 
  name[i] = '\0';
}

void AddIndex(int i)
{
  sprintf(name+strlen(name),"[%d]",i);
}

void RemoveIndex()
{
  int i;
 
  for (i = strlen(name);
    i > 0
      && name[i] != '[';
    --i)
    ;
 
  name[i] = '\0';
}

void GetULong(CORBA::ULong &v)
{
  v = ernie.getULong();
}

int CheckULong(const CORBA::ULong v)
{
  if (!ernie.checkULong(v))
  {
    tet_printf("Expected %s to contain %u, but found %u instead.",
      name,ernie.getULongAgain(),v);
    return 0;
  }
  return 1;
}

void GetLong(CORBA::Long &v)
{
  v = ernie.getLong();
}

int CheckLong(const CORBA::Long v)
{
  if (!ernie.checkLong(v))
  {
    tet_printf("Expected %s to contain %d, but found %d instead.",
      name,ernie.getLongAgain(),v);
    return 0;
  }
  return 1;
}

void GetUShort(CORBA::UShort &v)
{
  v = ernie.getUShort();
}

int CheckUShort(const CORBA::UShort v)
{
  if (!ernie.checkUShort(v))
  {
    tet_printf("Expected %s to contain %u, but found %u instead.",
      name,ernie.getUShortAgain(),v);
    return 0;
  }
  return 1;
}

void GetShort(CORBA::Short &v)
{
  v = ernie.getShort();
}

int CheckShort(const CORBA::Short v)
{
  if (!ernie.checkShort(v))
  {
    ::tet_printf("Expected %s to contain %d, but found %d instead.",
      name,ernie.getShortAgain(),v);
    return 0;
  }
  return 1;
}

void GetBoolean(CORBA::Boolean &v)
{
  v = ernie.getBoolean();
}

int CheckBoolean(const CORBA::Boolean v)
{
  if (!ernie.checkBoolean(v))
  {
    ::tet_printf("Expected %s to contain %s, but found %s instead.",
      name,BooleanToString(ernie.getBooleanAgain()),BooleanToString(v));
    return 0;
  }
  return 1;
}

void GetOctet(CORBA::Octet &v)
{
  v = ernie.getOctet();
}

int CheckOctet(const CORBA::Octet v)
{
  if (!ernie.checkOctet(v))
  {
    ::tet_printf("Expected %s to contain %s, but found %s instead.",
      name,OctetToString(ernie.getOctetAgain()),OctetToString(v));
    return 0;
  }
  return 1;
}

void GetChar(CORBA::Char &v)
{
  v = ernie.getChar();
}

int CheckChar(const CORBA::Char v)
{
  if (!ernie.checkChar(v))
  {
    ::tet_printf("Expected %s to contain %s, but found %s instead.",
      name,CharToString(ernie.getCharAgain()),CharToString(v));
    return 0;
  }
  return 1;
}

void GetFloat(CORBA::Float &v)
{
  v = ernie.getFloat();
}

int CheckFloat(const CORBA::Float v)
{
  if (!ernie.checkFloat(v))
  {
    tet_printf("Expected %s to contain %g, but found %g instead.",
      name,ernie.getFloatAgain(),v);
    return 0;
  }
  return 1;
}

int CompareFloats(float one, float two)
{
  float  delta;
  float  min;
  float max;

  delta = two/1000;
  if (delta < 0.0)
    delta = -delta;

  max = two + delta;
  min = two - delta;

  return ((one >= min) && (one <= max));
}

void GetDouble(CORBA::Double &v)
{
  v = ernie.getDouble();
}

int CheckDouble(const CORBA::Double v)
{
  if (!ernie.checkDouble(v))
  {
    tet_printf("Expected %s to contain %g, but found %g instead.",
      name,ernie.getDoubleAgain(),v);
    return 0;
  }
  return 1;
}

int CompareDoubles(double one,double two)
{
  double  delta;
  double  min;
  double  max;

  delta = two/100000;
  if (delta < 0.0)
    delta = -delta;

  max = two + delta;
  min = two - delta;

  return ((one >= min) && (one <= max));
}

char *GetString()
{
  char *v;
  v =CORBA::string_dup(ernie.getString());
  return v;
}

int CheckString(char * v)
{
  if (!ernie.checkString(v))
  {
    tet_printf("Expected %s to contain %s, but found %s instead.",
      name,ernie.getStringAgain(),v);
    return 0;
  }
  return 1;
}

int CheckString(const char * v)
{
  if (!ernie.checkString(v))
  {
    tet_printf("Expected %s to contain %s, but found %s instead.",
      name,ernie.getStringAgain(),v);
    return 0;
  }
  return 1;
}


char *GetString(int b)
{
  char *v;
  v =CORBA::string_dup(ernie.getString(b));
  return v;
}

int CheckString(char * v,int b)
{
  if (!ernie.checkString(v,b))
  {
    tet_printf("Expected %s to contain %s, but found %s instead.",
      name,ernie.getStringAgain(),v);
    return 0;
  }
  return 1;
}

int CheckString(const char * v,int b)
{
  if (!ernie.checkString(v,b))
  {
    tet_printf("Expected %s to contain %s, but found %s instead.",
      name,ernie.getStringAgain(),v);
    return 0;
  }
  return 1;
}

CORBA::Object_ptr Getobject()
{
  CORBA::Object_ptr v;
  v = orb->getObject((const char *)stringifiedIOR);
  return v;
}

int Checkobject(CORBA::Object_ptr v)
{
  char *s = 0;

  s = orb->getObjref(v);

  if (s == 0)
  {
    tet_printf("_object_to_string failed on Object reference %s.", name);
    return 0;
  }
#ifdef DEBUG
  tet_printf("_object_to_string returned %s",s);
#endif

  return 1;
}

 
void GetRecord2(::Record2 v)
{
#ifdef DEBUG
  ::tet_infoline("GetRecord2");
#endif

  {
    int	i1;
    for(i1=0; i1 < 9; ++i1)
    {
      AddIndex(i1);
      GetFloat(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckRecord2(const ::Record2 v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckRecord2");
#endif

  {
    int	i1;
    for(i1=0; i1 < 9; ++i1)
    {
      AddIndex(i1);
      if (!CheckFloat(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetChestnut1(::Chestnut1 v)
{
#ifdef DEBUG
  ::tet_infoline("GetChestnut1");
#endif

  {
    int	i1;
    for(i1=0; i1 < 5; ++i1)
    {
      AddIndex(i1);
      GetDouble(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckChestnut1(const ::Chestnut1 v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckChestnut1");
#endif

  {
    int	i1;
    for(i1=0; i1 < 5; ++i1)
    {
      AddIndex(i1);
      if (!CheckDouble(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetCarrots(::Carrots v)
{
#ifdef DEBUG
  ::tet_infoline("GetCarrots");
#endif

  {
    int	i1;
    for(i1=0; i1 < 8; ++i1)
    {
      AddIndex(i1);
      GetLong(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckCarrots(const ::Carrots v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckCarrots");
#endif

  {
    int	i1;
    for(i1=0; i1 < 8; ++i1)
    {
      AddIndex(i1);
      if (!CheckLong(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetDamsons2(::Damsons2 v)
{
#ifdef DEBUG
  ::tet_infoline("GetDamsons2");
#endif

  {
    int	i1;
    for(i1=0; i1 < 5; ++i1)
    {
      AddIndex(i1);
      GetShort(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckDamsons2(const ::Damsons2 v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckDamsons2");
#endif

  {
    int	i1;
    for(i1=0; i1 < 5; ++i1)
    {
      AddIndex(i1);
      if (!CheckShort(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetMullberry1(::Mullberry1 v)
{
#ifdef DEBUG
  ::tet_infoline("GetMullberry1");
#endif

  {
    int	i1;
    for(i1=0; i1 < 4; ++i1)
    {
      AddIndex(i1);
      GetULong(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckMullberry1(const ::Mullberry1 v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckMullberry1");
#endif

  {
    int	i1;
    for(i1=0; i1 < 4; ++i1)
    {
      AddIndex(i1);
      if (!CheckULong(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetWaterMelon3(::WaterMelon3 v)
{
#ifdef DEBUG
  ::tet_infoline("GetWaterMelon3");
#endif

  {
    int	i1;
    for(i1=0; i1 < 7; ++i1)
    {
      AddIndex(i1);
      GetUShort(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckWaterMelon3(const ::WaterMelon3 v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckWaterMelon3");
#endif

  {
    int	i1;
    for(i1=0; i1 < 7; ++i1)
    {
      AddIndex(i1);
      if (!CheckUShort(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetRaspberry1(::Raspberry1 v)
{
#ifdef DEBUG
  ::tet_infoline("GetRaspberry1");
#endif

  {
    int	i1;
    for(i1=0; i1 < 10; ++i1)
    {
      AddIndex(i1);
      GetChar(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckRaspberry1(const ::Raspberry1 v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckRaspberry1");
#endif

  {
    int	i1;
    for(i1=0; i1 < 10; ++i1)
    {
      AddIndex(i1);
      if (!CheckChar(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetCorn1(::Corn1 v)
{
#ifdef DEBUG
  ::tet_infoline("GetCorn1");
#endif

  {
    int	i1;
    for(i1=0; i1 < 5; ++i1)
    {
      AddIndex(i1);
      GetBoolean(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckCorn1(const ::Corn1 v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckCorn1");
#endif

  {
    int	i1;
    for(i1=0; i1 < 5; ++i1)
    {
      AddIndex(i1);
      if (!CheckBoolean(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetPumpkin(::Pumpkin v)
{
#ifdef DEBUG
  ::tet_infoline("GetPumpkin");
#endif

  {
    int	i1;
    for(i1=0; i1 < 6; ++i1)
    {
      AddIndex(i1);
      GetOctet(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckPumpkin(const ::Pumpkin v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckPumpkin");
#endif

  {
    int	i1;
    for(i1=0; i1 < 6; ++i1)
    {
      AddIndex(i1);
      if (!CheckOctet(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetPeanuts1(::Peanuts1 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetPeanuts1");
#endif

  GetLong(v);
}

int CheckPeanuts1(const ::Peanuts1 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckPeanuts1");
#endif

  if (!CheckLong(v))
    iRetCode = 0;
  return iRetCode;
}

 
void GetWaterMelon(::WaterMelon &v)
{
#ifdef DEBUG
  ::tet_infoline("GetWaterMelon");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetFloat(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckWaterMelon(const ::WaterMelon &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckWaterMelon");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckFloat(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetBambooShoot3(::BambooShoot3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetBambooShoot3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetDouble(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckBambooShoot3(const ::BambooShoot3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckBambooShoot3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckDouble(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetMangosteen3(::Mangosteen3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetMangosteen3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetLong(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckMangosteen3(const ::Mangosteen3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckMangosteen3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckLong(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetCurrant2(::Currant2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetCurrant2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetShort(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckCurrant2(const ::Currant2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckCurrant2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckShort(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetChives2(::Chives2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetChives2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetULong(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckChives2(const ::Chives2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckChives2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckULong(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetGardenPeas3(::GardenPeas3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetGardenPeas3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetUShort(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckGardenPeas3(const ::GardenPeas3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckGardenPeas3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckUShort(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetSquash1(::Squash1 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetSquash1");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetChar(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckSquash1(const ::Squash1 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckSquash1");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckChar(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetYellowPepper2(::YellowPepper2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetYellowPepper2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetBoolean(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckYellowPepper2(const ::YellowPepper2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckYellowPepper2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckBoolean(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetGrapefruit2(::Grapefruit2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetGrapefruit2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetOctet(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckGrapefruit2(const ::Grapefruit2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckGrapefruit2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckOctet(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetBlackEyedBeans(::BlackEyedBeans &v)
{
#ifdef DEBUG
  ::tet_infoline("GetBlackEyedBeans");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      v[i1] = GetString();
      RemoveIndex();
    }
  }
}

int CheckBlackEyedBeans(const ::BlackEyedBeans &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckBlackEyedBeans");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckString(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetGardenPeas1(::GardenPeas1 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetGardenPeas1");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(5);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetFloat(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckGardenPeas1(const ::GardenPeas1 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckGardenPeas1");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(5);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckFloat(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetCeleriac2(::Celeriac2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetCeleriac2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(2);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetDouble(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckCeleriac2(const ::Celeriac2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckCeleriac2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(2);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckDouble(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetGoldenDelicous3(::GoldenDelicous3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetGoldenDelicous3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(8);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetLong(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckGoldenDelicous3(const ::GoldenDelicous3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckGoldenDelicous3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(8);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckLong(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetAsparagus(::Asparagus &v)
{
#ifdef DEBUG
  ::tet_infoline("GetAsparagus");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(9);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetShort(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckAsparagus(const ::Asparagus &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckAsparagus");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(9);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckShort(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetOregano3(::Oregano3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetOregano3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(5);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetULong(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckOregano3(const ::Oregano3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckOregano3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(5);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckULong(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetBeechNut2(::BeechNut2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetBeechNut2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(1);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetUShort(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckBeechNut2(const ::BeechNut2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckBeechNut2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(1);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckUShort(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetBritishQueens1(::BritishQueens1 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetBritishQueens1");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(7);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetChar(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckBritishQueens1(const ::BritishQueens1 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckBritishQueens1");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(7);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckChar(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetBrazilNut3(::BrazilNut3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetBrazilNut3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(3);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetBoolean(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckBrazilNut3(const ::BrazilNut3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckBrazilNut3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(3);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckBoolean(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetLoganBerry(::LoganBerry &v)
{
#ifdef DEBUG
  ::tet_infoline("GetLoganBerry");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(4);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetOctet(v[i1]);
      RemoveIndex();
    }
  }
}

int CheckLoganBerry(const ::LoganBerry &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckLoganBerry");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(4);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckOctet(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetLychees3(::Lychees3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetLychees3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(10);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      v[i1] = GetString();
      RemoveIndex();
    }
  }
}

int CheckLychees3(const ::Lychees3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckLychees3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(10);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckString(v[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetCauliflower2(::Cauliflower2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetCauliflower2");
#endif

  GetFloat(v.Celery);
  GetDouble(v.WhiteCabbage);
  GetLong(v.Aniseed);
  GetShort(v.Bayleaf);
  GetULong(v.Dill);
  GetUShort(v.Oregano);
  GetChar(v.Beansprout);
  GetBoolean(v.Almonds);
  GetOctet(v.Guava);
  v.Mushroom = GetString();
}

int CheckCauliflower2(const ::Cauliflower2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckCauliflower2");
#endif

  AddName("Celery");
  if (!CheckFloat(v.Celery))
    iRetCode = 0;
  RemoveName();
  AddName("WhiteCabbage");
  if (!CheckDouble(v.WhiteCabbage))
    iRetCode = 0;
  RemoveName();
  AddName("Aniseed");
  if (!CheckLong(v.Aniseed))
    iRetCode = 0;
  RemoveName();
  AddName("Bayleaf");
  if (!CheckShort(v.Bayleaf))
    iRetCode = 0;
  RemoveName();
  AddName("Dill");
  if (!CheckULong(v.Dill))
    iRetCode = 0;
  RemoveName();
  AddName("Oregano");
  if (!CheckUShort(v.Oregano))
    iRetCode = 0;
  RemoveName();
  AddName("Beansprout");
  if (!CheckChar(v.Beansprout))
    iRetCode = 0;
  RemoveName();
  AddName("Almonds");
  if (!CheckBoolean(v.Almonds))
    iRetCode = 0;
  RemoveName();
  AddName("Guava");
  if (!CheckOctet(v.Guava))
    iRetCode = 0;
  RemoveName();
  AddName("Mushroom");
  if (!CheckString(v.Mushroom))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetBeetroot2(::Beetroot2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetBeetroot2");
#endif

  {
    int	i1;
    for(i1=0; i1 < 4; ++i1)
    {
      AddIndex(i1);
      GetFloat(v.Tofu[i1]);
      RemoveIndex();
    }
  }
  {
    int	i1;
    for(i1=0; i1 < 8; ++i1)
    {
      AddIndex(i1);
      GetDouble(v.Prunes[i1]);
      RemoveIndex();
    }
  }
  {
    int	i1;
    for(i1=0; i1 < 2; ++i1)
    {
      AddIndex(i1);
      GetLong(v.Brocoli[i1]);
      RemoveIndex();
    }
  }
  {
    int	i1;
    for(i1=0; i1 < 3; ++i1)
    {
      AddIndex(i1);
      GetShort(v.GrannySmith[i1]);
      RemoveIndex();
    }
  }
  {
    int	i1;
    for(i1=0; i1 < 8; ++i1)
    {
      AddIndex(i1);
      GetULong(v.BrazilNut[i1]);
      RemoveIndex();
    }
  }
  {
    int	i1;
    for(i1=0; i1 < 7; ++i1)
    {
      AddIndex(i1);
      GetUShort(v.SweetPotato[i1]);
      RemoveIndex();
    }
  }
  {
    int	i1;
    for(i1=0; i1 < 2; ++i1)
    {
      AddIndex(i1);
      GetChar(v.Tangarine[i1]);
      RemoveIndex();
    }
  }
  {
    int	i1;
    for(i1=0; i1 < 5; ++i1)
    {
      AddIndex(i1);
      GetBoolean(v.Billberries[i1]);
      RemoveIndex();
    }
  }
  {
    int	i1;
    for(i1=0; i1 < 1; ++i1)
    {
      AddIndex(i1);
      GetOctet(v.CantelopeMelon[i1]);
      RemoveIndex();
    }
  }
  {
    int	i1;
    for(i1=0; i1 < 3; ++i1)
    {
      AddIndex(i1);
      v.LolloRossa[i1] = GetString();
      RemoveIndex();
    }
  }
}

int CheckBeetroot2(const ::Beetroot2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckBeetroot2");
#endif

  AddName("Tofu");
  {
    int	i1;
    for(i1=0; i1 < 4; ++i1)
    {
      AddIndex(i1);
      if (!CheckFloat(v.Tofu[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("Prunes");
  {
    int	i1;
    for(i1=0; i1 < 8; ++i1)
    {
      AddIndex(i1);
      if (!CheckDouble(v.Prunes[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("Brocoli");
  {
    int	i1;
    for(i1=0; i1 < 2; ++i1)
    {
      AddIndex(i1);
      if (!CheckLong(v.Brocoli[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("GrannySmith");
  {
    int	i1;
    for(i1=0; i1 < 3; ++i1)
    {
      AddIndex(i1);
      if (!CheckShort(v.GrannySmith[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("BrazilNut");
  {
    int	i1;
    for(i1=0; i1 < 8; ++i1)
    {
      AddIndex(i1);
      if (!CheckULong(v.BrazilNut[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("SweetPotato");
  {
    int	i1;
    for(i1=0; i1 < 7; ++i1)
    {
      AddIndex(i1);
      if (!CheckUShort(v.SweetPotato[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("Tangarine");
  {
    int	i1;
    for(i1=0; i1 < 2; ++i1)
    {
      AddIndex(i1);
      if (!CheckChar(v.Tangarine[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("Billberries");
  {
    int	i1;
    for(i1=0; i1 < 5; ++i1)
    {
      AddIndex(i1);
      if (!CheckBoolean(v.Billberries[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("CantelopeMelon");
  {
    int	i1;
    for(i1=0; i1 < 1; ++i1)
    {
      AddIndex(i1);
      if (!CheckOctet(v.CantelopeMelon[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("LolloRossa");
  {
    int	i1;
    for(i1=0; i1 < 3; ++i1)
    {
      AddIndex(i1);
      if (!CheckString(v.LolloRossa[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  return iRetCode;
}

 
void GetFig(::Fig &v)
{
#ifdef DEBUG
  ::tet_infoline("GetFig");
#endif

  {
    int vSelection;
    CORBA::Long vArray[9] = {8606,-6010,-8262,-14738,16819,-6939,2160,4005,10729};
    
    vSelection = ernie.getULong() % 10;
    switch (vSelection) {
      case 0 :
              {
                CORBA::Float _value2;
                
                GetFloat(_value2);
                v. Sweetcorn(_value2);
              }
              break;
      case 1 :
              {
                CORBA::Double _value2;
                
                GetDouble(_value2);
                v. Onion(_value2);
              }
              break;
      case 2 :
              {
                CORBA::Long _value2;
                
                GetLong(_value2);
                v. Shallots(_value2);
              }
              break;
      default:
              {
                CORBA::Short _value2;
                
                GetShort(_value2);
                v. Plum(_value2);
              }
              break;
      case 3 :
              {
                CORBA::ULong _value2;
                
                GetULong(_value2);
                v. KidneyBean(_value2);
              }
              break;
      case 4 :
              {
                CORBA::UShort _value2;
                
                GetUShort(_value2);
                v. Zucchini(_value2);
              }
              break;
      case 5 :
              {
                CORBA::Char _value2;
                
                GetChar(_value2);
                v. ButterBean(_value2);
              }
              break;
      case 6 :
              {
                CORBA::Boolean _value2;
                
                GetBoolean(_value2);
                v. Avocado(_value2);
              }
              break;
      case 7 :
              {
                CORBA::Octet _value2;
                
                GetOctet(_value2);
                v. GoldenDelicous(_value2);
              }
              break;
      case 8 :
              {
                char * _value2;
                
                _value2 = GetString();
                v. Chervil(_value2);
              }
              break;
    }
    if (vSelection < 9)
      v._d(vArray[vSelection]);
  }
}

int CheckFig(const ::Fig &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckFig");
#endif

  {
    int vSelection;
    CORBA::Long vArray[9] = {8606,-6010,-8262,-14738,16819,-6939,2160,4005,10729};
    
    vSelection = ernie.getULong() % 10;
    
    if (vSelection < 9 && v._d() != vArray[vSelection])
    {
      ::tet_printf("ERROR: %s has an incorrect discriminator",name);
      iRetCode = 0;
    }
    switch (vSelection) {
      case 0 :
              AddName("Sweetcorn()");
              if (!CheckFloat(v.Sweetcorn()))
                iRetCode = 0;
              RemoveName();
              break;
      case 1 :
              AddName("Onion()");
              if (!CheckDouble(v.Onion()))
                iRetCode = 0;
              RemoveName();
              break;
      case 2 :
              AddName("Shallots()");
              if (!CheckLong(v.Shallots()))
                iRetCode = 0;
              RemoveName();
              break;
      default:
              AddName("Plum()");
              if (!CheckShort(v.Plum()))
                iRetCode = 0;
              RemoveName();
              break;
      case 3 :
              AddName("KidneyBean()");
              if (!CheckULong(v.KidneyBean()))
                iRetCode = 0;
              RemoveName();
              break;
      case 4 :
              AddName("Zucchini()");
              if (!CheckUShort(v.Zucchini()))
                iRetCode = 0;
              RemoveName();
              break;
      case 5 :
              AddName("ButterBean()");
              if (!CheckChar(v.ButterBean()))
                iRetCode = 0;
              RemoveName();
              break;
      case 6 :
              AddName("Avocado()");
              if (!CheckBoolean(v.Avocado()))
                iRetCode = 0;
              RemoveName();
              break;
      case 7 :
              AddName("GoldenDelicous()");
              if (!CheckOctet(v.GoldenDelicous()))
                iRetCode = 0;
              RemoveName();
              break;
      case 8 :
              AddName("Chervil()");
              if (!CheckString(v.Chervil()))
                iRetCode = 0;
              RemoveName();
              break;
    }
  }
  return iRetCode;
}

 
void GetIcebergLettuce2(::IcebergLettuce2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetIcebergLettuce2");
#endif

  {
    int vSelection;
    CORBA::Short vArray[10] = {-9071,10729,-2825,-6521,-2088,5861,5514,-11693,7475,12356};
    
    vSelection = ernie.getULong() % 10;
    switch (vSelection) {
      case 0 :
              {
                CORBA::Float _value2;
                
                GetFloat(_value2);
                v. CrabApple(_value2);
              }
              break;
      case 1 :
              {
                CORBA::Double _value2;
                
                GetDouble(_value2);
                v. RedOnion(_value2);
              }
              break;
      case 2 :
              {
                CORBA::Long _value2;
                
                GetLong(_value2);
                v. HazelNut(_value2);
              }
              break;
      case 3 :
              {
                CORBA::Short _value2;
                
                GetShort(_value2);
                v. Cinnamon(_value2);
              }
              break;
      case 4 :
              {
                CORBA::ULong _value2;
                
                GetULong(_value2);
                v. GooseBerry(_value2);
              }
              break;
      case 5 :
              {
                CORBA::UShort _value2;
                
                GetUShort(_value2);
                v. EggPlant(_value2);
              }
              break;
      case 6 :
              {
                CORBA::Char _value2;
                
                GetChar(_value2);
                v. JuniperBerries(_value2);
              }
              break;
      case 7 :
              {
                CORBA::Boolean _value2;
                
                GetBoolean(_value2);
                v. Cloves(_value2);
              }
              break;
      case 8 :
              {
                CORBA::Octet _value2;
                
                GetOctet(_value2);
                v. Daikan(_value2);
              }
              break;
      case 9 :
              {
                char * _value2;
                
                _value2 = GetString();
                v. Mustard(_value2);
              }
              break;
    }
    v._d(vArray[vSelection]);
  }
}

int CheckIcebergLettuce2(const ::IcebergLettuce2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckIcebergLettuce2");
#endif

  {
    int vSelection;
    CORBA::Short vArray[10] = {-9071,10729,-2825,-6521,-2088,5861,5514,-11693,7475,12356};
    
    vSelection = ernie.getULong() % 10;
    
    if (v._d() != vArray[vSelection])
    {
      ::tet_printf("ERROR: %s has an incorrect discriminator",name);
      iRetCode = 0;
    }
    switch (vSelection) {
      case 0 :
              AddName("CrabApple()");
              if (!CheckFloat(v.CrabApple()))
                iRetCode = 0;
              RemoveName();
              break;
      case 1 :
              AddName("RedOnion()");
              if (!CheckDouble(v.RedOnion()))
                iRetCode = 0;
              RemoveName();
              break;
      case 2 :
              AddName("HazelNut()");
              if (!CheckLong(v.HazelNut()))
                iRetCode = 0;
              RemoveName();
              break;
      case 3 :
              AddName("Cinnamon()");
              if (!CheckShort(v.Cinnamon()))
                iRetCode = 0;
              RemoveName();
              break;
      case 4 :
              AddName("GooseBerry()");
              if (!CheckULong(v.GooseBerry()))
                iRetCode = 0;
              RemoveName();
              break;
      case 5 :
              AddName("EggPlant()");
              if (!CheckUShort(v.EggPlant()))
                iRetCode = 0;
              RemoveName();
              break;
      case 6 :
              AddName("JuniperBerries()");
              if (!CheckChar(v.JuniperBerries()))
                iRetCode = 0;
              RemoveName();
              break;
      case 7 :
              AddName("Cloves()");
              if (!CheckBoolean(v.Cloves()))
                iRetCode = 0;
              RemoveName();
              break;
      case 8 :
              AddName("Daikan()");
              if (!CheckOctet(v.Daikan()))
                iRetCode = 0;
              RemoveName();
              break;
      case 9 :
              AddName("Mustard()");
              if (!CheckString(v.Mustard()))
                iRetCode = 0;
              RemoveName();
              break;
    }
  }
  return iRetCode;
}

 
void GetClementine2(::Clementine2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetClementine2");
#endif

  {
    int vSelection;
    CORBA::UShort vArray[9] = {8347,13439,7096,6372,4812,15857,13033,11406,4165};
    
    vSelection = ernie.getULong() % 10;
    switch (vSelection) {
      case 0 :
              {
                CORBA::Float _value2;
                
                GetFloat(_value2);
                v. BreadFruit(_value2);
              }
              break;
      case 1 :
              {
                CORBA::Double _value2;
                
                GetDouble(_value2);
                v. Thyme(_value2);
              }
              break;
      case 2 :
              {
                CORBA::Long _value2;
                
                GetLong(_value2);
                v. Sweedes(_value2);
              }
              break;
      case 3 :
              {
                CORBA::Short _value2;
                
                GetShort(_value2);
                v. Pomegranate(_value2);
              }
              break;
      case 4 :
              {
                CORBA::ULong _value2;
                
                GetULong(_value2);
                v. Tarragon(_value2);
              }
              break;
      default:
              {
                CORBA::UShort _value2;
                
                GetUShort(_value2);
                v. Lettuce(_value2);
              }
              break;
      case 5 :
              {
                CORBA::Char _value2;
                
                GetChar(_value2);
                v. Mangetout(_value2);
              }
              break;
      case 6 :
              {
                CORBA::Boolean _value2;
                
                GetBoolean(_value2);
                v. Cress(_value2);
              }
              break;
      case 7 :
              {
                CORBA::Octet _value2;
                
                GetOctet(_value2);
                v. BambooShoot(_value2);
              }
              break;
      case 8 :
              {
                char * _value2;
                
                _value2 = GetString();
                v. Cassava(_value2);
              }
              break;
    }
    if (vSelection < 9)
      v._d(vArray[vSelection]);
  }
}

int CheckClementine2(const ::Clementine2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckClementine2");
#endif

  {
    int vSelection;
    CORBA::UShort vArray[9] = {8347,13439,7096,6372,4812,15857,13033,11406,4165};
    
    vSelection = ernie.getULong() % 10;
    
    if (vSelection < 9 && v._d() != vArray[vSelection])
    {
      ::tet_printf("ERROR: %s has an incorrect discriminator",name);
      iRetCode = 0;
    }
    switch (vSelection) {
      case 0 :
              AddName("BreadFruit()");
              if (!CheckFloat(v.BreadFruit()))
                iRetCode = 0;
              RemoveName();
              break;
      case 1 :
              AddName("Thyme()");
              if (!CheckDouble(v.Thyme()))
                iRetCode = 0;
              RemoveName();
              break;
      case 2 :
              AddName("Sweedes()");
              if (!CheckLong(v.Sweedes()))
                iRetCode = 0;
              RemoveName();
              break;
      case 3 :
              AddName("Pomegranate()");
              if (!CheckShort(v.Pomegranate()))
                iRetCode = 0;
              RemoveName();
              break;
      case 4 :
              AddName("Tarragon()");
              if (!CheckULong(v.Tarragon()))
                iRetCode = 0;
              RemoveName();
              break;
      default:
              AddName("Lettuce()");
              if (!CheckUShort(v.Lettuce()))
                iRetCode = 0;
              RemoveName();
              break;
      case 5 :
              AddName("Mangetout()");
              if (!CheckChar(v.Mangetout()))
                iRetCode = 0;
              RemoveName();
              break;
      case 6 :
              AddName("Cress()");
              if (!CheckBoolean(v.Cress()))
                iRetCode = 0;
              RemoveName();
              break;
      case 7 :
              AddName("BambooShoot()");
              if (!CheckOctet(v.BambooShoot()))
                iRetCode = 0;
              RemoveName();
              break;
      case 8 :
              AddName("Cassava()");
              if (!CheckString(v.Cassava()))
                iRetCode = 0;
              RemoveName();
              break;
    }
  }
  return iRetCode;
}

 
void GetSusander2(::Susander2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetSusander2");
#endif

  {
    int vSelection;
    CORBA::ULong vArray[10] = {6597,5562,4834,11353,19080,9556,4803,12038,682,18578};
    
    vSelection = ernie.getULong() % 10;
    switch (vSelection) {
      case 0 :
              {
                CORBA::Float _value2;
                
                GetFloat(_value2);
                v. RedGrapes(_value2);
              }
              break;
      case 1 :
              {
                CORBA::Double _value2;
                
                GetDouble(_value2);
                v. BeechNut(_value2);
              }
              break;
      case 2 :
              {
                CORBA::Long _value2;
                
                GetLong(_value2);
                v. Elderberry(_value2);
              }
              break;
      case 3 :
              {
                CORBA::Short _value2;
                
                GetShort(_value2);
                v. Kumquats(_value2);
              }
              break;
      case 4 :
              {
                CORBA::ULong _value2;
                
                GetULong(_value2);
                v. Walnuts(_value2);
              }
              break;
      case 5 :
              {
                CORBA::UShort _value2;
                
                GetUShort(_value2);
                v. Fennell(_value2);
              }
              break;
      case 6 :
              {
                CORBA::Char _value2;
                
                GetChar(_value2);
                v. Uglifruit(_value2);
              }
              break;
      case 7 :
              {
                CORBA::Boolean _value2;
                
                GetBoolean(_value2);
                v. Waterchestnut(_value2);
              }
              break;
      case 8 :
              {
                CORBA::Octet _value2;
                
                GetOctet(_value2);
                v. Chickpea(_value2);
              }
              break;
      case 9 :
              {
                char * _value2;
                
                _value2 = GetString();
                v. Beetroot(_value2);
              }
              break;
    }
    v._d(vArray[vSelection]);
  }
}

int CheckSusander2(const ::Susander2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckSusander2");
#endif

  {
    int vSelection;
    CORBA::ULong vArray[10] = {6597,5562,4834,11353,19080,9556,4803,12038,682,18578};
    
    vSelection = ernie.getULong() % 10;
    
    if (v._d() != vArray[vSelection])
    {
      ::tet_printf("ERROR: %s has an incorrect discriminator",name);
      iRetCode = 0;
    }
    switch (vSelection) {
      case 0 :
              AddName("RedGrapes()");
              if (!CheckFloat(v.RedGrapes()))
                iRetCode = 0;
              RemoveName();
              break;
      case 1 :
              AddName("BeechNut()");
              if (!CheckDouble(v.BeechNut()))
                iRetCode = 0;
              RemoveName();
              break;
      case 2 :
              AddName("Elderberry()");
              if (!CheckLong(v.Elderberry()))
                iRetCode = 0;
              RemoveName();
              break;
      case 3 :
              AddName("Kumquats()");
              if (!CheckShort(v.Kumquats()))
                iRetCode = 0;
              RemoveName();
              break;
      case 4 :
              AddName("Walnuts()");
              if (!CheckULong(v.Walnuts()))
                iRetCode = 0;
              RemoveName();
              break;
      case 5 :
              AddName("Fennell()");
              if (!CheckUShort(v.Fennell()))
                iRetCode = 0;
              RemoveName();
              break;
      case 6 :
              AddName("Uglifruit()");
              if (!CheckChar(v.Uglifruit()))
                iRetCode = 0;
              RemoveName();
              break;
      case 7 :
              AddName("Waterchestnut()");
              if (!CheckBoolean(v.Waterchestnut()))
                iRetCode = 0;
              RemoveName();
              break;
      case 8 :
              AddName("Chickpea()");
              if (!CheckOctet(v.Chickpea()))
                iRetCode = 0;
              RemoveName();
              break;
      case 9 :
              AddName("Beetroot()");
              if (!CheckString(v.Beetroot()))
                iRetCode = 0;
              RemoveName();
              break;
    }
  }
  return iRetCode;
}

 
void GetBlackberry2(::Blackberry2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetBlackberry2");
#endif

  {
    int vSelection;
    CORBA::Char vArray[9] = {'3','&','s','T','\357','\000','F','g','\012'};
    
    vSelection = ernie.getULong() % 10;
    switch (vSelection) {
      case 0 :
              {
                CORBA::Float _value2;
                
                GetFloat(_value2);
                v. Plantain(_value2);
              }
              break;
      case 1 :
              {
                CORBA::Double _value2;
                
                GetDouble(_value2);
                v. Blackcurrant(_value2);
              }
              break;
      default:
              {
                CORBA::Long _value2;
                
                GetLong(_value2);
                v. Pineapple(_value2);
              }
              break;
      case 2 :
              {
                CORBA::Short _value2;
                
                GetShort(_value2);
                v. LimaBean(_value2);
              }
              break;
      case 3 :
              {
                CORBA::ULong _value2;
                
                GetULong(_value2);
                v. Mangosteen(_value2);
              }
              break;
      case 4 :
              {
                CORBA::UShort _value2;
                
                GetUShort(_value2);
                v. Pear(_value2);
              }
              break;
      case 5 :
              {
                CORBA::Char _value2;
                
                GetChar(_value2);
                v. RedPepper(_value2);
              }
              break;
      case 6 :
              {
                CORBA::Boolean _value2;
                
                GetBoolean(_value2);
                v. BeefTomatoes(_value2);
              }
              break;
      case 7 :
              {
                CORBA::Octet _value2;
                
                GetOctet(_value2);
                v. FrenchBeans(_value2);
              }
              break;
      case 8 :
              {
                char * _value2;
                
                _value2 = GetString();
                v. Sharanfruit(_value2);
              }
              break;
    }
    if (vSelection < 9)
      v._d(vArray[vSelection]);
  }
}

int CheckBlackberry2(const ::Blackberry2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckBlackberry2");
#endif

  {
    int vSelection;
    CORBA::Char vArray[9] = {'3','&','s','T','\357','\000','F','g','\012'};
    
    vSelection = ernie.getULong() % 10;
    
    if (vSelection < 9 && v._d() != vArray[vSelection])
    {
      ::tet_printf("ERROR: %s has an incorrect discriminator",name);
      iRetCode = 0;
    }
    switch (vSelection) {
      case 0 :
              AddName("Plantain()");
              if (!CheckFloat(v.Plantain()))
                iRetCode = 0;
              RemoveName();
              break;
      case 1 :
              AddName("Blackcurrant()");
              if (!CheckDouble(v.Blackcurrant()))
                iRetCode = 0;
              RemoveName();
              break;
      default:
              AddName("Pineapple()");
              if (!CheckLong(v.Pineapple()))
                iRetCode = 0;
              RemoveName();
              break;
      case 2 :
              AddName("LimaBean()");
              if (!CheckShort(v.LimaBean()))
                iRetCode = 0;
              RemoveName();
              break;
      case 3 :
              AddName("Mangosteen()");
              if (!CheckULong(v.Mangosteen()))
                iRetCode = 0;
              RemoveName();
              break;
      case 4 :
              AddName("Pear()");
              if (!CheckUShort(v.Pear()))
                iRetCode = 0;
              RemoveName();
              break;
      case 5 :
              AddName("RedPepper()");
              if (!CheckChar(v.RedPepper()))
                iRetCode = 0;
              RemoveName();
              break;
      case 6 :
              AddName("BeefTomatoes()");
              if (!CheckBoolean(v.BeefTomatoes()))
                iRetCode = 0;
              RemoveName();
              break;
      case 7 :
              AddName("FrenchBeans()");
              if (!CheckOctet(v.FrenchBeans()))
                iRetCode = 0;
              RemoveName();
              break;
      case 8 :
              AddName("Sharanfruit()");
              if (!CheckString(v.Sharanfruit()))
                iRetCode = 0;
              RemoveName();
              break;
    }
  }
  return iRetCode;
}

 
void GetPeppercorn(::Peppercorn &v)
{
#ifdef DEBUG
  ::tet_infoline("GetPeppercorn");
#endif

  {
    int vSelection;
    CORBA::Boolean vArray[2] = {1,0};
    
    vSelection = ernie.getULong() % 2;
    switch (vSelection) {
      case 0 :
              {
                CORBA::Short _value2;
                
                GetShort(_value2);
                v. Cherry(_value2);
              }
              break;
      case 1 :
              {
                CORBA::ULong _value2;
                
                GetULong(_value2);
                v. Papaya(_value2);
              }
              break;
    }
    v._d(vArray[vSelection]);
  }
}

int CheckPeppercorn(const ::Peppercorn &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckPeppercorn");
#endif

  {
    int vSelection;
    CORBA::Boolean vArray[2] = {1,0};
    
    vSelection = ernie.getULong() % 2;
    
    if (v._d() != vArray[vSelection])
    {
      ::tet_printf("ERROR: %s has an incorrect discriminator",name);
      iRetCode = 0;
    }
    switch (vSelection) {
      case 0 :
              AddName("Cherry()");
              if (!CheckShort(v.Cherry()))
                iRetCode = 0;
              RemoveName();
              break;
      case 1 :
              AddName("Papaya()");
              if (!CheckULong(v.Papaya()))
                iRetCode = 0;
              RemoveName();
              break;
    }
  }
  return iRetCode;
}

 
void GetTomato2(::Tomato2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetTomato2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(2);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      {
        CORBA::ULong	l3;
        CORBA::ULong	i3;
        l3 = ernie.getSequenceLength(14);
        v[i1].length(l3);
        for(i3=0; i3 < l3; ++i3)
        {
          AddIndex(i3);
          {
            CORBA::ULong	l5;
            CORBA::ULong	i5;
            l5 = ernie.getSequenceLength(5);
            v[i1][i3].length(l5);
            for(i5=0; i5 < l5; ++i5)
            {
              AddIndex(i5);
              {
                CORBA::ULong	l7;
                CORBA::ULong	i7;
                l7 = ernie.getSequenceLength(4);
                v[i1][i3][i5].length(l7);
                for(i7=0; i7 < l7; ++i7)
                {
                  AddIndex(i7);
                  GetLong(v[i1][i3][i5][i7]);
                  RemoveIndex();
                }
              }
              RemoveIndex();
            }
          }
          RemoveIndex();
        }
      }
      RemoveIndex();
    }
  }
}

int CheckTomato2(const ::Tomato2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckTomato2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(2);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      {
        CORBA::ULong	l3;
        CORBA::ULong	i3;
        l3 = ernie.getSequenceLength(14);
        if (l3 != v[i1].length())
        {
          ::tet_printf("ERROR: %s has incorrect length",name);
          ::tet_printf("Expected %d, but found %d instead", l3,v[i1].length());
          iRetCode = 0;
        }
        for(i3=0; i3 < l3; ++i3)
        {
          AddIndex(i3);
          {
            CORBA::ULong	l5;
            CORBA::ULong	i5;
            l5 = ernie.getSequenceLength(5);
            if (l5 != v[i1][i3].length())
            {
              ::tet_printf("ERROR: %s has incorrect length",name);
              ::tet_printf("Expected %d, but found %d instead", l5,v[i1][i3].length());
              iRetCode = 0;
            }
            for(i5=0; i5 < l5; ++i5)
            {
              AddIndex(i5);
              {
                CORBA::ULong	l7;
                CORBA::ULong	i7;
                l7 = ernie.getSequenceLength(4);
                if (l7 != v[i1][i3][i5].length())
                {
                  ::tet_printf("ERROR: %s has incorrect length",name);
                  ::tet_printf("Expected %d, but found %d instead", l7,v[i1][i3][i5].length());
                  iRetCode = 0;
                }
                for(i7=0; i7 < l7; ++i7)
                {
                  AddIndex(i7);
                  if (!CheckLong(v[i1][i3][i5][i7]))
                    iRetCode = 0;
                  RemoveIndex();
                }
              }
              RemoveIndex();
            }
          }
          RemoveIndex();
        }
      }
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetFennell3(::Fennell3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetFennell3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      {
        CORBA::ULong	l3;
        CORBA::ULong	i3;
        l3 = ernie.getSequenceLength(4);
        v[i1].length(l3);
        for(i3=0; i3 < l3; ++i3)
        {
          AddIndex(i3);
          {
            CORBA::ULong	l5;
            CORBA::ULong	i5;
            l5 = ernie.getSequenceLength(2);
            v[i1][i3].length(l5);
            for(i5=0; i5 < l5; ++i5)
            {
              AddIndex(i5);
              v[i1][i3][i5] = GetString();
              RemoveIndex();
            }
          }
          RemoveIndex();
        }
      }
      RemoveIndex();
    }
  }
}

int CheckFennell3(const ::Fennell3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckFennell3");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      {
        CORBA::ULong	l3;
        CORBA::ULong	i3;
        l3 = ernie.getSequenceLength(4);
        if (l3 != v[i1].length())
        {
          ::tet_printf("ERROR: %s has incorrect length",name);
          ::tet_printf("Expected %d, but found %d instead", l3,v[i1].length());
          iRetCode = 0;
        }
        for(i3=0; i3 < l3; ++i3)
        {
          AddIndex(i3);
          {
            CORBA::ULong	l5;
            CORBA::ULong	i5;
            l5 = ernie.getSequenceLength(2);
            if (l5 != v[i1][i3].length())
            {
              ::tet_printf("ERROR: %s has incorrect length",name);
              ::tet_printf("Expected %d, but found %d instead", l5,v[i1][i3].length());
              iRetCode = 0;
            }
            for(i5=0; i5 < l5; ++i5)
            {
              AddIndex(i5);
              if (!CheckString(v[i1][i3][i5]))
                iRetCode = 0;
              RemoveIndex();
            }
          }
          RemoveIndex();
        }
      }
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetRasin2(::Rasin2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetRasin2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      {
        CORBA::ULong	l3;
        CORBA::ULong	i3;
        l3 = ernie.getSequenceLength(0);
        v[i1].length(l3);
        for(i3=0; i3 < l3; ++i3)
        {
          AddIndex(i3);
          {
            CORBA::ULong	l5;
            CORBA::ULong	i5;
            l5 = ernie.getSequenceLength(0);
            v[i1][i3].length(l5);
            for(i5=0; i5 < l5; ++i5)
            {
              AddIndex(i5);
              GetCauliflower2(v[i1][i3][i5]);
              RemoveIndex();
            }
          }
          RemoveIndex();
        }
      }
      RemoveIndex();
    }
  }
}

int CheckRasin2(const ::Rasin2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckRasin2");
#endif

  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      {
        CORBA::ULong	l3;
        CORBA::ULong	i3;
        l3 = ernie.getSequenceLength(0);
        if (l3 != v[i1].length())
        {
          ::tet_printf("ERROR: %s has incorrect length",name);
          ::tet_printf("Expected %d, but found %d instead", l3,v[i1].length());
          iRetCode = 0;
        }
        for(i3=0; i3 < l3; ++i3)
        {
          AddIndex(i3);
          {
            CORBA::ULong	l5;
            CORBA::ULong	i5;
            l5 = ernie.getSequenceLength(0);
            if (l5 != v[i1][i3].length())
            {
              ::tet_printf("ERROR: %s has incorrect length",name);
              ::tet_printf("Expected %d, but found %d instead", l5,v[i1][i3].length());
              iRetCode = 0;
            }
            for(i5=0; i5 < l5; ++i5)
            {
              AddIndex(i5);
              if (!CheckCauliflower2(v[i1][i3][i5]))
                iRetCode = 0;
              RemoveIndex();
            }
          }
          RemoveIndex();
        }
      }
      RemoveIndex();
    }
  }
  return iRetCode;
}

 
void GetDates3(::Dates3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetDates3");
#endif

  GetTomato2(v.Peanutsquash);
  {
    int	i1;
    for(i1=0; i1 < 2; ++i1)
    {
      AddIndex(i1);
      GetLong(v.CherryTomatoes[i1]);
      RemoveIndex();
    }
  }
  {
    int	i1;
    for(i1=0; i1 < 5; ++i1)
    {
      AddIndex(i1);
      GetFloat(v.Artichokes[i1]);
      RemoveIndex();
    }
  }
}

int CheckDates3(const ::Dates3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckDates3");
#endif

  AddName("Peanutsquash");
  if (!CheckTomato2(v.Peanutsquash))
    iRetCode = 0;
  RemoveName();
  AddName("CherryTomatoes");
  {
    int	i1;
    for(i1=0; i1 < 2; ++i1)
    {
      AddIndex(i1);
      if (!CheckLong(v.CherryTomatoes[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("Artichokes");
  {
    int	i1;
    for(i1=0; i1 < 5; ++i1)
    {
      AddIndex(i1);
      if (!CheckFloat(v.Artichokes[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  return iRetCode;
}

 
void GetSquash2(::Squash2 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetSquash2");
#endif

  GetTomato2(v.CookingApple);
  {
    int	i1;
    for(i1=0; i1 < 2; ++i1)
    {
      AddIndex(i1);
      GetDates3(v.Mullberry[i1]);
      RemoveIndex();
    }
  }
  GetSusander2(v.Yam);
}

int CheckSquash2(const ::Squash2 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckSquash2");
#endif

  AddName("CookingApple");
  if (!CheckTomato2(v.CookingApple))
    iRetCode = 0;
  RemoveName();
  AddName("Mullberry");
  {
    int	i1;
    for(i1=0; i1 < 2; ++i1)
    {
      AddIndex(i1);
      if (!CheckDates3(v.Mullberry[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("Yam");
  if (!CheckSusander2(v.Yam))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetGaliaMelon3(::GaliaMelon3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetGaliaMelon3");
#endif

  GetSquash2(v.Peach);
  GetLong(v.Squash);
  GetFig(v.Parsley);
}

int CheckGaliaMelon3(const ::GaliaMelon3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckGaliaMelon3");
#endif

  AddName("Peach");
  if (!CheckSquash2(v.Peach))
    iRetCode = 0;
  RemoveName();
  AddName("Squash");
  if (!CheckLong(v.Squash))
    iRetCode = 0;
  RemoveName();
  AddName("Parsley");
  if (!CheckFig(v.Parsley))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetTurnip3(::Turnip3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetTurnip3");
#endif

  {
    int vSelection;
    CORBA::Long vArray[3] = {606,-6010,10729};
    
    vSelection = ernie.getULong() % 4;
    switch (vSelection) {
      case 0 :
              {
                ::GaliaMelon3 _value2;
                
                GetGaliaMelon3(_value2);
                v. Peanuts(_value2);
              }
              break;
      case 1 :
              {
                ::Fennell3 _value2;
                
                GetFennell3(_value2);
                v. CoxPippin(_value2);
              }
              break;
      default:
              {
                CORBA::Short _value2;
                
                GetShort(_value2);
                v. Blueberry(_value2);
              }
              break;
      case 2 :
              {
                ::Squash2 _value2;
                
                GetSquash2(_value2);
                v. Chervil(_value2);
              }
              break;
    }
    if (vSelection < 3)
      v._d(vArray[vSelection]);
  }
}

int CheckTurnip3(const ::Turnip3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckTurnip3");
#endif

  {
    int vSelection;
    CORBA::Long vArray[3] = {606,-6010,10729};
    
    vSelection = ernie.getULong() % 4;
    
    if (vSelection < 3 && v._d() != vArray[vSelection])
    {
      ::tet_printf("ERROR: %s has an incorrect discriminator",name);
      iRetCode = 0;
    }
    switch (vSelection) {
      case 0 :
              AddName("Peanuts()");
              if (!CheckGaliaMelon3(v.Peanuts()))
                iRetCode = 0;
              RemoveName();
              break;
      case 1 :
              AddName("CoxPippin()");
              if (!CheckFennell3(v.CoxPippin()))
                iRetCode = 0;
              RemoveName();
              break;
      default:
              AddName("Blueberry()");
              if (!CheckShort(v.Blueberry()))
                iRetCode = 0;
              RemoveName();
              break;
      case 2 :
              AddName("Chervil()");
              if (!CheckSquash2(v.Chervil()))
                iRetCode = 0;
              RemoveName();
              break;
    }
  }
  return iRetCode;
}

 
void GetTomato(::Tomato &v)
{
#ifdef DEBUG
  ::tet_infoline("GetTomato");
#endif

  {
    int vSelection;
    CORBA::Char vArray[3] = {'d','3','%'};
    
    vSelection = ernie.getULong() % 4;
    switch (vSelection) {
      case 0 :
              {
                ::Dates3 _value2;
                
                GetDates3(_value2);
                v. Scallion(_value2);
              }
              break;
      case 1 :
              {
                ::Rasin2 _value2;
                
                GetRasin2(_value2);
                v. Currant(_value2);
              }
              break;
      default:
              {
                CORBA::Short _value2;
                
                GetShort(_value2);
                v. Chestnut(_value2);
              }
              break;
      case 2 :
              {
                ::Cauliflower2 _value2;
                
                GetCauliflower2(_value2);
                v. KiwanoMelon(_value2);
              }
              break;
    }
    if (vSelection < 3)
      v._d(vArray[vSelection]);
  }
}

int CheckTomato(const ::Tomato &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckTomato");
#endif

  {
    int vSelection;
    CORBA::Char vArray[3] = {'d','3','%'};
    
    vSelection = ernie.getULong() % 4;
    
    if (vSelection < 3 && v._d() != vArray[vSelection])
    {
      ::tet_printf("ERROR: %s has an incorrect discriminator",name);
      iRetCode = 0;
    }
    switch (vSelection) {
      case 0 :
              AddName("Scallion()");
              if (!CheckDates3(v.Scallion()))
                iRetCode = 0;
              RemoveName();
              break;
      case 1 :
              AddName("Currant()");
              if (!CheckRasin2(v.Currant()))
                iRetCode = 0;
              RemoveName();
              break;
      default:
              AddName("Chestnut()");
              if (!CheckShort(v.Chestnut()))
                iRetCode = 0;
              RemoveName();
              break;
      case 2 :
              AddName("KiwanoMelon()");
              if (!CheckCauliflower2(v.KiwanoMelon()))
                iRetCode = 0;
              RemoveName();
              break;
    }
  }
  return iRetCode;
}

 
void GetBlackcurrant3(::Blackcurrant3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetBlackcurrant3");
#endif

}

int CheckBlackcurrant3(const ::Blackcurrant3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckBlackcurrant3");
#endif

  return iRetCode;
}

 
void GetDamsons3(::Damsons3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetDamsons3");
#endif

  GetFloat(v.RedChilly);
  GetDouble(v.Celeriac);
  GetLong(v.GalaApple);
  GetShort(v.Cauliflower);
  GetULong(v.Kiwi);
  GetUShort(v.Corriander);
  GetChar(v.Clementine);
  GetBoolean(v.RedCabbage);
  GetOctet(v.ChineseLeaves);
  v.Broadbeans = Getobject();
  v.Potato = GetString();
  v.Record = GetString(34);
}

int CheckDamsons3(const ::Damsons3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckDamsons3");
#endif

  AddName("RedChilly");
  if (!CheckFloat(v.RedChilly))
    iRetCode = 0;
  RemoveName();
  AddName("Celeriac");
  if (!CheckDouble(v.Celeriac))
    iRetCode = 0;
  RemoveName();
  AddName("GalaApple");
  if (!CheckLong(v.GalaApple))
    iRetCode = 0;
  RemoveName();
  AddName("Cauliflower");
  if (!CheckShort(v.Cauliflower))
    iRetCode = 0;
  RemoveName();
  AddName("Kiwi");
  if (!CheckULong(v.Kiwi))
    iRetCode = 0;
  RemoveName();
  AddName("Corriander");
  if (!CheckUShort(v.Corriander))
    iRetCode = 0;
  RemoveName();
  AddName("Clementine");
  if (!CheckChar(v.Clementine))
    iRetCode = 0;
  RemoveName();
  AddName("RedCabbage");
  if (!CheckBoolean(v.RedCabbage))
    iRetCode = 0;
  RemoveName();
  AddName("ChineseLeaves");
  if (!CheckOctet(v.ChineseLeaves))
    iRetCode = 0;
  RemoveName();
  AddName("Broadbeans");
  if (!Checkobject(v.Broadbeans.in()))
    iRetCode = 0;
  RemoveName();
  AddName("Potato");
  if (!CheckString(v.Potato))
    iRetCode = 0;
  RemoveName();
  AddName("Record");
  if (!CheckString(v.Record,34))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetRaspberry3(::Raspberry3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetRaspberry3");
#endif

  GetChives2(v.Celery1);
  GetBeetroot2(v.WhiteCabbage1);
  GetCeleriac2(v.Aniseed1);
}

int CheckRaspberry3(const ::Raspberry3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckRaspberry3");
#endif

  AddName("Celery1");
  if (!CheckChives2(v.Celery1))
    iRetCode = 0;
  RemoveName();
  AddName("WhiteCabbage1");
  if (!CheckBeetroot2(v.WhiteCabbage1))
    iRetCode = 0;
  RemoveName();
  AddName("Aniseed1");
  if (!CheckCeleriac2(v.Aniseed1))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetPeppercorn3(::Peppercorn3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetPeppercorn3");
#endif

  GetRecord2(v.Celery1);
  GetPeanuts1(v.WhiteCabbage1);
  GetAny(v.Aniseed1);
  GetMangosteen3(v.Bayleaf1);
}

int CheckPeppercorn3(const ::Peppercorn3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckPeppercorn3");
#endif

  AddName("Celery1");
  if (!CheckRecord2(v.Celery1))
    iRetCode = 0;
  RemoveName();
  AddName("WhiteCabbage1");
  if (!CheckPeanuts1(v.WhiteCabbage1))
    iRetCode = 0;
  RemoveName();
  AddName("Aniseed1");
  if (!CheckAny(v.Aniseed1))
    iRetCode = 0;
  RemoveName();
  AddName("Bayleaf1");
  if (!CheckMangosteen3(v.Bayleaf1))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetCherry3(::Cherry3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetCherry3");
#endif

  GetFloat(v.Celery1);
  v.WhiteCabbage1 = Getobject();
  GetAny(v.Aniseed1);
  GetFig(v.Guava1);
  GetLong(v.Mushroom1);
  {
    int	i1;
    for(i1=0; i1 < 3; ++i1)
    {
      AddIndex(i1);
      GetLong(v.Runnerbean1[i1]);
      RemoveIndex();
    }
  }
  GetLong(v.Tofu1);
}

int CheckCherry3(const ::Cherry3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckCherry3");
#endif

  AddName("Celery1");
  if (!CheckFloat(v.Celery1))
    iRetCode = 0;
  RemoveName();
  AddName("WhiteCabbage1");
  if (!Checkobject(v.WhiteCabbage1.in()))
    iRetCode = 0;
  RemoveName();
  AddName("Aniseed1");
  if (!CheckAny(v.Aniseed1))
    iRetCode = 0;
  RemoveName();
  AddName("Guava1");
  if (!CheckFig(v.Guava1))
    iRetCode = 0;
  RemoveName();
  AddName("Mushroom1");
  if (!CheckLong(v.Mushroom1))
    iRetCode = 0;
  RemoveName();
  AddName("Runnerbean1");
  {
    int	i1;
    for(i1=0; i1 < 3; ++i1)
    {
      AddIndex(i1);
      if (!CheckLong(v.Runnerbean1[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("Tofu1");
  if (!CheckLong(v.Tofu1))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetCherryTomatoes3(::CherryTomatoes3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetCherryTomatoes3");
#endif

  GetRasin2(v.Celery1);
  GetSquash2(v.Aniseed1);
}

int CheckCherryTomatoes3(const ::CherryTomatoes3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckCherryTomatoes3");
#endif

  AddName("Celery1");
  if (!CheckRasin2(v.Celery1))
    iRetCode = 0;
  RemoveName();
  AddName("Aniseed1");
  if (!CheckSquash2(v.Aniseed1))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetPapaya3(::Papaya3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetPapaya3");
#endif

  GetClementine2(v.Celery1);
  GetLychees3(v.WhiteCabbage1);
}

int CheckPapaya3(const ::Papaya3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckPapaya3");
#endif

  AddName("Celery1");
  if (!CheckClementine2(v.Celery1))
    iRetCode = 0;
  RemoveName();
  AddName("WhiteCabbage1");
  if (!CheckLychees3(v.WhiteCabbage1))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetSharanfruit3(::Sharanfruit3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetSharanfruit3");
#endif

  GetGardenPeas1(v.Celery1);
  GetBrazilNut3(v.WhiteCabbage1);
  GetChar(v.Aniseed1);
}

int CheckSharanfruit3(const ::Sharanfruit3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckSharanfruit3");
#endif

  AddName("Celery1");
  if (!CheckGardenPeas1(v.Celery1))
    iRetCode = 0;
  RemoveName();
  AddName("WhiteCabbage1");
  if (!CheckBrazilNut3(v.WhiteCabbage1))
    iRetCode = 0;
  RemoveName();
  AddName("Aniseed1");
  if (!CheckChar(v.Aniseed1))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetElderberry3(::Elderberry3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetElderberry3");
#endif

  GetAny(v.Celery1);
}

int CheckElderberry3(const ::Elderberry3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckElderberry3");
#endif

  AddName("Celery1");
  if (!CheckAny(v.Celery1))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetPeanutsquash3(::Peanutsquash3 &v)
{
#ifdef DEBUG
  ::tet_infoline("GetPeanutsquash3");
#endif

  GetAny(v.Celery1);
  GetAny(v.WhiteCabbage1);
  GetAny(v.Aniseed1);
  v.Almonds1 = Getobject();
  v.Guava1 = Getobject();
  v.Mushroom1 = Getobject();
  v.Runnerbean1 = Getobject();
  v.Tofu1 = Getobject();
}

int CheckPeanutsquash3(const ::Peanutsquash3 &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckPeanutsquash3");
#endif

  AddName("Celery1");
  if (!CheckAny(v.Celery1))
    iRetCode = 0;
  RemoveName();
  AddName("WhiteCabbage1");
  if (!CheckAny(v.WhiteCabbage1))
    iRetCode = 0;
  RemoveName();
  AddName("Aniseed1");
  if (!CheckAny(v.Aniseed1))
    iRetCode = 0;
  RemoveName();
  AddName("Almonds1");
  if (!Checkobject(v.Almonds1.in()))
    iRetCode = 0;
  RemoveName();
  AddName("Guava1");
  if (!Checkobject(v.Guava1.in()))
    iRetCode = 0;
  RemoveName();
  AddName("Mushroom1");
  if (!Checkobject(v.Mushroom1.in()))
    iRetCode = 0;
  RemoveName();
  AddName("Runnerbean1");
  if (!Checkobject(v.Runnerbean1.in()))
    iRetCode = 0;
  RemoveName();
  AddName("Tofu1");
  if (!Checkobject(v.Tofu1.in()))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void GetRadish(::Radish &v)
{
#ifdef DEBUG
  ::tet_infoline("GetRadish");
#endif

  GetOctet(v.Celery);
  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    v.Apple.length(l1);
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      GetRadish(v.Apple[i1]);
      RemoveIndex();
    }
  }
  GetDouble(v.Orange);
}

int CheckRadish(const ::Radish &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("CheckRadish");
#endif

  AddName("Celery");
  if (!CheckOctet(v.Celery))
    iRetCode = 0;
  RemoveName();
  AddName("Apple");
  {
    CORBA::ULong	l1;
    CORBA::ULong	i1;
    l1 = ernie.getSequenceLength(0);
    if (l1 != v.Apple.length())
    {
      ::tet_printf("ERROR: %s has incorrect length",name);
      ::tet_printf("Expected %d, but found %d instead", l1,v.Apple.length());
      iRetCode = 0;
    }
    for(i1=0; i1 < l1; ++i1)
    {
      AddIndex(i1);
      if (!CheckRadish(v.Apple[i1]))
        iRetCode = 0;
      RemoveIndex();
    }
  }
  RemoveName();
  AddName("Orange");
  if (!CheckDouble(v.Orange))
    iRetCode = 0;
  RemoveName();
  return iRetCode;
}

 
void Getstring_4(char* &v)
{
#ifdef DEBUG
  ::tet_infoline("Getstring_4");
#endif

  v = GetString(4);
}

int Checkstring_4(char* &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("Checkstring_4");
#endif

  if (!CheckString(v,4))
    iRetCode = 0;
  return iRetCode;
}

 
void Getstring_20(char* &v)
{
#ifdef DEBUG
  ::tet_infoline("Getstring_20");
#endif

  v = GetString(20);
}

int Checkstring_20(char* &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("Checkstring_20");
#endif

  if (!CheckString(v,20))
    iRetCode = 0;
  return iRetCode;
}

 
void Getstring_12(char* &v)
{
#ifdef DEBUG
  ::tet_infoline("Getstring_12");
#endif

  v = GetString(12);
}

int Checkstring_12(char* &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("Checkstring_12");
#endif

  if (!CheckString(v,12))
    iRetCode = 0;
  return iRetCode;
}

 
void Getstring_46(char* &v)
{
#ifdef DEBUG
  ::tet_infoline("Getstring_46");
#endif

  v = GetString(46);
}

int Checkstring_46(char* &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("Checkstring_46");
#endif

  if (!CheckString(v,46))
    iRetCode = 0;
  return iRetCode;
}

 
void Getstring_43(char* &v)
{
#ifdef DEBUG
  ::tet_infoline("Getstring_43");
#endif

  v = GetString(43);
}

int Checkstring_43(char* &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("Checkstring_43");
#endif

  if (!CheckString(v,43))
    iRetCode = 0;
  return iRetCode;
}

 
void Getstring_25(char* &v)
{
#ifdef DEBUG
  ::tet_infoline("Getstring_25");
#endif

  v = GetString(25);
}

int Checkstring_25(char* &v)
{
  int  iRetCode= 1;
#ifdef DEBUG
  ::tet_infoline("Checkstring_25");
#endif

  if (!CheckString(v,25))
    iRetCode = 0;
  return iRetCode;
}


