/*--------------------------------------------------------------------
 * Name:   valrep.c
 * Author: ITEX C-Code Generator
 *
 * Copyright 1991-1999 Telelogic AB, All rights reserved.
 *
 * This Program is owned by Telelogic and is protected by national
 * copyright laws and international copyright treaties. Telelogic
 * grants you the right to use this Program on one computer or in one
 * local computer network at any one time.  Under this License you may
 * only modify the source code for the purpose of adapting it to your
 * environment. You must reproduce and include any copyright and
 * trademark notices on all copies of the source code.  You may not
 * use, copy, merge, modify or transfer the Program except as provided
 * in this License.  Telelogic does not warrant that the Program will
 * meet your requirements or that the operation of the Program will be
 * uninterrupted and error free. You are solely responsible that the
 * selection of the Program and the modification of the source code
 * will achieve your intended results and that the results are
 * actually obtained.
 */



#include "gci.h"
#include <stdio.h>
#include <stddef.h>
#include <string.h>

int MTCD = 0;

/*--------------------------------------------------------------------
 * Forward declarations for local types.
 */

typedef struct VcLogType *VcLogType;
typedef struct VcFrameType VcFrameType;

/*--------------------------------------------------------------------
 * Forward declarations for local functions.
 */

/* Memory */

static Bool VcIsTemporary(void *);
static Bool VcIsLegalTemporary(void *);

/* Mk operations */
static VcValue *VcMkCHOICEContents(VcValue* cValue, 
                                   VcString name, 
                                   VcType* type);

/* Rm operations */

static void VcRmString(VcValue *);
static void VcRmSequence(VcValue *);
static void VcRmChoice(VcValue *);
static void VcRmReal(VcValue *);
static void VcRmOBJECT_IDENTIFIER(VcValue *);

static void VcRev(VcString, VcString);

/* Assignment */

static void VcAssignInteger(VcValue *, VcValue *);
static void VcAssignString(VcValue *, VcValue *);
static void VcAssignSequence(VcValue *, VcValue *);
static void VcAssignReal(VcValue *, VcValue *);
static void VcAssignChoice(VcValue *, VcValue *);
static void VcAssignOBJECT_IDENTIFIER(VcValue *, VcValue *);

/* Copying */

static VcValue *VcCopySimple(VcValue *);
static VcValue *VcCopySequence(VcValue *);
static VcValue *VcCopyNull(VcValue *);
static VcValue *VcCopyOBJECT_IDENTIFIER(VcValue *);
static VcValue *VcCopyObjectDescriptor(VcValue *);
static VcValue *VcCopyString(VcValue *);
static VcValue *VcCopyReal(VcValue *);
static VcValue *VcCopyChoice(VcValue *);

/* Query values */

static Bool VcIsInsteadOfValue(VcValue *);
static Bool VcIsInsideValue(VcValue *);
static Bool VcIsAttribute(VcValue *);

static Bool VcIsInteger(VcType *);
static Bool VcIsSimple(VcType *);
static Bool VcIsString(VcType *);
static Bool VcIsStructured(VcType *);
static Bool VcIsSequence(VcType *);
static Bool VcIsSequenceOf(VcType *);
static Bool VcIsChoice(VcType *);
static Bool VcIsReal(VcType *);
static Bool VcIsOBJECT_IDENTIFIER(VcType *);
static Bool VcIsOCTET_STRING(VcType *);

/* Restriction */

static int VcGetIntegerLength(VcType *);
static int VcGetStringLength(VcType *);

static VcRestriction *VcGetRes(VcType *);
static VcRestrictionType VcGetResType(VcRestriction *);
static VcRestriction *VcResNext(VcRestriction *);
static VcList *VcGetResValue(VcRestriction *);
static VcTypeList *VcGetResInclude(VcRestriction *);

static Bool VcRestrictSizeRange(VcRestriction *, VcValue *);
static Bool VcRestrictSize(VcRestriction *, VcValue *);
static Bool VcRestrictValueRange(VcRestriction *, VcValue *);
static Bool VcRestrictSingleValue(VcRestriction *, VcValue *);
static Bool VcRestrictPermittedAlphabet(VcRestriction *, VcValue *);
static Bool VcRestrictInclude(VcRestriction *, VcValue *);
static Bool VcRestrict(VcType *, VcValue *);

/* Matching */

static Bool VcMatchValuelist(VcValue *, VcValue *);
static Bool VcMatchComplement(VcValue *, VcValue *);
static Bool VcMatchSuperSet(VcValue *, VcValue *);
static Bool VcMatchSubSet(VcValue *, VcValue *);
static Bool VcMatchOmit(VcValue *, VcValue *);
static Bool VcMatchAnyOrNone(VcValue *, VcValue *);
static Bool VcMatchAnyone(VcValue *, VcValue *);
static Bool VcMatchIF_PRESENT(VcValue *, VcValue *);
static Bool VcMatchInsteadOfValue(VcValue *, VcValue *);
static Bool VcMatchAttribute(VcValue *, VcValue *);
static Bool VcMatchReal(VcValue *, VcValue *);
static Bool VcMatchChoice(VcValue *, VcValue *);
static Bool VcMatchSimple(VcValue *, VcValue *);
static Bool VcMatchClass(VcType *, VcType *);
static Bool VcMatchTag(VcType *, VcType *);
static Bool VcMatchTypeName(VcType *, VcType *);
static Bool VcMatchTypes(VcValue *, VcValue *);
static Bool VcMatchNumericString(VcValue *, VcValue *);
static Bool VcMatchBitString(VcValue *, VcValue *);
static Bool VcMatchOctetString(VcValue *, VcValue *);
static Bool VcMatchHexString(VcValue *, VcValue *);
static Bool VcMatchOtherString(VcValue *, VcValue *);
static Bool VcMatchTime(VcValue *, VcValue *);
static Bool VcMatchSequenceBacktrack(VcList *, int, VcList *, int);
static Bool VcMatchSequence(VcValue *, VcValue *);
static Bool VcMatchSetBacktrack(VcList *, int, VcList *, Bool *);
static Bool VcMatchSet(VcValue *, VcValue *);
static Bool VcMatchString(VcString, VcString);
static Bool VcMatchOBJECT_IDENTIFIER(VcValue *, VcValue *);
static Bool VcMatchValue(VcValue *, VcValue *);

/* Logging */

static VcLogType VcLogCreate(void);
static void VcLogDelete(VcLogType);

static char *VcLogBegin(VcLogType);

static void VcLogON(void);
static void VcLogOFF(void);

static void VcLogAddText(VcLogType, const char *);
static void VcLogAddInsteadOfValue(VcLogType, VcValue *);
static void VcLogAddInteger(VcLogType, int);
static void VcLogAddReal(VcLogType, VcReal *);
static void VcLogAddSequence(VcLogType, VcValue *);
static void VcLogAddOBJECT_IDENTIFIER(VcLogType, VcValue *);
static void VcLogAddValueValue(VcLogType, VcValue *);
static void VcLogAddValue(VcLogType, VcValue *);

static void VcLogMatch(VcValue *, VcValue *, Bool);
static void VcLogAssign(VcString, VcValue *);
static void VcLogRestriction(VcType *, VcValue *, Bool);
static void VcLogTypes(VcType *, VcType *);

static void VcMessage(const char *, const char *);

static void VcWarningON(void);
static void VcWarningOFF(void);
static void VcWarning(VcString, VcString);


static VcValue * VcListGetValue(VcList *, unsigned int);
static int VcListLength(VcList *);
static int VcListAllocated(VcList *);

/* String manipulation */
size_t Vcstrlen( const char* str);
char* Vcstrncpy( char* dest, const char* src, size_t count);
char* Vcstrcpy( char* dest, const char* src);
int Vcstrncmp( const char* str1, const char* str2, size_t count);
int Vcstrcmp( const char* str1, const char* str2);
char* Vcstrncat( char* dest, const char* src, size_t count);
char* Vcstrcat( char* dest, const char* src);


extern const int GcTS_VarDclD;
extern const int GcTC_VarDclD;

/*--------------------------------------------------------------------
 * struct VcType
 * Contains all data pertaining to a type.
 */

struct VcType
{
  VcType        *parent;           /* The types parent type. */
  VcType        *sequence_of_type;
  VcTypeName     name;             /* The type's name.       */
  VcTypeClass    class;            /* The type's class.      */
  VcTypeTag      tag;              /* The type's tag.        */
  int            implicit;         /* The IMPLICIT/EXPLICIT indicator */
  VcAttribute    attribute;
  VcRestriction *restriction;
  VcValue       *default_value;    /* The deault value.      */
  VcString       info;             /* The enumeration String */
  VcTypeList    *locals;
  VcTypeList    *members;
};

/*--------------------------------------------------------------------
 * The structure for a list of types
 */

struct VcTypeList
{
  VcType *type;
  char *name;
  VcTypeList *next;
};

/*--------------------------------------------------------------------
 * The structure for a list of members
 */

typedef struct VcMemberContext
{
  VcTypeList *members;
  struct VcMemberContext *next;
} VcMemberContext;

static VcMemberContext *members = NULL;


/*--------------------------------------------------------------------
 * The link type in the linked backpatchlist for restricted types.
 */

typedef struct tagVcValueDepBPLink 
{
  VcValue** value_location;
  VcValue** value;
  struct tagVcValueDepBPLink* next;
} VcValueDepBPLink;


/*--------------------------------------------------------------------
 * The global backpatch list for restricted types.
 */

static VcValueDepBPLink* VcValueDepBPList = NULL;


/*--------------------------------------------------------------------
 * Name:   VcMkType
 * Author: ITEX C-Code Generator
 *
 * Creates a new type with the given name. The class, tag and implicit
 * informaqtion is set to undefined.
 * All other fields are set to NULL and have to be set after this
 * call.  The function returns the newly built type.
 */

VcType * VcMkType(VcTypeName name)
{
  VcType *type = (VcType *) VCMALLOC(sizeof(VcType), "VcMkType");

  type->parent           = NULL;
  type->sequence_of_type = NULL;
  type->name             = name;
  type->class            = VcUndefinedD; 
  type->tag              = VcUndefinedD;
  type->implicit         = VcUndefinedD;  
  type->attribute        = 0;
  type->restriction      = NULL;
  type->default_value    = NULL;
  type->info             = NULL;
  type->locals           = NULL;
  type->members          = NULL;

  return type;
}

/*--------------------------------------------------------------------
 * Name:   VcReType
 * Author: ITEX C-Code Generator
 *
 * Changes the name and tag fields of a type.  The fields are
 * both changed to the given type name.  The function returns the
 * VcType of which we changed the type.
 */

VcType * VcReType(VcType * type, VcTypeName name)
{
  if (type != NULL) {
    type->name = name;
  }
  else
    VcWarning("VcReType", "<NULL> type");

  return type;
}

/*--------------------------------------------------------------------
 * Name:   VcGetName
 * Author: ITEX C-Code Generator
 *
 */

VcTypeName VcGetName(VcType * type)
{
  if (type != NULL)
    return type->name;

  return VcUndefinedD;
}

/*--------------------------------------------------------------------
 * Name:   VcGetTag
 * Author: ITEX C-Code Generator
 *
 */

VcTypeTag VcGetTag(VcType * type)
{
  if (type == NULL)
  {
    VcWarning("VcGetTag", "<NULL> type");
    return VcUndefinedD;
  }

  if (type->tag == VcUndefinedD)
    return VcGetTag(VcGetParent(type));

  return type->tag;
}

/*--------------------------------------------------------------------
 * Name:   VcGetClass
 * Author: ITEX C-Code Generator
 *
 */

VcTypeClass VcGetClass(VcType * type)
{
  if (type == NULL)
  {
    VcWarning("VcGetClass", "<NULL> type");
    return VcUndefinedD;
  }

  if (type->class == VcUndefinedD)
    return VcGetClass(VcGetParent(type));

  return type->class;
}

/*--------------------------------------------------------------------
 * Name:   VcGetImplicit
 * Author: ITEX C-Code Generator
 *
 */

Bool VcGetImplicit(VcType * type)
{
  if (type == NULL)
  {
    VcWarning("VcGetImplicit", "<NULL> type");
    return FALSE;
  }

  if (type->implicit == VcUndefinedD)
    return VcGetImplicit(VcGetParent(type));

  return (type->implicit == VcIMPLICIT);
}

/*--------------------------------------------------------------------
 * Name:   VcSetParent
 * Author: ITEX C-Code Generator
 *
 */

VcType * VcSetParent(VcType * parent, VcType * type)
{
  if (type != NULL)
    type->parent = parent;
  else
    VcWarning("VcSetParent", "<NULL> type");

  if (parent == NULL)
    VcWarning("VcSetParent", "Setting <NULL> parent");

  return type;
}

/*--------------------------------------------------------------------
 * Name:   VcGetParent
 * Author: ITEX C-Code Generator
 * 
 * This function returns the parent type of the type given as an
 * argument.  The function returns NULL if either there is no
 * parent type or if the given type is NULL.
 */

VcType * VcGetParent(VcType * type)
{
  if (type != NULL)
    return type->parent;
  else
    VcWarning("VcGetParent", "<NULL type");
  return NULL;
}

/*--------------------------------------------------------------------
 * Name:   VcTypeGetDefaultValue
 * Author: ITEX C-Code Generator
 *
 */

VcValue * VcTypeGetDefaultValue(VcType * type)
{
  if (type != NULL)
    return type->default_value;
  else
    VcWarning("VcTypeGetDefaultValue", "<NULL> type");
  return NULL;
}

/*--------------------------------------------------------------------
 * Name:   VcSetDefaultValue
 * Author: ITEX C-Code Generator
 *
 */

VcType * VcSetDefaultValue(VcValue** value, VcType * type)
{
  if (type != NULL)
  {
    if (*value != NULL)
      type->default_value = *value;
    else
    {
      type->default_value = VcMkINTEGER(0);  /* Dummy value! */
      VcValueDepForBackPatch(&(type->default_value), value);
    }
  }
  else
    VcWarning("VcSetDefaultValue", "<NULL> type");
  return type;
}

/*--------------------------------------------------------------------
 * Name:   

 * Author: ITEX C-Code Generator
 *
 */

VcType *VcSetTypeAttribute(VcAttribute attribute, VcType * type)
{
  if (type != NULL)
    type->attribute = attribute;
  else
    VcWarning("VcSetTypeAttribute", "<NULL> type");
  return type;
}

/*--------------------------------------------------------------------
 * Name:   VcGetTypeAttribute
 * Author: ITEX C-Code Generator
 *
 */

Bool VcGetTypeAttribute(VcAttribute attribute, VcType * type)
{
  if (type != NULL)
    return type->attribute == attribute;
  else
    VcWarning("VcGetTypeAttribute", "<NULL> type");
  return FALSE;
}

/*--------------------------------------------------------------------
 * Name:   VcGetBaseType
 * Author: ITEX C-Code Generator
 *
 */

VcTypeName VcGetBaseType(VcType * type)
{
  if (type == NULL)
    return VcUndefinedD;

  if (VcGetParent(type) == NULL)
    return VcGetName(type);
  else
    return VcGetBaseType(VcGetParent(type));
}

/*--------------------------------------------------------------------
 * Name:   VcGetBaseClass
 * Author: ITEX C-Code Generator
 *
 */

VcTypeClass VcGetBaseClass(VcType * type)
{

  /* An unknown type could result in a NULL pointer as a type.
   * We handle this by returning VcUndefined as the class */

  if (type == NULL)
    return VcUndefinedD;

  if (VcGetParent(type) == NULL) 
    return VcGetClass(type);
  return VcGetBaseClass(VcGetParent(type));
}

/*--------------------------------------------------------------------
 * Name:   VcGetBaseTag
 * Author: ITEX C-Code Generator
 *
 */

VcTypeTag VcGetBaseTag(VcType * type)
{
  if (type == NULL)
    return VcUndefinedD;

  if (VcGetParent(type) == NULL)
    return VcGetTag(type);
  else
    return VcGetBaseTag(VcGetParent(type));
}

/*--------------------------------------------------------------------
 * Name:   VcTagType
 * Author: ITEX C-Code Generator
 *
 * Set the tag information for the type.
 */

VcType *VcTagType(VcTypeClass class, VcTypeTag tag, Bool implicit, VcType *type)
{
  if (type != NULL) {
    if (class == VcNONE) {
      VcWarning("VcTagType", "No class.  Using CONTEXT");
      class = VcCONTEXT;
    }
    type->tag = tag;
    type->class = class;
    type->implicit = implicit ? VcIMPLICIT : VcEXPLICIT;
  }

  return type;
}

/*--------------------------------------------------------------------
 * Name:   

 * Author: ITEX C-Code Generator
 *
 * 
 */

void VcPush(void)
{
  VcMemberContext *current = (VcMemberContext *) VCMALLOC(sizeof(struct VcMemberContext), "VcPush");
  if (current) {
    current->members = 0;
    current->next    = members;
    members = current;
  }
}

/*--------------------------------------------------------------------
 * Name:   
 * Author: ITEX C-Code Generator
 *
 *
 */

void VcPop(void)
{
  if (members) {
    VcMemberContext *tmp = members;
    members = members->next;
    VCFREE(tmp, sizeof(VcMemberContext), "VcPop");
  }
}

/*--------------------------------------------------------------------
 * Name:   
 * Author: ITEX C-Code Generator
 *
 *
 */

void VcAddMember(VcType *type, const char *name)
{
  if (members) {
    VcTypeList *member = (VcTypeList *) VCMALLOC(sizeof(VcTypeList), "VcAddMember");
    if (member) {
      member->name = (char *) VCMALLOC(Vcstrlen(name) + 1, "VcAddMember");
      if (member->name)
        Vcstrcpy(member->name, name);
      member->type = type;
      member->next = NULL;

      if (members->members) {
        VcTypeList *current = members->members;
        while (current->next)
          current = current->next;
        current->next = member;
      }
      else
        members->members = member;
    }
  }
}

/*--------------------------------------------------------------------
 * Name:   
 * Author: ITEX C-Code Generator
 *
 *
 */

void VcRegisterMembers(VcType *owner)
{
  if (owner && members)
    owner->members = members->members;
  VcPop();
}

/*--------------------------------------------------------------------
 * Name:   
 * Author: ITEX C-Code Generator
 *
 *
 */

void VcSequenceOfType(VcType *sequence, VcType *type) 
{
  if (sequence)
    sequence->sequence_of_type = type;
}

/*--------------------------------------------------------------------
 * Name:   
 * Author: ITEX C-Code Generator
 *
 *
 */

unsigned VcGetMemberCount(int type)
{
  VcTypeList *member;
  VcType *vc_type;
  unsigned count;

  vc_type = VcTag2Type(type);
  if (!vc_type) {
    vc_type = IcSymTabStr2Type(IcSymTabTag2Str(type));
    if (!vc_type)
      return 0;
  }

  member = vc_type->members;
  count = 0;
  while (member) {
    member = member->next;
    ++count;
  }
  
  return count;
}

/*--------------------------------------------------------------------
 * Name:   
 * Author: ITEX C-Code Generator
 *
 *
 */

int VcGetMemberType(int type, unsigned index)
{
  VcTypeList *current;
  VcType *vc_type;
  unsigned count;

  vc_type = VcTag2Type(type);
  if (!vc_type) {
    vc_type = IcSymTabStr2Type(IcSymTabTag2Str(type));
    if (!vc_type)
      return VcUndefinedD;
  }

  current = vc_type->members;
  count = 0;
  while (current && count < index) {
    current = current->next;
    ++count;
  }

  if (count != index)
    return VcUndefinedD;
  else
    return VcGetName(current->type);
}

/*--------------------------------------------------------------------
 * Name:   
 * Author: ITEX C-Code Generator
 *
 *
 */

const char *VcGetMemberName(int type, unsigned index)
{
  VcTypeList *current;
  VcType *vc_type;
  unsigned count;

  vc_type = VcTag2Type(type);
  if (!vc_type) {
    vc_type = IcSymTabStr2Type(IcSymTabTag2Str(type));
    if (!vc_type)
      return NULL;
  }

  current = vc_type->members;
  count = 0;
  while (current && count < index) {
    current = current->next;
    ++count;
  }

  if (count != index)
    return NULL;
  else
    return current->name;
}

/*--------------------------------------------------------------------
 * Functions for handling local types.
 */

/*--------------------------------------------------------------------
 * Name:   VcAddLocal
 * Author: ITEX C-Code Generator
 *
 */

VcType * VcAddLocal(char *name, VcType * local_type, VcType * parent)
{
  if (parent) {
    VcTypeList *tmp = parent->locals;
    parent->locals = (VcTypeList *) VCMALLOC(sizeof(VcTypeList), "VcAddlocal");
    if (parent->locals) {
      parent->locals->next = tmp;
      parent->locals->type = local_type;
      if (local_type == NULL)
        VcWarning("VcAddLocal", "Adding <NULL> type");
      parent->locals->name = (char *) VCMALLOC(Vcstrlen(name) + 1, "VcAddlocal");
      Vcstrcpy(parent->locals->name, name);
    }
  }
  return parent;
}

/*--------------------------------------------------------------------
 * Name:   VcRegisterLocal
 * Author: ITEX C-Code Generator
 *
 */

VcType * VcRegisterLocal(VcType * holder, VcType * parent)
{
  if (parent) {
    parent->locals = holder->locals;
    
    /* Since the caller does not free the holder, I will.  This will
     * have to be removed if the CCG program ever is fixed in this
     * regard. */
    VCFREE(holder, sizeof(struct VcType), "VcRegisterLocal");
  }
  return parent;
}

/*--------------------------------------------------------------------
 * Name:   VcGetLocal
 * Author: ITEX C-Code Generator
 *
 * Find a local type stored in a global type.
 */

VcType * VcGetLocal(char *name, VcType * parent)
{
  if (name == NULL) {
    VcWarning("VcGetLocal", "Trying to locating unnamed local type");
    return NULL;
  }

  if (parent) {
    VcTypeList *tmp = parent->locals;
    
    while (tmp) {
      if (Vcstrcmp(tmp->name, name) == 0)
        return tmp->type;
      else
        tmp = tmp->next;
    }
  }
  return NULL;
}

/*--------------------------------------------------------------------
 * Functions for converting strings and tags to types
 *
 * The following nomenclature is used:
 *  type - This is a VcType* (VcINTEGER)
 *  tag  - This is a descriptor, both base and suite dependent
 *         (VcINTEGERD)
 *  str  - This is the name of the type as a string ("INTEGER")
 *  2    - Means "To"
 */

/*--------------------------------------------------------------------
 * Name:   VcTypeToStr 
 * Author: ITEX C-Code Generator
 *
 */

static 
const char * VcTypeToStr(VcType * type) 
{
  const char *name;

  if (type == NULL) {
    VcWarning("VcTypeToStr", "<NULL> type");
    name = "<NULL>";
  }
  else {
    VcWarningOFF();

    if ((name = VcTag2Str(VcGetName(type))) == NULL)
      if ((name = IcSymTabTag2Str(VcGetName(type))) == NULL)
        name = "UNKNOWN TYPE";

    VcWarningON();
  }
  return name;
}

static 
const char *VcTypeTagToStr(VcType *type)
{
  static char buffer[256];
  sprintf(buffer, "%d", VcGetTag(type));
  return buffer;
}

static 
const char *VcTypeClassToStr(VcType *type)
{
  VcTypeClass class = VcGetClass(type);
  if (class == VcNONE)
    return "NONE";
  else if (class == VcUNIVERSAL)
    return "UNIVERSAL";
  else if (class == VcAPPLICATION)
    return "APPLICATION";
  else if (class == VcPRIVATE)
    return "PRIVATE";
  else if (class == VcCONTEXT)
    return "CONTEXT";
  else
    return "UNDEFINED";
}

/*--------------------------------------------------------------------
 * Name:   VcStr2Type 
 * Author: ITEX C-Code Generator
 *
 */

VcType * VcStr2Type(const char *str)
{
  if (str == NULL) {
    VcWarning("VcStr2Type", "Type name <NULL>");
    return NULL;
  }

  if (Vcstrcmp("BOOLEAN", str) == 0)
    return VcBOOLEAN;
  if (Vcstrcmp("INTEGER", str) == 0)
    return VcINTEGER;
  if (Vcstrcmp("ENUMERATED", str) == 0)
    return VcENUMERATED;
  if (Vcstrcmp("NULL", str) == 0)
    return VcNULL;
  if (Vcstrcmp("BIT_STRING", str) == 0)
    return VcBIT_STRING;
  if (Vcstrcmp("HEXSTRING", str) == 0)
    return VcHEXSTRING;
  if (Vcstrcmp("OCTET_STRING", str) == 0)
    return VcOCTET_STRING;
  if (Vcstrcmp("SEQUENCE", str) == 0)
    return VcSEQUENCE;
  if (Vcstrcmp("SEQUENCE_OF", str) == 0)
    return VcSEQUENCE_OF;
  if (Vcstrcmp("SET", str) == 0)
    return VcSET;
  if (Vcstrcmp("SET_OF", str) == 0)
    return VcSET_OF;
  if (Vcstrcmp("REAL", str) == 0)
    return VcREAL;
  if (Vcstrcmp("CHOICE", str) == 0)
    return VcCHOICE;
  if (Vcstrcmp("PDU", str) == 0)
    return VcPDU;
  if (Vcstrcmp("NumericString", str) == 0)
    return VcNumericString;
  if (Vcstrcmp("PrintableString", str) == 0)
    return VcPrintableString;
  if (Vcstrcmp("TeletexString", str) == 0)
    return VcTeletexString;
  if (Vcstrcmp("VideotexString", str) == 0)
    return VcVideotexString;
  if (Vcstrcmp("IA5String", str) == 0)
    return VcIA5String;
  if (Vcstrcmp("UTCTime", str) == 0)
    return VcUTCTime;
  if (Vcstrcmp("GeneralizedTime", str) == 0)
    return VcGeneralizedTime;
  if (Vcstrcmp("GraphicString", str) == 0)
    return VcGraphicString;
  if (Vcstrcmp("VisibleString", str) == 0)
    return VcVisibleString;
  if (Vcstrcmp("GeneralString", str) == 0)
    return VcGeneralString;
  if (Vcstrcmp("ISO646String", str) == 0)
    return VcISO646String;
  if (Vcstrcmp("T61String", str) == 0)
    return VcT61String;
  if (Vcstrcmp("OBJECT_IDENTIFIER", str) == 0)
    return VcOBJECT_IDENTIFIER;
  if (Vcstrcmp("ObjectDescriptor", str) == 0)
    return VcObjectDescriptor;

  /* Not a base type */
  return NULL;
}

/*--------------------------------------------------------------------
 * Name:   VcTag2Type
 * Author: ITEX C-Code Generator
 *
 */

VcType * VcTag2Type(int tag)
{
  switch (tag) {
    case VcBOOLEAND:
      return VcBOOLEAN;
    case VcINTEGERD:
      return VcINTEGER;
    case VcBIT_STRINGD:
      return VcBIT_STRING;
    case VcHEXSTRINGD:
      return VcHEXSTRING;
    case VcOCTET_STRINGD:
      return VcOCTET_STRING;
    case VcSEQUENCED:
      return VcSEQUENCE;
    case VcSEQUENCE_OFD:
      return VcSEQUENCE_OF;
    case VcSETD:
      return VcSET;
    case VcSET_OFD:
      return VcSET_OF;
    case VcREALD:
      return VcREAL;
    case VcCHOICED:
      return VcCHOICE;
    case VcPDUD:
      return VcPDU;
    case VcNumericStringD:
      return VcNumericString;
    case VcPrintableStringD:
      return VcPrintableString;
    case VcTeletexStringD:
      return VcTeletexString;
    case VcVideotexStringD:
      return VcVideotexString;
    case VcIA5StringD:
      return VcIA5String;
    case VcUTCTimeD:
      return VcUTCTime;
    case VcGeneralizedTimeD:
      return VcGeneralizedTime;
    case VcGraphicStringD:
      return VcGraphicString;
    case VcVisibleStringD:
      return VcVisibleString;
    case VcGeneralStringD:
      return VcGeneralString;
    case VcISO646StringD:
      return VcISO646String;
    case VcT61StringD:
      return VcT61String;
    case VcOBJECT_IDENTIFIERD:
      return VcOBJECT_IDENTIFIER;
    case VcObjectDescriptorD:
      return VcObjectDescriptor;
    default:
      return NULL;
    }
}

/*--------------------------------------------------------------------
 * Name:   VcTag2Str
 * Author: ITEX C-Code Generator
 *
 */

const char * VcTag2Str(int tag)
{
  switch (tag) {
    case VcNULLD:
      return "NULL";
    case VcBOOLEAND:
      return "BOOLEAN";
    case VcINTEGERD:
      return "INTEGER";
    case VcENUMERATEDD:
      return "ENUMERATED";
    case VcBIT_STRINGD:
      return "BIT_STRING";
    case VcHEXSTRINGD:
      return "HEXSTRING";
    case VcOCTET_STRINGD:
      return "OCTET_STRING";
    case VcSEQUENCED:
      return "SEQUENCE";
    case VcSEQUENCE_OFD:
      return "SEQUENCE_OF";
    case VcSETD:
      return "SET";
    case VcSET_OFD:
      return "SET_OF";
    case VcREALD:
      return "REAL";
    case VcCHOICED:
      return "CHOICE";
    case VcPDUD:
      return "PDU";
    case VcNumericStringD:
      return "NumericString";
    case VcPrintableStringD:
      return "PrintableString";
    case VcTeletexStringD:
      return "TeletexString";
    case VcVideotexStringD:
      return "VideotexString";
    case VcIA5StringD:
      return "IA5String";
    case VcUTCTimeD:
      return "UTCTime";
    case VcGeneralizedTimeD:
      return "GeneralizedTime";
    case VcGraphicStringD:
      return "GraphicString";
    case VcVisibleStringD:
      return "VisibleString";
    case VcGeneralStringD:
      return "GeneralString";
    case VcISO646StringD:
      return "ISO646String";
    case VcT61StringD:
      return "T61String";
    case VcOBJECT_IDENTIFIERD:
      return "OBJECT_IDENTIFIER";
    case VcObjectDescriptorD:
      return "ObjectDescriptor";
    default:
      return NULL;
    }
}

/*--------------------------------------------------------------------
 * Name:   VcIsInteger
 * Author: ITEX C-Code Generator
 *
 * Returns TRUE if the argument value is an INTEGER type, FALSE
 * otherwise.
 */

static Bool VcIsInteger(VcType * type) 
{
  return VcGetBaseType(type) == VcINTEGERD;
}

/*--------------------------------------------------------------------
 * Name:   VcIsSimple
 * Author: ITEX C-Code Generator
 *
 * Returns TRUE if the argument value is an simple type, FALSE
 * otherwise.  A simple type is one of INTEGER, ENUMERATED or
 * BOOLEAN.
 */

static Bool VcIsSimple(VcType * type) 
{
  VcTypeName name = VcGetBaseType(type);

  return name == VcINTEGERD || name == VcENUMERATEDD || 
         name == VcBOOLEAND || name == VcR_TYPED;
}

/*--------------------------------------------------------------------
 * Name:   VcIsString
 * Author: ITEX C-Code Generator
 *
 * Returns TRUE if the argument value is an STRING type, FALSE
 * otherwise.
 */

static Bool VcIsString(VcType * type)
{
  VcTypeName name = VcGetBaseType(type);

  return
    name == VcBIT_STRINGD || name == VcHEXSTRINGD ||
    name == VcOCTET_STRINGD || name == VcNumericStringD ||
    name == VcPrintableStringD || name == VcTeletexStringD ||
    name == VcVideotexStringD || name == VcIA5StringD ||
    name == VcUTCTimeD || name == VcGeneralizedTimeD ||
    name == VcGraphicStringD || name == VcVisibleStringD ||
    name == VcGeneralStringD || name == VcISO646StringD ||
    name == VcT61StringD;
}

/*--------------------------------------------------------------------
 * Name:   VcIsStructured
 * Author: ITEX C-Code Generator
 *
 * Returns TRUE if the argument value is an SEQUENCE/SET or
 * SEQUENCE_OF/SET_OF type, FALSE otherwise.
 */

static Bool VcIsStructured(VcType * type)
{
  return VcIsSequence(type) || VcIsSequenceOf(type);
}

/*--------------------------------------------------------------------
 * Name:   VcIsSequence
 * Author: ITEX C-Code Generator
 *
 * Returns TRUE if the argument value is an SEQUENCE or SET type,
 * FALSE otherwise.
 */

static Bool VcIsSequence(VcType * type)
{
  VcTypeName name = VcGetBaseType(type);
  return name == VcSETD || name == VcSEQUENCED;
}

/*--------------------------------------------------------------------
 * Name:   VcIsSequenceOf
 * Author: ITEX C-Code Generator
 *
 * Returns TRUE if the argument value is an SEQUENCE_OF or SET_OF
 * type, FALSE otherwise.  
 */

static Bool VcIsSequenceOf(VcType * type)
{
  VcTypeName name = VcGetBaseType(type);
  return name == VcSET_OFD || name == VcSEQUENCE_OFD;
}

/*--------------------------------------------------------------------
 * Name:   VcIsChoice
 * Author: ITEX C-Code Generator
 *
 * Returns TRUE if the argument value is a CHOICE type, FALSE
 * otherwise.
 */

static Bool VcIsChoice(VcType * type) 
{
  return VcGetBaseType(type) == VcCHOICED;
}

/*--------------------------------------------------------------------
 * Name:   VcIsOBJECT_IDENTIFIER
 * Author: ITEX C-Code Generator
 *
 * Returns TRUE if the argument value is a OBJECT IDENTIFIER type, FALSE
 * otherwise.
 */

static Bool VcIsOBJECT_IDENTIFIER(VcType * type) 
{
  return VcGetBaseType(type) == VcOBJECT_IDENTIFIERD;
}

/*--------------------------------------------------------------------
 * Name:   VcIsReal
 * Author: ITEX C-Code Generator
 *
 * Returns TRUE if the argument value is a REAL type, FALSE otherwise.
 */

static Bool VcIsReal(VcType * type) 
{
  return VcGetBaseType(type) == VcREALD;
}

/*--------------------------------------------------------------------
 * Name:   VcIsOCTET
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcIsOCTET_STRING(VcType * type) 
{
  return VcGetBaseType(type) == VcOCTET_STRINGD ? TRUE : FALSE;
}

/*--------------------------------------------------------------------
 * VcList
 */

struct VcList
{
  VcType *type;                 /* The type of values we are storing.
                                 * Only valid if we are working with a
                                 * SEQUENCE_OF or a SET_OF. */

  VcValue **begin;              /* The array of values. */

  unsigned int length;          /* The length of the array of values. */
  unsigned int used;            /* How many elements of the array we
                                 * are using */
};

static 
VcList *VcListGrow(VcList * list, unsigned int min_length)
{
  const unsigned int increment = 16;
  
  if (list == NULL) {
    VcWarning("VcListGrow", "<NULL> list can not grow");
    return list;
  }

  if (min_length > list->length) {
    VcValue **hold;
    unsigned int new_length;
    
    new_length = list->length;
    while (new_length < min_length)
      new_length += increment;
    
    hold = list->begin;
    list->begin = VCMALLOC(sizeof(VcValue *) * new_length, "VcListGrow");
    if (hold == NULL)
      memset(list->begin, 0, new_length * sizeof(VcValue *));
    else {
      memcpy(list->begin, hold, list->used * sizeof(VcValue *));
      memset(list->begin + list->used, 0, 
	     (new_length - list->used) * sizeof(VcValue *));
      VCFREE(hold, list->length * sizeof(VcValue *), "VcListGrow");
    }
    list->length = new_length;
  }
  return list;
}

/*--------------------------------------------------------------------
 * Name:   VcListCreate
 * Author: ITEX C-Code Generator
 * 
 */

static
VcList *VcListCreate(int nfields)
{
  VcList *list = VCMALLOC(sizeof(VcList), "VcListCreate");

  list->type = VcUndefined;
  list->begin = NULL;
  list->length = 0;
  list->used = 0;

  VcListGrow(list, nfields);

  list->used = nfields;

  return list;
}

/*--------------------------------------------------------------------
 * Name:   VcListDelete
 * Author: ITEX C-Code Generator
 * 
 */

static void VcListDelete(VcList *list)
{
  int i;
  if (list) {
    for (i = 0; i < VcListLength(list); ++i)
      VcRmValue(VcListGetValue(list, i));
    VCFREE(list->begin, sizeof(VcValue *) * VcListAllocated(list), "VcListDelete");
  }
  VCFREE(list, sizeof(struct VcList), "VcListDelete");
}

static int VcListLength(VcList * list)
{
  if (list == NULL) {
    VcWarning("VcListLength", "getting length of NULL list");
    return 0;
  }
  return list->used;
}

static int VcListAllocated(VcList * list)
{
  if (list == NULL) {
    VcWarning("VcListLength", "getting allocation of NULL list");
    return 0;
  }
  return list->length;
}

static VcValue * VcListGetValue(VcList * list, unsigned int index)
{
  if (list == NULL) {
    VcWarning("VcListGetValue", "list not allocated");
    return NULL;
  }

  if (index + 1 > list->used) {
    VcWarning("VcListGetValue", "index out of bounds");
    return NULL;
  }
  return list->begin[index];
}

static VcValue * VcListSetValue(VcList * list, unsigned int index, VcValue * value)
{
  VcListGrow(list, index + 1);

  if (list == NULL) {
    VcWarning("VcListSetValue", "list not allocated");
    return NULL;
  }

  if (index + 1 > list->used)
    list->used = index + 1;

  return list->begin[index] = value;
}

static VcList * VcListAddLast(VcList * list, VcValue * value)
{
  if (list == NULL)
    list = VcListCreate(0);
  if (list->used == list->length)
    VcListGrow(list, list->length + 1);

  list->begin[list->used++] = value;

  return list;
}

static VcList * VcListAddFirst(VcList * list, VcValue * value)
{
  int i;
  
  if (list == NULL) {
    VcWarning("VcListGetValue", "list not allocated");
    return NULL;
  }

  VcListGrow(list, list->used + 1);

  for (i = list->used; i > 0; --i)
    list->begin[i] = list->begin[i - 1];
  ++(list->used);

  list->begin[0] = value;

  return list;
}

static VcList* VcListCopy(VcList* list)
{
  VcList *tmp = NULL;
  int i;

  if (list == NULL)
    return NULL;

  tmp = VcListCreate(0);
  for(i = 0; i < VcListLength(list); ++i)
    VcListAddLast(tmp, VcCopy(VcListGetValue(list, i)));

  return tmp;
}


/*--------------------------------------------------------------------
 * The structure for a restriction Value...
 */

struct VcRestriction
{
  VcRestrictionType type;       /* The type of the restriction */

  /* The following union depends on the type field above.  If the
   * type field indicates that this restriction is an
   * INCLUDE-restriction the include field in the union should be
   * used, and the body field otherwise. */

  union {
    VcList *value;		/* The restriction body */
    VcTypeList *include;	/* The included type    */
  }
  content;

  VcRestriction *next;
};

/*--------------------------------------------------------------------
 * VcReal
 */

struct VcReal
{
  VcBaseVal mantissa;
  VcBaseVal base;
  VcBaseVal exponent;
};

/*--------------------------------------------------------------------
 * Name:   VcGetMantissa
 * Author: ITEX C-Code Generator
 *
 * Gets the mantissa of the real.
 */

VcBaseVal VcGetMantissa(VcReal * real)
{
  if (real)
    return real->mantissa;
  else {
    VcWarning("VcGetMantissa", "<NULL> real value");
    return 0;
  }
}

/*--------------------------------------------------------------------
 * Name:   VcGetBase
 * Author: ITEX C-Code Generator
 *
 * Gets the base of the real.
 */

VcBaseVal VcGetBase(VcReal * real)
{
  if (real)
    return real->base;
  else {
    VcWarning("VcGetBase", "<NULL> real value");
    return 0;
  }
}

/*--------------------------------------------------------------------
 * Name:   VcGetExponent
 * Author: ITEX C-Code Generator
 *
 * Gets the exponent of the real.
 */

VcBaseVal VcGetExponent(VcReal * real)
{
  if (real)
    return real->exponent;
  else {
    VcWarning("VcGetExponent", "<NULL> real value");
    return 0;
  }
}

/*--------------------------------------------------------------------
 * Name:   VcSetMantissa
 * Author: ITEX C-Code Generator
 *
 * Sets the mantissa of the real to the indicated value.
 */

void VcSetMantissa(VcReal * real, VcBaseVal value)
{
  if (real)
    real->mantissa = value;
  else
    VcWarning("VcSetMantissa", "<NULL> real value");
}

/*--------------------------------------------------------------------
 * Name:   VcSetBase
 * Author: ITEX C-Code Generator
 *
 * Sets the base of the real to the indicated value.
 */

void VcSetBase(VcReal * real, VcBaseVal value)
{
  if (real)
    real->base = value;
  else
    VcWarning("VcSetBase", "<NULL> real value");
}

/*--------------------------------------------------------------------
 * Name:   VcSetExponent
 * Author: ITEX C-Code Generator
 *
 * Sets the exponent of the real to the indicated value.
 */

void VcSetExponent(VcReal * real, VcBaseVal value)
{
  if (real)
    real->exponent = value;
  else
    VcWarning("VcSetExponent", "<NULL> real value");
}

/*--------------------------------------------------------------------
 * VcChoice
 */

struct VcChoice
{
  VcValue *value;
  VcString name;
};

/*--------------------------------------------------------------------
 * VcValue
 */

struct VcValue
{
  VcType *type;			/* The value's type           */
  VcAttribute attribute;	/* The attribute of the value */
  union {
    VcBaseVal integer;		/* Integer value    */
    VcString string;		/* String value     */
    VcReal *real;		/* Real value       */
    VcChoice *choice;		/* Choice value     */
    VcList *list;		/* A list of values */
  }
  value;
  Bool is_choice_element;
};

/*--------------------------------------------------------------------
 * Name:   VcGetType
 * Author: ITEX C-Code Generator
 *
 * Returns the type of the given VcValue.  If the given value is
 * NULL a NULL value is returned indicating an undefined type.
 */

VcType * VcGetType(VcValue * value)
{
  return value ? value->type : NULL;
}

/*--------------------------------------------------------------------
 * Name:   VcBaseType
 * Author: ITEX C-Code Generator
 *
 */

static VcType * VcBaseType(VcValue * value)
{
  VcType *prev, *current;

  current = VcGetType(value);
  prev = NULL;
  while (current) {
    prev = current;
    current = VcGetParent(current);
  }

  return prev;
}

/*--------------------------------------------------------------------
 * Name:   VcSetType
 * Author: ITEX C-Code Generator
 * 
 * Sets the type pointer field in the given VcValue.  This is not
 * done if the type points to a PDU unless the value contains an
 * "instead of value" value.
 */

VcValue * VcSetType(VcType * type, VcValue * value)
{
  if (type && value) {
    VcTypeName base_type = VcGetBaseType(type);
    VcTypeName base_value = VcGetBaseType(VcGetType(value));
  
    if ((base_type == VcCHOICED) &&
        (base_value != VcCHOICED)) {
      /* If the type of the value is an INTEGER (the dummy type assigned 
       * to the value) we should simply set the type of the value to 
       * 'type' and create an unnamed ("") choice field. */
      value->type = type;
      VcMkCHOICEContents(value, "", NULL);
      return value;
    }
    
    if (base_type == VcPDUD) {
      if (VcIsInsteadOfValue(value))
        value->type = type;
    }
    else {
      if (base_value == VcSEQUENCE_OFD && !VcIsInsteadOfValue(value)) {
        if (VcSeqSize(value) > 0 &&
          VcGetBaseType(VcGetListType(value)) == VcUndefinedD)
          VcSetListType(VcGetType(VcGetElem(0, value)), value);
      }
      if (base_value == VcUndefinedD || base_type != VcUndefinedD)
        value->type = type;
    }
  }
  return value;
}


/*-------------------------------------------------------------------
 * Name:   VcValueAttribute
 * Author: ITEX C-Code Generator
 *
 * Returns the attributes currently set in the given value.
 * There should only be *ONE* attribute set at every given time.
 * If there are several attributes set it is an error!
 */

static VcAttribute VcValueAttribute(VcValue * value)
{
  if (value == NULL)
    return VcEMPTYATTR;
  return value->attribute;
}

/*--------------------------------------------------------------------
 * Name:   VcSetValueAttribute
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcSetValueAttribute(VcAttribute attribute, VcValue * value)
{
  if (value == NULL)
    return value;
  value->attribute |= attribute;
  return value;
}

/*--------------------------------------------------------------------
 * Name:   VcGetValueAttribute
 * Author: ITEX C-Code Generator
 * 
 */

Bool VcGetValueAttribute(VcAttribute attribute, VcValue * value)
{
  if (!value)
    return FALSE;
  if (value->attribute & attribute)
    return TRUE;
  else
    return FALSE;
}

/*--------------------------------------------------------------------
 * Name:   VcGetList
 * Author: ITEX C-Code Generator
 * 
 */

static VcList * VcGetList(VcValue * value)
{
  return value ? value->value.list : NULL;
}

/*--------------------------------------------------------------------
 * Functions for memory handling.
 */

/*--------------------------------------------------------------------
 * Some macros.
 */

/* Temporary memory size can be specified as a parameter to the
 * compiler. */

#ifndef VcTMPMEMSIZE
#define VcTMPMEMSIZE (100 * 1024)
#endif

enum
{
  VcStaticMemory = 1,
  VcTemporaryMemory = 2
};

/*--------------------------------------------------------------------
 * The types used in the memory handling.
 */

struct VcFrameType
{
  char *pointer;
  struct VcFrameType *next;
};

typedef struct VcMemoryHeap
{
  char *begin;			/* Start of temporary area.    */
  char *next;			/* Next free temporary byte.   */
  VcFrameType *frame;		/* The frame stack.            */
  VcMallocType context;		/* static or temporary memory. */
} VcMemoryHeap;

/*********************************************************************
 *
 ********************************************************************/

VcMemoryHeap *VcHeapCreate(void)
{
  VcMemoryHeap *heap = (VcMemoryHeap *) malloc(sizeof(VcMemoryHeap));
  heap->begin = (char *) malloc(sizeof(char) * VcTMPMEMSIZE);
  heap->next = heap->begin;
  heap->frame = NULL;
  heap->context = VcStaticMemory;

  return heap;
}

/*********************************************************************
 * 
 ********************************************************************/

void VcHeapDelete(VcMemoryHeap *heap)
{
  free((void *) heap->begin);

  while (heap->frame != NULL) {
    VcFrameType *hold = heap->frame;
    heap->frame = heap->frame->next;
    free((void *) hold);
  }
  
  free((void *) heap);
}

/*********************************************************************
 * 
 ********************************************************************/

/* The context that a concurrent test-case/step operates under. */

struct VcContext {
  VcList *variables;    /* TS and TC variables.          */
  VcMemoryHeap *heap;   /* Local heap for the test-step. */
  int verdict;          /* The local test-step verdict.  */
  int component;        /* The component descriptor.     */
  const char *name;     /* The name of the component.    */
  VcTimerType* timers; /* A local copy of all timers.   */
};

VcContext *context = NULL;


/*
 * The default context, i.e. a context containing the variables default values.
 */
static VcContext *default_context = NULL;

void VcCreateDefaultContext(void)
{
  IcSymTabEntry *current = IcSymTab;

  if (default_context) {
    context = NULL;
    return;
  }

  /* "Steal" the current context */
  default_context = context;

  while (current->Name != NULL) {
    if (current->Type == GcTS_VarDclD || current->Type == GcTC_VarDclD) {
      VcListAddLast(context->variables, *current->Val);
      /* Make sure that the symtab do not refer to the default values: */
      *(VcValue**)current->Val = NULL;
    }
    ++current;
  }

  context = NULL;
}

/* Initialize the Test Case variables. This should be done for
 * every MTC/PTC according to the standard. */
void VcResetTCVars(void)
{
  IcSymTabEntry *current = IcSymTab;
  int index = 0;

  while (current->Name != NULL) {
    if (current->Type == GcTC_VarDclD || current->Type == GcTS_VarDclD) {
      if (current->Type == GcTC_VarDclD) {
#ifdef DEBUG_VALREP
	fprintf(stderr, "VcResetTCVars: assigning %s @ %p.\n",
		current->Name, *current->Val);
#endif
	VcAssign(*current->Val,
		 VcListGetValue(default_context->variables, index),
		 VcGetAlloc(),
		 current->Name);
      }
      ++index;
    }
    ++current;
  }
}


void VcMemoryReport(const char *name, void *p)
{
#ifdef DEBUG_VALREP
  fprintf(stdout, "%s:\tC:%p ", name, context);
  if (context != NULL) {
    fprintf(stdout, "H:%p ", context->heap);
    if (context->heap != NULL)
      fprintf(stdout, "F:%p ", context->heap->frame);
  }
  if (p != NULL)
    fprintf(stdout, "P:%p", p);
  fprintf(stdout, "\n");
#endif
}

/*********************************************************************
 * 
 ********************************************************************/

void VcContextCreate(int component, const char *name)
{
  IcSymTabEntry *current = NULL;
  int index;

  context = (VcContext *) malloc(sizeof(VcContext));
  context->heap = NULL;

  context->verdict   = IcGetVerdict();
  context->heap      = VcHeapCreate();
  context->variables = VcListCreate(0);
  context->component = component;

  context->name = (const char *) malloc(sizeof(char) * (Vcstrlen(name) + 1));
  Vcstrcpy((char *) context->name, name);
  context->timers    = NULL;

  if (default_context) {
    current = IcSymTab;
    index = 0;
    while (current->Name != NULL) {
      if (current->Type == GcTS_VarDclD || current->Type == GcTC_VarDclD) {
	VcListAddLast(context->variables,
		      VcCopy(VcListGetValue(default_context->variables, index)));
	index++;
      }
      ++current;
    }
  }
}

/*********************************************************************
 *
 ********************************************************************/

void VcContextCreateMTC(int component, const char *name)
{
  IcSymTabEntry *current = NULL;
  int index = 0;
  
  MTCD = component;
  VcContextCreate(component, name);

  /* Need to reassign symbol table variables to refer to the *same*
   * values that the context refers:
   */

  current = IcSymTab;
  while (current->Name != NULL) {
    if (current->Type == GcTS_VarDclD || current->Type == GcTC_VarDclD) {
      *current->Val = VcListGetValue(context->variables, index++);
    }
    ++current;
  }

  VcMemoryReport("Context: create (MTC)", context);
}

/*********************************************************************
 *
 ********************************************************************/

void VcContextCreatePTC(int component, const char *name)
{
  IcSymTabEntry *current = NULL;
  int index = 0;

  VcContextCreate(component, name);

  /* Need to reassign symbol table variables to refer to the *same*
   * values that the context refers:
   */

  current = IcSymTab;
  while (current->Name != NULL) {
    if (current->Type == GcTS_VarDclD || current->Type == GcTC_VarDclD) {
      *current->Val = VcListGetValue(context->variables, index++);
    }
    ++current;
  }

  VcMemoryReport("Context: create (PTC)", context);
}

/*********************************************************************
 * 
 ********************************************************************/

void VcContextDelete(void)
{
  VcMemoryReport("Context: delete", NULL);
  
  if (context != NULL) {
    VcListDelete(context->variables);
    VcHeapDelete(context->heap);
    free((void *) context->name);
    free((void *) context->timers);
    free((void *) context);
    context = NULL;
  }
}

/*********************************************************************
 * 
 ********************************************************************/

void VcContextSwitch(VcContext *new_context)
{
  IcSymTabEntry *current = NULL;
  int index = 0;

  VcMemoryReport("Context: switch (b)", NULL);
  
  if (new_context == NULL) {
      VcWarning("VcContextSwitch", 
                "trying to switch to <NULL> context. No switch made.");
    return;
  }

  if (new_context == context)
    return;

  context = new_context;

  current = IcSymTab;
  while (current->Name != NULL) {
    if (current->Type == GcTS_VarDclD || current->Type == GcTC_VarDclD) {
      *current->Val = VcListGetValue(context->variables, index++);
    }
    ++current;
  }

  VcMemoryReport("Context: switch (a)", NULL);
}

/*********************************************************************
 * 
 ********************************************************************/

VcTimerType* VcContextGetTimers(void)
{
  return context->timers;
}


void VcContextSetTimers( int no_of_timers, VcTimerType* timers )
{
  if ( context->timers != NULL ) {
    free( context->timers );
  }
#ifdef DEBUG_VALREP
  fprintf( stderr, "Allocated %d bytes for %d context timers\n", 
                    sizeof(VcTimerType)*no_of_timers, no_of_timers);
#endif

  context->timers = (VcTimerType* )malloc( sizeof(VcTimerType)*no_of_timers);

  memcpy( context->timers, timers, sizeof(VcTimerType)*no_of_timers );
}

const char *VcContextGetComponentName(void)
{
  return context->name;
}

/*********************************************************************
 * 
 ********************************************************************/

int VcContextGetComponent(void)
{
  return context->component;
}

/*********************************************************************
 * 
 ********************************************************************/

int VcContextGetVerdict(void)
{
  return context->verdict;
}

/*********************************************************************
 * 
 ********************************************************************/

void VcContextSetVerdict(int verdict)
{
  context->verdict = verdict;
}

/*********************************************************************
 * 
 ********************************************************************/

void VcContextSetID(int component)
{
  context->component = component;
}

/*--------------------------------------------------------------------
 * Name:   VcIsTemporary
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcIsTemporary(void *p)
{
  return context->heap->begin <= (char *) p && (char *) p < context->heap->next;
}

/*--------------------------------------------------------------------
 * Name:   VcIsLegalTemporary
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcIsLegalTemporary(void *p)
{
  return
    context->heap->begin <= (char *) p &&
    (char *) p < context->heap->begin + VcTMPMEMSIZE;
}

/*--------------------------------------------------------------------
 * Name:   VcMalloc
 * Author: ITEX C-Code Generator
 *
 */

void * VcMalloc(size_t size, char * note)
{
  char *p;

  size = (size + 3) & ~3;

  if (VcGetAlloc() == VcStaticMemory) {
    p = (char *) malloc(size);
    memset(p, 0, size);
    VcMemoryReport("Malloc (S)", p);
  }
  else {
    p = context->heap->next;
    VcMemoryReport("Malloc (T)", p);
    
    if (VcIsLegalTemporary(p + size))
      context->heap->next = context->heap->next + size;
    else {
      VcWarning("VcMalloc", "out of temporary memory, using static memory");
      if ((p = (char *) malloc(size)) == NULL) {
	VcWarning("VcMalloc", "out of ALL memory, exiting!");
	exit(3770);
      }
    }
    memset(p, 0, size);
  }
  return (void *) p;
}

/*--------------------------------------------------------------------
 * Name:   VcFree
 * Author: ITEX C-Code Generator
 *
 */

void VcFree(void *p, size_t size, char * note)
{
  if (p != NULL)
    if (VcIsTemporary(p)) {
      VcMemoryReport("Free (T)", p);
      /* No deallocation of temporary memory.  At least not here.
       * This memory is later free'd by 'VcFreeFrame'. */
      return;
    }
    else {
      VcMemoryReport("Free (S)", p);
    
      /* Actually free the memory */
      free((void *) p);
    }
}

/*--------------------------------------------------------------------
 * Name:   VcMkFrame
 * Author: ITEX C-Code Generator
 *
 * Function for creating a new frame by storing the current
 * memory pointer in a new frame.
 *
 * This function is also responsible for setting the memory
 * allocation strategy to the temporary mode.
 */

void VcMkFrame(void)
{
  VcFrameType *frame;

  frame = (VcFrameType *) malloc(sizeof(VcFrameType));
  frame->next    = context->heap->frame;
  frame->pointer = context->heap->next;
  context->heap->frame = frame;

  VcMemoryReport("Frame:   create", NULL);

  VcSetTempAlloc();
}

/*--------------------------------------------------------------------
 * Name:   VcFreeFrame
 * Author: ITEX C-Code Generator
 *
 * Function for removing a frame.
 */

void VcFreeFrame(void)
{
  VcFrameType *frame;

  VcMemoryReport("Frame:   delete", NULL);

  if (context->heap->frame) {
    frame = context->heap->frame->next;
    if (context->heap->frame->pointer)
      context->heap->next = context->heap->frame->pointer;
    free(context->heap->frame);
    context->heap->frame = frame;
  }
}

/*--------------------------------------------------------------------
 * Name:   VcSetPermAlloc
 * Author: ITEX C-Code Generator
 *
 * Switching to permanent memory allocation.
 */

void VcSetPermAlloc(void)
{
  VcSetMalloc(VcStaticMemory);
}

/*--------------------------------------------------------------------
 * Name:   VcSetTempAlloc
 * Author: ITEX C-Code Generator
 *
 * Switching to temporary memory allocation.
 */

void VcSetTempAlloc(void)
{
  VcSetMalloc(VcTemporaryMemory);
}

/*--------------------------------------------------------------------
 * Name:   VcSetMalloc
 * Author: ITEX C-Code Generator
 *
 */

void VcSetMalloc(VcMallocType strategy)
{
  context->heap->context = strategy;
}

/*--------------------------------------------------------------------
 * Name:   VcGetAlloc
 * Author: ITEX C-Code Generator
 */

VcMallocType VcGetAlloc(void)
{
  return context->heap->context;
}

/*--------------------------------------------------------------------
 * Name:   VcInitMem
 * Author: ITEX C-Code Generator
 *
 * Init the temporary memory structure.  Must be called prior to
 * using temporary memory.
 */

void VcInitMem(void)
{
  VcContextCreate(0, "MTC");
}

/*--------------------------------------------------------------------
 * Name:   VcMemPrint
 * Author: ITEX C-Code Generator
 *
 * Memory-use display function.
 */

void VcMemPrint(void)
{
  VcMessage("VcMemPrint", "NOT IMPLEMENTED");
}

/*--------------------------------------------------------------------
 * Functions that handle the removal of VcValues.
 */

/*--------------------------------------------------------------------
 * Name:   VcRmString
 * Author: ITEX C-Code Generator
 *
 */

static void VcRmString(VcValue * value)
{
  VCFREE(VcGetCSTRING(value), Vcstrlen(VcGetCSTRING(value)) + 1, "VcRmString");
}

/*--------------------------------------------------------------------
 * Name:   VcRmSequence
 * Author: ITEX C-Code Generator
 *
 */

static
void VcRmSequence(VcValue * value)
{
  VcListDelete(VcGetList(value));
}

/*--------------------------------------------------------------------
 * Name:   VcRmOBJECT_IDENTIFIER
 * Author: ITEX C-Code Generator
 *
 */

static void VcRmOBJECT_IDENTIFIER(VcValue *value)
{
  VcListDelete(VcGetList(value));
}


/*--------------------------------------------------------------------
 * Name:   VcRmChoice
 * Author: ITEX C-Code Generator
 *
 * Remove the contents of a CHOICE value.  This is the "struct
 * VcChoice", the CHOICE value but NOT the CHOICE name.  The name
 * is a constant string defined in the file "s_constr.c".  It
 * should not be deallocated.
 */

static void VcRmChoice(VcValue * value)
{
  VcRmValue(VcGetCHOICE("", NULL, value));
  VCFREE(value->value.choice, sizeof(struct VcChoice), "VcRmChoice");
}

/*--------------------------------------------------------------------
 * Name:   VcRmReal
 * Author: ITEX C-Code Generator
 *
 */

static void VcRmReal(VcValue * value)
{
  VCFREE(VcGetREAL(value), sizeof(struct VcReal), "VcRmReal");
}

/*--------------------------------------------------------------------
 * Name:   VcRmValue
 * Author: ITEX C-Code Generator
 *
 */

void VcRmValue(VcValue * value)
{
  /* If there is no value to remove, just return. */
  if (value == NULL)
    return;

  if (VcIsInsteadOfValue(value)) {
    VcAttribute attribute = VcValueAttribute(value);
    if (attribute & (VcVALUERANGEATTR | VcVALUELISTATTR | 
		     VcCOMPLEMENTATTR | VcSUPERSETATTR | 
		     VcSUBSETATTR)) {
      VcRmSequence(value);
    }
  }
  else {
    switch (VcGetBaseType(VcGetType(value))) {
      case VcINTEGERD:
      case VcENUMERATEDD:
      case VcBOOLEAND:
      case VcNULLD:
      case VcR_TYPED:
	/* Values of these types are fully contained within the value
	 * given as argument to this function and are FREE'd at the
	 * end of this function. */
	break;
      case VcREALD:
	VcRmReal(value);
	break;
      case VcNumericStringD:
      case VcPrintableStringD:
      case VcTeletexStringD:
      case VcVideotexStringD:
      case VcIA5StringD:
      case VcUTCTimeD:
      case VcGeneralizedTimeD:
      case VcGraphicStringD:
      case VcVisibleStringD:
      case VcISO646StringD:
      case VcT61StringD:
      case VcGeneralStringD:
      case VcObjectDescriptorD:
      case VcHEXSTRINGD:
      case VcBIT_STRINGD:
      case VcOCTET_STRINGD:
	VcRmString(value);
	break;
      case VcOBJECT_IDENTIFIERD:
  VcRmOBJECT_IDENTIFIER(value);
  break;
      case VcSEQUENCE_OFD:
      case VcSET_OFD:
      case VcSEQUENCED:
      case VcSETD:
	VcRmSequence(value);
	break;
      case VcCHOICED:
	VcRmChoice(value);
	break;
      default:
	VcWarning("RmValue", "unknown type");
	break;
    }
  }
  VCFREE(value, sizeof(VcValue), "VcRmValue");
}

/*--------------------------------------------------------------------
 * The functions that handle VcType objects.
 */

/*--------------------------------------------------------------------
 * The base types
 *
 * The base types are defined as pointers to a type object.  The
 * initialialization is done in the function VcBaseTypes.  All
 * supported base-types must have an entry here.
 */

VcType *VcANY = NULL;
VcType *VcBIT_STRING = NULL;
VcType *VcBOOLEAN = NULL;
VcType *VcCHOICE = NULL;
VcType *VcENUMERATED = NULL;
VcType *VcGeneralString = NULL;
VcType *VcGeneralizedTime = NULL;
VcType *VcGraphicString = NULL;
VcType *VcHEXSTRING = NULL;
VcType *VcIA5String = NULL;
VcType *VcINTEGER = NULL;
VcType *VcISO646String = NULL;
VcType *VcNULL = NULL;
VcType *VcNumericString = NULL;
VcType *VcOBJECT_IDENTIFIER = NULL;
VcType *VcOCTET_STRING = NULL;
VcType *VcObjectDescriptor = NULL;
VcType *VcPDU = NULL;
VcType *VcPrintableString = NULL;
VcType *VcREAL = NULL;
VcType *VcR_TYPE = NULL;
VcType *VcSEQUENCE = NULL;
VcType *VcSEQUENCE_OF = NULL;
VcType *VcSET = NULL;
VcType *VcSET_OF = NULL;
VcType *VcT61String = NULL;
VcType *VcTAGGED = NULL;
VcType *VcTeletexString = NULL;
VcType *VcUTCTime = NULL;
VcType *VcUndefined = NULL;
VcType *VcVideotexString = NULL;
VcType *VcVisibleString = NULL;

/*--------------------------------------------------------------------
 * Name:   VcBaseTypes
 * Author: ITEX C-Code Generator
 *
 * Initializes all the base-types.  All this function does is to
 * create one instance of each base-type.
 */

void VcBaseTypes(void)
{
  VcANY = VcMkType(VcANYD);
  VcTagType(VcUNIVERSAL, VcANYD, FALSE, VcANY);
  VcBIT_STRING = VcMkType(VcBIT_STRINGD);
  VcTagType(VcUNIVERSAL, VcBIT_STRINGD, FALSE, VcBIT_STRING);
  VcBOOLEAN = VcMkType(VcBOOLEAND);
  VcTagType(VcUNIVERSAL, VcBOOLEAND, FALSE, VcBOOLEAN);
  VcCHOICE = VcMkType(VcCHOICED);
  VcTagType(VcUNIVERSAL, VcCHOICED, FALSE, VcCHOICE);
  VcENUMERATED = VcMkType(VcENUMERATEDD);
  VcTagType(VcUNIVERSAL, VcENUMERATEDD, FALSE, VcENUMERATED);
  VcGeneralString = VcMkType(VcGeneralStringD);
  VcTagType(VcUNIVERSAL, VcGeneralStringD, FALSE, VcGeneralString);
  VcGeneralizedTime = VcMkType(VcGeneralizedTimeD);
  VcTagType(VcUNIVERSAL, VcGeneralizedTimeD, FALSE, VcGeneralizedTime);
  VcGraphicString = VcMkType(VcGraphicStringD);
  VcTagType(VcUNIVERSAL, VcGraphicStringD, FALSE, VcGraphicString);
  VcHEXSTRING = VcMkType(VcHEXSTRINGD);
  VcTagType(VcUNIVERSAL, VcHEXSTRINGD, FALSE, VcHEXSTRING);
  VcIA5String = VcMkType(VcIA5StringD);
  VcTagType(VcUNIVERSAL, VcIA5StringD, FALSE, VcIA5String);
  VcINTEGER = VcMkType(VcINTEGERD);
  VcTagType(VcUNIVERSAL, VcINTEGERD, FALSE, VcINTEGER);
  VcISO646String = VcMkType(VcISO646StringD);
  VcTagType(VcUNIVERSAL, VcISO646StringD, FALSE, VcISO646String);
  VcNULL = VcMkType(VcNULLD);
  VcTagType(VcUNIVERSAL, VcNULLD, FALSE, VcNULL);
  VcNumericString = VcMkType(VcNumericStringD);
  VcTagType(VcUNIVERSAL, VcNumericStringD, FALSE, VcNumericString);
  VcOBJECT_IDENTIFIER = VcMkType(VcOBJECT_IDENTIFIERD);
  VcTagType(VcUNIVERSAL, VcOBJECT_IDENTIFIERD, FALSE, VcOBJECT_IDENTIFIER);
  VcOCTET_STRING = VcMkType(VcOCTET_STRINGD);
  VcTagType(VcUNIVERSAL, VcOCTET_STRINGD, FALSE, VcOCTET_STRING);
  VcObjectDescriptor = VcMkType(VcObjectDescriptorD);
  VcTagType(VcUNIVERSAL, VcObjectDescriptorD, FALSE, VcObjectDescriptor);
  VcPDU = VcMkType(VcPDUD);
  VcTagType(VcUNIVERSAL, VcPDUD, FALSE, VcPDU);
  VcPrintableString = VcMkType(VcPrintableStringD);
  VcTagType(VcUNIVERSAL, VcPrintableStringD, FALSE, VcPrintableString);
  VcREAL = VcMkType(VcREALD);
  VcTagType(VcUNIVERSAL, VcREALD, FALSE, VcREAL);
  VcSEQUENCE = VcMkType(VcSEQUENCED);
  VcTagType(VcUNIVERSAL, VcSEQUENCED, FALSE, VcSEQUENCE);
  VcSEQUENCE_OF = VcMkType(VcSEQUENCE_OFD);
  VcTagType(VcUNIVERSAL, VcSEQUENCE_OFD, FALSE, VcSEQUENCE_OF);
  VcSET = VcMkType(VcSETD);
  VcTagType(VcUNIVERSAL, VcSETD, FALSE, VcSET);
  VcSET_OF = VcMkType(VcSET_OFD);
  VcTagType(VcUNIVERSAL, VcSET_OFD, FALSE, VcSET_OF);
  VcT61String = VcMkType(VcT61StringD);
  VcTagType(VcUNIVERSAL, VcT61StringD, FALSE, VcT61String);
  VcTeletexString = VcMkType(VcTeletexStringD);
  VcTagType(VcUNIVERSAL, VcTeletexStringD, FALSE, VcTeletexString);
  VcUTCTime = VcMkType(VcUTCTimeD);
  VcTagType(VcUNIVERSAL, VcUTCTimeD, FALSE, VcUTCTime);
  VcUndefined = VcMkType(VcUndefinedD);
  VcTagType(VcUNIVERSAL, VcUndefinedD, FALSE, VcUndefined);
  VcVideotexString = VcMkType(VcVideotexStringD);
  VcTagType(VcUNIVERSAL, VcVideotexStringD, FALSE, VcVideotexString);
  VcVisibleString = VcMkType(VcVisibleStringD);
  VcTagType(VcUNIVERSAL, VcVisibleStringD, FALSE, VcVisibleString);
  VcR_TYPE = VcMkType(VcR_TYPED);
  VcTagType(VcUNIVERSAL, VcR_TYPED, FALSE, VcR_TYPE);
}

/*--------------------------------------------------------------------
 * Operations on VcValues with SEQUENCE/SEQUENCE_OF and
 * SET/SET_OF content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkLIST
 * Author: ITEX C-Code Generator
 *
 */

VcValue * VcMkLIST(void)
{
  VcValue *value;

  value = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkLIST");

  value->type       = VcSEQUENCE_OF;
  value->attribute  = VcEMPTYATTR;
  value->value.list = VcListCreate(0);

  return value;
}

/*--------------------------------------------------------------------
 * Name:   VcMkSEQUENCE
 * Author: ITEX C-Code Generator
 *
 * Create and return a sequence value with a given number of
 * fields.  All the fields contain values that are NULL pointers.
 * The Get/Set-Field operations initialize the field values to
 * real values.
 */

VcValue * VcMkSEQUENCE(int nfields)
{
  VcValue *sequence;

  sequence = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkSEQUENCE");

  sequence->type       = VcSEQUENCE;
  sequence->attribute  = VcEMPTYATTR;
  sequence->value.list = VcListCreate(nfields);

  return sequence;
}

/*--------------------------------------------------------------------
 * Name:   VcMkSEQUENCE
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkSEQUENCE_OF(VcType * type)
{
  return VcSetListType(type, VcSetType(VcSEQUENCE_OF, VcMkSEQUENCE(0)));
}

/*--------------------------------------------------------------------
 * Name:   VcSetListType
 * Author: ITEX C-Code Generator
 * 
 * This function sets the type field in a VcList to the indicated
 * type.  This function is intended to set the type for a
 * SET_OF/SEQUENCE_OF value.
 */

VcValue * VcSetListType(VcType * type, VcValue * sequence)
{
  if (VcIsSequenceOf(VcGetType(sequence))) {
    if (sequence->value.list == NULL)
      sequence->value.list = VcListCreate(0);
    if (VcGetBaseType(type) == VcUndefinedD && VcSeqSize(sequence) > 0)
      sequence->value.list->type = VcGetType(VcGetElem(0, sequence));
    else
      sequence->value.list->type = type;
  }
  else
    VcWarning("VcSetListType", "trying to set list-type on non-SEQUENCE_OF");
  return sequence;
}

/*--------------------------------------------------------------------
 * Name:   VcGetListType
 * Author: ITEX C-Code Generator
 *
 */

VcType * VcGetListType(VcValue * sequence)
{
  if (VcIsSequenceOf(VcGetType(sequence))) {
    if (sequence->value.list)
      return sequence->value.list->type;
    else
      return VcUndefined;
  }
  else {
    VcWarning("VcGetListType", "trying to get list-type from non-SEQUENCE_OF");
    return VcUndefined;
  }
}

/*--------------------------------------------------------------------
 * Name:   VcSeqSize
 * Author: ITEX C-Code Generator
 * 
 * Returns the size of a SEQUENCE.  If we have some part
 * missing/empty we return the size '0'.
 */

int VcSeqSize(VcValue * sequence)
{
  if (!VcIsStructured(VcGetType(sequence))) {
    VcWarning("VcSeqSize", "non-SEQUENCE/SET/SEQUENCE_OF/SET_OF value");
    return 0;
  }
  if (VcIsInsteadOfValue(sequence)) {
    VcWarning("VcSeqSize", "\"Not a Value\" value");
    return 0;
  }
  return VcListLength(VcGetList(sequence));
}

/*--------------------------------------------------------------------
 * Name:   VcSeqOfSize
 * Author: ITEX C-Code Generator
 * 
 * Returns the size of a SEQUENCE_OF.  If we have some part
 * missing/empty we return the size '0'.
 */

int VcSeqOfSize(VcValue * sequence_of)
{
  return VcSeqSize(sequence_of);
}

/*--------------------------------------------------------------------
 * Name:   VcMkSET
 * Author: ITEX C-Code Generator
 * 
 * Builds and returns a new SET with the indicated number of
 * list-elements.  The list-elements do not have any values
 * within them.  The value pointers are all set to NULL and have
 * to be allocated when a field is added.
 */

VcValue * VcMkSET(int nfields)
{
  return VcSetType(VcSET, VcMkSEQUENCE(nfields));
}

/*--------------------------------------------------------------------
 * Name:   VcMkSET
 * Author: ITEX C-Code Generator
 * 
 * Builds and returns a new SET_OF with the indicated number of
 * list-elements.  The list-elements do not have any values
 * within them.  The value pointers are all set to NULL and have
 * to be allocated when a field is added.
 */

VcValue * VcMkSET_OF(VcType * type)
{
  return VcSetType(VcSET_OF, VcMkSEQUENCE_OF(type));
}

/*--------------------------------------------------------------------
 * Name:   VcAddElem
 * Author: ITEX C-Code Generator
 * 
 * Adds an element first in a SEQUENCE_OF or SET_OF.
 * It works for all four cases, but is primary intended for
 * inserting elements into SEQUENCE_OF and SET_OF since they are
 * created without any size.
 */

VcValue * VcAddElem(VcValue * value, VcValue * sequence_of)
{
  if (sequence_of == NULL) {
    VcWarning("VcAddElem", "<NULL> SEQUENCE_OF");
    return NULL;
  }
  if (VcIsStructured(VcGetType(sequence_of)) && !VcIsInsteadOfValue(sequence_of)) {
    if (VcIsSequenceOf(VcGetType(sequence_of)) &&
	VcGetBaseType(VcGetType(value)) == VcUndefinedD && 
	VcGetBaseType(VcGetListType(sequence_of)) != VcUndefinedD)
      VcSetType(VcGetListType(sequence_of), value);
    (void) VcListAddFirst(VcGetList(sequence_of), value);
  }
  else
    VcWarning("VcAddElem", "trying to add element to non-SEQUENCE_OF");
  return sequence_of;
}

/*--------------------------------------------------------------------
 * Name:   VcAddElemZ
 * Author: ITEX C-Code Generator
 * 
 * Adds an element last in a SEQUENCE/SEQUENCE_OF or SET/SET_OF.
 * It works for all four cases, but is primary intended for
 * inserting elements into SEQUENCE_OF and SET_OF since they are
 * created without any size.
 */

VcValue * VcAddElemZ(VcValue * value, VcValue * sequence_of)
{
  if (!sequence_of) {
    VcWarning("VcAddElemZ", "<NULL> SEQUENCE_OF");
    return NULL;
  }
  if (VcIsStructured(VcGetType(sequence_of)) && !VcIsInsteadOfValue(sequence_of)) {
    if (VcIsSequenceOf(VcGetType(sequence_of)) &&
	VcGetBaseType(VcGetType(value)) == VcUndefinedD && 
	VcGetBaseType(VcGetListType(sequence_of)) != VcUndefinedD)
      VcSetType(VcGetListType(sequence_of), value);    
    (void) VcListAddLast(VcGetList(sequence_of), value);
  }
  else
    VcWarning("VcAddElemZ", "trying to add element to non-SEQUENCE_OF");
  return sequence_of;
}

/*--------------------------------------------------------------------
 * Name:   VcGetElem
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcGetElem(int index, VcValue * sequence_of)
{
  if (!sequence_of) {
    VcWarning("VcGetElem", "<NULL> SEQUENCE_OF");
    return NULL;
  }
  if (VcIsStructured(VcGetType(sequence_of)) && !VcIsInsteadOfValue(sequence_of))
    return VcListGetValue(VcGetList(sequence_of), index);
  else
    VcWarning("VcGetElem", "trying to get element to non-SEQUENCE_OF");
  return NULL;
}

/*--------------------------------------------------------------------
 * Name:   VcSetField
 * Author: ITEX C-Code Generator
 * 
 * Sets an already existing field to a value.  Can be used in
 * both SEQUENCE/SEQUENCE_OF and SET/SET_OF, but is primarily
 * intended for use with SEQUENCE and SET.  This is because they
 * get a size, but no elements, at creation time.
 */

VcValue * VcSetField(VcValue * value, int index, VcValue * sequence)
{
  if (!sequence) {
    VcWarning("VcSetField", "<NULL> SEQUENCE");
    return NULL;
  }
  if (VcIsStructured(VcGetType(sequence)) && !VcIsInsteadOfValue(sequence))
    (void) VcListSetValue(VcGetList(sequence), index - 1, value);
  else
    VcWarning("VcSetField", "trying to set element to non-SEQUENCE");
  return sequence;
}

/*--------------------------------------------------------------------
 * Name:   VcGetField
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcGetField(int index, VcValue * sequence)
{
  if (!sequence) {
    VcWarning("VcGetField", "<NULL> SEQUENCE");
    return NULL;
  }
  if (VcIsStructured(VcGetType(sequence)) && !VcIsInsteadOfValue(sequence))
    return VcListGetValue(VcGetList(sequence), index - 1);
  else
    VcWarning("VcGetField", "trying to extract element from non-SEQUENCE");
  return NULL;
}

/*--------------------------------------------------------------------
 * Operations on VcValues with OBJECT_IDENTIFIER/OBJECT_DESCRIPTOR
 * content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkOBJECT_IDENTIFIER
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkOBJECT_IDENTIFIER(VcValue* value)
{
  VcValue *tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkOBJECT_IDENTIFIER");

  tmp->type         = VcOBJECT_IDENTIFIER;
  tmp->attribute    = VcEMPTYATTR;

  if (value != NULL)
    tmp->value.list = VcListCopy(value->value.list);
  else
    tmp->value.list = NULL;
  
  return tmp;
}

/*--------------------------------------------------------------------
 * Name:   VcGetOBJECT_IDENTIFIER 
 * Author: ITEX C-Code Generator
 * 
 */

VcValue* VcGetOBJECT_IDENTIFIER(VcValue * value)
{
  if (value == NULL) {
    VcWarning("VcGetOBJECT_IDENTIFIER", "<NULL> value");
    return NULL;
  }
  return value;
}

/*--------------------------------------------------------------------
 * Name:   VcMkObjectDescriptor
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkObjectDescriptor(VcString value)
{
  VcValue *tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkObjectDescriptor");

  tmp->type         = VcObjectDescriptor;
  tmp->attribute    = VcEMPTYATTR;
  tmp->value.string = (VcString) VCMALLOC(Vcstrlen(value) + 1, "VcMkObjectDescriptor");

  Vcstrcpy(tmp->value.string, value);

  return tmp;
}

/*--------------------------------------------------------------------
 * Name:   VcGetObjectDescriptor 
 * Author: ITEX C-Code Generator
 * 
 */

VcString VcGetObjectDescriptor(VcValue * value)
{
  if (value == NULL) {
    VcWarning("VcGetObjectDescriptor", "<NULL> value");
    return NULL;
  }
  return value->value.string;
}

/*--------------------------------------------------------------------
 * Operations on VcValues with ENUMERATED content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkENUMERATED
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkENUMERATED(VcBaseVal value)
{
  VcValue *tmp;

  tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkENUMERATED");
  tmp->type          = VcENUMERATED;
  tmp->attribute     = VcEMPTYATTR;
  tmp->value.integer = value;

  return tmp;
}

/*--------------------------------------------------------------------
 * Name:   VcEnumMap
 * Author: ITEX C-Code Generator
 * 
 */

void VcEnumMap(char *data, VcType * type)
{
  type->info = (char *)
    VCMALLOC(sizeof(char) * (Vcstrlen(data) + 1), "VcEnumMap");
  Vcstrcpy(type->info, data);
}

/*--------------------------------------------------------------------
 * Name:   VcEnumGet
 * Author: ITEX C-Code Generator
 * 
 */

char * VcEnumGet(char *name, VcType * type)
{
  static char buffer[1024];
  char *p, *t;

  while (type && type->info == NULL)
    type = VcGetParent(type);

  /* Error somewhere */
  if (type == NULL || type->info == NULL || Vcstrlen(type->info) == 0 ||
      name == NULL) {
    VcWarning("VcEnumGet", "<NULL> pointer. No enumeration found.");
    Vcstrcpy(buffer, "Undefined enumeration");
    return buffer;
  }

  /* First is <num>$ */
  if (Vcstrncmp(type->info, name, Vcstrlen(name)) == 0) {
    /* Find the occurence of name */
    p = strstr(type->info, name);
    Vcstrncpy(buffer, p, Vcstrlen(p) + 1);
    p = strtok(buffer, "$");
    if (p == NULL) {
      Vcstrcpy(buffer, "Undefined enumeration");
      return buffer;
    }
    else {
      /* Find the name */
      p = strtok(NULL, "$");
      return p;
    }
  }
  else {
    int length = 0;

    /* Next is $<entity>$ */
    Vcstrcpy(buffer, "$");
    Vcstrcat(buffer, name);
    Vcstrcat(buffer, "$");
    p = strstr(type->info, buffer);
    if (p == NULL) {
      Vcstrcpy(buffer, "Undefined enumeration");
      return buffer;
    }
    ++p;

    length = Vcstrlen(p);
    if (length >= sizeof(buffer))
      length = sizeof(buffer);

    Vcstrncpy(buffer, p, length);
    t = strtok(buffer, "$");

    if (t == NULL) {
      Vcstrcpy(buffer, "Undefined enumeration");
      return buffer;
    }
    else {
      /* Find the name */
      t = strtok(NULL, "$");
      return t;
    }
  }
}

/*--------------------------------------------------------------------
 * Operations on OBJECT IDENTIFIERs.
 */

VcValue* VcAddOBJID_Component(VcValue* oid, int comp)
{
  if (oid != NULL)
  {
    if (oid->value.list == NULL)
      oid->value.list = VcListCreate(0);

    VcListAddLast(oid->value.list, VcMkINTEGER(comp));
  }

  return oid;
}

int VcGetOBJID_Size(VcValue* value)
{
  if (value == NULL)
    return 0;

  return VcListLength(value->value.list);
}

int VcGetOBJID_Component(VcValue* oid, unsigned int index)
{
  VcValue* tmp;

  if ((oid == NULL) || (index == 0)) /* Index starts at 1. */
    return -1;

  tmp = VcListGetValue(oid->value.list, index - 1);

  return (tmp == NULL) ? -1 : VcGetINTEGER(tmp); 
}

/*--------------------------------------------------------------------
 * Operations on VcValues with BOOLEAN content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkBOOLEAN
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkBOOLEAN(VcBaseVal base_value)
{
  VcValue *value = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkBOOLEAN");

  value->type          = VcBOOLEAN;
  value->attribute     = VcEMPTYATTR;
  value->value.integer = (base_value == 0) ? FALSE : TRUE;

  return value;
}

/*--------------------------------------------------------------------
 * Name:   VcGetBOOLEAN
 * Author: ITEX C-Code Generator
 * 
 */

VcBaseVal VcGetBOOLEAN(VcValue * value)
{
  if (!value) {
    VcWarning("VcGetBOOLEAN", "<NULL> value");
    return 0;
  }
  if (VcIsSimple(VcGetType(value)))
    return (value->value.integer != 0);
  else {
    VcWarning("VcGetBOOLEAN", "trying to get BOOLEAN value from non-BOOLEAN");
    return 0;
  }
}

/*--------------------------------------------------------------------
 * Operations on VcValues with INTEGER content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkINTEGER
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkINTEGER(VcBaseVal value)
{
  VcValue *tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkINTEGER");

  tmp->type          = VcINTEGER;
  tmp->attribute     = VcEMPTYATTR;
  tmp->value.integer = value;

  return tmp;
}

/*--------------------------------------------------------------------
 * Name:   VcGetINTEGER
 * Author: ITEX C-Code Generator
 * 
 */

VcBaseVal VcGetINTEGER(VcValue * value)
{
  if (!value) {
    VcWarning("VcGetINTEGER", "<NULL> value");
    return 0;
  }
  if (VcIsSimple(VcGetType(value)))
    return value->value.integer;
  else {
    VcWarning("VcGetINTEGER", "trying to get integer value from non-integer");
    return 0;
  }
}

/*--------------------------------------------------------------------
 * Operations on VcValues with *STRING content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkOCTET_STRING
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkOCTET_STRING(VcString value)
{
  VcValue *tmp = (VcValue *)
  VCMALLOC(sizeof(VcValue), "VcMkOCTET_STRING");

  tmp->type         = VcOCTET_STRING;
  tmp->attribute    = VcEMPTYATTR;
  tmp->value.string = (VcString) VCMALLOC(sizeof(char) * (Vcstrlen(value) + 1), "VcMkOCTET_STRING");

  Vcstrcpy(tmp->value.string, value);

  return tmp;
}

/*--------------------------------------------------------------------
 * Name:   VcGetOCTET_STRING
 * Author: ITEX C-Code Generator
 * 
 */

VcString VcGetOCTET_STRING(VcValue * value)
{
  if (!value) {
    VcWarning("VcGetOCTET", "<NULL> value");
    return NULL;
  }
  return value->value.string;
}

/*--------------------------------------------------------------------
 * Name:   VcMkCSTRING
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkCSTRING(VcString value)
{
  VcValue *tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkCSTRING");

  tmp->type         = VcIA5String;
  tmp->attribute    = VcEMPTYATTR;
  tmp->value.string = (VcString) VCMALLOC(sizeof(char) * (Vcstrlen(value) + 1), "VcMkCSTRING");

  Vcstrcpy(tmp->value.string, value);

  return tmp;
}

/*--------------------------------------------------------------------
 * Name:   VcGetCSTRING
 * Author: ITEX C-Code Generator
 * 
 */

VcString VcGetCSTRING(VcValue * value)
{
  if (!value) {
    VcWarning("VcGetCSTRING", "<NULL> value");
    return NULL;
  }
  if (VcIsString(VcGetType(value)))
    return value->value.string;
  else {
    VcWarning("VcGetCSTRING", "Trying to get CSTRING-value from non-STRING");
    return NULL;
  }
}

/*--------------------------------------------------------------------
 * Name:   VcMkBIT_STRING
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkBIT_STRING(VcString value)
{
  VcValue *tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkBIT_STRING");

  tmp->type         = VcBIT_STRING;
  tmp->attribute    = VcEMPTYATTR;
  tmp->value.string = (VcString)
    VCMALLOC(sizeof(char) * (Vcstrlen(value) + 1), "VcMkBIT_STRING");

  Vcstrcpy(tmp->value.string, value);

  return tmp;
}

/*--------------------------------------------------------------------
 * Name:   VcGetBIT_STRING
 * Author: ITEX C-Code Generator
 * 
 */

VcString VcGetBIT_STRING(VcValue * value)
{
  if (!value) {
    VcWarning("VcGetBIT", "<NULL> value");
    return NULL;
  }
  return VcGetCSTRING(value);
}

/*--------------------------------------------------------------------
 * Name:   VcMkHEXSTRING
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkHEXSTRING(VcString value)
{
  VcValue *tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkHEXSTRING");

  tmp->type         = VcHEXSTRING;
  tmp->attribute    = VcEMPTYATTR;
  tmp->value.string = (VcString) VCMALLOC(sizeof(char) * (Vcstrlen(value) + 1), "VcMkHEXSTRING");

  Vcstrcpy(tmp->value.string, value);

  return tmp;
}

/*--------------------------------------------------------------------
 * Name:   VcGetHEXSTRING
 * Author: ITEX C-Code Generator
 * 
 */

VcString VcGetHEXSTRING(VcValue * value)
{
  if (!value) {
    VcWarning("VcGetHEXSTRING", "<NULL> value");
    return NULL;
  }
  return VcGetCSTRING(value);
}

/*--------------------------------------------------------------------
 * Operations on VcValues with R_TYPE content.
 */

/*-------------------------------------------------------------------
 * Name:   VcMkR_TYPE
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkR_TYPE(VcBaseVal value)
{
  VcValue *tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkR_TYPE");

  tmp->type          = VcR_TYPE;
  tmp->attribute     = VcEMPTYATTR;
  tmp->value.integer = value;

  return tmp;
}

/*--------------------------------------------------------------------
 * Operations on VcValues with REAL content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkREAL
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkREAL(VcBaseVal mantissa, VcBaseVal base, VcBaseVal exponent)
{
  VcValue *tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkREAL");

  tmp->type      = VcREAL;
  tmp->attribute = VcEMPTYATTR;

  /* Allocate space and copy Value. */

  tmp->value.real = (VcReal *) VCMALLOC(sizeof(VcReal), "VcMkREAL");

  tmp->value.real->mantissa = mantissa;
  tmp->value.real->base = base;
  tmp->value.real->exponent = exponent;

  return tmp;
}

/*--------------------------------------------------------------------
 * Name:   VcGetREAL
 * Author: ITEX C-Code Generator
 * 
 */

VcReal * VcGetREAL(VcValue * value)
{
  if (!value) {
    VcWarning("VcGetREAL", "<NULL> value");
    return NULL;
  }
  if (VcIsReal(VcGetType(value)))
    return value->value.real;
  else {
    VcWarning("VcGetREAL", "trying to get REAL-value from non-REAL");
    return NULL;
  }
}

/*--------------------------------------------------------------------
 * Operations on VcValues with PDU content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkPDU
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkPDU(VcValue * value)
{
  return VcSetType(VcPDU, value);
}

/*--------------------------------------------------------------------
 * Name:   VcGetPDU
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcGetPDU(VcValue * value)
{
  return value;
}

/*--------------------------------------------------------------------
 * Operations on VcValues with NULL content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkNULL
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkNULL(void)
{
  VcValue *tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkNULL");

  tmp->type          = VcNULL;
  tmp->attribute     = VcEMPTYATTR;
  tmp->value.integer = 0;

  return tmp;
}

/*--------------------------------------------------------------------
 * Operations on VcValues with ANY content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkANY
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkANY(VcValue * value)
{
  return VcSetType(VcANY, value);
}

/*--------------------------------------------------------------------
 * Name:   VcGetANY
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcGetANY(VcValue * value)
{
  return value;
}

/*--------------------------------------------------------------------
 * Operations on VcValues with CHOICE content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkCHOICE
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkCHOICE(VcValue * value, VcString name)
{
  VcValue *tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkCHOICE");

  tmp->type         = VcCHOICE;
  tmp->attribute    = VcEMPTYATTR;

  VcMkCHOICEContents( tmp, name, NULL);
  tmp->value.choice->value = value;

  if (value != NULL)
    value->is_choice_element = TRUE;

  return tmp;
}


/*--------------------------------------------------------------------
 * Name:   VcMkCHOICEContents
 * Author: ITEX C-Code Generator
 * 
 */

VcValue *VcMkCHOICEContents(VcValue* cValue, VcString name, VcType* type) 
{
  VcChoice* choice;
  VcMallocType mt;
  
  if (!cValue) {
    VcWarning("VcMkCHOICEContents", "<NULL> value");
    return NULL;
  }

  if ((Vcstrcmp(name,"") == 0) && (type != NULL))
    VcWarning( "VcMkCHOICEContents", "creating typed unnamed value");

  /* Use the correct memory model. */
  mt = VcGetAlloc();
  if (VcIsTemporary(cValue))
    VcSetTempAlloc();
  else
    VcSetPermAlloc();
  
  /* Create the VcChoice container. */
  if (cValue->value.choice == NULL) 
    choice = (VcChoice *) VCMALLOC(sizeof(VcChoice), "VcMkCHOICEContents");      
  else 
    choice = cValue->value.choice;

  choice->name = name;
  choice->value = NULL;

  cValue->value.choice = choice;
  
  /* Create a new value of type 'type' if wanted.*/
  if (type != NULL) 
  {
    VcValue*  newValue;
    
    newValue = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkCHOICEContents");

    newValue->type              = type;
    newValue->attribute         = VcEMPTYATTR;
    newValue->is_choice_element = TRUE;

    cValue->value.choice->value = newValue;
  }

  VcSetMalloc( mt );

  return cValue;
}

/*--------------------------------------------------------------------
 * Name:   VcGetCHOICE
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcGetCHOICE(VcString name, VcType* type, VcValue * value)
{
  if (!value) {
    VcWarning("VcGetCHOICE", "<NULL> value");
    return NULL;
  }

  if (VcIsChoice(VcGetType(value)))
  {
    if (type == NULL)
    {
      if (value->value.choice != NULL)
        return value->value.choice->value;
      else
        return NULL;
    }

    if (value->value.choice != NULL) /* An initialized choice. */
    {
      if (Vcstrcmp(value->value.choice->name, name) == 0)
        return value->value.choice->value;
      else /* We request another choice element. Remove the value.*/
      {
        VcRmValue(value->value.choice->value);
        value->value.choice->value = NULL;
      }
    }

    /* Make a new choice contents with element named 'name' and value of 
     * type 'type'. 
     */
    VcMkCHOICEContents( value, name, type); 

    if (value->value.choice->value == NULL)
      VcWarning( "VcGetCHOICE", "Bad value returned!");

    return value->value.choice->value;
  }
  else {
    VcWarning("VcGetCHOICE", "trying to get CHOICE-value from non-CHOICE");
    return NULL;
  }
}

/*--------------------------------------------------------------------
 * Name:   VcGetCHOICEName
 * Author: ITEX C-Code Generator
 * 
 */

VcString VcGetCHOICEName(VcValue * value)
{
  if (!value) {
    VcWarning("VcGetCHOICEName", "<NULL> value");
    return NULL;
  }
  if (VcIsChoice(VcGetType(value)))
  {
    if (value->value.choice != NULL)
      return value->value.choice->name;
    else
      return "";
  }
  else {
    VcWarning("VcGetCHOICEName", "trying to get name from non-CHOICE");
    return NULL;
  }
}

/*--------------------------------------------------------------------
 * Operations on VcValues with TIME content.
 */

/*--------------------------------------------------------------------
 * Name:   VcMkUTCTime
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkUTCTime(VcString value)
{
  VcValue *tmp = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkUTCTime");

  tmp->type      = VcUTCTime;
  tmp->attribute = VcEMPTYATTR;

  /* Allocate space and copy Value. */

  tmp->value.string = (VcString)
    VCMALLOC(sizeof(char) * (Vcstrlen(value) + 1), "VcUTCTime");

  Vcstrcpy(tmp->value.string, value);

  return tmp;
}

/*--------------------------------------------------------------------
 * Name:   VcMkGeneralizedTime
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkGeneralizedTime(VcString value)
{
  VcValue *tmp =
  (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkGeneralizedTime");

  tmp->type      = VcGeneralizedTime;
  tmp->attribute = VcEMPTYATTR;

  /* Allocate space and copy Value. */

  tmp->value.string = (VcString)
    VCMALLOC(sizeof(char) * (Vcstrlen(value) + 1), "VcMkGeneralizedTime");

  Vcstrcpy(tmp->value.string, value);

  return tmp;
}

/*--------------------------------------------------------------------
 * Operations on "Instead Of Value" VcValues.
 */

/*--------------------------------------------------------------------
 * Name:   VcIsOmitable
 * Author: ITEX C-Code Generator
 * 
 */

static 
Bool VcIsOmitable(VcValue * value)
{

  /* If the value is NULL it is OMITed. Return TRUE. This case should
   * only occur for value constraints, never for reference
   * constraints. */

  if (value == NULL)
    return TRUE;

  /* If the value has an attribute set that allows it to be OMITed it
   * is definitly OMITable, otherwise it isn't. */
  
  if (VcValueAttribute(value) & (VcOMITATTR | VcANYORNONEATTR | VcIF_PRESENTATTR))
    return TRUE;
  else
    return FALSE;
}

/*--------------------------------------------------------------------
 * Name:   VcMkOmit
 * Author: ITEX C-Code Generator
 * */

VcValue * VcMkOmit(VcType * type)
{
  VcValue *value;

  value = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkOmit");

  value->type          = type;
  value->attribute     = VcOMITATTR;
  value->value.integer = 0;

  return value;
}

/*--------------------------------------------------------------------
 * Name:   VcMkAnyOne
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkAnyOne(VcType * type)
{
  VcValue *value;

  value = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkAnyOne");

  value->type          = type;
  value->attribute     = VcANYONEATTR;
  value->value.integer = 0;

  return value;
}

/*--------------------------------------------------------------------
 * Name:   VcMkAnyOrNone
 * Author: ITEX C-Code Generator
 * 
 */

VcValue * VcMkAnyOrNone(VcType * type)
{
  VcValue *value;

  value = (VcValue *) VCMALLOC(sizeof(VcValue), "VcMkAnyOrNone");

  value->type          = type;
  value->attribute     = VcANYORNONEATTR;
  value->value.integer = 0;

  return value;
}

/*--------------------------------------------------------------------
 * The functions that implement all the predefined operators for
 * the TTCN types.
 */

/*--------------------------------------------------------------------
 * Predeined Arithmetic Operators.
 * 
 * These are only defined for INTEGER values.  They all return an
 * INTEGER value.
 */

/*--------------------------------------------------------------------
 * Name:   VcNegate
 * Author: ITEX C-Code Generator
 *
 * Unary Arithmetic Operator.
 *
 * Allocates a new VcValue and stores the negated value of the
 * argument in it.  The new value is then returned.
 *
 * This function is only defined on INTEGER and REAL types.
 */

VcValue * VcNegate(VcValue * argument)
{
  if (argument == NULL) {
    VcWarning("VcNegate", "<NULL> argument.");
    return VcMkINTEGER(0);
  }
  if (VcGetBaseType(argument->type) != VcINTEGERD) {
    VcWarning("VcNegate", "type error");
    return VcMkINTEGER(0);
  }
  return VcMkINTEGER(-VcGetINTEGER(argument));
}

/*--------------------------------------------------------------------
 * Name:   VcAdd
 * Author: ITEX C-Code Generator
 *
 * Arithmetic binary operator.
 *
 * Adds the two values given together.  The resulting value is
 * allocated here and then returned.
 *
 * This operation is defined for INTEGERs only.
 */

VcValue * VcAdd(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcINTEGERD ||
      VcGetBaseType(VcGetType(right)) != VcINTEGERD) {
    VcWarning("VcAdd", "type error");
    return VcMkINTEGER(0);
  }
  return VcMkINTEGER(VcGetINTEGER(left) + VcGetINTEGER(right));
}

/*--------------------------------------------------------------------
 * Name:   VcSubtract
 * Author: ITEX C-Code Generator
 *
 * Arithmetic binary operator.
 *
 * Subtracts the second argument from the first and places the
 * result into a newly constructed value.  The new value is then
 * returned.
 *
 * This operator is defined for the INTEGER type.
 */

VcValue * VcSubtract(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcINTEGERD ||
      VcGetBaseType(VcGetType(right)) != VcINTEGERD) {
    VcWarning("VcSubtract", "type error");
    return VcMkINTEGER(0);
  }
  return VcMkINTEGER(VcGetINTEGER(left) - VcGetINTEGER(right));
}

/*--------------------------------------------------------------------
 * Name:   VcMultiply
 * Author: ITEX C-Code Generator
 *
 * Arithmetic binary operator.
 *
 * Multiplies the arguments with eachother and places the result
 * into a newly constructed value.  The new value is then
 * returned.
 *
 * This operator is defined for the INTEGER type.
 */

VcValue * VcMultiply(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcINTEGERD ||
      VcGetBaseType(VcGetType(right)) != VcINTEGERD) {
    VcWarning("VcMultiply", "type error");
    return VcMkINTEGER(0);
  }
  return VcMkINTEGER(VcGetINTEGER(left) * VcGetINTEGER(right));
}

/*--------------------------------------------------------------------
 * Name:   VcDivide
 * Author: ITEX C-Code Generator
 *
 * Arithmetic binary operator.
 *
 * Divides the first argument with the second and places the
 * result into a newly constructed value.  The new value is then
 * returned.
 *
 * This operator is defined for the INTEGER type.
 */

VcValue * VcDivide(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcINTEGERD ||
      VcGetBaseType(VcGetType(right)) != VcINTEGERD) {
    VcWarning("VcDivide", "type error");
    return VcMkINTEGER(0);
  }
  if (VcGetINTEGER(right) == 0) {
    VcWarning("VcDivide", "division by zero");
    return VcMkINTEGER(0);
  }
  return VcMkINTEGER(VcGetINTEGER(left) / VcGetINTEGER(right));
}

/*--------------------------------------------------------------------
 * Name:   VcModulo
 * Author: ITEX C-Code Generator
 *
 * Arithmetic binary operator.
 *
 * Calculates the modulo of the arguments and places the result
 * into a newly constructed value.  The new value is then
 * returned.
 *
 * This operator is defined for the INTEGER type.
 */

VcValue * VcModulo(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcINTEGERD ||
      VcGetBaseType(VcGetType(right)) != VcINTEGERD) {
    VcWarning("VcModulo", "type error");
    return VcMkINTEGER(0);
  }
  if (VcGetINTEGER(right) == 0) {
    VcWarning("VcModulo", "modulo by zero");
    return VcMkINTEGER(0);
  }
  return VcMkINTEGER(VcGetINTEGER(left) % VcGetINTEGER(right));
}

/*--------------------------------------------------------------------
 * Relational operators.
 */

/*--------------------------------------------------------------------
 * Name:   VcEQ
 * Author: ITEX C-Code Generator
 *
 * Relational binary operator.
 *
 * Determines if the arguments given are equal.  Returns a newly
 * built BOOLEAN value that is set to TRUE if they are equal,
 * FALSE otherwise.
 *
 * This operator is defined for all types.  The only provision is
 * that the arguments contain specific values, i.e. no wildcards.
 * This does not include wildcards within string values.  They
 * are allowed and should be compared using the normal
 * string-matching rules.
 */

VcValue * VcEQ(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcGetBaseType(VcGetType(right))) {
    VcWarning("VcEQ", "type error");
    return VcMkBOOLEAN(FALSE);
  }
  return VcMkBOOLEAN(VcMatchValue(left, right));
}

/*--------------------------------------------------------------------
 * Name:   VcNEQ
 * Author: ITEX C-Code Generator
 *
 * Relational binary operator.
 *
 * Determines if the arguments given are unequal.  Returns a
 * newly built BOOLEAN value that is set to FALSE if they are
 * equal and TRUE otherwise.
 *
 * This operator is defined for all types.  The only provision is
 * that the arguments contain specific values, i.e. no wildcards.
 * This does not include wildcards within string values.  They
 * are allowed and should be compared using the normal
 * string-matching rules.
 */

VcValue * VcNEQ(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcGetBaseType(VcGetType(right))) {
    VcWarning("VcNEQ", "type error");
    return VcMkBOOLEAN(FALSE);
  }
  return VcMkBOOLEAN(!VcMatchValue(left, right));
}

/*--------------------------------------------------------------------
 * Name:   VcGT
 * Author: ITEX C-Code Generator
 *
 * Relational binary operator.
 *
 * Determines if the first argument is greater than the second
 * argument.  Returns a newly built BOOLEAN value that is set to
 * TRUE if the first argument is greater to the second argument
 * and FALSE otherwise.
 *
 * This operator is defined for the INTEGER type.
 */

VcValue * VcGT(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcINTEGERD ||
      VcGetBaseType(VcGetType(right)) != VcINTEGERD) {
    VcWarning("VcGT", "type error");
    return VcMkBOOLEAN(FALSE);
  }
  return VcMkBOOLEAN(VcGetINTEGER(left) > VcGetINTEGER(right));
}

/*--------------------------------------------------------------------
 * Name:   VcLTN
 * Author: ITEX C-Code Generator
 *
 * Relational binary operator.
 *
 * Determines if the first argument is less than the second
 * argument.  Returns a newly built BOOLEAN value that is set to
 * TRUE if the first argument is less to the second argument and
 * FALSE otherwise.
 *
 * This operator is defined for the INTEGER type.
 */

VcValue * VcLTN(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcINTEGERD ||
      VcGetBaseType(VcGetType(right)) != VcINTEGERD) {
    VcWarning("VcLTN", "type error");
    return VcMkBOOLEAN(FALSE);
  }
  return VcMkBOOLEAN(VcGetINTEGER(left) < VcGetINTEGER(right));
}

/*--------------------------------------------------------------------
 * Name:   VcGEQ
 * Author: ITEX C-Code Generator
 *
 * Relational binary operator.
 *
 * Determines if the first argument is greater than or equal to
 * the second argument.  Returns a newly built BOOLEAN value that
 * is set to TRUE if the first argument is greater than or equal
 * to the second argument and FALSE otherwise.
 *
 * This operator is defined for the INTEGER type.
 */

VcValue * VcGEQ(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcINTEGERD ||
      VcGetBaseType(VcGetType(right)) != VcINTEGERD) {
    VcWarning("VcGEQ", "type error");
    return VcMkBOOLEAN(FALSE);
  }
  return VcMkBOOLEAN(VcGetINTEGER(left) >= VcGetINTEGER(right));
}

/*--------------------------------------------------------------------
 * Name:   VcLEQ
 * Author: ITEX C-Code Generator
 *
 * Relational binary operator.
 *
 * Determines if the first argument is less than or equal to the
 * second argument.  Returns a newly built BOOLEAN value that is
 * set to TRUE if the first argument is less than or equal to the
 * second argument and FALSE otherwise.
 *
 * This operator is defined for the INTEGER type.
 */

VcValue * VcLEQ(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcINTEGERD ||
      VcGetBaseType(VcGetType(right)) != VcINTEGERD) {
    VcWarning("VcLEQ", "type error");
    return VcMkBOOLEAN(FALSE);
  }
  return VcMkBOOLEAN(VcGetINTEGER(left) <= VcGetINTEGER(right));
}

/*--------------------------------------------------------------------
 * Logic operators.
 */

/*--------------------------------------------------------------------
 * Name:   VcNOT
 * Author: ITEX C-Code Generator
 *
 * Logic operator.
 *
 * Allocates a new VcValue and stores the logical complement of
 * the given argument in it.  The new value is then returned.
 *
 * This function is only defined on BOOLEAN types.
 */

VcValue * VcNOT(VcValue * argument)
{
  if (VcGetBaseType(VcGetType(argument)) != VcBOOLEAND) {
    VcWarning("VcNOT", "type error");
    return VcMkBOOLEAN(FALSE);
  }
  return VcMkBOOLEAN(VcGetBOOLEAN(argument) ? FALSE : TRUE);
}

/*--------------------------------------------------------------------
 * Name:   VcOR
 * Author: ITEX C-Code Generator
 *
 * Binary logic operator.
 *
 * Calculates the locical OR of the arguments and places the
 * result into a newly constructed value.  The new value is then
 * returned.
 *
 * This operator is defined for the BOOLEAN type.
 */

VcValue * VcOR(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcBOOLEAND ||
      VcGetBaseType(VcGetType(right)) != VcBOOLEAND) {
    VcWarning("VcOR", "type error");
    return VcMkBOOLEAN(FALSE);
  }
  return VcMkBOOLEAN(VcGetBOOLEAN(left) || VcGetBOOLEAN(right));
}

/*--------------------------------------------------------------------
 * Name:   VcAND
 * Author: ITEX C-Code Generator
 *
 * Logic binary operator.
 *
 * Calculates the locical AND of the arguments and places the
 * result into a newly constructed value.  The new value is then
 * returned.
 *
 * This operator is defined for the BOOLEAN type.
 */

VcValue * VcAND(VcValue * left, VcValue * right)
{
  if (VcGetBaseType(VcGetType(left)) != VcBOOLEAND ||
      VcGetBaseType(VcGetType(right)) != VcBOOLEAND) {
    VcWarning("VcAND", "type error");
    return VcMkBOOLEAN(FALSE);
  }
  return VcMkBOOLEAN(VcGetBOOLEAN(left) && VcGetBOOLEAN(right));
}

/*--------------------------------------------------------------------
 * Functions implementing the predefined operations in TTCN.
 */

/*--------------------------------------------------------------------
 * Name:   VcRev
 * Author: ITEX C-Code Generator
 *
 * Internal function used by the INT_TO_BIT function.
 */

static void VcRev(VcString wrong, VcString right)
{
  int i, length;
  
  if (wrong == NULL) {
    VcWarning("VcRev", "<NULL> argument.");
    return;
  }
    
  length = Vcstrlen(wrong);

  for (i = 0; i < length; i++)
    right[length - i - 1] = wrong[i];
  right[length] = '\0';
}

/*--------------------------------------------------------------------
 * Name:   VcINT_TO_BIT
 * Author:
 *
 * TTCN function converting an INTEGER Value to a BITSTRING 
 *
 * Note: This function uses a static buffer.
 */

VcValue * VcINT_TO_BIT(VcValue * Val, VcValue * length)
{
  VcString TmpString = (VcString) 
    VCMALLOC((VcGetINTEGER(length) + 1) * sizeof(char), "VcINT_TO_BIT");
  VcString String    = (VcString) 
    VCMALLOC((VcGetINTEGER(length) + 1) * sizeof(char), "VcINT_TO_BIT");
  int TmpVal = VcGetINTEGER(Val);
  int TmpLen = VcGetINTEGER(length);
  
  Vcstrcpy(TmpString, "");

  while (TmpLen-- > 0) {
    if ((TmpVal % 2) == 0)
      Vcstrcat(TmpString, "0");
    else
      Vcstrcat(TmpString, "1");
    TmpVal = TmpVal / 2;
  }
  
  VcRev(TmpString, String);
  VCFREE(TmpString, Vcstrlen(TmpString), "VcINT_TO_BIT");
  
  return VcMkBIT_STRING(String);
}

/*--------------------------------------------------------------------
 * Name:   VcINT_TO_HEX
 * Author: ITEX C-Code Generator 
 *
 * TTCN function converting an INTEGER Value to a HEXSTRING.
 */

VcValue * VcINT_TO_HEX(VcValue * integer, VcValue * length)
{
  char format[256];
  VcString str;
  VcValue *result;

  /* Here we build the format string used in the call to sprintf.
   * We have to build it explicitly since we have a variable
   * precision field that has to be changed each time this
   * function is called. */

  Vcstrcpy(format, "%.");
  sprintf(format + 2, "%i", (int) VcGetINTEGER(length));
  Vcstrcat(format, "X");

  /* Now we allocate memory for the hex-string and perform the
   * conversion to a hex-string. */

  str = (VcString)
    VCMALLOC(sizeof(char) * (VcGetINTEGER(length) + 1), "VcINT_TO_HEX");
  sprintf(str, format, VcGetINTEGER(integer));
  result = VcMkHEXSTRING(str);
  VCFREE(str, Vcstrlen(str) + 1, "VcINT_TO_HEX");

  return result;
}

/*--------------------------------------------------------------------
 * Name:   VcHEX_TO_INT
 * Author: ITEX C-Code Generator 
 *
 * TTCN function converting a hex Value to an INTEGER Value
 */

VcValue * VcHEX_TO_INT(VcValue * value)
{
  int v;

  sscanf(VcGetHEXSTRING(value), "%x", &v);
  return VcMkINTEGER(v);
}

/*--------------------------------------------------------------------
 * Name:   VcBIT_TO_INT
 * Author: ITEX C-Code Generator
 *
 * TTCN function converting a BITSTRING to an INTEGER Value
 */

VcValue * VcBIT_TO_INT(VcValue * Val)
{
  VcString tmpString = VcGetBIT_STRING(Val);
  unsigned int v, i;

  for (v = 0, i = 0; i < Vcstrlen(tmpString); ++i)
    v = v * 2 + tmpString[i] - '0';
  return VcMkINTEGER(v);
}

/*--------------------------------------------------------------------
 * Name:   VcLENGTH_OF
 * Author: ITEX C-Code Generator
 *
 * TTCN function returning the length of a string.
 */

VcValue * VcLENGTH_OF(VcValue * string)
{
  char *cstring;
  int length;

  if (!VcIsString(VcGetType(string))) {
    VcWarning("VcLENGTH_OF", "non-string value");
    return VcMkINTEGER(0);
  }

  cstring = VcGetCSTRING(string);

  if (!VcIsOCTET_STRING(VcGetType(string)))
    length = Vcstrlen(cstring);
  else {
    for (length = 0; *cstring; ++cstring) {
      if (*cstring == '?')
	length++;
      length++;
    }
    length = length/2;
  }

  return VcMkINTEGER(length);
}

/*--------------------------------------------------------------------
 * Name:   VcNUMBER_OF_ELEMENTS
 * Author: ITEX C-Code Generator
 *
 * TTCN function for retreiving the number of elements in a data
 * object Reference, whVch should be a SEQUENCE OF or a SET OF
 */

VcValue * VcNUMBER_OF_ELEMENTS(VcValue * value)
{
  return VcMkINTEGER(VcListLength(VcGetList(value)));
}

/*--------------------------------------------------------------------
 * Name:   VcIS_PRESENT
 * Author: ITEX C-Code Generator 
 *
 * TTCN function returning TRUE if an OPTIONAL or DEFAULT value
 * Reference is present in a received PDU, false otherwise.
 */

VcValue * VcIS_PRESENT(VcValue * value)
{
  VcMessage("VcIS_PRESENT", "NOT IMPLEMENTED");
  return VcMkBOOLEAN(value != NULL);    /* NOT IMPLEMENTED */
}

/*--------------------------------------------------------------------
 * Name:   VcIS_CHOSEN
 * Author: ITEX C-Code Generator 
 *
 * TTCN function used to indicate that we wich to accept a
 * particular element in a choice. It returns true if the data
 * object reference, which must be of CHOICE Type, matches the
 * received Value.
 */

VcValue * VcIS_CHOSEN(VcValue * dataObject)
{
  VcMessage("VcIS_CHOSEN", "NOT IMPLEMENTED");
  return VcMkBOOLEAN(TRUE);     /* NOT IMPLEMENTED. */
}

/*--------------------------------------------------------------------
 * Functions responsible for assignment of values.
 */

/*--------------------------------------------------------------------
 * Name:   VcAssignInteger
 * Author: ITEX C-Code Generator
 *
 */

static void VcAssignInteger(VcValue * left, VcValue * right)
{
#ifdef DEBUG_VALREP
  fprintf(stderr, "ASSIGN: %p = %ld -> %ld.\n", left, left->value.integer,
	  right->value.integer);
#endif
  left->value.integer = right->value.integer;
  left->attribute     = right->attribute;

  if (left->type == NULL)
    left->type = right->type;
}

/*--------------------------------------------------------------------
 * Name:   VcAssignString
 * Author: ITEX C-Code Generator
 *
 */

static void VcAssignString(VcValue * left, VcValue * right)
{
  char *lstr = VcGetCSTRING(left);
  char *rstr = VcGetCSTRING(right);

  if (lstr != NULL) {
    /* NULL if not assigned defaultvalue */
    VCFREE(lstr, Vcstrlen(lstr) + 1, "VcAssign");
  }

  lstr = (char *) VCMALLOC(sizeof(char) * (Vcstrlen(rstr) + 1), "VcAssign");
  Vcstrcpy(lstr, rstr);
  left->value.string = lstr;

  left->attribute = right->attribute;

  if (VcGetType(left) == NULL)
    VcSetType(VcGetType(right), left);
}

/*--------------------------------------------------------------------
 * Name:   VcAssignSequence
 * Author: ITEX C-Code Generator
 *
 */

static void VcAssignSequence(VcValue * left, VcValue * right)
{
  VcValue *tmp = VcCopy(right);
  VcValue *seq = VcMkINTEGER(0);

  *seq = *left;
  VcRmValue(seq);

  left->value     = tmp->value;
  left->attribute = right->attribute;

  if (left->type == NULL)
    left->type = right->type;
}

/*--------------------------------------------------------------------
 * Name:   VcAssignOBJECT_IDENTIFIER
 * Author: ITEX C-Code Generator
 *
 */

static void VcAssignOBJECT_IDENTIFIER(VcValue * left, VcValue * right)
{
  VcValue *tmp = VcCopyOBJECT_IDENTIFIER(right);

  if (left == NULL) 
    return;

  left->type        = right->type;
  left->attribute   = right->attribute;

  if (left->value.list != NULL)
    VcListDelete(left->value.list);

  left->value.list  = VcListCopy(right->value.list);
}

/*--------------------------------------------------------------------
 * Name:   VcAssignReal
 * Author: ITEX C-Code Generator
 *
 */

static void VcAssignReal(VcValue * left, VcValue * right)
{
  if (right->value.real == NULL)
  {
    VcWarning("VcAssignReal", "uninitialized r-value");
    return;
  }

  if (left->value.real == NULL) /* L-value uninitialized */ 
    left->value.real = (VcReal *) VCMALLOC(sizeof(VcReal), "VcCopyReal");

  left->value.real->mantissa = right->value.real->mantissa;
  left->value.real->base = right->value.real->base;
  left->value.real->exponent = right->value.real->exponent;

  left->attribute = right->attribute;

  if (left->type == NULL)
    left->type = right->type;
}

/*--------------------------------------------------------------------
 * Name:   VcAssignChoice
 * Author: ITEX C-Code Generator
 *
 */

static void VcAssignChoice(VcValue * left, VcValue * right)
{
  if (right->value.choice == NULL)
  {
    VcWarning("VcAssignChoice", "uninitialized r-value");
    return;
  }

  left->attribute = right->attribute;

  if (left->value.choice == NULL) /* l-value unitialized */
    left->value.choice = (VcChoice *) VCMALLOC(sizeof(VcChoice), "VcAssignChoice");
 
  left->value.choice->name = right->value.choice->name;
  VcRmValue(VcGetCHOICE("", NULL, left));
  
  if (VcGetCHOICE("", NULL, right) != NULL)
      left->value.choice->value = VcCopy(VcGetCHOICE("", NULL, right));

  if (left->value.choice->value != NULL)
    left->value.choice->value->is_choice_element = TRUE;
  else
    VcWarning("VcAssignChoice", "<NULL> value");

  if (VcGetType(left) == NULL)
    VcSetType(VcGetType(right), left);
}

/*--------------------------------------------------------------------
 * Name:   VcAssign
 * Author: ITEX C-Code Generator
 *
 * Description:
 *  The value in obj2 is copyed into Obj1.  Flag is either of:
 *    1 (Permanent memory)
 *    0 (Temporary memory)
 */

void VcAssign(VcValue * left, VcValue * right, int Mem, char *name)
{
  int base_type;
  VcMallocType current;

  /* Sanity check. */

  if (left == right)
    return;
  if ((left == NULL) || (right == NULL)) 
  {
    VcWarning("VcAssign", "Either operand (or both) is <NULL>");
    return;
  }

  /* Check if the type of the value we are assigning to has a
   * type restriction, and if so, that the type of the value we
   * are assigning falls within the restriction. */

  if (VcRestrict(VcGetType(left), right) == FALSE) {
    IcLog(GciLogAssign, "Assignment ignored.  Restriction violation");
    return;
  }

  /* Get the type we are assigning to.  If the value we are
   * assigning to has no type we assume the type of the value we
   * are assigning from.  If neither has a type, return without
   * doing anything. 
   * If we have a value that is an element of a choice, ignore the l-value
   * type and use the r-value type as basetype.
   */

  if (left->is_choice_element)
    base_type = VcGetBaseType(VcGetType(right));
  else if (VcGetType(left) != NULL)
    base_type = VcGetBaseType(VcGetType(left));
  else if (VcGetType(right) != NULL)
    base_type = VcGetBaseType(VcGetType(right));
  else {
    IcLog(GciLogAssign, "Assignment ignored. No R-Value.");
    return;
  }

  /* Set the memory allocation strategy to the indicated one. */

  current = VcGetAlloc();
  VcSetMalloc(Mem);

  switch (base_type) {
    case VcBOOLEAND:
    case VcINTEGERD:
    case VcENUMERATEDD:
      VcAssignInteger(left, right);
      break;
    case VcNULLD:
      VcAssignInteger(left, right);
      break;
    case VcNumericStringD:
    case VcPrintableStringD:
    case VcTeletexStringD:
    case VcVideotexStringD:
    case VcIA5StringD:
    case VcUTCTimeD:
    case VcGeneralizedTimeD:
    case VcGraphicStringD:
    case VcVisibleStringD:
    case VcISO646StringD:
    case VcT61StringD:
    case VcGeneralStringD:
    case VcObjectDescriptorD:
    case VcHEXSTRINGD:
    case VcBIT_STRINGD:
    case VcOCTET_STRINGD:
      VcAssignString(left, right);
      break;
    case VcREALD:
      VcAssignReal(left, right);
      break;
    case VcCHOICED:
      VcAssignChoice(left, right);
      break;
    case VcOBJECT_IDENTIFIERD:
      VcAssignOBJECT_IDENTIFIER(left, right);
      break;
    case VcSEQUENCED:
    case VcSETD:
    case VcSEQUENCE_OFD:
    case VcSET_OFD:
      VcAssignSequence(left, right);
      break;
    default:
      VcWarning("VcAssign", "unknown type");
      break;
    }

  VcLogAssign(name, right);
  VcSetMalloc(current);
}

/*--------------------------------------------------------------------
 * Copy related functions.
 *
 * These functions are responsible for making a deep copy of a
 * VcValue of an arbitrary type.
 */

static
VcValue *VcCopyVALUERANGEATTR(VcValue *value)
{
  VcValue *copy = NULL;

  copy            = VcMkSEQUENCE(2);
  copy->type      = VcGetType(value);
  copy->attribute = VcValueAttribute(value);

  VcListSetValue(VcGetList(copy), 0, VcCopy(VcListGetValue(VcGetList(value), 0)));
  VcListSetValue(VcGetList(copy), 1, VcCopy(VcListGetValue(VcGetList(value), 1)));

  return copy;
}

static
VcValue *VcCopyVALUELISTATTR(VcValue *value)
{
  int i = 0, imax = 0;
  VcValue *copy = NULL;

  copy            = VcMkSEQUENCE(0);
  copy->type      = VcGetType(value);
  copy->attribute = VcValueAttribute(value);

  imax = VcListLength(VcGetList(value));
  for (i = 0; i < imax; ++i)
    VcListSetValue(VcGetList(copy), i, VcCopy(VcListGetValue(VcGetList(value), i)));
  
  return copy;
}

static
VcValue *VcCopyOMITATTR(VcValue *value)
{
  return VcMkOmit(VcGetType(value));
}

static
VcValue *VcCopyANYONEATTR(VcValue *value)
{
  return VcMkAnyOne(VcGetType(value));
}

static
VcValue *VcCopyANYORNONEATTR(VcValue *value)
{
  return VcMkAnyOrNone(VcGetType(value));
}

static 
VcValue *VcCopyCOMPLEMENTATTR(VcValue *value)
{
  return VcCopyVALUELISTATTR(value);
}

static 
VcValue *VcCopySUPERSETATTR(VcValue *value)
{
  return VcCopyVALUELISTATTR(value);
}

static 
VcValue *VcCopySUBSETATTR(VcValue *value)
{
  return VcCopyVALUELISTATTR(value);
}

/*--------------------------------------------------------------------
 * Name:   VcCopySimple
 * Author: ITEX C-Code Generator
 *
 */

static VcValue * VcCopySimple(VcValue * value)
{
  return VcSetValueAttribute(VcValueAttribute(value),
                             VcSetType(VcGetType(value),
                                       VcMkINTEGER(VcGetINTEGER(value))));
}

/*--------------------------------------------------------------------
 * Name:   VcCopySequence
 * Author: ITEX C-Code Generator
 *
 */

static VcValue * VcCopySequence(VcValue * sequence)
{
  VcValue *copy = NULL;

  copy = VcMkSEQUENCE(0);
  if (VcGetList(sequence) != NULL) {
    VcList *from = NULL, *to = NULL;
    int items = 0, i = 0;
    
    from = VcGetList(sequence);
    to = VcGetList(copy);

    items = VcSeqSize(sequence);
    for (i = 0; i < items; ++i)
      VcListAddLast(to, VcCopy(VcListGetValue(from, i)));
  }

  /* In case of sequence beeing a SEQUENCE_OF/SET_OF. */
  copy->type = sequence->type;
  
  return VcSetValueAttribute(VcValueAttribute(sequence),
			     VcSetType(VcGetType(sequence), copy));
}

/*--------------------------------------------------------------------
 * Name:   VcCopyNull
 * Author: ITEX C-Code Generator
 *
 */

static VcValue * VcCopyNull(VcValue * null)
{
  return VcSetValueAttribute(VcValueAttribute(null),
                             VcSetType(VcGetType(null),
                                       VcMkNULL()));
}

/*--------------------------------------------------------------------
 * Name:   VcCopyOBJECT
 * Author: ITEX C-Code Generator
 *
 */

static VcValue * VcCopyOBJECT_IDENTIFIER(VcValue * value)
{
  return
    VcSetValueAttribute(VcValueAttribute(value),
                        VcSetType(VcGetType(value),
                     VcMkOBJECT_IDENTIFIER(value)));
}

/*--------------------------------------------------------------------
 * Name:   VcCopyObjectDescriptor
 * Author: ITEX C-Code Generator
 *
 */

static VcValue * VcCopyObjectDescriptor(VcValue * value)
{
  return VcSetValueAttribute(VcValueAttribute(value),
                             VcSetType(VcGetType(value),
                       VcMkObjectDescriptor(VcGetObjectDescriptor(value))));
}

/*--------------------------------------------------------------------
 * Name:   VcCopyString
 * Author: ITEX C-Code Generator
 *
 */

static VcValue * VcCopyString(VcValue * value)
{
  if (VcGetCSTRING(value) == NULL)
    return VcSetValueAttribute(VcValueAttribute(value),
			       VcSetType(VcGetType(value), VcMkCSTRING("")));
  else
    return VcSetValueAttribute(VcValueAttribute(value),
			       VcSetType(VcGetType(value),
					 VcMkCSTRING(VcGetCSTRING(value))));
}

/*--------------------------------------------------------------------
 * Name:   VcCopyReal
 * Author: ITEX C-Code Generator
 *
 */

static VcValue * VcCopyReal(VcValue * value)
{
  if (VcGetREAL(value) == NULL)
  return VcSetValueAttribute(VcValueAttribute(value),
                             VcSetType(VcGetType(value), VcMkREAL(0, 0, 0)));
  else
    return VcSetValueAttribute(VcValueAttribute(value),
			       VcSetType(VcGetType(value),
					 VcMkREAL(VcGetREAL(value)->mantissa,
						  VcGetREAL(value)->base,
						  VcGetREAL(value)->exponent)));

}

/*--------------------------------------------------------------------
 * Name:   VcCopyChoice
 * Author: ITEX C-Code Generator
 *
 */

static VcValue * VcCopyChoice(VcValue * value)
{
  VcValue *choice;

  if (value == NULL)
  { 
    VcWarning("VcCopyChoice", "<NULL> value");
    return NULL;
  }

  choice = (VcValue *) VCMALLOC(sizeof(VcValue), "VcCopyChoice");
  
  VcSetType(VcGetType(value), choice);
  VcSetValueAttribute(VcValueAttribute(value), choice);

  if (value->value.choice == NULL) /* Unnitialized value, nothing to copy.*/
    return choice;

  choice->value.choice = (VcChoice *) VCMALLOC(sizeof(VcChoice), "VcCopyChoice");
  if (VcGetCHOICE("", NULL, value) != NULL)
      choice->value.choice->value = VcCopy(VcGetCHOICE("", NULL, value));
  if (choice->value.choice->value != NULL)
    choice->value.choice->value->is_choice_element = TRUE;
  choice->value.choice->name  = VcGetCHOICEName(value);

  return choice;
}

/*--------------------------------------------------------------------
 * Name:   VcCopy
 * Author: ITEX C-Code Generator
 *
 */

VcValue * VcCopy(VcValue * value)
{
  VcValue *copy = NULL;

  if (value == NULL) {
    VcWarning("VcCopy", "<NULL> value (OK if value is OMITed)");
    copy = NULL;
  }
  else if (VcGetType(value) == NULL) {
    VcWarning("VcCopy", "untyped value");
    copy = NULL;
  }
  else if (VcIsInsteadOfValue(value)) {
    VcAttribute attribute = VcValueAttribute(value);
    if (attribute & VcVALUERANGEATTR)
      copy = VcCopyVALUERANGEATTR(value);
    else if (attribute & VcVALUELISTATTR)
      copy = VcCopyVALUELISTATTR(value);
    else if (attribute & VcOMITATTR)
      copy = VcCopyOMITATTR(value);
    else if (attribute & VcANYONEATTR)
      copy = VcCopyANYONEATTR(value);
    else if (attribute & VcANYORNONEATTR)
      copy = VcCopyANYORNONEATTR(value);
    else if (attribute & VcCOMPLEMENTATTR)
      copy = VcCopyCOMPLEMENTATTR(value);
    else if (attribute & VcSUPERSETATTR)
      copy = VcCopySUPERSETATTR(value);
    else if (attribute & VcSUBSETATTR)
      copy = VcCopySUBSETATTR(value);
    
    if (VcIsAttribute(value))
      VcSetValueAttribute(VcIF_PRESENTATTR, copy);
  }
  else {
    switch (VcGetBaseType(VcGetType(value))) {
      case VcBOOLEAND:
      case VcINTEGERD:
      case VcENUMERATEDD:
      case VcR_TYPED:
	copy = VcCopySimple(value);
	break;
      case VcNULLD:
	copy = VcCopyNull(value);
	break;
      case VcOBJECT_IDENTIFIERD:
	copy = VcCopyOBJECT_IDENTIFIER(value);
	break;
      case VcObjectDescriptorD:
	copy = VcCopyObjectDescriptor(value);
	break;
      case VcNumericStringD:
      case VcPrintableStringD:
      case VcTeletexStringD:
      case VcVideotexStringD:
      case VcIA5StringD:
      case VcUTCTimeD:
      case VcGeneralizedTimeD:
      case VcGraphicStringD:
      case VcVisibleStringD:
      case VcISO646StringD:
      case VcT61StringD:
      case VcGeneralStringD:
      case VcHEXSTRINGD:
      case VcBIT_STRINGD:
      case VcOCTET_STRINGD:
	copy = VcCopyString(value);
	break;
      case VcSEQUENCE_OFD:
      case VcSET_OFD:
      case VcSEQUENCED:
      case VcSETD:
	copy = VcCopySequence(value);
	break;
      case VcREALD:
	copy = VcCopyReal(value);
	break;
      case VcCHOICED:
	copy = VcCopyChoice(value);
	break;
      default:
	VcWarning("VcCopy", "unknown type");
	copy = NULL;
	break;
    }
  }
  
  return copy;
}

/*--------------------------------------------------------------------
 * Name:   VcGetRes
 * Author: ITEX C-Code Generator
 *
 */

static VcRestriction * VcGetRes(VcType * type)
{
  if (type == NULL) {
    VcWarning( "VcGetRes", "<NULL> type.");
    return NULL;
  }

  return type->restriction;
}

/*--------------------------------------------------------------------
 * Name:   VcGetResType
 * Author: ITEX C-Code Generator
 *
 */

static VcRestrictionType VcGetResType(VcRestriction * restriction)
{
  if (restriction == NULL) {
    VcWarning( "VcGetResType", "<NULL> restriction.");
    return 0;
  }

  return restriction->type;
}

/*--------------------------------------------------------------------
 * Name:   VcResNext
 * Author: ITEX C-Code Generator
 *
 */

static VcRestriction * VcResNext(VcRestriction * restriction)
{
  if (!restriction) {
    VcWarning("VcResNext", "<NULL> restriction");
    return NULL;
  }
  return restriction->next;
}

/*--------------------------------------------------------------------
 * Name:   VcGetIntegerLength
 * Author: ITEX C-Code Generator
 * 
 * This function gets the legal range of values for the given
 * type.  It then calculates the minimum number of eight-bit
 * bytes needed to represent that range.  The resulting number of
 * bytes is returned.  If there is no legal range or something
 * was not right the function returns '-1' to indicate that the
 * size needed to represent the integer type was unknown.
 */

static int VcGetIntegerLength(VcType * type)
{
  VcBaseVal lower = INT_MIN;
  VcBaseVal upper = INT_MAX;
  VcBaseVal range;
  VcBaseVal bits;

  if (!VcIsInteger(type))
    return -1;

  while (type != NULL) {
    VcRestriction *restriction = VcGetRes(type);
    
    while (restriction) {
      if (VcGetResType(restriction) != VcCONTAINED) {
	VcBaseVal l = INT_MIN;
	VcBaseVal u = INT_MAX;
	VcList *list = VcGetResValue(restriction);
	
	if (VcGetResType(restriction) == VcVALUERANGE) {
	  l = VcGetINTEGER(VcListGetValue(list, 0));
	  u = VcGetINTEGER(VcListGetValue(list, 1));
	}

	lower = (l < lower) ? lower : l;
	upper = (u > upper) ? upper : u;
      }
      
      restriction = VcResNext(restriction);
    }

    type = VcGetParent(type);
  }

  /* Count the number of bits needed. */

  bits = 0;
  range = (lower > upper) ? 0 : upper - lower + 1;
  while (range != 0) {
    range >>= 1;
    ++bits;
  }

  /* Calculate the number of bytes and return it. */
  return (bits != 0) ? ((bits - 1) / 8) + 1 : 0;
}

/*--------------------------------------------------------------------
 * Name:   VcGetStringLength
 * Author: ITEX C-Code Generator
 *
 */

static int VcGetStringLength(VcType * type)
{
  VcBaseVal lower = INT_MIN;
  VcBaseVal upper = INT_MAX;

  if (!VcIsString(type))
    return -1;

  while (type != NULL) {
    VcRestriction *restriction = VcGetRes(type);
    
    while (restriction) {
      if (VcGetResType(restriction) != VcCONTAINED) {
	VcBaseVal l = INT_MIN;
	VcBaseVal u = INT_MAX;
	VcList *list = VcGetResValue(restriction);
	
	if (VcGetResType(restriction) == VcSIZE) {
	  l = 0;
	  u = VcGetINTEGER(VcListGetValue(list, 0));
	}
	else if (VcGetResType(restriction) == VcSIZERANGE) {
	  l = VcGetINTEGER(VcListGetValue(list, 0));
	  u = VcGetINTEGER(VcListGetValue(list, 1));
	}

	lower = (l < lower) ? lower : l;
	upper = (u > upper) ? upper : u;
      }

      restriction = VcResNext(restriction);
    }
    type = VcGetParent(type);
  }

  return upper - lower;
}

/*--------------------------------------------------------------------
 * Name:   VcGetResValue
 * Author: ITEX C-Code Generator
 *
 */

static VcList * VcGetResValue(VcRestriction * restriction)
{
  if (VcGetResType(restriction) == VcCONTAINED) {
    VcWarning("VcGetResValue",
	      "restriction value requested from contained restriction");
    return NULL;
  }
  return restriction->content.value;
}

/*--------------------------------------------------------------------
 * Name:   VcGetResInclude
 * Author: ITEX C-Code Generator
 *
 * Asks the given restriction to return its contents as a
 * VcTypeList.  Asumes that they are contained restrictions.
 */

static VcTypeList * VcGetResInclude(VcRestriction * restriction)
{
  if (VcGetResType(restriction) != VcCONTAINED) {
    VcWarning("VcGetResInclude",
	      "contained value requested from restriction value");
    return NULL;
  }
  return restriction->content.include;
}

/*--------------------------------------------------------------------
 * Name:   VcRestrictSizeRange
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcRestrictSizeRange(VcRestriction * res, VcValue * value)
{
  int length, upper, lower;
  VcList *list;

  list = VcGetResValue(res);
  if (list == NULL || VcListLength(list) == 0) {
    VcWarning("VcRestrictSizeRange", "empty size-range");
    return FALSE;
  }

  lower = VcGetINTEGER(VcListGetValue(list, 0));
  upper = VcGetINTEGER(VcListGetValue(list, 1));

  if (VcIsStructured(VcGetType(value)))
    length = VcSeqSize(value);
  else if (VcIsOCTET_STRING(VcGetType(value)))
    length = Vcstrlen(VcGetCSTRING(value)) / 2;
  else
    length = Vcstrlen(VcGetCSTRING(value));

  return lower <= length && length <= upper;
}

/*--------------------------------------------------------------------
 * Name:   VcRestrictSize
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcRestrictSize(VcRestriction * res, VcValue * value)
{
  int length, size;
  VcList *list;

  list = VcGetResValue(res);
  if (list == NULL || VcListLength(list) == 0) {
    VcWarning("VcRestrictSizeRange", "empty size-range");
    return FALSE;
  }

  size = VcGetINTEGER(VcListGetValue(list, 0));

  if (VcIsStructured(VcGetType(value)))
    length = VcSeqSize(value);
  else if (VcIsOCTET_STRING(VcGetType(value)))
    length = Vcstrlen(VcGetCSTRING(value)) / 2;
  else
    length = Vcstrlen(VcGetCSTRING(value));

  return size == length;
}

/*--------------------------------------------------------------------
 * Name:   VcRestrictValueRange
 * Author: ITEX C-Code Generator
 *
 * This function checks if the value given lies in the value
 * range defined by the restriction.
 */

static Bool VcRestrictValueRange(VcRestriction * res, VcValue * value)
{
  int upper, lower, current;
  VcList *list;

  list = VcGetResValue(res);
  if (list == NULL || VcListLength(list) == 0) {
    VcWarning("VcRestrictValueRange", "empty value-range");
    return FALSE;
  }

  lower = VcGetINTEGER(VcListGetValue(list, 0));
  upper = VcGetINTEGER(VcListGetValue(list, 1));
  current = VcGetINTEGER(value);

  return lower <= current && current <= upper;
}

/*--------------------------------------------------------------------
 * Name:   VcRestrictSingleValue
 * Author: ITEX C-Code Generator
 *
 * This function checks if the value given matches at least one
 * of the values in the restrictions list of values.  If so we
 * return TRUE and false otherwise.
 */

static Bool VcRestrictSingleValue(VcRestriction * res, VcValue * value)
{
  VcList *list = VcGetResValue(res);
  int i;

  for (i = 0; i < VcListLength(list); ++i)
    if (VcMatchValue(VcListGetValue(list, i), value))
      return TRUE;
  return FALSE;
}

/*--------------------------------------------------------------------
 * Name:   VcRestrictPermittedAlphabet
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcRestrictPermittedAlphabet(VcRestriction * res, VcValue * value)
{
  char alphabet[512] = "";
  int i;

  VcString string = VcGetCSTRING(value);
  VcList *character = VcGetResValue(res);

  for (i = 0; i < VcListLength(character); ++i)
    Vcstrcat(alphabet, VcGetCSTRING(VcListGetValue(character, i)));

  return (strspn(string, alphabet) == Vcstrlen(string)) ? TRUE : FALSE;
}

/*--------------------------------------------------------------------
 * Name:   VcRestrictInclude
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcRestrictInclude(VcRestriction * res, VcValue * value)
{
  VcTypeList *list = VcGetResInclude(res);

  while (list != NULL) {
    if (VcRestrict(list->type, value) == TRUE)
      return TRUE;
    list = list->next;
  }
  return FALSE;
}

/*--------------------------------------------------------------------
 * Name:   VcRestrict
 * Author: ITEX C-Code Generator
 *
 * This function checks that the value give matches the
 * restrictions in the type given.  The value must match all
 * restrictions up to the uppermost type.  We only log if we do
 * not match the restrictions
 */

static Bool VcRestrict(VcType * type, VcValue * value)
{
  Bool match;
  VcType *current;

  /* Omited values match their restrictions per default. */

  if (VcIsOmitable(value))
    return TRUE;

  /* We now turn of logging.  The checking of restrictions gives
   * birth to a lot of redundant logging.  This is because we
   * reuse the matching mechanism for the checking.  We are only
   * interested in the logs when we fail a match because of a
   * restriction. */

  VcLogOFF();

  /* Here we check the current type and its parent type until we
   * reach a type that contains a restriction.  When we reach a
   * type with a restriction we check it. */

  match = TRUE;
  current = type;
  while (current != NULL) {
    VcRestriction *restriction = VcGetRes(type);
    Bool result = TRUE;
    
    while (restriction) {
      result = FALSE;

      switch (VcGetResType(restriction)) {
	case VcSIZERANGE:
	  result = VcRestrictSizeRange(restriction, value);
	  break;
	case VcSIZE:
	  result = VcRestrictSize(restriction, value);
	  break;
	case VcVALUERANGE:
	  result = VcRestrictValueRange(restriction, value);
	  break;
	case VcSINGLEVALUE:
	  result = VcRestrictSingleValue(restriction, value);
	  break;
	case VcPERMITTEDALPHABET:
	  result = VcRestrictPermittedAlphabet(restriction, value);
	  break;
	case VcCONTAINED:
	  result = VcRestrictInclude(restriction, value);
	  break;
	default:
	  VcWarning("VcRestrict", "unknown restriction");
	  break;
      }

      if (result)
	break;
      
      restriction = VcResNext(restriction);
    }
    
    if (!(match = match & result))
      break;
    
    /* Climb up the parent type ladder one step. */
    
    current = VcGetParent(current);
  }

  /* We now turn the logging back on. */

  VcLogON();

  VcLogRestriction(type, value, match);

  return match;
}

/*--------------------------------------------------------------------
 * Name:   VcGetLength
 * Author: ITEX C-Code Generator
 *
 * This function returns the single length restriction for a
 * string.
 *
 * If the type is an INTEGER or equivalent, the function will
 * return the number of bytes it will take to encode it,
 * depending on the range.
 *
 * If no valid restriction is given, the function will return -1.
 */

int VcGetLength(VcType * type)
{

  /* If there is no restriction, return -1 */

  if (VcGetRes(type) == NULL)
    return -1;

  /* See what the given types basetype is, and handle it
   * specifically.  The only cases handled are strings and
   * integers. */

  if (VcIsInteger(type))
    return VcGetIntegerLength(type);
  else if (VcIsString(type))
    return VcGetStringLength(type);
  return -1;
}

/*--------------------------------------------------------------------
 * Name:   VcMkRes
 * Author: ITEX C-Code Generator
 *
 * Allocates and returns a new restriction of the indicated type.
 * The type of the restriction shall be one of these types:
 *
 *  VcSIZERANGE
 *    The restriction contains a list of length two.  The first
 *    element is a lower bound and the second an upper bound on
 *    the size of values.  A values size has to be within
 *    (inclusive) of the lower and upper bounds in order to match
 *    the restriction.  Only STRING, SEQUENCE and SET types have
 *    this restriction.
 *
 *  VcSIZE
 *    The restriction contains a list of length one.  The element
 *    is a bound on the size of values.  A values size has to be
 *    the one given in the bound in order to match the
 *    restriction.  Only STRING, SEQUENCE and SET types have this
 *    restriction.
 *
 *  VcVALUERANGE
 *    The restriction contains a list of length two.  The first
 *    element is a lower bound and the second an upper bound on
 *    values.  A value has to be within (inclusive) of the lower
 *    and upper bounds in order to match the restriction.  Only
 *    INTEGERS have this restriction.
 *
 *  VcSINGLEVALUE
 *    The restriction contains a list of values.  A value has to
 *    match (at least) one of them in order to match the
 *    restriction.
 *
 *  VcPERMITTEDALPHABET
 *    The restriction contains a list of string-typed values,
 *    representing a permitted alphabet.  A string value has to
 *    be entierly built from elements from the alphabet in order
 *    to match the restriction.
 *
 *  VcCONTAINED
 *    The restriction contains a list of pointers to types.  The
 *    value must fulfill the restrictions of at least one of the
 *    types in order to match the restriction.
 */

VcRestriction * VcMkRes(VcRestrictionType type)
{
  VcRestriction *res = (VcRestriction *)
  VCMALLOC(sizeof(VcRestriction), "VcMkRes");

  res->type = type;
  res->content.value = NULL;
  res->next = NULL;

  return res;
}

/*--------------------------------------------------------------------
 * Name:   VcResAddElem 
 * Author: ITEX C-Code Generator
 *
 */

VcRestriction * VcResAddElem(VcValue** value, VcRestriction * res)
{
  VcList* reslist = NULL;
  VcValue** dummyloc = NULL;

  if (*value != NULL)
    res->content.value = VcListAddLast(res->content.value, *value);
  else
  {
    res->content.value = VcListAddLast(res->content.value, VcMkINTEGER(0));
    reslist = res->content.value;
    dummyloc = &(reslist->begin[ VcListLength(reslist) - 1 ]);
    VcValueDepForBackPatch(dummyloc, value);
  }

  return res;
}

/*--------------------------------------------------------------------
 * Name:   VcAddRes 
 * Author: ITEX C-Code Generator
 *
 */

VcType * VcAddRes(VcRestriction * res, VcType * type)
{
  if (type == NULL) {
    VcWarning("VcAddRes", "<NULL> type");
    return NULL;
  }
  res->next = type->restriction;
  type->restriction = res;
  return type;
}

/*--------------------------------------------------------------------
 * Name:   VcAddContainedType
 * Author: ITEX C-Code Generator
 *
 */

VcRestriction * VcAddContainedType(VcType * include, VcRestriction * res)
{
  VcTypeList *node;

  if (VcGetResType(res) != VcCONTAINED) {
    VcWarning("VcAddContainedType",
	      "adding type reference to non-contained restriction");
    return res;
  }
  node = (VcTypeList *) VCMALLOC(sizeof(VcTypeList), "VcAddContainedType");
  node->type = include;
  node->next = res->content.include;
  res->content.include = node;
  return res;
}

/*--------------------------------------------------------------------
 * Name:    VcValueDepForBackPatch
 * Author:  Mikael Gustafsson 
 *
 * Adds the value<->restriction dependency to the back-pass list. 
 *
 */

Bool VcValueDepForBackPatch(VcValue** valueloc, VcValue** resval)
{
  VcValueDepBPLink* link = (VcValueDepBPLink*) malloc(sizeof(VcValueDepBPLink));

  if (link != NULL)
  {
    link->value_location = valueloc;
    link->value = resval;
    link->next = VcValueDepBPList;

    VcValueDepBPList = link;
    
    return TRUE;
  }

  return FALSE;
}

/*--------------------------------------------------------------------
 * Name:    VcBackPatchValueDep
 * Author:  Mikael Gustafsson
 *
 * Iterates the back-pass list and updates the restricted types with correct
 * values instead of the previously introduced dummy values.
 *
 * If something goes wrong a warning (an error!?) is issued.
 *
 * NOTE: The backpatch list is deallocated as the unresolved restrictions are 
 * back-passed!
 */

Bool VcBackPatchValueDep()
{
  VcValueDepBPLink* tmp = VcValueDepBPList;

  while (tmp != NULL)
  {
    if ((tmp->value_location != NULL) &&
        (tmp->value != NULL) &&
        (*(tmp->value) != NULL))
      *(tmp->value_location) = *(tmp->value); 
    else
      VcWarning( "VcBackPatchValueDep", "Leaving restriction undefined"); /* Should say more?! */

    /* Proceed to next and deallocate the handled element. */
    tmp = tmp->next;
    free(VcValueDepBPList);
    VcValueDepBPList = tmp;
  }

  return TRUE;
}


/*--------------------------------------------------------------------
 * The set of functions that are responsible for the matching of
 * values.
 */

/*--------------------------------------------------------------------
 * Name:   VcMatchValuelist
 * Author: ITEX C-Code Generator
 *
 * Matches an "Instead of Value" that is a VALUELIST.  This means
 * that the value received will match if it matches one or more
 * values in the VALUELIST's list of values.
 */

static Bool VcMatchValuelist(VcValue * reference, VcValue * value)
{
  Bool match;
  VcList *list = VcGetList(reference);
  int i;

  if (list == NULL) {
    IcLog(IcLogNoMatchValue, "Matching empty VALUELIST");
    return FALSE;
  }

  match = FALSE;
  for (i = 0; i < VcListLength(list); ++i)
    if (VcMatchValue(VcListGetValue(list, i), value)) {
      match = TRUE;
      break;
    }

  VcLogMatch(reference, value, match);

  return match;
}

/*--------------------------------------------------------------------
 * Name:   VcMatchComplement
 * Author: ITEX C-Code Generator
 *
 * Matches an "Instead of Value" that is a COMPLEMENT.  This
 * means that the value received will match if it doesn't match
 * any value in the COMPLEMENT's list of values.
 */

static Bool VcMatchComplement(VcValue * reference, VcValue * value)
{
  Bool match = FALSE;

  if (VcIsOmitable(value))
    match = TRUE;
  else
    match = (VcMatchValuelist(reference, value) == FALSE);

  VcLogMatch(reference, value, match);

  return match;
}

/*--------------------------------------------------------------------
 * Name:   VcMatchSuperSet
 * Author: ITEX C-Code Generator
 *
 * Matches an "Instead of Value" that is a SUPERSET.  This
 * means that the value received will match if it contains at least 
 * the element defined in the SUPERSET but may contain more.
 */

static Bool VcMatchSuperSet(VcValue * reference, VcValue * value)
{
  Bool match = FALSE;
  VcList* rlist = VcGetList(reference);
  VcList* vlist = VcGetList(value);

  if (VcIsOmitable(value))
    match = TRUE;
  else if (vlist != NULL) {   
    match = TRUE;    
    if (VcListLength(vlist) >= VcListLength(rlist)) {
      int i; 
      for (i = 0; i < VcListLength(rlist); ++i)
	if (!VcMatchValuelist(value, VcListGetValue(rlist, i))) {
	  match = FALSE;
	  break;
	}  
    }
  }
  
  VcLogMatch(reference, value, match);
  
  return match;
}

/*--------------------------------------------------------------------
 * Name:   VcMatchSubSet
 * Author: ITEX C-Code Generator
 *
 * Matches an "Instead of Value" that is a SUBSET. This
 * means that the value received will match if it contains only
 * the elements defined in the SUBSET but may contain less.
 */

static Bool VcMatchSubSet(VcValue * reference, VcValue * value)
{
  Bool match = FALSE;
  VcList* rlist = VcGetList(reference);
  VcList* vlist = VcGetList(value);

  if (VcIsOmitable(value))
    match = TRUE;
  else if (vlist != NULL) {   
    match = TRUE;    
    if (VcListLength(vlist) <= VcListLength(rlist)) {
      int i;
      for (i = 0; i < VcListLength(vlist); ++i)
	if (!VcMatchValuelist(reference, VcListGetValue(vlist, i))) {
	  match = FALSE;
	  break;
	}  
    }
  }
  
  VcLogMatch(reference, value, match);

  return match;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchOmit
 * Author: ITEX C-Code Generator
 *
 * This function matches OMIT values.  It returns true if the
 * value is omited, FALSE otherwise.
 */

static Bool VcMatchOmit(VcValue * reference, VcValue * value)
{
  Bool match = VcIsOmitable(value) ? TRUE : FALSE;

  VcLogMatch(reference, value, match);
  return match;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchAnyOrNone
 * Author: ITEX C-Code Generator
 *
 * This function matches ANYORNONE values.  It returns always
 * TRUE since it matches anything.
 */

static Bool VcMatchAnyOrNone(VcValue * reference, VcValue * value)
{
  VcLogMatch(reference, value, TRUE);
  return TRUE;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchAnyone
 * Author: ITEX C-Code Generator
 *
 * This function matches ANYONE values.  It returns TRUE if the
 * value we are matching is not omited, FALSE otherwise.
 */

static Bool VcMatchAnyone(VcValue * reference, VcValue * value)
{
  Bool match = VcIsOmitable(value) ? FALSE : TRUE;

  VcLogMatch(reference, value, match);
  return match;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchIF_PRESENT
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchValueRange(VcValue * reference, VcValue * value)
{
  int lower, upper, current;
  VcList *list;

  list = VcGetList(reference);
  if (VcListLength(list) != 2) {
    VcWarning("VcMatchValueRange", "trying to match empty valuerange");
    return FALSE;
  }

  lower = VcGetINTEGER(VcListGetValue(list, 0));
  upper = VcGetINTEGER(VcListGetValue(list, 1));
  if (lower > upper) {
    int tmp = lower;
    lower = upper;
    upper = tmp;
  }
  current = VcGetINTEGER(value);

  return lower <= current && current <= upper;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchIF_PRESENT
 * Author: ITEX C-Code Generator
 *
 * Matches IF_PRESENT values.  The function returns TRUE if
 * either the value we are matching is omited or if it matches
 * the actual value.  It returns FALSE otherwise.
 */

static Bool VcMatchIF_PRESENT(VcValue * reference, VcValue * value)
{
  /* If the value we are trying to match is omitable it has to be
   * either a NULL-value or a value with the OMIT attribute set.
   * The value is thus not present and we have a match.  We
   * return here since we have logged the result of the match. */

  if (VcIsOmitable(value)) {
    VcLogMatch(reference, value, TRUE);
    return TRUE;
  }

  /* If the value we are matching isn't omitted we match the
   * actual values.  We do not log anything here since that is
   * handled in the "VcMatchValue" function.  We just return the
   * result of the match. */

  return VcMatchValue(reference, value);
}

/*-------------------------------------------------------------------
 * Name:   VcMatchInsteadOfValue
 * Author: ITEX C-Code Generator
 *
 * This function is responsible for the matching of values
 * against a constraint that isn't a real value.  All it does is
 * to determine what kind of "Instead of Value" value we have and
 * calls the matching function that knows how to handle it.
 */

static Bool VcMatchInsteadOfValue(VcValue * reference, VcValue * value)
{
  VcAttribute attribute = VcValueAttribute(reference);

  if (attribute & VcOMITATTR)
    return VcMatchOmit(reference, value);
  else if (attribute & VcVALUELISTATTR)
    return VcMatchValuelist(reference, value);
  else if (attribute & VcCOMPLEMENTATTR)
    return VcMatchComplement(reference, value);
  else if (attribute & VcANYORNONEATTR)
    return VcMatchAnyOrNone(reference, value);
  else if (attribute & VcANYONEATTR)
    return VcMatchAnyone(reference, value);
  else if (attribute & VcVALUERANGEATTR)
    return VcMatchValueRange(reference, value);
  else if (attribute & VcSUPERSETATTR)
    return VcMatchSuperSet(reference, value);
  else if (attribute & VcSUBSETATTR)
    return VcMatchSubSet(reference, value);
  
  /* If we get here no attributes were set.  If we have a NULL
   * value we can only match it if we have one of the attributes
   * ANYORNONE, OMIT or IF_PRESENT.  Thus a NULL value should
   * give a mismatch on the attributes. */

  return value == NULL ? FALSE : TRUE;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchAttribute
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchAttribute(VcValue * reference, VcValue * value)
{
  if (VcValueAttribute(reference) & VcIF_PRESENTATTR)
    return VcMatchIF_PRESENT(reference, value);
  return FALSE;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchReal
 * Author: ITEX C-Code Generator
 *
 * Matches two real values against each other.  Returns TRUE if
 * the value matches the reference, FALSE otherwise.
 */

static Bool VcMatchReal(VcValue * reference, VcValue * value)
{
  Bool match;

  if (VcGetBOOLEAN(VcEQ(reference, value)) == TRUE)
    match = TRUE;
  else
    match = FALSE;

  VcLogMatch(reference, value, match);

  return match;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchOBJECT_IDENTIFIER
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchOBJECT_IDENTIFIER(VcValue * reference, VcValue * value)
{
  Bool match;
  VcList *rlist, *vlist;
  int i;

  IcLog(IcLogMatchValue, "Attempting to match OBJECT IDENTIFIER");
  
  if (reference == NULL) {
    VcWarning("VcMatchOBJECT_IDENTIFIER", "untyped reference constraint");
    return FALSE;
  }
  if (value == NULL) {
    VcWarning("VcMatchOBJECT_IDENTIFIER", "untyped value constraint");
    return FALSE;
  }

  rlist = VcGetList(reference);
  vlist = VcGetList(value);

  if (rlist == NULL) {
    VcWarning("VcMatchOBJECT_IDENTIFIER", "<NULL> component list in reference constraint");
    return FALSE;
  }
  if (vlist == NULL) {
    VcWarning("VcMatchOBJECT_IDENTIFIER", "<NULL> component list in value constraint");
    return FALSE;
  }

  match = FALSE;
  if (VcListLength(rlist) == VcListLength(vlist)) /* Keep checking */
  {
    match = TRUE; /* Match so far... */
    for(i = 0; i < VcListLength(rlist); ++i)
    {
      VcValue *rc, *vc;
      rc = VcListGetValue(rlist, i);
      vc = VcListGetValue(vlist, i);

      if ((rc == NULL) || (vc == NULL) || 
          (VcGetINTEGER(rc) != VcGetINTEGER(vc)))
      {
        match = FALSE;
        break;
      }
    }
  }

  if (match)
    IcLog(IcLogMatchValue, "Matching OBJECT IDENTIFIER");
  else
    IcLog(IcLogNoMatchValue, "Matching OBJECT IDENTIFIER");

  return match;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchChoice 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchChoice(VcValue * reference, VcValue * value)
{
  Bool match;

  if (VcGetBaseType(VcGetType(value)) == VcCHOICED)
    match = VcMatch(VcGetCHOICE("", NULL, reference), VcGetCHOICE("", NULL, value));
  else
    match = VcMatch(VcGetCHOICE("", NULL, reference), value);

  if (match)
    IcLog(IcLogMatchValue, "Matched CHOICE");
  else
    IcLog(IcLogNoMatchValue, "Matched CHOICE");
  return match;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchSimple 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchSimple(VcValue * reference, VcValue * value)
{
  Bool match;

  VcWarningOFF();
  match = (VcGetINTEGER(reference) == VcGetINTEGER(value));
  VcWarningON();

  VcLogMatch(reference, value, match);

  return match;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchClass 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchClass(VcType * r_type, VcType * v_type)
{
  if (VcGetClass(r_type) == VcGetClass(v_type))
    return TRUE;
  return FALSE;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchTag 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchTag(VcType * r_type, VcType * v_type)
{
  if (VcGetTag(v_type) == VcNULLD)
    return TRUE;
  else if (VcGetTag(r_type) == VcGetTag(v_type))
    return TRUE;
  return FALSE;
}
/*-------------------------------------------------------------------
 * Name:   VcMatchTypeName 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchTypeName(VcType * r_type, VcType * v_type)
{
  if (VcGetName(r_type) == VcGetName(v_type))
    return TRUE;
  return FALSE;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchTypes 
 * Author: ITEX C-Code Generator
 *
 * This function determines if the types of its arguments are
 * compatible.  If they are it returns TRUE, and FALSE otherwise.
 */

static Bool VcMatchTypes(VcValue * reference, VcValue * value)
{
  if (reference == NULL) {
    VcWarning("VcMatchTypes", "<NULL> reference constraint");
    return FALSE;
  }
  if (VcGetBaseType(VcGetType(reference)) == VcUndefinedD) {
    VcWarning("VcMatchTypes", "untyped reference constraint");
    return FALSE;
  }

  /* In case we have an undefined value or a value with an
   * undefined type.  In this case we assume the types match and
   * hope that the value has an attribute set.  This should
   * really be a FAIL.  It should not happen! */

  if (VcGetBaseType(VcGetType(value)) == VcUndefinedD)
    return TRUE;

  /* If the type of the reference value is any of CHOICE, ANY or
   * PDU we defer the type matching.  All these values contain
   * references to other values, and the matching function will
   * dereference and call itself recursively.  When that happens
   * we will perform the type matching. */

  if (VcGetBaseType(VcGetType(reference)) == VcCHOICED ||
      VcGetBaseType(VcGetType(reference)) == VcANYD ||
      VcGetBaseType(VcGetType(reference)) == VcPDUD)
    return TRUE;

  /* If the value we are matching is OMITED we don't care 
   * about the type. This is because the CCG generates 
   * "VcMkOMIT(VcNULL)" statements. */

  if (VcIsOmitable(value))
    return TRUE;
    
  /* See if the actual types are compatible.  This implements the
   * hard part of the type check.  If we really were hard we
   * would just use this test.*/

  if (VcMatchClass(VcGetType(reference), VcGetType(value)) &&
      VcMatchTag(VcGetType(reference), VcGetType(value)) &&
      VcMatchTypeName(VcGetType(reference), VcGetType(value)))
    return TRUE;

  /* See if the base-class of the reference value is compatible
   * with the user-provided value.  This is here because we
   * assume that the user could quite possibly be lazy, and not
   * build the specific types used by the test suite. */

  if (VcMatchClass(VcBaseType(reference), VcGetType(value)) &&
      VcMatchTag(VcBaseType(reference), VcGetType(value)) &&
      VcMatchTypeName(VcBaseType(reference), VcGetType(value)))
    return TRUE;

  /* Type mismatch. */

  VcLogTypes(VcGetType(reference), VcGetType(value));

  return FALSE;
}

/*-------------------------------------------------------------------
 * Name:   VcIsInsteadOfValue
 * Author: ITEX C-Code Generator
 *
 * This function checks if the value given as argument is an
 * "Instead of Value" value.  If so we return TRUE, and FALSE
 * otherwise.
 */

static Bool VcIsInsteadOfValue(VcValue * value)
{
  if (VcValueAttribute(value) & 
      (VcOMITATTR | VcANYONEATTR | VcANYORNONEATTR | VcVALUELISTATTR | 
       VcCOMPLEMENTATTR | VcVALUERANGEATTR | VcSUBSETATTR | VcSUPERSETATTR))
    return TRUE;
  else
    return FALSE;
}

/*-------------------------------------------------------------------
 * Name:   VcIsInsideValue
 * Author: ITEX C-Code Generator
 *
 * This function checks if the value given as an argument
 * contains any wildcard symbols.  If it does we have an
 * Inside-Value value.
 *
 * It currently only works with string values.  SEQUENCE_OF and
 * SET_OF values will be added.
 *
 * PERMUTATION belongs here!
 */

static Bool VcIsInsideValue(VcValue * value)
{
  if (VcIsString(VcGetType(value))) {
    VcString str = VcGetCSTRING(value);
    VcString current;

    /* Check for AnyOne (?) in the string. */

    current = str;
    while ((current = strchr(current, '?')) != NULL)
      if (current == str || *(current - 1) != '\\')
	return TRUE;
    
    /* Check for AnyOrNone (*) in the string. */
    
    current = str;
    while ((current = strchr(current, '*')) != NULL)
      if (current == str || *(current - 1) != '\\')
	return TRUE;
    
    /* We found neither a AnyOrNone or a AnyOne, return FALSE. */
    
    return FALSE;
  }
  return TRUE;
}

/*-------------------------------------------------------------------
 * Name:   VcIsAttribute 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcIsAttribute(VcValue * value)
{
  return (VcValueAttribute(value) & VcIF_PRESENTATTR) ? TRUE : FALSE;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchNumericString 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchNumericString(VcValue * reference, VcValue * value)
{
  if (!VcIsNum(VcGetCSTRING(value))) {
    IcLog(IcLogNoMatchValue, "Non Numeric Value received for NumericString.");
    return FALSE;
  }  return VcMatchOtherString(reference, value);
}

/*-------------------------------------------------------------------
 * Name:   VcMatchBitString 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchBitString(VcValue * reference, VcValue * value)
{
  if (!VcIsBit(VcGetCSTRING(value))) {
    IcLog(IcLogNoMatchValue, "Non Bitstring Value received for BIT STRING.");
    return FALSE;
  }
  return VcMatchOtherString(reference, value);
}

/*-------------------------------------------------------------------
 * Name:   VcMatchOctetString 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchOctetString(VcValue * reference, VcValue * value)
{
  if (!VcIsOCTET_STRING(VcGetType(value)) || !VcIsOct(VcGetCSTRING(value))) {
    IcLog(IcLogNoMatchValue, "Non OCTET Value received for OCTET STRING.");
    return FALSE;
  }
  return VcMatchOtherString(reference, value);
}

/*-------------------------------------------------------------------
 * Name:   VcMatchHexString 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchHexString(VcValue * reference, VcValue * value)
{
  if (!VcIsHex(VcGetCSTRING(value))) {
    IcLog(IcLogNoMatchValue, "Non HEX Value received for HEX STRING.");
    return FALSE;
  }
  return VcMatchOtherString(reference, value);
}

/*-------------------------------------------------------------------
 * Name:   VcMatchOtherString 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchOtherString(VcValue * reference, VcValue * value)
{
  Bool match;

  /* Check for wildcards. If on the wrong side (receiving) swap
   * places. This is not absolutely correct, but it makes the code
   * more robust. We also check for wildcards in both sides of the
   * string and warn for that.*/

  /* H5318 Value can't contain any wildcards!!! So, swaping isn't permited!*/
  /* Yes, it's quite necessary. */

  if (VcIsInsideValue(value)) {
    if (VcIsInsideValue(reference))
      VcWarning("VcMatchOtherString", "wildcards in both sides of string match");    
    else {
      VcValue *tmp;

      tmp       = reference;
      reference = value;
      value     = tmp;
    }
  }
  match = VcMatchString(VcGetCSTRING(reference), VcGetCSTRING(value));

  VcLogMatch(reference, value, match);

  return match;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchTime 
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchTime(VcValue * reference, VcValue * value)
{
  if (!VcIsTime(VcGetCSTRING(value))) {
    IcLog(IcLogNoMatchValue, "Non Time Value received.");
    return FALSE;
  }
  return VcMatchOtherString(reference, value);
}

/*-------------------------------------------------------------------
 * Name:   VcMatchSequenceBacktrack
 * Author: ITEX C-Code Generator
 *
 * Tries to match two sequence constraints.  The value sequence
 * can contain implicitly omited values, optional values that
 * just has been left out.  In order to be able to match such
 * values we have to have backtracking.
 *
 * The algorithm used is:
 *
 *  We iterate over the refrence sequence.  For each element we
 *  do the following:
 *
 *    See if the current pair of values match and if the rest of
 *    the sequences match.  If so we are done and return TRUE.
 *
 *    Otherwise if the current refrence value is omitable we
 *    assume that it was implicitly omitted and move along the
 *    reference sequence.  If the assumption was incorrect we
 *    will fail later.
 *
 *    If it wasn't omitable we failed.  Return FALSE.
 *  
 *  We have reached the end of one of the two sequences.  See if
 *  we used up all the elements. If so, return TRUE otherwise
 *  FALSE.
 */

static Bool VcMatchSequenceBacktrack(VcList * r, int ri, VcList * v, int vi)
{
  Bool b1;
  int offset = 0,
    new_vi, count_stars, i;
       
  VcAttribute attribute;

  while (ri != VcListLength(r) && vi != VcListLength(v)) {
    b1 = VcMatch(VcListGetValue(r, ri), VcListGetValue(v, vi)) &&
	VcMatchSequenceBacktrack(r, ri + 1, v, vi + 1);
    if(!b1)
      {
	/* Failed matching */
	attribute = VcValueAttribute(VcListGetValue(r, ri));
	if(attribute & VcANYORNONEATTR)
	  {
	    /* Wildcard "*" presents */
	    if(ri+1 == VcListLength(r))
	      return TRUE;
	    count_stars = 0;
	    /*It counts a number of "*" */
	    for(i=ri+1; i<VcListLength(r); i++)
	      {
		attribute = VcValueAttribute(VcListGetValue(r, i));
		if(attribute & VcANYORNONEATTR)
		  count_stars++;
	      }
	    /* It counts new high boundary of vi */
	    offset = VcListLength(r) - ri - 1 - count_stars;
	    new_vi = VcListLength(v) - offset;
	    if (new_vi<vi)
	      return FALSE;
	    /* Vi is changed. It tries to match lists for different vi */
	    for(i=vi; i<new_vi+1; i++)
	      {
		if (VcMatchSequenceBacktrack(r, ri + 1, v, i))
		  return TRUE;
	      }
	    return FALSE;
	  }
      }

    if (b1)
      return TRUE;
    else if (VcIsOmitable(VcListGetValue(r, ri)))
      ++ri;
    else
      return FALSE;
  }

  if (vi == VcListLength(v)) {
    while (ri != VcListLength(r) && VcIsOmitable(VcListGetValue(r, ri)))
      ++ri;
    return ri == VcListLength(r);
  }
  else
    return FALSE;
}


/*-------------------------------------------------------------------
 * Name:   VcMatchSequence
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchSequence(VcValue * reference, VcValue * value)
{
  Bool match;
  VcList *rlist, *vlist;

  IcLog(IcLogMatchValue, "Attempting to match SEQUENCE/SEQUENCE_OF");

  if (reference == NULL) {
    VcWarning("VcMatchSequence", "untyped reference constraint");
    return FALSE;
  }
  if (value == NULL) {
    VcWarning("VcMatchSequence", "untyped value constraint");
    return FALSE;
  }

  rlist = VcGetList(reference);
  vlist = VcGetList(value);

  if (rlist == NULL) {
    VcWarning("VcMatchSequence", "<NULL> sequence in reference constraint");
    return FALSE;
  }
  if (vlist == NULL) {
    VcWarning("VcMatchSequence", "<NULL> sequence in value constraint");
    return FALSE;
  }

  match = VcMatchSequenceBacktrack(rlist, 0, vlist, 0);

  if (match)
    IcLog(IcLogMatchValue, "Matching SEQUENCE/SEQUENCE_OF");
  else
    IcLog(IcLogNoMatchValue, "Matching SEQUENCE/SEQUENCE_OF");

  return match;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchSetBacktrack
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchSetBacktrack(VcList * r, int ri, VcList * v, Bool * used)
{
  int i;

  while (ri != VcListLength(r)) {
    for (i = 0; i < VcListLength(v); ++i)
      if (!used[i] && VcMatch(VcListGetValue(r, ri), VcListGetValue(v, i)))  {
	used[i] = TRUE;;
	if (VcMatchSetBacktrack(r, ri + 1, v, used))
	  return TRUE;
	else
	  used[i] = FALSE;
      }
    if (VcIsOmitable(VcListGetValue(r, ri)))
      ++ri;
    else
      return FALSE;
  }

  for (i = 0; i < VcListLength(v); ++i)
    if (!used[i])
      return FALSE;
  return TRUE;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchSet
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchSet(VcValue * reference, VcValue * value)
{
  Bool match, *used;
  VcList *rlist, *vlist;
  int i;

  IcLog(IcLogMatchValue, "Attempting to match SET/SET_OF");

  if (reference == NULL) {
    VcWarning("VcMatchSet", "untyped reference constraint");
    return FALSE;
  }
  if (value == NULL) {
    VcWarning("VcMatchSet", "untyped value constraint");
    return FALSE;
  }

  rlist = VcGetList(reference);
  vlist = VcGetList(value);

  if (rlist == NULL) {
    VcWarning("VcMatchSet", "<NULL> sequence in reference constraint");
    return FALSE;
  }
  if (vlist == NULL) {
    VcWarning("VcMatchSet", "<NULL> sequence in value constraint");
    return FALSE;
  }

  used = (Bool *) malloc(sizeof(Bool) * VcListLength(vlist));

  for (i = 0; i < VcListLength(vlist); ++i)
    used[i] = FALSE;

  match = VcMatchSetBacktrack(rlist, 0, vlist, used);

  free(used);

  if (match)
    IcLog(IcLogMatchValue, "Matching SET/SET_OF");
  else
    IcLog(IcLogNoMatchValue, "Matching SET/SET_OF");

  return match;
}

/*--------------------------------------------------------------------
 * Name:   VcMatchString 
 * Author: ITEX C-Code Generator
 *
 * String matching function. pattern may contain two different
 * wildcards:
 *  '*' meaning ANY OR NONE 
 *  '?' meaning ANY ONE
 *
 * The function checks for the occurance of sub-strings delimeted by
 * '*' from the pattern in the string.  If we can find all such
 * sub-strings in the strin in the same order we have a match.
 */

static Bool VcMatchString(VcString pattern, VcString string)
{
  register char *s = string, *p = pattern;
  int p_slash = 0;

  if (s == p)
    return TRUE;

  if (s == NULL || p == NULL)
    return FALSE;

  while (*p != '\0' || *s != '\0') {
    p_slash = 0;
    if(*p == '\\')
    {
      p++;
      p_slash = 1;
    }
    if ((*p != '*')||(p_slash == 1)) {
      if (*p == *s || (*p == '?' && *s != '\0' && p_slash == 0))
	++p, ++s;
      else
	return FALSE;
    }
    else {
      register char c, *p_end;
      
      ++p;
      
      if (*p == '\0')
	return TRUE;
      
      /* Find the end of the next token, ie substring not containing
       * '*' or the end of the string.  When the end has been found,
       * save the character and replace it with a '\0' character. */

      for (p_end = p; *p_end != '\0' && *p_end != '*'; ++p_end)
	{}
      c = *p_end;
      *p_end = '\0';
      
      /* Now we try to find the token within the string that we have
       * left. */

      while (*s != '\0') {
	register char *ts = s, *tp = p;
	
	/* Try to match the current position in the string with the
	 * current token in the pattern. */
	
	while ((*ts == *tp || (*tp == '?' && *ts != '\0')) && *tp != '\0')
	  ++ts, ++tp;
	
	/* If we haven't reached the end of the current token in the
	 * pattern, the we move forward in the string. */
	
	if (*tp != '\0') {
	  
	  /* Advance ano step. */
	  
	  ++s;
	  
	  /* If we reached the end of the string, restore the pattern
	   * and return FALSE. */
	  
	  if (*ts == '\0') {
	    *p_end = c;
	    return FALSE;
	  }
	}
	else if (c == '\0' && *ts != '\0')
	  ++s;
	else {
	  p = tp, s = ts, *p_end = c;
	  break;
	}
      }
    }
  }
  return TRUE;
}

/*-------------------------------------------------------------------
 * Name:   VcMatchValue
 * Author: ITEX C-Code Generator
 *
 */

static Bool VcMatchValue(VcValue * reference, VcValue * value)
{
  switch (VcGetBaseType(VcGetType(reference))) {
    case VcNULLD:
    case VcR_TYPED:
    case VcBOOLEAND:
    case VcINTEGERD:
    case VcENUMERATEDD:
      return VcMatchSimple(reference, value);
    case VcREALD:
      return VcMatchReal(reference, value);
    case VcCHOICED:
      return VcMatchChoice(reference, value);
    case VcPDUD:
    case VcANYD:
      /* We should never get here!  Values can only have the type
       * PDU or ANY if they have an "Instead of Value" attribute
       * set, and those cases should have been caught earlier. */
      return TRUE;
    case VcSETD:
    case VcSET_OFD:
      return VcMatchSet(reference, value);
    case VcSEQUENCED:
    case VcSEQUENCE_OFD:
      return VcMatchSequence(reference, value);
    case VcNumericStringD:
      return VcMatchNumericString(reference, value);
    case VcBIT_STRINGD:
      return VcMatchBitString(reference, value);
    case VcOCTET_STRINGD:
      return VcMatchOctetString(reference, value);
    case VcHEXSTRINGD:
      return VcMatchHexString(reference, value);
    case VcPrintableStringD:
    case VcTeletexStringD:
    case VcVideotexStringD:
    case VcIA5StringD:
    case VcGraphicStringD:
    case VcVisibleStringD:
    case VcISO646StringD:
    case VcT61StringD:
    case VcGeneralStringD:
    case VcObjectDescriptorD:
      return VcMatchOtherString(reference, value);
    case VcOBJECT_IDENTIFIERD:
      return VcMatchOBJECT_IDENTIFIER(reference, value);
    case VcUTCTimeD:
    case VcGeneralizedTimeD:
      return VcMatchTime(reference, value);
    default:
      VcWarning("VcMatch", "unknown type");
      return FALSE;
    }
}

/*--------------------------------------------------------------------
 * Name:   VcMatch
 * Author: ITEX C-Code Generator
 *
 * Function to compare two diferent objects.  The reference
 * object is the only one that may contain wildcards.
 *
 * The types of the objects and the sub-objects are determined by
 * the type of the reference object.
 *
 * This function is called for all matchings.  In most cases an
 * utility function will be called in order to perform more
 * specific matching.  It is very possible that this utility
 * function will call this function again, since we have values
 * within values...
 *
 * It first tries to determine if the given values are of a
 * compatible type.  If they aren't of compatible types the
 * matching has failed and we stop we stop, otherwise we
 * continue.
 *
 * We now check if the reference value is a "Instead of Value"
 * value.  If it isn't we continue, otherwise we check which one
 * it is and handle it.  The handler returns a result that is the
 * result of the match.
 *
 * Now we try to handle the restrictions.  Currently, mismatches
 * are only logged.  Matching continues in all cases here.
 *
 * Now we see what the basetype the reference value has.  The
 * basetype determines which kind of match we will do.
 *
 * This function returns TRUE if the arguments did match, FALSE
 * otherwise.
 */

Bool VcMatch(VcValue * reference, VcValue * value)
{
  /* Two NULL values are matching. */ 
  if (reference == NULL && value == NULL) {
    VcWarning("VcMatch", "<NULL> matches <NULL>");
    return TRUE;
  }

  /* See if the two values given here have compatible types.  If
   * they havn't compatible types abort the match, otherwise
   * continue... */
  if (VcMatchTypes(reference, value) == FALSE)
    return FALSE;

  /* Check so that the recived value is within it's types
   * restrictions.  The restrictions are collected from the type
   * of the reference.  This is done because we can't trust the
   * user to be able to identify the correct sub-type in each
   * case. */
  if (VcRestrict(VcGetType(reference), value) == FALSE)
    return FALSE;

  if (VcIsAttribute(reference) && VcIsInsteadOfValue(reference)) {
    if (VcMatchAttribute(reference, value))
      return TRUE;
    else
      return VcMatchInsteadOfValue(reference, value);
  }

  if (VcIsAttribute(reference))
    return VcMatchAttribute(reference, value);
 
  if (VcIsInsteadOfValue(reference))
    return VcMatchInsteadOfValue(reference, value);

  /* Handle "Specific Value" constraints */
  return VcMatchValue(reference, value);
}

/*--------------------------------------------------------------------
 * The functions responsible for building a lexical
 * representation ao an arbitrary value.  Mainly used building a
 * log-text.
 */

/*--------------------------------------------------------------------
 * Types used within the logging.
 */

struct VcLogType
{
  int length;
  int used;
  char *begin;
  char *end;
};

/* Used to turn the logging of selectively. */

static int VcLogState = 0;

/*--------------------------------------------------------------------
 * Name:   VcLogCreate
 * Author: ITEX C-Code Generator
 *
 * This function creates and initialises a log value.
 */

static VcLogType VcLogCreate(void)
{
  const int initial_size = 1024;

  VcLogType log = (VcLogType) malloc(sizeof(struct VcLogType));

  log->length = initial_size;
  log->used = 1;
  log->begin = (char *) malloc(initial_size * sizeof(char));

  log->end = log->begin;

  Vcstrcpy(log->begin, "");

  return log;
}

/*--------------------------------------------------------------------
 * Name:   VcLogDelete
 * Author: ITEX C-Code Generator
 *
 * This function frees the memory used by a log value.
 */

static void VcLogDelete(VcLogType log)
{
  free(log->begin);
  free(log);
}

/*--------------------------------------------------------------------
 * Name:   VcLogBegin
 * Author: ITEX C-Code Generator
 *
 * An accessor function that returns the start of the log-string.
 */

static char * VcLogBegin(VcLogType log)
{
  return log != NULL ? log->begin : "";
}

/*--------------------------------------------------------------------
 * Name:   VcLogON 
 * Author: ITEX C-Code Generator
 *
 */

static void VcLogON(void)
{
  ++VcLogState;
}

/*--------------------------------------------------------------------
 * Name:   VcLogOFF 
 * Author: ITEX C-Code Generator
 *
 */

static void VcLogOFF(void)
{
  --VcLogState;
}

/*--------------------------------------------------------------------
 * Name:   VcLogAddText
 * Author: ITEX C-Code Generator
 *
 * The workhorse of the logging functions.  This function handles
 * the actual insertion of text into the logstring.  If the
 * logstring would overflow because of the addition the
 * log-string is reallocated with a bigger size.
 *
 * The VcLogType contains an "end" pointer which is used here to
 * be able to know where to add the new text.  This eliminates a
 * search through the log-string for every insertion.
 */

static void VcLogAddText(VcLogType log, const char *text)
{
  int needed, length;
  
  if (log == NULL) {
    VcWarning( "VcLogAddText", "no log type.");
    return;
  }

  if (text == NULL) {
    VcWarning( "VcLogAddText", "empty log text.");
    return;
  }

  needed = log->used + Vcstrlen(text);
  length = log->length;

  while (length < needed)
    length *= 2;

  if (length > log->length) {
    char *tmp = (char *) malloc(sizeof(char) * length);
    
    Vcstrcpy(tmp, log->begin);
    free(log->begin);
    
    log->length = length;
    log->begin = tmp;
    log->end = log->begin + log->used - 1;
  }
  Vcstrcpy(log->end, text);
  log->end = log->begin + needed - 1;
  log->used = needed;
}

/*--------------------------------------------------------------------
 * Name:   VcLogAddInsteadOfValue
 * Author: ITEX C-Code Generator
 *
 * Logs "Instead of Value" values.  Check which type of "Instead
 * of Value" we are working with and add a token describing it.
 */

static void VcLogAddInsteadOfValue(VcLogType log, VcValue * value)
{
  VcAttribute attribute = VcValueAttribute(value);
  if (attribute & VcVALUELISTATTR)
    VcLogAddText(log, "VALUELIST ");
  else if (attribute & VcCOMPLEMENTATTR)
    VcLogAddText(log, "COMPLEMENT ");
  else if (attribute & VcANYORNONEATTR)
    VcLogAddText(log, "* ");
  else if (attribute & VcANYONEATTR)
    VcLogAddText(log, "? ");
  else if (attribute & VcOMITATTR)
    VcLogAddText(log, "OMIT ");
  else if (attribute & VcSUPERSETATTR)
    VcLogAddText(log, "SUPERSET ");
  else if (attribute & VcSUBSETATTR)
    VcLogAddText(log, "SUBSET ");
}

/*--------------------------------------------------------------------
 * Name:   VcLogAddAttribute
 * Author: ITEX C-Code Generator
 *
 * Adds the lexical representation of an value with an attribute
 * set to the log.  Currently there is only one attribute
 * handled, IF_PRESENT.
 */

static void VcLogAddAttribute(VcLogType log, VcValue * value)
{
  if (VcValueAttribute(value) & VcIF_PRESENTATTR) {
    VcLogAddValueValue(log, value);
    VcLogAddText(log, "IF_PRESENT ");
  }
}

/*--------------------------------------------------------------------
 * Name:   VcLogAddInteger
 * Author: ITEX C-Code Generator
 *
 * Builds the lexical representation of an INTEGER value.
 */

static 
void VcLogAddEnumerated(VcLogType log, VcValue * value)
{
  char buffer[256], string[64];

  sprintf(string, "%ld", VcGetINTEGER(value));
  sprintf(buffer, "%s (%s) ", VcEnumGet(string, VcGetType(value)), string);

  VcLogAddText(log, buffer);
}

/*--------------------------------------------------------------------
 * Name:   VcLogAddInteger
 * Author: ITEX C-Code Generator
 *
 * Builds the lexical representation of an INTEGER value.
 */

static void VcLogAddInteger(VcLogType log, int integer)
{
  static char tmp[50];

  sprintf(tmp, "%i ", integer);
  VcLogAddText(log, tmp);
}

/*--------------------------------------------------------------------
 * Name:   VcLogAddReal
 * Author: ITEX C-Code Generator
 *
 * Builds the lexical representation of a REAL value.
 */

static void VcLogAddReal(VcLogType log, VcReal * real)
{
  static char tmp[150];

  if (real == NULL) {
    VcWarning("VcLogAddReal", "<NULL> real value.");
    Vcstrcpy(tmp, "<NULL>");
  } else
    sprintf(tmp, "(%ld, %ld, %ld) ", real->mantissa, real->base, real->exponent);

  VcLogAddText(log, tmp);
}

/*--------------------------------------------------------------------
 * Name:   VcLogAddSequence
 * Author: ITEX C-Code Generator
 *
 * Logs a sequence.  Just add the sequence delimeters and then
 * recursively log the elements in the sequence.
 */

static void VcLogAddSequence(VcLogType log, VcValue * value)
{
  int i;
  VcList *list = VcGetList(value);

  VcLogAddText(log, "{ ");
  for (i = 0; i < VcListLength(list); ++i) {
    VcLogAddValue(log, VcListGetValue(list, i));
    if (i + 1 != VcListLength(list))
      VcLogAddText(log, ", ");
  }
  VcLogAddText(log, "} ");
}

/*--------------------------------------------------------------------
 * Name:   VcLogAddOBJECT_IDENTIFIER
 * Author: ITEX C-Code Generator
 *
 * Logs an OBJECT IDENTIFIER.  Just add the curly brace delimeters and 
 * then recursively log the elements in the list.
 */

static void VcLogAddOBJECT_IDENTIFIER(VcLogType log, VcValue *value)
{
  int i;
  VcList *list = VcGetList(value);

  VcLogAddText(log, "{ ");
  for (i = 0; i < VcListLength(list); ++i) {
    VcLogAddValue(log, VcListGetValue(list, i));
    if (i + 1 != VcListLength(list))
      VcLogAddText(log, ", ");
  }
  VcLogAddText(log, "} ");
}

/*--------------------------------------------------------------------
 * Name:   VcLogAddValueValue
 * Author: ITEX C-Code Generator
 *
 * This function handles actual values.  It checks the type and
 * logs it accordingly.  If the handling is complex enough it
 * calls a specialized function that performs the addition to the
 * log.
 */

static void VcLogAddValueValue(VcLogType log, VcValue * value)
{
  switch (VcGetBaseType(VcGetType(value))) {
    case VcBOOLEAND:
      VcLogAddText(log, VcGetBOOLEAN(value) ? "TRUE " : "FALSE ");
      break;
    case VcENUMERATEDD:
      VcLogAddEnumerated(log, value);
      break;
    case VcINTEGERD:
    case VcR_TYPED:
      VcLogAddInteger(log, VcGetINTEGER(value));
      break;
    case VcNULLD:
      VcLogAddText(log, "NULL");
      break;
    case VcREALD:
      VcLogAddReal(log, VcGetREAL(value));
      break;
    case VcBIT_STRINGD:
      VcLogAddText(log, "'");
      VcLogAddText(log, VcGetCSTRING(value));
      VcLogAddText(log, "'B ");
      break;
    case VcOCTET_STRINGD:
      VcLogAddText(log, "'");
      VcLogAddText(log, VcGetCSTRING(value));
      VcLogAddText(log, "'O ");
      break;
    case VcHEXSTRINGD:
      VcLogAddText(log, "'");
      VcLogAddText(log, VcGetCSTRING(value));
      VcLogAddText(log, "'H ");
      break;
    case VcObjectDescriptorD:
    case VcPrintableStringD:
    case VcNumericStringD:
    case VcTeletexStringD:
    case VcVideotexStringD:
    case VcIA5StringD:
    case VcUTCTimeD:
    case VcGeneralizedTimeD:
    case VcGraphicStringD:
    case VcVisibleStringD:
    case VcGeneralStringD:
      VcLogAddText(log, "\"");
      VcLogAddText(log, VcGetCSTRING(value));
      VcLogAddText(log, "\" ");
      break;
    case VcOBJECT_IDENTIFIERD:
      VcLogAddOBJECT_IDENTIFIER(log, value);
      break;
    case VcSETD:
    case VcSEQUENCED:
    case VcSEQUENCE_OFD:
  case VcSET_OFD:
      VcLogAddSequence(log, value);
      break;
  case VcCHOICED:
      VcLogAddText(log, VcGetCHOICEName( value ));
      VcLogAddText(log, " ");
      VcLogAddValue( log, VcGetCHOICE( "", NULL, value ));
      break;
    default:
      break;
    }
}

/*--------------------------------------------------------------------
 * Name:   VcLogAddValue
 * Author: ITEX C-Code Generator
 *
 * This function sees what kind of value we are trying to log and
 * defers the actual building of the lexical representation to
 * the correct function.
 */

static void VcLogAddValue(VcLogType log, VcValue * value)
{
  if (value == NULL)
    VcLogAddText(log, "<NULL> ");
  else if (VcIsInsteadOfValue(value))
    VcLogAddInsteadOfValue(log, value);
  else if (VcIsAttribute(value))
    VcLogAddAttribute(log, value);
  else
    VcLogAddValueValue(log, value);
}

/*--------------------------------------------------------------------
 * Name:   VcLogMatch
 * Author: ITEX C-Code Generator
 *
 * This function builds a log-string "Matching <value-rep>
 * against <constraint-rep>".  It then emits the log_string as a
 * match or no-match according to the 'match' argument.
 *
 * This function is intended to be used at the end of the value
 * matching functions.  These functions all match values against
 * constraints and they all log the result of the match.
 */

static void VcLogMatch(VcValue * reference, VcValue * value, Bool match)
{
  if (VcLogState == 0) {
    VcLogType log = VcLogCreate();
    
    VcLogAddText(log, "Matching ");
    VcLogAddValue(log, value);
    VcLogAddText(log, "against ");
    VcLogAddValue(log, reference);
    
    if (match)
      IcLog(IcLogMatchValue, VcLogBegin(log));
    else
      IcLog(IcLogNoMatchValue, VcLogBegin(log));
    
    VcLogDelete(log);
  }
}

/*--------------------------------------------------------------------
 * Name:   VcLogAssign
 * Author: ITEX C-Code Generator
 *
 */

static void VcLogAssign(VcString name, VcValue * value)
{
  if (VcLogState == 0) {
    VcLogType log = VcLogCreate();
    
    VcLogAddText(log, name);
    VcLogAddText(log, " := ");
    VcLogAddValue(log, value);
    
    IcLog(GciLogAssign, VcLogBegin(log));
    
    VcLogDelete(log);
  }
}

/*--------------------------------------------------------------------
 * Name:   VcLogRestriction
 * Author: ITEX C-Code Generator
 *
 */

static void VcLogRestriction(VcType * type, VcValue * value, Bool match)
{
  if (VcLogState == 0 && !match) {
    VcLogType log = VcLogCreate();
    
    VcLogAddText(log, "Matching ");
    VcLogAddValue(log, value);
    VcLogAddText(log, "against the restrictions on type \"");
    VcLogAddText(log, VcTypeToStr(type));
    VcLogAddText(log, "\"");

    IcLog(IcLogNoMatchValue, VcLogBegin(log));
    
    VcLogDelete(log);
  }
}

/*--------------------------------------------------------------------
 * Name:   VcLogTypes
 * Author: ITEX C-Code Generator
 *
 */

static void VcLogTypes(VcType * reference, VcType * value)
{
  if (VcLogState == 0) {
    VcLogType log = VcLogCreate();
    
    VcLogAddText(log, "Incorrect message type.  Got a \"[");
    VcLogAddText(log, VcTypeClassToStr(value));
    VcLogAddText(log, " ");
    VcLogAddText(log, VcTypeTagToStr(value));
    VcLogAddText(log, "] ");
    VcLogAddText(log, VcTypeToStr(value));
    VcLogAddText(log, "\", expected a \"[");
    VcLogAddText(log, VcTypeClassToStr(reference));
    VcLogAddText(log, " ");
    VcLogAddText(log, VcTypeTagToStr(reference));
    VcLogAddText(log, "] ");
    VcLogAddText(log, VcTypeToStr(reference));
    
    IcLog(IcLogNoMatchValue, VcLogBegin(log));
    
    VcLogDelete(log);
  }
}

/*--------------------------------------------------------------------
 * Functions responsible for informative messages follow here.
 */

/*--------------------------------------------------------------------
 * Name:   VcMessage 
 * Author: ITEX C-Code Generator
 *
 */

static void VcMessage(const char *name, const char *message)
{
  fprintf(stderr, "%s: %s\n", name, message);
}

/*--------------------------------------------------------------------
 * Functions responsible for warning- and error-messages follow
 * here.
 */

/* Used to turn the warning messages of selectively. */

int VcWarningState = 0;

/*--------------------------------------------------------------------
 * Name:   VcWarningON 
 * Author: ITEX C-Code Generator
 *
 */

static void VcWarningON(void)
{
  ++VcWarningState;
}

/*--------------------------------------------------------------------
 * Name:   VcWarningOFF 
 * Author: ITEX C-Code Generator
 *
 */

static void VcWarningOFF(void)
{
  --VcWarningState;
}

/*--------------------------------------------------------------------
 * Name:   VcWarning
 * Author: ITEX C-Code Generator
 *
 */

static void VcWarning(VcString name, VcString message)
{
  if (VcWarningState == 0)
    fprintf(stderr, "WARNING: %s in function \"%s\"\n", message, name);
}

/*--------------------------------------------------------------------
 * Miscellaneous functions follow here.
 */

/*--------------------------------------------------------------------
 * Name:   VcBefore
 * Author: ITEX C-Code Generator
 *
 */

Bool VcBefore(VcTime Start, VcValue * Duration, VcUnit Unit, VcTime End)
{
  VcMessage("VcBefore", "NOT IMPLEMENTED");
  return TRUE;                  /* NOT IMPLEMENTED. */
}

/*--------------------------------------------------------------------
 * Name:   VcSUBSET
 * Author: ITEX C-Code Generator
 *
 */

VcValue * VcSUBSET(VcValue * dataObject1, VcValue * dataObject2)
{
  VcMessage("VcSUBSET", "NOT IMPLEMENTED");
  return VcMkBOOLEAN(TRUE);     /* NOT IMPLEMENTED. */

}

/*--------------------------------------------------------------------
 * Name:   VcSUPERSET
 * Author: ITEX C-Code Generator
 *
 */

VcValue * VcSUPERSET(VcValue * dataObject1, VcValue * dataObject2)
{
  VcMessage("VcSUPERSET", "NOT IMPLEMENTED");
  return VcMkBOOLEAN(TRUE);     /* NOT IMPLEMENTED. */
}

/*--------------------------------------------------------------------
 * Name:   VcPERMUTATION
 * Author: ITEX C-Code Generator
 *
 */

VcValue * VcPERMUTATION(VcValue * dataObject1, VcValue * dataObject2)
{
  VcMessage("VcPERMUTATION", "NOT IMPLEMENTED");
  return FALSE;                 /* NOT IMPLEMENTED. */
}

/*--------------------------------------------------------------------
 * Name:   VcIsOct
 * Author: ITEX C-Code Generator
 *
 */

Bool VcIsOct(VcString String)
{
  return VcIsHex(String) && Vcstrlen(String) % 2 == 0;
}

/*--------------------------------------------------------------------
 * Name:   VcIsHex
 * Author: ITEX C-Code Generator
 *
 */

Bool VcIsHex(VcString String)
{
  return strspn(String, "0123456789ABCDEFabcdef*?") == Vcstrlen(String);
}

/*--------------------------------------------------------------------
 * Name:   VcIsBit
 * Author: ITEX C-Code Generator
 *
 */

Bool VcIsBit(VcString String)
{
  return strspn(String, "01*?") == Vcstrlen(String);
}

/*--------------------------------------------------------------------
 * Name:   VcIsNum
 * Author: ITEX C-Code Generator
 *
 */

Bool VcIsNum(VcString String)
{
  return strspn(String, "0123456789*?") == Vcstrlen(String);
}

/*--------------------------------------------------------------------
 * Name:   VcIsTime
 * Author: ITEX C-Code Generator
 *
 */

Bool VcIsTime(VcString String)
{
  return strspn(String, "0123456789.-+Zz*?") == Vcstrlen(String);
}


/*--------------------------------------------------------------------
 * String manipulation functions.
 *
 * Implementations to handle NULL parameters for the str... functions 
 * which don't.
 *
 * When comparing (Vcstrcmp, strncmp), two NULL strings are considered 
 * equal (with warning) and a NULL string is lexicographically less 
 * than a non-NULL string.
 *
 */


/*--------------------------------------------------------------------
 * Vcstrlen
 *
 */
size_t Vcstrlen( const char* str) {
  if (str == NULL) {
    VcWarning("Vcstrlen", "<NULL> string. Returning 0 (zero) length.");
    return 0;
  } else
    return strlen(str);
}

/*--------------------------------------------------------------------
 * Vcstrncpy
 *
 */
char* Vcstrncpy( char* dest, const char* src, size_t count) {
  if (dest == NULL) {
    VcWarning("Vcstrncpy", "<NULL> destination string input. Returning <NULL>.");
    return NULL;
  }
  
  if (src == NULL) {
    VcWarning("Vcstrncpy", "<NULL> source string input. Returning destination string");
    return dest;
  }
  
  return strncpy(dest, src, count);
}

/*--------------------------------------------------------------------
 * Vcstrcpy
 *
 */
char* Vcstrcpy( char* dest, const char* src) {
  if (dest == NULL) {
    VcWarning("Vcstrcpy", "<NULL> destination string input. Returning <NULL>.");
    return NULL;
  }
  
  if (src == NULL) {
    VcWarning("Vcstrcpy", "<NULL> source string input. Returning destination string");
    return dest;
  }
  
  return strcpy(dest, src);
}

/*--------------------------------------------------------------------
 * Vcstrncmp
 *
 */
int Vcstrncmp( const char* str1, const char* str2, size_t count) {
  if (str1 == NULL && str2 == NULL) {
    VcWarning("Vcstrncmp", "considering <NULL> strings to be equal.");
    return 0;
  }

  if (str1 == NULL) {
    VcWarning("Vcstrncmp", "first argument <NULL>.");
    return -1;
  }

  if (str2 == NULL) {
    VcWarning("Vcstrncmp", "second argument <NULL>.");
    return 1;
  }

  return strncmp(str1, str2, count);
}

/*--------------------------------------------------------------------
 * Vcstrcmp
 *
 */
int Vcstrcmp( const char* str1, const char* str2) {
  if (str1 == NULL && str2 == NULL) {
    VcWarning("Vcstrcmp", "considering <NULL> strings to be equal.");
    return 0;
  }

  if (str1 == NULL) {
    VcWarning("Vcstrcmp", "first argument <NULL>.");
    return -1;
  }

  if (str2 == NULL) {
    VcWarning("Vcstrcmp", "second argument <NULL>.");
    return 1;
  }

  return strcmp(str1, str2);
}


/*--------------------------------------------------------------------
 * Vcstrncat
 *
 */
char* Vcstrncat( char* dest, const char* src, size_t count) {
  if (dest == NULL) {
    VcWarning("Vcstrncat", "<NULL> destination string input. Returning <NULL>.");
    return NULL;
  }
  
  if (src == NULL) {
    VcWarning("Vcstrncat", "<NULL> source string input. Returning destination string");
    return dest;
  }
  
  return strncat(dest, src, count);
}


/*--------------------------------------------------------------------
 * Vcstrcat
 *
 */
char* Vcstrcat( char* dest, const char* src) {
  if (dest == NULL) {
    VcWarning("Vcstrcat", "<NULL> destination string input. Returning <NULL>.");
    return NULL;
  }
  
  if (src == NULL) {
    VcWarning("Vcstrcat", "<NULL> source string input. Returning destination string");
    return dest;
  }
  
  return strcat(dest, src);
}

/* End of File. */
