//
// Copyright (c) 1997 X/Open Company Ltd., A member of The Open Group.
//
// All rights reserved. No part of this source code may be reproduced,
// stored in a retrieval system, or transmitted, in any form or by any
// means, electronic, mechanical, photocopying, recording or otherwise,
// except as stated in the end-user licence agreement, without the prior
// permission of the copyright owners.
//
// Developed for X/Open by ApTest Ireland Limited.
//
// Project: CORVAL2
//

#include <stdio.h>
#include <stdlib.h>
#include <tet_api.h>
#include "orb_supp.hh"
#include "ernie.hh"
#include "tin.hh"
#include "types.hh"

extern Ernie ernie;
extern OrbSupport *orb;
extern char *stringifiedIOR;

static int initialised = 0;
CORBA::Object_ptr obj;
siiServer_ptr  tobj;

void
initialise()
{
    if (!initialised)
    {
#ifdef DEBUG
      ::tet_infoline("client: connecting to proxy");
      ::tet_printf("client: using %s", stringifiedIOR);
#endif
    
      try {
        obj = orb->getObject((const char *)stringifiedIOR);
      }
      catch (const CORBA::SystemException &ex) {
        ::tet_infoline("client: getObj failed");
        ::tet_printf("client: minor code %d", ex.minor());
        ::tet_result(TET_FAIL);
        return;
      }
    
      if (obj == 0)
      {
        ::tet_infoline("client: failed connecting to proxy");
        ::tet_result(TET_FAIL);
        return;
      }
    
      try {
        tobj = siiServer::_narrow(obj);
      }
      catch (CORBA::Exception &ex) {
        ::tet_infoline("client: failed to narrow object reference.");
      }
      initialised = 1;

      char * tout = 0;
      tout = getenv("VSORB_EVENT_TIMEOUT");

      unsigned long timeout = (tout) ? atoi(tout) : 15;
      orb->setTimeout(tobj, timeout);
    }
    
}

#ifdef VSORB_IDL_TESTS

void testIDL()
{
  char buf1[1024];

::tet_result(TET_PASS);
}
 
#endif



 
void testBasicTestsPrunes1(void)
{
  
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::BasicTests::Prunes1()");
#endif
  
  ernie.seed("Prunes1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Prunes1Return");
  try {
     tobj->Prunes1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Prunes1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsBrocoli1(void)
{
  
  CORBA::Float ReturnValue;
  CORBA::Float p1;
  CORBA::Float p2;
  CORBA::Float p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Float ::BasicTests::Brocoli1(CORBA::Float p1,CORBA::Float &p2,CORBA::Float &p3)");
#endif
  
  ernie.seed("Brocoli1Call");
  ernie.mode(0);
  GetFloat(p1);
  ernie.mode(1);
  GetFloat(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Brocoli1Return");
  try {
    ReturnValue = tobj->Brocoli1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Brocoli1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFloat(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckFloat(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckFloat(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsGrannySmith1(void)
{
  
  CORBA::Double ReturnValue;
  CORBA::Double p1;
  CORBA::Double p2;
  CORBA::Double p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Double ::BasicTests::GrannySmith1(CORBA::Double p1,CORBA::Double &p2,CORBA::Double &p3)");
#endif
  
  ernie.seed("GrannySmith1Call");
  ernie.mode(0);
  GetDouble(p1);
  ernie.mode(1);
  GetDouble(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GrannySmith1Return");
  try {
    ReturnValue = tobj->GrannySmith1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::GrannySmith1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDouble(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckDouble(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckDouble(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsBrazilNut1(void)
{
  
  CORBA::Long ReturnValue;
  CORBA::Long p1;
  CORBA::Long p2;
  CORBA::Long p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::BasicTests::BrazilNut1(CORBA::Long p1,CORBA::Long &p2,CORBA::Long &p3)");
#endif
  
  ernie.seed("BrazilNut1Call");
  ernie.mode(0);
  GetLong(p1);
  ernie.mode(1);
  GetLong(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BrazilNut1Return");
  try {
    ReturnValue = tobj->BrazilNut1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::BrazilNut1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckLong(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckLong(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsSweetPotato1(void)
{
  
  CORBA::Short ReturnValue;
  CORBA::Short p1;
  CORBA::Short p2;
  CORBA::Short p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Short ::BasicTests::SweetPotato1(CORBA::Short p1,CORBA::Short &p2,CORBA::Short &p3)");
#endif
  
  ernie.seed("SweetPotato1Call");
  ernie.mode(0);
  GetShort(p1);
  ernie.mode(1);
  GetShort(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetPotato1Return");
  try {
    ReturnValue = tobj->SweetPotato1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::SweetPotato1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckShort(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckShort(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsTangarine1(void)
{
  
  CORBA::ULong ReturnValue;
  CORBA::ULong p1;
  CORBA::ULong p2;
  CORBA::ULong p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::ULong ::BasicTests::Tangarine1(CORBA::ULong p1,CORBA::ULong &p2,CORBA::ULong &p3)");
#endif
  
  ernie.seed("Tangarine1Call");
  ernie.mode(0);
  GetULong(p1);
  ernie.mode(1);
  GetULong(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Tangarine1Return");
  try {
    ReturnValue = tobj->Tangarine1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Tangarine1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckULong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckULong(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckULong(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsBillberries1(void)
{
  
  CORBA::UShort ReturnValue;
  CORBA::UShort p1;
  CORBA::UShort p2;
  CORBA::UShort p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::BasicTests::Billberries1(CORBA::UShort p1,CORBA::UShort &p2,CORBA::UShort &p3)");
#endif
  
  ernie.seed("Billberries1Call");
  ernie.mode(0);
  GetUShort(p1);
  ernie.mode(1);
  GetUShort(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Billberries1Return");
  try {
    ReturnValue = tobj->Billberries1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Billberries1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckUShort(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckUShort(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsCantelopeMelon1(void)
{
  
  CORBA::Char ReturnValue;
  CORBA::Char p1;
  CORBA::Char p2;
  CORBA::Char p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char ::BasicTests::CantelopeMelon1(CORBA::Char p1,CORBA::Char &p2,CORBA::Char &p3)");
#endif
  
  ernie.seed("CantelopeMelon1Call");
  ernie.mode(0);
  GetChar(p1);
  ernie.mode(1);
  GetChar(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CantelopeMelon1Return");
  try {
    ReturnValue = tobj->CantelopeMelon1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::CantelopeMelon1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChar(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckChar(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckChar(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsLolloRossa1(void)
{
  
  CORBA::Boolean ReturnValue;
  CORBA::Boolean p1;
  CORBA::Boolean p2;
  CORBA::Boolean p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Boolean ::BasicTests::LolloRossa1(CORBA::Boolean p1,CORBA::Boolean &p2,CORBA::Boolean &p3)");
#endif
  
  ernie.seed("LolloRossa1Call");
  ernie.mode(0);
  GetBoolean(p1);
  ernie.mode(1);
  GetBoolean(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LolloRossa1Return");
  try {
    ReturnValue = tobj->LolloRossa1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::LolloRossa1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBoolean(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckBoolean(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckBoolean(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsRowanBerry1(void)
{
  
  CORBA::Octet ReturnValue;
  CORBA::Octet p1;
  CORBA::Octet p2;
  CORBA::Octet p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Octet ::BasicTests::RowanBerry1(CORBA::Octet p1,CORBA::Octet &p2,CORBA::Octet &p3)");
#endif
  
  ernie.seed("RowanBerry1Call");
  ernie.mode(0);
  GetOctet(p1);
  ernie.mode(1);
  GetOctet(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RowanBerry1Return");
  try {
    ReturnValue = tobj->RowanBerry1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::RowanBerry1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOctet(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckOctet(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckOctet(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsSweetcorn1(void)
{
  
  CORBA::Object_ptr ReturnValue;
  CORBA::Object_ptr p1;
  CORBA::Object_ptr p2;
  CORBA::Object_ptr p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Object_ptr ::BasicTests::Sweetcorn1(CORBA::Object_ptr p1,CORBA::Object_ptr &p2,CORBA::Object_ptr &p3)");
#endif
  
  ernie.seed("Sweetcorn1Call");
  ernie.mode(0);
  p1 = Getobject();
  ernie.mode(1);
  p3 = Getobject();

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sweetcorn1Return");
  try {
    ReturnValue = tobj->Sweetcorn1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Sweetcorn1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!Checkobject(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!Checkobject(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!Checkobject(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsOnion1(void)
{
  
  CORBA::Char * ReturnValue;
  CORBA::Char * p1;
  CORBA::Char * p2;
  CORBA::Char * p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::BasicTests::Onion1(const CORBA::Char * p1,CORBA::Char * &p2,CORBA::Char * &p3)");
#endif
  
  ernie.seed("Onion1Call");
  ernie.mode(0);
  p1 = GetString();
  ernie.mode(1);
  p3 = GetString();

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Onion1Return");
  try {
    ReturnValue = tobj->Onion1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Onion1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckString(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckString(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsShallots1(void)
{
  
  CORBA::Char * ReturnValue;
  CORBA::Char * p1;
  CORBA::Char * p2;
  CORBA::Char * p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::BasicTests::Shallots1(const CORBA::Char * p1,CORBA::Char * &p2,CORBA::Char * &p3)");
#endif
  
  ernie.seed("Shallots1Call");
  ernie.mode(0);
  p1 = GetString(20);
  ernie.mode(1);
  p3 = GetString(46);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Shallots1Return");
  try {
    ReturnValue = tobj->Shallots1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Shallots1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue,4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckString(p2,12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckString(p3,46))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsPlum1(void)
{
  
  CORBA::Long ReturnValue;
  CORBA::Boolean p1;
  CORBA::Octet p2;
  CORBA::UShort p3;
  CORBA::Boolean p4;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::BasicTests::Plum1(CORBA::Boolean &p1,CORBA::Octet p2,CORBA::UShort p3,CORBA::Boolean &p4)");
#endif
  
  ernie.seed("Plum1Call");
  ernie.mode(0);
  GetOctet(p2);
  ernie.mode(0);
  GetUShort(p3);
  ernie.mode(1);
  GetBoolean(p4);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Plum1Return");
  try {
    ReturnValue = tobj->Plum1(p1,p2,p3,p4);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Plum1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckBoolean(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckBoolean(p4))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsKidneyBean1(void)
{
  
  CORBA::UShort ReturnValue;
  CORBA::ULong p1;
  CORBA::Char * p2;
  CORBA::Double p3;
  CORBA::Char * p4;
  CORBA::Boolean p5;
  CORBA::Char p6;
  CORBA::Short p7;
  CORBA::Octet p8;
  CORBA::Char * p9;
  CORBA::Octet p10;
  CORBA::Short p11;
  CORBA::UShort p12;
  CORBA::Float p13;
  CORBA::UShort p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::BasicTests::KidneyBean1(CORBA::ULong &p1,const CORBA::Char * p2,CORBA::Double &p3,CORBA::Char * &p4,CORBA::Boolean &p5,CORBA::Char &p6,CORBA::Short &p7,CORBA::Octet &p8,const CORBA::Char * p9,CORBA::Octet p10,CORBA::Short &p11,CORBA::UShort &p12,CORBA::Float &p13,CORBA::UShort &p14)");
#endif
  
  ernie.seed("KidneyBean1Call");
  ernie.mode(0);
  p2 = GetString();
  ernie.mode(0);
  p9 = GetString();
  ernie.mode(0);
  GetOctet(p10);
  ernie.mode(1);
  GetUShort(p12);
  ernie.mode(1);
  GetUShort(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KidneyBean1Return");
  try {
    ReturnValue = tobj->KidneyBean1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::KidneyBean1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckULong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckDouble(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckString(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckBoolean(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckChar(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckShort(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckOctet(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckShort(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckUShort(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckFloat(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckUShort(p14))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsZucchini1(void)
{
  
  CORBA::UShort ReturnValue;
  CORBA::Long p1;
  CORBA::Char * p2;
  CORBA::Char * p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::BasicTests::Zucchini1(CORBA::Long &p1,CORBA::Char * &p2,const CORBA::Char * p3)");
#endif
  
  ernie.seed("Zucchini1Call");
  ernie.mode(1);
  GetLong(p1);
  ernie.mode(1);
  p2 = GetString();
  ernie.mode(0);
  p3 = GetString();

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Zucchini1Return");
  try {
    ReturnValue = tobj->Zucchini1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Zucchini1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckLong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckString(p2))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsButterBean1(void)
{
  
  CORBA::Char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::BasicTests::ButterBean1()");
#endif
  
  ernie.seed("ButterBean1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ButterBean1Return");
  try {
    ReturnValue = tobj->ButterBean1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::ButterBean1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsAvocado1(void)
{
  
  CORBA::Octet ReturnValue;
  CORBA::Boolean p1;
  CORBA::Char p2;
  CORBA::Char p3;
  CORBA::Octet p4;
  CORBA::Char p5;
  CORBA::Boolean p6;
  CORBA::Double p7;
  CORBA::Boolean p8;
  CORBA::Char * p9;
  CORBA::Long p10;
  CORBA::UShort p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Octet ::BasicTests::Avocado1(CORBA::Boolean &p1,CORBA::Char p2,CORBA::Char &p3,CORBA::Octet &p4,CORBA::Char p5,CORBA::Boolean &p6,CORBA::Double p7,CORBA::Boolean &p8,const CORBA::Char * p9,CORBA::Long &p10,CORBA::UShort &p11)");
#endif
  
  ernie.seed("Avocado1Call");
  ernie.mode(0);
  GetChar(p2);
  ernie.mode(1);
  GetOctet(p4);
  ernie.mode(0);
  GetChar(p5);
  ernie.mode(1);
  GetBoolean(p6);
  ernie.mode(0);
  GetDouble(p7);
  ernie.mode(1);
  GetBoolean(p8);
  ernie.mode(0);
  p9 = GetString();

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Avocado1Return");
  try {
    ReturnValue = tobj->Avocado1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Avocado1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOctet(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckBoolean(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckChar(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckOctet(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckBoolean(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckBoolean(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckLong(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckUShort(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsGoldenDelicous1(void)
{
  
  CORBA::Char * ReturnValue;
  CORBA::Boolean p1;
  CORBA::Long p2;
  CORBA::Octet p3;
  CORBA::Boolean p4;
  CORBA::Long p5;
  CORBA::UShort p6;
  CORBA::Short p7;
  CORBA::Float p8;
  CORBA::Boolean p9;
  CORBA::ULong p10;
  CORBA::Char * p11;
  CORBA::ULong p12;
  CORBA::Double p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::BasicTests::GoldenDelicous1(CORBA::Boolean p1,CORBA::Long p2,CORBA::Octet p3,CORBA::Boolean &p4,CORBA::Long &p5,CORBA::UShort &p6,CORBA::Short p7,CORBA::Float p8,CORBA::Boolean &p9,CORBA::ULong &p10,CORBA::Char * &p11,CORBA::ULong &p12,CORBA::Double p13)");
#endif
  
  ernie.seed("GoldenDelicous1Call");
  ernie.mode(0);
  GetBoolean(p1);
  ernie.mode(0);
  GetLong(p2);
  ernie.mode(0);
  GetOctet(p3);
  ernie.mode(0);
  GetShort(p7);
  ernie.mode(0);
  GetFloat(p8);
  ernie.mode(1);
  GetBoolean(p9);
  ernie.mode(1);
  GetULong(p10);
  ernie.mode(1);
  p11 = GetString();
  ernie.mode(1);
  GetULong(p12);
  ernie.mode(0);
  GetDouble(p13);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GoldenDelicous1Return");
  try {
    ReturnValue = tobj->GoldenDelicous1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::GoldenDelicous1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckBoolean(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckLong(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckUShort(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckBoolean(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckULong(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckString(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckULong(p12))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsLentil1(void)
{
  
  CORBA::Char * ReturnValue;
  CORBA::Char * p1;
  CORBA::Float p2;
  CORBA::Boolean p3;
  CORBA::Long p4;
  CORBA::Octet p5;
  CORBA::ULong p6;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::BasicTests::Lentil1(CORBA::Char * &p1,CORBA::Float p2,CORBA::Boolean &p3,CORBA::Long &p4,CORBA::Octet &p5,CORBA::ULong &p6)");
#endif
  
  ernie.seed("Lentil1Call");
  ernie.mode(1);
  p1 = GetString();
  ernie.mode(0);
  GetFloat(p2);
  ernie.mode(1);
  GetBoolean(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lentil1Return");
  try {
    ReturnValue = tobj->Lentil1(p1,p2,p3,p4,p5,p6);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Lentil1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckString(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckBoolean(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckLong(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckOctet(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckULong(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsFig1(void)
{
  
  CORBA::UShort ReturnValue;
  CORBA::Float p1;
  CORBA::Float p2;
  CORBA::Float p3;
  CORBA::Double p4;
  CORBA::Char p5;
  CORBA::Short p6;
  CORBA::ULong p7;
  CORBA::Double p8;
  CORBA::UShort p9;
  CORBA::Char p10;
  CORBA::ULong p11;
  CORBA::UShort p12;
  CORBA::Char * p13;
  CORBA::Float p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::BasicTests::Fig1(CORBA::Float &p1,CORBA::Float &p2,CORBA::Float p3,CORBA::Double p4,CORBA::Char &p5,CORBA::Short &p6,CORBA::ULong p7,CORBA::Double &p8,CORBA::UShort &p9,CORBA::Char &p10,CORBA::ULong &p11,CORBA::UShort p12,CORBA::Char * &p13,CORBA::Float p14)");
#endif
  
  ernie.seed("Fig1Call");
  ernie.mode(1);
  GetFloat(p2);
  ernie.mode(0);
  GetFloat(p3);
  ernie.mode(0);
  GetDouble(p4);
  ernie.mode(1);
  GetChar(p5);
  ernie.mode(1);
  GetShort(p6);
  ernie.mode(0);
  GetULong(p7);
  ernie.mode(1);
  GetULong(p11);
  ernie.mode(0);
  GetUShort(p12);
  ernie.mode(1);
  p13 = GetString();
  ernie.mode(0);
  GetFloat(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Fig1Return");
  try {
    ReturnValue = tobj->Fig1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Fig1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckFloat(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckFloat(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckChar(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckShort(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckDouble(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckUShort(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckChar(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckULong(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckString(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsCrabApple1(void)
{
  
  CORBA::Char ReturnValue;
  CORBA::Double p1;
  CORBA::Octet p2;
  CORBA::Boolean p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char ::BasicTests::CrabApple1(CORBA::Double p1,CORBA::Octet &p2,CORBA::Boolean &p3)");
#endif
  
  ernie.seed("CrabApple1Call");
  ernie.mode(0);
  GetDouble(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CrabApple1Return");
  try {
    ReturnValue = tobj->CrabApple1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::CrabApple1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChar(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckOctet(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckBoolean(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsRedOnion1(void)
{
  
  CORBA::Char * ReturnValue;
  CORBA::ULong p1;
  CORBA::Short p2;
  CORBA::Double p3;
  CORBA::Short p4;
  CORBA::Boolean p5;
  CORBA::Float p6;
  CORBA::Char p7;
  CORBA::Octet p8;
  CORBA::Double p9;
  CORBA::Octet p10;
  CORBA::Long p11;
  CORBA::Float p12;
  CORBA::UShort p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::BasicTests::RedOnion1(CORBA::ULong &p1,CORBA::Short &p2,CORBA::Double &p3,CORBA::Short &p4,CORBA::Boolean &p5,CORBA::Float p6,CORBA::Char p7,CORBA::Octet p8,CORBA::Double &p9,CORBA::Octet &p10,CORBA::Long &p11,CORBA::Float p12,CORBA::UShort &p13)");
#endif
  
  ernie.seed("RedOnion1Call");
  ernie.mode(1);
  GetULong(p1);
  ernie.mode(1);
  GetShort(p2);
  ernie.mode(0);
  GetFloat(p6);
  ernie.mode(0);
  GetChar(p7);
  ernie.mode(0);
  GetOctet(p8);
  ernie.mode(1);
  GetLong(p11);
  ernie.mode(0);
  GetFloat(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedOnion1Return");
  try {
    ReturnValue = tobj->RedOnion1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::RedOnion1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckULong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckShort(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckDouble(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckShort(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckBoolean(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckDouble(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckOctet(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckLong(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckUShort(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsHazelNut1(void)
{
  
  CORBA::Octet ReturnValue;
  CORBA::Short p1;
  CORBA::Octet p2;
  CORBA::Double p3;
  CORBA::Char * p4;
  CORBA::Short p5;
  CORBA::Double p6;
  CORBA::Double p7;
  CORBA::ULong p8;
  CORBA::Boolean p9;
  CORBA::ULong p10;
  CORBA::Float p11;
  CORBA::Float p12;
  CORBA::UShort p13;
  CORBA::Octet p14;
  CORBA::UShort p15;
  CORBA::Long p16;
  CORBA::Short p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Octet ::BasicTests::HazelNut1(CORBA::Short &p1,CORBA::Octet &p2,CORBA::Double p3,const CORBA::Char * p4,CORBA::Short p5,CORBA::Double &p6,CORBA::Double &p7,CORBA::ULong p8,CORBA::Boolean &p9,CORBA::ULong &p10,CORBA::Float &p11,CORBA::Float &p12,CORBA::UShort &p13,CORBA::Octet &p14,CORBA::UShort p15,CORBA::Long &p16,CORBA::Short p17)");
#endif
  
  ernie.seed("HazelNut1Call");
  ernie.mode(1);
  GetShort(p1);
  ernie.mode(1);
  GetOctet(p2);
  ernie.mode(0);
  GetDouble(p3);
  ernie.mode(0);
  p4 = GetString();
  ernie.mode(0);
  GetShort(p5);
  ernie.mode(1);
  GetDouble(p6);
  ernie.mode(1);
  GetDouble(p7);
  ernie.mode(0);
  GetULong(p8);
  ernie.mode(1);
  GetULong(p10);
  ernie.mode(1);
  GetFloat(p12);
  ernie.mode(1);
  GetUShort(p13);
  ernie.mode(0);
  GetUShort(p15);
  ernie.mode(0);
  GetShort(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("HazelNut1Return");
  try {
    ReturnValue = tobj->HazelNut1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::HazelNut1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOctet(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckShort(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckOctet(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckDouble(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckDouble(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckBoolean(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckULong(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckFloat(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckFloat(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckUShort(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckOctet(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckLong(p16))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsCinnamon1(void)
{
  
  CORBA::Short ReturnValue;
  CORBA::Long p1;
  CORBA::Short p2;
  CORBA::Octet p3;
  CORBA::ULong p4;
  CORBA::ULong p5;
  CORBA::Double p6;
  CORBA::Char p7;
  CORBA::Short p8;
  CORBA::Char p9;
  CORBA::UShort p10;
  CORBA::Char p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Short ::BasicTests::Cinnamon1(CORBA::Long &p1,CORBA::Short &p2,CORBA::Octet p3,CORBA::ULong p4,CORBA::ULong &p5,CORBA::Double p6,CORBA::Char &p7,CORBA::Short &p8,CORBA::Char p9,CORBA::UShort &p10,CORBA::Char &p11)");
#endif
  
  ernie.seed("Cinnamon1Call");
  ernie.mode(1);
  GetShort(p2);
  ernie.mode(0);
  GetOctet(p3);
  ernie.mode(0);
  GetULong(p4);
  ernie.mode(0);
  GetDouble(p6);
  ernie.mode(0);
  GetChar(p9);
  ernie.mode(1);
  GetUShort(p10);
  ernie.mode(1);
  GetChar(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cinnamon1Return");
  try {
    ReturnValue = tobj->Cinnamon1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Cinnamon1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckLong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckShort(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckULong(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckChar(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckShort(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckUShort(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckChar(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsGooseBerry1(void)
{
  
  CORBA::Long ReturnValue;
  CORBA::Boolean p1;
  CORBA::ULong p2;
  CORBA::Double p3;
  CORBA::UShort p4;
  CORBA::Char * p5;
  CORBA::Float p6;
  CORBA::UShort p7;
  CORBA::Long p8;
  CORBA::UShort p9;
  CORBA::Long p10;
  CORBA::Boolean p11;
  CORBA::Octet p12;
  CORBA::Short p13;
  CORBA::Char p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::BasicTests::GooseBerry1(CORBA::Boolean &p1,CORBA::ULong p2,CORBA::Double &p3,CORBA::UShort &p4,const CORBA::Char * p5,CORBA::Float &p6,CORBA::UShort p7,CORBA::Long &p8,CORBA::UShort p9,CORBA::Long &p10,CORBA::Boolean &p11,CORBA::Octet &p12,CORBA::Short &p13,CORBA::Char p14)");
#endif
  
  ernie.seed("GooseBerry1Call");
  ernie.mode(0);
  GetULong(p2);
  ernie.mode(1);
  GetDouble(p3);
  ernie.mode(0);
  p5 = GetString();
  ernie.mode(1);
  GetFloat(p6);
  ernie.mode(0);
  GetUShort(p7);
  ernie.mode(1);
  GetLong(p8);
  ernie.mode(0);
  GetUShort(p9);
  ernie.mode(1);
  GetLong(p10);
  ernie.mode(1);
  GetOctet(p12);
  ernie.mode(1);
  GetShort(p13);
  ernie.mode(0);
  GetChar(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GooseBerry1Return");
  try {
    ReturnValue = tobj->GooseBerry1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::GooseBerry1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckBoolean(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckDouble(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckUShort(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckFloat(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckLong(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckLong(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckBoolean(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckOctet(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckShort(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsEggPlant1(void)
{
  
  CORBA::Double ReturnValue;
  CORBA::Long p1;
  CORBA::UShort p2;
  CORBA::Short p3;
  CORBA::Short p4;
  CORBA::UShort p5;
  CORBA::Double p6;
  CORBA::Float p7;
  CORBA::Octet p8;
  CORBA::Char * p9;
  CORBA::Char p10;
  CORBA::Short p11;
  CORBA::ULong p12;
  CORBA::Short p13;
  CORBA::ULong p14;
  CORBA::Long p15;
  CORBA::Char p16;
  CORBA::ULong p17;
  CORBA::Float p18;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Double ::BasicTests::EggPlant1(CORBA::Long &p1,CORBA::UShort &p2,CORBA::Short &p3,CORBA::Short p4,CORBA::UShort &p5,CORBA::Double &p6,CORBA::Float &p7,CORBA::Octet p8,CORBA::Char * &p9,CORBA::Char &p10,CORBA::Short &p11,CORBA::ULong p12,CORBA::Short &p13,CORBA::ULong &p14,CORBA::Long &p15,CORBA::Char &p16,CORBA::ULong p17,CORBA::Float &p18)");
#endif
  
  ernie.seed("EggPlant1Call");
  ernie.mode(1);
  GetLong(p1);
  ernie.mode(1);
  GetUShort(p2);
  ernie.mode(0);
  GetShort(p4);
  ernie.mode(1);
  GetUShort(p5);
  ernie.mode(1);
  GetFloat(p7);
  ernie.mode(0);
  GetOctet(p8);
  ernie.mode(0);
  GetULong(p12);
  ernie.mode(1);
  GetShort(p13);
  ernie.mode(1);
  GetLong(p15);
  ernie.mode(0);
  GetULong(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("EggPlant1Return");
  try {
    ReturnValue = tobj->EggPlant1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::EggPlant1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDouble(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckLong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckUShort(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckShort(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckUShort(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckDouble(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckFloat(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckString(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckChar(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckShort(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckShort(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckULong(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckLong(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckChar(p16))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p18");
  if (!CheckFloat(p18))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsJuniperBerries1(void)
{
  
  CORBA::Short ReturnValue;
  CORBA::Double p1;
  CORBA::Boolean p2;
  CORBA::Octet p3;
  CORBA::Float p4;
  CORBA::Short p5;
  CORBA::UShort p6;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Short ::BasicTests::JuniperBerries1(CORBA::Double &p1,CORBA::Boolean &p2,CORBA::Octet &p3,CORBA::Float &p4,CORBA::Short &p5,CORBA::UShort &p6)");
#endif
  
  ernie.seed("JuniperBerries1Call");
  ernie.mode(1);
  GetOctet(p3);
  ernie.mode(1);
  GetFloat(p4);
  ernie.mode(1);
  GetUShort(p6);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("JuniperBerries1Return");
  try {
    ReturnValue = tobj->JuniperBerries1(p1,p2,p3,p4,p5,p6);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::JuniperBerries1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckDouble(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckBoolean(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckOctet(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckFloat(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckShort(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckUShort(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsCloves1(void)
{
  
  CORBA::UShort ReturnValue;
  CORBA::Double p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::BasicTests::Cloves1(CORBA::Double p1)");
#endif
  
  ernie.seed("Cloves1Call");
  ernie.mode(0);
  GetDouble(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cloves1Return");
  try {
    ReturnValue = tobj->Cloves1(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Cloves1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsDaikan1(void)
{
  
  CORBA::Char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::BasicTests::Daikan1()");
#endif
  
  ernie.seed("Daikan1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Daikan1Return");
  try {
    ReturnValue = tobj->Daikan1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Daikan1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsMustard1(void)
{
  
  CORBA::Float ReturnValue;
  CORBA::Octet p1;
  CORBA::Boolean p2;
  CORBA::Char * p3;
  CORBA::Double p4;
  CORBA::Float p5;
  CORBA::Char p6;
  CORBA::Char p7;
  CORBA::Octet p8;
  CORBA::Long p9;
  CORBA::UShort p10;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Float ::BasicTests::Mustard1(CORBA::Octet p1,CORBA::Boolean p2,const CORBA::Char * p3,CORBA::Double p4,CORBA::Float &p5,CORBA::Char p6,CORBA::Char &p7,CORBA::Octet p8,CORBA::Long &p9,CORBA::UShort &p10)");
#endif
  
  ernie.seed("Mustard1Call");
  ernie.mode(0);
  GetOctet(p1);
  ernie.mode(0);
  GetBoolean(p2);
  ernie.mode(0);
  p3 = GetString();
  ernie.mode(0);
  GetDouble(p4);
  ernie.mode(1);
  GetFloat(p5);
  ernie.mode(0);
  GetChar(p6);
  ernie.mode(0);
  GetOctet(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mustard1Return");
  try {
    ReturnValue = tobj->Mustard1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Mustard1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFloat(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckFloat(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckChar(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckLong(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckUShort(p10))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsStrawberry1(void)
{
  
  CORBA::UShort ReturnValue;
  CORBA::Long p1;
  CORBA::Char p2;
  CORBA::Double p3;
  CORBA::Char * p4;
  CORBA::Long p5;
  CORBA::Boolean p6;
  CORBA::Octet p7;
  CORBA::Long p8;
  CORBA::ULong p9;
  CORBA::Float p10;
  CORBA::UShort p11;
  CORBA::Short p12;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::BasicTests::Strawberry1(CORBA::Long p1,CORBA::Char &p2,CORBA::Double p3,const CORBA::Char * p4,CORBA::Long &p5,CORBA::Boolean &p6,CORBA::Octet &p7,CORBA::Long &p8,CORBA::ULong p9,CORBA::Float &p10,CORBA::UShort &p11,CORBA::Short &p12)");
#endif
  
  ernie.seed("Strawberry1Call");
  ernie.mode(0);
  GetLong(p1);
  ernie.mode(0);
  GetDouble(p3);
  ernie.mode(0);
  p4 = GetString();
  ernie.mode(1);
  GetLong(p5);
  ernie.mode(1);
  GetBoolean(p6);
  ernie.mode(1);
  GetOctet(p7);
  ernie.mode(0);
  GetULong(p9);
  ernie.mode(1);
  GetShort(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Strawberry1Return");
  try {
    ReturnValue = tobj->Strawberry1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Strawberry1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckChar(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckLong(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckBoolean(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckOctet(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckLong(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckFloat(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckUShort(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckShort(p12))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsBreadFruit1(void)
{
  
  CORBA::Char * ReturnValue;
  CORBA::Char p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::BasicTests::BreadFruit1(CORBA::Char p1)");
#endif
  
  ernie.seed("BreadFruit1Call");
  ernie.mode(0);
  GetChar(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BreadFruit1Return");
  try {
    ReturnValue = tobj->BreadFruit1(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::BreadFruit1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsThyme1(void)
{
  
  CORBA::UShort ReturnValue;
  CORBA::Long p1;
  CORBA::Float p2;
  CORBA::Char * p3;
  CORBA::Short p4;
  CORBA::Double p5;
  CORBA::Octet p6;
  CORBA::Char * p7;
  CORBA::Char p8;
  CORBA::Long p9;
  CORBA::Octet p10;
  CORBA::Long p11;
  CORBA::Boolean p12;
  CORBA::ULong p13;
  CORBA::ULong p14;
  CORBA::Long p15;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::BasicTests::Thyme1(CORBA::Long &p1,CORBA::Float &p2,CORBA::Char * &p3,CORBA::Short &p4,CORBA::Double p5,CORBA::Octet &p6,const CORBA::Char * p7,CORBA::Char &p8,CORBA::Long p9,CORBA::Octet p10,CORBA::Long &p11,CORBA::Boolean p12,CORBA::ULong &p13,CORBA::ULong &p14,CORBA::Long &p15)");
#endif
  
  ernie.seed("Thyme1Call");
  ernie.mode(1);
  GetLong(p1);
  ernie.mode(1);
  GetFloat(p2);
  ernie.mode(1);
  p3 = GetString();
  ernie.mode(0);
  GetDouble(p5);
  ernie.mode(0);
  p7 = GetString();
  ernie.mode(0);
  GetLong(p9);
  ernie.mode(0);
  GetOctet(p10);
  ernie.mode(1);
  GetLong(p11);
  ernie.mode(0);
  GetBoolean(p12);
  ernie.mode(1);
  GetLong(p15);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Thyme1Return");
  try {
    ReturnValue = tobj->Thyme1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Thyme1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckLong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckFloat(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckString(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckShort(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckOctet(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckChar(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckLong(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckULong(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckULong(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckLong(p15))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsSweedes1(void)
{
  
  CORBA::Char * ReturnValue;
  CORBA::Float p1;
  CORBA::Short p2;
  CORBA::Char p3;
  CORBA::Octet p4;
  CORBA::Short p5;
  CORBA::Float p6;
  CORBA::Long p7;
  CORBA::Short p8;
  CORBA::Float p9;
  CORBA::Boolean p10;
  CORBA::Char p11;
  CORBA::Boolean p12;
  CORBA::Short p13;
  CORBA::Char p14;
  CORBA::Long p15;
  CORBA::Float p16;
  CORBA::Long p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::BasicTests::Sweedes1(CORBA::Float &p1,CORBA::Short p2,CORBA::Char p3,CORBA::Octet &p4,CORBA::Short p5,CORBA::Float &p6,CORBA::Long p7,CORBA::Short &p8,CORBA::Float &p9,CORBA::Boolean &p10,CORBA::Char p11,CORBA::Boolean &p12,CORBA::Short p13,CORBA::Char &p14,CORBA::Long &p15,CORBA::Float p16,CORBA::Long &p17)");
#endif
  
  ernie.seed("Sweedes1Call");
  ernie.mode(1);
  GetFloat(p1);
  ernie.mode(0);
  GetShort(p2);
  ernie.mode(0);
  GetChar(p3);
  ernie.mode(0);
  GetShort(p5);
  ernie.mode(0);
  GetLong(p7);
  ernie.mode(1);
  GetShort(p8);
  ernie.mode(1);
  GetFloat(p9);
  ernie.mode(1);
  GetBoolean(p10);
  ernie.mode(0);
  GetChar(p11);
  ernie.mode(0);
  GetShort(p13);
  ernie.mode(1);
  GetChar(p14);
  ernie.mode(0);
  GetFloat(p16);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sweedes1Return");
  try {
    ReturnValue = tobj->Sweedes1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Sweedes1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckFloat(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckOctet(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckFloat(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckShort(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckFloat(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckBoolean(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckBoolean(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckChar(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckLong(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p17");
  if (!CheckLong(p17))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsPomegranate1(void)
{
  
  CORBA::Double ReturnValue;
  CORBA::UShort p1;
  CORBA::Boolean p2;
  CORBA::Char * p3;
  CORBA::Double p4;
  CORBA::Long p5;
  CORBA::ULong p6;
  CORBA::Char p7;
  CORBA::Short p8;
  CORBA::Char * p9;
  CORBA::Long p10;
  CORBA::ULong p11;
  CORBA::Float p12;
  CORBA::Double p13;
  CORBA::Octet p14;
  CORBA::Long p15;
  CORBA::Boolean p16;
  CORBA::ULong p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Double ::BasicTests::Pomegranate1(CORBA::UShort &p1,CORBA::Boolean &p2,const CORBA::Char * p3,CORBA::Double p4,CORBA::Long &p5,CORBA::ULong &p6,CORBA::Char &p7,CORBA::Short &p8,CORBA::Char * &p9,CORBA::Long p10,CORBA::ULong p11,CORBA::Float &p12,CORBA::Double &p13,CORBA::Octet &p14,CORBA::Long &p15,CORBA::Boolean p16,CORBA::ULong p17)");
#endif
  
  ernie.seed("Pomegranate1Call");
  ernie.mode(0);
  p3 = GetString();
  ernie.mode(0);
  GetDouble(p4);
  ernie.mode(1);
  GetLong(p5);
  ernie.mode(1);
  GetShort(p8);
  ernie.mode(0);
  GetLong(p10);
  ernie.mode(0);
  GetULong(p11);
  ernie.mode(1);
  GetFloat(p12);
  ernie.mode(1);
  GetDouble(p13);
  ernie.mode(1);
  GetOctet(p14);
  ernie.mode(0);
  GetBoolean(p16);
  ernie.mode(0);
  GetULong(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pomegranate1Return");
  try {
    ReturnValue = tobj->Pomegranate1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Pomegranate1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDouble(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckUShort(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckBoolean(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckLong(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckULong(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckChar(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckShort(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckString(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckFloat(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckDouble(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckOctet(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckLong(p15))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsTarragon1(void)
{
  
  CORBA::ULong ReturnValue;
  CORBA::Long p1;
  CORBA::Octet p2;
  CORBA::Octet p3;
  CORBA::Long p4;
  CORBA::Octet p5;
  CORBA::ULong p6;
  CORBA::UShort p7;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::ULong ::BasicTests::Tarragon1(CORBA::Long p1,CORBA::Octet p2,CORBA::Octet p3,CORBA::Long p4,CORBA::Octet &p5,CORBA::ULong p6,CORBA::UShort &p7)");
#endif
  
  ernie.seed("Tarragon1Call");
  ernie.mode(0);
  GetLong(p1);
  ernie.mode(0);
  GetOctet(p2);
  ernie.mode(0);
  GetOctet(p3);
  ernie.mode(0);
  GetLong(p4);
  ernie.mode(1);
  GetOctet(p5);
  ernie.mode(0);
  GetULong(p6);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Tarragon1Return");
  try {
    ReturnValue = tobj->Tarragon1(p1,p2,p3,p4,p5,p6,p7);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Tarragon1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckULong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckOctet(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckUShort(p7))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsLettuce1(void)
{
  
  CORBA::Char ReturnValue;
  CORBA::Char p1;
  CORBA::ULong p2;
  CORBA::Double p3;
  CORBA::Char p4;
  CORBA::Octet p5;
  CORBA::Char p6;
  CORBA::Char * p7;
  CORBA::Float p8;
  CORBA::Octet p9;
  CORBA::Boolean p10;
  CORBA::Boolean p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char ::BasicTests::Lettuce1(CORBA::Char &p1,CORBA::ULong p2,CORBA::Double p3,CORBA::Char &p4,CORBA::Octet p5,CORBA::Char &p6,CORBA::Char * &p7,CORBA::Float p8,CORBA::Octet p9,CORBA::Boolean p10,CORBA::Boolean &p11)");
#endif
  
  ernie.seed("Lettuce1Call");
  ernie.mode(0);
  GetULong(p2);
  ernie.mode(0);
  GetDouble(p3);
  ernie.mode(0);
  GetOctet(p5);
  ernie.mode(0);
  GetFloat(p8);
  ernie.mode(0);
  GetOctet(p9);
  ernie.mode(0);
  GetBoolean(p10);
  ernie.mode(1);
  GetBoolean(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lettuce1Return");
  try {
    ReturnValue = tobj->Lettuce1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Lettuce1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChar(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckChar(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckChar(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckChar(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckString(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckBoolean(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsMangetout1(void)
{
  
  CORBA::UShort ReturnValue;
  CORBA::Float p1;
  CORBA::Float p2;
  CORBA::UShort p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::BasicTests::Mangetout1(CORBA::Float &p1,CORBA::Float &p2,CORBA::UShort &p3)");
#endif
  
  ernie.seed("Mangetout1Call");
  ernie.mode(1);
  GetFloat(p2);
  ernie.mode(1);
  GetUShort(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mangetout1Return");
  try {
    ReturnValue = tobj->Mangetout1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Mangetout1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckFloat(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckFloat(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckUShort(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsCress1(void)
{
  
  CORBA::Double ReturnValue;
  CORBA::Char * p1;
  CORBA::Char p2;
  CORBA::Double p3;
  CORBA::Char p4;
  CORBA::Long p5;
  CORBA::Double p6;
  CORBA::Long p7;
  CORBA::ULong p8;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Double ::BasicTests::Cress1(CORBA::Char * &p1,CORBA::Char &p2,CORBA::Double &p3,CORBA::Char &p4,CORBA::Long p5,CORBA::Double &p6,CORBA::Long p7,CORBA::ULong p8)");
#endif
  
  ernie.seed("Cress1Call");
  ernie.mode(1);
  GetChar(p2);
  ernie.mode(1);
  GetChar(p4);
  ernie.mode(0);
  GetLong(p5);
  ernie.mode(1);
  GetDouble(p6);
  ernie.mode(0);
  GetLong(p7);
  ernie.mode(0);
  GetULong(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cress1Return");
  try {
    ReturnValue = tobj->Cress1(p1,p2,p3,p4,p5,p6,p7,p8);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Cress1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDouble(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckString(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckChar(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckDouble(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckChar(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckDouble(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsBambooShoot1(void)
{
  
  CORBA::Short ReturnValue;
  CORBA::Char * p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Short ::BasicTests::BambooShoot1(const CORBA::Char * p1)");
#endif
  
  ernie.seed("BambooShoot1Call");
  ernie.mode(0);
  p1 = GetString();

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BambooShoot1Return");
  try {
    ReturnValue = tobj->BambooShoot1(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::BambooShoot1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsCassava1(void)
{
  
  CORBA::Octet ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Octet ::BasicTests::Cassava1()");
#endif
  
  ernie.seed("Cassava1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cassava1Return");
  try {
    ReturnValue = tobj->Cassava1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Cassava1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOctet(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testBasicTestsMelon1(void)
{
  
  CORBA::Float ReturnValue;
  CORBA::UShort p1;
  CORBA::Boolean p2;
  CORBA::UShort p3;
  CORBA::Long p4;
  CORBA::Short p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Float ::BasicTests::Melon1(CORBA::UShort &p1,CORBA::Boolean &p2,CORBA::UShort &p3,CORBA::Long p4,CORBA::Short p5)");
#endif
  
  ernie.seed("Melon1Call");
  ernie.mode(1);
  GetBoolean(p2);
  ernie.mode(1);
  GetUShort(p3);
  ernie.mode(0);
  GetLong(p4);
  ernie.mode(0);
  GetShort(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Melon1Return");
  try {
    ReturnValue = tobj->Melon1(p1,p2,p3,p4,p5);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::BasicTests::Melon1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFloat(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckUShort(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckBoolean(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckUShort(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsRedGrapes1(void)
{
  
  CORBA::Float ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Float ::AttrTests::RedGrapes1()");
#endif
  
  ernie.seed("RedGrapes1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedGrapes1Return");
  try {
    ReturnValue = tobj->RedGrapes1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::RedGrapes1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFloat(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsRedGrapes1W(void)
{
  
  CORBA::Float _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::RedGrapes1(CORBA::Float _attr)");
#endif
  
  ernie.seed("RedGrapes1Call");
  ernie.mode(0);
  GetFloat(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedGrapes1Return");
  try {
     tobj->RedGrapes1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::RedGrapes1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsBeechNut1(void)
{
  
  CORBA::Double ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Double ::AttrTests::BeechNut1()");
#endif
  
  ernie.seed("BeechNut1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BeechNut1Return");
  try {
    ReturnValue = tobj->BeechNut1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::BeechNut1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDouble(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsBeechNut1W(void)
{
  
  CORBA::Double _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::BeechNut1(CORBA::Double _attr)");
#endif
  
  ernie.seed("BeechNut1Call");
  ernie.mode(0);
  GetDouble(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BeechNut1Return");
  try {
     tobj->BeechNut1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::BeechNut1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsElderberry1(void)
{
  
  CORBA::Long ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::AttrTests::Elderberry1()");
#endif
  
  ernie.seed("Elderberry1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Elderberry1Return");
  try {
    ReturnValue = tobj->Elderberry1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Elderberry1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsElderberry1W(void)
{
  
  CORBA::Long _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::Elderberry1(CORBA::Long _attr)");
#endif
  
  ernie.seed("Elderberry1Call");
  ernie.mode(0);
  GetLong(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Elderberry1Return");
  try {
     tobj->Elderberry1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Elderberry1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsKumquats1(void)
{
  
  CORBA::Short ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Short ::AttrTests::Kumquats1()");
#endif
  
  ernie.seed("Kumquats1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Kumquats1Return");
  try {
    ReturnValue = tobj->Kumquats1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Kumquats1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsKumquats1W(void)
{
  
  CORBA::Short _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::Kumquats1(CORBA::Short _attr)");
#endif
  
  ernie.seed("Kumquats1Call");
  ernie.mode(0);
  GetShort(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Kumquats1Return");
  try {
     tobj->Kumquats1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Kumquats1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsWalnuts1(void)
{
  
  CORBA::ULong ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::ULong ::AttrTests::Walnuts1()");
#endif
  
  ernie.seed("Walnuts1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Walnuts1Return");
  try {
    ReturnValue = tobj->Walnuts1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Walnuts1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckULong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsWalnuts1W(void)
{
  
  CORBA::ULong _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::Walnuts1(CORBA::ULong _attr)");
#endif
  
  ernie.seed("Walnuts1Call");
  ernie.mode(0);
  GetULong(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Walnuts1Return");
  try {
     tobj->Walnuts1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Walnuts1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsFennell1(void)
{
  
  CORBA::UShort ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::AttrTests::Fennell1()");
#endif
  
  ernie.seed("Fennell1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Fennell1Return");
  try {
    ReturnValue = tobj->Fennell1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Fennell1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsFennell1W(void)
{
  
  CORBA::UShort _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::Fennell1(CORBA::UShort _attr)");
#endif
  
  ernie.seed("Fennell1Call");
  ernie.mode(0);
  GetUShort(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Fennell1Return");
  try {
     tobj->Fennell1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Fennell1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsUglifruit1(void)
{
  
  CORBA::Char ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char ::AttrTests::Uglifruit1()");
#endif
  
  ernie.seed("Uglifruit1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Uglifruit1Return");
  try {
    ReturnValue = tobj->Uglifruit1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Uglifruit1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChar(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsUglifruit1W(void)
{
  
  CORBA::Char _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::Uglifruit1(CORBA::Char _attr)");
#endif
  
  ernie.seed("Uglifruit1Call");
  ernie.mode(0);
  GetChar(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Uglifruit1Return");
  try {
     tobj->Uglifruit1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Uglifruit1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsWaterchestnut1(void)
{
  
  CORBA::Boolean ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Boolean ::AttrTests::Waterchestnut1()");
#endif
  
  ernie.seed("Waterchestnut1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Waterchestnut1Return");
  try {
    ReturnValue = tobj->Waterchestnut1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Waterchestnut1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBoolean(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsWaterchestnut1W(void)
{
  
  CORBA::Boolean _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::Waterchestnut1(CORBA::Boolean _attr)");
#endif
  
  ernie.seed("Waterchestnut1Call");
  ernie.mode(0);
  GetBoolean(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Waterchestnut1Return");
  try {
     tobj->Waterchestnut1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Waterchestnut1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsChickpea1(void)
{
  
  CORBA::Octet ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Octet ::AttrTests::Chickpea1()");
#endif
  
  ernie.seed("Chickpea1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Chickpea1Return");
  try {
    ReturnValue = tobj->Chickpea1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Chickpea1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOctet(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsChickpea1W(void)
{
  
  CORBA::Octet _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::Chickpea1(CORBA::Octet _attr)");
#endif
  
  ernie.seed("Chickpea1Call");
  ernie.mode(0);
  GetOctet(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Chickpea1Return");
  try {
     tobj->Chickpea1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Chickpea1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsBeetroot1(void)
{
  
  CORBA::Object_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Object_ptr ::AttrTests::Beetroot1()");
#endif
  
  ernie.seed("Beetroot1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Beetroot1Return");
  try {
    ReturnValue = tobj->Beetroot1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Beetroot1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!Checkobject(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsBeetroot1W(void)
{
  
  CORBA::Object_ptr _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::Beetroot1(CORBA::Object_ptr _attr)");
#endif
  
  ernie.seed("Beetroot1Call");
  ernie.mode(0);
  _attr = Getobject();

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Beetroot1Return");
  try {
     tobj->Beetroot1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Beetroot1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsOkra1(void)
{
  
  CORBA::Char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::AttrTests::Okra1()");
#endif
  
  ernie.seed("Okra1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Okra1Return");
  try {
    ReturnValue = tobj->Okra1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Okra1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsOkra1W(void)
{
  
  CORBA::Char * _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::Okra1(const CORBA::Char * _attr)");
#endif
  
  ernie.seed("Okra1Call");
  ernie.mode(0);
  _attr = GetString();

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Okra1Return");
  try {
     tobj->Okra1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Okra1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsPlantain1(void)
{
  
  CORBA::Char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::AttrTests::Plantain1()");
#endif
  
  ernie.seed("Plantain1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Plantain1Return");
  try {
    ReturnValue = tobj->Plantain1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Plantain1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue,43))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsPlantain1W(void)
{
  
  CORBA::Char * _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AttrTests::Plantain1(const CORBA::Char * _attr)");
#endif
  
  ernie.seed("Plantain1Call");
  ernie.mode(0);
  _attr = GetString(43);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Plantain1Return");
  try {
     tobj->Plantain1(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Plantain1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsBlackcurrant1(void)
{
  
  CORBA::Float ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Float ::AttrTests::Blackcurrant1()");
#endif
  
  ernie.seed("Blackcurrant1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Blackcurrant1Return");
  try {
    ReturnValue = tobj->Blackcurrant1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Blackcurrant1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFloat(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsPineapple1(void)
{
  
  CORBA::Double ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Double ::AttrTests::Pineapple1()");
#endif
  
  ernie.seed("Pineapple1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pineapple1Return");
  try {
    ReturnValue = tobj->Pineapple1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Pineapple1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDouble(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsLimaBean1(void)
{
  
  CORBA::Long ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Long ::AttrTests::LimaBean1()");
#endif
  
  ernie.seed("LimaBean1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LimaBean1Return");
  try {
    ReturnValue = tobj->LimaBean1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::LimaBean1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsMangosteen1(void)
{
  
  CORBA::Short ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Short ::AttrTests::Mangosteen1()");
#endif
  
  ernie.seed("Mangosteen1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mangosteen1Return");
  try {
    ReturnValue = tobj->Mangosteen1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Mangosteen1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsPear1(void)
{
  
  CORBA::ULong ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::ULong ::AttrTests::Pear1()");
#endif
  
  ernie.seed("Pear1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pear1Return");
  try {
    ReturnValue = tobj->Pear1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Pear1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckULong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsRedPepper1(void)
{
  
  CORBA::UShort ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::AttrTests::RedPepper1()");
#endif
  
  ernie.seed("RedPepper1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedPepper1Return");
  try {
    ReturnValue = tobj->RedPepper1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::RedPepper1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsBeefTomatoes1(void)
{
  
  CORBA::Char ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char ::AttrTests::BeefTomatoes1()");
#endif
  
  ernie.seed("BeefTomatoes1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BeefTomatoes1Return");
  try {
    ReturnValue = tobj->BeefTomatoes1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::BeefTomatoes1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChar(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsFrenchBeans1(void)
{
  
  CORBA::Boolean ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Boolean ::AttrTests::FrenchBeans1()");
#endif
  
  ernie.seed("FrenchBeans1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("FrenchBeans1Return");
  try {
    ReturnValue = tobj->FrenchBeans1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::FrenchBeans1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBoolean(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsSharanfruit1(void)
{
  
  CORBA::Octet ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Octet ::AttrTests::Sharanfruit1()");
#endif
  
  ernie.seed("Sharanfruit1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sharanfruit1Return");
  try {
    ReturnValue = tobj->Sharanfruit1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Sharanfruit1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOctet(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsRadish1(void)
{
  
  CORBA::Object_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Object_ptr ::AttrTests::Radish1()");
#endif
  
  ernie.seed("Radish1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Radish1Return");
  try {
    ReturnValue = tobj->Radish1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Radish1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!Checkobject(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsCherry1(void)
{
  
  CORBA::Char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::AttrTests::Cherry1()");
#endif
  
  ernie.seed("Cherry1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cherry1Return");
  try {
    ReturnValue = tobj->Cherry1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Cherry1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAttrTestsPapaya1(void)
{
  
  CORBA::Char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::AttrTests::Papaya1()");
#endif
  
  ernie.seed("Papaya1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Papaya1Return");
  try {
    ReturnValue = tobj->Papaya1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AttrTests::Papaya1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue,25))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsSusander1(void)
{
  
  ::Record2_slice * ReturnValue;
  CORBA::Char * p1;
  ::Radish p2;
  ::GaliaMelon3 p3;
  ::Rasin2 p4;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Record2_slice * ::ConstructedTypeTests::Susander1(CORBA::Char * &p1,const ::Radish &p2,const ::GaliaMelon3 &p3,::Rasin2 &p4)");
#endif
  
  ernie.seed("Susander1Call");
  ernie.mode(0);
  GetRadish(p2);
  ernie.mode(0);
  GetGaliaMelon3(p3);
  ernie.mode(1);
  GetRasin2(p4);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Susander1Return");
  try {
    ReturnValue = tobj->Susander1(p1,p2,p3,p4);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Susander1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRecord2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckString(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckRasin2(p4))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsGrapefruit1(void)
{
  
  ::Raspberry1_slice * ReturnValue;
  
  ::Fig* p1;
  ::Chestnut1 p2;
  ::Squash2* p3;
  ::GoldenDelicous3* p4;
  
  ::Fig* p5;
  
  ::Peppercorn p6;
  CORBA::UShort p7;
  ::Cauliflower2* p8;
  ::BlackEyedBeans p9;
  ::WaterMelon3 p10;
  
  ::Turnip3* p11;
  
  ::Fig p12;
  ::Cauliflower2* p13;
  ::Radish p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Raspberry1_slice * ::ConstructedTypeTests::Grapefruit1(::Fig *&p1,const ::Chestnut1 p2,::Squash2 *&p3,::GoldenDelicous3 *&p4,::Fig *&p5,::Peppercorn &p6,CORBA::UShort &p7,::Cauliflower2 *&p8,const ::BlackEyedBeans &p9,const ::WaterMelon3 p10,::Turnip3 *&p11,::Fig &p12,::Cauliflower2 *&p13,::Radish &p14)");
#endif
  
  ernie.seed("Grapefruit1Call");
  ernie.mode(0);
  GetChestnut1(p2);
  ernie.mode(0);
  GetBlackEyedBeans(p9);
  ernie.mode(0);
  GetWaterMelon3(p10);
  ernie.mode(1);
  GetFig(p12);
  ernie.mode(1);
  GetRadish(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Grapefruit1Return");
  try {
    ReturnValue = tobj->Grapefruit1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Grapefruit1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRaspberry1(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckFig(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckSquash2(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckGoldenDelicous3(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckFig(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckPeppercorn(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckUShort(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckCauliflower2(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckTurnip3(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckFig(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckCauliflower2(*p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckRadish(p14))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsPepinoMelon1(void)
{
  
  CORBA::UShort ReturnValue;
  CORBA::Long p1;
  ::Tomato2 p2;
  
  ::Fig p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::UShort ::ConstructedTypeTests::PepinoMelon1(CORBA::Long &p1,::Tomato2 &p2,const ::Fig &p3)");
#endif
  
  ernie.seed("PepinoMelon1Call");
  ernie.mode(1);
  GetLong(p1);
  ernie.mode(1);
  GetTomato2(p2);
  ernie.mode(0);
  GetFig(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PepinoMelon1Return");
  try {
    ReturnValue = tobj->PepinoMelon1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::PepinoMelon1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckUShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckLong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckTomato2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsPeanutsquash1(void)
{
  
  ::Cauliflower2* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Cauliflower2* ::ConstructedTypeTests::Peanutsquash1()");
#endif
  
  ernie.seed("Peanutsquash1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Peanutsquash1Return");
  try {
    ReturnValue = tobj->Peanutsquash1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Peanutsquash1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCauliflower2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsArtichokes1(void)
{
  
  ::Tomato2* ReturnValue;
  ::Squash1* p1;
  CORBA::ULong p2;
  CORBA::ULong p3;
  ::GardenPeas1 p4;
  ::Tomato2 p5;
  ::YellowPepper2 p6;
  ::Dates3 p7;
  ::Radish p8;
  
  ::Blackberry2 p9;
  ::Rasin2* p10;
  ::Mangosteen3* p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Tomato2* ::ConstructedTypeTests::Artichokes1(::Squash1 *&p1,CORBA::ULong p2,CORBA::ULong &p3,::GardenPeas1 &p4,const ::Tomato2 &p5,::YellowPepper2 &p6,const ::Dates3 &p7,::Radish &p8,const ::Blackberry2 &p9,::Rasin2 *&p10,::Mangosteen3 *&p11)");
#endif
  
  ernie.seed("Artichokes1Call");
  ernie.mode(0);
  GetULong(p2);
  ernie.mode(1);
  GetGardenPeas1(p4);
  ernie.mode(0);
  GetTomato2(p5);
  ernie.mode(1);
  GetYellowPepper2(p6);
  ernie.mode(0);
  GetDates3(p7);
  ernie.mode(1);
  GetRadish(p8);
  ernie.mode(0);
  GetBlackberry2(p9);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Artichokes1Return");
  try {
    ReturnValue = tobj->Artichokes1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Artichokes1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTomato2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckSquash1(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckULong(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckGardenPeas1(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckYellowPepper2(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckRadish(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckRasin2(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckMangosteen3(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsRasin1(void)
{
  
  ::Damsons2_slice * ReturnValue;
  ::Raspberry1 p1;
  ::Mangosteen3 p2;
  
  ::Fig p3;
  ::Lychees3* p4;
  ::Carrots p5;
  ::Rasin2* p6;
  ::GoldenDelicous3 p7;
  ::Mullberry1 p8;
  ::Squash1 p9;
  ::YellowPepper2 p10;
  ::YellowPepper2 p11;
  ::GardenPeas3 p12;
  ::Tomato2 p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Damsons2_slice * ::ConstructedTypeTests::Rasin1(const ::Raspberry1 p1,const ::Mangosteen3 &p2,const ::Fig &p3,::Lychees3 *&p4,::Carrots p5,::Rasin2 *&p6,const ::GoldenDelicous3 &p7,const ::Mullberry1 p8,::Squash1 &p9,::YellowPepper2 &p10,::YellowPepper2 &p11,::GardenPeas3 &p12,const ::Tomato2 &p13)");
#endif
  
  ernie.seed("Rasin1Call");
  ernie.mode(0);
  GetRaspberry1(p1);
  ernie.mode(0);
  GetMangosteen3(p2);
  ernie.mode(0);
  GetFig(p3);
  ernie.mode(0);
  GetGoldenDelicous3(p7);
  ernie.mode(0);
  GetMullberry1(p8);
  ernie.mode(1);
  GetSquash1(p9);
  ernie.mode(1);
  GetYellowPepper2(p10);
  ernie.mode(1);
  GetYellowPepper2(p11);
  ernie.mode(1);
  GetGardenPeas3(p12);
  ernie.mode(0);
  GetTomato2(p13);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Rasin1Return");
  try {
    ReturnValue = tobj->Rasin1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Rasin1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDamsons2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckLychees3(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckCarrots(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckRasin2(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckSquash1(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckYellowPepper2(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckYellowPepper2(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckGardenPeas3(p12))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsChives1(void)
{
  
  ::Chives2* ReturnValue;
  
  ::Tomato p1;
  ::Peanuts1 p2;
  
  ::Clementine2 p3;
  ::Squash1* p4;
  ::Squash2* p5;
  ::Oregano3* p6;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Chives2* ::ConstructedTypeTests::Chives1(::Tomato &p1,::Peanuts1 p2,::Clementine2 &p3,::Squash1 *&p4,::Squash2 *&p5,::Oregano3 *&p6)");
#endif
  
  ernie.seed("Chives1Call");
  ernie.mode(1);
  GetTomato(p1);
  ernie.mode(0);
  GetPeanuts1(p2);
  ernie.mode(1);
  GetClementine2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Chives1Return");
  try {
    ReturnValue = tobj->Chives1(p1,p2,p3,p4,p5,p6);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Chives1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChives2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckTomato(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckClementine2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckSquash1(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckSquash2(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckOregano3(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsEddo1(void)
{
  
  ::Mullberry1_slice * ReturnValue;
  ::Chestnut1 p1;
  ::Dates3 p2;
  ::Oregano3 p3;
  ::Rasin2 p4;
  ::Mangosteen3 p5;
  ::Squash2 p6;
  ::Oregano3 p7;
  ::Tomato2* p8;
  ::BambooShoot3* p9;
  
  ::Turnip3* p10;
  ::Rasin2 p11;
  ::Fennell3 p12;
  ::GoldenDelicous3 p13;
  ::Mullberry1 p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mullberry1_slice * ::ConstructedTypeTests::Eddo1(::Chestnut1 p1,::Dates3 &p2,const ::Oregano3 &p3,const ::Rasin2 &p4,::Mangosteen3 &p5,::Squash2 &p6,const ::Oregano3 &p7,::Tomato2 *&p8,::BambooShoot3 *&p9,::Turnip3 *&p10,::Rasin2 &p11,const ::Fennell3 &p12,::GoldenDelicous3 &p13,const ::Mullberry1 p14)");
#endif
  
  ernie.seed("Eddo1Call");
  ernie.mode(1);
  GetDates3(p2);
  ernie.mode(0);
  GetOregano3(p3);
  ernie.mode(0);
  GetRasin2(p4);
  ernie.mode(1);
  GetMangosteen3(p5);
  ernie.mode(1);
  GetSquash2(p6);
  ernie.mode(0);
  GetOregano3(p7);
  ernie.mode(1);
  GetRasin2(p11);
  ernie.mode(0);
  GetFennell3(p12);
  ernie.mode(1);
  GetGoldenDelicous3(p13);
  ernie.mode(0);
  GetMullberry1(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Eddo1Return");
  try {
    ReturnValue = tobj->Eddo1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Eddo1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMullberry1(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckChestnut1(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckDates3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckMangosteen3(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckSquash2(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckTomato2(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckBambooShoot3(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckTurnip3(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckRasin2(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckGoldenDelicous3(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsGreenChilly1(void)
{
  
  ::Chives2* ReturnValue;
  ::Chestnut1 p1;
  ::Carrots p2;
  ::Dates3* p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Chives2* ::ConstructedTypeTests::GreenChilly1(const ::Chestnut1 p1,::Carrots p2,::Dates3 *&p3)");
#endif
  
  ernie.seed("GreenChilly1Call");
  ernie.mode(0);
  GetChestnut1(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GreenChilly1Return");
  try {
    ReturnValue = tobj->GreenChilly1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::GreenChilly1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChives2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckCarrots(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckDates3(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsPeach1(void)
{
  
  ::Corn1_slice * ReturnValue;
  CORBA::Long p1;
  ::Tomato2 p2;
  
  ::Tomato* p3;
  ::Asparagus* p4;
  ::Cauliflower2* p5;
  ::Raspberry1 p6;
  
  ::Peppercorn p7;
  ::WaterMelon p8;
  ::Cauliflower2* p9;
  ::Chives2* p10;
  ::Mullberry1 p11;
  
  ::Blackberry2 p12;
  ::Fennell3* p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Corn1_slice * ::ConstructedTypeTests::Peach1(CORBA::Long &p1,::Tomato2 &p2,::Tomato *&p3,::Asparagus *&p4,::Cauliflower2 *&p5,const ::Raspberry1 p6,const ::Peppercorn &p7,const ::WaterMelon &p8,::Cauliflower2 *&p9,::Chives2 *&p10,::Mullberry1 p11,const ::Blackberry2 &p12,::Fennell3 *&p13)");
#endif
  
  ernie.seed("Peach1Call");
  ernie.mode(1);
  GetLong(p1);
  ernie.mode(1);
  GetTomato2(p2);
  ernie.mode(0);
  GetRaspberry1(p6);
  ernie.mode(0);
  GetPeppercorn(p7);
  ernie.mode(0);
  GetWaterMelon(p8);
  ernie.mode(1);
  GetMullberry1(p11);
  ernie.mode(0);
  GetBlackberry2(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Peach1Return");
  try {
    ReturnValue = tobj->Peach1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Peach1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCorn1(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckLong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckTomato2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckTomato(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckAsparagus(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckCauliflower2(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCauliflower2(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckChives2(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckMullberry1(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckFennell3(*p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsParsley1(void)
{
  
  ::Squash1* ReturnValue;
  ::GardenPeas3 p1;
  ::Squash2 p2;
  ::Dates3 p3;
  ::Peanuts1 p4;
  CORBA::Long p5;
  ::Grapefruit2 p6;
  ::WaterMelon3 p7;
  CORBA::ULong p8;
  ::WaterMelon3 p9;
  ::Squash2 p10;
  ::Radish* p11;
  ::Mangosteen3 p12;
  ::BlackEyedBeans p13;
  CORBA::UShort p14;
  ::BritishQueens1 p15;
  ::YellowPepper2* p16;
  CORBA::UShort p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Squash1* ::ConstructedTypeTests::Parsley1(::GardenPeas3 &p1,::Squash2 &p2,const ::Dates3 &p3,::Peanuts1 p4,CORBA::Long p5,::Grapefruit2 &p6,::WaterMelon3 p7,CORBA::ULong p8,::WaterMelon3 p9,::Squash2 &p10,::Radish *&p11,::Mangosteen3 &p12,::BlackEyedBeans &p13,CORBA::UShort &p14,const ::BritishQueens1 &p15,::YellowPepper2 *&p16,CORBA::UShort p17)");
#endif
  
  ernie.seed("Parsley1Call");
  ernie.mode(1);
  GetGardenPeas3(p1);
  ernie.mode(1);
  GetSquash2(p2);
  ernie.mode(0);
  GetDates3(p3);
  ernie.mode(0);
  GetPeanuts1(p4);
  ernie.mode(0);
  GetLong(p5);
  ernie.mode(1);
  GetGrapefruit2(p6);
  ernie.mode(1);
  GetWaterMelon3(p7);
  ernie.mode(0);
  GetULong(p8);
  ernie.mode(1);
  GetSquash2(p10);
  ernie.mode(1);
  GetMangosteen3(p12);
  ernie.mode(1);
  GetBlackEyedBeans(p13);
  ernie.mode(0);
  GetBritishQueens1(p15);
  ernie.mode(0);
  GetUShort(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsley1Return");
  try {
    ReturnValue = tobj->Parsley1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Parsley1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSquash1(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckGardenPeas3(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckSquash2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckGrapefruit2(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckWaterMelon3(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckWaterMelon3(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckSquash2(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckRadish(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckMangosteen3(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckBlackEyedBeans(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckUShort(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckYellowPepper2(*p16))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsSatsuma1(void)
{
  
  ::GaliaMelon3* ReturnValue;
  ::Pumpkin p1;
  CORBA::Short p2;
  ::Mullberry1 p3;
  
  ::Clementine2 p4;
  ::BambooShoot3* p5;
  ::BlackEyedBeans p6;
  ::Raspberry1 p7;
  ::Oregano3* p8;
  ::Record2 p9;
  ::Celeriac2 p10;
  ::Grapefruit2 p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GaliaMelon3* ::ConstructedTypeTests::Satsuma1(::Pumpkin p1,CORBA::Short &p2,const ::Mullberry1 p3,const ::Clementine2 &p4,::BambooShoot3 *&p5,const ::BlackEyedBeans &p6,::Raspberry1 p7,::Oregano3 *&p8,const ::Record2 p9,::Celeriac2 &p10,::Grapefruit2 &p11)");
#endif
  
  ernie.seed("Satsuma1Call");
  ernie.mode(1);
  GetShort(p2);
  ernie.mode(0);
  GetMullberry1(p3);
  ernie.mode(0);
  GetClementine2(p4);
  ernie.mode(0);
  GetBlackEyedBeans(p6);
  ernie.mode(0);
  GetRecord2(p9);
  ernie.mode(1);
  GetCeleriac2(p10);
  ernie.mode(1);
  GetGrapefruit2(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Satsuma1Return");
  try {
    ReturnValue = tobj->Satsuma1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Satsuma1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGaliaMelon3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckPumpkin(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckShort(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckBambooShoot3(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckRaspberry1(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckOregano3(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckCeleriac2(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckGrapefruit2(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsBasil1(void)
{
  
  ::Peanuts1 ReturnValue;
  ::Mullberry1 p1;
  ::Dates3 p2;
  
  ::Blackberry2 p3;
  ::Squash2* p4;
  ::Damsons2 p5;
  ::Corn1 p6;
  ::Oregano3 p7;
  
  ::Turnip3 p8;
  ::Mullberry1 p9;
  
  ::Clementine2 p10;
  ::Grapefruit2* p11;
  ::Peanuts1 p12;
  ::Tomato2 p13;
  ::Dates3 p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Peanuts1 ::ConstructedTypeTests::Basil1(::Mullberry1 p1,const ::Dates3 &p2,::Blackberry2 &p3,::Squash2 *&p4,const ::Damsons2 p5,::Corn1 p6,const ::Oregano3 &p7,::Turnip3 &p8,const ::Mullberry1 p9,::Clementine2 &p10,::Grapefruit2 *&p11,::Peanuts1 &p12,::Tomato2 &p13,const ::Dates3 &p14)");
#endif
  
  ernie.seed("Basil1Call");
  ernie.mode(0);
  GetDates3(p2);
  ernie.mode(1);
  GetBlackberry2(p3);
  ernie.mode(0);
  GetDamsons2(p5);
  ernie.mode(1);
  GetCorn1(p6);
  ernie.mode(0);
  GetOregano3(p7);
  ernie.mode(1);
  GetTurnip3(p8);
  ernie.mode(0);
  GetMullberry1(p9);
  ernie.mode(1);
  GetClementine2(p10);
  ernie.mode(1);
  GetPeanuts1(p12);
  ernie.mode(1);
  GetTomato2(p13);
  ernie.mode(0);
  GetDates3(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Basil1Return");
  try {
    ReturnValue = tobj->Basil1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Basil1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckPeanuts1(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckMullberry1(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckBlackberry2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckSquash2(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckCorn1(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckTurnip3(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckClementine2(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckGrapefruit2(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckPeanuts1(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckTomato2(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsCoxPippin1(void)
{
  
  CORBA::Octet ReturnValue;
  ::YellowPepper2 p1;
  ::BrazilNut3 p2;
  ::Grapefruit2* p3;
  CORBA::Double p4;
  ::Corn1 p5;
  
  ::Fig* p6;
  
  ::Susander2 p7;
  ::Peanuts1 p8;
  ::Mullberry1 p9;
  ::Celeriac2* p10;
  ::Cauliflower2* p11;
  ::Beetroot2 p12;
  
  ::Clementine2 p13;
  ::WaterMelon3 p14;
  ::Dates3 p15;
  ::Squash2* p16;
  CORBA::ULong p17;
  ::Damsons2 p18;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Octet ::ConstructedTypeTests::CoxPippin1(::YellowPepper2 &p1,::BrazilNut3 &p2,::Grapefruit2 *&p3,CORBA::Double p4,::Corn1 p5,::Fig *&p6,::Susander2 &p7,::Peanuts1 p8,::Mullberry1 p9,::Celeriac2 *&p10,::Cauliflower2 *&p11,const ::Beetroot2 &p12,::Clementine2 &p13,::WaterMelon3 p14,::Dates3 &p15,::Squash2 *&p16,CORBA::ULong p17,::Damsons2 p18)");
#endif
  
  ernie.seed("CoxPippin1Call");
  ernie.mode(1);
  GetYellowPepper2(p1);
  ernie.mode(1);
  GetBrazilNut3(p2);
  ernie.mode(0);
  GetDouble(p4);
  ernie.mode(1);
  GetCorn1(p5);
  ernie.mode(1);
  GetSusander2(p7);
  ernie.mode(0);
  GetPeanuts1(p8);
  ernie.mode(0);
  GetBeetroot2(p12);
  ernie.mode(1);
  GetClementine2(p13);
  ernie.mode(1);
  GetDates3(p15);
  ernie.mode(0);
  GetULong(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CoxPippin1Return");
  try {
    ReturnValue = tobj->CoxPippin1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::CoxPippin1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOctet(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckYellowPepper2(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckBrazilNut3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckGrapefruit2(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCorn1(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckFig(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckSusander2(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckMullberry1(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckCeleriac2(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckCauliflower2(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckClementine2(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckWaterMelon3(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckDates3(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckSquash2(*p16))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p18");
  if (!CheckDamsons2(p18))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsBlackberry1(void)
{
  
  ::Chives2* ReturnValue;
  ::Rasin2* p1;
  ::Squash1* p2;
  ::BritishQueens1 p3;
  ::Fennell3 p4;
  ::Tomato2* p5;
  
  ::Blackberry2 p6;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Chives2* ::ConstructedTypeTests::Blackberry1(::Rasin2 *&p1,::Squash1 *&p2,::BritishQueens1 &p3,::Fennell3 &p4,::Tomato2 *&p5,::Blackberry2 &p6)");
#endif
  
  ernie.seed("Blackberry1Call");
  ernie.mode(1);
  GetBritishQueens1(p3);
  ernie.mode(1);
  GetFennell3(p4);
  ernie.mode(1);
  GetBlackberry2(p6);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Blackberry1Return");
  try {
    ReturnValue = tobj->Blackberry1(p1,p2,p3,p4,p5,p6);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Blackberry1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChives2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckRasin2(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckSquash1(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckBritishQueens1(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckFennell3(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckTomato2(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckBlackberry2(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsGreenPepper1(void)
{
  
  ::Tomato2* ReturnValue;
  ::Mullberry1 p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Tomato2* ::ConstructedTypeTests::GreenPepper1(const ::Mullberry1 p1)");
#endif
  
  ernie.seed("GreenPepper1Call");
  ernie.mode(0);
  GetMullberry1(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GreenPepper1Return");
  try {
    ReturnValue = tobj->GreenPepper1(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::GreenPepper1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTomato2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsScallion1(void)
{
  
  ::Chives2* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Chives2* ::ConstructedTypeTests::Scallion1()");
#endif
  
  ernie.seed("Scallion1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Scallion1Return");
  try {
    ReturnValue = tobj->Scallion1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Scallion1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChives2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsCurrant1(void)
{
  
  ::YellowPepper2* ReturnValue;
  ::Asparagus p1;
  ::LoganBerry p2;
  ::Squash2 p3;
  ::Lychees3 p4;
  CORBA::ULong p5;
  ::Corn1 p6;
  ::BlackEyedBeans* p7;
  ::GardenPeas3 p8;
  ::Dates3* p9;
  CORBA::Float p10;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::YellowPepper2* ::ConstructedTypeTests::Currant1(const ::Asparagus &p1,const ::LoganBerry &p2,const ::Squash2 &p3,const ::Lychees3 &p4,CORBA::ULong &p5,const ::Corn1 p6,::BlackEyedBeans *&p7,const ::GardenPeas3 &p8,::Dates3 *&p9,CORBA::Float &p10)");
#endif
  
  ernie.seed("Currant1Call");
  ernie.mode(0);
  GetAsparagus(p1);
  ernie.mode(0);
  GetLoganBerry(p2);
  ernie.mode(0);
  GetSquash2(p3);
  ernie.mode(0);
  GetLychees3(p4);
  ernie.mode(1);
  GetULong(p5);
  ernie.mode(0);
  GetCorn1(p6);
  ernie.mode(0);
  GetGardenPeas3(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Currant1Return");
  try {
    ReturnValue = tobj->Currant1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Currant1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckYellowPepper2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckULong(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckBlackEyedBeans(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckDates3(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckFloat(p10))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsIcebergLettuce1(void)
{
  
  ::Chestnut1_slice * ReturnValue;
  
  ::Susander2 p1;
  ::Squash2* p2;
  CORBA::Boolean p3;
  
  ::Fig p4;
  CORBA::Long p5;
  CORBA::Double p6;
  ::Dates3 p7;
  ::Oregano3* p8;
  ::BritishQueens1 p9;
  ::Tomato2* p10;
  ::Squash2* p11;
  ::Peanuts1 p12;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Chestnut1_slice * ::ConstructedTypeTests::IcebergLettuce1(const ::Susander2 &p1,::Squash2 *&p2,CORBA::Boolean p3,const ::Fig &p4,CORBA::Long &p5,CORBA::Double &p6,::Dates3 &p7,::Oregano3 *&p8,const ::BritishQueens1 &p9,::Tomato2 *&p10,::Squash2 *&p11,::Peanuts1 &p12)");
#endif
  
  ernie.seed("IcebergLettuce1Call");
  ernie.mode(0);
  GetSusander2(p1);
  ernie.mode(0);
  GetBoolean(p3);
  ernie.mode(0);
  GetFig(p4);
  ernie.mode(1);
  GetLong(p5);
  ernie.mode(1);
  GetDouble(p6);
  ernie.mode(1);
  GetDates3(p7);
  ernie.mode(0);
  GetBritishQueens1(p9);
  ernie.mode(1);
  GetPeanuts1(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("IcebergLettuce1Return");
  try {
    ReturnValue = tobj->IcebergLettuce1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::IcebergLettuce1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChestnut1(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckSquash2(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckLong(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckDouble(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckDates3(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckOregano3(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckTomato2(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckSquash2(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckPeanuts1(p12))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsTomato1(void)
{
  
  ::Fennell3* ReturnValue;
  ::Squash2 p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Fennell3* ::ConstructedTypeTests::Tomato1(const ::Squash2 &p1)");
#endif
  
  ernie.seed("Tomato1Call");
  ernie.mode(0);
  GetSquash2(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Tomato1Return");
  try {
    ReturnValue = tobj->Tomato1(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Tomato1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFennell3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsPippinApple1(void)
{
  
  ::Celeriac2* ReturnValue;
  ::BambooShoot3 p1;
  ::BeechNut2 p2;
  CORBA::Float p3;
  ::Chestnut1 p4;
  ::BambooShoot3 p5;
  ::Corn1 p6;
  CORBA::Long p7;
  
  ::Blackberry2* p8;
  ::Tomato2 p9;
  
  ::Fig p10;
  ::Squash1 p11;
  
  ::Clementine2 p12;
  ::Squash1* p13;
  ::GaliaMelon3* p14;
  
  ::Fig p15;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Celeriac2* ::ConstructedTypeTests::PippinApple1(::BambooShoot3 &p1,::BeechNut2 &p2,CORBA::Float &p3,::Chestnut1 p4,const ::BambooShoot3 &p5,::Corn1 p6,CORBA::Long p7,::Blackberry2 *&p8,const ::Tomato2 &p9,const ::Fig &p10,::Squash1 &p11,const ::Clementine2 &p12,::Squash1 *&p13,::GaliaMelon3 *&p14,::Fig &p15)");
#endif
  
  ernie.seed("PippinApple1Call");
  ernie.mode(1);
  GetBambooShoot3(p1);
  ernie.mode(1);
  GetBeechNut2(p2);
  ernie.mode(1);
  GetFloat(p3);
  ernie.mode(0);
  GetBambooShoot3(p5);
  ernie.mode(0);
  GetLong(p7);
  ernie.mode(0);
  GetTomato2(p9);
  ernie.mode(0);
  GetFig(p10);
  ernie.mode(1);
  GetSquash1(p11);
  ernie.mode(0);
  GetClementine2(p12);
  ernie.mode(1);
  GetFig(p15);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PippinApple1Return");
  try {
    ReturnValue = tobj->PippinApple1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::PippinApple1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCeleriac2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckBambooShoot3(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckBeechNut2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckFloat(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckChestnut1(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckCorn1(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckBlackberry2(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckSquash1(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckSquash1(*p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckGaliaMelon3(*p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckFig(p15))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsCeleriac1(void)
{
  
  ::Rasin2* ReturnValue;
  ::Corn1 p1;
  ::Dates3 p2;
  ::Squash2 p3;
  
  ::Fig* p4;
  ::LoganBerry p5;
  ::BritishQueens1* p6;
  ::Record2 p7;
  ::Mullberry1 p8;
  ::YellowPepper2 p9;
  ::GaliaMelon3 p10;
  ::BrazilNut3 p11;
  CORBA::UShort p12;
  
  ::Fig p13;
  ::BeechNut2 p14;
  CORBA::Float p15;
  ::Grapefruit2 p16;
  CORBA::Long p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Rasin2* ::ConstructedTypeTests::Celeriac1(::Corn1 p1,const ::Dates3 &p2,const ::Squash2 &p3,::Fig *&p4,const ::LoganBerry &p5,::BritishQueens1 *&p6,const ::Record2 p7,::Mullberry1 p8,::YellowPepper2 &p9,::GaliaMelon3 &p10,const ::BrazilNut3 &p11,CORBA::UShort &p12,const ::Fig &p13,::BeechNut2 &p14,CORBA::Float &p15,const ::Grapefruit2 &p16,CORBA::Long &p17)");
#endif
  
  ernie.seed("Celeriac1Call");
  ernie.mode(1);
  GetCorn1(p1);
  ernie.mode(0);
  GetDates3(p2);
  ernie.mode(0);
  GetSquash2(p3);
  ernie.mode(0);
  GetLoganBerry(p5);
  ernie.mode(0);
  GetRecord2(p7);
  ernie.mode(1);
  GetMullberry1(p8);
  ernie.mode(1);
  GetYellowPepper2(p9);
  ernie.mode(1);
  GetGaliaMelon3(p10);
  ernie.mode(0);
  GetBrazilNut3(p11);
  ernie.mode(0);
  GetFig(p13);
  ernie.mode(1);
  GetBeechNut2(p14);
  ernie.mode(0);
  GetGrapefruit2(p16);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Celeriac1Return");
  try {
    ReturnValue = tobj->Celeriac1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Celeriac1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRasin2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckCorn1(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckFig(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckBritishQueens1(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckMullberry1(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckYellowPepper2(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckGaliaMelon3(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckUShort(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckBeechNut2(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckFloat(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p17");
  if (!CheckLong(p17))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsCauliflower1(void)
{
  
  ::Tomato2* ReturnValue;
  ::Dates3* p1;
  ::LoganBerry* p2;
  ::Lychees3 p3;
  ::Celeriac2 p4;
  ::BambooShoot3 p5;
  
  ::Susander2* p6;
  ::Peanuts1 p7;
  CORBA::Double p8;
  ::Currant2* p9;
  ::Tomato2 p10;
  ::BeechNut2 p11;
  CORBA::UShort p12;
  ::YellowPepper2 p13;
  ::GaliaMelon3 p14;
  CORBA::ULong p15;
  ::GaliaMelon3 p16;
  ::Beetroot2 p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Tomato2* ::ConstructedTypeTests::Cauliflower1(::Dates3 *&p1,::LoganBerry *&p2,const ::Lychees3 &p3,const ::Celeriac2 &p4,::BambooShoot3 &p5,::Susander2 *&p6,::Peanuts1 &p7,CORBA::Double &p8,::Currant2 *&p9,const ::Tomato2 &p10,const ::BeechNut2 &p11,CORBA::UShort &p12,::YellowPepper2 &p13,::GaliaMelon3 &p14,CORBA::ULong &p15,const ::GaliaMelon3 &p16,const ::Beetroot2 &p17)");
#endif
  
  ernie.seed("Cauliflower1Call");
  ernie.mode(0);
  GetLychees3(p3);
  ernie.mode(0);
  GetCeleriac2(p4);
  ernie.mode(1);
  GetBambooShoot3(p5);
  ernie.mode(1);
  GetDouble(p8);
  ernie.mode(0);
  GetTomato2(p10);
  ernie.mode(0);
  GetBeechNut2(p11);
  ernie.mode(1);
  GetUShort(p12);
  ernie.mode(1);
  GetYellowPepper2(p13);
  ernie.mode(1);
  GetGaliaMelon3(p14);
  ernie.mode(0);
  GetGaliaMelon3(p16);
  ernie.mode(0);
  GetBeetroot2(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cauliflower1Return");
  try {
    ReturnValue = tobj->Cauliflower1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Cauliflower1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTomato2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckDates3(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckLoganBerry(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckBambooShoot3(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckSusander2(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckPeanuts1(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckDouble(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCurrant2(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckUShort(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckYellowPepper2(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckGaliaMelon3(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckULong(p15))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsCorriander1(void)
{
  
  ::Lychees3* ReturnValue;
  ::Record2 p1;
  ::BambooShoot3 p2;
  ::Oregano3 p3;
  CORBA::ULong p4;
  ::Chives2 p5;
  
  ::Turnip3 p6;
  ::Cauliflower2* p7;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lychees3* ::ConstructedTypeTests::Corriander1(const ::Record2 p1,const ::BambooShoot3 &p2,const ::Oregano3 &p3,CORBA::ULong p4,::Chives2 &p5,const ::Turnip3 &p6,::Cauliflower2 *&p7)");
#endif
  
  ernie.seed("Corriander1Call");
  ernie.mode(0);
  GetRecord2(p1);
  ernie.mode(0);
  GetBambooShoot3(p2);
  ernie.mode(0);
  GetOregano3(p3);
  ernie.mode(0);
  GetULong(p4);
  ernie.mode(1);
  GetChives2(p5);
  ernie.mode(0);
  GetTurnip3(p6);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Corriander1Return");
  try {
    ReturnValue = tobj->Corriander1(p1,p2,p3,p4,p5,p6,p7);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Corriander1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLychees3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckChives2(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCauliflower2(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsRedCabbage1(void)
{
  
  ::BrazilNut3* ReturnValue;
  ::BritishQueens1* p1;
  ::Celeriac2 p2;
  ::Squash2 p3;
  CORBA::Octet p4;
  
  ::Fig p5;
  ::Record2 p6;
  
  ::Turnip3* p7;
  CORBA::Float p8;
  ::BritishQueens1 p9;
  ::YellowPepper2 p10;
  ::Cauliflower2 p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BrazilNut3* ::ConstructedTypeTests::RedCabbage1(::BritishQueens1 *&p1,const ::Celeriac2 &p2,const ::Squash2 &p3,CORBA::Octet &p4,const ::Fig &p5,::Record2 p6,::Turnip3 *&p7,CORBA::Float p8,const ::BritishQueens1 &p9,const ::YellowPepper2 &p10,::Cauliflower2 &p11)");
#endif
  
  ernie.seed("RedCabbage1Call");
  ernie.mode(0);
  GetCeleriac2(p2);
  ernie.mode(0);
  GetSquash2(p3);
  ernie.mode(0);
  GetFig(p5);
  ernie.mode(0);
  GetFloat(p8);
  ernie.mode(0);
  GetBritishQueens1(p9);
  ernie.mode(0);
  GetYellowPepper2(p10);
  ernie.mode(1);
  GetCauliflower2(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedCabbage1Return");
  try {
    ReturnValue = tobj->RedCabbage1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::RedCabbage1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBrazilNut3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckBritishQueens1(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckOctet(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckRecord2(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckTurnip3(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckCauliflower2(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsChineseLeaves1(void)
{
  
  ::Radish* ReturnValue;
  ::BritishQueens1* p1;
  ::GaliaMelon3 p2;
  ::Mangosteen3 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Radish* ::ConstructedTypeTests::ChineseLeaves1(::BritishQueens1 *&p1,::GaliaMelon3 &p2,::Mangosteen3 &p3)");
#endif
  
  ernie.seed("ChineseLeaves1Call");
  ernie.mode(1);
  GetGaliaMelon3(p2);
  ernie.mode(1);
  GetMangosteen3(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ChineseLeaves1Return");
  try {
    ReturnValue = tobj->ChineseLeaves1(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::ChineseLeaves1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRadish(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckBritishQueens1(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckGaliaMelon3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckMangosteen3(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsPotato1(void)
{
  
  ::Squash2* ReturnValue;
  ::Squash2* p1;
  ::Lychees3 p2;
  
  ::Turnip3* p3;
  ::Mangosteen3 p4;
  ::Currant2 p5;
  ::WaterMelon3 p6;
  ::Lychees3 p7;
  ::BritishQueens1 p8;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Squash2* ::ConstructedTypeTests::Potato1(::Squash2 *&p1,::Lychees3 &p2,::Turnip3 *&p3,::Mangosteen3 &p4,const ::Currant2 &p5,::WaterMelon3 p6,const ::Lychees3 &p7,const ::BritishQueens1 &p8)");
#endif
  
  ernie.seed("Potato1Call");
  ernie.mode(1);
  GetLychees3(p2);
  ernie.mode(1);
  GetMangosteen3(p4);
  ernie.mode(0);
  GetCurrant2(p5);
  ernie.mode(1);
  GetWaterMelon3(p6);
  ernie.mode(0);
  GetLychees3(p7);
  ernie.mode(0);
  GetBritishQueens1(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Potato1Return");
  try {
    ReturnValue = tobj->Potato1(p1,p2,p3,p4,p5,p6,p7,p8);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Potato1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSquash2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckSquash2(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckLychees3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckTurnip3(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckMangosteen3(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckWaterMelon3(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsHomeguard1(void)
{
  
  ::BrazilNut3* ReturnValue;
  ::Chives2 p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BrazilNut3* ::ConstructedTypeTests::Homeguard1(const ::Chives2 &p1)");
#endif
  
  ernie.seed("Homeguard1Call");
  ernie.mode(0);
  GetChives2(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Homeguard1Return");
  try {
    ReturnValue = tobj->Homeguard1(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Homeguard1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBrazilNut3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsWilja1(void)
{
  
  ::BritishQueens1* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BritishQueens1* ::ConstructedTypeTests::Wilja1()");
#endif
  
  ernie.seed("Wilja1Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Wilja1Return");
  try {
    ReturnValue = tobj->Wilja1();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::Wilja1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBritishQueens1(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testConstructedTypeTestsKerrsPinks1(void)
{
  
  ::Grapefruit2* ReturnValue;
  ::Dates3* p1;
  CORBA::Char * p2;
  ::Squash1 p3;
  
  ::Clementine2 p4;
  ::Cauliflower2 p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Grapefruit2* ::ConstructedTypeTests::KerrsPinks1(::Dates3 *&p1,CORBA::Char * &p2,::Squash1 &p3,const ::Clementine2 &p4,const ::Cauliflower2 &p5)");
#endif
  
  ernie.seed("KerrsPinks1Call");
  ernie.mode(1);
  p2 = GetString();
  ernie.mode(1);
  GetSquash1(p3);
  ernie.mode(0);
  GetClementine2(p4);
  ernie.mode(0);
  GetCauliflower2(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KerrsPinks1Return");
  try {
    ReturnValue = tobj->KerrsPinks1(p1,p2,p3,p4,p5);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ConstructedTypeTests::KerrsPinks1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGrapefruit2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckDates3(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckString(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckSquash1(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsGoldenWonder1(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
  CORBA::TypeCode_ptr p11;
  CORBA::TypeCode_ptr p12;
  CORBA::TypeCode_ptr p13;
  CORBA::TypeCode_ptr p14;
  CORBA::TypeCode_ptr p15;
  CORBA::TypeCode_ptr p16;
  CORBA::TypeCode_ptr p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::GoldenWonder1(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr &p6,CORBA::TypeCode_ptr p7,CORBA::TypeCode_ptr p8,CORBA::TypeCode_ptr p9,CORBA::TypeCode_ptr p10,CORBA::TypeCode_ptr &p11,CORBA::TypeCode_ptr &p12,CORBA::TypeCode_ptr p13,CORBA::TypeCode_ptr &p14,CORBA::TypeCode_ptr &p15,CORBA::TypeCode_ptr p16,CORBA::TypeCode_ptr &p17)");
#endif
  
  ernie.seed("GoldenWonder1Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p7);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p8);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p9);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p10);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p13);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p14);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p16);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GoldenWonder1Return");
  try {
    ReturnValue = tobj->GoldenWonder1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::GoldenWonder1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckCORBATypeCode_ptr(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckCORBATypeCode_ptr(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckCORBATypeCode_ptr(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckCORBATypeCode_ptr(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p17");
  if (!CheckCORBATypeCode_ptr(p17))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsPimpernel1(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Pimpernel1(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr p3,CORBA::TypeCode_ptr p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr &p6)");
#endif
  
  ernie.seed("Pimpernel1Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pimpernel1Return");
  try {
    ReturnValue = tobj->Pimpernel1(p1,p2,p3,p4,p5,p6);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Pimpernel1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsCultra1(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Cultra1(CORBA::TypeCode_ptr &p1)");
#endif
  
  ernie.seed("Cultra1Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cultra1Return");
  try {
    ReturnValue = tobj->Cultra1(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Cultra1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsRooster1(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
  CORBA::TypeCode_ptr p11;
  CORBA::TypeCode_ptr p12;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Rooster1(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr p5,CORBA::TypeCode_ptr &p6,CORBA::TypeCode_ptr &p7,CORBA::TypeCode_ptr &p8,CORBA::TypeCode_ptr &p9,CORBA::TypeCode_ptr &p10,CORBA::TypeCode_ptr &p11,CORBA::TypeCode_ptr &p12)");
#endif
  
  ernie.seed("Rooster1Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p9);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Rooster1Return");
  try {
    ReturnValue = tobj->Rooster1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Rooster1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckCORBATypeCode_ptr(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckCORBATypeCode_ptr(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckCORBATypeCode_ptr(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckCORBATypeCode_ptr(p12))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsMarisPiper1(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
  CORBA::TypeCode_ptr p11;
  CORBA::TypeCode_ptr p12;
  CORBA::TypeCode_ptr p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::MarisPiper1(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr &p6,CORBA::TypeCode_ptr &p7,CORBA::TypeCode_ptr &p8,CORBA::TypeCode_ptr &p9,CORBA::TypeCode_ptr p10,CORBA::TypeCode_ptr &p11,CORBA::TypeCode_ptr &p12,CORBA::TypeCode_ptr &p13)");
#endif
  
  ernie.seed("MarisPiper1Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p7);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p10);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p11);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p13);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MarisPiper1Return");
  try {
    ReturnValue = tobj->MarisPiper1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::MarisPiper1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckCORBATypeCode_ptr(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckCORBATypeCode_ptr(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckCORBATypeCode_ptr(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckCORBATypeCode_ptr(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsCara1(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Cara1(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr p5)");
#endif
  
  ernie.seed("Cara1Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cara1Return");
  try {
    ReturnValue = tobj->Cara1(p1,p2,p3,p4,p5);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Cara1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsKingEdward1(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
  CORBA::TypeCode_ptr p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::KingEdward1(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr p3,CORBA::TypeCode_ptr p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr &p6,CORBA::TypeCode_ptr &p7,CORBA::TypeCode_ptr p8,CORBA::TypeCode_ptr &p9,CORBA::TypeCode_ptr &p10,CORBA::TypeCode_ptr &p11)");
#endif
  
  ernie.seed("KingEdward1Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p7);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p8);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KingEdward1Return");
  try {
    ReturnValue = tobj->KingEdward1(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::KingEdward1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckCORBATypeCode_ptr(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckCORBATypeCode_ptr(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsPentlandDell1(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::PentlandDell1(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr p5,CORBA::TypeCode_ptr &p6,CORBA::TypeCode_ptr p7,CORBA::TypeCode_ptr p8)");
#endif
  
  ernie.seed("PentlandDell1Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p7);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PentlandDell1Return");
  try {
    ReturnValue = tobj->PentlandDell1(p1,p2,p3,p4,p5,p6,p7,p8);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::PentlandDell1");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsApple2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
  CORBA::TypeCode_ptr p11;
  CORBA::TypeCode_ptr p12;
  CORBA::TypeCode_ptr p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Apple2(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr p6,CORBA::TypeCode_ptr &p7,CORBA::TypeCode_ptr p8,CORBA::TypeCode_ptr &p9,CORBA::TypeCode_ptr &p10,CORBA::TypeCode_ptr p11,CORBA::TypeCode_ptr p12,CORBA::TypeCode_ptr &p13)");
#endif
  
  ernie.seed("Apple2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p7);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p8);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p9);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p11);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p12);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p13);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Apple2Return");
  try {
    ReturnValue = tobj->Apple2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Apple2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckCORBATypeCode_ptr(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckCORBATypeCode_ptr(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsBanana2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Banana2(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5)");
#endif
  
  ernie.seed("Banana2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p4);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Banana2Return");
  try {
    ReturnValue = tobj->Banana2(p1,p2,p3,p4,p5);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Banana2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsCarrots2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Carrots2(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr &p6,CORBA::TypeCode_ptr &p7,CORBA::TypeCode_ptr &p8,CORBA::TypeCode_ptr &p9)");
#endif
  
  ernie.seed("Carrots2Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p7);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Carrots2Return");
  try {
    ReturnValue = tobj->Carrots2(p1,p2,p3,p4,p5,p6,p7,p8,p9);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Carrots2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckCORBATypeCode_ptr(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsLemon2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Lemon2(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr p6,CORBA::TypeCode_ptr &p7,CORBA::TypeCode_ptr &p8,CORBA::TypeCode_ptr p9,CORBA::TypeCode_ptr p10)");
#endif
  
  ernie.seed("Lemon2Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p9);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p10);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lemon2Return");
  try {
    ReturnValue = tobj->Lemon2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Lemon2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckCORBATypeCode_ptr(p8))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsBlackCherry2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::BlackCherry2(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr p5,CORBA::TypeCode_ptr p6,CORBA::TypeCode_ptr p7,CORBA::TypeCode_ptr &p8,CORBA::TypeCode_ptr &p9)");
#endif
  
  ernie.seed("BlackCherry2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p7);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackCherry2Return");
  try {
    ReturnValue = tobj->BlackCherry2(p1,p2,p3,p4,p5,p6,p7,p8,p9);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::BlackCherry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckCORBATypeCode_ptr(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsStarfruit2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
  CORBA::TypeCode_ptr p11;
  CORBA::TypeCode_ptr p12;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Starfruit2(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr &p6,CORBA::TypeCode_ptr &p7,CORBA::TypeCode_ptr p8,CORBA::TypeCode_ptr &p9,CORBA::TypeCode_ptr p10,CORBA::TypeCode_ptr &p11,CORBA::TypeCode_ptr &p12)");
#endif
  
  ernie.seed("Starfruit2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p8);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p10);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p11);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Starfruit2Return");
  try {
    ReturnValue = tobj->Starfruit2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Starfruit2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckCORBATypeCode_ptr(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckCORBATypeCode_ptr(p12))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsRhubarb2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Rhubarb2(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr &p6,CORBA::TypeCode_ptr &p7)");
#endif
  
  ernie.seed("Rhubarb2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Rhubarb2Return");
  try {
    ReturnValue = tobj->Rhubarb2(p1,p2,p3,p4,p5,p6,p7);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Rhubarb2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsPassionFruit2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::PassionFruit2()");
#endif
  
  ernie.seed("PassionFruit2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PassionFruit2Return");
  try {
    ReturnValue = tobj->PassionFruit2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::PassionFruit2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsPumpkin2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
  CORBA::TypeCode_ptr p11;
  CORBA::TypeCode_ptr p12;
  CORBA::TypeCode_ptr p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Pumpkin2(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr p6,CORBA::TypeCode_ptr p7,CORBA::TypeCode_ptr p8,CORBA::TypeCode_ptr &p9,CORBA::TypeCode_ptr &p10,CORBA::TypeCode_ptr p11,CORBA::TypeCode_ptr p12,CORBA::TypeCode_ptr &p13)");
#endif
  
  ernie.seed("Pumpkin2Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p7);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p8);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p10);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p11);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pumpkin2Return");
  try {
    ReturnValue = tobj->Pumpkin2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Pumpkin2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckCORBATypeCode_ptr(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckCORBATypeCode_ptr(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsMango2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
  CORBA::TypeCode_ptr p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Mango2(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr p6,CORBA::TypeCode_ptr &p7,CORBA::TypeCode_ptr p8,CORBA::TypeCode_ptr &p9,CORBA::TypeCode_ptr &p10,CORBA::TypeCode_ptr &p11)");
#endif
  
  ernie.seed("Mango2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p8);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p10);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mango2Return");
  try {
    ReturnValue = tobj->Mango2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Mango2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckCORBATypeCode_ptr(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckCORBATypeCode_ptr(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsWaterMelon2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
  CORBA::TypeCode_ptr p11;
  CORBA::TypeCode_ptr p12;
  CORBA::TypeCode_ptr p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::WaterMelon2(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr p5,CORBA::TypeCode_ptr p6,CORBA::TypeCode_ptr p7,CORBA::TypeCode_ptr &p8,CORBA::TypeCode_ptr &p9,CORBA::TypeCode_ptr p10,CORBA::TypeCode_ptr p11,CORBA::TypeCode_ptr &p12,CORBA::TypeCode_ptr &p13)");
#endif
  
  ernie.seed("WaterMelon2Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p7);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p10);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p11);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p12);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p13);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("WaterMelon2Return");
  try {
    ReturnValue = tobj->WaterMelon2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::WaterMelon2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckCORBATypeCode_ptr(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckCORBATypeCode_ptr(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckCORBATypeCode_ptr(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsLime2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Lime2(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr &p6,CORBA::TypeCode_ptr &p7)");
#endif
  
  ernie.seed("Lime2Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p7);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lime2Return");
  try {
    ReturnValue = tobj->Lime2(p1,p2,p3,p4,p5,p6,p7);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Lime2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsDates2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Dates2(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr &p6,CORBA::TypeCode_ptr &p7,CORBA::TypeCode_ptr p8)");
#endif
  
  ernie.seed("Dates2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Dates2Return");
  try {
    ReturnValue = tobj->Dates2(p1,p2,p3,p4,p5,p6,p7,p8);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Dates2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsCoconut2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Coconut2()");
#endif
  
  ernie.seed("Coconut2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Coconut2Return");
  try {
    ReturnValue = tobj->Coconut2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Coconut2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsLychees2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Lychees2(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5)");
#endif
  
  ernie.seed("Lychees2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lychees2Return");
  try {
    ReturnValue = tobj->Lychees2(p1,p2,p3,p4,p5);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Lychees2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsCabbage2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Cabbage2(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr &p6,CORBA::TypeCode_ptr p7)");
#endif
  
  ernie.seed("Cabbage2Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p7);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cabbage2Return");
  try {
    ReturnValue = tobj->Cabbage2(p1,p2,p3,p4,p5,p6,p7);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Cabbage2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsCucumber2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Cucumber2(CORBA::TypeCode_ptr &p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4)");
#endif
  
  ernie.seed("Cucumber2Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cucumber2Return");
  try {
    ReturnValue = tobj->Cucumber2(p1,p2,p3,p4);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Cucumber2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsGinger2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Ginger2(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr p2)");
#endif
  
  ernie.seed("Ginger2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Ginger2Return");
  try {
    ReturnValue = tobj->Ginger2(p1,p2);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Ginger2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsApricots2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
  CORBA::TypeCode_ptr p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Apricots2(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr p3,CORBA::TypeCode_ptr &p4,CORBA::TypeCode_ptr &p5,CORBA::TypeCode_ptr p6,CORBA::TypeCode_ptr p7,CORBA::TypeCode_ptr &p8,CORBA::TypeCode_ptr &p9,CORBA::TypeCode_ptr &p10,CORBA::TypeCode_ptr &p11)");
#endif
  
  ernie.seed("Apricots2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p7);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p10);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Apricots2Return");
  try {
    ReturnValue = tobj->Apricots2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Apricots2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckCORBATypeCode_ptr(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckCORBATypeCode_ptr(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckCORBATypeCode_ptr(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckCORBATypeCode_ptr(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsBlackEyedBeans2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
  CORBA::TypeCode_ptr p5;
  CORBA::TypeCode_ptr p6;
  CORBA::TypeCode_ptr p7;
  CORBA::TypeCode_ptr p8;
  CORBA::TypeCode_ptr p9;
  CORBA::TypeCode_ptr p10;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::BlackEyedBeans2(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr &p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr p4,CORBA::TypeCode_ptr p5,CORBA::TypeCode_ptr p6,CORBA::TypeCode_ptr &p7,CORBA::TypeCode_ptr &p8,CORBA::TypeCode_ptr &p9,CORBA::TypeCode_ptr p10)");
#endif
  
  ernie.seed("BlackEyedBeans2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p5);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p10);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackEyedBeans2Return");
  try {
    ReturnValue = tobj->BlackEyedBeans2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::BlackEyedBeans2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckCORBATypeCode_ptr(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCORBATypeCode_ptr(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckCORBATypeCode_ptr(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCORBATypeCode_ptr(p9))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsLeek2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::TypeCode_ptr p2;
  CORBA::TypeCode_ptr p3;
  CORBA::TypeCode_ptr p4;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Leek2(CORBA::TypeCode_ptr p1,CORBA::TypeCode_ptr p2,CORBA::TypeCode_ptr &p3,CORBA::TypeCode_ptr &p4)");
#endif
  
  ernie.seed("Leek2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Leek2Return");
  try {
    ReturnValue = tobj->Leek2(p1,p2,p3,p4);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Leek2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsMint2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Mint2()");
#endif
  
  ernie.seed("Mint2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mint2Return");
  try {
    ReturnValue = tobj->Mint2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Mint2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsGardenPeas2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::GardenPeas2()");
#endif
  
  ernie.seed("GardenPeas2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GardenPeas2Return");
  try {
    ReturnValue = tobj->GardenPeas2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::GardenPeas2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsGardenPeas2W(void)
{
  
  CORBA::TypeCode_ptr _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TypeCodeTests::GardenPeas2(CORBA::TypeCode_ptr _attr)");
#endif
  
  ernie.seed("GardenPeas2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GardenPeas2Return");
  try {
     tobj->GardenPeas2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::GardenPeas2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsAsparagus2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Asparagus2()");
#endif
  
  ernie.seed("Asparagus2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Asparagus2Return");
  try {
    ReturnValue = tobj->Asparagus2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Asparagus2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsAsparagus2W(void)
{
  
  CORBA::TypeCode_ptr _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TypeCodeTests::Asparagus2(CORBA::TypeCode_ptr _attr)");
#endif
  
  ernie.seed("Asparagus2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Asparagus2Return");
  try {
     tobj->Asparagus2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Asparagus2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsGaliaMelon2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::GaliaMelon2()");
#endif
  
  ernie.seed("GaliaMelon2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GaliaMelon2Return");
  try {
    ReturnValue = tobj->GaliaMelon2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::GaliaMelon2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsGaliaMelon2W(void)
{
  
  CORBA::TypeCode_ptr _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TypeCodeTests::GaliaMelon2(CORBA::TypeCode_ptr _attr)");
#endif
  
  ernie.seed("GaliaMelon2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GaliaMelon2Return");
  try {
     tobj->GaliaMelon2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::GaliaMelon2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsSultana2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Sultana2()");
#endif
  
  ernie.seed("Sultana2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sultana2Return");
  try {
    ReturnValue = tobj->Sultana2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Sultana2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsSultana2W(void)
{
  
  CORBA::TypeCode_ptr _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TypeCodeTests::Sultana2(CORBA::TypeCode_ptr _attr)");
#endif
  
  ernie.seed("Sultana2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sultana2Return");
  try {
     tobj->Sultana2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Sultana2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsTurnip2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Turnip2()");
#endif
  
  ernie.seed("Turnip2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Turnip2Return");
  try {
    ReturnValue = tobj->Turnip2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Turnip2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsTurnip2W(void)
{
  
  CORBA::TypeCode_ptr _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TypeCodeTests::Turnip2(CORBA::TypeCode_ptr _attr)");
#endif
  
  ernie.seed("Turnip2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Turnip2Return");
  try {
     tobj->Turnip2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Turnip2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsOrange2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Orange2()");
#endif
  
  ernie.seed("Orange2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Orange2Return");
  try {
    ReturnValue = tobj->Orange2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Orange2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsOrange2W(void)
{
  
  CORBA::TypeCode_ptr _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TypeCodeTests::Orange2(CORBA::TypeCode_ptr _attr)");
#endif
  
  ernie.seed("Orange2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Orange2Return");
  try {
     tobj->Orange2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Orange2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsLoganBerry2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::LoganBerry2()");
#endif
  
  ernie.seed("LoganBerry2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerry2Return");
  try {
    ReturnValue = tobj->LoganBerry2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::LoganBerry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsLoganBerry2W(void)
{
  
  CORBA::TypeCode_ptr _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::TypeCodeTests::LoganBerry2(CORBA::TypeCode_ptr _attr)");
#endif
  
  ernie.seed("LoganBerry2Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LoganBerry2Return");
  try {
     tobj->LoganBerry2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::LoganBerry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsParsnip2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Parsnip2()");
#endif
  
  ernie.seed("Parsnip2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsnip2Return");
  try {
    ReturnValue = tobj->Parsnip2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Parsnip2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsCelery2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Celery2()");
#endif
  
  ernie.seed("Celery2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Celery2Return");
  try {
    ReturnValue = tobj->Celery2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Celery2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsWhiteCabbage2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::WhiteCabbage2()");
#endif
  
  ernie.seed("WhiteCabbage2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("WhiteCabbage2Return");
  try {
    ReturnValue = tobj->WhiteCabbage2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::WhiteCabbage2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsAniseed2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Aniseed2()");
#endif
  
  ernie.seed("Aniseed2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Aniseed2Return");
  try {
    ReturnValue = tobj->Aniseed2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Aniseed2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsBayleaf2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Bayleaf2()");
#endif
  
  ernie.seed("Bayleaf2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Bayleaf2Return");
  try {
    ReturnValue = tobj->Bayleaf2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Bayleaf2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsDill2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Dill2()");
#endif
  
  ernie.seed("Dill2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Dill2Return");
  try {
    ReturnValue = tobj->Dill2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Dill2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testTypeCodeTestsOregano2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::TypeCodeTests::Oregano2()");
#endif
  
  ernie.seed("Oregano2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Oregano2Return");
  try {
    ReturnValue = tobj->Oregano2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::TypeCodeTests::Oregano2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsBeansprout2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any* p2;
  CORBA::Any p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Beansprout2(const CORBA::Any &p1,CORBA::Any *&p2,CORBA::Any &p3)");
#endif
  
  ernie.seed("Beansprout2Call");
  ernie.mode(0);
  GetAny(p1);
  ernie.mode(1);
  GetAny(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Beansprout2Return");
  try {
    ReturnValue = tobj->Beansprout2(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Beansprout2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckAny(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsAlmonds2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any* p2;
  CORBA::Any* p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any p6;
  CORBA::Any p7;
  CORBA::Any p8;
  CORBA::Any* p9;
  CORBA::Any p10;
  CORBA::Any p11;
  CORBA::Any p12;
  CORBA::Any p13;
  CORBA::Any p14;
  CORBA::Any p15;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Almonds2(CORBA::Any &p1,CORBA::Any *&p2,CORBA::Any *&p3,const CORBA::Any &p4,CORBA::Any &p5,const CORBA::Any &p6,const CORBA::Any &p7,const CORBA::Any &p8,CORBA::Any *&p9,const CORBA::Any &p10,const CORBA::Any &p11,CORBA::Any &p12,const CORBA::Any &p13,const CORBA::Any &p14,const CORBA::Any &p15)");
#endif
  
  ernie.seed("Almonds2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p4);
  ernie.mode(1);
  GetAny(p5);
  ernie.mode(0);
  GetAny(p6);
  ernie.mode(0);
  GetAny(p7);
  ernie.mode(0);
  GetAny(p8);
  ernie.mode(0);
  GetAny(p10);
  ernie.mode(0);
  GetAny(p11);
  ernie.mode(1);
  GetAny(p12);
  ernie.mode(0);
  GetAny(p13);
  ernie.mode(0);
  GetAny(p14);
  ernie.mode(0);
  GetAny(p15);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Almonds2Return");
  try {
    ReturnValue = tobj->Almonds2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Almonds2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckAny(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckAny(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckAny(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckAny(p12))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsGuava2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any* p2;
  CORBA::Any p3;
  CORBA::Any p4;
  CORBA::Any* p5;
  CORBA::Any* p6;
  CORBA::Any* p7;
  CORBA::Any p8;
  CORBA::Any* p9;
  CORBA::Any* p10;
  CORBA::Any p11;
  CORBA::Any p12;
  CORBA::Any* p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Guava2(const CORBA::Any &p1,CORBA::Any *&p2,const CORBA::Any &p3,const CORBA::Any &p4,CORBA::Any *&p5,CORBA::Any *&p6,CORBA::Any *&p7,CORBA::Any &p8,CORBA::Any *&p9,CORBA::Any *&p10,const CORBA::Any &p11,CORBA::Any &p12,CORBA::Any *&p13)");
#endif
  
  ernie.seed("Guava2Call");
  ernie.mode(0);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p3);
  ernie.mode(0);
  GetAny(p4);
  ernie.mode(1);
  GetAny(p8);
  ernie.mode(0);
  GetAny(p11);
  ernie.mode(1);
  GetAny(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Guava2Return");
  try {
    ReturnValue = tobj->Guava2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Guava2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckAny(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckAny(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckAny(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckAny(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckAny(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckAny(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckAny(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckAny(*p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsMushroom2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any p2;
  CORBA::Any p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Mushroom2(CORBA::Any &p1,const CORBA::Any &p2,CORBA::Any &p3)");
#endif
  
  ernie.seed("Mushroom2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p2);
  ernie.mode(1);
  GetAny(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mushroom2Return");
  try {
    ReturnValue = tobj->Mushroom2(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Mushroom2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckAny(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsRunnerbean2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any* p1;
  CORBA::Any* p2;
  CORBA::Any p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any p6;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Runnerbean2(CORBA::Any *&p1,CORBA::Any *&p2,const CORBA::Any &p3,CORBA::Any &p4,const CORBA::Any &p5,const CORBA::Any &p6)");
#endif
  
  ernie.seed("Runnerbean2Call");
  ernie.mode(0);
  GetAny(p3);
  ernie.mode(1);
  GetAny(p4);
  ernie.mode(0);
  GetAny(p5);
  ernie.mode(0);
  GetAny(p6);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Runnerbean2Return");
  try {
    ReturnValue = tobj->Runnerbean2(p1,p2,p3,p4,p5,p6);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Runnerbean2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckAny(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAny(p4))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsTofu2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any p2;
  CORBA::Any p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any p6;
  CORBA::Any* p7;
  CORBA::Any p8;
  CORBA::Any p9;
  CORBA::Any p10;
  CORBA::Any* p11;
  CORBA::Any* p12;
  CORBA::Any p13;
  CORBA::Any* p14;
  CORBA::Any* p15;
  CORBA::Any p16;
  CORBA::Any p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Tofu2(CORBA::Any &p1,const CORBA::Any &p2,const CORBA::Any &p3,CORBA::Any &p4,CORBA::Any &p5,const CORBA::Any &p6,CORBA::Any *&p7,CORBA::Any &p8,const CORBA::Any &p9,CORBA::Any &p10,CORBA::Any *&p11,CORBA::Any *&p12,const CORBA::Any &p13,CORBA::Any *&p14,CORBA::Any *&p15,const CORBA::Any &p16,const CORBA::Any &p17)");
#endif
  
  ernie.seed("Tofu2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p2);
  ernie.mode(0);
  GetAny(p3);
  ernie.mode(1);
  GetAny(p4);
  ernie.mode(1);
  GetAny(p5);
  ernie.mode(0);
  GetAny(p6);
  ernie.mode(1);
  GetAny(p8);
  ernie.mode(0);
  GetAny(p9);
  ernie.mode(1);
  GetAny(p10);
  ernie.mode(0);
  GetAny(p13);
  ernie.mode(0);
  GetAny(p16);
  ernie.mode(0);
  GetAny(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Tofu2Return");
  try {
    ReturnValue = tobj->Tofu2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Tofu2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAny(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckAny(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckAny(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckAny(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckAny(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckAny(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckAny(*p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckAny(*p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckAny(*p15))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsPrunes2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any* p1;
  CORBA::Any* p2;
  CORBA::Any* p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any p6;
  CORBA::Any* p7;
  CORBA::Any* p8;
  CORBA::Any p9;
  CORBA::Any* p10;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Prunes2(CORBA::Any *&p1,CORBA::Any *&p2,CORBA::Any *&p3,CORBA::Any &p4,CORBA::Any &p5,CORBA::Any &p6,CORBA::Any *&p7,CORBA::Any *&p8,CORBA::Any &p9,CORBA::Any *&p10)");
#endif
  
  ernie.seed("Prunes2Call");
  ernie.mode(1);
  GetAny(p4);
  ernie.mode(1);
  GetAny(p5);
  ernie.mode(1);
  GetAny(p6);
  ernie.mode(1);
  GetAny(p9);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Prunes2Return");
  try {
    ReturnValue = tobj->Prunes2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Prunes2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckAny(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckAny(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAny(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckAny(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckAny(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckAny(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckAny(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckAny(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckAny(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsBrocoli2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any* p2;
  CORBA::Any p3;
  CORBA::Any* p4;
  CORBA::Any* p5;
  CORBA::Any* p6;
  CORBA::Any p7;
  CORBA::Any p8;
  CORBA::Any p9;
  CORBA::Any* p10;
  CORBA::Any* p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Brocoli2(CORBA::Any &p1,CORBA::Any *&p2,const CORBA::Any &p3,CORBA::Any *&p4,CORBA::Any *&p5,CORBA::Any *&p6,const CORBA::Any &p7,const CORBA::Any &p8,CORBA::Any &p9,CORBA::Any *&p10,CORBA::Any *&p11)");
#endif
  
  ernie.seed("Brocoli2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p3);
  ernie.mode(0);
  GetAny(p7);
  ernie.mode(0);
  GetAny(p8);
  ernie.mode(1);
  GetAny(p9);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Brocoli2Return");
  try {
    ReturnValue = tobj->Brocoli2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Brocoli2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckAny(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckAny(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckAny(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckAny(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckAny(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckAny(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsGrannySmith2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any* p2;
  CORBA::Any p3;
  CORBA::Any* p4;
  CORBA::Any* p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::GrannySmith2(const CORBA::Any &p1,CORBA::Any *&p2,const CORBA::Any &p3,CORBA::Any *&p4,CORBA::Any *&p5)");
#endif
  
  ernie.seed("GrannySmith2Call");
  ernie.mode(0);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GrannySmith2Return");
  try {
    ReturnValue = tobj->GrannySmith2(p1,p2,p3,p4,p5);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::GrannySmith2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckAny(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckAny(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsBrazilNut2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any* p1;
  CORBA::Any* p2;
  CORBA::Any p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any p6;
  CORBA::Any p7;
  CORBA::Any p8;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::BrazilNut2(CORBA::Any *&p1,CORBA::Any *&p2,CORBA::Any &p3,CORBA::Any &p4,CORBA::Any &p5,CORBA::Any &p6,const CORBA::Any &p7,const CORBA::Any &p8)");
#endif
  
  ernie.seed("BrazilNut2Call");
  ernie.mode(1);
  GetAny(p3);
  ernie.mode(1);
  GetAny(p4);
  ernie.mode(1);
  GetAny(p5);
  ernie.mode(1);
  GetAny(p6);
  ernie.mode(0);
  GetAny(p7);
  ernie.mode(0);
  GetAny(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BrazilNut2Return");
  try {
    ReturnValue = tobj->BrazilNut2(p1,p2,p3,p4,p5,p6,p7,p8);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::BrazilNut2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckAny(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckAny(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAny(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckAny(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckAny(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsSweetPotato2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any* p1;
  CORBA::Any p2;
  CORBA::Any* p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any* p6;
  CORBA::Any p7;
  CORBA::Any* p8;
  CORBA::Any* p9;
  CORBA::Any* p10;
  CORBA::Any* p11;
  CORBA::Any* p12;
  CORBA::Any p13;
  CORBA::Any p14;
  CORBA::Any p15;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::SweetPotato2(CORBA::Any *&p1,CORBA::Any &p2,CORBA::Any *&p3,CORBA::Any &p4,const CORBA::Any &p5,CORBA::Any *&p6,const CORBA::Any &p7,CORBA::Any *&p8,CORBA::Any *&p9,CORBA::Any *&p10,CORBA::Any *&p11,CORBA::Any *&p12,const CORBA::Any &p13,CORBA::Any &p14,CORBA::Any &p15)");
#endif
  
  ernie.seed("SweetPotato2Call");
  ernie.mode(1);
  GetAny(p2);
  ernie.mode(1);
  GetAny(p4);
  ernie.mode(0);
  GetAny(p5);
  ernie.mode(0);
  GetAny(p7);
  ernie.mode(0);
  GetAny(p13);
  ernie.mode(1);
  GetAny(p14);
  ernie.mode(1);
  GetAny(p15);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetPotato2Return");
  try {
    ReturnValue = tobj->SweetPotato2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::SweetPotato2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckAny(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckAny(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckAny(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAny(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckAny(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckAny(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckAny(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckAny(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckAny(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckAny(*p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckAny(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckAny(p15))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsTangarine2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any p2;
  CORBA::Any p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any p6;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Tangarine2(const CORBA::Any &p1,const CORBA::Any &p2,const CORBA::Any &p3,const CORBA::Any &p4,CORBA::Any &p5,const CORBA::Any &p6)");
#endif
  
  ernie.seed("Tangarine2Call");
  ernie.mode(0);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p2);
  ernie.mode(0);
  GetAny(p3);
  ernie.mode(0);
  GetAny(p4);
  ernie.mode(1);
  GetAny(p5);
  ernie.mode(0);
  GetAny(p6);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Tangarine2Return");
  try {
    ReturnValue = tobj->Tangarine2(p1,p2,p3,p4,p5,p6);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Tangarine2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckAny(p5))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsBillberries2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any p2;
  CORBA::Any p3;
  CORBA::Any* p4;
  CORBA::Any p5;
  CORBA::Any p6;
  CORBA::Any* p7;
  CORBA::Any p8;
  CORBA::Any p9;
  CORBA::Any p10;
  CORBA::Any p11;
  CORBA::Any* p12;
  CORBA::Any p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Billberries2(const CORBA::Any &p1,const CORBA::Any &p2,CORBA::Any &p3,CORBA::Any *&p4,CORBA::Any &p5,const CORBA::Any &p6,CORBA::Any *&p7,const CORBA::Any &p8,CORBA::Any &p9,const CORBA::Any &p10,CORBA::Any &p11,CORBA::Any *&p12,CORBA::Any &p13)");
#endif
  
  ernie.seed("Billberries2Call");
  ernie.mode(0);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p2);
  ernie.mode(1);
  GetAny(p3);
  ernie.mode(1);
  GetAny(p5);
  ernie.mode(0);
  GetAny(p6);
  ernie.mode(0);
  GetAny(p8);
  ernie.mode(1);
  GetAny(p9);
  ernie.mode(0);
  GetAny(p10);
  ernie.mode(1);
  GetAny(p11);
  ernie.mode(1);
  GetAny(p13);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Billberries2Return");
  try {
    ReturnValue = tobj->Billberries2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Billberries2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckAny(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckAny(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckAny(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckAny(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckAny(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckAny(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckAny(*p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckAny(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsCantelopeMelon2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any p2;
  CORBA::Any* p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any p6;
  CORBA::Any* p7;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::CantelopeMelon2(CORBA::Any &p1,const CORBA::Any &p2,CORBA::Any *&p3,CORBA::Any &p4,const CORBA::Any &p5,CORBA::Any &p6,CORBA::Any *&p7)");
#endif
  
  ernie.seed("CantelopeMelon2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p2);
  ernie.mode(1);
  GetAny(p4);
  ernie.mode(0);
  GetAny(p5);
  ernie.mode(1);
  GetAny(p6);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CantelopeMelon2Return");
  try {
    ReturnValue = tobj->CantelopeMelon2(p1,p2,p3,p4,p5,p6,p7);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::CantelopeMelon2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckAny(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAny(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckAny(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckAny(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsLolloRossa2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::LolloRossa2()");
#endif
  
  ernie.seed("LolloRossa2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("LolloRossa2Return");
  try {
    ReturnValue = tobj->LolloRossa2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::LolloRossa2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsRowanBerry2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any p2;
  CORBA::Any* p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any p6;
  CORBA::Any* p7;
  CORBA::Any* p8;
  CORBA::Any p9;
  CORBA::Any p10;
  CORBA::Any* p11;
  CORBA::Any p12;
  CORBA::Any* p13;
  CORBA::Any* p14;
  CORBA::Any p15;
  CORBA::Any p16;
  CORBA::Any* p17;
  CORBA::Any p18;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::RowanBerry2(CORBA::Any &p1,CORBA::Any &p2,CORBA::Any *&p3,CORBA::Any &p4,const CORBA::Any &p5,CORBA::Any &p6,CORBA::Any *&p7,CORBA::Any *&p8,const CORBA::Any &p9,const CORBA::Any &p10,CORBA::Any *&p11,CORBA::Any &p12,CORBA::Any *&p13,CORBA::Any *&p14,CORBA::Any &p15,CORBA::Any &p16,CORBA::Any *&p17,const CORBA::Any &p18)");
#endif
  
  ernie.seed("RowanBerry2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(1);
  GetAny(p2);
  ernie.mode(1);
  GetAny(p4);
  ernie.mode(0);
  GetAny(p5);
  ernie.mode(1);
  GetAny(p6);
  ernie.mode(0);
  GetAny(p9);
  ernie.mode(0);
  GetAny(p10);
  ernie.mode(1);
  GetAny(p12);
  ernie.mode(1);
  GetAny(p15);
  ernie.mode(1);
  GetAny(p16);
  ernie.mode(0);
  GetAny(p18);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RowanBerry2Return");
  try {
    ReturnValue = tobj->RowanBerry2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::RowanBerry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckAny(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckAny(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAny(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckAny(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckAny(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckAny(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckAny(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckAny(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckAny(*p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckAny(*p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckAny(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p16");
  if (!CheckAny(p16))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p17");
  if (!CheckAny(*p17))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsSweetcorn2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any* p1;
  CORBA::Any p2;
  CORBA::Any* p3;
  CORBA::Any* p4;
  CORBA::Any p5;
  CORBA::Any* p6;
  CORBA::Any p7;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Sweetcorn2(CORBA::Any *&p1,const CORBA::Any &p2,CORBA::Any *&p3,CORBA::Any *&p4,CORBA::Any &p5,CORBA::Any *&p6,const CORBA::Any &p7)");
#endif
  
  ernie.seed("Sweetcorn2Call");
  ernie.mode(0);
  GetAny(p2);
  ernie.mode(1);
  GetAny(p5);
  ernie.mode(0);
  GetAny(p7);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sweetcorn2Return");
  try {
    ReturnValue = tobj->Sweetcorn2(p1,p2,p3,p4,p5,p6,p7);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Sweetcorn2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckAny(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckAny(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckAny(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckAny(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckAny(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsOnion2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any* p2;
  CORBA::Any p3;
  CORBA::Any* p4;
  CORBA::Any* p5;
  CORBA::Any* p6;
  CORBA::Any p7;
  CORBA::Any p8;
  CORBA::Any p9;
  CORBA::Any* p10;
  CORBA::Any* p11;
  CORBA::Any p12;
  CORBA::Any p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Onion2(const CORBA::Any &p1,CORBA::Any *&p2,const CORBA::Any &p3,CORBA::Any *&p4,CORBA::Any *&p5,CORBA::Any *&p6,const CORBA::Any &p7,CORBA::Any &p8,CORBA::Any &p9,CORBA::Any *&p10,CORBA::Any *&p11,const CORBA::Any &p12,CORBA::Any &p13)");
#endif
  
  ernie.seed("Onion2Call");
  ernie.mode(0);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p3);
  ernie.mode(0);
  GetAny(p7);
  ernie.mode(1);
  GetAny(p8);
  ernie.mode(1);
  GetAny(p9);
  ernie.mode(0);
  GetAny(p12);
  ernie.mode(1);
  GetAny(p13);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Onion2Return");
  try {
    ReturnValue = tobj->Onion2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Onion2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckAny(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckAny(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckAny(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckAny(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckAny(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckAny(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckAny(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckAny(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsShallots2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any p2;
  CORBA::Any p3;
  CORBA::Any p4;
  CORBA::Any* p5;
  CORBA::Any p6;
  CORBA::Any* p7;
  CORBA::Any* p8;
  CORBA::Any* p9;
  CORBA::Any p10;
  CORBA::Any p11;
  CORBA::Any p12;
  CORBA::Any p13;
  CORBA::Any p14;
  CORBA::Any p15;
  CORBA::Any* p16;
  CORBA::Any* p17;
  CORBA::Any p18;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Shallots2(CORBA::Any &p1,CORBA::Any &p2,CORBA::Any &p3,const CORBA::Any &p4,CORBA::Any *&p5,CORBA::Any &p6,CORBA::Any *&p7,CORBA::Any *&p8,CORBA::Any *&p9,const CORBA::Any &p10,const CORBA::Any &p11,CORBA::Any &p12,const CORBA::Any &p13,CORBA::Any &p14,const CORBA::Any &p15,CORBA::Any *&p16,CORBA::Any *&p17,const CORBA::Any &p18)");
#endif
  
  ernie.seed("Shallots2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(1);
  GetAny(p2);
  ernie.mode(1);
  GetAny(p3);
  ernie.mode(0);
  GetAny(p4);
  ernie.mode(1);
  GetAny(p6);
  ernie.mode(0);
  GetAny(p10);
  ernie.mode(0);
  GetAny(p11);
  ernie.mode(1);
  GetAny(p12);
  ernie.mode(0);
  GetAny(p13);
  ernie.mode(1);
  GetAny(p14);
  ernie.mode(0);
  GetAny(p15);
  ernie.mode(0);
  GetAny(p18);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Shallots2Return");
  try {
    ReturnValue = tobj->Shallots2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Shallots2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckAny(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckAny(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckAny(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckAny(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckAny(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckAny(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckAny(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckAny(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckAny(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckAny(*p16))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p17");
  if (!CheckAny(*p17))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsPlum2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any* p2;
  CORBA::Any p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any* p6;
  CORBA::Any* p7;
  CORBA::Any p8;
  CORBA::Any* p9;
  CORBA::Any p10;
  CORBA::Any p11;
  CORBA::Any p12;
  CORBA::Any p13;
  CORBA::Any p14;
  CORBA::Any p15;
  CORBA::Any* p16;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Plum2(CORBA::Any &p1,CORBA::Any *&p2,const CORBA::Any &p3,const CORBA::Any &p4,const CORBA::Any &p5,CORBA::Any *&p6,CORBA::Any *&p7,const CORBA::Any &p8,CORBA::Any *&p9,const CORBA::Any &p10,const CORBA::Any &p11,const CORBA::Any &p12,const CORBA::Any &p13,CORBA::Any &p14,CORBA::Any &p15,CORBA::Any *&p16)");
#endif
  
  ernie.seed("Plum2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p3);
  ernie.mode(0);
  GetAny(p4);
  ernie.mode(0);
  GetAny(p5);
  ernie.mode(0);
  GetAny(p8);
  ernie.mode(0);
  GetAny(p10);
  ernie.mode(0);
  GetAny(p11);
  ernie.mode(0);
  GetAny(p12);
  ernie.mode(0);
  GetAny(p13);
  ernie.mode(1);
  GetAny(p14);
  ernie.mode(1);
  GetAny(p15);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Plum2Return");
  try {
    ReturnValue = tobj->Plum2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Plum2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckAny(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckAny(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckAny(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckAny(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckAny(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckAny(*p16))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsKidneyBean2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any p2;
  CORBA::Any p3;
  CORBA::Any* p4;
  CORBA::Any* p5;
  CORBA::Any p6;
  CORBA::Any p7;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::KidneyBean2(CORBA::Any &p1,const CORBA::Any &p2,const CORBA::Any &p3,CORBA::Any *&p4,CORBA::Any *&p5,const CORBA::Any &p6,CORBA::Any &p7)");
#endif
  
  ernie.seed("KidneyBean2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(0);
  GetAny(p2);
  ernie.mode(0);
  GetAny(p3);
  ernie.mode(0);
  GetAny(p6);
  ernie.mode(1);
  GetAny(p7);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KidneyBean2Return");
  try {
    ReturnValue = tobj->KidneyBean2(p1,p2,p3,p4,p5,p6,p7);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::KidneyBean2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckAny(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckAny(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckAny(p7))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsZucchini2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any* p2;
  CORBA::Any p3;
  CORBA::Any p4;
  CORBA::Any* p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Zucchini2(CORBA::Any &p1,CORBA::Any *&p2,CORBA::Any &p3,const CORBA::Any &p4,CORBA::Any *&p5)");
#endif
  
  ernie.seed("Zucchini2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(1);
  GetAny(p3);
  ernie.mode(0);
  GetAny(p4);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Zucchini2Return");
  try {
    ReturnValue = tobj->Zucchini2(p1,p2,p3,p4,p5);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Zucchini2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckAny(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckAny(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsButterBean2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any* p2;
  CORBA::Any* p3;
  CORBA::Any p4;
  CORBA::Any* p5;
  CORBA::Any* p6;
  CORBA::Any p7;
  CORBA::Any* p8;
  CORBA::Any p9;
  CORBA::Any p10;
  CORBA::Any p11;
  CORBA::Any p12;
  CORBA::Any p13;
  CORBA::Any* p14;
  CORBA::Any p15;
  CORBA::Any* p16;
  CORBA::Any p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::ButterBean2(CORBA::Any &p1,CORBA::Any *&p2,CORBA::Any *&p3,CORBA::Any &p4,CORBA::Any *&p5,CORBA::Any *&p6,const CORBA::Any &p7,CORBA::Any *&p8,CORBA::Any &p9,CORBA::Any &p10,const CORBA::Any &p11,const CORBA::Any &p12,CORBA::Any &p13,CORBA::Any *&p14,const CORBA::Any &p15,CORBA::Any *&p16,const CORBA::Any &p17)");
#endif
  
  ernie.seed("ButterBean2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(1);
  GetAny(p4);
  ernie.mode(0);
  GetAny(p7);
  ernie.mode(1);
  GetAny(p9);
  ernie.mode(1);
  GetAny(p10);
  ernie.mode(0);
  GetAny(p11);
  ernie.mode(0);
  GetAny(p12);
  ernie.mode(1);
  GetAny(p13);
  ernie.mode(0);
  GetAny(p15);
  ernie.mode(0);
  GetAny(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("ButterBean2Return");
  try {
    ReturnValue = tobj->ButterBean2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::ButterBean2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckAny(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAny(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckAny(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckAny(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckAny(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckAny(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckAny(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckAny(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckAny(*p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckAny(*p16))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsAvocado2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any* p1;
  CORBA::Any* p2;
  CORBA::Any* p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any p6;
  CORBA::Any p7;
  CORBA::Any p8;
  CORBA::Any p9;
  CORBA::Any* p10;
  CORBA::Any* p11;
  CORBA::Any p12;
  CORBA::Any p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Avocado2(CORBA::Any *&p1,CORBA::Any *&p2,CORBA::Any *&p3,CORBA::Any &p4,const CORBA::Any &p5,CORBA::Any &p6,CORBA::Any &p7,const CORBA::Any &p8,const CORBA::Any &p9,CORBA::Any *&p10,CORBA::Any *&p11,const CORBA::Any &p12,CORBA::Any &p13)");
#endif
  
  ernie.seed("Avocado2Call");
  ernie.mode(1);
  GetAny(p4);
  ernie.mode(0);
  GetAny(p5);
  ernie.mode(1);
  GetAny(p6);
  ernie.mode(1);
  GetAny(p7);
  ernie.mode(0);
  GetAny(p8);
  ernie.mode(0);
  GetAny(p9);
  ernie.mode(0);
  GetAny(p12);
  ernie.mode(1);
  GetAny(p13);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Avocado2Return");
  try {
    ReturnValue = tobj->Avocado2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Avocado2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckAny(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckAny(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAny(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckAny(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckAny(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckAny(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckAny(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckAny(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsGoldenDelicous2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any* p1;
  CORBA::Any* p2;
  CORBA::Any* p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::GoldenDelicous2(CORBA::Any *&p1,CORBA::Any *&p2,CORBA::Any *&p3)");
#endif
  
  ernie.seed("GoldenDelicous2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GoldenDelicous2Return");
  try {
    ReturnValue = tobj->GoldenDelicous2(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::GoldenDelicous2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckAny(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckAny(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsLentil2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any p2;
  CORBA::Any p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any p6;
  CORBA::Any p7;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Lentil2(CORBA::Any &p1,CORBA::Any &p2,const CORBA::Any &p3,const CORBA::Any &p4,CORBA::Any &p5,CORBA::Any &p6,CORBA::Any &p7)");
#endif
  
  ernie.seed("Lentil2Call");
  ernie.mode(1);
  GetAny(p1);
  ernie.mode(1);
  GetAny(p2);
  ernie.mode(0);
  GetAny(p3);
  ernie.mode(0);
  GetAny(p4);
  ernie.mode(1);
  GetAny(p5);
  ernie.mode(1);
  GetAny(p6);
  ernie.mode(1);
  GetAny(p7);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lentil2Return");
  try {
    ReturnValue = tobj->Lentil2(p1,p2,p3,p4,p5,p6,p7);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Lentil2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAny(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckAny(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckAny(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckAny(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckAny(p7))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsFig2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any* p1;
  CORBA::Any p2;
  CORBA::Any p3;
  CORBA::Any* p4;
  CORBA::Any p5;
  CORBA::Any p6;
  CORBA::Any* p7;
  CORBA::Any* p8;
  CORBA::Any p9;
  CORBA::Any p10;
  CORBA::Any p11;
  CORBA::Any* p12;
  CORBA::Any p13;
  CORBA::Any p14;
  CORBA::Any p15;
  CORBA::Any p16;
  CORBA::Any p17;
  CORBA::Any p18;
  CORBA::Any* p19;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Fig2(CORBA::Any *&p1,const CORBA::Any &p2,CORBA::Any &p3,CORBA::Any *&p4,const CORBA::Any &p5,CORBA::Any &p6,CORBA::Any *&p7,CORBA::Any *&p8,const CORBA::Any &p9,const CORBA::Any &p10,CORBA::Any &p11,CORBA::Any *&p12,CORBA::Any &p13,const CORBA::Any &p14,CORBA::Any &p15,const CORBA::Any &p16,CORBA::Any &p17,CORBA::Any &p18,CORBA::Any *&p19)");
#endif
  
  ernie.seed("Fig2Call");
  ernie.mode(0);
  GetAny(p2);
  ernie.mode(1);
  GetAny(p3);
  ernie.mode(0);
  GetAny(p5);
  ernie.mode(1);
  GetAny(p6);
  ernie.mode(0);
  GetAny(p9);
  ernie.mode(0);
  GetAny(p10);
  ernie.mode(1);
  GetAny(p11);
  ernie.mode(1);
  GetAny(p13);
  ernie.mode(0);
  GetAny(p14);
  ernie.mode(1);
  GetAny(p15);
  ernie.mode(0);
  GetAny(p16);
  ernie.mode(1);
  GetAny(p17);
  ernie.mode(1);
  GetAny(p18);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Fig2Return");
  try {
    ReturnValue = tobj->Fig2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18,p19);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Fig2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckAny(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckAny(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckAny(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckAny(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckAny(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckAny(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckAny(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckAny(*p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckAny(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckAny(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p17");
  if (!CheckAny(p17))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p18");
  if (!CheckAny(p18))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p19");
  if (!CheckAny(*p19))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsCrabApple2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any* p1;
  CORBA::Any* p2;
  CORBA::Any p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::CrabApple2(CORBA::Any *&p1,CORBA::Any *&p2,CORBA::Any &p3)");
#endif
  
  ernie.seed("CrabApple2Call");
  ernie.mode(1);
  GetAny(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CrabApple2Return");
  try {
    ReturnValue = tobj->CrabApple2(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::CrabApple2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckAny(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckAny(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckAny(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsRedOnion2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any* p1;
  CORBA::Any p2;
  CORBA::Any* p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any p6;
  CORBA::Any p7;
  CORBA::Any p8;
  CORBA::Any* p9;
  CORBA::Any p10;
  CORBA::Any p11;
  CORBA::Any p12;
  CORBA::Any p13;
  CORBA::Any p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::RedOnion2(CORBA::Any *&p1,CORBA::Any &p2,CORBA::Any *&p3,const CORBA::Any &p4,CORBA::Any &p5,const CORBA::Any &p6,CORBA::Any &p7,CORBA::Any &p8,CORBA::Any *&p9,CORBA::Any &p10,const CORBA::Any &p11,CORBA::Any &p12,CORBA::Any &p13,const CORBA::Any &p14)");
#endif
  
  ernie.seed("RedOnion2Call");
  ernie.mode(1);
  GetAny(p2);
  ernie.mode(0);
  GetAny(p4);
  ernie.mode(1);
  GetAny(p5);
  ernie.mode(0);
  GetAny(p6);
  ernie.mode(1);
  GetAny(p7);
  ernie.mode(1);
  GetAny(p8);
  ernie.mode(1);
  GetAny(p10);
  ernie.mode(0);
  GetAny(p11);
  ernie.mode(1);
  GetAny(p12);
  ernie.mode(1);
  GetAny(p13);
  ernie.mode(0);
  GetAny(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedOnion2Return");
  try {
    ReturnValue = tobj->RedOnion2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::RedOnion2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckAny(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckAny(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckAny(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckAny(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckAny(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckAny(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckAny(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckAny(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckAny(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckAny(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsHazelNut2(void)
{
  
  CORBA::Any* ReturnValue;
  CORBA::Any p1;
  CORBA::Any p2;
  CORBA::Any p3;
  CORBA::Any p4;
  CORBA::Any p5;
  CORBA::Any* p6;
  CORBA::Any* p7;
  CORBA::Any p8;
  CORBA::Any p9;
  CORBA::Any p10;
  CORBA::Any* p11;
  CORBA::Any p12;
  CORBA::Any* p13;
  CORBA::Any* p14;
  CORBA::Any* p15;
  CORBA::Any* p16;
  CORBA::Any* p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::HazelNut2(const CORBA::Any &p1,CORBA::Any &p2,CORBA::Any &p3,CORBA::Any &p4,const CORBA::Any &p5,CORBA::Any *&p6,CORBA::Any *&p7,const CORBA::Any &p8,CORBA::Any &p9,const CORBA::Any &p10,CORBA::Any *&p11,CORBA::Any &p12,CORBA::Any *&p13,CORBA::Any *&p14,CORBA::Any *&p15,CORBA::Any *&p16,CORBA::Any *&p17)");
#endif
  
  ernie.seed("HazelNut2Call");
  ernie.mode(0);
  GetAny(p1);
  ernie.mode(1);
  GetAny(p2);
  ernie.mode(1);
  GetAny(p3);
  ernie.mode(1);
  GetAny(p4);
  ernie.mode(0);
  GetAny(p5);
  ernie.mode(0);
  GetAny(p8);
  ernie.mode(1);
  GetAny(p9);
  ernie.mode(0);
  GetAny(p10);
  ernie.mode(1);
  GetAny(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("HazelNut2Return");
  try {
    ReturnValue = tobj->HazelNut2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::HazelNut2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckAny(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckAny(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAny(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckAny(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckAny(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckAny(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckAny(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckAny(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckAny(*p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckAny(*p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckAny(*p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckAny(*p16))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p17");
  if (!CheckAny(*p17))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsCinnamon2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Cinnamon2()");
#endif
  
  ernie.seed("Cinnamon2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cinnamon2Return");
  try {
    ReturnValue = tobj->Cinnamon2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Cinnamon2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsCinnamon2W(void)
{
  
  CORBA::Any _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AnyTests::Cinnamon2(const CORBA::Any &_attr)");
#endif
  
  ernie.seed("Cinnamon2Call");
  ernie.mode(0);
  GetAny(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cinnamon2Return");
  try {
     tobj->Cinnamon2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Cinnamon2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsGooseBerry2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::GooseBerry2()");
#endif
  
  ernie.seed("GooseBerry2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GooseBerry2Return");
  try {
    ReturnValue = tobj->GooseBerry2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::GooseBerry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsGooseBerry2W(void)
{
  
  CORBA::Any _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AnyTests::GooseBerry2(const CORBA::Any &_attr)");
#endif
  
  ernie.seed("GooseBerry2Call");
  ernie.mode(0);
  GetAny(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GooseBerry2Return");
  try {
     tobj->GooseBerry2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::GooseBerry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsEggPlant2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::EggPlant2()");
#endif
  
  ernie.seed("EggPlant2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("EggPlant2Return");
  try {
    ReturnValue = tobj->EggPlant2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::EggPlant2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsEggPlant2W(void)
{
  
  CORBA::Any _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AnyTests::EggPlant2(const CORBA::Any &_attr)");
#endif
  
  ernie.seed("EggPlant2Call");
  ernie.mode(0);
  GetAny(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("EggPlant2Return");
  try {
     tobj->EggPlant2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::EggPlant2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsJuniperBerries2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::JuniperBerries2()");
#endif
  
  ernie.seed("JuniperBerries2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("JuniperBerries2Return");
  try {
    ReturnValue = tobj->JuniperBerries2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::JuniperBerries2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsJuniperBerries2W(void)
{
  
  CORBA::Any _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AnyTests::JuniperBerries2(const CORBA::Any &_attr)");
#endif
  
  ernie.seed("JuniperBerries2Call");
  ernie.mode(0);
  GetAny(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("JuniperBerries2Return");
  try {
     tobj->JuniperBerries2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::JuniperBerries2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsCloves2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Cloves2()");
#endif
  
  ernie.seed("Cloves2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cloves2Return");
  try {
    ReturnValue = tobj->Cloves2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Cloves2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsCloves2W(void)
{
  
  CORBA::Any _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AnyTests::Cloves2(const CORBA::Any &_attr)");
#endif
  
  ernie.seed("Cloves2Call");
  ernie.mode(0);
  GetAny(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cloves2Return");
  try {
     tobj->Cloves2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Cloves2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsDaikan2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Daikan2()");
#endif
  
  ernie.seed("Daikan2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Daikan2Return");
  try {
    ReturnValue = tobj->Daikan2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Daikan2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsDaikan2W(void)
{
  
  CORBA::Any _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AnyTests::Daikan2(const CORBA::Any &_attr)");
#endif
  
  ernie.seed("Daikan2Call");
  ernie.mode(0);
  GetAny(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Daikan2Return");
  try {
     tobj->Daikan2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Daikan2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsMustard2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Mustard2()");
#endif
  
  ernie.seed("Mustard2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mustard2Return");
  try {
    ReturnValue = tobj->Mustard2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Mustard2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsMustard2W(void)
{
  
  CORBA::Any _attr;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AnyTests::Mustard2(const CORBA::Any &_attr)");
#endif
  
  ernie.seed("Mustard2Call");
  ernie.mode(0);
  GetAny(_attr);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mustard2Return");
  try {
     tobj->Mustard2(_attr);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Mustard2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsStrawberry2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Strawberry2()");
#endif
  
  ernie.seed("Strawberry2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Strawberry2Return");
  try {
    ReturnValue = tobj->Strawberry2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Strawberry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsBreadFruit2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::BreadFruit2()");
#endif
  
  ernie.seed("BreadFruit2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BreadFruit2Return");
  try {
    ReturnValue = tobj->BreadFruit2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::BreadFruit2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsThyme2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Thyme2()");
#endif
  
  ernie.seed("Thyme2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Thyme2Return");
  try {
    ReturnValue = tobj->Thyme2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Thyme2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsSweedes2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Sweedes2()");
#endif
  
  ernie.seed("Sweedes2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sweedes2Return");
  try {
    ReturnValue = tobj->Sweedes2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Sweedes2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsPomegranate2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Pomegranate2()");
#endif
  
  ernie.seed("Pomegranate2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pomegranate2Return");
  try {
    ReturnValue = tobj->Pomegranate2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Pomegranate2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsTarragon2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Tarragon2()");
#endif
  
  ernie.seed("Tarragon2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Tarragon2Return");
  try {
    ReturnValue = tobj->Tarragon2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Tarragon2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAnyTestsLettuce2(void)
{
  
  CORBA::Any* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::AnyTests::Lettuce2()");
#endif
  
  ernie.seed("Lettuce2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lettuce2Return");
  try {
    ReturnValue = tobj->Lettuce2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AnyTests::Lettuce2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsMangetout2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  ::YellowPepper2* p1;
  ::Dates3 p2;
  ::Pumpkin p3;
  ::Record2 p4;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::AllTypesTests::Mangetout2(::YellowPepper2 *&p1,const ::Dates3 &p2,const ::Pumpkin p3,::Record2 p4)");
#endif
  
  ernie.seed("Mangetout2Call");
  ernie.mode(0);
  GetDates3(p2);
  ernie.mode(0);
  GetPumpkin(p3);
  ernie.mode(1);
  GetRecord2(p4);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mangetout2Return");
  try {
    ReturnValue = tobj->Mangetout2(p1,p2,p3,p4);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Mangetout2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckYellowPepper2(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckRecord2(p4))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsBambooShoot2(void)
{
  
  
  ::Turnip3* ReturnValue;
  ::Currant2* p1;
  ::Fennell3 p2;
  ::Peanuts1 p3;
  ::GardenPeas3* p4;
  ::Squash2* p5;
  ::BritishQueens1* p6;
  ::Celeriac2* p7;
  ::Rasin2* p8;
  ::Peanuts1 p9;
  ::Fennell3 p10;
  ::Dates3* p11;
  ::Squash1 p12;
  CORBA::UShort p13;
  ::Dates3 p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Turnip3* ::AllTypesTests::BambooShoot2(::Currant2 *&p1,const ::Fennell3 &p2,::Peanuts1 &p3,::GardenPeas3 *&p4,::Squash2 *&p5,::BritishQueens1 *&p6,::Celeriac2 *&p7,::Rasin2 *&p8,::Peanuts1 p9,const ::Fennell3 &p10,::Dates3 *&p11,::Squash1 &p12,CORBA::UShort &p13,::Dates3 &p14)");
#endif
  
  ernie.seed("BambooShoot2Call");
  ernie.mode(0);
  GetFennell3(p2);
  ernie.mode(0);
  GetPeanuts1(p9);
  ernie.mode(0);
  GetFennell3(p10);
  ernie.mode(1);
  GetSquash1(p12);
  ernie.mode(1);
  GetDates3(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BambooShoot2Return");
  try {
    ReturnValue = tobj->BambooShoot2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::BambooShoot2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTurnip3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckCurrant2(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckPeanuts1(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckGardenPeas3(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckSquash2(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckBritishQueens1(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCeleriac2(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckRasin2(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckDates3(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckSquash1(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckUShort(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckDates3(p14))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsMelon2(void)
{
  
  ::Squash2* ReturnValue;
  ::Asparagus p1;
  ::Peanuts1 p2;
  ::Fennell3 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Squash2* ::AllTypesTests::Melon2(::Asparagus &p1,::Peanuts1 &p2,const ::Fennell3 &p3)");
#endif
  
  ernie.seed("Melon2Call");
  ernie.mode(1);
  GetAsparagus(p1);
  ernie.mode(1);
  GetPeanuts1(p2);
  ernie.mode(0);
  GetFennell3(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Melon2Return");
  try {
    ReturnValue = tobj->Melon2(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Melon2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSquash2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAsparagus(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckPeanuts1(p2))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsRedGrapes2(void)
{
  
  CORBA::Char * ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char * ::AllTypesTests::RedGrapes2()");
#endif
  
  ernie.seed("RedGrapes2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedGrapes2Return");
  try {
    ReturnValue = tobj->RedGrapes2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::RedGrapes2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckString(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsElderberry2(void)
{
  
  ::BeechNut2* ReturnValue;
  ::Oregano3* p1;
  ::Squash2 p2;
  CORBA::TypeCode_ptr p3;
  ::GoldenDelicous3 p4;
  ::Corn1 p5;
  ::GardenPeas3 p6;
  ::Chives2 p7;
  ::Squash2 p8;
  CORBA::Long p9;
  ::Raspberry1 p10;
  ::Corn1 p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BeechNut2* ::AllTypesTests::Elderberry2(::Oregano3 *&p1,const ::Squash2 &p2,CORBA::TypeCode_ptr &p3,::GoldenDelicous3 &p4,const ::Corn1 p5,::GardenPeas3 &p6,const ::Chives2 &p7,::Squash2 &p8,CORBA::Long p9,::Raspberry1 p10,::Corn1 p11)");
#endif
  
  ernie.seed("Elderberry2Call");
  ernie.mode(0);
  GetSquash2(p2);
  ernie.mode(1);
  GetGoldenDelicous3(p4);
  ernie.mode(0);
  GetCorn1(p5);
  ernie.mode(1);
  GetGardenPeas3(p6);
  ernie.mode(0);
  GetChives2(p7);
  ernie.mode(1);
  GetSquash2(p8);
  ernie.mode(0);
  GetLong(p9);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Elderberry2Return");
  try {
    ReturnValue = tobj->Elderberry2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Elderberry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBeechNut2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckOregano3(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckGoldenDelicous3(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckGardenPeas3(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckSquash2(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckRaspberry1(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckCorn1(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsKumquats2(void)
{
  
  CORBA::Char ReturnValue;
  ::Rasin2 p1;
  ::Squash2 p2;
  ::Mullberry1 p3;
  CORBA::Double p4;
  ::GaliaMelon3* p5;
  CORBA::Double p6;
  ::Pumpkin p7;
  ::Peanuts1 p8;
  CORBA::Any p9;
  ::Oregano3 p10;
  ::GaliaMelon3 p11;
  CORBA::UShort p12;
  CORBA::Float p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char ::AllTypesTests::Kumquats2(const ::Rasin2 &p1,const ::Squash2 &p2,const ::Mullberry1 p3,CORBA::Double &p4,::GaliaMelon3 *&p5,CORBA::Double &p6,const ::Pumpkin p7,::Peanuts1 p8,CORBA::Any &p9,::Oregano3 &p10,::GaliaMelon3 &p11,CORBA::UShort &p12,CORBA::Float p13)");
#endif
  
  ernie.seed("Kumquats2Call");
  ernie.mode(0);
  GetRasin2(p1);
  ernie.mode(0);
  GetSquash2(p2);
  ernie.mode(0);
  GetMullberry1(p3);
  ernie.mode(0);
  GetPumpkin(p7);
  ernie.mode(0);
  GetPeanuts1(p8);
  ernie.mode(1);
  GetAny(p9);
  ernie.mode(1);
  GetOregano3(p10);
  ernie.mode(1);
  GetGaliaMelon3(p11);
  ernie.mode(1);
  GetUShort(p12);
  ernie.mode(0);
  GetFloat(p13);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Kumquats2Return");
  try {
    ReturnValue = tobj->Kumquats2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Kumquats2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChar(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckDouble(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckGaliaMelon3(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckDouble(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckAny(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckOregano3(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckGaliaMelon3(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckUShort(p12))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsFennell2(void)
{
  
  ::Cauliflower2* ReturnValue;
  ::Beetroot2 p1;
  ::Mullberry1 p2;
  ::Carrots p3;
  CORBA::TypeCode_ptr p4;
  ::BeechNut2* p5;
  CORBA::Long p6;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Cauliflower2* ::AllTypesTests::Fennell2(::Beetroot2 &p1,const ::Mullberry1 p2,::Carrots p3,CORBA::TypeCode_ptr &p4,::BeechNut2 *&p5,CORBA::Long &p6)");
#endif
  
  ernie.seed("Fennell2Call");
  ernie.mode(1);
  GetBeetroot2(p1);
  ernie.mode(0);
  GetMullberry1(p2);
  ernie.mode(1);
  GetCarrots(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Fennell2Return");
  try {
    ReturnValue = tobj->Fennell2(p1,p2,p3,p4,p5,p6);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Fennell2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCauliflower2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckBeetroot2(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCarrots(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCORBATypeCode_ptr(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckBeechNut2(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckLong(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsUglifruit2(void)
{
  
  CORBA::Char * p1;
  
  ::Turnip3 p2;
  
  ::Peppercorn p3;
  CORBA::ULong p4;
  ::GardenPeas1 p5;
  ::Squash2 p6;
  ::Raspberry1 p7;
  
  ::Susander2* p8;
  CORBA::Object_ptr p9;
  ::BrazilNut3* p10;
  ::Corn1 p11;
  ::GaliaMelon3 p12;
  ::BritishQueens1 p13;
  ::BritishQueens1 p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AllTypesTests::Uglifruit2(CORBA::Char * &p1,::Turnip3 &p2,const ::Peppercorn &p3,CORBA::ULong p4,::GardenPeas1 &p5,::Squash2 &p6,const ::Raspberry1 p7,::Susander2 *&p8,CORBA::Object_ptr &p9,::BrazilNut3 *&p10,::Corn1 p11,const ::GaliaMelon3 &p12,::BritishQueens1 &p13,const ::BritishQueens1 &p14)");
#endif
  
  ernie.seed("Uglifruit2Call");
  ernie.mode(1);
  GetTurnip3(p2);
  ernie.mode(0);
  GetPeppercorn(p3);
  ernie.mode(0);
  GetULong(p4);
  ernie.mode(1);
  GetGardenPeas1(p5);
  ernie.mode(1);
  GetSquash2(p6);
  ernie.mode(0);
  GetRaspberry1(p7);
  ernie.mode(1);
  GetCorn1(p11);
  ernie.mode(0);
  GetGaliaMelon3(p12);
  ernie.mode(1);
  GetBritishQueens1(p13);
  ernie.mode(0);
  GetBritishQueens1(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Uglifruit2Return");
  try {
     tobj->Uglifruit2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Uglifruit2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  ernie.mode(0);
  AddName("p1");
  if (!CheckString(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckTurnip3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckGardenPeas1(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckSquash2(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckSusander2(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!Checkobject(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckBrazilNut3(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckCorn1(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckBritishQueens1(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsChickpea2(void)
{
  
  ::Dates3* ReturnValue;
  ::Squash1 p1;
  ::Tomato2* p2;
  CORBA::Char * p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Dates3* ::AllTypesTests::Chickpea2(const ::Squash1 &p1,::Tomato2 *&p2,CORBA::Char * &p3)");
#endif
  
  ernie.seed("Chickpea2Call");
  ernie.mode(0);
  GetSquash1(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Chickpea2Return");
  try {
    ReturnValue = tobj->Chickpea2(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Chickpea2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDates3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckTomato2(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckString(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsOkra2(void)
{
  
  ::Beetroot2* ReturnValue;
  ::Squash1 p1;
  CORBA::Float p2;
  ::Tomato2* p3;
  ::Chives2* p4;
  ::GaliaMelon3* p5;
  CORBA::Object_ptr p6;
  
  ::Blackberry2 p7;
  ::Rasin2 p8;
  ::Fennell3* p9;
  CORBA::Char * p10;
  ::Oregano3 p11;
  ::GaliaMelon3 p12;
  ::Peanuts1 p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Beetroot2* ::AllTypesTests::Okra2(::Squash1 &p1,CORBA::Float &p2,::Tomato2 *&p3,::Chives2 *&p4,::GaliaMelon3 *&p5,CORBA::Object_ptr p6,const ::Blackberry2 &p7,const ::Rasin2 &p8,::Fennell3 *&p9,CORBA::Char * &p10,::Oregano3 &p11,const ::GaliaMelon3 &p12,::Peanuts1 &p13)");
#endif
  
  ernie.seed("Okra2Call");
  ernie.mode(1);
  GetSquash1(p1);
  ernie.mode(1);
  GetFloat(p2);
  ernie.mode(0);
  p6 = Getobject();
  ernie.mode(0);
  GetBlackberry2(p7);
  ernie.mode(0);
  GetRasin2(p8);
  ernie.mode(1);
  GetOregano3(p11);
  ernie.mode(0);
  GetGaliaMelon3(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Okra2Return");
  try {
    ReturnValue = tobj->Okra2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Okra2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBeetroot2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckSquash1(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckFloat(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckTomato2(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckChives2(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckGaliaMelon3(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckFennell3(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckString(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckOregano3(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckPeanuts1(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsPlantain2(void)
{
  
  CORBA::Float ReturnValue;
  CORBA::Long p1;
  
  ::Susander2 p2;
  
  ::Fig p3;
  ::Tomato2 p4;
  ::Cauliflower2 p5;
  ::Oregano3 p6;
  ::Mangosteen3 p7;
  ::Squash1 p8;
  ::LoganBerry* p9;
  ::Squash2 p10;
  
  ::Tomato* p11;
  ::LoganBerry p12;
  ::Tomato2 p13;
  ::GaliaMelon3* p14;
  CORBA::TypeCode_ptr p15;
  ::BritishQueens1* p16;
  
  ::Susander2 p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Float ::AllTypesTests::Plantain2(CORBA::Long &p1,::Susander2 &p2,const ::Fig &p3,const ::Tomato2 &p4,const ::Cauliflower2 &p5,::Oregano3 &p6,::Mangosteen3 &p7,const ::Squash1 &p8,::LoganBerry *&p9,::Squash2 &p10,::Tomato *&p11,::LoganBerry &p12,::Tomato2 &p13,::GaliaMelon3 *&p14,CORBA::TypeCode_ptr p15,::BritishQueens1 *&p16,const ::Susander2 &p17)");
#endif
  
  ernie.seed("Plantain2Call");
  ernie.mode(1);
  GetLong(p1);
  ernie.mode(1);
  GetSusander2(p2);
  ernie.mode(0);
  GetFig(p3);
  ernie.mode(0);
  GetTomato2(p4);
  ernie.mode(0);
  GetCauliflower2(p5);
  ernie.mode(1);
  GetOregano3(p6);
  ernie.mode(1);
  GetMangosteen3(p7);
  ernie.mode(0);
  GetSquash1(p8);
  ernie.mode(1);
  GetSquash2(p10);
  ernie.mode(1);
  GetLoganBerry(p12);
  ernie.mode(1);
  GetTomato2(p13);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p15);
  ernie.mode(0);
  GetSusander2(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Plantain2Return");
  try {
    ReturnValue = tobj->Plantain2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Plantain2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFloat(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckLong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckSusander2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckOregano3(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckMangosteen3(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckLoganBerry(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckSquash2(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckTomato(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckLoganBerry(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckTomato2(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckGaliaMelon3(*p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckBritishQueens1(*p16))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsBlackcurrant2(void)
{
  
  
  ::Susander2* p1;
  ::Chives2 p2;
  ::Damsons2 p3;
  ::GardenPeas1 p4;
  
  ::Tomato* p5;
  ::WaterMelon3 p6;
  
  ::Susander2* p7;
  CORBA::TypeCode_ptr p8;
  ::Damsons2 p9;
  ::Currant2 p10;
  CORBA::Boolean p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::AllTypesTests::Blackcurrant2(::Susander2 *&p1,::Chives2 &p2,const ::Damsons2 p3,const ::GardenPeas1 &p4,::Tomato *&p5,const ::WaterMelon3 p6,::Susander2 *&p7,CORBA::TypeCode_ptr &p8,const ::Damsons2 p9,::Currant2 &p10,CORBA::Boolean &p11)");
#endif
  
  ernie.seed("Blackcurrant2Call");
  ernie.mode(1);
  GetChives2(p2);
  ernie.mode(0);
  GetDamsons2(p3);
  ernie.mode(0);
  GetGardenPeas1(p4);
  ernie.mode(0);
  GetWaterMelon3(p6);
  ernie.mode(0);
  GetDamsons2(p9);
  ernie.mode(1);
  GetCurrant2(p10);
  ernie.mode(1);
  GetBoolean(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Blackcurrant2Return");
  try {
     tobj->Blackcurrant2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Blackcurrant2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  ernie.mode(0);
  AddName("p1");
  if (!CheckSusander2(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckChives2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckTomato(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckSusander2(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckCORBATypeCode_ptr(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckCurrant2(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckBoolean(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsPineapple2(void)
{
  
  CORBA::Object_ptr ReturnValue;
  ::Fennell3* p1;
  ::Carrots p2;
  ::Chestnut1 p3;
  ::GardenPeas1* p4;
  CORBA::Float p5;
  CORBA::TypeCode_ptr p6;
  ::Dates3 p7;
  ::BeechNut2 p8;
  CORBA::Float p9;
  ::Rasin2 p10;
  CORBA::Char p11;
  ::Mullberry1 p12;
  ::BlackEyedBeans p13;
  ::Squash2 p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Object_ptr ::AllTypesTests::Pineapple2(::Fennell3 *&p1,const ::Carrots p2,::Chestnut1 p3,::GardenPeas1 *&p4,CORBA::Float p5,CORBA::TypeCode_ptr &p6,const ::Dates3 &p7,::BeechNut2 &p8,CORBA::Float p9,::Rasin2 &p10,CORBA::Char &p11,::Mullberry1 p12,::BlackEyedBeans &p13,const ::Squash2 &p14)");
#endif
  
  ernie.seed("Pineapple2Call");
  ernie.mode(0);
  GetCarrots(p2);
  ernie.mode(1);
  GetChestnut1(p3);
  ernie.mode(0);
  GetFloat(p5);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(0);
  GetDates3(p7);
  ernie.mode(1);
  GetBeechNut2(p8);
  ernie.mode(0);
  GetFloat(p9);
  ernie.mode(1);
  GetRasin2(p10);
  ernie.mode(1);
  GetMullberry1(p12);
  ernie.mode(1);
  GetBlackEyedBeans(p13);
  ernie.mode(0);
  GetSquash2(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pineapple2Return");
  try {
    ReturnValue = tobj->Pineapple2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Pineapple2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!Checkobject(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckFennell3(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckChestnut1(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckGardenPeas1(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckBeechNut2(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckRasin2(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckChar(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckMullberry1(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckBlackEyedBeans(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsMangosteen2(void)
{
  
  ::Cauliflower2* ReturnValue;
  CORBA::Octet p1;
  CORBA::Any p2;
  
  ::Fig* p3;
  ::Damsons2 p4;
  CORBA::Double p5;
  ::GaliaMelon3* p6;
  CORBA::Object_ptr p7;
  CORBA::Short p8;
  ::WaterMelon3 p9;
  ::Currant2* p10;
  ::Peanuts1 p11;
  CORBA::Float p12;
  ::BrazilNut3 p13;
  ::WaterMelon3 p14;
  
  ::Susander2 p15;
  ::Tomato2* p16;
  ::GaliaMelon3 p17;
  ::Fennell3* p18;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Cauliflower2* ::AllTypesTests::Mangosteen2(CORBA::Octet &p1,CORBA::Any &p2,::Fig *&p3,const ::Damsons2 p4,CORBA::Double &p5,::GaliaMelon3 *&p6,CORBA::Object_ptr &p7,CORBA::Short p8,::WaterMelon3 p9,::Currant2 *&p10,::Peanuts1 &p11,CORBA::Float p12,::BrazilNut3 &p13,::WaterMelon3 p14,::Susander2 &p15,::Tomato2 *&p16,const ::GaliaMelon3 &p17,::Fennell3 *&p18)");
#endif
  
  ernie.seed("Mangosteen2Call");
  ernie.mode(1);
  GetOctet(p1);
  ernie.mode(1);
  GetAny(p2);
  ernie.mode(0);
  GetDamsons2(p4);
  ernie.mode(1);
  GetDouble(p5);
  ernie.mode(1);
  p7 = Getobject();
  ernie.mode(0);
  GetShort(p8);
  ernie.mode(0);
  GetFloat(p12);
  ernie.mode(1);
  GetBrazilNut3(p13);
  ernie.mode(1);
  GetSusander2(p15);
  ernie.mode(0);
  GetGaliaMelon3(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mangosteen2Return");
  try {
    ReturnValue = tobj->Mangosteen2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Mangosteen2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCauliflower2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckOctet(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckAny(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckFig(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckDouble(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckGaliaMelon3(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!Checkobject(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckWaterMelon3(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckCurrant2(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckPeanuts1(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckBrazilNut3(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckWaterMelon3(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckSusander2(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckTomato2(*p16))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p18");
  if (!CheckFennell3(*p18))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsPear2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  ::Squash1* p1;
  
  ::Fig* p2;
  CORBA::TypeCode_ptr p3;
  CORBA::Octet p4;
  ::BritishQueens1* p5;
  ::Dates3 p6;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::AllTypesTests::Pear2(::Squash1 *&p1,::Fig *&p2,CORBA::TypeCode_ptr &p3,CORBA::Octet &p4,::BritishQueens1 *&p5,::Dates3 &p6)");
#endif
  
  ernie.seed("Pear2Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(1);
  GetOctet(p4);
  ernie.mode(1);
  GetDates3(p6);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pear2Return");
  try {
    ReturnValue = tobj->Pear2(p1,p2,p3,p4,p5,p6);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Pear2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckSquash1(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckFig(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckOctet(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckBritishQueens1(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckDates3(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsBeefTomatoes2(void)
{
  
  ::Celeriac2* ReturnValue;
  
  ::Fig p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Celeriac2* ::AllTypesTests::BeefTomatoes2(const ::Fig &p1)");
#endif
  
  ernie.seed("BeefTomatoes2Call");
  ernie.mode(0);
  GetFig(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BeefTomatoes2Return");
  try {
    ReturnValue = tobj->BeefTomatoes2(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::BeefTomatoes2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCeleriac2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsSharanfruit2(void)
{
  
  ::Fennell3* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Fennell3* ::AllTypesTests::Sharanfruit2()");
#endif
  
  ernie.seed("Sharanfruit2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sharanfruit2Return");
  try {
    ReturnValue = tobj->Sharanfruit2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Sharanfruit2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFennell3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsCherry2(void)
{
  
  ::Cauliflower2* ReturnValue;
  ::GoldenDelicous3 p1;
  CORBA::Object_ptr p2;
  
  ::Susander2 p3;
  CORBA::TypeCode_ptr p4;
  ::Cauliflower2 p5;
  
  ::Fig p6;
  ::Fennell3* p7;
  ::Rasin2 p8;
  ::Currant2* p9;
  CORBA::Boolean p10;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Cauliflower2* ::AllTypesTests::Cherry2(const ::GoldenDelicous3 &p1,CORBA::Object_ptr p2,const ::Susander2 &p3,CORBA::TypeCode_ptr p4,::Cauliflower2 &p5,const ::Fig &p6,::Fennell3 *&p7,const ::Rasin2 &p8,::Currant2 *&p9,CORBA::Boolean &p10)");
#endif
  
  ernie.seed("Cherry2Call");
  ernie.mode(0);
  GetGoldenDelicous3(p1);
  ernie.mode(0);
  p2 = Getobject();
  ernie.mode(0);
  GetSusander2(p3);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p4);
  ernie.mode(1);
  GetCauliflower2(p5);
  ernie.mode(0);
  GetFig(p6);
  ernie.mode(0);
  GetRasin2(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cherry2Return");
  try {
    ReturnValue = tobj->Cherry2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Cherry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCauliflower2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCauliflower2(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckFennell3(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckCurrant2(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckBoolean(p10))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsPeppercorn2(void)
{
  
  ::Oregano3* ReturnValue;
  ::GoldenDelicous3 p1;
  ::Dates3* p2;
  ::Tomato2 p3;
  ::Fennell3 p4;
  CORBA::Octet p5;
  ::Peanuts1 p6;
  ::Raspberry1 p7;
  CORBA::Short p8;
  ::WaterMelon p9;
  CORBA::Any* p10;
  ::Fennell3* p11;
  ::Fennell3 p12;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Oregano3* ::AllTypesTests::Peppercorn2(const ::GoldenDelicous3 &p1,::Dates3 *&p2,const ::Tomato2 &p3,const ::Fennell3 &p4,CORBA::Octet &p5,::Peanuts1 &p6,::Raspberry1 p7,CORBA::Short &p8,const ::WaterMelon &p9,CORBA::Any *&p10,::Fennell3 *&p11,::Fennell3 &p12)");
#endif
  
  ernie.seed("Peppercorn2Call");
  ernie.mode(0);
  GetGoldenDelicous3(p1);
  ernie.mode(0);
  GetTomato2(p3);
  ernie.mode(0);
  GetFennell3(p4);
  ernie.mode(1);
  GetOctet(p5);
  ernie.mode(1);
  GetPeanuts1(p6);
  ernie.mode(1);
  GetRaspberry1(p7);
  ernie.mode(0);
  GetWaterMelon(p9);
  ernie.mode(1);
  GetFennell3(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Peppercorn2Return");
  try {
    ReturnValue = tobj->Peppercorn2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Peppercorn2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOregano3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckDates3(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckOctet(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckPeanuts1(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckRaspberry1(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckShort(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckAny(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckFennell3(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckFennell3(p12))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsRaspberry2(void)
{
  
  
  ::Susander2* ReturnValue;
  ::Mullberry1 p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Susander2* ::AllTypesTests::Raspberry2(const ::Mullberry1 p1)");
#endif
  
  ernie.seed("Raspberry2Call");
  ernie.mode(0);
  GetMullberry1(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Raspberry2Return");
  try {
    ReturnValue = tobj->Raspberry2(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Raspberry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSusander2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsPepinoMelon2(void)
{
  
  ::Grapefruit2* ReturnValue;
  ::Dates3 p1;
  
  ::Peppercorn p2;
  
  ::Fig p3;
  ::Tomato2* p4;
  ::Cauliflower2 p5;
  ::BritishQueens1* p6;
  CORBA::ULong p7;
  ::Mullberry1 p8;
  ::Cauliflower2 p9;
  ::Squash2 p10;
  ::Asparagus p11;
  ::Squash2 p12;
  ::Beetroot2* p13;
  CORBA::Object_ptr p14;
  ::Rasin2 p15;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Grapefruit2* ::AllTypesTests::PepinoMelon2(::Dates3 &p1,::Peppercorn &p2,::Fig &p3,::Tomato2 *&p4,const ::Cauliflower2 &p5,::BritishQueens1 *&p6,CORBA::ULong p7,::Mullberry1 p8,const ::Cauliflower2 &p9,const ::Squash2 &p10,::Asparagus &p11,const ::Squash2 &p12,::Beetroot2 *&p13,CORBA::Object_ptr &p14,::Rasin2 &p15)");
#endif
  
  ernie.seed("PepinoMelon2Call");
  ernie.mode(1);
  GetDates3(p1);
  ernie.mode(1);
  GetPeppercorn(p2);
  ernie.mode(1);
  GetFig(p3);
  ernie.mode(0);
  GetCauliflower2(p5);
  ernie.mode(0);
  GetULong(p7);
  ernie.mode(0);
  GetCauliflower2(p9);
  ernie.mode(0);
  GetSquash2(p10);
  ernie.mode(1);
  GetAsparagus(p11);
  ernie.mode(0);
  GetSquash2(p12);
  ernie.mode(1);
  GetRasin2(p15);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PepinoMelon2Return");
  try {
    ReturnValue = tobj->PepinoMelon2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::PepinoMelon2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGrapefruit2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckDates3(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckPeppercorn(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckFig(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckTomato2(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckBritishQueens1(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckMullberry1(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckAsparagus(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckBeetroot2(*p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!Checkobject(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckRasin2(p15))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsPeanutsquash2(void)
{
  
  ::Damsons2_slice * ReturnValue;
  ::Rasin2 p1;
  CORBA::ULong p2;
  ::Mangosteen3 p3;
  ::Chestnut1 p4;
  ::Dates3 p5;
  CORBA::ULong p6;
  ::Squash2 p7;
  
  ::Susander2 p8;
  
  ::Fig p9;
  ::Grapefruit2 p10;
  CORBA::Object_ptr p11;
  
  ::Turnip3* p12;
  
  ::Peppercorn p13;
  ::Squash2 p14;
  ::Dates3* p15;
  ::Rasin2 p16;
  
  ::Tomato* p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Damsons2_slice * ::AllTypesTests::Peanutsquash2(::Rasin2 &p1,CORBA::ULong p2,const ::Mangosteen3 &p3,::Chestnut1 p4,const ::Dates3 &p5,CORBA::ULong &p6,const ::Squash2 &p7,::Susander2 &p8,::Fig &p9,::Grapefruit2 &p10,CORBA::Object_ptr p11,::Turnip3 *&p12,const ::Peppercorn &p13,::Squash2 &p14,::Dates3 *&p15,const ::Rasin2 &p16,::Tomato *&p17)");
#endif
  
  ernie.seed("Peanutsquash2Call");
  ernie.mode(1);
  GetRasin2(p1);
  ernie.mode(0);
  GetULong(p2);
  ernie.mode(0);
  GetMangosteen3(p3);
  ernie.mode(0);
  GetDates3(p5);
  ernie.mode(0);
  GetSquash2(p7);
  ernie.mode(1);
  GetSusander2(p8);
  ernie.mode(1);
  GetFig(p9);
  ernie.mode(1);
  GetGrapefruit2(p10);
  ernie.mode(0);
  p11 = Getobject();
  ernie.mode(0);
  GetPeppercorn(p13);
  ernie.mode(1);
  GetSquash2(p14);
  ernie.mode(0);
  GetRasin2(p16);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Peanutsquash2Return");
  try {
    ReturnValue = tobj->Peanutsquash2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Peanutsquash2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDamsons2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckRasin2(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckChestnut1(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckULong(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckSusander2(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckFig(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckGrapefruit2(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckTurnip3(*p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckSquash2(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckDates3(*p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p17");
  if (!CheckTomato(*p17))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsArtichokes2(void)
{
  
  ::GaliaMelon3* ReturnValue;
  
  ::Blackberry2* p1;
  ::Dates3* p2;
  ::Tomato2 p3;
  ::Squash1 p4;
  ::Mangosteen3 p5;
  ::WaterMelon* p6;
  CORBA::Octet p7;
  
  ::Susander2 p8;
  CORBA::UShort p9;
  CORBA::TypeCode_ptr p10;
  ::Grapefruit2 p11;
  ::Cauliflower2 p12;
  ::Currant2 p13;
  
  ::Fig p14;
  CORBA::TypeCode_ptr p15;
  ::Chestnut1 p16;
  ::Corn1 p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GaliaMelon3* ::AllTypesTests::Artichokes2(::Blackberry2 *&p1,::Dates3 *&p2,const ::Tomato2 &p3,const ::Squash1 &p4,::Mangosteen3 &p5,::WaterMelon *&p6,CORBA::Octet &p7,::Susander2 &p8,CORBA::UShort &p9,CORBA::TypeCode_ptr p10,const ::Grapefruit2 &p11,::Cauliflower2 &p12,::Currant2 &p13,::Fig &p14,CORBA::TypeCode_ptr &p15,const ::Chestnut1 p16,const ::Corn1 p17)");
#endif
  
  ernie.seed("Artichokes2Call");
  ernie.mode(0);
  GetTomato2(p3);
  ernie.mode(0);
  GetSquash1(p4);
  ernie.mode(1);
  GetMangosteen3(p5);
  ernie.mode(1);
  GetSusander2(p8);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p10);
  ernie.mode(0);
  GetGrapefruit2(p11);
  ernie.mode(1);
  GetCauliflower2(p12);
  ernie.mode(1);
  GetCurrant2(p13);
  ernie.mode(1);
  GetFig(p14);
  ernie.mode(0);
  GetChestnut1(p16);
  ernie.mode(0);
  GetCorn1(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Artichokes2Return");
  try {
    ReturnValue = tobj->Artichokes2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Artichokes2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGaliaMelon3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckBlackberry2(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckDates3(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckMangosteen3(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckWaterMelon(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckOctet(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckSusander2(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckUShort(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckCauliflower2(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckCurrant2(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckFig(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckCORBATypeCode_ptr(p15))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsGreenGrapes2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  ::WaterMelon p1;
  
  ::Fig p2;
  ::Celeriac2 p3;
  ::Corn1 p4;
  CORBA::Char * p5;
  ::GoldenDelicous3 p6;
  ::GaliaMelon3* p7;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::AllTypesTests::GreenGrapes2(const ::WaterMelon &p1,const ::Fig &p2,const ::Celeriac2 &p3,const ::Corn1 p4,CORBA::Char * &p5,const ::GoldenDelicous3 &p6,::GaliaMelon3 *&p7)");
#endif
  
  ernie.seed("GreenGrapes2Call");
  ernie.mode(0);
  GetWaterMelon(p1);
  ernie.mode(0);
  GetFig(p2);
  ernie.mode(0);
  GetCeleriac2(p3);
  ernie.mode(0);
  GetCorn1(p4);
  ernie.mode(1);
  p5 = GetString();
  ernie.mode(0);
  GetGoldenDelicous3(p6);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GreenGrapes2Return");
  try {
    ReturnValue = tobj->GreenGrapes2(p1,p2,p3,p4,p5,p6,p7);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::GreenGrapes2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckString(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckGaliaMelon3(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsCookingApple2(void)
{
  
  ::Rasin2* ReturnValue;
  CORBA::Long p1;
  ::Cauliflower2 p2;
  ::Tomato2 p3;
  ::Tomato2* p4;
  ::Celeriac2 p5;
  ::BambooShoot3* p6;
  ::Currant2* p7;
  ::Cauliflower2 p8;
  ::BlackEyedBeans p9;
  ::BlackEyedBeans p10;
  ::Dates3 p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Rasin2* ::AllTypesTests::CookingApple2(CORBA::Long &p1,const ::Cauliflower2 &p2,const ::Tomato2 &p3,::Tomato2 *&p4,const ::Celeriac2 &p5,::BambooShoot3 *&p6,::Currant2 *&p7,const ::Cauliflower2 &p8,const ::BlackEyedBeans &p9,const ::BlackEyedBeans &p10,::Dates3 &p11)");
#endif
  
  ernie.seed("CookingApple2Call");
  ernie.mode(0);
  GetCauliflower2(p2);
  ernie.mode(0);
  GetTomato2(p3);
  ernie.mode(0);
  GetCeleriac2(p5);
  ernie.mode(0);
  GetCauliflower2(p8);
  ernie.mode(0);
  GetBlackEyedBeans(p9);
  ernie.mode(0);
  GetBlackEyedBeans(p10);
  ernie.mode(1);
  GetDates3(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("CookingApple2Return");
  try {
    ReturnValue = tobj->CookingApple2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::CookingApple2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRasin2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckLong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckTomato2(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckBambooShoot3(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCurrant2(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckDates3(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsMullberry2(void)
{
  
  ::Chives2* ReturnValue;
  CORBA::Object_ptr p1;
  ::Cauliflower2 p2;
  ::Tomato2 p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Chives2* ::AllTypesTests::Mullberry2(CORBA::Object_ptr &p1,::Cauliflower2 &p2,::Tomato2 &p3)");
#endif
  
  ernie.seed("Mullberry2Call");
  ernie.mode(1);
  GetCauliflower2(p2);
  ernie.mode(1);
  GetTomato2(p3);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mullberry2Return");
  try {
    ReturnValue = tobj->Mullberry2(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Mullberry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChives2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!Checkobject(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckCauliflower2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckTomato2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsYam2(void)
{
  
  ::GoldenDelicous3* ReturnValue;
  ::WaterMelon3 p1;
  CORBA::Any p2;
  CORBA::Char * p3;
  
  ::IcebergLettuce2 p4;
  CORBA::UShort p5;
  ::Dates3 p6;
  CORBA::Boolean p7;
  ::GaliaMelon3 p8;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GoldenDelicous3* ::AllTypesTests::Yam2(::WaterMelon3 p1,CORBA::Any &p2,CORBA::Char * &p3,::IcebergLettuce2 &p4,CORBA::UShort p5,::Dates3 &p6,CORBA::Boolean p7,const ::GaliaMelon3 &p8)");
#endif
  
  ernie.seed("Yam2Call");
  ernie.mode(1);
  GetAny(p2);
  ernie.mode(1);
  GetIcebergLettuce2(p4);
  ernie.mode(0);
  GetUShort(p5);
  ernie.mode(1);
  GetDates3(p6);
  ernie.mode(0);
  GetBoolean(p7);
  ernie.mode(0);
  GetGaliaMelon3(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Yam2Return");
  try {
    ReturnValue = tobj->Yam2(p1,p2,p3,p4,p5,p6,p7,p8);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Yam2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGoldenDelicous3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckWaterMelon3(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckAny(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckString(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckIcebergLettuce2(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckDates3(p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsGreenChilly2(void)
{
  
  CORBA::Short ReturnValue;
  ::Mullberry1 p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Short ::AllTypesTests::GreenChilly2(const ::Mullberry1 p1)");
#endif
  
  ernie.seed("GreenChilly2Call");
  ernie.mode(0);
  GetMullberry1(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GreenChilly2Return");
  try {
    ReturnValue = tobj->GreenChilly2(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::GreenChilly2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsCorn2(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::AllTypesTests::Corn2()");
#endif
  
  ernie.seed("Corn2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Corn2Return");
  try {
    ReturnValue = tobj->Corn2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Corn2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsPeach2(void)
{
  
  CORBA::Char ReturnValue;
  CORBA::Double p1;
  ::Asparagus p2;
  ::Carrots p3;
  ::Peanuts1 p4;
  ::GaliaMelon3 p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char ::AllTypesTests::Peach2(CORBA::Double &p1,::Asparagus &p2,::Carrots p3,::Peanuts1 p4,const ::GaliaMelon3 &p5)");
#endif
  
  ernie.seed("Peach2Call");
  ernie.mode(1);
  GetAsparagus(p2);
  ernie.mode(1);
  GetCarrots(p3);
  ernie.mode(0);
  GetPeanuts1(p4);
  ernie.mode(0);
  GetGaliaMelon3(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Peach2Return");
  try {
    ReturnValue = tobj->Peach2(p1,p2,p3,p4,p5);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Peach2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChar(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckDouble(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckAsparagus(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCarrots(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsParsley2(void)
{
  
  ::Squash2* ReturnValue;
  ::Pumpkin p1;
  ::LoganBerry p2;
  ::Cauliflower2 p3;
  ::YellowPepper2* p4;
  ::Mangosteen3 p5;
  
  ::Susander2 p6;
  ::Asparagus p7;
  CORBA::Float p8;
  CORBA::Octet p9;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Squash2* ::AllTypesTests::Parsley2(const ::Pumpkin p1,const ::LoganBerry &p2,const ::Cauliflower2 &p3,::YellowPepper2 *&p4,const ::Mangosteen3 &p5,::Susander2 &p6,::Asparagus &p7,CORBA::Float &p8,CORBA::Octet &p9)");
#endif
  
  ernie.seed("Parsley2Call");
  ernie.mode(0);
  GetPumpkin(p1);
  ernie.mode(0);
  GetLoganBerry(p2);
  ernie.mode(0);
  GetCauliflower2(p3);
  ernie.mode(0);
  GetMangosteen3(p5);
  ernie.mode(1);
  GetSusander2(p6);
  ernie.mode(1);
  GetAsparagus(p7);
  ernie.mode(1);
  GetFloat(p8);
  ernie.mode(1);
  GetOctet(p9);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Parsley2Return");
  try {
    ReturnValue = tobj->Parsley2(p1,p2,p3,p4,p5,p6,p7,p8,p9);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Parsley2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckSquash2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckYellowPepper2(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckSusander2(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckAsparagus(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckFloat(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckOctet(p9))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsSatsuma2(void)
{
  
  ::YellowPepper2* ReturnValue;
  
  ::Susander2 p1;
  ::Squash1 p2;
  ::GardenPeas3* p3;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::YellowPepper2* ::AllTypesTests::Satsuma2(const ::Susander2 &p1,const ::Squash1 &p2,::GardenPeas3 *&p3)");
#endif
  
  ernie.seed("Satsuma2Call");
  ernie.mode(0);
  GetSusander2(p1);
  ernie.mode(0);
  GetSquash1(p2);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Satsuma2Return");
  try {
    ReturnValue = tobj->Satsuma2(p1,p2,p3);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Satsuma2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckYellowPepper2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckGardenPeas3(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsBasil2(void)
{
  
  ::BambooShoot3* ReturnValue;
  
  ::Peppercorn p1;
  ::Oregano3 p2;
  
  ::Fig p3;
  ::Dates3 p4;
  
  ::Clementine2 p5;
  CORBA::Boolean p6;
  ::Carrots p7;
  ::Raspberry1 p8;
  ::Rasin2 p9;
  
  ::Clementine2 p10;
  ::Mangosteen3* p11;
  ::Pumpkin p12;
  CORBA::Float p13;
  ::Chestnut1 p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BambooShoot3* ::AllTypesTests::Basil2(::Peppercorn &p1,::Oregano3 &p2,::Fig &p3,::Dates3 &p4,const ::Clementine2 &p5,CORBA::Boolean p6,::Carrots p7,::Raspberry1 p8,::Rasin2 &p9,::Clementine2 &p10,::Mangosteen3 *&p11,::Pumpkin p12,CORBA::Float &p13,::Chestnut1 p14)");
#endif
  
  ernie.seed("Basil2Call");
  ernie.mode(1);
  GetPeppercorn(p1);
  ernie.mode(1);
  GetOregano3(p2);
  ernie.mode(1);
  GetFig(p3);
  ernie.mode(1);
  GetDates3(p4);
  ernie.mode(0);
  GetClementine2(p5);
  ernie.mode(0);
  GetBoolean(p6);
  ernie.mode(1);
  GetRaspberry1(p8);
  ernie.mode(1);
  GetRasin2(p9);
  ernie.mode(1);
  GetClementine2(p10);
  ernie.mode(1);
  GetPumpkin(p12);
  ernie.mode(1);
  GetFloat(p13);
  ernie.mode(1);
  GetChestnut1(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Basil2Return");
  try {
    ReturnValue = tobj->Basil2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Basil2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBambooShoot3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckPeppercorn(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckOregano3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckFig(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckDates3(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCarrots(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckRaspberry1(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckRasin2(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckClementine2(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckMangosteen3(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckPumpkin(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckFloat(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckChestnut1(p14))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsBlueberry2(void)
{
  
  ::Record2_slice * ReturnValue;
  ::Cauliflower2 p1;
  ::Asparagus p2;
  ::WaterMelon3 p3;
  ::WaterMelon3 p4;
  ::Dates3* p5;
  ::Lychees3* p6;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Record2_slice * ::AllTypesTests::Blueberry2(const ::Cauliflower2 &p1,::Asparagus &p2,::WaterMelon3 p3,::WaterMelon3 p4,::Dates3 *&p5,::Lychees3 *&p6)");
#endif
  
  ernie.seed("Blueberry2Call");
  ernie.mode(0);
  GetCauliflower2(p1);
  ernie.mode(1);
  GetAsparagus(p2);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Blueberry2Return");
  try {
    ReturnValue = tobj->Blueberry2(p1,p2,p3,p4,p5,p6);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Blueberry2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRecord2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckAsparagus(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckWaterMelon3(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckWaterMelon3(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckDates3(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckLychees3(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsChervil2(void)
{
  
  
  ::Blackberry2* ReturnValue;
  CORBA::Long p1;
  
  ::Susander2 p2;
  CORBA::TypeCode_ptr p3;
  
  ::Peppercorn p4;
  ::Cauliflower2 p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Blackberry2* ::AllTypesTests::Chervil2(CORBA::Long &p1,const ::Susander2 &p2,CORBA::TypeCode_ptr &p3,const ::Peppercorn &p4,const ::Cauliflower2 &p5)");
#endif
  
  ernie.seed("Chervil2Call");
  ernie.mode(0);
  GetSusander2(p2);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p3);
  ernie.mode(0);
  GetPeppercorn(p4);
  ernie.mode(0);
  GetCauliflower2(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Chervil2Return");
  try {
    ReturnValue = tobj->Chervil2(p1,p2,p3,p4,p5);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Chervil2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBlackberry2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckLong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCORBATypeCode_ptr(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsGreenPepper2(void)
{
  
  CORBA::Boolean ReturnValue;
  ::Lychees3 p1;
  CORBA::Float p2;
  ::BambooShoot3 p3;
  ::Record2 p4;
  CORBA::Char * p5;
  
  ::Tomato* p6;
  ::LoganBerry p7;
  CORBA::Long p8;
  ::Corn1 p9;
  CORBA::Float p10;
  CORBA::Long p11;
  CORBA::Float p12;
  ::GardenPeas3 p13;
  ::BambooShoot3* p14;
  CORBA::ULong p15;
  ::Squash1 p16;
  ::BrazilNut3* p17;
  ::Cauliflower2* p18;
  ::GardenPeas3 p19;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Boolean ::AllTypesTests::GreenPepper2(::Lychees3 &p1,CORBA::Float &p2,const ::BambooShoot3 &p3,::Record2 p4,const CORBA::Char * p5,::Tomato *&p6,::LoganBerry &p7,CORBA::Long &p8,const ::Corn1 p9,CORBA::Float &p10,CORBA::Long &p11,CORBA::Float &p12,const ::GardenPeas3 &p13,::BambooShoot3 *&p14,CORBA::ULong &p15,const ::Squash1 &p16,::BrazilNut3 *&p17,::Cauliflower2 *&p18,const ::GardenPeas3 &p19)");
#endif
  
  ernie.seed("GreenPepper2Call");
  ernie.mode(1);
  GetLychees3(p1);
  ernie.mode(0);
  GetBambooShoot3(p3);
  ernie.mode(0);
  p5 = GetString();
  ernie.mode(1);
  GetLoganBerry(p7);
  ernie.mode(1);
  GetLong(p8);
  ernie.mode(0);
  GetCorn1(p9);
  ernie.mode(0);
  GetGardenPeas3(p13);
  ernie.mode(1);
  GetULong(p15);
  ernie.mode(0);
  GetSquash1(p16);
  ernie.mode(0);
  GetGardenPeas3(p19);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GreenPepper2Return");
  try {
    ReturnValue = tobj->GreenPepper2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18,p19);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::GreenPepper2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBoolean(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckLychees3(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckFloat(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckRecord2(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckTomato(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckLoganBerry(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckLong(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckFloat(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckLong(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckFloat(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckBambooShoot3(*p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckULong(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p17");
  if (!CheckBrazilNut3(*p17))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p18");
  if (!CheckCauliflower2(*p18))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsScallion2(void)
{
  
  ::Mangosteen3* ReturnValue;
  ::Squash1 p1;
  ::Chestnut1 p2;
  ::Raspberry1 p3;
  ::Cauliflower2* p4;
  ::Oregano3* p5;
  ::Dates3 p6;
  CORBA::Boolean p7;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mangosteen3* ::AllTypesTests::Scallion2(::Squash1 &p1,::Chestnut1 p2,::Raspberry1 p3,::Cauliflower2 *&p4,::Oregano3 *&p5,const ::Dates3 &p6,CORBA::Boolean p7)");
#endif
  
  ernie.seed("Scallion2Call");
  ernie.mode(1);
  GetSquash1(p1);
  ernie.mode(1);
  GetRaspberry1(p3);
  ernie.mode(0);
  GetDates3(p6);
  ernie.mode(0);
  GetBoolean(p7);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Scallion2Return");
  try {
    ReturnValue = tobj->Scallion2(p1,p2,p3,p4,p5,p6,p7);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Scallion2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMangosteen3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckSquash1(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckChestnut1(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckRaspberry1(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCauliflower2(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckOregano3(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsPricklyPear2(void)
{
  
  CORBA::Char ReturnValue;
  ::Asparagus p1;
  
  ::Fig p2;
  
  ::Fig p3;
  ::Beetroot2 p4;
  ::GoldenDelicous3 p5;
  ::GaliaMelon3* p6;
  ::Squash2 p7;
  ::Beetroot2 p8;
  
  ::Clementine2 p9;
  
  ::Tomato p10;
  ::GoldenDelicous3 p11;
  ::BeechNut2* p12;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Char ::AllTypesTests::PricklyPear2(::Asparagus &p1,const ::Fig &p2,::Fig &p3,const ::Beetroot2 &p4,::GoldenDelicous3 &p5,::GaliaMelon3 *&p6,::Squash2 &p7,::Beetroot2 &p8,::Clementine2 &p9,::Tomato &p10,const ::GoldenDelicous3 &p11,::BeechNut2 *&p12)");
#endif
  
  ernie.seed("PricklyPear2Call");
  ernie.mode(1);
  GetAsparagus(p1);
  ernie.mode(0);
  GetFig(p2);
  ernie.mode(1);
  GetFig(p3);
  ernie.mode(0);
  GetBeetroot2(p4);
  ernie.mode(1);
  GetGoldenDelicous3(p5);
  ernie.mode(1);
  GetSquash2(p7);
  ernie.mode(1);
  GetBeetroot2(p8);
  ernie.mode(1);
  GetClementine2(p9);
  ernie.mode(1);
  GetTomato(p10);
  ernie.mode(0);
  GetGoldenDelicous3(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PricklyPear2Return");
  try {
    ReturnValue = tobj->PricklyPear2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::PricklyPear2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckChar(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckAsparagus(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckFig(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckGoldenDelicous3(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckGaliaMelon3(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckSquash2(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckBeetroot2(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckClementine2(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckTomato(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p12");
  if (!CheckBeechNut2(*p12))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsChestnut2(void)
{
  
  ::Currant2* ReturnValue;
  ::Carrots p1;
  ::Grapefruit2 p2;
  CORBA::Boolean p3;
  
  ::Fig p4;
  ::Fennell3* p5;
  ::Cauliflower2 p6;
  CORBA::Any p7;
  ::GaliaMelon3 p8;
  
  ::Turnip3* p9;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Currant2* ::AllTypesTests::Chestnut2(::Carrots p1,const ::Grapefruit2 &p2,CORBA::Boolean p3,const ::Fig &p4,::Fennell3 *&p5,const ::Cauliflower2 &p6,CORBA::Any &p7,::GaliaMelon3 &p8,::Turnip3 *&p9)");
#endif
  
  ernie.seed("Chestnut2Call");
  ernie.mode(1);
  GetCarrots(p1);
  ernie.mode(0);
  GetGrapefruit2(p2);
  ernie.mode(0);
  GetBoolean(p3);
  ernie.mode(0);
  GetFig(p4);
  ernie.mode(0);
  GetCauliflower2(p6);
  ernie.mode(1);
  GetAny(p7);
  ernie.mode(1);
  GetGaliaMelon3(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Chestnut2Return");
  try {
    ReturnValue = tobj->Chestnut2(p1,p2,p3,p4,p5,p6,p7,p8,p9);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Chestnut2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCurrant2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckCarrots(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckFennell3(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckAny(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckGaliaMelon3(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckTurnip3(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsKiwanoMelon2(void)
{
  
  
  ::IcebergLettuce2* ReturnValue;
  ::Asparagus* p1;
  ::Corn1 p2;
  
  ::Peppercorn p3;
  
  ::Peppercorn p4;
  ::Fennell3 p5;
  ::Peanuts1 p6;
  ::Squash1 p7;
  
  ::Turnip3 p8;
  CORBA::Any p9;
  ::BambooShoot3 p10;
  ::Lychees3* p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::IcebergLettuce2* ::AllTypesTests::KiwanoMelon2(::Asparagus *&p1,const ::Corn1 p2,::Peppercorn &p3,::Peppercorn &p4,::Fennell3 &p5,::Peanuts1 &p6,::Squash1 &p7,const ::Turnip3 &p8,CORBA::Any &p9,const ::BambooShoot3 &p10,::Lychees3 *&p11)");
#endif
  
  ernie.seed("KiwanoMelon2Call");
  ernie.mode(0);
  GetCorn1(p2);
  ernie.mode(1);
  GetFennell3(p5);
  ernie.mode(1);
  GetSquash1(p7);
  ernie.mode(0);
  GetTurnip3(p8);
  ernie.mode(1);
  GetAny(p9);
  ernie.mode(0);
  GetBambooShoot3(p10);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KiwanoMelon2Return");
  try {
    ReturnValue = tobj->KiwanoMelon2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::KiwanoMelon2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckIcebergLettuce2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckAsparagus(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckPeppercorn(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckPeppercorn(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckFennell3(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckPeanuts1(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckSquash1(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckAny(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckLychees3(*p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsChayato2(void)
{
  
  ::Tomato2* ReturnValue;
  ::Beetroot2 p1;
  ::BambooShoot3* p2;
  ::BlackEyedBeans* p3;
  ::Squash2 p4;
  
  ::Turnip3 p5;
  CORBA::TypeCode_ptr p6;
  ::Chestnut1 p7;
  CORBA::Char * p8;
  ::Record2 p9;
  
  ::Tomato* p10;
  
  ::IcebergLettuce2 p11;
  ::Squash1 p12;
  ::Fennell3 p13;
  ::Pumpkin p14;
  CORBA::Char p15;
  CORBA::TypeCode_ptr p16;
  ::BritishQueens1 p17;
  
  ::Turnip3* p18;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Tomato2* ::AllTypesTests::Chayato2(::Beetroot2 &p1,::BambooShoot3 *&p2,::BlackEyedBeans *&p3,const ::Squash2 &p4,::Turnip3 &p5,CORBA::TypeCode_ptr &p6,::Chestnut1 p7,CORBA::Char * &p8,::Record2 p9,::Tomato *&p10,const ::IcebergLettuce2 &p11,::Squash1 &p12,const ::Fennell3 &p13,const ::Pumpkin p14,CORBA::Char &p15,CORBA::TypeCode_ptr &p16,::BritishQueens1 &p17,::Turnip3 *&p18)");
#endif
  
  ernie.seed("Chayato2Call");
  ernie.mode(1);
  GetBeetroot2(p1);
  ernie.mode(0);
  GetSquash2(p4);
  ernie.mode(1);
  GetTurnip3(p5);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p6);
  ernie.mode(1);
  GetChestnut1(p7);
  ernie.mode(1);
  p8 = GetString();
  ernie.mode(0);
  GetIcebergLettuce2(p11);
  ernie.mode(1);
  GetSquash1(p12);
  ernie.mode(0);
  GetFennell3(p13);
  ernie.mode(0);
  GetPumpkin(p14);
  ernie.mode(1);
  GetCORBATypeCode_ptr(p16);
  ernie.mode(1);
  GetBritishQueens1(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Chayato2Return");
  try {
    ReturnValue = tobj->Chayato2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Chayato2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTomato2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckBeetroot2(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckBambooShoot3(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckBlackEyedBeans(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckTurnip3(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckCORBATypeCode_ptr(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckChestnut1(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckString(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckRecord2(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckTomato(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckSquash1(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckChar(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p16");
  if (!CheckCORBATypeCode_ptr(p16))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p17");
  if (!CheckBritishQueens1(p17))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p18");
  if (!CheckTurnip3(*p18))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsRedChilly2(void)
{
  
  ::GaliaMelon3* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GaliaMelon3* ::AllTypesTests::RedChilly2()");
#endif
  
  ernie.seed("RedChilly2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedChilly2Return");
  try {
    ReturnValue = tobj->RedChilly2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::RedChilly2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGaliaMelon3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsGalaApple2(void)
{
  
  ::Celeriac2* ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Celeriac2* ::AllTypesTests::GalaApple2()");
#endif
  
  ernie.seed("GalaApple2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GalaApple2Return");
  try {
    ReturnValue = tobj->GalaApple2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::GalaApple2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCeleriac2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsKiwi2(void)
{
  
  ::Cauliflower2* ReturnValue;
  ::Record2 p1;
  CORBA::Short p2;
  ::Mullberry1 p3;
  
  ::Susander2 p4;
  
  ::Tomato p5;
  ::Squash2* p6;
  ::Cauliflower2 p7;
  ::Carrots p8;
  ::Record2 p9;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Cauliflower2* ::AllTypesTests::Kiwi2(const ::Record2 p1,CORBA::Short p2,const ::Mullberry1 p3,::Susander2 &p4,const ::Tomato &p5,::Squash2 *&p6,::Cauliflower2 &p7,::Carrots p8,::Record2 p9)");
#endif
  
  ernie.seed("Kiwi2Call");
  ernie.mode(0);
  GetRecord2(p1);
  ernie.mode(0);
  GetShort(p2);
  ernie.mode(0);
  GetMullberry1(p3);
  ernie.mode(1);
  GetSusander2(p4);
  ernie.mode(0);
  GetTomato(p5);
  ernie.mode(1);
  GetCauliflower2(p7);
  ernie.mode(1);
  GetRecord2(p9);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Kiwi2Return");
  try {
    ReturnValue = tobj->Kiwi2(p1,p2,p3,p4,p5,p6,p7,p8,p9);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Kiwi2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCauliflower2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckSusander2(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckSquash2(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckCauliflower2(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckCarrots(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckRecord2(p9))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsCorriander2(void)
{
  
  CORBA::ULong ReturnValue;
  
  ::Peppercorn p1;
  
  ::Susander2 p2;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::ULong ::AllTypesTests::Corriander2(::Peppercorn &p1,const ::Susander2 &p2)");
#endif
  
  ernie.seed("Corriander2Call");
  ernie.mode(0);
  GetSusander2(p2);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Corriander2Return");
  try {
    ReturnValue = tobj->Corriander2(p1,p2);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Corriander2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckULong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckPeppercorn(p1))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsRedCabbage2(void)
{
  
  
  ::Clementine2* ReturnValue;
  ::Peanuts1 p1;
  ::Tomato2* p2;
  CORBA::Object_ptr p3;
  ::Cauliflower2 p4;
  ::Asparagus* p5;
  ::Raspberry1 p6;
  ::Record2 p7;
  
  ::Susander2 p8;
  ::Asparagus p9;
  ::Rasin2* p10;
  ::Mangosteen3 p11;
  ::Chives2 p12;
  ::GaliaMelon3* p13;
  ::Grapefruit2 p14;
  ::Dates3 p15;
  ::BlackEyedBeans* p16;
  ::WaterMelon3 p17;
  CORBA::Object_ptr p18;
  ::WaterMelon3 p19;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Clementine2* ::AllTypesTests::RedCabbage2(::Peanuts1 &p1,::Tomato2 *&p2,CORBA::Object_ptr &p3,const ::Cauliflower2 &p4,::Asparagus *&p5,::Raspberry1 p6,::Record2 p7,::Susander2 &p8,const ::Asparagus &p9,::Rasin2 *&p10,const ::Mangosteen3 &p11,const ::Chives2 &p12,::GaliaMelon3 *&p13,const ::Grapefruit2 &p14,const ::Dates3 &p15,::BlackEyedBeans *&p16,::WaterMelon3 p17,CORBA::Object_ptr &p18,const ::WaterMelon3 p19)");
#endif
  
  ernie.seed("RedCabbage2Call");
  ernie.mode(1);
  p3 = Getobject();
  ernie.mode(0);
  GetCauliflower2(p4);
  ernie.mode(1);
  GetRaspberry1(p6);
  ernie.mode(1);
  GetSusander2(p8);
  ernie.mode(0);
  GetAsparagus(p9);
  ernie.mode(0);
  GetMangosteen3(p11);
  ernie.mode(0);
  GetChives2(p12);
  ernie.mode(0);
  GetGrapefruit2(p14);
  ernie.mode(0);
  GetDates3(p15);
  ernie.mode(1);
  GetWaterMelon3(p17);
  ernie.mode(1);
  p18 = Getobject();
  ernie.mode(0);
  GetWaterMelon3(p19);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("RedCabbage2Return");
  try {
    ReturnValue = tobj->RedCabbage2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18,p19);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::RedCabbage2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckClementine2(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckPeanuts1(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckTomato2(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!Checkobject(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckAsparagus(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckRaspberry1(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckRecord2(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckSusander2(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckRasin2(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckGaliaMelon3(*p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckBlackEyedBeans(*p16))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p17");
  if (!CheckWaterMelon3(p17))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p18");
  if (!Checkobject(p18))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsBroadbeans2(void)
{
  
  ::Dates3* ReturnValue;
  ::WaterMelon* p1;
  ::Fennell3 p2;
  ::Oregano3 p3;
  ::Squash2 p4;
  
  ::Fig* p5;
  ::Grapefruit2* p6;
  ::WaterMelon p7;
  
  ::Fig p8;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Dates3* ::AllTypesTests::Broadbeans2(::WaterMelon *&p1,::Fennell3 &p2,::Oregano3 &p3,const ::Squash2 &p4,::Fig *&p5,::Grapefruit2 *&p6,const ::WaterMelon &p7,::Fig &p8)");
#endif
  
  ernie.seed("Broadbeans2Call");
  ernie.mode(1);
  GetFennell3(p2);
  ernie.mode(1);
  GetOregano3(p3);
  ernie.mode(0);
  GetSquash2(p4);
  ernie.mode(0);
  GetWaterMelon(p7);
  ernie.mode(1);
  GetFig(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Broadbeans2Return");
  try {
    ReturnValue = tobj->Broadbeans2(p1,p2,p3,p4,p5,p6,p7,p8);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Broadbeans2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDates3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckWaterMelon(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckFennell3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckOregano3(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckFig(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckGrapefruit2(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckFig(p8))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsPotato2(void)
{
  
  CORBA::ULong ReturnValue;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::ULong ::AllTypesTests::Potato2()");
#endif
  
  ernie.seed("Potato2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Potato2Return");
  try {
    ReturnValue = tobj->Potato2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Potato2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckULong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsHomeguard2(void)
{
  
  ::Record2_slice * ReturnValue;
  CORBA::Boolean p1;
  ::Lychees3 p2;
  
  ::Susander2 p3;
  ::Grapefruit2* p4;
  CORBA::Char p5;
  ::Dates3 p6;
  CORBA::Double p7;
  ::GaliaMelon3 p8;
  ::Grapefruit2 p9;
  ::GardenPeas1 p10;
  ::Mullberry1 p11;
  CORBA::Long p12;
  CORBA::Any p13;
  CORBA::ULong p14;
  ::Peanuts1 p15;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Record2_slice * ::AllTypesTests::Homeguard2(CORBA::Boolean &p1,::Lychees3 &p2,::Susander2 &p3,::Grapefruit2 *&p4,CORBA::Char &p5,const ::Dates3 &p6,CORBA::Double p7,::GaliaMelon3 &p8,::Grapefruit2 &p9,const ::GardenPeas1 &p10,::Mullberry1 p11,CORBA::Long &p12,const CORBA::Any &p13,CORBA::ULong &p14,::Peanuts1 p15)");
#endif
  
  ernie.seed("Homeguard2Call");
  ernie.mode(1);
  GetBoolean(p1);
  ernie.mode(1);
  GetLychees3(p2);
  ernie.mode(1);
  GetSusander2(p3);
  ernie.mode(1);
  GetChar(p5);
  ernie.mode(0);
  GetDates3(p6);
  ernie.mode(0);
  GetDouble(p7);
  ernie.mode(1);
  GetGaliaMelon3(p8);
  ernie.mode(1);
  GetGrapefruit2(p9);
  ernie.mode(0);
  GetGardenPeas1(p10);
  ernie.mode(1);
  GetMullberry1(p11);
  ernie.mode(1);
  GetLong(p12);
  ernie.mode(0);
  GetAny(p13);
  ernie.mode(0);
  GetPeanuts1(p15);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Homeguard2Return");
  try {
    ReturnValue = tobj->Homeguard2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Homeguard2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckRecord2(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckBoolean(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckLychees3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckSusander2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckGrapefruit2(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckChar(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckGaliaMelon3(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckGrapefruit2(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckMullberry1(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckLong(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckULong(p14))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testAllTypesTestsWilja2(void)
{
  
  ::WaterMelon3_slice * ReturnValue;
  
  ::Fig p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::WaterMelon3_slice * ::AllTypesTests::Wilja2(::Fig &p1)");
#endif
  
  ernie.seed("Wilja2Call");
  ernie.mode(1);
  GetFig(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Wilja2Return");
  try {
    ReturnValue = tobj->Wilja2(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::AllTypesTests::Wilja2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckWaterMelon3(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckFig(p1))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsBalmoral2(void)
{
  
  ::Currant2 p1;
  
  ::Turnip3 p2;
  ::BlackEyedBeans p3;
  CORBA::Double p4;
  ::Dates3 p5;
  ::Tomato2 p6;
  ::Raspberry1 p7;
  ::Peanuts1 p8;
  CORBA::TypeCode_ptr p9;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Balmoral2(const ::Currant2 &p1,const ::Turnip3 &p2,const ::BlackEyedBeans &p3,CORBA::Double p4,const ::Dates3 &p5,const ::Tomato2 &p6,const ::Raspberry1 p7,::Peanuts1 p8,CORBA::TypeCode_ptr p9)");
#endif
  
  ernie.seed("Balmoral2Call");
  ernie.mode(0);
  GetCurrant2(p1);
  ernie.mode(0);
  GetTurnip3(p2);
  ernie.mode(0);
  GetBlackEyedBeans(p3);
  ernie.mode(0);
  GetDouble(p4);
  ernie.mode(0);
  GetDates3(p5);
  ernie.mode(0);
  GetTomato2(p6);
  ernie.mode(0);
  GetRaspberry1(p7);
  ernie.mode(0);
  GetPeanuts1(p8);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p9);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Balmoral2Return");
  try {
     tobj->Balmoral2(p1,p2,p3,p4,p5,p6,p7,p8,p9);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Balmoral2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsKerrsPinks2(void)
{
  
  ::WaterMelon3 p1;
  ::GaliaMelon3 p2;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::KerrsPinks2(const ::WaterMelon3 p1,const ::GaliaMelon3 &p2)");
#endif
  
  ernie.seed("KerrsPinks2Call");
  ernie.mode(0);
  GetWaterMelon3(p1);
  ernie.mode(0);
  GetGaliaMelon3(p2);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KerrsPinks2Return");
  try {
     tobj->KerrsPinks2(p1,p2);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::KerrsPinks2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsGoldenWonder2(void)
{
  
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::GoldenWonder2()");
#endif
  
  ernie.seed("GoldenWonder2Call");

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("GoldenWonder2Return");
  try {
     tobj->GoldenWonder2();
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::GoldenWonder2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsPimpernel2(void)
{
  
  
  ::Tomato p1;
  
  ::Clementine2 p2;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Pimpernel2(const ::Tomato &p1,const ::Clementine2 &p2)");
#endif
  
  ernie.seed("Pimpernel2Call");
  ernie.mode(0);
  GetTomato(p1);
  ernie.mode(0);
  GetClementine2(p2);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pimpernel2Return");
  try {
     tobj->Pimpernel2(p1,p2);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Pimpernel2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsCultra2(void)
{
  
  ::Chestnut1 p1;
  CORBA::Char p2;
  CORBA::ULong p3;
  
  ::IcebergLettuce2 p4;
  CORBA::Short p5;
  ::WaterMelon p6;
  ::Cauliflower2 p7;
  ::GardenPeas1 p8;
  ::Cauliflower2 p9;
  ::Corn1 p10;
  ::Corn1 p11;
  CORBA::TypeCode_ptr p12;
  ::WaterMelon3 p13;
  ::Lychees3 p14;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Cultra2(const ::Chestnut1 p1,CORBA::Char p2,CORBA::ULong p3,const ::IcebergLettuce2 &p4,CORBA::Short p5,const ::WaterMelon &p6,const ::Cauliflower2 &p7,const ::GardenPeas1 &p8,const ::Cauliflower2 &p9,const ::Corn1 p10,const ::Corn1 p11,CORBA::TypeCode_ptr p12,const ::WaterMelon3 p13,const ::Lychees3 &p14)");
#endif
  
  ernie.seed("Cultra2Call");
  ernie.mode(0);
  GetChestnut1(p1);
  ernie.mode(0);
  GetChar(p2);
  ernie.mode(0);
  GetULong(p3);
  ernie.mode(0);
  GetIcebergLettuce2(p4);
  ernie.mode(0);
  GetShort(p5);
  ernie.mode(0);
  GetWaterMelon(p6);
  ernie.mode(0);
  GetCauliflower2(p7);
  ernie.mode(0);
  GetGardenPeas1(p8);
  ernie.mode(0);
  GetCauliflower2(p9);
  ernie.mode(0);
  GetCorn1(p10);
  ernie.mode(0);
  GetCorn1(p11);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p12);
  ernie.mode(0);
  GetWaterMelon3(p13);
  ernie.mode(0);
  GetLychees3(p14);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cultra2Return");
  try {
     tobj->Cultra2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Cultra2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsRooster2(void)
{
  
  ::GaliaMelon3 p1;
  ::Celeriac2 p2;
  CORBA::Any p3;
  ::BeechNut2 p4;
  ::BeechNut2 p5;
  ::Fennell3 p6;
  ::Dates3 p7;
  
  ::Blackberry2 p8;
  ::Fennell3 p9;
  CORBA::UShort p10;
  ::Cauliflower2 p11;
  ::WaterMelon3 p12;
  ::Rasin2 p13;
  ::Dates3 p14;
  
  ::Fig p15;
  ::GaliaMelon3 p16;
  ::BritishQueens1 p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Rooster2(const ::GaliaMelon3 &p1,const ::Celeriac2 &p2,const CORBA::Any &p3,const ::BeechNut2 &p4,const ::BeechNut2 &p5,const ::Fennell3 &p6,const ::Dates3 &p7,const ::Blackberry2 &p8,const ::Fennell3 &p9,CORBA::UShort p10,const ::Cauliflower2 &p11,const ::WaterMelon3 p12,const ::Rasin2 &p13,const ::Dates3 &p14,const ::Fig &p15,const ::GaliaMelon3 &p16,const ::BritishQueens1 &p17)");
#endif
  
  ernie.seed("Rooster2Call");
  ernie.mode(0);
  GetGaliaMelon3(p1);
  ernie.mode(0);
  GetCeleriac2(p2);
  ernie.mode(0);
  GetAny(p3);
  ernie.mode(0);
  GetBeechNut2(p4);
  ernie.mode(0);
  GetBeechNut2(p5);
  ernie.mode(0);
  GetFennell3(p6);
  ernie.mode(0);
  GetDates3(p7);
  ernie.mode(0);
  GetBlackberry2(p8);
  ernie.mode(0);
  GetFennell3(p9);
  ernie.mode(0);
  GetUShort(p10);
  ernie.mode(0);
  GetCauliflower2(p11);
  ernie.mode(0);
  GetWaterMelon3(p12);
  ernie.mode(0);
  GetRasin2(p13);
  ernie.mode(0);
  GetDates3(p14);
  ernie.mode(0);
  GetFig(p15);
  ernie.mode(0);
  GetGaliaMelon3(p16);
  ernie.mode(0);
  GetBritishQueens1(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Rooster2Return");
  try {
     tobj->Rooster2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Rooster2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsMarisPiper2(void)
{
  
  ::BeechNut2 p1;
  ::Pumpkin p2;
  CORBA::Double p3;
  ::Tomato2 p4;
  CORBA::Octet p5;
  ::Record2 p6;
  CORBA::Double p7;
  ::WaterMelon3 p8;
  ::Squash2 p9;
  ::Record2 p10;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::MarisPiper2(const ::BeechNut2 &p1,const ::Pumpkin p2,CORBA::Double p3,const ::Tomato2 &p4,CORBA::Octet p5,const ::Record2 p6,CORBA::Double p7,const ::WaterMelon3 p8,const ::Squash2 &p9,const ::Record2 p10)");
#endif
  
  ernie.seed("MarisPiper2Call");
  ernie.mode(0);
  GetBeechNut2(p1);
  ernie.mode(0);
  GetPumpkin(p2);
  ernie.mode(0);
  GetDouble(p3);
  ernie.mode(0);
  GetTomato2(p4);
  ernie.mode(0);
  GetOctet(p5);
  ernie.mode(0);
  GetRecord2(p6);
  ernie.mode(0);
  GetDouble(p7);
  ernie.mode(0);
  GetWaterMelon3(p8);
  ernie.mode(0);
  GetSquash2(p9);
  ernie.mode(0);
  GetRecord2(p10);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("MarisPiper2Return");
  try {
     tobj->MarisPiper2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::MarisPiper2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsCara2(void)
{
  
  CORBA::Short p1;
  ::BrazilNut3 p2;
  CORBA::Char p3;
  ::GardenPeas3 p4;
  ::Fennell3 p5;
  CORBA::Boolean p6;
  ::Mullberry1 p7;
  
  ::Susander2 p8;
  CORBA::UShort p9;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Cara2(CORBA::Short p1,const ::BrazilNut3 &p2,CORBA::Char p3,const ::GardenPeas3 &p4,const ::Fennell3 &p5,CORBA::Boolean p6,const ::Mullberry1 p7,const ::Susander2 &p8,CORBA::UShort p9)");
#endif
  
  ernie.seed("Cara2Call");
  ernie.mode(0);
  GetShort(p1);
  ernie.mode(0);
  GetBrazilNut3(p2);
  ernie.mode(0);
  GetChar(p3);
  ernie.mode(0);
  GetGardenPeas3(p4);
  ernie.mode(0);
  GetFennell3(p5);
  ernie.mode(0);
  GetBoolean(p6);
  ernie.mode(0);
  GetMullberry1(p7);
  ernie.mode(0);
  GetSusander2(p8);
  ernie.mode(0);
  GetUShort(p9);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cara2Return");
  try {
     tobj->Cara2(p1,p2,p3,p4,p5,p6,p7,p8,p9);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Cara2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsKingEdward2(void)
{
  
  ::Squash2 p1;
  CORBA::Float p2;
  ::Cauliflower2 p3;
  ::Chives2 p4;
  
  ::Turnip3 p5;
  ::Squash2 p6;
  
  ::Tomato p7;
  
  ::Fig p8;
  CORBA::Char p9;
  ::GaliaMelon3 p10;
  ::Carrots p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::KingEdward2(const ::Squash2 &p1,CORBA::Float p2,const ::Cauliflower2 &p3,const ::Chives2 &p4,const ::Turnip3 &p5,const ::Squash2 &p6,const ::Tomato &p7,const ::Fig &p8,CORBA::Char p9,const ::GaliaMelon3 &p10,const ::Carrots p11)");
#endif
  
  ernie.seed("KingEdward2Call");
  ernie.mode(0);
  GetSquash2(p1);
  ernie.mode(0);
  GetFloat(p2);
  ernie.mode(0);
  GetCauliflower2(p3);
  ernie.mode(0);
  GetChives2(p4);
  ernie.mode(0);
  GetTurnip3(p5);
  ernie.mode(0);
  GetSquash2(p6);
  ernie.mode(0);
  GetTomato(p7);
  ernie.mode(0);
  GetFig(p8);
  ernie.mode(0);
  GetChar(p9);
  ernie.mode(0);
  GetGaliaMelon3(p10);
  ernie.mode(0);
  GetCarrots(p11);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KingEdward2Return");
  try {
     tobj->KingEdward2(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::KingEdward2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsPentlandDell2(void)
{
  
  CORBA::Char * p1;
  ::Tomato2 p2;
  ::Asparagus p3;
  ::Dates3 p4;
  ::Tomato2 p5;
  ::Tomato2 p6;
  CORBA::Octet p7;
  
  ::Peppercorn p8;
  
  ::IcebergLettuce2 p9;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::PentlandDell2(const CORBA::Char * p1,const ::Tomato2 &p2,const ::Asparagus &p3,const ::Dates3 &p4,const ::Tomato2 &p5,const ::Tomato2 &p6,CORBA::Octet p7,const ::Peppercorn &p8,const ::IcebergLettuce2 &p9)");
#endif
  
  ernie.seed("PentlandDell2Call");
  ernie.mode(0);
  p1 = GetString();
  ernie.mode(0);
  GetTomato2(p2);
  ernie.mode(0);
  GetAsparagus(p3);
  ernie.mode(0);
  GetDates3(p4);
  ernie.mode(0);
  GetTomato2(p5);
  ernie.mode(0);
  GetTomato2(p6);
  ernie.mode(0);
  GetOctet(p7);
  ernie.mode(0);
  GetPeppercorn(p8);
  ernie.mode(0);
  GetIcebergLettuce2(p9);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PentlandDell2Return");
  try {
     tobj->PentlandDell2(p1,p2,p3,p4,p5,p6,p7,p8,p9);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::PentlandDell2");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsApple3(void)
{
  
  ::Oregano3 p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Apple3(const ::Oregano3 &p1)");
#endif
  
  ernie.seed("Apple3Call");
  ernie.mode(0);
  GetOregano3(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Apple3Return");
  try {
     tobj->Apple3(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Apple3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsBanana3(void)
{
  
  ::Rasin2 p1;
  CORBA::Double p2;
  ::Fennell3 p3;
  
  ::IcebergLettuce2 p4;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Banana3(const ::Rasin2 &p1,CORBA::Double p2,const ::Fennell3 &p3,const ::IcebergLettuce2 &p4)");
#endif
  
  ernie.seed("Banana3Call");
  ernie.mode(0);
  GetRasin2(p1);
  ernie.mode(0);
  GetDouble(p2);
  ernie.mode(0);
  GetFennell3(p3);
  ernie.mode(0);
  GetIcebergLettuce2(p4);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Banana3Return");
  try {
     tobj->Banana3(p1,p2,p3,p4);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Banana3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsCarrots3(void)
{
  
  CORBA::Boolean p1;
  ::Damsons2 p2;
  ::Corn1 p3;
  ::Peanuts1 p4;
  ::Rasin2 p5;
  ::Chestnut1 p6;
  ::WaterMelon3 p7;
  CORBA::Long p8;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Carrots3(CORBA::Boolean p1,const ::Damsons2 p2,const ::Corn1 p3,::Peanuts1 p4,const ::Rasin2 &p5,const ::Chestnut1 p6,const ::WaterMelon3 p7,CORBA::Long p8)");
#endif
  
  ernie.seed("Carrots3Call");
  ernie.mode(0);
  GetBoolean(p1);
  ernie.mode(0);
  GetDamsons2(p2);
  ernie.mode(0);
  GetCorn1(p3);
  ernie.mode(0);
  GetPeanuts1(p4);
  ernie.mode(0);
  GetRasin2(p5);
  ernie.mode(0);
  GetChestnut1(p6);
  ernie.mode(0);
  GetWaterMelon3(p7);
  ernie.mode(0);
  GetLong(p8);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Carrots3Return");
  try {
     tobj->Carrots3(p1,p2,p3,p4,p5,p6,p7,p8);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Carrots3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsLemon3(void)
{
  
  ::Fennell3 p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Lemon3(const ::Fennell3 &p1)");
#endif
  
  ernie.seed("Lemon3Call");
  ernie.mode(0);
  GetFennell3(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lemon3Return");
  try {
     tobj->Lemon3(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Lemon3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsBlackCherry3(void)
{
  
  
  ::IcebergLettuce2 p1;
  ::Squash2 p2;
  ::Raspberry1 p3;
  
  ::Susander2 p4;
  
  ::IcebergLettuce2 p5;
  ::Tomato2 p6;
  ::Mangosteen3 p7;
  ::BlackEyedBeans p8;
  ::Tomato2 p9;
  ::Record2 p10;
  CORBA::TypeCode_ptr p11;
  ::Tomato2 p12;
  ::Fennell3 p13;
  ::Tomato2 p14;
  ::Tomato2 p15;
  ::Tomato2 p16;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::BlackCherry3(const ::IcebergLettuce2 &p1,const ::Squash2 &p2,const ::Raspberry1 p3,const ::Susander2 &p4,const ::IcebergLettuce2 &p5,const ::Tomato2 &p6,const ::Mangosteen3 &p7,const ::BlackEyedBeans &p8,const ::Tomato2 &p9,const ::Record2 p10,CORBA::TypeCode_ptr p11,const ::Tomato2 &p12,const ::Fennell3 &p13,const ::Tomato2 &p14,const ::Tomato2 &p15,const ::Tomato2 &p16)");
#endif
  
  ernie.seed("BlackCherry3Call");
  ernie.mode(0);
  GetIcebergLettuce2(p1);
  ernie.mode(0);
  GetSquash2(p2);
  ernie.mode(0);
  GetRaspberry1(p3);
  ernie.mode(0);
  GetSusander2(p4);
  ernie.mode(0);
  GetIcebergLettuce2(p5);
  ernie.mode(0);
  GetTomato2(p6);
  ernie.mode(0);
  GetMangosteen3(p7);
  ernie.mode(0);
  GetBlackEyedBeans(p8);
  ernie.mode(0);
  GetTomato2(p9);
  ernie.mode(0);
  GetRecord2(p10);
  ernie.mode(0);
  GetCORBATypeCode_ptr(p11);
  ernie.mode(0);
  GetTomato2(p12);
  ernie.mode(0);
  GetFennell3(p13);
  ernie.mode(0);
  GetTomato2(p14);
  ernie.mode(0);
  GetTomato2(p15);
  ernie.mode(0);
  GetTomato2(p16);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackCherry3Return");
  try {
     tobj->BlackCherry3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::BlackCherry3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsStarfruit3(void)
{
  
  ::Asparagus p1;
  ::Squash2 p2;
  ::Tomato2 p3;
  
  ::Turnip3 p4;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Starfruit3(const ::Asparagus &p1,const ::Squash2 &p2,const ::Tomato2 &p3,const ::Turnip3 &p4)");
#endif
  
  ernie.seed("Starfruit3Call");
  ernie.mode(0);
  GetAsparagus(p1);
  ernie.mode(0);
  GetSquash2(p2);
  ernie.mode(0);
  GetTomato2(p3);
  ernie.mode(0);
  GetTurnip3(p4);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Starfruit3Return");
  try {
     tobj->Starfruit3(p1,p2,p3,p4);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Starfruit3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsRhubarb3(void)
{
  
  
  ::IcebergLettuce2 p1;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Rhubarb3(const ::IcebergLettuce2 &p1)");
#endif
  
  ernie.seed("Rhubarb3Call");
  ernie.mode(0);
  GetIcebergLettuce2(p1);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Rhubarb3Return");
  try {
     tobj->Rhubarb3(p1);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Rhubarb3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsPassionFruit3(void)
{
  
  ::Cauliflower2 p1;
  ::WaterMelon p2;
  ::Peanuts1 p3;
  ::GoldenDelicous3 p4;
  CORBA::Long p5;
  ::Oregano3 p6;
  ::Record2 p7;
  ::Squash1 p8;
  ::Asparagus p9;
  ::Chives2 p10;
  CORBA::Boolean p11;
  ::BrazilNut3 p12;
  
  ::Susander2 p13;
  CORBA::ULong p14;
  ::Chives2 p15;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::PassionFruit3(const ::Cauliflower2 &p1,const ::WaterMelon &p2,::Peanuts1 p3,const ::GoldenDelicous3 &p4,CORBA::Long p5,const ::Oregano3 &p6,const ::Record2 p7,const ::Squash1 &p8,const ::Asparagus &p9,const ::Chives2 &p10,CORBA::Boolean p11,const ::BrazilNut3 &p12,const ::Susander2 &p13,CORBA::ULong p14,const ::Chives2 &p15)");
#endif
  
  ernie.seed("PassionFruit3Call");
  ernie.mode(0);
  GetCauliflower2(p1);
  ernie.mode(0);
  GetWaterMelon(p2);
  ernie.mode(0);
  GetPeanuts1(p3);
  ernie.mode(0);
  GetGoldenDelicous3(p4);
  ernie.mode(0);
  GetLong(p5);
  ernie.mode(0);
  GetOregano3(p6);
  ernie.mode(0);
  GetRecord2(p7);
  ernie.mode(0);
  GetSquash1(p8);
  ernie.mode(0);
  GetAsparagus(p9);
  ernie.mode(0);
  GetChives2(p10);
  ernie.mode(0);
  GetBoolean(p11);
  ernie.mode(0);
  GetBrazilNut3(p12);
  ernie.mode(0);
  GetSusander2(p13);
  ernie.mode(0);
  GetULong(p14);
  ernie.mode(0);
  GetChives2(p15);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("PassionFruit3Return");
  try {
     tobj->PassionFruit3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::PassionFruit3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testOneWayTestsPumpkin3(void)
{
  
  
  ::Turnip3 p1;
  ::Tomato2 p2;
  ::Cauliflower2 p3;
  ::Chives2 p4;
  
  ::Susander2 p5;
  ::Pumpkin p6;
  
  ::Tomato p7;
  ::Mullberry1 p8;
  ::Tomato2 p9;
  ::LoganBerry p10;
  ::GaliaMelon3 p11;
  ::Grapefruit2 p12;
  ::Record2 p13;
  ::GaliaMelon3 p14;
  ::Cauliflower2 p15;
  ::Rasin2 p16;
  
  ::IcebergLettuce2 p17;
  ::Cauliflower2 p18;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function void ::OneWayTests::Pumpkin3(const ::Turnip3 &p1,const ::Tomato2 &p2,const ::Cauliflower2 &p3,const ::Chives2 &p4,const ::Susander2 &p5,const ::Pumpkin p6,const ::Tomato &p7,const ::Mullberry1 p8,const ::Tomato2 &p9,const ::LoganBerry &p10,const ::GaliaMelon3 &p11,const ::Grapefruit2 &p12,const ::Record2 p13,const ::GaliaMelon3 &p14,const ::Cauliflower2 &p15,const ::Rasin2 &p16,const ::IcebergLettuce2 &p17,const ::Cauliflower2 &p18)");
#endif
  
  ernie.seed("Pumpkin3Call");
  ernie.mode(0);
  GetTurnip3(p1);
  ernie.mode(0);
  GetTomato2(p2);
  ernie.mode(0);
  GetCauliflower2(p3);
  ernie.mode(0);
  GetChives2(p4);
  ernie.mode(0);
  GetSusander2(p5);
  ernie.mode(0);
  GetPumpkin(p6);
  ernie.mode(0);
  GetTomato(p7);
  ernie.mode(0);
  GetMullberry1(p8);
  ernie.mode(0);
  GetTomato2(p9);
  ernie.mode(0);
  GetLoganBerry(p10);
  ernie.mode(0);
  GetGaliaMelon3(p11);
  ernie.mode(0);
  GetGrapefruit2(p12);
  ernie.mode(0);
  GetRecord2(p13);
  ernie.mode(0);
  GetGaliaMelon3(p14);
  ernie.mode(0);
  GetCauliflower2(p15);
  ernie.mode(0);
  GetRasin2(p16);
  ernie.mode(0);
  GetIcebergLettuce2(p17);
  ernie.mode(0);
  GetCauliflower2(p18);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pumpkin3Return");
  try {
     tobj->Pumpkin3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::OneWayTests::Pumpkin3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif



  ::tet_result(TET_PASS);
  return;
}


 
void testContextTestsMango3(void)
{
  
  CORBA::Double ReturnValue;
  ::LoganBerry_var p1;
  CORBA::Context_var ctx;
  CORBA::Any ctxAny;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Double ::ContextTests::Mango3(::LoganBerry_out p1,CORBA::Context_ptr ctx)");
#endif
  
  ernie.seed("Mango3Call");
  orb->orb()->get_default_context(ctx);
  ctxAny <<= GetString();
  ctx->set_one_value("Rasin1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Eddo1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Leek1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Mango1",ctxAny);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mango3Return");
  try {
    ReturnValue = tobj->Mango3(p1,ctx);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ContextTests::Mango3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDouble(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckLoganBerry(p1))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testContextTestsLime3(void)
{
  
  ::WaterMelon3_slice * ReturnValue;
  CORBA::ULong p1;
  ::Grapefruit2 p2;
  ::GaliaMelon3 p3;
  CORBA::Any p4;
  CORBA::Any p5;
  ::LoganBerry* p6;
  ::Cauliflower2* p7;
  CORBA::Float p8;
  ::Currant2 p9;
  ::BritishQueens1* p10;
  CORBA::Char * p11;
  ::Corn1 p12;
  ::Mangosteen3 p13;
  
  ::Fig* p14;
  ::Pumpkin p15;
  ::Chives2 p16;
  ::Fennell3* p17;
  
  ::Fig p18;
  CORBA::Context_var ctx;
  CORBA::Any ctxAny;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::WaterMelon3_slice * ::ContextTests::Lime3(CORBA::ULong p1,::Grapefruit2 &p2,const ::GaliaMelon3 &p3,CORBA::Any &p4,const CORBA::Any &p5,::LoganBerry *&p6,::Cauliflower2 *&p7,CORBA::Float p8,const ::Currant2 &p9,::BritishQueens1 *&p10,const CORBA::Char * p11,::Corn1 p12,const ::Mangosteen3 &p13,::Fig *&p14,::Pumpkin p15,const ::Chives2 &p16,::Fennell3 *&p17,::Fig &p18,CORBA::Context_ptr ctx)");
#endif
  
  ernie.seed("Lime3Call");
  ernie.mode(0);
  GetULong(p1);
  ernie.mode(1);
  GetGrapefruit2(p2);
  ernie.mode(0);
  GetGaliaMelon3(p3);
  ernie.mode(1);
  GetAny(p4);
  ernie.mode(0);
  GetAny(p5);
  ernie.mode(0);
  GetFloat(p8);
  ernie.mode(0);
  GetCurrant2(p9);
  ernie.mode(0);
  p11 = GetString();
  ernie.mode(1);
  GetCorn1(p12);
  ernie.mode(0);
  GetMangosteen3(p13);
  ernie.mode(0);
  GetChives2(p16);
  ernie.mode(1);
  GetFig(p18);
  orb->orb()->get_default_context(ctx);
  ctxAny <<= GetString();
  ctx->set_one_value("Sultana1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Rasin1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Banana1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Dates1",ctxAny);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lime3Return");
  try {
    ReturnValue = tobj->Lime3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18,ctx);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ContextTests::Lime3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckWaterMelon3(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckGrapefruit2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAny(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckLoganBerry(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckCauliflower2(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckBritishQueens1(*p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckCorn1(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckFig(*p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckPumpkin(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p17");
  if (!CheckFennell3(*p17))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p18");
  if (!CheckFig(p18))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testContextTestsCoconut3(void)
{
  
  ::Dates3* ReturnValue;
  CORBA::Context_var ctx;
  CORBA::Any ctxAny;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Dates3* ::ContextTests::Coconut3(CORBA::Context_ptr ctx)");
#endif
  
  ernie.seed("Coconut3Call");
  orb->orb()->get_default_context(ctx);
  ctxAny <<= GetString();
  ctx->set_one_value("Chervil1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Coconut1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("GardenPeas1",ctxAny);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Coconut3Return");
  try {
    ReturnValue = tobj->Coconut3(ctx);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ContextTests::Coconut3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckDates3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testContextTestsCabbage3(void)
{
  
  ::Lychees3* ReturnValue;
  
  ::IcebergLettuce2 p1;
  ::Cauliflower2* p2;
  ::Rasin2 p3;
  
  ::Susander2 p4;
  ::Currant2 p5;
  ::Squash2 p6;
  ::Fennell3 p7;
  ::Record2 p8;
  CORBA::Context_var ctx;
  CORBA::Any ctxAny;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Lychees3* ::ContextTests::Cabbage3(const ::IcebergLettuce2 &p1,::Cauliflower2 *&p2,::Rasin2 &p3,::Susander2 &p4,::Currant2 &p5,::Squash2 &p6,const ::Fennell3 &p7,::Record2 p8,CORBA::Context_ptr ctx)");
#endif
  
  ernie.seed("Cabbage3Call");
  ernie.mode(0);
  GetIcebergLettuce2(p1);
  ernie.mode(1);
  GetRasin2(p3);
  ernie.mode(1);
  GetSusander2(p4);
  ernie.mode(1);
  GetCurrant2(p5);
  ernie.mode(1);
  GetSquash2(p6);
  ernie.mode(0);
  GetFennell3(p7);
  ernie.mode(1);
  GetRecord2(p8);
  orb->orb()->get_default_context(ctx);
  ctxAny <<= GetString();
  ctx->set_one_value("LoganBerry1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("PassionFruit1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Strawberry1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("KiwanoMelon1",ctxAny);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cabbage3Return");
  try {
    ReturnValue = tobj->Cabbage3(p1,p2,p3,p4,p5,p6,p7,p8,ctx);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ContextTests::Cabbage3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckLychees3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckCauliflower2(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckRasin2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckSusander2(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckCurrant2(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckSquash2(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckRecord2(p8))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testContextTestsGinger3(void)
{
  
  ::GaliaMelon3* ReturnValue;
  ::Squash2 p1;
  CORBA::Char * p2;
  ::BrazilNut3* p3;
  ::Cauliflower2* p4;
  ::BlackEyedBeans* p5;
  ::BambooShoot3* p6;
  CORBA::Context_var ctx;
  CORBA::Any ctxAny;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GaliaMelon3* ::ContextTests::Ginger3(const ::Squash2 &p1,const CORBA::Char * p2,::BrazilNut3 *&p3,::Cauliflower2 *&p4,::BlackEyedBeans *&p5,::BambooShoot3 *&p6,CORBA::Context_ptr ctx)");
#endif
  
  ernie.seed("Ginger3Call");
  ernie.mode(0);
  GetSquash2(p1);
  ernie.mode(0);
  p2 = GetString();
  orb->orb()->get_default_context(ctx);
  ctxAny <<= GetString();
  ctx->set_one_value("BlackCherry1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Scallion1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Carrots1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Orange1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Rasin1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("WaterMelon1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Eddo1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Parsnip1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Asparagus1",ctxAny);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Ginger3Return");
  try {
    ReturnValue = tobj->Ginger3(p1,p2,p3,p4,p5,p6,ctx);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ContextTests::Ginger3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGaliaMelon3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckBrazilNut3(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCauliflower2(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckBlackEyedBeans(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckBambooShoot3(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testContextTestsBlackEyedBeans3(void)
{
  
  ::Mangosteen3* ReturnValue;
  ::WaterMelon3 p1;
  ::Dates3 p2;
  ::Dates3 p3;
  ::Squash2 p4;
  ::Pumpkin p5;
  CORBA::Boolean p6;
  ::Beetroot2 p7;
  ::BritishQueens1 p8;
  
  ::Fig p9;
  CORBA::Long p10;
  CORBA::Context_var ctx;
  CORBA::Any ctxAny;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mangosteen3* ::ContextTests::BlackEyedBeans3(const ::WaterMelon3 p1,const ::Dates3 &p2,::Dates3 &p3,const ::Squash2 &p4,const ::Pumpkin p5,CORBA::Boolean &p6,::Beetroot2 &p7,::BritishQueens1 &p8,const ::Fig &p9,CORBA::Long p10,CORBA::Context_ptr ctx)");
#endif
  
  ernie.seed("BlackEyedBeans3Call");
  ernie.mode(0);
  GetWaterMelon3(p1);
  ernie.mode(0);
  GetDates3(p2);
  ernie.mode(1);
  GetDates3(p3);
  ernie.mode(0);
  GetSquash2(p4);
  ernie.mode(0);
  GetPumpkin(p5);
  ernie.mode(1);
  GetBoolean(p6);
  ernie.mode(1);
  GetBeetroot2(p7);
  ernie.mode(1);
  GetBritishQueens1(p8);
  ernie.mode(0);
  GetFig(p9);
  ernie.mode(0);
  GetLong(p10);
  orb->orb()->get_default_context(ctx);
  ctxAny <<= GetString();
  ctx->set_one_value("PassionFruit1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Eddo1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Rasin1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Parsley1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Lime1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("WaterMelon1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Banana1",ctxAny);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("BlackEyedBeans3Return");
  try {
    ReturnValue = tobj->BlackEyedBeans3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,ctx);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ContextTests::BlackEyedBeans3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMangosteen3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckDates3(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckBoolean(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckBeetroot2(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckBritishQueens1(p8))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testContextTestsLeek3(void)
{
  
  
  ::Fig* ReturnValue;
  ::Corn1 p1;
  ::Chives2 p2;
  ::Fennell3 p3;
  ::Cauliflower2* p4;
  CORBA::Long p5;
  ::Damsons2 p6;
  ::BritishQueens1* p7;
  ::Chestnut1 p8;
  ::GaliaMelon3 p9;
  ::WaterMelon3 p10;
  CORBA::Context_var ctx;
  CORBA::Any ctxAny;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Fig* ::ContextTests::Leek3(::Corn1 p1,::Chives2 &p2,::Fennell3 &p3,::Cauliflower2 *&p4,CORBA::Long &p5,const ::Damsons2 p6,::BritishQueens1 *&p7,const ::Chestnut1 p8,::GaliaMelon3 &p9,::WaterMelon3 p10,CORBA::Context_ptr ctx)");
#endif
  
  ernie.seed("Leek3Call");
  ernie.mode(1);
  GetChives2(p2);
  ernie.mode(1);
  GetFennell3(p3);
  ernie.mode(0);
  GetDamsons2(p6);
  ernie.mode(0);
  GetChestnut1(p8);
  ernie.mode(1);
  GetGaliaMelon3(p9);
  orb->orb()->get_default_context(ctx);
  ctxAny <<= GetString();
  ctx->set_one_value("Coconut1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Ginger1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Eddo1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("PassionFruit1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Turnip1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("LoganBerry1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("WaterMelon1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Lemon1",ctxAny);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Leek3Return");
  try {
    ReturnValue = tobj->Leek3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,ctx);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ContextTests::Leek3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFig(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckCorn1(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckChives2(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckFennell3(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCauliflower2(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckLong(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckBritishQueens1(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckGaliaMelon3(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p10");
  if (!CheckWaterMelon3(p10))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testContextTestsMint3(void)
{
  
  CORBA::TypeCode_ptr ReturnValue;
  ::Chestnut1 p1;
  ::Dates3 p2;
  ::Chives2 p3;
  ::GoldenDelicous3 p4;
  ::Oregano3* p5;
  ::Mullberry1 p6;
  CORBA::Context_var ctx;
  CORBA::Any ctxAny;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::TypeCode_ptr ::ContextTests::Mint3(::Chestnut1 p1,::Dates3 &p2,const ::Chives2 &p3,const ::GoldenDelicous3 &p4,::Oregano3 *&p5,const ::Mullberry1 p6,CORBA::Context_ptr ctx)");
#endif
  
  ernie.seed("Mint3Call");
  ernie.mode(1);
  GetDates3(p2);
  ernie.mode(0);
  GetChives2(p3);
  ernie.mode(0);
  GetGoldenDelicous3(p4);
  ernie.mode(0);
  GetMullberry1(p6);
  orb->orb()->get_default_context(ctx);
  ctxAny <<= GetString();
  ctx->set_one_value("Banana1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Lime1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Strawberry1",ctxAny);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Mint3Return");
  try {
    ReturnValue = tobj->Mint3(p1,p2,p3,p4,p5,p6,ctx);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ContextTests::Mint3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckCORBATypeCode_ptr(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckChestnut1(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckDates3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckOregano3(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testContextTestsAsparagus3(void)
{
  
  ::GardenPeas3* ReturnValue;
  ::Raspberry1 p1;
  CORBA::Context_var ctx;
  CORBA::Any ctxAny;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GardenPeas3* ::ContextTests::Asparagus3(::Raspberry1 p1,CORBA::Context_ptr ctx)");
#endif
  
  ernie.seed("Asparagus3Call");
  orb->orb()->get_default_context(ctx);
  ctxAny <<= GetString();
  ctx->set_one_value("BlackEyedBeans1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Scallion1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Peanuts1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Lychees1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Melon1",ctxAny);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Asparagus3Return");
  try {
    ReturnValue = tobj->Asparagus3(p1,ctx);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ContextTests::Asparagus3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGardenPeas3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckRaspberry1(p1))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testContextTestsSultana3(void)
{
  
  ::GaliaMelon3* ReturnValue;
  ::GaliaMelon3 p1;
  ::Pumpkin p2;
  CORBA::Long p3;
  
  ::Fig p4;
  ::Beetroot2* p5;
  ::Chives2 p6;
  ::Squash1* p7;
  ::Tomato2* p8;
  ::Celeriac2 p9;
  ::Rasin2 p10;
  CORBA::UShort p11;
  ::GaliaMelon3 p12;
  ::Raspberry1 p13;
  ::Mullberry1 p14;
  ::Squash2 p15;
  ::Rasin2* p16;
  CORBA::Context_var ctx;
  CORBA::Any ctxAny;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GaliaMelon3* ::ContextTests::Sultana3(const ::GaliaMelon3 &p1,const ::Pumpkin p2,CORBA::Long &p3,::Fig &p4,::Beetroot2 *&p5,const ::Chives2 &p6,::Squash1 *&p7,::Tomato2 *&p8,::Celeriac2 &p9,const ::Rasin2 &p10,CORBA::UShort p11,::GaliaMelon3 &p12,::Raspberry1 p13,::Mullberry1 p14,::Squash2 &p15,::Rasin2 *&p16,CORBA::Context_ptr ctx)");
#endif
  
  ernie.seed("Sultana3Call");
  ernie.mode(0);
  GetGaliaMelon3(p1);
  ernie.mode(0);
  GetPumpkin(p2);
  ernie.mode(1);
  GetFig(p4);
  ernie.mode(0);
  GetChives2(p6);
  ernie.mode(1);
  GetCeleriac2(p9);
  ernie.mode(0);
  GetRasin2(p10);
  ernie.mode(0);
  GetUShort(p11);
  ernie.mode(1);
  GetGaliaMelon3(p12);
  ernie.mode(1);
  GetRaspberry1(p13);
  ernie.mode(1);
  GetMullberry1(p14);
  ernie.mode(1);
  GetSquash2(p15);
  orb->orb()->get_default_context(ctx);
  ctxAny <<= GetString();
  ctx->set_one_value("Peanuts1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("KiwanoMelon1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Eddo1",ctxAny);
  ctxAny <<= GetString();
  ctx->set_one_value("Currant1",ctxAny);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Sultana3Return");
  try {
    ReturnValue = tobj->Sultana3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,ctx);
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ContextTests::Sultana3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGaliaMelon3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckLong(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckFig(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckBeetroot2(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckSquash1(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckTomato2(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckCeleriac2(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckGaliaMelon3(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckRaspberry1(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckMullberry1(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckSquash2(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckRasin2(*p16))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testExceptionTestsOrange3(void)
{
  
  
  ::Turnip3* ReturnValue;
  ::Lychees3 p1;
  ::Rasin2 p2;
  ::Grapefruit2* p3;
  ::GaliaMelon3 p4;
  ::Fennell3 p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Turnip3* ::ExceptionTests::Orange3(const ::Lychees3 &p1,const ::Rasin2 &p2,::Grapefruit2 *&p3,::GaliaMelon3 &p4,const ::Fennell3 &p5)");
#endif
  
  ernie.seed("Orange3Call");
  ernie.mode(0);
  GetLychees3(p1);
  ernie.mode(0);
  GetRasin2(p2);
  ernie.mode(1);
  GetGaliaMelon3(p4);
  ernie.mode(0);
  GetFennell3(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Orange3Return");
  try {
    ReturnValue = tobj->Orange3(p1,p2,p3,p4,p5);
  }
  catch (::CherryTomatoes3 &ex){
    if (ernie.getULong() % 6 != 0)
    {
        ::tet_infoline("Unexpected user exception (::CherryTomatoes3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherryTomatoes3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Raspberry3 &ex){
    if (ernie.getULong() % 6 != 1)
    {
        ::tet_infoline("Unexpected user exception (::Raspberry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckRaspberry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Peppercorn3 &ex){
    if (ernie.getULong() % 6 != 2)
    {
        ::tet_infoline("Unexpected user exception (::Peppercorn3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPeppercorn3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Elderberry3 &ex){
    if (ernie.getULong() % 6 != 3)
    {
        ::tet_infoline("Unexpected user exception (::Elderberry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckElderberry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Peanutsquash3 &ex){
    if (ernie.getULong() % 6 != 4)
    {
        ::tet_infoline("Unexpected user exception (::Peanutsquash3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPeanutsquash3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Damsons3 &ex){
    if (ernie.getULong() % 6 != 5)
    {
        ::tet_infoline("Unexpected user exception (::Damsons3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckDamsons3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ExceptionTests::Orange3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckTurnip3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckGrapefruit2(*p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckGaliaMelon3(p4))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testExceptionTestsBeansprout3(void)
{
  
  ::Oregano3* ReturnValue;
  ::Rasin2* p1;
  CORBA::Octet p2;
  ::Oregano3 p3;
  ::Rasin2* p4;
  ::BlackEyedBeans p5;
  ::GaliaMelon3 p6;
  ::Squash2* p7;
  ::GoldenDelicous3 p8;
  ::Peanuts1 p9;
  ::Squash1 p10;
  CORBA::Long p11;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Oregano3* ::ExceptionTests::Beansprout3(::Rasin2 *&p1,CORBA::Octet &p2,const ::Oregano3 &p3,::Rasin2 *&p4,const ::BlackEyedBeans &p5,::GaliaMelon3 &p6,::Squash2 *&p7,const ::GoldenDelicous3 &p8,::Peanuts1 &p9,const ::Squash1 &p10,CORBA::Long &p11)");
#endif
  
  ernie.seed("Beansprout3Call");
  ernie.mode(1);
  GetOctet(p2);
  ernie.mode(0);
  GetOregano3(p3);
  ernie.mode(0);
  GetBlackEyedBeans(p5);
  ernie.mode(1);
  GetGaliaMelon3(p6);
  ernie.mode(0);
  GetGoldenDelicous3(p8);
  ernie.mode(1);
  GetPeanuts1(p9);
  ernie.mode(0);
  GetSquash1(p10);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Beansprout3Return");
  try {
    ReturnValue = tobj->Beansprout3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11);
  }
  catch (::Cherry3 &ex){
    if (ernie.getULong() % 4 != 0)
    {
        ::tet_infoline("Unexpected user exception (::Cherry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Papaya3 &ex){
    if (ernie.getULong() % 4 != 1)
    {
        ::tet_infoline("Unexpected user exception (::Papaya3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPapaya3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Sharanfruit3 &ex){
    if (ernie.getULong() % 4 != 2)
    {
        ::tet_infoline("Unexpected user exception (::Sharanfruit3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckSharanfruit3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Raspberry3 &ex){
    if (ernie.getULong() % 4 != 3)
    {
        ::tet_infoline("Unexpected user exception (::Raspberry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckRaspberry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ExceptionTests::Beansprout3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckOregano3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckRasin2(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckOctet(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckRasin2(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckGaliaMelon3(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckSquash2(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckPeanuts1(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p11");
  if (!CheckLong(p11))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testExceptionTestsSweetPotato3(void)
{
  
  ::BrazilNut3* ReturnValue;
  ::Oregano3 p1;
  
  ::Tomato* p2;
  ::Cauliflower2 p3;
  ::YellowPepper2 p4;
  ::GardenPeas3* p5;
  ::Chives2* p6;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BrazilNut3* ::ExceptionTests::SweetPotato3(::Oregano3 &p1,::Tomato *&p2,const ::Cauliflower2 &p3,const ::YellowPepper2 &p4,::GardenPeas3 *&p5,::Chives2 *&p6)");
#endif
  
  ernie.seed("SweetPotato3Call");
  ernie.mode(1);
  GetOregano3(p1);
  ernie.mode(0);
  GetCauliflower2(p3);
  ernie.mode(0);
  GetYellowPepper2(p4);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("SweetPotato3Return");
  try {
    ReturnValue = tobj->SweetPotato3(p1,p2,p3,p4,p5,p6);
  }
  catch (::Raspberry3 &ex){
    if (ernie.getULong() % 5 != 0)
    {
        ::tet_infoline("Unexpected user exception (::Raspberry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckRaspberry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Sharanfruit3 &ex){
    if (ernie.getULong() % 5 != 1)
    {
        ::tet_infoline("Unexpected user exception (::Sharanfruit3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckSharanfruit3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Blackcurrant3 &ex){
    if (ernie.getULong() % 5 != 2)
    {
        ::tet_infoline("Unexpected user exception (::Blackcurrant3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckBlackcurrant3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Damsons3 &ex){
    if (ernie.getULong() % 5 != 3)
    {
        ::tet_infoline("Unexpected user exception (::Damsons3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckDamsons3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Papaya3 &ex){
    if (ernie.getULong() % 5 != 4)
    {
        ::tet_infoline("Unexpected user exception (::Papaya3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPapaya3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ExceptionTests::SweetPotato3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBrazilNut3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckOregano3(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckTomato(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckGardenPeas3(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckChives2(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testExceptionTestsKidneyBean3(void)
{
  
  CORBA::ULong ReturnValue;
  ::GoldenDelicous3* p1;
  ::Rasin2 p2;
  ::Cauliflower2 p3;
  ::Dates3 p4;
  ::Pumpkin p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::ULong ::ExceptionTests::KidneyBean3(::GoldenDelicous3 *&p1,const ::Rasin2 &p2,::Cauliflower2 &p3,const ::Dates3 &p4,const ::Pumpkin p5)");
#endif
  
  ernie.seed("KidneyBean3Call");
  ernie.mode(0);
  GetRasin2(p2);
  ernie.mode(1);
  GetCauliflower2(p3);
  ernie.mode(0);
  GetDates3(p4);
  ernie.mode(0);
  GetPumpkin(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("KidneyBean3Return");
  try {
    ReturnValue = tobj->KidneyBean3(p1,p2,p3,p4,p5);
  }
  catch (::Papaya3 &ex){
    if (ernie.getULong() % 3 != 0)
    {
        ::tet_infoline("Unexpected user exception (::Papaya3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPapaya3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::CherryTomatoes3 &ex){
    if (ernie.getULong() % 3 != 1)
    {
        ::tet_infoline("Unexpected user exception (::CherryTomatoes3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherryTomatoes3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Peanutsquash3 &ex){
    if (ernie.getULong() % 3 != 2)
    {
        ::tet_infoline("Unexpected user exception (::Peanutsquash3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPeanutsquash3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ExceptionTests::KidneyBean3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckULong(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckGoldenDelicous3(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckCauliflower2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testExceptionTestsLentil3(void)
{
  
  ::GoldenDelicous3* ReturnValue;
  CORBA::Char * p1;
  ::Squash2 p2;
  CORBA::Any p3;
  ::WaterMelon3 p4;
  
  ::Fig* p5;
  ::WaterMelon p6;
  ::Celeriac2 p7;
  CORBA::Octet p8;
  
  ::Fig* p9;
  
  ::Blackberry2 p10;
  ::Rasin2 p11;
  ::Squash2 p12;
  ::Pumpkin p13;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::GoldenDelicous3* ::ExceptionTests::Lentil3(CORBA::Char * &p1,const ::Squash2 &p2,const CORBA::Any &p3,::WaterMelon3 p4,::Fig *&p5,::WaterMelon &p6,::Celeriac2 &p7,CORBA::Octet &p8,::Fig *&p9,::Blackberry2 &p10,::Rasin2 &p11,const ::Squash2 &p12,::Pumpkin p13)");
#endif
  
  ernie.seed("Lentil3Call");
  ernie.mode(1);
  p1 = GetString();
  ernie.mode(0);
  GetSquash2(p2);
  ernie.mode(0);
  GetAny(p3);
  ernie.mode(1);
  GetWaterMelon(p6);
  ernie.mode(1);
  GetCeleriac2(p7);
  ernie.mode(1);
  GetBlackberry2(p10);
  ernie.mode(1);
  GetRasin2(p11);
  ernie.mode(0);
  GetSquash2(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Lentil3Return");
  try {
    ReturnValue = tobj->Lentil3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13);
  }
  catch (::Elderberry3 &ex){
    if (ernie.getULong() % 7 != 0)
    {
        ::tet_infoline("Unexpected user exception (::Elderberry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckElderberry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Cherry3 &ex){
    if (ernie.getULong() % 7 != 1)
    {
        ::tet_infoline("Unexpected user exception (::Cherry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Peanutsquash3 &ex){
    if (ernie.getULong() % 7 != 2)
    {
        ::tet_infoline("Unexpected user exception (::Peanutsquash3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPeanutsquash3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::CherryTomatoes3 &ex){
    if (ernie.getULong() % 7 != 3)
    {
        ::tet_infoline("Unexpected user exception (::CherryTomatoes3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherryTomatoes3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Peppercorn3 &ex){
    if (ernie.getULong() % 7 != 4)
    {
        ::tet_infoline("Unexpected user exception (::Peppercorn3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPeppercorn3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Raspberry3 &ex){
    if (ernie.getULong() % 7 != 5)
    {
        ::tet_infoline("Unexpected user exception (::Raspberry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckRaspberry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Blackcurrant3 &ex){
    if (ernie.getULong() % 7 != 6)
    {
        ::tet_infoline("Unexpected user exception (::Blackcurrant3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckBlackcurrant3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ExceptionTests::Lentil3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckGoldenDelicous3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckString(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckWaterMelon3(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckFig(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckWaterMelon(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckCeleriac2(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckOctet(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckFig(*p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckBlackberry2(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p11");
  if (!CheckRasin2(p11))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p13");
  if (!CheckPumpkin(p13))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testExceptionTestsDaikan3(void)
{
  
  CORBA::Short ReturnValue;
  CORBA::TypeCode_ptr p1;
  CORBA::Octet p2;
  ::Chestnut1 p3;
  ::Asparagus p4;
  ::GaliaMelon3 p5;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Short ::ExceptionTests::Daikan3(CORBA::TypeCode_ptr &p1,CORBA::Octet &p2,::Chestnut1 p3,::Asparagus &p4,::GaliaMelon3 &p5)");
#endif
  
  ernie.seed("Daikan3Call");
  ernie.mode(1);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(1);
  GetOctet(p2);
  ernie.mode(1);
  GetAsparagus(p4);
  ernie.mode(1);
  GetGaliaMelon3(p5);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Daikan3Return");
  try {
    ReturnValue = tobj->Daikan3(p1,p2,p3,p4,p5);
  }
  catch (::Raspberry3 &ex){
    if (ernie.getULong() % 7 != 0)
    {
        ::tet_infoline("Unexpected user exception (::Raspberry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckRaspberry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Damsons3 &ex){
    if (ernie.getULong() % 7 != 1)
    {
        ::tet_infoline("Unexpected user exception (::Damsons3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckDamsons3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::CherryTomatoes3 &ex){
    if (ernie.getULong() % 7 != 2)
    {
        ::tet_infoline("Unexpected user exception (::CherryTomatoes3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherryTomatoes3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Cherry3 &ex){
    if (ernie.getULong() % 7 != 3)
    {
        ::tet_infoline("Unexpected user exception (::Cherry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Sharanfruit3 &ex){
    if (ernie.getULong() % 7 != 4)
    {
        ::tet_infoline("Unexpected user exception (::Sharanfruit3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckSharanfruit3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Papaya3 &ex){
    if (ernie.getULong() % 7 != 5)
    {
        ::tet_infoline("Unexpected user exception (::Papaya3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPapaya3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Blackcurrant3 &ex){
    if (ernie.getULong() % 7 != 6)
    {
        ::tet_infoline("Unexpected user exception (::Blackcurrant3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckBlackcurrant3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ExceptionTests::Daikan3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckShort(ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p1");
  if (!CheckCORBATypeCode_ptr(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckOctet(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p3");
  if (!CheckChestnut1(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p4");
  if (!CheckAsparagus(p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckGaliaMelon3(p5))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testExceptionTestsCassava3(void)
{
  
  ::BambooShoot3* ReturnValue;
  CORBA::TypeCode_ptr p1;
  ::Chives2 p2;
  CORBA::UShort p3;
  ::Squash1 p4;
  ::LoganBerry* p5;
  ::Chestnut1 p6;
  ::Tomato2 p7;
  ::Tomato2 p8;
  CORBA::Any p9;
  ::Oregano3 p10;
  ::Asparagus p11;
  ::Rasin2 p12;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::BambooShoot3* ::ExceptionTests::Cassava3(CORBA::TypeCode_ptr p1,const ::Chives2 &p2,CORBA::UShort &p3,const ::Squash1 &p4,::LoganBerry *&p5,::Chestnut1 p6,::Tomato2 &p7,::Tomato2 &p8,const CORBA::Any &p9,::Oregano3 &p10,const ::Asparagus &p11,const ::Rasin2 &p12)");
#endif
  
  ernie.seed("Cassava3Call");
  ernie.mode(0);
  GetCORBATypeCode_ptr(p1);
  ernie.mode(0);
  GetChives2(p2);
  ernie.mode(1);
  GetUShort(p3);
  ernie.mode(0);
  GetSquash1(p4);
  ernie.mode(1);
  GetTomato2(p7);
  ernie.mode(1);
  GetTomato2(p8);
  ernie.mode(0);
  GetAny(p9);
  ernie.mode(1);
  GetOregano3(p10);
  ernie.mode(0);
  GetAsparagus(p11);
  ernie.mode(0);
  GetRasin2(p12);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Cassava3Return");
  try {
    ReturnValue = tobj->Cassava3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12);
  }
  catch (::Sharanfruit3 &ex){
    if (ernie.getULong() % 6 != 0)
    {
        ::tet_infoline("Unexpected user exception (::Sharanfruit3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckSharanfruit3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::CherryTomatoes3 &ex){
    if (ernie.getULong() % 6 != 1)
    {
        ::tet_infoline("Unexpected user exception (::CherryTomatoes3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherryTomatoes3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Peanutsquash3 &ex){
    if (ernie.getULong() % 6 != 2)
    {
        ::tet_infoline("Unexpected user exception (::Peanutsquash3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPeanutsquash3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Elderberry3 &ex){
    if (ernie.getULong() % 6 != 3)
    {
        ::tet_infoline("Unexpected user exception (::Elderberry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckElderberry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Blackcurrant3 &ex){
    if (ernie.getULong() % 6 != 4)
    {
        ::tet_infoline("Unexpected user exception (::Blackcurrant3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckBlackcurrant3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Peppercorn3 &ex){
    if (ernie.getULong() % 6 != 5)
    {
        ::tet_infoline("Unexpected user exception (::Peppercorn3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPeppercorn3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ExceptionTests::Cassava3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckBambooShoot3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckUShort(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p5");
  if (!CheckLoganBerry(*p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckChestnut1(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckTomato2(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckTomato2(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckOregano3(p10))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testExceptionTestsUglifruit3(void)
{
  
  ::Fennell3* ReturnValue;
  CORBA::Long p1;
  ::BlackEyedBeans p2;
  CORBA::UShort p3;
  
  ::IcebergLettuce2 p4;
  ::Asparagus p5;
  ::Rasin2 p6;
  ::Oregano3* p7;
  ::Squash1* p8;
  ::Fennell3 p9;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Fennell3* ::ExceptionTests::Uglifruit3(CORBA::Long &p1,::BlackEyedBeans &p2,CORBA::UShort &p3,const ::IcebergLettuce2 &p4,::Asparagus &p5,::Rasin2 &p6,::Oregano3 *&p7,::Squash1 *&p8,::Fennell3 &p9)");
#endif
  
  ernie.seed("Uglifruit3Call");
  ernie.mode(1);
  GetBlackEyedBeans(p2);
  ernie.mode(1);
  GetUShort(p3);
  ernie.mode(0);
  GetIcebergLettuce2(p4);
  ernie.mode(1);
  GetAsparagus(p5);
  ernie.mode(1);
  GetRasin2(p6);
  ernie.mode(1);
  GetFennell3(p9);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Uglifruit3Return");
  try {
    ReturnValue = tobj->Uglifruit3(p1,p2,p3,p4,p5,p6,p7,p8,p9);
  }
  catch (::CherryTomatoes3 &ex){
    if (ernie.getULong() % 7 != 0)
    {
        ::tet_infoline("Unexpected user exception (::CherryTomatoes3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherryTomatoes3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Papaya3 &ex){
    if (ernie.getULong() % 7 != 1)
    {
        ::tet_infoline("Unexpected user exception (::Papaya3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPapaya3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Peanutsquash3 &ex){
    if (ernie.getULong() % 7 != 2)
    {
        ::tet_infoline("Unexpected user exception (::Peanutsquash3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPeanutsquash3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Sharanfruit3 &ex){
    if (ernie.getULong() % 7 != 3)
    {
        ::tet_infoline("Unexpected user exception (::Sharanfruit3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckSharanfruit3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Blackcurrant3 &ex){
    if (ernie.getULong() % 7 != 4)
    {
        ::tet_infoline("Unexpected user exception (::Blackcurrant3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckBlackcurrant3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Damsons3 &ex){
    if (ernie.getULong() % 7 != 5)
    {
        ::tet_infoline("Unexpected user exception (::Damsons3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckDamsons3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Peppercorn3 &ex){
    if (ernie.getULong() % 7 != 6)
    {
        ::tet_infoline("Unexpected user exception (::Peppercorn3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPeppercorn3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ExceptionTests::Uglifruit3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckFennell3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckLong(p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckBlackEyedBeans(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckUShort(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckAsparagus(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckRasin2(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p7");
  if (!CheckOregano3(*p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckSquash1(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckFennell3(p9))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testExceptionTestsPear3(void)
{
  
  ::Mangosteen3* ReturnValue;
  ::Raspberry1 p1;
  ::BeechNut2* p2;
  ::Tomato2 p3;
  ::Cauliflower2* p4;
  
  ::Blackberry2 p5;
  ::Oregano3* p6;
  ::Chives2 p7;
  ::GaliaMelon3 p8;
  CORBA::Octet p9;
  ::Corn1 p10;
  ::Cauliflower2 p11;
  ::BritishQueens1 p12;
  ::BeechNut2 p13;
  ::Rasin2 p14;
  ::GaliaMelon3 p15;
  ::LoganBerry p16;
  ::GaliaMelon3 p17;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function ::Mangosteen3* ::ExceptionTests::Pear3(const ::Raspberry1 p1,::BeechNut2 *&p2,::Tomato2 &p3,::Cauliflower2 *&p4,::Blackberry2 &p5,::Oregano3 *&p6,::Chives2 &p7,::GaliaMelon3 &p8,CORBA::Octet &p9,const ::Corn1 p10,const ::Cauliflower2 &p11,::BritishQueens1 &p12,const ::BeechNut2 &p13,::Rasin2 &p14,::GaliaMelon3 &p15,const ::LoganBerry &p16,::GaliaMelon3 &p17)");
#endif
  
  ernie.seed("Pear3Call");
  ernie.mode(0);
  GetRaspberry1(p1);
  ernie.mode(1);
  GetTomato2(p3);
  ernie.mode(1);
  GetBlackberry2(p5);
  ernie.mode(1);
  GetChives2(p7);
  ernie.mode(1);
  GetGaliaMelon3(p8);
  ernie.mode(0);
  GetCorn1(p10);
  ernie.mode(0);
  GetCauliflower2(p11);
  ernie.mode(1);
  GetBritishQueens1(p12);
  ernie.mode(0);
  GetBeechNut2(p13);
  ernie.mode(1);
  GetRasin2(p14);
  ernie.mode(1);
  GetGaliaMelon3(p15);
  ernie.mode(0);
  GetLoganBerry(p16);
  ernie.mode(1);
  GetGaliaMelon3(p17);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Pear3Return");
  try {
    ReturnValue = tobj->Pear3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17);
  }
  catch (::CherryTomatoes3 &ex){
    if (ernie.getULong() % 6 != 0)
    {
        ::tet_infoline("Unexpected user exception (::CherryTomatoes3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherryTomatoes3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Sharanfruit3 &ex){
    if (ernie.getULong() % 6 != 1)
    {
        ::tet_infoline("Unexpected user exception (::Sharanfruit3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckSharanfruit3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Raspberry3 &ex){
    if (ernie.getULong() % 6 != 2)
    {
        ::tet_infoline("Unexpected user exception (::Raspberry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckRaspberry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Cherry3 &ex){
    if (ernie.getULong() % 6 != 3)
    {
        ::tet_infoline("Unexpected user exception (::Cherry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Papaya3 &ex){
    if (ernie.getULong() % 6 != 4)
    {
        ::tet_infoline("Unexpected user exception (::Papaya3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPapaya3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Peppercorn3 &ex){
    if (ernie.getULong() % 6 != 5)
    {
        ::tet_infoline("Unexpected user exception (::Peppercorn3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPeppercorn3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ExceptionTests::Pear3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckMangosteen3(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p2");
  if (!CheckBeechNut2(*p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckTomato2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckCauliflower2(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p5");
  if (!CheckBlackberry2(p5))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p6");
  if (!CheckOregano3(*p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckChives2(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p8");
  if (!CheckGaliaMelon3(p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p9");
  if (!CheckOctet(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckBritishQueens1(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p14");
  if (!CheckRasin2(p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p15");
  if (!CheckGaliaMelon3(p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p17");
  if (!CheckGaliaMelon3(p17))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


 
void testExceptionTestsSusander3(void)
{
  
  CORBA::Any* ReturnValue;
  ::BambooShoot3* p1;
  
  ::Turnip3 p2;
  ::Beetroot2 p3;
  
  ::Turnip3* p4;
  ::BambooShoot3 p5;
  ::BlackEyedBeans p6;
  ::Fennell3 p7;
  ::Cauliflower2* p8;
  ::Dates3 p9;
  ::Squash2 p10;
  ::Cauliflower2 p11;
  ::LoganBerry p12;
  ::BeechNut2 p13;
  ::Dates3* p14;
  ::GardenPeas3* p15;
  ::Asparagus* p16;
  ::Currant2 p17;
  ::Raspberry1 p18;
  ::BrazilNut3 p19;
    
  initialise();

#ifdef DEBUG
  ::tet_infoline("client: testing the function CORBA::Any* ::ExceptionTests::Susander3(::BambooShoot3 *&p1,::Turnip3 &p2,::Beetroot2 &p3,::Turnip3 *&p4,const ::BambooShoot3 &p5,::BlackEyedBeans &p6,::Fennell3 &p7,::Cauliflower2 *&p8,::Dates3 &p9,::Squash2 &p10,const ::Cauliflower2 &p11,::LoganBerry &p12,::BeechNut2 &p13,::Dates3 *&p14,::GardenPeas3 *&p15,::Asparagus *&p16,const ::Currant2 &p17,const ::Raspberry1 p18,::BrazilNut3 &p19)");
#endif
  
  ernie.seed("Susander3Call");
  ernie.mode(1);
  GetTurnip3(p2);
  ernie.mode(1);
  GetBeetroot2(p3);
  ernie.mode(0);
  GetBambooShoot3(p5);
  ernie.mode(1);
  GetBlackEyedBeans(p6);
  ernie.mode(1);
  GetFennell3(p7);
  ernie.mode(1);
  GetDates3(p9);
  ernie.mode(1);
  GetSquash2(p10);
  ernie.mode(0);
  GetCauliflower2(p11);
  ernie.mode(1);
  GetLoganBerry(p12);
  ernie.mode(1);
  GetBeechNut2(p13);
  ernie.mode(0);
  GetCurrant2(p17);
  ernie.mode(0);
  GetRaspberry1(p18);
  ernie.mode(1);
  GetBrazilNut3(p19);

#ifdef DEBUG
  ::tet_infoline("client: making Call");
#endif
  
  ernie.seed("Susander3Return");
  try {
    ReturnValue = tobj->Susander3(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18,p19);
  }
  catch (::Raspberry3 &ex){
    if (ernie.getULong() % 4 != 0)
    {
        ::tet_infoline("Unexpected user exception (::Raspberry3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckRaspberry3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Damsons3 &ex){
    if (ernie.getULong() % 4 != 1)
    {
        ::tet_infoline("Unexpected user exception (::Damsons3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckDamsons3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::Peanutsquash3 &ex){
    if (ernie.getULong() % 4 != 2)
    {
        ::tet_infoline("Unexpected user exception (::Peanutsquash3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckPeanutsquash3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (::CherryTomatoes3 &ex){
    if (ernie.getULong() % 4 != 3)
    {
        ::tet_infoline("Unexpected user exception (::CherryTomatoes3) caught");
        ::tet_result(TET_FAIL);
        return;
    }
    if (!CheckCherryTomatoes3(ex))
      ::tet_result(TET_FAIL);
    else
      ::tet_result(TET_PASS);
    return;
  }
  catch (const CORBA::SystemException &ex) {
    ::tet_infoline("client: Failed on ::ExceptionTests::Susander3");
    ::tet_printf("client: minor code %d", ex.minor());
    ::tet_result(TET_FAIL);
    return;
  }
#ifdef DEBUG
  ::tet_infoline("client: returned from Call");
#endif


  AddName("ReturnValue");
  if (!CheckAny(*ReturnValue))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p1");
  if (!CheckBambooShoot3(*p1))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p2");
  if (!CheckTurnip3(p2))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p3");
  if (!CheckBeetroot2(p3))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p4");
  if (!CheckTurnip3(*p4))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p6");
  if (!CheckBlackEyedBeans(p6))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p7");
  if (!CheckFennell3(p7))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p8");
  if (!CheckCauliflower2(*p8))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p9");
  if (!CheckDates3(p9))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p10");
  if (!CheckSquash2(p10))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p12");
  if (!CheckLoganBerry(p12))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p13");
  if (!CheckBeechNut2(p13))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p14");
  if (!CheckDates3(*p14))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p15");
  if (!CheckGardenPeas3(*p15))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(0);
  AddName("p16");
  if (!CheckAsparagus(*p16))
    ::tet_result(TET_FAIL);
  RemoveName();
  ernie.mode(2);
  AddName("p19");
  if (!CheckBrazilNut3(p19))
    ::tet_result(TET_FAIL);
  RemoveName();

  ::tet_result(TET_PASS);
  return;
}


